struct CharacterName
{
  gunichar uc;
  const char *name;
};
static const struct CharacterName character_names[] =
  {
    { 0x0020, "SPACE" },
    { 0x0021, "EXCLAMATION MARK" },
    { 0x0022, "QUOTATION MARK" },
    { 0x0023, "NUMBER SIGN" },
    { 0x0024, "DOLLAR SIGN" },
    { 0x0025, "PERCENT SIGN" },
    { 0x0026, "AMPERSAND" },
    { 0x0027, "APOSTROPHE" },
    { 0x0028, "LEFT PARENTHESIS" },
    { 0x0029, "RIGHT PARENTHESIS" },
    { 0x002A, "ASTERISK" },
    { 0x002B, "PLUS SIGN" },
    { 0x002C, "COMMA" },
    { 0x002D, "HYPHEN-MINUS" },
    { 0x002E, "FULL STOP" },
    { 0x002F, "SOLIDUS" },
    { 0x0030, "DIGIT ZERO" },
    { 0x0031, "DIGIT ONE" },
    { 0x0032, "DIGIT TWO" },
    { 0x0033, "DIGIT THREE" },
    { 0x0034, "DIGIT FOUR" },
    { 0x0035, "DIGIT FIVE" },
    { 0x0036, "DIGIT SIX" },
    { 0x0037, "DIGIT SEVEN" },
    { 0x0038, "DIGIT EIGHT" },
    { 0x0039, "DIGIT NINE" },
    { 0x003A, "COLON" },
    { 0x003B, "SEMICOLON" },
    { 0x003C, "LESS-THAN SIGN" },
    { 0x003D, "EQUALS SIGN" },
    { 0x003E, "GREATER-THAN SIGN" },
    { 0x003F, "QUESTION MARK" },
    { 0x0040, "COMMERCIAL AT" },
    { 0x0041, "LATIN CAPITAL LETTER A" },
    { 0x0042, "LATIN CAPITAL LETTER B" },
    { 0x0043, "LATIN CAPITAL LETTER C" },
    { 0x0044, "LATIN CAPITAL LETTER D" },
    { 0x0045, "LATIN CAPITAL LETTER E" },
    { 0x0046, "LATIN CAPITAL LETTER F" },
    { 0x0047, "LATIN CAPITAL LETTER G" },
    { 0x0048, "LATIN CAPITAL LETTER H" },
    { 0x0049, "LATIN CAPITAL LETTER I" },
    { 0x004A, "LATIN CAPITAL LETTER J" },
    { 0x004B, "LATIN CAPITAL LETTER K" },
    { 0x004C, "LATIN CAPITAL LETTER L" },
    { 0x004D, "LATIN CAPITAL LETTER M" },
    { 0x004E, "LATIN CAPITAL LETTER N" },
    { 0x004F, "LATIN CAPITAL LETTER O" },
    { 0x0050, "LATIN CAPITAL LETTER P" },
    { 0x0051, "LATIN CAPITAL LETTER Q" },
    { 0x0052, "LATIN CAPITAL LETTER R" },
    { 0x0053, "LATIN CAPITAL LETTER S" },
    { 0x0054, "LATIN CAPITAL LETTER T" },
    { 0x0055, "LATIN CAPITAL LETTER U" },
    { 0x0056, "LATIN CAPITAL LETTER V" },
    { 0x0057, "LATIN CAPITAL LETTER W" },
    { 0x0058, "LATIN CAPITAL LETTER X" },
    { 0x0059, "LATIN CAPITAL LETTER Y" },
    { 0x005A, "LATIN CAPITAL LETTER Z" },
    { 0x005B, "LEFT SQUARE BRACKET" },
    { 0x005C, "REVERSE SOLIDUS" },
    { 0x005D, "RIGHT SQUARE BRACKET" },
    { 0x005E, "CIRCUMFLEX ACCENT" },
    { 0x005F, "LOW LINE" },
    { 0x0060, "GRAVE ACCENT" },
    { 0x0061, "LATIN SMALL LETTER A" },
    { 0x0062, "LATIN SMALL LETTER B" },
    { 0x0063, "LATIN SMALL LETTER C" },
    { 0x0064, "LATIN SMALL LETTER D" },
    { 0x0065, "LATIN SMALL LETTER E" },
    { 0x0066, "LATIN SMALL LETTER F" },
    { 0x0067, "LATIN SMALL LETTER G" },
    { 0x0068, "LATIN SMALL LETTER H" },
    { 0x0069, "LATIN SMALL LETTER I" },
    { 0x006A, "LATIN SMALL LETTER J" },
    { 0x006B, "LATIN SMALL LETTER K" },
    { 0x006C, "LATIN SMALL LETTER L" },
    { 0x006D, "LATIN SMALL LETTER M" },
    { 0x006E, "LATIN SMALL LETTER N" },
    { 0x006F, "LATIN SMALL LETTER O" },
    { 0x0070, "LATIN SMALL LETTER P" },
    { 0x0071, "LATIN SMALL LETTER Q" },
    { 0x0072, "LATIN SMALL LETTER R" },
    { 0x0073, "LATIN SMALL LETTER S" },
    { 0x0074, "LATIN SMALL LETTER T" },
    { 0x0075, "LATIN SMALL LETTER U" },
    { 0x0076, "LATIN SMALL LETTER V" },
    { 0x0077, "LATIN SMALL LETTER W" },
    { 0x0078, "LATIN SMALL LETTER X" },
    { 0x0079, "LATIN SMALL LETTER Y" },
    { 0x007A, "LATIN SMALL LETTER Z" },
    { 0x007B, "LEFT CURLY BRACKET" },
    { 0x007C, "VERTICAL LINE" },
    { 0x007D, "RIGHT CURLY BRACKET" },
    { 0x007E, "TILDE" },
    { 0x00A0, "NO-BREAK SPACE" },
    { 0x00A1, "INVERTED EXCLAMATION MARK" },
    { 0x00A2, "CENT SIGN" },
    { 0x00A3, "POUND SIGN" },
    { 0x00A4, "CURRENCY SIGN" },
    { 0x00A5, "YEN SIGN" },
    { 0x00A6, "BROKEN BAR" },
    { 0x00A7, "SECTION SIGN" },
    { 0x00A8, "DIAERESIS" },
    { 0x00A9, "COPYRIGHT SIGN" },
    { 0x00AA, "FEMININE ORDINAL INDICATOR" },
    { 0x00AB, "LEFT-POINTING DOUBLE ANGLE QUOTATION MARK" },
    { 0x00AC, "NOT SIGN" },
    { 0x00AD, "SOFT HYPHEN" },
    { 0x00AE, "REGISTERED SIGN" },
    { 0x00AF, "MACRON" },
    { 0x00B0, "DEGREE SIGN" },
    { 0x00B1, "PLUS-MINUS SIGN" },
    { 0x00B2, "SUPERSCRIPT TWO" },
    { 0x00B3, "SUPERSCRIPT THREE" },
    { 0x00B4, "ACUTE ACCENT" },
    { 0x00B5, "MICRO SIGN" },
    { 0x00B6, "PILCROW SIGN" },
    { 0x00B7, "MIDDLE DOT" },
    { 0x00B8, "CEDILLA" },
    { 0x00B9, "SUPERSCRIPT ONE" },
    { 0x00BA, "MASCULINE ORDINAL INDICATOR" },
    { 0x00BB, "RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK" },
    { 0x00BC, "VULGAR FRACTION ONE QUARTER" },
    { 0x00BD, "VULGAR FRACTION ONE HALF" },
    { 0x00BE, "VULGAR FRACTION THREE QUARTERS" },
    { 0x00BF, "INVERTED QUESTION MARK" },
    { 0x00C0, "LATIN CAPITAL LETTER A WITH GRAVE" },
    { 0x00C1, "LATIN CAPITAL LETTER A WITH ACUTE" },
    { 0x00C2, "LATIN CAPITAL LETTER A WITH CIRCUMFLEX" },
    { 0x00C3, "LATIN CAPITAL LETTER A WITH TILDE" },
    { 0x00C4, "LATIN CAPITAL LETTER A WITH DIAERESIS" },
    { 0x00C5, "LATIN CAPITAL LETTER A WITH RING ABOVE" },
    { 0x00C6, "LATIN CAPITAL LETTER AE" },
    { 0x00C7, "LATIN CAPITAL LETTER C WITH CEDILLA" },
    { 0x00C8, "LATIN CAPITAL LETTER E WITH GRAVE" },
    { 0x00C9, "LATIN CAPITAL LETTER E WITH ACUTE" },
    { 0x00CA, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX" },
    { 0x00CB, "LATIN CAPITAL LETTER E WITH DIAERESIS" },
    { 0x00CC, "LATIN CAPITAL LETTER I WITH GRAVE" },
    { 0x00CD, "LATIN CAPITAL LETTER I WITH ACUTE" },
    { 0x00CE, "LATIN CAPITAL LETTER I WITH CIRCUMFLEX" },
    { 0x00CF, "LATIN CAPITAL LETTER I WITH DIAERESIS" },
    { 0x00D0, "LATIN CAPITAL LETTER ETH" },
    { 0x00D1, "LATIN CAPITAL LETTER N WITH TILDE" },
    { 0x00D2, "LATIN CAPITAL LETTER O WITH GRAVE" },
    { 0x00D3, "LATIN CAPITAL LETTER O WITH ACUTE" },
    { 0x00D4, "LATIN CAPITAL LETTER O WITH CIRCUMFLEX" },
    { 0x00D5, "LATIN CAPITAL LETTER O WITH TILDE" },
    { 0x00D6, "LATIN CAPITAL LETTER O WITH DIAERESIS" },
    { 0x00D7, "MULTIPLICATION SIGN" },
    { 0x00D8, "LATIN CAPITAL LETTER O WITH STROKE" },
    { 0x00D9, "LATIN CAPITAL LETTER U WITH GRAVE" },
    { 0x00DA, "LATIN CAPITAL LETTER U WITH ACUTE" },
    { 0x00DB, "LATIN CAPITAL LETTER U WITH CIRCUMFLEX" },
    { 0x00DC, "LATIN CAPITAL LETTER U WITH DIAERESIS" },
    { 0x00DD, "LATIN CAPITAL LETTER Y WITH ACUTE" },
    { 0x00DE, "LATIN CAPITAL LETTER THORN" },
    { 0x00DF, "LATIN SMALL LETTER SHARP S" },
    { 0x00E0, "LATIN SMALL LETTER A WITH GRAVE" },
    { 0x00E1, "LATIN SMALL LETTER A WITH ACUTE" },
    { 0x00E2, "LATIN SMALL LETTER A WITH CIRCUMFLEX" },
    { 0x00E3, "LATIN SMALL LETTER A WITH TILDE" },
    { 0x00E4, "LATIN SMALL LETTER A WITH DIAERESIS" },
    { 0x00E5, "LATIN SMALL LETTER A WITH RING ABOVE" },
    { 0x00E6, "LATIN SMALL LETTER AE" },
    { 0x00E7, "LATIN SMALL LETTER C WITH CEDILLA" },
    { 0x00E8, "LATIN SMALL LETTER E WITH GRAVE" },
    { 0x00E9, "LATIN SMALL LETTER E WITH ACUTE" },
    { 0x00EA, "LATIN SMALL LETTER E WITH CIRCUMFLEX" },
    { 0x00EB, "LATIN SMALL LETTER E WITH DIAERESIS" },
    { 0x00EC, "LATIN SMALL LETTER I WITH GRAVE" },
    { 0x00ED, "LATIN SMALL LETTER I WITH ACUTE" },
    { 0x00EE, "LATIN SMALL LETTER I WITH CIRCUMFLEX" },
    { 0x00EF, "LATIN SMALL LETTER I WITH DIAERESIS" },
    { 0x00F0, "LATIN SMALL LETTER ETH" },
    { 0x00F1, "LATIN SMALL LETTER N WITH TILDE" },
    { 0x00F2, "LATIN SMALL LETTER O WITH GRAVE" },
    { 0x00F3, "LATIN SMALL LETTER O WITH ACUTE" },
    { 0x00F4, "LATIN SMALL LETTER O WITH CIRCUMFLEX" },
    { 0x00F5, "LATIN SMALL LETTER O WITH TILDE" },
    { 0x00F6, "LATIN SMALL LETTER O WITH DIAERESIS" },
    { 0x00F7, "DIVISION SIGN" },
    { 0x00F8, "LATIN SMALL LETTER O WITH STROKE" },
    { 0x00F9, "LATIN SMALL LETTER U WITH GRAVE" },
    { 0x00FA, "LATIN SMALL LETTER U WITH ACUTE" },
    { 0x00FB, "LATIN SMALL LETTER U WITH CIRCUMFLEX" },
    { 0x00FC, "LATIN SMALL LETTER U WITH DIAERESIS" },
    { 0x00FD, "LATIN SMALL LETTER Y WITH ACUTE" },
    { 0x00FE, "LATIN SMALL LETTER THORN" },
    { 0x00FF, "LATIN SMALL LETTER Y WITH DIAERESIS" },
    { 0x0100, "LATIN CAPITAL LETTER A WITH MACRON" },
    { 0x0101, "LATIN SMALL LETTER A WITH MACRON" },
    { 0x0102, "LATIN CAPITAL LETTER A WITH BREVE" },
    { 0x0103, "LATIN SMALL LETTER A WITH BREVE" },
    { 0x0104, "LATIN CAPITAL LETTER A WITH OGONEK" },
    { 0x0105, "LATIN SMALL LETTER A WITH OGONEK" },
    { 0x0106, "LATIN CAPITAL LETTER C WITH ACUTE" },
    { 0x0107, "LATIN SMALL LETTER C WITH ACUTE" },
    { 0x0108, "LATIN CAPITAL LETTER C WITH CIRCUMFLEX" },
    { 0x0109, "LATIN SMALL LETTER C WITH CIRCUMFLEX" },
    { 0x010A, "LATIN CAPITAL LETTER C WITH DOT ABOVE" },
    { 0x010B, "LATIN SMALL LETTER C WITH DOT ABOVE" },
    { 0x010C, "LATIN CAPITAL LETTER C WITH CARON" },
    { 0x010D, "LATIN SMALL LETTER C WITH CARON" },
    { 0x010E, "LATIN CAPITAL LETTER D WITH CARON" },
    { 0x010F, "LATIN SMALL LETTER D WITH CARON" },
    { 0x0110, "LATIN CAPITAL LETTER D WITH STROKE" },
    { 0x0111, "LATIN SMALL LETTER D WITH STROKE" },
    { 0x0112, "LATIN CAPITAL LETTER E WITH MACRON" },
    { 0x0113, "LATIN SMALL LETTER E WITH MACRON" },
    { 0x0114, "LATIN CAPITAL LETTER E WITH BREVE" },
    { 0x0115, "LATIN SMALL LETTER E WITH BREVE" },
    { 0x0116, "LATIN CAPITAL LETTER E WITH DOT ABOVE" },
    { 0x0117, "LATIN SMALL LETTER E WITH DOT ABOVE" },
    { 0x0118, "LATIN CAPITAL LETTER E WITH OGONEK" },
    { 0x0119, "LATIN SMALL LETTER E WITH OGONEK" },
    { 0x011A, "LATIN CAPITAL LETTER E WITH CARON" },
    { 0x011B, "LATIN SMALL LETTER E WITH CARON" },
    { 0x011C, "LATIN CAPITAL LETTER G WITH CIRCUMFLEX" },
    { 0x011D, "LATIN SMALL LETTER G WITH CIRCUMFLEX" },
    { 0x011E, "LATIN CAPITAL LETTER G WITH BREVE" },
    { 0x011F, "LATIN SMALL LETTER G WITH BREVE" },
    { 0x0120, "LATIN CAPITAL LETTER G WITH DOT ABOVE" },
    { 0x0121, "LATIN SMALL LETTER G WITH DOT ABOVE" },
    { 0x0122, "LATIN CAPITAL LETTER G WITH CEDILLA" },
    { 0x0123, "LATIN SMALL LETTER G WITH CEDILLA" },
    { 0x0124, "LATIN CAPITAL LETTER H WITH CIRCUMFLEX" },
    { 0x0125, "LATIN SMALL LETTER H WITH CIRCUMFLEX" },
    { 0x0126, "LATIN CAPITAL LETTER H WITH STROKE" },
    { 0x0127, "LATIN SMALL LETTER H WITH STROKE" },
    { 0x0128, "LATIN CAPITAL LETTER I WITH TILDE" },
    { 0x0129, "LATIN SMALL LETTER I WITH TILDE" },
    { 0x012A, "LATIN CAPITAL LETTER I WITH MACRON" },
    { 0x012B, "LATIN SMALL LETTER I WITH MACRON" },
    { 0x012C, "LATIN CAPITAL LETTER I WITH BREVE" },
    { 0x012D, "LATIN SMALL LETTER I WITH BREVE" },
    { 0x012E, "LATIN CAPITAL LETTER I WITH OGONEK" },
    { 0x012F, "LATIN SMALL LETTER I WITH OGONEK" },
    { 0x0130, "LATIN CAPITAL LETTER I WITH DOT ABOVE" },
    { 0x0131, "LATIN SMALL LETTER DOTLESS I" },
    { 0x0132, "LATIN CAPITAL LIGATURE IJ" },
    { 0x0133, "LATIN SMALL LIGATURE IJ" },
    { 0x0134, "LATIN CAPITAL LETTER J WITH CIRCUMFLEX" },
    { 0x0135, "LATIN SMALL LETTER J WITH CIRCUMFLEX" },
    { 0x0136, "LATIN CAPITAL LETTER K WITH CEDILLA" },
    { 0x0137, "LATIN SMALL LETTER K WITH CEDILLA" },
    { 0x0138, "LATIN SMALL LETTER KRA" },
    { 0x0139, "LATIN CAPITAL LETTER L WITH ACUTE" },
    { 0x013A, "LATIN SMALL LETTER L WITH ACUTE" },
    { 0x013B, "LATIN CAPITAL LETTER L WITH CEDILLA" },
    { 0x013C, "LATIN SMALL LETTER L WITH CEDILLA" },
    { 0x013D, "LATIN CAPITAL LETTER L WITH CARON" },
    { 0x013E, "LATIN SMALL LETTER L WITH CARON" },
    { 0x013F, "LATIN CAPITAL LETTER L WITH MIDDLE DOT" },
    { 0x0140, "LATIN SMALL LETTER L WITH MIDDLE DOT" },
    { 0x0141, "LATIN CAPITAL LETTER L WITH STROKE" },
    { 0x0142, "LATIN SMALL LETTER L WITH STROKE" },
    { 0x0143, "LATIN CAPITAL LETTER N WITH ACUTE" },
    { 0x0144, "LATIN SMALL LETTER N WITH ACUTE" },
    { 0x0145, "LATIN CAPITAL LETTER N WITH CEDILLA" },
    { 0x0146, "LATIN SMALL LETTER N WITH CEDILLA" },
    { 0x0147, "LATIN CAPITAL LETTER N WITH CARON" },
    { 0x0148, "LATIN SMALL LETTER N WITH CARON" },
    { 0x0149, "LATIN SMALL LETTER N PRECEDED BY APOSTROPHE" },
    { 0x014A, "LATIN CAPITAL LETTER ENG" },
    { 0x014B, "LATIN SMALL LETTER ENG" },
    { 0x014C, "LATIN CAPITAL LETTER O WITH MACRON" },
    { 0x014D, "LATIN SMALL LETTER O WITH MACRON" },
    { 0x014E, "LATIN CAPITAL LETTER O WITH BREVE" },
    { 0x014F, "LATIN SMALL LETTER O WITH BREVE" },
    { 0x0150, "LATIN CAPITAL LETTER O WITH DOUBLE ACUTE" },
    { 0x0151, "LATIN SMALL LETTER O WITH DOUBLE ACUTE" },
    { 0x0152, "LATIN CAPITAL LIGATURE OE" },
    { 0x0153, "LATIN SMALL LIGATURE OE" },
    { 0x0154, "LATIN CAPITAL LETTER R WITH ACUTE" },
    { 0x0155, "LATIN SMALL LETTER R WITH ACUTE" },
    { 0x0156, "LATIN CAPITAL LETTER R WITH CEDILLA" },
    { 0x0157, "LATIN SMALL LETTER R WITH CEDILLA" },
    { 0x0158, "LATIN CAPITAL LETTER R WITH CARON" },
    { 0x0159, "LATIN SMALL LETTER R WITH CARON" },
    { 0x015A, "LATIN CAPITAL LETTER S WITH ACUTE" },
    { 0x015B, "LATIN SMALL LETTER S WITH ACUTE" },
    { 0x015C, "LATIN CAPITAL LETTER S WITH CIRCUMFLEX" },
    { 0x015D, "LATIN SMALL LETTER S WITH CIRCUMFLEX" },
    { 0x015E, "LATIN CAPITAL LETTER S WITH CEDILLA" },
    { 0x015F, "LATIN SMALL LETTER S WITH CEDILLA" },
    { 0x0160, "LATIN CAPITAL LETTER S WITH CARON" },
    { 0x0161, "LATIN SMALL LETTER S WITH CARON" },
    { 0x0162, "LATIN CAPITAL LETTER T WITH CEDILLA" },
    { 0x0163, "LATIN SMALL LETTER T WITH CEDILLA" },
    { 0x0164, "LATIN CAPITAL LETTER T WITH CARON" },
    { 0x0165, "LATIN SMALL LETTER T WITH CARON" },
    { 0x0166, "LATIN CAPITAL LETTER T WITH STROKE" },
    { 0x0167, "LATIN SMALL LETTER T WITH STROKE" },
    { 0x0168, "LATIN CAPITAL LETTER U WITH TILDE" },
    { 0x0169, "LATIN SMALL LETTER U WITH TILDE" },
    { 0x016A, "LATIN CAPITAL LETTER U WITH MACRON" },
    { 0x016B, "LATIN SMALL LETTER U WITH MACRON" },
    { 0x016C, "LATIN CAPITAL LETTER U WITH BREVE" },
    { 0x016D, "LATIN SMALL LETTER U WITH BREVE" },
    { 0x016E, "LATIN CAPITAL LETTER U WITH RING ABOVE" },
    { 0x016F, "LATIN SMALL LETTER U WITH RING ABOVE" },
    { 0x0170, "LATIN CAPITAL LETTER U WITH DOUBLE ACUTE" },
    { 0x0171, "LATIN SMALL LETTER U WITH DOUBLE ACUTE" },
    { 0x0172, "LATIN CAPITAL LETTER U WITH OGONEK" },
    { 0x0173, "LATIN SMALL LETTER U WITH OGONEK" },
    { 0x0174, "LATIN CAPITAL LETTER W WITH CIRCUMFLEX" },
    { 0x0175, "LATIN SMALL LETTER W WITH CIRCUMFLEX" },
    { 0x0176, "LATIN CAPITAL LETTER Y WITH CIRCUMFLEX" },
    { 0x0177, "LATIN SMALL LETTER Y WITH CIRCUMFLEX" },
    { 0x0178, "LATIN CAPITAL LETTER Y WITH DIAERESIS" },
    { 0x0179, "LATIN CAPITAL LETTER Z WITH ACUTE" },
    { 0x017A, "LATIN SMALL LETTER Z WITH ACUTE" },
    { 0x017B, "LATIN CAPITAL LETTER Z WITH DOT ABOVE" },
    { 0x017C, "LATIN SMALL LETTER Z WITH DOT ABOVE" },
    { 0x017D, "LATIN CAPITAL LETTER Z WITH CARON" },
    { 0x017E, "LATIN SMALL LETTER Z WITH CARON" },
    { 0x017F, "LATIN SMALL LETTER LONG S" },
    { 0x0180, "LATIN SMALL LETTER B WITH STROKE" },
    { 0x0181, "LATIN CAPITAL LETTER B WITH HOOK" },
    { 0x0182, "LATIN CAPITAL LETTER B WITH TOPBAR" },
    { 0x0183, "LATIN SMALL LETTER B WITH TOPBAR" },
    { 0x0184, "LATIN CAPITAL LETTER TONE SIX" },
    { 0x0185, "LATIN SMALL LETTER TONE SIX" },
    { 0x0186, "LATIN CAPITAL LETTER OPEN O" },
    { 0x0187, "LATIN CAPITAL LETTER C WITH HOOK" },
    { 0x0188, "LATIN SMALL LETTER C WITH HOOK" },
    { 0x0189, "LATIN CAPITAL LETTER AFRICAN D" },
    { 0x018A, "LATIN CAPITAL LETTER D WITH HOOK" },
    { 0x018B, "LATIN CAPITAL LETTER D WITH TOPBAR" },
    { 0x018C, "LATIN SMALL LETTER D WITH TOPBAR" },
    { 0x018D, "LATIN SMALL LETTER TURNED DELTA" },
    { 0x018E, "LATIN CAPITAL LETTER REVERSED E" },
    { 0x018F, "LATIN CAPITAL LETTER SCHWA" },
    { 0x0190, "LATIN CAPITAL LETTER OPEN E" },
    { 0x0191, "LATIN CAPITAL LETTER F WITH HOOK" },
    { 0x0192, "LATIN SMALL LETTER F WITH HOOK" },
    { 0x0193, "LATIN CAPITAL LETTER G WITH HOOK" },
    { 0x0194, "LATIN CAPITAL LETTER GAMMA" },
    { 0x0195, "LATIN SMALL LETTER HV" },
    { 0x0196, "LATIN CAPITAL LETTER IOTA" },
    { 0x0197, "LATIN CAPITAL LETTER I WITH STROKE" },
    { 0x0198, "LATIN CAPITAL LETTER K WITH HOOK" },
    { 0x0199, "LATIN SMALL LETTER K WITH HOOK" },
    { 0x019A, "LATIN SMALL LETTER L WITH BAR" },
    { 0x019B, "LATIN SMALL LETTER LAMBDA WITH STROKE" },
    { 0x019C, "LATIN CAPITAL LETTER TURNED M" },
    { 0x019D, "LATIN CAPITAL LETTER N WITH LEFT HOOK" },
    { 0x019E, "LATIN SMALL LETTER N WITH LONG RIGHT LEG" },
    { 0x019F, "LATIN CAPITAL LETTER O WITH MIDDLE TILDE" },
    { 0x01A0, "LATIN CAPITAL LETTER O WITH HORN" },
    { 0x01A1, "LATIN SMALL LETTER O WITH HORN" },
    { 0x01A2, "LATIN CAPITAL LETTER OI" },
    { 0x01A3, "LATIN SMALL LETTER OI" },
    { 0x01A4, "LATIN CAPITAL LETTER P WITH HOOK" },
    { 0x01A5, "LATIN SMALL LETTER P WITH HOOK" },
    { 0x01A6, "LATIN LETTER YR" },
    { 0x01A7, "LATIN CAPITAL LETTER TONE TWO" },
    { 0x01A8, "LATIN SMALL LETTER TONE TWO" },
    { 0x01A9, "LATIN CAPITAL LETTER ESH" },
    { 0x01AA, "LATIN LETTER REVERSED ESH LOOP" },
    { 0x01AB, "LATIN SMALL LETTER T WITH PALATAL HOOK" },
    { 0x01AC, "LATIN CAPITAL LETTER T WITH HOOK" },
    { 0x01AD, "LATIN SMALL LETTER T WITH HOOK" },
    { 0x01AE, "LATIN CAPITAL LETTER T WITH RETROFLEX HOOK" },
    { 0x01AF, "LATIN CAPITAL LETTER U WITH HORN" },
    { 0x01B0, "LATIN SMALL LETTER U WITH HORN" },
    { 0x01B1, "LATIN CAPITAL LETTER UPSILON" },
    { 0x01B2, "LATIN CAPITAL LETTER V WITH HOOK" },
    { 0x01B3, "LATIN CAPITAL LETTER Y WITH HOOK" },
    { 0x01B4, "LATIN SMALL LETTER Y WITH HOOK" },
    { 0x01B5, "LATIN CAPITAL LETTER Z WITH STROKE" },
    { 0x01B6, "LATIN SMALL LETTER Z WITH STROKE" },
    { 0x01B7, "LATIN CAPITAL LETTER EZH" },
    { 0x01B8, "LATIN CAPITAL LETTER EZH REVERSED" },
    { 0x01B9, "LATIN SMALL LETTER EZH REVERSED" },
    { 0x01BA, "LATIN SMALL LETTER EZH WITH TAIL" },
    { 0x01BB, "LATIN LETTER TWO WITH STROKE" },
    { 0x01BC, "LATIN CAPITAL LETTER TONE FIVE" },
    { 0x01BD, "LATIN SMALL LETTER TONE FIVE" },
    { 0x01BE, "LATIN LETTER INVERTED GLOTTAL STOP WITH STROKE" },
    { 0x01BF, "LATIN LETTER WYNN" },
    { 0x01C0, "LATIN LETTER DENTAL CLICK" },
    { 0x01C1, "LATIN LETTER LATERAL CLICK" },
    { 0x01C2, "LATIN LETTER ALVEOLAR CLICK" },
    { 0x01C3, "LATIN LETTER RETROFLEX CLICK" },
    { 0x01C4, "LATIN CAPITAL LETTER DZ WITH CARON" },
    { 0x01C5, "LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON" },
    { 0x01C6, "LATIN SMALL LETTER DZ WITH CARON" },
    { 0x01C7, "LATIN CAPITAL LETTER LJ" },
    { 0x01C8, "LATIN CAPITAL LETTER L WITH SMALL LETTER J" },
    { 0x01C9, "LATIN SMALL LETTER LJ" },
    { 0x01CA, "LATIN CAPITAL LETTER NJ" },
    { 0x01CB, "LATIN CAPITAL LETTER N WITH SMALL LETTER J" },
    { 0x01CC, "LATIN SMALL LETTER NJ" },
    { 0x01CD, "LATIN CAPITAL LETTER A WITH CARON" },
    { 0x01CE, "LATIN SMALL LETTER A WITH CARON" },
    { 0x01CF, "LATIN CAPITAL LETTER I WITH CARON" },
    { 0x01D0, "LATIN SMALL LETTER I WITH CARON" },
    { 0x01D1, "LATIN CAPITAL LETTER O WITH CARON" },
    { 0x01D2, "LATIN SMALL LETTER O WITH CARON" },
    { 0x01D3, "LATIN CAPITAL LETTER U WITH CARON" },
    { 0x01D4, "LATIN SMALL LETTER U WITH CARON" },
    { 0x01D5, "LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON" },
    { 0x01D6, "LATIN SMALL LETTER U WITH DIAERESIS AND MACRON" },
    { 0x01D7, "LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE" },
    { 0x01D8, "LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE" },
    { 0x01D9, "LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON" },
    { 0x01DA, "LATIN SMALL LETTER U WITH DIAERESIS AND CARON" },
    { 0x01DB, "LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE" },
    { 0x01DC, "LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE" },
    { 0x01DD, "LATIN SMALL LETTER TURNED E" },
    { 0x01DE, "LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON" },
    { 0x01DF, "LATIN SMALL LETTER A WITH DIAERESIS AND MACRON" },
    { 0x01E0, "LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON" },
    { 0x01E1, "LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON" },
    { 0x01E2, "LATIN CAPITAL LETTER AE WITH MACRON" },
    { 0x01E3, "LATIN SMALL LETTER AE WITH MACRON" },
    { 0x01E4, "LATIN CAPITAL LETTER G WITH STROKE" },
    { 0x01E5, "LATIN SMALL LETTER G WITH STROKE" },
    { 0x01E6, "LATIN CAPITAL LETTER G WITH CARON" },
    { 0x01E7, "LATIN SMALL LETTER G WITH CARON" },
    { 0x01E8, "LATIN CAPITAL LETTER K WITH CARON" },
    { 0x01E9, "LATIN SMALL LETTER K WITH CARON" },
    { 0x01EA, "LATIN CAPITAL LETTER O WITH OGONEK" },
    { 0x01EB, "LATIN SMALL LETTER O WITH OGONEK" },
    { 0x01EC, "LATIN CAPITAL LETTER O WITH OGONEK AND MACRON" },
    { 0x01ED, "LATIN SMALL LETTER O WITH OGONEK AND MACRON" },
    { 0x01EE, "LATIN CAPITAL LETTER EZH WITH CARON" },
    { 0x01EF, "LATIN SMALL LETTER EZH WITH CARON" },
    { 0x01F0, "LATIN SMALL LETTER J WITH CARON" },
    { 0x01F1, "LATIN CAPITAL LETTER DZ" },
    { 0x01F2, "LATIN CAPITAL LETTER D WITH SMALL LETTER Z" },
    { 0x01F3, "LATIN SMALL LETTER DZ" },
    { 0x01F4, "LATIN CAPITAL LETTER G WITH ACUTE" },
    { 0x01F5, "LATIN SMALL LETTER G WITH ACUTE" },
    { 0x01F6, "LATIN CAPITAL LETTER HWAIR" },
    { 0x01F7, "LATIN CAPITAL LETTER WYNN" },
    { 0x01F8, "LATIN CAPITAL LETTER N WITH GRAVE" },
    { 0x01F9, "LATIN SMALL LETTER N WITH GRAVE" },
    { 0x01FA, "LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE" },
    { 0x01FB, "LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE" },
    { 0x01FC, "LATIN CAPITAL LETTER AE WITH ACUTE" },
    { 0x01FD, "LATIN SMALL LETTER AE WITH ACUTE" },
    { 0x01FE, "LATIN CAPITAL LETTER O WITH STROKE AND ACUTE" },
    { 0x01FF, "LATIN SMALL LETTER O WITH STROKE AND ACUTE" },
    { 0x0200, "LATIN CAPITAL LETTER A WITH DOUBLE GRAVE" },
    { 0x0201, "LATIN SMALL LETTER A WITH DOUBLE GRAVE" },
    { 0x0202, "LATIN CAPITAL LETTER A WITH INVERTED BREVE" },
    { 0x0203, "LATIN SMALL LETTER A WITH INVERTED BREVE" },
    { 0x0204, "LATIN CAPITAL LETTER E WITH DOUBLE GRAVE" },
    { 0x0205, "LATIN SMALL LETTER E WITH DOUBLE GRAVE" },
    { 0x0206, "LATIN CAPITAL LETTER E WITH INVERTED BREVE" },
    { 0x0207, "LATIN SMALL LETTER E WITH INVERTED BREVE" },
    { 0x0208, "LATIN CAPITAL LETTER I WITH DOUBLE GRAVE" },
    { 0x0209, "LATIN SMALL LETTER I WITH DOUBLE GRAVE" },
    { 0x020A, "LATIN CAPITAL LETTER I WITH INVERTED BREVE" },
    { 0x020B, "LATIN SMALL LETTER I WITH INVERTED BREVE" },
    { 0x020C, "LATIN CAPITAL LETTER O WITH DOUBLE GRAVE" },
    { 0x020D, "LATIN SMALL LETTER O WITH DOUBLE GRAVE" },
    { 0x020E, "LATIN CAPITAL LETTER O WITH INVERTED BREVE" },
    { 0x020F, "LATIN SMALL LETTER O WITH INVERTED BREVE" },
    { 0x0210, "LATIN CAPITAL LETTER R WITH DOUBLE GRAVE" },
    { 0x0211, "LATIN SMALL LETTER R WITH DOUBLE GRAVE" },
    { 0x0212, "LATIN CAPITAL LETTER R WITH INVERTED BREVE" },
    { 0x0213, "LATIN SMALL LETTER R WITH INVERTED BREVE" },
    { 0x0214, "LATIN CAPITAL LETTER U WITH DOUBLE GRAVE" },
    { 0x0215, "LATIN SMALL LETTER U WITH DOUBLE GRAVE" },
    { 0x0216, "LATIN CAPITAL LETTER U WITH INVERTED BREVE" },
    { 0x0217, "LATIN SMALL LETTER U WITH INVERTED BREVE" },
    { 0x0218, "LATIN CAPITAL LETTER S WITH COMMA BELOW" },
    { 0x0219, "LATIN SMALL LETTER S WITH COMMA BELOW" },
    { 0x021A, "LATIN CAPITAL LETTER T WITH COMMA BELOW" },
    { 0x021B, "LATIN SMALL LETTER T WITH COMMA BELOW" },
    { 0x021C, "LATIN CAPITAL LETTER YOGH" },
    { 0x021D, "LATIN SMALL LETTER YOGH" },
    { 0x021E, "LATIN CAPITAL LETTER H WITH CARON" },
    { 0x021F, "LATIN SMALL LETTER H WITH CARON" },
    { 0x0220, "LATIN CAPITAL LETTER N WITH LONG RIGHT LEG" },
    { 0x0221, "LATIN SMALL LETTER D WITH CURL" },
    { 0x0222, "LATIN CAPITAL LETTER OU" },
    { 0x0223, "LATIN SMALL LETTER OU" },
    { 0x0224, "LATIN CAPITAL LETTER Z WITH HOOK" },
    { 0x0225, "LATIN SMALL LETTER Z WITH HOOK" },
    { 0x0226, "LATIN CAPITAL LETTER A WITH DOT ABOVE" },
    { 0x0227, "LATIN SMALL LETTER A WITH DOT ABOVE" },
    { 0x0228, "LATIN CAPITAL LETTER E WITH CEDILLA" },
    { 0x0229, "LATIN SMALL LETTER E WITH CEDILLA" },
    { 0x022A, "LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON" },
    { 0x022B, "LATIN SMALL LETTER O WITH DIAERESIS AND MACRON" },
    { 0x022C, "LATIN CAPITAL LETTER O WITH TILDE AND MACRON" },
    { 0x022D, "LATIN SMALL LETTER O WITH TILDE AND MACRON" },
    { 0x022E, "LATIN CAPITAL LETTER O WITH DOT ABOVE" },
    { 0x022F, "LATIN SMALL LETTER O WITH DOT ABOVE" },
    { 0x0230, "LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON" },
    { 0x0231, "LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON" },
    { 0x0232, "LATIN CAPITAL LETTER Y WITH MACRON" },
    { 0x0233, "LATIN SMALL LETTER Y WITH MACRON" },
    { 0x0234, "LATIN SMALL LETTER L WITH CURL" },
    { 0x0235, "LATIN SMALL LETTER N WITH CURL" },
    { 0x0236, "LATIN SMALL LETTER T WITH CURL" },
    { 0x0237, "LATIN SMALL LETTER DOTLESS J" },
    { 0x0238, "LATIN SMALL LETTER DB DIGRAPH" },
    { 0x0239, "LATIN SMALL LETTER QP DIGRAPH" },
    { 0x023A, "LATIN CAPITAL LETTER A WITH STROKE" },
    { 0x023B, "LATIN CAPITAL LETTER C WITH STROKE" },
    { 0x023C, "LATIN SMALL LETTER C WITH STROKE" },
    { 0x023D, "LATIN CAPITAL LETTER L WITH BAR" },
    { 0x023E, "LATIN CAPITAL LETTER T WITH DIAGONAL STROKE" },
    { 0x023F, "LATIN SMALL LETTER S WITH SWASH TAIL" },
    { 0x0240, "LATIN SMALL LETTER Z WITH SWASH TAIL" },
    { 0x0241, "LATIN CAPITAL LETTER GLOTTAL STOP" },
    { 0x0242, "LATIN SMALL LETTER GLOTTAL STOP" },
    { 0x0243, "LATIN CAPITAL LETTER B WITH STROKE" },
    { 0x0244, "LATIN CAPITAL LETTER U BAR" },
    { 0x0245, "LATIN CAPITAL LETTER TURNED V" },
    { 0x0246, "LATIN CAPITAL LETTER E WITH STROKE" },
    { 0x0247, "LATIN SMALL LETTER E WITH STROKE" },
    { 0x0248, "LATIN CAPITAL LETTER J WITH STROKE" },
    { 0x0249, "LATIN SMALL LETTER J WITH STROKE" },
    { 0x024A, "LATIN CAPITAL LETTER SMALL Q WITH HOOK TAIL" },
    { 0x024B, "LATIN SMALL LETTER Q WITH HOOK TAIL" },
    { 0x024C, "LATIN CAPITAL LETTER R WITH STROKE" },
    { 0x024D, "LATIN SMALL LETTER R WITH STROKE" },
    { 0x024E, "LATIN CAPITAL LETTER Y WITH STROKE" },
    { 0x024F, "LATIN SMALL LETTER Y WITH STROKE" },
    { 0x0250, "LATIN SMALL LETTER TURNED A" },
    { 0x0251, "LATIN SMALL LETTER ALPHA" },
    { 0x0252, "LATIN SMALL LETTER TURNED ALPHA" },
    { 0x0253, "LATIN SMALL LETTER B WITH HOOK" },
    { 0x0254, "LATIN SMALL LETTER OPEN O" },
    { 0x0255, "LATIN SMALL LETTER C WITH CURL" },
    { 0x0256, "LATIN SMALL LETTER D WITH TAIL" },
    { 0x0257, "LATIN SMALL LETTER D WITH HOOK" },
    { 0x0258, "LATIN SMALL LETTER REVERSED E" },
    { 0x0259, "LATIN SMALL LETTER SCHWA" },
    { 0x025A, "LATIN SMALL LETTER SCHWA WITH HOOK" },
    { 0x025B, "LATIN SMALL LETTER OPEN E" },
    { 0x025C, "LATIN SMALL LETTER REVERSED OPEN E" },
    { 0x025D, "LATIN SMALL LETTER REVERSED OPEN E WITH HOOK" },
    { 0x025E, "LATIN SMALL LETTER CLOSED REVERSED OPEN E" },
    { 0x025F, "LATIN SMALL LETTER DOTLESS J WITH STROKE" },
    { 0x0260, "LATIN SMALL LETTER G WITH HOOK" },
    { 0x0261, "LATIN SMALL LETTER SCRIPT G" },
    { 0x0262, "LATIN LETTER SMALL CAPITAL G" },
    { 0x0263, "LATIN SMALL LETTER GAMMA" },
    { 0x0264, "LATIN SMALL LETTER RAMS HORN" },
    { 0x0265, "LATIN SMALL LETTER TURNED H" },
    { 0x0266, "LATIN SMALL LETTER H WITH HOOK" },
    { 0x0267, "LATIN SMALL LETTER HENG WITH HOOK" },
    { 0x0268, "LATIN SMALL LETTER I WITH STROKE" },
    { 0x0269, "LATIN SMALL LETTER IOTA" },
    { 0x026A, "LATIN LETTER SMALL CAPITAL I" },
    { 0x026B, "LATIN SMALL LETTER L WITH MIDDLE TILDE" },
    { 0x026C, "LATIN SMALL LETTER L WITH BELT" },
    { 0x026D, "LATIN SMALL LETTER L WITH RETROFLEX HOOK" },
    { 0x026E, "LATIN SMALL LETTER LEZH" },
    { 0x026F, "LATIN SMALL LETTER TURNED M" },
    { 0x0270, "LATIN SMALL LETTER TURNED M WITH LONG LEG" },
    { 0x0271, "LATIN SMALL LETTER M WITH HOOK" },
    { 0x0272, "LATIN SMALL LETTER N WITH LEFT HOOK" },
    { 0x0273, "LATIN SMALL LETTER N WITH RETROFLEX HOOK" },
    { 0x0274, "LATIN LETTER SMALL CAPITAL N" },
    { 0x0275, "LATIN SMALL LETTER BARRED O" },
    { 0x0276, "LATIN LETTER SMALL CAPITAL OE" },
    { 0x0277, "LATIN SMALL LETTER CLOSED OMEGA" },
    { 0x0278, "LATIN SMALL LETTER PHI" },
    { 0x0279, "LATIN SMALL LETTER TURNED R" },
    { 0x027A, "LATIN SMALL LETTER TURNED R WITH LONG LEG" },
    { 0x027B, "LATIN SMALL LETTER TURNED R WITH HOOK" },
    { 0x027C, "LATIN SMALL LETTER R WITH LONG LEG" },
    { 0x027D, "LATIN SMALL LETTER R WITH TAIL" },
    { 0x027E, "LATIN SMALL LETTER R WITH FISHHOOK" },
    { 0x027F, "LATIN SMALL LETTER REVERSED R WITH FISHHOOK" },
    { 0x0280, "LATIN LETTER SMALL CAPITAL R" },
    { 0x0281, "LATIN LETTER SMALL CAPITAL INVERTED R" },
    { 0x0282, "LATIN SMALL LETTER S WITH HOOK" },
    { 0x0283, "LATIN SMALL LETTER ESH" },
    { 0x0284, "LATIN SMALL LETTER DOTLESS J WITH STROKE AND HOOK" },
    { 0x0285, "LATIN SMALL LETTER SQUAT REVERSED ESH" },
    { 0x0286, "LATIN SMALL LETTER ESH WITH CURL" },
    { 0x0287, "LATIN SMALL LETTER TURNED T" },
    { 0x0288, "LATIN SMALL LETTER T WITH RETROFLEX HOOK" },
    { 0x0289, "LATIN SMALL LETTER U BAR" },
    { 0x028A, "LATIN SMALL LETTER UPSILON" },
    { 0x028B, "LATIN SMALL LETTER V WITH HOOK" },
    { 0x028C, "LATIN SMALL LETTER TURNED V" },
    { 0x028D, "LATIN SMALL LETTER TURNED W" },
    { 0x028E, "LATIN SMALL LETTER TURNED Y" },
    { 0x028F, "LATIN LETTER SMALL CAPITAL Y" },
    { 0x0290, "LATIN SMALL LETTER Z WITH RETROFLEX HOOK" },
    { 0x0291, "LATIN SMALL LETTER Z WITH CURL" },
    { 0x0292, "LATIN SMALL LETTER EZH" },
    { 0x0293, "LATIN SMALL LETTER EZH WITH CURL" },
    { 0x0294, "LATIN LETTER GLOTTAL STOP" },
    { 0x0295, "LATIN LETTER PHARYNGEAL VOICED FRICATIVE" },
    { 0x0296, "LATIN LETTER INVERTED GLOTTAL STOP" },
    { 0x0297, "LATIN LETTER STRETCHED C" },
    { 0x0298, "LATIN LETTER BILABIAL CLICK" },
    { 0x0299, "LATIN LETTER SMALL CAPITAL B" },
    { 0x029A, "LATIN SMALL LETTER CLOSED OPEN E" },
    { 0x029B, "LATIN LETTER SMALL CAPITAL G WITH HOOK" },
    { 0x029C, "LATIN LETTER SMALL CAPITAL H" },
    { 0x029D, "LATIN SMALL LETTER J WITH CROSSED-TAIL" },
    { 0x029E, "LATIN SMALL LETTER TURNED K" },
    { 0x029F, "LATIN LETTER SMALL CAPITAL L" },
    { 0x02A0, "LATIN SMALL LETTER Q WITH HOOK" },
    { 0x02A1, "LATIN LETTER GLOTTAL STOP WITH STROKE" },
    { 0x02A2, "LATIN LETTER REVERSED GLOTTAL STOP WITH STROKE" },
    { 0x02A3, "LATIN SMALL LETTER DZ DIGRAPH" },
    { 0x02A4, "LATIN SMALL LETTER DEZH DIGRAPH" },
    { 0x02A5, "LATIN SMALL LETTER DZ DIGRAPH WITH CURL" },
    { 0x02A6, "LATIN SMALL LETTER TS DIGRAPH" },
    { 0x02A7, "LATIN SMALL LETTER TESH DIGRAPH" },
    { 0x02A8, "LATIN SMALL LETTER TC DIGRAPH WITH CURL" },
    { 0x02A9, "LATIN SMALL LETTER FENG DIGRAPH" },
    { 0x02AA, "LATIN SMALL LETTER LS DIGRAPH" },
    { 0x02AB, "LATIN SMALL LETTER LZ DIGRAPH" },
    { 0x02AC, "LATIN LETTER BILABIAL PERCUSSIVE" },
    { 0x02AD, "LATIN LETTER BIDENTAL PERCUSSIVE" },
    { 0x02AE, "LATIN SMALL LETTER TURNED H WITH FISHHOOK" },
    { 0x02AF, "LATIN SMALL LETTER TURNED H WITH FISHHOOK AND TAIL" },
    { 0x02B0, "MODIFIER LETTER SMALL H" },
    { 0x02B1, "MODIFIER LETTER SMALL H WITH HOOK" },
    { 0x02B2, "MODIFIER LETTER SMALL J" },
    { 0x02B3, "MODIFIER LETTER SMALL R" },
    { 0x02B4, "MODIFIER LETTER SMALL TURNED R" },
    { 0x02B5, "MODIFIER LETTER SMALL TURNED R WITH HOOK" },
    { 0x02B6, "MODIFIER LETTER SMALL CAPITAL INVERTED R" },
    { 0x02B7, "MODIFIER LETTER SMALL W" },
    { 0x02B8, "MODIFIER LETTER SMALL Y" },
    { 0x02B9, "MODIFIER LETTER PRIME" },
    { 0x02BA, "MODIFIER LETTER DOUBLE PRIME" },
    { 0x02BB, "MODIFIER LETTER TURNED COMMA" },
    { 0x02BC, "MODIFIER LETTER APOSTROPHE" },
    { 0x02BD, "MODIFIER LETTER REVERSED COMMA" },
    { 0x02BE, "MODIFIER LETTER RIGHT HALF RING" },
    { 0x02BF, "MODIFIER LETTER LEFT HALF RING" },
    { 0x02C0, "MODIFIER LETTER GLOTTAL STOP" },
    { 0x02C1, "MODIFIER LETTER REVERSED GLOTTAL STOP" },
    { 0x02C2, "MODIFIER LETTER LEFT ARROWHEAD" },
    { 0x02C3, "MODIFIER LETTER RIGHT ARROWHEAD" },
    { 0x02C4, "MODIFIER LETTER UP ARROWHEAD" },
    { 0x02C5, "MODIFIER LETTER DOWN ARROWHEAD" },
    { 0x02C6, "MODIFIER LETTER CIRCUMFLEX ACCENT" },
    { 0x02C7, "CARON" },
    { 0x02C8, "MODIFIER LETTER VERTICAL LINE" },
    { 0x02C9, "MODIFIER LETTER MACRON" },
    { 0x02CA, "MODIFIER LETTER ACUTE ACCENT" },
    { 0x02CB, "MODIFIER LETTER GRAVE ACCENT" },
    { 0x02CC, "MODIFIER LETTER LOW VERTICAL LINE" },
    { 0x02CD, "MODIFIER LETTER LOW MACRON" },
    { 0x02CE, "MODIFIER LETTER LOW GRAVE ACCENT" },
    { 0x02CF, "MODIFIER LETTER LOW ACUTE ACCENT" },
    { 0x02D0, "MODIFIER LETTER TRIANGULAR COLON" },
    { 0x02D1, "MODIFIER LETTER HALF TRIANGULAR COLON" },
    { 0x02D2, "MODIFIER LETTER CENTRED RIGHT HALF RING" },
    { 0x02D3, "MODIFIER LETTER CENTRED LEFT HALF RING" },
    { 0x02D4, "MODIFIER LETTER UP TACK" },
    { 0x02D5, "MODIFIER LETTER DOWN TACK" },
    { 0x02D6, "MODIFIER LETTER PLUS SIGN" },
    { 0x02D7, "MODIFIER LETTER MINUS SIGN" },
    { 0x02D8, "BREVE" },
    { 0x02D9, "DOT ABOVE" },
    { 0x02DA, "RING ABOVE" },
    { 0x02DB, "OGONEK" },
    { 0x02DC, "SMALL TILDE" },
    { 0x02DD, "DOUBLE ACUTE ACCENT" },
    { 0x02DE, "MODIFIER LETTER RHOTIC HOOK" },
    { 0x02DF, "MODIFIER LETTER CROSS ACCENT" },
    { 0x02E0, "MODIFIER LETTER SMALL GAMMA" },
    { 0x02E1, "MODIFIER LETTER SMALL L" },
    { 0x02E2, "MODIFIER LETTER SMALL S" },
    { 0x02E3, "MODIFIER LETTER SMALL X" },
    { 0x02E4, "MODIFIER LETTER SMALL REVERSED GLOTTAL STOP" },
    { 0x02E5, "MODIFIER LETTER EXTRA-HIGH TONE BAR" },
    { 0x02E6, "MODIFIER LETTER HIGH TONE BAR" },
    { 0x02E7, "MODIFIER LETTER MID TONE BAR" },
    { 0x02E8, "MODIFIER LETTER LOW TONE BAR" },
    { 0x02E9, "MODIFIER LETTER EXTRA-LOW TONE BAR" },
    { 0x02EA, "MODIFIER LETTER YIN DEPARTING TONE MARK" },
    { 0x02EB, "MODIFIER LETTER YANG DEPARTING TONE MARK" },
    { 0x02EC, "MODIFIER LETTER VOICING" },
    { 0x02ED, "MODIFIER LETTER UNASPIRATED" },
    { 0x02EE, "MODIFIER LETTER DOUBLE APOSTROPHE" },
    { 0x02EF, "MODIFIER LETTER LOW DOWN ARROWHEAD" },
    { 0x02F0, "MODIFIER LETTER LOW UP ARROWHEAD" },
    { 0x02F1, "MODIFIER LETTER LOW LEFT ARROWHEAD" },
    { 0x02F2, "MODIFIER LETTER LOW RIGHT ARROWHEAD" },
    { 0x02F3, "MODIFIER LETTER LOW RING" },
    { 0x02F4, "MODIFIER LETTER MIDDLE GRAVE ACCENT" },
    { 0x02F5, "MODIFIER LETTER MIDDLE DOUBLE GRAVE ACCENT" },
    { 0x02F6, "MODIFIER LETTER MIDDLE DOUBLE ACUTE ACCENT" },
    { 0x02F7, "MODIFIER LETTER LOW TILDE" },
    { 0x02F8, "MODIFIER LETTER RAISED COLON" },
    { 0x02F9, "MODIFIER LETTER BEGIN HIGH TONE" },
    { 0x02FA, "MODIFIER LETTER END HIGH TONE" },
    { 0x02FB, "MODIFIER LETTER BEGIN LOW TONE" },
    { 0x02FC, "MODIFIER LETTER END LOW TONE" },
    { 0x02FD, "MODIFIER LETTER SHELF" },
    { 0x02FE, "MODIFIER LETTER OPEN SHELF" },
    { 0x02FF, "MODIFIER LETTER LOW LEFT ARROW" },
    { 0x0300, "COMBINING GRAVE ACCENT" },
    { 0x0301, "COMBINING ACUTE ACCENT" },
    { 0x0302, "COMBINING CIRCUMFLEX ACCENT" },
    { 0x0303, "COMBINING TILDE" },
    { 0x0304, "COMBINING MACRON" },
    { 0x0305, "COMBINING OVERLINE" },
    { 0x0306, "COMBINING BREVE" },
    { 0x0307, "COMBINING DOT ABOVE" },
    { 0x0308, "COMBINING DIAERESIS" },
    { 0x0309, "COMBINING HOOK ABOVE" },
    { 0x030A, "COMBINING RING ABOVE" },
    { 0x030B, "COMBINING DOUBLE ACUTE ACCENT" },
    { 0x030C, "COMBINING CARON" },
    { 0x030D, "COMBINING VERTICAL LINE ABOVE" },
    { 0x030E, "COMBINING DOUBLE VERTICAL LINE ABOVE" },
    { 0x030F, "COMBINING DOUBLE GRAVE ACCENT" },
    { 0x0310, "COMBINING CANDRABINDU" },
    { 0x0311, "COMBINING INVERTED BREVE" },
    { 0x0312, "COMBINING TURNED COMMA ABOVE" },
    { 0x0313, "COMBINING COMMA ABOVE" },
    { 0x0314, "COMBINING REVERSED COMMA ABOVE" },
    { 0x0315, "COMBINING COMMA ABOVE RIGHT" },
    { 0x0316, "COMBINING GRAVE ACCENT BELOW" },
    { 0x0317, "COMBINING ACUTE ACCENT BELOW" },
    { 0x0318, "COMBINING LEFT TACK BELOW" },
    { 0x0319, "COMBINING RIGHT TACK BELOW" },
    { 0x031A, "COMBINING LEFT ANGLE ABOVE" },
    { 0x031B, "COMBINING HORN" },
    { 0x031C, "COMBINING LEFT HALF RING BELOW" },
    { 0x031D, "COMBINING UP TACK BELOW" },
    { 0x031E, "COMBINING DOWN TACK BELOW" },
    { 0x031F, "COMBINING PLUS SIGN BELOW" },
    { 0x0320, "COMBINING MINUS SIGN BELOW" },
    { 0x0321, "COMBINING PALATALIZED HOOK BELOW" },
    { 0x0322, "COMBINING RETROFLEX HOOK BELOW" },
    { 0x0323, "COMBINING DOT BELOW" },
    { 0x0324, "COMBINING DIAERESIS BELOW" },
    { 0x0325, "COMBINING RING BELOW" },
    { 0x0326, "COMBINING COMMA BELOW" },
    { 0x0327, "COMBINING CEDILLA" },
    { 0x0328, "COMBINING OGONEK" },
    { 0x0329, "COMBINING VERTICAL LINE BELOW" },
    { 0x032A, "COMBINING BRIDGE BELOW" },
    { 0x032B, "COMBINING INVERTED DOUBLE ARCH BELOW" },
    { 0x032C, "COMBINING CARON BELOW" },
    { 0x032D, "COMBINING CIRCUMFLEX ACCENT BELOW" },
    { 0x032E, "COMBINING BREVE BELOW" },
    { 0x032F, "COMBINING INVERTED BREVE BELOW" },
    { 0x0330, "COMBINING TILDE BELOW" },
    { 0x0331, "COMBINING MACRON BELOW" },
    { 0x0332, "COMBINING LOW LINE" },
    { 0x0333, "COMBINING DOUBLE LOW LINE" },
    { 0x0334, "COMBINING TILDE OVERLAY" },
    { 0x0335, "COMBINING SHORT STROKE OVERLAY" },
    { 0x0336, "COMBINING LONG STROKE OVERLAY" },
    { 0x0337, "COMBINING SHORT SOLIDUS OVERLAY" },
    { 0x0338, "COMBINING LONG SOLIDUS OVERLAY" },
    { 0x0339, "COMBINING RIGHT HALF RING BELOW" },
    { 0x033A, "COMBINING INVERTED BRIDGE BELOW" },
    { 0x033B, "COMBINING SQUARE BELOW" },
    { 0x033C, "COMBINING SEAGULL BELOW" },
    { 0x033D, "COMBINING X ABOVE" },
    { 0x033E, "COMBINING VERTICAL TILDE" },
    { 0x033F, "COMBINING DOUBLE OVERLINE" },
    { 0x0340, "COMBINING GRAVE TONE MARK" },
    { 0x0341, "COMBINING ACUTE TONE MARK" },
    { 0x0342, "COMBINING GREEK PERISPOMENI" },
    { 0x0343, "COMBINING GREEK KORONIS" },
    { 0x0344, "COMBINING GREEK DIALYTIKA TONOS" },
    { 0x0345, "COMBINING GREEK YPOGEGRAMMENI" },
    { 0x0346, "COMBINING BRIDGE ABOVE" },
    { 0x0347, "COMBINING EQUALS SIGN BELOW" },
    { 0x0348, "COMBINING DOUBLE VERTICAL LINE BELOW" },
    { 0x0349, "COMBINING LEFT ANGLE BELOW" },
    { 0x034A, "COMBINING NOT TILDE ABOVE" },
    { 0x034B, "COMBINING HOMOTHETIC ABOVE" },
    { 0x034C, "COMBINING ALMOST EQUAL TO ABOVE" },
    { 0x034D, "COMBINING LEFT RIGHT ARROW BELOW" },
    { 0x034E, "COMBINING UPWARDS ARROW BELOW" },
    { 0x034F, "COMBINING GRAPHEME JOINER" },
    { 0x0350, "COMBINING RIGHT ARROWHEAD ABOVE" },
    { 0x0351, "COMBINING LEFT HALF RING ABOVE" },
    { 0x0352, "COMBINING FERMATA" },
    { 0x0353, "COMBINING X BELOW" },
    { 0x0354, "COMBINING LEFT ARROWHEAD BELOW" },
    { 0x0355, "COMBINING RIGHT ARROWHEAD BELOW" },
    { 0x0356, "COMBINING RIGHT ARROWHEAD AND UP ARROWHEAD BELOW" },
    { 0x0357, "COMBINING RIGHT HALF RING ABOVE" },
    { 0x0358, "COMBINING DOT ABOVE RIGHT" },
    { 0x0359, "COMBINING ASTERISK BELOW" },
    { 0x035A, "COMBINING DOUBLE RING BELOW" },
    { 0x035B, "COMBINING ZIGZAG ABOVE" },
    { 0x035C, "COMBINING DOUBLE BREVE BELOW" },
    { 0x035D, "COMBINING DOUBLE BREVE" },
    { 0x035E, "COMBINING DOUBLE MACRON" },
    { 0x035F, "COMBINING DOUBLE MACRON BELOW" },
    { 0x0360, "COMBINING DOUBLE TILDE" },
    { 0x0361, "COMBINING DOUBLE INVERTED BREVE" },
    { 0x0362, "COMBINING DOUBLE RIGHTWARDS ARROW BELOW" },
    { 0x0363, "COMBINING LATIN SMALL LETTER A" },
    { 0x0364, "COMBINING LATIN SMALL LETTER E" },
    { 0x0365, "COMBINING LATIN SMALL LETTER I" },
    { 0x0366, "COMBINING LATIN SMALL LETTER O" },
    { 0x0367, "COMBINING LATIN SMALL LETTER U" },
    { 0x0368, "COMBINING LATIN SMALL LETTER C" },
    { 0x0369, "COMBINING LATIN SMALL LETTER D" },
    { 0x036A, "COMBINING LATIN SMALL LETTER H" },
    { 0x036B, "COMBINING LATIN SMALL LETTER M" },
    { 0x036C, "COMBINING LATIN SMALL LETTER R" },
    { 0x036D, "COMBINING LATIN SMALL LETTER T" },
    { 0x036E, "COMBINING LATIN SMALL LETTER V" },
    { 0x036F, "COMBINING LATIN SMALL LETTER X" },
    { 0x0370, "GREEK CAPITAL LETTER HETA" },
    { 0x0371, "GREEK SMALL LETTER HETA" },
    { 0x0372, "GREEK CAPITAL LETTER ARCHAIC SAMPI" },
    { 0x0373, "GREEK SMALL LETTER ARCHAIC SAMPI" },
    { 0x0374, "GREEK NUMERAL SIGN" },
    { 0x0375, "GREEK LOWER NUMERAL SIGN" },
    { 0x0376, "GREEK CAPITAL LETTER PAMPHYLIAN DIGAMMA" },
    { 0x0377, "GREEK SMALL LETTER PAMPHYLIAN DIGAMMA" },
    { 0x037A, "GREEK YPOGEGRAMMENI" },
    { 0x037B, "GREEK SMALL REVERSED LUNATE SIGMA SYMBOL" },
    { 0x037C, "GREEK SMALL DOTTED LUNATE SIGMA SYMBOL" },
    { 0x037D, "GREEK SMALL REVERSED DOTTED LUNATE SIGMA SYMBOL" },
    { 0x037E, "GREEK QUESTION MARK" },
    { 0x037F, "GREEK CAPITAL LETTER YOT" },
    { 0x0384, "GREEK TONOS" },
    { 0x0385, "GREEK DIALYTIKA TONOS" },
    { 0x0386, "GREEK CAPITAL LETTER ALPHA WITH TONOS" },
    { 0x0387, "GREEK ANO TELEIA" },
    { 0x0388, "GREEK CAPITAL LETTER EPSILON WITH TONOS" },
    { 0x0389, "GREEK CAPITAL LETTER ETA WITH TONOS" },
    { 0x038A, "GREEK CAPITAL LETTER IOTA WITH TONOS" },
    { 0x038C, "GREEK CAPITAL LETTER OMICRON WITH TONOS" },
    { 0x038E, "GREEK CAPITAL LETTER UPSILON WITH TONOS" },
    { 0x038F, "GREEK CAPITAL LETTER OMEGA WITH TONOS" },
    { 0x0390, "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS" },
    { 0x0391, "GREEK CAPITAL LETTER ALPHA" },
    { 0x0392, "GREEK CAPITAL LETTER BETA" },
    { 0x0393, "GREEK CAPITAL LETTER GAMMA" },
    { 0x0394, "GREEK CAPITAL LETTER DELTA" },
    { 0x0395, "GREEK CAPITAL LETTER EPSILON" },
    { 0x0396, "GREEK CAPITAL LETTER ZETA" },
    { 0x0397, "GREEK CAPITAL LETTER ETA" },
    { 0x0398, "GREEK CAPITAL LETTER THETA" },
    { 0x0399, "GREEK CAPITAL LETTER IOTA" },
    { 0x039A, "GREEK CAPITAL LETTER KAPPA" },
    { 0x039B, "GREEK CAPITAL LETTER LAMDA" },
    { 0x039C, "GREEK CAPITAL LETTER MU" },
    { 0x039D, "GREEK CAPITAL LETTER NU" },
    { 0x039E, "GREEK CAPITAL LETTER XI" },
    { 0x039F, "GREEK CAPITAL LETTER OMICRON" },
    { 0x03A0, "GREEK CAPITAL LETTER PI" },
    { 0x03A1, "GREEK CAPITAL LETTER RHO" },
    { 0x03A3, "GREEK CAPITAL LETTER SIGMA" },
    { 0x03A4, "GREEK CAPITAL LETTER TAU" },
    { 0x03A5, "GREEK CAPITAL LETTER UPSILON" },
    { 0x03A6, "GREEK CAPITAL LETTER PHI" },
    { 0x03A7, "GREEK CAPITAL LETTER CHI" },
    { 0x03A8, "GREEK CAPITAL LETTER PSI" },
    { 0x03A9, "GREEK CAPITAL LETTER OMEGA" },
    { 0x03AA, "GREEK CAPITAL LETTER IOTA WITH DIALYTIKA" },
    { 0x03AB, "GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA" },
    { 0x03AC, "GREEK SMALL LETTER ALPHA WITH TONOS" },
    { 0x03AD, "GREEK SMALL LETTER EPSILON WITH TONOS" },
    { 0x03AE, "GREEK SMALL LETTER ETA WITH TONOS" },
    { 0x03AF, "GREEK SMALL LETTER IOTA WITH TONOS" },
    { 0x03B0, "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS" },
    { 0x03B1, "GREEK SMALL LETTER ALPHA" },
    { 0x03B2, "GREEK SMALL LETTER BETA" },
    { 0x03B3, "GREEK SMALL LETTER GAMMA" },
    { 0x03B4, "GREEK SMALL LETTER DELTA" },
    { 0x03B5, "GREEK SMALL LETTER EPSILON" },
    { 0x03B6, "GREEK SMALL LETTER ZETA" },
    { 0x03B7, "GREEK SMALL LETTER ETA" },
    { 0x03B8, "GREEK SMALL LETTER THETA" },
    { 0x03B9, "GREEK SMALL LETTER IOTA" },
    { 0x03BA, "GREEK SMALL LETTER KAPPA" },
    { 0x03BB, "GREEK SMALL LETTER LAMDA" },
    { 0x03BC, "GREEK SMALL LETTER MU" },
    { 0x03BD, "GREEK SMALL LETTER NU" },
    { 0x03BE, "GREEK SMALL LETTER XI" },
    { 0x03BF, "GREEK SMALL LETTER OMICRON" },
    { 0x03C0, "GREEK SMALL LETTER PI" },
    { 0x03C1, "GREEK SMALL LETTER RHO" },
    { 0x03C2, "GREEK SMALL LETTER FINAL SIGMA" },
    { 0x03C3, "GREEK SMALL LETTER SIGMA" },
    { 0x03C4, "GREEK SMALL LETTER TAU" },
    { 0x03C5, "GREEK SMALL LETTER UPSILON" },
    { 0x03C6, "GREEK SMALL LETTER PHI" },
    { 0x03C7, "GREEK SMALL LETTER CHI" },
    { 0x03C8, "GREEK SMALL LETTER PSI" },
    { 0x03C9, "GREEK SMALL LETTER OMEGA" },
    { 0x03CA, "GREEK SMALL LETTER IOTA WITH DIALYTIKA" },
    { 0x03CB, "GREEK SMALL LETTER UPSILON WITH DIALYTIKA" },
    { 0x03CC, "GREEK SMALL LETTER OMICRON WITH TONOS" },
    { 0x03CD, "GREEK SMALL LETTER UPSILON WITH TONOS" },
    { 0x03CE, "GREEK SMALL LETTER OMEGA WITH TONOS" },
    { 0x03CF, "GREEK CAPITAL KAI SYMBOL" },
    { 0x03D0, "GREEK BETA SYMBOL" },
    { 0x03D1, "GREEK THETA SYMBOL" },
    { 0x03D2, "GREEK UPSILON WITH HOOK SYMBOL" },
    { 0x03D3, "GREEK UPSILON WITH ACUTE AND HOOK SYMBOL" },
    { 0x03D4, "GREEK UPSILON WITH DIAERESIS AND HOOK SYMBOL" },
    { 0x03D5, "GREEK PHI SYMBOL" },
    { 0x03D6, "GREEK PI SYMBOL" },
    { 0x03D7, "GREEK KAI SYMBOL" },
    { 0x03D8, "GREEK LETTER ARCHAIC KOPPA" },
    { 0x03D9, "GREEK SMALL LETTER ARCHAIC KOPPA" },
    { 0x03DA, "GREEK LETTER STIGMA" },
    { 0x03DB, "GREEK SMALL LETTER STIGMA" },
    { 0x03DC, "GREEK LETTER DIGAMMA" },
    { 0x03DD, "GREEK SMALL LETTER DIGAMMA" },
    { 0x03DE, "GREEK LETTER KOPPA" },
    { 0x03DF, "GREEK SMALL LETTER KOPPA" },
    { 0x03E0, "GREEK LETTER SAMPI" },
    { 0x03E1, "GREEK SMALL LETTER SAMPI" },
    { 0x03E2, "COPTIC CAPITAL LETTER SHEI" },
    { 0x03E3, "COPTIC SMALL LETTER SHEI" },
    { 0x03E4, "COPTIC CAPITAL LETTER FEI" },
    { 0x03E5, "COPTIC SMALL LETTER FEI" },
    { 0x03E6, "COPTIC CAPITAL LETTER KHEI" },
    { 0x03E7, "COPTIC SMALL LETTER KHEI" },
    { 0x03E8, "COPTIC CAPITAL LETTER HORI" },
    { 0x03E9, "COPTIC SMALL LETTER HORI" },
    { 0x03EA, "COPTIC CAPITAL LETTER GANGIA" },
    { 0x03EB, "COPTIC SMALL LETTER GANGIA" },
    { 0x03EC, "COPTIC CAPITAL LETTER SHIMA" },
    { 0x03ED, "COPTIC SMALL LETTER SHIMA" },
    { 0x03EE, "COPTIC CAPITAL LETTER DEI" },
    { 0x03EF, "COPTIC SMALL LETTER DEI" },
    { 0x03F0, "GREEK KAPPA SYMBOL" },
    { 0x03F1, "GREEK RHO SYMBOL" },
    { 0x03F2, "GREEK LUNATE SIGMA SYMBOL" },
    { 0x03F3, "GREEK LETTER YOT" },
    { 0x03F4, "GREEK CAPITAL THETA SYMBOL" },
    { 0x03F5, "GREEK LUNATE EPSILON SYMBOL" },
    { 0x03F6, "GREEK REVERSED LUNATE EPSILON SYMBOL" },
    { 0x03F7, "GREEK CAPITAL LETTER SHO" },
    { 0x03F8, "GREEK SMALL LETTER SHO" },
    { 0x03F9, "GREEK CAPITAL LUNATE SIGMA SYMBOL" },
    { 0x03FA, "GREEK CAPITAL LETTER SAN" },
    { 0x03FB, "GREEK SMALL LETTER SAN" },
    { 0x03FC, "GREEK RHO WITH STROKE SYMBOL" },
    { 0x03FD, "GREEK CAPITAL REVERSED LUNATE SIGMA SYMBOL" },
    { 0x03FE, "GREEK CAPITAL DOTTED LUNATE SIGMA SYMBOL" },
    { 0x03FF, "GREEK CAPITAL REVERSED DOTTED LUNATE SIGMA SYMBOL" },
    { 0x0400, "CYRILLIC CAPITAL LETTER IE WITH GRAVE" },
    { 0x0401, "CYRILLIC CAPITAL LETTER IO" },
    { 0x0402, "CYRILLIC CAPITAL LETTER DJE" },
    { 0x0403, "CYRILLIC CAPITAL LETTER GJE" },
    { 0x0404, "CYRILLIC CAPITAL LETTER UKRAINIAN IE" },
    { 0x0405, "CYRILLIC CAPITAL LETTER DZE" },
    { 0x0406, "CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I" },
    { 0x0407, "CYRILLIC CAPITAL LETTER YI" },
    { 0x0408, "CYRILLIC CAPITAL LETTER JE" },
    { 0x0409, "CYRILLIC CAPITAL LETTER LJE" },
    { 0x040A, "CYRILLIC CAPITAL LETTER NJE" },
    { 0x040B, "CYRILLIC CAPITAL LETTER TSHE" },
    { 0x040C, "CYRILLIC CAPITAL LETTER KJE" },
    { 0x040D, "CYRILLIC CAPITAL LETTER I WITH GRAVE" },
    { 0x040E, "CYRILLIC CAPITAL LETTER SHORT U" },
    { 0x040F, "CYRILLIC CAPITAL LETTER DZHE" },
    { 0x0410, "CYRILLIC CAPITAL LETTER A" },
    { 0x0411, "CYRILLIC CAPITAL LETTER BE" },
    { 0x0412, "CYRILLIC CAPITAL LETTER VE" },
    { 0x0413, "CYRILLIC CAPITAL LETTER GHE" },
    { 0x0414, "CYRILLIC CAPITAL LETTER DE" },
    { 0x0415, "CYRILLIC CAPITAL LETTER IE" },
    { 0x0416, "CYRILLIC CAPITAL LETTER ZHE" },
    { 0x0417, "CYRILLIC CAPITAL LETTER ZE" },
    { 0x0418, "CYRILLIC CAPITAL LETTER I" },
    { 0x0419, "CYRILLIC CAPITAL LETTER SHORT I" },
    { 0x041A, "CYRILLIC CAPITAL LETTER KA" },
    { 0x041B, "CYRILLIC CAPITAL LETTER EL" },
    { 0x041C, "CYRILLIC CAPITAL LETTER EM" },
    { 0x041D, "CYRILLIC CAPITAL LETTER EN" },
    { 0x041E, "CYRILLIC CAPITAL LETTER O" },
    { 0x041F, "CYRILLIC CAPITAL LETTER PE" },
    { 0x0420, "CYRILLIC CAPITAL LETTER ER" },
    { 0x0421, "CYRILLIC CAPITAL LETTER ES" },
    { 0x0422, "CYRILLIC CAPITAL LETTER TE" },
    { 0x0423, "CYRILLIC CAPITAL LETTER U" },
    { 0x0424, "CYRILLIC CAPITAL LETTER EF" },
    { 0x0425, "CYRILLIC CAPITAL LETTER HA" },
    { 0x0426, "CYRILLIC CAPITAL LETTER TSE" },
    { 0x0427, "CYRILLIC CAPITAL LETTER CHE" },
    { 0x0428, "CYRILLIC CAPITAL LETTER SHA" },
    { 0x0429, "CYRILLIC CAPITAL LETTER SHCHA" },
    { 0x042A, "CYRILLIC CAPITAL LETTER HARD SIGN" },
    { 0x042B, "CYRILLIC CAPITAL LETTER YERU" },
    { 0x042C, "CYRILLIC CAPITAL LETTER SOFT SIGN" },
    { 0x042D, "CYRILLIC CAPITAL LETTER E" },
    { 0x042E, "CYRILLIC CAPITAL LETTER YU" },
    { 0x042F, "CYRILLIC CAPITAL LETTER YA" },
    { 0x0430, "CYRILLIC SMALL LETTER A" },
    { 0x0431, "CYRILLIC SMALL LETTER BE" },
    { 0x0432, "CYRILLIC SMALL LETTER VE" },
    { 0x0433, "CYRILLIC SMALL LETTER GHE" },
    { 0x0434, "CYRILLIC SMALL LETTER DE" },
    { 0x0435, "CYRILLIC SMALL LETTER IE" },
    { 0x0436, "CYRILLIC SMALL LETTER ZHE" },
    { 0x0437, "CYRILLIC SMALL LETTER ZE" },
    { 0x0438, "CYRILLIC SMALL LETTER I" },
    { 0x0439, "CYRILLIC SMALL LETTER SHORT I" },
    { 0x043A, "CYRILLIC SMALL LETTER KA" },
    { 0x043B, "CYRILLIC SMALL LETTER EL" },
    { 0x043C, "CYRILLIC SMALL LETTER EM" },
    { 0x043D, "CYRILLIC SMALL LETTER EN" },
    { 0x043E, "CYRILLIC SMALL LETTER O" },
    { 0x043F, "CYRILLIC SMALL LETTER PE" },
    { 0x0440, "CYRILLIC SMALL LETTER ER" },
    { 0x0441, "CYRILLIC SMALL LETTER ES" },
    { 0x0442, "CYRILLIC SMALL LETTER TE" },
    { 0x0443, "CYRILLIC SMALL LETTER U" },
    { 0x0444, "CYRILLIC SMALL LETTER EF" },
    { 0x0445, "CYRILLIC SMALL LETTER HA" },
    { 0x0446, "CYRILLIC SMALL LETTER TSE" },
    { 0x0447, "CYRILLIC SMALL LETTER CHE" },
    { 0x0448, "CYRILLIC SMALL LETTER SHA" },
    { 0x0449, "CYRILLIC SMALL LETTER SHCHA" },
    { 0x044A, "CYRILLIC SMALL LETTER HARD SIGN" },
    { 0x044B, "CYRILLIC SMALL LETTER YERU" },
    { 0x044C, "CYRILLIC SMALL LETTER SOFT SIGN" },
    { 0x044D, "CYRILLIC SMALL LETTER E" },
    { 0x044E, "CYRILLIC SMALL LETTER YU" },
    { 0x044F, "CYRILLIC SMALL LETTER YA" },
    { 0x0450, "CYRILLIC SMALL LETTER IE WITH GRAVE" },
    { 0x0451, "CYRILLIC SMALL LETTER IO" },
    { 0x0452, "CYRILLIC SMALL LETTER DJE" },
    { 0x0453, "CYRILLIC SMALL LETTER GJE" },
    { 0x0454, "CYRILLIC SMALL LETTER UKRAINIAN IE" },
    { 0x0455, "CYRILLIC SMALL LETTER DZE" },
    { 0x0456, "CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I" },
    { 0x0457, "CYRILLIC SMALL LETTER YI" },
    { 0x0458, "CYRILLIC SMALL LETTER JE" },
    { 0x0459, "CYRILLIC SMALL LETTER LJE" },
    { 0x045A, "CYRILLIC SMALL LETTER NJE" },
    { 0x045B, "CYRILLIC SMALL LETTER TSHE" },
    { 0x045C, "CYRILLIC SMALL LETTER KJE" },
    { 0x045D, "CYRILLIC SMALL LETTER I WITH GRAVE" },
    { 0x045E, "CYRILLIC SMALL LETTER SHORT U" },
    { 0x045F, "CYRILLIC SMALL LETTER DZHE" },
    { 0x0460, "CYRILLIC CAPITAL LETTER OMEGA" },
    { 0x0461, "CYRILLIC SMALL LETTER OMEGA" },
    { 0x0462, "CYRILLIC CAPITAL LETTER YAT" },
    { 0x0463, "CYRILLIC SMALL LETTER YAT" },
    { 0x0464, "CYRILLIC CAPITAL LETTER IOTIFIED E" },
    { 0x0465, "CYRILLIC SMALL LETTER IOTIFIED E" },
    { 0x0466, "CYRILLIC CAPITAL LETTER LITTLE YUS" },
    { 0x0467, "CYRILLIC SMALL LETTER LITTLE YUS" },
    { 0x0468, "CYRILLIC CAPITAL LETTER IOTIFIED LITTLE YUS" },
    { 0x0469, "CYRILLIC SMALL LETTER IOTIFIED LITTLE YUS" },
    { 0x046A, "CYRILLIC CAPITAL LETTER BIG YUS" },
    { 0x046B, "CYRILLIC SMALL LETTER BIG YUS" },
    { 0x046C, "CYRILLIC CAPITAL LETTER IOTIFIED BIG YUS" },
    { 0x046D, "CYRILLIC SMALL LETTER IOTIFIED BIG YUS" },
    { 0x046E, "CYRILLIC CAPITAL LETTER KSI" },
    { 0x046F, "CYRILLIC SMALL LETTER KSI" },
    { 0x0470, "CYRILLIC CAPITAL LETTER PSI" },
    { 0x0471, "CYRILLIC SMALL LETTER PSI" },
    { 0x0472, "CYRILLIC CAPITAL LETTER FITA" },
    { 0x0473, "CYRILLIC SMALL LETTER FITA" },
    { 0x0474, "CYRILLIC CAPITAL LETTER IZHITSA" },
    { 0x0475, "CYRILLIC SMALL LETTER IZHITSA" },
    { 0x0476, "CYRILLIC CAPITAL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT" },
    { 0x0477, "CYRILLIC SMALL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT" },
    { 0x0478, "CYRILLIC CAPITAL LETTER UK" },
    { 0x0479, "CYRILLIC SMALL LETTER UK" },
    { 0x047A, "CYRILLIC CAPITAL LETTER ROUND OMEGA" },
    { 0x047B, "CYRILLIC SMALL LETTER ROUND OMEGA" },
    { 0x047C, "CYRILLIC CAPITAL LETTER OMEGA WITH TITLO" },
    { 0x047D, "CYRILLIC SMALL LETTER OMEGA WITH TITLO" },
    { 0x047E, "CYRILLIC CAPITAL LETTER OT" },
    { 0x047F, "CYRILLIC SMALL LETTER OT" },
    { 0x0480, "CYRILLIC CAPITAL LETTER KOPPA" },
    { 0x0481, "CYRILLIC SMALL LETTER KOPPA" },
    { 0x0482, "CYRILLIC THOUSANDS SIGN" },
    { 0x0483, "COMBINING CYRILLIC TITLO" },
    { 0x0484, "COMBINING CYRILLIC PALATALIZATION" },
    { 0x0485, "COMBINING CYRILLIC DASIA PNEUMATA" },
    { 0x0486, "COMBINING CYRILLIC PSILI PNEUMATA" },
    { 0x0487, "COMBINING CYRILLIC POKRYTIE" },
    { 0x0488, "COMBINING CYRILLIC HUNDRED THOUSANDS SIGN" },
    { 0x0489, "COMBINING CYRILLIC MILLIONS SIGN" },
    { 0x048A, "CYRILLIC CAPITAL LETTER SHORT I WITH TAIL" },
    { 0x048B, "CYRILLIC SMALL LETTER SHORT I WITH TAIL" },
    { 0x048C, "CYRILLIC CAPITAL LETTER SEMISOFT SIGN" },
    { 0x048D, "CYRILLIC SMALL LETTER SEMISOFT SIGN" },
    { 0x048E, "CYRILLIC CAPITAL LETTER ER WITH TICK" },
    { 0x048F, "CYRILLIC SMALL LETTER ER WITH TICK" },
    { 0x0490, "CYRILLIC CAPITAL LETTER GHE WITH UPTURN" },
    { 0x0491, "CYRILLIC SMALL LETTER GHE WITH UPTURN" },
    { 0x0492, "CYRILLIC CAPITAL LETTER GHE WITH STROKE" },
    { 0x0493, "CYRILLIC SMALL LETTER GHE WITH STROKE" },
    { 0x0494, "CYRILLIC CAPITAL LETTER GHE WITH MIDDLE HOOK" },
    { 0x0495, "CYRILLIC SMALL LETTER GHE WITH MIDDLE HOOK" },
    { 0x0496, "CYRILLIC CAPITAL LETTER ZHE WITH DESCENDER" },
    { 0x0497, "CYRILLIC SMALL LETTER ZHE WITH DESCENDER" },
    { 0x0498, "CYRILLIC CAPITAL LETTER ZE WITH DESCENDER" },
    { 0x0499, "CYRILLIC SMALL LETTER ZE WITH DESCENDER" },
    { 0x049A, "CYRILLIC CAPITAL LETTER KA WITH DESCENDER" },
    { 0x049B, "CYRILLIC SMALL LETTER KA WITH DESCENDER" },
    { 0x049C, "CYRILLIC CAPITAL LETTER KA WITH VERTICAL STROKE" },
    { 0x049D, "CYRILLIC SMALL LETTER KA WITH VERTICAL STROKE" },
    { 0x049E, "CYRILLIC CAPITAL LETTER KA WITH STROKE" },
    { 0x049F, "CYRILLIC SMALL LETTER KA WITH STROKE" },
    { 0x04A0, "CYRILLIC CAPITAL LETTER BASHKIR KA" },
    { 0x04A1, "CYRILLIC SMALL LETTER BASHKIR KA" },
    { 0x04A2, "CYRILLIC CAPITAL LETTER EN WITH DESCENDER" },
    { 0x04A3, "CYRILLIC SMALL LETTER EN WITH DESCENDER" },
    { 0x04A4, "CYRILLIC CAPITAL LIGATURE EN GHE" },
    { 0x04A5, "CYRILLIC SMALL LIGATURE EN GHE" },
    { 0x04A6, "CYRILLIC CAPITAL LETTER PE WITH MIDDLE HOOK" },
    { 0x04A7, "CYRILLIC SMALL LETTER PE WITH MIDDLE HOOK" },
    { 0x04A8, "CYRILLIC CAPITAL LETTER ABKHASIAN HA" },
    { 0x04A9, "CYRILLIC SMALL LETTER ABKHASIAN HA" },
    { 0x04AA, "CYRILLIC CAPITAL LETTER ES WITH DESCENDER" },
    { 0x04AB, "CYRILLIC SMALL LETTER ES WITH DESCENDER" },
    { 0x04AC, "CYRILLIC CAPITAL LETTER TE WITH DESCENDER" },
    { 0x04AD, "CYRILLIC SMALL LETTER TE WITH DESCENDER" },
    { 0x04AE, "CYRILLIC CAPITAL LETTER STRAIGHT U" },
    { 0x04AF, "CYRILLIC SMALL LETTER STRAIGHT U" },
    { 0x04B0, "CYRILLIC CAPITAL LETTER STRAIGHT U WITH STROKE" },
    { 0x04B1, "CYRILLIC SMALL LETTER STRAIGHT U WITH STROKE" },
    { 0x04B2, "CYRILLIC CAPITAL LETTER HA WITH DESCENDER" },
    { 0x04B3, "CYRILLIC SMALL LETTER HA WITH DESCENDER" },
    { 0x04B4, "CYRILLIC CAPITAL LIGATURE TE TSE" },
    { 0x04B5, "CYRILLIC SMALL LIGATURE TE TSE" },
    { 0x04B6, "CYRILLIC CAPITAL LETTER CHE WITH DESCENDER" },
    { 0x04B7, "CYRILLIC SMALL LETTER CHE WITH DESCENDER" },
    { 0x04B8, "CYRILLIC CAPITAL LETTER CHE WITH VERTICAL STROKE" },
    { 0x04B9, "CYRILLIC SMALL LETTER CHE WITH VERTICAL STROKE" },
    { 0x04BA, "CYRILLIC CAPITAL LETTER SHHA" },
    { 0x04BB, "CYRILLIC SMALL LETTER SHHA" },
    { 0x04BC, "CYRILLIC CAPITAL LETTER ABKHASIAN CHE" },
    { 0x04BD, "CYRILLIC SMALL LETTER ABKHASIAN CHE" },
    { 0x04BE, "CYRILLIC CAPITAL LETTER ABKHASIAN CHE WITH DESCENDER" },
    { 0x04BF, "CYRILLIC SMALL LETTER ABKHASIAN CHE WITH DESCENDER" },
    { 0x04C0, "CYRILLIC LETTER PALOCHKA" },
    { 0x04C1, "CYRILLIC CAPITAL LETTER ZHE WITH BREVE" },
    { 0x04C2, "CYRILLIC SMALL LETTER ZHE WITH BREVE" },
    { 0x04C3, "CYRILLIC CAPITAL LETTER KA WITH HOOK" },
    { 0x04C4, "CYRILLIC SMALL LETTER KA WITH HOOK" },
    { 0x04C5, "CYRILLIC CAPITAL LETTER EL WITH TAIL" },
    { 0x04C6, "CYRILLIC SMALL LETTER EL WITH TAIL" },
    { 0x04C7, "CYRILLIC CAPITAL LETTER EN WITH HOOK" },
    { 0x04C8, "CYRILLIC SMALL LETTER EN WITH HOOK" },
    { 0x04C9, "CYRILLIC CAPITAL LETTER EN WITH TAIL" },
    { 0x04CA, "CYRILLIC SMALL LETTER EN WITH TAIL" },
    { 0x04CB, "CYRILLIC CAPITAL LETTER KHAKASSIAN CHE" },
    { 0x04CC, "CYRILLIC SMALL LETTER KHAKASSIAN CHE" },
    { 0x04CD, "CYRILLIC CAPITAL LETTER EM WITH TAIL" },
    { 0x04CE, "CYRILLIC SMALL LETTER EM WITH TAIL" },
    { 0x04CF, "CYRILLIC SMALL LETTER PALOCHKA" },
    { 0x04D0, "CYRILLIC CAPITAL LETTER A WITH BREVE" },
    { 0x04D1, "CYRILLIC SMALL LETTER A WITH BREVE" },
    { 0x04D2, "CYRILLIC CAPITAL LETTER A WITH DIAERESIS" },
    { 0x04D3, "CYRILLIC SMALL LETTER A WITH DIAERESIS" },
    { 0x04D4, "CYRILLIC CAPITAL LIGATURE A IE" },
    { 0x04D5, "CYRILLIC SMALL LIGATURE A IE" },
    { 0x04D6, "CYRILLIC CAPITAL LETTER IE WITH BREVE" },
    { 0x04D7, "CYRILLIC SMALL LETTER IE WITH BREVE" },
    { 0x04D8, "CYRILLIC CAPITAL LETTER SCHWA" },
    { 0x04D9, "CYRILLIC SMALL LETTER SCHWA" },
    { 0x04DA, "CYRILLIC CAPITAL LETTER SCHWA WITH DIAERESIS" },
    { 0x04DB, "CYRILLIC SMALL LETTER SCHWA WITH DIAERESIS" },
    { 0x04DC, "CYRILLIC CAPITAL LETTER ZHE WITH DIAERESIS" },
    { 0x04DD, "CYRILLIC SMALL LETTER ZHE WITH DIAERESIS" },
    { 0x04DE, "CYRILLIC CAPITAL LETTER ZE WITH DIAERESIS" },
    { 0x04DF, "CYRILLIC SMALL LETTER ZE WITH DIAERESIS" },
    { 0x04E0, "CYRILLIC CAPITAL LETTER ABKHASIAN DZE" },
    { 0x04E1, "CYRILLIC SMALL LETTER ABKHASIAN DZE" },
    { 0x04E2, "CYRILLIC CAPITAL LETTER I WITH MACRON" },
    { 0x04E3, "CYRILLIC SMALL LETTER I WITH MACRON" },
    { 0x04E4, "CYRILLIC CAPITAL LETTER I WITH DIAERESIS" },
    { 0x04E5, "CYRILLIC SMALL LETTER I WITH DIAERESIS" },
    { 0x04E6, "CYRILLIC CAPITAL LETTER O WITH DIAERESIS" },
    { 0x04E7, "CYRILLIC SMALL LETTER O WITH DIAERESIS" },
    { 0x04E8, "CYRILLIC CAPITAL LETTER BARRED O" },
    { 0x04E9, "CYRILLIC SMALL LETTER BARRED O" },
    { 0x04EA, "CYRILLIC CAPITAL LETTER BARRED O WITH DIAERESIS" },
    { 0x04EB, "CYRILLIC SMALL LETTER BARRED O WITH DIAERESIS" },
    { 0x04EC, "CYRILLIC CAPITAL LETTER E WITH DIAERESIS" },
    { 0x04ED, "CYRILLIC SMALL LETTER E WITH DIAERESIS" },
    { 0x04EE, "CYRILLIC CAPITAL LETTER U WITH MACRON" },
    { 0x04EF, "CYRILLIC SMALL LETTER U WITH MACRON" },
    { 0x04F0, "CYRILLIC CAPITAL LETTER U WITH DIAERESIS" },
    { 0x04F1, "CYRILLIC SMALL LETTER U WITH DIAERESIS" },
    { 0x04F2, "CYRILLIC CAPITAL LETTER U WITH DOUBLE ACUTE" },
    { 0x04F3, "CYRILLIC SMALL LETTER U WITH DOUBLE ACUTE" },
    { 0x04F4, "CYRILLIC CAPITAL LETTER CHE WITH DIAERESIS" },
    { 0x04F5, "CYRILLIC SMALL LETTER CHE WITH DIAERESIS" },
    { 0x04F6, "CYRILLIC CAPITAL LETTER GHE WITH DESCENDER" },
    { 0x04F7, "CYRILLIC SMALL LETTER GHE WITH DESCENDER" },
    { 0x04F8, "CYRILLIC CAPITAL LETTER YERU WITH DIAERESIS" },
    { 0x04F9, "CYRILLIC SMALL LETTER YERU WITH DIAERESIS" },
    { 0x04FA, "CYRILLIC CAPITAL LETTER GHE WITH STROKE AND HOOK" },
    { 0x04FB, "CYRILLIC SMALL LETTER GHE WITH STROKE AND HOOK" },
    { 0x04FC, "CYRILLIC CAPITAL LETTER HA WITH HOOK" },
    { 0x04FD, "CYRILLIC SMALL LETTER HA WITH HOOK" },
    { 0x04FE, "CYRILLIC CAPITAL LETTER HA WITH STROKE" },
    { 0x04FF, "CYRILLIC SMALL LETTER HA WITH STROKE" },
    { 0x0500, "CYRILLIC CAPITAL LETTER KOMI DE" },
    { 0x0501, "CYRILLIC SMALL LETTER KOMI DE" },
    { 0x0502, "CYRILLIC CAPITAL LETTER KOMI DJE" },
    { 0x0503, "CYRILLIC SMALL LETTER KOMI DJE" },
    { 0x0504, "CYRILLIC CAPITAL LETTER KOMI ZJE" },
    { 0x0505, "CYRILLIC SMALL LETTER KOMI ZJE" },
    { 0x0506, "CYRILLIC CAPITAL LETTER KOMI DZJE" },
    { 0x0507, "CYRILLIC SMALL LETTER KOMI DZJE" },
    { 0x0508, "CYRILLIC CAPITAL LETTER KOMI LJE" },
    { 0x0509, "CYRILLIC SMALL LETTER KOMI LJE" },
    { 0x050A, "CYRILLIC CAPITAL LETTER KOMI NJE" },
    { 0x050B, "CYRILLIC SMALL LETTER KOMI NJE" },
    { 0x050C, "CYRILLIC CAPITAL LETTER KOMI SJE" },
    { 0x050D, "CYRILLIC SMALL LETTER KOMI SJE" },
    { 0x050E, "CYRILLIC CAPITAL LETTER KOMI TJE" },
    { 0x050F, "CYRILLIC SMALL LETTER KOMI TJE" },
    { 0x0510, "CYRILLIC CAPITAL LETTER REVERSED ZE" },
    { 0x0511, "CYRILLIC SMALL LETTER REVERSED ZE" },
    { 0x0512, "CYRILLIC CAPITAL LETTER EL WITH HOOK" },
    { 0x0513, "CYRILLIC SMALL LETTER EL WITH HOOK" },
    { 0x0514, "CYRILLIC CAPITAL LETTER LHA" },
    { 0x0515, "CYRILLIC SMALL LETTER LHA" },
    { 0x0516, "CYRILLIC CAPITAL LETTER RHA" },
    { 0x0517, "CYRILLIC SMALL LETTER RHA" },
    { 0x0518, "CYRILLIC CAPITAL LETTER YAE" },
    { 0x0519, "CYRILLIC SMALL LETTER YAE" },
    { 0x051A, "CYRILLIC CAPITAL LETTER QA" },
    { 0x051B, "CYRILLIC SMALL LETTER QA" },
    { 0x051C, "CYRILLIC CAPITAL LETTER WE" },
    { 0x051D, "CYRILLIC SMALL LETTER WE" },
    { 0x051E, "CYRILLIC CAPITAL LETTER ALEUT KA" },
    { 0x051F, "CYRILLIC SMALL LETTER ALEUT KA" },
    { 0x0520, "CYRILLIC CAPITAL LETTER EL WITH MIDDLE HOOK" },
    { 0x0521, "CYRILLIC SMALL LETTER EL WITH MIDDLE HOOK" },
    { 0x0522, "CYRILLIC CAPITAL LETTER EN WITH MIDDLE HOOK" },
    { 0x0523, "CYRILLIC SMALL LETTER EN WITH MIDDLE HOOK" },
    { 0x0524, "CYRILLIC CAPITAL LETTER PE WITH DESCENDER" },
    { 0x0525, "CYRILLIC SMALL LETTER PE WITH DESCENDER" },
    { 0x0526, "CYRILLIC CAPITAL LETTER SHHA WITH DESCENDER" },
    { 0x0527, "CYRILLIC SMALL LETTER SHHA WITH DESCENDER" },
    { 0x0528, "CYRILLIC CAPITAL LETTER EN WITH LEFT HOOK" },
    { 0x0529, "CYRILLIC SMALL LETTER EN WITH LEFT HOOK" },
    { 0x052A, "CYRILLIC CAPITAL LETTER DZZHE" },
    { 0x052B, "CYRILLIC SMALL LETTER DZZHE" },
    { 0x052C, "CYRILLIC CAPITAL LETTER DCHE" },
    { 0x052D, "CYRILLIC SMALL LETTER DCHE" },
    { 0x052E, "CYRILLIC CAPITAL LETTER EL WITH DESCENDER" },
    { 0x052F, "CYRILLIC SMALL LETTER EL WITH DESCENDER" },
    { 0x0531, "ARMENIAN CAPITAL LETTER AYB" },
    { 0x0532, "ARMENIAN CAPITAL LETTER BEN" },
    { 0x0533, "ARMENIAN CAPITAL LETTER GIM" },
    { 0x0534, "ARMENIAN CAPITAL LETTER DA" },
    { 0x0535, "ARMENIAN CAPITAL LETTER ECH" },
    { 0x0536, "ARMENIAN CAPITAL LETTER ZA" },
    { 0x0537, "ARMENIAN CAPITAL LETTER EH" },
    { 0x0538, "ARMENIAN CAPITAL LETTER ET" },
    { 0x0539, "ARMENIAN CAPITAL LETTER TO" },
    { 0x053A, "ARMENIAN CAPITAL LETTER ZHE" },
    { 0x053B, "ARMENIAN CAPITAL LETTER INI" },
    { 0x053C, "ARMENIAN CAPITAL LETTER LIWN" },
    { 0x053D, "ARMENIAN CAPITAL LETTER XEH" },
    { 0x053E, "ARMENIAN CAPITAL LETTER CA" },
    { 0x053F, "ARMENIAN CAPITAL LETTER KEN" },
    { 0x0540, "ARMENIAN CAPITAL LETTER HO" },
    { 0x0541, "ARMENIAN CAPITAL LETTER JA" },
    { 0x0542, "ARMENIAN CAPITAL LETTER GHAD" },
    { 0x0543, "ARMENIAN CAPITAL LETTER CHEH" },
    { 0x0544, "ARMENIAN CAPITAL LETTER MEN" },
    { 0x0545, "ARMENIAN CAPITAL LETTER YI" },
    { 0x0546, "ARMENIAN CAPITAL LETTER NOW" },
    { 0x0547, "ARMENIAN CAPITAL LETTER SHA" },
    { 0x0548, "ARMENIAN CAPITAL LETTER VO" },
    { 0x0549, "ARMENIAN CAPITAL LETTER CHA" },
    { 0x054A, "ARMENIAN CAPITAL LETTER PEH" },
    { 0x054B, "ARMENIAN CAPITAL LETTER JHEH" },
    { 0x054C, "ARMENIAN CAPITAL LETTER RA" },
    { 0x054D, "ARMENIAN CAPITAL LETTER SEH" },
    { 0x054E, "ARMENIAN CAPITAL LETTER VEW" },
    { 0x054F, "ARMENIAN CAPITAL LETTER TIWN" },
    { 0x0550, "ARMENIAN CAPITAL LETTER REH" },
    { 0x0551, "ARMENIAN CAPITAL LETTER CO" },
    { 0x0552, "ARMENIAN CAPITAL LETTER YIWN" },
    { 0x0553, "ARMENIAN CAPITAL LETTER PIWR" },
    { 0x0554, "ARMENIAN CAPITAL LETTER KEH" },
    { 0x0555, "ARMENIAN CAPITAL LETTER OH" },
    { 0x0556, "ARMENIAN CAPITAL LETTER FEH" },
    { 0x0559, "ARMENIAN MODIFIER LETTER LEFT HALF RING" },
    { 0x055A, "ARMENIAN APOSTROPHE" },
    { 0x055B, "ARMENIAN EMPHASIS MARK" },
    { 0x055C, "ARMENIAN EXCLAMATION MARK" },
    { 0x055D, "ARMENIAN COMMA" },
    { 0x055E, "ARMENIAN QUESTION MARK" },
    { 0x055F, "ARMENIAN ABBREVIATION MARK" },
    { 0x0560, "ARMENIAN SMALL LETTER TURNED AYB" },
    { 0x0561, "ARMENIAN SMALL LETTER AYB" },
    { 0x0562, "ARMENIAN SMALL LETTER BEN" },
    { 0x0563, "ARMENIAN SMALL LETTER GIM" },
    { 0x0564, "ARMENIAN SMALL LETTER DA" },
    { 0x0565, "ARMENIAN SMALL LETTER ECH" },
    { 0x0566, "ARMENIAN SMALL LETTER ZA" },
    { 0x0567, "ARMENIAN SMALL LETTER EH" },
    { 0x0568, "ARMENIAN SMALL LETTER ET" },
    { 0x0569, "ARMENIAN SMALL LETTER TO" },
    { 0x056A, "ARMENIAN SMALL LETTER ZHE" },
    { 0x056B, "ARMENIAN SMALL LETTER INI" },
    { 0x056C, "ARMENIAN SMALL LETTER LIWN" },
    { 0x056D, "ARMENIAN SMALL LETTER XEH" },
    { 0x056E, "ARMENIAN SMALL LETTER CA" },
    { 0x056F, "ARMENIAN SMALL LETTER KEN" },
    { 0x0570, "ARMENIAN SMALL LETTER HO" },
    { 0x0571, "ARMENIAN SMALL LETTER JA" },
    { 0x0572, "ARMENIAN SMALL LETTER GHAD" },
    { 0x0573, "ARMENIAN SMALL LETTER CHEH" },
    { 0x0574, "ARMENIAN SMALL LETTER MEN" },
    { 0x0575, "ARMENIAN SMALL LETTER YI" },
    { 0x0576, "ARMENIAN SMALL LETTER NOW" },
    { 0x0577, "ARMENIAN SMALL LETTER SHA" },
    { 0x0578, "ARMENIAN SMALL LETTER VO" },
    { 0x0579, "ARMENIAN SMALL LETTER CHA" },
    { 0x057A, "ARMENIAN SMALL LETTER PEH" },
    { 0x057B, "ARMENIAN SMALL LETTER JHEH" },
    { 0x057C, "ARMENIAN SMALL LETTER RA" },
    { 0x057D, "ARMENIAN SMALL LETTER SEH" },
    { 0x057E, "ARMENIAN SMALL LETTER VEW" },
    { 0x057F, "ARMENIAN SMALL LETTER TIWN" },
    { 0x0580, "ARMENIAN SMALL LETTER REH" },
    { 0x0581, "ARMENIAN SMALL LETTER CO" },
    { 0x0582, "ARMENIAN SMALL LETTER YIWN" },
    { 0x0583, "ARMENIAN SMALL LETTER PIWR" },
    { 0x0584, "ARMENIAN SMALL LETTER KEH" },
    { 0x0585, "ARMENIAN SMALL LETTER OH" },
    { 0x0586, "ARMENIAN SMALL LETTER FEH" },
    { 0x0587, "ARMENIAN SMALL LIGATURE ECH YIWN" },
    { 0x0588, "ARMENIAN SMALL LETTER YI WITH STROKE" },
    { 0x0589, "ARMENIAN FULL STOP" },
    { 0x058A, "ARMENIAN HYPHEN" },
    { 0x058D, "RIGHT-FACING ARMENIAN ETERNITY SIGN" },
    { 0x058E, "LEFT-FACING ARMENIAN ETERNITY SIGN" },
    { 0x058F, "ARMENIAN DRAM SIGN" },
    { 0x0591, "HEBREW ACCENT ETNAHTA" },
    { 0x0592, "HEBREW ACCENT SEGOL" },
    { 0x0593, "HEBREW ACCENT SHALSHELET" },
    { 0x0594, "HEBREW ACCENT ZAQEF QATAN" },
    { 0x0595, "HEBREW ACCENT ZAQEF GADOL" },
    { 0x0596, "HEBREW ACCENT TIPEHA" },
    { 0x0597, "HEBREW ACCENT REVIA" },
    { 0x0598, "HEBREW ACCENT ZARQA" },
    { 0x0599, "HEBREW ACCENT PASHTA" },
    { 0x059A, "HEBREW ACCENT YETIV" },
    { 0x059B, "HEBREW ACCENT TEVIR" },
    { 0x059C, "HEBREW ACCENT GERESH" },
    { 0x059D, "HEBREW ACCENT GERESH MUQDAM" },
    { 0x059E, "HEBREW ACCENT GERSHAYIM" },
    { 0x059F, "HEBREW ACCENT QARNEY PARA" },
    { 0x05A0, "HEBREW ACCENT TELISHA GEDOLA" },
    { 0x05A1, "HEBREW ACCENT PAZER" },
    { 0x05A2, "HEBREW ACCENT ATNAH HAFUKH" },
    { 0x05A3, "HEBREW ACCENT MUNAH" },
    { 0x05A4, "HEBREW ACCENT MAHAPAKH" },
    { 0x05A5, "HEBREW ACCENT MERKHA" },
    { 0x05A6, "HEBREW ACCENT MERKHA KEFULA" },
    { 0x05A7, "HEBREW ACCENT DARGA" },
    { 0x05A8, "HEBREW ACCENT QADMA" },
    { 0x05A9, "HEBREW ACCENT TELISHA QETANA" },
    { 0x05AA, "HEBREW ACCENT YERAH BEN YOMO" },
    { 0x05AB, "HEBREW ACCENT OLE" },
    { 0x05AC, "HEBREW ACCENT ILUY" },
    { 0x05AD, "HEBREW ACCENT DEHI" },
    { 0x05AE, "HEBREW ACCENT ZINOR" },
    { 0x05AF, "HEBREW MARK MASORA CIRCLE" },
    { 0x05B0, "HEBREW POINT SHEVA" },
    { 0x05B1, "HEBREW POINT HATAF SEGOL" },
    { 0x05B2, "HEBREW POINT HATAF PATAH" },
    { 0x05B3, "HEBREW POINT HATAF QAMATS" },
    { 0x05B4, "HEBREW POINT HIRIQ" },
    { 0x05B5, "HEBREW POINT TSERE" },
    { 0x05B6, "HEBREW POINT SEGOL" },
    { 0x05B7, "HEBREW POINT PATAH" },
    { 0x05B8, "HEBREW POINT QAMATS" },
    { 0x05B9, "HEBREW POINT HOLAM" },
    { 0x05BA, "HEBREW POINT HOLAM HASER FOR VAV" },
    { 0x05BB, "HEBREW POINT QUBUTS" },
    { 0x05BC, "HEBREW POINT DAGESH OR MAPIQ" },
    { 0x05BD, "HEBREW POINT METEG" },
    { 0x05BE, "HEBREW PUNCTUATION MAQAF" },
    { 0x05BF, "HEBREW POINT RAFE" },
    { 0x05C0, "HEBREW PUNCTUATION PASEQ" },
    { 0x05C1, "HEBREW POINT SHIN DOT" },
    { 0x05C2, "HEBREW POINT SIN DOT" },
    { 0x05C3, "HEBREW PUNCTUATION SOF PASUQ" },
    { 0x05C4, "HEBREW MARK UPPER DOT" },
    { 0x05C5, "HEBREW MARK LOWER DOT" },
    { 0x05C6, "HEBREW PUNCTUATION NUN HAFUKHA" },
    { 0x05C7, "HEBREW POINT QAMATS QATAN" },
    { 0x05D0, "HEBREW LETTER ALEF" },
    { 0x05D1, "HEBREW LETTER BET" },
    { 0x05D2, "HEBREW LETTER GIMEL" },
    { 0x05D3, "HEBREW LETTER DALET" },
    { 0x05D4, "HEBREW LETTER HE" },
    { 0x05D5, "HEBREW LETTER VAV" },
    { 0x05D6, "HEBREW LETTER ZAYIN" },
    { 0x05D7, "HEBREW LETTER HET" },
    { 0x05D8, "HEBREW LETTER TET" },
    { 0x05D9, "HEBREW LETTER YOD" },
    { 0x05DA, "HEBREW LETTER FINAL KAF" },
    { 0x05DB, "HEBREW LETTER KAF" },
    { 0x05DC, "HEBREW LETTER LAMED" },
    { 0x05DD, "HEBREW LETTER FINAL MEM" },
    { 0x05DE, "HEBREW LETTER MEM" },
    { 0x05DF, "HEBREW LETTER FINAL NUN" },
    { 0x05E0, "HEBREW LETTER NUN" },
    { 0x05E1, "HEBREW LETTER SAMEKH" },
    { 0x05E2, "HEBREW LETTER AYIN" },
    { 0x05E3, "HEBREW LETTER FINAL PE" },
    { 0x05E4, "HEBREW LETTER PE" },
    { 0x05E5, "HEBREW LETTER FINAL TSADI" },
    { 0x05E6, "HEBREW LETTER TSADI" },
    { 0x05E7, "HEBREW LETTER QOF" },
    { 0x05E8, "HEBREW LETTER RESH" },
    { 0x05E9, "HEBREW LETTER SHIN" },
    { 0x05EA, "HEBREW LETTER TAV" },
    { 0x05EF, "HEBREW YOD TRIANGLE" },
    { 0x05F0, "HEBREW LIGATURE YIDDISH DOUBLE VAV" },
    { 0x05F1, "HEBREW LIGATURE YIDDISH VAV YOD" },
    { 0x05F2, "HEBREW LIGATURE YIDDISH DOUBLE YOD" },
    { 0x05F3, "HEBREW PUNCTUATION GERESH" },
    { 0x05F4, "HEBREW PUNCTUATION GERSHAYIM" },
    { 0x0600, "ARABIC NUMBER SIGN" },
    { 0x0601, "ARABIC SIGN SANAH" },
    { 0x0602, "ARABIC FOOTNOTE MARKER" },
    { 0x0603, "ARABIC SIGN SAFHA" },
    { 0x0604, "ARABIC SIGN SAMVAT" },
    { 0x0605, "ARABIC NUMBER MARK ABOVE" },
    { 0x0606, "ARABIC-INDIC CUBE ROOT" },
    { 0x0607, "ARABIC-INDIC FOURTH ROOT" },
    { 0x0608, "ARABIC RAY" },
    { 0x0609, "ARABIC-INDIC PER MILLE SIGN" },
    { 0x060A, "ARABIC-INDIC PER TEN THOUSAND SIGN" },
    { 0x060B, "AFGHANI SIGN" },
    { 0x060C, "ARABIC COMMA" },
    { 0x060D, "ARABIC DATE SEPARATOR" },
    { 0x060E, "ARABIC POETIC VERSE SIGN" },
    { 0x060F, "ARABIC SIGN MISRA" },
    { 0x0610, "ARABIC SIGN SALLALLAHOU ALAYHE WASSALLAM" },
    { 0x0611, "ARABIC SIGN ALAYHE ASSALLAM" },
    { 0x0612, "ARABIC SIGN RAHMATULLAH ALAYHE" },
    { 0x0613, "ARABIC SIGN RADI ALLAHOU ANHU" },
    { 0x0614, "ARABIC SIGN TAKHALLUS" },
    { 0x0615, "ARABIC SMALL HIGH TAH" },
    { 0x0616, "ARABIC SMALL HIGH LIGATURE ALEF WITH LAM WITH YEH" },
    { 0x0617, "ARABIC SMALL HIGH ZAIN" },
    { 0x0618, "ARABIC SMALL FATHA" },
    { 0x0619, "ARABIC SMALL DAMMA" },
    { 0x061A, "ARABIC SMALL KASRA" },
    { 0x061B, "ARABIC SEMICOLON" },
    { 0x061C, "ARABIC LETTER MARK" },
    { 0x061D, "ARABIC END OF TEXT MARK" },
    { 0x061E, "ARABIC TRIPLE DOT PUNCTUATION MARK" },
    { 0x061F, "ARABIC QUESTION MARK" },
    { 0x0620, "ARABIC LETTER KASHMIRI YEH" },
    { 0x0621, "ARABIC LETTER HAMZA" },
    { 0x0622, "ARABIC LETTER ALEF WITH MADDA ABOVE" },
    { 0x0623, "ARABIC LETTER ALEF WITH HAMZA ABOVE" },
    { 0x0624, "ARABIC LETTER WAW WITH HAMZA ABOVE" },
    { 0x0625, "ARABIC LETTER ALEF WITH HAMZA BELOW" },
    { 0x0626, "ARABIC LETTER YEH WITH HAMZA ABOVE" },
    { 0x0627, "ARABIC LETTER ALEF" },
    { 0x0628, "ARABIC LETTER BEH" },
    { 0x0629, "ARABIC LETTER TEH MARBUTA" },
    { 0x062A, "ARABIC LETTER TEH" },
    { 0x062B, "ARABIC LETTER THEH" },
    { 0x062C, "ARABIC LETTER JEEM" },
    { 0x062D, "ARABIC LETTER HAH" },
    { 0x062E, "ARABIC LETTER KHAH" },
    { 0x062F, "ARABIC LETTER DAL" },
    { 0x0630, "ARABIC LETTER THAL" },
    { 0x0631, "ARABIC LETTER REH" },
    { 0x0632, "ARABIC LETTER ZAIN" },
    { 0x0633, "ARABIC LETTER SEEN" },
    { 0x0634, "ARABIC LETTER SHEEN" },
    { 0x0635, "ARABIC LETTER SAD" },
    { 0x0636, "ARABIC LETTER DAD" },
    { 0x0637, "ARABIC LETTER TAH" },
    { 0x0638, "ARABIC LETTER ZAH" },
    { 0x0639, "ARABIC LETTER AIN" },
    { 0x063A, "ARABIC LETTER GHAIN" },
    { 0x063B, "ARABIC LETTER KEHEH WITH TWO DOTS ABOVE" },
    { 0x063C, "ARABIC LETTER KEHEH WITH THREE DOTS BELOW" },
    { 0x063D, "ARABIC LETTER FARSI YEH WITH INVERTED V" },
    { 0x063E, "ARABIC LETTER FARSI YEH WITH TWO DOTS ABOVE" },
    { 0x063F, "ARABIC LETTER FARSI YEH WITH THREE DOTS ABOVE" },
    { 0x0640, "ARABIC TATWEEL" },
    { 0x0641, "ARABIC LETTER FEH" },
    { 0x0642, "ARABIC LETTER QAF" },
    { 0x0643, "ARABIC LETTER KAF" },
    { 0x0644, "ARABIC LETTER LAM" },
    { 0x0645, "ARABIC LETTER MEEM" },
    { 0x0646, "ARABIC LETTER NOON" },
    { 0x0647, "ARABIC LETTER HEH" },
    { 0x0648, "ARABIC LETTER WAW" },
    { 0x0649, "ARABIC LETTER ALEF MAKSURA" },
    { 0x064A, "ARABIC LETTER YEH" },
    { 0x064B, "ARABIC FATHATAN" },
    { 0x064C, "ARABIC DAMMATAN" },
    { 0x064D, "ARABIC KASRATAN" },
    { 0x064E, "ARABIC FATHA" },
    { 0x064F, "ARABIC DAMMA" },
    { 0x0650, "ARABIC KASRA" },
    { 0x0651, "ARABIC SHADDA" },
    { 0x0652, "ARABIC SUKUN" },
    { 0x0653, "ARABIC MADDAH ABOVE" },
    { 0x0654, "ARABIC HAMZA ABOVE" },
    { 0x0655, "ARABIC HAMZA BELOW" },
    { 0x0656, "ARABIC SUBSCRIPT ALEF" },
    { 0x0657, "ARABIC INVERTED DAMMA" },
    { 0x0658, "ARABIC MARK NOON GHUNNA" },
    { 0x0659, "ARABIC ZWARAKAY" },
    { 0x065A, "ARABIC VOWEL SIGN SMALL V ABOVE" },
    { 0x065B, "ARABIC VOWEL SIGN INVERTED SMALL V ABOVE" },
    { 0x065C, "ARABIC VOWEL SIGN DOT BELOW" },
    { 0x065D, "ARABIC REVERSED DAMMA" },
    { 0x065E, "ARABIC FATHA WITH TWO DOTS" },
    { 0x065F, "ARABIC WAVY HAMZA BELOW" },
    { 0x0660, "ARABIC-INDIC DIGIT ZERO" },
    { 0x0661, "ARABIC-INDIC DIGIT ONE" },
    { 0x0662, "ARABIC-INDIC DIGIT TWO" },
    { 0x0663, "ARABIC-INDIC DIGIT THREE" },
    { 0x0664, "ARABIC-INDIC DIGIT FOUR" },
    { 0x0665, "ARABIC-INDIC DIGIT FIVE" },
    { 0x0666, "ARABIC-INDIC DIGIT SIX" },
    { 0x0667, "ARABIC-INDIC DIGIT SEVEN" },
    { 0x0668, "ARABIC-INDIC DIGIT EIGHT" },
    { 0x0669, "ARABIC-INDIC DIGIT NINE" },
    { 0x066A, "ARABIC PERCENT SIGN" },
    { 0x066B, "ARABIC DECIMAL SEPARATOR" },
    { 0x066C, "ARABIC THOUSANDS SEPARATOR" },
    { 0x066D, "ARABIC FIVE POINTED STAR" },
    { 0x066E, "ARABIC LETTER DOTLESS BEH" },
    { 0x066F, "ARABIC LETTER DOTLESS QAF" },
    { 0x0670, "ARABIC LETTER SUPERSCRIPT ALEF" },
    { 0x0671, "ARABIC LETTER ALEF WASLA" },
    { 0x0672, "ARABIC LETTER ALEF WITH WAVY HAMZA ABOVE" },
    { 0x0673, "ARABIC LETTER ALEF WITH WAVY HAMZA BELOW" },
    { 0x0674, "ARABIC LETTER HIGH HAMZA" },
    { 0x0675, "ARABIC LETTER HIGH HAMZA ALEF" },
    { 0x0676, "ARABIC LETTER HIGH HAMZA WAW" },
    { 0x0677, "ARABIC LETTER U WITH HAMZA ABOVE" },
    { 0x0678, "ARABIC LETTER HIGH HAMZA YEH" },
    { 0x0679, "ARABIC LETTER TTEH" },
    { 0x067A, "ARABIC LETTER TTEHEH" },
    { 0x067B, "ARABIC LETTER BEEH" },
    { 0x067C, "ARABIC LETTER TEH WITH RING" },
    { 0x067D, "ARABIC LETTER TEH WITH THREE DOTS ABOVE DOWNWARDS" },
    { 0x067E, "ARABIC LETTER PEH" },
    { 0x067F, "ARABIC LETTER TEHEH" },
    { 0x0680, "ARABIC LETTER BEHEH" },
    { 0x0681, "ARABIC LETTER HAH WITH HAMZA ABOVE" },
    { 0x0682, "ARABIC LETTER HAH WITH TWO DOTS VERTICAL ABOVE" },
    { 0x0683, "ARABIC LETTER NYEH" },
    { 0x0684, "ARABIC LETTER DYEH" },
    { 0x0685, "ARABIC LETTER HAH WITH THREE DOTS ABOVE" },
    { 0x0686, "ARABIC LETTER TCHEH" },
    { 0x0687, "ARABIC LETTER TCHEHEH" },
    { 0x0688, "ARABIC LETTER DDAL" },
    { 0x0689, "ARABIC LETTER DAL WITH RING" },
    { 0x068A, "ARABIC LETTER DAL WITH DOT BELOW" },
    { 0x068B, "ARABIC LETTER DAL WITH DOT BELOW AND SMALL TAH" },
    { 0x068C, "ARABIC LETTER DAHAL" },
    { 0x068D, "ARABIC LETTER DDAHAL" },
    { 0x068E, "ARABIC LETTER DUL" },
    { 0x068F, "ARABIC LETTER DAL WITH THREE DOTS ABOVE DOWNWARDS" },
    { 0x0690, "ARABIC LETTER DAL WITH FOUR DOTS ABOVE" },
    { 0x0691, "ARABIC LETTER RREH" },
    { 0x0692, "ARABIC LETTER REH WITH SMALL V" },
    { 0x0693, "ARABIC LETTER REH WITH RING" },
    { 0x0694, "ARABIC LETTER REH WITH DOT BELOW" },
    { 0x0695, "ARABIC LETTER REH WITH SMALL V BELOW" },
    { 0x0696, "ARABIC LETTER REH WITH DOT BELOW AND DOT ABOVE" },
    { 0x0697, "ARABIC LETTER REH WITH TWO DOTS ABOVE" },
    { 0x0698, "ARABIC LETTER JEH" },
    { 0x0699, "ARABIC LETTER REH WITH FOUR DOTS ABOVE" },
    { 0x069A, "ARABIC LETTER SEEN WITH DOT BELOW AND DOT ABOVE" },
    { 0x069B, "ARABIC LETTER SEEN WITH THREE DOTS BELOW" },
    { 0x069C, "ARABIC LETTER SEEN WITH THREE DOTS BELOW AND THREE DOTS ABOVE" },
    { 0x069D, "ARABIC LETTER SAD WITH TWO DOTS BELOW" },
    { 0x069E, "ARABIC LETTER SAD WITH THREE DOTS ABOVE" },
    { 0x069F, "ARABIC LETTER TAH WITH THREE DOTS ABOVE" },
    { 0x06A0, "ARABIC LETTER AIN WITH THREE DOTS ABOVE" },
    { 0x06A1, "ARABIC LETTER DOTLESS FEH" },
    { 0x06A2, "ARABIC LETTER FEH WITH DOT MOVED BELOW" },
    { 0x06A3, "ARABIC LETTER FEH WITH DOT BELOW" },
    { 0x06A4, "ARABIC LETTER VEH" },
    { 0x06A5, "ARABIC LETTER FEH WITH THREE DOTS BELOW" },
    { 0x06A6, "ARABIC LETTER PEHEH" },
    { 0x06A7, "ARABIC LETTER QAF WITH DOT ABOVE" },
    { 0x06A8, "ARABIC LETTER QAF WITH THREE DOTS ABOVE" },
    { 0x06A9, "ARABIC LETTER KEHEH" },
    { 0x06AA, "ARABIC LETTER SWASH KAF" },
    { 0x06AB, "ARABIC LETTER KAF WITH RING" },
    { 0x06AC, "ARABIC LETTER KAF WITH DOT ABOVE" },
    { 0x06AD, "ARABIC LETTER NG" },
    { 0x06AE, "ARABIC LETTER KAF WITH THREE DOTS BELOW" },
    { 0x06AF, "ARABIC LETTER GAF" },
    { 0x06B0, "ARABIC LETTER GAF WITH RING" },
    { 0x06B1, "ARABIC LETTER NGOEH" },
    { 0x06B2, "ARABIC LETTER GAF WITH TWO DOTS BELOW" },
    { 0x06B3, "ARABIC LETTER GUEH" },
    { 0x06B4, "ARABIC LETTER GAF WITH THREE DOTS ABOVE" },
    { 0x06B5, "ARABIC LETTER LAM WITH SMALL V" },
    { 0x06B6, "ARABIC LETTER LAM WITH DOT ABOVE" },
    { 0x06B7, "ARABIC LETTER LAM WITH THREE DOTS ABOVE" },
    { 0x06B8, "ARABIC LETTER LAM WITH THREE DOTS BELOW" },
    { 0x06B9, "ARABIC LETTER NOON WITH DOT BELOW" },
    { 0x06BA, "ARABIC LETTER NOON GHUNNA" },
    { 0x06BB, "ARABIC LETTER RNOON" },
    { 0x06BC, "ARABIC LETTER NOON WITH RING" },
    { 0x06BD, "ARABIC LETTER NOON WITH THREE DOTS ABOVE" },
    { 0x06BE, "ARABIC LETTER HEH DOACHASHMEE" },
    { 0x06BF, "ARABIC LETTER TCHEH WITH DOT ABOVE" },
    { 0x06C0, "ARABIC LETTER HEH WITH YEH ABOVE" },
    { 0x06C1, "ARABIC LETTER HEH GOAL" },
    { 0x06C2, "ARABIC LETTER HEH GOAL WITH HAMZA ABOVE" },
    { 0x06C3, "ARABIC LETTER TEH MARBUTA GOAL" },
    { 0x06C4, "ARABIC LETTER WAW WITH RING" },
    { 0x06C5, "ARABIC LETTER KIRGHIZ OE" },
    { 0x06C6, "ARABIC LETTER OE" },
    { 0x06C7, "ARABIC LETTER U" },
    { 0x06C8, "ARABIC LETTER YU" },
    { 0x06C9, "ARABIC LETTER KIRGHIZ YU" },
    { 0x06CA, "ARABIC LETTER WAW WITH TWO DOTS ABOVE" },
    { 0x06CB, "ARABIC LETTER VE" },
    { 0x06CC, "ARABIC LETTER FARSI YEH" },
    { 0x06CD, "ARABIC LETTER YEH WITH TAIL" },
    { 0x06CE, "ARABIC LETTER YEH WITH SMALL V" },
    { 0x06CF, "ARABIC LETTER WAW WITH DOT ABOVE" },
    { 0x06D0, "ARABIC LETTER E" },
    { 0x06D1, "ARABIC LETTER YEH WITH THREE DOTS BELOW" },
    { 0x06D2, "ARABIC LETTER YEH BARREE" },
    { 0x06D3, "ARABIC LETTER YEH BARREE WITH HAMZA ABOVE" },
    { 0x06D4, "ARABIC FULL STOP" },
    { 0x06D5, "ARABIC LETTER AE" },
    { 0x06D6, "ARABIC SMALL HIGH LIGATURE SAD WITH LAM WITH ALEF MAKSURA" },
    { 0x06D7, "ARABIC SMALL HIGH LIGATURE QAF WITH LAM WITH ALEF MAKSURA" },
    { 0x06D8, "ARABIC SMALL HIGH MEEM INITIAL FORM" },
    { 0x06D9, "ARABIC SMALL HIGH LAM ALEF" },
    { 0x06DA, "ARABIC SMALL HIGH JEEM" },
    { 0x06DB, "ARABIC SMALL HIGH THREE DOTS" },
    { 0x06DC, "ARABIC SMALL HIGH SEEN" },
    { 0x06DD, "ARABIC END OF AYAH" },
    { 0x06DE, "ARABIC START OF RUB EL HIZB" },
    { 0x06DF, "ARABIC SMALL HIGH ROUNDED ZERO" },
    { 0x06E0, "ARABIC SMALL HIGH UPRIGHT RECTANGULAR ZERO" },
    { 0x06E1, "ARABIC SMALL HIGH DOTLESS HEAD OF KHAH" },
    { 0x06E2, "ARABIC SMALL HIGH MEEM ISOLATED FORM" },
    { 0x06E3, "ARABIC SMALL LOW SEEN" },
    { 0x06E4, "ARABIC SMALL HIGH MADDA" },
    { 0x06E5, "ARABIC SMALL WAW" },
    { 0x06E6, "ARABIC SMALL YEH" },
    { 0x06E7, "ARABIC SMALL HIGH YEH" },
    { 0x06E8, "ARABIC SMALL HIGH NOON" },
    { 0x06E9, "ARABIC PLACE OF SAJDAH" },
    { 0x06EA, "ARABIC EMPTY CENTRE LOW STOP" },
    { 0x06EB, "ARABIC EMPTY CENTRE HIGH STOP" },
    { 0x06EC, "ARABIC ROUNDED HIGH STOP WITH FILLED CENTRE" },
    { 0x06ED, "ARABIC SMALL LOW MEEM" },
    { 0x06EE, "ARABIC LETTER DAL WITH INVERTED V" },
    { 0x06EF, "ARABIC LETTER REH WITH INVERTED V" },
    { 0x06F0, "EXTENDED ARABIC-INDIC DIGIT ZERO" },
    { 0x06F1, "EXTENDED ARABIC-INDIC DIGIT ONE" },
    { 0x06F2, "EXTENDED ARABIC-INDIC DIGIT TWO" },
    { 0x06F3, "EXTENDED ARABIC-INDIC DIGIT THREE" },
    { 0x06F4, "EXTENDED ARABIC-INDIC DIGIT FOUR" },
    { 0x06F5, "EXTENDED ARABIC-INDIC DIGIT FIVE" },
    { 0x06F6, "EXTENDED ARABIC-INDIC DIGIT SIX" },
    { 0x06F7, "EXTENDED ARABIC-INDIC DIGIT SEVEN" },
    { 0x06F8, "EXTENDED ARABIC-INDIC DIGIT EIGHT" },
    { 0x06F9, "EXTENDED ARABIC-INDIC DIGIT NINE" },
    { 0x06FA, "ARABIC LETTER SHEEN WITH DOT BELOW" },
    { 0x06FB, "ARABIC LETTER DAD WITH DOT BELOW" },
    { 0x06FC, "ARABIC LETTER GHAIN WITH DOT BELOW" },
    { 0x06FD, "ARABIC SIGN SINDHI AMPERSAND" },
    { 0x06FE, "ARABIC SIGN SINDHI POSTPOSITION MEN" },
    { 0x06FF, "ARABIC LETTER HEH WITH INVERTED V" },
    { 0x0700, "SYRIAC END OF PARAGRAPH" },
    { 0x0701, "SYRIAC SUPRALINEAR FULL STOP" },
    { 0x0702, "SYRIAC SUBLINEAR FULL STOP" },
    { 0x0703, "SYRIAC SUPRALINEAR COLON" },
    { 0x0704, "SYRIAC SUBLINEAR COLON" },
    { 0x0705, "SYRIAC HORIZONTAL COLON" },
    { 0x0706, "SYRIAC COLON SKEWED LEFT" },
    { 0x0707, "SYRIAC COLON SKEWED RIGHT" },
    { 0x0708, "SYRIAC SUPRALINEAR COLON SKEWED LEFT" },
    { 0x0709, "SYRIAC SUBLINEAR COLON SKEWED RIGHT" },
    { 0x070A, "SYRIAC CONTRACTION" },
    { 0x070B, "SYRIAC HARKLEAN OBELUS" },
    { 0x070C, "SYRIAC HARKLEAN METOBELUS" },
    { 0x070D, "SYRIAC HARKLEAN ASTERISCUS" },
    { 0x070F, "SYRIAC ABBREVIATION MARK" },
    { 0x0710, "SYRIAC LETTER ALAPH" },
    { 0x0711, "SYRIAC LETTER SUPERSCRIPT ALAPH" },
    { 0x0712, "SYRIAC LETTER BETH" },
    { 0x0713, "SYRIAC LETTER GAMAL" },
    { 0x0714, "SYRIAC LETTER GAMAL GARSHUNI" },
    { 0x0715, "SYRIAC LETTER DALATH" },
    { 0x0716, "SYRIAC LETTER DOTLESS DALATH RISH" },
    { 0x0717, "SYRIAC LETTER HE" },
    { 0x0718, "SYRIAC LETTER WAW" },
    { 0x0719, "SYRIAC LETTER ZAIN" },
    { 0x071A, "SYRIAC LETTER HETH" },
    { 0x071B, "SYRIAC LETTER TETH" },
    { 0x071C, "SYRIAC LETTER TETH GARSHUNI" },
    { 0x071D, "SYRIAC LETTER YUDH" },
    { 0x071E, "SYRIAC LETTER YUDH HE" },
    { 0x071F, "SYRIAC LETTER KAPH" },
    { 0x0720, "SYRIAC LETTER LAMADH" },
    { 0x0721, "SYRIAC LETTER MIM" },
    { 0x0722, "SYRIAC LETTER NUN" },
    { 0x0723, "SYRIAC LETTER SEMKATH" },
    { 0x0724, "SYRIAC LETTER FINAL SEMKATH" },
    { 0x0725, "SYRIAC LETTER E" },
    { 0x0726, "SYRIAC LETTER PE" },
    { 0x0727, "SYRIAC LETTER REVERSED PE" },
    { 0x0728, "SYRIAC LETTER SADHE" },
    { 0x0729, "SYRIAC LETTER QAPH" },
    { 0x072A, "SYRIAC LETTER RISH" },
    { 0x072B, "SYRIAC LETTER SHIN" },
    { 0x072C, "SYRIAC LETTER TAW" },
    { 0x072D, "SYRIAC LETTER PERSIAN BHETH" },
    { 0x072E, "SYRIAC LETTER PERSIAN GHAMAL" },
    { 0x072F, "SYRIAC LETTER PERSIAN DHALATH" },
    { 0x0730, "SYRIAC PTHAHA ABOVE" },
    { 0x0731, "SYRIAC PTHAHA BELOW" },
    { 0x0732, "SYRIAC PTHAHA DOTTED" },
    { 0x0733, "SYRIAC ZQAPHA ABOVE" },
    { 0x0734, "SYRIAC ZQAPHA BELOW" },
    { 0x0735, "SYRIAC ZQAPHA DOTTED" },
    { 0x0736, "SYRIAC RBASA ABOVE" },
    { 0x0737, "SYRIAC RBASA BELOW" },
    { 0x0738, "SYRIAC DOTTED ZLAMA HORIZONTAL" },
    { 0x0739, "SYRIAC DOTTED ZLAMA ANGULAR" },
    { 0x073A, "SYRIAC HBASA ABOVE" },
    { 0x073B, "SYRIAC HBASA BELOW" },
    { 0x073C, "SYRIAC HBASA-ESASA DOTTED" },
    { 0x073D, "SYRIAC ESASA ABOVE" },
    { 0x073E, "SYRIAC ESASA BELOW" },
    { 0x073F, "SYRIAC RWAHA" },
    { 0x0740, "SYRIAC FEMININE DOT" },
    { 0x0741, "SYRIAC QUSHSHAYA" },
    { 0x0742, "SYRIAC RUKKAKHA" },
    { 0x0743, "SYRIAC TWO VERTICAL DOTS ABOVE" },
    { 0x0744, "SYRIAC TWO VERTICAL DOTS BELOW" },
    { 0x0745, "SYRIAC THREE DOTS ABOVE" },
    { 0x0746, "SYRIAC THREE DOTS BELOW" },
    { 0x0747, "SYRIAC OBLIQUE LINE ABOVE" },
    { 0x0748, "SYRIAC OBLIQUE LINE BELOW" },
    { 0x0749, "SYRIAC MUSIC" },
    { 0x074A, "SYRIAC BARREKH" },
    { 0x074D, "SYRIAC LETTER SOGDIAN ZHAIN" },
    { 0x074E, "SYRIAC LETTER SOGDIAN KHAPH" },
    { 0x074F, "SYRIAC LETTER SOGDIAN FE" },
    { 0x0750, "ARABIC LETTER BEH WITH THREE DOTS HORIZONTALLY BELOW" },
    { 0x0751, "ARABIC LETTER BEH WITH DOT BELOW AND THREE DOTS ABOVE" },
    { 0x0752, "ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW" },
    { 0x0753, "ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW AND TWO DOTS ABOVE" },
    { 0x0754, "ARABIC LETTER BEH WITH TWO DOTS BELOW AND DOT ABOVE" },
    { 0x0755, "ARABIC LETTER BEH WITH INVERTED SMALL V BELOW" },
    { 0x0756, "ARABIC LETTER BEH WITH SMALL V" },
    { 0x0757, "ARABIC LETTER HAH WITH TWO DOTS ABOVE" },
    { 0x0758, "ARABIC LETTER HAH WITH THREE DOTS POINTING UPWARDS BELOW" },
    { 0x0759, "ARABIC LETTER DAL WITH TWO DOTS VERTICALLY BELOW AND SMALL TAH" },
    { 0x075A, "ARABIC LETTER DAL WITH INVERTED SMALL V BELOW" },
    { 0x075B, "ARABIC LETTER REH WITH STROKE" },
    { 0x075C, "ARABIC LETTER SEEN WITH FOUR DOTS ABOVE" },
    { 0x075D, "ARABIC LETTER AIN WITH TWO DOTS ABOVE" },
    { 0x075E, "ARABIC LETTER AIN WITH THREE DOTS POINTING DOWNWARDS ABOVE" },
    { 0x075F, "ARABIC LETTER AIN WITH TWO DOTS VERTICALLY ABOVE" },
    { 0x0760, "ARABIC LETTER FEH WITH TWO DOTS BELOW" },
    { 0x0761, "ARABIC LETTER FEH WITH THREE DOTS POINTING UPWARDS BELOW" },
    { 0x0762, "ARABIC LETTER KEHEH WITH DOT ABOVE" },
    { 0x0763, "ARABIC LETTER KEHEH WITH THREE DOTS ABOVE" },
    { 0x0764, "ARABIC LETTER KEHEH WITH THREE DOTS POINTING UPWARDS BELOW" },
    { 0x0765, "ARABIC LETTER MEEM WITH DOT ABOVE" },
    { 0x0766, "ARABIC LETTER MEEM WITH DOT BELOW" },
    { 0x0767, "ARABIC LETTER NOON WITH TWO DOTS BELOW" },
    { 0x0768, "ARABIC LETTER NOON WITH SMALL TAH" },
    { 0x0769, "ARABIC LETTER NOON WITH SMALL V" },
    { 0x076A, "ARABIC LETTER LAM WITH BAR" },
    { 0x076B, "ARABIC LETTER REH WITH TWO DOTS VERTICALLY ABOVE" },
    { 0x076C, "ARABIC LETTER REH WITH HAMZA ABOVE" },
    { 0x076D, "ARABIC LETTER SEEN WITH TWO DOTS VERTICALLY ABOVE" },
    { 0x076E, "ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH BELOW" },
    { 0x076F, "ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH AND TWO DOTS" },
    { 0x0770, "ARABIC LETTER SEEN WITH SMALL ARABIC LETTER TAH AND TWO DOTS" },
    { 0x0771, "ARABIC LETTER REH WITH SMALL ARABIC LETTER TAH AND TWO DOTS" },
    { 0x0772, "ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH ABOVE" },
    { 0x0773, "ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE" },
    { 0x0774, "ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE" },
    { 0x0775, "ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE" },
    { 0x0776, "ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE" },
    { 0x0777, "ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW" },
    { 0x0778, "ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE" },
    { 0x0779, "ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE" },
    { 0x077A, "ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE" },
    { 0x077B, "ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE" },
    { 0x077C, "ARABIC LETTER HAH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW" },
    { 0x077D, "ARABIC LETTER SEEN WITH EXTENDED ARABIC-INDIC DIGIT FOUR ABOVE" },
    { 0x077E, "ARABIC LETTER SEEN WITH INVERTED V" },
    { 0x077F, "ARABIC LETTER KAF WITH TWO DOTS ABOVE" },
    { 0x0780, "THAANA LETTER HAA" },
    { 0x0781, "THAANA LETTER SHAVIYANI" },
    { 0x0782, "THAANA LETTER NOONU" },
    { 0x0783, "THAANA LETTER RAA" },
    { 0x0784, "THAANA LETTER BAA" },
    { 0x0785, "THAANA LETTER LHAVIYANI" },
    { 0x0786, "THAANA LETTER KAAFU" },
    { 0x0787, "THAANA LETTER ALIFU" },
    { 0x0788, "THAANA LETTER VAAVU" },
    { 0x0789, "THAANA LETTER MEEMU" },
    { 0x078A, "THAANA LETTER FAAFU" },
    { 0x078B, "THAANA LETTER DHAALU" },
    { 0x078C, "THAANA LETTER THAA" },
    { 0x078D, "THAANA LETTER LAAMU" },
    { 0x078E, "THAANA LETTER GAAFU" },
    { 0x078F, "THAANA LETTER GNAVIYANI" },
    { 0x0790, "THAANA LETTER SEENU" },
    { 0x0791, "THAANA LETTER DAVIYANI" },
    { 0x0792, "THAANA LETTER ZAVIYANI" },
    { 0x0793, "THAANA LETTER TAVIYANI" },
    { 0x0794, "THAANA LETTER YAA" },
    { 0x0795, "THAANA LETTER PAVIYANI" },
    { 0x0796, "THAANA LETTER JAVIYANI" },
    { 0x0797, "THAANA LETTER CHAVIYANI" },
    { 0x0798, "THAANA LETTER TTAA" },
    { 0x0799, "THAANA LETTER HHAA" },
    { 0x079A, "THAANA LETTER KHAA" },
    { 0x079B, "THAANA LETTER THAALU" },
    { 0x079C, "THAANA LETTER ZAA" },
    { 0x079D, "THAANA LETTER SHEENU" },
    { 0x079E, "THAANA LETTER SAADHU" },
    { 0x079F, "THAANA LETTER DAADHU" },
    { 0x07A0, "THAANA LETTER TO" },
    { 0x07A1, "THAANA LETTER ZO" },
    { 0x07A2, "THAANA LETTER AINU" },
    { 0x07A3, "THAANA LETTER GHAINU" },
    { 0x07A4, "THAANA LETTER QAAFU" },
    { 0x07A5, "THAANA LETTER WAAVU" },
    { 0x07A6, "THAANA ABAFILI" },
    { 0x07A7, "THAANA AABAAFILI" },
    { 0x07A8, "THAANA IBIFILI" },
    { 0x07A9, "THAANA EEBEEFILI" },
    { 0x07AA, "THAANA UBUFILI" },
    { 0x07AB, "THAANA OOBOOFILI" },
    { 0x07AC, "THAANA EBEFILI" },
    { 0x07AD, "THAANA EYBEYFILI" },
    { 0x07AE, "THAANA OBOFILI" },
    { 0x07AF, "THAANA OABOAFILI" },
    { 0x07B0, "THAANA SUKUN" },
    { 0x07B1, "THAANA LETTER NAA" },
    { 0x07C0, "NKO DIGIT ZERO" },
    { 0x07C1, "NKO DIGIT ONE" },
    { 0x07C2, "NKO DIGIT TWO" },
    { 0x07C3, "NKO DIGIT THREE" },
    { 0x07C4, "NKO DIGIT FOUR" },
    { 0x07C5, "NKO DIGIT FIVE" },
    { 0x07C6, "NKO DIGIT SIX" },
    { 0x07C7, "NKO DIGIT SEVEN" },
    { 0x07C8, "NKO DIGIT EIGHT" },
    { 0x07C9, "NKO DIGIT NINE" },
    { 0x07CA, "NKO LETTER A" },
    { 0x07CB, "NKO LETTER EE" },
    { 0x07CC, "NKO LETTER I" },
    { 0x07CD, "NKO LETTER E" },
    { 0x07CE, "NKO LETTER U" },
    { 0x07CF, "NKO LETTER OO" },
    { 0x07D0, "NKO LETTER O" },
    { 0x07D1, "NKO LETTER DAGBASINNA" },
    { 0x07D2, "NKO LETTER N" },
    { 0x07D3, "NKO LETTER BA" },
    { 0x07D4, "NKO LETTER PA" },
    { 0x07D5, "NKO LETTER TA" },
    { 0x07D6, "NKO LETTER JA" },
    { 0x07D7, "NKO LETTER CHA" },
    { 0x07D8, "NKO LETTER DA" },
    { 0x07D9, "NKO LETTER RA" },
    { 0x07DA, "NKO LETTER RRA" },
    { 0x07DB, "NKO LETTER SA" },
    { 0x07DC, "NKO LETTER GBA" },
    { 0x07DD, "NKO LETTER FA" },
    { 0x07DE, "NKO LETTER KA" },
    { 0x07DF, "NKO LETTER LA" },
    { 0x07E0, "NKO LETTER NA WOLOSO" },
    { 0x07E1, "NKO LETTER MA" },
    { 0x07E2, "NKO LETTER NYA" },
    { 0x07E3, "NKO LETTER NA" },
    { 0x07E4, "NKO LETTER HA" },
    { 0x07E5, "NKO LETTER WA" },
    { 0x07E6, "NKO LETTER YA" },
    { 0x07E7, "NKO LETTER NYA WOLOSO" },
    { 0x07E8, "NKO LETTER JONA JA" },
    { 0x07E9, "NKO LETTER JONA CHA" },
    { 0x07EA, "NKO LETTER JONA RA" },
    { 0x07EB, "NKO COMBINING SHORT HIGH TONE" },
    { 0x07EC, "NKO COMBINING SHORT LOW TONE" },
    { 0x07ED, "NKO COMBINING SHORT RISING TONE" },
    { 0x07EE, "NKO COMBINING LONG DESCENDING TONE" },
    { 0x07EF, "NKO COMBINING LONG HIGH TONE" },
    { 0x07F0, "NKO COMBINING LONG LOW TONE" },
    { 0x07F1, "NKO COMBINING LONG RISING TONE" },
    { 0x07F2, "NKO COMBINING NASALIZATION MARK" },
    { 0x07F3, "NKO COMBINING DOUBLE DOT ABOVE" },
    { 0x07F4, "NKO HIGH TONE APOSTROPHE" },
    { 0x07F5, "NKO LOW TONE APOSTROPHE" },
    { 0x07F6, "NKO SYMBOL OO DENNEN" },
    { 0x07F7, "NKO SYMBOL GBAKURUNEN" },
    { 0x07F8, "NKO COMMA" },
    { 0x07F9, "NKO EXCLAMATION MARK" },
    { 0x07FA, "NKO LAJANYALAN" },
    { 0x07FD, "NKO DANTAYALAN" },
    { 0x07FE, "NKO DOROME SIGN" },
    { 0x07FF, "NKO TAMAN SIGN" },
    { 0x0800, "SAMARITAN LETTER ALAF" },
    { 0x0801, "SAMARITAN LETTER BIT" },
    { 0x0802, "SAMARITAN LETTER GAMAN" },
    { 0x0803, "SAMARITAN LETTER DALAT" },
    { 0x0804, "SAMARITAN LETTER IY" },
    { 0x0805, "SAMARITAN LETTER BAA" },
    { 0x0806, "SAMARITAN LETTER ZEN" },
    { 0x0807, "SAMARITAN LETTER IT" },
    { 0x0808, "SAMARITAN LETTER TIT" },
    { 0x0809, "SAMARITAN LETTER YUT" },
    { 0x080A, "SAMARITAN LETTER KAAF" },
    { 0x080B, "SAMARITAN LETTER LABAT" },
    { 0x080C, "SAMARITAN LETTER MIM" },
    { 0x080D, "SAMARITAN LETTER NUN" },
    { 0x080E, "SAMARITAN LETTER SINGAAT" },
    { 0x080F, "SAMARITAN LETTER IN" },
    { 0x0810, "SAMARITAN LETTER FI" },
    { 0x0811, "SAMARITAN LETTER TSAADIY" },
    { 0x0812, "SAMARITAN LETTER QUF" },
    { 0x0813, "SAMARITAN LETTER RISH" },
    { 0x0814, "SAMARITAN LETTER SHAN" },
    { 0x0815, "SAMARITAN LETTER TAAF" },
    { 0x0816, "SAMARITAN MARK IN" },
    { 0x0817, "SAMARITAN MARK IN-ALAF" },
    { 0x0818, "SAMARITAN MARK OCCLUSION" },
    { 0x0819, "SAMARITAN MARK DAGESH" },
    { 0x081A, "SAMARITAN MODIFIER LETTER EPENTHETIC YUT" },
    { 0x081B, "SAMARITAN MARK EPENTHETIC YUT" },
    { 0x081C, "SAMARITAN VOWEL SIGN LONG E" },
    { 0x081D, "SAMARITAN VOWEL SIGN E" },
    { 0x081E, "SAMARITAN VOWEL SIGN OVERLONG AA" },
    { 0x081F, "SAMARITAN VOWEL SIGN LONG AA" },
    { 0x0820, "SAMARITAN VOWEL SIGN AA" },
    { 0x0821, "SAMARITAN VOWEL SIGN OVERLONG A" },
    { 0x0822, "SAMARITAN VOWEL SIGN LONG A" },
    { 0x0823, "SAMARITAN VOWEL SIGN A" },
    { 0x0824, "SAMARITAN MODIFIER LETTER SHORT A" },
    { 0x0825, "SAMARITAN VOWEL SIGN SHORT A" },
    { 0x0826, "SAMARITAN VOWEL SIGN LONG U" },
    { 0x0827, "SAMARITAN VOWEL SIGN U" },
    { 0x0828, "SAMARITAN MODIFIER LETTER I" },
    { 0x0829, "SAMARITAN VOWEL SIGN LONG I" },
    { 0x082A, "SAMARITAN VOWEL SIGN I" },
    { 0x082B, "SAMARITAN VOWEL SIGN O" },
    { 0x082C, "SAMARITAN VOWEL SIGN SUKUN" },
    { 0x082D, "SAMARITAN MARK NEQUDAA" },
    { 0x0830, "SAMARITAN PUNCTUATION NEQUDAA" },
    { 0x0831, "SAMARITAN PUNCTUATION AFSAAQ" },
    { 0x0832, "SAMARITAN PUNCTUATION ANGED" },
    { 0x0833, "SAMARITAN PUNCTUATION BAU" },
    { 0x0834, "SAMARITAN PUNCTUATION ATMAAU" },
    { 0x0835, "SAMARITAN PUNCTUATION SHIYYAALAA" },
    { 0x0836, "SAMARITAN ABBREVIATION MARK" },
    { 0x0837, "SAMARITAN PUNCTUATION MELODIC QITSA" },
    { 0x0838, "SAMARITAN PUNCTUATION ZIQAA" },
    { 0x0839, "SAMARITAN PUNCTUATION QITSA" },
    { 0x083A, "SAMARITAN PUNCTUATION ZAEF" },
    { 0x083B, "SAMARITAN PUNCTUATION TURU" },
    { 0x083C, "SAMARITAN PUNCTUATION ARKAANU" },
    { 0x083D, "SAMARITAN PUNCTUATION SOF MASHFAAT" },
    { 0x083E, "SAMARITAN PUNCTUATION ANNAAU" },
    { 0x0840, "MANDAIC LETTER HALQA" },
    { 0x0841, "MANDAIC LETTER AB" },
    { 0x0842, "MANDAIC LETTER AG" },
    { 0x0843, "MANDAIC LETTER AD" },
    { 0x0844, "MANDAIC LETTER AH" },
    { 0x0845, "MANDAIC LETTER USHENNA" },
    { 0x0846, "MANDAIC LETTER AZ" },
    { 0x0847, "MANDAIC LETTER IT" },
    { 0x0848, "MANDAIC LETTER ATT" },
    { 0x0849, "MANDAIC LETTER AKSA" },
    { 0x084A, "MANDAIC LETTER AK" },
    { 0x084B, "MANDAIC LETTER AL" },
    { 0x084C, "MANDAIC LETTER AM" },
    { 0x084D, "MANDAIC LETTER AN" },
    { 0x084E, "MANDAIC LETTER AS" },
    { 0x084F, "MANDAIC LETTER IN" },
    { 0x0850, "MANDAIC LETTER AP" },
    { 0x0851, "MANDAIC LETTER ASZ" },
    { 0x0852, "MANDAIC LETTER AQ" },
    { 0x0853, "MANDAIC LETTER AR" },
    { 0x0854, "MANDAIC LETTER ASH" },
    { 0x0855, "MANDAIC LETTER AT" },
    { 0x0856, "MANDAIC LETTER DUSHENNA" },
    { 0x0857, "MANDAIC LETTER KAD" },
    { 0x0858, "MANDAIC LETTER AIN" },
    { 0x0859, "MANDAIC AFFRICATION MARK" },
    { 0x085A, "MANDAIC VOCALIZATION MARK" },
    { 0x085B, "MANDAIC GEMINATION MARK" },
    { 0x085E, "MANDAIC PUNCTUATION" },
    { 0x0860, "SYRIAC LETTER MALAYALAM NGA" },
    { 0x0861, "SYRIAC LETTER MALAYALAM JA" },
    { 0x0862, "SYRIAC LETTER MALAYALAM NYA" },
    { 0x0863, "SYRIAC LETTER MALAYALAM TTA" },
    { 0x0864, "SYRIAC LETTER MALAYALAM NNA" },
    { 0x0865, "SYRIAC LETTER MALAYALAM NNNA" },
    { 0x0866, "SYRIAC LETTER MALAYALAM BHA" },
    { 0x0867, "SYRIAC LETTER MALAYALAM RA" },
    { 0x0868, "SYRIAC LETTER MALAYALAM LLA" },
    { 0x0869, "SYRIAC LETTER MALAYALAM LLLA" },
    { 0x086A, "SYRIAC LETTER MALAYALAM SSA" },
    { 0x0870, "ARABIC LETTER ALEF WITH ATTACHED FATHA" },
    { 0x0871, "ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA" },
    { 0x0872, "ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE" },
    { 0x0873, "ARABIC LETTER ALEF WITH LEFT MIDDLE STROKE" },
    { 0x0874, "ARABIC LETTER ALEF WITH ATTACHED KASRA" },
    { 0x0875, "ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA" },
    { 0x0876, "ARABIC LETTER ALEF WITH ATTACHED ROUND DOT ABOVE" },
    { 0x0877, "ARABIC LETTER ALEF WITH ATTACHED RIGHT ROUND DOT" },
    { 0x0878, "ARABIC LETTER ALEF WITH ATTACHED LEFT ROUND DOT" },
    { 0x0879, "ARABIC LETTER ALEF WITH ATTACHED ROUND DOT BELOW" },
    { 0x087A, "ARABIC LETTER ALEF WITH DOT ABOVE" },
    { 0x087B, "ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA AND DOT ABOVE" },
    { 0x087C, "ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE AND DOT ABOVE" },
    { 0x087D, "ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA AND DOT ABOVE" },
    { 0x087E, "ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA AND LEFT RING" },
    { 0x087F, "ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE AND LEFT RING" },
    { 0x0880, "ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA AND LEFT RING" },
    { 0x0881, "ARABIC LETTER ALEF WITH ATTACHED RIGHT HAMZA" },
    { 0x0882, "ARABIC LETTER ALEF WITH ATTACHED LEFT HAMZA" },
    { 0x0883, "ARABIC TATWEEL WITH OVERSTRUCK HAMZA" },
    { 0x0884, "ARABIC TATWEEL WITH OVERSTRUCK WAW" },
    { 0x0885, "ARABIC TATWEEL WITH TWO DOTS BELOW" },
    { 0x0886, "ARABIC LETTER THIN YEH" },
    { 0x0887, "ARABIC BASELINE ROUND DOT" },
    { 0x0888, "ARABIC RAISED ROUND DOT" },
    { 0x0889, "ARABIC LETTER NOON WITH INVERTED SMALL V" },
    { 0x088A, "ARABIC LETTER HAH WITH INVERTED SMALL V BELOW" },
    { 0x088B, "ARABIC LETTER TAH WITH DOT BELOW" },
    { 0x088C, "ARABIC LETTER TAH WITH THREE DOTS BELOW" },
    { 0x088D, "ARABIC LETTER KEHEH WITH TWO DOTS VERTICALLY BELOW" },
    { 0x088E, "ARABIC VERTICAL TAIL" },
    { 0x0890, "ARABIC POUND MARK ABOVE" },
    { 0x0891, "ARABIC PIASTRE MARK ABOVE" },
    { 0x0898, "ARABIC SMALL HIGH WORD AL-JUZ" },
    { 0x0899, "ARABIC SMALL LOW WORD ISHMAAM" },
    { 0x089A, "ARABIC SMALL LOW WORD IMAALA" },
    { 0x089B, "ARABIC SMALL LOW WORD TASHEEL" },
    { 0x089C, "ARABIC MADDA WAAJIB" },
    { 0x089D, "ARABIC SUPERSCRIPT ALEF MOKHASSAS" },
    { 0x089E, "ARABIC DOUBLED MADDA" },
    { 0x089F, "ARABIC HALF MADDA OVER MADDA" },
    { 0x08A0, "ARABIC LETTER BEH WITH SMALL V BELOW" },
    { 0x08A1, "ARABIC LETTER BEH WITH HAMZA ABOVE" },
    { 0x08A2, "ARABIC LETTER JEEM WITH TWO DOTS ABOVE" },
    { 0x08A3, "ARABIC LETTER TAH WITH TWO DOTS ABOVE" },
    { 0x08A4, "ARABIC LETTER FEH WITH DOT BELOW AND THREE DOTS ABOVE" },
    { 0x08A5, "ARABIC LETTER QAF WITH DOT BELOW" },
    { 0x08A6, "ARABIC LETTER LAM WITH DOUBLE BAR" },
    { 0x08A7, "ARABIC LETTER MEEM WITH THREE DOTS ABOVE" },
    { 0x08A8, "ARABIC LETTER YEH WITH TWO DOTS BELOW AND HAMZA ABOVE" },
    { 0x08A9, "ARABIC LETTER YEH WITH TWO DOTS BELOW AND DOT ABOVE" },
    { 0x08AA, "ARABIC LETTER REH WITH LOOP" },
    { 0x08AB, "ARABIC LETTER WAW WITH DOT WITHIN" },
    { 0x08AC, "ARABIC LETTER ROHINGYA YEH" },
    { 0x08AD, "ARABIC LETTER LOW ALEF" },
    { 0x08AE, "ARABIC LETTER DAL WITH THREE DOTS BELOW" },
    { 0x08AF, "ARABIC LETTER SAD WITH THREE DOTS BELOW" },
    { 0x08B0, "ARABIC LETTER GAF WITH INVERTED STROKE" },
    { 0x08B1, "ARABIC LETTER STRAIGHT WAW" },
    { 0x08B2, "ARABIC LETTER ZAIN WITH INVERTED V ABOVE" },
    { 0x08B3, "ARABIC LETTER AIN WITH THREE DOTS BELOW" },
    { 0x08B4, "ARABIC LETTER KAF WITH DOT BELOW" },
    { 0x08B5, "ARABIC LETTER QAF WITH DOT BELOW AND NO DOTS ABOVE" },
    { 0x08B6, "ARABIC LETTER BEH WITH SMALL MEEM ABOVE" },
    { 0x08B7, "ARABIC LETTER PEH WITH SMALL MEEM ABOVE" },
    { 0x08B8, "ARABIC LETTER TEH WITH SMALL TEH ABOVE" },
    { 0x08B9, "ARABIC LETTER REH WITH SMALL NOON ABOVE" },
    { 0x08BA, "ARABIC LETTER YEH WITH TWO DOTS BELOW AND SMALL NOON ABOVE" },
    { 0x08BB, "ARABIC LETTER AFRICAN FEH" },
    { 0x08BC, "ARABIC LETTER AFRICAN QAF" },
    { 0x08BD, "ARABIC LETTER AFRICAN NOON" },
    { 0x08BE, "ARABIC LETTER PEH WITH SMALL V" },
    { 0x08BF, "ARABIC LETTER TEH WITH SMALL V" },
    { 0x08C0, "ARABIC LETTER TTEH WITH SMALL V" },
    { 0x08C1, "ARABIC LETTER TCHEH WITH SMALL V" },
    { 0x08C2, "ARABIC LETTER KEHEH WITH SMALL V" },
    { 0x08C3, "ARABIC LETTER GHAIN WITH THREE DOTS ABOVE" },
    { 0x08C4, "ARABIC LETTER AFRICAN QAF WITH THREE DOTS ABOVE" },
    { 0x08C5, "ARABIC LETTER JEEM WITH THREE DOTS ABOVE" },
    { 0x08C6, "ARABIC LETTER JEEM WITH THREE DOTS BELOW" },
    { 0x08C7, "ARABIC LETTER LAM WITH SMALL ARABIC LETTER TAH ABOVE" },
    { 0x08C8, "ARABIC LETTER GRAF" },
    { 0x08C9, "ARABIC SMALL FARSI YEH" },
    { 0x08CA, "ARABIC SMALL HIGH FARSI YEH" },
    { 0x08CB, "ARABIC SMALL HIGH YEH BARREE WITH TWO DOTS BELOW" },
    { 0x08CC, "ARABIC SMALL HIGH WORD SAH" },
    { 0x08CD, "ARABIC SMALL HIGH ZAH" },
    { 0x08CE, "ARABIC LARGE ROUND DOT ABOVE" },
    { 0x08CF, "ARABIC LARGE ROUND DOT BELOW" },
    { 0x08D0, "ARABIC SUKUN BELOW" },
    { 0x08D1, "ARABIC LARGE CIRCLE BELOW" },
    { 0x08D2, "ARABIC LARGE ROUND DOT INSIDE CIRCLE BELOW" },
    { 0x08D3, "ARABIC SMALL LOW WAW" },
    { 0x08D4, "ARABIC SMALL HIGH WORD AR-RUB" },
    { 0x08D5, "ARABIC SMALL HIGH SAD" },
    { 0x08D6, "ARABIC SMALL HIGH AIN" },
    { 0x08D7, "ARABIC SMALL HIGH QAF" },
    { 0x08D8, "ARABIC SMALL HIGH NOON WITH KASRA" },
    { 0x08D9, "ARABIC SMALL LOW NOON WITH KASRA" },
    { 0x08DA, "ARABIC SMALL HIGH WORD ATH-THALATHA" },
    { 0x08DB, "ARABIC SMALL HIGH WORD AS-SAJDA" },
    { 0x08DC, "ARABIC SMALL HIGH WORD AN-NISF" },
    { 0x08DD, "ARABIC SMALL HIGH WORD SAKTA" },
    { 0x08DE, "ARABIC SMALL HIGH WORD QIF" },
    { 0x08DF, "ARABIC SMALL HIGH WORD WAQFA" },
    { 0x08E0, "ARABIC SMALL HIGH FOOTNOTE MARKER" },
    { 0x08E1, "ARABIC SMALL HIGH SIGN SAFHA" },
    { 0x08E2, "ARABIC DISPUTED END OF AYAH" },
    { 0x08E3, "ARABIC TURNED DAMMA BELOW" },
    { 0x08E4, "ARABIC CURLY FATHA" },
    { 0x08E5, "ARABIC CURLY DAMMA" },
    { 0x08E6, "ARABIC CURLY KASRA" },
    { 0x08E7, "ARABIC CURLY FATHATAN" },
    { 0x08E8, "ARABIC CURLY DAMMATAN" },
    { 0x08E9, "ARABIC CURLY KASRATAN" },
    { 0x08EA, "ARABIC TONE ONE DOT ABOVE" },
    { 0x08EB, "ARABIC TONE TWO DOTS ABOVE" },
    { 0x08EC, "ARABIC TONE LOOP ABOVE" },
    { 0x08ED, "ARABIC TONE ONE DOT BELOW" },
    { 0x08EE, "ARABIC TONE TWO DOTS BELOW" },
    { 0x08EF, "ARABIC TONE LOOP BELOW" },
    { 0x08F0, "ARABIC OPEN FATHATAN" },
    { 0x08F1, "ARABIC OPEN DAMMATAN" },
    { 0x08F2, "ARABIC OPEN KASRATAN" },
    { 0x08F3, "ARABIC SMALL HIGH WAW" },
    { 0x08F4, "ARABIC FATHA WITH RING" },
    { 0x08F5, "ARABIC FATHA WITH DOT ABOVE" },
    { 0x08F6, "ARABIC KASRA WITH DOT BELOW" },
    { 0x08F7, "ARABIC LEFT ARROWHEAD ABOVE" },
    { 0x08F8, "ARABIC RIGHT ARROWHEAD ABOVE" },
    { 0x08F9, "ARABIC LEFT ARROWHEAD BELOW" },
    { 0x08FA, "ARABIC RIGHT ARROWHEAD BELOW" },
    { 0x08FB, "ARABIC DOUBLE RIGHT ARROWHEAD ABOVE" },
    { 0x08FC, "ARABIC DOUBLE RIGHT ARROWHEAD ABOVE WITH DOT" },
    { 0x08FD, "ARABIC RIGHT ARROWHEAD ABOVE WITH DOT" },
    { 0x08FE, "ARABIC DAMMA WITH DOT" },
    { 0x08FF, "ARABIC MARK SIDEWAYS NOON GHUNNA" },
    { 0x0900, "DEVANAGARI SIGN INVERTED CANDRABINDU" },
    { 0x0901, "DEVANAGARI SIGN CANDRABINDU" },
    { 0x0902, "DEVANAGARI SIGN ANUSVARA" },
    { 0x0903, "DEVANAGARI SIGN VISARGA" },
    { 0x0904, "DEVANAGARI LETTER SHORT A" },
    { 0x0905, "DEVANAGARI LETTER A" },
    { 0x0906, "DEVANAGARI LETTER AA" },
    { 0x0907, "DEVANAGARI LETTER I" },
    { 0x0908, "DEVANAGARI LETTER II" },
    { 0x0909, "DEVANAGARI LETTER U" },
    { 0x090A, "DEVANAGARI LETTER UU" },
    { 0x090B, "DEVANAGARI LETTER VOCALIC R" },
    { 0x090C, "DEVANAGARI LETTER VOCALIC L" },
    { 0x090D, "DEVANAGARI LETTER CANDRA E" },
    { 0x090E, "DEVANAGARI LETTER SHORT E" },
    { 0x090F, "DEVANAGARI LETTER E" },
    { 0x0910, "DEVANAGARI LETTER AI" },
    { 0x0911, "DEVANAGARI LETTER CANDRA O" },
    { 0x0912, "DEVANAGARI LETTER SHORT O" },
    { 0x0913, "DEVANAGARI LETTER O" },
    { 0x0914, "DEVANAGARI LETTER AU" },
    { 0x0915, "DEVANAGARI LETTER KA" },
    { 0x0916, "DEVANAGARI LETTER KHA" },
    { 0x0917, "DEVANAGARI LETTER GA" },
    { 0x0918, "DEVANAGARI LETTER GHA" },
    { 0x0919, "DEVANAGARI LETTER NGA" },
    { 0x091A, "DEVANAGARI LETTER CA" },
    { 0x091B, "DEVANAGARI LETTER CHA" },
    { 0x091C, "DEVANAGARI LETTER JA" },
    { 0x091D, "DEVANAGARI LETTER JHA" },
    { 0x091E, "DEVANAGARI LETTER NYA" },
    { 0x091F, "DEVANAGARI LETTER TTA" },
    { 0x0920, "DEVANAGARI LETTER TTHA" },
    { 0x0921, "DEVANAGARI LETTER DDA" },
    { 0x0922, "DEVANAGARI LETTER DDHA" },
    { 0x0923, "DEVANAGARI LETTER NNA" },
    { 0x0924, "DEVANAGARI LETTER TA" },
    { 0x0925, "DEVANAGARI LETTER THA" },
    { 0x0926, "DEVANAGARI LETTER DA" },
    { 0x0927, "DEVANAGARI LETTER DHA" },
    { 0x0928, "DEVANAGARI LETTER NA" },
    { 0x0929, "DEVANAGARI LETTER NNNA" },
    { 0x092A, "DEVANAGARI LETTER PA" },
    { 0x092B, "DEVANAGARI LETTER PHA" },
    { 0x092C, "DEVANAGARI LETTER BA" },
    { 0x092D, "DEVANAGARI LETTER BHA" },
    { 0x092E, "DEVANAGARI LETTER MA" },
    { 0x092F, "DEVANAGARI LETTER YA" },
    { 0x0930, "DEVANAGARI LETTER RA" },
    { 0x0931, "DEVANAGARI LETTER RRA" },
    { 0x0932, "DEVANAGARI LETTER LA" },
    { 0x0933, "DEVANAGARI LETTER LLA" },
    { 0x0934, "DEVANAGARI LETTER LLLA" },
    { 0x0935, "DEVANAGARI LETTER VA" },
    { 0x0936, "DEVANAGARI LETTER SHA" },
    { 0x0937, "DEVANAGARI LETTER SSA" },
    { 0x0938, "DEVANAGARI LETTER SA" },
    { 0x0939, "DEVANAGARI LETTER HA" },
    { 0x093A, "DEVANAGARI VOWEL SIGN OE" },
    { 0x093B, "DEVANAGARI VOWEL SIGN OOE" },
    { 0x093C, "DEVANAGARI SIGN NUKTA" },
    { 0x093D, "DEVANAGARI SIGN AVAGRAHA" },
    { 0x093E, "DEVANAGARI VOWEL SIGN AA" },
    { 0x093F, "DEVANAGARI VOWEL SIGN I" },
    { 0x0940, "DEVANAGARI VOWEL SIGN II" },
    { 0x0941, "DEVANAGARI VOWEL SIGN U" },
    { 0x0942, "DEVANAGARI VOWEL SIGN UU" },
    { 0x0943, "DEVANAGARI VOWEL SIGN VOCALIC R" },
    { 0x0944, "DEVANAGARI VOWEL SIGN VOCALIC RR" },
    { 0x0945, "DEVANAGARI VOWEL SIGN CANDRA E" },
    { 0x0946, "DEVANAGARI VOWEL SIGN SHORT E" },
    { 0x0947, "DEVANAGARI VOWEL SIGN E" },
    { 0x0948, "DEVANAGARI VOWEL SIGN AI" },
    { 0x0949, "DEVANAGARI VOWEL SIGN CANDRA O" },
    { 0x094A, "DEVANAGARI VOWEL SIGN SHORT O" },
    { 0x094B, "DEVANAGARI VOWEL SIGN O" },
    { 0x094C, "DEVANAGARI VOWEL SIGN AU" },
    { 0x094D, "DEVANAGARI SIGN VIRAMA" },
    { 0x094E, "DEVANAGARI VOWEL SIGN PRISHTHAMATRA E" },
    { 0x094F, "DEVANAGARI VOWEL SIGN AW" },
    { 0x0950, "DEVANAGARI OM" },
    { 0x0951, "DEVANAGARI STRESS SIGN UDATTA" },
    { 0x0952, "DEVANAGARI STRESS SIGN ANUDATTA" },
    { 0x0953, "DEVANAGARI GRAVE ACCENT" },
    { 0x0954, "DEVANAGARI ACUTE ACCENT" },
    { 0x0955, "DEVANAGARI VOWEL SIGN CANDRA LONG E" },
    { 0x0956, "DEVANAGARI VOWEL SIGN UE" },
    { 0x0957, "DEVANAGARI VOWEL SIGN UUE" },
    { 0x0958, "DEVANAGARI LETTER QA" },
    { 0x0959, "DEVANAGARI LETTER KHHA" },
    { 0x095A, "DEVANAGARI LETTER GHHA" },
    { 0x095B, "DEVANAGARI LETTER ZA" },
    { 0x095C, "DEVANAGARI LETTER DDDHA" },
    { 0x095D, "DEVANAGARI LETTER RHA" },
    { 0x095E, "DEVANAGARI LETTER FA" },
    { 0x095F, "DEVANAGARI LETTER YYA" },
    { 0x0960, "DEVANAGARI LETTER VOCALIC RR" },
    { 0x0961, "DEVANAGARI LETTER VOCALIC LL" },
    { 0x0962, "DEVANAGARI VOWEL SIGN VOCALIC L" },
    { 0x0963, "DEVANAGARI VOWEL SIGN VOCALIC LL" },
    { 0x0964, "DEVANAGARI DANDA" },
    { 0x0965, "DEVANAGARI DOUBLE DANDA" },
    { 0x0966, "DEVANAGARI DIGIT ZERO" },
    { 0x0967, "DEVANAGARI DIGIT ONE" },
    { 0x0968, "DEVANAGARI DIGIT TWO" },
    { 0x0969, "DEVANAGARI DIGIT THREE" },
    { 0x096A, "DEVANAGARI DIGIT FOUR" },
    { 0x096B, "DEVANAGARI DIGIT FIVE" },
    { 0x096C, "DEVANAGARI DIGIT SIX" },
    { 0x096D, "DEVANAGARI DIGIT SEVEN" },
    { 0x096E, "DEVANAGARI DIGIT EIGHT" },
    { 0x096F, "DEVANAGARI DIGIT NINE" },
    { 0x0970, "DEVANAGARI ABBREVIATION SIGN" },
    { 0x0971, "DEVANAGARI SIGN HIGH SPACING DOT" },
    { 0x0972, "DEVANAGARI LETTER CANDRA A" },
    { 0x0973, "DEVANAGARI LETTER OE" },
    { 0x0974, "DEVANAGARI LETTER OOE" },
    { 0x0975, "DEVANAGARI LETTER AW" },
    { 0x0976, "DEVANAGARI LETTER UE" },
    { 0x0977, "DEVANAGARI LETTER UUE" },
    { 0x0978, "DEVANAGARI LETTER MARWARI DDA" },
    { 0x0979, "DEVANAGARI LETTER ZHA" },
    { 0x097A, "DEVANAGARI LETTER HEAVY YA" },
    { 0x097B, "DEVANAGARI LETTER GGA" },
    { 0x097C, "DEVANAGARI LETTER JJA" },
    { 0x097D, "DEVANAGARI LETTER GLOTTAL STOP" },
    { 0x097E, "DEVANAGARI LETTER DDDA" },
    { 0x097F, "DEVANAGARI LETTER BBA" },
    { 0x0980, "BENGALI ANJI" },
    { 0x0981, "BENGALI SIGN CANDRABINDU" },
    { 0x0982, "BENGALI SIGN ANUSVARA" },
    { 0x0983, "BENGALI SIGN VISARGA" },
    { 0x0985, "BENGALI LETTER A" },
    { 0x0986, "BENGALI LETTER AA" },
    { 0x0987, "BENGALI LETTER I" },
    { 0x0988, "BENGALI LETTER II" },
    { 0x0989, "BENGALI LETTER U" },
    { 0x098A, "BENGALI LETTER UU" },
    { 0x098B, "BENGALI LETTER VOCALIC R" },
    { 0x098C, "BENGALI LETTER VOCALIC L" },
    { 0x098F, "BENGALI LETTER E" },
    { 0x0990, "BENGALI LETTER AI" },
    { 0x0993, "BENGALI LETTER O" },
    { 0x0994, "BENGALI LETTER AU" },
    { 0x0995, "BENGALI LETTER KA" },
    { 0x0996, "BENGALI LETTER KHA" },
    { 0x0997, "BENGALI LETTER GA" },
    { 0x0998, "BENGALI LETTER GHA" },
    { 0x0999, "BENGALI LETTER NGA" },
    { 0x099A, "BENGALI LETTER CA" },
    { 0x099B, "BENGALI LETTER CHA" },
    { 0x099C, "BENGALI LETTER JA" },
    { 0x099D, "BENGALI LETTER JHA" },
    { 0x099E, "BENGALI LETTER NYA" },
    { 0x099F, "BENGALI LETTER TTA" },
    { 0x09A0, "BENGALI LETTER TTHA" },
    { 0x09A1, "BENGALI LETTER DDA" },
    { 0x09A2, "BENGALI LETTER DDHA" },
    { 0x09A3, "BENGALI LETTER NNA" },
    { 0x09A4, "BENGALI LETTER TA" },
    { 0x09A5, "BENGALI LETTER THA" },
    { 0x09A6, "BENGALI LETTER DA" },
    { 0x09A7, "BENGALI LETTER DHA" },
    { 0x09A8, "BENGALI LETTER NA" },
    { 0x09AA, "BENGALI LETTER PA" },
    { 0x09AB, "BENGALI LETTER PHA" },
    { 0x09AC, "BENGALI LETTER BA" },
    { 0x09AD, "BENGALI LETTER BHA" },
    { 0x09AE, "BENGALI LETTER MA" },
    { 0x09AF, "BENGALI LETTER YA" },
    { 0x09B0, "BENGALI LETTER RA" },
    { 0x09B2, "BENGALI LETTER LA" },
    { 0x09B6, "BENGALI LETTER SHA" },
    { 0x09B7, "BENGALI LETTER SSA" },
    { 0x09B8, "BENGALI LETTER SA" },
    { 0x09B9, "BENGALI LETTER HA" },
    { 0x09BC, "BENGALI SIGN NUKTA" },
    { 0x09BD, "BENGALI SIGN AVAGRAHA" },
    { 0x09BE, "BENGALI VOWEL SIGN AA" },
    { 0x09BF, "BENGALI VOWEL SIGN I" },
    { 0x09C0, "BENGALI VOWEL SIGN II" },
    { 0x09C1, "BENGALI VOWEL SIGN U" },
    { 0x09C2, "BENGALI VOWEL SIGN UU" },
    { 0x09C3, "BENGALI VOWEL SIGN VOCALIC R" },
    { 0x09C4, "BENGALI VOWEL SIGN VOCALIC RR" },
    { 0x09C7, "BENGALI VOWEL SIGN E" },
    { 0x09C8, "BENGALI VOWEL SIGN AI" },
    { 0x09CB, "BENGALI VOWEL SIGN O" },
    { 0x09CC, "BENGALI VOWEL SIGN AU" },
    { 0x09CD, "BENGALI SIGN VIRAMA" },
    { 0x09CE, "BENGALI LETTER KHANDA TA" },
    { 0x09D7, "BENGALI AU LENGTH MARK" },
    { 0x09DC, "BENGALI LETTER RRA" },
    { 0x09DD, "BENGALI LETTER RHA" },
    { 0x09DF, "BENGALI LETTER YYA" },
    { 0x09E0, "BENGALI LETTER VOCALIC RR" },
    { 0x09E1, "BENGALI LETTER VOCALIC LL" },
    { 0x09E2, "BENGALI VOWEL SIGN VOCALIC L" },
    { 0x09E3, "BENGALI VOWEL SIGN VOCALIC LL" },
    { 0x09E6, "BENGALI DIGIT ZERO" },
    { 0x09E7, "BENGALI DIGIT ONE" },
    { 0x09E8, "BENGALI DIGIT TWO" },
    { 0x09E9, "BENGALI DIGIT THREE" },
    { 0x09EA, "BENGALI DIGIT FOUR" },
    { 0x09EB, "BENGALI DIGIT FIVE" },
    { 0x09EC, "BENGALI DIGIT SIX" },
    { 0x09ED, "BENGALI DIGIT SEVEN" },
    { 0x09EE, "BENGALI DIGIT EIGHT" },
    { 0x09EF, "BENGALI DIGIT NINE" },
    { 0x09F0, "BENGALI LETTER RA WITH MIDDLE DIAGONAL" },
    { 0x09F1, "BENGALI LETTER RA WITH LOWER DIAGONAL" },
    { 0x09F2, "BENGALI RUPEE MARK" },
    { 0x09F3, "BENGALI RUPEE SIGN" },
    { 0x09F4, "BENGALI CURRENCY NUMERATOR ONE" },
    { 0x09F5, "BENGALI CURRENCY NUMERATOR TWO" },
    { 0x09F6, "BENGALI CURRENCY NUMERATOR THREE" },
    { 0x09F7, "BENGALI CURRENCY NUMERATOR FOUR" },
    { 0x09F8, "BENGALI CURRENCY NUMERATOR ONE LESS THAN THE DENOMINATOR" },
    { 0x09F9, "BENGALI CURRENCY DENOMINATOR SIXTEEN" },
    { 0x09FA, "BENGALI ISSHAR" },
    { 0x09FB, "BENGALI GANDA MARK" },
    { 0x09FC, "BENGALI LETTER VEDIC ANUSVARA" },
    { 0x09FD, "BENGALI ABBREVIATION SIGN" },
    { 0x09FE, "BENGALI SANDHI MARK" },
    { 0x0A01, "GURMUKHI SIGN ADAK BINDI" },
    { 0x0A02, "GURMUKHI SIGN BINDI" },
    { 0x0A03, "GURMUKHI SIGN VISARGA" },
    { 0x0A05, "GURMUKHI LETTER A" },
    { 0x0A06, "GURMUKHI LETTER AA" },
    { 0x0A07, "GURMUKHI LETTER I" },
    { 0x0A08, "GURMUKHI LETTER II" },
    { 0x0A09, "GURMUKHI LETTER U" },
    { 0x0A0A, "GURMUKHI LETTER UU" },
    { 0x0A0F, "GURMUKHI LETTER EE" },
    { 0x0A10, "GURMUKHI LETTER AI" },
    { 0x0A13, "GURMUKHI LETTER OO" },
    { 0x0A14, "GURMUKHI LETTER AU" },
    { 0x0A15, "GURMUKHI LETTER KA" },
    { 0x0A16, "GURMUKHI LETTER KHA" },
    { 0x0A17, "GURMUKHI LETTER GA" },
    { 0x0A18, "GURMUKHI LETTER GHA" },
    { 0x0A19, "GURMUKHI LETTER NGA" },
    { 0x0A1A, "GURMUKHI LETTER CA" },
    { 0x0A1B, "GURMUKHI LETTER CHA" },
    { 0x0A1C, "GURMUKHI LETTER JA" },
    { 0x0A1D, "GURMUKHI LETTER JHA" },
    { 0x0A1E, "GURMUKHI LETTER NYA" },
    { 0x0A1F, "GURMUKHI LETTER TTA" },
    { 0x0A20, "GURMUKHI LETTER TTHA" },
    { 0x0A21, "GURMUKHI LETTER DDA" },
    { 0x0A22, "GURMUKHI LETTER DDHA" },
    { 0x0A23, "GURMUKHI LETTER NNA" },
    { 0x0A24, "GURMUKHI LETTER TA" },
    { 0x0A25, "GURMUKHI LETTER THA" },
    { 0x0A26, "GURMUKHI LETTER DA" },
    { 0x0A27, "GURMUKHI LETTER DHA" },
    { 0x0A28, "GURMUKHI LETTER NA" },
    { 0x0A2A, "GURMUKHI LETTER PA" },
    { 0x0A2B, "GURMUKHI LETTER PHA" },
    { 0x0A2C, "GURMUKHI LETTER BA" },
    { 0x0A2D, "GURMUKHI LETTER BHA" },
    { 0x0A2E, "GURMUKHI LETTER MA" },
    { 0x0A2F, "GURMUKHI LETTER YA" },
    { 0x0A30, "GURMUKHI LETTER RA" },
    { 0x0A32, "GURMUKHI LETTER LA" },
    { 0x0A33, "GURMUKHI LETTER LLA" },
    { 0x0A35, "GURMUKHI LETTER VA" },
    { 0x0A36, "GURMUKHI LETTER SHA" },
    { 0x0A38, "GURMUKHI LETTER SA" },
    { 0x0A39, "GURMUKHI LETTER HA" },
    { 0x0A3C, "GURMUKHI SIGN NUKTA" },
    { 0x0A3E, "GURMUKHI VOWEL SIGN AA" },
    { 0x0A3F, "GURMUKHI VOWEL SIGN I" },
    { 0x0A40, "GURMUKHI VOWEL SIGN II" },
    { 0x0A41, "GURMUKHI VOWEL SIGN U" },
    { 0x0A42, "GURMUKHI VOWEL SIGN UU" },
    { 0x0A47, "GURMUKHI VOWEL SIGN EE" },
    { 0x0A48, "GURMUKHI VOWEL SIGN AI" },
    { 0x0A4B, "GURMUKHI VOWEL SIGN OO" },
    { 0x0A4C, "GURMUKHI VOWEL SIGN AU" },
    { 0x0A4D, "GURMUKHI SIGN VIRAMA" },
    { 0x0A51, "GURMUKHI SIGN UDAAT" },
    { 0x0A59, "GURMUKHI LETTER KHHA" },
    { 0x0A5A, "GURMUKHI LETTER GHHA" },
    { 0x0A5B, "GURMUKHI LETTER ZA" },
    { 0x0A5C, "GURMUKHI LETTER RRA" },
    { 0x0A5E, "GURMUKHI LETTER FA" },
    { 0x0A66, "GURMUKHI DIGIT ZERO" },
    { 0x0A67, "GURMUKHI DIGIT ONE" },
    { 0x0A68, "GURMUKHI DIGIT TWO" },
    { 0x0A69, "GURMUKHI DIGIT THREE" },
    { 0x0A6A, "GURMUKHI DIGIT FOUR" },
    { 0x0A6B, "GURMUKHI DIGIT FIVE" },
    { 0x0A6C, "GURMUKHI DIGIT SIX" },
    { 0x0A6D, "GURMUKHI DIGIT SEVEN" },
    { 0x0A6E, "GURMUKHI DIGIT EIGHT" },
    { 0x0A6F, "GURMUKHI DIGIT NINE" },
    { 0x0A70, "GURMUKHI TIPPI" },
    { 0x0A71, "GURMUKHI ADDAK" },
    { 0x0A72, "GURMUKHI IRI" },
    { 0x0A73, "GURMUKHI URA" },
    { 0x0A74, "GURMUKHI EK ONKAR" },
    { 0x0A75, "GURMUKHI SIGN YAKASH" },
    { 0x0A76, "GURMUKHI ABBREVIATION SIGN" },
    { 0x0A81, "GUJARATI SIGN CANDRABINDU" },
    { 0x0A82, "GUJARATI SIGN ANUSVARA" },
    { 0x0A83, "GUJARATI SIGN VISARGA" },
    { 0x0A85, "GUJARATI LETTER A" },
    { 0x0A86, "GUJARATI LETTER AA" },
    { 0x0A87, "GUJARATI LETTER I" },
    { 0x0A88, "GUJARATI LETTER II" },
    { 0x0A89, "GUJARATI LETTER U" },
    { 0x0A8A, "GUJARATI LETTER UU" },
    { 0x0A8B, "GUJARATI LETTER VOCALIC R" },
    { 0x0A8C, "GUJARATI LETTER VOCALIC L" },
    { 0x0A8D, "GUJARATI VOWEL CANDRA E" },
    { 0x0A8F, "GUJARATI LETTER E" },
    { 0x0A90, "GUJARATI LETTER AI" },
    { 0x0A91, "GUJARATI VOWEL CANDRA O" },
    { 0x0A93, "GUJARATI LETTER O" },
    { 0x0A94, "GUJARATI LETTER AU" },
    { 0x0A95, "GUJARATI LETTER KA" },
    { 0x0A96, "GUJARATI LETTER KHA" },
    { 0x0A97, "GUJARATI LETTER GA" },
    { 0x0A98, "GUJARATI LETTER GHA" },
    { 0x0A99, "GUJARATI LETTER NGA" },
    { 0x0A9A, "GUJARATI LETTER CA" },
    { 0x0A9B, "GUJARATI LETTER CHA" },
    { 0x0A9C, "GUJARATI LETTER JA" },
    { 0x0A9D, "GUJARATI LETTER JHA" },
    { 0x0A9E, "GUJARATI LETTER NYA" },
    { 0x0A9F, "GUJARATI LETTER TTA" },
    { 0x0AA0, "GUJARATI LETTER TTHA" },
    { 0x0AA1, "GUJARATI LETTER DDA" },
    { 0x0AA2, "GUJARATI LETTER DDHA" },
    { 0x0AA3, "GUJARATI LETTER NNA" },
    { 0x0AA4, "GUJARATI LETTER TA" },
    { 0x0AA5, "GUJARATI LETTER THA" },
    { 0x0AA6, "GUJARATI LETTER DA" },
    { 0x0AA7, "GUJARATI LETTER DHA" },
    { 0x0AA8, "GUJARATI LETTER NA" },
    { 0x0AAA, "GUJARATI LETTER PA" },
    { 0x0AAB, "GUJARATI LETTER PHA" },
    { 0x0AAC, "GUJARATI LETTER BA" },
    { 0x0AAD, "GUJARATI LETTER BHA" },
    { 0x0AAE, "GUJARATI LETTER MA" },
    { 0x0AAF, "GUJARATI LETTER YA" },
    { 0x0AB0, "GUJARATI LETTER RA" },
    { 0x0AB2, "GUJARATI LETTER LA" },
    { 0x0AB3, "GUJARATI LETTER LLA" },
    { 0x0AB5, "GUJARATI LETTER VA" },
    { 0x0AB6, "GUJARATI LETTER SHA" },
    { 0x0AB7, "GUJARATI LETTER SSA" },
    { 0x0AB8, "GUJARATI LETTER SA" },
    { 0x0AB9, "GUJARATI LETTER HA" },
    { 0x0ABC, "GUJARATI SIGN NUKTA" },
    { 0x0ABD, "GUJARATI SIGN AVAGRAHA" },
    { 0x0ABE, "GUJARATI VOWEL SIGN AA" },
    { 0x0ABF, "GUJARATI VOWEL SIGN I" },
    { 0x0AC0, "GUJARATI VOWEL SIGN II" },
    { 0x0AC1, "GUJARATI VOWEL SIGN U" },
    { 0x0AC2, "GUJARATI VOWEL SIGN UU" },
    { 0x0AC3, "GUJARATI VOWEL SIGN VOCALIC R" },
    { 0x0AC4, "GUJARATI VOWEL SIGN VOCALIC RR" },
    { 0x0AC5, "GUJARATI VOWEL SIGN CANDRA E" },
    { 0x0AC7, "GUJARATI VOWEL SIGN E" },
    { 0x0AC8, "GUJARATI VOWEL SIGN AI" },
    { 0x0AC9, "GUJARATI VOWEL SIGN CANDRA O" },
    { 0x0ACB, "GUJARATI VOWEL SIGN O" },
    { 0x0ACC, "GUJARATI VOWEL SIGN AU" },
    { 0x0ACD, "GUJARATI SIGN VIRAMA" },
    { 0x0AD0, "GUJARATI OM" },
    { 0x0AE0, "GUJARATI LETTER VOCALIC RR" },
    { 0x0AE1, "GUJARATI LETTER VOCALIC LL" },
    { 0x0AE2, "GUJARATI VOWEL SIGN VOCALIC L" },
    { 0x0AE3, "GUJARATI VOWEL SIGN VOCALIC LL" },
    { 0x0AE6, "GUJARATI DIGIT ZERO" },
    { 0x0AE7, "GUJARATI DIGIT ONE" },
    { 0x0AE8, "GUJARATI DIGIT TWO" },
    { 0x0AE9, "GUJARATI DIGIT THREE" },
    { 0x0AEA, "GUJARATI DIGIT FOUR" },
    { 0x0AEB, "GUJARATI DIGIT FIVE" },
    { 0x0AEC, "GUJARATI DIGIT SIX" },
    { 0x0AED, "GUJARATI DIGIT SEVEN" },
    { 0x0AEE, "GUJARATI DIGIT EIGHT" },
    { 0x0AEF, "GUJARATI DIGIT NINE" },
    { 0x0AF0, "GUJARATI ABBREVIATION SIGN" },
    { 0x0AF1, "GUJARATI RUPEE SIGN" },
    { 0x0AF9, "GUJARATI LETTER ZHA" },
    { 0x0AFA, "GUJARATI SIGN SUKUN" },
    { 0x0AFB, "GUJARATI SIGN SHADDA" },
    { 0x0AFC, "GUJARATI SIGN MADDAH" },
    { 0x0AFD, "GUJARATI SIGN THREE-DOT NUKTA ABOVE" },
    { 0x0AFE, "GUJARATI SIGN CIRCLE NUKTA ABOVE" },
    { 0x0AFF, "GUJARATI SIGN TWO-CIRCLE NUKTA ABOVE" },
    { 0x0B01, "ORIYA SIGN CANDRABINDU" },
    { 0x0B02, "ORIYA SIGN ANUSVARA" },
    { 0x0B03, "ORIYA SIGN VISARGA" },
    { 0x0B05, "ORIYA LETTER A" },
    { 0x0B06, "ORIYA LETTER AA" },
    { 0x0B07, "ORIYA LETTER I" },
    { 0x0B08, "ORIYA LETTER II" },
    { 0x0B09, "ORIYA LETTER U" },
    { 0x0B0A, "ORIYA LETTER UU" },
    { 0x0B0B, "ORIYA LETTER VOCALIC R" },
    { 0x0B0C, "ORIYA LETTER VOCALIC L" },
    { 0x0B0F, "ORIYA LETTER E" },
    { 0x0B10, "ORIYA LETTER AI" },
    { 0x0B13, "ORIYA LETTER O" },
    { 0x0B14, "ORIYA LETTER AU" },
    { 0x0B15, "ORIYA LETTER KA" },
    { 0x0B16, "ORIYA LETTER KHA" },
    { 0x0B17, "ORIYA LETTER GA" },
    { 0x0B18, "ORIYA LETTER GHA" },
    { 0x0B19, "ORIYA LETTER NGA" },
    { 0x0B1A, "ORIYA LETTER CA" },
    { 0x0B1B, "ORIYA LETTER CHA" },
    { 0x0B1C, "ORIYA LETTER JA" },
    { 0x0B1D, "ORIYA LETTER JHA" },
    { 0x0B1E, "ORIYA LETTER NYA" },
    { 0x0B1F, "ORIYA LETTER TTA" },
    { 0x0B20, "ORIYA LETTER TTHA" },
    { 0x0B21, "ORIYA LETTER DDA" },
    { 0x0B22, "ORIYA LETTER DDHA" },
    { 0x0B23, "ORIYA LETTER NNA" },
    { 0x0B24, "ORIYA LETTER TA" },
    { 0x0B25, "ORIYA LETTER THA" },
    { 0x0B26, "ORIYA LETTER DA" },
    { 0x0B27, "ORIYA LETTER DHA" },
    { 0x0B28, "ORIYA LETTER NA" },
    { 0x0B2A, "ORIYA LETTER PA" },
    { 0x0B2B, "ORIYA LETTER PHA" },
    { 0x0B2C, "ORIYA LETTER BA" },
    { 0x0B2D, "ORIYA LETTER BHA" },
    { 0x0B2E, "ORIYA LETTER MA" },
    { 0x0B2F, "ORIYA LETTER YA" },
    { 0x0B30, "ORIYA LETTER RA" },
    { 0x0B32, "ORIYA LETTER LA" },
    { 0x0B33, "ORIYA LETTER LLA" },
    { 0x0B35, "ORIYA LETTER VA" },
    { 0x0B36, "ORIYA LETTER SHA" },
    { 0x0B37, "ORIYA LETTER SSA" },
    { 0x0B38, "ORIYA LETTER SA" },
    { 0x0B39, "ORIYA LETTER HA" },
    { 0x0B3C, "ORIYA SIGN NUKTA" },
    { 0x0B3D, "ORIYA SIGN AVAGRAHA" },
    { 0x0B3E, "ORIYA VOWEL SIGN AA" },
    { 0x0B3F, "ORIYA VOWEL SIGN I" },
    { 0x0B40, "ORIYA VOWEL SIGN II" },
    { 0x0B41, "ORIYA VOWEL SIGN U" },
    { 0x0B42, "ORIYA VOWEL SIGN UU" },
    { 0x0B43, "ORIYA VOWEL SIGN VOCALIC R" },
    { 0x0B44, "ORIYA VOWEL SIGN VOCALIC RR" },
    { 0x0B47, "ORIYA VOWEL SIGN E" },
    { 0x0B48, "ORIYA VOWEL SIGN AI" },
    { 0x0B4B, "ORIYA VOWEL SIGN O" },
    { 0x0B4C, "ORIYA VOWEL SIGN AU" },
    { 0x0B4D, "ORIYA SIGN VIRAMA" },
    { 0x0B55, "ORIYA SIGN OVERLINE" },
    { 0x0B56, "ORIYA AI LENGTH MARK" },
    { 0x0B57, "ORIYA AU LENGTH MARK" },
    { 0x0B5C, "ORIYA LETTER RRA" },
    { 0x0B5D, "ORIYA LETTER RHA" },
    { 0x0B5F, "ORIYA LETTER YYA" },
    { 0x0B60, "ORIYA LETTER VOCALIC RR" },
    { 0x0B61, "ORIYA LETTER VOCALIC LL" },
    { 0x0B62, "ORIYA VOWEL SIGN VOCALIC L" },
    { 0x0B63, "ORIYA VOWEL SIGN VOCALIC LL" },
    { 0x0B66, "ORIYA DIGIT ZERO" },
    { 0x0B67, "ORIYA DIGIT ONE" },
    { 0x0B68, "ORIYA DIGIT TWO" },
    { 0x0B69, "ORIYA DIGIT THREE" },
    { 0x0B6A, "ORIYA DIGIT FOUR" },
    { 0x0B6B, "ORIYA DIGIT FIVE" },
    { 0x0B6C, "ORIYA DIGIT SIX" },
    { 0x0B6D, "ORIYA DIGIT SEVEN" },
    { 0x0B6E, "ORIYA DIGIT EIGHT" },
    { 0x0B6F, "ORIYA DIGIT NINE" },
    { 0x0B70, "ORIYA ISSHAR" },
    { 0x0B71, "ORIYA LETTER WA" },
    { 0x0B72, "ORIYA FRACTION ONE QUARTER" },
    { 0x0B73, "ORIYA FRACTION ONE HALF" },
    { 0x0B74, "ORIYA FRACTION THREE QUARTERS" },
    { 0x0B75, "ORIYA FRACTION ONE SIXTEENTH" },
    { 0x0B76, "ORIYA FRACTION ONE EIGHTH" },
    { 0x0B77, "ORIYA FRACTION THREE SIXTEENTHS" },
    { 0x0B82, "TAMIL SIGN ANUSVARA" },
    { 0x0B83, "TAMIL SIGN VISARGA" },
    { 0x0B85, "TAMIL LETTER A" },
    { 0x0B86, "TAMIL LETTER AA" },
    { 0x0B87, "TAMIL LETTER I" },
    { 0x0B88, "TAMIL LETTER II" },
    { 0x0B89, "TAMIL LETTER U" },
    { 0x0B8A, "TAMIL LETTER UU" },
    { 0x0B8E, "TAMIL LETTER E" },
    { 0x0B8F, "TAMIL LETTER EE" },
    { 0x0B90, "TAMIL LETTER AI" },
    { 0x0B92, "TAMIL LETTER O" },
    { 0x0B93, "TAMIL LETTER OO" },
    { 0x0B94, "TAMIL LETTER AU" },
    { 0x0B95, "TAMIL LETTER KA" },
    { 0x0B99, "TAMIL LETTER NGA" },
    { 0x0B9A, "TAMIL LETTER CA" },
    { 0x0B9C, "TAMIL LETTER JA" },
    { 0x0B9E, "TAMIL LETTER NYA" },
    { 0x0B9F, "TAMIL LETTER TTA" },
    { 0x0BA3, "TAMIL LETTER NNA" },
    { 0x0BA4, "TAMIL LETTER TA" },
    { 0x0BA8, "TAMIL LETTER NA" },
    { 0x0BA9, "TAMIL LETTER NNNA" },
    { 0x0BAA, "TAMIL LETTER PA" },
    { 0x0BAE, "TAMIL LETTER MA" },
    { 0x0BAF, "TAMIL LETTER YA" },
    { 0x0BB0, "TAMIL LETTER RA" },
    { 0x0BB1, "TAMIL LETTER RRA" },
    { 0x0BB2, "TAMIL LETTER LA" },
    { 0x0BB3, "TAMIL LETTER LLA" },
    { 0x0BB4, "TAMIL LETTER LLLA" },
    { 0x0BB5, "TAMIL LETTER VA" },
    { 0x0BB6, "TAMIL LETTER SHA" },
    { 0x0BB7, "TAMIL LETTER SSA" },
    { 0x0BB8, "TAMIL LETTER SA" },
    { 0x0BB9, "TAMIL LETTER HA" },
    { 0x0BBE, "TAMIL VOWEL SIGN AA" },
    { 0x0BBF, "TAMIL VOWEL SIGN I" },
    { 0x0BC0, "TAMIL VOWEL SIGN II" },
    { 0x0BC1, "TAMIL VOWEL SIGN U" },
    { 0x0BC2, "TAMIL VOWEL SIGN UU" },
    { 0x0BC6, "TAMIL VOWEL SIGN E" },
    { 0x0BC7, "TAMIL VOWEL SIGN EE" },
    { 0x0BC8, "TAMIL VOWEL SIGN AI" },
    { 0x0BCA, "TAMIL VOWEL SIGN O" },
    { 0x0BCB, "TAMIL VOWEL SIGN OO" },
    { 0x0BCC, "TAMIL VOWEL SIGN AU" },
    { 0x0BCD, "TAMIL SIGN VIRAMA" },
    { 0x0BD0, "TAMIL OM" },
    { 0x0BD7, "TAMIL AU LENGTH MARK" },
    { 0x0BE6, "TAMIL DIGIT ZERO" },
    { 0x0BE7, "TAMIL DIGIT ONE" },
    { 0x0BE8, "TAMIL DIGIT TWO" },
    { 0x0BE9, "TAMIL DIGIT THREE" },
    { 0x0BEA, "TAMIL DIGIT FOUR" },
    { 0x0BEB, "TAMIL DIGIT FIVE" },
    { 0x0BEC, "TAMIL DIGIT SIX" },
    { 0x0BED, "TAMIL DIGIT SEVEN" },
    { 0x0BEE, "TAMIL DIGIT EIGHT" },
    { 0x0BEF, "TAMIL DIGIT NINE" },
    { 0x0BF0, "TAMIL NUMBER TEN" },
    { 0x0BF1, "TAMIL NUMBER ONE HUNDRED" },
    { 0x0BF2, "TAMIL NUMBER ONE THOUSAND" },
    { 0x0BF3, "TAMIL DAY SIGN" },
    { 0x0BF4, "TAMIL MONTH SIGN" },
    { 0x0BF5, "TAMIL YEAR SIGN" },
    { 0x0BF6, "TAMIL DEBIT SIGN" },
    { 0x0BF7, "TAMIL CREDIT SIGN" },
    { 0x0BF8, "TAMIL AS ABOVE SIGN" },
    { 0x0BF9, "TAMIL RUPEE SIGN" },
    { 0x0BFA, "TAMIL NUMBER SIGN" },
    { 0x0C00, "TELUGU SIGN COMBINING CANDRABINDU ABOVE" },
    { 0x0C01, "TELUGU SIGN CANDRABINDU" },
    { 0x0C02, "TELUGU SIGN ANUSVARA" },
    { 0x0C03, "TELUGU SIGN VISARGA" },
    { 0x0C04, "TELUGU SIGN COMBINING ANUSVARA ABOVE" },
    { 0x0C05, "TELUGU LETTER A" },
    { 0x0C06, "TELUGU LETTER AA" },
    { 0x0C07, "TELUGU LETTER I" },
    { 0x0C08, "TELUGU LETTER II" },
    { 0x0C09, "TELUGU LETTER U" },
    { 0x0C0A, "TELUGU LETTER UU" },
    { 0x0C0B, "TELUGU LETTER VOCALIC R" },
    { 0x0C0C, "TELUGU LETTER VOCALIC L" },
    { 0x0C0E, "TELUGU LETTER E" },
    { 0x0C0F, "TELUGU LETTER EE" },
    { 0x0C10, "TELUGU LETTER AI" },
    { 0x0C12, "TELUGU LETTER O" },
    { 0x0C13, "TELUGU LETTER OO" },
    { 0x0C14, "TELUGU LETTER AU" },
    { 0x0C15, "TELUGU LETTER KA" },
    { 0x0C16, "TELUGU LETTER KHA" },
    { 0x0C17, "TELUGU LETTER GA" },
    { 0x0C18, "TELUGU LETTER GHA" },
    { 0x0C19, "TELUGU LETTER NGA" },
    { 0x0C1A, "TELUGU LETTER CA" },
    { 0x0C1B, "TELUGU LETTER CHA" },
    { 0x0C1C, "TELUGU LETTER JA" },
    { 0x0C1D, "TELUGU LETTER JHA" },
    { 0x0C1E, "TELUGU LETTER NYA" },
    { 0x0C1F, "TELUGU LETTER TTA" },
    { 0x0C20, "TELUGU LETTER TTHA" },
    { 0x0C21, "TELUGU LETTER DDA" },
    { 0x0C22, "TELUGU LETTER DDHA" },
    { 0x0C23, "TELUGU LETTER NNA" },
    { 0x0C24, "TELUGU LETTER TA" },
    { 0x0C25, "TELUGU LETTER THA" },
    { 0x0C26, "TELUGU LETTER DA" },
    { 0x0C27, "TELUGU LETTER DHA" },
    { 0x0C28, "TELUGU LETTER NA" },
    { 0x0C2A, "TELUGU LETTER PA" },
    { 0x0C2B, "TELUGU LETTER PHA" },
    { 0x0C2C, "TELUGU LETTER BA" },
    { 0x0C2D, "TELUGU LETTER BHA" },
    { 0x0C2E, "TELUGU LETTER MA" },
    { 0x0C2F, "TELUGU LETTER YA" },
    { 0x0C30, "TELUGU LETTER RA" },
    { 0x0C31, "TELUGU LETTER RRA" },
    { 0x0C32, "TELUGU LETTER LA" },
    { 0x0C33, "TELUGU LETTER LLA" },
    { 0x0C34, "TELUGU LETTER LLLA" },
    { 0x0C35, "TELUGU LETTER VA" },
    { 0x0C36, "TELUGU LETTER SHA" },
    { 0x0C37, "TELUGU LETTER SSA" },
    { 0x0C38, "TELUGU LETTER SA" },
    { 0x0C39, "TELUGU LETTER HA" },
    { 0x0C3C, "TELUGU SIGN NUKTA" },
    { 0x0C3D, "TELUGU SIGN AVAGRAHA" },
    { 0x0C3E, "TELUGU VOWEL SIGN AA" },
    { 0x0C3F, "TELUGU VOWEL SIGN I" },
    { 0x0C40, "TELUGU VOWEL SIGN II" },
    { 0x0C41, "TELUGU VOWEL SIGN U" },
    { 0x0C42, "TELUGU VOWEL SIGN UU" },
    { 0x0C43, "TELUGU VOWEL SIGN VOCALIC R" },
    { 0x0C44, "TELUGU VOWEL SIGN VOCALIC RR" },
    { 0x0C46, "TELUGU VOWEL SIGN E" },
    { 0x0C47, "TELUGU VOWEL SIGN EE" },
    { 0x0C48, "TELUGU VOWEL SIGN AI" },
    { 0x0C4A, "TELUGU VOWEL SIGN O" },
    { 0x0C4B, "TELUGU VOWEL SIGN OO" },
    { 0x0C4C, "TELUGU VOWEL SIGN AU" },
    { 0x0C4D, "TELUGU SIGN VIRAMA" },
    { 0x0C55, "TELUGU LENGTH MARK" },
    { 0x0C56, "TELUGU AI LENGTH MARK" },
    { 0x0C58, "TELUGU LETTER TSA" },
    { 0x0C59, "TELUGU LETTER DZA" },
    { 0x0C5A, "TELUGU LETTER RRRA" },
    { 0x0C5D, "TELUGU LETTER NAKAARA POLLU" },
    { 0x0C60, "TELUGU LETTER VOCALIC RR" },
    { 0x0C61, "TELUGU LETTER VOCALIC LL" },
    { 0x0C62, "TELUGU VOWEL SIGN VOCALIC L" },
    { 0x0C63, "TELUGU VOWEL SIGN VOCALIC LL" },
    { 0x0C66, "TELUGU DIGIT ZERO" },
    { 0x0C67, "TELUGU DIGIT ONE" },
    { 0x0C68, "TELUGU DIGIT TWO" },
    { 0x0C69, "TELUGU DIGIT THREE" },
    { 0x0C6A, "TELUGU DIGIT FOUR" },
    { 0x0C6B, "TELUGU DIGIT FIVE" },
    { 0x0C6C, "TELUGU DIGIT SIX" },
    { 0x0C6D, "TELUGU DIGIT SEVEN" },
    { 0x0C6E, "TELUGU DIGIT EIGHT" },
    { 0x0C6F, "TELUGU DIGIT NINE" },
    { 0x0C77, "TELUGU SIGN SIDDHAM" },
    { 0x0C78, "TELUGU FRACTION DIGIT ZERO FOR ODD POWERS OF FOUR" },
    { 0x0C79, "TELUGU FRACTION DIGIT ONE FOR ODD POWERS OF FOUR" },
    { 0x0C7A, "TELUGU FRACTION DIGIT TWO FOR ODD POWERS OF FOUR" },
    { 0x0C7B, "TELUGU FRACTION DIGIT THREE FOR ODD POWERS OF FOUR" },
    { 0x0C7C, "TELUGU FRACTION DIGIT ONE FOR EVEN POWERS OF FOUR" },
    { 0x0C7D, "TELUGU FRACTION DIGIT TWO FOR EVEN POWERS OF FOUR" },
    { 0x0C7E, "TELUGU FRACTION DIGIT THREE FOR EVEN POWERS OF FOUR" },
    { 0x0C7F, "TELUGU SIGN TUUMU" },
    { 0x0C80, "KANNADA SIGN SPACING CANDRABINDU" },
    { 0x0C81, "KANNADA SIGN CANDRABINDU" },
    { 0x0C82, "KANNADA SIGN ANUSVARA" },
    { 0x0C83, "KANNADA SIGN VISARGA" },
    { 0x0C84, "KANNADA SIGN SIDDHAM" },
    { 0x0C85, "KANNADA LETTER A" },
    { 0x0C86, "KANNADA LETTER AA" },
    { 0x0C87, "KANNADA LETTER I" },
    { 0x0C88, "KANNADA LETTER II" },
    { 0x0C89, "KANNADA LETTER U" },
    { 0x0C8A, "KANNADA LETTER UU" },
    { 0x0C8B, "KANNADA LETTER VOCALIC R" },
    { 0x0C8C, "KANNADA LETTER VOCALIC L" },
    { 0x0C8E, "KANNADA LETTER E" },
    { 0x0C8F, "KANNADA LETTER EE" },
    { 0x0C90, "KANNADA LETTER AI" },
    { 0x0C92, "KANNADA LETTER O" },
    { 0x0C93, "KANNADA LETTER OO" },
    { 0x0C94, "KANNADA LETTER AU" },
    { 0x0C95, "KANNADA LETTER KA" },
    { 0x0C96, "KANNADA LETTER KHA" },
    { 0x0C97, "KANNADA LETTER GA" },
    { 0x0C98, "KANNADA LETTER GHA" },
    { 0x0C99, "KANNADA LETTER NGA" },
    { 0x0C9A, "KANNADA LETTER CA" },
    { 0x0C9B, "KANNADA LETTER CHA" },
    { 0x0C9C, "KANNADA LETTER JA" },
    { 0x0C9D, "KANNADA LETTER JHA" },
    { 0x0C9E, "KANNADA LETTER NYA" },
    { 0x0C9F, "KANNADA LETTER TTA" },
    { 0x0CA0, "KANNADA LETTER TTHA" },
    { 0x0CA1, "KANNADA LETTER DDA" },
    { 0x0CA2, "KANNADA LETTER DDHA" },
    { 0x0CA3, "KANNADA LETTER NNA" },
    { 0x0CA4, "KANNADA LETTER TA" },
    { 0x0CA5, "KANNADA LETTER THA" },
    { 0x0CA6, "KANNADA LETTER DA" },
    { 0x0CA7, "KANNADA LETTER DHA" },
    { 0x0CA8, "KANNADA LETTER NA" },
    { 0x0CAA, "KANNADA LETTER PA" },
    { 0x0CAB, "KANNADA LETTER PHA" },
    { 0x0CAC, "KANNADA LETTER BA" },
    { 0x0CAD, "KANNADA LETTER BHA" },
    { 0x0CAE, "KANNADA LETTER MA" },
    { 0x0CAF, "KANNADA LETTER YA" },
    { 0x0CB0, "KANNADA LETTER RA" },
    { 0x0CB1, "KANNADA LETTER RRA" },
    { 0x0CB2, "KANNADA LETTER LA" },
    { 0x0CB3, "KANNADA LETTER LLA" },
    { 0x0CB5, "KANNADA LETTER VA" },
    { 0x0CB6, "KANNADA LETTER SHA" },
    { 0x0CB7, "KANNADA LETTER SSA" },
    { 0x0CB8, "KANNADA LETTER SA" },
    { 0x0CB9, "KANNADA LETTER HA" },
    { 0x0CBC, "KANNADA SIGN NUKTA" },
    { 0x0CBD, "KANNADA SIGN AVAGRAHA" },
    { 0x0CBE, "KANNADA VOWEL SIGN AA" },
    { 0x0CBF, "KANNADA VOWEL SIGN I" },
    { 0x0CC0, "KANNADA VOWEL SIGN II" },
    { 0x0CC1, "KANNADA VOWEL SIGN U" },
    { 0x0CC2, "KANNADA VOWEL SIGN UU" },
    { 0x0CC3, "KANNADA VOWEL SIGN VOCALIC R" },
    { 0x0CC4, "KANNADA VOWEL SIGN VOCALIC RR" },
    { 0x0CC6, "KANNADA VOWEL SIGN E" },
    { 0x0CC7, "KANNADA VOWEL SIGN EE" },
    { 0x0CC8, "KANNADA VOWEL SIGN AI" },
    { 0x0CCA, "KANNADA VOWEL SIGN O" },
    { 0x0CCB, "KANNADA VOWEL SIGN OO" },
    { 0x0CCC, "KANNADA VOWEL SIGN AU" },
    { 0x0CCD, "KANNADA SIGN VIRAMA" },
    { 0x0CD5, "KANNADA LENGTH MARK" },
    { 0x0CD6, "KANNADA AI LENGTH MARK" },
    { 0x0CDD, "KANNADA LETTER NAKAARA POLLU" },
    { 0x0CDE, "KANNADA LETTER FA" },
    { 0x0CE0, "KANNADA LETTER VOCALIC RR" },
    { 0x0CE1, "KANNADA LETTER VOCALIC LL" },
    { 0x0CE2, "KANNADA VOWEL SIGN VOCALIC L" },
    { 0x0CE3, "KANNADA VOWEL SIGN VOCALIC LL" },
    { 0x0CE6, "KANNADA DIGIT ZERO" },
    { 0x0CE7, "KANNADA DIGIT ONE" },
    { 0x0CE8, "KANNADA DIGIT TWO" },
    { 0x0CE9, "KANNADA DIGIT THREE" },
    { 0x0CEA, "KANNADA DIGIT FOUR" },
    { 0x0CEB, "KANNADA DIGIT FIVE" },
    { 0x0CEC, "KANNADA DIGIT SIX" },
    { 0x0CED, "KANNADA DIGIT SEVEN" },
    { 0x0CEE, "KANNADA DIGIT EIGHT" },
    { 0x0CEF, "KANNADA DIGIT NINE" },
    { 0x0CF1, "KANNADA SIGN JIHVAMULIYA" },
    { 0x0CF2, "KANNADA SIGN UPADHMANIYA" },
    { 0x0CF3, "KANNADA SIGN COMBINING ANUSVARA ABOVE RIGHT" },
    { 0x0D00, "MALAYALAM SIGN COMBINING ANUSVARA ABOVE" },
    { 0x0D01, "MALAYALAM SIGN CANDRABINDU" },
    { 0x0D02, "MALAYALAM SIGN ANUSVARA" },
    { 0x0D03, "MALAYALAM SIGN VISARGA" },
    { 0x0D04, "MALAYALAM LETTER VEDIC ANUSVARA" },
    { 0x0D05, "MALAYALAM LETTER A" },
    { 0x0D06, "MALAYALAM LETTER AA" },
    { 0x0D07, "MALAYALAM LETTER I" },
    { 0x0D08, "MALAYALAM LETTER II" },
    { 0x0D09, "MALAYALAM LETTER U" },
    { 0x0D0A, "MALAYALAM LETTER UU" },
    { 0x0D0B, "MALAYALAM LETTER VOCALIC R" },
    { 0x0D0C, "MALAYALAM LETTER VOCALIC L" },
    { 0x0D0E, "MALAYALAM LETTER E" },
    { 0x0D0F, "MALAYALAM LETTER EE" },
    { 0x0D10, "MALAYALAM LETTER AI" },
    { 0x0D12, "MALAYALAM LETTER O" },
    { 0x0D13, "MALAYALAM LETTER OO" },
    { 0x0D14, "MALAYALAM LETTER AU" },
    { 0x0D15, "MALAYALAM LETTER KA" },
    { 0x0D16, "MALAYALAM LETTER KHA" },
    { 0x0D17, "MALAYALAM LETTER GA" },
    { 0x0D18, "MALAYALAM LETTER GHA" },
    { 0x0D19, "MALAYALAM LETTER NGA" },
    { 0x0D1A, "MALAYALAM LETTER CA" },
    { 0x0D1B, "MALAYALAM LETTER CHA" },
    { 0x0D1C, "MALAYALAM LETTER JA" },
    { 0x0D1D, "MALAYALAM LETTER JHA" },
    { 0x0D1E, "MALAYALAM LETTER NYA" },
    { 0x0D1F, "MALAYALAM LETTER TTA" },
    { 0x0D20, "MALAYALAM LETTER TTHA" },
    { 0x0D21, "MALAYALAM LETTER DDA" },
    { 0x0D22, "MALAYALAM LETTER DDHA" },
    { 0x0D23, "MALAYALAM LETTER NNA" },
    { 0x0D24, "MALAYALAM LETTER TA" },
    { 0x0D25, "MALAYALAM LETTER THA" },
    { 0x0D26, "MALAYALAM LETTER DA" },
    { 0x0D27, "MALAYALAM LETTER DHA" },
    { 0x0D28, "MALAYALAM LETTER NA" },
    { 0x0D29, "MALAYALAM LETTER NNNA" },
    { 0x0D2A, "MALAYALAM LETTER PA" },
    { 0x0D2B, "MALAYALAM LETTER PHA" },
    { 0x0D2C, "MALAYALAM LETTER BA" },
    { 0x0D2D, "MALAYALAM LETTER BHA" },
    { 0x0D2E, "MALAYALAM LETTER MA" },
    { 0x0D2F, "MALAYALAM LETTER YA" },
    { 0x0D30, "MALAYALAM LETTER RA" },
    { 0x0D31, "MALAYALAM LETTER RRA" },
    { 0x0D32, "MALAYALAM LETTER LA" },
    { 0x0D33, "MALAYALAM LETTER LLA" },
    { 0x0D34, "MALAYALAM LETTER LLLA" },
    { 0x0D35, "MALAYALAM LETTER VA" },
    { 0x0D36, "MALAYALAM LETTER SHA" },
    { 0x0D37, "MALAYALAM LETTER SSA" },
    { 0x0D38, "MALAYALAM LETTER SA" },
    { 0x0D39, "MALAYALAM LETTER HA" },
    { 0x0D3A, "MALAYALAM LETTER TTTA" },
    { 0x0D3B, "MALAYALAM SIGN VERTICAL BAR VIRAMA" },
    { 0x0D3C, "MALAYALAM SIGN CIRCULAR VIRAMA" },
    { 0x0D3D, "MALAYALAM SIGN AVAGRAHA" },
    { 0x0D3E, "MALAYALAM VOWEL SIGN AA" },
    { 0x0D3F, "MALAYALAM VOWEL SIGN I" },
    { 0x0D40, "MALAYALAM VOWEL SIGN II" },
    { 0x0D41, "MALAYALAM VOWEL SIGN U" },
    { 0x0D42, "MALAYALAM VOWEL SIGN UU" },
    { 0x0D43, "MALAYALAM VOWEL SIGN VOCALIC R" },
    { 0x0D44, "MALAYALAM VOWEL SIGN VOCALIC RR" },
    { 0x0D46, "MALAYALAM VOWEL SIGN E" },
    { 0x0D47, "MALAYALAM VOWEL SIGN EE" },
    { 0x0D48, "MALAYALAM VOWEL SIGN AI" },
    { 0x0D4A, "MALAYALAM VOWEL SIGN O" },
    { 0x0D4B, "MALAYALAM VOWEL SIGN OO" },
    { 0x0D4C, "MALAYALAM VOWEL SIGN AU" },
    { 0x0D4D, "MALAYALAM SIGN VIRAMA" },
    { 0x0D4E, "MALAYALAM LETTER DOT REPH" },
    { 0x0D4F, "MALAYALAM SIGN PARA" },
    { 0x0D54, "MALAYALAM LETTER CHILLU M" },
    { 0x0D55, "MALAYALAM LETTER CHILLU Y" },
    { 0x0D56, "MALAYALAM LETTER CHILLU LLL" },
    { 0x0D57, "MALAYALAM AU LENGTH MARK" },
    { 0x0D58, "MALAYALAM FRACTION ONE ONE-HUNDRED-AND-SIXTIETH" },
    { 0x0D59, "MALAYALAM FRACTION ONE FORTIETH" },
    { 0x0D5A, "MALAYALAM FRACTION THREE EIGHTIETHS" },
    { 0x0D5B, "MALAYALAM FRACTION ONE TWENTIETH" },
    { 0x0D5C, "MALAYALAM FRACTION ONE TENTH" },
    { 0x0D5D, "MALAYALAM FRACTION THREE TWENTIETHS" },
    { 0x0D5E, "MALAYALAM FRACTION ONE FIFTH" },
    { 0x0D5F, "MALAYALAM LETTER ARCHAIC II" },
    { 0x0D60, "MALAYALAM LETTER VOCALIC RR" },
    { 0x0D61, "MALAYALAM LETTER VOCALIC LL" },
    { 0x0D62, "MALAYALAM VOWEL SIGN VOCALIC L" },
    { 0x0D63, "MALAYALAM VOWEL SIGN VOCALIC LL" },
    { 0x0D66, "MALAYALAM DIGIT ZERO" },
    { 0x0D67, "MALAYALAM DIGIT ONE" },
    { 0x0D68, "MALAYALAM DIGIT TWO" },
    { 0x0D69, "MALAYALAM DIGIT THREE" },
    { 0x0D6A, "MALAYALAM DIGIT FOUR" },
    { 0x0D6B, "MALAYALAM DIGIT FIVE" },
    { 0x0D6C, "MALAYALAM DIGIT SIX" },
    { 0x0D6D, "MALAYALAM DIGIT SEVEN" },
    { 0x0D6E, "MALAYALAM DIGIT EIGHT" },
    { 0x0D6F, "MALAYALAM DIGIT NINE" },
    { 0x0D70, "MALAYALAM NUMBER TEN" },
    { 0x0D71, "MALAYALAM NUMBER ONE HUNDRED" },
    { 0x0D72, "MALAYALAM NUMBER ONE THOUSAND" },
    { 0x0D73, "MALAYALAM FRACTION ONE QUARTER" },
    { 0x0D74, "MALAYALAM FRACTION ONE HALF" },
    { 0x0D75, "MALAYALAM FRACTION THREE QUARTERS" },
    { 0x0D76, "MALAYALAM FRACTION ONE SIXTEENTH" },
    { 0x0D77, "MALAYALAM FRACTION ONE EIGHTH" },
    { 0x0D78, "MALAYALAM FRACTION THREE SIXTEENTHS" },
    { 0x0D79, "MALAYALAM DATE MARK" },
    { 0x0D7A, "MALAYALAM LETTER CHILLU NN" },
    { 0x0D7B, "MALAYALAM LETTER CHILLU N" },
    { 0x0D7C, "MALAYALAM LETTER CHILLU RR" },
    { 0x0D7D, "MALAYALAM LETTER CHILLU L" },
    { 0x0D7E, "MALAYALAM LETTER CHILLU LL" },
    { 0x0D7F, "MALAYALAM LETTER CHILLU K" },
    { 0x0D81, "SINHALA SIGN CANDRABINDU" },
    { 0x0D82, "SINHALA SIGN ANUSVARAYA" },
    { 0x0D83, "SINHALA SIGN VISARGAYA" },
    { 0x0D85, "SINHALA LETTER AYANNA" },
    { 0x0D86, "SINHALA LETTER AAYANNA" },
    { 0x0D87, "SINHALA LETTER AEYANNA" },
    { 0x0D88, "SINHALA LETTER AEEYANNA" },
    { 0x0D89, "SINHALA LETTER IYANNA" },
    { 0x0D8A, "SINHALA LETTER IIYANNA" },
    { 0x0D8B, "SINHALA LETTER UYANNA" },
    { 0x0D8C, "SINHALA LETTER UUYANNA" },
    { 0x0D8D, "SINHALA LETTER IRUYANNA" },
    { 0x0D8E, "SINHALA LETTER IRUUYANNA" },
    { 0x0D8F, "SINHALA LETTER ILUYANNA" },
    { 0x0D90, "SINHALA LETTER ILUUYANNA" },
    { 0x0D91, "SINHALA LETTER EYANNA" },
    { 0x0D92, "SINHALA LETTER EEYANNA" },
    { 0x0D93, "SINHALA LETTER AIYANNA" },
    { 0x0D94, "SINHALA LETTER OYANNA" },
    { 0x0D95, "SINHALA LETTER OOYANNA" },
    { 0x0D96, "SINHALA LETTER AUYANNA" },
    { 0x0D9A, "SINHALA LETTER ALPAPRAANA KAYANNA" },
    { 0x0D9B, "SINHALA LETTER MAHAAPRAANA KAYANNA" },
    { 0x0D9C, "SINHALA LETTER ALPAPRAANA GAYANNA" },
    { 0x0D9D, "SINHALA LETTER MAHAAPRAANA GAYANNA" },
    { 0x0D9E, "SINHALA LETTER KANTAJA NAASIKYAYA" },
    { 0x0D9F, "SINHALA LETTER SANYAKA GAYANNA" },
    { 0x0DA0, "SINHALA LETTER ALPAPRAANA CAYANNA" },
    { 0x0DA1, "SINHALA LETTER MAHAAPRAANA CAYANNA" },
    { 0x0DA2, "SINHALA LETTER ALPAPRAANA JAYANNA" },
    { 0x0DA3, "SINHALA LETTER MAHAAPRAANA JAYANNA" },
    { 0x0DA4, "SINHALA LETTER TAALUJA NAASIKYAYA" },
    { 0x0DA5, "SINHALA LETTER TAALUJA SANYOOGA NAAKSIKYAYA" },
    { 0x0DA6, "SINHALA LETTER SANYAKA JAYANNA" },
    { 0x0DA7, "SINHALA LETTER ALPAPRAANA TTAYANNA" },
    { 0x0DA8, "SINHALA LETTER MAHAAPRAANA TTAYANNA" },
    { 0x0DA9, "SINHALA LETTER ALPAPRAANA DDAYANNA" },
    { 0x0DAA, "SINHALA LETTER MAHAAPRAANA DDAYANNA" },
    { 0x0DAB, "SINHALA LETTER MUURDHAJA NAYANNA" },
    { 0x0DAC, "SINHALA LETTER SANYAKA DDAYANNA" },
    { 0x0DAD, "SINHALA LETTER ALPAPRAANA TAYANNA" },
    { 0x0DAE, "SINHALA LETTER MAHAAPRAANA TAYANNA" },
    { 0x0DAF, "SINHALA LETTER ALPAPRAANA DAYANNA" },
    { 0x0DB0, "SINHALA LETTER MAHAAPRAANA DAYANNA" },
    { 0x0DB1, "SINHALA LETTER DANTAJA NAYANNA" },
    { 0x0DB3, "SINHALA LETTER SANYAKA DAYANNA" },
    { 0x0DB4, "SINHALA LETTER ALPAPRAANA PAYANNA" },
    { 0x0DB5, "SINHALA LETTER MAHAAPRAANA PAYANNA" },
    { 0x0DB6, "SINHALA LETTER ALPAPRAANA BAYANNA" },
    { 0x0DB7, "SINHALA LETTER MAHAAPRAANA BAYANNA" },
    { 0x0DB8, "SINHALA LETTER MAYANNA" },
    { 0x0DB9, "SINHALA LETTER AMBA BAYANNA" },
    { 0x0DBA, "SINHALA LETTER YAYANNA" },
    { 0x0DBB, "SINHALA LETTER RAYANNA" },
    { 0x0DBD, "SINHALA LETTER DANTAJA LAYANNA" },
    { 0x0DC0, "SINHALA LETTER VAYANNA" },
    { 0x0DC1, "SINHALA LETTER TAALUJA SAYANNA" },
    { 0x0DC2, "SINHALA LETTER MUURDHAJA SAYANNA" },
    { 0x0DC3, "SINHALA LETTER DANTAJA SAYANNA" },
    { 0x0DC4, "SINHALA LETTER HAYANNA" },
    { 0x0DC5, "SINHALA LETTER MUURDHAJA LAYANNA" },
    { 0x0DC6, "SINHALA LETTER FAYANNA" },
    { 0x0DCA, "SINHALA SIGN AL-LAKUNA" },
    { 0x0DCF, "SINHALA VOWEL SIGN AELA-PILLA" },
    { 0x0DD0, "SINHALA VOWEL SIGN KETTI AEDA-PILLA" },
    { 0x0DD1, "SINHALA VOWEL SIGN DIGA AEDA-PILLA" },
    { 0x0DD2, "SINHALA VOWEL SIGN KETTI IS-PILLA" },
    { 0x0DD3, "SINHALA VOWEL SIGN DIGA IS-PILLA" },
    { 0x0DD4, "SINHALA VOWEL SIGN KETTI PAA-PILLA" },
    { 0x0DD6, "SINHALA VOWEL SIGN DIGA PAA-PILLA" },
    { 0x0DD8, "SINHALA VOWEL SIGN GAETTA-PILLA" },
    { 0x0DD9, "SINHALA VOWEL SIGN KOMBUVA" },
    { 0x0DDA, "SINHALA VOWEL SIGN DIGA KOMBUVA" },
    { 0x0DDB, "SINHALA VOWEL SIGN KOMBU DEKA" },
    { 0x0DDC, "SINHALA VOWEL SIGN KOMBUVA HAA AELA-PILLA" },
    { 0x0DDD, "SINHALA VOWEL SIGN KOMBUVA HAA DIGA AELA-PILLA" },
    { 0x0DDE, "SINHALA VOWEL SIGN KOMBUVA HAA GAYANUKITTA" },
    { 0x0DDF, "SINHALA VOWEL SIGN GAYANUKITTA" },
    { 0x0DE6, "SINHALA LITH DIGIT ZERO" },
    { 0x0DE7, "SINHALA LITH DIGIT ONE" },
    { 0x0DE8, "SINHALA LITH DIGIT TWO" },
    { 0x0DE9, "SINHALA LITH DIGIT THREE" },
    { 0x0DEA, "SINHALA LITH DIGIT FOUR" },
    { 0x0DEB, "SINHALA LITH DIGIT FIVE" },
    { 0x0DEC, "SINHALA LITH DIGIT SIX" },
    { 0x0DED, "SINHALA LITH DIGIT SEVEN" },
    { 0x0DEE, "SINHALA LITH DIGIT EIGHT" },
    { 0x0DEF, "SINHALA LITH DIGIT NINE" },
    { 0x0DF2, "SINHALA VOWEL SIGN DIGA GAETTA-PILLA" },
    { 0x0DF3, "SINHALA VOWEL SIGN DIGA GAYANUKITTA" },
    { 0x0DF4, "SINHALA PUNCTUATION KUNDDALIYA" },
    { 0x0E01, "THAI CHARACTER KO KAI" },
    { 0x0E02, "THAI CHARACTER KHO KHAI" },
    { 0x0E03, "THAI CHARACTER KHO KHUAT" },
    { 0x0E04, "THAI CHARACTER KHO KHWAI" },
    { 0x0E05, "THAI CHARACTER KHO KHON" },
    { 0x0E06, "THAI CHARACTER KHO RAKHANG" },
    { 0x0E07, "THAI CHARACTER NGO NGU" },
    { 0x0E08, "THAI CHARACTER CHO CHAN" },
    { 0x0E09, "THAI CHARACTER CHO CHING" },
    { 0x0E0A, "THAI CHARACTER CHO CHANG" },
    { 0x0E0B, "THAI CHARACTER SO SO" },
    { 0x0E0C, "THAI CHARACTER CHO CHOE" },
    { 0x0E0D, "THAI CHARACTER YO YING" },
    { 0x0E0E, "THAI CHARACTER DO CHADA" },
    { 0x0E0F, "THAI CHARACTER TO PATAK" },
    { 0x0E10, "THAI CHARACTER THO THAN" },
    { 0x0E11, "THAI CHARACTER THO NANGMONTHO" },
    { 0x0E12, "THAI CHARACTER THO PHUTHAO" },
    { 0x0E13, "THAI CHARACTER NO NEN" },
    { 0x0E14, "THAI CHARACTER DO DEK" },
    { 0x0E15, "THAI CHARACTER TO TAO" },
    { 0x0E16, "THAI CHARACTER THO THUNG" },
    { 0x0E17, "THAI CHARACTER THO THAHAN" },
    { 0x0E18, "THAI CHARACTER THO THONG" },
    { 0x0E19, "THAI CHARACTER NO NU" },
    { 0x0E1A, "THAI CHARACTER BO BAIMAI" },
    { 0x0E1B, "THAI CHARACTER PO PLA" },
    { 0x0E1C, "THAI CHARACTER PHO PHUNG" },
    { 0x0E1D, "THAI CHARACTER FO FA" },
    { 0x0E1E, "THAI CHARACTER PHO PHAN" },
    { 0x0E1F, "THAI CHARACTER FO FAN" },
    { 0x0E20, "THAI CHARACTER PHO SAMPHAO" },
    { 0x0E21, "THAI CHARACTER MO MA" },
    { 0x0E22, "THAI CHARACTER YO YAK" },
    { 0x0E23, "THAI CHARACTER RO RUA" },
    { 0x0E24, "THAI CHARACTER RU" },
    { 0x0E25, "THAI CHARACTER LO LING" },
    { 0x0E26, "THAI CHARACTER LU" },
    { 0x0E27, "THAI CHARACTER WO WAEN" },
    { 0x0E28, "THAI CHARACTER SO SALA" },
    { 0x0E29, "THAI CHARACTER SO RUSI" },
    { 0x0E2A, "THAI CHARACTER SO SUA" },
    { 0x0E2B, "THAI CHARACTER HO HIP" },
    { 0x0E2C, "THAI CHARACTER LO CHULA" },
    { 0x0E2D, "THAI CHARACTER O ANG" },
    { 0x0E2E, "THAI CHARACTER HO NOKHUK" },
    { 0x0E2F, "THAI CHARACTER PAIYANNOI" },
    { 0x0E30, "THAI CHARACTER SARA A" },
    { 0x0E31, "THAI CHARACTER MAI HAN-AKAT" },
    { 0x0E32, "THAI CHARACTER SARA AA" },
    { 0x0E33, "THAI CHARACTER SARA AM" },
    { 0x0E34, "THAI CHARACTER SARA I" },
    { 0x0E35, "THAI CHARACTER SARA II" },
    { 0x0E36, "THAI CHARACTER SARA UE" },
    { 0x0E37, "THAI CHARACTER SARA UEE" },
    { 0x0E38, "THAI CHARACTER SARA U" },
    { 0x0E39, "THAI CHARACTER SARA UU" },
    { 0x0E3A, "THAI CHARACTER PHINTHU" },
    { 0x0E3F, "THAI CURRENCY SYMBOL BAHT" },
    { 0x0E40, "THAI CHARACTER SARA E" },
    { 0x0E41, "THAI CHARACTER SARA AE" },
    { 0x0E42, "THAI CHARACTER SARA O" },
    { 0x0E43, "THAI CHARACTER SARA AI MAIMUAN" },
    { 0x0E44, "THAI CHARACTER SARA AI MAIMALAI" },
    { 0x0E45, "THAI CHARACTER LAKKHANGYAO" },
    { 0x0E46, "THAI CHARACTER MAIYAMOK" },
    { 0x0E47, "THAI CHARACTER MAITAIKHU" },
    { 0x0E48, "THAI CHARACTER MAI EK" },
    { 0x0E49, "THAI CHARACTER MAI THO" },
    { 0x0E4A, "THAI CHARACTER MAI TRI" },
    { 0x0E4B, "THAI CHARACTER MAI CHATTAWA" },
    { 0x0E4C, "THAI CHARACTER THANTHAKHAT" },
    { 0x0E4D, "THAI CHARACTER NIKHAHIT" },
    { 0x0E4E, "THAI CHARACTER YAMAKKAN" },
    { 0x0E4F, "THAI CHARACTER FONGMAN" },
    { 0x0E50, "THAI DIGIT ZERO" },
    { 0x0E51, "THAI DIGIT ONE" },
    { 0x0E52, "THAI DIGIT TWO" },
    { 0x0E53, "THAI DIGIT THREE" },
    { 0x0E54, "THAI DIGIT FOUR" },
    { 0x0E55, "THAI DIGIT FIVE" },
    { 0x0E56, "THAI DIGIT SIX" },
    { 0x0E57, "THAI DIGIT SEVEN" },
    { 0x0E58, "THAI DIGIT EIGHT" },
    { 0x0E59, "THAI DIGIT NINE" },
    { 0x0E5A, "THAI CHARACTER ANGKHANKHU" },
    { 0x0E5B, "THAI CHARACTER KHOMUT" },
    { 0x0E81, "LAO LETTER KO" },
    { 0x0E82, "LAO LETTER KHO SUNG" },
    { 0x0E84, "LAO LETTER KHO TAM" },
    { 0x0E86, "LAO LETTER PALI GHA" },
    { 0x0E87, "LAO LETTER NGO" },
    { 0x0E88, "LAO LETTER CO" },
    { 0x0E89, "LAO LETTER PALI CHA" },
    { 0x0E8A, "LAO LETTER SO TAM" },
    { 0x0E8C, "LAO LETTER PALI JHA" },
    { 0x0E8D, "LAO LETTER NYO" },
    { 0x0E8E, "LAO LETTER PALI NYA" },
    { 0x0E8F, "LAO LETTER PALI TTA" },
    { 0x0E90, "LAO LETTER PALI TTHA" },
    { 0x0E91, "LAO LETTER PALI DDA" },
    { 0x0E92, "LAO LETTER PALI DDHA" },
    { 0x0E93, "LAO LETTER PALI NNA" },
    { 0x0E94, "LAO LETTER DO" },
    { 0x0E95, "LAO LETTER TO" },
    { 0x0E96, "LAO LETTER THO SUNG" },
    { 0x0E97, "LAO LETTER THO TAM" },
    { 0x0E98, "LAO LETTER PALI DHA" },
    { 0x0E99, "LAO LETTER NO" },
    { 0x0E9A, "LAO LETTER BO" },
    { 0x0E9B, "LAO LETTER PO" },
    { 0x0E9C, "LAO LETTER PHO SUNG" },
    { 0x0E9D, "LAO LETTER FO TAM" },
    { 0x0E9E, "LAO LETTER PHO TAM" },
    { 0x0E9F, "LAO LETTER FO SUNG" },
    { 0x0EA0, "LAO LETTER PALI BHA" },
    { 0x0EA1, "LAO LETTER MO" },
    { 0x0EA2, "LAO LETTER YO" },
    { 0x0EA3, "LAO LETTER LO LING" },
    { 0x0EA5, "LAO LETTER LO LOOT" },
    { 0x0EA7, "LAO LETTER WO" },
    { 0x0EA8, "LAO LETTER SANSKRIT SHA" },
    { 0x0EA9, "LAO LETTER SANSKRIT SSA" },
    { 0x0EAA, "LAO LETTER SO SUNG" },
    { 0x0EAB, "LAO LETTER HO SUNG" },
    { 0x0EAC, "LAO LETTER PALI LLA" },
    { 0x0EAD, "LAO LETTER O" },
    { 0x0EAE, "LAO LETTER HO TAM" },
    { 0x0EAF, "LAO ELLIPSIS" },
    { 0x0EB0, "LAO VOWEL SIGN A" },
    { 0x0EB1, "LAO VOWEL SIGN MAI KAN" },
    { 0x0EB2, "LAO VOWEL SIGN AA" },
    { 0x0EB3, "LAO VOWEL SIGN AM" },
    { 0x0EB4, "LAO VOWEL SIGN I" },
    { 0x0EB5, "LAO VOWEL SIGN II" },
    { 0x0EB6, "LAO VOWEL SIGN Y" },
    { 0x0EB7, "LAO VOWEL SIGN YY" },
    { 0x0EB8, "LAO VOWEL SIGN U" },
    { 0x0EB9, "LAO VOWEL SIGN UU" },
    { 0x0EBA, "LAO SIGN PALI VIRAMA" },
    { 0x0EBB, "LAO VOWEL SIGN MAI KON" },
    { 0x0EBC, "LAO SEMIVOWEL SIGN LO" },
    { 0x0EBD, "LAO SEMIVOWEL SIGN NYO" },
    { 0x0EC0, "LAO VOWEL SIGN E" },
    { 0x0EC1, "LAO VOWEL SIGN EI" },
    { 0x0EC2, "LAO VOWEL SIGN O" },
    { 0x0EC3, "LAO VOWEL SIGN AY" },
    { 0x0EC4, "LAO VOWEL SIGN AI" },
    { 0x0EC6, "LAO KO LA" },
    { 0x0EC8, "LAO TONE MAI EK" },
    { 0x0EC9, "LAO TONE MAI THO" },
    { 0x0ECA, "LAO TONE MAI TI" },
    { 0x0ECB, "LAO TONE MAI CATAWA" },
    { 0x0ECC, "LAO CANCELLATION MARK" },
    { 0x0ECD, "LAO NIGGAHITA" },
    { 0x0ECE, "LAO YAMAKKAN" },
    { 0x0ED0, "LAO DIGIT ZERO" },
    { 0x0ED1, "LAO DIGIT ONE" },
    { 0x0ED2, "LAO DIGIT TWO" },
    { 0x0ED3, "LAO DIGIT THREE" },
    { 0x0ED4, "LAO DIGIT FOUR" },
    { 0x0ED5, "LAO DIGIT FIVE" },
    { 0x0ED6, "LAO DIGIT SIX" },
    { 0x0ED7, "LAO DIGIT SEVEN" },
    { 0x0ED8, "LAO DIGIT EIGHT" },
    { 0x0ED9, "LAO DIGIT NINE" },
    { 0x0EDC, "LAO HO NO" },
    { 0x0EDD, "LAO HO MO" },
    { 0x0EDE, "LAO LETTER KHMU GO" },
    { 0x0EDF, "LAO LETTER KHMU NYO" },
    { 0x0F00, "TIBETAN SYLLABLE OM" },
    { 0x0F01, "TIBETAN MARK GTER YIG MGO TRUNCATED A" },
    { 0x0F02, "TIBETAN MARK GTER YIG MGO -UM RNAM BCAD MA" },
    { 0x0F03, "TIBETAN MARK GTER YIG MGO -UM GTER TSHEG MA" },
    { 0x0F04, "TIBETAN MARK INITIAL YIG MGO MDUN MA" },
    { 0x0F05, "TIBETAN MARK CLOSING YIG MGO SGAB MA" },
    { 0x0F06, "TIBETAN MARK CARET YIG MGO PHUR SHAD MA" },
    { 0x0F07, "TIBETAN MARK YIG MGO TSHEG SHAD MA" },
    { 0x0F08, "TIBETAN MARK SBRUL SHAD" },
    { 0x0F09, "TIBETAN MARK BSKUR YIG MGO" },
    { 0x0F0A, "TIBETAN MARK BKA- SHOG YIG MGO" },
    { 0x0F0B, "TIBETAN MARK INTERSYLLABIC TSHEG" },
    { 0x0F0C, "TIBETAN MARK DELIMITER TSHEG BSTAR" },
    { 0x0F0D, "TIBETAN MARK SHAD" },
    { 0x0F0E, "TIBETAN MARK NYIS SHAD" },
    { 0x0F0F, "TIBETAN MARK TSHEG SHAD" },
    { 0x0F10, "TIBETAN MARK NYIS TSHEG SHAD" },
    { 0x0F11, "TIBETAN MARK RIN CHEN SPUNGS SHAD" },
    { 0x0F12, "TIBETAN MARK RGYA GRAM SHAD" },
    { 0x0F13, "TIBETAN MARK CARET -DZUD RTAGS ME LONG CAN" },
    { 0x0F14, "TIBETAN MARK GTER TSHEG" },
    { 0x0F15, "TIBETAN LOGOTYPE SIGN CHAD RTAGS" },
    { 0x0F16, "TIBETAN LOGOTYPE SIGN LHAG RTAGS" },
    { 0x0F17, "TIBETAN ASTROLOGICAL SIGN SGRA GCAN -CHAR RTAGS" },
    { 0x0F18, "TIBETAN ASTROLOGICAL SIGN -KHYUD PA" },
    { 0x0F19, "TIBETAN ASTROLOGICAL SIGN SDONG TSHUGS" },
    { 0x0F1A, "TIBETAN SIGN RDEL DKAR GCIG" },
    { 0x0F1B, "TIBETAN SIGN RDEL DKAR GNYIS" },
    { 0x0F1C, "TIBETAN SIGN RDEL DKAR GSUM" },
    { 0x0F1D, "TIBETAN SIGN RDEL NAG GCIG" },
    { 0x0F1E, "TIBETAN SIGN RDEL NAG GNYIS" },
    { 0x0F1F, "TIBETAN SIGN RDEL DKAR RDEL NAG" },
    { 0x0F20, "TIBETAN DIGIT ZERO" },
    { 0x0F21, "TIBETAN DIGIT ONE" },
    { 0x0F22, "TIBETAN DIGIT TWO" },
    { 0x0F23, "TIBETAN DIGIT THREE" },
    { 0x0F24, "TIBETAN DIGIT FOUR" },
    { 0x0F25, "TIBETAN DIGIT FIVE" },
    { 0x0F26, "TIBETAN DIGIT SIX" },
    { 0x0F27, "TIBETAN DIGIT SEVEN" },
    { 0x0F28, "TIBETAN DIGIT EIGHT" },
    { 0x0F29, "TIBETAN DIGIT NINE" },
    { 0x0F2A, "TIBETAN DIGIT HALF ONE" },
    { 0x0F2B, "TIBETAN DIGIT HALF TWO" },
    { 0x0F2C, "TIBETAN DIGIT HALF THREE" },
    { 0x0F2D, "TIBETAN DIGIT HALF FOUR" },
    { 0x0F2E, "TIBETAN DIGIT HALF FIVE" },
    { 0x0F2F, "TIBETAN DIGIT HALF SIX" },
    { 0x0F30, "TIBETAN DIGIT HALF SEVEN" },
    { 0x0F31, "TIBETAN DIGIT HALF EIGHT" },
    { 0x0F32, "TIBETAN DIGIT HALF NINE" },
    { 0x0F33, "TIBETAN DIGIT HALF ZERO" },
    { 0x0F34, "TIBETAN MARK BSDUS RTAGS" },
    { 0x0F35, "TIBETAN MARK NGAS BZUNG NYI ZLA" },
    { 0x0F36, "TIBETAN MARK CARET -DZUD RTAGS BZHI MIG CAN" },
    { 0x0F37, "TIBETAN MARK NGAS BZUNG SGOR RTAGS" },
    { 0x0F38, "TIBETAN MARK CHE MGO" },
    { 0x0F39, "TIBETAN MARK TSA -PHRU" },
    { 0x0F3A, "TIBETAN MARK GUG RTAGS GYON" },
    { 0x0F3B, "TIBETAN MARK GUG RTAGS GYAS" },
    { 0x0F3C, "TIBETAN MARK ANG KHANG GYON" },
    { 0x0F3D, "TIBETAN MARK ANG KHANG GYAS" },
    { 0x0F3E, "TIBETAN SIGN YAR TSHES" },
    { 0x0F3F, "TIBETAN SIGN MAR TSHES" },
    { 0x0F40, "TIBETAN LETTER KA" },
    { 0x0F41, "TIBETAN LETTER KHA" },
    { 0x0F42, "TIBETAN LETTER GA" },
    { 0x0F43, "TIBETAN LETTER GHA" },
    { 0x0F44, "TIBETAN LETTER NGA" },
    { 0x0F45, "TIBETAN LETTER CA" },
    { 0x0F46, "TIBETAN LETTER CHA" },
    { 0x0F47, "TIBETAN LETTER JA" },
    { 0x0F49, "TIBETAN LETTER NYA" },
    { 0x0F4A, "TIBETAN LETTER TTA" },
    { 0x0F4B, "TIBETAN LETTER TTHA" },
    { 0x0F4C, "TIBETAN LETTER DDA" },
    { 0x0F4D, "TIBETAN LETTER DDHA" },
    { 0x0F4E, "TIBETAN LETTER NNA" },
    { 0x0F4F, "TIBETAN LETTER TA" },
    { 0x0F50, "TIBETAN LETTER THA" },
    { 0x0F51, "TIBETAN LETTER DA" },
    { 0x0F52, "TIBETAN LETTER DHA" },
    { 0x0F53, "TIBETAN LETTER NA" },
    { 0x0F54, "TIBETAN LETTER PA" },
    { 0x0F55, "TIBETAN LETTER PHA" },
    { 0x0F56, "TIBETAN LETTER BA" },
    { 0x0F57, "TIBETAN LETTER BHA" },
    { 0x0F58, "TIBETAN LETTER MA" },
    { 0x0F59, "TIBETAN LETTER TSA" },
    { 0x0F5A, "TIBETAN LETTER TSHA" },
    { 0x0F5B, "TIBETAN LETTER DZA" },
    { 0x0F5C, "TIBETAN LETTER DZHA" },
    { 0x0F5D, "TIBETAN LETTER WA" },
    { 0x0F5E, "TIBETAN LETTER ZHA" },
    { 0x0F5F, "TIBETAN LETTER ZA" },
    { 0x0F60, "TIBETAN LETTER -A" },
    { 0x0F61, "TIBETAN LETTER YA" },
    { 0x0F62, "TIBETAN LETTER RA" },
    { 0x0F63, "TIBETAN LETTER LA" },
    { 0x0F64, "TIBETAN LETTER SHA" },
    { 0x0F65, "TIBETAN LETTER SSA" },
    { 0x0F66, "TIBETAN LETTER SA" },
    { 0x0F67, "TIBETAN LETTER HA" },
    { 0x0F68, "TIBETAN LETTER A" },
    { 0x0F69, "TIBETAN LETTER KSSA" },
    { 0x0F6A, "TIBETAN LETTER FIXED-FORM RA" },
    { 0x0F6B, "TIBETAN LETTER KKA" },
    { 0x0F6C, "TIBETAN LETTER RRA" },
    { 0x0F71, "TIBETAN VOWEL SIGN AA" },
    { 0x0F72, "TIBETAN VOWEL SIGN I" },
    { 0x0F73, "TIBETAN VOWEL SIGN II" },
    { 0x0F74, "TIBETAN VOWEL SIGN U" },
    { 0x0F75, "TIBETAN VOWEL SIGN UU" },
    { 0x0F76, "TIBETAN VOWEL SIGN VOCALIC R" },
    { 0x0F77, "TIBETAN VOWEL SIGN VOCALIC RR" },
    { 0x0F78, "TIBETAN VOWEL SIGN VOCALIC L" },
    { 0x0F79, "TIBETAN VOWEL SIGN VOCALIC LL" },
    { 0x0F7A, "TIBETAN VOWEL SIGN E" },
    { 0x0F7B, "TIBETAN VOWEL SIGN EE" },
    { 0x0F7C, "TIBETAN VOWEL SIGN O" },
    { 0x0F7D, "TIBETAN VOWEL SIGN OO" },
    { 0x0F7E, "TIBETAN SIGN RJES SU NGA RO" },
    { 0x0F7F, "TIBETAN SIGN RNAM BCAD" },
    { 0x0F80, "TIBETAN VOWEL SIGN REVERSED I" },
    { 0x0F81, "TIBETAN VOWEL SIGN REVERSED II" },
    { 0x0F82, "TIBETAN SIGN NYI ZLA NAA DA" },
    { 0x0F83, "TIBETAN SIGN SNA LDAN" },
    { 0x0F84, "TIBETAN MARK HALANTA" },
    { 0x0F85, "TIBETAN MARK PALUTA" },
    { 0x0F86, "TIBETAN SIGN LCI RTAGS" },
    { 0x0F87, "TIBETAN SIGN YANG RTAGS" },
    { 0x0F88, "TIBETAN SIGN LCE TSA CAN" },
    { 0x0F89, "TIBETAN SIGN MCHU CAN" },
    { 0x0F8A, "TIBETAN SIGN GRU CAN RGYINGS" },
    { 0x0F8B, "TIBETAN SIGN GRU MED RGYINGS" },
    { 0x0F8C, "TIBETAN SIGN INVERTED MCHU CAN" },
    { 0x0F8D, "TIBETAN SUBJOINED SIGN LCE TSA CAN" },
    { 0x0F8E, "TIBETAN SUBJOINED SIGN MCHU CAN" },
    { 0x0F8F, "TIBETAN SUBJOINED SIGN INVERTED MCHU CAN" },
    { 0x0F90, "TIBETAN SUBJOINED LETTER KA" },
    { 0x0F91, "TIBETAN SUBJOINED LETTER KHA" },
    { 0x0F92, "TIBETAN SUBJOINED LETTER GA" },
    { 0x0F93, "TIBETAN SUBJOINED LETTER GHA" },
    { 0x0F94, "TIBETAN SUBJOINED LETTER NGA" },
    { 0x0F95, "TIBETAN SUBJOINED LETTER CA" },
    { 0x0F96, "TIBETAN SUBJOINED LETTER CHA" },
    { 0x0F97, "TIBETAN SUBJOINED LETTER JA" },
    { 0x0F99, "TIBETAN SUBJOINED LETTER NYA" },
    { 0x0F9A, "TIBETAN SUBJOINED LETTER TTA" },
    { 0x0F9B, "TIBETAN SUBJOINED LETTER TTHA" },
    { 0x0F9C, "TIBETAN SUBJOINED LETTER DDA" },
    { 0x0F9D, "TIBETAN SUBJOINED LETTER DDHA" },
    { 0x0F9E, "TIBETAN SUBJOINED LETTER NNA" },
    { 0x0F9F, "TIBETAN SUBJOINED LETTER TA" },
    { 0x0FA0, "TIBETAN SUBJOINED LETTER THA" },
    { 0x0FA1, "TIBETAN SUBJOINED LETTER DA" },
    { 0x0FA2, "TIBETAN SUBJOINED LETTER DHA" },
    { 0x0FA3, "TIBETAN SUBJOINED LETTER NA" },
    { 0x0FA4, "TIBETAN SUBJOINED LETTER PA" },
    { 0x0FA5, "TIBETAN SUBJOINED LETTER PHA" },
    { 0x0FA6, "TIBETAN SUBJOINED LETTER BA" },
    { 0x0FA7, "TIBETAN SUBJOINED LETTER BHA" },
    { 0x0FA8, "TIBETAN SUBJOINED LETTER MA" },
    { 0x0FA9, "TIBETAN SUBJOINED LETTER TSA" },
    { 0x0FAA, "TIBETAN SUBJOINED LETTER TSHA" },
    { 0x0FAB, "TIBETAN SUBJOINED LETTER DZA" },
    { 0x0FAC, "TIBETAN SUBJOINED LETTER DZHA" },
    { 0x0FAD, "TIBETAN SUBJOINED LETTER WA" },
    { 0x0FAE, "TIBETAN SUBJOINED LETTER ZHA" },
    { 0x0FAF, "TIBETAN SUBJOINED LETTER ZA" },
    { 0x0FB0, "TIBETAN SUBJOINED LETTER -A" },
    { 0x0FB1, "TIBETAN SUBJOINED LETTER YA" },
    { 0x0FB2, "TIBETAN SUBJOINED LETTER RA" },
    { 0x0FB3, "TIBETAN SUBJOINED LETTER LA" },
    { 0x0FB4, "TIBETAN SUBJOINED LETTER SHA" },
    { 0x0FB5, "TIBETAN SUBJOINED LETTER SSA" },
    { 0x0FB6, "TIBETAN SUBJOINED LETTER SA" },
    { 0x0FB7, "TIBETAN SUBJOINED LETTER HA" },
    { 0x0FB8, "TIBETAN SUBJOINED LETTER A" },
    { 0x0FB9, "TIBETAN SUBJOINED LETTER KSSA" },
    { 0x0FBA, "TIBETAN SUBJOINED LETTER FIXED-FORM WA" },
    { 0x0FBB, "TIBETAN SUBJOINED LETTER FIXED-FORM YA" },
    { 0x0FBC, "TIBETAN SUBJOINED LETTER FIXED-FORM RA" },
    { 0x0FBE, "TIBETAN KU RU KHA" },
    { 0x0FBF, "TIBETAN KU RU KHA BZHI MIG CAN" },
    { 0x0FC0, "TIBETAN CANTILLATION SIGN HEAVY BEAT" },
    { 0x0FC1, "TIBETAN CANTILLATION SIGN LIGHT BEAT" },
    { 0x0FC2, "TIBETAN CANTILLATION SIGN CANG TE-U" },
    { 0x0FC3, "TIBETAN CANTILLATION SIGN SBUB -CHAL" },
    { 0x0FC4, "TIBETAN SYMBOL DRIL BU" },
    { 0x0FC5, "TIBETAN SYMBOL RDO RJE" },
    { 0x0FC6, "TIBETAN SYMBOL PADMA GDAN" },
    { 0x0FC7, "TIBETAN SYMBOL RDO RJE RGYA GRAM" },
    { 0x0FC8, "TIBETAN SYMBOL PHUR PA" },
    { 0x0FC9, "TIBETAN SYMBOL NOR BU" },
    { 0x0FCA, "TIBETAN SYMBOL NOR BU NYIS -KHYIL" },
    { 0x0FCB, "TIBETAN SYMBOL NOR BU GSUM -KHYIL" },
    { 0x0FCC, "TIBETAN SYMBOL NOR BU BZHI -KHYIL" },
    { 0x0FCE, "TIBETAN SIGN RDEL NAG RDEL DKAR" },
    { 0x0FCF, "TIBETAN SIGN RDEL NAG GSUM" },
    { 0x0FD0, "TIBETAN MARK BSKA- SHOG GI MGO RGYAN" },
    { 0x0FD1, "TIBETAN MARK MNYAM YIG GI MGO RGYAN" },
    { 0x0FD2, "TIBETAN MARK NYIS TSHEG" },
    { 0x0FD3, "TIBETAN MARK INITIAL BRDA RNYING YIG MGO MDUN MA" },
    { 0x0FD4, "TIBETAN MARK CLOSING BRDA RNYING YIG MGO SGAB MA" },
    { 0x0FD5, "RIGHT-FACING SVASTI SIGN" },
    { 0x0FD6, "LEFT-FACING SVASTI SIGN" },
    { 0x0FD7, "RIGHT-FACING SVASTI SIGN WITH DOTS" },
    { 0x0FD8, "LEFT-FACING SVASTI SIGN WITH DOTS" },
    { 0x0FD9, "TIBETAN MARK LEADING MCHAN RTAGS" },
    { 0x0FDA, "TIBETAN MARK TRAILING MCHAN RTAGS" },
    { 0x1000, "MYANMAR LETTER KA" },
    { 0x1001, "MYANMAR LETTER KHA" },
    { 0x1002, "MYANMAR LETTER GA" },
    { 0x1003, "MYANMAR LETTER GHA" },
    { 0x1004, "MYANMAR LETTER NGA" },
    { 0x1005, "MYANMAR LETTER CA" },
    { 0x1006, "MYANMAR LETTER CHA" },
    { 0x1007, "MYANMAR LETTER JA" },
    { 0x1008, "MYANMAR LETTER JHA" },
    { 0x1009, "MYANMAR LETTER NYA" },
    { 0x100A, "MYANMAR LETTER NNYA" },
    { 0x100B, "MYANMAR LETTER TTA" },
    { 0x100C, "MYANMAR LETTER TTHA" },
    { 0x100D, "MYANMAR LETTER DDA" },
    { 0x100E, "MYANMAR LETTER DDHA" },
    { 0x100F, "MYANMAR LETTER NNA" },
    { 0x1010, "MYANMAR LETTER TA" },
    { 0x1011, "MYANMAR LETTER THA" },
    { 0x1012, "MYANMAR LETTER DA" },
    { 0x1013, "MYANMAR LETTER DHA" },
    { 0x1014, "MYANMAR LETTER NA" },
    { 0x1015, "MYANMAR LETTER PA" },
    { 0x1016, "MYANMAR LETTER PHA" },
    { 0x1017, "MYANMAR LETTER BA" },
    { 0x1018, "MYANMAR LETTER BHA" },
    { 0x1019, "MYANMAR LETTER MA" },
    { 0x101A, "MYANMAR LETTER YA" },
    { 0x101B, "MYANMAR LETTER RA" },
    { 0x101C, "MYANMAR LETTER LA" },
    { 0x101D, "MYANMAR LETTER WA" },
    { 0x101E, "MYANMAR LETTER SA" },
    { 0x101F, "MYANMAR LETTER HA" },
    { 0x1020, "MYANMAR LETTER LLA" },
    { 0x1021, "MYANMAR LETTER A" },
    { 0x1022, "MYANMAR LETTER SHAN A" },
    { 0x1023, "MYANMAR LETTER I" },
    { 0x1024, "MYANMAR LETTER II" },
    { 0x1025, "MYANMAR LETTER U" },
    { 0x1026, "MYANMAR LETTER UU" },
    { 0x1027, "MYANMAR LETTER E" },
    { 0x1028, "MYANMAR LETTER MON E" },
    { 0x1029, "MYANMAR LETTER O" },
    { 0x102A, "MYANMAR LETTER AU" },
    { 0x102B, "MYANMAR VOWEL SIGN TALL AA" },
    { 0x102C, "MYANMAR VOWEL SIGN AA" },
    { 0x102D, "MYANMAR VOWEL SIGN I" },
    { 0x102E, "MYANMAR VOWEL SIGN II" },
    { 0x102F, "MYANMAR VOWEL SIGN U" },
    { 0x1030, "MYANMAR VOWEL SIGN UU" },
    { 0x1031, "MYANMAR VOWEL SIGN E" },
    { 0x1032, "MYANMAR VOWEL SIGN AI" },
    { 0x1033, "MYANMAR VOWEL SIGN MON II" },
    { 0x1034, "MYANMAR VOWEL SIGN MON O" },
    { 0x1035, "MYANMAR VOWEL SIGN E ABOVE" },
    { 0x1036, "MYANMAR SIGN ANUSVARA" },
    { 0x1037, "MYANMAR SIGN DOT BELOW" },
    { 0x1038, "MYANMAR SIGN VISARGA" },
    { 0x1039, "MYANMAR SIGN VIRAMA" },
    { 0x103A, "MYANMAR SIGN ASAT" },
    { 0x103B, "MYANMAR CONSONANT SIGN MEDIAL YA" },
    { 0x103C, "MYANMAR CONSONANT SIGN MEDIAL RA" },
    { 0x103D, "MYANMAR CONSONANT SIGN MEDIAL WA" },
    { 0x103E, "MYANMAR CONSONANT SIGN MEDIAL HA" },
    { 0x103F, "MYANMAR LETTER GREAT SA" },
    { 0x1040, "MYANMAR DIGIT ZERO" },
    { 0x1041, "MYANMAR DIGIT ONE" },
    { 0x1042, "MYANMAR DIGIT TWO" },
    { 0x1043, "MYANMAR DIGIT THREE" },
    { 0x1044, "MYANMAR DIGIT FOUR" },
    { 0x1045, "MYANMAR DIGIT FIVE" },
    { 0x1046, "MYANMAR DIGIT SIX" },
    { 0x1047, "MYANMAR DIGIT SEVEN" },
    { 0x1048, "MYANMAR DIGIT EIGHT" },
    { 0x1049, "MYANMAR DIGIT NINE" },
    { 0x104A, "MYANMAR SIGN LITTLE SECTION" },
    { 0x104B, "MYANMAR SIGN SECTION" },
    { 0x104C, "MYANMAR SYMBOL LOCATIVE" },
    { 0x104D, "MYANMAR SYMBOL COMPLETED" },
    { 0x104E, "MYANMAR SYMBOL AFOREMENTIONED" },
    { 0x104F, "MYANMAR SYMBOL GENITIVE" },
    { 0x1050, "MYANMAR LETTER SHA" },
    { 0x1051, "MYANMAR LETTER SSA" },
    { 0x1052, "MYANMAR LETTER VOCALIC R" },
    { 0x1053, "MYANMAR LETTER VOCALIC RR" },
    { 0x1054, "MYANMAR LETTER VOCALIC L" },
    { 0x1055, "MYANMAR LETTER VOCALIC LL" },
    { 0x1056, "MYANMAR VOWEL SIGN VOCALIC R" },
    { 0x1057, "MYANMAR VOWEL SIGN VOCALIC RR" },
    { 0x1058, "MYANMAR VOWEL SIGN VOCALIC L" },
    { 0x1059, "MYANMAR VOWEL SIGN VOCALIC LL" },
    { 0x105A, "MYANMAR LETTER MON NGA" },
    { 0x105B, "MYANMAR LETTER MON JHA" },
    { 0x105C, "MYANMAR LETTER MON BBA" },
    { 0x105D, "MYANMAR LETTER MON BBE" },
    { 0x105E, "MYANMAR CONSONANT SIGN MON MEDIAL NA" },
    { 0x105F, "MYANMAR CONSONANT SIGN MON MEDIAL MA" },
    { 0x1060, "MYANMAR CONSONANT SIGN MON MEDIAL LA" },
    { 0x1061, "MYANMAR LETTER SGAW KAREN SHA" },
    { 0x1062, "MYANMAR VOWEL SIGN SGAW KAREN EU" },
    { 0x1063, "MYANMAR TONE MARK SGAW KAREN HATHI" },
    { 0x1064, "MYANMAR TONE MARK SGAW KAREN KE PHO" },
    { 0x1065, "MYANMAR LETTER WESTERN PWO KAREN THA" },
    { 0x1066, "MYANMAR LETTER WESTERN PWO KAREN PWA" },
    { 0x1067, "MYANMAR VOWEL SIGN WESTERN PWO KAREN EU" },
    { 0x1068, "MYANMAR VOWEL SIGN WESTERN PWO KAREN UE" },
    { 0x1069, "MYANMAR SIGN WESTERN PWO KAREN TONE-1" },
    { 0x106A, "MYANMAR SIGN WESTERN PWO KAREN TONE-2" },
    { 0x106B, "MYANMAR SIGN WESTERN PWO KAREN TONE-3" },
    { 0x106C, "MYANMAR SIGN WESTERN PWO KAREN TONE-4" },
    { 0x106D, "MYANMAR SIGN WESTERN PWO KAREN TONE-5" },
    { 0x106E, "MYANMAR LETTER EASTERN PWO KAREN NNA" },
    { 0x106F, "MYANMAR LETTER EASTERN PWO KAREN YWA" },
    { 0x1070, "MYANMAR LETTER EASTERN PWO KAREN GHWA" },
    { 0x1071, "MYANMAR VOWEL SIGN GEBA KAREN I" },
    { 0x1072, "MYANMAR VOWEL SIGN KAYAH OE" },
    { 0x1073, "MYANMAR VOWEL SIGN KAYAH U" },
    { 0x1074, "MYANMAR VOWEL SIGN KAYAH EE" },
    { 0x1075, "MYANMAR LETTER SHAN KA" },
    { 0x1076, "MYANMAR LETTER SHAN KHA" },
    { 0x1077, "MYANMAR LETTER SHAN GA" },
    { 0x1078, "MYANMAR LETTER SHAN CA" },
    { 0x1079, "MYANMAR LETTER SHAN ZA" },
    { 0x107A, "MYANMAR LETTER SHAN NYA" },
    { 0x107B, "MYANMAR LETTER SHAN DA" },
    { 0x107C, "MYANMAR LETTER SHAN NA" },
    { 0x107D, "MYANMAR LETTER SHAN PHA" },
    { 0x107E, "MYANMAR LETTER SHAN FA" },
    { 0x107F, "MYANMAR LETTER SHAN BA" },
    { 0x1080, "MYANMAR LETTER SHAN THA" },
    { 0x1081, "MYANMAR LETTER SHAN HA" },
    { 0x1082, "MYANMAR CONSONANT SIGN SHAN MEDIAL WA" },
    { 0x1083, "MYANMAR VOWEL SIGN SHAN AA" },
    { 0x1084, "MYANMAR VOWEL SIGN SHAN E" },
    { 0x1085, "MYANMAR VOWEL SIGN SHAN E ABOVE" },
    { 0x1086, "MYANMAR VOWEL SIGN SHAN FINAL Y" },
    { 0x1087, "MYANMAR SIGN SHAN TONE-2" },
    { 0x1088, "MYANMAR SIGN SHAN TONE-3" },
    { 0x1089, "MYANMAR SIGN SHAN TONE-5" },
    { 0x108A, "MYANMAR SIGN SHAN TONE-6" },
    { 0x108B, "MYANMAR SIGN SHAN COUNCIL TONE-2" },
    { 0x108C, "MYANMAR SIGN SHAN COUNCIL TONE-3" },
    { 0x108D, "MYANMAR SIGN SHAN COUNCIL EMPHATIC TONE" },
    { 0x108E, "MYANMAR LETTER RUMAI PALAUNG FA" },
    { 0x108F, "MYANMAR SIGN RUMAI PALAUNG TONE-5" },
    { 0x1090, "MYANMAR SHAN DIGIT ZERO" },
    { 0x1091, "MYANMAR SHAN DIGIT ONE" },
    { 0x1092, "MYANMAR SHAN DIGIT TWO" },
    { 0x1093, "MYANMAR SHAN DIGIT THREE" },
    { 0x1094, "MYANMAR SHAN DIGIT FOUR" },
    { 0x1095, "MYANMAR SHAN DIGIT FIVE" },
    { 0x1096, "MYANMAR SHAN DIGIT SIX" },
    { 0x1097, "MYANMAR SHAN DIGIT SEVEN" },
    { 0x1098, "MYANMAR SHAN DIGIT EIGHT" },
    { 0x1099, "MYANMAR SHAN DIGIT NINE" },
    { 0x109A, "MYANMAR SIGN KHAMTI TONE-1" },
    { 0x109B, "MYANMAR SIGN KHAMTI TONE-3" },
    { 0x109C, "MYANMAR VOWEL SIGN AITON A" },
    { 0x109D, "MYANMAR VOWEL SIGN AITON AI" },
    { 0x109E, "MYANMAR SYMBOL SHAN ONE" },
    { 0x109F, "MYANMAR SYMBOL SHAN EXCLAMATION" },
    { 0x10A0, "GEORGIAN CAPITAL LETTER AN" },
    { 0x10A1, "GEORGIAN CAPITAL LETTER BAN" },
    { 0x10A2, "GEORGIAN CAPITAL LETTER GAN" },
    { 0x10A3, "GEORGIAN CAPITAL LETTER DON" },
    { 0x10A4, "GEORGIAN CAPITAL LETTER EN" },
    { 0x10A5, "GEORGIAN CAPITAL LETTER VIN" },
    { 0x10A6, "GEORGIAN CAPITAL LETTER ZEN" },
    { 0x10A7, "GEORGIAN CAPITAL LETTER TAN" },
    { 0x10A8, "GEORGIAN CAPITAL LETTER IN" },
    { 0x10A9, "GEORGIAN CAPITAL LETTER KAN" },
    { 0x10AA, "GEORGIAN CAPITAL LETTER LAS" },
    { 0x10AB, "GEORGIAN CAPITAL LETTER MAN" },
    { 0x10AC, "GEORGIAN CAPITAL LETTER NAR" },
    { 0x10AD, "GEORGIAN CAPITAL LETTER ON" },
    { 0x10AE, "GEORGIAN CAPITAL LETTER PAR" },
    { 0x10AF, "GEORGIAN CAPITAL LETTER ZHAR" },
    { 0x10B0, "GEORGIAN CAPITAL LETTER RAE" },
    { 0x10B1, "GEORGIAN CAPITAL LETTER SAN" },
    { 0x10B2, "GEORGIAN CAPITAL LETTER TAR" },
    { 0x10B3, "GEORGIAN CAPITAL LETTER UN" },
    { 0x10B4, "GEORGIAN CAPITAL LETTER PHAR" },
    { 0x10B5, "GEORGIAN CAPITAL LETTER KHAR" },
    { 0x10B6, "GEORGIAN CAPITAL LETTER GHAN" },
    { 0x10B7, "GEORGIAN CAPITAL LETTER QAR" },
    { 0x10B8, "GEORGIAN CAPITAL LETTER SHIN" },
    { 0x10B9, "GEORGIAN CAPITAL LETTER CHIN" },
    { 0x10BA, "GEORGIAN CAPITAL LETTER CAN" },
    { 0x10BB, "GEORGIAN CAPITAL LETTER JIL" },
    { 0x10BC, "GEORGIAN CAPITAL LETTER CIL" },
    { 0x10BD, "GEORGIAN CAPITAL LETTER CHAR" },
    { 0x10BE, "GEORGIAN CAPITAL LETTER XAN" },
    { 0x10BF, "GEORGIAN CAPITAL LETTER JHAN" },
    { 0x10C0, "GEORGIAN CAPITAL LETTER HAE" },
    { 0x10C1, "GEORGIAN CAPITAL LETTER HE" },
    { 0x10C2, "GEORGIAN CAPITAL LETTER HIE" },
    { 0x10C3, "GEORGIAN CAPITAL LETTER WE" },
    { 0x10C4, "GEORGIAN CAPITAL LETTER HAR" },
    { 0x10C5, "GEORGIAN CAPITAL LETTER HOE" },
    { 0x10C7, "GEORGIAN CAPITAL LETTER YN" },
    { 0x10CD, "GEORGIAN CAPITAL LETTER AEN" },
    { 0x10D0, "GEORGIAN LETTER AN" },
    { 0x10D1, "GEORGIAN LETTER BAN" },
    { 0x10D2, "GEORGIAN LETTER GAN" },
    { 0x10D3, "GEORGIAN LETTER DON" },
    { 0x10D4, "GEORGIAN LETTER EN" },
    { 0x10D5, "GEORGIAN LETTER VIN" },
    { 0x10D6, "GEORGIAN LETTER ZEN" },
    { 0x10D7, "GEORGIAN LETTER TAN" },
    { 0x10D8, "GEORGIAN LETTER IN" },
    { 0x10D9, "GEORGIAN LETTER KAN" },
    { 0x10DA, "GEORGIAN LETTER LAS" },
    { 0x10DB, "GEORGIAN LETTER MAN" },
    { 0x10DC, "GEORGIAN LETTER NAR" },
    { 0x10DD, "GEORGIAN LETTER ON" },
    { 0x10DE, "GEORGIAN LETTER PAR" },
    { 0x10DF, "GEORGIAN LETTER ZHAR" },
    { 0x10E0, "GEORGIAN LETTER RAE" },
    { 0x10E1, "GEORGIAN LETTER SAN" },
    { 0x10E2, "GEORGIAN LETTER TAR" },
    { 0x10E3, "GEORGIAN LETTER UN" },
    { 0x10E4, "GEORGIAN LETTER PHAR" },
    { 0x10E5, "GEORGIAN LETTER KHAR" },
    { 0x10E6, "GEORGIAN LETTER GHAN" },
    { 0x10E7, "GEORGIAN LETTER QAR" },
    { 0x10E8, "GEORGIAN LETTER SHIN" },
    { 0x10E9, "GEORGIAN LETTER CHIN" },
    { 0x10EA, "GEORGIAN LETTER CAN" },
    { 0x10EB, "GEORGIAN LETTER JIL" },
    { 0x10EC, "GEORGIAN LETTER CIL" },
    { 0x10ED, "GEORGIAN LETTER CHAR" },
    { 0x10EE, "GEORGIAN LETTER XAN" },
    { 0x10EF, "GEORGIAN LETTER JHAN" },
    { 0x10F0, "GEORGIAN LETTER HAE" },
    { 0x10F1, "GEORGIAN LETTER HE" },
    { 0x10F2, "GEORGIAN LETTER HIE" },
    { 0x10F3, "GEORGIAN LETTER WE" },
    { 0x10F4, "GEORGIAN LETTER HAR" },
    { 0x10F5, "GEORGIAN LETTER HOE" },
    { 0x10F6, "GEORGIAN LETTER FI" },
    { 0x10F7, "GEORGIAN LETTER YN" },
    { 0x10F8, "GEORGIAN LETTER ELIFI" },
    { 0x10F9, "GEORGIAN LETTER TURNED GAN" },
    { 0x10FA, "GEORGIAN LETTER AIN" },
    { 0x10FB, "GEORGIAN PARAGRAPH SEPARATOR" },
    { 0x10FC, "MODIFIER LETTER GEORGIAN NAR" },
    { 0x10FD, "GEORGIAN LETTER AEN" },
    { 0x10FE, "GEORGIAN LETTER HARD SIGN" },
    { 0x10FF, "GEORGIAN LETTER LABIAL SIGN" },
    { 0x1100, "HANGUL CHOSEONG KIYEOK" },
    { 0x1101, "HANGUL CHOSEONG SSANGKIYEOK" },
    { 0x1102, "HANGUL CHOSEONG NIEUN" },
    { 0x1103, "HANGUL CHOSEONG TIKEUT" },
    { 0x1104, "HANGUL CHOSEONG SSANGTIKEUT" },
    { 0x1105, "HANGUL CHOSEONG RIEUL" },
    { 0x1106, "HANGUL CHOSEONG MIEUM" },
    { 0x1107, "HANGUL CHOSEONG PIEUP" },
    { 0x1108, "HANGUL CHOSEONG SSANGPIEUP" },
    { 0x1109, "HANGUL CHOSEONG SIOS" },
    { 0x110A, "HANGUL CHOSEONG SSANGSIOS" },
    { 0x110B, "HANGUL CHOSEONG IEUNG" },
    { 0x110C, "HANGUL CHOSEONG CIEUC" },
    { 0x110D, "HANGUL CHOSEONG SSANGCIEUC" },
    { 0x110E, "HANGUL CHOSEONG CHIEUCH" },
    { 0x110F, "HANGUL CHOSEONG KHIEUKH" },
    { 0x1110, "HANGUL CHOSEONG THIEUTH" },
    { 0x1111, "HANGUL CHOSEONG PHIEUPH" },
    { 0x1112, "HANGUL CHOSEONG HIEUH" },
    { 0x1113, "HANGUL CHOSEONG NIEUN-KIYEOK" },
    { 0x1114, "HANGUL CHOSEONG SSANGNIEUN" },
    { 0x1115, "HANGUL CHOSEONG NIEUN-TIKEUT" },
    { 0x1116, "HANGUL CHOSEONG NIEUN-PIEUP" },
    { 0x1117, "HANGUL CHOSEONG TIKEUT-KIYEOK" },
    { 0x1118, "HANGUL CHOSEONG RIEUL-NIEUN" },
    { 0x1119, "HANGUL CHOSEONG SSANGRIEUL" },
    { 0x111A, "HANGUL CHOSEONG RIEUL-HIEUH" },
    { 0x111B, "HANGUL CHOSEONG KAPYEOUNRIEUL" },
    { 0x111C, "HANGUL CHOSEONG MIEUM-PIEUP" },
    { 0x111D, "HANGUL CHOSEONG KAPYEOUNMIEUM" },
    { 0x111E, "HANGUL CHOSEONG PIEUP-KIYEOK" },
    { 0x111F, "HANGUL CHOSEONG PIEUP-NIEUN" },
    { 0x1120, "HANGUL CHOSEONG PIEUP-TIKEUT" },
    { 0x1121, "HANGUL CHOSEONG PIEUP-SIOS" },
    { 0x1122, "HANGUL CHOSEONG PIEUP-SIOS-KIYEOK" },
    { 0x1123, "HANGUL CHOSEONG PIEUP-SIOS-TIKEUT" },
    { 0x1124, "HANGUL CHOSEONG PIEUP-SIOS-PIEUP" },
    { 0x1125, "HANGUL CHOSEONG PIEUP-SSANGSIOS" },
    { 0x1126, "HANGUL CHOSEONG PIEUP-SIOS-CIEUC" },
    { 0x1127, "HANGUL CHOSEONG PIEUP-CIEUC" },
    { 0x1128, "HANGUL CHOSEONG PIEUP-CHIEUCH" },
    { 0x1129, "HANGUL CHOSEONG PIEUP-THIEUTH" },
    { 0x112A, "HANGUL CHOSEONG PIEUP-PHIEUPH" },
    { 0x112B, "HANGUL CHOSEONG KAPYEOUNPIEUP" },
    { 0x112C, "HANGUL CHOSEONG KAPYEOUNSSANGPIEUP" },
    { 0x112D, "HANGUL CHOSEONG SIOS-KIYEOK" },
    { 0x112E, "HANGUL CHOSEONG SIOS-NIEUN" },
    { 0x112F, "HANGUL CHOSEONG SIOS-TIKEUT" },
    { 0x1130, "HANGUL CHOSEONG SIOS-RIEUL" },
    { 0x1131, "HANGUL CHOSEONG SIOS-MIEUM" },
    { 0x1132, "HANGUL CHOSEONG SIOS-PIEUP" },
    { 0x1133, "HANGUL CHOSEONG SIOS-PIEUP-KIYEOK" },
    { 0x1134, "HANGUL CHOSEONG SIOS-SSANGSIOS" },
    { 0x1135, "HANGUL CHOSEONG SIOS-IEUNG" },
    { 0x1136, "HANGUL CHOSEONG SIOS-CIEUC" },
    { 0x1137, "HANGUL CHOSEONG SIOS-CHIEUCH" },
    { 0x1138, "HANGUL CHOSEONG SIOS-KHIEUKH" },
    { 0x1139, "HANGUL CHOSEONG SIOS-THIEUTH" },
    { 0x113A, "HANGUL CHOSEONG SIOS-PHIEUPH" },
    { 0x113B, "HANGUL CHOSEONG SIOS-HIEUH" },
    { 0x113C, "HANGUL CHOSEONG CHITUEUMSIOS" },
    { 0x113D, "HANGUL CHOSEONG CHITUEUMSSANGSIOS" },
    { 0x113E, "HANGUL CHOSEONG CEONGCHIEUMSIOS" },
    { 0x113F, "HANGUL CHOSEONG CEONGCHIEUMSSANGSIOS" },
    { 0x1140, "HANGUL CHOSEONG PANSIOS" },
    { 0x1141, "HANGUL CHOSEONG IEUNG-KIYEOK" },
    { 0x1142, "HANGUL CHOSEONG IEUNG-TIKEUT" },
    { 0x1143, "HANGUL CHOSEONG IEUNG-MIEUM" },
    { 0x1144, "HANGUL CHOSEONG IEUNG-PIEUP" },
    { 0x1145, "HANGUL CHOSEONG IEUNG-SIOS" },
    { 0x1146, "HANGUL CHOSEONG IEUNG-PANSIOS" },
    { 0x1147, "HANGUL CHOSEONG SSANGIEUNG" },
    { 0x1148, "HANGUL CHOSEONG IEUNG-CIEUC" },
    { 0x1149, "HANGUL CHOSEONG IEUNG-CHIEUCH" },
    { 0x114A, "HANGUL CHOSEONG IEUNG-THIEUTH" },
    { 0x114B, "HANGUL CHOSEONG IEUNG-PHIEUPH" },
    { 0x114C, "HANGUL CHOSEONG YESIEUNG" },
    { 0x114D, "HANGUL CHOSEONG CIEUC-IEUNG" },
    { 0x114E, "HANGUL CHOSEONG CHITUEUMCIEUC" },
    { 0x114F, "HANGUL CHOSEONG CHITUEUMSSANGCIEUC" },
    { 0x1150, "HANGUL CHOSEONG CEONGCHIEUMCIEUC" },
    { 0x1151, "HANGUL CHOSEONG CEONGCHIEUMSSANGCIEUC" },
    { 0x1152, "HANGUL CHOSEONG CHIEUCH-KHIEUKH" },
    { 0x1153, "HANGUL CHOSEONG CHIEUCH-HIEUH" },
    { 0x1154, "HANGUL CHOSEONG CHITUEUMCHIEUCH" },
    { 0x1155, "HANGUL CHOSEONG CEONGCHIEUMCHIEUCH" },
    { 0x1156, "HANGUL CHOSEONG PHIEUPH-PIEUP" },
    { 0x1157, "HANGUL CHOSEONG KAPYEOUNPHIEUPH" },
    { 0x1158, "HANGUL CHOSEONG SSANGHIEUH" },
    { 0x1159, "HANGUL CHOSEONG YEORINHIEUH" },
    { 0x115A, "HANGUL CHOSEONG KIYEOK-TIKEUT" },
    { 0x115B, "HANGUL CHOSEONG NIEUN-SIOS" },
    { 0x115C, "HANGUL CHOSEONG NIEUN-CIEUC" },
    { 0x115D, "HANGUL CHOSEONG NIEUN-HIEUH" },
    { 0x115E, "HANGUL CHOSEONG TIKEUT-RIEUL" },
    { 0x115F, "HANGUL CHOSEONG FILLER" },
    { 0x1160, "HANGUL JUNGSEONG FILLER" },
    { 0x1161, "HANGUL JUNGSEONG A" },
    { 0x1162, "HANGUL JUNGSEONG AE" },
    { 0x1163, "HANGUL JUNGSEONG YA" },
    { 0x1164, "HANGUL JUNGSEONG YAE" },
    { 0x1165, "HANGUL JUNGSEONG EO" },
    { 0x1166, "HANGUL JUNGSEONG E" },
    { 0x1167, "HANGUL JUNGSEONG YEO" },
    { 0x1168, "HANGUL JUNGSEONG YE" },
    { 0x1169, "HANGUL JUNGSEONG O" },
    { 0x116A, "HANGUL JUNGSEONG WA" },
    { 0x116B, "HANGUL JUNGSEONG WAE" },
    { 0x116C, "HANGUL JUNGSEONG OE" },
    { 0x116D, "HANGUL JUNGSEONG YO" },
    { 0x116E, "HANGUL JUNGSEONG U" },
    { 0x116F, "HANGUL JUNGSEONG WEO" },
    { 0x1170, "HANGUL JUNGSEONG WE" },
    { 0x1171, "HANGUL JUNGSEONG WI" },
    { 0x1172, "HANGUL JUNGSEONG YU" },
    { 0x1173, "HANGUL JUNGSEONG EU" },
    { 0x1174, "HANGUL JUNGSEONG YI" },
    { 0x1175, "HANGUL JUNGSEONG I" },
    { 0x1176, "HANGUL JUNGSEONG A-O" },
    { 0x1177, "HANGUL JUNGSEONG A-U" },
    { 0x1178, "HANGUL JUNGSEONG YA-O" },
    { 0x1179, "HANGUL JUNGSEONG YA-YO" },
    { 0x117A, "HANGUL JUNGSEONG EO-O" },
    { 0x117B, "HANGUL JUNGSEONG EO-U" },
    { 0x117C, "HANGUL JUNGSEONG EO-EU" },
    { 0x117D, "HANGUL JUNGSEONG YEO-O" },
    { 0x117E, "HANGUL JUNGSEONG YEO-U" },
    { 0x117F, "HANGUL JUNGSEONG O-EO" },
    { 0x1180, "HANGUL JUNGSEONG O-E" },
    { 0x1181, "HANGUL JUNGSEONG O-YE" },
    { 0x1182, "HANGUL JUNGSEONG O-O" },
    { 0x1183, "HANGUL JUNGSEONG O-U" },
    { 0x1184, "HANGUL JUNGSEONG YO-YA" },
    { 0x1185, "HANGUL JUNGSEONG YO-YAE" },
    { 0x1186, "HANGUL JUNGSEONG YO-YEO" },
    { 0x1187, "HANGUL JUNGSEONG YO-O" },
    { 0x1188, "HANGUL JUNGSEONG YO-I" },
    { 0x1189, "HANGUL JUNGSEONG U-A" },
    { 0x118A, "HANGUL JUNGSEONG U-AE" },
    { 0x118B, "HANGUL JUNGSEONG U-EO-EU" },
    { 0x118C, "HANGUL JUNGSEONG U-YE" },
    { 0x118D, "HANGUL JUNGSEONG U-U" },
    { 0x118E, "HANGUL JUNGSEONG YU-A" },
    { 0x118F, "HANGUL JUNGSEONG YU-EO" },
    { 0x1190, "HANGUL JUNGSEONG YU-E" },
    { 0x1191, "HANGUL JUNGSEONG YU-YEO" },
    { 0x1192, "HANGUL JUNGSEONG YU-YE" },
    { 0x1193, "HANGUL JUNGSEONG YU-U" },
    { 0x1194, "HANGUL JUNGSEONG YU-I" },
    { 0x1195, "HANGUL JUNGSEONG EU-U" },
    { 0x1196, "HANGUL JUNGSEONG EU-EU" },
    { 0x1197, "HANGUL JUNGSEONG YI-U" },
    { 0x1198, "HANGUL JUNGSEONG I-A" },
    { 0x1199, "HANGUL JUNGSEONG I-YA" },
    { 0x119A, "HANGUL JUNGSEONG I-O" },
    { 0x119B, "HANGUL JUNGSEONG I-U" },
    { 0x119C, "HANGUL JUNGSEONG I-EU" },
    { 0x119D, "HANGUL JUNGSEONG I-ARAEA" },
    { 0x119E, "HANGUL JUNGSEONG ARAEA" },
    { 0x119F, "HANGUL JUNGSEONG ARAEA-EO" },
    { 0x11A0, "HANGUL JUNGSEONG ARAEA-U" },
    { 0x11A1, "HANGUL JUNGSEONG ARAEA-I" },
    { 0x11A2, "HANGUL JUNGSEONG SSANGARAEA" },
    { 0x11A3, "HANGUL JUNGSEONG A-EU" },
    { 0x11A4, "HANGUL JUNGSEONG YA-U" },
    { 0x11A5, "HANGUL JUNGSEONG YEO-YA" },
    { 0x11A6, "HANGUL JUNGSEONG O-YA" },
    { 0x11A7, "HANGUL JUNGSEONG O-YAE" },
    { 0x11A8, "HANGUL JONGSEONG KIYEOK" },
    { 0x11A9, "HANGUL JONGSEONG SSANGKIYEOK" },
    { 0x11AA, "HANGUL JONGSEONG KIYEOK-SIOS" },
    { 0x11AB, "HANGUL JONGSEONG NIEUN" },
    { 0x11AC, "HANGUL JONGSEONG NIEUN-CIEUC" },
    { 0x11AD, "HANGUL JONGSEONG NIEUN-HIEUH" },
    { 0x11AE, "HANGUL JONGSEONG TIKEUT" },
    { 0x11AF, "HANGUL JONGSEONG RIEUL" },
    { 0x11B0, "HANGUL JONGSEONG RIEUL-KIYEOK" },
    { 0x11B1, "HANGUL JONGSEONG RIEUL-MIEUM" },
    { 0x11B2, "HANGUL JONGSEONG RIEUL-PIEUP" },
    { 0x11B3, "HANGUL JONGSEONG RIEUL-SIOS" },
    { 0x11B4, "HANGUL JONGSEONG RIEUL-THIEUTH" },
    { 0x11B5, "HANGUL JONGSEONG RIEUL-PHIEUPH" },
    { 0x11B6, "HANGUL JONGSEONG RIEUL-HIEUH" },
    { 0x11B7, "HANGUL JONGSEONG MIEUM" },
    { 0x11B8, "HANGUL JONGSEONG PIEUP" },
    { 0x11B9, "HANGUL JONGSEONG PIEUP-SIOS" },
    { 0x11BA, "HANGUL JONGSEONG SIOS" },
    { 0x11BB, "HANGUL JONGSEONG SSANGSIOS" },
    { 0x11BC, "HANGUL JONGSEONG IEUNG" },
    { 0x11BD, "HANGUL JONGSEONG CIEUC" },
    { 0x11BE, "HANGUL JONGSEONG CHIEUCH" },
    { 0x11BF, "HANGUL JONGSEONG KHIEUKH" },
    { 0x11C0, "HANGUL JONGSEONG THIEUTH" },
    { 0x11C1, "HANGUL JONGSEONG PHIEUPH" },
    { 0x11C2, "HANGUL JONGSEONG HIEUH" },
    { 0x11C3, "HANGUL JONGSEONG KIYEOK-RIEUL" },
    { 0x11C4, "HANGUL JONGSEONG KIYEOK-SIOS-KIYEOK" },
    { 0x11C5, "HANGUL JONGSEONG NIEUN-KIYEOK" },
    { 0x11C6, "HANGUL JONGSEONG NIEUN-TIKEUT" },
    { 0x11C7, "HANGUL JONGSEONG NIEUN-SIOS" },
    { 0x11C8, "HANGUL JONGSEONG NIEUN-PANSIOS" },
    { 0x11C9, "HANGUL JONGSEONG NIEUN-THIEUTH" },
    { 0x11CA, "HANGUL JONGSEONG TIKEUT-KIYEOK" },
    { 0x11CB, "HANGUL JONGSEONG TIKEUT-RIEUL" },
    { 0x11CC, "HANGUL JONGSEONG RIEUL-KIYEOK-SIOS" },
    { 0x11CD, "HANGUL JONGSEONG RIEUL-NIEUN" },
    { 0x11CE, "HANGUL JONGSEONG RIEUL-TIKEUT" },
    { 0x11CF, "HANGUL JONGSEONG RIEUL-TIKEUT-HIEUH" },
    { 0x11D0, "HANGUL JONGSEONG SSANGRIEUL" },
    { 0x11D1, "HANGUL JONGSEONG RIEUL-MIEUM-KIYEOK" },
    { 0x11D2, "HANGUL JONGSEONG RIEUL-MIEUM-SIOS" },
    { 0x11D3, "HANGUL JONGSEONG RIEUL-PIEUP-SIOS" },
    { 0x11D4, "HANGUL JONGSEONG RIEUL-PIEUP-HIEUH" },
    { 0x11D5, "HANGUL JONGSEONG RIEUL-KAPYEOUNPIEUP" },
    { 0x11D6, "HANGUL JONGSEONG RIEUL-SSANGSIOS" },
    { 0x11D7, "HANGUL JONGSEONG RIEUL-PANSIOS" },
    { 0x11D8, "HANGUL JONGSEONG RIEUL-KHIEUKH" },
    { 0x11D9, "HANGUL JONGSEONG RIEUL-YEORINHIEUH" },
    { 0x11DA, "HANGUL JONGSEONG MIEUM-KIYEOK" },
    { 0x11DB, "HANGUL JONGSEONG MIEUM-RIEUL" },
    { 0x11DC, "HANGUL JONGSEONG MIEUM-PIEUP" },
    { 0x11DD, "HANGUL JONGSEONG MIEUM-SIOS" },
    { 0x11DE, "HANGUL JONGSEONG MIEUM-SSANGSIOS" },
    { 0x11DF, "HANGUL JONGSEONG MIEUM-PANSIOS" },
    { 0x11E0, "HANGUL JONGSEONG MIEUM-CHIEUCH" },
    { 0x11E1, "HANGUL JONGSEONG MIEUM-HIEUH" },
    { 0x11E2, "HANGUL JONGSEONG KAPYEOUNMIEUM" },
    { 0x11E3, "HANGUL JONGSEONG PIEUP-RIEUL" },
    { 0x11E4, "HANGUL JONGSEONG PIEUP-PHIEUPH" },
    { 0x11E5, "HANGUL JONGSEONG PIEUP-HIEUH" },
    { 0x11E6, "HANGUL JONGSEONG KAPYEOUNPIEUP" },
    { 0x11E7, "HANGUL JONGSEONG SIOS-KIYEOK" },
    { 0x11E8, "HANGUL JONGSEONG SIOS-TIKEUT" },
    { 0x11E9, "HANGUL JONGSEONG SIOS-RIEUL" },
    { 0x11EA, "HANGUL JONGSEONG SIOS-PIEUP" },
    { 0x11EB, "HANGUL JONGSEONG PANSIOS" },
    { 0x11EC, "HANGUL JONGSEONG IEUNG-KIYEOK" },
    { 0x11ED, "HANGUL JONGSEONG IEUNG-SSANGKIYEOK" },
    { 0x11EE, "HANGUL JONGSEONG SSANGIEUNG" },
    { 0x11EF, "HANGUL JONGSEONG IEUNG-KHIEUKH" },
    { 0x11F0, "HANGUL JONGSEONG YESIEUNG" },
    { 0x11F1, "HANGUL JONGSEONG YESIEUNG-SIOS" },
    { 0x11F2, "HANGUL JONGSEONG YESIEUNG-PANSIOS" },
    { 0x11F3, "HANGUL JONGSEONG PHIEUPH-PIEUP" },
    { 0x11F4, "HANGUL JONGSEONG KAPYEOUNPHIEUPH" },
    { 0x11F5, "HANGUL JONGSEONG HIEUH-NIEUN" },
    { 0x11F6, "HANGUL JONGSEONG HIEUH-RIEUL" },
    { 0x11F7, "HANGUL JONGSEONG HIEUH-MIEUM" },
    { 0x11F8, "HANGUL JONGSEONG HIEUH-PIEUP" },
    { 0x11F9, "HANGUL JONGSEONG YEORINHIEUH" },
    { 0x11FA, "HANGUL JONGSEONG KIYEOK-NIEUN" },
    { 0x11FB, "HANGUL JONGSEONG KIYEOK-PIEUP" },
    { 0x11FC, "HANGUL JONGSEONG KIYEOK-CHIEUCH" },
    { 0x11FD, "HANGUL JONGSEONG KIYEOK-KHIEUKH" },
    { 0x11FE, "HANGUL JONGSEONG KIYEOK-HIEUH" },
    { 0x11FF, "HANGUL JONGSEONG SSANGNIEUN" },
    { 0x1200, "ETHIOPIC SYLLABLE HA" },
    { 0x1201, "ETHIOPIC SYLLABLE HU" },
    { 0x1202, "ETHIOPIC SYLLABLE HI" },
    { 0x1203, "ETHIOPIC SYLLABLE HAA" },
    { 0x1204, "ETHIOPIC SYLLABLE HEE" },
    { 0x1205, "ETHIOPIC SYLLABLE HE" },
    { 0x1206, "ETHIOPIC SYLLABLE HO" },
    { 0x1207, "ETHIOPIC SYLLABLE HOA" },
    { 0x1208, "ETHIOPIC SYLLABLE LA" },
    { 0x1209, "ETHIOPIC SYLLABLE LU" },
    { 0x120A, "ETHIOPIC SYLLABLE LI" },
    { 0x120B, "ETHIOPIC SYLLABLE LAA" },
    { 0x120C, "ETHIOPIC SYLLABLE LEE" },
    { 0x120D, "ETHIOPIC SYLLABLE LE" },
    { 0x120E, "ETHIOPIC SYLLABLE LO" },
    { 0x120F, "ETHIOPIC SYLLABLE LWA" },
    { 0x1210, "ETHIOPIC SYLLABLE HHA" },
    { 0x1211, "ETHIOPIC SYLLABLE HHU" },
    { 0x1212, "ETHIOPIC SYLLABLE HHI" },
    { 0x1213, "ETHIOPIC SYLLABLE HHAA" },
    { 0x1214, "ETHIOPIC SYLLABLE HHEE" },
    { 0x1215, "ETHIOPIC SYLLABLE HHE" },
    { 0x1216, "ETHIOPIC SYLLABLE HHO" },
    { 0x1217, "ETHIOPIC SYLLABLE HHWA" },
    { 0x1218, "ETHIOPIC SYLLABLE MA" },
    { 0x1219, "ETHIOPIC SYLLABLE MU" },
    { 0x121A, "ETHIOPIC SYLLABLE MI" },
    { 0x121B, "ETHIOPIC SYLLABLE MAA" },
    { 0x121C, "ETHIOPIC SYLLABLE MEE" },
    { 0x121D, "ETHIOPIC SYLLABLE ME" },
    { 0x121E, "ETHIOPIC SYLLABLE MO" },
    { 0x121F, "ETHIOPIC SYLLABLE MWA" },
    { 0x1220, "ETHIOPIC SYLLABLE SZA" },
    { 0x1221, "ETHIOPIC SYLLABLE SZU" },
    { 0x1222, "ETHIOPIC SYLLABLE SZI" },
    { 0x1223, "ETHIOPIC SYLLABLE SZAA" },
    { 0x1224, "ETHIOPIC SYLLABLE SZEE" },
    { 0x1225, "ETHIOPIC SYLLABLE SZE" },
    { 0x1226, "ETHIOPIC SYLLABLE SZO" },
    { 0x1227, "ETHIOPIC SYLLABLE SZWA" },
    { 0x1228, "ETHIOPIC SYLLABLE RA" },
    { 0x1229, "ETHIOPIC SYLLABLE RU" },
    { 0x122A, "ETHIOPIC SYLLABLE RI" },
    { 0x122B, "ETHIOPIC SYLLABLE RAA" },
    { 0x122C, "ETHIOPIC SYLLABLE REE" },
    { 0x122D, "ETHIOPIC SYLLABLE RE" },
    { 0x122E, "ETHIOPIC SYLLABLE RO" },
    { 0x122F, "ETHIOPIC SYLLABLE RWA" },
    { 0x1230, "ETHIOPIC SYLLABLE SA" },
    { 0x1231, "ETHIOPIC SYLLABLE SU" },
    { 0x1232, "ETHIOPIC SYLLABLE SI" },
    { 0x1233, "ETHIOPIC SYLLABLE SAA" },
    { 0x1234, "ETHIOPIC SYLLABLE SEE" },
    { 0x1235, "ETHIOPIC SYLLABLE SE" },
    { 0x1236, "ETHIOPIC SYLLABLE SO" },
    { 0x1237, "ETHIOPIC SYLLABLE SWA" },
    { 0x1238, "ETHIOPIC SYLLABLE SHA" },
    { 0x1239, "ETHIOPIC SYLLABLE SHU" },
    { 0x123A, "ETHIOPIC SYLLABLE SHI" },
    { 0x123B, "ETHIOPIC SYLLABLE SHAA" },
    { 0x123C, "ETHIOPIC SYLLABLE SHEE" },
    { 0x123D, "ETHIOPIC SYLLABLE SHE" },
    { 0x123E, "ETHIOPIC SYLLABLE SHO" },
    { 0x123F, "ETHIOPIC SYLLABLE SHWA" },
    { 0x1240, "ETHIOPIC SYLLABLE QA" },
    { 0x1241, "ETHIOPIC SYLLABLE QU" },
    { 0x1242, "ETHIOPIC SYLLABLE QI" },
    { 0x1243, "ETHIOPIC SYLLABLE QAA" },
    { 0x1244, "ETHIOPIC SYLLABLE QEE" },
    { 0x1245, "ETHIOPIC SYLLABLE QE" },
    { 0x1246, "ETHIOPIC SYLLABLE QO" },
    { 0x1247, "ETHIOPIC SYLLABLE QOA" },
    { 0x1248, "ETHIOPIC SYLLABLE QWA" },
    { 0x124A, "ETHIOPIC SYLLABLE QWI" },
    { 0x124B, "ETHIOPIC SYLLABLE QWAA" },
    { 0x124C, "ETHIOPIC SYLLABLE QWEE" },
    { 0x124D, "ETHIOPIC SYLLABLE QWE" },
    { 0x1250, "ETHIOPIC SYLLABLE QHA" },
    { 0x1251, "ETHIOPIC SYLLABLE QHU" },
    { 0x1252, "ETHIOPIC SYLLABLE QHI" },
    { 0x1253, "ETHIOPIC SYLLABLE QHAA" },
    { 0x1254, "ETHIOPIC SYLLABLE QHEE" },
    { 0x1255, "ETHIOPIC SYLLABLE QHE" },
    { 0x1256, "ETHIOPIC SYLLABLE QHO" },
    { 0x1258, "ETHIOPIC SYLLABLE QHWA" },
    { 0x125A, "ETHIOPIC SYLLABLE QHWI" },
    { 0x125B, "ETHIOPIC SYLLABLE QHWAA" },
    { 0x125C, "ETHIOPIC SYLLABLE QHWEE" },
    { 0x125D, "ETHIOPIC SYLLABLE QHWE" },
    { 0x1260, "ETHIOPIC SYLLABLE BA" },
    { 0x1261, "ETHIOPIC SYLLABLE BU" },
    { 0x1262, "ETHIOPIC SYLLABLE BI" },
    { 0x1263, "ETHIOPIC SYLLABLE BAA" },
    { 0x1264, "ETHIOPIC SYLLABLE BEE" },
    { 0x1265, "ETHIOPIC SYLLABLE BE" },
    { 0x1266, "ETHIOPIC SYLLABLE BO" },
    { 0x1267, "ETHIOPIC SYLLABLE BWA" },
    { 0x1268, "ETHIOPIC SYLLABLE VA" },
    { 0x1269, "ETHIOPIC SYLLABLE VU" },
    { 0x126A, "ETHIOPIC SYLLABLE VI" },
    { 0x126B, "ETHIOPIC SYLLABLE VAA" },
    { 0x126C, "ETHIOPIC SYLLABLE VEE" },
    { 0x126D, "ETHIOPIC SYLLABLE VE" },
    { 0x126E, "ETHIOPIC SYLLABLE VO" },
    { 0x126F, "ETHIOPIC SYLLABLE VWA" },
    { 0x1270, "ETHIOPIC SYLLABLE TA" },
    { 0x1271, "ETHIOPIC SYLLABLE TU" },
    { 0x1272, "ETHIOPIC SYLLABLE TI" },
    { 0x1273, "ETHIOPIC SYLLABLE TAA" },
    { 0x1274, "ETHIOPIC SYLLABLE TEE" },
    { 0x1275, "ETHIOPIC SYLLABLE TE" },
    { 0x1276, "ETHIOPIC SYLLABLE TO" },
    { 0x1277, "ETHIOPIC SYLLABLE TWA" },
    { 0x1278, "ETHIOPIC SYLLABLE CA" },
    { 0x1279, "ETHIOPIC SYLLABLE CU" },
    { 0x127A, "ETHIOPIC SYLLABLE CI" },
    { 0x127B, "ETHIOPIC SYLLABLE CAA" },
    { 0x127C, "ETHIOPIC SYLLABLE CEE" },
    { 0x127D, "ETHIOPIC SYLLABLE CE" },
    { 0x127E, "ETHIOPIC SYLLABLE CO" },
    { 0x127F, "ETHIOPIC SYLLABLE CWA" },
    { 0x1280, "ETHIOPIC SYLLABLE XA" },
    { 0x1281, "ETHIOPIC SYLLABLE XU" },
    { 0x1282, "ETHIOPIC SYLLABLE XI" },
    { 0x1283, "ETHIOPIC SYLLABLE XAA" },
    { 0x1284, "ETHIOPIC SYLLABLE XEE" },
    { 0x1285, "ETHIOPIC SYLLABLE XE" },
    { 0x1286, "ETHIOPIC SYLLABLE XO" },
    { 0x1287, "ETHIOPIC SYLLABLE XOA" },
    { 0x1288, "ETHIOPIC SYLLABLE XWA" },
    { 0x128A, "ETHIOPIC SYLLABLE XWI" },
    { 0x128B, "ETHIOPIC SYLLABLE XWAA" },
    { 0x128C, "ETHIOPIC SYLLABLE XWEE" },
    { 0x128D, "ETHIOPIC SYLLABLE XWE" },
    { 0x1290, "ETHIOPIC SYLLABLE NA" },
    { 0x1291, "ETHIOPIC SYLLABLE NU" },
    { 0x1292, "ETHIOPIC SYLLABLE NI" },
    { 0x1293, "ETHIOPIC SYLLABLE NAA" },
    { 0x1294, "ETHIOPIC SYLLABLE NEE" },
    { 0x1295, "ETHIOPIC SYLLABLE NE" },
    { 0x1296, "ETHIOPIC SYLLABLE NO" },
    { 0x1297, "ETHIOPIC SYLLABLE NWA" },
    { 0x1298, "ETHIOPIC SYLLABLE NYA" },
    { 0x1299, "ETHIOPIC SYLLABLE NYU" },
    { 0x129A, "ETHIOPIC SYLLABLE NYI" },
    { 0x129B, "ETHIOPIC SYLLABLE NYAA" },
    { 0x129C, "ETHIOPIC SYLLABLE NYEE" },
    { 0x129D, "ETHIOPIC SYLLABLE NYE" },
    { 0x129E, "ETHIOPIC SYLLABLE NYO" },
    { 0x129F, "ETHIOPIC SYLLABLE NYWA" },
    { 0x12A0, "ETHIOPIC SYLLABLE GLOTTAL A" },
    { 0x12A1, "ETHIOPIC SYLLABLE GLOTTAL U" },
    { 0x12A2, "ETHIOPIC SYLLABLE GLOTTAL I" },
    { 0x12A3, "ETHIOPIC SYLLABLE GLOTTAL AA" },
    { 0x12A4, "ETHIOPIC SYLLABLE GLOTTAL EE" },
    { 0x12A5, "ETHIOPIC SYLLABLE GLOTTAL E" },
    { 0x12A6, "ETHIOPIC SYLLABLE GLOTTAL O" },
    { 0x12A7, "ETHIOPIC SYLLABLE GLOTTAL WA" },
    { 0x12A8, "ETHIOPIC SYLLABLE KA" },
    { 0x12A9, "ETHIOPIC SYLLABLE KU" },
    { 0x12AA, "ETHIOPIC SYLLABLE KI" },
    { 0x12AB, "ETHIOPIC SYLLABLE KAA" },
    { 0x12AC, "ETHIOPIC SYLLABLE KEE" },
    { 0x12AD, "ETHIOPIC SYLLABLE KE" },
    { 0x12AE, "ETHIOPIC SYLLABLE KO" },
    { 0x12AF, "ETHIOPIC SYLLABLE KOA" },
    { 0x12B0, "ETHIOPIC SYLLABLE KWA" },
    { 0x12B2, "ETHIOPIC SYLLABLE KWI" },
    { 0x12B3, "ETHIOPIC SYLLABLE KWAA" },
    { 0x12B4, "ETHIOPIC SYLLABLE KWEE" },
    { 0x12B5, "ETHIOPIC SYLLABLE KWE" },
    { 0x12B8, "ETHIOPIC SYLLABLE KXA" },
    { 0x12B9, "ETHIOPIC SYLLABLE KXU" },
    { 0x12BA, "ETHIOPIC SYLLABLE KXI" },
    { 0x12BB, "ETHIOPIC SYLLABLE KXAA" },
    { 0x12BC, "ETHIOPIC SYLLABLE KXEE" },
    { 0x12BD, "ETHIOPIC SYLLABLE KXE" },
    { 0x12BE, "ETHIOPIC SYLLABLE KXO" },
    { 0x12C0, "ETHIOPIC SYLLABLE KXWA" },
    { 0x12C2, "ETHIOPIC SYLLABLE KXWI" },
    { 0x12C3, "ETHIOPIC SYLLABLE KXWAA" },
    { 0x12C4, "ETHIOPIC SYLLABLE KXWEE" },
    { 0x12C5, "ETHIOPIC SYLLABLE KXWE" },
    { 0x12C8, "ETHIOPIC SYLLABLE WA" },
    { 0x12C9, "ETHIOPIC SYLLABLE WU" },
    { 0x12CA, "ETHIOPIC SYLLABLE WI" },
    { 0x12CB, "ETHIOPIC SYLLABLE WAA" },
    { 0x12CC, "ETHIOPIC SYLLABLE WEE" },
    { 0x12CD, "ETHIOPIC SYLLABLE WE" },
    { 0x12CE, "ETHIOPIC SYLLABLE WO" },
    { 0x12CF, "ETHIOPIC SYLLABLE WOA" },
    { 0x12D0, "ETHIOPIC SYLLABLE PHARYNGEAL A" },
    { 0x12D1, "ETHIOPIC SYLLABLE PHARYNGEAL U" },
    { 0x12D2, "ETHIOPIC SYLLABLE PHARYNGEAL I" },
    { 0x12D3, "ETHIOPIC SYLLABLE PHARYNGEAL AA" },
    { 0x12D4, "ETHIOPIC SYLLABLE PHARYNGEAL EE" },
    { 0x12D5, "ETHIOPIC SYLLABLE PHARYNGEAL E" },
    { 0x12D6, "ETHIOPIC SYLLABLE PHARYNGEAL O" },
    { 0x12D8, "ETHIOPIC SYLLABLE ZA" },
    { 0x12D9, "ETHIOPIC SYLLABLE ZU" },
    { 0x12DA, "ETHIOPIC SYLLABLE ZI" },
    { 0x12DB, "ETHIOPIC SYLLABLE ZAA" },
    { 0x12DC, "ETHIOPIC SYLLABLE ZEE" },
    { 0x12DD, "ETHIOPIC SYLLABLE ZE" },
    { 0x12DE, "ETHIOPIC SYLLABLE ZO" },
    { 0x12DF, "ETHIOPIC SYLLABLE ZWA" },
    { 0x12E0, "ETHIOPIC SYLLABLE ZHA" },
    { 0x12E1, "ETHIOPIC SYLLABLE ZHU" },
    { 0x12E2, "ETHIOPIC SYLLABLE ZHI" },
    { 0x12E3, "ETHIOPIC SYLLABLE ZHAA" },
    { 0x12E4, "ETHIOPIC SYLLABLE ZHEE" },
    { 0x12E5, "ETHIOPIC SYLLABLE ZHE" },
    { 0x12E6, "ETHIOPIC SYLLABLE ZHO" },
    { 0x12E7, "ETHIOPIC SYLLABLE ZHWA" },
    { 0x12E8, "ETHIOPIC SYLLABLE YA" },
    { 0x12E9, "ETHIOPIC SYLLABLE YU" },
    { 0x12EA, "ETHIOPIC SYLLABLE YI" },
    { 0x12EB, "ETHIOPIC SYLLABLE YAA" },
    { 0x12EC, "ETHIOPIC SYLLABLE YEE" },
    { 0x12ED, "ETHIOPIC SYLLABLE YE" },
    { 0x12EE, "ETHIOPIC SYLLABLE YO" },
    { 0x12EF, "ETHIOPIC SYLLABLE YOA" },
    { 0x12F0, "ETHIOPIC SYLLABLE DA" },
    { 0x12F1, "ETHIOPIC SYLLABLE DU" },
    { 0x12F2, "ETHIOPIC SYLLABLE DI" },
    { 0x12F3, "ETHIOPIC SYLLABLE DAA" },
    { 0x12F4, "ETHIOPIC SYLLABLE DEE" },
    { 0x12F5, "ETHIOPIC SYLLABLE DE" },
    { 0x12F6, "ETHIOPIC SYLLABLE DO" },
    { 0x12F7, "ETHIOPIC SYLLABLE DWA" },
    { 0x12F8, "ETHIOPIC SYLLABLE DDA" },
    { 0x12F9, "ETHIOPIC SYLLABLE DDU" },
    { 0x12FA, "ETHIOPIC SYLLABLE DDI" },
    { 0x12FB, "ETHIOPIC SYLLABLE DDAA" },
    { 0x12FC, "ETHIOPIC SYLLABLE DDEE" },
    { 0x12FD, "ETHIOPIC SYLLABLE DDE" },
    { 0x12FE, "ETHIOPIC SYLLABLE DDO" },
    { 0x12FF, "ETHIOPIC SYLLABLE DDWA" },
    { 0x1300, "ETHIOPIC SYLLABLE JA" },
    { 0x1301, "ETHIOPIC SYLLABLE JU" },
    { 0x1302, "ETHIOPIC SYLLABLE JI" },
    { 0x1303, "ETHIOPIC SYLLABLE JAA" },
    { 0x1304, "ETHIOPIC SYLLABLE JEE" },
    { 0x1305, "ETHIOPIC SYLLABLE JE" },
    { 0x1306, "ETHIOPIC SYLLABLE JO" },
    { 0x1307, "ETHIOPIC SYLLABLE JWA" },
    { 0x1308, "ETHIOPIC SYLLABLE GA" },
    { 0x1309, "ETHIOPIC SYLLABLE GU" },
    { 0x130A, "ETHIOPIC SYLLABLE GI" },
    { 0x130B, "ETHIOPIC SYLLABLE GAA" },
    { 0x130C, "ETHIOPIC SYLLABLE GEE" },
    { 0x130D, "ETHIOPIC SYLLABLE GE" },
    { 0x130E, "ETHIOPIC SYLLABLE GO" },
    { 0x130F, "ETHIOPIC SYLLABLE GOA" },
    { 0x1310, "ETHIOPIC SYLLABLE GWA" },
    { 0x1312, "ETHIOPIC SYLLABLE GWI" },
    { 0x1313, "ETHIOPIC SYLLABLE GWAA" },
    { 0x1314, "ETHIOPIC SYLLABLE GWEE" },
    { 0x1315, "ETHIOPIC SYLLABLE GWE" },
    { 0x1318, "ETHIOPIC SYLLABLE GGA" },
    { 0x1319, "ETHIOPIC SYLLABLE GGU" },
    { 0x131A, "ETHIOPIC SYLLABLE GGI" },
    { 0x131B, "ETHIOPIC SYLLABLE GGAA" },
    { 0x131C, "ETHIOPIC SYLLABLE GGEE" },
    { 0x131D, "ETHIOPIC SYLLABLE GGE" },
    { 0x131E, "ETHIOPIC SYLLABLE GGO" },
    { 0x131F, "ETHIOPIC SYLLABLE GGWAA" },
    { 0x1320, "ETHIOPIC SYLLABLE THA" },
    { 0x1321, "ETHIOPIC SYLLABLE THU" },
    { 0x1322, "ETHIOPIC SYLLABLE THI" },
    { 0x1323, "ETHIOPIC SYLLABLE THAA" },
    { 0x1324, "ETHIOPIC SYLLABLE THEE" },
    { 0x1325, "ETHIOPIC SYLLABLE THE" },
    { 0x1326, "ETHIOPIC SYLLABLE THO" },
    { 0x1327, "ETHIOPIC SYLLABLE THWA" },
    { 0x1328, "ETHIOPIC SYLLABLE CHA" },
    { 0x1329, "ETHIOPIC SYLLABLE CHU" },
    { 0x132A, "ETHIOPIC SYLLABLE CHI" },
    { 0x132B, "ETHIOPIC SYLLABLE CHAA" },
    { 0x132C, "ETHIOPIC SYLLABLE CHEE" },
    { 0x132D, "ETHIOPIC SYLLABLE CHE" },
    { 0x132E, "ETHIOPIC SYLLABLE CHO" },
    { 0x132F, "ETHIOPIC SYLLABLE CHWA" },
    { 0x1330, "ETHIOPIC SYLLABLE PHA" },
    { 0x1331, "ETHIOPIC SYLLABLE PHU" },
    { 0x1332, "ETHIOPIC SYLLABLE PHI" },
    { 0x1333, "ETHIOPIC SYLLABLE PHAA" },
    { 0x1334, "ETHIOPIC SYLLABLE PHEE" },
    { 0x1335, "ETHIOPIC SYLLABLE PHE" },
    { 0x1336, "ETHIOPIC SYLLABLE PHO" },
    { 0x1337, "ETHIOPIC SYLLABLE PHWA" },
    { 0x1338, "ETHIOPIC SYLLABLE TSA" },
    { 0x1339, "ETHIOPIC SYLLABLE TSU" },
    { 0x133A, "ETHIOPIC SYLLABLE TSI" },
    { 0x133B, "ETHIOPIC SYLLABLE TSAA" },
    { 0x133C, "ETHIOPIC SYLLABLE TSEE" },
    { 0x133D, "ETHIOPIC SYLLABLE TSE" },
    { 0x133E, "ETHIOPIC SYLLABLE TSO" },
    { 0x133F, "ETHIOPIC SYLLABLE TSWA" },
    { 0x1340, "ETHIOPIC SYLLABLE TZA" },
    { 0x1341, "ETHIOPIC SYLLABLE TZU" },
    { 0x1342, "ETHIOPIC SYLLABLE TZI" },
    { 0x1343, "ETHIOPIC SYLLABLE TZAA" },
    { 0x1344, "ETHIOPIC SYLLABLE TZEE" },
    { 0x1345, "ETHIOPIC SYLLABLE TZE" },
    { 0x1346, "ETHIOPIC SYLLABLE TZO" },
    { 0x1347, "ETHIOPIC SYLLABLE TZOA" },
    { 0x1348, "ETHIOPIC SYLLABLE FA" },
    { 0x1349, "ETHIOPIC SYLLABLE FU" },
    { 0x134A, "ETHIOPIC SYLLABLE FI" },
    { 0x134B, "ETHIOPIC SYLLABLE FAA" },
    { 0x134C, "ETHIOPIC SYLLABLE FEE" },
    { 0x134D, "ETHIOPIC SYLLABLE FE" },
    { 0x134E, "ETHIOPIC SYLLABLE FO" },
    { 0x134F, "ETHIOPIC SYLLABLE FWA" },
    { 0x1350, "ETHIOPIC SYLLABLE PA" },
    { 0x1351, "ETHIOPIC SYLLABLE PU" },
    { 0x1352, "ETHIOPIC SYLLABLE PI" },
    { 0x1353, "ETHIOPIC SYLLABLE PAA" },
    { 0x1354, "ETHIOPIC SYLLABLE PEE" },
    { 0x1355, "ETHIOPIC SYLLABLE PE" },
    { 0x1356, "ETHIOPIC SYLLABLE PO" },
    { 0x1357, "ETHIOPIC SYLLABLE PWA" },
    { 0x1358, "ETHIOPIC SYLLABLE RYA" },
    { 0x1359, "ETHIOPIC SYLLABLE MYA" },
    { 0x135A, "ETHIOPIC SYLLABLE FYA" },
    { 0x135D, "ETHIOPIC COMBINING GEMINATION AND VOWEL LENGTH MARK" },
    { 0x135E, "ETHIOPIC COMBINING VOWEL LENGTH MARK" },
    { 0x135F, "ETHIOPIC COMBINING GEMINATION MARK" },
    { 0x1360, "ETHIOPIC SECTION MARK" },
    { 0x1361, "ETHIOPIC WORDSPACE" },
    { 0x1362, "ETHIOPIC FULL STOP" },
    { 0x1363, "ETHIOPIC COMMA" },
    { 0x1364, "ETHIOPIC SEMICOLON" },
    { 0x1365, "ETHIOPIC COLON" },
    { 0x1366, "ETHIOPIC PREFACE COLON" },
    { 0x1367, "ETHIOPIC QUESTION MARK" },
    { 0x1368, "ETHIOPIC PARAGRAPH SEPARATOR" },
    { 0x1369, "ETHIOPIC DIGIT ONE" },
    { 0x136A, "ETHIOPIC DIGIT TWO" },
    { 0x136B, "ETHIOPIC DIGIT THREE" },
    { 0x136C, "ETHIOPIC DIGIT FOUR" },
    { 0x136D, "ETHIOPIC DIGIT FIVE" },
    { 0x136E, "ETHIOPIC DIGIT SIX" },
    { 0x136F, "ETHIOPIC DIGIT SEVEN" },
    { 0x1370, "ETHIOPIC DIGIT EIGHT" },
    { 0x1371, "ETHIOPIC DIGIT NINE" },
    { 0x1372, "ETHIOPIC NUMBER TEN" },
    { 0x1373, "ETHIOPIC NUMBER TWENTY" },
    { 0x1374, "ETHIOPIC NUMBER THIRTY" },
    { 0x1375, "ETHIOPIC NUMBER FORTY" },
    { 0x1376, "ETHIOPIC NUMBER FIFTY" },
    { 0x1377, "ETHIOPIC NUMBER SIXTY" },
    { 0x1378, "ETHIOPIC NUMBER SEVENTY" },
    { 0x1379, "ETHIOPIC NUMBER EIGHTY" },
    { 0x137A, "ETHIOPIC NUMBER NINETY" },
    { 0x137B, "ETHIOPIC NUMBER HUNDRED" },
    { 0x137C, "ETHIOPIC NUMBER TEN THOUSAND" },
    { 0x1380, "ETHIOPIC SYLLABLE SEBATBEIT MWA" },
    { 0x1381, "ETHIOPIC SYLLABLE MWI" },
    { 0x1382, "ETHIOPIC SYLLABLE MWEE" },
    { 0x1383, "ETHIOPIC SYLLABLE MWE" },
    { 0x1384, "ETHIOPIC SYLLABLE SEBATBEIT BWA" },
    { 0x1385, "ETHIOPIC SYLLABLE BWI" },
    { 0x1386, "ETHIOPIC SYLLABLE BWEE" },
    { 0x1387, "ETHIOPIC SYLLABLE BWE" },
    { 0x1388, "ETHIOPIC SYLLABLE SEBATBEIT FWA" },
    { 0x1389, "ETHIOPIC SYLLABLE FWI" },
    { 0x138A, "ETHIOPIC SYLLABLE FWEE" },
    { 0x138B, "ETHIOPIC SYLLABLE FWE" },
    { 0x138C, "ETHIOPIC SYLLABLE SEBATBEIT PWA" },
    { 0x138D, "ETHIOPIC SYLLABLE PWI" },
    { 0x138E, "ETHIOPIC SYLLABLE PWEE" },
    { 0x138F, "ETHIOPIC SYLLABLE PWE" },
    { 0x1390, "ETHIOPIC TONAL MARK YIZET" },
    { 0x1391, "ETHIOPIC TONAL MARK DERET" },
    { 0x1392, "ETHIOPIC TONAL MARK RIKRIK" },
    { 0x1393, "ETHIOPIC TONAL MARK SHORT RIKRIK" },
    { 0x1394, "ETHIOPIC TONAL MARK DIFAT" },
    { 0x1395, "ETHIOPIC TONAL MARK KENAT" },
    { 0x1396, "ETHIOPIC TONAL MARK CHIRET" },
    { 0x1397, "ETHIOPIC TONAL MARK HIDET" },
    { 0x1398, "ETHIOPIC TONAL MARK DERET-HIDET" },
    { 0x1399, "ETHIOPIC TONAL MARK KURT" },
    { 0x13A0, "CHEROKEE LETTER A" },
    { 0x13A1, "CHEROKEE LETTER E" },
    { 0x13A2, "CHEROKEE LETTER I" },
    { 0x13A3, "CHEROKEE LETTER O" },
    { 0x13A4, "CHEROKEE LETTER U" },
    { 0x13A5, "CHEROKEE LETTER V" },
    { 0x13A6, "CHEROKEE LETTER GA" },
    { 0x13A7, "CHEROKEE LETTER KA" },
    { 0x13A8, "CHEROKEE LETTER GE" },
    { 0x13A9, "CHEROKEE LETTER GI" },
    { 0x13AA, "CHEROKEE LETTER GO" },
    { 0x13AB, "CHEROKEE LETTER GU" },
    { 0x13AC, "CHEROKEE LETTER GV" },
    { 0x13AD, "CHEROKEE LETTER HA" },
    { 0x13AE, "CHEROKEE LETTER HE" },
    { 0x13AF, "CHEROKEE LETTER HI" },
    { 0x13B0, "CHEROKEE LETTER HO" },
    { 0x13B1, "CHEROKEE LETTER HU" },
    { 0x13B2, "CHEROKEE LETTER HV" },
    { 0x13B3, "CHEROKEE LETTER LA" },
    { 0x13B4, "CHEROKEE LETTER LE" },
    { 0x13B5, "CHEROKEE LETTER LI" },
    { 0x13B6, "CHEROKEE LETTER LO" },
    { 0x13B7, "CHEROKEE LETTER LU" },
    { 0x13B8, "CHEROKEE LETTER LV" },
    { 0x13B9, "CHEROKEE LETTER MA" },
    { 0x13BA, "CHEROKEE LETTER ME" },
    { 0x13BB, "CHEROKEE LETTER MI" },
    { 0x13BC, "CHEROKEE LETTER MO" },
    { 0x13BD, "CHEROKEE LETTER MU" },
    { 0x13BE, "CHEROKEE LETTER NA" },
    { 0x13BF, "CHEROKEE LETTER HNA" },
    { 0x13C0, "CHEROKEE LETTER NAH" },
    { 0x13C1, "CHEROKEE LETTER NE" },
    { 0x13C2, "CHEROKEE LETTER NI" },
    { 0x13C3, "CHEROKEE LETTER NO" },
    { 0x13C4, "CHEROKEE LETTER NU" },
    { 0x13C5, "CHEROKEE LETTER NV" },
    { 0x13C6, "CHEROKEE LETTER QUA" },
    { 0x13C7, "CHEROKEE LETTER QUE" },
    { 0x13C8, "CHEROKEE LETTER QUI" },
    { 0x13C9, "CHEROKEE LETTER QUO" },
    { 0x13CA, "CHEROKEE LETTER QUU" },
    { 0x13CB, "CHEROKEE LETTER QUV" },
    { 0x13CC, "CHEROKEE LETTER SA" },
    { 0x13CD, "CHEROKEE LETTER S" },
    { 0x13CE, "CHEROKEE LETTER SE" },
    { 0x13CF, "CHEROKEE LETTER SI" },
    { 0x13D0, "CHEROKEE LETTER SO" },
    { 0x13D1, "CHEROKEE LETTER SU" },
    { 0x13D2, "CHEROKEE LETTER SV" },
    { 0x13D3, "CHEROKEE LETTER DA" },
    { 0x13D4, "CHEROKEE LETTER TA" },
    { 0x13D5, "CHEROKEE LETTER DE" },
    { 0x13D6, "CHEROKEE LETTER TE" },
    { 0x13D7, "CHEROKEE LETTER DI" },
    { 0x13D8, "CHEROKEE LETTER TI" },
    { 0x13D9, "CHEROKEE LETTER DO" },
    { 0x13DA, "CHEROKEE LETTER DU" },
    { 0x13DB, "CHEROKEE LETTER DV" },
    { 0x13DC, "CHEROKEE LETTER DLA" },
    { 0x13DD, "CHEROKEE LETTER TLA" },
    { 0x13DE, "CHEROKEE LETTER TLE" },
    { 0x13DF, "CHEROKEE LETTER TLI" },
    { 0x13E0, "CHEROKEE LETTER TLO" },
    { 0x13E1, "CHEROKEE LETTER TLU" },
    { 0x13E2, "CHEROKEE LETTER TLV" },
    { 0x13E3, "CHEROKEE LETTER TSA" },
    { 0x13E4, "CHEROKEE LETTER TSE" },
    { 0x13E5, "CHEROKEE LETTER TSI" },
    { 0x13E6, "CHEROKEE LETTER TSO" },
    { 0x13E7, "CHEROKEE LETTER TSU" },
    { 0x13E8, "CHEROKEE LETTER TSV" },
    { 0x13E9, "CHEROKEE LETTER WA" },
    { 0x13EA, "CHEROKEE LETTER WE" },
    { 0x13EB, "CHEROKEE LETTER WI" },
    { 0x13EC, "CHEROKEE LETTER WO" },
    { 0x13ED, "CHEROKEE LETTER WU" },
    { 0x13EE, "CHEROKEE LETTER WV" },
    { 0x13EF, "CHEROKEE LETTER YA" },
    { 0x13F0, "CHEROKEE LETTER YE" },
    { 0x13F1, "CHEROKEE LETTER YI" },
    { 0x13F2, "CHEROKEE LETTER YO" },
    { 0x13F3, "CHEROKEE LETTER YU" },
    { 0x13F4, "CHEROKEE LETTER YV" },
    { 0x13F5, "CHEROKEE LETTER MV" },
    { 0x13F8, "CHEROKEE SMALL LETTER YE" },
    { 0x13F9, "CHEROKEE SMALL LETTER YI" },
    { 0x13FA, "CHEROKEE SMALL LETTER YO" },
    { 0x13FB, "CHEROKEE SMALL LETTER YU" },
    { 0x13FC, "CHEROKEE SMALL LETTER YV" },
    { 0x13FD, "CHEROKEE SMALL LETTER MV" },
    { 0x1400, "CANADIAN SYLLABICS HYPHEN" },
    { 0x1401, "CANADIAN SYLLABICS E" },
    { 0x1402, "CANADIAN SYLLABICS AAI" },
    { 0x1403, "CANADIAN SYLLABICS I" },
    { 0x1404, "CANADIAN SYLLABICS II" },
    { 0x1405, "CANADIAN SYLLABICS O" },
    { 0x1406, "CANADIAN SYLLABICS OO" },
    { 0x1407, "CANADIAN SYLLABICS Y-CREE OO" },
    { 0x1408, "CANADIAN SYLLABICS CARRIER EE" },
    { 0x1409, "CANADIAN SYLLABICS CARRIER I" },
    { 0x140A, "CANADIAN SYLLABICS A" },
    { 0x140B, "CANADIAN SYLLABICS AA" },
    { 0x140C, "CANADIAN SYLLABICS WE" },
    { 0x140D, "CANADIAN SYLLABICS WEST-CREE WE" },
    { 0x140E, "CANADIAN SYLLABICS WI" },
    { 0x140F, "CANADIAN SYLLABICS WEST-CREE WI" },
    { 0x1410, "CANADIAN SYLLABICS WII" },
    { 0x1411, "CANADIAN SYLLABICS WEST-CREE WII" },
    { 0x1412, "CANADIAN SYLLABICS WO" },
    { 0x1413, "CANADIAN SYLLABICS WEST-CREE WO" },
    { 0x1414, "CANADIAN SYLLABICS WOO" },
    { 0x1415, "CANADIAN SYLLABICS WEST-CREE WOO" },
    { 0x1416, "CANADIAN SYLLABICS NASKAPI WOO" },
    { 0x1417, "CANADIAN SYLLABICS WA" },
    { 0x1418, "CANADIAN SYLLABICS WEST-CREE WA" },
    { 0x1419, "CANADIAN SYLLABICS WAA" },
    { 0x141A, "CANADIAN SYLLABICS WEST-CREE WAA" },
    { 0x141B, "CANADIAN SYLLABICS NASKAPI WAA" },
    { 0x141C, "CANADIAN SYLLABICS AI" },
    { 0x141D, "CANADIAN SYLLABICS Y-CREE W" },
    { 0x141E, "CANADIAN SYLLABICS GLOTTAL STOP" },
    { 0x141F, "CANADIAN SYLLABICS FINAL ACUTE" },
    { 0x1420, "CANADIAN SYLLABICS FINAL GRAVE" },
    { 0x1421, "CANADIAN SYLLABICS FINAL BOTTOM HALF RING" },
    { 0x1422, "CANADIAN SYLLABICS FINAL TOP HALF RING" },
    { 0x1423, "CANADIAN SYLLABICS FINAL RIGHT HALF RING" },
    { 0x1424, "CANADIAN SYLLABICS FINAL RING" },
    { 0x1425, "CANADIAN SYLLABICS FINAL DOUBLE ACUTE" },
    { 0x1426, "CANADIAN SYLLABICS FINAL DOUBLE SHORT VERTICAL STROKES" },
    { 0x1427, "CANADIAN SYLLABICS FINAL MIDDLE DOT" },
    { 0x1428, "CANADIAN SYLLABICS FINAL SHORT HORIZONTAL STROKE" },
    { 0x1429, "CANADIAN SYLLABICS FINAL PLUS" },
    { 0x142A, "CANADIAN SYLLABICS FINAL DOWN TACK" },
    { 0x142B, "CANADIAN SYLLABICS EN" },
    { 0x142C, "CANADIAN SYLLABICS IN" },
    { 0x142D, "CANADIAN SYLLABICS ON" },
    { 0x142E, "CANADIAN SYLLABICS AN" },
    { 0x142F, "CANADIAN SYLLABICS PE" },
    { 0x1430, "CANADIAN SYLLABICS PAAI" },
    { 0x1431, "CANADIAN SYLLABICS PI" },
    { 0x1432, "CANADIAN SYLLABICS PII" },
    { 0x1433, "CANADIAN SYLLABICS PO" },
    { 0x1434, "CANADIAN SYLLABICS POO" },
    { 0x1435, "CANADIAN SYLLABICS Y-CREE POO" },
    { 0x1436, "CANADIAN SYLLABICS CARRIER HEE" },
    { 0x1437, "CANADIAN SYLLABICS CARRIER HI" },
    { 0x1438, "CANADIAN SYLLABICS PA" },
    { 0x1439, "CANADIAN SYLLABICS PAA" },
    { 0x143A, "CANADIAN SYLLABICS PWE" },
    { 0x143B, "CANADIAN SYLLABICS WEST-CREE PWE" },
    { 0x143C, "CANADIAN SYLLABICS PWI" },
    { 0x143D, "CANADIAN SYLLABICS WEST-CREE PWI" },
    { 0x143E, "CANADIAN SYLLABICS PWII" },
    { 0x143F, "CANADIAN SYLLABICS WEST-CREE PWII" },
    { 0x1440, "CANADIAN SYLLABICS PWO" },
    { 0x1441, "CANADIAN SYLLABICS WEST-CREE PWO" },
    { 0x1442, "CANADIAN SYLLABICS PWOO" },
    { 0x1443, "CANADIAN SYLLABICS WEST-CREE PWOO" },
    { 0x1444, "CANADIAN SYLLABICS PWA" },
    { 0x1445, "CANADIAN SYLLABICS WEST-CREE PWA" },
    { 0x1446, "CANADIAN SYLLABICS PWAA" },
    { 0x1447, "CANADIAN SYLLABICS WEST-CREE PWAA" },
    { 0x1448, "CANADIAN SYLLABICS Y-CREE PWAA" },
    { 0x1449, "CANADIAN SYLLABICS P" },
    { 0x144A, "CANADIAN SYLLABICS WEST-CREE P" },
    { 0x144B, "CANADIAN SYLLABICS CARRIER H" },
    { 0x144C, "CANADIAN SYLLABICS TE" },
    { 0x144D, "CANADIAN SYLLABICS TAAI" },
    { 0x144E, "CANADIAN SYLLABICS TI" },
    { 0x144F, "CANADIAN SYLLABICS TII" },
    { 0x1450, "CANADIAN SYLLABICS TO" },
    { 0x1451, "CANADIAN SYLLABICS TOO" },
    { 0x1452, "CANADIAN SYLLABICS Y-CREE TOO" },
    { 0x1453, "CANADIAN SYLLABICS CARRIER DEE" },
    { 0x1454, "CANADIAN SYLLABICS CARRIER DI" },
    { 0x1455, "CANADIAN SYLLABICS TA" },
    { 0x1456, "CANADIAN SYLLABICS TAA" },
    { 0x1457, "CANADIAN SYLLABICS TWE" },
    { 0x1458, "CANADIAN SYLLABICS WEST-CREE TWE" },
    { 0x1459, "CANADIAN SYLLABICS TWI" },
    { 0x145A, "CANADIAN SYLLABICS WEST-CREE TWI" },
    { 0x145B, "CANADIAN SYLLABICS TWII" },
    { 0x145C, "CANADIAN SYLLABICS WEST-CREE TWII" },
    { 0x145D, "CANADIAN SYLLABICS TWO" },
    { 0x145E, "CANADIAN SYLLABICS WEST-CREE TWO" },
    { 0x145F, "CANADIAN SYLLABICS TWOO" },
    { 0x1460, "CANADIAN SYLLABICS WEST-CREE TWOO" },
    { 0x1461, "CANADIAN SYLLABICS TWA" },
    { 0x1462, "CANADIAN SYLLABICS WEST-CREE TWA" },
    { 0x1463, "CANADIAN SYLLABICS TWAA" },
    { 0x1464, "CANADIAN SYLLABICS WEST-CREE TWAA" },
    { 0x1465, "CANADIAN SYLLABICS NASKAPI TWAA" },
    { 0x1466, "CANADIAN SYLLABICS T" },
    { 0x1467, "CANADIAN SYLLABICS TTE" },
    { 0x1468, "CANADIAN SYLLABICS TTI" },
    { 0x1469, "CANADIAN SYLLABICS TTO" },
    { 0x146A, "CANADIAN SYLLABICS TTA" },
    { 0x146B, "CANADIAN SYLLABICS KE" },
    { 0x146C, "CANADIAN SYLLABICS KAAI" },
    { 0x146D, "CANADIAN SYLLABICS KI" },
    { 0x146E, "CANADIAN SYLLABICS KII" },
    { 0x146F, "CANADIAN SYLLABICS KO" },
    { 0x1470, "CANADIAN SYLLABICS KOO" },
    { 0x1471, "CANADIAN SYLLABICS Y-CREE KOO" },
    { 0x1472, "CANADIAN SYLLABICS KA" },
    { 0x1473, "CANADIAN SYLLABICS KAA" },
    { 0x1474, "CANADIAN SYLLABICS KWE" },
    { 0x1475, "CANADIAN SYLLABICS WEST-CREE KWE" },
    { 0x1476, "CANADIAN SYLLABICS KWI" },
    { 0x1477, "CANADIAN SYLLABICS WEST-CREE KWI" },
    { 0x1478, "CANADIAN SYLLABICS KWII" },
    { 0x1479, "CANADIAN SYLLABICS WEST-CREE KWII" },
    { 0x147A, "CANADIAN SYLLABICS KWO" },
    { 0x147B, "CANADIAN SYLLABICS WEST-CREE KWO" },
    { 0x147C, "CANADIAN SYLLABICS KWOO" },
    { 0x147D, "CANADIAN SYLLABICS WEST-CREE KWOO" },
    { 0x147E, "CANADIAN SYLLABICS KWA" },
    { 0x147F, "CANADIAN SYLLABICS WEST-CREE KWA" },
    { 0x1480, "CANADIAN SYLLABICS KWAA" },
    { 0x1481, "CANADIAN SYLLABICS WEST-CREE KWAA" },
    { 0x1482, "CANADIAN SYLLABICS NASKAPI KWAA" },
    { 0x1483, "CANADIAN SYLLABICS K" },
    { 0x1484, "CANADIAN SYLLABICS KW" },
    { 0x1485, "CANADIAN SYLLABICS SOUTH-SLAVEY KEH" },
    { 0x1486, "CANADIAN SYLLABICS SOUTH-SLAVEY KIH" },
    { 0x1487, "CANADIAN SYLLABICS SOUTH-SLAVEY KOH" },
    { 0x1488, "CANADIAN SYLLABICS SOUTH-SLAVEY KAH" },
    { 0x1489, "CANADIAN SYLLABICS CE" },
    { 0x148A, "CANADIAN SYLLABICS CAAI" },
    { 0x148B, "CANADIAN SYLLABICS CI" },
    { 0x148C, "CANADIAN SYLLABICS CII" },
    { 0x148D, "CANADIAN SYLLABICS CO" },
    { 0x148E, "CANADIAN SYLLABICS COO" },
    { 0x148F, "CANADIAN SYLLABICS Y-CREE COO" },
    { 0x1490, "CANADIAN SYLLABICS CA" },
    { 0x1491, "CANADIAN SYLLABICS CAA" },
    { 0x1492, "CANADIAN SYLLABICS CWE" },
    { 0x1493, "CANADIAN SYLLABICS WEST-CREE CWE" },
    { 0x1494, "CANADIAN SYLLABICS CWI" },
    { 0x1495, "CANADIAN SYLLABICS WEST-CREE CWI" },
    { 0x1496, "CANADIAN SYLLABICS CWII" },
    { 0x1497, "CANADIAN SYLLABICS WEST-CREE CWII" },
    { 0x1498, "CANADIAN SYLLABICS CWO" },
    { 0x1499, "CANADIAN SYLLABICS WEST-CREE CWO" },
    { 0x149A, "CANADIAN SYLLABICS CWOO" },
    { 0x149B, "CANADIAN SYLLABICS WEST-CREE CWOO" },
    { 0x149C, "CANADIAN SYLLABICS CWA" },
    { 0x149D, "CANADIAN SYLLABICS WEST-CREE CWA" },
    { 0x149E, "CANADIAN SYLLABICS CWAA" },
    { 0x149F, "CANADIAN SYLLABICS WEST-CREE CWAA" },
    { 0x14A0, "CANADIAN SYLLABICS NASKAPI CWAA" },
    { 0x14A1, "CANADIAN SYLLABICS C" },
    { 0x14A2, "CANADIAN SYLLABICS SAYISI TH" },
    { 0x14A3, "CANADIAN SYLLABICS ME" },
    { 0x14A4, "CANADIAN SYLLABICS MAAI" },
    { 0x14A5, "CANADIAN SYLLABICS MI" },
    { 0x14A6, "CANADIAN SYLLABICS MII" },
    { 0x14A7, "CANADIAN SYLLABICS MO" },
    { 0x14A8, "CANADIAN SYLLABICS MOO" },
    { 0x14A9, "CANADIAN SYLLABICS Y-CREE MOO" },
    { 0x14AA, "CANADIAN SYLLABICS MA" },
    { 0x14AB, "CANADIAN SYLLABICS MAA" },
    { 0x14AC, "CANADIAN SYLLABICS MWE" },
    { 0x14AD, "CANADIAN SYLLABICS WEST-CREE MWE" },
    { 0x14AE, "CANADIAN SYLLABICS MWI" },
    { 0x14AF, "CANADIAN SYLLABICS WEST-CREE MWI" },
    { 0x14B0, "CANADIAN SYLLABICS MWII" },
    { 0x14B1, "CANADIAN SYLLABICS WEST-CREE MWII" },
    { 0x14B2, "CANADIAN SYLLABICS MWO" },
    { 0x14B3, "CANADIAN SYLLABICS WEST-CREE MWO" },
    { 0x14B4, "CANADIAN SYLLABICS MWOO" },
    { 0x14B5, "CANADIAN SYLLABICS WEST-CREE MWOO" },
    { 0x14B6, "CANADIAN SYLLABICS MWA" },
    { 0x14B7, "CANADIAN SYLLABICS WEST-CREE MWA" },
    { 0x14B8, "CANADIAN SYLLABICS MWAA" },
    { 0x14B9, "CANADIAN SYLLABICS WEST-CREE MWAA" },
    { 0x14BA, "CANADIAN SYLLABICS NASKAPI MWAA" },
    { 0x14BB, "CANADIAN SYLLABICS M" },
    { 0x14BC, "CANADIAN SYLLABICS WEST-CREE M" },
    { 0x14BD, "CANADIAN SYLLABICS MH" },
    { 0x14BE, "CANADIAN SYLLABICS ATHAPASCAN M" },
    { 0x14BF, "CANADIAN SYLLABICS SAYISI M" },
    { 0x14C0, "CANADIAN SYLLABICS NE" },
    { 0x14C1, "CANADIAN SYLLABICS NAAI" },
    { 0x14C2, "CANADIAN SYLLABICS NI" },
    { 0x14C3, "CANADIAN SYLLABICS NII" },
    { 0x14C4, "CANADIAN SYLLABICS NO" },
    { 0x14C5, "CANADIAN SYLLABICS NOO" },
    { 0x14C6, "CANADIAN SYLLABICS Y-CREE NOO" },
    { 0x14C7, "CANADIAN SYLLABICS NA" },
    { 0x14C8, "CANADIAN SYLLABICS NAA" },
    { 0x14C9, "CANADIAN SYLLABICS NWE" },
    { 0x14CA, "CANADIAN SYLLABICS WEST-CREE NWE" },
    { 0x14CB, "CANADIAN SYLLABICS NWA" },
    { 0x14CC, "CANADIAN SYLLABICS WEST-CREE NWA" },
    { 0x14CD, "CANADIAN SYLLABICS NWAA" },
    { 0x14CE, "CANADIAN SYLLABICS WEST-CREE NWAA" },
    { 0x14CF, "CANADIAN SYLLABICS NASKAPI NWAA" },
    { 0x14D0, "CANADIAN SYLLABICS N" },
    { 0x14D1, "CANADIAN SYLLABICS CARRIER NG" },
    { 0x14D2, "CANADIAN SYLLABICS NH" },
    { 0x14D3, "CANADIAN SYLLABICS LE" },
    { 0x14D4, "CANADIAN SYLLABICS LAAI" },
    { 0x14D5, "CANADIAN SYLLABICS LI" },
    { 0x14D6, "CANADIAN SYLLABICS LII" },
    { 0x14D7, "CANADIAN SYLLABICS LO" },
    { 0x14D8, "CANADIAN SYLLABICS LOO" },
    { 0x14D9, "CANADIAN SYLLABICS Y-CREE LOO" },
    { 0x14DA, "CANADIAN SYLLABICS LA" },
    { 0x14DB, "CANADIAN SYLLABICS LAA" },
    { 0x14DC, "CANADIAN SYLLABICS LWE" },
    { 0x14DD, "CANADIAN SYLLABICS WEST-CREE LWE" },
    { 0x14DE, "CANADIAN SYLLABICS LWI" },
    { 0x14DF, "CANADIAN SYLLABICS WEST-CREE LWI" },
    { 0x14E0, "CANADIAN SYLLABICS LWII" },
    { 0x14E1, "CANADIAN SYLLABICS WEST-CREE LWII" },
    { 0x14E2, "CANADIAN SYLLABICS LWO" },
    { 0x14E3, "CANADIAN SYLLABICS WEST-CREE LWO" },
    { 0x14E4, "CANADIAN SYLLABICS LWOO" },
    { 0x14E5, "CANADIAN SYLLABICS WEST-CREE LWOO" },
    { 0x14E6, "CANADIAN SYLLABICS LWA" },
    { 0x14E7, "CANADIAN SYLLABICS WEST-CREE LWA" },
    { 0x14E8, "CANADIAN SYLLABICS LWAA" },
    { 0x14E9, "CANADIAN SYLLABICS WEST-CREE LWAA" },
    { 0x14EA, "CANADIAN SYLLABICS L" },
    { 0x14EB, "CANADIAN SYLLABICS WEST-CREE L" },
    { 0x14EC, "CANADIAN SYLLABICS MEDIAL L" },
    { 0x14ED, "CANADIAN SYLLABICS SE" },
    { 0x14EE, "CANADIAN SYLLABICS SAAI" },
    { 0x14EF, "CANADIAN SYLLABICS SI" },
    { 0x14F0, "CANADIAN SYLLABICS SII" },
    { 0x14F1, "CANADIAN SYLLABICS SO" },
    { 0x14F2, "CANADIAN SYLLABICS SOO" },
    { 0x14F3, "CANADIAN SYLLABICS Y-CREE SOO" },
    { 0x14F4, "CANADIAN SYLLABICS SA" },
    { 0x14F5, "CANADIAN SYLLABICS SAA" },
    { 0x14F6, "CANADIAN SYLLABICS SWE" },
    { 0x14F7, "CANADIAN SYLLABICS WEST-CREE SWE" },
    { 0x14F8, "CANADIAN SYLLABICS SWI" },
    { 0x14F9, "CANADIAN SYLLABICS WEST-CREE SWI" },
    { 0x14FA, "CANADIAN SYLLABICS SWII" },
    { 0x14FB, "CANADIAN SYLLABICS WEST-CREE SWII" },
    { 0x14FC, "CANADIAN SYLLABICS SWO" },
    { 0x14FD, "CANADIAN SYLLABICS WEST-CREE SWO" },
    { 0x14FE, "CANADIAN SYLLABICS SWOO" },
    { 0x14FF, "CANADIAN SYLLABICS WEST-CREE SWOO" },
    { 0x1500, "CANADIAN SYLLABICS SWA" },
    { 0x1501, "CANADIAN SYLLABICS WEST-CREE SWA" },
    { 0x1502, "CANADIAN SYLLABICS SWAA" },
    { 0x1503, "CANADIAN SYLLABICS WEST-CREE SWAA" },
    { 0x1504, "CANADIAN SYLLABICS NASKAPI SWAA" },
    { 0x1505, "CANADIAN SYLLABICS S" },
    { 0x1506, "CANADIAN SYLLABICS ATHAPASCAN S" },
    { 0x1507, "CANADIAN SYLLABICS SW" },
    { 0x1508, "CANADIAN SYLLABICS BLACKFOOT S" },
    { 0x1509, "CANADIAN SYLLABICS MOOSE-CREE SK" },
    { 0x150A, "CANADIAN SYLLABICS NASKAPI SKW" },
    { 0x150B, "CANADIAN SYLLABICS NASKAPI S-W" },
    { 0x150C, "CANADIAN SYLLABICS NASKAPI SPWA" },
    { 0x150D, "CANADIAN SYLLABICS NASKAPI STWA" },
    { 0x150E, "CANADIAN SYLLABICS NASKAPI SKWA" },
    { 0x150F, "CANADIAN SYLLABICS NASKAPI SCWA" },
    { 0x1510, "CANADIAN SYLLABICS SHE" },
    { 0x1511, "CANADIAN SYLLABICS SHI" },
    { 0x1512, "CANADIAN SYLLABICS SHII" },
    { 0x1513, "CANADIAN SYLLABICS SHO" },
    { 0x1514, "CANADIAN SYLLABICS SHOO" },
    { 0x1515, "CANADIAN SYLLABICS SHA" },
    { 0x1516, "CANADIAN SYLLABICS SHAA" },
    { 0x1517, "CANADIAN SYLLABICS SHWE" },
    { 0x1518, "CANADIAN SYLLABICS WEST-CREE SHWE" },
    { 0x1519, "CANADIAN SYLLABICS SHWI" },
    { 0x151A, "CANADIAN SYLLABICS WEST-CREE SHWI" },
    { 0x151B, "CANADIAN SYLLABICS SHWII" },
    { 0x151C, "CANADIAN SYLLABICS WEST-CREE SHWII" },
    { 0x151D, "CANADIAN SYLLABICS SHWO" },
    { 0x151E, "CANADIAN SYLLABICS WEST-CREE SHWO" },
    { 0x151F, "CANADIAN SYLLABICS SHWOO" },
    { 0x1520, "CANADIAN SYLLABICS WEST-CREE SHWOO" },
    { 0x1521, "CANADIAN SYLLABICS SHWA" },
    { 0x1522, "CANADIAN SYLLABICS WEST-CREE SHWA" },
    { 0x1523, "CANADIAN SYLLABICS SHWAA" },
    { 0x1524, "CANADIAN SYLLABICS WEST-CREE SHWAA" },
    { 0x1525, "CANADIAN SYLLABICS SH" },
    { 0x1526, "CANADIAN SYLLABICS YE" },
    { 0x1527, "CANADIAN SYLLABICS YAAI" },
    { 0x1528, "CANADIAN SYLLABICS YI" },
    { 0x1529, "CANADIAN SYLLABICS YII" },
    { 0x152A, "CANADIAN SYLLABICS YO" },
    { 0x152B, "CANADIAN SYLLABICS YOO" },
    { 0x152C, "CANADIAN SYLLABICS Y-CREE YOO" },
    { 0x152D, "CANADIAN SYLLABICS YA" },
    { 0x152E, "CANADIAN SYLLABICS YAA" },
    { 0x152F, "CANADIAN SYLLABICS YWE" },
    { 0x1530, "CANADIAN SYLLABICS WEST-CREE YWE" },
    { 0x1531, "CANADIAN SYLLABICS YWI" },
    { 0x1532, "CANADIAN SYLLABICS WEST-CREE YWI" },
    { 0x1533, "CANADIAN SYLLABICS YWII" },
    { 0x1534, "CANADIAN SYLLABICS WEST-CREE YWII" },
    { 0x1535, "CANADIAN SYLLABICS YWO" },
    { 0x1536, "CANADIAN SYLLABICS WEST-CREE YWO" },
    { 0x1537, "CANADIAN SYLLABICS YWOO" },
    { 0x1538, "CANADIAN SYLLABICS WEST-CREE YWOO" },
    { 0x1539, "CANADIAN SYLLABICS YWA" },
    { 0x153A, "CANADIAN SYLLABICS WEST-CREE YWA" },
    { 0x153B, "CANADIAN SYLLABICS YWAA" },
    { 0x153C, "CANADIAN SYLLABICS WEST-CREE YWAA" },
    { 0x153D, "CANADIAN SYLLABICS NASKAPI YWAA" },
    { 0x153E, "CANADIAN SYLLABICS Y" },
    { 0x153F, "CANADIAN SYLLABICS BIBLE-CREE Y" },
    { 0x1540, "CANADIAN SYLLABICS WEST-CREE Y" },
    { 0x1541, "CANADIAN SYLLABICS SAYISI YI" },
    { 0x1542, "CANADIAN SYLLABICS RE" },
    { 0x1543, "CANADIAN SYLLABICS R-CREE RE" },
    { 0x1544, "CANADIAN SYLLABICS WEST-CREE LE" },
    { 0x1545, "CANADIAN SYLLABICS RAAI" },
    { 0x1546, "CANADIAN SYLLABICS RI" },
    { 0x1547, "CANADIAN SYLLABICS RII" },
    { 0x1548, "CANADIAN SYLLABICS RO" },
    { 0x1549, "CANADIAN SYLLABICS ROO" },
    { 0x154A, "CANADIAN SYLLABICS WEST-CREE LO" },
    { 0x154B, "CANADIAN SYLLABICS RA" },
    { 0x154C, "CANADIAN SYLLABICS RAA" },
    { 0x154D, "CANADIAN SYLLABICS WEST-CREE LA" },
    { 0x154E, "CANADIAN SYLLABICS RWAA" },
    { 0x154F, "CANADIAN SYLLABICS WEST-CREE RWAA" },
    { 0x1550, "CANADIAN SYLLABICS R" },
    { 0x1551, "CANADIAN SYLLABICS WEST-CREE R" },
    { 0x1552, "CANADIAN SYLLABICS MEDIAL R" },
    { 0x1553, "CANADIAN SYLLABICS FE" },
    { 0x1554, "CANADIAN SYLLABICS FAAI" },
    { 0x1555, "CANADIAN SYLLABICS FI" },
    { 0x1556, "CANADIAN SYLLABICS FII" },
    { 0x1557, "CANADIAN SYLLABICS FO" },
    { 0x1558, "CANADIAN SYLLABICS FOO" },
    { 0x1559, "CANADIAN SYLLABICS FA" },
    { 0x155A, "CANADIAN SYLLABICS FAA" },
    { 0x155B, "CANADIAN SYLLABICS FWAA" },
    { 0x155C, "CANADIAN SYLLABICS WEST-CREE FWAA" },
    { 0x155D, "CANADIAN SYLLABICS F" },
    { 0x155E, "CANADIAN SYLLABICS THE" },
    { 0x155F, "CANADIAN SYLLABICS N-CREE THE" },
    { 0x1560, "CANADIAN SYLLABICS THI" },
    { 0x1561, "CANADIAN SYLLABICS N-CREE THI" },
    { 0x1562, "CANADIAN SYLLABICS THII" },
    { 0x1563, "CANADIAN SYLLABICS N-CREE THII" },
    { 0x1564, "CANADIAN SYLLABICS THO" },
    { 0x1565, "CANADIAN SYLLABICS THOO" },
    { 0x1566, "CANADIAN SYLLABICS THA" },
    { 0x1567, "CANADIAN SYLLABICS THAA" },
    { 0x1568, "CANADIAN SYLLABICS THWAA" },
    { 0x1569, "CANADIAN SYLLABICS WEST-CREE THWAA" },
    { 0x156A, "CANADIAN SYLLABICS TH" },
    { 0x156B, "CANADIAN SYLLABICS TTHE" },
    { 0x156C, "CANADIAN SYLLABICS TTHI" },
    { 0x156D, "CANADIAN SYLLABICS TTHO" },
    { 0x156E, "CANADIAN SYLLABICS TTHA" },
    { 0x156F, "CANADIAN SYLLABICS TTH" },
    { 0x1570, "CANADIAN SYLLABICS TYE" },
    { 0x1571, "CANADIAN SYLLABICS TYI" },
    { 0x1572, "CANADIAN SYLLABICS TYO" },
    { 0x1573, "CANADIAN SYLLABICS TYA" },
    { 0x1574, "CANADIAN SYLLABICS NUNAVIK HE" },
    { 0x1575, "CANADIAN SYLLABICS NUNAVIK HI" },
    { 0x1576, "CANADIAN SYLLABICS NUNAVIK HII" },
    { 0x1577, "CANADIAN SYLLABICS NUNAVIK HO" },
    { 0x1578, "CANADIAN SYLLABICS NUNAVIK HOO" },
    { 0x1579, "CANADIAN SYLLABICS NUNAVIK HA" },
    { 0x157A, "CANADIAN SYLLABICS NUNAVIK HAA" },
    { 0x157B, "CANADIAN SYLLABICS NUNAVIK H" },
    { 0x157C, "CANADIAN SYLLABICS NUNAVUT H" },
    { 0x157D, "CANADIAN SYLLABICS HK" },
    { 0x157E, "CANADIAN SYLLABICS QAAI" },
    { 0x157F, "CANADIAN SYLLABICS QI" },
    { 0x1580, "CANADIAN SYLLABICS QII" },
    { 0x1581, "CANADIAN SYLLABICS QO" },
    { 0x1582, "CANADIAN SYLLABICS QOO" },
    { 0x1583, "CANADIAN SYLLABICS QA" },
    { 0x1584, "CANADIAN SYLLABICS QAA" },
    { 0x1585, "CANADIAN SYLLABICS Q" },
    { 0x1586, "CANADIAN SYLLABICS TLHE" },
    { 0x1587, "CANADIAN SYLLABICS TLHI" },
    { 0x1588, "CANADIAN SYLLABICS TLHO" },
    { 0x1589, "CANADIAN SYLLABICS TLHA" },
    { 0x158A, "CANADIAN SYLLABICS WEST-CREE RE" },
    { 0x158B, "CANADIAN SYLLABICS WEST-CREE RI" },
    { 0x158C, "CANADIAN SYLLABICS WEST-CREE RO" },
    { 0x158D, "CANADIAN SYLLABICS WEST-CREE RA" },
    { 0x158E, "CANADIAN SYLLABICS NGAAI" },
    { 0x158F, "CANADIAN SYLLABICS NGI" },
    { 0x1590, "CANADIAN SYLLABICS NGII" },
    { 0x1591, "CANADIAN SYLLABICS NGO" },
    { 0x1592, "CANADIAN SYLLABICS NGOO" },
    { 0x1593, "CANADIAN SYLLABICS NGA" },
    { 0x1594, "CANADIAN SYLLABICS NGAA" },
    { 0x1595, "CANADIAN SYLLABICS NG" },
    { 0x1596, "CANADIAN SYLLABICS NNG" },
    { 0x1597, "CANADIAN SYLLABICS SAYISI SHE" },
    { 0x1598, "CANADIAN SYLLABICS SAYISI SHI" },
    { 0x1599, "CANADIAN SYLLABICS SAYISI SHO" },
    { 0x159A, "CANADIAN SYLLABICS SAYISI SHA" },
    { 0x159B, "CANADIAN SYLLABICS WOODS-CREE THE" },
    { 0x159C, "CANADIAN SYLLABICS WOODS-CREE THI" },
    { 0x159D, "CANADIAN SYLLABICS WOODS-CREE THO" },
    { 0x159E, "CANADIAN SYLLABICS WOODS-CREE THA" },
    { 0x159F, "CANADIAN SYLLABICS WOODS-CREE TH" },
    { 0x15A0, "CANADIAN SYLLABICS LHI" },
    { 0x15A1, "CANADIAN SYLLABICS LHII" },
    { 0x15A2, "CANADIAN SYLLABICS LHO" },
    { 0x15A3, "CANADIAN SYLLABICS LHOO" },
    { 0x15A4, "CANADIAN SYLLABICS LHA" },
    { 0x15A5, "CANADIAN SYLLABICS LHAA" },
    { 0x15A6, "CANADIAN SYLLABICS LH" },
    { 0x15A7, "CANADIAN SYLLABICS TH-CREE THE" },
    { 0x15A8, "CANADIAN SYLLABICS TH-CREE THI" },
    { 0x15A9, "CANADIAN SYLLABICS TH-CREE THII" },
    { 0x15AA, "CANADIAN SYLLABICS TH-CREE THO" },
    { 0x15AB, "CANADIAN SYLLABICS TH-CREE THOO" },
    { 0x15AC, "CANADIAN SYLLABICS TH-CREE THA" },
    { 0x15AD, "CANADIAN SYLLABICS TH-CREE THAA" },
    { 0x15AE, "CANADIAN SYLLABICS TH-CREE TH" },
    { 0x15AF, "CANADIAN SYLLABICS AIVILIK B" },
    { 0x15B0, "CANADIAN SYLLABICS BLACKFOOT E" },
    { 0x15B1, "CANADIAN SYLLABICS BLACKFOOT I" },
    { 0x15B2, "CANADIAN SYLLABICS BLACKFOOT O" },
    { 0x15B3, "CANADIAN SYLLABICS BLACKFOOT A" },
    { 0x15B4, "CANADIAN SYLLABICS BLACKFOOT WE" },
    { 0x15B5, "CANADIAN SYLLABICS BLACKFOOT WI" },
    { 0x15B6, "CANADIAN SYLLABICS BLACKFOOT WO" },
    { 0x15B7, "CANADIAN SYLLABICS BLACKFOOT WA" },
    { 0x15B8, "CANADIAN SYLLABICS BLACKFOOT NE" },
    { 0x15B9, "CANADIAN SYLLABICS BLACKFOOT NI" },
    { 0x15BA, "CANADIAN SYLLABICS BLACKFOOT NO" },
    { 0x15BB, "CANADIAN SYLLABICS BLACKFOOT NA" },
    { 0x15BC, "CANADIAN SYLLABICS BLACKFOOT KE" },
    { 0x15BD, "CANADIAN SYLLABICS BLACKFOOT KI" },
    { 0x15BE, "CANADIAN SYLLABICS BLACKFOOT KO" },
    { 0x15BF, "CANADIAN SYLLABICS BLACKFOOT KA" },
    { 0x15C0, "CANADIAN SYLLABICS SAYISI HE" },
    { 0x15C1, "CANADIAN SYLLABICS SAYISI HI" },
    { 0x15C2, "CANADIAN SYLLABICS SAYISI HO" },
    { 0x15C3, "CANADIAN SYLLABICS SAYISI HA" },
    { 0x15C4, "CANADIAN SYLLABICS CARRIER GHU" },
    { 0x15C5, "CANADIAN SYLLABICS CARRIER GHO" },
    { 0x15C6, "CANADIAN SYLLABICS CARRIER GHE" },
    { 0x15C7, "CANADIAN SYLLABICS CARRIER GHEE" },
    { 0x15C8, "CANADIAN SYLLABICS CARRIER GHI" },
    { 0x15C9, "CANADIAN SYLLABICS CARRIER GHA" },
    { 0x15CA, "CANADIAN SYLLABICS CARRIER RU" },
    { 0x15CB, "CANADIAN SYLLABICS CARRIER RO" },
    { 0x15CC, "CANADIAN SYLLABICS CARRIER RE" },
    { 0x15CD, "CANADIAN SYLLABICS CARRIER REE" },
    { 0x15CE, "CANADIAN SYLLABICS CARRIER RI" },
    { 0x15CF, "CANADIAN SYLLABICS CARRIER RA" },
    { 0x15D0, "CANADIAN SYLLABICS CARRIER WU" },
    { 0x15D1, "CANADIAN SYLLABICS CARRIER WO" },
    { 0x15D2, "CANADIAN SYLLABICS CARRIER WE" },
    { 0x15D3, "CANADIAN SYLLABICS CARRIER WEE" },
    { 0x15D4, "CANADIAN SYLLABICS CARRIER WI" },
    { 0x15D5, "CANADIAN SYLLABICS CARRIER WA" },
    { 0x15D6, "CANADIAN SYLLABICS CARRIER HWU" },
    { 0x15D7, "CANADIAN SYLLABICS CARRIER HWO" },
    { 0x15D8, "CANADIAN SYLLABICS CARRIER HWE" },
    { 0x15D9, "CANADIAN SYLLABICS CARRIER HWEE" },
    { 0x15DA, "CANADIAN SYLLABICS CARRIER HWI" },
    { 0x15DB, "CANADIAN SYLLABICS CARRIER HWA" },
    { 0x15DC, "CANADIAN SYLLABICS CARRIER THU" },
    { 0x15DD, "CANADIAN SYLLABICS CARRIER THO" },
    { 0x15DE, "CANADIAN SYLLABICS CARRIER THE" },
    { 0x15DF, "CANADIAN SYLLABICS CARRIER THEE" },
    { 0x15E0, "CANADIAN SYLLABICS CARRIER THI" },
    { 0x15E1, "CANADIAN SYLLABICS CARRIER THA" },
    { 0x15E2, "CANADIAN SYLLABICS CARRIER TTU" },
    { 0x15E3, "CANADIAN SYLLABICS CARRIER TTO" },
    { 0x15E4, "CANADIAN SYLLABICS CARRIER TTE" },
    { 0x15E5, "CANADIAN SYLLABICS CARRIER TTEE" },
    { 0x15E6, "CANADIAN SYLLABICS CARRIER TTI" },
    { 0x15E7, "CANADIAN SYLLABICS CARRIER TTA" },
    { 0x15E8, "CANADIAN SYLLABICS CARRIER PU" },
    { 0x15E9, "CANADIAN SYLLABICS CARRIER PO" },
    { 0x15EA, "CANADIAN SYLLABICS CARRIER PE" },
    { 0x15EB, "CANADIAN SYLLABICS CARRIER PEE" },
    { 0x15EC, "CANADIAN SYLLABICS CARRIER PI" },
    { 0x15ED, "CANADIAN SYLLABICS CARRIER PA" },
    { 0x15EE, "CANADIAN SYLLABICS CARRIER P" },
    { 0x15EF, "CANADIAN SYLLABICS CARRIER GU" },
    { 0x15F0, "CANADIAN SYLLABICS CARRIER GO" },
    { 0x15F1, "CANADIAN SYLLABICS CARRIER GE" },
    { 0x15F2, "CANADIAN SYLLABICS CARRIER GEE" },
    { 0x15F3, "CANADIAN SYLLABICS CARRIER GI" },
    { 0x15F4, "CANADIAN SYLLABICS CARRIER GA" },
    { 0x15F5, "CANADIAN SYLLABICS CARRIER KHU" },
    { 0x15F6, "CANADIAN SYLLABICS CARRIER KHO" },
    { 0x15F7, "CANADIAN SYLLABICS CARRIER KHE" },
    { 0x15F8, "CANADIAN SYLLABICS CARRIER KHEE" },
    { 0x15F9, "CANADIAN SYLLABICS CARRIER KHI" },
    { 0x15FA, "CANADIAN SYLLABICS CARRIER KHA" },
    { 0x15FB, "CANADIAN SYLLABICS CARRIER KKU" },
    { 0x15FC, "CANADIAN SYLLABICS CARRIER KKO" },
    { 0x15FD, "CANADIAN SYLLABICS CARRIER KKE" },
    { 0x15FE, "CANADIAN SYLLABICS CARRIER KKEE" },
    { 0x15FF, "CANADIAN SYLLABICS CARRIER KKI" },
    { 0x1600, "CANADIAN SYLLABICS CARRIER KKA" },
    { 0x1601, "CANADIAN SYLLABICS CARRIER KK" },
    { 0x1602, "CANADIAN SYLLABICS CARRIER NU" },
    { 0x1603, "CANADIAN SYLLABICS CARRIER NO" },
    { 0x1604, "CANADIAN SYLLABICS CARRIER NE" },
    { 0x1605, "CANADIAN SYLLABICS CARRIER NEE" },
    { 0x1606, "CANADIAN SYLLABICS CARRIER NI" },
    { 0x1607, "CANADIAN SYLLABICS CARRIER NA" },
    { 0x1608, "CANADIAN SYLLABICS CARRIER MU" },
    { 0x1609, "CANADIAN SYLLABICS CARRIER MO" },
    { 0x160A, "CANADIAN SYLLABICS CARRIER ME" },
    { 0x160B, "CANADIAN SYLLABICS CARRIER MEE" },
    { 0x160C, "CANADIAN SYLLABICS CARRIER MI" },
    { 0x160D, "CANADIAN SYLLABICS CARRIER MA" },
    { 0x160E, "CANADIAN SYLLABICS CARRIER YU" },
    { 0x160F, "CANADIAN SYLLABICS CARRIER YO" },
    { 0x1610, "CANADIAN SYLLABICS CARRIER YE" },
    { 0x1611, "CANADIAN SYLLABICS CARRIER YEE" },
    { 0x1612, "CANADIAN SYLLABICS CARRIER YI" },
    { 0x1613, "CANADIAN SYLLABICS CARRIER YA" },
    { 0x1614, "CANADIAN SYLLABICS CARRIER JU" },
    { 0x1615, "CANADIAN SYLLABICS SAYISI JU" },
    { 0x1616, "CANADIAN SYLLABICS CARRIER JO" },
    { 0x1617, "CANADIAN SYLLABICS CARRIER JE" },
    { 0x1618, "CANADIAN SYLLABICS CARRIER JEE" },
    { 0x1619, "CANADIAN SYLLABICS CARRIER JI" },
    { 0x161A, "CANADIAN SYLLABICS SAYISI JI" },
    { 0x161B, "CANADIAN SYLLABICS CARRIER JA" },
    { 0x161C, "CANADIAN SYLLABICS CARRIER JJU" },
    { 0x161D, "CANADIAN SYLLABICS CARRIER JJO" },
    { 0x161E, "CANADIAN SYLLABICS CARRIER JJE" },
    { 0x161F, "CANADIAN SYLLABICS CARRIER JJEE" },
    { 0x1620, "CANADIAN SYLLABICS CARRIER JJI" },
    { 0x1621, "CANADIAN SYLLABICS CARRIER JJA" },
    { 0x1622, "CANADIAN SYLLABICS CARRIER LU" },
    { 0x1623, "CANADIAN SYLLABICS CARRIER LO" },
    { 0x1624, "CANADIAN SYLLABICS CARRIER LE" },
    { 0x1625, "CANADIAN SYLLABICS CARRIER LEE" },
    { 0x1626, "CANADIAN SYLLABICS CARRIER LI" },
    { 0x1627, "CANADIAN SYLLABICS CARRIER LA" },
    { 0x1628, "CANADIAN SYLLABICS CARRIER DLU" },
    { 0x1629, "CANADIAN SYLLABICS CARRIER DLO" },
    { 0x162A, "CANADIAN SYLLABICS CARRIER DLE" },
    { 0x162B, "CANADIAN SYLLABICS CARRIER DLEE" },
    { 0x162C, "CANADIAN SYLLABICS CARRIER DLI" },
    { 0x162D, "CANADIAN SYLLABICS CARRIER DLA" },
    { 0x162E, "CANADIAN SYLLABICS CARRIER LHU" },
    { 0x162F, "CANADIAN SYLLABICS CARRIER LHO" },
    { 0x1630, "CANADIAN SYLLABICS CARRIER LHE" },
    { 0x1631, "CANADIAN SYLLABICS CARRIER LHEE" },
    { 0x1632, "CANADIAN SYLLABICS CARRIER LHI" },
    { 0x1633, "CANADIAN SYLLABICS CARRIER LHA" },
    { 0x1634, "CANADIAN SYLLABICS CARRIER TLHU" },
    { 0x1635, "CANADIAN SYLLABICS CARRIER TLHO" },
    { 0x1636, "CANADIAN SYLLABICS CARRIER TLHE" },
    { 0x1637, "CANADIAN SYLLABICS CARRIER TLHEE" },
    { 0x1638, "CANADIAN SYLLABICS CARRIER TLHI" },
    { 0x1639, "CANADIAN SYLLABICS CARRIER TLHA" },
    { 0x163A, "CANADIAN SYLLABICS CARRIER TLU" },
    { 0x163B, "CANADIAN SYLLABICS CARRIER TLO" },
    { 0x163C, "CANADIAN SYLLABICS CARRIER TLE" },
    { 0x163D, "CANADIAN SYLLABICS CARRIER TLEE" },
    { 0x163E, "CANADIAN SYLLABICS CARRIER TLI" },
    { 0x163F, "CANADIAN SYLLABICS CARRIER TLA" },
    { 0x1640, "CANADIAN SYLLABICS CARRIER ZU" },
    { 0x1641, "CANADIAN SYLLABICS CARRIER ZO" },
    { 0x1642, "CANADIAN SYLLABICS CARRIER ZE" },
    { 0x1643, "CANADIAN SYLLABICS CARRIER ZEE" },
    { 0x1644, "CANADIAN SYLLABICS CARRIER ZI" },
    { 0x1645, "CANADIAN SYLLABICS CARRIER ZA" },
    { 0x1646, "CANADIAN SYLLABICS CARRIER Z" },
    { 0x1647, "CANADIAN SYLLABICS CARRIER INITIAL Z" },
    { 0x1648, "CANADIAN SYLLABICS CARRIER DZU" },
    { 0x1649, "CANADIAN SYLLABICS CARRIER DZO" },
    { 0x164A, "CANADIAN SYLLABICS CARRIER DZE" },
    { 0x164B, "CANADIAN SYLLABICS CARRIER DZEE" },
    { 0x164C, "CANADIAN SYLLABICS CARRIER DZI" },
    { 0x164D, "CANADIAN SYLLABICS CARRIER DZA" },
    { 0x164E, "CANADIAN SYLLABICS CARRIER SU" },
    { 0x164F, "CANADIAN SYLLABICS CARRIER SO" },
    { 0x1650, "CANADIAN SYLLABICS CARRIER SE" },
    { 0x1651, "CANADIAN SYLLABICS CARRIER SEE" },
    { 0x1652, "CANADIAN SYLLABICS CARRIER SI" },
    { 0x1653, "CANADIAN SYLLABICS CARRIER SA" },
    { 0x1654, "CANADIAN SYLLABICS CARRIER SHU" },
    { 0x1655, "CANADIAN SYLLABICS CARRIER SHO" },
    { 0x1656, "CANADIAN SYLLABICS CARRIER SHE" },
    { 0x1657, "CANADIAN SYLLABICS CARRIER SHEE" },
    { 0x1658, "CANADIAN SYLLABICS CARRIER SHI" },
    { 0x1659, "CANADIAN SYLLABICS CARRIER SHA" },
    { 0x165A, "CANADIAN SYLLABICS CARRIER SH" },
    { 0x165B, "CANADIAN SYLLABICS CARRIER TSU" },
    { 0x165C, "CANADIAN SYLLABICS CARRIER TSO" },
    { 0x165D, "CANADIAN SYLLABICS CARRIER TSE" },
    { 0x165E, "CANADIAN SYLLABICS CARRIER TSEE" },
    { 0x165F, "CANADIAN SYLLABICS CARRIER TSI" },
    { 0x1660, "CANADIAN SYLLABICS CARRIER TSA" },
    { 0x1661, "CANADIAN SYLLABICS CARRIER CHU" },
    { 0x1662, "CANADIAN SYLLABICS CARRIER CHO" },
    { 0x1663, "CANADIAN SYLLABICS CARRIER CHE" },
    { 0x1664, "CANADIAN SYLLABICS CARRIER CHEE" },
    { 0x1665, "CANADIAN SYLLABICS CARRIER CHI" },
    { 0x1666, "CANADIAN SYLLABICS CARRIER CHA" },
    { 0x1667, "CANADIAN SYLLABICS CARRIER TTSU" },
    { 0x1668, "CANADIAN SYLLABICS CARRIER TTSO" },
    { 0x1669, "CANADIAN SYLLABICS CARRIER TTSE" },
    { 0x166A, "CANADIAN SYLLABICS CARRIER TTSEE" },
    { 0x166B, "CANADIAN SYLLABICS CARRIER TTSI" },
    { 0x166C, "CANADIAN SYLLABICS CARRIER TTSA" },
    { 0x166D, "CANADIAN SYLLABICS CHI SIGN" },
    { 0x166E, "CANADIAN SYLLABICS FULL STOP" },
    { 0x166F, "CANADIAN SYLLABICS QAI" },
    { 0x1670, "CANADIAN SYLLABICS NGAI" },
    { 0x1671, "CANADIAN SYLLABICS NNGI" },
    { 0x1672, "CANADIAN SYLLABICS NNGII" },
    { 0x1673, "CANADIAN SYLLABICS NNGO" },
    { 0x1674, "CANADIAN SYLLABICS NNGOO" },
    { 0x1675, "CANADIAN SYLLABICS NNGA" },
    { 0x1676, "CANADIAN SYLLABICS NNGAA" },
    { 0x1677, "CANADIAN SYLLABICS WOODS-CREE THWEE" },
    { 0x1678, "CANADIAN SYLLABICS WOODS-CREE THWI" },
    { 0x1679, "CANADIAN SYLLABICS WOODS-CREE THWII" },
    { 0x167A, "CANADIAN SYLLABICS WOODS-CREE THWO" },
    { 0x167B, "CANADIAN SYLLABICS WOODS-CREE THWOO" },
    { 0x167C, "CANADIAN SYLLABICS WOODS-CREE THWA" },
    { 0x167D, "CANADIAN SYLLABICS WOODS-CREE THWAA" },
    { 0x167E, "CANADIAN SYLLABICS WOODS-CREE FINAL TH" },
    { 0x167F, "CANADIAN SYLLABICS BLACKFOOT W" },
    { 0x1680, "OGHAM SPACE MARK" },
    { 0x1681, "OGHAM LETTER BEITH" },
    { 0x1682, "OGHAM LETTER LUIS" },
    { 0x1683, "OGHAM LETTER FEARN" },
    { 0x1684, "OGHAM LETTER SAIL" },
    { 0x1685, "OGHAM LETTER NION" },
    { 0x1686, "OGHAM LETTER UATH" },
    { 0x1687, "OGHAM LETTER DAIR" },
    { 0x1688, "OGHAM LETTER TINNE" },
    { 0x1689, "OGHAM LETTER COLL" },
    { 0x168A, "OGHAM LETTER CEIRT" },
    { 0x168B, "OGHAM LETTER MUIN" },
    { 0x168C, "OGHAM LETTER GORT" },
    { 0x168D, "OGHAM LETTER NGEADAL" },
    { 0x168E, "OGHAM LETTER STRAIF" },
    { 0x168F, "OGHAM LETTER RUIS" },
    { 0x1690, "OGHAM LETTER AILM" },
    { 0x1691, "OGHAM LETTER ONN" },
    { 0x1692, "OGHAM LETTER UR" },
    { 0x1693, "OGHAM LETTER EADHADH" },
    { 0x1694, "OGHAM LETTER IODHADH" },
    { 0x1695, "OGHAM LETTER EABHADH" },
    { 0x1696, "OGHAM LETTER OR" },
    { 0x1697, "OGHAM LETTER UILLEANN" },
    { 0x1698, "OGHAM LETTER IFIN" },
    { 0x1699, "OGHAM LETTER EAMHANCHOLL" },
    { 0x169A, "OGHAM LETTER PEITH" },
    { 0x169B, "OGHAM FEATHER MARK" },
    { 0x169C, "OGHAM REVERSED FEATHER MARK" },
    { 0x16A0, "RUNIC LETTER FEHU FEOH FE F" },
    { 0x16A1, "RUNIC LETTER V" },
    { 0x16A2, "RUNIC LETTER URUZ UR U" },
    { 0x16A3, "RUNIC LETTER YR" },
    { 0x16A4, "RUNIC LETTER Y" },
    { 0x16A5, "RUNIC LETTER W" },
    { 0x16A6, "RUNIC LETTER THURISAZ THURS THORN" },
    { 0x16A7, "RUNIC LETTER ETH" },
    { 0x16A8, "RUNIC LETTER ANSUZ A" },
    { 0x16A9, "RUNIC LETTER OS O" },
    { 0x16AA, "RUNIC LETTER AC A" },
    { 0x16AB, "RUNIC LETTER AESC" },
    { 0x16AC, "RUNIC LETTER LONG-BRANCH-OSS O" },
    { 0x16AD, "RUNIC LETTER SHORT-TWIG-OSS O" },
    { 0x16AE, "RUNIC LETTER O" },
    { 0x16AF, "RUNIC LETTER OE" },
    { 0x16B0, "RUNIC LETTER ON" },
    { 0x16B1, "RUNIC LETTER RAIDO RAD REID R" },
    { 0x16B2, "RUNIC LETTER KAUNA" },
    { 0x16B3, "RUNIC LETTER CEN" },
    { 0x16B4, "RUNIC LETTER KAUN K" },
    { 0x16B5, "RUNIC LETTER G" },
    { 0x16B6, "RUNIC LETTER ENG" },
    { 0x16B7, "RUNIC LETTER GEBO GYFU G" },
    { 0x16B8, "RUNIC LETTER GAR" },
    { 0x16B9, "RUNIC LETTER WUNJO WYNN W" },
    { 0x16BA, "RUNIC LETTER HAGLAZ H" },
    { 0x16BB, "RUNIC LETTER HAEGL H" },
    { 0x16BC, "RUNIC LETTER LONG-BRANCH-HAGALL H" },
    { 0x16BD, "RUNIC LETTER SHORT-TWIG-HAGALL H" },
    { 0x16BE, "RUNIC LETTER NAUDIZ NYD NAUD N" },
    { 0x16BF, "RUNIC LETTER SHORT-TWIG-NAUD N" },
    { 0x16C0, "RUNIC LETTER DOTTED-N" },
    { 0x16C1, "RUNIC LETTER ISAZ IS ISS I" },
    { 0x16C2, "RUNIC LETTER E" },
    { 0x16C3, "RUNIC LETTER JERAN J" },
    { 0x16C4, "RUNIC LETTER GER" },
    { 0x16C5, "RUNIC LETTER LONG-BRANCH-AR AE" },
    { 0x16C6, "RUNIC LETTER SHORT-TWIG-AR A" },
    { 0x16C7, "RUNIC LETTER IWAZ EOH" },
    { 0x16C8, "RUNIC LETTER PERTHO PEORTH P" },
    { 0x16C9, "RUNIC LETTER ALGIZ EOLHX" },
    { 0x16CA, "RUNIC LETTER SOWILO S" },
    { 0x16CB, "RUNIC LETTER SIGEL LONG-BRANCH-SOL S" },
    { 0x16CC, "RUNIC LETTER SHORT-TWIG-SOL S" },
    { 0x16CD, "RUNIC LETTER C" },
    { 0x16CE, "RUNIC LETTER Z" },
    { 0x16CF, "RUNIC LETTER TIWAZ TIR TYR T" },
    { 0x16D0, "RUNIC LETTER SHORT-TWIG-TYR T" },
    { 0x16D1, "RUNIC LETTER D" },
    { 0x16D2, "RUNIC LETTER BERKANAN BEORC BJARKAN B" },
    { 0x16D3, "RUNIC LETTER SHORT-TWIG-BJARKAN B" },
    { 0x16D4, "RUNIC LETTER DOTTED-P" },
    { 0x16D5, "RUNIC LETTER OPEN-P" },
    { 0x16D6, "RUNIC LETTER EHWAZ EH E" },
    { 0x16D7, "RUNIC LETTER MANNAZ MAN M" },
    { 0x16D8, "RUNIC LETTER LONG-BRANCH-MADR M" },
    { 0x16D9, "RUNIC LETTER SHORT-TWIG-MADR M" },
    { 0x16DA, "RUNIC LETTER LAUKAZ LAGU LOGR L" },
    { 0x16DB, "RUNIC LETTER DOTTED-L" },
    { 0x16DC, "RUNIC LETTER INGWAZ" },
    { 0x16DD, "RUNIC LETTER ING" },
    { 0x16DE, "RUNIC LETTER DAGAZ DAEG D" },
    { 0x16DF, "RUNIC LETTER OTHALAN ETHEL O" },
    { 0x16E0, "RUNIC LETTER EAR" },
    { 0x16E1, "RUNIC LETTER IOR" },
    { 0x16E2, "RUNIC LETTER CWEORTH" },
    { 0x16E3, "RUNIC LETTER CALC" },
    { 0x16E4, "RUNIC LETTER CEALC" },
    { 0x16E5, "RUNIC LETTER STAN" },
    { 0x16E6, "RUNIC LETTER LONG-BRANCH-YR" },
    { 0x16E7, "RUNIC LETTER SHORT-TWIG-YR" },
    { 0x16E8, "RUNIC LETTER ICELANDIC-YR" },
    { 0x16E9, "RUNIC LETTER Q" },
    { 0x16EA, "RUNIC LETTER X" },
    { 0x16EB, "RUNIC SINGLE PUNCTUATION" },
    { 0x16EC, "RUNIC MULTIPLE PUNCTUATION" },
    { 0x16ED, "RUNIC CROSS PUNCTUATION" },
    { 0x16EE, "RUNIC ARLAUG SYMBOL" },
    { 0x16EF, "RUNIC TVIMADUR SYMBOL" },
    { 0x16F0, "RUNIC BELGTHOR SYMBOL" },
    { 0x16F1, "RUNIC LETTER K" },
    { 0x16F2, "RUNIC LETTER SH" },
    { 0x16F3, "RUNIC LETTER OO" },
    { 0x16F4, "RUNIC LETTER FRANKS CASKET OS" },
    { 0x16F5, "RUNIC LETTER FRANKS CASKET IS" },
    { 0x16F6, "RUNIC LETTER FRANKS CASKET EH" },
    { 0x16F7, "RUNIC LETTER FRANKS CASKET AC" },
    { 0x16F8, "RUNIC LETTER FRANKS CASKET AESC" },
    { 0x1700, "TAGALOG LETTER A" },
    { 0x1701, "TAGALOG LETTER I" },
    { 0x1702, "TAGALOG LETTER U" },
    { 0x1703, "TAGALOG LETTER KA" },
    { 0x1704, "TAGALOG LETTER GA" },
    { 0x1705, "TAGALOG LETTER NGA" },
    { 0x1706, "TAGALOG LETTER TA" },
    { 0x1707, "TAGALOG LETTER DA" },
    { 0x1708, "TAGALOG LETTER NA" },
    { 0x1709, "TAGALOG LETTER PA" },
    { 0x170A, "TAGALOG LETTER BA" },
    { 0x170B, "TAGALOG LETTER MA" },
    { 0x170C, "TAGALOG LETTER YA" },
    { 0x170D, "TAGALOG LETTER RA" },
    { 0x170E, "TAGALOG LETTER LA" },
    { 0x170F, "TAGALOG LETTER WA" },
    { 0x1710, "TAGALOG LETTER SA" },
    { 0x1711, "TAGALOG LETTER HA" },
    { 0x1712, "TAGALOG VOWEL SIGN I" },
    { 0x1713, "TAGALOG VOWEL SIGN U" },
    { 0x1714, "TAGALOG SIGN VIRAMA" },
    { 0x1715, "TAGALOG SIGN PAMUDPOD" },
    { 0x171F, "TAGALOG LETTER ARCHAIC RA" },
    { 0x1720, "HANUNOO LETTER A" },
    { 0x1721, "HANUNOO LETTER I" },
    { 0x1722, "HANUNOO LETTER U" },
    { 0x1723, "HANUNOO LETTER KA" },
    { 0x1724, "HANUNOO LETTER GA" },
    { 0x1725, "HANUNOO LETTER NGA" },
    { 0x1726, "HANUNOO LETTER TA" },
    { 0x1727, "HANUNOO LETTER DA" },
    { 0x1728, "HANUNOO LETTER NA" },
    { 0x1729, "HANUNOO LETTER PA" },
    { 0x172A, "HANUNOO LETTER BA" },
    { 0x172B, "HANUNOO LETTER MA" },
    { 0x172C, "HANUNOO LETTER YA" },
    { 0x172D, "HANUNOO LETTER RA" },
    { 0x172E, "HANUNOO LETTER LA" },
    { 0x172F, "HANUNOO LETTER WA" },
    { 0x1730, "HANUNOO LETTER SA" },
    { 0x1731, "HANUNOO LETTER HA" },
    { 0x1732, "HANUNOO VOWEL SIGN I" },
    { 0x1733, "HANUNOO VOWEL SIGN U" },
    { 0x1734, "HANUNOO SIGN PAMUDPOD" },
    { 0x1735, "PHILIPPINE SINGLE PUNCTUATION" },
    { 0x1736, "PHILIPPINE DOUBLE PUNCTUATION" },
    { 0x1740, "BUHID LETTER A" },
    { 0x1741, "BUHID LETTER I" },
    { 0x1742, "BUHID LETTER U" },
    { 0x1743, "BUHID LETTER KA" },
    { 0x1744, "BUHID LETTER GA" },
    { 0x1745, "BUHID LETTER NGA" },
    { 0x1746, "BUHID LETTER TA" },
    { 0x1747, "BUHID LETTER DA" },
    { 0x1748, "BUHID LETTER NA" },
    { 0x1749, "BUHID LETTER PA" },
    { 0x174A, "BUHID LETTER BA" },
    { 0x174B, "BUHID LETTER MA" },
    { 0x174C, "BUHID LETTER YA" },
    { 0x174D, "BUHID LETTER RA" },
    { 0x174E, "BUHID LETTER LA" },
    { 0x174F, "BUHID LETTER WA" },
    { 0x1750, "BUHID LETTER SA" },
    { 0x1751, "BUHID LETTER HA" },
    { 0x1752, "BUHID VOWEL SIGN I" },
    { 0x1753, "BUHID VOWEL SIGN U" },
    { 0x1760, "TAGBANWA LETTER A" },
    { 0x1761, "TAGBANWA LETTER I" },
    { 0x1762, "TAGBANWA LETTER U" },
    { 0x1763, "TAGBANWA LETTER KA" },
    { 0x1764, "TAGBANWA LETTER GA" },
    { 0x1765, "TAGBANWA LETTER NGA" },
    { 0x1766, "TAGBANWA LETTER TA" },
    { 0x1767, "TAGBANWA LETTER DA" },
    { 0x1768, "TAGBANWA LETTER NA" },
    { 0x1769, "TAGBANWA LETTER PA" },
    { 0x176A, "TAGBANWA LETTER BA" },
    { 0x176B, "TAGBANWA LETTER MA" },
    { 0x176C, "TAGBANWA LETTER YA" },
    { 0x176E, "TAGBANWA LETTER LA" },
    { 0x176F, "TAGBANWA LETTER WA" },
    { 0x1770, "TAGBANWA LETTER SA" },
    { 0x1772, "TAGBANWA VOWEL SIGN I" },
    { 0x1773, "TAGBANWA VOWEL SIGN U" },
    { 0x1780, "KHMER LETTER KA" },
    { 0x1781, "KHMER LETTER KHA" },
    { 0x1782, "KHMER LETTER KO" },
    { 0x1783, "KHMER LETTER KHO" },
    { 0x1784, "KHMER LETTER NGO" },
    { 0x1785, "KHMER LETTER CA" },
    { 0x1786, "KHMER LETTER CHA" },
    { 0x1787, "KHMER LETTER CO" },
    { 0x1788, "KHMER LETTER CHO" },
    { 0x1789, "KHMER LETTER NYO" },
    { 0x178A, "KHMER LETTER DA" },
    { 0x178B, "KHMER LETTER TTHA" },
    { 0x178C, "KHMER LETTER DO" },
    { 0x178D, "KHMER LETTER TTHO" },
    { 0x178E, "KHMER LETTER NNO" },
    { 0x178F, "KHMER LETTER TA" },
    { 0x1790, "KHMER LETTER THA" },
    { 0x1791, "KHMER LETTER TO" },
    { 0x1792, "KHMER LETTER THO" },
    { 0x1793, "KHMER LETTER NO" },
    { 0x1794, "KHMER LETTER BA" },
    { 0x1795, "KHMER LETTER PHA" },
    { 0x1796, "KHMER LETTER PO" },
    { 0x1797, "KHMER LETTER PHO" },
    { 0x1798, "KHMER LETTER MO" },
    { 0x1799, "KHMER LETTER YO" },
    { 0x179A, "KHMER LETTER RO" },
    { 0x179B, "KHMER LETTER LO" },
    { 0x179C, "KHMER LETTER VO" },
    { 0x179D, "KHMER LETTER SHA" },
    { 0x179E, "KHMER LETTER SSO" },
    { 0x179F, "KHMER LETTER SA" },
    { 0x17A0, "KHMER LETTER HA" },
    { 0x17A1, "KHMER LETTER LA" },
    { 0x17A2, "KHMER LETTER QA" },
    { 0x17A3, "KHMER INDEPENDENT VOWEL QAQ" },
    { 0x17A4, "KHMER INDEPENDENT VOWEL QAA" },
    { 0x17A5, "KHMER INDEPENDENT VOWEL QI" },
    { 0x17A6, "KHMER INDEPENDENT VOWEL QII" },
    { 0x17A7, "KHMER INDEPENDENT VOWEL QU" },
    { 0x17A8, "KHMER INDEPENDENT VOWEL QUK" },
    { 0x17A9, "KHMER INDEPENDENT VOWEL QUU" },
    { 0x17AA, "KHMER INDEPENDENT VOWEL QUUV" },
    { 0x17AB, "KHMER INDEPENDENT VOWEL RY" },
    { 0x17AC, "KHMER INDEPENDENT VOWEL RYY" },
    { 0x17AD, "KHMER INDEPENDENT VOWEL LY" },
    { 0x17AE, "KHMER INDEPENDENT VOWEL LYY" },
    { 0x17AF, "KHMER INDEPENDENT VOWEL QE" },
    { 0x17B0, "KHMER INDEPENDENT VOWEL QAI" },
    { 0x17B1, "KHMER INDEPENDENT VOWEL QOO TYPE ONE" },
    { 0x17B2, "KHMER INDEPENDENT VOWEL QOO TYPE TWO" },
    { 0x17B3, "KHMER INDEPENDENT VOWEL QAU" },
    { 0x17B4, "KHMER VOWEL INHERENT AQ" },
    { 0x17B5, "KHMER VOWEL INHERENT AA" },
    { 0x17B6, "KHMER VOWEL SIGN AA" },
    { 0x17B7, "KHMER VOWEL SIGN I" },
    { 0x17B8, "KHMER VOWEL SIGN II" },
    { 0x17B9, "KHMER VOWEL SIGN Y" },
    { 0x17BA, "KHMER VOWEL SIGN YY" },
    { 0x17BB, "KHMER VOWEL SIGN U" },
    { 0x17BC, "KHMER VOWEL SIGN UU" },
    { 0x17BD, "KHMER VOWEL SIGN UA" },
    { 0x17BE, "KHMER VOWEL SIGN OE" },
    { 0x17BF, "KHMER VOWEL SIGN YA" },
    { 0x17C0, "KHMER VOWEL SIGN IE" },
    { 0x17C1, "KHMER VOWEL SIGN E" },
    { 0x17C2, "KHMER VOWEL SIGN AE" },
    { 0x17C3, "KHMER VOWEL SIGN AI" },
    { 0x17C4, "KHMER VOWEL SIGN OO" },
    { 0x17C5, "KHMER VOWEL SIGN AU" },
    { 0x17C6, "KHMER SIGN NIKAHIT" },
    { 0x17C7, "KHMER SIGN REAHMUK" },
    { 0x17C8, "KHMER SIGN YUUKALEAPINTU" },
    { 0x17C9, "KHMER SIGN MUUSIKATOAN" },
    { 0x17CA, "KHMER SIGN TRIISAP" },
    { 0x17CB, "KHMER SIGN BANTOC" },
    { 0x17CC, "KHMER SIGN ROBAT" },
    { 0x17CD, "KHMER SIGN TOANDAKHIAT" },
    { 0x17CE, "KHMER SIGN KAKABAT" },
    { 0x17CF, "KHMER SIGN AHSDA" },
    { 0x17D0, "KHMER SIGN SAMYOK SANNYA" },
    { 0x17D1, "KHMER SIGN VIRIAM" },
    { 0x17D2, "KHMER SIGN COENG" },
    { 0x17D3, "KHMER SIGN BATHAMASAT" },
    { 0x17D4, "KHMER SIGN KHAN" },
    { 0x17D5, "KHMER SIGN BARIYOOSAN" },
    { 0x17D6, "KHMER SIGN CAMNUC PII KUUH" },
    { 0x17D7, "KHMER SIGN LEK TOO" },
    { 0x17D8, "KHMER SIGN BEYYAL" },
    { 0x17D9, "KHMER SIGN PHNAEK MUAN" },
    { 0x17DA, "KHMER SIGN KOOMUUT" },
    { 0x17DB, "KHMER CURRENCY SYMBOL RIEL" },
    { 0x17DC, "KHMER SIGN AVAKRAHASANYA" },
    { 0x17DD, "KHMER SIGN ATTHACAN" },
    { 0x17E0, "KHMER DIGIT ZERO" },
    { 0x17E1, "KHMER DIGIT ONE" },
    { 0x17E2, "KHMER DIGIT TWO" },
    { 0x17E3, "KHMER DIGIT THREE" },
    { 0x17E4, "KHMER DIGIT FOUR" },
    { 0x17E5, "KHMER DIGIT FIVE" },
    { 0x17E6, "KHMER DIGIT SIX" },
    { 0x17E7, "KHMER DIGIT SEVEN" },
    { 0x17E8, "KHMER DIGIT EIGHT" },
    { 0x17E9, "KHMER DIGIT NINE" },
    { 0x17F0, "KHMER SYMBOL LEK ATTAK SON" },
    { 0x17F1, "KHMER SYMBOL LEK ATTAK MUOY" },
    { 0x17F2, "KHMER SYMBOL LEK ATTAK PII" },
    { 0x17F3, "KHMER SYMBOL LEK ATTAK BEI" },
    { 0x17F4, "KHMER SYMBOL LEK ATTAK BUON" },
    { 0x17F5, "KHMER SYMBOL LEK ATTAK PRAM" },
    { 0x17F6, "KHMER SYMBOL LEK ATTAK PRAM-MUOY" },
    { 0x17F7, "KHMER SYMBOL LEK ATTAK PRAM-PII" },
    { 0x17F8, "KHMER SYMBOL LEK ATTAK PRAM-BEI" },
    { 0x17F9, "KHMER SYMBOL LEK ATTAK PRAM-BUON" },
    { 0x1800, "MONGOLIAN BIRGA" },
    { 0x1801, "MONGOLIAN ELLIPSIS" },
    { 0x1802, "MONGOLIAN COMMA" },
    { 0x1803, "MONGOLIAN FULL STOP" },
    { 0x1804, "MONGOLIAN COLON" },
    { 0x1805, "MONGOLIAN FOUR DOTS" },
    { 0x1806, "MONGOLIAN TODO SOFT HYPHEN" },
    { 0x1807, "MONGOLIAN SIBE SYLLABLE BOUNDARY MARKER" },
    { 0x1808, "MONGOLIAN MANCHU COMMA" },
    { 0x1809, "MONGOLIAN MANCHU FULL STOP" },
    { 0x180A, "MONGOLIAN NIRUGU" },
    { 0x180B, "MONGOLIAN FREE VARIATION SELECTOR ONE" },
    { 0x180C, "MONGOLIAN FREE VARIATION SELECTOR TWO" },
    { 0x180D, "MONGOLIAN FREE VARIATION SELECTOR THREE" },
    { 0x180E, "MONGOLIAN VOWEL SEPARATOR" },
    { 0x180F, "MONGOLIAN FREE VARIATION SELECTOR FOUR" },
    { 0x1810, "MONGOLIAN DIGIT ZERO" },
    { 0x1811, "MONGOLIAN DIGIT ONE" },
    { 0x1812, "MONGOLIAN DIGIT TWO" },
    { 0x1813, "MONGOLIAN DIGIT THREE" },
    { 0x1814, "MONGOLIAN DIGIT FOUR" },
    { 0x1815, "MONGOLIAN DIGIT FIVE" },
    { 0x1816, "MONGOLIAN DIGIT SIX" },
    { 0x1817, "MONGOLIAN DIGIT SEVEN" },
    { 0x1818, "MONGOLIAN DIGIT EIGHT" },
    { 0x1819, "MONGOLIAN DIGIT NINE" },
    { 0x1820, "MONGOLIAN LETTER A" },
    { 0x1821, "MONGOLIAN LETTER E" },
    { 0x1822, "MONGOLIAN LETTER I" },
    { 0x1823, "MONGOLIAN LETTER O" },
    { 0x1824, "MONGOLIAN LETTER U" },
    { 0x1825, "MONGOLIAN LETTER OE" },
    { 0x1826, "MONGOLIAN LETTER UE" },
    { 0x1827, "MONGOLIAN LETTER EE" },
    { 0x1828, "MONGOLIAN LETTER NA" },
    { 0x1829, "MONGOLIAN LETTER ANG" },
    { 0x182A, "MONGOLIAN LETTER BA" },
    { 0x182B, "MONGOLIAN LETTER PA" },
    { 0x182C, "MONGOLIAN LETTER QA" },
    { 0x182D, "MONGOLIAN LETTER GA" },
    { 0x182E, "MONGOLIAN LETTER MA" },
    { 0x182F, "MONGOLIAN LETTER LA" },
    { 0x1830, "MONGOLIAN LETTER SA" },
    { 0x1831, "MONGOLIAN LETTER SHA" },
    { 0x1832, "MONGOLIAN LETTER TA" },
    { 0x1833, "MONGOLIAN LETTER DA" },
    { 0x1834, "MONGOLIAN LETTER CHA" },
    { 0x1835, "MONGOLIAN LETTER JA" },
    { 0x1836, "MONGOLIAN LETTER YA" },
    { 0x1837, "MONGOLIAN LETTER RA" },
    { 0x1838, "MONGOLIAN LETTER WA" },
    { 0x1839, "MONGOLIAN LETTER FA" },
    { 0x183A, "MONGOLIAN LETTER KA" },
    { 0x183B, "MONGOLIAN LETTER KHA" },
    { 0x183C, "MONGOLIAN LETTER TSA" },
    { 0x183D, "MONGOLIAN LETTER ZA" },
    { 0x183E, "MONGOLIAN LETTER HAA" },
    { 0x183F, "MONGOLIAN LETTER ZRA" },
    { 0x1840, "MONGOLIAN LETTER LHA" },
    { 0x1841, "MONGOLIAN LETTER ZHI" },
    { 0x1842, "MONGOLIAN LETTER CHI" },
    { 0x1843, "MONGOLIAN LETTER TODO LONG VOWEL SIGN" },
    { 0x1844, "MONGOLIAN LETTER TODO E" },
    { 0x1845, "MONGOLIAN LETTER TODO I" },
    { 0x1846, "MONGOLIAN LETTER TODO O" },
    { 0x1847, "MONGOLIAN LETTER TODO U" },
    { 0x1848, "MONGOLIAN LETTER TODO OE" },
    { 0x1849, "MONGOLIAN LETTER TODO UE" },
    { 0x184A, "MONGOLIAN LETTER TODO ANG" },
    { 0x184B, "MONGOLIAN LETTER TODO BA" },
    { 0x184C, "MONGOLIAN LETTER TODO PA" },
    { 0x184D, "MONGOLIAN LETTER TODO QA" },
    { 0x184E, "MONGOLIAN LETTER TODO GA" },
    { 0x184F, "MONGOLIAN LETTER TODO MA" },
    { 0x1850, "MONGOLIAN LETTER TODO TA" },
    { 0x1851, "MONGOLIAN LETTER TODO DA" },
    { 0x1852, "MONGOLIAN LETTER TODO CHA" },
    { 0x1853, "MONGOLIAN LETTER TODO JA" },
    { 0x1854, "MONGOLIAN LETTER TODO TSA" },
    { 0x1855, "MONGOLIAN LETTER TODO YA" },
    { 0x1856, "MONGOLIAN LETTER TODO WA" },
    { 0x1857, "MONGOLIAN LETTER TODO KA" },
    { 0x1858, "MONGOLIAN LETTER TODO GAA" },
    { 0x1859, "MONGOLIAN LETTER TODO HAA" },
    { 0x185A, "MONGOLIAN LETTER TODO JIA" },
    { 0x185B, "MONGOLIAN LETTER TODO NIA" },
    { 0x185C, "MONGOLIAN LETTER TODO DZA" },
    { 0x185D, "MONGOLIAN LETTER SIBE E" },
    { 0x185E, "MONGOLIAN LETTER SIBE I" },
    { 0x185F, "MONGOLIAN LETTER SIBE IY" },
    { 0x1860, "MONGOLIAN LETTER SIBE UE" },
    { 0x1861, "MONGOLIAN LETTER SIBE U" },
    { 0x1862, "MONGOLIAN LETTER SIBE ANG" },
    { 0x1863, "MONGOLIAN LETTER SIBE KA" },
    { 0x1864, "MONGOLIAN LETTER SIBE GA" },
    { 0x1865, "MONGOLIAN LETTER SIBE HA" },
    { 0x1866, "MONGOLIAN LETTER SIBE PA" },
    { 0x1867, "MONGOLIAN LETTER SIBE SHA" },
    { 0x1868, "MONGOLIAN LETTER SIBE TA" },
    { 0x1869, "MONGOLIAN LETTER SIBE DA" },
    { 0x186A, "MONGOLIAN LETTER SIBE JA" },
    { 0x186B, "MONGOLIAN LETTER SIBE FA" },
    { 0x186C, "MONGOLIAN LETTER SIBE GAA" },
    { 0x186D, "MONGOLIAN LETTER SIBE HAA" },
    { 0x186E, "MONGOLIAN LETTER SIBE TSA" },
    { 0x186F, "MONGOLIAN LETTER SIBE ZA" },
    { 0x1870, "MONGOLIAN LETTER SIBE RAA" },
    { 0x1871, "MONGOLIAN LETTER SIBE CHA" },
    { 0x1872, "MONGOLIAN LETTER SIBE ZHA" },
    { 0x1873, "MONGOLIAN LETTER MANCHU I" },
    { 0x1874, "MONGOLIAN LETTER MANCHU KA" },
    { 0x1875, "MONGOLIAN LETTER MANCHU RA" },
    { 0x1876, "MONGOLIAN LETTER MANCHU FA" },
    { 0x1877, "MONGOLIAN LETTER MANCHU ZHA" },
    { 0x1878, "MONGOLIAN LETTER CHA WITH TWO DOTS" },
    { 0x1880, "MONGOLIAN LETTER ALI GALI ANUSVARA ONE" },
    { 0x1881, "MONGOLIAN LETTER ALI GALI VISARGA ONE" },
    { 0x1882, "MONGOLIAN LETTER ALI GALI DAMARU" },
    { 0x1883, "MONGOLIAN LETTER ALI GALI UBADAMA" },
    { 0x1884, "MONGOLIAN LETTER ALI GALI INVERTED UBADAMA" },
    { 0x1885, "MONGOLIAN LETTER ALI GALI BALUDA" },
    { 0x1886, "MONGOLIAN LETTER ALI GALI THREE BALUDA" },
    { 0x1887, "MONGOLIAN LETTER ALI GALI A" },
    { 0x1888, "MONGOLIAN LETTER ALI GALI I" },
    { 0x1889, "MONGOLIAN LETTER ALI GALI KA" },
    { 0x188A, "MONGOLIAN LETTER ALI GALI NGA" },
    { 0x188B, "MONGOLIAN LETTER ALI GALI CA" },
    { 0x188C, "MONGOLIAN LETTER ALI GALI TTA" },
    { 0x188D, "MONGOLIAN LETTER ALI GALI TTHA" },
    { 0x188E, "MONGOLIAN LETTER ALI GALI DDA" },
    { 0x188F, "MONGOLIAN LETTER ALI GALI NNA" },
    { 0x1890, "MONGOLIAN LETTER ALI GALI TA" },
    { 0x1891, "MONGOLIAN LETTER ALI GALI DA" },
    { 0x1892, "MONGOLIAN LETTER ALI GALI PA" },
    { 0x1893, "MONGOLIAN LETTER ALI GALI PHA" },
    { 0x1894, "MONGOLIAN LETTER ALI GALI SSA" },
    { 0x1895, "MONGOLIAN LETTER ALI GALI ZHA" },
    { 0x1896, "MONGOLIAN LETTER ALI GALI ZA" },
    { 0x1897, "MONGOLIAN LETTER ALI GALI AH" },
    { 0x1898, "MONGOLIAN LETTER TODO ALI GALI TA" },
    { 0x1899, "MONGOLIAN LETTER TODO ALI GALI ZHA" },
    { 0x189A, "MONGOLIAN LETTER MANCHU ALI GALI GHA" },
    { 0x189B, "MONGOLIAN LETTER MANCHU ALI GALI NGA" },
    { 0x189C, "MONGOLIAN LETTER MANCHU ALI GALI CA" },
    { 0x189D, "MONGOLIAN LETTER MANCHU ALI GALI JHA" },
    { 0x189E, "MONGOLIAN LETTER MANCHU ALI GALI TTA" },
    { 0x189F, "MONGOLIAN LETTER MANCHU ALI GALI DDHA" },
    { 0x18A0, "MONGOLIAN LETTER MANCHU ALI GALI TA" },
    { 0x18A1, "MONGOLIAN LETTER MANCHU ALI GALI DHA" },
    { 0x18A2, "MONGOLIAN LETTER MANCHU ALI GALI SSA" },
    { 0x18A3, "MONGOLIAN LETTER MANCHU ALI GALI CYA" },
    { 0x18A4, "MONGOLIAN LETTER MANCHU ALI GALI ZHA" },
    { 0x18A5, "MONGOLIAN LETTER MANCHU ALI GALI ZA" },
    { 0x18A6, "MONGOLIAN LETTER ALI GALI HALF U" },
    { 0x18A7, "MONGOLIAN LETTER ALI GALI HALF YA" },
    { 0x18A8, "MONGOLIAN LETTER MANCHU ALI GALI BHA" },
    { 0x18A9, "MONGOLIAN LETTER ALI GALI DAGALGA" },
    { 0x18AA, "MONGOLIAN LETTER MANCHU ALI GALI LHA" },
    { 0x18B0, "CANADIAN SYLLABICS OY" },
    { 0x18B1, "CANADIAN SYLLABICS AY" },
    { 0x18B2, "CANADIAN SYLLABICS AAY" },
    { 0x18B3, "CANADIAN SYLLABICS WAY" },
    { 0x18B4, "CANADIAN SYLLABICS POY" },
    { 0x18B5, "CANADIAN SYLLABICS PAY" },
    { 0x18B6, "CANADIAN SYLLABICS PWOY" },
    { 0x18B7, "CANADIAN SYLLABICS TAY" },
    { 0x18B8, "CANADIAN SYLLABICS KAY" },
    { 0x18B9, "CANADIAN SYLLABICS KWAY" },
    { 0x18BA, "CANADIAN SYLLABICS MAY" },
    { 0x18BB, "CANADIAN SYLLABICS NOY" },
    { 0x18BC, "CANADIAN SYLLABICS NAY" },
    { 0x18BD, "CANADIAN SYLLABICS LAY" },
    { 0x18BE, "CANADIAN SYLLABICS SOY" },
    { 0x18BF, "CANADIAN SYLLABICS SAY" },
    { 0x18C0, "CANADIAN SYLLABICS SHOY" },
    { 0x18C1, "CANADIAN SYLLABICS SHAY" },
    { 0x18C2, "CANADIAN SYLLABICS SHWOY" },
    { 0x18C3, "CANADIAN SYLLABICS YOY" },
    { 0x18C4, "CANADIAN SYLLABICS YAY" },
    { 0x18C5, "CANADIAN SYLLABICS RAY" },
    { 0x18C6, "CANADIAN SYLLABICS NWI" },
    { 0x18C7, "CANADIAN SYLLABICS OJIBWAY NWI" },
    { 0x18C8, "CANADIAN SYLLABICS NWII" },
    { 0x18C9, "CANADIAN SYLLABICS OJIBWAY NWII" },
    { 0x18CA, "CANADIAN SYLLABICS NWO" },
    { 0x18CB, "CANADIAN SYLLABICS OJIBWAY NWO" },
    { 0x18CC, "CANADIAN SYLLABICS NWOO" },
    { 0x18CD, "CANADIAN SYLLABICS OJIBWAY NWOO" },
    { 0x18CE, "CANADIAN SYLLABICS RWEE" },
    { 0x18CF, "CANADIAN SYLLABICS RWI" },
    { 0x18D0, "CANADIAN SYLLABICS RWII" },
    { 0x18D1, "CANADIAN SYLLABICS RWO" },
    { 0x18D2, "CANADIAN SYLLABICS RWOO" },
    { 0x18D3, "CANADIAN SYLLABICS RWA" },
    { 0x18D4, "CANADIAN SYLLABICS OJIBWAY P" },
    { 0x18D5, "CANADIAN SYLLABICS OJIBWAY T" },
    { 0x18D6, "CANADIAN SYLLABICS OJIBWAY K" },
    { 0x18D7, "CANADIAN SYLLABICS OJIBWAY C" },
    { 0x18D8, "CANADIAN SYLLABICS OJIBWAY M" },
    { 0x18D9, "CANADIAN SYLLABICS OJIBWAY N" },
    { 0x18DA, "CANADIAN SYLLABICS OJIBWAY S" },
    { 0x18DB, "CANADIAN SYLLABICS OJIBWAY SH" },
    { 0x18DC, "CANADIAN SYLLABICS EASTERN W" },
    { 0x18DD, "CANADIAN SYLLABICS WESTERN W" },
    { 0x18DE, "CANADIAN SYLLABICS FINAL SMALL RING" },
    { 0x18DF, "CANADIAN SYLLABICS FINAL RAISED DOT" },
    { 0x18E0, "CANADIAN SYLLABICS R-CREE RWE" },
    { 0x18E1, "CANADIAN SYLLABICS WEST-CREE LOO" },
    { 0x18E2, "CANADIAN SYLLABICS WEST-CREE LAA" },
    { 0x18E3, "CANADIAN SYLLABICS THWE" },
    { 0x18E4, "CANADIAN SYLLABICS THWA" },
    { 0x18E5, "CANADIAN SYLLABICS TTHWE" },
    { 0x18E6, "CANADIAN SYLLABICS TTHOO" },
    { 0x18E7, "CANADIAN SYLLABICS TTHAA" },
    { 0x18E8, "CANADIAN SYLLABICS TLHWE" },
    { 0x18E9, "CANADIAN SYLLABICS TLHOO" },
    { 0x18EA, "CANADIAN SYLLABICS SAYISI SHWE" },
    { 0x18EB, "CANADIAN SYLLABICS SAYISI SHOO" },
    { 0x18EC, "CANADIAN SYLLABICS SAYISI HOO" },
    { 0x18ED, "CANADIAN SYLLABICS CARRIER GWU" },
    { 0x18EE, "CANADIAN SYLLABICS CARRIER DENE GEE" },
    { 0x18EF, "CANADIAN SYLLABICS CARRIER GAA" },
    { 0x18F0, "CANADIAN SYLLABICS CARRIER GWA" },
    { 0x18F1, "CANADIAN SYLLABICS SAYISI JUU" },
    { 0x18F2, "CANADIAN SYLLABICS CARRIER JWA" },
    { 0x18F3, "CANADIAN SYLLABICS BEAVER DENE L" },
    { 0x18F4, "CANADIAN SYLLABICS BEAVER DENE R" },
    { 0x18F5, "CANADIAN SYLLABICS CARRIER DENTAL S" },
    { 0x1900, "LIMBU VOWEL-CARRIER LETTER" },
    { 0x1901, "LIMBU LETTER KA" },
    { 0x1902, "LIMBU LETTER KHA" },
    { 0x1903, "LIMBU LETTER GA" },
    { 0x1904, "LIMBU LETTER GHA" },
    { 0x1905, "LIMBU LETTER NGA" },
    { 0x1906, "LIMBU LETTER CA" },
    { 0x1907, "LIMBU LETTER CHA" },
    { 0x1908, "LIMBU LETTER JA" },
    { 0x1909, "LIMBU LETTER JHA" },
    { 0x190A, "LIMBU LETTER YAN" },
    { 0x190B, "LIMBU LETTER TA" },
    { 0x190C, "LIMBU LETTER THA" },
    { 0x190D, "LIMBU LETTER DA" },
    { 0x190E, "LIMBU LETTER DHA" },
    { 0x190F, "LIMBU LETTER NA" },
    { 0x1910, "LIMBU LETTER PA" },
    { 0x1911, "LIMBU LETTER PHA" },
    { 0x1912, "LIMBU LETTER BA" },
    { 0x1913, "LIMBU LETTER BHA" },
    { 0x1914, "LIMBU LETTER MA" },
    { 0x1915, "LIMBU LETTER YA" },
    { 0x1916, "LIMBU LETTER RA" },
    { 0x1917, "LIMBU LETTER LA" },
    { 0x1918, "LIMBU LETTER WA" },
    { 0x1919, "LIMBU LETTER SHA" },
    { 0x191A, "LIMBU LETTER SSA" },
    { 0x191B, "LIMBU LETTER SA" },
    { 0x191C, "LIMBU LETTER HA" },
    { 0x191D, "LIMBU LETTER GYAN" },
    { 0x191E, "LIMBU LETTER TRA" },
    { 0x1920, "LIMBU VOWEL SIGN A" },
    { 0x1921, "LIMBU VOWEL SIGN I" },
    { 0x1922, "LIMBU VOWEL SIGN U" },
    { 0x1923, "LIMBU VOWEL SIGN EE" },
    { 0x1924, "LIMBU VOWEL SIGN AI" },
    { 0x1925, "LIMBU VOWEL SIGN OO" },
    { 0x1926, "LIMBU VOWEL SIGN AU" },
    { 0x1927, "LIMBU VOWEL SIGN E" },
    { 0x1928, "LIMBU VOWEL SIGN O" },
    { 0x1929, "LIMBU SUBJOINED LETTER YA" },
    { 0x192A, "LIMBU SUBJOINED LETTER RA" },
    { 0x192B, "LIMBU SUBJOINED LETTER WA" },
    { 0x1930, "LIMBU SMALL LETTER KA" },
    { 0x1931, "LIMBU SMALL LETTER NGA" },
    { 0x1932, "LIMBU SMALL LETTER ANUSVARA" },
    { 0x1933, "LIMBU SMALL LETTER TA" },
    { 0x1934, "LIMBU SMALL LETTER NA" },
    { 0x1935, "LIMBU SMALL LETTER PA" },
    { 0x1936, "LIMBU SMALL LETTER MA" },
    { 0x1937, "LIMBU SMALL LETTER RA" },
    { 0x1938, "LIMBU SMALL LETTER LA" },
    { 0x1939, "LIMBU SIGN MUKPHRENG" },
    { 0x193A, "LIMBU SIGN KEMPHRENG" },
    { 0x193B, "LIMBU SIGN SA-I" },
    { 0x1940, "LIMBU SIGN LOO" },
    { 0x1944, "LIMBU EXCLAMATION MARK" },
    { 0x1945, "LIMBU QUESTION MARK" },
    { 0x1946, "LIMBU DIGIT ZERO" },
    { 0x1947, "LIMBU DIGIT ONE" },
    { 0x1948, "LIMBU DIGIT TWO" },
    { 0x1949, "LIMBU DIGIT THREE" },
    { 0x194A, "LIMBU DIGIT FOUR" },
    { 0x194B, "LIMBU DIGIT FIVE" },
    { 0x194C, "LIMBU DIGIT SIX" },
    { 0x194D, "LIMBU DIGIT SEVEN" },
    { 0x194E, "LIMBU DIGIT EIGHT" },
    { 0x194F, "LIMBU DIGIT NINE" },
    { 0x1950, "TAI LE LETTER KA" },
    { 0x1951, "TAI LE LETTER XA" },
    { 0x1952, "TAI LE LETTER NGA" },
    { 0x1953, "TAI LE LETTER TSA" },
    { 0x1954, "TAI LE LETTER SA" },
    { 0x1955, "TAI LE LETTER YA" },
    { 0x1956, "TAI LE LETTER TA" },
    { 0x1957, "TAI LE LETTER THA" },
    { 0x1958, "TAI LE LETTER LA" },
    { 0x1959, "TAI LE LETTER PA" },
    { 0x195A, "TAI LE LETTER PHA" },
    { 0x195B, "TAI LE LETTER MA" },
    { 0x195C, "TAI LE LETTER FA" },
    { 0x195D, "TAI LE LETTER VA" },
    { 0x195E, "TAI LE LETTER HA" },
    { 0x195F, "TAI LE LETTER QA" },
    { 0x1960, "TAI LE LETTER KHA" },
    { 0x1961, "TAI LE LETTER TSHA" },
    { 0x1962, "TAI LE LETTER NA" },
    { 0x1963, "TAI LE LETTER A" },
    { 0x1964, "TAI LE LETTER I" },
    { 0x1965, "TAI LE LETTER EE" },
    { 0x1966, "TAI LE LETTER EH" },
    { 0x1967, "TAI LE LETTER U" },
    { 0x1968, "TAI LE LETTER OO" },
    { 0x1969, "TAI LE LETTER O" },
    { 0x196A, "TAI LE LETTER UE" },
    { 0x196B, "TAI LE LETTER E" },
    { 0x196C, "TAI LE LETTER AUE" },
    { 0x196D, "TAI LE LETTER AI" },
    { 0x1970, "TAI LE LETTER TONE-2" },
    { 0x1971, "TAI LE LETTER TONE-3" },
    { 0x1972, "TAI LE LETTER TONE-4" },
    { 0x1973, "TAI LE LETTER TONE-5" },
    { 0x1974, "TAI LE LETTER TONE-6" },
    { 0x1980, "NEW TAI LUE LETTER HIGH QA" },
    { 0x1981, "NEW TAI LUE LETTER LOW QA" },
    { 0x1982, "NEW TAI LUE LETTER HIGH KA" },
    { 0x1983, "NEW TAI LUE LETTER HIGH XA" },
    { 0x1984, "NEW TAI LUE LETTER HIGH NGA" },
    { 0x1985, "NEW TAI LUE LETTER LOW KA" },
    { 0x1986, "NEW TAI LUE LETTER LOW XA" },
    { 0x1987, "NEW TAI LUE LETTER LOW NGA" },
    { 0x1988, "NEW TAI LUE LETTER HIGH TSA" },
    { 0x1989, "NEW TAI LUE LETTER HIGH SA" },
    { 0x198A, "NEW TAI LUE LETTER HIGH YA" },
    { 0x198B, "NEW TAI LUE LETTER LOW TSA" },
    { 0x198C, "NEW TAI LUE LETTER LOW SA" },
    { 0x198D, "NEW TAI LUE LETTER LOW YA" },
    { 0x198E, "NEW TAI LUE LETTER HIGH TA" },
    { 0x198F, "NEW TAI LUE LETTER HIGH THA" },
    { 0x1990, "NEW TAI LUE LETTER HIGH NA" },
    { 0x1991, "NEW TAI LUE LETTER LOW TA" },
    { 0x1992, "NEW TAI LUE LETTER LOW THA" },
    { 0x1993, "NEW TAI LUE LETTER LOW NA" },
    { 0x1994, "NEW TAI LUE LETTER HIGH PA" },
    { 0x1995, "NEW TAI LUE LETTER HIGH PHA" },
    { 0x1996, "NEW TAI LUE LETTER HIGH MA" },
    { 0x1997, "NEW TAI LUE LETTER LOW PA" },
    { 0x1998, "NEW TAI LUE LETTER LOW PHA" },
    { 0x1999, "NEW TAI LUE LETTER LOW MA" },
    { 0x199A, "NEW TAI LUE LETTER HIGH FA" },
    { 0x199B, "NEW TAI LUE LETTER HIGH VA" },
    { 0x199C, "NEW TAI LUE LETTER HIGH LA" },
    { 0x199D, "NEW TAI LUE LETTER LOW FA" },
    { 0x199E, "NEW TAI LUE LETTER LOW VA" },
    { 0x199F, "NEW TAI LUE LETTER LOW LA" },
    { 0x19A0, "NEW TAI LUE LETTER HIGH HA" },
    { 0x19A1, "NEW TAI LUE LETTER HIGH DA" },
    { 0x19A2, "NEW TAI LUE LETTER HIGH BA" },
    { 0x19A3, "NEW TAI LUE LETTER LOW HA" },
    { 0x19A4, "NEW TAI LUE LETTER LOW DA" },
    { 0x19A5, "NEW TAI LUE LETTER LOW BA" },
    { 0x19A6, "NEW TAI LUE LETTER HIGH KVA" },
    { 0x19A7, "NEW TAI LUE LETTER HIGH XVA" },
    { 0x19A8, "NEW TAI LUE LETTER LOW KVA" },
    { 0x19A9, "NEW TAI LUE LETTER LOW XVA" },
    { 0x19AA, "NEW TAI LUE LETTER HIGH SUA" },
    { 0x19AB, "NEW TAI LUE LETTER LOW SUA" },
    { 0x19B0, "NEW TAI LUE VOWEL SIGN VOWEL SHORTENER" },
    { 0x19B1, "NEW TAI LUE VOWEL SIGN AA" },
    { 0x19B2, "NEW TAI LUE VOWEL SIGN II" },
    { 0x19B3, "NEW TAI LUE VOWEL SIGN U" },
    { 0x19B4, "NEW TAI LUE VOWEL SIGN UU" },
    { 0x19B5, "NEW TAI LUE VOWEL SIGN E" },
    { 0x19B6, "NEW TAI LUE VOWEL SIGN AE" },
    { 0x19B7, "NEW TAI LUE VOWEL SIGN O" },
    { 0x19B8, "NEW TAI LUE VOWEL SIGN OA" },
    { 0x19B9, "NEW TAI LUE VOWEL SIGN UE" },
    { 0x19BA, "NEW TAI LUE VOWEL SIGN AY" },
    { 0x19BB, "NEW TAI LUE VOWEL SIGN AAY" },
    { 0x19BC, "NEW TAI LUE VOWEL SIGN UY" },
    { 0x19BD, "NEW TAI LUE VOWEL SIGN OY" },
    { 0x19BE, "NEW TAI LUE VOWEL SIGN OAY" },
    { 0x19BF, "NEW TAI LUE VOWEL SIGN UEY" },
    { 0x19C0, "NEW TAI LUE VOWEL SIGN IY" },
    { 0x19C1, "NEW TAI LUE LETTER FINAL V" },
    { 0x19C2, "NEW TAI LUE LETTER FINAL NG" },
    { 0x19C3, "NEW TAI LUE LETTER FINAL N" },
    { 0x19C4, "NEW TAI LUE LETTER FINAL M" },
    { 0x19C5, "NEW TAI LUE LETTER FINAL K" },
    { 0x19C6, "NEW TAI LUE LETTER FINAL D" },
    { 0x19C7, "NEW TAI LUE LETTER FINAL B" },
    { 0x19C8, "NEW TAI LUE TONE MARK-1" },
    { 0x19C9, "NEW TAI LUE TONE MARK-2" },
    { 0x19D0, "NEW TAI LUE DIGIT ZERO" },
    { 0x19D1, "NEW TAI LUE DIGIT ONE" },
    { 0x19D2, "NEW TAI LUE DIGIT TWO" },
    { 0x19D3, "NEW TAI LUE DIGIT THREE" },
    { 0x19D4, "NEW TAI LUE DIGIT FOUR" },
    { 0x19D5, "NEW TAI LUE DIGIT FIVE" },
    { 0x19D6, "NEW TAI LUE DIGIT SIX" },
    { 0x19D7, "NEW TAI LUE DIGIT SEVEN" },
    { 0x19D8, "NEW TAI LUE DIGIT EIGHT" },
    { 0x19D9, "NEW TAI LUE DIGIT NINE" },
    { 0x19DA, "NEW TAI LUE THAM DIGIT ONE" },
    { 0x19DE, "NEW TAI LUE SIGN LAE" },
    { 0x19DF, "NEW TAI LUE SIGN LAEV" },
    { 0x19E0, "KHMER SYMBOL PATHAMASAT" },
    { 0x19E1, "KHMER SYMBOL MUOY KOET" },
    { 0x19E2, "KHMER SYMBOL PII KOET" },
    { 0x19E3, "KHMER SYMBOL BEI KOET" },
    { 0x19E4, "KHMER SYMBOL BUON KOET" },
    { 0x19E5, "KHMER SYMBOL PRAM KOET" },
    { 0x19E6, "KHMER SYMBOL PRAM-MUOY KOET" },
    { 0x19E7, "KHMER SYMBOL PRAM-PII KOET" },
    { 0x19E8, "KHMER SYMBOL PRAM-BEI KOET" },
    { 0x19E9, "KHMER SYMBOL PRAM-BUON KOET" },
    { 0x19EA, "KHMER SYMBOL DAP KOET" },
    { 0x19EB, "KHMER SYMBOL DAP-MUOY KOET" },
    { 0x19EC, "KHMER SYMBOL DAP-PII KOET" },
    { 0x19ED, "KHMER SYMBOL DAP-BEI KOET" },
    { 0x19EE, "KHMER SYMBOL DAP-BUON KOET" },
    { 0x19EF, "KHMER SYMBOL DAP-PRAM KOET" },
    { 0x19F0, "KHMER SYMBOL TUTEYASAT" },
    { 0x19F1, "KHMER SYMBOL MUOY ROC" },
    { 0x19F2, "KHMER SYMBOL PII ROC" },
    { 0x19F3, "KHMER SYMBOL BEI ROC" },
    { 0x19F4, "KHMER SYMBOL BUON ROC" },
    { 0x19F5, "KHMER SYMBOL PRAM ROC" },
    { 0x19F6, "KHMER SYMBOL PRAM-MUOY ROC" },
    { 0x19F7, "KHMER SYMBOL PRAM-PII ROC" },
    { 0x19F8, "KHMER SYMBOL PRAM-BEI ROC" },
    { 0x19F9, "KHMER SYMBOL PRAM-BUON ROC" },
    { 0x19FA, "KHMER SYMBOL DAP ROC" },
    { 0x19FB, "KHMER SYMBOL DAP-MUOY ROC" },
    { 0x19FC, "KHMER SYMBOL DAP-PII ROC" },
    { 0x19FD, "KHMER SYMBOL DAP-BEI ROC" },
    { 0x19FE, "KHMER SYMBOL DAP-BUON ROC" },
    { 0x19FF, "KHMER SYMBOL DAP-PRAM ROC" },
    { 0x1A00, "BUGINESE LETTER KA" },
    { 0x1A01, "BUGINESE LETTER GA" },
    { 0x1A02, "BUGINESE LETTER NGA" },
    { 0x1A03, "BUGINESE LETTER NGKA" },
    { 0x1A04, "BUGINESE LETTER PA" },
    { 0x1A05, "BUGINESE LETTER BA" },
    { 0x1A06, "BUGINESE LETTER MA" },
    { 0x1A07, "BUGINESE LETTER MPA" },
    { 0x1A08, "BUGINESE LETTER TA" },
    { 0x1A09, "BUGINESE LETTER DA" },
    { 0x1A0A, "BUGINESE LETTER NA" },
    { 0x1A0B, "BUGINESE LETTER NRA" },
    { 0x1A0C, "BUGINESE LETTER CA" },
    { 0x1A0D, "BUGINESE LETTER JA" },
    { 0x1A0E, "BUGINESE LETTER NYA" },
    { 0x1A0F, "BUGINESE LETTER NYCA" },
    { 0x1A10, "BUGINESE LETTER YA" },
    { 0x1A11, "BUGINESE LETTER RA" },
    { 0x1A12, "BUGINESE LETTER LA" },
    { 0x1A13, "BUGINESE LETTER VA" },
    { 0x1A14, "BUGINESE LETTER SA" },
    { 0x1A15, "BUGINESE LETTER A" },
    { 0x1A16, "BUGINESE LETTER HA" },
    { 0x1A17, "BUGINESE VOWEL SIGN I" },
    { 0x1A18, "BUGINESE VOWEL SIGN U" },
    { 0x1A19, "BUGINESE VOWEL SIGN E" },
    { 0x1A1A, "BUGINESE VOWEL SIGN O" },
    { 0x1A1B, "BUGINESE VOWEL SIGN AE" },
    { 0x1A1E, "BUGINESE PALLAWA" },
    { 0x1A1F, "BUGINESE END OF SECTION" },
    { 0x1A20, "TAI THAM LETTER HIGH KA" },
    { 0x1A21, "TAI THAM LETTER HIGH KHA" },
    { 0x1A22, "TAI THAM LETTER HIGH KXA" },
    { 0x1A23, "TAI THAM LETTER LOW KA" },
    { 0x1A24, "TAI THAM LETTER LOW KXA" },
    { 0x1A25, "TAI THAM LETTER LOW KHA" },
    { 0x1A26, "TAI THAM LETTER NGA" },
    { 0x1A27, "TAI THAM LETTER HIGH CA" },
    { 0x1A28, "TAI THAM LETTER HIGH CHA" },
    { 0x1A29, "TAI THAM LETTER LOW CA" },
    { 0x1A2A, "TAI THAM LETTER LOW SA" },
    { 0x1A2B, "TAI THAM LETTER LOW CHA" },
    { 0x1A2C, "TAI THAM LETTER NYA" },
    { 0x1A2D, "TAI THAM LETTER RATA" },
    { 0x1A2E, "TAI THAM LETTER HIGH RATHA" },
    { 0x1A2F, "TAI THAM LETTER DA" },
    { 0x1A30, "TAI THAM LETTER LOW RATHA" },
    { 0x1A31, "TAI THAM LETTER RANA" },
    { 0x1A32, "TAI THAM LETTER HIGH TA" },
    { 0x1A33, "TAI THAM LETTER HIGH THA" },
    { 0x1A34, "TAI THAM LETTER LOW TA" },
    { 0x1A35, "TAI THAM LETTER LOW THA" },
    { 0x1A36, "TAI THAM LETTER NA" },
    { 0x1A37, "TAI THAM LETTER BA" },
    { 0x1A38, "TAI THAM LETTER HIGH PA" },
    { 0x1A39, "TAI THAM LETTER HIGH PHA" },
    { 0x1A3A, "TAI THAM LETTER HIGH FA" },
    { 0x1A3B, "TAI THAM LETTER LOW PA" },
    { 0x1A3C, "TAI THAM LETTER LOW FA" },
    { 0x1A3D, "TAI THAM LETTER LOW PHA" },
    { 0x1A3E, "TAI THAM LETTER MA" },
    { 0x1A3F, "TAI THAM LETTER LOW YA" },
    { 0x1A40, "TAI THAM LETTER HIGH YA" },
    { 0x1A41, "TAI THAM LETTER RA" },
    { 0x1A42, "TAI THAM LETTER RUE" },
    { 0x1A43, "TAI THAM LETTER LA" },
    { 0x1A44, "TAI THAM LETTER LUE" },
    { 0x1A45, "TAI THAM LETTER WA" },
    { 0x1A46, "TAI THAM LETTER HIGH SHA" },
    { 0x1A47, "TAI THAM LETTER HIGH SSA" },
    { 0x1A48, "TAI THAM LETTER HIGH SA" },
    { 0x1A49, "TAI THAM LETTER HIGH HA" },
    { 0x1A4A, "TAI THAM LETTER LLA" },
    { 0x1A4B, "TAI THAM LETTER A" },
    { 0x1A4C, "TAI THAM LETTER LOW HA" },
    { 0x1A4D, "TAI THAM LETTER I" },
    { 0x1A4E, "TAI THAM LETTER II" },
    { 0x1A4F, "TAI THAM LETTER U" },
    { 0x1A50, "TAI THAM LETTER UU" },
    { 0x1A51, "TAI THAM LETTER EE" },
    { 0x1A52, "TAI THAM LETTER OO" },
    { 0x1A53, "TAI THAM LETTER LAE" },
    { 0x1A54, "TAI THAM LETTER GREAT SA" },
    { 0x1A55, "TAI THAM CONSONANT SIGN MEDIAL RA" },
    { 0x1A56, "TAI THAM CONSONANT SIGN MEDIAL LA" },
    { 0x1A57, "TAI THAM CONSONANT SIGN LA TANG LAI" },
    { 0x1A58, "TAI THAM SIGN MAI KANG LAI" },
    { 0x1A59, "TAI THAM CONSONANT SIGN FINAL NGA" },
    { 0x1A5A, "TAI THAM CONSONANT SIGN LOW PA" },
    { 0x1A5B, "TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA" },
    { 0x1A5C, "TAI THAM CONSONANT SIGN MA" },
    { 0x1A5D, "TAI THAM CONSONANT SIGN BA" },
    { 0x1A5E, "TAI THAM CONSONANT SIGN SA" },
    { 0x1A60, "TAI THAM SIGN SAKOT" },
    { 0x1A61, "TAI THAM VOWEL SIGN A" },
    { 0x1A62, "TAI THAM VOWEL SIGN MAI SAT" },
    { 0x1A63, "TAI THAM VOWEL SIGN AA" },
    { 0x1A64, "TAI THAM VOWEL SIGN TALL AA" },
    { 0x1A65, "TAI THAM VOWEL SIGN I" },
    { 0x1A66, "TAI THAM VOWEL SIGN II" },
    { 0x1A67, "TAI THAM VOWEL SIGN UE" },
    { 0x1A68, "TAI THAM VOWEL SIGN UUE" },
    { 0x1A69, "TAI THAM VOWEL SIGN U" },
    { 0x1A6A, "TAI THAM VOWEL SIGN UU" },
    { 0x1A6B, "TAI THAM VOWEL SIGN O" },
    { 0x1A6C, "TAI THAM VOWEL SIGN OA BELOW" },
    { 0x1A6D, "TAI THAM VOWEL SIGN OY" },
    { 0x1A6E, "TAI THAM VOWEL SIGN E" },
    { 0x1A6F, "TAI THAM VOWEL SIGN AE" },
    { 0x1A70, "TAI THAM VOWEL SIGN OO" },
    { 0x1A71, "TAI THAM VOWEL SIGN AI" },
    { 0x1A72, "TAI THAM VOWEL SIGN THAM AI" },
    { 0x1A73, "TAI THAM VOWEL SIGN OA ABOVE" },
    { 0x1A74, "TAI THAM SIGN MAI KANG" },
    { 0x1A75, "TAI THAM SIGN TONE-1" },
    { 0x1A76, "TAI THAM SIGN TONE-2" },
    { 0x1A77, "TAI THAM SIGN KHUEN TONE-3" },
    { 0x1A78, "TAI THAM SIGN KHUEN TONE-4" },
    { 0x1A79, "TAI THAM SIGN KHUEN TONE-5" },
    { 0x1A7A, "TAI THAM SIGN RA HAAM" },
    { 0x1A7B, "TAI THAM SIGN MAI SAM" },
    { 0x1A7C, "TAI THAM SIGN KHUEN-LUE KARAN" },
    { 0x1A7F, "TAI THAM COMBINING CRYPTOGRAMMIC DOT" },
    { 0x1A80, "TAI THAM HORA DIGIT ZERO" },
    { 0x1A81, "TAI THAM HORA DIGIT ONE" },
    { 0x1A82, "TAI THAM HORA DIGIT TWO" },
    { 0x1A83, "TAI THAM HORA DIGIT THREE" },
    { 0x1A84, "TAI THAM HORA DIGIT FOUR" },
    { 0x1A85, "TAI THAM HORA DIGIT FIVE" },
    { 0x1A86, "TAI THAM HORA DIGIT SIX" },
    { 0x1A87, "TAI THAM HORA DIGIT SEVEN" },
    { 0x1A88, "TAI THAM HORA DIGIT EIGHT" },
    { 0x1A89, "TAI THAM HORA DIGIT NINE" },
    { 0x1A90, "TAI THAM THAM DIGIT ZERO" },
    { 0x1A91, "TAI THAM THAM DIGIT ONE" },
    { 0x1A92, "TAI THAM THAM DIGIT TWO" },
    { 0x1A93, "TAI THAM THAM DIGIT THREE" },
    { 0x1A94, "TAI THAM THAM DIGIT FOUR" },
    { 0x1A95, "TAI THAM THAM DIGIT FIVE" },
    { 0x1A96, "TAI THAM THAM DIGIT SIX" },
    { 0x1A97, "TAI THAM THAM DIGIT SEVEN" },
    { 0x1A98, "TAI THAM THAM DIGIT EIGHT" },
    { 0x1A99, "TAI THAM THAM DIGIT NINE" },
    { 0x1AA0, "TAI THAM SIGN WIANG" },
    { 0x1AA1, "TAI THAM SIGN WIANGWAAK" },
    { 0x1AA2, "TAI THAM SIGN SAWAN" },
    { 0x1AA3, "TAI THAM SIGN KEOW" },
    { 0x1AA4, "TAI THAM SIGN HOY" },
    { 0x1AA5, "TAI THAM SIGN DOKMAI" },
    { 0x1AA6, "TAI THAM SIGN REVERSED ROTATED RANA" },
    { 0x1AA7, "TAI THAM SIGN MAI YAMOK" },
    { 0x1AA8, "TAI THAM SIGN KAAN" },
    { 0x1AA9, "TAI THAM SIGN KAANKUU" },
    { 0x1AAA, "TAI THAM SIGN SATKAAN" },
    { 0x1AAB, "TAI THAM SIGN SATKAANKUU" },
    { 0x1AAC, "TAI THAM SIGN HANG" },
    { 0x1AAD, "TAI THAM SIGN CAANG" },
    { 0x1AB0, "COMBINING DOUBLED CIRCUMFLEX ACCENT" },
    { 0x1AB1, "COMBINING DIAERESIS-RING" },
    { 0x1AB2, "COMBINING INFINITY" },
    { 0x1AB3, "COMBINING DOWNWARDS ARROW" },
    { 0x1AB4, "COMBINING TRIPLE DOT" },
    { 0x1AB5, "COMBINING X-X BELOW" },
    { 0x1AB6, "COMBINING WIGGLY LINE BELOW" },
    { 0x1AB7, "COMBINING OPEN MARK BELOW" },
    { 0x1AB8, "COMBINING DOUBLE OPEN MARK BELOW" },
    { 0x1AB9, "COMBINING LIGHT CENTRALIZATION STROKE BELOW" },
    { 0x1ABA, "COMBINING STRONG CENTRALIZATION STROKE BELOW" },
    { 0x1ABB, "COMBINING PARENTHESES ABOVE" },
    { 0x1ABC, "COMBINING DOUBLE PARENTHESES ABOVE" },
    { 0x1ABD, "COMBINING PARENTHESES BELOW" },
    { 0x1ABE, "COMBINING PARENTHESES OVERLAY" },
    { 0x1ABF, "COMBINING LATIN SMALL LETTER W BELOW" },
    { 0x1AC0, "COMBINING LATIN SMALL LETTER TURNED W BELOW" },
    { 0x1AC1, "COMBINING LEFT PARENTHESIS ABOVE LEFT" },
    { 0x1AC2, "COMBINING RIGHT PARENTHESIS ABOVE RIGHT" },
    { 0x1AC3, "COMBINING LEFT PARENTHESIS BELOW LEFT" },
    { 0x1AC4, "COMBINING RIGHT PARENTHESIS BELOW RIGHT" },
    { 0x1AC5, "COMBINING SQUARE BRACKETS ABOVE" },
    { 0x1AC6, "COMBINING NUMBER SIGN ABOVE" },
    { 0x1AC7, "COMBINING INVERTED DOUBLE ARCH ABOVE" },
    { 0x1AC8, "COMBINING PLUS SIGN ABOVE" },
    { 0x1AC9, "COMBINING DOUBLE PLUS SIGN ABOVE" },
    { 0x1ACA, "COMBINING DOUBLE PLUS SIGN BELOW" },
    { 0x1ACB, "COMBINING TRIPLE ACUTE ACCENT" },
    { 0x1ACC, "COMBINING LATIN SMALL LETTER INSULAR G" },
    { 0x1ACD, "COMBINING LATIN SMALL LETTER INSULAR R" },
    { 0x1ACE, "COMBINING LATIN SMALL LETTER INSULAR T" },
    { 0x1B00, "BALINESE SIGN ULU RICEM" },
    { 0x1B01, "BALINESE SIGN ULU CANDRA" },
    { 0x1B02, "BALINESE SIGN CECEK" },
    { 0x1B03, "BALINESE SIGN SURANG" },
    { 0x1B04, "BALINESE SIGN BISAH" },
    { 0x1B05, "BALINESE LETTER AKARA" },
    { 0x1B06, "BALINESE LETTER AKARA TEDUNG" },
    { 0x1B07, "BALINESE LETTER IKARA" },
    { 0x1B08, "BALINESE LETTER IKARA TEDUNG" },
    { 0x1B09, "BALINESE LETTER UKARA" },
    { 0x1B0A, "BALINESE LETTER UKARA TEDUNG" },
    { 0x1B0B, "BALINESE LETTER RA REPA" },
    { 0x1B0C, "BALINESE LETTER RA REPA TEDUNG" },
    { 0x1B0D, "BALINESE LETTER LA LENGA" },
    { 0x1B0E, "BALINESE LETTER LA LENGA TEDUNG" },
    { 0x1B0F, "BALINESE LETTER EKARA" },
    { 0x1B10, "BALINESE LETTER AIKARA" },
    { 0x1B11, "BALINESE LETTER OKARA" },
    { 0x1B12, "BALINESE LETTER OKARA TEDUNG" },
    { 0x1B13, "BALINESE LETTER KA" },
    { 0x1B14, "BALINESE LETTER KA MAHAPRANA" },
    { 0x1B15, "BALINESE LETTER GA" },
    { 0x1B16, "BALINESE LETTER GA GORA" },
    { 0x1B17, "BALINESE LETTER NGA" },
    { 0x1B18, "BALINESE LETTER CA" },
    { 0x1B19, "BALINESE LETTER CA LACA" },
    { 0x1B1A, "BALINESE LETTER JA" },
    { 0x1B1B, "BALINESE LETTER JA JERA" },
    { 0x1B1C, "BALINESE LETTER NYA" },
    { 0x1B1D, "BALINESE LETTER TA LATIK" },
    { 0x1B1E, "BALINESE LETTER TA MURDA MAHAPRANA" },
    { 0x1B1F, "BALINESE LETTER DA MURDA ALPAPRANA" },
    { 0x1B20, "BALINESE LETTER DA MURDA MAHAPRANA" },
    { 0x1B21, "BALINESE LETTER NA RAMBAT" },
    { 0x1B22, "BALINESE LETTER TA" },
    { 0x1B23, "BALINESE LETTER TA TAWA" },
    { 0x1B24, "BALINESE LETTER DA" },
    { 0x1B25, "BALINESE LETTER DA MADU" },
    { 0x1B26, "BALINESE LETTER NA" },
    { 0x1B27, "BALINESE LETTER PA" },
    { 0x1B28, "BALINESE LETTER PA KAPAL" },
    { 0x1B29, "BALINESE LETTER BA" },
    { 0x1B2A, "BALINESE LETTER BA KEMBANG" },
    { 0x1B2B, "BALINESE LETTER MA" },
    { 0x1B2C, "BALINESE LETTER YA" },
    { 0x1B2D, "BALINESE LETTER RA" },
    { 0x1B2E, "BALINESE LETTER LA" },
    { 0x1B2F, "BALINESE LETTER WA" },
    { 0x1B30, "BALINESE LETTER SA SAGA" },
    { 0x1B31, "BALINESE LETTER SA SAPA" },
    { 0x1B32, "BALINESE LETTER SA" },
    { 0x1B33, "BALINESE LETTER HA" },
    { 0x1B34, "BALINESE SIGN REREKAN" },
    { 0x1B35, "BALINESE VOWEL SIGN TEDUNG" },
    { 0x1B36, "BALINESE VOWEL SIGN ULU" },
    { 0x1B37, "BALINESE VOWEL SIGN ULU SARI" },
    { 0x1B38, "BALINESE VOWEL SIGN SUKU" },
    { 0x1B39, "BALINESE VOWEL SIGN SUKU ILUT" },
    { 0x1B3A, "BALINESE VOWEL SIGN RA REPA" },
    { 0x1B3B, "BALINESE VOWEL SIGN RA REPA TEDUNG" },
    { 0x1B3C, "BALINESE VOWEL SIGN LA LENGA" },
    { 0x1B3D, "BALINESE VOWEL SIGN LA LENGA TEDUNG" },
    { 0x1B3E, "BALINESE VOWEL SIGN TALING" },
    { 0x1B3F, "BALINESE VOWEL SIGN TALING REPA" },
    { 0x1B40, "BALINESE VOWEL SIGN TALING TEDUNG" },
    { 0x1B41, "BALINESE VOWEL SIGN TALING REPA TEDUNG" },
    { 0x1B42, "BALINESE VOWEL SIGN PEPET" },
    { 0x1B43, "BALINESE VOWEL SIGN PEPET TEDUNG" },
    { 0x1B44, "BALINESE ADEG ADEG" },
    { 0x1B45, "BALINESE LETTER KAF SASAK" },
    { 0x1B46, "BALINESE LETTER KHOT SASAK" },
    { 0x1B47, "BALINESE LETTER TZIR SASAK" },
    { 0x1B48, "BALINESE LETTER EF SASAK" },
    { 0x1B49, "BALINESE LETTER VE SASAK" },
    { 0x1B4A, "BALINESE LETTER ZAL SASAK" },
    { 0x1B4B, "BALINESE LETTER ASYURA SASAK" },
    { 0x1B4C, "BALINESE LETTER ARCHAIC JNYA" },
    { 0x1B50, "BALINESE DIGIT ZERO" },
    { 0x1B51, "BALINESE DIGIT ONE" },
    { 0x1B52, "BALINESE DIGIT TWO" },
    { 0x1B53, "BALINESE DIGIT THREE" },
    { 0x1B54, "BALINESE DIGIT FOUR" },
    { 0x1B55, "BALINESE DIGIT FIVE" },
    { 0x1B56, "BALINESE DIGIT SIX" },
    { 0x1B57, "BALINESE DIGIT SEVEN" },
    { 0x1B58, "BALINESE DIGIT EIGHT" },
    { 0x1B59, "BALINESE DIGIT NINE" },
    { 0x1B5A, "BALINESE PANTI" },
    { 0x1B5B, "BALINESE PAMADA" },
    { 0x1B5C, "BALINESE WINDU" },
    { 0x1B5D, "BALINESE CARIK PAMUNGKAH" },
    { 0x1B5E, "BALINESE CARIK SIKI" },
    { 0x1B5F, "BALINESE CARIK PAREREN" },
    { 0x1B60, "BALINESE PAMENENG" },
    { 0x1B61, "BALINESE MUSICAL SYMBOL DONG" },
    { 0x1B62, "BALINESE MUSICAL SYMBOL DENG" },
    { 0x1B63, "BALINESE MUSICAL SYMBOL DUNG" },
    { 0x1B64, "BALINESE MUSICAL SYMBOL DANG" },
    { 0x1B65, "BALINESE MUSICAL SYMBOL DANG SURANG" },
    { 0x1B66, "BALINESE MUSICAL SYMBOL DING" },
    { 0x1B67, "BALINESE MUSICAL SYMBOL DAENG" },
    { 0x1B68, "BALINESE MUSICAL SYMBOL DEUNG" },
    { 0x1B69, "BALINESE MUSICAL SYMBOL DAING" },
    { 0x1B6A, "BALINESE MUSICAL SYMBOL DANG GEDE" },
    { 0x1B6B, "BALINESE MUSICAL SYMBOL COMBINING TEGEH" },
    { 0x1B6C, "BALINESE MUSICAL SYMBOL COMBINING ENDEP" },
    { 0x1B6D, "BALINESE MUSICAL SYMBOL COMBINING KEMPUL" },
    { 0x1B6E, "BALINESE MUSICAL SYMBOL COMBINING KEMPLI" },
    { 0x1B6F, "BALINESE MUSICAL SYMBOL COMBINING JEGOGAN" },
    { 0x1B70, "BALINESE MUSICAL SYMBOL COMBINING KEMPUL WITH JEGOGAN" },
    { 0x1B71, "BALINESE MUSICAL SYMBOL COMBINING KEMPLI WITH JEGOGAN" },
    { 0x1B72, "BALINESE MUSICAL SYMBOL COMBINING BENDE" },
    { 0x1B73, "BALINESE MUSICAL SYMBOL COMBINING GONG" },
    { 0x1B74, "BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DUG" },
    { 0x1B75, "BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DAG" },
    { 0x1B76, "BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TUK" },
    { 0x1B77, "BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TAK" },
    { 0x1B78, "BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PANG" },
    { 0x1B79, "BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PUNG" },
    { 0x1B7A, "BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLAK" },
    { 0x1B7B, "BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLUK" },
    { 0x1B7C, "BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PING" },
    { 0x1B7D, "BALINESE PANTI LANTANG" },
    { 0x1B7E, "BALINESE PAMADA LANTANG" },
    { 0x1B80, "SUNDANESE SIGN PANYECEK" },
    { 0x1B81, "SUNDANESE SIGN PANGLAYAR" },
    { 0x1B82, "SUNDANESE SIGN PANGWISAD" },
    { 0x1B83, "SUNDANESE LETTER A" },
    { 0x1B84, "SUNDANESE LETTER I" },
    { 0x1B85, "SUNDANESE LETTER U" },
    { 0x1B86, "SUNDANESE LETTER AE" },
    { 0x1B87, "SUNDANESE LETTER O" },
    { 0x1B88, "SUNDANESE LETTER E" },
    { 0x1B89, "SUNDANESE LETTER EU" },
    { 0x1B8A, "SUNDANESE LETTER KA" },
    { 0x1B8B, "SUNDANESE LETTER QA" },
    { 0x1B8C, "SUNDANESE LETTER GA" },
    { 0x1B8D, "SUNDANESE LETTER NGA" },
    { 0x1B8E, "SUNDANESE LETTER CA" },
    { 0x1B8F, "SUNDANESE LETTER JA" },
    { 0x1B90, "SUNDANESE LETTER ZA" },
    { 0x1B91, "SUNDANESE LETTER NYA" },
    { 0x1B92, "SUNDANESE LETTER TA" },
    { 0x1B93, "SUNDANESE LETTER DA" },
    { 0x1B94, "SUNDANESE LETTER NA" },
    { 0x1B95, "SUNDANESE LETTER PA" },
    { 0x1B96, "SUNDANESE LETTER FA" },
    { 0x1B97, "SUNDANESE LETTER VA" },
    { 0x1B98, "SUNDANESE LETTER BA" },
    { 0x1B99, "SUNDANESE LETTER MA" },
    { 0x1B9A, "SUNDANESE LETTER YA" },
    { 0x1B9B, "SUNDANESE LETTER RA" },
    { 0x1B9C, "SUNDANESE LETTER LA" },
    { 0x1B9D, "SUNDANESE LETTER WA" },
    { 0x1B9E, "SUNDANESE LETTER SA" },
    { 0x1B9F, "SUNDANESE LETTER XA" },
    { 0x1BA0, "SUNDANESE LETTER HA" },
    { 0x1BA1, "SUNDANESE CONSONANT SIGN PAMINGKAL" },
    { 0x1BA2, "SUNDANESE CONSONANT SIGN PANYAKRA" },
    { 0x1BA3, "SUNDANESE CONSONANT SIGN PANYIKU" },
    { 0x1BA4, "SUNDANESE VOWEL SIGN PANGHULU" },
    { 0x1BA5, "SUNDANESE VOWEL SIGN PANYUKU" },
    { 0x1BA6, "SUNDANESE VOWEL SIGN PANAELAENG" },
    { 0x1BA7, "SUNDANESE VOWEL SIGN PANOLONG" },
    { 0x1BA8, "SUNDANESE VOWEL SIGN PAMEPET" },
    { 0x1BA9, "SUNDANESE VOWEL SIGN PANEULEUNG" },
    { 0x1BAA, "SUNDANESE SIGN PAMAAEH" },
    { 0x1BAB, "SUNDANESE SIGN VIRAMA" },
    { 0x1BAC, "SUNDANESE CONSONANT SIGN PASANGAN MA" },
    { 0x1BAD, "SUNDANESE CONSONANT SIGN PASANGAN WA" },
    { 0x1BAE, "SUNDANESE LETTER KHA" },
    { 0x1BAF, "SUNDANESE LETTER SYA" },
    { 0x1BB0, "SUNDANESE DIGIT ZERO" },
    { 0x1BB1, "SUNDANESE DIGIT ONE" },
    { 0x1BB2, "SUNDANESE DIGIT TWO" },
    { 0x1BB3, "SUNDANESE DIGIT THREE" },
    { 0x1BB4, "SUNDANESE DIGIT FOUR" },
    { 0x1BB5, "SUNDANESE DIGIT FIVE" },
    { 0x1BB6, "SUNDANESE DIGIT SIX" },
    { 0x1BB7, "SUNDANESE DIGIT SEVEN" },
    { 0x1BB8, "SUNDANESE DIGIT EIGHT" },
    { 0x1BB9, "SUNDANESE DIGIT NINE" },
    { 0x1BBA, "SUNDANESE AVAGRAHA" },
    { 0x1BBB, "SUNDANESE LETTER REU" },
    { 0x1BBC, "SUNDANESE LETTER LEU" },
    { 0x1BBD, "SUNDANESE LETTER BHA" },
    { 0x1BBE, "SUNDANESE LETTER FINAL K" },
    { 0x1BBF, "SUNDANESE LETTER FINAL M" },
    { 0x1BC0, "BATAK LETTER A" },
    { 0x1BC1, "BATAK LETTER SIMALUNGUN A" },
    { 0x1BC2, "BATAK LETTER HA" },
    { 0x1BC3, "BATAK LETTER SIMALUNGUN HA" },
    { 0x1BC4, "BATAK LETTER MANDAILING HA" },
    { 0x1BC5, "BATAK LETTER BA" },
    { 0x1BC6, "BATAK LETTER KARO BA" },
    { 0x1BC7, "BATAK LETTER PA" },
    { 0x1BC8, "BATAK LETTER SIMALUNGUN PA" },
    { 0x1BC9, "BATAK LETTER NA" },
    { 0x1BCA, "BATAK LETTER MANDAILING NA" },
    { 0x1BCB, "BATAK LETTER WA" },
    { 0x1BCC, "BATAK LETTER SIMALUNGUN WA" },
    { 0x1BCD, "BATAK LETTER PAKPAK WA" },
    { 0x1BCE, "BATAK LETTER GA" },
    { 0x1BCF, "BATAK LETTER SIMALUNGUN GA" },
    { 0x1BD0, "BATAK LETTER JA" },
    { 0x1BD1, "BATAK LETTER DA" },
    { 0x1BD2, "BATAK LETTER RA" },
    { 0x1BD3, "BATAK LETTER SIMALUNGUN RA" },
    { 0x1BD4, "BATAK LETTER MA" },
    { 0x1BD5, "BATAK LETTER SIMALUNGUN MA" },
    { 0x1BD6, "BATAK LETTER SOUTHERN TA" },
    { 0x1BD7, "BATAK LETTER NORTHERN TA" },
    { 0x1BD8, "BATAK LETTER SA" },
    { 0x1BD9, "BATAK LETTER SIMALUNGUN SA" },
    { 0x1BDA, "BATAK LETTER MANDAILING SA" },
    { 0x1BDB, "BATAK LETTER YA" },
    { 0x1BDC, "BATAK LETTER SIMALUNGUN YA" },
    { 0x1BDD, "BATAK LETTER NGA" },
    { 0x1BDE, "BATAK LETTER LA" },
    { 0x1BDF, "BATAK LETTER SIMALUNGUN LA" },
    { 0x1BE0, "BATAK LETTER NYA" },
    { 0x1BE1, "BATAK LETTER CA" },
    { 0x1BE2, "BATAK LETTER NDA" },
    { 0x1BE3, "BATAK LETTER MBA" },
    { 0x1BE4, "BATAK LETTER I" },
    { 0x1BE5, "BATAK LETTER U" },
    { 0x1BE6, "BATAK SIGN TOMPI" },
    { 0x1BE7, "BATAK VOWEL SIGN E" },
    { 0x1BE8, "BATAK VOWEL SIGN PAKPAK E" },
    { 0x1BE9, "BATAK VOWEL SIGN EE" },
    { 0x1BEA, "BATAK VOWEL SIGN I" },
    { 0x1BEB, "BATAK VOWEL SIGN KARO I" },
    { 0x1BEC, "BATAK VOWEL SIGN O" },
    { 0x1BED, "BATAK VOWEL SIGN KARO O" },
    { 0x1BEE, "BATAK VOWEL SIGN U" },
    { 0x1BEF, "BATAK VOWEL SIGN U FOR SIMALUNGUN SA" },
    { 0x1BF0, "BATAK CONSONANT SIGN NG" },
    { 0x1BF1, "BATAK CONSONANT SIGN H" },
    { 0x1BF2, "BATAK PANGOLAT" },
    { 0x1BF3, "BATAK PANONGONAN" },
    { 0x1BFC, "BATAK SYMBOL BINDU NA METEK" },
    { 0x1BFD, "BATAK SYMBOL BINDU PINARBORAS" },
    { 0x1BFE, "BATAK SYMBOL BINDU JUDUL" },
    { 0x1BFF, "BATAK SYMBOL BINDU PANGOLAT" },
    { 0x1C00, "LEPCHA LETTER KA" },
    { 0x1C01, "LEPCHA LETTER KLA" },
    { 0x1C02, "LEPCHA LETTER KHA" },
    { 0x1C03, "LEPCHA LETTER GA" },
    { 0x1C04, "LEPCHA LETTER GLA" },
    { 0x1C05, "LEPCHA LETTER NGA" },
    { 0x1C06, "LEPCHA LETTER CA" },
    { 0x1C07, "LEPCHA LETTER CHA" },
    { 0x1C08, "LEPCHA LETTER JA" },
    { 0x1C09, "LEPCHA LETTER NYA" },
    { 0x1C0A, "LEPCHA LETTER TA" },
    { 0x1C0B, "LEPCHA LETTER THA" },
    { 0x1C0C, "LEPCHA LETTER DA" },
    { 0x1C0D, "LEPCHA LETTER NA" },
    { 0x1C0E, "LEPCHA LETTER PA" },
    { 0x1C0F, "LEPCHA LETTER PLA" },
    { 0x1C10, "LEPCHA LETTER PHA" },
    { 0x1C11, "LEPCHA LETTER FA" },
    { 0x1C12, "LEPCHA LETTER FLA" },
    { 0x1C13, "LEPCHA LETTER BA" },
    { 0x1C14, "LEPCHA LETTER BLA" },
    { 0x1C15, "LEPCHA LETTER MA" },
    { 0x1C16, "LEPCHA LETTER MLA" },
    { 0x1C17, "LEPCHA LETTER TSA" },
    { 0x1C18, "LEPCHA LETTER TSHA" },
    { 0x1C19, "LEPCHA LETTER DZA" },
    { 0x1C1A, "LEPCHA LETTER YA" },
    { 0x1C1B, "LEPCHA LETTER RA" },
    { 0x1C1C, "LEPCHA LETTER LA" },
    { 0x1C1D, "LEPCHA LETTER HA" },
    { 0x1C1E, "LEPCHA LETTER HLA" },
    { 0x1C1F, "LEPCHA LETTER VA" },
    { 0x1C20, "LEPCHA LETTER SA" },
    { 0x1C21, "LEPCHA LETTER SHA" },
    { 0x1C22, "LEPCHA LETTER WA" },
    { 0x1C23, "LEPCHA LETTER A" },
    { 0x1C24, "LEPCHA SUBJOINED LETTER YA" },
    { 0x1C25, "LEPCHA SUBJOINED LETTER RA" },
    { 0x1C26, "LEPCHA VOWEL SIGN AA" },
    { 0x1C27, "LEPCHA VOWEL SIGN I" },
    { 0x1C28, "LEPCHA VOWEL SIGN O" },
    { 0x1C29, "LEPCHA VOWEL SIGN OO" },
    { 0x1C2A, "LEPCHA VOWEL SIGN U" },
    { 0x1C2B, "LEPCHA VOWEL SIGN UU" },
    { 0x1C2C, "LEPCHA VOWEL SIGN E" },
    { 0x1C2D, "LEPCHA CONSONANT SIGN K" },
    { 0x1C2E, "LEPCHA CONSONANT SIGN M" },
    { 0x1C2F, "LEPCHA CONSONANT SIGN L" },
    { 0x1C30, "LEPCHA CONSONANT SIGN N" },
    { 0x1C31, "LEPCHA CONSONANT SIGN P" },
    { 0x1C32, "LEPCHA CONSONANT SIGN R" },
    { 0x1C33, "LEPCHA CONSONANT SIGN T" },
    { 0x1C34, "LEPCHA CONSONANT SIGN NYIN-DO" },
    { 0x1C35, "LEPCHA CONSONANT SIGN KANG" },
    { 0x1C36, "LEPCHA SIGN RAN" },
    { 0x1C37, "LEPCHA SIGN NUKTA" },
    { 0x1C3B, "LEPCHA PUNCTUATION TA-ROL" },
    { 0x1C3C, "LEPCHA PUNCTUATION NYET THYOOM TA-ROL" },
    { 0x1C3D, "LEPCHA PUNCTUATION CER-WA" },
    { 0x1C3E, "LEPCHA PUNCTUATION TSHOOK CER-WA" },
    { 0x1C3F, "LEPCHA PUNCTUATION TSHOOK" },
    { 0x1C40, "LEPCHA DIGIT ZERO" },
    { 0x1C41, "LEPCHA DIGIT ONE" },
    { 0x1C42, "LEPCHA DIGIT TWO" },
    { 0x1C43, "LEPCHA DIGIT THREE" },
    { 0x1C44, "LEPCHA DIGIT FOUR" },
    { 0x1C45, "LEPCHA DIGIT FIVE" },
    { 0x1C46, "LEPCHA DIGIT SIX" },
    { 0x1C47, "LEPCHA DIGIT SEVEN" },
    { 0x1C48, "LEPCHA DIGIT EIGHT" },
    { 0x1C49, "LEPCHA DIGIT NINE" },
    { 0x1C4D, "LEPCHA LETTER TTA" },
    { 0x1C4E, "LEPCHA LETTER TTHA" },
    { 0x1C4F, "LEPCHA LETTER DDA" },
    { 0x1C50, "OL CHIKI DIGIT ZERO" },
    { 0x1C51, "OL CHIKI DIGIT ONE" },
    { 0x1C52, "OL CHIKI DIGIT TWO" },
    { 0x1C53, "OL CHIKI DIGIT THREE" },
    { 0x1C54, "OL CHIKI DIGIT FOUR" },
    { 0x1C55, "OL CHIKI DIGIT FIVE" },
    { 0x1C56, "OL CHIKI DIGIT SIX" },
    { 0x1C57, "OL CHIKI DIGIT SEVEN" },
    { 0x1C58, "OL CHIKI DIGIT EIGHT" },
    { 0x1C59, "OL CHIKI DIGIT NINE" },
    { 0x1C5A, "OL CHIKI LETTER LA" },
    { 0x1C5B, "OL CHIKI LETTER AT" },
    { 0x1C5C, "OL CHIKI LETTER AG" },
    { 0x1C5D, "OL CHIKI LETTER ANG" },
    { 0x1C5E, "OL CHIKI LETTER AL" },
    { 0x1C5F, "OL CHIKI LETTER LAA" },
    { 0x1C60, "OL CHIKI LETTER AAK" },
    { 0x1C61, "OL CHIKI LETTER AAJ" },
    { 0x1C62, "OL CHIKI LETTER AAM" },
    { 0x1C63, "OL CHIKI LETTER AAW" },
    { 0x1C64, "OL CHIKI LETTER LI" },
    { 0x1C65, "OL CHIKI LETTER IS" },
    { 0x1C66, "OL CHIKI LETTER IH" },
    { 0x1C67, "OL CHIKI LETTER INY" },
    { 0x1C68, "OL CHIKI LETTER IR" },
    { 0x1C69, "OL CHIKI LETTER LU" },
    { 0x1C6A, "OL CHIKI LETTER UC" },
    { 0x1C6B, "OL CHIKI LETTER UD" },
    { 0x1C6C, "OL CHIKI LETTER UNN" },
    { 0x1C6D, "OL CHIKI LETTER UY" },
    { 0x1C6E, "OL CHIKI LETTER LE" },
    { 0x1C6F, "OL CHIKI LETTER EP" },
    { 0x1C70, "OL CHIKI LETTER EDD" },
    { 0x1C71, "OL CHIKI LETTER EN" },
    { 0x1C72, "OL CHIKI LETTER ERR" },
    { 0x1C73, "OL CHIKI LETTER LO" },
    { 0x1C74, "OL CHIKI LETTER OTT" },
    { 0x1C75, "OL CHIKI LETTER OB" },
    { 0x1C76, "OL CHIKI LETTER OV" },
    { 0x1C77, "OL CHIKI LETTER OH" },
    { 0x1C78, "OL CHIKI MU TTUDDAG" },
    { 0x1C79, "OL CHIKI GAAHLAA TTUDDAAG" },
    { 0x1C7A, "OL CHIKI MU-GAAHLAA TTUDDAAG" },
    { 0x1C7B, "OL CHIKI RELAA" },
    { 0x1C7C, "OL CHIKI PHAARKAA" },
    { 0x1C7D, "OL CHIKI AHAD" },
    { 0x1C7E, "OL CHIKI PUNCTUATION MUCAAD" },
    { 0x1C7F, "OL CHIKI PUNCTUATION DOUBLE MUCAAD" },
    { 0x1C80, "CYRILLIC SMALL LETTER ROUNDED VE" },
    { 0x1C81, "CYRILLIC SMALL LETTER LONG-LEGGED DE" },
    { 0x1C82, "CYRILLIC SMALL LETTER NARROW O" },
    { 0x1C83, "CYRILLIC SMALL LETTER WIDE ES" },
    { 0x1C84, "CYRILLIC SMALL LETTER TALL TE" },
    { 0x1C85, "CYRILLIC SMALL LETTER THREE-LEGGED TE" },
    { 0x1C86, "CYRILLIC SMALL LETTER TALL HARD SIGN" },
    { 0x1C87, "CYRILLIC SMALL LETTER TALL YAT" },
    { 0x1C88, "CYRILLIC SMALL LETTER UNBLENDED UK" },
    { 0x1C90, "GEORGIAN MTAVRULI CAPITAL LETTER AN" },
    { 0x1C91, "GEORGIAN MTAVRULI CAPITAL LETTER BAN" },
    { 0x1C92, "GEORGIAN MTAVRULI CAPITAL LETTER GAN" },
    { 0x1C93, "GEORGIAN MTAVRULI CAPITAL LETTER DON" },
    { 0x1C94, "GEORGIAN MTAVRULI CAPITAL LETTER EN" },
    { 0x1C95, "GEORGIAN MTAVRULI CAPITAL LETTER VIN" },
    { 0x1C96, "GEORGIAN MTAVRULI CAPITAL LETTER ZEN" },
    { 0x1C97, "GEORGIAN MTAVRULI CAPITAL LETTER TAN" },
    { 0x1C98, "GEORGIAN MTAVRULI CAPITAL LETTER IN" },
    { 0x1C99, "GEORGIAN MTAVRULI CAPITAL LETTER KAN" },
    { 0x1C9A, "GEORGIAN MTAVRULI CAPITAL LETTER LAS" },
    { 0x1C9B, "GEORGIAN MTAVRULI CAPITAL LETTER MAN" },
    { 0x1C9C, "GEORGIAN MTAVRULI CAPITAL LETTER NAR" },
    { 0x1C9D, "GEORGIAN MTAVRULI CAPITAL LETTER ON" },
    { 0x1C9E, "GEORGIAN MTAVRULI CAPITAL LETTER PAR" },
    { 0x1C9F, "GEORGIAN MTAVRULI CAPITAL LETTER ZHAR" },
    { 0x1CA0, "GEORGIAN MTAVRULI CAPITAL LETTER RAE" },
    { 0x1CA1, "GEORGIAN MTAVRULI CAPITAL LETTER SAN" },
    { 0x1CA2, "GEORGIAN MTAVRULI CAPITAL LETTER TAR" },
    { 0x1CA3, "GEORGIAN MTAVRULI CAPITAL LETTER UN" },
    { 0x1CA4, "GEORGIAN MTAVRULI CAPITAL LETTER PHAR" },
    { 0x1CA5, "GEORGIAN MTAVRULI CAPITAL LETTER KHAR" },
    { 0x1CA6, "GEORGIAN MTAVRULI CAPITAL LETTER GHAN" },
    { 0x1CA7, "GEORGIAN MTAVRULI CAPITAL LETTER QAR" },
    { 0x1CA8, "GEORGIAN MTAVRULI CAPITAL LETTER SHIN" },
    { 0x1CA9, "GEORGIAN MTAVRULI CAPITAL LETTER CHIN" },
    { 0x1CAA, "GEORGIAN MTAVRULI CAPITAL LETTER CAN" },
    { 0x1CAB, "GEORGIAN MTAVRULI CAPITAL LETTER JIL" },
    { 0x1CAC, "GEORGIAN MTAVRULI CAPITAL LETTER CIL" },
    { 0x1CAD, "GEORGIAN MTAVRULI CAPITAL LETTER CHAR" },
    { 0x1CAE, "GEORGIAN MTAVRULI CAPITAL LETTER XAN" },
    { 0x1CAF, "GEORGIAN MTAVRULI CAPITAL LETTER JHAN" },
    { 0x1CB0, "GEORGIAN MTAVRULI CAPITAL LETTER HAE" },
    { 0x1CB1, "GEORGIAN MTAVRULI CAPITAL LETTER HE" },
    { 0x1CB2, "GEORGIAN MTAVRULI CAPITAL LETTER HIE" },
    { 0x1CB3, "GEORGIAN MTAVRULI CAPITAL LETTER WE" },
    { 0x1CB4, "GEORGIAN MTAVRULI CAPITAL LETTER HAR" },
    { 0x1CB5, "GEORGIAN MTAVRULI CAPITAL LETTER HOE" },
    { 0x1CB6, "GEORGIAN MTAVRULI CAPITAL LETTER FI" },
    { 0x1CB7, "GEORGIAN MTAVRULI CAPITAL LETTER YN" },
    { 0x1CB8, "GEORGIAN MTAVRULI CAPITAL LETTER ELIFI" },
    { 0x1CB9, "GEORGIAN MTAVRULI CAPITAL LETTER TURNED GAN" },
    { 0x1CBA, "GEORGIAN MTAVRULI CAPITAL LETTER AIN" },
    { 0x1CBD, "GEORGIAN MTAVRULI CAPITAL LETTER AEN" },
    { 0x1CBE, "GEORGIAN MTAVRULI CAPITAL LETTER HARD SIGN" },
    { 0x1CBF, "GEORGIAN MTAVRULI CAPITAL LETTER LABIAL SIGN" },
    { 0x1CC0, "SUNDANESE PUNCTUATION BINDU SURYA" },
    { 0x1CC1, "SUNDANESE PUNCTUATION BINDU PANGLONG" },
    { 0x1CC2, "SUNDANESE PUNCTUATION BINDU PURNAMA" },
    { 0x1CC3, "SUNDANESE PUNCTUATION BINDU CAKRA" },
    { 0x1CC4, "SUNDANESE PUNCTUATION BINDU LEU SATANGA" },
    { 0x1CC5, "SUNDANESE PUNCTUATION BINDU KA SATANGA" },
    { 0x1CC6, "SUNDANESE PUNCTUATION BINDU DA SATANGA" },
    { 0x1CC7, "SUNDANESE PUNCTUATION BINDU BA SATANGA" },
    { 0x1CD0, "VEDIC TONE KARSHANA" },
    { 0x1CD1, "VEDIC TONE SHARA" },
    { 0x1CD2, "VEDIC TONE PRENKHA" },
    { 0x1CD3, "VEDIC SIGN NIHSHVASA" },
    { 0x1CD4, "VEDIC SIGN YAJURVEDIC MIDLINE SVARITA" },
    { 0x1CD5, "VEDIC TONE YAJURVEDIC AGGRAVATED INDEPENDENT SVARITA" },
    { 0x1CD6, "VEDIC TONE YAJURVEDIC INDEPENDENT SVARITA" },
    { 0x1CD7, "VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA" },
    { 0x1CD8, "VEDIC TONE CANDRA BELOW" },
    { 0x1CD9, "VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA SCHROEDER" },
    { 0x1CDA, "VEDIC TONE DOUBLE SVARITA" },
    { 0x1CDB, "VEDIC TONE TRIPLE SVARITA" },
    { 0x1CDC, "VEDIC TONE KATHAKA ANUDATTA" },
    { 0x1CDD, "VEDIC TONE DOT BELOW" },
    { 0x1CDE, "VEDIC TONE TWO DOTS BELOW" },
    { 0x1CDF, "VEDIC TONE THREE DOTS BELOW" },
    { 0x1CE0, "VEDIC TONE RIGVEDIC KASHMIRI INDEPENDENT SVARITA" },
    { 0x1CE1, "VEDIC TONE ATHARVAVEDIC INDEPENDENT SVARITA" },
    { 0x1CE2, "VEDIC SIGN VISARGA SVARITA" },
    { 0x1CE3, "VEDIC SIGN VISARGA UDATTA" },
    { 0x1CE4, "VEDIC SIGN REVERSED VISARGA UDATTA" },
    { 0x1CE5, "VEDIC SIGN VISARGA ANUDATTA" },
    { 0x1CE6, "VEDIC SIGN REVERSED VISARGA ANUDATTA" },
    { 0x1CE7, "VEDIC SIGN VISARGA UDATTA WITH TAIL" },
    { 0x1CE8, "VEDIC SIGN VISARGA ANUDATTA WITH TAIL" },
    { 0x1CE9, "VEDIC SIGN ANUSVARA ANTARGOMUKHA" },
    { 0x1CEA, "VEDIC SIGN ANUSVARA BAHIRGOMUKHA" },
    { 0x1CEB, "VEDIC SIGN ANUSVARA VAMAGOMUKHA" },
    { 0x1CEC, "VEDIC SIGN ANUSVARA VAMAGOMUKHA WITH TAIL" },
    { 0x1CED, "VEDIC SIGN TIRYAK" },
    { 0x1CEE, "VEDIC SIGN HEXIFORM LONG ANUSVARA" },
    { 0x1CEF, "VEDIC SIGN LONG ANUSVARA" },
    { 0x1CF0, "VEDIC SIGN RTHANG LONG ANUSVARA" },
    { 0x1CF1, "VEDIC SIGN ANUSVARA UBHAYATO MUKHA" },
    { 0x1CF2, "VEDIC SIGN ARDHAVISARGA" },
    { 0x1CF3, "VEDIC SIGN ROTATED ARDHAVISARGA" },
    { 0x1CF4, "VEDIC TONE CANDRA ABOVE" },
    { 0x1CF5, "VEDIC SIGN JIHVAMULIYA" },
    { 0x1CF6, "VEDIC SIGN UPADHMANIYA" },
    { 0x1CF7, "VEDIC SIGN ATIKRAMA" },
    { 0x1CF8, "VEDIC TONE RING ABOVE" },
    { 0x1CF9, "VEDIC TONE DOUBLE RING ABOVE" },
    { 0x1CFA, "VEDIC SIGN DOUBLE ANUSVARA ANTARGOMUKHA" },
    { 0x1D00, "LATIN LETTER SMALL CAPITAL A" },
    { 0x1D01, "LATIN LETTER SMALL CAPITAL AE" },
    { 0x1D02, "LATIN SMALL LETTER TURNED AE" },
    { 0x1D03, "LATIN LETTER SMALL CAPITAL BARRED B" },
    { 0x1D04, "LATIN LETTER SMALL CAPITAL C" },
    { 0x1D05, "LATIN LETTER SMALL CAPITAL D" },
    { 0x1D06, "LATIN LETTER SMALL CAPITAL ETH" },
    { 0x1D07, "LATIN LETTER SMALL CAPITAL E" },
    { 0x1D08, "LATIN SMALL LETTER TURNED OPEN E" },
    { 0x1D09, "LATIN SMALL LETTER TURNED I" },
    { 0x1D0A, "LATIN LETTER SMALL CAPITAL J" },
    { 0x1D0B, "LATIN LETTER SMALL CAPITAL K" },
    { 0x1D0C, "LATIN LETTER SMALL CAPITAL L WITH STROKE" },
    { 0x1D0D, "LATIN LETTER SMALL CAPITAL M" },
    { 0x1D0E, "LATIN LETTER SMALL CAPITAL REVERSED N" },
    { 0x1D0F, "LATIN LETTER SMALL CAPITAL O" },
    { 0x1D10, "LATIN LETTER SMALL CAPITAL OPEN O" },
    { 0x1D11, "LATIN SMALL LETTER SIDEWAYS O" },
    { 0x1D12, "LATIN SMALL LETTER SIDEWAYS OPEN O" },
    { 0x1D13, "LATIN SMALL LETTER SIDEWAYS O WITH STROKE" },
    { 0x1D14, "LATIN SMALL LETTER TURNED OE" },
    { 0x1D15, "LATIN LETTER SMALL CAPITAL OU" },
    { 0x1D16, "LATIN SMALL LETTER TOP HALF O" },
    { 0x1D17, "LATIN SMALL LETTER BOTTOM HALF O" },
    { 0x1D18, "LATIN LETTER SMALL CAPITAL P" },
    { 0x1D19, "LATIN LETTER SMALL CAPITAL REVERSED R" },
    { 0x1D1A, "LATIN LETTER SMALL CAPITAL TURNED R" },
    { 0x1D1B, "LATIN LETTER SMALL CAPITAL T" },
    { 0x1D1C, "LATIN LETTER SMALL CAPITAL U" },
    { 0x1D1D, "LATIN SMALL LETTER SIDEWAYS U" },
    { 0x1D1E, "LATIN SMALL LETTER SIDEWAYS DIAERESIZED U" },
    { 0x1D1F, "LATIN SMALL LETTER SIDEWAYS TURNED M" },
    { 0x1D20, "LATIN LETTER SMALL CAPITAL V" },
    { 0x1D21, "LATIN LETTER SMALL CAPITAL W" },
    { 0x1D22, "LATIN LETTER SMALL CAPITAL Z" },
    { 0x1D23, "LATIN LETTER SMALL CAPITAL EZH" },
    { 0x1D24, "LATIN LETTER VOICED LARYNGEAL SPIRANT" },
    { 0x1D25, "LATIN LETTER AIN" },
    { 0x1D26, "GREEK LETTER SMALL CAPITAL GAMMA" },
    { 0x1D27, "GREEK LETTER SMALL CAPITAL LAMDA" },
    { 0x1D28, "GREEK LETTER SMALL CAPITAL PI" },
    { 0x1D29, "GREEK LETTER SMALL CAPITAL RHO" },
    { 0x1D2A, "GREEK LETTER SMALL CAPITAL PSI" },
    { 0x1D2B, "CYRILLIC LETTER SMALL CAPITAL EL" },
    { 0x1D2C, "MODIFIER LETTER CAPITAL A" },
    { 0x1D2D, "MODIFIER LETTER CAPITAL AE" },
    { 0x1D2E, "MODIFIER LETTER CAPITAL B" },
    { 0x1D2F, "MODIFIER LETTER CAPITAL BARRED B" },
    { 0x1D30, "MODIFIER LETTER CAPITAL D" },
    { 0x1D31, "MODIFIER LETTER CAPITAL E" },
    { 0x1D32, "MODIFIER LETTER CAPITAL REVERSED E" },
    { 0x1D33, "MODIFIER LETTER CAPITAL G" },
    { 0x1D34, "MODIFIER LETTER CAPITAL H" },
    { 0x1D35, "MODIFIER LETTER CAPITAL I" },
    { 0x1D36, "MODIFIER LETTER CAPITAL J" },
    { 0x1D37, "MODIFIER LETTER CAPITAL K" },
    { 0x1D38, "MODIFIER LETTER CAPITAL L" },
    { 0x1D39, "MODIFIER LETTER CAPITAL M" },
    { 0x1D3A, "MODIFIER LETTER CAPITAL N" },
    { 0x1D3B, "MODIFIER LETTER CAPITAL REVERSED N" },
    { 0x1D3C, "MODIFIER LETTER CAPITAL O" },
    { 0x1D3D, "MODIFIER LETTER CAPITAL OU" },
    { 0x1D3E, "MODIFIER LETTER CAPITAL P" },
    { 0x1D3F, "MODIFIER LETTER CAPITAL R" },
    { 0x1D40, "MODIFIER LETTER CAPITAL T" },
    { 0x1D41, "MODIFIER LETTER CAPITAL U" },
    { 0x1D42, "MODIFIER LETTER CAPITAL W" },
    { 0x1D43, "MODIFIER LETTER SMALL A" },
    { 0x1D44, "MODIFIER LETTER SMALL TURNED A" },
    { 0x1D45, "MODIFIER LETTER SMALL ALPHA" },
    { 0x1D46, "MODIFIER LETTER SMALL TURNED AE" },
    { 0x1D47, "MODIFIER LETTER SMALL B" },
    { 0x1D48, "MODIFIER LETTER SMALL D" },
    { 0x1D49, "MODIFIER LETTER SMALL E" },
    { 0x1D4A, "MODIFIER LETTER SMALL SCHWA" },
    { 0x1D4B, "MODIFIER LETTER SMALL OPEN E" },
    { 0x1D4C, "MODIFIER LETTER SMALL TURNED OPEN E" },
    { 0x1D4D, "MODIFIER LETTER SMALL G" },
    { 0x1D4E, "MODIFIER LETTER SMALL TURNED I" },
    { 0x1D4F, "MODIFIER LETTER SMALL K" },
    { 0x1D50, "MODIFIER LETTER SMALL M" },
    { 0x1D51, "MODIFIER LETTER SMALL ENG" },
    { 0x1D52, "MODIFIER LETTER SMALL O" },
    { 0x1D53, "MODIFIER LETTER SMALL OPEN O" },
    { 0x1D54, "MODIFIER LETTER SMALL TOP HALF O" },
    { 0x1D55, "MODIFIER LETTER SMALL BOTTOM HALF O" },
    { 0x1D56, "MODIFIER LETTER SMALL P" },
    { 0x1D57, "MODIFIER LETTER SMALL T" },
    { 0x1D58, "MODIFIER LETTER SMALL U" },
    { 0x1D59, "MODIFIER LETTER SMALL SIDEWAYS U" },
    { 0x1D5A, "MODIFIER LETTER SMALL TURNED M" },
    { 0x1D5B, "MODIFIER LETTER SMALL V" },
    { 0x1D5C, "MODIFIER LETTER SMALL AIN" },
    { 0x1D5D, "MODIFIER LETTER SMALL BETA" },
    { 0x1D5E, "MODIFIER LETTER SMALL GREEK GAMMA" },
    { 0x1D5F, "MODIFIER LETTER SMALL DELTA" },
    { 0x1D60, "MODIFIER LETTER SMALL GREEK PHI" },
    { 0x1D61, "MODIFIER LETTER SMALL CHI" },
    { 0x1D62, "LATIN SUBSCRIPT SMALL LETTER I" },
    { 0x1D63, "LATIN SUBSCRIPT SMALL LETTER R" },
    { 0x1D64, "LATIN SUBSCRIPT SMALL LETTER U" },
    { 0x1D65, "LATIN SUBSCRIPT SMALL LETTER V" },
    { 0x1D66, "GREEK SUBSCRIPT SMALL LETTER BETA" },
    { 0x1D67, "GREEK SUBSCRIPT SMALL LETTER GAMMA" },
    { 0x1D68, "GREEK SUBSCRIPT SMALL LETTER RHO" },
    { 0x1D69, "GREEK SUBSCRIPT SMALL LETTER PHI" },
    { 0x1D6A, "GREEK SUBSCRIPT SMALL LETTER CHI" },
    { 0x1D6B, "LATIN SMALL LETTER UE" },
    { 0x1D6C, "LATIN SMALL LETTER B WITH MIDDLE TILDE" },
    { 0x1D6D, "LATIN SMALL LETTER D WITH MIDDLE TILDE" },
    { 0x1D6E, "LATIN SMALL LETTER F WITH MIDDLE TILDE" },
    { 0x1D6F, "LATIN SMALL LETTER M WITH MIDDLE TILDE" },
    { 0x1D70, "LATIN SMALL LETTER N WITH MIDDLE TILDE" },
    { 0x1D71, "LATIN SMALL LETTER P WITH MIDDLE TILDE" },
    { 0x1D72, "LATIN SMALL LETTER R WITH MIDDLE TILDE" },
    { 0x1D73, "LATIN SMALL LETTER R WITH FISHHOOK AND MIDDLE TILDE" },
    { 0x1D74, "LATIN SMALL LETTER S WITH MIDDLE TILDE" },
    { 0x1D75, "LATIN SMALL LETTER T WITH MIDDLE TILDE" },
    { 0x1D76, "LATIN SMALL LETTER Z WITH MIDDLE TILDE" },
    { 0x1D77, "LATIN SMALL LETTER TURNED G" },
    { 0x1D78, "MODIFIER LETTER CYRILLIC EN" },
    { 0x1D79, "LATIN SMALL LETTER INSULAR G" },
    { 0x1D7A, "LATIN SMALL LETTER TH WITH STRIKETHROUGH" },
    { 0x1D7B, "LATIN SMALL CAPITAL LETTER I WITH STROKE" },
    { 0x1D7C, "LATIN SMALL LETTER IOTA WITH STROKE" },
    { 0x1D7D, "LATIN SMALL LETTER P WITH STROKE" },
    { 0x1D7E, "LATIN SMALL CAPITAL LETTER U WITH STROKE" },
    { 0x1D7F, "LATIN SMALL LETTER UPSILON WITH STROKE" },
    { 0x1D80, "LATIN SMALL LETTER B WITH PALATAL HOOK" },
    { 0x1D81, "LATIN SMALL LETTER D WITH PALATAL HOOK" },
    { 0x1D82, "LATIN SMALL LETTER F WITH PALATAL HOOK" },
    { 0x1D83, "LATIN SMALL LETTER G WITH PALATAL HOOK" },
    { 0x1D84, "LATIN SMALL LETTER K WITH PALATAL HOOK" },
    { 0x1D85, "LATIN SMALL LETTER L WITH PALATAL HOOK" },
    { 0x1D86, "LATIN SMALL LETTER M WITH PALATAL HOOK" },
    { 0x1D87, "LATIN SMALL LETTER N WITH PALATAL HOOK" },
    { 0x1D88, "LATIN SMALL LETTER P WITH PALATAL HOOK" },
    { 0x1D89, "LATIN SMALL LETTER R WITH PALATAL HOOK" },
    { 0x1D8A, "LATIN SMALL LETTER S WITH PALATAL HOOK" },
    { 0x1D8B, "LATIN SMALL LETTER ESH WITH PALATAL HOOK" },
    { 0x1D8C, "LATIN SMALL LETTER V WITH PALATAL HOOK" },
    { 0x1D8D, "LATIN SMALL LETTER X WITH PALATAL HOOK" },
    { 0x1D8E, "LATIN SMALL LETTER Z WITH PALATAL HOOK" },
    { 0x1D8F, "LATIN SMALL LETTER A WITH RETROFLEX HOOK" },
    { 0x1D90, "LATIN SMALL LETTER ALPHA WITH RETROFLEX HOOK" },
    { 0x1D91, "LATIN SMALL LETTER D WITH HOOK AND TAIL" },
    { 0x1D92, "LATIN SMALL LETTER E WITH RETROFLEX HOOK" },
    { 0x1D93, "LATIN SMALL LETTER OPEN E WITH RETROFLEX HOOK" },
    { 0x1D94, "LATIN SMALL LETTER REVERSED OPEN E WITH RETROFLEX HOOK" },
    { 0x1D95, "LATIN SMALL LETTER SCHWA WITH RETROFLEX HOOK" },
    { 0x1D96, "LATIN SMALL LETTER I WITH RETROFLEX HOOK" },
    { 0x1D97, "LATIN SMALL LETTER OPEN O WITH RETROFLEX HOOK" },
    { 0x1D98, "LATIN SMALL LETTER ESH WITH RETROFLEX HOOK" },
    { 0x1D99, "LATIN SMALL LETTER U WITH RETROFLEX HOOK" },
    { 0x1D9A, "LATIN SMALL LETTER EZH WITH RETROFLEX HOOK" },
    { 0x1D9B, "MODIFIER LETTER SMALL TURNED ALPHA" },
    { 0x1D9C, "MODIFIER LETTER SMALL C" },
    { 0x1D9D, "MODIFIER LETTER SMALL C WITH CURL" },
    { 0x1D9E, "MODIFIER LETTER SMALL ETH" },
    { 0x1D9F, "MODIFIER LETTER SMALL REVERSED OPEN E" },
    { 0x1DA0, "MODIFIER LETTER SMALL F" },
    { 0x1DA1, "MODIFIER LETTER SMALL DOTLESS J WITH STROKE" },
    { 0x1DA2, "MODIFIER LETTER SMALL SCRIPT G" },
    { 0x1DA3, "MODIFIER LETTER SMALL TURNED H" },
    { 0x1DA4, "MODIFIER LETTER SMALL I WITH STROKE" },
    { 0x1DA5, "MODIFIER LETTER SMALL IOTA" },
    { 0x1DA6, "MODIFIER LETTER SMALL CAPITAL I" },
    { 0x1DA7, "MODIFIER LETTER SMALL CAPITAL I WITH STROKE" },
    { 0x1DA8, "MODIFIER LETTER SMALL J WITH CROSSED-TAIL" },
    { 0x1DA9, "MODIFIER LETTER SMALL L WITH RETROFLEX HOOK" },
    { 0x1DAA, "MODIFIER LETTER SMALL L WITH PALATAL HOOK" },
    { 0x1DAB, "MODIFIER LETTER SMALL CAPITAL L" },
    { 0x1DAC, "MODIFIER LETTER SMALL M WITH HOOK" },
    { 0x1DAD, "MODIFIER LETTER SMALL TURNED M WITH LONG LEG" },
    { 0x1DAE, "MODIFIER LETTER SMALL N WITH LEFT HOOK" },
    { 0x1DAF, "MODIFIER LETTER SMALL N WITH RETROFLEX HOOK" },
    { 0x1DB0, "MODIFIER LETTER SMALL CAPITAL N" },
    { 0x1DB1, "MODIFIER LETTER SMALL BARRED O" },
    { 0x1DB2, "MODIFIER LETTER SMALL PHI" },
    { 0x1DB3, "MODIFIER LETTER SMALL S WITH HOOK" },
    { 0x1DB4, "MODIFIER LETTER SMALL ESH" },
    { 0x1DB5, "MODIFIER LETTER SMALL T WITH PALATAL HOOK" },
    { 0x1DB6, "MODIFIER LETTER SMALL U BAR" },
    { 0x1DB7, "MODIFIER LETTER SMALL UPSILON" },
    { 0x1DB8, "MODIFIER LETTER SMALL CAPITAL U" },
    { 0x1DB9, "MODIFIER LETTER SMALL V WITH HOOK" },
    { 0x1DBA, "MODIFIER LETTER SMALL TURNED V" },
    { 0x1DBB, "MODIFIER LETTER SMALL Z" },
    { 0x1DBC, "MODIFIER LETTER SMALL Z WITH RETROFLEX HOOK" },
    { 0x1DBD, "MODIFIER LETTER SMALL Z WITH CURL" },
    { 0x1DBE, "MODIFIER LETTER SMALL EZH" },
    { 0x1DBF, "MODIFIER LETTER SMALL THETA" },
    { 0x1DC0, "COMBINING DOTTED GRAVE ACCENT" },
    { 0x1DC1, "COMBINING DOTTED ACUTE ACCENT" },
    { 0x1DC2, "COMBINING SNAKE BELOW" },
    { 0x1DC3, "COMBINING SUSPENSION MARK" },
    { 0x1DC4, "COMBINING MACRON-ACUTE" },
    { 0x1DC5, "COMBINING GRAVE-MACRON" },
    { 0x1DC6, "COMBINING MACRON-GRAVE" },
    { 0x1DC7, "COMBINING ACUTE-MACRON" },
    { 0x1DC8, "COMBINING GRAVE-ACUTE-GRAVE" },
    { 0x1DC9, "COMBINING ACUTE-GRAVE-ACUTE" },
    { 0x1DCA, "COMBINING LATIN SMALL LETTER R BELOW" },
    { 0x1DCB, "COMBINING BREVE-MACRON" },
    { 0x1DCC, "COMBINING MACRON-BREVE" },
    { 0x1DCD, "COMBINING DOUBLE CIRCUMFLEX ABOVE" },
    { 0x1DCE, "COMBINING OGONEK ABOVE" },
    { 0x1DCF, "COMBINING ZIGZAG BELOW" },
    { 0x1DD0, "COMBINING IS BELOW" },
    { 0x1DD1, "COMBINING UR ABOVE" },
    { 0x1DD2, "COMBINING US ABOVE" },
    { 0x1DD3, "COMBINING LATIN SMALL LETTER FLATTENED OPEN A ABOVE" },
    { 0x1DD4, "COMBINING LATIN SMALL LETTER AE" },
    { 0x1DD5, "COMBINING LATIN SMALL LETTER AO" },
    { 0x1DD6, "COMBINING LATIN SMALL LETTER AV" },
    { 0x1DD7, "COMBINING LATIN SMALL LETTER C CEDILLA" },
    { 0x1DD8, "COMBINING LATIN SMALL LETTER INSULAR D" },
    { 0x1DD9, "COMBINING LATIN SMALL LETTER ETH" },
    { 0x1DDA, "COMBINING LATIN SMALL LETTER G" },
    { 0x1DDB, "COMBINING LATIN LETTER SMALL CAPITAL G" },
    { 0x1DDC, "COMBINING LATIN SMALL LETTER K" },
    { 0x1DDD, "COMBINING LATIN SMALL LETTER L" },
    { 0x1DDE, "COMBINING LATIN LETTER SMALL CAPITAL L" },
    { 0x1DDF, "COMBINING LATIN LETTER SMALL CAPITAL M" },
    { 0x1DE0, "COMBINING LATIN SMALL LETTER N" },
    { 0x1DE1, "COMBINING LATIN LETTER SMALL CAPITAL N" },
    { 0x1DE2, "COMBINING LATIN LETTER SMALL CAPITAL R" },
    { 0x1DE3, "COMBINING LATIN SMALL LETTER R ROTUNDA" },
    { 0x1DE4, "COMBINING LATIN SMALL LETTER S" },
    { 0x1DE5, "COMBINING LATIN SMALL LETTER LONG S" },
    { 0x1DE6, "COMBINING LATIN SMALL LETTER Z" },
    { 0x1DE7, "COMBINING LATIN SMALL LETTER ALPHA" },
    { 0x1DE8, "COMBINING LATIN SMALL LETTER B" },
    { 0x1DE9, "COMBINING LATIN SMALL LETTER BETA" },
    { 0x1DEA, "COMBINING LATIN SMALL LETTER SCHWA" },
    { 0x1DEB, "COMBINING LATIN SMALL LETTER F" },
    { 0x1DEC, "COMBINING LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE" },
    { 0x1DED, "COMBINING LATIN SMALL LETTER O WITH LIGHT CENTRALIZATION STROKE" },
    { 0x1DEE, "COMBINING LATIN SMALL LETTER P" },
    { 0x1DEF, "COMBINING LATIN SMALL LETTER ESH" },
    { 0x1DF0, "COMBINING LATIN SMALL LETTER U WITH LIGHT CENTRALIZATION STROKE" },
    { 0x1DF1, "COMBINING LATIN SMALL LETTER W" },
    { 0x1DF2, "COMBINING LATIN SMALL LETTER A WITH DIAERESIS" },
    { 0x1DF3, "COMBINING LATIN SMALL LETTER O WITH DIAERESIS" },
    { 0x1DF4, "COMBINING LATIN SMALL LETTER U WITH DIAERESIS" },
    { 0x1DF5, "COMBINING UP TACK ABOVE" },
    { 0x1DF6, "COMBINING KAVYKA ABOVE RIGHT" },
    { 0x1DF7, "COMBINING KAVYKA ABOVE LEFT" },
    { 0x1DF8, "COMBINING DOT ABOVE LEFT" },
    { 0x1DF9, "COMBINING WIDE INVERTED BRIDGE BELOW" },
    { 0x1DFA, "COMBINING DOT BELOW LEFT" },
    { 0x1DFB, "COMBINING DELETION MARK" },
    { 0x1DFC, "COMBINING DOUBLE INVERTED BREVE BELOW" },
    { 0x1DFD, "COMBINING ALMOST EQUAL TO BELOW" },
    { 0x1DFE, "COMBINING LEFT ARROWHEAD ABOVE" },
    { 0x1DFF, "COMBINING RIGHT ARROWHEAD AND DOWN ARROWHEAD BELOW" },
    { 0x1E00, "LATIN CAPITAL LETTER A WITH RING BELOW" },
    { 0x1E01, "LATIN SMALL LETTER A WITH RING BELOW" },
    { 0x1E02, "LATIN CAPITAL LETTER B WITH DOT ABOVE" },
    { 0x1E03, "LATIN SMALL LETTER B WITH DOT ABOVE" },
    { 0x1E04, "LATIN CAPITAL LETTER B WITH DOT BELOW" },
    { 0x1E05, "LATIN SMALL LETTER B WITH DOT BELOW" },
    { 0x1E06, "LATIN CAPITAL LETTER B WITH LINE BELOW" },
    { 0x1E07, "LATIN SMALL LETTER B WITH LINE BELOW" },
    { 0x1E08, "LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE" },
    { 0x1E09, "LATIN SMALL LETTER C WITH CEDILLA AND ACUTE" },
    { 0x1E0A, "LATIN CAPITAL LETTER D WITH DOT ABOVE" },
    { 0x1E0B, "LATIN SMALL LETTER D WITH DOT ABOVE" },
    { 0x1E0C, "LATIN CAPITAL LETTER D WITH DOT BELOW" },
    { 0x1E0D, "LATIN SMALL LETTER D WITH DOT BELOW" },
    { 0x1E0E, "LATIN CAPITAL LETTER D WITH LINE BELOW" },
    { 0x1E0F, "LATIN SMALL LETTER D WITH LINE BELOW" },
    { 0x1E10, "LATIN CAPITAL LETTER D WITH CEDILLA" },
    { 0x1E11, "LATIN SMALL LETTER D WITH CEDILLA" },
    { 0x1E12, "LATIN CAPITAL LETTER D WITH CIRCUMFLEX BELOW" },
    { 0x1E13, "LATIN SMALL LETTER D WITH CIRCUMFLEX BELOW" },
    { 0x1E14, "LATIN CAPITAL LETTER E WITH MACRON AND GRAVE" },
    { 0x1E15, "LATIN SMALL LETTER E WITH MACRON AND GRAVE" },
    { 0x1E16, "LATIN CAPITAL LETTER E WITH MACRON AND ACUTE" },
    { 0x1E17, "LATIN SMALL LETTER E WITH MACRON AND ACUTE" },
    { 0x1E18, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX BELOW" },
    { 0x1E19, "LATIN SMALL LETTER E WITH CIRCUMFLEX BELOW" },
    { 0x1E1A, "LATIN CAPITAL LETTER E WITH TILDE BELOW" },
    { 0x1E1B, "LATIN SMALL LETTER E WITH TILDE BELOW" },
    { 0x1E1C, "LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE" },
    { 0x1E1D, "LATIN SMALL LETTER E WITH CEDILLA AND BREVE" },
    { 0x1E1E, "LATIN CAPITAL LETTER F WITH DOT ABOVE" },
    { 0x1E1F, "LATIN SMALL LETTER F WITH DOT ABOVE" },
    { 0x1E20, "LATIN CAPITAL LETTER G WITH MACRON" },
    { 0x1E21, "LATIN SMALL LETTER G WITH MACRON" },
    { 0x1E22, "LATIN CAPITAL LETTER H WITH DOT ABOVE" },
    { 0x1E23, "LATIN SMALL LETTER H WITH DOT ABOVE" },
    { 0x1E24, "LATIN CAPITAL LETTER H WITH DOT BELOW" },
    { 0x1E25, "LATIN SMALL LETTER H WITH DOT BELOW" },
    { 0x1E26, "LATIN CAPITAL LETTER H WITH DIAERESIS" },
    { 0x1E27, "LATIN SMALL LETTER H WITH DIAERESIS" },
    { 0x1E28, "LATIN CAPITAL LETTER H WITH CEDILLA" },
    { 0x1E29, "LATIN SMALL LETTER H WITH CEDILLA" },
    { 0x1E2A, "LATIN CAPITAL LETTER H WITH BREVE BELOW" },
    { 0x1E2B, "LATIN SMALL LETTER H WITH BREVE BELOW" },
    { 0x1E2C, "LATIN CAPITAL LETTER I WITH TILDE BELOW" },
    { 0x1E2D, "LATIN SMALL LETTER I WITH TILDE BELOW" },
    { 0x1E2E, "LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE" },
    { 0x1E2F, "LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE" },
    { 0x1E30, "LATIN CAPITAL LETTER K WITH ACUTE" },
    { 0x1E31, "LATIN SMALL LETTER K WITH ACUTE" },
    { 0x1E32, "LATIN CAPITAL LETTER K WITH DOT BELOW" },
    { 0x1E33, "LATIN SMALL LETTER K WITH DOT BELOW" },
    { 0x1E34, "LATIN CAPITAL LETTER K WITH LINE BELOW" },
    { 0x1E35, "LATIN SMALL LETTER K WITH LINE BELOW" },
    { 0x1E36, "LATIN CAPITAL LETTER L WITH DOT BELOW" },
    { 0x1E37, "LATIN SMALL LETTER L WITH DOT BELOW" },
    { 0x1E38, "LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON" },
    { 0x1E39, "LATIN SMALL LETTER L WITH DOT BELOW AND MACRON" },
    { 0x1E3A, "LATIN CAPITAL LETTER L WITH LINE BELOW" },
    { 0x1E3B, "LATIN SMALL LETTER L WITH LINE BELOW" },
    { 0x1E3C, "LATIN CAPITAL LETTER L WITH CIRCUMFLEX BELOW" },
    { 0x1E3D, "LATIN SMALL LETTER L WITH CIRCUMFLEX BELOW" },
    { 0x1E3E, "LATIN CAPITAL LETTER M WITH ACUTE" },
    { 0x1E3F, "LATIN SMALL LETTER M WITH ACUTE" },
    { 0x1E40, "LATIN CAPITAL LETTER M WITH DOT ABOVE" },
    { 0x1E41, "LATIN SMALL LETTER M WITH DOT ABOVE" },
    { 0x1E42, "LATIN CAPITAL LETTER M WITH DOT BELOW" },
    { 0x1E43, "LATIN SMALL LETTER M WITH DOT BELOW" },
    { 0x1E44, "LATIN CAPITAL LETTER N WITH DOT ABOVE" },
    { 0x1E45, "LATIN SMALL LETTER N WITH DOT ABOVE" },
    { 0x1E46, "LATIN CAPITAL LETTER N WITH DOT BELOW" },
    { 0x1E47, "LATIN SMALL LETTER N WITH DOT BELOW" },
    { 0x1E48, "LATIN CAPITAL LETTER N WITH LINE BELOW" },
    { 0x1E49, "LATIN SMALL LETTER N WITH LINE BELOW" },
    { 0x1E4A, "LATIN CAPITAL LETTER N WITH CIRCUMFLEX BELOW" },
    { 0x1E4B, "LATIN SMALL LETTER N WITH CIRCUMFLEX BELOW" },
    { 0x1E4C, "LATIN CAPITAL LETTER O WITH TILDE AND ACUTE" },
    { 0x1E4D, "LATIN SMALL LETTER O WITH TILDE AND ACUTE" },
    { 0x1E4E, "LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS" },
    { 0x1E4F, "LATIN SMALL LETTER O WITH TILDE AND DIAERESIS" },
    { 0x1E50, "LATIN CAPITAL LETTER O WITH MACRON AND GRAVE" },
    { 0x1E51, "LATIN SMALL LETTER O WITH MACRON AND GRAVE" },
    { 0x1E52, "LATIN CAPITAL LETTER O WITH MACRON AND ACUTE" },
    { 0x1E53, "LATIN SMALL LETTER O WITH MACRON AND ACUTE" },
    { 0x1E54, "LATIN CAPITAL LETTER P WITH ACUTE" },
    { 0x1E55, "LATIN SMALL LETTER P WITH ACUTE" },
    { 0x1E56, "LATIN CAPITAL LETTER P WITH DOT ABOVE" },
    { 0x1E57, "LATIN SMALL LETTER P WITH DOT ABOVE" },
    { 0x1E58, "LATIN CAPITAL LETTER R WITH DOT ABOVE" },
    { 0x1E59, "LATIN SMALL LETTER R WITH DOT ABOVE" },
    { 0x1E5A, "LATIN CAPITAL LETTER R WITH DOT BELOW" },
    { 0x1E5B, "LATIN SMALL LETTER R WITH DOT BELOW" },
    { 0x1E5C, "LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON" },
    { 0x1E5D, "LATIN SMALL LETTER R WITH DOT BELOW AND MACRON" },
    { 0x1E5E, "LATIN CAPITAL LETTER R WITH LINE BELOW" },
    { 0x1E5F, "LATIN SMALL LETTER R WITH LINE BELOW" },
    { 0x1E60, "LATIN CAPITAL LETTER S WITH DOT ABOVE" },
    { 0x1E61, "LATIN SMALL LETTER S WITH DOT ABOVE" },
    { 0x1E62, "LATIN CAPITAL LETTER S WITH DOT BELOW" },
    { 0x1E63, "LATIN SMALL LETTER S WITH DOT BELOW" },
    { 0x1E64, "LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE" },
    { 0x1E65, "LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE" },
    { 0x1E66, "LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE" },
    { 0x1E67, "LATIN SMALL LETTER S WITH CARON AND DOT ABOVE" },
    { 0x1E68, "LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE" },
    { 0x1E69, "LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE" },
    { 0x1E6A, "LATIN CAPITAL LETTER T WITH DOT ABOVE" },
    { 0x1E6B, "LATIN SMALL LETTER T WITH DOT ABOVE" },
    { 0x1E6C, "LATIN CAPITAL LETTER T WITH DOT BELOW" },
    { 0x1E6D, "LATIN SMALL LETTER T WITH DOT BELOW" },
    { 0x1E6E, "LATIN CAPITAL LETTER T WITH LINE BELOW" },
    { 0x1E6F, "LATIN SMALL LETTER T WITH LINE BELOW" },
    { 0x1E70, "LATIN CAPITAL LETTER T WITH CIRCUMFLEX BELOW" },
    { 0x1E71, "LATIN SMALL LETTER T WITH CIRCUMFLEX BELOW" },
    { 0x1E72, "LATIN CAPITAL LETTER U WITH DIAERESIS BELOW" },
    { 0x1E73, "LATIN SMALL LETTER U WITH DIAERESIS BELOW" },
    { 0x1E74, "LATIN CAPITAL LETTER U WITH TILDE BELOW" },
    { 0x1E75, "LATIN SMALL LETTER U WITH TILDE BELOW" },
    { 0x1E76, "LATIN CAPITAL LETTER U WITH CIRCUMFLEX BELOW" },
    { 0x1E77, "LATIN SMALL LETTER U WITH CIRCUMFLEX BELOW" },
    { 0x1E78, "LATIN CAPITAL LETTER U WITH TILDE AND ACUTE" },
    { 0x1E79, "LATIN SMALL LETTER U WITH TILDE AND ACUTE" },
    { 0x1E7A, "LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS" },
    { 0x1E7B, "LATIN SMALL LETTER U WITH MACRON AND DIAERESIS" },
    { 0x1E7C, "LATIN CAPITAL LETTER V WITH TILDE" },
    { 0x1E7D, "LATIN SMALL LETTER V WITH TILDE" },
    { 0x1E7E, "LATIN CAPITAL LETTER V WITH DOT BELOW" },
    { 0x1E7F, "LATIN SMALL LETTER V WITH DOT BELOW" },
    { 0x1E80, "LATIN CAPITAL LETTER W WITH GRAVE" },
    { 0x1E81, "LATIN SMALL LETTER W WITH GRAVE" },
    { 0x1E82, "LATIN CAPITAL LETTER W WITH ACUTE" },
    { 0x1E83, "LATIN SMALL LETTER W WITH ACUTE" },
    { 0x1E84, "LATIN CAPITAL LETTER W WITH DIAERESIS" },
    { 0x1E85, "LATIN SMALL LETTER W WITH DIAERESIS" },
    { 0x1E86, "LATIN CAPITAL LETTER W WITH DOT ABOVE" },
    { 0x1E87, "LATIN SMALL LETTER W WITH DOT ABOVE" },
    { 0x1E88, "LATIN CAPITAL LETTER W WITH DOT BELOW" },
    { 0x1E89, "LATIN SMALL LETTER W WITH DOT BELOW" },
    { 0x1E8A, "LATIN CAPITAL LETTER X WITH DOT ABOVE" },
    { 0x1E8B, "LATIN SMALL LETTER X WITH DOT ABOVE" },
    { 0x1E8C, "LATIN CAPITAL LETTER X WITH DIAERESIS" },
    { 0x1E8D, "LATIN SMALL LETTER X WITH DIAERESIS" },
    { 0x1E8E, "LATIN CAPITAL LETTER Y WITH DOT ABOVE" },
    { 0x1E8F, "LATIN SMALL LETTER Y WITH DOT ABOVE" },
    { 0x1E90, "LATIN CAPITAL LETTER Z WITH CIRCUMFLEX" },
    { 0x1E91, "LATIN SMALL LETTER Z WITH CIRCUMFLEX" },
    { 0x1E92, "LATIN CAPITAL LETTER Z WITH DOT BELOW" },
    { 0x1E93, "LATIN SMALL LETTER Z WITH DOT BELOW" },
    { 0x1E94, "LATIN CAPITAL LETTER Z WITH LINE BELOW" },
    { 0x1E95, "LATIN SMALL LETTER Z WITH LINE BELOW" },
    { 0x1E96, "LATIN SMALL LETTER H WITH LINE BELOW" },
    { 0x1E97, "LATIN SMALL LETTER T WITH DIAERESIS" },
    { 0x1E98, "LATIN SMALL LETTER W WITH RING ABOVE" },
    { 0x1E99, "LATIN SMALL LETTER Y WITH RING ABOVE" },
    { 0x1E9A, "LATIN SMALL LETTER A WITH RIGHT HALF RING" },
    { 0x1E9B, "LATIN SMALL LETTER LONG S WITH DOT ABOVE" },
    { 0x1E9C, "LATIN SMALL LETTER LONG S WITH DIAGONAL STROKE" },
    { 0x1E9D, "LATIN SMALL LETTER LONG S WITH HIGH STROKE" },
    { 0x1E9E, "LATIN CAPITAL LETTER SHARP S" },
    { 0x1E9F, "LATIN SMALL LETTER DELTA" },
    { 0x1EA0, "LATIN CAPITAL LETTER A WITH DOT BELOW" },
    { 0x1EA1, "LATIN SMALL LETTER A WITH DOT BELOW" },
    { 0x1EA2, "LATIN CAPITAL LETTER A WITH HOOK ABOVE" },
    { 0x1EA3, "LATIN SMALL LETTER A WITH HOOK ABOVE" },
    { 0x1EA4, "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE" },
    { 0x1EA5, "LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE" },
    { 0x1EA6, "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE" },
    { 0x1EA7, "LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE" },
    { 0x1EA8, "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE" },
    { 0x1EA9, "LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE" },
    { 0x1EAA, "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE" },
    { 0x1EAB, "LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE" },
    { 0x1EAC, "LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW" },
    { 0x1EAD, "LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW" },
    { 0x1EAE, "LATIN CAPITAL LETTER A WITH BREVE AND ACUTE" },
    { 0x1EAF, "LATIN SMALL LETTER A WITH BREVE AND ACUTE" },
    { 0x1EB0, "LATIN CAPITAL LETTER A WITH BREVE AND GRAVE" },
    { 0x1EB1, "LATIN SMALL LETTER A WITH BREVE AND GRAVE" },
    { 0x1EB2, "LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE" },
    { 0x1EB3, "LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE" },
    { 0x1EB4, "LATIN CAPITAL LETTER A WITH BREVE AND TILDE" },
    { 0x1EB5, "LATIN SMALL LETTER A WITH BREVE AND TILDE" },
    { 0x1EB6, "LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW" },
    { 0x1EB7, "LATIN SMALL LETTER A WITH BREVE AND DOT BELOW" },
    { 0x1EB8, "LATIN CAPITAL LETTER E WITH DOT BELOW" },
    { 0x1EB9, "LATIN SMALL LETTER E WITH DOT BELOW" },
    { 0x1EBA, "LATIN CAPITAL LETTER E WITH HOOK ABOVE" },
    { 0x1EBB, "LATIN SMALL LETTER E WITH HOOK ABOVE" },
    { 0x1EBC, "LATIN CAPITAL LETTER E WITH TILDE" },
    { 0x1EBD, "LATIN SMALL LETTER E WITH TILDE" },
    { 0x1EBE, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE" },
    { 0x1EBF, "LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE" },
    { 0x1EC0, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE" },
    { 0x1EC1, "LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE" },
    { 0x1EC2, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE" },
    { 0x1EC3, "LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE" },
    { 0x1EC4, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE" },
    { 0x1EC5, "LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE" },
    { 0x1EC6, "LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW" },
    { 0x1EC7, "LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW" },
    { 0x1EC8, "LATIN CAPITAL LETTER I WITH HOOK ABOVE" },
    { 0x1EC9, "LATIN SMALL LETTER I WITH HOOK ABOVE" },
    { 0x1ECA, "LATIN CAPITAL LETTER I WITH DOT BELOW" },
    { 0x1ECB, "LATIN SMALL LETTER I WITH DOT BELOW" },
    { 0x1ECC, "LATIN CAPITAL LETTER O WITH DOT BELOW" },
    { 0x1ECD, "LATIN SMALL LETTER O WITH DOT BELOW" },
    { 0x1ECE, "LATIN CAPITAL LETTER O WITH HOOK ABOVE" },
    { 0x1ECF, "LATIN SMALL LETTER O WITH HOOK ABOVE" },
    { 0x1ED0, "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE" },
    { 0x1ED1, "LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE" },
    { 0x1ED2, "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE" },
    { 0x1ED3, "LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE" },
    { 0x1ED4, "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE" },
    { 0x1ED5, "LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE" },
    { 0x1ED6, "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE" },
    { 0x1ED7, "LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE" },
    { 0x1ED8, "LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW" },
    { 0x1ED9, "LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW" },
    { 0x1EDA, "LATIN CAPITAL LETTER O WITH HORN AND ACUTE" },
    { 0x1EDB, "LATIN SMALL LETTER O WITH HORN AND ACUTE" },
    { 0x1EDC, "LATIN CAPITAL LETTER O WITH HORN AND GRAVE" },
    { 0x1EDD, "LATIN SMALL LETTER O WITH HORN AND GRAVE" },
    { 0x1EDE, "LATIN CAPITAL LETTER O WITH HORN AND HOOK ABOVE" },
    { 0x1EDF, "LATIN SMALL LETTER O WITH HORN AND HOOK ABOVE" },
    { 0x1EE0, "LATIN CAPITAL LETTER O WITH HORN AND TILDE" },
    { 0x1EE1, "LATIN SMALL LETTER O WITH HORN AND TILDE" },
    { 0x1EE2, "LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW" },
    { 0x1EE3, "LATIN SMALL LETTER O WITH HORN AND DOT BELOW" },
    { 0x1EE4, "LATIN CAPITAL LETTER U WITH DOT BELOW" },
    { 0x1EE5, "LATIN SMALL LETTER U WITH DOT BELOW" },
    { 0x1EE6, "LATIN CAPITAL LETTER U WITH HOOK ABOVE" },
    { 0x1EE7, "LATIN SMALL LETTER U WITH HOOK ABOVE" },
    { 0x1EE8, "LATIN CAPITAL LETTER U WITH HORN AND ACUTE" },
    { 0x1EE9, "LATIN SMALL LETTER U WITH HORN AND ACUTE" },
    { 0x1EEA, "LATIN CAPITAL LETTER U WITH HORN AND GRAVE" },
    { 0x1EEB, "LATIN SMALL LETTER U WITH HORN AND GRAVE" },
    { 0x1EEC, "LATIN CAPITAL LETTER U WITH HORN AND HOOK ABOVE" },
    { 0x1EED, "LATIN SMALL LETTER U WITH HORN AND HOOK ABOVE" },
    { 0x1EEE, "LATIN CAPITAL LETTER U WITH HORN AND TILDE" },
    { 0x1EEF, "LATIN SMALL LETTER U WITH HORN AND TILDE" },
    { 0x1EF0, "LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW" },
    { 0x1EF1, "LATIN SMALL LETTER U WITH HORN AND DOT BELOW" },
    { 0x1EF2, "LATIN CAPITAL LETTER Y WITH GRAVE" },
    { 0x1EF3, "LATIN SMALL LETTER Y WITH GRAVE" },
    { 0x1EF4, "LATIN CAPITAL LETTER Y WITH DOT BELOW" },
    { 0x1EF5, "LATIN SMALL LETTER Y WITH DOT BELOW" },
    { 0x1EF6, "LATIN CAPITAL LETTER Y WITH HOOK ABOVE" },
    { 0x1EF7, "LATIN SMALL LETTER Y WITH HOOK ABOVE" },
    { 0x1EF8, "LATIN CAPITAL LETTER Y WITH TILDE" },
    { 0x1EF9, "LATIN SMALL LETTER Y WITH TILDE" },
    { 0x1EFA, "LATIN CAPITAL LETTER MIDDLE-WELSH LL" },
    { 0x1EFB, "LATIN SMALL LETTER MIDDLE-WELSH LL" },
    { 0x1EFC, "LATIN CAPITAL LETTER MIDDLE-WELSH V" },
    { 0x1EFD, "LATIN SMALL LETTER MIDDLE-WELSH V" },
    { 0x1EFE, "LATIN CAPITAL LETTER Y WITH LOOP" },
    { 0x1EFF, "LATIN SMALL LETTER Y WITH LOOP" },
    { 0x1F00, "GREEK SMALL LETTER ALPHA WITH PSILI" },
    { 0x1F01, "GREEK SMALL LETTER ALPHA WITH DASIA" },
    { 0x1F02, "GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA" },
    { 0x1F03, "GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA" },
    { 0x1F04, "GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA" },
    { 0x1F05, "GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA" },
    { 0x1F06, "GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI" },
    { 0x1F07, "GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI" },
    { 0x1F08, "GREEK CAPITAL LETTER ALPHA WITH PSILI" },
    { 0x1F09, "GREEK CAPITAL LETTER ALPHA WITH DASIA" },
    { 0x1F0A, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA" },
    { 0x1F0B, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA" },
    { 0x1F0C, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA" },
    { 0x1F0D, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA" },
    { 0x1F0E, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI" },
    { 0x1F0F, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI" },
    { 0x1F10, "GREEK SMALL LETTER EPSILON WITH PSILI" },
    { 0x1F11, "GREEK SMALL LETTER EPSILON WITH DASIA" },
    { 0x1F12, "GREEK SMALL LETTER EPSILON WITH PSILI AND VARIA" },
    { 0x1F13, "GREEK SMALL LETTER EPSILON WITH DASIA AND VARIA" },
    { 0x1F14, "GREEK SMALL LETTER EPSILON WITH PSILI AND OXIA" },
    { 0x1F15, "GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA" },
    { 0x1F18, "GREEK CAPITAL LETTER EPSILON WITH PSILI" },
    { 0x1F19, "GREEK CAPITAL LETTER EPSILON WITH DASIA" },
    { 0x1F1A, "GREEK CAPITAL LETTER EPSILON WITH PSILI AND VARIA" },
    { 0x1F1B, "GREEK CAPITAL LETTER EPSILON WITH DASIA AND VARIA" },
    { 0x1F1C, "GREEK CAPITAL LETTER EPSILON WITH PSILI AND OXIA" },
    { 0x1F1D, "GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA" },
    { 0x1F20, "GREEK SMALL LETTER ETA WITH PSILI" },
    { 0x1F21, "GREEK SMALL LETTER ETA WITH DASIA" },
    { 0x1F22, "GREEK SMALL LETTER ETA WITH PSILI AND VARIA" },
    { 0x1F23, "GREEK SMALL LETTER ETA WITH DASIA AND VARIA" },
    { 0x1F24, "GREEK SMALL LETTER ETA WITH PSILI AND OXIA" },
    { 0x1F25, "GREEK SMALL LETTER ETA WITH DASIA AND OXIA" },
    { 0x1F26, "GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI" },
    { 0x1F27, "GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI" },
    { 0x1F28, "GREEK CAPITAL LETTER ETA WITH PSILI" },
    { 0x1F29, "GREEK CAPITAL LETTER ETA WITH DASIA" },
    { 0x1F2A, "GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA" },
    { 0x1F2B, "GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA" },
    { 0x1F2C, "GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA" },
    { 0x1F2D, "GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA" },
    { 0x1F2E, "GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI" },
    { 0x1F2F, "GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI" },
    { 0x1F30, "GREEK SMALL LETTER IOTA WITH PSILI" },
    { 0x1F31, "GREEK SMALL LETTER IOTA WITH DASIA" },
    { 0x1F32, "GREEK SMALL LETTER IOTA WITH PSILI AND VARIA" },
    { 0x1F33, "GREEK SMALL LETTER IOTA WITH DASIA AND VARIA" },
    { 0x1F34, "GREEK SMALL LETTER IOTA WITH PSILI AND OXIA" },
    { 0x1F35, "GREEK SMALL LETTER IOTA WITH DASIA AND OXIA" },
    { 0x1F36, "GREEK SMALL LETTER IOTA WITH PSILI AND PERISPOMENI" },
    { 0x1F37, "GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI" },
    { 0x1F38, "GREEK CAPITAL LETTER IOTA WITH PSILI" },
    { 0x1F39, "GREEK CAPITAL LETTER IOTA WITH DASIA" },
    { 0x1F3A, "GREEK CAPITAL LETTER IOTA WITH PSILI AND VARIA" },
    { 0x1F3B, "GREEK CAPITAL LETTER IOTA WITH DASIA AND VARIA" },
    { 0x1F3C, "GREEK CAPITAL LETTER IOTA WITH PSILI AND OXIA" },
    { 0x1F3D, "GREEK CAPITAL LETTER IOTA WITH DASIA AND OXIA" },
    { 0x1F3E, "GREEK CAPITAL LETTER IOTA WITH PSILI AND PERISPOMENI" },
    { 0x1F3F, "GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI" },
    { 0x1F40, "GREEK SMALL LETTER OMICRON WITH PSILI" },
    { 0x1F41, "GREEK SMALL LETTER OMICRON WITH DASIA" },
    { 0x1F42, "GREEK SMALL LETTER OMICRON WITH PSILI AND VARIA" },
    { 0x1F43, "GREEK SMALL LETTER OMICRON WITH DASIA AND VARIA" },
    { 0x1F44, "GREEK SMALL LETTER OMICRON WITH PSILI AND OXIA" },
    { 0x1F45, "GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA" },
    { 0x1F48, "GREEK CAPITAL LETTER OMICRON WITH PSILI" },
    { 0x1F49, "GREEK CAPITAL LETTER OMICRON WITH DASIA" },
    { 0x1F4A, "GREEK CAPITAL LETTER OMICRON WITH PSILI AND VARIA" },
    { 0x1F4B, "GREEK CAPITAL LETTER OMICRON WITH DASIA AND VARIA" },
    { 0x1F4C, "GREEK CAPITAL LETTER OMICRON WITH PSILI AND OXIA" },
    { 0x1F4D, "GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA" },
    { 0x1F50, "GREEK SMALL LETTER UPSILON WITH PSILI" },
    { 0x1F51, "GREEK SMALL LETTER UPSILON WITH DASIA" },
    { 0x1F52, "GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA" },
    { 0x1F53, "GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA" },
    { 0x1F54, "GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA" },
    { 0x1F55, "GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA" },
    { 0x1F56, "GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI" },
    { 0x1F57, "GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI" },
    { 0x1F59, "GREEK CAPITAL LETTER UPSILON WITH DASIA" },
    { 0x1F5B, "GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA" },
    { 0x1F5D, "GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA" },
    { 0x1F5F, "GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI" },
    { 0x1F60, "GREEK SMALL LETTER OMEGA WITH PSILI" },
    { 0x1F61, "GREEK SMALL LETTER OMEGA WITH DASIA" },
    { 0x1F62, "GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA" },
    { 0x1F63, "GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA" },
    { 0x1F64, "GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA" },
    { 0x1F65, "GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA" },
    { 0x1F66, "GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI" },
    { 0x1F67, "GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI" },
    { 0x1F68, "GREEK CAPITAL LETTER OMEGA WITH PSILI" },
    { 0x1F69, "GREEK CAPITAL LETTER OMEGA WITH DASIA" },
    { 0x1F6A, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA" },
    { 0x1F6B, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA" },
    { 0x1F6C, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA" },
    { 0x1F6D, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA" },
    { 0x1F6E, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI" },
    { 0x1F6F, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI" },
    { 0x1F70, "GREEK SMALL LETTER ALPHA WITH VARIA" },
    { 0x1F71, "GREEK SMALL LETTER ALPHA WITH OXIA" },
    { 0x1F72, "GREEK SMALL LETTER EPSILON WITH VARIA" },
    { 0x1F73, "GREEK SMALL LETTER EPSILON WITH OXIA" },
    { 0x1F74, "GREEK SMALL LETTER ETA WITH VARIA" },
    { 0x1F75, "GREEK SMALL LETTER ETA WITH OXIA" },
    { 0x1F76, "GREEK SMALL LETTER IOTA WITH VARIA" },
    { 0x1F77, "GREEK SMALL LETTER IOTA WITH OXIA" },
    { 0x1F78, "GREEK SMALL LETTER OMICRON WITH VARIA" },
    { 0x1F79, "GREEK SMALL LETTER OMICRON WITH OXIA" },
    { 0x1F7A, "GREEK SMALL LETTER UPSILON WITH VARIA" },
    { 0x1F7B, "GREEK SMALL LETTER UPSILON WITH OXIA" },
    { 0x1F7C, "GREEK SMALL LETTER OMEGA WITH VARIA" },
    { 0x1F7D, "GREEK SMALL LETTER OMEGA WITH OXIA" },
    { 0x1F80, "GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI" },
    { 0x1F81, "GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI" },
    { 0x1F82, "GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA AND YPOGEGRAMMENI" },
    { 0x1F83, "GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA AND YPOGEGRAMMENI" },
    { 0x1F84, "GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA AND YPOGEGRAMMENI" },
    { 0x1F85, "GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA AND YPOGEGRAMMENI" },
    { 0x1F86, "GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1F87, "GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1F88, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI" },
    { 0x1F89, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI" },
    { 0x1F8A, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI" },
    { 0x1F8B, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI" },
    { 0x1F8C, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI" },
    { 0x1F8D, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI" },
    { 0x1F8E, "GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI" },
    { 0x1F8F, "GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI" },
    { 0x1F90, "GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI" },
    { 0x1F91, "GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI" },
    { 0x1F92, "GREEK SMALL LETTER ETA WITH PSILI AND VARIA AND YPOGEGRAMMENI" },
    { 0x1F93, "GREEK SMALL LETTER ETA WITH DASIA AND VARIA AND YPOGEGRAMMENI" },
    { 0x1F94, "GREEK SMALL LETTER ETA WITH PSILI AND OXIA AND YPOGEGRAMMENI" },
    { 0x1F95, "GREEK SMALL LETTER ETA WITH DASIA AND OXIA AND YPOGEGRAMMENI" },
    { 0x1F96, "GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1F97, "GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1F98, "GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI" },
    { 0x1F99, "GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI" },
    { 0x1F9A, "GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI" },
    { 0x1F9B, "GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI" },
    { 0x1F9C, "GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI" },
    { 0x1F9D, "GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI" },
    { 0x1F9E, "GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI" },
    { 0x1F9F, "GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI" },
    { 0x1FA0, "GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI" },
    { 0x1FA1, "GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI" },
    { 0x1FA2, "GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA AND YPOGEGRAMMENI" },
    { 0x1FA3, "GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA AND YPOGEGRAMMENI" },
    { 0x1FA4, "GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA AND YPOGEGRAMMENI" },
    { 0x1FA5, "GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA AND YPOGEGRAMMENI" },
    { 0x1FA6, "GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1FA7, "GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1FA8, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI" },
    { 0x1FA9, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI" },
    { 0x1FAA, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI" },
    { 0x1FAB, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI" },
    { 0x1FAC, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI" },
    { 0x1FAD, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI" },
    { 0x1FAE, "GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI" },
    { 0x1FAF, "GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI" },
    { 0x1FB0, "GREEK SMALL LETTER ALPHA WITH VRACHY" },
    { 0x1FB1, "GREEK SMALL LETTER ALPHA WITH MACRON" },
    { 0x1FB2, "GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI" },
    { 0x1FB3, "GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI" },
    { 0x1FB4, "GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI" },
    { 0x1FB6, "GREEK SMALL LETTER ALPHA WITH PERISPOMENI" },
    { 0x1FB7, "GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1FB8, "GREEK CAPITAL LETTER ALPHA WITH VRACHY" },
    { 0x1FB9, "GREEK CAPITAL LETTER ALPHA WITH MACRON" },
    { 0x1FBA, "GREEK CAPITAL LETTER ALPHA WITH VARIA" },
    { 0x1FBB, "GREEK CAPITAL LETTER ALPHA WITH OXIA" },
    { 0x1FBC, "GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI" },
    { 0x1FBD, "GREEK KORONIS" },
    { 0x1FBE, "GREEK PROSGEGRAMMENI" },
    { 0x1FBF, "GREEK PSILI" },
    { 0x1FC0, "GREEK PERISPOMENI" },
    { 0x1FC1, "GREEK DIALYTIKA AND PERISPOMENI" },
    { 0x1FC2, "GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI" },
    { 0x1FC3, "GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI" },
    { 0x1FC4, "GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI" },
    { 0x1FC6, "GREEK SMALL LETTER ETA WITH PERISPOMENI" },
    { 0x1FC7, "GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1FC8, "GREEK CAPITAL LETTER EPSILON WITH VARIA" },
    { 0x1FC9, "GREEK CAPITAL LETTER EPSILON WITH OXIA" },
    { 0x1FCA, "GREEK CAPITAL LETTER ETA WITH VARIA" },
    { 0x1FCB, "GREEK CAPITAL LETTER ETA WITH OXIA" },
    { 0x1FCC, "GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI" },
    { 0x1FCD, "GREEK PSILI AND VARIA" },
    { 0x1FCE, "GREEK PSILI AND OXIA" },
    { 0x1FCF, "GREEK PSILI AND PERISPOMENI" },
    { 0x1FD0, "GREEK SMALL LETTER IOTA WITH VRACHY" },
    { 0x1FD1, "GREEK SMALL LETTER IOTA WITH MACRON" },
    { 0x1FD2, "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA" },
    { 0x1FD3, "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA" },
    { 0x1FD6, "GREEK SMALL LETTER IOTA WITH PERISPOMENI" },
    { 0x1FD7, "GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI" },
    { 0x1FD8, "GREEK CAPITAL LETTER IOTA WITH VRACHY" },
    { 0x1FD9, "GREEK CAPITAL LETTER IOTA WITH MACRON" },
    { 0x1FDA, "GREEK CAPITAL LETTER IOTA WITH VARIA" },
    { 0x1FDB, "GREEK CAPITAL LETTER IOTA WITH OXIA" },
    { 0x1FDD, "GREEK DASIA AND VARIA" },
    { 0x1FDE, "GREEK DASIA AND OXIA" },
    { 0x1FDF, "GREEK DASIA AND PERISPOMENI" },
    { 0x1FE0, "GREEK SMALL LETTER UPSILON WITH VRACHY" },
    { 0x1FE1, "GREEK SMALL LETTER UPSILON WITH MACRON" },
    { 0x1FE2, "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA" },
    { 0x1FE3, "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA" },
    { 0x1FE4, "GREEK SMALL LETTER RHO WITH PSILI" },
    { 0x1FE5, "GREEK SMALL LETTER RHO WITH DASIA" },
    { 0x1FE6, "GREEK SMALL LETTER UPSILON WITH PERISPOMENI" },
    { 0x1FE7, "GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI" },
    { 0x1FE8, "GREEK CAPITAL LETTER UPSILON WITH VRACHY" },
    { 0x1FE9, "GREEK CAPITAL LETTER UPSILON WITH MACRON" },
    { 0x1FEA, "GREEK CAPITAL LETTER UPSILON WITH VARIA" },
    { 0x1FEB, "GREEK CAPITAL LETTER UPSILON WITH OXIA" },
    { 0x1FEC, "GREEK CAPITAL LETTER RHO WITH DASIA" },
    { 0x1FED, "GREEK DIALYTIKA AND VARIA" },
    { 0x1FEE, "GREEK DIALYTIKA AND OXIA" },
    { 0x1FEF, "GREEK VARIA" },
    { 0x1FF2, "GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI" },
    { 0x1FF3, "GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI" },
    { 0x1FF4, "GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI" },
    { 0x1FF6, "GREEK SMALL LETTER OMEGA WITH PERISPOMENI" },
    { 0x1FF7, "GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI" },
    { 0x1FF8, "GREEK CAPITAL LETTER OMICRON WITH VARIA" },
    { 0x1FF9, "GREEK CAPITAL LETTER OMICRON WITH OXIA" },
    { 0x1FFA, "GREEK CAPITAL LETTER OMEGA WITH VARIA" },
    { 0x1FFB, "GREEK CAPITAL LETTER OMEGA WITH OXIA" },
    { 0x1FFC, "GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI" },
    { 0x1FFD, "GREEK OXIA" },
    { 0x1FFE, "GREEK DASIA" },
    { 0x2000, "EN QUAD" },
    { 0x2001, "EM QUAD" },
    { 0x2002, "EN SPACE" },
    { 0x2003, "EM SPACE" },
    { 0x2004, "THREE-PER-EM SPACE" },
    { 0x2005, "FOUR-PER-EM SPACE" },
    { 0x2006, "SIX-PER-EM SPACE" },
    { 0x2007, "FIGURE SPACE" },
    { 0x2008, "PUNCTUATION SPACE" },
    { 0x2009, "THIN SPACE" },
    { 0x200A, "HAIR SPACE" },
    { 0x200B, "ZERO WIDTH SPACE" },
    { 0x200C, "ZERO WIDTH NON-JOINER" },
    { 0x200D, "ZERO WIDTH JOINER" },
    { 0x200E, "LEFT-TO-RIGHT MARK" },
    { 0x200F, "RIGHT-TO-LEFT MARK" },
    { 0x2010, "HYPHEN" },
    { 0x2011, "NON-BREAKING HYPHEN" },
    { 0x2012, "FIGURE DASH" },
    { 0x2013, "EN DASH" },
    { 0x2014, "EM DASH" },
    { 0x2015, "HORIZONTAL BAR" },
    { 0x2016, "DOUBLE VERTICAL LINE" },
    { 0x2017, "DOUBLE LOW LINE" },
    { 0x2018, "LEFT SINGLE QUOTATION MARK" },
    { 0x2019, "RIGHT SINGLE QUOTATION MARK" },
    { 0x201A, "SINGLE LOW-9 QUOTATION MARK" },
    { 0x201B, "SINGLE HIGH-REVERSED-9 QUOTATION MARK" },
    { 0x201C, "LEFT DOUBLE QUOTATION MARK" },
    { 0x201D, "RIGHT DOUBLE QUOTATION MARK" },
    { 0x201E, "DOUBLE LOW-9 QUOTATION MARK" },
    { 0x201F, "DOUBLE HIGH-REVERSED-9 QUOTATION MARK" },
    { 0x2020, "DAGGER" },
    { 0x2021, "DOUBLE DAGGER" },
    { 0x2022, "BULLET" },
    { 0x2023, "TRIANGULAR BULLET" },
    { 0x2024, "ONE DOT LEADER" },
    { 0x2025, "TWO DOT LEADER" },
    { 0x2026, "HORIZONTAL ELLIPSIS" },
    { 0x2027, "HYPHENATION POINT" },
    { 0x2028, "LINE SEPARATOR" },
    { 0x2029, "PARAGRAPH SEPARATOR" },
    { 0x202A, "LEFT-TO-RIGHT EMBEDDING" },
    { 0x202B, "RIGHT-TO-LEFT EMBEDDING" },
    { 0x202C, "POP DIRECTIONAL FORMATTING" },
    { 0x202D, "LEFT-TO-RIGHT OVERRIDE" },
    { 0x202E, "RIGHT-TO-LEFT OVERRIDE" },
    { 0x202F, "NARROW NO-BREAK SPACE" },
    { 0x2030, "PER MILLE SIGN" },
    { 0x2031, "PER TEN THOUSAND SIGN" },
    { 0x2032, "PRIME" },
    { 0x2033, "DOUBLE PRIME" },
    { 0x2034, "TRIPLE PRIME" },
    { 0x2035, "REVERSED PRIME" },
    { 0x2036, "REVERSED DOUBLE PRIME" },
    { 0x2037, "REVERSED TRIPLE PRIME" },
    { 0x2038, "CARET" },
    { 0x2039, "SINGLE LEFT-POINTING ANGLE QUOTATION MARK" },
    { 0x203A, "SINGLE RIGHT-POINTING ANGLE QUOTATION MARK" },
    { 0x203B, "REFERENCE MARK" },
    { 0x203C, "DOUBLE EXCLAMATION MARK" },
    { 0x203D, "INTERROBANG" },
    { 0x203E, "OVERLINE" },
    { 0x203F, "UNDERTIE" },
    { 0x2040, "CHARACTER TIE" },
    { 0x2041, "CARET INSERTION POINT" },
    { 0x2042, "ASTERISM" },
    { 0x2043, "HYPHEN BULLET" },
    { 0x2044, "FRACTION SLASH" },
    { 0x2045, "LEFT SQUARE BRACKET WITH QUILL" },
    { 0x2046, "RIGHT SQUARE BRACKET WITH QUILL" },
    { 0x2047, "DOUBLE QUESTION MARK" },
    { 0x2048, "QUESTION EXCLAMATION MARK" },
    { 0x2049, "EXCLAMATION QUESTION MARK" },
    { 0x204A, "TIRONIAN SIGN ET" },
    { 0x204B, "REVERSED PILCROW SIGN" },
    { 0x204C, "BLACK LEFTWARDS BULLET" },
    { 0x204D, "BLACK RIGHTWARDS BULLET" },
    { 0x204E, "LOW ASTERISK" },
    { 0x204F, "REVERSED SEMICOLON" },
    { 0x2050, "CLOSE UP" },
    { 0x2051, "TWO ASTERISKS ALIGNED VERTICALLY" },
    { 0x2052, "COMMERCIAL MINUS SIGN" },
    { 0x2053, "SWUNG DASH" },
    { 0x2054, "INVERTED UNDERTIE" },
    { 0x2055, "FLOWER PUNCTUATION MARK" },
    { 0x2056, "THREE DOT PUNCTUATION" },
    { 0x2057, "QUADRUPLE PRIME" },
    { 0x2058, "FOUR DOT PUNCTUATION" },
    { 0x2059, "FIVE DOT PUNCTUATION" },
    { 0x205A, "TWO DOT PUNCTUATION" },
    { 0x205B, "FOUR DOT MARK" },
    { 0x205C, "DOTTED CROSS" },
    { 0x205D, "TRICOLON" },
    { 0x205E, "VERTICAL FOUR DOTS" },
    { 0x205F, "MEDIUM MATHEMATICAL SPACE" },
    { 0x2060, "WORD JOINER" },
    { 0x2061, "FUNCTION APPLICATION" },
    { 0x2062, "INVISIBLE TIMES" },
    { 0x2063, "INVISIBLE SEPARATOR" },
    { 0x2064, "INVISIBLE PLUS" },
    { 0x2066, "LEFT-TO-RIGHT ISOLATE" },
    { 0x2067, "RIGHT-TO-LEFT ISOLATE" },
    { 0x2068, "FIRST STRONG ISOLATE" },
    { 0x2069, "POP DIRECTIONAL ISOLATE" },
    { 0x206A, "INHIBIT SYMMETRIC SWAPPING" },
    { 0x206B, "ACTIVATE SYMMETRIC SWAPPING" },
    { 0x206C, "INHIBIT ARABIC FORM SHAPING" },
    { 0x206D, "ACTIVATE ARABIC FORM SHAPING" },
    { 0x206E, "NATIONAL DIGIT SHAPES" },
    { 0x206F, "NOMINAL DIGIT SHAPES" },
    { 0x2070, "SUPERSCRIPT ZERO" },
    { 0x2071, "SUPERSCRIPT LATIN SMALL LETTER I" },
    { 0x2074, "SUPERSCRIPT FOUR" },
    { 0x2075, "SUPERSCRIPT FIVE" },
    { 0x2076, "SUPERSCRIPT SIX" },
    { 0x2077, "SUPERSCRIPT SEVEN" },
    { 0x2078, "SUPERSCRIPT EIGHT" },
    { 0x2079, "SUPERSCRIPT NINE" },
    { 0x207A, "SUPERSCRIPT PLUS SIGN" },
    { 0x207B, "SUPERSCRIPT MINUS" },
    { 0x207C, "SUPERSCRIPT EQUALS SIGN" },
    { 0x207D, "SUPERSCRIPT LEFT PARENTHESIS" },
    { 0x207E, "SUPERSCRIPT RIGHT PARENTHESIS" },
    { 0x207F, "SUPERSCRIPT LATIN SMALL LETTER N" },
    { 0x2080, "SUBSCRIPT ZERO" },
    { 0x2081, "SUBSCRIPT ONE" },
    { 0x2082, "SUBSCRIPT TWO" },
    { 0x2083, "SUBSCRIPT THREE" },
    { 0x2084, "SUBSCRIPT FOUR" },
    { 0x2085, "SUBSCRIPT FIVE" },
    { 0x2086, "SUBSCRIPT SIX" },
    { 0x2087, "SUBSCRIPT SEVEN" },
    { 0x2088, "SUBSCRIPT EIGHT" },
    { 0x2089, "SUBSCRIPT NINE" },
    { 0x208A, "SUBSCRIPT PLUS SIGN" },
    { 0x208B, "SUBSCRIPT MINUS" },
    { 0x208C, "SUBSCRIPT EQUALS SIGN" },
    { 0x208D, "SUBSCRIPT LEFT PARENTHESIS" },
    { 0x208E, "SUBSCRIPT RIGHT PARENTHESIS" },
    { 0x2090, "LATIN SUBSCRIPT SMALL LETTER A" },
    { 0x2091, "LATIN SUBSCRIPT SMALL LETTER E" },
    { 0x2092, "LATIN SUBSCRIPT SMALL LETTER O" },
    { 0x2093, "LATIN SUBSCRIPT SMALL LETTER X" },
    { 0x2094, "LATIN SUBSCRIPT SMALL LETTER SCHWA" },
    { 0x2095, "LATIN SUBSCRIPT SMALL LETTER H" },
    { 0x2096, "LATIN SUBSCRIPT SMALL LETTER K" },
    { 0x2097, "LATIN SUBSCRIPT SMALL LETTER L" },
    { 0x2098, "LATIN SUBSCRIPT SMALL LETTER M" },
    { 0x2099, "LATIN SUBSCRIPT SMALL LETTER N" },
    { 0x209A, "LATIN SUBSCRIPT SMALL LETTER P" },
    { 0x209B, "LATIN SUBSCRIPT SMALL LETTER S" },
    { 0x209C, "LATIN SUBSCRIPT SMALL LETTER T" },
    { 0x20A0, "EURO-CURRENCY SIGN" },
    { 0x20A1, "COLON SIGN" },
    { 0x20A2, "CRUZEIRO SIGN" },
    { 0x20A3, "FRENCH FRANC SIGN" },
    { 0x20A4, "LIRA SIGN" },
    { 0x20A5, "MILL SIGN" },
    { 0x20A6, "NAIRA SIGN" },
    { 0x20A7, "PESETA SIGN" },
    { 0x20A8, "RUPEE SIGN" },
    { 0x20A9, "WON SIGN" },
    { 0x20AA, "NEW SHEQEL SIGN" },
    { 0x20AB, "DONG SIGN" },
    { 0x20AC, "EURO SIGN" },
    { 0x20AD, "KIP SIGN" },
    { 0x20AE, "TUGRIK SIGN" },
    { 0x20AF, "DRACHMA SIGN" },
    { 0x20B0, "GERMAN PENNY SIGN" },
    { 0x20B1, "PESO SIGN" },
    { 0x20B2, "GUARANI SIGN" },
    { 0x20B3, "AUSTRAL SIGN" },
    { 0x20B4, "HRYVNIA SIGN" },
    { 0x20B5, "CEDI SIGN" },
    { 0x20B6, "LIVRE TOURNOIS SIGN" },
    { 0x20B7, "SPESMILO SIGN" },
    { 0x20B8, "TENGE SIGN" },
    { 0x20B9, "INDIAN RUPEE SIGN" },
    { 0x20BA, "TURKISH LIRA SIGN" },
    { 0x20BB, "NORDIC MARK SIGN" },
    { 0x20BC, "MANAT SIGN" },
    { 0x20BD, "RUBLE SIGN" },
    { 0x20BE, "LARI SIGN" },
    { 0x20BF, "BITCOIN SIGN" },
    { 0x20C0, "SOM SIGN" },
    { 0x20D0, "COMBINING LEFT HARPOON ABOVE" },
    { 0x20D1, "COMBINING RIGHT HARPOON ABOVE" },
    { 0x20D2, "COMBINING LONG VERTICAL LINE OVERLAY" },
    { 0x20D3, "COMBINING SHORT VERTICAL LINE OVERLAY" },
    { 0x20D4, "COMBINING ANTICLOCKWISE ARROW ABOVE" },
    { 0x20D5, "COMBINING CLOCKWISE ARROW ABOVE" },
    { 0x20D6, "COMBINING LEFT ARROW ABOVE" },
    { 0x20D7, "COMBINING RIGHT ARROW ABOVE" },
    { 0x20D8, "COMBINING RING OVERLAY" },
    { 0x20D9, "COMBINING CLOCKWISE RING OVERLAY" },
    { 0x20DA, "COMBINING ANTICLOCKWISE RING OVERLAY" },
    { 0x20DB, "COMBINING THREE DOTS ABOVE" },
    { 0x20DC, "COMBINING FOUR DOTS ABOVE" },
    { 0x20DD, "COMBINING ENCLOSING CIRCLE" },
    { 0x20DE, "COMBINING ENCLOSING SQUARE" },
    { 0x20DF, "COMBINING ENCLOSING DIAMOND" },
    { 0x20E0, "COMBINING ENCLOSING CIRCLE BACKSLASH" },
    { 0x20E1, "COMBINING LEFT RIGHT ARROW ABOVE" },
    { 0x20E2, "COMBINING ENCLOSING SCREEN" },
    { 0x20E3, "COMBINING ENCLOSING KEYCAP" },
    { 0x20E4, "COMBINING ENCLOSING UPWARD POINTING TRIANGLE" },
    { 0x20E5, "COMBINING REVERSE SOLIDUS OVERLAY" },
    { 0x20E6, "COMBINING DOUBLE VERTICAL STROKE OVERLAY" },
    { 0x20E7, "COMBINING ANNUITY SYMBOL" },
    { 0x20E8, "COMBINING TRIPLE UNDERDOT" },
    { 0x20E9, "COMBINING WIDE BRIDGE ABOVE" },
    { 0x20EA, "COMBINING LEFTWARDS ARROW OVERLAY" },
    { 0x20EB, "COMBINING LONG DOUBLE SOLIDUS OVERLAY" },
    { 0x20EC, "COMBINING RIGHTWARDS HARPOON WITH BARB DOWNWARDS" },
    { 0x20ED, "COMBINING LEFTWARDS HARPOON WITH BARB DOWNWARDS" },
    { 0x20EE, "COMBINING LEFT ARROW BELOW" },
    { 0x20EF, "COMBINING RIGHT ARROW BELOW" },
    { 0x20F0, "COMBINING ASTERISK ABOVE" },
    { 0x2100, "ACCOUNT OF" },
    { 0x2101, "ADDRESSED TO THE SUBJECT" },
    { 0x2102, "DOUBLE-STRUCK CAPITAL C" },
    { 0x2103, "DEGREE CELSIUS" },
    { 0x2104, "CENTRE LINE SYMBOL" },
    { 0x2105, "CARE OF" },
    { 0x2106, "CADA UNA" },
    { 0x2107, "EULER CONSTANT" },
    { 0x2108, "SCRUPLE" },
    { 0x2109, "DEGREE FAHRENHEIT" },
    { 0x210A, "SCRIPT SMALL G" },
    { 0x210B, "SCRIPT CAPITAL H" },
    { 0x210C, "BLACK-LETTER CAPITAL H" },
    { 0x210D, "DOUBLE-STRUCK CAPITAL H" },
    { 0x210E, "PLANCK CONSTANT" },
    { 0x210F, "PLANCK CONSTANT OVER TWO PI" },
    { 0x2110, "SCRIPT CAPITAL I" },
    { 0x2111, "BLACK-LETTER CAPITAL I" },
    { 0x2112, "SCRIPT CAPITAL L" },
    { 0x2113, "SCRIPT SMALL L" },
    { 0x2114, "L B BAR SYMBOL" },
    { 0x2115, "DOUBLE-STRUCK CAPITAL N" },
    { 0x2116, "NUMERO SIGN" },
    { 0x2117, "SOUND RECORDING COPYRIGHT" },
    { 0x2118, "SCRIPT CAPITAL P" },
    { 0x2119, "DOUBLE-STRUCK CAPITAL P" },
    { 0x211A, "DOUBLE-STRUCK CAPITAL Q" },
    { 0x211B, "SCRIPT CAPITAL R" },
    { 0x211C, "BLACK-LETTER CAPITAL R" },
    { 0x211D, "DOUBLE-STRUCK CAPITAL R" },
    { 0x211E, "PRESCRIPTION TAKE" },
    { 0x211F, "RESPONSE" },
    { 0x2120, "SERVICE MARK" },
    { 0x2121, "TELEPHONE SIGN" },
    { 0x2122, "TRADE MARK SIGN" },
    { 0x2123, "VERSICLE" },
    { 0x2124, "DOUBLE-STRUCK CAPITAL Z" },
    { 0x2125, "OUNCE SIGN" },
    { 0x2126, "OHM SIGN" },
    { 0x2127, "INVERTED OHM SIGN" },
    { 0x2128, "BLACK-LETTER CAPITAL Z" },
    { 0x2129, "TURNED GREEK SMALL LETTER IOTA" },
    { 0x212A, "KELVIN SIGN" },
    { 0x212B, "ANGSTROM SIGN" },
    { 0x212C, "SCRIPT CAPITAL B" },
    { 0x212D, "BLACK-LETTER CAPITAL C" },
    { 0x212E, "ESTIMATED SYMBOL" },
    { 0x212F, "SCRIPT SMALL E" },
    { 0x2130, "SCRIPT CAPITAL E" },
    { 0x2131, "SCRIPT CAPITAL F" },
    { 0x2132, "TURNED CAPITAL F" },
    { 0x2133, "SCRIPT CAPITAL M" },
    { 0x2134, "SCRIPT SMALL O" },
    { 0x2135, "ALEF SYMBOL" },
    { 0x2136, "BET SYMBOL" },
    { 0x2137, "GIMEL SYMBOL" },
    { 0x2138, "DALET SYMBOL" },
    { 0x2139, "INFORMATION SOURCE" },
    { 0x213A, "ROTATED CAPITAL Q" },
    { 0x213B, "FACSIMILE SIGN" },
    { 0x213C, "DOUBLE-STRUCK SMALL PI" },
    { 0x213D, "DOUBLE-STRUCK SMALL GAMMA" },
    { 0x213E, "DOUBLE-STRUCK CAPITAL GAMMA" },
    { 0x213F, "DOUBLE-STRUCK CAPITAL PI" },
    { 0x2140, "DOUBLE-STRUCK N-ARY SUMMATION" },
    { 0x2141, "TURNED SANS-SERIF CAPITAL G" },
    { 0x2142, "TURNED SANS-SERIF CAPITAL L" },
    { 0x2143, "REVERSED SANS-SERIF CAPITAL L" },
    { 0x2144, "TURNED SANS-SERIF CAPITAL Y" },
    { 0x2145, "DOUBLE-STRUCK ITALIC CAPITAL D" },
    { 0x2146, "DOUBLE-STRUCK ITALIC SMALL D" },
    { 0x2147, "DOUBLE-STRUCK ITALIC SMALL E" },
    { 0x2148, "DOUBLE-STRUCK ITALIC SMALL I" },
    { 0x2149, "DOUBLE-STRUCK ITALIC SMALL J" },
    { 0x214A, "PROPERTY LINE" },
    { 0x214B, "TURNED AMPERSAND" },
    { 0x214C, "PER SIGN" },
    { 0x214D, "AKTIESELSKAB" },
    { 0x214E, "TURNED SMALL F" },
    { 0x214F, "SYMBOL FOR SAMARITAN SOURCE" },
    { 0x2150, "VULGAR FRACTION ONE SEVENTH" },
    { 0x2151, "VULGAR FRACTION ONE NINTH" },
    { 0x2152, "VULGAR FRACTION ONE TENTH" },
    { 0x2153, "VULGAR FRACTION ONE THIRD" },
    { 0x2154, "VULGAR FRACTION TWO THIRDS" },
    { 0x2155, "VULGAR FRACTION ONE FIFTH" },
    { 0x2156, "VULGAR FRACTION TWO FIFTHS" },
    { 0x2157, "VULGAR FRACTION THREE FIFTHS" },
    { 0x2158, "VULGAR FRACTION FOUR FIFTHS" },
    { 0x2159, "VULGAR FRACTION ONE SIXTH" },
    { 0x215A, "VULGAR FRACTION FIVE SIXTHS" },
    { 0x215B, "VULGAR FRACTION ONE EIGHTH" },
    { 0x215C, "VULGAR FRACTION THREE EIGHTHS" },
    { 0x215D, "VULGAR FRACTION FIVE EIGHTHS" },
    { 0x215E, "VULGAR FRACTION SEVEN EIGHTHS" },
    { 0x215F, "FRACTION NUMERATOR ONE" },
    { 0x2160, "ROMAN NUMERAL ONE" },
    { 0x2161, "ROMAN NUMERAL TWO" },
    { 0x2162, "ROMAN NUMERAL THREE" },
    { 0x2163, "ROMAN NUMERAL FOUR" },
    { 0x2164, "ROMAN NUMERAL FIVE" },
    { 0x2165, "ROMAN NUMERAL SIX" },
    { 0x2166, "ROMAN NUMERAL SEVEN" },
    { 0x2167, "ROMAN NUMERAL EIGHT" },
    { 0x2168, "ROMAN NUMERAL NINE" },
    { 0x2169, "ROMAN NUMERAL TEN" },
    { 0x216A, "ROMAN NUMERAL ELEVEN" },
    { 0x216B, "ROMAN NUMERAL TWELVE" },
    { 0x216C, "ROMAN NUMERAL FIFTY" },
    { 0x216D, "ROMAN NUMERAL ONE HUNDRED" },
    { 0x216E, "ROMAN NUMERAL FIVE HUNDRED" },
    { 0x216F, "ROMAN NUMERAL ONE THOUSAND" },
    { 0x2170, "SMALL ROMAN NUMERAL ONE" },
    { 0x2171, "SMALL ROMAN NUMERAL TWO" },
    { 0x2172, "SMALL ROMAN NUMERAL THREE" },
    { 0x2173, "SMALL ROMAN NUMERAL FOUR" },
    { 0x2174, "SMALL ROMAN NUMERAL FIVE" },
    { 0x2175, "SMALL ROMAN NUMERAL SIX" },
    { 0x2176, "SMALL ROMAN NUMERAL SEVEN" },
    { 0x2177, "SMALL ROMAN NUMERAL EIGHT" },
    { 0x2178, "SMALL ROMAN NUMERAL NINE" },
    { 0x2179, "SMALL ROMAN NUMERAL TEN" },
    { 0x217A, "SMALL ROMAN NUMERAL ELEVEN" },
    { 0x217B, "SMALL ROMAN NUMERAL TWELVE" },
    { 0x217C, "SMALL ROMAN NUMERAL FIFTY" },
    { 0x217D, "SMALL ROMAN NUMERAL ONE HUNDRED" },
    { 0x217E, "SMALL ROMAN NUMERAL FIVE HUNDRED" },
    { 0x217F, "SMALL ROMAN NUMERAL ONE THOUSAND" },
    { 0x2180, "ROMAN NUMERAL ONE THOUSAND C D" },
    { 0x2181, "ROMAN NUMERAL FIVE THOUSAND" },
    { 0x2182, "ROMAN NUMERAL TEN THOUSAND" },
    { 0x2183, "ROMAN NUMERAL REVERSED ONE HUNDRED" },
    { 0x2184, "LATIN SMALL LETTER REVERSED C" },
    { 0x2185, "ROMAN NUMERAL SIX LATE FORM" },
    { 0x2186, "ROMAN NUMERAL FIFTY EARLY FORM" },
    { 0x2187, "ROMAN NUMERAL FIFTY THOUSAND" },
    { 0x2188, "ROMAN NUMERAL ONE HUNDRED THOUSAND" },
    { 0x2189, "VULGAR FRACTION ZERO THIRDS" },
    { 0x218A, "TURNED DIGIT TWO" },
    { 0x218B, "TURNED DIGIT THREE" },
    { 0x2190, "LEFTWARDS ARROW" },
    { 0x2191, "UPWARDS ARROW" },
    { 0x2192, "RIGHTWARDS ARROW" },
    { 0x2193, "DOWNWARDS ARROW" },
    { 0x2194, "LEFT RIGHT ARROW" },
    { 0x2195, "UP DOWN ARROW" },
    { 0x2196, "NORTH WEST ARROW" },
    { 0x2197, "NORTH EAST ARROW" },
    { 0x2198, "SOUTH EAST ARROW" },
    { 0x2199, "SOUTH WEST ARROW" },
    { 0x219A, "LEFTWARDS ARROW WITH STROKE" },
    { 0x219B, "RIGHTWARDS ARROW WITH STROKE" },
    { 0x219C, "LEFTWARDS WAVE ARROW" },
    { 0x219D, "RIGHTWARDS WAVE ARROW" },
    { 0x219E, "LEFTWARDS TWO HEADED ARROW" },
    { 0x219F, "UPWARDS TWO HEADED ARROW" },
    { 0x21A0, "RIGHTWARDS TWO HEADED ARROW" },
    { 0x21A1, "DOWNWARDS TWO HEADED ARROW" },
    { 0x21A2, "LEFTWARDS ARROW WITH TAIL" },
    { 0x21A3, "RIGHTWARDS ARROW WITH TAIL" },
    { 0x21A4, "LEFTWARDS ARROW FROM BAR" },
    { 0x21A5, "UPWARDS ARROW FROM BAR" },
    { 0x21A6, "RIGHTWARDS ARROW FROM BAR" },
    { 0x21A7, "DOWNWARDS ARROW FROM BAR" },
    { 0x21A8, "UP DOWN ARROW WITH BASE" },
    { 0x21A9, "LEFTWARDS ARROW WITH HOOK" },
    { 0x21AA, "RIGHTWARDS ARROW WITH HOOK" },
    { 0x21AB, "LEFTWARDS ARROW WITH LOOP" },
    { 0x21AC, "RIGHTWARDS ARROW WITH LOOP" },
    { 0x21AD, "LEFT RIGHT WAVE ARROW" },
    { 0x21AE, "LEFT RIGHT ARROW WITH STROKE" },
    { 0x21AF, "DOWNWARDS ZIGZAG ARROW" },
    { 0x21B0, "UPWARDS ARROW WITH TIP LEFTWARDS" },
    { 0x21B1, "UPWARDS ARROW WITH TIP RIGHTWARDS" },
    { 0x21B2, "DOWNWARDS ARROW WITH TIP LEFTWARDS" },
    { 0x21B3, "DOWNWARDS ARROW WITH TIP RIGHTWARDS" },
    { 0x21B4, "RIGHTWARDS ARROW WITH CORNER DOWNWARDS" },
    { 0x21B5, "DOWNWARDS ARROW WITH CORNER LEFTWARDS" },
    { 0x21B6, "ANTICLOCKWISE TOP SEMICIRCLE ARROW" },
    { 0x21B7, "CLOCKWISE TOP SEMICIRCLE ARROW" },
    { 0x21B8, "NORTH WEST ARROW TO LONG BAR" },
    { 0x21B9, "LEFTWARDS ARROW TO BAR OVER RIGHTWARDS ARROW TO BAR" },
    { 0x21BA, "ANTICLOCKWISE OPEN CIRCLE ARROW" },
    { 0x21BB, "CLOCKWISE OPEN CIRCLE ARROW" },
    { 0x21BC, "LEFTWARDS HARPOON WITH BARB UPWARDS" },
    { 0x21BD, "LEFTWARDS HARPOON WITH BARB DOWNWARDS" },
    { 0x21BE, "UPWARDS HARPOON WITH BARB RIGHTWARDS" },
    { 0x21BF, "UPWARDS HARPOON WITH BARB LEFTWARDS" },
    { 0x21C0, "RIGHTWARDS HARPOON WITH BARB UPWARDS" },
    { 0x21C1, "RIGHTWARDS HARPOON WITH BARB DOWNWARDS" },
    { 0x21C2, "DOWNWARDS HARPOON WITH BARB RIGHTWARDS" },
    { 0x21C3, "DOWNWARDS HARPOON WITH BARB LEFTWARDS" },
    { 0x21C4, "RIGHTWARDS ARROW OVER LEFTWARDS ARROW" },
    { 0x21C5, "UPWARDS ARROW LEFTWARDS OF DOWNWARDS ARROW" },
    { 0x21C6, "LEFTWARDS ARROW OVER RIGHTWARDS ARROW" },
    { 0x21C7, "LEFTWARDS PAIRED ARROWS" },
    { 0x21C8, "UPWARDS PAIRED ARROWS" },
    { 0x21C9, "RIGHTWARDS PAIRED ARROWS" },
    { 0x21CA, "DOWNWARDS PAIRED ARROWS" },
    { 0x21CB, "LEFTWARDS HARPOON OVER RIGHTWARDS HARPOON" },
    { 0x21CC, "RIGHTWARDS HARPOON OVER LEFTWARDS HARPOON" },
    { 0x21CD, "LEFTWARDS DOUBLE ARROW WITH STROKE" },
    { 0x21CE, "LEFT RIGHT DOUBLE ARROW WITH STROKE" },
    { 0x21CF, "RIGHTWARDS DOUBLE ARROW WITH STROKE" },
    { 0x21D0, "LEFTWARDS DOUBLE ARROW" },
    { 0x21D1, "UPWARDS DOUBLE ARROW" },
    { 0x21D2, "RIGHTWARDS DOUBLE ARROW" },
    { 0x21D3, "DOWNWARDS DOUBLE ARROW" },
    { 0x21D4, "LEFT RIGHT DOUBLE ARROW" },
    { 0x21D5, "UP DOWN DOUBLE ARROW" },
    { 0x21D6, "NORTH WEST DOUBLE ARROW" },
    { 0x21D7, "NORTH EAST DOUBLE ARROW" },
    { 0x21D8, "SOUTH EAST DOUBLE ARROW" },
    { 0x21D9, "SOUTH WEST DOUBLE ARROW" },
    { 0x21DA, "LEFTWARDS TRIPLE ARROW" },
    { 0x21DB, "RIGHTWARDS TRIPLE ARROW" },
    { 0x21DC, "LEFTWARDS SQUIGGLE ARROW" },
    { 0x21DD, "RIGHTWARDS SQUIGGLE ARROW" },
    { 0x21DE, "UPWARDS ARROW WITH DOUBLE STROKE" },
    { 0x21DF, "DOWNWARDS ARROW WITH DOUBLE STROKE" },
    { 0x21E0, "LEFTWARDS DASHED ARROW" },
    { 0x21E1, "UPWARDS DASHED ARROW" },
    { 0x21E2, "RIGHTWARDS DASHED ARROW" },
    { 0x21E3, "DOWNWARDS DASHED ARROW" },
    { 0x21E4, "LEFTWARDS ARROW TO BAR" },
    { 0x21E5, "RIGHTWARDS ARROW TO BAR" },
    { 0x21E6, "LEFTWARDS WHITE ARROW" },
    { 0x21E7, "UPWARDS WHITE ARROW" },
    { 0x21E8, "RIGHTWARDS WHITE ARROW" },
    { 0x21E9, "DOWNWARDS WHITE ARROW" },
    { 0x21EA, "UPWARDS WHITE ARROW FROM BAR" },
    { 0x21EB, "UPWARDS WHITE ARROW ON PEDESTAL" },
    { 0x21EC, "UPWARDS WHITE ARROW ON PEDESTAL WITH HORIZONTAL BAR" },
    { 0x21ED, "UPWARDS WHITE ARROW ON PEDESTAL WITH VERTICAL BAR" },
    { 0x21EE, "UPWARDS WHITE DOUBLE ARROW" },
    { 0x21EF, "UPWARDS WHITE DOUBLE ARROW ON PEDESTAL" },
    { 0x21F0, "RIGHTWARDS WHITE ARROW FROM WALL" },
    { 0x21F1, "NORTH WEST ARROW TO CORNER" },
    { 0x21F2, "SOUTH EAST ARROW TO CORNER" },
    { 0x21F3, "UP DOWN WHITE ARROW" },
    { 0x21F4, "RIGHT ARROW WITH SMALL CIRCLE" },
    { 0x21F5, "DOWNWARDS ARROW LEFTWARDS OF UPWARDS ARROW" },
    { 0x21F6, "THREE RIGHTWARDS ARROWS" },
    { 0x21F7, "LEFTWARDS ARROW WITH VERTICAL STROKE" },
    { 0x21F8, "RIGHTWARDS ARROW WITH VERTICAL STROKE" },
    { 0x21F9, "LEFT RIGHT ARROW WITH VERTICAL STROKE" },
    { 0x21FA, "LEFTWARDS ARROW WITH DOUBLE VERTICAL STROKE" },
    { 0x21FB, "RIGHTWARDS ARROW WITH DOUBLE VERTICAL STROKE" },
    { 0x21FC, "LEFT RIGHT ARROW WITH DOUBLE VERTICAL STROKE" },
    { 0x21FD, "LEFTWARDS OPEN-HEADED ARROW" },
    { 0x21FE, "RIGHTWARDS OPEN-HEADED ARROW" },
    { 0x21FF, "LEFT RIGHT OPEN-HEADED ARROW" },
    { 0x2200, "FOR ALL" },
    { 0x2201, "COMPLEMENT" },
    { 0x2202, "PARTIAL DIFFERENTIAL" },
    { 0x2203, "THERE EXISTS" },
    { 0x2204, "THERE DOES NOT EXIST" },
    { 0x2205, "EMPTY SET" },
    { 0x2206, "INCREMENT" },
    { 0x2207, "NABLA" },
    { 0x2208, "ELEMENT OF" },
    { 0x2209, "NOT AN ELEMENT OF" },
    { 0x220A, "SMALL ELEMENT OF" },
    { 0x220B, "CONTAINS AS MEMBER" },
    { 0x220C, "DOES NOT CONTAIN AS MEMBER" },
    { 0x220D, "SMALL CONTAINS AS MEMBER" },
    { 0x220E, "END OF PROOF" },
    { 0x220F, "N-ARY PRODUCT" },
    { 0x2210, "N-ARY COPRODUCT" },
    { 0x2211, "N-ARY SUMMATION" },
    { 0x2212, "MINUS SIGN" },
    { 0x2213, "MINUS-OR-PLUS SIGN" },
    { 0x2214, "DOT PLUS" },
    { 0x2215, "DIVISION SLASH" },
    { 0x2216, "SET MINUS" },
    { 0x2217, "ASTERISK OPERATOR" },
    { 0x2218, "RING OPERATOR" },
    { 0x2219, "BULLET OPERATOR" },
    { 0x221A, "SQUARE ROOT" },
    { 0x221B, "CUBE ROOT" },
    { 0x221C, "FOURTH ROOT" },
    { 0x221D, "PROPORTIONAL TO" },
    { 0x221E, "INFINITY" },
    { 0x221F, "RIGHT ANGLE" },
    { 0x2220, "ANGLE" },
    { 0x2221, "MEASURED ANGLE" },
    { 0x2222, "SPHERICAL ANGLE" },
    { 0x2223, "DIVIDES" },
    { 0x2224, "DOES NOT DIVIDE" },
    { 0x2225, "PARALLEL TO" },
    { 0x2226, "NOT PARALLEL TO" },
    { 0x2227, "LOGICAL AND" },
    { 0x2228, "LOGICAL OR" },
    { 0x2229, "INTERSECTION" },
    { 0x222A, "UNION" },
    { 0x222B, "INTEGRAL" },
    { 0x222C, "DOUBLE INTEGRAL" },
    { 0x222D, "TRIPLE INTEGRAL" },
    { 0x222E, "CONTOUR INTEGRAL" },
    { 0x222F, "SURFACE INTEGRAL" },
    { 0x2230, "VOLUME INTEGRAL" },
    { 0x2231, "CLOCKWISE INTEGRAL" },
    { 0x2232, "CLOCKWISE CONTOUR INTEGRAL" },
    { 0x2233, "ANTICLOCKWISE CONTOUR INTEGRAL" },
    { 0x2234, "THEREFORE" },
    { 0x2235, "BECAUSE" },
    { 0x2236, "RATIO" },
    { 0x2237, "PROPORTION" },
    { 0x2238, "DOT MINUS" },
    { 0x2239, "EXCESS" },
    { 0x223A, "GEOMETRIC PROPORTION" },
    { 0x223B, "HOMOTHETIC" },
    { 0x223C, "TILDE OPERATOR" },
    { 0x223D, "REVERSED TILDE" },
    { 0x223E, "INVERTED LAZY S" },
    { 0x223F, "SINE WAVE" },
    { 0x2240, "WREATH PRODUCT" },
    { 0x2241, "NOT TILDE" },
    { 0x2242, "MINUS TILDE" },
    { 0x2243, "ASYMPTOTICALLY EQUAL TO" },
    { 0x2244, "NOT ASYMPTOTICALLY EQUAL TO" },
    { 0x2245, "APPROXIMATELY EQUAL TO" },
    { 0x2246, "APPROXIMATELY BUT NOT ACTUALLY EQUAL TO" },
    { 0x2247, "NEITHER APPROXIMATELY NOR ACTUALLY EQUAL TO" },
    { 0x2248, "ALMOST EQUAL TO" },
    { 0x2249, "NOT ALMOST EQUAL TO" },
    { 0x224A, "ALMOST EQUAL OR EQUAL TO" },
    { 0x224B, "TRIPLE TILDE" },
    { 0x224C, "ALL EQUAL TO" },
    { 0x224D, "EQUIVALENT TO" },
    { 0x224E, "GEOMETRICALLY EQUIVALENT TO" },
    { 0x224F, "DIFFERENCE BETWEEN" },
    { 0x2250, "APPROACHES THE LIMIT" },
    { 0x2251, "GEOMETRICALLY EQUAL TO" },
    { 0x2252, "APPROXIMATELY EQUAL TO OR THE IMAGE OF" },
    { 0x2253, "IMAGE OF OR APPROXIMATELY EQUAL TO" },
    { 0x2254, "COLON EQUALS" },
    { 0x2255, "EQUALS COLON" },
    { 0x2256, "RING IN EQUAL TO" },
    { 0x2257, "RING EQUAL TO" },
    { 0x2258, "CORRESPONDS TO" },
    { 0x2259, "ESTIMATES" },
    { 0x225A, "EQUIANGULAR TO" },
    { 0x225B, "STAR EQUALS" },
    { 0x225C, "DELTA EQUAL TO" },
    { 0x225D, "EQUAL TO BY DEFINITION" },
    { 0x225E, "MEASURED BY" },
    { 0x225F, "QUESTIONED EQUAL TO" },
    { 0x2260, "NOT EQUAL TO" },
    { 0x2261, "IDENTICAL TO" },
    { 0x2262, "NOT IDENTICAL TO" },
    { 0x2263, "STRICTLY EQUIVALENT TO" },
    { 0x2264, "LESS-THAN OR EQUAL TO" },
    { 0x2265, "GREATER-THAN OR EQUAL TO" },
    { 0x2266, "LESS-THAN OVER EQUAL TO" },
    { 0x2267, "GREATER-THAN OVER EQUAL TO" },
    { 0x2268, "LESS-THAN BUT NOT EQUAL TO" },
    { 0x2269, "GREATER-THAN BUT NOT EQUAL TO" },
    { 0x226A, "MUCH LESS-THAN" },
    { 0x226B, "MUCH GREATER-THAN" },
    { 0x226C, "BETWEEN" },
    { 0x226D, "NOT EQUIVALENT TO" },
    { 0x226E, "NOT LESS-THAN" },
    { 0x226F, "NOT GREATER-THAN" },
    { 0x2270, "NEITHER LESS-THAN NOR EQUAL TO" },
    { 0x2271, "NEITHER GREATER-THAN NOR EQUAL TO" },
    { 0x2272, "LESS-THAN OR EQUIVALENT TO" },
    { 0x2273, "GREATER-THAN OR EQUIVALENT TO" },
    { 0x2274, "NEITHER LESS-THAN NOR EQUIVALENT TO" },
    { 0x2275, "NEITHER GREATER-THAN NOR EQUIVALENT TO" },
    { 0x2276, "LESS-THAN OR GREATER-THAN" },
    { 0x2277, "GREATER-THAN OR LESS-THAN" },
    { 0x2278, "NEITHER LESS-THAN NOR GREATER-THAN" },
    { 0x2279, "NEITHER GREATER-THAN NOR LESS-THAN" },
    { 0x227A, "PRECEDES" },
    { 0x227B, "SUCCEEDS" },
    { 0x227C, "PRECEDES OR EQUAL TO" },
    { 0x227D, "SUCCEEDS OR EQUAL TO" },
    { 0x227E, "PRECEDES OR EQUIVALENT TO" },
    { 0x227F, "SUCCEEDS OR EQUIVALENT TO" },
    { 0x2280, "DOES NOT PRECEDE" },
    { 0x2281, "DOES NOT SUCCEED" },
    { 0x2282, "SUBSET OF" },
    { 0x2283, "SUPERSET OF" },
    { 0x2284, "NOT A SUBSET OF" },
    { 0x2285, "NOT A SUPERSET OF" },
    { 0x2286, "SUBSET OF OR EQUAL TO" },
    { 0x2287, "SUPERSET OF OR EQUAL TO" },
    { 0x2288, "NEITHER A SUBSET OF NOR EQUAL TO" },
    { 0x2289, "NEITHER A SUPERSET OF NOR EQUAL TO" },
    { 0x228A, "SUBSET OF WITH NOT EQUAL TO" },
    { 0x228B, "SUPERSET OF WITH NOT EQUAL TO" },
    { 0x228C, "MULTISET" },
    { 0x228D, "MULTISET MULTIPLICATION" },
    { 0x228E, "MULTISET UNION" },
    { 0x228F, "SQUARE IMAGE OF" },
    { 0x2290, "SQUARE ORIGINAL OF" },
    { 0x2291, "SQUARE IMAGE OF OR EQUAL TO" },
    { 0x2292, "SQUARE ORIGINAL OF OR EQUAL TO" },
    { 0x2293, "SQUARE CAP" },
    { 0x2294, "SQUARE CUP" },
    { 0x2295, "CIRCLED PLUS" },
    { 0x2296, "CIRCLED MINUS" },
    { 0x2297, "CIRCLED TIMES" },
    { 0x2298, "CIRCLED DIVISION SLASH" },
    { 0x2299, "CIRCLED DOT OPERATOR" },
    { 0x229A, "CIRCLED RING OPERATOR" },
    { 0x229B, "CIRCLED ASTERISK OPERATOR" },
    { 0x229C, "CIRCLED EQUALS" },
    { 0x229D, "CIRCLED DASH" },
    { 0x229E, "SQUARED PLUS" },
    { 0x229F, "SQUARED MINUS" },
    { 0x22A0, "SQUARED TIMES" },
    { 0x22A1, "SQUARED DOT OPERATOR" },
    { 0x22A2, "RIGHT TACK" },
    { 0x22A3, "LEFT TACK" },
    { 0x22A4, "DOWN TACK" },
    { 0x22A5, "UP TACK" },
    { 0x22A6, "ASSERTION" },
    { 0x22A7, "MODELS" },
    { 0x22A8, "TRUE" },
    { 0x22A9, "FORCES" },
    { 0x22AA, "TRIPLE VERTICAL BAR RIGHT TURNSTILE" },
    { 0x22AB, "DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE" },
    { 0x22AC, "DOES NOT PROVE" },
    { 0x22AD, "NOT TRUE" },
    { 0x22AE, "DOES NOT FORCE" },
    { 0x22AF, "NEGATED DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE" },
    { 0x22B0, "PRECEDES UNDER RELATION" },
    { 0x22B1, "SUCCEEDS UNDER RELATION" },
    { 0x22B2, "NORMAL SUBGROUP OF" },
    { 0x22B3, "CONTAINS AS NORMAL SUBGROUP" },
    { 0x22B4, "NORMAL SUBGROUP OF OR EQUAL TO" },
    { 0x22B5, "CONTAINS AS NORMAL SUBGROUP OR EQUAL TO" },
    { 0x22B6, "ORIGINAL OF" },
    { 0x22B7, "IMAGE OF" },
    { 0x22B8, "MULTIMAP" },
    { 0x22B9, "HERMITIAN CONJUGATE MATRIX" },
    { 0x22BA, "INTERCALATE" },
    { 0x22BB, "XOR" },
    { 0x22BC, "NAND" },
    { 0x22BD, "NOR" },
    { 0x22BE, "RIGHT ANGLE WITH ARC" },
    { 0x22BF, "RIGHT TRIANGLE" },
    { 0x22C0, "N-ARY LOGICAL AND" },
    { 0x22C1, "N-ARY LOGICAL OR" },
    { 0x22C2, "N-ARY INTERSECTION" },
    { 0x22C3, "N-ARY UNION" },
    { 0x22C4, "DIAMOND OPERATOR" },
    { 0x22C5, "DOT OPERATOR" },
    { 0x22C6, "STAR OPERATOR" },
    { 0x22C7, "DIVISION TIMES" },
    { 0x22C8, "BOWTIE" },
    { 0x22C9, "LEFT NORMAL FACTOR SEMIDIRECT PRODUCT" },
    { 0x22CA, "RIGHT NORMAL FACTOR SEMIDIRECT PRODUCT" },
    { 0x22CB, "LEFT SEMIDIRECT PRODUCT" },
    { 0x22CC, "RIGHT SEMIDIRECT PRODUCT" },
    { 0x22CD, "REVERSED TILDE EQUALS" },
    { 0x22CE, "CURLY LOGICAL OR" },
    { 0x22CF, "CURLY LOGICAL AND" },
    { 0x22D0, "DOUBLE SUBSET" },
    { 0x22D1, "DOUBLE SUPERSET" },
    { 0x22D2, "DOUBLE INTERSECTION" },
    { 0x22D3, "DOUBLE UNION" },
    { 0x22D4, "PITCHFORK" },
    { 0x22D5, "EQUAL AND PARALLEL TO" },
    { 0x22D6, "LESS-THAN WITH DOT" },
    { 0x22D7, "GREATER-THAN WITH DOT" },
    { 0x22D8, "VERY MUCH LESS-THAN" },
    { 0x22D9, "VERY MUCH GREATER-THAN" },
    { 0x22DA, "LESS-THAN EQUAL TO OR GREATER-THAN" },
    { 0x22DB, "GREATER-THAN EQUAL TO OR LESS-THAN" },
    { 0x22DC, "EQUAL TO OR LESS-THAN" },
    { 0x22DD, "EQUAL TO OR GREATER-THAN" },
    { 0x22DE, "EQUAL TO OR PRECEDES" },
    { 0x22DF, "EQUAL TO OR SUCCEEDS" },
    { 0x22E0, "DOES NOT PRECEDE OR EQUAL" },
    { 0x22E1, "DOES NOT SUCCEED OR EQUAL" },
    { 0x22E2, "NOT SQUARE IMAGE OF OR EQUAL TO" },
    { 0x22E3, "NOT SQUARE ORIGINAL OF OR EQUAL TO" },
    { 0x22E4, "SQUARE IMAGE OF OR NOT EQUAL TO" },
    { 0x22E5, "SQUARE ORIGINAL OF OR NOT EQUAL TO" },
    { 0x22E6, "LESS-THAN BUT NOT EQUIVALENT TO" },
    { 0x22E7, "GREATER-THAN BUT NOT EQUIVALENT TO" },
    { 0x22E8, "PRECEDES BUT NOT EQUIVALENT TO" },
    { 0x22E9, "SUCCEEDS BUT NOT EQUIVALENT TO" },
    { 0x22EA, "NOT NORMAL SUBGROUP OF" },
    { 0x22EB, "DOES NOT CONTAIN AS NORMAL SUBGROUP" },
    { 0x22EC, "NOT NORMAL SUBGROUP OF OR EQUAL TO" },
    { 0x22ED, "DOES NOT CONTAIN AS NORMAL SUBGROUP OR EQUAL" },
    { 0x22EE, "VERTICAL ELLIPSIS" },
    { 0x22EF, "MIDLINE HORIZONTAL ELLIPSIS" },
    { 0x22F0, "UP RIGHT DIAGONAL ELLIPSIS" },
    { 0x22F1, "DOWN RIGHT DIAGONAL ELLIPSIS" },
    { 0x22F2, "ELEMENT OF WITH LONG HORIZONTAL STROKE" },
    { 0x22F3, "ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE" },
    { 0x22F4, "SMALL ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE" },
    { 0x22F5, "ELEMENT OF WITH DOT ABOVE" },
    { 0x22F6, "ELEMENT OF WITH OVERBAR" },
    { 0x22F7, "SMALL ELEMENT OF WITH OVERBAR" },
    { 0x22F8, "ELEMENT OF WITH UNDERBAR" },
    { 0x22F9, "ELEMENT OF WITH TWO HORIZONTAL STROKES" },
    { 0x22FA, "CONTAINS WITH LONG HORIZONTAL STROKE" },
    { 0x22FB, "CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE" },
    { 0x22FC, "SMALL CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE" },
    { 0x22FD, "CONTAINS WITH OVERBAR" },
    { 0x22FE, "SMALL CONTAINS WITH OVERBAR" },
    { 0x22FF, "Z NOTATION BAG MEMBERSHIP" },
    { 0x2300, "DIAMETER SIGN" },
    { 0x2301, "ELECTRIC ARROW" },
    { 0x2302, "HOUSE" },
    { 0x2303, "UP ARROWHEAD" },
    { 0x2304, "DOWN ARROWHEAD" },
    { 0x2305, "PROJECTIVE" },
    { 0x2306, "PERSPECTIVE" },
    { 0x2307, "WAVY LINE" },
    { 0x2308, "LEFT CEILING" },
    { 0x2309, "RIGHT CEILING" },
    { 0x230A, "LEFT FLOOR" },
    { 0x230B, "RIGHT FLOOR" },
    { 0x230C, "BOTTOM RIGHT CROP" },
    { 0x230D, "BOTTOM LEFT CROP" },
    { 0x230E, "TOP RIGHT CROP" },
    { 0x230F, "TOP LEFT CROP" },
    { 0x2310, "REVERSED NOT SIGN" },
    { 0x2311, "SQUARE LOZENGE" },
    { 0x2312, "ARC" },
    { 0x2313, "SEGMENT" },
    { 0x2314, "SECTOR" },
    { 0x2315, "TELEPHONE RECORDER" },
    { 0x2316, "POSITION INDICATOR" },
    { 0x2317, "VIEWDATA SQUARE" },
    { 0x2318, "PLACE OF INTEREST SIGN" },
    { 0x2319, "TURNED NOT SIGN" },
    { 0x231C, "TOP LEFT CORNER" },
    { 0x231D, "TOP RIGHT CORNER" },
    { 0x231E, "BOTTOM LEFT CORNER" },
    { 0x231F, "BOTTOM RIGHT CORNER" },
    { 0x2320, "TOP HALF INTEGRAL" },
    { 0x2321, "BOTTOM HALF INTEGRAL" },
    { 0x2322, "FROWN" },
    { 0x2323, "SMILE" },
    { 0x2324, "UP ARROWHEAD BETWEEN TWO HORIZONTAL BARS" },
    { 0x2325, "OPTION KEY" },
    { 0x2326, "ERASE TO THE RIGHT" },
    { 0x2327, "X IN A RECTANGLE BOX" },
    { 0x2328, "KEYBOARD" },
    { 0x2329, "LEFT-POINTING ANGLE BRACKET" },
    { 0x232A, "RIGHT-POINTING ANGLE BRACKET" },
    { 0x232B, "ERASE TO THE LEFT" },
    { 0x232C, "BENZENE RING" },
    { 0x232D, "CYLINDRICITY" },
    { 0x232E, "ALL AROUND-PROFILE" },
    { 0x232F, "SYMMETRY" },
    { 0x2330, "TOTAL RUNOUT" },
    { 0x2331, "DIMENSION ORIGIN" },
    { 0x2332, "CONICAL TAPER" },
    { 0x2333, "SLOPE" },
    { 0x2334, "COUNTERBORE" },
    { 0x2335, "COUNTERSINK" },
    { 0x2336, "APL FUNCTIONAL SYMBOL I-BEAM" },
    { 0x2337, "APL FUNCTIONAL SYMBOL SQUISH QUAD" },
    { 0x2338, "APL FUNCTIONAL SYMBOL QUAD EQUAL" },
    { 0x2339, "APL FUNCTIONAL SYMBOL QUAD DIVIDE" },
    { 0x233A, "APL FUNCTIONAL SYMBOL QUAD DIAMOND" },
    { 0x233B, "APL FUNCTIONAL SYMBOL QUAD JOT" },
    { 0x233C, "APL FUNCTIONAL SYMBOL QUAD CIRCLE" },
    { 0x233D, "APL FUNCTIONAL SYMBOL CIRCLE STILE" },
    { 0x233E, "APL FUNCTIONAL SYMBOL CIRCLE JOT" },
    { 0x233F, "APL FUNCTIONAL SYMBOL SLASH BAR" },
    { 0x2340, "APL FUNCTIONAL SYMBOL BACKSLASH BAR" },
    { 0x2341, "APL FUNCTIONAL SYMBOL QUAD SLASH" },
    { 0x2342, "APL FUNCTIONAL SYMBOL QUAD BACKSLASH" },
    { 0x2343, "APL FUNCTIONAL SYMBOL QUAD LESS-THAN" },
    { 0x2344, "APL FUNCTIONAL SYMBOL QUAD GREATER-THAN" },
    { 0x2345, "APL FUNCTIONAL SYMBOL LEFTWARDS VANE" },
    { 0x2346, "APL FUNCTIONAL SYMBOL RIGHTWARDS VANE" },
    { 0x2347, "APL FUNCTIONAL SYMBOL QUAD LEFTWARDS ARROW" },
    { 0x2348, "APL FUNCTIONAL SYMBOL QUAD RIGHTWARDS ARROW" },
    { 0x2349, "APL FUNCTIONAL SYMBOL CIRCLE BACKSLASH" },
    { 0x234A, "APL FUNCTIONAL SYMBOL DOWN TACK UNDERBAR" },
    { 0x234B, "APL FUNCTIONAL SYMBOL DELTA STILE" },
    { 0x234C, "APL FUNCTIONAL SYMBOL QUAD DOWN CARET" },
    { 0x234D, "APL FUNCTIONAL SYMBOL QUAD DELTA" },
    { 0x234E, "APL FUNCTIONAL SYMBOL DOWN TACK JOT" },
    { 0x234F, "APL FUNCTIONAL SYMBOL UPWARDS VANE" },
    { 0x2350, "APL FUNCTIONAL SYMBOL QUAD UPWARDS ARROW" },
    { 0x2351, "APL FUNCTIONAL SYMBOL UP TACK OVERBAR" },
    { 0x2352, "APL FUNCTIONAL SYMBOL DEL STILE" },
    { 0x2353, "APL FUNCTIONAL SYMBOL QUAD UP CARET" },
    { 0x2354, "APL FUNCTIONAL SYMBOL QUAD DEL" },
    { 0x2355, "APL FUNCTIONAL SYMBOL UP TACK JOT" },
    { 0x2356, "APL FUNCTIONAL SYMBOL DOWNWARDS VANE" },
    { 0x2357, "APL FUNCTIONAL SYMBOL QUAD DOWNWARDS ARROW" },
    { 0x2358, "APL FUNCTIONAL SYMBOL QUOTE UNDERBAR" },
    { 0x2359, "APL FUNCTIONAL SYMBOL DELTA UNDERBAR" },
    { 0x235A, "APL FUNCTIONAL SYMBOL DIAMOND UNDERBAR" },
    { 0x235B, "APL FUNCTIONAL SYMBOL JOT UNDERBAR" },
    { 0x235C, "APL FUNCTIONAL SYMBOL CIRCLE UNDERBAR" },
    { 0x235D, "APL FUNCTIONAL SYMBOL UP SHOE JOT" },
    { 0x235E, "APL FUNCTIONAL SYMBOL QUOTE QUAD" },
    { 0x235F, "APL FUNCTIONAL SYMBOL CIRCLE STAR" },
    { 0x2360, "APL FUNCTIONAL SYMBOL QUAD COLON" },
    { 0x2361, "APL FUNCTIONAL SYMBOL UP TACK DIAERESIS" },
    { 0x2362, "APL FUNCTIONAL SYMBOL DEL DIAERESIS" },
    { 0x2363, "APL FUNCTIONAL SYMBOL STAR DIAERESIS" },
    { 0x2364, "APL FUNCTIONAL SYMBOL JOT DIAERESIS" },
    { 0x2365, "APL FUNCTIONAL SYMBOL CIRCLE DIAERESIS" },
    { 0x2366, "APL FUNCTIONAL SYMBOL DOWN SHOE STILE" },
    { 0x2367, "APL FUNCTIONAL SYMBOL LEFT SHOE STILE" },
    { 0x2368, "APL FUNCTIONAL SYMBOL TILDE DIAERESIS" },
    { 0x2369, "APL FUNCTIONAL SYMBOL GREATER-THAN DIAERESIS" },
    { 0x236A, "APL FUNCTIONAL SYMBOL COMMA BAR" },
    { 0x236B, "APL FUNCTIONAL SYMBOL DEL TILDE" },
    { 0x236C, "APL FUNCTIONAL SYMBOL ZILDE" },
    { 0x236D, "APL FUNCTIONAL SYMBOL STILE TILDE" },
    { 0x236E, "APL FUNCTIONAL SYMBOL SEMICOLON UNDERBAR" },
    { 0x236F, "APL FUNCTIONAL SYMBOL QUAD NOT EQUAL" },
    { 0x2370, "APL FUNCTIONAL SYMBOL QUAD QUESTION" },
    { 0x2371, "APL FUNCTIONAL SYMBOL DOWN CARET TILDE" },
    { 0x2372, "APL FUNCTIONAL SYMBOL UP CARET TILDE" },
    { 0x2373, "APL FUNCTIONAL SYMBOL IOTA" },
    { 0x2374, "APL FUNCTIONAL SYMBOL RHO" },
    { 0x2375, "APL FUNCTIONAL SYMBOL OMEGA" },
    { 0x2376, "APL FUNCTIONAL SYMBOL ALPHA UNDERBAR" },
    { 0x2377, "APL FUNCTIONAL SYMBOL EPSILON UNDERBAR" },
    { 0x2378, "APL FUNCTIONAL SYMBOL IOTA UNDERBAR" },
    { 0x2379, "APL FUNCTIONAL SYMBOL OMEGA UNDERBAR" },
    { 0x237A, "APL FUNCTIONAL SYMBOL ALPHA" },
    { 0x237B, "NOT CHECK MARK" },
    { 0x237C, "RIGHT ANGLE WITH DOWNWARDS ZIGZAG ARROW" },
    { 0x237D, "SHOULDERED OPEN BOX" },
    { 0x237E, "BELL SYMBOL" },
    { 0x237F, "VERTICAL LINE WITH MIDDLE DOT" },
    { 0x2380, "INSERTION SYMBOL" },
    { 0x2381, "CONTINUOUS UNDERLINE SYMBOL" },
    { 0x2382, "DISCONTINUOUS UNDERLINE SYMBOL" },
    { 0x2383, "EMPHASIS SYMBOL" },
    { 0x2384, "COMPOSITION SYMBOL" },
    { 0x2385, "WHITE SQUARE WITH CENTRE VERTICAL LINE" },
    { 0x2386, "ENTER SYMBOL" },
    { 0x2387, "ALTERNATIVE KEY SYMBOL" },
    { 0x2388, "HELM SYMBOL" },
    { 0x2389, "CIRCLED HORIZONTAL BAR WITH NOTCH" },
    { 0x238A, "CIRCLED TRIANGLE DOWN" },
    { 0x238B, "BROKEN CIRCLE WITH NORTHWEST ARROW" },
    { 0x238C, "UNDO SYMBOL" },
    { 0x238D, "MONOSTABLE SYMBOL" },
    { 0x238E, "HYSTERESIS SYMBOL" },
    { 0x238F, "OPEN-CIRCUIT-OUTPUT H-TYPE SYMBOL" },
    { 0x2390, "OPEN-CIRCUIT-OUTPUT L-TYPE SYMBOL" },
    { 0x2391, "PASSIVE-PULL-DOWN-OUTPUT SYMBOL" },
    { 0x2392, "PASSIVE-PULL-UP-OUTPUT SYMBOL" },
    { 0x2393, "DIRECT CURRENT SYMBOL FORM TWO" },
    { 0x2394, "SOFTWARE-FUNCTION SYMBOL" },
    { 0x2395, "APL FUNCTIONAL SYMBOL QUAD" },
    { 0x2396, "DECIMAL SEPARATOR KEY SYMBOL" },
    { 0x2397, "PREVIOUS PAGE" },
    { 0x2398, "NEXT PAGE" },
    { 0x2399, "PRINT SCREEN SYMBOL" },
    { 0x239A, "CLEAR SCREEN SYMBOL" },
    { 0x239B, "LEFT PARENTHESIS UPPER HOOK" },
    { 0x239C, "LEFT PARENTHESIS EXTENSION" },
    { 0x239D, "LEFT PARENTHESIS LOWER HOOK" },
    { 0x239E, "RIGHT PARENTHESIS UPPER HOOK" },
    { 0x239F, "RIGHT PARENTHESIS EXTENSION" },
    { 0x23A0, "RIGHT PARENTHESIS LOWER HOOK" },
    { 0x23A1, "LEFT SQUARE BRACKET UPPER CORNER" },
    { 0x23A2, "LEFT SQUARE BRACKET EXTENSION" },
    { 0x23A3, "LEFT SQUARE BRACKET LOWER CORNER" },
    { 0x23A4, "RIGHT SQUARE BRACKET UPPER CORNER" },
    { 0x23A5, "RIGHT SQUARE BRACKET EXTENSION" },
    { 0x23A6, "RIGHT SQUARE BRACKET LOWER CORNER" },
    { 0x23A7, "LEFT CURLY BRACKET UPPER HOOK" },
    { 0x23A8, "LEFT CURLY BRACKET MIDDLE PIECE" },
    { 0x23A9, "LEFT CURLY BRACKET LOWER HOOK" },
    { 0x23AA, "CURLY BRACKET EXTENSION" },
    { 0x23AB, "RIGHT CURLY BRACKET UPPER HOOK" },
    { 0x23AC, "RIGHT CURLY BRACKET MIDDLE PIECE" },
    { 0x23AD, "RIGHT CURLY BRACKET LOWER HOOK" },
    { 0x23AE, "INTEGRAL EXTENSION" },
    { 0x23AF, "HORIZONTAL LINE EXTENSION" },
    { 0x23B0, "UPPER LEFT OR LOWER RIGHT CURLY BRACKET SECTION" },
    { 0x23B1, "UPPER RIGHT OR LOWER LEFT CURLY BRACKET SECTION" },
    { 0x23B2, "SUMMATION TOP" },
    { 0x23B3, "SUMMATION BOTTOM" },
    { 0x23B4, "TOP SQUARE BRACKET" },
    { 0x23B5, "BOTTOM SQUARE BRACKET" },
    { 0x23B6, "BOTTOM SQUARE BRACKET OVER TOP SQUARE BRACKET" },
    { 0x23B7, "RADICAL SYMBOL BOTTOM" },
    { 0x23B8, "LEFT VERTICAL BOX LINE" },
    { 0x23B9, "RIGHT VERTICAL BOX LINE" },
    { 0x23BA, "HORIZONTAL SCAN LINE-1" },
    { 0x23BB, "HORIZONTAL SCAN LINE-3" },
    { 0x23BC, "HORIZONTAL SCAN LINE-7" },
    { 0x23BD, "HORIZONTAL SCAN LINE-9" },
    { 0x23BE, "DENTISTRY SYMBOL LIGHT VERTICAL AND TOP RIGHT" },
    { 0x23BF, "DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM RIGHT" },
    { 0x23C0, "DENTISTRY SYMBOL LIGHT VERTICAL WITH CIRCLE" },
    { 0x23C1, "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH CIRCLE" },
    { 0x23C2, "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH CIRCLE" },
    { 0x23C3, "DENTISTRY SYMBOL LIGHT VERTICAL WITH TRIANGLE" },
    { 0x23C4, "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH TRIANGLE" },
    { 0x23C5, "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH TRIANGLE" },
    { 0x23C6, "DENTISTRY SYMBOL LIGHT VERTICAL AND WAVE" },
    { 0x23C7, "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH WAVE" },
    { 0x23C8, "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH WAVE" },
    { 0x23C9, "DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL" },
    { 0x23CA, "DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL" },
    { 0x23CB, "DENTISTRY SYMBOL LIGHT VERTICAL AND TOP LEFT" },
    { 0x23CC, "DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM LEFT" },
    { 0x23CD, "SQUARE FOOT" },
    { 0x23CE, "RETURN SYMBOL" },
    { 0x23CF, "EJECT SYMBOL" },
    { 0x23D0, "VERTICAL LINE EXTENSION" },
    { 0x23D1, "METRICAL BREVE" },
    { 0x23D2, "METRICAL LONG OVER SHORT" },
    { 0x23D3, "METRICAL SHORT OVER LONG" },
    { 0x23D4, "METRICAL LONG OVER TWO SHORTS" },
    { 0x23D5, "METRICAL TWO SHORTS OVER LONG" },
    { 0x23D6, "METRICAL TWO SHORTS JOINED" },
    { 0x23D7, "METRICAL TRISEME" },
    { 0x23D8, "METRICAL TETRASEME" },
    { 0x23D9, "METRICAL PENTASEME" },
    { 0x23DA, "EARTH GROUND" },
    { 0x23DB, "FUSE" },
    { 0x23DC, "TOP PARENTHESIS" },
    { 0x23DD, "BOTTOM PARENTHESIS" },
    { 0x23DE, "TOP CURLY BRACKET" },
    { 0x23DF, "BOTTOM CURLY BRACKET" },
    { 0x23E0, "TOP TORTOISE SHELL BRACKET" },
    { 0x23E1, "BOTTOM TORTOISE SHELL BRACKET" },
    { 0x23E2, "WHITE TRAPEZIUM" },
    { 0x23E3, "BENZENE RING WITH CIRCLE" },
    { 0x23E4, "STRAIGHTNESS" },
    { 0x23E5, "FLATNESS" },
    { 0x23E6, "AC CURRENT" },
    { 0x23E7, "ELECTRICAL INTERSECTION" },
    { 0x23E8, "DECIMAL EXPONENT SYMBOL" },
    { 0x23ED, "BLACK RIGHT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR" },
    { 0x23EE, "BLACK LEFT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR" },
    { 0x23EF, "BLACK RIGHT-POINTING TRIANGLE WITH DOUBLE VERTICAL BAR" },
    { 0x23F1, "STOPWATCH" },
    { 0x23F2, "TIMER CLOCK" },
    { 0x23F4, "BLACK MEDIUM LEFT-POINTING TRIANGLE" },
    { 0x23F5, "BLACK MEDIUM RIGHT-POINTING TRIANGLE" },
    { 0x23F6, "BLACK MEDIUM UP-POINTING TRIANGLE" },
    { 0x23F7, "BLACK MEDIUM DOWN-POINTING TRIANGLE" },
    { 0x23F8, "DOUBLE VERTICAL BAR" },
    { 0x23F9, "BLACK SQUARE FOR STOP" },
    { 0x23FA, "BLACK CIRCLE FOR RECORD" },
    { 0x23FB, "POWER SYMBOL" },
    { 0x23FC, "POWER ON-OFF SYMBOL" },
    { 0x23FD, "POWER ON SYMBOL" },
    { 0x23FE, "POWER SLEEP SYMBOL" },
    { 0x23FF, "OBSERVER EYE SYMBOL" },
    { 0x2400, "SYMBOL FOR NULL" },
    { 0x2401, "SYMBOL FOR START OF HEADING" },
    { 0x2402, "SYMBOL FOR START OF TEXT" },
    { 0x2403, "SYMBOL FOR END OF TEXT" },
    { 0x2404, "SYMBOL FOR END OF TRANSMISSION" },
    { 0x2405, "SYMBOL FOR ENQUIRY" },
    { 0x2406, "SYMBOL FOR ACKNOWLEDGE" },
    { 0x2407, "SYMBOL FOR BELL" },
    { 0x2408, "SYMBOL FOR BACKSPACE" },
    { 0x2409, "SYMBOL FOR HORIZONTAL TABULATION" },
    { 0x240A, "SYMBOL FOR LINE FEED" },
    { 0x240B, "SYMBOL FOR VERTICAL TABULATION" },
    { 0x240C, "SYMBOL FOR FORM FEED" },
    { 0x240D, "SYMBOL FOR CARRIAGE RETURN" },
    { 0x240E, "SYMBOL FOR SHIFT OUT" },
    { 0x240F, "SYMBOL FOR SHIFT IN" },
    { 0x2410, "SYMBOL FOR DATA LINK ESCAPE" },
    { 0x2411, "SYMBOL FOR DEVICE CONTROL ONE" },
    { 0x2412, "SYMBOL FOR DEVICE CONTROL TWO" },
    { 0x2413, "SYMBOL FOR DEVICE CONTROL THREE" },
    { 0x2414, "SYMBOL FOR DEVICE CONTROL FOUR" },
    { 0x2415, "SYMBOL FOR NEGATIVE ACKNOWLEDGE" },
    { 0x2416, "SYMBOL FOR SYNCHRONOUS IDLE" },
    { 0x2417, "SYMBOL FOR END OF TRANSMISSION BLOCK" },
    { 0x2418, "SYMBOL FOR CANCEL" },
    { 0x2419, "SYMBOL FOR END OF MEDIUM" },
    { 0x241A, "SYMBOL FOR SUBSTITUTE" },
    { 0x241B, "SYMBOL FOR ESCAPE" },
    { 0x241C, "SYMBOL FOR FILE SEPARATOR" },
    { 0x241D, "SYMBOL FOR GROUP SEPARATOR" },
    { 0x241E, "SYMBOL FOR RECORD SEPARATOR" },
    { 0x241F, "SYMBOL FOR UNIT SEPARATOR" },
    { 0x2420, "SYMBOL FOR SPACE" },
    { 0x2421, "SYMBOL FOR DELETE" },
    { 0x2422, "BLANK SYMBOL" },
    { 0x2423, "OPEN BOX" },
    { 0x2424, "SYMBOL FOR NEWLINE" },
    { 0x2425, "SYMBOL FOR DELETE FORM TWO" },
    { 0x2426, "SYMBOL FOR SUBSTITUTE FORM TWO" },
    { 0x2440, "OCR HOOK" },
    { 0x2441, "OCR CHAIR" },
    { 0x2442, "OCR FORK" },
    { 0x2443, "OCR INVERTED FORK" },
    { 0x2444, "OCR BELT BUCKLE" },
    { 0x2445, "OCR BOW TIE" },
    { 0x2446, "OCR BRANCH BANK IDENTIFICATION" },
    { 0x2447, "OCR AMOUNT OF CHECK" },
    { 0x2448, "OCR DASH" },
    { 0x2449, "OCR CUSTOMER ACCOUNT NUMBER" },
    { 0x244A, "OCR DOUBLE BACKSLASH" },
    { 0x2460, "CIRCLED DIGIT ONE" },
    { 0x2461, "CIRCLED DIGIT TWO" },
    { 0x2462, "CIRCLED DIGIT THREE" },
    { 0x2463, "CIRCLED DIGIT FOUR" },
    { 0x2464, "CIRCLED DIGIT FIVE" },
    { 0x2465, "CIRCLED DIGIT SIX" },
    { 0x2466, "CIRCLED DIGIT SEVEN" },
    { 0x2467, "CIRCLED DIGIT EIGHT" },
    { 0x2468, "CIRCLED DIGIT NINE" },
    { 0x2469, "CIRCLED NUMBER TEN" },
    { 0x246A, "CIRCLED NUMBER ELEVEN" },
    { 0x246B, "CIRCLED NUMBER TWELVE" },
    { 0x246C, "CIRCLED NUMBER THIRTEEN" },
    { 0x246D, "CIRCLED NUMBER FOURTEEN" },
    { 0x246E, "CIRCLED NUMBER FIFTEEN" },
    { 0x246F, "CIRCLED NUMBER SIXTEEN" },
    { 0x2470, "CIRCLED NUMBER SEVENTEEN" },
    { 0x2471, "CIRCLED NUMBER EIGHTEEN" },
    { 0x2472, "CIRCLED NUMBER NINETEEN" },
    { 0x2473, "CIRCLED NUMBER TWENTY" },
    { 0x2474, "PARENTHESIZED DIGIT ONE" },
    { 0x2475, "PARENTHESIZED DIGIT TWO" },
    { 0x2476, "PARENTHESIZED DIGIT THREE" },
    { 0x2477, "PARENTHESIZED DIGIT FOUR" },
    { 0x2478, "PARENTHESIZED DIGIT FIVE" },
    { 0x2479, "PARENTHESIZED DIGIT SIX" },
    { 0x247A, "PARENTHESIZED DIGIT SEVEN" },
    { 0x247B, "PARENTHESIZED DIGIT EIGHT" },
    { 0x247C, "PARENTHESIZED DIGIT NINE" },
    { 0x247D, "PARENTHESIZED NUMBER TEN" },
    { 0x247E, "PARENTHESIZED NUMBER ELEVEN" },
    { 0x247F, "PARENTHESIZED NUMBER TWELVE" },
    { 0x2480, "PARENTHESIZED NUMBER THIRTEEN" },
    { 0x2481, "PARENTHESIZED NUMBER FOURTEEN" },
    { 0x2482, "PARENTHESIZED NUMBER FIFTEEN" },
    { 0x2483, "PARENTHESIZED NUMBER SIXTEEN" },
    { 0x2484, "PARENTHESIZED NUMBER SEVENTEEN" },
    { 0x2485, "PARENTHESIZED NUMBER EIGHTEEN" },
    { 0x2486, "PARENTHESIZED NUMBER NINETEEN" },
    { 0x2487, "PARENTHESIZED NUMBER TWENTY" },
    { 0x2488, "DIGIT ONE FULL STOP" },
    { 0x2489, "DIGIT TWO FULL STOP" },
    { 0x248A, "DIGIT THREE FULL STOP" },
    { 0x248B, "DIGIT FOUR FULL STOP" },
    { 0x248C, "DIGIT FIVE FULL STOP" },
    { 0x248D, "DIGIT SIX FULL STOP" },
    { 0x248E, "DIGIT SEVEN FULL STOP" },
    { 0x248F, "DIGIT EIGHT FULL STOP" },
    { 0x2490, "DIGIT NINE FULL STOP" },
    { 0x2491, "NUMBER TEN FULL STOP" },
    { 0x2492, "NUMBER ELEVEN FULL STOP" },
    { 0x2493, "NUMBER TWELVE FULL STOP" },
    { 0x2494, "NUMBER THIRTEEN FULL STOP" },
    { 0x2495, "NUMBER FOURTEEN FULL STOP" },
    { 0x2496, "NUMBER FIFTEEN FULL STOP" },
    { 0x2497, "NUMBER SIXTEEN FULL STOP" },
    { 0x2498, "NUMBER SEVENTEEN FULL STOP" },
    { 0x2499, "NUMBER EIGHTEEN FULL STOP" },
    { 0x249A, "NUMBER NINETEEN FULL STOP" },
    { 0x249B, "NUMBER TWENTY FULL STOP" },
    { 0x249C, "PARENTHESIZED LATIN SMALL LETTER A" },
    { 0x249D, "PARENTHESIZED LATIN SMALL LETTER B" },
    { 0x249E, "PARENTHESIZED LATIN SMALL LETTER C" },
    { 0x249F, "PARENTHESIZED LATIN SMALL LETTER D" },
    { 0x24A0, "PARENTHESIZED LATIN SMALL LETTER E" },
    { 0x24A1, "PARENTHESIZED LATIN SMALL LETTER F" },
    { 0x24A2, "PARENTHESIZED LATIN SMALL LETTER G" },
    { 0x24A3, "PARENTHESIZED LATIN SMALL LETTER H" },
    { 0x24A4, "PARENTHESIZED LATIN SMALL LETTER I" },
    { 0x24A5, "PARENTHESIZED LATIN SMALL LETTER J" },
    { 0x24A6, "PARENTHESIZED LATIN SMALL LETTER K" },
    { 0x24A7, "PARENTHESIZED LATIN SMALL LETTER L" },
    { 0x24A8, "PARENTHESIZED LATIN SMALL LETTER M" },
    { 0x24A9, "PARENTHESIZED LATIN SMALL LETTER N" },
    { 0x24AA, "PARENTHESIZED LATIN SMALL LETTER O" },
    { 0x24AB, "PARENTHESIZED LATIN SMALL LETTER P" },
    { 0x24AC, "PARENTHESIZED LATIN SMALL LETTER Q" },
    { 0x24AD, "PARENTHESIZED LATIN SMALL LETTER R" },
    { 0x24AE, "PARENTHESIZED LATIN SMALL LETTER S" },
    { 0x24AF, "PARENTHESIZED LATIN SMALL LETTER T" },
    { 0x24B0, "PARENTHESIZED LATIN SMALL LETTER U" },
    { 0x24B1, "PARENTHESIZED LATIN SMALL LETTER V" },
    { 0x24B2, "PARENTHESIZED LATIN SMALL LETTER W" },
    { 0x24B3, "PARENTHESIZED LATIN SMALL LETTER X" },
    { 0x24B4, "PARENTHESIZED LATIN SMALL LETTER Y" },
    { 0x24B5, "PARENTHESIZED LATIN SMALL LETTER Z" },
    { 0x24B6, "CIRCLED LATIN CAPITAL LETTER A" },
    { 0x24B7, "CIRCLED LATIN CAPITAL LETTER B" },
    { 0x24B8, "CIRCLED LATIN CAPITAL LETTER C" },
    { 0x24B9, "CIRCLED LATIN CAPITAL LETTER D" },
    { 0x24BA, "CIRCLED LATIN CAPITAL LETTER E" },
    { 0x24BB, "CIRCLED LATIN CAPITAL LETTER F" },
    { 0x24BC, "CIRCLED LATIN CAPITAL LETTER G" },
    { 0x24BD, "CIRCLED LATIN CAPITAL LETTER H" },
    { 0x24BE, "CIRCLED LATIN CAPITAL LETTER I" },
    { 0x24BF, "CIRCLED LATIN CAPITAL LETTER J" },
    { 0x24C0, "CIRCLED LATIN CAPITAL LETTER K" },
    { 0x24C1, "CIRCLED LATIN CAPITAL LETTER L" },
    { 0x24C2, "CIRCLED LATIN CAPITAL LETTER M" },
    { 0x24C3, "CIRCLED LATIN CAPITAL LETTER N" },
    { 0x24C4, "CIRCLED LATIN CAPITAL LETTER O" },
    { 0x24C5, "CIRCLED LATIN CAPITAL LETTER P" },
    { 0x24C6, "CIRCLED LATIN CAPITAL LETTER Q" },
    { 0x24C7, "CIRCLED LATIN CAPITAL LETTER R" },
    { 0x24C8, "CIRCLED LATIN CAPITAL LETTER S" },
    { 0x24C9, "CIRCLED LATIN CAPITAL LETTER T" },
    { 0x24CA, "CIRCLED LATIN CAPITAL LETTER U" },
    { 0x24CB, "CIRCLED LATIN CAPITAL LETTER V" },
    { 0x24CC, "CIRCLED LATIN CAPITAL LETTER W" },
    { 0x24CD, "CIRCLED LATIN CAPITAL LETTER X" },
    { 0x24CE, "CIRCLED LATIN CAPITAL LETTER Y" },
    { 0x24CF, "CIRCLED LATIN CAPITAL LETTER Z" },
    { 0x24D0, "CIRCLED LATIN SMALL LETTER A" },
    { 0x24D1, "CIRCLED LATIN SMALL LETTER B" },
    { 0x24D2, "CIRCLED LATIN SMALL LETTER C" },
    { 0x24D3, "CIRCLED LATIN SMALL LETTER D" },
    { 0x24D4, "CIRCLED LATIN SMALL LETTER E" },
    { 0x24D5, "CIRCLED LATIN SMALL LETTER F" },
    { 0x24D6, "CIRCLED LATIN SMALL LETTER G" },
    { 0x24D7, "CIRCLED LATIN SMALL LETTER H" },
    { 0x24D8, "CIRCLED LATIN SMALL LETTER I" },
    { 0x24D9, "CIRCLED LATIN SMALL LETTER J" },
    { 0x24DA, "CIRCLED LATIN SMALL LETTER K" },
    { 0x24DB, "CIRCLED LATIN SMALL LETTER L" },
    { 0x24DC, "CIRCLED LATIN SMALL LETTER M" },
    { 0x24DD, "CIRCLED LATIN SMALL LETTER N" },
    { 0x24DE, "CIRCLED LATIN SMALL LETTER O" },
    { 0x24DF, "CIRCLED LATIN SMALL LETTER P" },
    { 0x24E0, "CIRCLED LATIN SMALL LETTER Q" },
    { 0x24E1, "CIRCLED LATIN SMALL LETTER R" },
    { 0x24E2, "CIRCLED LATIN SMALL LETTER S" },
    { 0x24E3, "CIRCLED LATIN SMALL LETTER T" },
    { 0x24E4, "CIRCLED LATIN SMALL LETTER U" },
    { 0x24E5, "CIRCLED LATIN SMALL LETTER V" },
    { 0x24E6, "CIRCLED LATIN SMALL LETTER W" },
    { 0x24E7, "CIRCLED LATIN SMALL LETTER X" },
    { 0x24E8, "CIRCLED LATIN SMALL LETTER Y" },
    { 0x24E9, "CIRCLED LATIN SMALL LETTER Z" },
    { 0x24EA, "CIRCLED DIGIT ZERO" },
    { 0x24EB, "NEGATIVE CIRCLED NUMBER ELEVEN" },
    { 0x24EC, "NEGATIVE CIRCLED NUMBER TWELVE" },
    { 0x24ED, "NEGATIVE CIRCLED NUMBER THIRTEEN" },
    { 0x24EE, "NEGATIVE CIRCLED NUMBER FOURTEEN" },
    { 0x24EF, "NEGATIVE CIRCLED NUMBER FIFTEEN" },
    { 0x24F0, "NEGATIVE CIRCLED NUMBER SIXTEEN" },
    { 0x24F1, "NEGATIVE CIRCLED NUMBER SEVENTEEN" },
    { 0x24F2, "NEGATIVE CIRCLED NUMBER EIGHTEEN" },
    { 0x24F3, "NEGATIVE CIRCLED NUMBER NINETEEN" },
    { 0x24F4, "NEGATIVE CIRCLED NUMBER TWENTY" },
    { 0x24F5, "DOUBLE CIRCLED DIGIT ONE" },
    { 0x24F6, "DOUBLE CIRCLED DIGIT TWO" },
    { 0x24F7, "DOUBLE CIRCLED DIGIT THREE" },
    { 0x24F8, "DOUBLE CIRCLED DIGIT FOUR" },
    { 0x24F9, "DOUBLE CIRCLED DIGIT FIVE" },
    { 0x24FA, "DOUBLE CIRCLED DIGIT SIX" },
    { 0x24FB, "DOUBLE CIRCLED DIGIT SEVEN" },
    { 0x24FC, "DOUBLE CIRCLED DIGIT EIGHT" },
    { 0x24FD, "DOUBLE CIRCLED DIGIT NINE" },
    { 0x24FE, "DOUBLE CIRCLED NUMBER TEN" },
    { 0x24FF, "NEGATIVE CIRCLED DIGIT ZERO" },
    { 0x2500, "BOX DRAWINGS LIGHT HORIZONTAL" },
    { 0x2501, "BOX DRAWINGS HEAVY HORIZONTAL" },
    { 0x2502, "BOX DRAWINGS LIGHT VERTICAL" },
    { 0x2503, "BOX DRAWINGS HEAVY VERTICAL" },
    { 0x2504, "BOX DRAWINGS LIGHT TRIPLE DASH HORIZONTAL" },
    { 0x2505, "BOX DRAWINGS HEAVY TRIPLE DASH HORIZONTAL" },
    { 0x2506, "BOX DRAWINGS LIGHT TRIPLE DASH VERTICAL" },
    { 0x2507, "BOX DRAWINGS HEAVY TRIPLE DASH VERTICAL" },
    { 0x2508, "BOX DRAWINGS LIGHT QUADRUPLE DASH HORIZONTAL" },
    { 0x2509, "BOX DRAWINGS HEAVY QUADRUPLE DASH HORIZONTAL" },
    { 0x250A, "BOX DRAWINGS LIGHT QUADRUPLE DASH VERTICAL" },
    { 0x250B, "BOX DRAWINGS HEAVY QUADRUPLE DASH VERTICAL" },
    { 0x250C, "BOX DRAWINGS LIGHT DOWN AND RIGHT" },
    { 0x250D, "BOX DRAWINGS DOWN LIGHT AND RIGHT HEAVY" },
    { 0x250E, "BOX DRAWINGS DOWN HEAVY AND RIGHT LIGHT" },
    { 0x250F, "BOX DRAWINGS HEAVY DOWN AND RIGHT" },
    { 0x2510, "BOX DRAWINGS LIGHT DOWN AND LEFT" },
    { 0x2511, "BOX DRAWINGS DOWN LIGHT AND LEFT HEAVY" },
    { 0x2512, "BOX DRAWINGS DOWN HEAVY AND LEFT LIGHT" },
    { 0x2513, "BOX DRAWINGS HEAVY DOWN AND LEFT" },
    { 0x2514, "BOX DRAWINGS LIGHT UP AND RIGHT" },
    { 0x2515, "BOX DRAWINGS UP LIGHT AND RIGHT HEAVY" },
    { 0x2516, "BOX DRAWINGS UP HEAVY AND RIGHT LIGHT" },
    { 0x2517, "BOX DRAWINGS HEAVY UP AND RIGHT" },
    { 0x2518, "BOX DRAWINGS LIGHT UP AND LEFT" },
    { 0x2519, "BOX DRAWINGS UP LIGHT AND LEFT HEAVY" },
    { 0x251A, "BOX DRAWINGS UP HEAVY AND LEFT LIGHT" },
    { 0x251B, "BOX DRAWINGS HEAVY UP AND LEFT" },
    { 0x251C, "BOX DRAWINGS LIGHT VERTICAL AND RIGHT" },
    { 0x251D, "BOX DRAWINGS VERTICAL LIGHT AND RIGHT HEAVY" },
    { 0x251E, "BOX DRAWINGS UP HEAVY AND RIGHT DOWN LIGHT" },
    { 0x251F, "BOX DRAWINGS DOWN HEAVY AND RIGHT UP LIGHT" },
    { 0x2520, "BOX DRAWINGS VERTICAL HEAVY AND RIGHT LIGHT" },
    { 0x2521, "BOX DRAWINGS DOWN LIGHT AND RIGHT UP HEAVY" },
    { 0x2522, "BOX DRAWINGS UP LIGHT AND RIGHT DOWN HEAVY" },
    { 0x2523, "BOX DRAWINGS HEAVY VERTICAL AND RIGHT" },
    { 0x2524, "BOX DRAWINGS LIGHT VERTICAL AND LEFT" },
    { 0x2525, "BOX DRAWINGS VERTICAL LIGHT AND LEFT HEAVY" },
    { 0x2526, "BOX DRAWINGS UP HEAVY AND LEFT DOWN LIGHT" },
    { 0x2527, "BOX DRAWINGS DOWN HEAVY AND LEFT UP LIGHT" },
    { 0x2528, "BOX DRAWINGS VERTICAL HEAVY AND LEFT LIGHT" },
    { 0x2529, "BOX DRAWINGS DOWN LIGHT AND LEFT UP HEAVY" },
    { 0x252A, "BOX DRAWINGS UP LIGHT AND LEFT DOWN HEAVY" },
    { 0x252B, "BOX DRAWINGS HEAVY VERTICAL AND LEFT" },
    { 0x252C, "BOX DRAWINGS LIGHT DOWN AND HORIZONTAL" },
    { 0x252D, "BOX DRAWINGS LEFT HEAVY AND RIGHT DOWN LIGHT" },
    { 0x252E, "BOX DRAWINGS RIGHT HEAVY AND LEFT DOWN LIGHT" },
    { 0x252F, "BOX DRAWINGS DOWN LIGHT AND HORIZONTAL HEAVY" },
    { 0x2530, "BOX DRAWINGS DOWN HEAVY AND HORIZONTAL LIGHT" },
    { 0x2531, "BOX DRAWINGS RIGHT LIGHT AND LEFT DOWN HEAVY" },
    { 0x2532, "BOX DRAWINGS LEFT LIGHT AND RIGHT DOWN HEAVY" },
    { 0x2533, "BOX DRAWINGS HEAVY DOWN AND HORIZONTAL" },
    { 0x2534, "BOX DRAWINGS LIGHT UP AND HORIZONTAL" },
    { 0x2535, "BOX DRAWINGS LEFT HEAVY AND RIGHT UP LIGHT" },
    { 0x2536, "BOX DRAWINGS RIGHT HEAVY AND LEFT UP LIGHT" },
    { 0x2537, "BOX DRAWINGS UP LIGHT AND HORIZONTAL HEAVY" },
    { 0x2538, "BOX DRAWINGS UP HEAVY AND HORIZONTAL LIGHT" },
    { 0x2539, "BOX DRAWINGS RIGHT LIGHT AND LEFT UP HEAVY" },
    { 0x253A, "BOX DRAWINGS LEFT LIGHT AND RIGHT UP HEAVY" },
    { 0x253B, "BOX DRAWINGS HEAVY UP AND HORIZONTAL" },
    { 0x253C, "BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL" },
    { 0x253D, "BOX DRAWINGS LEFT HEAVY AND RIGHT VERTICAL LIGHT" },
    { 0x253E, "BOX DRAWINGS RIGHT HEAVY AND LEFT VERTICAL LIGHT" },
    { 0x253F, "BOX DRAWINGS VERTICAL LIGHT AND HORIZONTAL HEAVY" },
    { 0x2540, "BOX DRAWINGS UP HEAVY AND DOWN HORIZONTAL LIGHT" },
    { 0x2541, "BOX DRAWINGS DOWN HEAVY AND UP HORIZONTAL LIGHT" },
    { 0x2542, "BOX DRAWINGS VERTICAL HEAVY AND HORIZONTAL LIGHT" },
    { 0x2543, "BOX DRAWINGS LEFT UP HEAVY AND RIGHT DOWN LIGHT" },
    { 0x2544, "BOX DRAWINGS RIGHT UP HEAVY AND LEFT DOWN LIGHT" },
    { 0x2545, "BOX DRAWINGS LEFT DOWN HEAVY AND RIGHT UP LIGHT" },
    { 0x2546, "BOX DRAWINGS RIGHT DOWN HEAVY AND LEFT UP LIGHT" },
    { 0x2547, "BOX DRAWINGS DOWN LIGHT AND UP HORIZONTAL HEAVY" },
    { 0x2548, "BOX DRAWINGS UP LIGHT AND DOWN HORIZONTAL HEAVY" },
    { 0x2549, "BOX DRAWINGS RIGHT LIGHT AND LEFT VERTICAL HEAVY" },
    { 0x254A, "BOX DRAWINGS LEFT LIGHT AND RIGHT VERTICAL HEAVY" },
    { 0x254B, "BOX DRAWINGS HEAVY VERTICAL AND HORIZONTAL" },
    { 0x254C, "BOX DRAWINGS LIGHT DOUBLE DASH HORIZONTAL" },
    { 0x254D, "BOX DRAWINGS HEAVY DOUBLE DASH HORIZONTAL" },
    { 0x254E, "BOX DRAWINGS LIGHT DOUBLE DASH VERTICAL" },
    { 0x254F, "BOX DRAWINGS HEAVY DOUBLE DASH VERTICAL" },
    { 0x2550, "BOX DRAWINGS DOUBLE HORIZONTAL" },
    { 0x2551, "BOX DRAWINGS DOUBLE VERTICAL" },
    { 0x2552, "BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE" },
    { 0x2553, "BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE" },
    { 0x2554, "BOX DRAWINGS DOUBLE DOWN AND RIGHT" },
    { 0x2555, "BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE" },
    { 0x2556, "BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE" },
    { 0x2557, "BOX DRAWINGS DOUBLE DOWN AND LEFT" },
    { 0x2558, "BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE" },
    { 0x2559, "BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE" },
    { 0x255A, "BOX DRAWINGS DOUBLE UP AND RIGHT" },
    { 0x255B, "BOX DRAWINGS UP SINGLE AND LEFT DOUBLE" },
    { 0x255C, "BOX DRAWINGS UP DOUBLE AND LEFT SINGLE" },
    { 0x255D, "BOX DRAWINGS DOUBLE UP AND LEFT" },
    { 0x255E, "BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE" },
    { 0x255F, "BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE" },
    { 0x2560, "BOX DRAWINGS DOUBLE VERTICAL AND RIGHT" },
    { 0x2561, "BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE" },
    { 0x2562, "BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE" },
    { 0x2563, "BOX DRAWINGS DOUBLE VERTICAL AND LEFT" },
    { 0x2564, "BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE" },
    { 0x2565, "BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE" },
    { 0x2566, "BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL" },
    { 0x2567, "BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE" },
    { 0x2568, "BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE" },
    { 0x2569, "BOX DRAWINGS DOUBLE UP AND HORIZONTAL" },
    { 0x256A, "BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE" },
    { 0x256B, "BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE" },
    { 0x256C, "BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL" },
    { 0x256D, "BOX DRAWINGS LIGHT ARC DOWN AND RIGHT" },
    { 0x256E, "BOX DRAWINGS LIGHT ARC DOWN AND LEFT" },
    { 0x256F, "BOX DRAWINGS LIGHT ARC UP AND LEFT" },
    { 0x2570, "BOX DRAWINGS LIGHT ARC UP AND RIGHT" },
    { 0x2571, "BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT" },
    { 0x2572, "BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT" },
    { 0x2573, "BOX DRAWINGS LIGHT DIAGONAL CROSS" },
    { 0x2574, "BOX DRAWINGS LIGHT LEFT" },
    { 0x2575, "BOX DRAWINGS LIGHT UP" },
    { 0x2576, "BOX DRAWINGS LIGHT RIGHT" },
    { 0x2577, "BOX DRAWINGS LIGHT DOWN" },
    { 0x2578, "BOX DRAWINGS HEAVY LEFT" },
    { 0x2579, "BOX DRAWINGS HEAVY UP" },
    { 0x257A, "BOX DRAWINGS HEAVY RIGHT" },
    { 0x257B, "BOX DRAWINGS HEAVY DOWN" },
    { 0x257C, "BOX DRAWINGS LIGHT LEFT AND HEAVY RIGHT" },
    { 0x257D, "BOX DRAWINGS LIGHT UP AND HEAVY DOWN" },
    { 0x257E, "BOX DRAWINGS HEAVY LEFT AND LIGHT RIGHT" },
    { 0x257F, "BOX DRAWINGS HEAVY UP AND LIGHT DOWN" },
    { 0x2580, "UPPER HALF BLOCK" },
    { 0x2581, "LOWER ONE EIGHTH BLOCK" },
    { 0x2582, "LOWER ONE QUARTER BLOCK" },
    { 0x2583, "LOWER THREE EIGHTHS BLOCK" },
    { 0x2584, "LOWER HALF BLOCK" },
    { 0x2585, "LOWER FIVE EIGHTHS BLOCK" },
    { 0x2586, "LOWER THREE QUARTERS BLOCK" },
    { 0x2587, "LOWER SEVEN EIGHTHS BLOCK" },
    { 0x2588, "FULL BLOCK" },
    { 0x2589, "LEFT SEVEN EIGHTHS BLOCK" },
    { 0x258A, "LEFT THREE QUARTERS BLOCK" },
    { 0x258B, "LEFT FIVE EIGHTHS BLOCK" },
    { 0x258C, "LEFT HALF BLOCK" },
    { 0x258D, "LEFT THREE EIGHTHS BLOCK" },
    { 0x258E, "LEFT ONE QUARTER BLOCK" },
    { 0x258F, "LEFT ONE EIGHTH BLOCK" },
    { 0x2590, "RIGHT HALF BLOCK" },
    { 0x2591, "LIGHT SHADE" },
    { 0x2592, "MEDIUM SHADE" },
    { 0x2593, "DARK SHADE" },
    { 0x2594, "UPPER ONE EIGHTH BLOCK" },
    { 0x2595, "RIGHT ONE EIGHTH BLOCK" },
    { 0x2596, "QUADRANT LOWER LEFT" },
    { 0x2597, "QUADRANT LOWER RIGHT" },
    { 0x2598, "QUADRANT UPPER LEFT" },
    { 0x2599, "QUADRANT UPPER LEFT AND LOWER LEFT AND LOWER RIGHT" },
    { 0x259A, "QUADRANT UPPER LEFT AND LOWER RIGHT" },
    { 0x259B, "QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER LEFT" },
    { 0x259C, "QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER RIGHT" },
    { 0x259D, "QUADRANT UPPER RIGHT" },
    { 0x259E, "QUADRANT UPPER RIGHT AND LOWER LEFT" },
    { 0x259F, "QUADRANT UPPER RIGHT AND LOWER LEFT AND LOWER RIGHT" },
    { 0x25A0, "BLACK SQUARE" },
    { 0x25A1, "WHITE SQUARE" },
    { 0x25A2, "WHITE SQUARE WITH ROUNDED CORNERS" },
    { 0x25A3, "WHITE SQUARE CONTAINING BLACK SMALL SQUARE" },
    { 0x25A4, "SQUARE WITH HORIZONTAL FILL" },
    { 0x25A5, "SQUARE WITH VERTICAL FILL" },
    { 0x25A6, "SQUARE WITH ORTHOGONAL CROSSHATCH FILL" },
    { 0x25A7, "SQUARE WITH UPPER LEFT TO LOWER RIGHT FILL" },
    { 0x25A8, "SQUARE WITH UPPER RIGHT TO LOWER LEFT FILL" },
    { 0x25A9, "SQUARE WITH DIAGONAL CROSSHATCH FILL" },
    { 0x25AA, "BLACK SMALL SQUARE" },
    { 0x25AB, "WHITE SMALL SQUARE" },
    { 0x25AC, "BLACK RECTANGLE" },
    { 0x25AD, "WHITE RECTANGLE" },
    { 0x25AE, "BLACK VERTICAL RECTANGLE" },
    { 0x25AF, "WHITE VERTICAL RECTANGLE" },
    { 0x25B0, "BLACK PARALLELOGRAM" },
    { 0x25B1, "WHITE PARALLELOGRAM" },
    { 0x25B2, "BLACK UP-POINTING TRIANGLE" },
    { 0x25B3, "WHITE UP-POINTING TRIANGLE" },
    { 0x25B4, "BLACK UP-POINTING SMALL TRIANGLE" },
    { 0x25B5, "WHITE UP-POINTING SMALL TRIANGLE" },
    { 0x25B6, "BLACK RIGHT-POINTING TRIANGLE" },
    { 0x25B7, "WHITE RIGHT-POINTING TRIANGLE" },
    { 0x25B8, "BLACK RIGHT-POINTING SMALL TRIANGLE" },
    { 0x25B9, "WHITE RIGHT-POINTING SMALL TRIANGLE" },
    { 0x25BA, "BLACK RIGHT-POINTING POINTER" },
    { 0x25BB, "WHITE RIGHT-POINTING POINTER" },
    { 0x25BC, "BLACK DOWN-POINTING TRIANGLE" },
    { 0x25BD, "WHITE DOWN-POINTING TRIANGLE" },
    { 0x25BE, "BLACK DOWN-POINTING SMALL TRIANGLE" },
    { 0x25BF, "WHITE DOWN-POINTING SMALL TRIANGLE" },
    { 0x25C0, "BLACK LEFT-POINTING TRIANGLE" },
    { 0x25C1, "WHITE LEFT-POINTING TRIANGLE" },
    { 0x25C2, "BLACK LEFT-POINTING SMALL TRIANGLE" },
    { 0x25C3, "WHITE LEFT-POINTING SMALL TRIANGLE" },
    { 0x25C4, "BLACK LEFT-POINTING POINTER" },
    { 0x25C5, "WHITE LEFT-POINTING POINTER" },
    { 0x25C6, "BLACK DIAMOND" },
    { 0x25C7, "WHITE DIAMOND" },
    { 0x25C8, "WHITE DIAMOND CONTAINING BLACK SMALL DIAMOND" },
    { 0x25C9, "FISHEYE" },
    { 0x25CA, "LOZENGE" },
    { 0x25CB, "WHITE CIRCLE" },
    { 0x25CC, "DOTTED CIRCLE" },
    { 0x25CD, "CIRCLE WITH VERTICAL FILL" },
    { 0x25CE, "BULLSEYE" },
    { 0x25CF, "BLACK CIRCLE" },
    { 0x25D0, "CIRCLE WITH LEFT HALF BLACK" },
    { 0x25D1, "CIRCLE WITH RIGHT HALF BLACK" },
    { 0x25D2, "CIRCLE WITH LOWER HALF BLACK" },
    { 0x25D3, "CIRCLE WITH UPPER HALF BLACK" },
    { 0x25D4, "CIRCLE WITH UPPER RIGHT QUADRANT BLACK" },
    { 0x25D5, "CIRCLE WITH ALL BUT UPPER LEFT QUADRANT BLACK" },
    { 0x25D6, "LEFT HALF BLACK CIRCLE" },
    { 0x25D7, "RIGHT HALF BLACK CIRCLE" },
    { 0x25D8, "INVERSE BULLET" },
    { 0x25D9, "INVERSE WHITE CIRCLE" },
    { 0x25DA, "UPPER HALF INVERSE WHITE CIRCLE" },
    { 0x25DB, "LOWER HALF INVERSE WHITE CIRCLE" },
    { 0x25DC, "UPPER LEFT QUADRANT CIRCULAR ARC" },
    { 0x25DD, "UPPER RIGHT QUADRANT CIRCULAR ARC" },
    { 0x25DE, "LOWER RIGHT QUADRANT CIRCULAR ARC" },
    { 0x25DF, "LOWER LEFT QUADRANT CIRCULAR ARC" },
    { 0x25E0, "UPPER HALF CIRCLE" },
    { 0x25E1, "LOWER HALF CIRCLE" },
    { 0x25E2, "BLACK LOWER RIGHT TRIANGLE" },
    { 0x25E3, "BLACK LOWER LEFT TRIANGLE" },
    { 0x25E4, "BLACK UPPER LEFT TRIANGLE" },
    { 0x25E5, "BLACK UPPER RIGHT TRIANGLE" },
    { 0x25E6, "WHITE BULLET" },
    { 0x25E7, "SQUARE WITH LEFT HALF BLACK" },
    { 0x25E8, "SQUARE WITH RIGHT HALF BLACK" },
    { 0x25E9, "SQUARE WITH UPPER LEFT DIAGONAL HALF BLACK" },
    { 0x25EA, "SQUARE WITH LOWER RIGHT DIAGONAL HALF BLACK" },
    { 0x25EB, "WHITE SQUARE WITH VERTICAL BISECTING LINE" },
    { 0x25EC, "WHITE UP-POINTING TRIANGLE WITH DOT" },
    { 0x25ED, "UP-POINTING TRIANGLE WITH LEFT HALF BLACK" },
    { 0x25EE, "UP-POINTING TRIANGLE WITH RIGHT HALF BLACK" },
    { 0x25EF, "LARGE CIRCLE" },
    { 0x25F0, "WHITE SQUARE WITH UPPER LEFT QUADRANT" },
    { 0x25F1, "WHITE SQUARE WITH LOWER LEFT QUADRANT" },
    { 0x25F2, "WHITE SQUARE WITH LOWER RIGHT QUADRANT" },
    { 0x25F3, "WHITE SQUARE WITH UPPER RIGHT QUADRANT" },
    { 0x25F4, "WHITE CIRCLE WITH UPPER LEFT QUADRANT" },
    { 0x25F5, "WHITE CIRCLE WITH LOWER LEFT QUADRANT" },
    { 0x25F6, "WHITE CIRCLE WITH LOWER RIGHT QUADRANT" },
    { 0x25F7, "WHITE CIRCLE WITH UPPER RIGHT QUADRANT" },
    { 0x25F8, "UPPER LEFT TRIANGLE" },
    { 0x25F9, "UPPER RIGHT TRIANGLE" },
    { 0x25FA, "LOWER LEFT TRIANGLE" },
    { 0x25FB, "WHITE MEDIUM SQUARE" },
    { 0x25FC, "BLACK MEDIUM SQUARE" },
    { 0x25FF, "LOWER RIGHT TRIANGLE" },
    { 0x2600, "BLACK SUN WITH RAYS" },
    { 0x2601, "CLOUD" },
    { 0x2602, "UMBRELLA" },
    { 0x2603, "SNOWMAN" },
    { 0x2604, "COMET" },
    { 0x2605, "BLACK STAR" },
    { 0x2606, "WHITE STAR" },
    { 0x2607, "LIGHTNING" },
    { 0x2608, "THUNDERSTORM" },
    { 0x2609, "SUN" },
    { 0x260A, "ASCENDING NODE" },
    { 0x260B, "DESCENDING NODE" },
    { 0x260C, "CONJUNCTION" },
    { 0x260D, "OPPOSITION" },
    { 0x260E, "BLACK TELEPHONE" },
    { 0x260F, "WHITE TELEPHONE" },
    { 0x2610, "BALLOT BOX" },
    { 0x2611, "BALLOT BOX WITH CHECK" },
    { 0x2612, "BALLOT BOX WITH X" },
    { 0x2613, "SALTIRE" },
    { 0x2616, "WHITE SHOGI PIECE" },
    { 0x2617, "BLACK SHOGI PIECE" },
    { 0x2618, "SHAMROCK" },
    { 0x2619, "REVERSED ROTATED FLORAL HEART BULLET" },
    { 0x261A, "BLACK LEFT POINTING INDEX" },
    { 0x261B, "BLACK RIGHT POINTING INDEX" },
    { 0x261C, "WHITE LEFT POINTING INDEX" },
    { 0x261D, "WHITE UP POINTING INDEX" },
    { 0x261E, "WHITE RIGHT POINTING INDEX" },
    { 0x261F, "WHITE DOWN POINTING INDEX" },
    { 0x2620, "SKULL AND CROSSBONES" },
    { 0x2621, "CAUTION SIGN" },
    { 0x2622, "RADIOACTIVE SIGN" },
    { 0x2623, "BIOHAZARD SIGN" },
    { 0x2624, "CADUCEUS" },
    { 0x2625, "ANKH" },
    { 0x2626, "ORTHODOX CROSS" },
    { 0x2627, "CHI RHO" },
    { 0x2628, "CROSS OF LORRAINE" },
    { 0x2629, "CROSS OF JERUSALEM" },
    { 0x262A, "STAR AND CRESCENT" },
    { 0x262B, "FARSI SYMBOL" },
    { 0x262C, "ADI SHAKTI" },
    { 0x262D, "HAMMER AND SICKLE" },
    { 0x262E, "PEACE SYMBOL" },
    { 0x262F, "YIN YANG" },
    { 0x2630, "TRIGRAM FOR HEAVEN" },
    { 0x2631, "TRIGRAM FOR LAKE" },
    { 0x2632, "TRIGRAM FOR FIRE" },
    { 0x2633, "TRIGRAM FOR THUNDER" },
    { 0x2634, "TRIGRAM FOR WIND" },
    { 0x2635, "TRIGRAM FOR WATER" },
    { 0x2636, "TRIGRAM FOR MOUNTAIN" },
    { 0x2637, "TRIGRAM FOR EARTH" },
    { 0x2638, "WHEEL OF DHARMA" },
    { 0x2639, "WHITE FROWNING FACE" },
    { 0x263A, "WHITE SMILING FACE" },
    { 0x263B, "BLACK SMILING FACE" },
    { 0x263C, "WHITE SUN WITH RAYS" },
    { 0x263D, "FIRST QUARTER MOON" },
    { 0x263E, "LAST QUARTER MOON" },
    { 0x263F, "MERCURY" },
    { 0x2640, "FEMALE SIGN" },
    { 0x2641, "EARTH" },
    { 0x2642, "MALE SIGN" },
    { 0x2643, "JUPITER" },
    { 0x2644, "SATURN" },
    { 0x2645, "URANUS" },
    { 0x2646, "NEPTUNE" },
    { 0x2647, "PLUTO" },
    { 0x2654, "WHITE CHESS KING" },
    { 0x2655, "WHITE CHESS QUEEN" },
    { 0x2656, "WHITE CHESS ROOK" },
    { 0x2657, "WHITE CHESS BISHOP" },
    { 0x2658, "WHITE CHESS KNIGHT" },
    { 0x2659, "WHITE CHESS PAWN" },
    { 0x265A, "BLACK CHESS KING" },
    { 0x265B, "BLACK CHESS QUEEN" },
    { 0x265C, "BLACK CHESS ROOK" },
    { 0x265D, "BLACK CHESS BISHOP" },
    { 0x265E, "BLACK CHESS KNIGHT" },
    { 0x265F, "BLACK CHESS PAWN" },
    { 0x2660, "BLACK SPADE SUIT" },
    { 0x2661, "WHITE HEART SUIT" },
    { 0x2662, "WHITE DIAMOND SUIT" },
    { 0x2663, "BLACK CLUB SUIT" },
    { 0x2664, "WHITE SPADE SUIT" },
    { 0x2665, "BLACK HEART SUIT" },
    { 0x2666, "BLACK DIAMOND SUIT" },
    { 0x2667, "WHITE CLUB SUIT" },
    { 0x2668, "HOT SPRINGS" },
    { 0x2669, "QUARTER NOTE" },
    { 0x266A, "EIGHTH NOTE" },
    { 0x266B, "BEAMED EIGHTH NOTES" },
    { 0x266C, "BEAMED SIXTEENTH NOTES" },
    { 0x266D, "MUSIC FLAT SIGN" },
    { 0x266E, "MUSIC NATURAL SIGN" },
    { 0x266F, "MUSIC SHARP SIGN" },
    { 0x2670, "WEST SYRIAC CROSS" },
    { 0x2671, "EAST SYRIAC CROSS" },
    { 0x2672, "UNIVERSAL RECYCLING SYMBOL" },
    { 0x2673, "RECYCLING SYMBOL FOR TYPE-1 PLASTICS" },
    { 0x2674, "RECYCLING SYMBOL FOR TYPE-2 PLASTICS" },
    { 0x2675, "RECYCLING SYMBOL FOR TYPE-3 PLASTICS" },
    { 0x2676, "RECYCLING SYMBOL FOR TYPE-4 PLASTICS" },
    { 0x2677, "RECYCLING SYMBOL FOR TYPE-5 PLASTICS" },
    { 0x2678, "RECYCLING SYMBOL FOR TYPE-6 PLASTICS" },
    { 0x2679, "RECYCLING SYMBOL FOR TYPE-7 PLASTICS" },
    { 0x267A, "RECYCLING SYMBOL FOR GENERIC MATERIALS" },
    { 0x267B, "BLACK UNIVERSAL RECYCLING SYMBOL" },
    { 0x267C, "RECYCLED PAPER SYMBOL" },
    { 0x267D, "PARTIALLY-RECYCLED PAPER SYMBOL" },
    { 0x267E, "PERMANENT PAPER SIGN" },
    { 0x2680, "DIE FACE-1" },
    { 0x2681, "DIE FACE-2" },
    { 0x2682, "DIE FACE-3" },
    { 0x2683, "DIE FACE-4" },
    { 0x2684, "DIE FACE-5" },
    { 0x2685, "DIE FACE-6" },
    { 0x2686, "WHITE CIRCLE WITH DOT RIGHT" },
    { 0x2687, "WHITE CIRCLE WITH TWO DOTS" },
    { 0x2688, "BLACK CIRCLE WITH WHITE DOT RIGHT" },
    { 0x2689, "BLACK CIRCLE WITH TWO WHITE DOTS" },
    { 0x268A, "MONOGRAM FOR YANG" },
    { 0x268B, "MONOGRAM FOR YIN" },
    { 0x268C, "DIGRAM FOR GREATER YANG" },
    { 0x268D, "DIGRAM FOR LESSER YIN" },
    { 0x268E, "DIGRAM FOR LESSER YANG" },
    { 0x268F, "DIGRAM FOR GREATER YIN" },
    { 0x2690, "WHITE FLAG" },
    { 0x2691, "BLACK FLAG" },
    { 0x2692, "HAMMER AND PICK" },
    { 0x2694, "CROSSED SWORDS" },
    { 0x2695, "STAFF OF AESCULAPIUS" },
    { 0x2696, "SCALES" },
    { 0x2697, "ALEMBIC" },
    { 0x2698, "FLOWER" },
    { 0x2699, "GEAR" },
    { 0x269A, "STAFF OF HERMES" },
    { 0x269B, "ATOM SYMBOL" },
    { 0x269C, "FLEUR-DE-LIS" },
    { 0x269D, "OUTLINED WHITE STAR" },
    { 0x269E, "THREE LINES CONVERGING RIGHT" },
    { 0x269F, "THREE LINES CONVERGING LEFT" },
    { 0x26A0, "WARNING SIGN" },
    { 0x26A2, "DOUBLED FEMALE SIGN" },
    { 0x26A3, "DOUBLED MALE SIGN" },
    { 0x26A4, "INTERLOCKED FEMALE AND MALE SIGN" },
    { 0x26A5, "MALE AND FEMALE SIGN" },
    { 0x26A6, "MALE WITH STROKE SIGN" },
    { 0x26A7, "MALE WITH STROKE AND MALE AND FEMALE SIGN" },
    { 0x26A8, "VERTICAL MALE WITH STROKE SIGN" },
    { 0x26A9, "HORIZONTAL MALE WITH STROKE SIGN" },
    { 0x26AC, "MEDIUM SMALL WHITE CIRCLE" },
    { 0x26AD, "MARRIAGE SYMBOL" },
    { 0x26AE, "DIVORCE SYMBOL" },
    { 0x26AF, "UNMARRIED PARTNERSHIP SYMBOL" },
    { 0x26B0, "COFFIN" },
    { 0x26B1, "FUNERAL URN" },
    { 0x26B2, "NEUTER" },
    { 0x26B3, "CERES" },
    { 0x26B4, "PALLAS" },
    { 0x26B5, "JUNO" },
    { 0x26B6, "VESTA" },
    { 0x26B7, "CHIRON" },
    { 0x26B8, "BLACK MOON LILITH" },
    { 0x26B9, "SEXTILE" },
    { 0x26BA, "SEMISEXTILE" },
    { 0x26BB, "QUINCUNX" },
    { 0x26BC, "SESQUIQUADRATE" },
    { 0x26BF, "SQUARED KEY" },
    { 0x26C0, "WHITE DRAUGHTS MAN" },
    { 0x26C1, "WHITE DRAUGHTS KING" },
    { 0x26C2, "BLACK DRAUGHTS MAN" },
    { 0x26C3, "BLACK DRAUGHTS KING" },
    { 0x26C6, "RAIN" },
    { 0x26C7, "BLACK SNOWMAN" },
    { 0x26C8, "THUNDER CLOUD AND RAIN" },
    { 0x26C9, "TURNED WHITE SHOGI PIECE" },
    { 0x26CA, "TURNED BLACK SHOGI PIECE" },
    { 0x26CB, "WHITE DIAMOND IN SQUARE" },
    { 0x26CC, "CROSSING LANES" },
    { 0x26CD, "DISABLED CAR" },
    { 0x26CF, "PICK" },
    { 0x26D0, "CAR SLIDING" },
    { 0x26D1, "HELMET WITH WHITE CROSS" },
    { 0x26D2, "CIRCLED CROSSING LANES" },
    { 0x26D3, "CHAINS" },
    { 0x26D5, "ALTERNATE ONE-WAY LEFT WAY TRAFFIC" },
    { 0x26D6, "BLACK TWO-WAY LEFT WAY TRAFFIC" },
    { 0x26D7, "WHITE TWO-WAY LEFT WAY TRAFFIC" },
    { 0x26D8, "BLACK LEFT LANE MERGE" },
    { 0x26D9, "WHITE LEFT LANE MERGE" },
    { 0x26DA, "DRIVE SLOW SIGN" },
    { 0x26DB, "HEAVY WHITE DOWN-POINTING TRIANGLE" },
    { 0x26DC, "LEFT CLOSED ENTRY" },
    { 0x26DD, "SQUARED SALTIRE" },
    { 0x26DE, "FALLING DIAGONAL IN WHITE CIRCLE IN BLACK SQUARE" },
    { 0x26DF, "BLACK TRUCK" },
    { 0x26E0, "RESTRICTED LEFT ENTRY-1" },
    { 0x26E1, "RESTRICTED LEFT ENTRY-2" },
    { 0x26E2, "ASTRONOMICAL SYMBOL FOR URANUS" },
    { 0x26E3, "HEAVY CIRCLE WITH STROKE AND TWO DOTS ABOVE" },
    { 0x26E4, "PENTAGRAM" },
    { 0x26E5, "RIGHT-HANDED INTERLACED PENTAGRAM" },
    { 0x26E6, "LEFT-HANDED INTERLACED PENTAGRAM" },
    { 0x26E7, "INVERTED PENTAGRAM" },
    { 0x26E8, "BLACK CROSS ON SHIELD" },
    { 0x26E9, "SHINTO SHRINE" },
    { 0x26EB, "CASTLE" },
    { 0x26EC, "HISTORIC SITE" },
    { 0x26ED, "GEAR WITHOUT HUB" },
    { 0x26EE, "GEAR WITH HANDLES" },
    { 0x26EF, "MAP SYMBOL FOR LIGHTHOUSE" },
    { 0x26F0, "MOUNTAIN" },
    { 0x26F1, "UMBRELLA ON GROUND" },
    { 0x26F4, "FERRY" },
    { 0x26F6, "SQUARE FOUR CORNERS" },
    { 0x26F7, "SKIER" },
    { 0x26F8, "ICE SKATE" },
    { 0x26F9, "PERSON WITH BALL" },
    { 0x26FB, "JAPANESE BANK SYMBOL" },
    { 0x26FC, "HEADSTONE GRAVEYARD SYMBOL" },
    { 0x26FE, "CUP ON BLACK SQUARE" },
    { 0x26FF, "WHITE FLAG WITH HORIZONTAL MIDDLE BLACK STRIPE" },
    { 0x2700, "BLACK SAFETY SCISSORS" },
    { 0x2701, "UPPER BLADE SCISSORS" },
    { 0x2702, "BLACK SCISSORS" },
    { 0x2703, "LOWER BLADE SCISSORS" },
    { 0x2704, "WHITE SCISSORS" },
    { 0x2706, "TELEPHONE LOCATION SIGN" },
    { 0x2707, "TAPE DRIVE" },
    { 0x2708, "AIRPLANE" },
    { 0x2709, "ENVELOPE" },
    { 0x270C, "VICTORY HAND" },
    { 0x270D, "WRITING HAND" },
    { 0x270E, "LOWER RIGHT PENCIL" },
    { 0x270F, "PENCIL" },
    { 0x2710, "UPPER RIGHT PENCIL" },
    { 0x2711, "WHITE NIB" },
    { 0x2712, "BLACK NIB" },
    { 0x2713, "CHECK MARK" },
    { 0x2714, "HEAVY CHECK MARK" },
    { 0x2715, "MULTIPLICATION X" },
    { 0x2716, "HEAVY MULTIPLICATION X" },
    { 0x2717, "BALLOT X" },
    { 0x2718, "HEAVY BALLOT X" },
    { 0x2719, "OUTLINED GREEK CROSS" },
    { 0x271A, "HEAVY GREEK CROSS" },
    { 0x271B, "OPEN CENTRE CROSS" },
    { 0x271C, "HEAVY OPEN CENTRE CROSS" },
    { 0x271D, "LATIN CROSS" },
    { 0x271E, "SHADOWED WHITE LATIN CROSS" },
    { 0x271F, "OUTLINED LATIN CROSS" },
    { 0x2720, "MALTESE CROSS" },
    { 0x2721, "STAR OF DAVID" },
    { 0x2722, "FOUR TEARDROP-SPOKED ASTERISK" },
    { 0x2723, "FOUR BALLOON-SPOKED ASTERISK" },
    { 0x2724, "HEAVY FOUR BALLOON-SPOKED ASTERISK" },
    { 0x2725, "FOUR CLUB-SPOKED ASTERISK" },
    { 0x2726, "BLACK FOUR POINTED STAR" },
    { 0x2727, "WHITE FOUR POINTED STAR" },
    { 0x2729, "STRESS OUTLINED WHITE STAR" },
    { 0x272A, "CIRCLED WHITE STAR" },
    { 0x272B, "OPEN CENTRE BLACK STAR" },
    { 0x272C, "BLACK CENTRE WHITE STAR" },
    { 0x272D, "OUTLINED BLACK STAR" },
    { 0x272E, "HEAVY OUTLINED BLACK STAR" },
    { 0x272F, "PINWHEEL STAR" },
    { 0x2730, "SHADOWED WHITE STAR" },
    { 0x2731, "HEAVY ASTERISK" },
    { 0x2732, "OPEN CENTRE ASTERISK" },
    { 0x2733, "EIGHT SPOKED ASTERISK" },
    { 0x2734, "EIGHT POINTED BLACK STAR" },
    { 0x2735, "EIGHT POINTED PINWHEEL STAR" },
    { 0x2736, "SIX POINTED BLACK STAR" },
    { 0x2737, "EIGHT POINTED RECTILINEAR BLACK STAR" },
    { 0x2738, "HEAVY EIGHT POINTED RECTILINEAR BLACK STAR" },
    { 0x2739, "TWELVE POINTED BLACK STAR" },
    { 0x273A, "SIXTEEN POINTED ASTERISK" },
    { 0x273B, "TEARDROP-SPOKED ASTERISK" },
    { 0x273C, "OPEN CENTRE TEARDROP-SPOKED ASTERISK" },
    { 0x273D, "HEAVY TEARDROP-SPOKED ASTERISK" },
    { 0x273E, "SIX PETALLED BLACK AND WHITE FLORETTE" },
    { 0x273F, "BLACK FLORETTE" },
    { 0x2740, "WHITE FLORETTE" },
    { 0x2741, "EIGHT PETALLED OUTLINED BLACK FLORETTE" },
    { 0x2742, "CIRCLED OPEN CENTRE EIGHT POINTED STAR" },
    { 0x2743, "HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK" },
    { 0x2744, "SNOWFLAKE" },
    { 0x2745, "TIGHT TRIFOLIATE SNOWFLAKE" },
    { 0x2746, "HEAVY CHEVRON SNOWFLAKE" },
    { 0x2747, "SPARKLE" },
    { 0x2748, "HEAVY SPARKLE" },
    { 0x2749, "BALLOON-SPOKED ASTERISK" },
    { 0x274A, "EIGHT TEARDROP-SPOKED PROPELLER ASTERISK" },
    { 0x274B, "HEAVY EIGHT TEARDROP-SPOKED PROPELLER ASTERISK" },
    { 0x274D, "SHADOWED WHITE CIRCLE" },
    { 0x274F, "LOWER RIGHT DROP-SHADOWED WHITE SQUARE" },
    { 0x2750, "UPPER RIGHT DROP-SHADOWED WHITE SQUARE" },
    { 0x2751, "LOWER RIGHT SHADOWED WHITE SQUARE" },
    { 0x2752, "UPPER RIGHT SHADOWED WHITE SQUARE" },
    { 0x2756, "BLACK DIAMOND MINUS WHITE X" },
    { 0x2758, "LIGHT VERTICAL BAR" },
    { 0x2759, "MEDIUM VERTICAL BAR" },
    { 0x275A, "HEAVY VERTICAL BAR" },
    { 0x275B, "HEAVY SINGLE TURNED COMMA QUOTATION MARK ORNAMENT" },
    { 0x275C, "HEAVY SINGLE COMMA QUOTATION MARK ORNAMENT" },
    { 0x275D, "HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT" },
    { 0x275E, "HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT" },
    { 0x275F, "HEAVY LOW SINGLE COMMA QUOTATION MARK ORNAMENT" },
    { 0x2760, "HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT" },
    { 0x2761, "CURVED STEM PARAGRAPH SIGN ORNAMENT" },
    { 0x2762, "HEAVY EXCLAMATION MARK ORNAMENT" },
    { 0x2763, "HEAVY HEART EXCLAMATION MARK ORNAMENT" },
    { 0x2764, "HEAVY BLACK HEART" },
    { 0x2765, "ROTATED HEAVY BLACK HEART BULLET" },
    { 0x2766, "FLORAL HEART" },
    { 0x2767, "ROTATED FLORAL HEART BULLET" },
    { 0x2768, "MEDIUM LEFT PARENTHESIS ORNAMENT" },
    { 0x2769, "MEDIUM RIGHT PARENTHESIS ORNAMENT" },
    { 0x276A, "MEDIUM FLATTENED LEFT PARENTHESIS ORNAMENT" },
    { 0x276B, "MEDIUM FLATTENED RIGHT PARENTHESIS ORNAMENT" },
    { 0x276C, "MEDIUM LEFT-POINTING ANGLE BRACKET ORNAMENT" },
    { 0x276D, "MEDIUM RIGHT-POINTING ANGLE BRACKET ORNAMENT" },
    { 0x276E, "HEAVY LEFT-POINTING ANGLE QUOTATION MARK ORNAMENT" },
    { 0x276F, "HEAVY RIGHT-POINTING ANGLE QUOTATION MARK ORNAMENT" },
    { 0x2770, "HEAVY LEFT-POINTING ANGLE BRACKET ORNAMENT" },
    { 0x2771, "HEAVY RIGHT-POINTING ANGLE BRACKET ORNAMENT" },
    { 0x2772, "LIGHT LEFT TORTOISE SHELL BRACKET ORNAMENT" },
    { 0x2773, "LIGHT RIGHT TORTOISE SHELL BRACKET ORNAMENT" },
    { 0x2774, "MEDIUM LEFT CURLY BRACKET ORNAMENT" },
    { 0x2775, "MEDIUM RIGHT CURLY BRACKET ORNAMENT" },
    { 0x2776, "DINGBAT NEGATIVE CIRCLED DIGIT ONE" },
    { 0x2777, "DINGBAT NEGATIVE CIRCLED DIGIT TWO" },
    { 0x2778, "DINGBAT NEGATIVE CIRCLED DIGIT THREE" },
    { 0x2779, "DINGBAT NEGATIVE CIRCLED DIGIT FOUR" },
    { 0x277A, "DINGBAT NEGATIVE CIRCLED DIGIT FIVE" },
    { 0x277B, "DINGBAT NEGATIVE CIRCLED DIGIT SIX" },
    { 0x277C, "DINGBAT NEGATIVE CIRCLED DIGIT SEVEN" },
    { 0x277D, "DINGBAT NEGATIVE CIRCLED DIGIT EIGHT" },
    { 0x277E, "DINGBAT NEGATIVE CIRCLED DIGIT NINE" },
    { 0x277F, "DINGBAT NEGATIVE CIRCLED NUMBER TEN" },
    { 0x2780, "DINGBAT CIRCLED SANS-SERIF DIGIT ONE" },
    { 0x2781, "DINGBAT CIRCLED SANS-SERIF DIGIT TWO" },
    { 0x2782, "DINGBAT CIRCLED SANS-SERIF DIGIT THREE" },
    { 0x2783, "DINGBAT CIRCLED SANS-SERIF DIGIT FOUR" },
    { 0x2784, "DINGBAT CIRCLED SANS-SERIF DIGIT FIVE" },
    { 0x2785, "DINGBAT CIRCLED SANS-SERIF DIGIT SIX" },
    { 0x2786, "DINGBAT CIRCLED SANS-SERIF DIGIT SEVEN" },
    { 0x2787, "DINGBAT CIRCLED SANS-SERIF DIGIT EIGHT" },
    { 0x2788, "DINGBAT CIRCLED SANS-SERIF DIGIT NINE" },
    { 0x2789, "DINGBAT CIRCLED SANS-SERIF NUMBER TEN" },
    { 0x278A, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE" },
    { 0x278B, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TWO" },
    { 0x278C, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT THREE" },
    { 0x278D, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FOUR" },
    { 0x278E, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FIVE" },
    { 0x278F, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SIX" },
    { 0x2790, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN" },
    { 0x2791, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT" },
    { 0x2792, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE" },
    { 0x2793, "DINGBAT NEGATIVE CIRCLED SANS-SERIF NUMBER TEN" },
    { 0x2794, "HEAVY WIDE-HEADED RIGHTWARDS ARROW" },
    { 0x2798, "HEAVY SOUTH EAST ARROW" },
    { 0x2799, "HEAVY RIGHTWARDS ARROW" },
    { 0x279A, "HEAVY NORTH EAST ARROW" },
    { 0x279B, "DRAFTING POINT RIGHTWARDS ARROW" },
    { 0x279C, "HEAVY ROUND-TIPPED RIGHTWARDS ARROW" },
    { 0x279D, "TRIANGLE-HEADED RIGHTWARDS ARROW" },
    { 0x279E, "HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW" },
    { 0x279F, "DASHED TRIANGLE-HEADED RIGHTWARDS ARROW" },
    { 0x27A0, "HEAVY DASHED TRIANGLE-HEADED RIGHTWARDS ARROW" },
    { 0x27A1, "BLACK RIGHTWARDS ARROW" },
    { 0x27A2, "THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD" },
    { 0x27A3, "THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD" },
    { 0x27A4, "BLACK RIGHTWARDS ARROWHEAD" },
    { 0x27A5, "HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW" },
    { 0x27A6, "HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW" },
    { 0x27A7, "SQUAT BLACK RIGHTWARDS ARROW" },
    { 0x27A8, "HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW" },
    { 0x27A9, "RIGHT-SHADED WHITE RIGHTWARDS ARROW" },
    { 0x27AA, "LEFT-SHADED WHITE RIGHTWARDS ARROW" },
    { 0x27AB, "BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW" },
    { 0x27AC, "FRONT-TILTED SHADOWED WHITE RIGHTWARDS ARROW" },
    { 0x27AD, "HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW" },
    { 0x27AE, "HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW" },
    { 0x27AF, "NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW" },
    { 0x27B1, "NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW" },
    { 0x27B2, "CIRCLED HEAVY WHITE RIGHTWARDS ARROW" },
    { 0x27B3, "WHITE-FEATHERED RIGHTWARDS ARROW" },
    { 0x27B4, "BLACK-FEATHERED SOUTH EAST ARROW" },
    { 0x27B5, "BLACK-FEATHERED RIGHTWARDS ARROW" },
    { 0x27B6, "BLACK-FEATHERED NORTH EAST ARROW" },
    { 0x27B7, "HEAVY BLACK-FEATHERED SOUTH EAST ARROW" },
    { 0x27B8, "HEAVY BLACK-FEATHERED RIGHTWARDS ARROW" },
    { 0x27B9, "HEAVY BLACK-FEATHERED NORTH EAST ARROW" },
    { 0x27BA, "TEARDROP-BARBED RIGHTWARDS ARROW" },
    { 0x27BB, "HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW" },
    { 0x27BC, "WEDGE-TAILED RIGHTWARDS ARROW" },
    { 0x27BD, "HEAVY WEDGE-TAILED RIGHTWARDS ARROW" },
    { 0x27BE, "OPEN-OUTLINED RIGHTWARDS ARROW" },
    { 0x27C0, "THREE DIMENSIONAL ANGLE" },
    { 0x27C1, "WHITE TRIANGLE CONTAINING SMALL WHITE TRIANGLE" },
    { 0x27C2, "PERPENDICULAR" },
    { 0x27C3, "OPEN SUBSET" },
    { 0x27C4, "OPEN SUPERSET" },
    { 0x27C5, "LEFT S-SHAPED BAG DELIMITER" },
    { 0x27C6, "RIGHT S-SHAPED BAG DELIMITER" },
    { 0x27C7, "OR WITH DOT INSIDE" },
    { 0x27C8, "REVERSE SOLIDUS PRECEDING SUBSET" },
    { 0x27C9, "SUPERSET PRECEDING SOLIDUS" },
    { 0x27CA, "VERTICAL BAR WITH HORIZONTAL STROKE" },
    { 0x27CB, "MATHEMATICAL RISING DIAGONAL" },
    { 0x27CC, "LONG DIVISION" },
    { 0x27CD, "MATHEMATICAL FALLING DIAGONAL" },
    { 0x27CE, "SQUARED LOGICAL AND" },
    { 0x27CF, "SQUARED LOGICAL OR" },
    { 0x27D0, "WHITE DIAMOND WITH CENTRED DOT" },
    { 0x27D1, "AND WITH DOT" },
    { 0x27D2, "ELEMENT OF OPENING UPWARDS" },
    { 0x27D3, "LOWER RIGHT CORNER WITH DOT" },
    { 0x27D4, "UPPER LEFT CORNER WITH DOT" },
    { 0x27D5, "LEFT OUTER JOIN" },
    { 0x27D6, "RIGHT OUTER JOIN" },
    { 0x27D7, "FULL OUTER JOIN" },
    { 0x27D8, "LARGE UP TACK" },
    { 0x27D9, "LARGE DOWN TACK" },
    { 0x27DA, "LEFT AND RIGHT DOUBLE TURNSTILE" },
    { 0x27DB, "LEFT AND RIGHT TACK" },
    { 0x27DC, "LEFT MULTIMAP" },
    { 0x27DD, "LONG RIGHT TACK" },
    { 0x27DE, "LONG LEFT TACK" },
    { 0x27DF, "UP TACK WITH CIRCLE ABOVE" },
    { 0x27E0, "LOZENGE DIVIDED BY HORIZONTAL RULE" },
    { 0x27E1, "WHITE CONCAVE-SIDED DIAMOND" },
    { 0x27E2, "WHITE CONCAVE-SIDED DIAMOND WITH LEFTWARDS TICK" },
    { 0x27E3, "WHITE CONCAVE-SIDED DIAMOND WITH RIGHTWARDS TICK" },
    { 0x27E4, "WHITE SQUARE WITH LEFTWARDS TICK" },
    { 0x27E5, "WHITE SQUARE WITH RIGHTWARDS TICK" },
    { 0x27E6, "MATHEMATICAL LEFT WHITE SQUARE BRACKET" },
    { 0x27E7, "MATHEMATICAL RIGHT WHITE SQUARE BRACKET" },
    { 0x27E8, "MATHEMATICAL LEFT ANGLE BRACKET" },
    { 0x27E9, "MATHEMATICAL RIGHT ANGLE BRACKET" },
    { 0x27EA, "MATHEMATICAL LEFT DOUBLE ANGLE BRACKET" },
    { 0x27EB, "MATHEMATICAL RIGHT DOUBLE ANGLE BRACKET" },
    { 0x27EC, "MATHEMATICAL LEFT WHITE TORTOISE SHELL BRACKET" },
    { 0x27ED, "MATHEMATICAL RIGHT WHITE TORTOISE SHELL BRACKET" },
    { 0x27EE, "MATHEMATICAL LEFT FLATTENED PARENTHESIS" },
    { 0x27EF, "MATHEMATICAL RIGHT FLATTENED PARENTHESIS" },
    { 0x27F0, "UPWARDS QUADRUPLE ARROW" },
    { 0x27F1, "DOWNWARDS QUADRUPLE ARROW" },
    { 0x27F2, "ANTICLOCKWISE GAPPED CIRCLE ARROW" },
    { 0x27F3, "CLOCKWISE GAPPED CIRCLE ARROW" },
    { 0x27F4, "RIGHT ARROW WITH CIRCLED PLUS" },
    { 0x27F5, "LONG LEFTWARDS ARROW" },
    { 0x27F6, "LONG RIGHTWARDS ARROW" },
    { 0x27F7, "LONG LEFT RIGHT ARROW" },
    { 0x27F8, "LONG LEFTWARDS DOUBLE ARROW" },
    { 0x27F9, "LONG RIGHTWARDS DOUBLE ARROW" },
    { 0x27FA, "LONG LEFT RIGHT DOUBLE ARROW" },
    { 0x27FB, "LONG LEFTWARDS ARROW FROM BAR" },
    { 0x27FC, "LONG RIGHTWARDS ARROW FROM BAR" },
    { 0x27FD, "LONG LEFTWARDS DOUBLE ARROW FROM BAR" },
    { 0x27FE, "LONG RIGHTWARDS DOUBLE ARROW FROM BAR" },
    { 0x27FF, "LONG RIGHTWARDS SQUIGGLE ARROW" },
    { 0x2800, "BRAILLE PATTERN BLANK" },
    { 0x2801, "BRAILLE PATTERN DOTS-1" },
    { 0x2802, "BRAILLE PATTERN DOTS-2" },
    { 0x2803, "BRAILLE PATTERN DOTS-12" },
    { 0x2804, "BRAILLE PATTERN DOTS-3" },
    { 0x2805, "BRAILLE PATTERN DOTS-13" },
    { 0x2806, "BRAILLE PATTERN DOTS-23" },
    { 0x2807, "BRAILLE PATTERN DOTS-123" },
    { 0x2808, "BRAILLE PATTERN DOTS-4" },
    { 0x2809, "BRAILLE PATTERN DOTS-14" },
    { 0x280A, "BRAILLE PATTERN DOTS-24" },
    { 0x280B, "BRAILLE PATTERN DOTS-124" },
    { 0x280C, "BRAILLE PATTERN DOTS-34" },
    { 0x280D, "BRAILLE PATTERN DOTS-134" },
    { 0x280E, "BRAILLE PATTERN DOTS-234" },
    { 0x280F, "BRAILLE PATTERN DOTS-1234" },
    { 0x2810, "BRAILLE PATTERN DOTS-5" },
    { 0x2811, "BRAILLE PATTERN DOTS-15" },
    { 0x2812, "BRAILLE PATTERN DOTS-25" },
    { 0x2813, "BRAILLE PATTERN DOTS-125" },
    { 0x2814, "BRAILLE PATTERN DOTS-35" },
    { 0x2815, "BRAILLE PATTERN DOTS-135" },
    { 0x2816, "BRAILLE PATTERN DOTS-235" },
    { 0x2817, "BRAILLE PATTERN DOTS-1235" },
    { 0x2818, "BRAILLE PATTERN DOTS-45" },
    { 0x2819, "BRAILLE PATTERN DOTS-145" },
    { 0x281A, "BRAILLE PATTERN DOTS-245" },
    { 0x281B, "BRAILLE PATTERN DOTS-1245" },
    { 0x281C, "BRAILLE PATTERN DOTS-345" },
    { 0x281D, "BRAILLE PATTERN DOTS-1345" },
    { 0x281E, "BRAILLE PATTERN DOTS-2345" },
    { 0x281F, "BRAILLE PATTERN DOTS-12345" },
    { 0x2820, "BRAILLE PATTERN DOTS-6" },
    { 0x2821, "BRAILLE PATTERN DOTS-16" },
    { 0x2822, "BRAILLE PATTERN DOTS-26" },
    { 0x2823, "BRAILLE PATTERN DOTS-126" },
    { 0x2824, "BRAILLE PATTERN DOTS-36" },
    { 0x2825, "BRAILLE PATTERN DOTS-136" },
    { 0x2826, "BRAILLE PATTERN DOTS-236" },
    { 0x2827, "BRAILLE PATTERN DOTS-1236" },
    { 0x2828, "BRAILLE PATTERN DOTS-46" },
    { 0x2829, "BRAILLE PATTERN DOTS-146" },
    { 0x282A, "BRAILLE PATTERN DOTS-246" },
    { 0x282B, "BRAILLE PATTERN DOTS-1246" },
    { 0x282C, "BRAILLE PATTERN DOTS-346" },
    { 0x282D, "BRAILLE PATTERN DOTS-1346" },
    { 0x282E, "BRAILLE PATTERN DOTS-2346" },
    { 0x282F, "BRAILLE PATTERN DOTS-12346" },
    { 0x2830, "BRAILLE PATTERN DOTS-56" },
    { 0x2831, "BRAILLE PATTERN DOTS-156" },
    { 0x2832, "BRAILLE PATTERN DOTS-256" },
    { 0x2833, "BRAILLE PATTERN DOTS-1256" },
    { 0x2834, "BRAILLE PATTERN DOTS-356" },
    { 0x2835, "BRAILLE PATTERN DOTS-1356" },
    { 0x2836, "BRAILLE PATTERN DOTS-2356" },
    { 0x2837, "BRAILLE PATTERN DOTS-12356" },
    { 0x2838, "BRAILLE PATTERN DOTS-456" },
    { 0x2839, "BRAILLE PATTERN DOTS-1456" },
    { 0x283A, "BRAILLE PATTERN DOTS-2456" },
    { 0x283B, "BRAILLE PATTERN DOTS-12456" },
    { 0x283C, "BRAILLE PATTERN DOTS-3456" },
    { 0x283D, "BRAILLE PATTERN DOTS-13456" },
    { 0x283E, "BRAILLE PATTERN DOTS-23456" },
    { 0x283F, "BRAILLE PATTERN DOTS-123456" },
    { 0x2840, "BRAILLE PATTERN DOTS-7" },
    { 0x2841, "BRAILLE PATTERN DOTS-17" },
    { 0x2842, "BRAILLE PATTERN DOTS-27" },
    { 0x2843, "BRAILLE PATTERN DOTS-127" },
    { 0x2844, "BRAILLE PATTERN DOTS-37" },
    { 0x2845, "BRAILLE PATTERN DOTS-137" },
    { 0x2846, "BRAILLE PATTERN DOTS-237" },
    { 0x2847, "BRAILLE PATTERN DOTS-1237" },
    { 0x2848, "BRAILLE PATTERN DOTS-47" },
    { 0x2849, "BRAILLE PATTERN DOTS-147" },
    { 0x284A, "BRAILLE PATTERN DOTS-247" },
    { 0x284B, "BRAILLE PATTERN DOTS-1247" },
    { 0x284C, "BRAILLE PATTERN DOTS-347" },
    { 0x284D, "BRAILLE PATTERN DOTS-1347" },
    { 0x284E, "BRAILLE PATTERN DOTS-2347" },
    { 0x284F, "BRAILLE PATTERN DOTS-12347" },
    { 0x2850, "BRAILLE PATTERN DOTS-57" },
    { 0x2851, "BRAILLE PATTERN DOTS-157" },
    { 0x2852, "BRAILLE PATTERN DOTS-257" },
    { 0x2853, "BRAILLE PATTERN DOTS-1257" },
    { 0x2854, "BRAILLE PATTERN DOTS-357" },
    { 0x2855, "BRAILLE PATTERN DOTS-1357" },
    { 0x2856, "BRAILLE PATTERN DOTS-2357" },
    { 0x2857, "BRAILLE PATTERN DOTS-12357" },
    { 0x2858, "BRAILLE PATTERN DOTS-457" },
    { 0x2859, "BRAILLE PATTERN DOTS-1457" },
    { 0x285A, "BRAILLE PATTERN DOTS-2457" },
    { 0x285B, "BRAILLE PATTERN DOTS-12457" },
    { 0x285C, "BRAILLE PATTERN DOTS-3457" },
    { 0x285D, "BRAILLE PATTERN DOTS-13457" },
    { 0x285E, "BRAILLE PATTERN DOTS-23457" },
    { 0x285F, "BRAILLE PATTERN DOTS-123457" },
    { 0x2860, "BRAILLE PATTERN DOTS-67" },
    { 0x2861, "BRAILLE PATTERN DOTS-167" },
    { 0x2862, "BRAILLE PATTERN DOTS-267" },
    { 0x2863, "BRAILLE PATTERN DOTS-1267" },
    { 0x2864, "BRAILLE PATTERN DOTS-367" },
    { 0x2865, "BRAILLE PATTERN DOTS-1367" },
    { 0x2866, "BRAILLE PATTERN DOTS-2367" },
    { 0x2867, "BRAILLE PATTERN DOTS-12367" },
    { 0x2868, "BRAILLE PATTERN DOTS-467" },
    { 0x2869, "BRAILLE PATTERN DOTS-1467" },
    { 0x286A, "BRAILLE PATTERN DOTS-2467" },
    { 0x286B, "BRAILLE PATTERN DOTS-12467" },
    { 0x286C, "BRAILLE PATTERN DOTS-3467" },
    { 0x286D, "BRAILLE PATTERN DOTS-13467" },
    { 0x286E, "BRAILLE PATTERN DOTS-23467" },
    { 0x286F, "BRAILLE PATTERN DOTS-123467" },
    { 0x2870, "BRAILLE PATTERN DOTS-567" },
    { 0x2871, "BRAILLE PATTERN DOTS-1567" },
    { 0x2872, "BRAILLE PATTERN DOTS-2567" },
    { 0x2873, "BRAILLE PATTERN DOTS-12567" },
    { 0x2874, "BRAILLE PATTERN DOTS-3567" },
    { 0x2875, "BRAILLE PATTERN DOTS-13567" },
    { 0x2876, "BRAILLE PATTERN DOTS-23567" },
    { 0x2877, "BRAILLE PATTERN DOTS-123567" },
    { 0x2878, "BRAILLE PATTERN DOTS-4567" },
    { 0x2879, "BRAILLE PATTERN DOTS-14567" },
    { 0x287A, "BRAILLE PATTERN DOTS-24567" },
    { 0x287B, "BRAILLE PATTERN DOTS-124567" },
    { 0x287C, "BRAILLE PATTERN DOTS-34567" },
    { 0x287D, "BRAILLE PATTERN DOTS-134567" },
    { 0x287E, "BRAILLE PATTERN DOTS-234567" },
    { 0x287F, "BRAILLE PATTERN DOTS-1234567" },
    { 0x2880, "BRAILLE PATTERN DOTS-8" },
    { 0x2881, "BRAILLE PATTERN DOTS-18" },
    { 0x2882, "BRAILLE PATTERN DOTS-28" },
    { 0x2883, "BRAILLE PATTERN DOTS-128" },
    { 0x2884, "BRAILLE PATTERN DOTS-38" },
    { 0x2885, "BRAILLE PATTERN DOTS-138" },
    { 0x2886, "BRAILLE PATTERN DOTS-238" },
    { 0x2887, "BRAILLE PATTERN DOTS-1238" },
    { 0x2888, "BRAILLE PATTERN DOTS-48" },
    { 0x2889, "BRAILLE PATTERN DOTS-148" },
    { 0x288A, "BRAILLE PATTERN DOTS-248" },
    { 0x288B, "BRAILLE PATTERN DOTS-1248" },
    { 0x288C, "BRAILLE PATTERN DOTS-348" },
    { 0x288D, "BRAILLE PATTERN DOTS-1348" },
    { 0x288E, "BRAILLE PATTERN DOTS-2348" },
    { 0x288F, "BRAILLE PATTERN DOTS-12348" },
    { 0x2890, "BRAILLE PATTERN DOTS-58" },
    { 0x2891, "BRAILLE PATTERN DOTS-158" },
    { 0x2892, "BRAILLE PATTERN DOTS-258" },
    { 0x2893, "BRAILLE PATTERN DOTS-1258" },
    { 0x2894, "BRAILLE PATTERN DOTS-358" },
    { 0x2895, "BRAILLE PATTERN DOTS-1358" },
    { 0x2896, "BRAILLE PATTERN DOTS-2358" },
    { 0x2897, "BRAILLE PATTERN DOTS-12358" },
    { 0x2898, "BRAILLE PATTERN DOTS-458" },
    { 0x2899, "BRAILLE PATTERN DOTS-1458" },
    { 0x289A, "BRAILLE PATTERN DOTS-2458" },
    { 0x289B, "BRAILLE PATTERN DOTS-12458" },
    { 0x289C, "BRAILLE PATTERN DOTS-3458" },
    { 0x289D, "BRAILLE PATTERN DOTS-13458" },
    { 0x289E, "BRAILLE PATTERN DOTS-23458" },
    { 0x289F, "BRAILLE PATTERN DOTS-123458" },
    { 0x28A0, "BRAILLE PATTERN DOTS-68" },
    { 0x28A1, "BRAILLE PATTERN DOTS-168" },
    { 0x28A2, "BRAILLE PATTERN DOTS-268" },
    { 0x28A3, "BRAILLE PATTERN DOTS-1268" },
    { 0x28A4, "BRAILLE PATTERN DOTS-368" },
    { 0x28A5, "BRAILLE PATTERN DOTS-1368" },
    { 0x28A6, "BRAILLE PATTERN DOTS-2368" },
    { 0x28A7, "BRAILLE PATTERN DOTS-12368" },
    { 0x28A8, "BRAILLE PATTERN DOTS-468" },
    { 0x28A9, "BRAILLE PATTERN DOTS-1468" },
    { 0x28AA, "BRAILLE PATTERN DOTS-2468" },
    { 0x28AB, "BRAILLE PATTERN DOTS-12468" },
    { 0x28AC, "BRAILLE PATTERN DOTS-3468" },
    { 0x28AD, "BRAILLE PATTERN DOTS-13468" },
    { 0x28AE, "BRAILLE PATTERN DOTS-23468" },
    { 0x28AF, "BRAILLE PATTERN DOTS-123468" },
    { 0x28B0, "BRAILLE PATTERN DOTS-568" },
    { 0x28B1, "BRAILLE PATTERN DOTS-1568" },
    { 0x28B2, "BRAILLE PATTERN DOTS-2568" },
    { 0x28B3, "BRAILLE PATTERN DOTS-12568" },
    { 0x28B4, "BRAILLE PATTERN DOTS-3568" },
    { 0x28B5, "BRAILLE PATTERN DOTS-13568" },
    { 0x28B6, "BRAILLE PATTERN DOTS-23568" },
    { 0x28B7, "BRAILLE PATTERN DOTS-123568" },
    { 0x28B8, "BRAILLE PATTERN DOTS-4568" },
    { 0x28B9, "BRAILLE PATTERN DOTS-14568" },
    { 0x28BA, "BRAILLE PATTERN DOTS-24568" },
    { 0x28BB, "BRAILLE PATTERN DOTS-124568" },
    { 0x28BC, "BRAILLE PATTERN DOTS-34568" },
    { 0x28BD, "BRAILLE PATTERN DOTS-134568" },
    { 0x28BE, "BRAILLE PATTERN DOTS-234568" },
    { 0x28BF, "BRAILLE PATTERN DOTS-1234568" },
    { 0x28C0, "BRAILLE PATTERN DOTS-78" },
    { 0x28C1, "BRAILLE PATTERN DOTS-178" },
    { 0x28C2, "BRAILLE PATTERN DOTS-278" },
    { 0x28C3, "BRAILLE PATTERN DOTS-1278" },
    { 0x28C4, "BRAILLE PATTERN DOTS-378" },
    { 0x28C5, "BRAILLE PATTERN DOTS-1378" },
    { 0x28C6, "BRAILLE PATTERN DOTS-2378" },
    { 0x28C7, "BRAILLE PATTERN DOTS-12378" },
    { 0x28C8, "BRAILLE PATTERN DOTS-478" },
    { 0x28C9, "BRAILLE PATTERN DOTS-1478" },
    { 0x28CA, "BRAILLE PATTERN DOTS-2478" },
    { 0x28CB, "BRAILLE PATTERN DOTS-12478" },
    { 0x28CC, "BRAILLE PATTERN DOTS-3478" },
    { 0x28CD, "BRAILLE PATTERN DOTS-13478" },
    { 0x28CE, "BRAILLE PATTERN DOTS-23478" },
    { 0x28CF, "BRAILLE PATTERN DOTS-123478" },
    { 0x28D0, "BRAILLE PATTERN DOTS-578" },
    { 0x28D1, "BRAILLE PATTERN DOTS-1578" },
    { 0x28D2, "BRAILLE PATTERN DOTS-2578" },
    { 0x28D3, "BRAILLE PATTERN DOTS-12578" },
    { 0x28D4, "BRAILLE PATTERN DOTS-3578" },
    { 0x28D5, "BRAILLE PATTERN DOTS-13578" },
    { 0x28D6, "BRAILLE PATTERN DOTS-23578" },
    { 0x28D7, "BRAILLE PATTERN DOTS-123578" },
    { 0x28D8, "BRAILLE PATTERN DOTS-4578" },
    { 0x28D9, "BRAILLE PATTERN DOTS-14578" },
    { 0x28DA, "BRAILLE PATTERN DOTS-24578" },
    { 0x28DB, "BRAILLE PATTERN DOTS-124578" },
    { 0x28DC, "BRAILLE PATTERN DOTS-34578" },
    { 0x28DD, "BRAILLE PATTERN DOTS-134578" },
    { 0x28DE, "BRAILLE PATTERN DOTS-234578" },
    { 0x28DF, "BRAILLE PATTERN DOTS-1234578" },
    { 0x28E0, "BRAILLE PATTERN DOTS-678" },
    { 0x28E1, "BRAILLE PATTERN DOTS-1678" },
    { 0x28E2, "BRAILLE PATTERN DOTS-2678" },
    { 0x28E3, "BRAILLE PATTERN DOTS-12678" },
    { 0x28E4, "BRAILLE PATTERN DOTS-3678" },
    { 0x28E5, "BRAILLE PATTERN DOTS-13678" },
    { 0x28E6, "BRAILLE PATTERN DOTS-23678" },
    { 0x28E7, "BRAILLE PATTERN DOTS-123678" },
    { 0x28E8, "BRAILLE PATTERN DOTS-4678" },
    { 0x28E9, "BRAILLE PATTERN DOTS-14678" },
    { 0x28EA, "BRAILLE PATTERN DOTS-24678" },
    { 0x28EB, "BRAILLE PATTERN DOTS-124678" },
    { 0x28EC, "BRAILLE PATTERN DOTS-34678" },
    { 0x28ED, "BRAILLE PATTERN DOTS-134678" },
    { 0x28EE, "BRAILLE PATTERN DOTS-234678" },
    { 0x28EF, "BRAILLE PATTERN DOTS-1234678" },
    { 0x28F0, "BRAILLE PATTERN DOTS-5678" },
    { 0x28F1, "BRAILLE PATTERN DOTS-15678" },
    { 0x28F2, "BRAILLE PATTERN DOTS-25678" },
    { 0x28F3, "BRAILLE PATTERN DOTS-125678" },
    { 0x28F4, "BRAILLE PATTERN DOTS-35678" },
    { 0x28F5, "BRAILLE PATTERN DOTS-135678" },
    { 0x28F6, "BRAILLE PATTERN DOTS-235678" },
    { 0x28F7, "BRAILLE PATTERN DOTS-1235678" },
    { 0x28F8, "BRAILLE PATTERN DOTS-45678" },
    { 0x28F9, "BRAILLE PATTERN DOTS-145678" },
    { 0x28FA, "BRAILLE PATTERN DOTS-245678" },
    { 0x28FB, "BRAILLE PATTERN DOTS-1245678" },
    { 0x28FC, "BRAILLE PATTERN DOTS-345678" },
    { 0x28FD, "BRAILLE PATTERN DOTS-1345678" },
    { 0x28FE, "BRAILLE PATTERN DOTS-2345678" },
    { 0x28FF, "BRAILLE PATTERN DOTS-12345678" },
    { 0x2900, "RIGHTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE" },
    { 0x2901, "RIGHTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE" },
    { 0x2902, "LEFTWARDS DOUBLE ARROW WITH VERTICAL STROKE" },
    { 0x2903, "RIGHTWARDS DOUBLE ARROW WITH VERTICAL STROKE" },
    { 0x2904, "LEFT RIGHT DOUBLE ARROW WITH VERTICAL STROKE" },
    { 0x2905, "RIGHTWARDS TWO-HEADED ARROW FROM BAR" },
    { 0x2906, "LEFTWARDS DOUBLE ARROW FROM BAR" },
    { 0x2907, "RIGHTWARDS DOUBLE ARROW FROM BAR" },
    { 0x2908, "DOWNWARDS ARROW WITH HORIZONTAL STROKE" },
    { 0x2909, "UPWARDS ARROW WITH HORIZONTAL STROKE" },
    { 0x290A, "UPWARDS TRIPLE ARROW" },
    { 0x290B, "DOWNWARDS TRIPLE ARROW" },
    { 0x290C, "LEFTWARDS DOUBLE DASH ARROW" },
    { 0x290D, "RIGHTWARDS DOUBLE DASH ARROW" },
    { 0x290E, "LEFTWARDS TRIPLE DASH ARROW" },
    { 0x290F, "RIGHTWARDS TRIPLE DASH ARROW" },
    { 0x2910, "RIGHTWARDS TWO-HEADED TRIPLE DASH ARROW" },
    { 0x2911, "RIGHTWARDS ARROW WITH DOTTED STEM" },
    { 0x2912, "UPWARDS ARROW TO BAR" },
    { 0x2913, "DOWNWARDS ARROW TO BAR" },
    { 0x2914, "RIGHTWARDS ARROW WITH TAIL WITH VERTICAL STROKE" },
    { 0x2915, "RIGHTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE" },
    { 0x2916, "RIGHTWARDS TWO-HEADED ARROW WITH TAIL" },
    { 0x2917, "RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE" },
    { 0x2918, "RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE" },
    { 0x2919, "LEFTWARDS ARROW-TAIL" },
    { 0x291A, "RIGHTWARDS ARROW-TAIL" },
    { 0x291B, "LEFTWARDS DOUBLE ARROW-TAIL" },
    { 0x291C, "RIGHTWARDS DOUBLE ARROW-TAIL" },
    { 0x291D, "LEFTWARDS ARROW TO BLACK DIAMOND" },
    { 0x291E, "RIGHTWARDS ARROW TO BLACK DIAMOND" },
    { 0x291F, "LEFTWARDS ARROW FROM BAR TO BLACK DIAMOND" },
    { 0x2920, "RIGHTWARDS ARROW FROM BAR TO BLACK DIAMOND" },
    { 0x2921, "NORTH WEST AND SOUTH EAST ARROW" },
    { 0x2922, "NORTH EAST AND SOUTH WEST ARROW" },
    { 0x2923, "NORTH WEST ARROW WITH HOOK" },
    { 0x2924, "NORTH EAST ARROW WITH HOOK" },
    { 0x2925, "SOUTH EAST ARROW WITH HOOK" },
    { 0x2926, "SOUTH WEST ARROW WITH HOOK" },
    { 0x2927, "NORTH WEST ARROW AND NORTH EAST ARROW" },
    { 0x2928, "NORTH EAST ARROW AND SOUTH EAST ARROW" },
    { 0x2929, "SOUTH EAST ARROW AND SOUTH WEST ARROW" },
    { 0x292A, "SOUTH WEST ARROW AND NORTH WEST ARROW" },
    { 0x292B, "RISING DIAGONAL CROSSING FALLING DIAGONAL" },
    { 0x292C, "FALLING DIAGONAL CROSSING RISING DIAGONAL" },
    { 0x292D, "SOUTH EAST ARROW CROSSING NORTH EAST ARROW" },
    { 0x292E, "NORTH EAST ARROW CROSSING SOUTH EAST ARROW" },
    { 0x292F, "FALLING DIAGONAL CROSSING NORTH EAST ARROW" },
    { 0x2930, "RISING DIAGONAL CROSSING SOUTH EAST ARROW" },
    { 0x2931, "NORTH EAST ARROW CROSSING NORTH WEST ARROW" },
    { 0x2932, "NORTH WEST ARROW CROSSING NORTH EAST ARROW" },
    { 0x2933, "WAVE ARROW POINTING DIRECTLY RIGHT" },
    { 0x2934, "ARROW POINTING RIGHTWARDS THEN CURVING UPWARDS" },
    { 0x2935, "ARROW POINTING RIGHTWARDS THEN CURVING DOWNWARDS" },
    { 0x2936, "ARROW POINTING DOWNWARDS THEN CURVING LEFTWARDS" },
    { 0x2937, "ARROW POINTING DOWNWARDS THEN CURVING RIGHTWARDS" },
    { 0x2938, "RIGHT-SIDE ARC CLOCKWISE ARROW" },
    { 0x2939, "LEFT-SIDE ARC ANTICLOCKWISE ARROW" },
    { 0x293A, "TOP ARC ANTICLOCKWISE ARROW" },
    { 0x293B, "BOTTOM ARC ANTICLOCKWISE ARROW" },
    { 0x293C, "TOP ARC CLOCKWISE ARROW WITH MINUS" },
    { 0x293D, "TOP ARC ANTICLOCKWISE ARROW WITH PLUS" },
    { 0x293E, "LOWER RIGHT SEMICIRCULAR CLOCKWISE ARROW" },
    { 0x293F, "LOWER LEFT SEMICIRCULAR ANTICLOCKWISE ARROW" },
    { 0x2940, "ANTICLOCKWISE CLOSED CIRCLE ARROW" },
    { 0x2941, "CLOCKWISE CLOSED CIRCLE ARROW" },
    { 0x2942, "RIGHTWARDS ARROW ABOVE SHORT LEFTWARDS ARROW" },
    { 0x2943, "LEFTWARDS ARROW ABOVE SHORT RIGHTWARDS ARROW" },
    { 0x2944, "SHORT RIGHTWARDS ARROW ABOVE LEFTWARDS ARROW" },
    { 0x2945, "RIGHTWARDS ARROW WITH PLUS BELOW" },
    { 0x2946, "LEFTWARDS ARROW WITH PLUS BELOW" },
    { 0x2947, "RIGHTWARDS ARROW THROUGH X" },
    { 0x2948, "LEFT RIGHT ARROW THROUGH SMALL CIRCLE" },
    { 0x2949, "UPWARDS TWO-HEADED ARROW FROM SMALL CIRCLE" },
    { 0x294A, "LEFT BARB UP RIGHT BARB DOWN HARPOON" },
    { 0x294B, "LEFT BARB DOWN RIGHT BARB UP HARPOON" },
    { 0x294C, "UP BARB RIGHT DOWN BARB LEFT HARPOON" },
    { 0x294D, "UP BARB LEFT DOWN BARB RIGHT HARPOON" },
    { 0x294E, "LEFT BARB UP RIGHT BARB UP HARPOON" },
    { 0x294F, "UP BARB RIGHT DOWN BARB RIGHT HARPOON" },
    { 0x2950, "LEFT BARB DOWN RIGHT BARB DOWN HARPOON" },
    { 0x2951, "UP BARB LEFT DOWN BARB LEFT HARPOON" },
    { 0x2952, "LEFTWARDS HARPOON WITH BARB UP TO BAR" },
    { 0x2953, "RIGHTWARDS HARPOON WITH BARB UP TO BAR" },
    { 0x2954, "UPWARDS HARPOON WITH BARB RIGHT TO BAR" },
    { 0x2955, "DOWNWARDS HARPOON WITH BARB RIGHT TO BAR" },
    { 0x2956, "LEFTWARDS HARPOON WITH BARB DOWN TO BAR" },
    { 0x2957, "RIGHTWARDS HARPOON WITH BARB DOWN TO BAR" },
    { 0x2958, "UPWARDS HARPOON WITH BARB LEFT TO BAR" },
    { 0x2959, "DOWNWARDS HARPOON WITH BARB LEFT TO BAR" },
    { 0x295A, "LEFTWARDS HARPOON WITH BARB UP FROM BAR" },
    { 0x295B, "RIGHTWARDS HARPOON WITH BARB UP FROM BAR" },
    { 0x295C, "UPWARDS HARPOON WITH BARB RIGHT FROM BAR" },
    { 0x295D, "DOWNWARDS HARPOON WITH BARB RIGHT FROM BAR" },
    { 0x295E, "LEFTWARDS HARPOON WITH BARB DOWN FROM BAR" },
    { 0x295F, "RIGHTWARDS HARPOON WITH BARB DOWN FROM BAR" },
    { 0x2960, "UPWARDS HARPOON WITH BARB LEFT FROM BAR" },
    { 0x2961, "DOWNWARDS HARPOON WITH BARB LEFT FROM BAR" },
    { 0x2962, "LEFTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB DOWN" },
    { 0x2963, "UPWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT" },
    { 0x2964, "RIGHTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB DOWN" },
    { 0x2965, "DOWNWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT" },
    { 0x2966, "LEFTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB UP" },
    { 0x2967, "LEFTWARDS HARPOON WITH BARB DOWN ABOVE RIGHTWARDS HARPOON WITH BARB DOWN" },
    { 0x2968, "RIGHTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB UP" },
    { 0x2969, "RIGHTWARDS HARPOON WITH BARB DOWN ABOVE LEFTWARDS HARPOON WITH BARB DOWN" },
    { 0x296A, "LEFTWARDS HARPOON WITH BARB UP ABOVE LONG DASH" },
    { 0x296B, "LEFTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH" },
    { 0x296C, "RIGHTWARDS HARPOON WITH BARB UP ABOVE LONG DASH" },
    { 0x296D, "RIGHTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH" },
    { 0x296E, "UPWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT" },
    { 0x296F, "DOWNWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT" },
    { 0x2970, "RIGHT DOUBLE ARROW WITH ROUNDED HEAD" },
    { 0x2971, "EQUALS SIGN ABOVE RIGHTWARDS ARROW" },
    { 0x2972, "TILDE OPERATOR ABOVE RIGHTWARDS ARROW" },
    { 0x2973, "LEFTWARDS ARROW ABOVE TILDE OPERATOR" },
    { 0x2974, "RIGHTWARDS ARROW ABOVE TILDE OPERATOR" },
    { 0x2975, "RIGHTWARDS ARROW ABOVE ALMOST EQUAL TO" },
    { 0x2976, "LESS-THAN ABOVE LEFTWARDS ARROW" },
    { 0x2977, "LEFTWARDS ARROW THROUGH LESS-THAN" },
    { 0x2978, "GREATER-THAN ABOVE RIGHTWARDS ARROW" },
    { 0x2979, "SUBSET ABOVE RIGHTWARDS ARROW" },
    { 0x297A, "LEFTWARDS ARROW THROUGH SUBSET" },
    { 0x297B, "SUPERSET ABOVE LEFTWARDS ARROW" },
    { 0x297C, "LEFT FISH TAIL" },
    { 0x297D, "RIGHT FISH TAIL" },
    { 0x297E, "UP FISH TAIL" },
    { 0x297F, "DOWN FISH TAIL" },
    { 0x2980, "TRIPLE VERTICAL BAR DELIMITER" },
    { 0x2981, "Z NOTATION SPOT" },
    { 0x2982, "Z NOTATION TYPE COLON" },
    { 0x2983, "LEFT WHITE CURLY BRACKET" },
    { 0x2984, "RIGHT WHITE CURLY BRACKET" },
    { 0x2985, "LEFT WHITE PARENTHESIS" },
    { 0x2986, "RIGHT WHITE PARENTHESIS" },
    { 0x2987, "Z NOTATION LEFT IMAGE BRACKET" },
    { 0x2988, "Z NOTATION RIGHT IMAGE BRACKET" },
    { 0x2989, "Z NOTATION LEFT BINDING BRACKET" },
    { 0x298A, "Z NOTATION RIGHT BINDING BRACKET" },
    { 0x298B, "LEFT SQUARE BRACKET WITH UNDERBAR" },
    { 0x298C, "RIGHT SQUARE BRACKET WITH UNDERBAR" },
    { 0x298D, "LEFT SQUARE BRACKET WITH TICK IN TOP CORNER" },
    { 0x298E, "RIGHT SQUARE BRACKET WITH TICK IN BOTTOM CORNER" },
    { 0x298F, "LEFT SQUARE BRACKET WITH TICK IN BOTTOM CORNER" },
    { 0x2990, "RIGHT SQUARE BRACKET WITH TICK IN TOP CORNER" },
    { 0x2991, "LEFT ANGLE BRACKET WITH DOT" },
    { 0x2992, "RIGHT ANGLE BRACKET WITH DOT" },
    { 0x2993, "LEFT ARC LESS-THAN BRACKET" },
    { 0x2994, "RIGHT ARC GREATER-THAN BRACKET" },
    { 0x2995, "DOUBLE LEFT ARC GREATER-THAN BRACKET" },
    { 0x2996, "DOUBLE RIGHT ARC LESS-THAN BRACKET" },
    { 0x2997, "LEFT BLACK TORTOISE SHELL BRACKET" },
    { 0x2998, "RIGHT BLACK TORTOISE SHELL BRACKET" },
    { 0x2999, "DOTTED FENCE" },
    { 0x299A, "VERTICAL ZIGZAG LINE" },
    { 0x299B, "MEASURED ANGLE OPENING LEFT" },
    { 0x299C, "RIGHT ANGLE VARIANT WITH SQUARE" },
    { 0x299D, "MEASURED RIGHT ANGLE WITH DOT" },
    { 0x299E, "ANGLE WITH S INSIDE" },
    { 0x299F, "ACUTE ANGLE" },
    { 0x29A0, "SPHERICAL ANGLE OPENING LEFT" },
    { 0x29A1, "SPHERICAL ANGLE OPENING UP" },
    { 0x29A2, "TURNED ANGLE" },
    { 0x29A3, "REVERSED ANGLE" },
    { 0x29A4, "ANGLE WITH UNDERBAR" },
    { 0x29A5, "REVERSED ANGLE WITH UNDERBAR" },
    { 0x29A6, "OBLIQUE ANGLE OPENING UP" },
    { 0x29A7, "OBLIQUE ANGLE OPENING DOWN" },
    { 0x29A8, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND RIGHT" },
    { 0x29A9, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND LEFT" },
    { 0x29AA, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND RIGHT" },
    { 0x29AB, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND LEFT" },
    { 0x29AC, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND UP" },
    { 0x29AD, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND UP" },
    { 0x29AE, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND DOWN" },
    { 0x29AF, "MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND DOWN" },
    { 0x29B0, "REVERSED EMPTY SET" },
    { 0x29B1, "EMPTY SET WITH OVERBAR" },
    { 0x29B2, "EMPTY SET WITH SMALL CIRCLE ABOVE" },
    { 0x29B3, "EMPTY SET WITH RIGHT ARROW ABOVE" },
    { 0x29B4, "EMPTY SET WITH LEFT ARROW ABOVE" },
    { 0x29B5, "CIRCLE WITH HORIZONTAL BAR" },
    { 0x29B6, "CIRCLED VERTICAL BAR" },
    { 0x29B7, "CIRCLED PARALLEL" },
    { 0x29B8, "CIRCLED REVERSE SOLIDUS" },
    { 0x29B9, "CIRCLED PERPENDICULAR" },
    { 0x29BA, "CIRCLE DIVIDED BY HORIZONTAL BAR AND TOP HALF DIVIDED BY VERTICAL BAR" },
    { 0x29BB, "CIRCLE WITH SUPERIMPOSED X" },
    { 0x29BC, "CIRCLED ANTICLOCKWISE-ROTATED DIVISION SIGN" },
    { 0x29BD, "UP ARROW THROUGH CIRCLE" },
    { 0x29BE, "CIRCLED WHITE BULLET" },
    { 0x29BF, "CIRCLED BULLET" },
    { 0x29C0, "CIRCLED LESS-THAN" },
    { 0x29C1, "CIRCLED GREATER-THAN" },
    { 0x29C2, "CIRCLE WITH SMALL CIRCLE TO THE RIGHT" },
    { 0x29C3, "CIRCLE WITH TWO HORIZONTAL STROKES TO THE RIGHT" },
    { 0x29C4, "SQUARED RISING DIAGONAL SLASH" },
    { 0x29C5, "SQUARED FALLING DIAGONAL SLASH" },
    { 0x29C6, "SQUARED ASTERISK" },
    { 0x29C7, "SQUARED SMALL CIRCLE" },
    { 0x29C8, "SQUARED SQUARE" },
    { 0x29C9, "TWO JOINED SQUARES" },
    { 0x29CA, "TRIANGLE WITH DOT ABOVE" },
    { 0x29CB, "TRIANGLE WITH UNDERBAR" },
    { 0x29CC, "S IN TRIANGLE" },
    { 0x29CD, "TRIANGLE WITH SERIFS AT BOTTOM" },
    { 0x29CE, "RIGHT TRIANGLE ABOVE LEFT TRIANGLE" },
    { 0x29CF, "LEFT TRIANGLE BESIDE VERTICAL BAR" },
    { 0x29D0, "VERTICAL BAR BESIDE RIGHT TRIANGLE" },
    { 0x29D1, "BOWTIE WITH LEFT HALF BLACK" },
    { 0x29D2, "BOWTIE WITH RIGHT HALF BLACK" },
    { 0x29D3, "BLACK BOWTIE" },
    { 0x29D4, "TIMES WITH LEFT HALF BLACK" },
    { 0x29D5, "TIMES WITH RIGHT HALF BLACK" },
    { 0x29D6, "WHITE HOURGLASS" },
    { 0x29D7, "BLACK HOURGLASS" },
    { 0x29D8, "LEFT WIGGLY FENCE" },
    { 0x29D9, "RIGHT WIGGLY FENCE" },
    { 0x29DA, "LEFT DOUBLE WIGGLY FENCE" },
    { 0x29DB, "RIGHT DOUBLE WIGGLY FENCE" },
    { 0x29DC, "INCOMPLETE INFINITY" },
    { 0x29DD, "TIE OVER INFINITY" },
    { 0x29DE, "INFINITY NEGATED WITH VERTICAL BAR" },
    { 0x29DF, "DOUBLE-ENDED MULTIMAP" },
    { 0x29E0, "SQUARE WITH CONTOURED OUTLINE" },
    { 0x29E1, "INCREASES AS" },
    { 0x29E2, "SHUFFLE PRODUCT" },
    { 0x29E3, "EQUALS SIGN AND SLANTED PARALLEL" },
    { 0x29E4, "EQUALS SIGN AND SLANTED PARALLEL WITH TILDE ABOVE" },
    { 0x29E5, "IDENTICAL TO AND SLANTED PARALLEL" },
    { 0x29E6, "GLEICH STARK" },
    { 0x29E7, "THERMODYNAMIC" },
    { 0x29E8, "DOWN-POINTING TRIANGLE WITH LEFT HALF BLACK" },
    { 0x29E9, "DOWN-POINTING TRIANGLE WITH RIGHT HALF BLACK" },
    { 0x29EA, "BLACK DIAMOND WITH DOWN ARROW" },
    { 0x29EB, "BLACK LOZENGE" },
    { 0x29EC, "WHITE CIRCLE WITH DOWN ARROW" },
    { 0x29ED, "BLACK CIRCLE WITH DOWN ARROW" },
    { 0x29EE, "ERROR-BARRED WHITE SQUARE" },
    { 0x29EF, "ERROR-BARRED BLACK SQUARE" },
    { 0x29F0, "ERROR-BARRED WHITE DIAMOND" },
    { 0x29F1, "ERROR-BARRED BLACK DIAMOND" },
    { 0x29F2, "ERROR-BARRED WHITE CIRCLE" },
    { 0x29F3, "ERROR-BARRED BLACK CIRCLE" },
    { 0x29F4, "RULE-DELAYED" },
    { 0x29F5, "REVERSE SOLIDUS OPERATOR" },
    { 0x29F6, "SOLIDUS WITH OVERBAR" },
    { 0x29F7, "REVERSE SOLIDUS WITH HORIZONTAL STROKE" },
    { 0x29F8, "BIG SOLIDUS" },
    { 0x29F9, "BIG REVERSE SOLIDUS" },
    { 0x29FA, "DOUBLE PLUS" },
    { 0x29FB, "TRIPLE PLUS" },
    { 0x29FC, "LEFT-POINTING CURVED ANGLE BRACKET" },
    { 0x29FD, "RIGHT-POINTING CURVED ANGLE BRACKET" },
    { 0x29FE, "TINY" },
    { 0x29FF, "MINY" },
    { 0x2A00, "N-ARY CIRCLED DOT OPERATOR" },
    { 0x2A01, "N-ARY CIRCLED PLUS OPERATOR" },
    { 0x2A02, "N-ARY CIRCLED TIMES OPERATOR" },
    { 0x2A03, "N-ARY UNION OPERATOR WITH DOT" },
    { 0x2A04, "N-ARY UNION OPERATOR WITH PLUS" },
    { 0x2A05, "N-ARY SQUARE INTERSECTION OPERATOR" },
    { 0x2A06, "N-ARY SQUARE UNION OPERATOR" },
    { 0x2A07, "TWO LOGICAL AND OPERATOR" },
    { 0x2A08, "TWO LOGICAL OR OPERATOR" },
    { 0x2A09, "N-ARY TIMES OPERATOR" },
    { 0x2A0A, "MODULO TWO SUM" },
    { 0x2A0B, "SUMMATION WITH INTEGRAL" },
    { 0x2A0C, "QUADRUPLE INTEGRAL OPERATOR" },
    { 0x2A0D, "FINITE PART INTEGRAL" },
    { 0x2A0E, "INTEGRAL WITH DOUBLE STROKE" },
    { 0x2A0F, "INTEGRAL AVERAGE WITH SLASH" },
    { 0x2A10, "CIRCULATION FUNCTION" },
    { 0x2A11, "ANTICLOCKWISE INTEGRATION" },
    { 0x2A12, "LINE INTEGRATION WITH RECTANGULAR PATH AROUND POLE" },
    { 0x2A13, "LINE INTEGRATION WITH SEMICIRCULAR PATH AROUND POLE" },
    { 0x2A14, "LINE INTEGRATION NOT INCLUDING THE POLE" },
    { 0x2A15, "INTEGRAL AROUND A POINT OPERATOR" },
    { 0x2A16, "QUATERNION INTEGRAL OPERATOR" },
    { 0x2A17, "INTEGRAL WITH LEFTWARDS ARROW WITH HOOK" },
    { 0x2A18, "INTEGRAL WITH TIMES SIGN" },
    { 0x2A19, "INTEGRAL WITH INTERSECTION" },
    { 0x2A1A, "INTEGRAL WITH UNION" },
    { 0x2A1B, "INTEGRAL WITH OVERBAR" },
    { 0x2A1C, "INTEGRAL WITH UNDERBAR" },
    { 0x2A1D, "JOIN" },
    { 0x2A1E, "LARGE LEFT TRIANGLE OPERATOR" },
    { 0x2A1F, "Z NOTATION SCHEMA COMPOSITION" },
    { 0x2A20, "Z NOTATION SCHEMA PIPING" },
    { 0x2A21, "Z NOTATION SCHEMA PROJECTION" },
    { 0x2A22, "PLUS SIGN WITH SMALL CIRCLE ABOVE" },
    { 0x2A23, "PLUS SIGN WITH CIRCUMFLEX ACCENT ABOVE" },
    { 0x2A24, "PLUS SIGN WITH TILDE ABOVE" },
    { 0x2A25, "PLUS SIGN WITH DOT BELOW" },
    { 0x2A26, "PLUS SIGN WITH TILDE BELOW" },
    { 0x2A27, "PLUS SIGN WITH SUBSCRIPT TWO" },
    { 0x2A28, "PLUS SIGN WITH BLACK TRIANGLE" },
    { 0x2A29, "MINUS SIGN WITH COMMA ABOVE" },
    { 0x2A2A, "MINUS SIGN WITH DOT BELOW" },
    { 0x2A2B, "MINUS SIGN WITH FALLING DOTS" },
    { 0x2A2C, "MINUS SIGN WITH RISING DOTS" },
    { 0x2A2D, "PLUS SIGN IN LEFT HALF CIRCLE" },
    { 0x2A2E, "PLUS SIGN IN RIGHT HALF CIRCLE" },
    { 0x2A2F, "VECTOR OR CROSS PRODUCT" },
    { 0x2A30, "MULTIPLICATION SIGN WITH DOT ABOVE" },
    { 0x2A31, "MULTIPLICATION SIGN WITH UNDERBAR" },
    { 0x2A32, "SEMIDIRECT PRODUCT WITH BOTTOM CLOSED" },
    { 0x2A33, "SMASH PRODUCT" },
    { 0x2A34, "MULTIPLICATION SIGN IN LEFT HALF CIRCLE" },
    { 0x2A35, "MULTIPLICATION SIGN IN RIGHT HALF CIRCLE" },
    { 0x2A36, "CIRCLED MULTIPLICATION SIGN WITH CIRCUMFLEX ACCENT" },
    { 0x2A37, "MULTIPLICATION SIGN IN DOUBLE CIRCLE" },
    { 0x2A38, "CIRCLED DIVISION SIGN" },
    { 0x2A39, "PLUS SIGN IN TRIANGLE" },
    { 0x2A3A, "MINUS SIGN IN TRIANGLE" },
    { 0x2A3B, "MULTIPLICATION SIGN IN TRIANGLE" },
    { 0x2A3C, "INTERIOR PRODUCT" },
    { 0x2A3D, "RIGHTHAND INTERIOR PRODUCT" },
    { 0x2A3E, "Z NOTATION RELATIONAL COMPOSITION" },
    { 0x2A3F, "AMALGAMATION OR COPRODUCT" },
    { 0x2A40, "INTERSECTION WITH DOT" },
    { 0x2A41, "UNION WITH MINUS SIGN" },
    { 0x2A42, "UNION WITH OVERBAR" },
    { 0x2A43, "INTERSECTION WITH OVERBAR" },
    { 0x2A44, "INTERSECTION WITH LOGICAL AND" },
    { 0x2A45, "UNION WITH LOGICAL OR" },
    { 0x2A46, "UNION ABOVE INTERSECTION" },
    { 0x2A47, "INTERSECTION ABOVE UNION" },
    { 0x2A48, "UNION ABOVE BAR ABOVE INTERSECTION" },
    { 0x2A49, "INTERSECTION ABOVE BAR ABOVE UNION" },
    { 0x2A4A, "UNION BESIDE AND JOINED WITH UNION" },
    { 0x2A4B, "INTERSECTION BESIDE AND JOINED WITH INTERSECTION" },
    { 0x2A4C, "CLOSED UNION WITH SERIFS" },
    { 0x2A4D, "CLOSED INTERSECTION WITH SERIFS" },
    { 0x2A4E, "DOUBLE SQUARE INTERSECTION" },
    { 0x2A4F, "DOUBLE SQUARE UNION" },
    { 0x2A50, "CLOSED UNION WITH SERIFS AND SMASH PRODUCT" },
    { 0x2A51, "LOGICAL AND WITH DOT ABOVE" },
    { 0x2A52, "LOGICAL OR WITH DOT ABOVE" },
    { 0x2A53, "DOUBLE LOGICAL AND" },
    { 0x2A54, "DOUBLE LOGICAL OR" },
    { 0x2A55, "TWO INTERSECTING LOGICAL AND" },
    { 0x2A56, "TWO INTERSECTING LOGICAL OR" },
    { 0x2A57, "SLOPING LARGE OR" },
    { 0x2A58, "SLOPING LARGE AND" },
    { 0x2A59, "LOGICAL OR OVERLAPPING LOGICAL AND" },
    { 0x2A5A, "LOGICAL AND WITH MIDDLE STEM" },
    { 0x2A5B, "LOGICAL OR WITH MIDDLE STEM" },
    { 0x2A5C, "LOGICAL AND WITH HORIZONTAL DASH" },
    { 0x2A5D, "LOGICAL OR WITH HORIZONTAL DASH" },
    { 0x2A5E, "LOGICAL AND WITH DOUBLE OVERBAR" },
    { 0x2A5F, "LOGICAL AND WITH UNDERBAR" },
    { 0x2A60, "LOGICAL AND WITH DOUBLE UNDERBAR" },
    { 0x2A61, "SMALL VEE WITH UNDERBAR" },
    { 0x2A62, "LOGICAL OR WITH DOUBLE OVERBAR" },
    { 0x2A63, "LOGICAL OR WITH DOUBLE UNDERBAR" },
    { 0x2A64, "Z NOTATION DOMAIN ANTIRESTRICTION" },
    { 0x2A65, "Z NOTATION RANGE ANTIRESTRICTION" },
    { 0x2A66, "EQUALS SIGN WITH DOT BELOW" },
    { 0x2A67, "IDENTICAL WITH DOT ABOVE" },
    { 0x2A68, "TRIPLE HORIZONTAL BAR WITH DOUBLE VERTICAL STROKE" },
    { 0x2A69, "TRIPLE HORIZONTAL BAR WITH TRIPLE VERTICAL STROKE" },
    { 0x2A6A, "TILDE OPERATOR WITH DOT ABOVE" },
    { 0x2A6B, "TILDE OPERATOR WITH RISING DOTS" },
    { 0x2A6C, "SIMILAR MINUS SIMILAR" },
    { 0x2A6D, "CONGRUENT WITH DOT ABOVE" },
    { 0x2A6E, "EQUALS WITH ASTERISK" },
    { 0x2A6F, "ALMOST EQUAL TO WITH CIRCUMFLEX ACCENT" },
    { 0x2A70, "APPROXIMATELY EQUAL OR EQUAL TO" },
    { 0x2A71, "EQUALS SIGN ABOVE PLUS SIGN" },
    { 0x2A72, "PLUS SIGN ABOVE EQUALS SIGN" },
    { 0x2A73, "EQUALS SIGN ABOVE TILDE OPERATOR" },
    { 0x2A74, "DOUBLE COLON EQUAL" },
    { 0x2A75, "TWO CONSECUTIVE EQUALS SIGNS" },
    { 0x2A76, "THREE CONSECUTIVE EQUALS SIGNS" },
    { 0x2A77, "EQUALS SIGN WITH TWO DOTS ABOVE AND TWO DOTS BELOW" },
    { 0x2A78, "EQUIVALENT WITH FOUR DOTS ABOVE" },
    { 0x2A79, "LESS-THAN WITH CIRCLE INSIDE" },
    { 0x2A7A, "GREATER-THAN WITH CIRCLE INSIDE" },
    { 0x2A7B, "LESS-THAN WITH QUESTION MARK ABOVE" },
    { 0x2A7C, "GREATER-THAN WITH QUESTION MARK ABOVE" },
    { 0x2A7D, "LESS-THAN OR SLANTED EQUAL TO" },
    { 0x2A7E, "GREATER-THAN OR SLANTED EQUAL TO" },
    { 0x2A7F, "LESS-THAN OR SLANTED EQUAL TO WITH DOT INSIDE" },
    { 0x2A80, "GREATER-THAN OR SLANTED EQUAL TO WITH DOT INSIDE" },
    { 0x2A81, "LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE" },
    { 0x2A82, "GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE" },
    { 0x2A83, "LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE RIGHT" },
    { 0x2A84, "GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE LEFT" },
    { 0x2A85, "LESS-THAN OR APPROXIMATE" },
    { 0x2A86, "GREATER-THAN OR APPROXIMATE" },
    { 0x2A87, "LESS-THAN AND SINGLE-LINE NOT EQUAL TO" },
    { 0x2A88, "GREATER-THAN AND SINGLE-LINE NOT EQUAL TO" },
    { 0x2A89, "LESS-THAN AND NOT APPROXIMATE" },
    { 0x2A8A, "GREATER-THAN AND NOT APPROXIMATE" },
    { 0x2A8B, "LESS-THAN ABOVE DOUBLE-LINE EQUAL ABOVE GREATER-THAN" },
    { 0x2A8C, "GREATER-THAN ABOVE DOUBLE-LINE EQUAL ABOVE LESS-THAN" },
    { 0x2A8D, "LESS-THAN ABOVE SIMILAR OR EQUAL" },
    { 0x2A8E, "GREATER-THAN ABOVE SIMILAR OR EQUAL" },
    { 0x2A8F, "LESS-THAN ABOVE SIMILAR ABOVE GREATER-THAN" },
    { 0x2A90, "GREATER-THAN ABOVE SIMILAR ABOVE LESS-THAN" },
    { 0x2A91, "LESS-THAN ABOVE GREATER-THAN ABOVE DOUBLE-LINE EQUAL" },
    { 0x2A92, "GREATER-THAN ABOVE LESS-THAN ABOVE DOUBLE-LINE EQUAL" },
    { 0x2A93, "LESS-THAN ABOVE SLANTED EQUAL ABOVE GREATER-THAN ABOVE SLANTED EQUAL" },
    { 0x2A94, "GREATER-THAN ABOVE SLANTED EQUAL ABOVE LESS-THAN ABOVE SLANTED EQUAL" },
    { 0x2A95, "SLANTED EQUAL TO OR LESS-THAN" },
    { 0x2A96, "SLANTED EQUAL TO OR GREATER-THAN" },
    { 0x2A97, "SLANTED EQUAL TO OR LESS-THAN WITH DOT INSIDE" },
    { 0x2A98, "SLANTED EQUAL TO OR GREATER-THAN WITH DOT INSIDE" },
    { 0x2A99, "DOUBLE-LINE EQUAL TO OR LESS-THAN" },
    { 0x2A9A, "DOUBLE-LINE EQUAL TO OR GREATER-THAN" },
    { 0x2A9B, "DOUBLE-LINE SLANTED EQUAL TO OR LESS-THAN" },
    { 0x2A9C, "DOUBLE-LINE SLANTED EQUAL TO OR GREATER-THAN" },
    { 0x2A9D, "SIMILAR OR LESS-THAN" },
    { 0x2A9E, "SIMILAR OR GREATER-THAN" },
    { 0x2A9F, "SIMILAR ABOVE LESS-THAN ABOVE EQUALS SIGN" },
    { 0x2AA0, "SIMILAR ABOVE GREATER-THAN ABOVE EQUALS SIGN" },
    { 0x2AA1, "DOUBLE NESTED LESS-THAN" },
    { 0x2AA2, "DOUBLE NESTED GREATER-THAN" },
    { 0x2AA3, "DOUBLE NESTED LESS-THAN WITH UNDERBAR" },
    { 0x2AA4, "GREATER-THAN OVERLAPPING LESS-THAN" },
    { 0x2AA5, "GREATER-THAN BESIDE LESS-THAN" },
    { 0x2AA6, "LESS-THAN CLOSED BY CURVE" },
    { 0x2AA7, "GREATER-THAN CLOSED BY CURVE" },
    { 0x2AA8, "LESS-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL" },
    { 0x2AA9, "GREATER-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL" },
    { 0x2AAA, "SMALLER THAN" },
    { 0x2AAB, "LARGER THAN" },
    { 0x2AAC, "SMALLER THAN OR EQUAL TO" },
    { 0x2AAD, "LARGER THAN OR EQUAL TO" },
    { 0x2AAE, "EQUALS SIGN WITH BUMPY ABOVE" },
    { 0x2AAF, "PRECEDES ABOVE SINGLE-LINE EQUALS SIGN" },
    { 0x2AB0, "SUCCEEDS ABOVE SINGLE-LINE EQUALS SIGN" },
    { 0x2AB1, "PRECEDES ABOVE SINGLE-LINE NOT EQUAL TO" },
    { 0x2AB2, "SUCCEEDS ABOVE SINGLE-LINE NOT EQUAL TO" },
    { 0x2AB3, "PRECEDES ABOVE EQUALS SIGN" },
    { 0x2AB4, "SUCCEEDS ABOVE EQUALS SIGN" },
    { 0x2AB5, "PRECEDES ABOVE NOT EQUAL TO" },
    { 0x2AB6, "SUCCEEDS ABOVE NOT EQUAL TO" },
    { 0x2AB7, "PRECEDES ABOVE ALMOST EQUAL TO" },
    { 0x2AB8, "SUCCEEDS ABOVE ALMOST EQUAL TO" },
    { 0x2AB9, "PRECEDES ABOVE NOT ALMOST EQUAL TO" },
    { 0x2ABA, "SUCCEEDS ABOVE NOT ALMOST EQUAL TO" },
    { 0x2ABB, "DOUBLE PRECEDES" },
    { 0x2ABC, "DOUBLE SUCCEEDS" },
    { 0x2ABD, "SUBSET WITH DOT" },
    { 0x2ABE, "SUPERSET WITH DOT" },
    { 0x2ABF, "SUBSET WITH PLUS SIGN BELOW" },
    { 0x2AC0, "SUPERSET WITH PLUS SIGN BELOW" },
    { 0x2AC1, "SUBSET WITH MULTIPLICATION SIGN BELOW" },
    { 0x2AC2, "SUPERSET WITH MULTIPLICATION SIGN BELOW" },
    { 0x2AC3, "SUBSET OF OR EQUAL TO WITH DOT ABOVE" },
    { 0x2AC4, "SUPERSET OF OR EQUAL TO WITH DOT ABOVE" },
    { 0x2AC5, "SUBSET OF ABOVE EQUALS SIGN" },
    { 0x2AC6, "SUPERSET OF ABOVE EQUALS SIGN" },
    { 0x2AC7, "SUBSET OF ABOVE TILDE OPERATOR" },
    { 0x2AC8, "SUPERSET OF ABOVE TILDE OPERATOR" },
    { 0x2AC9, "SUBSET OF ABOVE ALMOST EQUAL TO" },
    { 0x2ACA, "SUPERSET OF ABOVE ALMOST EQUAL TO" },
    { 0x2ACB, "SUBSET OF ABOVE NOT EQUAL TO" },
    { 0x2ACC, "SUPERSET OF ABOVE NOT EQUAL TO" },
    { 0x2ACD, "SQUARE LEFT OPEN BOX OPERATOR" },
    { 0x2ACE, "SQUARE RIGHT OPEN BOX OPERATOR" },
    { 0x2ACF, "CLOSED SUBSET" },
    { 0x2AD0, "CLOSED SUPERSET" },
    { 0x2AD1, "CLOSED SUBSET OR EQUAL TO" },
    { 0x2AD2, "CLOSED SUPERSET OR EQUAL TO" },
    { 0x2AD3, "SUBSET ABOVE SUPERSET" },
    { 0x2AD4, "SUPERSET ABOVE SUBSET" },
    { 0x2AD5, "SUBSET ABOVE SUBSET" },
    { 0x2AD6, "SUPERSET ABOVE SUPERSET" },
    { 0x2AD7, "SUPERSET BESIDE SUBSET" },
    { 0x2AD8, "SUPERSET BESIDE AND JOINED BY DASH WITH SUBSET" },
    { 0x2AD9, "ELEMENT OF OPENING DOWNWARDS" },
    { 0x2ADA, "PITCHFORK WITH TEE TOP" },
    { 0x2ADB, "TRANSVERSAL INTERSECTION" },
    { 0x2ADC, "FORKING" },
    { 0x2ADD, "NONFORKING" },
    { 0x2ADE, "SHORT LEFT TACK" },
    { 0x2ADF, "SHORT DOWN TACK" },
    { 0x2AE0, "SHORT UP TACK" },
    { 0x2AE1, "PERPENDICULAR WITH S" },
    { 0x2AE2, "VERTICAL BAR TRIPLE RIGHT TURNSTILE" },
    { 0x2AE3, "DOUBLE VERTICAL BAR LEFT TURNSTILE" },
    { 0x2AE4, "VERTICAL BAR DOUBLE LEFT TURNSTILE" },
    { 0x2AE5, "DOUBLE VERTICAL BAR DOUBLE LEFT TURNSTILE" },
    { 0x2AE6, "LONG DASH FROM LEFT MEMBER OF DOUBLE VERTICAL" },
    { 0x2AE7, "SHORT DOWN TACK WITH OVERBAR" },
    { 0x2AE8, "SHORT UP TACK WITH UNDERBAR" },
    { 0x2AE9, "SHORT UP TACK ABOVE SHORT DOWN TACK" },
    { 0x2AEA, "DOUBLE DOWN TACK" },
    { 0x2AEB, "DOUBLE UP TACK" },
    { 0x2AEC, "DOUBLE STROKE NOT SIGN" },
    { 0x2AED, "REVERSED DOUBLE STROKE NOT SIGN" },
    { 0x2AEE, "DOES NOT DIVIDE WITH REVERSED NEGATION SLASH" },
    { 0x2AEF, "VERTICAL LINE WITH CIRCLE ABOVE" },
    { 0x2AF0, "VERTICAL LINE WITH CIRCLE BELOW" },
    { 0x2AF1, "DOWN TACK WITH CIRCLE BELOW" },
    { 0x2AF2, "PARALLEL WITH HORIZONTAL STROKE" },
    { 0x2AF3, "PARALLEL WITH TILDE OPERATOR" },
    { 0x2AF4, "TRIPLE VERTICAL BAR BINARY RELATION" },
    { 0x2AF5, "TRIPLE VERTICAL BAR WITH HORIZONTAL STROKE" },
    { 0x2AF6, "TRIPLE COLON OPERATOR" },
    { 0x2AF7, "TRIPLE NESTED LESS-THAN" },
    { 0x2AF8, "TRIPLE NESTED GREATER-THAN" },
    { 0x2AF9, "DOUBLE-LINE SLANTED LESS-THAN OR EQUAL TO" },
    { 0x2AFA, "DOUBLE-LINE SLANTED GREATER-THAN OR EQUAL TO" },
    { 0x2AFB, "TRIPLE SOLIDUS BINARY RELATION" },
    { 0x2AFC, "LARGE TRIPLE VERTICAL BAR OPERATOR" },
    { 0x2AFD, "DOUBLE SOLIDUS OPERATOR" },
    { 0x2AFE, "WHITE VERTICAL BAR" },
    { 0x2AFF, "N-ARY WHITE VERTICAL BAR" },
    { 0x2B00, "NORTH EAST WHITE ARROW" },
    { 0x2B01, "NORTH WEST WHITE ARROW" },
    { 0x2B02, "SOUTH EAST WHITE ARROW" },
    { 0x2B03, "SOUTH WEST WHITE ARROW" },
    { 0x2B04, "LEFT RIGHT WHITE ARROW" },
    { 0x2B05, "LEFTWARDS BLACK ARROW" },
    { 0x2B06, "UPWARDS BLACK ARROW" },
    { 0x2B07, "DOWNWARDS BLACK ARROW" },
    { 0x2B08, "NORTH EAST BLACK ARROW" },
    { 0x2B09, "NORTH WEST BLACK ARROW" },
    { 0x2B0A, "SOUTH EAST BLACK ARROW" },
    { 0x2B0B, "SOUTH WEST BLACK ARROW" },
    { 0x2B0C, "LEFT RIGHT BLACK ARROW" },
    { 0x2B0D, "UP DOWN BLACK ARROW" },
    { 0x2B0E, "RIGHTWARDS ARROW WITH TIP DOWNWARDS" },
    { 0x2B0F, "RIGHTWARDS ARROW WITH TIP UPWARDS" },
    { 0x2B10, "LEFTWARDS ARROW WITH TIP DOWNWARDS" },
    { 0x2B11, "LEFTWARDS ARROW WITH TIP UPWARDS" },
    { 0x2B12, "SQUARE WITH TOP HALF BLACK" },
    { 0x2B13, "SQUARE WITH BOTTOM HALF BLACK" },
    { 0x2B14, "SQUARE WITH UPPER RIGHT DIAGONAL HALF BLACK" },
    { 0x2B15, "SQUARE WITH LOWER LEFT DIAGONAL HALF BLACK" },
    { 0x2B16, "DIAMOND WITH LEFT HALF BLACK" },
    { 0x2B17, "DIAMOND WITH RIGHT HALF BLACK" },
    { 0x2B18, "DIAMOND WITH TOP HALF BLACK" },
    { 0x2B19, "DIAMOND WITH BOTTOM HALF BLACK" },
    { 0x2B1A, "DOTTED SQUARE" },
    { 0x2B1D, "BLACK VERY SMALL SQUARE" },
    { 0x2B1E, "WHITE VERY SMALL SQUARE" },
    { 0x2B1F, "BLACK PENTAGON" },
    { 0x2B20, "WHITE PENTAGON" },
    { 0x2B21, "WHITE HEXAGON" },
    { 0x2B22, "BLACK HEXAGON" },
    { 0x2B23, "HORIZONTAL BLACK HEXAGON" },
    { 0x2B24, "BLACK LARGE CIRCLE" },
    { 0x2B25, "BLACK MEDIUM DIAMOND" },
    { 0x2B26, "WHITE MEDIUM DIAMOND" },
    { 0x2B27, "BLACK MEDIUM LOZENGE" },
    { 0x2B28, "WHITE MEDIUM LOZENGE" },
    { 0x2B29, "BLACK SMALL DIAMOND" },
    { 0x2B2A, "BLACK SMALL LOZENGE" },
    { 0x2B2B, "WHITE SMALL LOZENGE" },
    { 0x2B2C, "BLACK HORIZONTAL ELLIPSE" },
    { 0x2B2D, "WHITE HORIZONTAL ELLIPSE" },
    { 0x2B2E, "BLACK VERTICAL ELLIPSE" },
    { 0x2B2F, "WHITE VERTICAL ELLIPSE" },
    { 0x2B30, "LEFT ARROW WITH SMALL CIRCLE" },
    { 0x2B31, "THREE LEFTWARDS ARROWS" },
    { 0x2B32, "LEFT ARROW WITH CIRCLED PLUS" },
    { 0x2B33, "LONG LEFTWARDS SQUIGGLE ARROW" },
    { 0x2B34, "LEFTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE" },
    { 0x2B35, "LEFTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE" },
    { 0x2B36, "LEFTWARDS TWO-HEADED ARROW FROM BAR" },
    { 0x2B37, "LEFTWARDS TWO-HEADED TRIPLE DASH ARROW" },
    { 0x2B38, "LEFTWARDS ARROW WITH DOTTED STEM" },
    { 0x2B39, "LEFTWARDS ARROW WITH TAIL WITH VERTICAL STROKE" },
    { 0x2B3A, "LEFTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE" },
    { 0x2B3B, "LEFTWARDS TWO-HEADED ARROW WITH TAIL" },
    { 0x2B3C, "LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE" },
    { 0x2B3D, "LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE" },
    { 0x2B3E, "LEFTWARDS ARROW THROUGH X" },
    { 0x2B3F, "WAVE ARROW POINTING DIRECTLY LEFT" },
    { 0x2B40, "EQUALS SIGN ABOVE LEFTWARDS ARROW" },
    { 0x2B41, "REVERSE TILDE OPERATOR ABOVE LEFTWARDS ARROW" },
    { 0x2B42, "LEFTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO" },
    { 0x2B43, "RIGHTWARDS ARROW THROUGH GREATER-THAN" },
    { 0x2B44, "RIGHTWARDS ARROW THROUGH SUPERSET" },
    { 0x2B45, "LEFTWARDS QUADRUPLE ARROW" },
    { 0x2B46, "RIGHTWARDS QUADRUPLE ARROW" },
    { 0x2B47, "REVERSE TILDE OPERATOR ABOVE RIGHTWARDS ARROW" },
    { 0x2B48, "RIGHTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO" },
    { 0x2B49, "TILDE OPERATOR ABOVE LEFTWARDS ARROW" },
    { 0x2B4A, "LEFTWARDS ARROW ABOVE ALMOST EQUAL TO" },
    { 0x2B4B, "LEFTWARDS ARROW ABOVE REVERSE TILDE OPERATOR" },
    { 0x2B4C, "RIGHTWARDS ARROW ABOVE REVERSE TILDE OPERATOR" },
    { 0x2B4D, "DOWNWARDS TRIANGLE-HEADED ZIGZAG ARROW" },
    { 0x2B4E, "SHORT SLANTED NORTH ARROW" },
    { 0x2B4F, "SHORT BACKSLANTED SOUTH ARROW" },
    { 0x2B51, "BLACK SMALL STAR" },
    { 0x2B52, "WHITE SMALL STAR" },
    { 0x2B53, "BLACK RIGHT-POINTING PENTAGON" },
    { 0x2B54, "WHITE RIGHT-POINTING PENTAGON" },
    { 0x2B56, "HEAVY OVAL WITH OVAL INSIDE" },
    { 0x2B57, "HEAVY CIRCLE WITH CIRCLE INSIDE" },
    { 0x2B58, "HEAVY CIRCLE" },
    { 0x2B59, "HEAVY CIRCLED SALTIRE" },
    { 0x2B5A, "SLANTED NORTH ARROW WITH HOOKED HEAD" },
    { 0x2B5B, "BACKSLANTED SOUTH ARROW WITH HOOKED TAIL" },
    { 0x2B5C, "SLANTED NORTH ARROW WITH HORIZONTAL TAIL" },
    { 0x2B5D, "BACKSLANTED SOUTH ARROW WITH HORIZONTAL TAIL" },
    { 0x2B5E, "BENT ARROW POINTING DOWNWARDS THEN NORTH EAST" },
    { 0x2B5F, "SHORT BENT ARROW POINTING DOWNWARDS THEN NORTH EAST" },
    { 0x2B60, "LEFTWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B61, "UPWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B62, "RIGHTWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B63, "DOWNWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B64, "LEFT RIGHT TRIANGLE-HEADED ARROW" },
    { 0x2B65, "UP DOWN TRIANGLE-HEADED ARROW" },
    { 0x2B66, "NORTH WEST TRIANGLE-HEADED ARROW" },
    { 0x2B67, "NORTH EAST TRIANGLE-HEADED ARROW" },
    { 0x2B68, "SOUTH EAST TRIANGLE-HEADED ARROW" },
    { 0x2B69, "SOUTH WEST TRIANGLE-HEADED ARROW" },
    { 0x2B6A, "LEFTWARDS TRIANGLE-HEADED DASHED ARROW" },
    { 0x2B6B, "UPWARDS TRIANGLE-HEADED DASHED ARROW" },
    { 0x2B6C, "RIGHTWARDS TRIANGLE-HEADED DASHED ARROW" },
    { 0x2B6D, "DOWNWARDS TRIANGLE-HEADED DASHED ARROW" },
    { 0x2B6E, "CLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW" },
    { 0x2B6F, "ANTICLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW" },
    { 0x2B70, "LEFTWARDS TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B71, "UPWARDS TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B72, "RIGHTWARDS TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B73, "DOWNWARDS TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B76, "NORTH WEST TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B77, "NORTH EAST TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B78, "SOUTH EAST TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B79, "SOUTH WEST TRIANGLE-HEADED ARROW TO BAR" },
    { 0x2B7A, "LEFTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE" },
    { 0x2B7B, "UPWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE" },
    { 0x2B7C, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE" },
    { 0x2B7D, "DOWNWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE" },
    { 0x2B7E, "HORIZONTAL TAB KEY" },
    { 0x2B7F, "VERTICAL TAB KEY" },
    { 0x2B80, "LEFTWARDS TRIANGLE-HEADED ARROW OVER RIGHTWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B81, "UPWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF DOWNWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B82, "RIGHTWARDS TRIANGLE-HEADED ARROW OVER LEFTWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B83, "DOWNWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF UPWARDS TRIANGLE-HEADED ARROW" },
    { 0x2B84, "LEFTWARDS TRIANGLE-HEADED PAIRED ARROWS" },
    { 0x2B85, "UPWARDS TRIANGLE-HEADED PAIRED ARROWS" },
    { 0x2B86, "RIGHTWARDS TRIANGLE-HEADED PAIRED ARROWS" },
    { 0x2B87, "DOWNWARDS TRIANGLE-HEADED PAIRED ARROWS" },
    { 0x2B88, "LEFTWARDS BLACK CIRCLED WHITE ARROW" },
    { 0x2B89, "UPWARDS BLACK CIRCLED WHITE ARROW" },
    { 0x2B8A, "RIGHTWARDS BLACK CIRCLED WHITE ARROW" },
    { 0x2B8B, "DOWNWARDS BLACK CIRCLED WHITE ARROW" },
    { 0x2B8C, "ANTICLOCKWISE TRIANGLE-HEADED RIGHT U-SHAPED ARROW" },
    { 0x2B8D, "ANTICLOCKWISE TRIANGLE-HEADED BOTTOM U-SHAPED ARROW" },
    { 0x2B8E, "ANTICLOCKWISE TRIANGLE-HEADED LEFT U-SHAPED ARROW" },
    { 0x2B8F, "ANTICLOCKWISE TRIANGLE-HEADED TOP U-SHAPED ARROW" },
    { 0x2B90, "RETURN LEFT" },
    { 0x2B91, "RETURN RIGHT" },
    { 0x2B92, "NEWLINE LEFT" },
    { 0x2B93, "NEWLINE RIGHT" },
    { 0x2B94, "FOUR CORNER ARROWS CIRCLING ANTICLOCKWISE" },
    { 0x2B95, "RIGHTWARDS BLACK ARROW" },
    { 0x2B97, "SYMBOL FOR TYPE A ELECTRONICS" },
    { 0x2B98, "THREE-D TOP-LIGHTED LEFTWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B99, "THREE-D RIGHT-LIGHTED UPWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B9A, "THREE-D TOP-LIGHTED RIGHTWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B9B, "THREE-D LEFT-LIGHTED DOWNWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B9C, "BLACK LEFTWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B9D, "BLACK UPWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B9E, "BLACK RIGHTWARDS EQUILATERAL ARROWHEAD" },
    { 0x2B9F, "BLACK DOWNWARDS EQUILATERAL ARROWHEAD" },
    { 0x2BA0, "DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS" },
    { 0x2BA1, "DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS" },
    { 0x2BA2, "UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS" },
    { 0x2BA3, "UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS" },
    { 0x2BA4, "LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS" },
    { 0x2BA5, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS" },
    { 0x2BA6, "LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS" },
    { 0x2BA7, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS" },
    { 0x2BA8, "BLACK CURVED DOWNWARDS AND LEFTWARDS ARROW" },
    { 0x2BA9, "BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW" },
    { 0x2BAA, "BLACK CURVED UPWARDS AND LEFTWARDS ARROW" },
    { 0x2BAB, "BLACK CURVED UPWARDS AND RIGHTWARDS ARROW" },
    { 0x2BAC, "BLACK CURVED LEFTWARDS AND UPWARDS ARROW" },
    { 0x2BAD, "BLACK CURVED RIGHTWARDS AND UPWARDS ARROW" },
    { 0x2BAE, "BLACK CURVED LEFTWARDS AND DOWNWARDS ARROW" },
    { 0x2BAF, "BLACK CURVED RIGHTWARDS AND DOWNWARDS ARROW" },
    { 0x2BB0, "RIBBON ARROW DOWN LEFT" },
    { 0x2BB1, "RIBBON ARROW DOWN RIGHT" },
    { 0x2BB2, "RIBBON ARROW UP LEFT" },
    { 0x2BB3, "RIBBON ARROW UP RIGHT" },
    { 0x2BB4, "RIBBON ARROW LEFT UP" },
    { 0x2BB5, "RIBBON ARROW RIGHT UP" },
    { 0x2BB6, "RIBBON ARROW LEFT DOWN" },
    { 0x2BB7, "RIBBON ARROW RIGHT DOWN" },
    { 0x2BB8, "UPWARDS WHITE ARROW FROM BAR WITH HORIZONTAL BAR" },
    { 0x2BB9, "UP ARROWHEAD IN A RECTANGLE BOX" },
    { 0x2BBA, "OVERLAPPING WHITE SQUARES" },
    { 0x2BBB, "OVERLAPPING WHITE AND BLACK SQUARES" },
    { 0x2BBC, "OVERLAPPING BLACK SQUARES" },
    { 0x2BBD, "BALLOT BOX WITH LIGHT X" },
    { 0x2BBE, "CIRCLED X" },
    { 0x2BBF, "CIRCLED BOLD X" },
    { 0x2BC0, "BLACK SQUARE CENTRED" },
    { 0x2BC1, "BLACK DIAMOND CENTRED" },
    { 0x2BC2, "TURNED BLACK PENTAGON" },
    { 0x2BC3, "HORIZONTAL BLACK OCTAGON" },
    { 0x2BC4, "BLACK OCTAGON" },
    { 0x2BC5, "BLACK MEDIUM UP-POINTING TRIANGLE CENTRED" },
    { 0x2BC6, "BLACK MEDIUM DOWN-POINTING TRIANGLE CENTRED" },
    { 0x2BC7, "BLACK MEDIUM LEFT-POINTING TRIANGLE CENTRED" },
    { 0x2BC8, "BLACK MEDIUM RIGHT-POINTING TRIANGLE CENTRED" },
    { 0x2BC9, "NEPTUNE FORM TWO" },
    { 0x2BCA, "TOP HALF BLACK CIRCLE" },
    { 0x2BCB, "BOTTOM HALF BLACK CIRCLE" },
    { 0x2BCC, "LIGHT FOUR POINTED BLACK CUSP" },
    { 0x2BCD, "ROTATED LIGHT FOUR POINTED BLACK CUSP" },
    { 0x2BCE, "WHITE FOUR POINTED CUSP" },
    { 0x2BCF, "ROTATED WHITE FOUR POINTED CUSP" },
    { 0x2BD0, "SQUARE POSITION INDICATOR" },
    { 0x2BD1, "UNCERTAINTY SIGN" },
    { 0x2BD2, "GROUP MARK" },
    { 0x2BD3, "PLUTO FORM TWO" },
    { 0x2BD4, "PLUTO FORM THREE" },
    { 0x2BD5, "PLUTO FORM FOUR" },
    { 0x2BD6, "PLUTO FORM FIVE" },
    { 0x2BD7, "TRANSPLUTO" },
    { 0x2BD8, "PROSERPINA" },
    { 0x2BD9, "ASTRAEA" },
    { 0x2BDA, "HYGIEA" },
    { 0x2BDB, "PHOLUS" },
    { 0x2BDC, "NESSUS" },
    { 0x2BDD, "WHITE MOON SELENA" },
    { 0x2BDE, "BLACK DIAMOND ON CROSS" },
    { 0x2BDF, "TRUE LIGHT MOON ARTA" },
    { 0x2BE0, "CUPIDO" },
    { 0x2BE1, "HADES" },
    { 0x2BE2, "ZEUS" },
    { 0x2BE3, "KRONOS" },
    { 0x2BE4, "APOLLON" },
    { 0x2BE5, "ADMETOS" },
    { 0x2BE6, "VULCANUS" },
    { 0x2BE7, "POSEIDON" },
    { 0x2BE8, "LEFT HALF BLACK STAR" },
    { 0x2BE9, "RIGHT HALF BLACK STAR" },
    { 0x2BEA, "STAR WITH LEFT HALF BLACK" },
    { 0x2BEB, "STAR WITH RIGHT HALF BLACK" },
    { 0x2BEC, "LEFTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS" },
    { 0x2BED, "UPWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS" },
    { 0x2BEE, "RIGHTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS" },
    { 0x2BEF, "DOWNWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS" },
    { 0x2BF0, "ERIS FORM ONE" },
    { 0x2BF1, "ERIS FORM TWO" },
    { 0x2BF2, "SEDNA" },
    { 0x2BF3, "RUSSIAN ASTROLOGICAL SYMBOL VIGINTILE" },
    { 0x2BF4, "RUSSIAN ASTROLOGICAL SYMBOL NOVILE" },
    { 0x2BF5, "RUSSIAN ASTROLOGICAL SYMBOL QUINTILE" },
    { 0x2BF6, "RUSSIAN ASTROLOGICAL SYMBOL BINOVILE" },
    { 0x2BF7, "RUSSIAN ASTROLOGICAL SYMBOL SENTAGON" },
    { 0x2BF8, "RUSSIAN ASTROLOGICAL SYMBOL TREDECILE" },
    { 0x2BF9, "EQUALS SIGN WITH INFINITY BELOW" },
    { 0x2BFA, "UNITED SYMBOL" },
    { 0x2BFB, "SEPARATED SYMBOL" },
    { 0x2BFC, "DOUBLED SYMBOL" },
    { 0x2BFD, "PASSED SYMBOL" },
    { 0x2BFE, "REVERSED RIGHT ANGLE" },
    { 0x2BFF, "HELLSCHREIBER PAUSE SYMBOL" },
    { 0x2C00, "GLAGOLITIC CAPITAL LETTER AZU" },
    { 0x2C01, "GLAGOLITIC CAPITAL LETTER BUKY" },
    { 0x2C02, "GLAGOLITIC CAPITAL LETTER VEDE" },
    { 0x2C03, "GLAGOLITIC CAPITAL LETTER GLAGOLI" },
    { 0x2C04, "GLAGOLITIC CAPITAL LETTER DOBRO" },
    { 0x2C05, "GLAGOLITIC CAPITAL LETTER YESTU" },
    { 0x2C06, "GLAGOLITIC CAPITAL LETTER ZHIVETE" },
    { 0x2C07, "GLAGOLITIC CAPITAL LETTER DZELO" },
    { 0x2C08, "GLAGOLITIC CAPITAL LETTER ZEMLJA" },
    { 0x2C09, "GLAGOLITIC CAPITAL LETTER IZHE" },
    { 0x2C0A, "GLAGOLITIC CAPITAL LETTER INITIAL IZHE" },
    { 0x2C0B, "GLAGOLITIC CAPITAL LETTER I" },
    { 0x2C0C, "GLAGOLITIC CAPITAL LETTER DJERVI" },
    { 0x2C0D, "GLAGOLITIC CAPITAL LETTER KAKO" },
    { 0x2C0E, "GLAGOLITIC CAPITAL LETTER LJUDIJE" },
    { 0x2C0F, "GLAGOLITIC CAPITAL LETTER MYSLITE" },
    { 0x2C10, "GLAGOLITIC CAPITAL LETTER NASHI" },
    { 0x2C11, "GLAGOLITIC CAPITAL LETTER ONU" },
    { 0x2C12, "GLAGOLITIC CAPITAL LETTER POKOJI" },
    { 0x2C13, "GLAGOLITIC CAPITAL LETTER RITSI" },
    { 0x2C14, "GLAGOLITIC CAPITAL LETTER SLOVO" },
    { 0x2C15, "GLAGOLITIC CAPITAL LETTER TVRIDO" },
    { 0x2C16, "GLAGOLITIC CAPITAL LETTER UKU" },
    { 0x2C17, "GLAGOLITIC CAPITAL LETTER FRITU" },
    { 0x2C18, "GLAGOLITIC CAPITAL LETTER HERU" },
    { 0x2C19, "GLAGOLITIC CAPITAL LETTER OTU" },
    { 0x2C1A, "GLAGOLITIC CAPITAL LETTER PE" },
    { 0x2C1B, "GLAGOLITIC CAPITAL LETTER SHTA" },
    { 0x2C1C, "GLAGOLITIC CAPITAL LETTER TSI" },
    { 0x2C1D, "GLAGOLITIC CAPITAL LETTER CHRIVI" },
    { 0x2C1E, "GLAGOLITIC CAPITAL LETTER SHA" },
    { 0x2C1F, "GLAGOLITIC CAPITAL LETTER YERU" },
    { 0x2C20, "GLAGOLITIC CAPITAL LETTER YERI" },
    { 0x2C21, "GLAGOLITIC CAPITAL LETTER YATI" },
    { 0x2C22, "GLAGOLITIC CAPITAL LETTER SPIDERY HA" },
    { 0x2C23, "GLAGOLITIC CAPITAL LETTER YU" },
    { 0x2C24, "GLAGOLITIC CAPITAL LETTER SMALL YUS" },
    { 0x2C25, "GLAGOLITIC CAPITAL LETTER SMALL YUS WITH TAIL" },
    { 0x2C26, "GLAGOLITIC CAPITAL LETTER YO" },
    { 0x2C27, "GLAGOLITIC CAPITAL LETTER IOTATED SMALL YUS" },
    { 0x2C28, "GLAGOLITIC CAPITAL LETTER BIG YUS" },
    { 0x2C29, "GLAGOLITIC CAPITAL LETTER IOTATED BIG YUS" },
    { 0x2C2A, "GLAGOLITIC CAPITAL LETTER FITA" },
    { 0x2C2B, "GLAGOLITIC CAPITAL LETTER IZHITSA" },
    { 0x2C2C, "GLAGOLITIC CAPITAL LETTER SHTAPIC" },
    { 0x2C2D, "GLAGOLITIC CAPITAL LETTER TROKUTASTI A" },
    { 0x2C2E, "GLAGOLITIC CAPITAL LETTER LATINATE MYSLITE" },
    { 0x2C2F, "GLAGOLITIC CAPITAL LETTER CAUDATE CHRIVI" },
    { 0x2C30, "GLAGOLITIC SMALL LETTER AZU" },
    { 0x2C31, "GLAGOLITIC SMALL LETTER BUKY" },
    { 0x2C32, "GLAGOLITIC SMALL LETTER VEDE" },
    { 0x2C33, "GLAGOLITIC SMALL LETTER GLAGOLI" },
    { 0x2C34, "GLAGOLITIC SMALL LETTER DOBRO" },
    { 0x2C35, "GLAGOLITIC SMALL LETTER YESTU" },
    { 0x2C36, "GLAGOLITIC SMALL LETTER ZHIVETE" },
    { 0x2C37, "GLAGOLITIC SMALL LETTER DZELO" },
    { 0x2C38, "GLAGOLITIC SMALL LETTER ZEMLJA" },
    { 0x2C39, "GLAGOLITIC SMALL LETTER IZHE" },
    { 0x2C3A, "GLAGOLITIC SMALL LETTER INITIAL IZHE" },
    { 0x2C3B, "GLAGOLITIC SMALL LETTER I" },
    { 0x2C3C, "GLAGOLITIC SMALL LETTER DJERVI" },
    { 0x2C3D, "GLAGOLITIC SMALL LETTER KAKO" },
    { 0x2C3E, "GLAGOLITIC SMALL LETTER LJUDIJE" },
    { 0x2C3F, "GLAGOLITIC SMALL LETTER MYSLITE" },
    { 0x2C40, "GLAGOLITIC SMALL LETTER NASHI" },
    { 0x2C41, "GLAGOLITIC SMALL LETTER ONU" },
    { 0x2C42, "GLAGOLITIC SMALL LETTER POKOJI" },
    { 0x2C43, "GLAGOLITIC SMALL LETTER RITSI" },
    { 0x2C44, "GLAGOLITIC SMALL LETTER SLOVO" },
    { 0x2C45, "GLAGOLITIC SMALL LETTER TVRIDO" },
    { 0x2C46, "GLAGOLITIC SMALL LETTER UKU" },
    { 0x2C47, "GLAGOLITIC SMALL LETTER FRITU" },
    { 0x2C48, "GLAGOLITIC SMALL LETTER HERU" },
    { 0x2C49, "GLAGOLITIC SMALL LETTER OTU" },
    { 0x2C4A, "GLAGOLITIC SMALL LETTER PE" },
    { 0x2C4B, "GLAGOLITIC SMALL LETTER SHTA" },
    { 0x2C4C, "GLAGOLITIC SMALL LETTER TSI" },
    { 0x2C4D, "GLAGOLITIC SMALL LETTER CHRIVI" },
    { 0x2C4E, "GLAGOLITIC SMALL LETTER SHA" },
    { 0x2C4F, "GLAGOLITIC SMALL LETTER YERU" },
    { 0x2C50, "GLAGOLITIC SMALL LETTER YERI" },
    { 0x2C51, "GLAGOLITIC SMALL LETTER YATI" },
    { 0x2C52, "GLAGOLITIC SMALL LETTER SPIDERY HA" },
    { 0x2C53, "GLAGOLITIC SMALL LETTER YU" },
    { 0x2C54, "GLAGOLITIC SMALL LETTER SMALL YUS" },
    { 0x2C55, "GLAGOLITIC SMALL LETTER SMALL YUS WITH TAIL" },
    { 0x2C56, "GLAGOLITIC SMALL LETTER YO" },
    { 0x2C57, "GLAGOLITIC SMALL LETTER IOTATED SMALL YUS" },
    { 0x2C58, "GLAGOLITIC SMALL LETTER BIG YUS" },
    { 0x2C59, "GLAGOLITIC SMALL LETTER IOTATED BIG YUS" },
    { 0x2C5A, "GLAGOLITIC SMALL LETTER FITA" },
    { 0x2C5B, "GLAGOLITIC SMALL LETTER IZHITSA" },
    { 0x2C5C, "GLAGOLITIC SMALL LETTER SHTAPIC" },
    { 0x2C5D, "GLAGOLITIC SMALL LETTER TROKUTASTI A" },
    { 0x2C5E, "GLAGOLITIC SMALL LETTER LATINATE MYSLITE" },
    { 0x2C5F, "GLAGOLITIC SMALL LETTER CAUDATE CHRIVI" },
    { 0x2C60, "LATIN CAPITAL LETTER L WITH DOUBLE BAR" },
    { 0x2C61, "LATIN SMALL LETTER L WITH DOUBLE BAR" },
    { 0x2C62, "LATIN CAPITAL LETTER L WITH MIDDLE TILDE" },
    { 0x2C63, "LATIN CAPITAL LETTER P WITH STROKE" },
    { 0x2C64, "LATIN CAPITAL LETTER R WITH TAIL" },
    { 0x2C65, "LATIN SMALL LETTER A WITH STROKE" },
    { 0x2C66, "LATIN SMALL LETTER T WITH DIAGONAL STROKE" },
    { 0x2C67, "LATIN CAPITAL LETTER H WITH DESCENDER" },
    { 0x2C68, "LATIN SMALL LETTER H WITH DESCENDER" },
    { 0x2C69, "LATIN CAPITAL LETTER K WITH DESCENDER" },
    { 0x2C6A, "LATIN SMALL LETTER K WITH DESCENDER" },
    { 0x2C6B, "LATIN CAPITAL LETTER Z WITH DESCENDER" },
    { 0x2C6C, "LATIN SMALL LETTER Z WITH DESCENDER" },
    { 0x2C6D, "LATIN CAPITAL LETTER ALPHA" },
    { 0x2C6E, "LATIN CAPITAL LETTER M WITH HOOK" },
    { 0x2C6F, "LATIN CAPITAL LETTER TURNED A" },
    { 0x2C70, "LATIN CAPITAL LETTER TURNED ALPHA" },
    { 0x2C71, "LATIN SMALL LETTER V WITH RIGHT HOOK" },
    { 0x2C72, "LATIN CAPITAL LETTER W WITH HOOK" },
    { 0x2C73, "LATIN SMALL LETTER W WITH HOOK" },
    { 0x2C74, "LATIN SMALL LETTER V WITH CURL" },
    { 0x2C75, "LATIN CAPITAL LETTER HALF H" },
    { 0x2C76, "LATIN SMALL LETTER HALF H" },
    { 0x2C77, "LATIN SMALL LETTER TAILLESS PHI" },
    { 0x2C78, "LATIN SMALL LETTER E WITH NOTCH" },
    { 0x2C79, "LATIN SMALL LETTER TURNED R WITH TAIL" },
    { 0x2C7A, "LATIN SMALL LETTER O WITH LOW RING INSIDE" },
    { 0x2C7B, "LATIN LETTER SMALL CAPITAL TURNED E" },
    { 0x2C7C, "LATIN SUBSCRIPT SMALL LETTER J" },
    { 0x2C7D, "MODIFIER LETTER CAPITAL V" },
    { 0x2C7E, "LATIN CAPITAL LETTER S WITH SWASH TAIL" },
    { 0x2C7F, "LATIN CAPITAL LETTER Z WITH SWASH TAIL" },
    { 0x2C80, "COPTIC CAPITAL LETTER ALFA" },
    { 0x2C81, "COPTIC SMALL LETTER ALFA" },
    { 0x2C82, "COPTIC CAPITAL LETTER VIDA" },
    { 0x2C83, "COPTIC SMALL LETTER VIDA" },
    { 0x2C84, "COPTIC CAPITAL LETTER GAMMA" },
    { 0x2C85, "COPTIC SMALL LETTER GAMMA" },
    { 0x2C86, "COPTIC CAPITAL LETTER DALDA" },
    { 0x2C87, "COPTIC SMALL LETTER DALDA" },
    { 0x2C88, "COPTIC CAPITAL LETTER EIE" },
    { 0x2C89, "COPTIC SMALL LETTER EIE" },
    { 0x2C8A, "COPTIC CAPITAL LETTER SOU" },
    { 0x2C8B, "COPTIC SMALL LETTER SOU" },
    { 0x2C8C, "COPTIC CAPITAL LETTER ZATA" },
    { 0x2C8D, "COPTIC SMALL LETTER ZATA" },
    { 0x2C8E, "COPTIC CAPITAL LETTER HATE" },
    { 0x2C8F, "COPTIC SMALL LETTER HATE" },
    { 0x2C90, "COPTIC CAPITAL LETTER THETHE" },
    { 0x2C91, "COPTIC SMALL LETTER THETHE" },
    { 0x2C92, "COPTIC CAPITAL LETTER IAUDA" },
    { 0x2C93, "COPTIC SMALL LETTER IAUDA" },
    { 0x2C94, "COPTIC CAPITAL LETTER KAPA" },
    { 0x2C95, "COPTIC SMALL LETTER KAPA" },
    { 0x2C96, "COPTIC CAPITAL LETTER LAULA" },
    { 0x2C97, "COPTIC SMALL LETTER LAULA" },
    { 0x2C98, "COPTIC CAPITAL LETTER MI" },
    { 0x2C99, "COPTIC SMALL LETTER MI" },
    { 0x2C9A, "COPTIC CAPITAL LETTER NI" },
    { 0x2C9B, "COPTIC SMALL LETTER NI" },
    { 0x2C9C, "COPTIC CAPITAL LETTER KSI" },
    { 0x2C9D, "COPTIC SMALL LETTER KSI" },
    { 0x2C9E, "COPTIC CAPITAL LETTER O" },
    { 0x2C9F, "COPTIC SMALL LETTER O" },
    { 0x2CA0, "COPTIC CAPITAL LETTER PI" },
    { 0x2CA1, "COPTIC SMALL LETTER PI" },
    { 0x2CA2, "COPTIC CAPITAL LETTER RO" },
    { 0x2CA3, "COPTIC SMALL LETTER RO" },
    { 0x2CA4, "COPTIC CAPITAL LETTER SIMA" },
    { 0x2CA5, "COPTIC SMALL LETTER SIMA" },
    { 0x2CA6, "COPTIC CAPITAL LETTER TAU" },
    { 0x2CA7, "COPTIC SMALL LETTER TAU" },
    { 0x2CA8, "COPTIC CAPITAL LETTER UA" },
    { 0x2CA9, "COPTIC SMALL LETTER UA" },
    { 0x2CAA, "COPTIC CAPITAL LETTER FI" },
    { 0x2CAB, "COPTIC SMALL LETTER FI" },
    { 0x2CAC, "COPTIC CAPITAL LETTER KHI" },
    { 0x2CAD, "COPTIC SMALL LETTER KHI" },
    { 0x2CAE, "COPTIC CAPITAL LETTER PSI" },
    { 0x2CAF, "COPTIC SMALL LETTER PSI" },
    { 0x2CB0, "COPTIC CAPITAL LETTER OOU" },
    { 0x2CB1, "COPTIC SMALL LETTER OOU" },
    { 0x2CB2, "COPTIC CAPITAL LETTER DIALECT-P ALEF" },
    { 0x2CB3, "COPTIC SMALL LETTER DIALECT-P ALEF" },
    { 0x2CB4, "COPTIC CAPITAL LETTER OLD COPTIC AIN" },
    { 0x2CB5, "COPTIC SMALL LETTER OLD COPTIC AIN" },
    { 0x2CB6, "COPTIC CAPITAL LETTER CRYPTOGRAMMIC EIE" },
    { 0x2CB7, "COPTIC SMALL LETTER CRYPTOGRAMMIC EIE" },
    { 0x2CB8, "COPTIC CAPITAL LETTER DIALECT-P KAPA" },
    { 0x2CB9, "COPTIC SMALL LETTER DIALECT-P KAPA" },
    { 0x2CBA, "COPTIC CAPITAL LETTER DIALECT-P NI" },
    { 0x2CBB, "COPTIC SMALL LETTER DIALECT-P NI" },
    { 0x2CBC, "COPTIC CAPITAL LETTER CRYPTOGRAMMIC NI" },
    { 0x2CBD, "COPTIC SMALL LETTER CRYPTOGRAMMIC NI" },
    { 0x2CBE, "COPTIC CAPITAL LETTER OLD COPTIC OOU" },
    { 0x2CBF, "COPTIC SMALL LETTER OLD COPTIC OOU" },
    { 0x2CC0, "COPTIC CAPITAL LETTER SAMPI" },
    { 0x2CC1, "COPTIC SMALL LETTER SAMPI" },
    { 0x2CC2, "COPTIC CAPITAL LETTER CROSSED SHEI" },
    { 0x2CC3, "COPTIC SMALL LETTER CROSSED SHEI" },
    { 0x2CC4, "COPTIC CAPITAL LETTER OLD COPTIC SHEI" },
    { 0x2CC5, "COPTIC SMALL LETTER OLD COPTIC SHEI" },
    { 0x2CC6, "COPTIC CAPITAL LETTER OLD COPTIC ESH" },
    { 0x2CC7, "COPTIC SMALL LETTER OLD COPTIC ESH" },
    { 0x2CC8, "COPTIC CAPITAL LETTER AKHMIMIC KHEI" },
    { 0x2CC9, "COPTIC SMALL LETTER AKHMIMIC KHEI" },
    { 0x2CCA, "COPTIC CAPITAL LETTER DIALECT-P HORI" },
    { 0x2CCB, "COPTIC SMALL LETTER DIALECT-P HORI" },
    { 0x2CCC, "COPTIC CAPITAL LETTER OLD COPTIC HORI" },
    { 0x2CCD, "COPTIC SMALL LETTER OLD COPTIC HORI" },
    { 0x2CCE, "COPTIC CAPITAL LETTER OLD COPTIC HA" },
    { 0x2CCF, "COPTIC SMALL LETTER OLD COPTIC HA" },
    { 0x2CD0, "COPTIC CAPITAL LETTER L-SHAPED HA" },
    { 0x2CD1, "COPTIC SMALL LETTER L-SHAPED HA" },
    { 0x2CD2, "COPTIC CAPITAL LETTER OLD COPTIC HEI" },
    { 0x2CD3, "COPTIC SMALL LETTER OLD COPTIC HEI" },
    { 0x2CD4, "COPTIC CAPITAL LETTER OLD COPTIC HAT" },
    { 0x2CD5, "COPTIC SMALL LETTER OLD COPTIC HAT" },
    { 0x2CD6, "COPTIC CAPITAL LETTER OLD COPTIC GANGIA" },
    { 0x2CD7, "COPTIC SMALL LETTER OLD COPTIC GANGIA" },
    { 0x2CD8, "COPTIC CAPITAL LETTER OLD COPTIC DJA" },
    { 0x2CD9, "COPTIC SMALL LETTER OLD COPTIC DJA" },
    { 0x2CDA, "COPTIC CAPITAL LETTER OLD COPTIC SHIMA" },
    { 0x2CDB, "COPTIC SMALL LETTER OLD COPTIC SHIMA" },
    { 0x2CDC, "COPTIC CAPITAL LETTER OLD NUBIAN SHIMA" },
    { 0x2CDD, "COPTIC SMALL LETTER OLD NUBIAN SHIMA" },
    { 0x2CDE, "COPTIC CAPITAL LETTER OLD NUBIAN NGI" },
    { 0x2CDF, "COPTIC SMALL LETTER OLD NUBIAN NGI" },
    { 0x2CE0, "COPTIC CAPITAL LETTER OLD NUBIAN NYI" },
    { 0x2CE1, "COPTIC SMALL LETTER OLD NUBIAN NYI" },
    { 0x2CE2, "COPTIC CAPITAL LETTER OLD NUBIAN WAU" },
    { 0x2CE3, "COPTIC SMALL LETTER OLD NUBIAN WAU" },
    { 0x2CE4, "COPTIC SYMBOL KAI" },
    { 0x2CE5, "COPTIC SYMBOL MI RO" },
    { 0x2CE6, "COPTIC SYMBOL PI RO" },
    { 0x2CE7, "COPTIC SYMBOL STAUROS" },
    { 0x2CE8, "COPTIC SYMBOL TAU RO" },
    { 0x2CE9, "COPTIC SYMBOL KHI RO" },
    { 0x2CEA, "COPTIC SYMBOL SHIMA SIMA" },
    { 0x2CEB, "COPTIC CAPITAL LETTER CRYPTOGRAMMIC SHEI" },
    { 0x2CEC, "COPTIC SMALL LETTER CRYPTOGRAMMIC SHEI" },
    { 0x2CED, "COPTIC CAPITAL LETTER CRYPTOGRAMMIC GANGIA" },
    { 0x2CEE, "COPTIC SMALL LETTER CRYPTOGRAMMIC GANGIA" },
    { 0x2CEF, "COPTIC COMBINING NI ABOVE" },
    { 0x2CF0, "COPTIC COMBINING SPIRITUS ASPER" },
    { 0x2CF1, "COPTIC COMBINING SPIRITUS LENIS" },
    { 0x2CF2, "COPTIC CAPITAL LETTER BOHAIRIC KHEI" },
    { 0x2CF3, "COPTIC SMALL LETTER BOHAIRIC KHEI" },
    { 0x2CF9, "COPTIC OLD NUBIAN FULL STOP" },
    { 0x2CFA, "COPTIC OLD NUBIAN DIRECT QUESTION MARK" },
    { 0x2CFB, "COPTIC OLD NUBIAN INDIRECT QUESTION MARK" },
    { 0x2CFC, "COPTIC OLD NUBIAN VERSE DIVIDER" },
    { 0x2CFD, "COPTIC FRACTION ONE HALF" },
    { 0x2CFE, "COPTIC FULL STOP" },
    { 0x2CFF, "COPTIC MORPHOLOGICAL DIVIDER" },
    { 0x2D00, "GEORGIAN SMALL LETTER AN" },
    { 0x2D01, "GEORGIAN SMALL LETTER BAN" },
    { 0x2D02, "GEORGIAN SMALL LETTER GAN" },
    { 0x2D03, "GEORGIAN SMALL LETTER DON" },
    { 0x2D04, "GEORGIAN SMALL LETTER EN" },
    { 0x2D05, "GEORGIAN SMALL LETTER VIN" },
    { 0x2D06, "GEORGIAN SMALL LETTER ZEN" },
    { 0x2D07, "GEORGIAN SMALL LETTER TAN" },
    { 0x2D08, "GEORGIAN SMALL LETTER IN" },
    { 0x2D09, "GEORGIAN SMALL LETTER KAN" },
    { 0x2D0A, "GEORGIAN SMALL LETTER LAS" },
    { 0x2D0B, "GEORGIAN SMALL LETTER MAN" },
    { 0x2D0C, "GEORGIAN SMALL LETTER NAR" },
    { 0x2D0D, "GEORGIAN SMALL LETTER ON" },
    { 0x2D0E, "GEORGIAN SMALL LETTER PAR" },
    { 0x2D0F, "GEORGIAN SMALL LETTER ZHAR" },
    { 0x2D10, "GEORGIAN SMALL LETTER RAE" },
    { 0x2D11, "GEORGIAN SMALL LETTER SAN" },
    { 0x2D12, "GEORGIAN SMALL LETTER TAR" },
    { 0x2D13, "GEORGIAN SMALL LETTER UN" },
    { 0x2D14, "GEORGIAN SMALL LETTER PHAR" },
    { 0x2D15, "GEORGIAN SMALL LETTER KHAR" },
    { 0x2D16, "GEORGIAN SMALL LETTER GHAN" },
    { 0x2D17, "GEORGIAN SMALL LETTER QAR" },
    { 0x2D18, "GEORGIAN SMALL LETTER SHIN" },
    { 0x2D19, "GEORGIAN SMALL LETTER CHIN" },
    { 0x2D1A, "GEORGIAN SMALL LETTER CAN" },
    { 0x2D1B, "GEORGIAN SMALL LETTER JIL" },
    { 0x2D1C, "GEORGIAN SMALL LETTER CIL" },
    { 0x2D1D, "GEORGIAN SMALL LETTER CHAR" },
    { 0x2D1E, "GEORGIAN SMALL LETTER XAN" },
    { 0x2D1F, "GEORGIAN SMALL LETTER JHAN" },
    { 0x2D20, "GEORGIAN SMALL LETTER HAE" },
    { 0x2D21, "GEORGIAN SMALL LETTER HE" },
    { 0x2D22, "GEORGIAN SMALL LETTER HIE" },
    { 0x2D23, "GEORGIAN SMALL LETTER WE" },
    { 0x2D24, "GEORGIAN SMALL LETTER HAR" },
    { 0x2D25, "GEORGIAN SMALL LETTER HOE" },
    { 0x2D27, "GEORGIAN SMALL LETTER YN" },
    { 0x2D2D, "GEORGIAN SMALL LETTER AEN" },
    { 0x2D30, "TIFINAGH LETTER YA" },
    { 0x2D31, "TIFINAGH LETTER YAB" },
    { 0x2D32, "TIFINAGH LETTER YABH" },
    { 0x2D33, "TIFINAGH LETTER YAG" },
    { 0x2D34, "TIFINAGH LETTER YAGHH" },
    { 0x2D35, "TIFINAGH LETTER BERBER ACADEMY YAJ" },
    { 0x2D36, "TIFINAGH LETTER YAJ" },
    { 0x2D37, "TIFINAGH LETTER YAD" },
    { 0x2D38, "TIFINAGH LETTER YADH" },
    { 0x2D39, "TIFINAGH LETTER YADD" },
    { 0x2D3A, "TIFINAGH LETTER YADDH" },
    { 0x2D3B, "TIFINAGH LETTER YEY" },
    { 0x2D3C, "TIFINAGH LETTER YAF" },
    { 0x2D3D, "TIFINAGH LETTER YAK" },
    { 0x2D3E, "TIFINAGH LETTER TUAREG YAK" },
    { 0x2D3F, "TIFINAGH LETTER YAKHH" },
    { 0x2D40, "TIFINAGH LETTER YAH" },
    { 0x2D41, "TIFINAGH LETTER BERBER ACADEMY YAH" },
    { 0x2D42, "TIFINAGH LETTER TUAREG YAH" },
    { 0x2D43, "TIFINAGH LETTER YAHH" },
    { 0x2D44, "TIFINAGH LETTER YAA" },
    { 0x2D45, "TIFINAGH LETTER YAKH" },
    { 0x2D46, "TIFINAGH LETTER TUAREG YAKH" },
    { 0x2D47, "TIFINAGH LETTER YAQ" },
    { 0x2D48, "TIFINAGH LETTER TUAREG YAQ" },
    { 0x2D49, "TIFINAGH LETTER YI" },
    { 0x2D4A, "TIFINAGH LETTER YAZH" },
    { 0x2D4B, "TIFINAGH LETTER AHAGGAR YAZH" },
    { 0x2D4C, "TIFINAGH LETTER TUAREG YAZH" },
    { 0x2D4D, "TIFINAGH LETTER YAL" },
    { 0x2D4E, "TIFINAGH LETTER YAM" },
    { 0x2D4F, "TIFINAGH LETTER YAN" },
    { 0x2D50, "TIFINAGH LETTER TUAREG YAGN" },
    { 0x2D51, "TIFINAGH LETTER TUAREG YANG" },
    { 0x2D52, "TIFINAGH LETTER YAP" },
    { 0x2D53, "TIFINAGH LETTER YU" },
    { 0x2D54, "TIFINAGH LETTER YAR" },
    { 0x2D55, "TIFINAGH LETTER YARR" },
    { 0x2D56, "TIFINAGH LETTER YAGH" },
    { 0x2D57, "TIFINAGH LETTER TUAREG YAGH" },
    { 0x2D58, "TIFINAGH LETTER AYER YAGH" },
    { 0x2D59, "TIFINAGH LETTER YAS" },
    { 0x2D5A, "TIFINAGH LETTER YASS" },
    { 0x2D5B, "TIFINAGH LETTER YASH" },
    { 0x2D5C, "TIFINAGH LETTER YAT" },
    { 0x2D5D, "TIFINAGH LETTER YATH" },
    { 0x2D5E, "TIFINAGH LETTER YACH" },
    { 0x2D5F, "TIFINAGH LETTER YATT" },
    { 0x2D60, "TIFINAGH LETTER YAV" },
    { 0x2D61, "TIFINAGH LETTER YAW" },
    { 0x2D62, "TIFINAGH LETTER YAY" },
    { 0x2D63, "TIFINAGH LETTER YAZ" },
    { 0x2D64, "TIFINAGH LETTER TAWELLEMET YAZ" },
    { 0x2D65, "TIFINAGH LETTER YAZZ" },
    { 0x2D66, "TIFINAGH LETTER YE" },
    { 0x2D67, "TIFINAGH LETTER YO" },
    { 0x2D6F, "TIFINAGH MODIFIER LETTER LABIALIZATION MARK" },
    { 0x2D70, "TIFINAGH SEPARATOR MARK" },
    { 0x2D7F, "TIFINAGH CONSONANT JOINER" },
    { 0x2D80, "ETHIOPIC SYLLABLE LOA" },
    { 0x2D81, "ETHIOPIC SYLLABLE MOA" },
    { 0x2D82, "ETHIOPIC SYLLABLE ROA" },
    { 0x2D83, "ETHIOPIC SYLLABLE SOA" },
    { 0x2D84, "ETHIOPIC SYLLABLE SHOA" },
    { 0x2D85, "ETHIOPIC SYLLABLE BOA" },
    { 0x2D86, "ETHIOPIC SYLLABLE TOA" },
    { 0x2D87, "ETHIOPIC SYLLABLE COA" },
    { 0x2D88, "ETHIOPIC SYLLABLE NOA" },
    { 0x2D89, "ETHIOPIC SYLLABLE NYOA" },
    { 0x2D8A, "ETHIOPIC SYLLABLE GLOTTAL OA" },
    { 0x2D8B, "ETHIOPIC SYLLABLE ZOA" },
    { 0x2D8C, "ETHIOPIC SYLLABLE DOA" },
    { 0x2D8D, "ETHIOPIC SYLLABLE DDOA" },
    { 0x2D8E, "ETHIOPIC SYLLABLE JOA" },
    { 0x2D8F, "ETHIOPIC SYLLABLE THOA" },
    { 0x2D90, "ETHIOPIC SYLLABLE CHOA" },
    { 0x2D91, "ETHIOPIC SYLLABLE PHOA" },
    { 0x2D92, "ETHIOPIC SYLLABLE POA" },
    { 0x2D93, "ETHIOPIC SYLLABLE GGWA" },
    { 0x2D94, "ETHIOPIC SYLLABLE GGWI" },
    { 0x2D95, "ETHIOPIC SYLLABLE GGWEE" },
    { 0x2D96, "ETHIOPIC SYLLABLE GGWE" },
    { 0x2DA0, "ETHIOPIC SYLLABLE SSA" },
    { 0x2DA1, "ETHIOPIC SYLLABLE SSU" },
    { 0x2DA2, "ETHIOPIC SYLLABLE SSI" },
    { 0x2DA3, "ETHIOPIC SYLLABLE SSAA" },
    { 0x2DA4, "ETHIOPIC SYLLABLE SSEE" },
    { 0x2DA5, "ETHIOPIC SYLLABLE SSE" },
    { 0x2DA6, "ETHIOPIC SYLLABLE SSO" },
    { 0x2DA8, "ETHIOPIC SYLLABLE CCA" },
    { 0x2DA9, "ETHIOPIC SYLLABLE CCU" },
    { 0x2DAA, "ETHIOPIC SYLLABLE CCI" },
    { 0x2DAB, "ETHIOPIC SYLLABLE CCAA" },
    { 0x2DAC, "ETHIOPIC SYLLABLE CCEE" },
    { 0x2DAD, "ETHIOPIC SYLLABLE CCE" },
    { 0x2DAE, "ETHIOPIC SYLLABLE CCO" },
    { 0x2DB0, "ETHIOPIC SYLLABLE ZZA" },
    { 0x2DB1, "ETHIOPIC SYLLABLE ZZU" },
    { 0x2DB2, "ETHIOPIC SYLLABLE ZZI" },
    { 0x2DB3, "ETHIOPIC SYLLABLE ZZAA" },
    { 0x2DB4, "ETHIOPIC SYLLABLE ZZEE" },
    { 0x2DB5, "ETHIOPIC SYLLABLE ZZE" },
    { 0x2DB6, "ETHIOPIC SYLLABLE ZZO" },
    { 0x2DB8, "ETHIOPIC SYLLABLE CCHA" },
    { 0x2DB9, "ETHIOPIC SYLLABLE CCHU" },
    { 0x2DBA, "ETHIOPIC SYLLABLE CCHI" },
    { 0x2DBB, "ETHIOPIC SYLLABLE CCHAA" },
    { 0x2DBC, "ETHIOPIC SYLLABLE CCHEE" },
    { 0x2DBD, "ETHIOPIC SYLLABLE CCHE" },
    { 0x2DBE, "ETHIOPIC SYLLABLE CCHO" },
    { 0x2DC0, "ETHIOPIC SYLLABLE QYA" },
    { 0x2DC1, "ETHIOPIC SYLLABLE QYU" },
    { 0x2DC2, "ETHIOPIC SYLLABLE QYI" },
    { 0x2DC3, "ETHIOPIC SYLLABLE QYAA" },
    { 0x2DC4, "ETHIOPIC SYLLABLE QYEE" },
    { 0x2DC5, "ETHIOPIC SYLLABLE QYE" },
    { 0x2DC6, "ETHIOPIC SYLLABLE QYO" },
    { 0x2DC8, "ETHIOPIC SYLLABLE KYA" },
    { 0x2DC9, "ETHIOPIC SYLLABLE KYU" },
    { 0x2DCA, "ETHIOPIC SYLLABLE KYI" },
    { 0x2DCB, "ETHIOPIC SYLLABLE KYAA" },
    { 0x2DCC, "ETHIOPIC SYLLABLE KYEE" },
    { 0x2DCD, "ETHIOPIC SYLLABLE KYE" },
    { 0x2DCE, "ETHIOPIC SYLLABLE KYO" },
    { 0x2DD0, "ETHIOPIC SYLLABLE XYA" },
    { 0x2DD1, "ETHIOPIC SYLLABLE XYU" },
    { 0x2DD2, "ETHIOPIC SYLLABLE XYI" },
    { 0x2DD3, "ETHIOPIC SYLLABLE XYAA" },
    { 0x2DD4, "ETHIOPIC SYLLABLE XYEE" },
    { 0x2DD5, "ETHIOPIC SYLLABLE XYE" },
    { 0x2DD6, "ETHIOPIC SYLLABLE XYO" },
    { 0x2DD8, "ETHIOPIC SYLLABLE GYA" },
    { 0x2DD9, "ETHIOPIC SYLLABLE GYU" },
    { 0x2DDA, "ETHIOPIC SYLLABLE GYI" },
    { 0x2DDB, "ETHIOPIC SYLLABLE GYAA" },
    { 0x2DDC, "ETHIOPIC SYLLABLE GYEE" },
    { 0x2DDD, "ETHIOPIC SYLLABLE GYE" },
    { 0x2DDE, "ETHIOPIC SYLLABLE GYO" },
    { 0x2DE0, "COMBINING CYRILLIC LETTER BE" },
    { 0x2DE1, "COMBINING CYRILLIC LETTER VE" },
    { 0x2DE2, "COMBINING CYRILLIC LETTER GHE" },
    { 0x2DE3, "COMBINING CYRILLIC LETTER DE" },
    { 0x2DE4, "COMBINING CYRILLIC LETTER ZHE" },
    { 0x2DE5, "COMBINING CYRILLIC LETTER ZE" },
    { 0x2DE6, "COMBINING CYRILLIC LETTER KA" },
    { 0x2DE7, "COMBINING CYRILLIC LETTER EL" },
    { 0x2DE8, "COMBINING CYRILLIC LETTER EM" },
    { 0x2DE9, "COMBINING CYRILLIC LETTER EN" },
    { 0x2DEA, "COMBINING CYRILLIC LETTER O" },
    { 0x2DEB, "COMBINING CYRILLIC LETTER PE" },
    { 0x2DEC, "COMBINING CYRILLIC LETTER ER" },
    { 0x2DED, "COMBINING CYRILLIC LETTER ES" },
    { 0x2DEE, "COMBINING CYRILLIC LETTER TE" },
    { 0x2DEF, "COMBINING CYRILLIC LETTER HA" },
    { 0x2DF0, "COMBINING CYRILLIC LETTER TSE" },
    { 0x2DF1, "COMBINING CYRILLIC LETTER CHE" },
    { 0x2DF2, "COMBINING CYRILLIC LETTER SHA" },
    { 0x2DF3, "COMBINING CYRILLIC LETTER SHCHA" },
    { 0x2DF4, "COMBINING CYRILLIC LETTER FITA" },
    { 0x2DF5, "COMBINING CYRILLIC LETTER ES-TE" },
    { 0x2DF6, "COMBINING CYRILLIC LETTER A" },
    { 0x2DF7, "COMBINING CYRILLIC LETTER IE" },
    { 0x2DF8, "COMBINING CYRILLIC LETTER DJERV" },
    { 0x2DF9, "COMBINING CYRILLIC LETTER MONOGRAPH UK" },
    { 0x2DFA, "COMBINING CYRILLIC LETTER YAT" },
    { 0x2DFB, "COMBINING CYRILLIC LETTER YU" },
    { 0x2DFC, "COMBINING CYRILLIC LETTER IOTIFIED A" },
    { 0x2DFD, "COMBINING CYRILLIC LETTER LITTLE YUS" },
    { 0x2DFE, "COMBINING CYRILLIC LETTER BIG YUS" },
    { 0x2DFF, "COMBINING CYRILLIC LETTER IOTIFIED BIG YUS" },
    { 0x2E00, "RIGHT ANGLE SUBSTITUTION MARKER" },
    { 0x2E01, "RIGHT ANGLE DOTTED SUBSTITUTION MARKER" },
    { 0x2E02, "LEFT SUBSTITUTION BRACKET" },
    { 0x2E03, "RIGHT SUBSTITUTION BRACKET" },
    { 0x2E04, "LEFT DOTTED SUBSTITUTION BRACKET" },
    { 0x2E05, "RIGHT DOTTED SUBSTITUTION BRACKET" },
    { 0x2E06, "RAISED INTERPOLATION MARKER" },
    { 0x2E07, "RAISED DOTTED INTERPOLATION MARKER" },
    { 0x2E08, "DOTTED TRANSPOSITION MARKER" },
    { 0x2E09, "LEFT TRANSPOSITION BRACKET" },
    { 0x2E0A, "RIGHT TRANSPOSITION BRACKET" },
    { 0x2E0B, "RAISED SQUARE" },
    { 0x2E0C, "LEFT RAISED OMISSION BRACKET" },
    { 0x2E0D, "RIGHT RAISED OMISSION BRACKET" },
    { 0x2E0E, "EDITORIAL CORONIS" },
    { 0x2E0F, "PARAGRAPHOS" },
    { 0x2E10, "FORKED PARAGRAPHOS" },
    { 0x2E11, "REVERSED FORKED PARAGRAPHOS" },
    { 0x2E12, "HYPODIASTOLE" },
    { 0x2E13, "DOTTED OBELOS" },
    { 0x2E14, "DOWNWARDS ANCORA" },
    { 0x2E15, "UPWARDS ANCORA" },
    { 0x2E16, "DOTTED RIGHT-POINTING ANGLE" },
    { 0x2E17, "DOUBLE OBLIQUE HYPHEN" },
    { 0x2E18, "INVERTED INTERROBANG" },
    { 0x2E19, "PALM BRANCH" },
    { 0x2E1A, "HYPHEN WITH DIAERESIS" },
    { 0x2E1B, "TILDE WITH RING ABOVE" },
    { 0x2E1C, "LEFT LOW PARAPHRASE BRACKET" },
    { 0x2E1D, "RIGHT LOW PARAPHRASE BRACKET" },
    { 0x2E1E, "TILDE WITH DOT ABOVE" },
    { 0x2E1F, "TILDE WITH DOT BELOW" },
    { 0x2E20, "LEFT VERTICAL BAR WITH QUILL" },
    { 0x2E21, "RIGHT VERTICAL BAR WITH QUILL" },
    { 0x2E22, "TOP LEFT HALF BRACKET" },
    { 0x2E23, "TOP RIGHT HALF BRACKET" },
    { 0x2E24, "BOTTOM LEFT HALF BRACKET" },
    { 0x2E25, "BOTTOM RIGHT HALF BRACKET" },
    { 0x2E26, "LEFT SIDEWAYS U BRACKET" },
    { 0x2E27, "RIGHT SIDEWAYS U BRACKET" },
    { 0x2E28, "LEFT DOUBLE PARENTHESIS" },
    { 0x2E29, "RIGHT DOUBLE PARENTHESIS" },
    { 0x2E2A, "TWO DOTS OVER ONE DOT PUNCTUATION" },
    { 0x2E2B, "ONE DOT OVER TWO DOTS PUNCTUATION" },
    { 0x2E2C, "SQUARED FOUR DOT PUNCTUATION" },
    { 0x2E2D, "FIVE DOT MARK" },
    { 0x2E2E, "REVERSED QUESTION MARK" },
    { 0x2E2F, "VERTICAL TILDE" },
    { 0x2E30, "RING POINT" },
    { 0x2E31, "WORD SEPARATOR MIDDLE DOT" },
    { 0x2E32, "TURNED COMMA" },
    { 0x2E33, "RAISED DOT" },
    { 0x2E34, "RAISED COMMA" },
    { 0x2E35, "TURNED SEMICOLON" },
    { 0x2E36, "DAGGER WITH LEFT GUARD" },
    { 0x2E37, "DAGGER WITH RIGHT GUARD" },
    { 0x2E38, "TURNED DAGGER" },
    { 0x2E39, "TOP HALF SECTION SIGN" },
    { 0x2E3A, "TWO-EM DASH" },
    { 0x2E3B, "THREE-EM DASH" },
    { 0x2E3C, "STENOGRAPHIC FULL STOP" },
    { 0x2E3D, "VERTICAL SIX DOTS" },
    { 0x2E3E, "WIGGLY VERTICAL LINE" },
    { 0x2E3F, "CAPITULUM" },
    { 0x2E40, "DOUBLE HYPHEN" },
    { 0x2E41, "REVERSED COMMA" },
    { 0x2E42, "DOUBLE LOW-REVERSED-9 QUOTATION MARK" },
    { 0x2E43, "DASH WITH LEFT UPTURN" },
    { 0x2E44, "DOUBLE SUSPENSION MARK" },
    { 0x2E45, "INVERTED LOW KAVYKA" },
    { 0x2E46, "INVERTED LOW KAVYKA WITH KAVYKA ABOVE" },
    { 0x2E47, "LOW KAVYKA" },
    { 0x2E48, "LOW KAVYKA WITH DOT" },
    { 0x2E49, "DOUBLE STACKED COMMA" },
    { 0x2E4A, "DOTTED SOLIDUS" },
    { 0x2E4B, "TRIPLE DAGGER" },
    { 0x2E4C, "MEDIEVAL COMMA" },
    { 0x2E4D, "PARAGRAPHUS MARK" },
    { 0x2E4E, "PUNCTUS ELEVATUS MARK" },
    { 0x2E4F, "CORNISH VERSE DIVIDER" },
    { 0x2E50, "CROSS PATTY WITH RIGHT CROSSBAR" },
    { 0x2E51, "CROSS PATTY WITH LEFT CROSSBAR" },
    { 0x2E52, "TIRONIAN SIGN CAPITAL ET" },
    { 0x2E53, "MEDIEVAL EXCLAMATION MARK" },
    { 0x2E54, "MEDIEVAL QUESTION MARK" },
    { 0x2E55, "LEFT SQUARE BRACKET WITH STROKE" },
    { 0x2E56, "RIGHT SQUARE BRACKET WITH STROKE" },
    { 0x2E57, "LEFT SQUARE BRACKET WITH DOUBLE STROKE" },
    { 0x2E58, "RIGHT SQUARE BRACKET WITH DOUBLE STROKE" },
    { 0x2E59, "TOP HALF LEFT PARENTHESIS" },
    { 0x2E5A, "TOP HALF RIGHT PARENTHESIS" },
    { 0x2E5B, "BOTTOM HALF LEFT PARENTHESIS" },
    { 0x2E5C, "BOTTOM HALF RIGHT PARENTHESIS" },
    { 0x2E5D, "OBLIQUE HYPHEN" },
    { 0x2E80, "CJK RADICAL REPEAT" },
    { 0x2E81, "CJK RADICAL CLIFF" },
    { 0x2E82, "CJK RADICAL SECOND ONE" },
    { 0x2E83, "CJK RADICAL SECOND TWO" },
    { 0x2E84, "CJK RADICAL SECOND THREE" },
    { 0x2E85, "CJK RADICAL PERSON" },
    { 0x2E86, "CJK RADICAL BOX" },
    { 0x2E87, "CJK RADICAL TABLE" },
    { 0x2E88, "CJK RADICAL KNIFE ONE" },
    { 0x2E89, "CJK RADICAL KNIFE TWO" },
    { 0x2E8A, "CJK RADICAL DIVINATION" },
    { 0x2E8B, "CJK RADICAL SEAL" },
    { 0x2E8C, "CJK RADICAL SMALL ONE" },
    { 0x2E8D, "CJK RADICAL SMALL TWO" },
    { 0x2E8E, "CJK RADICAL LAME ONE" },
    { 0x2E8F, "CJK RADICAL LAME TWO" },
    { 0x2E90, "CJK RADICAL LAME THREE" },
    { 0x2E91, "CJK RADICAL LAME FOUR" },
    { 0x2E92, "CJK RADICAL SNAKE" },
    { 0x2E93, "CJK RADICAL THREAD" },
    { 0x2E94, "CJK RADICAL SNOUT ONE" },
    { 0x2E95, "CJK RADICAL SNOUT TWO" },
    { 0x2E96, "CJK RADICAL HEART ONE" },
    { 0x2E97, "CJK RADICAL HEART TWO" },
    { 0x2E98, "CJK RADICAL HAND" },
    { 0x2E99, "CJK RADICAL RAP" },
    { 0x2E9B, "CJK RADICAL CHOKE" },
    { 0x2E9C, "CJK RADICAL SUN" },
    { 0x2E9D, "CJK RADICAL MOON" },
    { 0x2E9E, "CJK RADICAL DEATH" },
    { 0x2E9F, "CJK RADICAL MOTHER" },
    { 0x2EA0, "CJK RADICAL CIVILIAN" },
    { 0x2EA1, "CJK RADICAL WATER ONE" },
    { 0x2EA2, "CJK RADICAL WATER TWO" },
    { 0x2EA3, "CJK RADICAL FIRE" },
    { 0x2EA4, "CJK RADICAL PAW ONE" },
    { 0x2EA5, "CJK RADICAL PAW TWO" },
    { 0x2EA6, "CJK RADICAL SIMPLIFIED HALF TREE TRUNK" },
    { 0x2EA7, "CJK RADICAL COW" },
    { 0x2EA8, "CJK RADICAL DOG" },
    { 0x2EA9, "CJK RADICAL JADE" },
    { 0x2EAA, "CJK RADICAL BOLT OF CLOTH" },
    { 0x2EAB, "CJK RADICAL EYE" },
    { 0x2EAC, "CJK RADICAL SPIRIT ONE" },
    { 0x2EAD, "CJK RADICAL SPIRIT TWO" },
    { 0x2EAE, "CJK RADICAL BAMBOO" },
    { 0x2EAF, "CJK RADICAL SILK" },
    { 0x2EB0, "CJK RADICAL C-SIMPLIFIED SILK" },
    { 0x2EB1, "CJK RADICAL NET ONE" },
    { 0x2EB2, "CJK RADICAL NET TWO" },
    { 0x2EB3, "CJK RADICAL NET THREE" },
    { 0x2EB4, "CJK RADICAL NET FOUR" },
    { 0x2EB5, "CJK RADICAL MESH" },
    { 0x2EB6, "CJK RADICAL SHEEP" },
    { 0x2EB7, "CJK RADICAL RAM" },
    { 0x2EB8, "CJK RADICAL EWE" },
    { 0x2EB9, "CJK RADICAL OLD" },
    { 0x2EBA, "CJK RADICAL BRUSH ONE" },
    { 0x2EBB, "CJK RADICAL BRUSH TWO" },
    { 0x2EBC, "CJK RADICAL MEAT" },
    { 0x2EBD, "CJK RADICAL MORTAR" },
    { 0x2EBE, "CJK RADICAL GRASS ONE" },
    { 0x2EBF, "CJK RADICAL GRASS TWO" },
    { 0x2EC0, "CJK RADICAL GRASS THREE" },
    { 0x2EC1, "CJK RADICAL TIGER" },
    { 0x2EC2, "CJK RADICAL CLOTHES" },
    { 0x2EC3, "CJK RADICAL WEST ONE" },
    { 0x2EC4, "CJK RADICAL WEST TWO" },
    { 0x2EC5, "CJK RADICAL C-SIMPLIFIED SEE" },
    { 0x2EC6, "CJK RADICAL SIMPLIFIED HORN" },
    { 0x2EC7, "CJK RADICAL HORN" },
    { 0x2EC8, "CJK RADICAL C-SIMPLIFIED SPEECH" },
    { 0x2EC9, "CJK RADICAL C-SIMPLIFIED SHELL" },
    { 0x2ECA, "CJK RADICAL FOOT" },
    { 0x2ECB, "CJK RADICAL C-SIMPLIFIED CART" },
    { 0x2ECC, "CJK RADICAL SIMPLIFIED WALK" },
    { 0x2ECD, "CJK RADICAL WALK ONE" },
    { 0x2ECE, "CJK RADICAL WALK TWO" },
    { 0x2ECF, "CJK RADICAL CITY" },
    { 0x2ED0, "CJK RADICAL C-SIMPLIFIED GOLD" },
    { 0x2ED1, "CJK RADICAL LONG ONE" },
    { 0x2ED2, "CJK RADICAL LONG TWO" },
    { 0x2ED3, "CJK RADICAL C-SIMPLIFIED LONG" },
    { 0x2ED4, "CJK RADICAL C-SIMPLIFIED GATE" },
    { 0x2ED5, "CJK RADICAL MOUND ONE" },
    { 0x2ED6, "CJK RADICAL MOUND TWO" },
    { 0x2ED7, "CJK RADICAL RAIN" },
    { 0x2ED8, "CJK RADICAL BLUE" },
    { 0x2ED9, "CJK RADICAL C-SIMPLIFIED TANNED LEATHER" },
    { 0x2EDA, "CJK RADICAL C-SIMPLIFIED LEAF" },
    { 0x2EDB, "CJK RADICAL C-SIMPLIFIED WIND" },
    { 0x2EDC, "CJK RADICAL C-SIMPLIFIED FLY" },
    { 0x2EDD, "CJK RADICAL EAT ONE" },
    { 0x2EDE, "CJK RADICAL EAT TWO" },
    { 0x2EDF, "CJK RADICAL EAT THREE" },
    { 0x2EE0, "CJK RADICAL C-SIMPLIFIED EAT" },
    { 0x2EE1, "CJK RADICAL HEAD" },
    { 0x2EE2, "CJK RADICAL C-SIMPLIFIED HORSE" },
    { 0x2EE3, "CJK RADICAL BONE" },
    { 0x2EE4, "CJK RADICAL GHOST" },
    { 0x2EE5, "CJK RADICAL C-SIMPLIFIED FISH" },
    { 0x2EE6, "CJK RADICAL C-SIMPLIFIED BIRD" },
    { 0x2EE7, "CJK RADICAL C-SIMPLIFIED SALT" },
    { 0x2EE8, "CJK RADICAL SIMPLIFIED WHEAT" },
    { 0x2EE9, "CJK RADICAL SIMPLIFIED YELLOW" },
    { 0x2EEA, "CJK RADICAL C-SIMPLIFIED FROG" },
    { 0x2EEB, "CJK RADICAL J-SIMPLIFIED EVEN" },
    { 0x2EEC, "CJK RADICAL C-SIMPLIFIED EVEN" },
    { 0x2EED, "CJK RADICAL J-SIMPLIFIED TOOTH" },
    { 0x2EEE, "CJK RADICAL C-SIMPLIFIED TOOTH" },
    { 0x2EEF, "CJK RADICAL J-SIMPLIFIED DRAGON" },
    { 0x2EF0, "CJK RADICAL C-SIMPLIFIED DRAGON" },
    { 0x2EF1, "CJK RADICAL TURTLE" },
    { 0x2EF2, "CJK RADICAL J-SIMPLIFIED TURTLE" },
    { 0x2EF3, "CJK RADICAL C-SIMPLIFIED TURTLE" },
    { 0x2F00, "KANGXI RADICAL ONE" },
    { 0x2F01, "KANGXI RADICAL LINE" },
    { 0x2F02, "KANGXI RADICAL DOT" },
    { 0x2F03, "KANGXI RADICAL SLASH" },
    { 0x2F04, "KANGXI RADICAL SECOND" },
    { 0x2F05, "KANGXI RADICAL HOOK" },
    { 0x2F06, "KANGXI RADICAL TWO" },
    { 0x2F07, "KANGXI RADICAL LID" },
    { 0x2F08, "KANGXI RADICAL MAN" },
    { 0x2F09, "KANGXI RADICAL LEGS" },
    { 0x2F0A, "KANGXI RADICAL ENTER" },
    { 0x2F0B, "KANGXI RADICAL EIGHT" },
    { 0x2F0C, "KANGXI RADICAL DOWN BOX" },
    { 0x2F0D, "KANGXI RADICAL COVER" },
    { 0x2F0E, "KANGXI RADICAL ICE" },
    { 0x2F0F, "KANGXI RADICAL TABLE" },
    { 0x2F10, "KANGXI RADICAL OPEN BOX" },
    { 0x2F11, "KANGXI RADICAL KNIFE" },
    { 0x2F12, "KANGXI RADICAL POWER" },
    { 0x2F13, "KANGXI RADICAL WRAP" },
    { 0x2F14, "KANGXI RADICAL SPOON" },
    { 0x2F15, "KANGXI RADICAL RIGHT OPEN BOX" },
    { 0x2F16, "KANGXI RADICAL HIDING ENCLOSURE" },
    { 0x2F17, "KANGXI RADICAL TEN" },
    { 0x2F18, "KANGXI RADICAL DIVINATION" },
    { 0x2F19, "KANGXI RADICAL SEAL" },
    { 0x2F1A, "KANGXI RADICAL CLIFF" },
    { 0x2F1B, "KANGXI RADICAL PRIVATE" },
    { 0x2F1C, "KANGXI RADICAL AGAIN" },
    { 0x2F1D, "KANGXI RADICAL MOUTH" },
    { 0x2F1E, "KANGXI RADICAL ENCLOSURE" },
    { 0x2F1F, "KANGXI RADICAL EARTH" },
    { 0x2F20, "KANGXI RADICAL SCHOLAR" },
    { 0x2F21, "KANGXI RADICAL GO" },
    { 0x2F22, "KANGXI RADICAL GO SLOWLY" },
    { 0x2F23, "KANGXI RADICAL EVENING" },
    { 0x2F24, "KANGXI RADICAL BIG" },
    { 0x2F25, "KANGXI RADICAL WOMAN" },
    { 0x2F26, "KANGXI RADICAL CHILD" },
    { 0x2F27, "KANGXI RADICAL ROOF" },
    { 0x2F28, "KANGXI RADICAL INCH" },
    { 0x2F29, "KANGXI RADICAL SMALL" },
    { 0x2F2A, "KANGXI RADICAL LAME" },
    { 0x2F2B, "KANGXI RADICAL CORPSE" },
    { 0x2F2C, "KANGXI RADICAL SPROUT" },
    { 0x2F2D, "KANGXI RADICAL MOUNTAIN" },
    { 0x2F2E, "KANGXI RADICAL RIVER" },
    { 0x2F2F, "KANGXI RADICAL WORK" },
    { 0x2F30, "KANGXI RADICAL ONESELF" },
    { 0x2F31, "KANGXI RADICAL TURBAN" },
    { 0x2F32, "KANGXI RADICAL DRY" },
    { 0x2F33, "KANGXI RADICAL SHORT THREAD" },
    { 0x2F34, "KANGXI RADICAL DOTTED CLIFF" },
    { 0x2F35, "KANGXI RADICAL LONG STRIDE" },
    { 0x2F36, "KANGXI RADICAL TWO HANDS" },
    { 0x2F37, "KANGXI RADICAL SHOOT" },
    { 0x2F38, "KANGXI RADICAL BOW" },
    { 0x2F39, "KANGXI RADICAL SNOUT" },
    { 0x2F3A, "KANGXI RADICAL BRISTLE" },
    { 0x2F3B, "KANGXI RADICAL STEP" },
    { 0x2F3C, "KANGXI RADICAL HEART" },
    { 0x2F3D, "KANGXI RADICAL HALBERD" },
    { 0x2F3E, "KANGXI RADICAL DOOR" },
    { 0x2F3F, "KANGXI RADICAL HAND" },
    { 0x2F40, "KANGXI RADICAL BRANCH" },
    { 0x2F41, "KANGXI RADICAL RAP" },
    { 0x2F42, "KANGXI RADICAL SCRIPT" },
    { 0x2F43, "KANGXI RADICAL DIPPER" },
    { 0x2F44, "KANGXI RADICAL AXE" },
    { 0x2F45, "KANGXI RADICAL SQUARE" },
    { 0x2F46, "KANGXI RADICAL NOT" },
    { 0x2F47, "KANGXI RADICAL SUN" },
    { 0x2F48, "KANGXI RADICAL SAY" },
    { 0x2F49, "KANGXI RADICAL MOON" },
    { 0x2F4A, "KANGXI RADICAL TREE" },
    { 0x2F4B, "KANGXI RADICAL LACK" },
    { 0x2F4C, "KANGXI RADICAL STOP" },
    { 0x2F4D, "KANGXI RADICAL DEATH" },
    { 0x2F4E, "KANGXI RADICAL WEAPON" },
    { 0x2F4F, "KANGXI RADICAL DO NOT" },
    { 0x2F50, "KANGXI RADICAL COMPARE" },
    { 0x2F51, "KANGXI RADICAL FUR" },
    { 0x2F52, "KANGXI RADICAL CLAN" },
    { 0x2F53, "KANGXI RADICAL STEAM" },
    { 0x2F54, "KANGXI RADICAL WATER" },
    { 0x2F55, "KANGXI RADICAL FIRE" },
    { 0x2F56, "KANGXI RADICAL CLAW" },
    { 0x2F57, "KANGXI RADICAL FATHER" },
    { 0x2F58, "KANGXI RADICAL DOUBLE X" },
    { 0x2F59, "KANGXI RADICAL HALF TREE TRUNK" },
    { 0x2F5A, "KANGXI RADICAL SLICE" },
    { 0x2F5B, "KANGXI RADICAL FANG" },
    { 0x2F5C, "KANGXI RADICAL COW" },
    { 0x2F5D, "KANGXI RADICAL DOG" },
    { 0x2F5E, "KANGXI RADICAL PROFOUND" },
    { 0x2F5F, "KANGXI RADICAL JADE" },
    { 0x2F60, "KANGXI RADICAL MELON" },
    { 0x2F61, "KANGXI RADICAL TILE" },
    { 0x2F62, "KANGXI RADICAL SWEET" },
    { 0x2F63, "KANGXI RADICAL LIFE" },
    { 0x2F64, "KANGXI RADICAL USE" },
    { 0x2F65, "KANGXI RADICAL FIELD" },
    { 0x2F66, "KANGXI RADICAL BOLT OF CLOTH" },
    { 0x2F67, "KANGXI RADICAL SICKNESS" },
    { 0x2F68, "KANGXI RADICAL DOTTED TENT" },
    { 0x2F69, "KANGXI RADICAL WHITE" },
    { 0x2F6A, "KANGXI RADICAL SKIN" },
    { 0x2F6B, "KANGXI RADICAL DISH" },
    { 0x2F6C, "KANGXI RADICAL EYE" },
    { 0x2F6D, "KANGXI RADICAL SPEAR" },
    { 0x2F6E, "KANGXI RADICAL ARROW" },
    { 0x2F6F, "KANGXI RADICAL STONE" },
    { 0x2F70, "KANGXI RADICAL SPIRIT" },
    { 0x2F71, "KANGXI RADICAL TRACK" },
    { 0x2F72, "KANGXI RADICAL GRAIN" },
    { 0x2F73, "KANGXI RADICAL CAVE" },
    { 0x2F74, "KANGXI RADICAL STAND" },
    { 0x2F75, "KANGXI RADICAL BAMBOO" },
    { 0x2F76, "KANGXI RADICAL RICE" },
    { 0x2F77, "KANGXI RADICAL SILK" },
    { 0x2F78, "KANGXI RADICAL JAR" },
    { 0x2F79, "KANGXI RADICAL NET" },
    { 0x2F7A, "KANGXI RADICAL SHEEP" },
    { 0x2F7B, "KANGXI RADICAL FEATHER" },
    { 0x2F7C, "KANGXI RADICAL OLD" },
    { 0x2F7D, "KANGXI RADICAL AND" },
    { 0x2F7E, "KANGXI RADICAL PLOW" },
    { 0x2F7F, "KANGXI RADICAL EAR" },
    { 0x2F80, "KANGXI RADICAL BRUSH" },
    { 0x2F81, "KANGXI RADICAL MEAT" },
    { 0x2F82, "KANGXI RADICAL MINISTER" },
    { 0x2F83, "KANGXI RADICAL SELF" },
    { 0x2F84, "KANGXI RADICAL ARRIVE" },
    { 0x2F85, "KANGXI RADICAL MORTAR" },
    { 0x2F86, "KANGXI RADICAL TONGUE" },
    { 0x2F87, "KANGXI RADICAL OPPOSE" },
    { 0x2F88, "KANGXI RADICAL BOAT" },
    { 0x2F89, "KANGXI RADICAL STOPPING" },
    { 0x2F8A, "KANGXI RADICAL COLOR" },
    { 0x2F8B, "KANGXI RADICAL GRASS" },
    { 0x2F8C, "KANGXI RADICAL TIGER" },
    { 0x2F8D, "KANGXI RADICAL INSECT" },
    { 0x2F8E, "KANGXI RADICAL BLOOD" },
    { 0x2F8F, "KANGXI RADICAL WALK ENCLOSURE" },
    { 0x2F90, "KANGXI RADICAL CLOTHES" },
    { 0x2F91, "KANGXI RADICAL WEST" },
    { 0x2F92, "KANGXI RADICAL SEE" },
    { 0x2F93, "KANGXI RADICAL HORN" },
    { 0x2F94, "KANGXI RADICAL SPEECH" },
    { 0x2F95, "KANGXI RADICAL VALLEY" },
    { 0x2F96, "KANGXI RADICAL BEAN" },
    { 0x2F97, "KANGXI RADICAL PIG" },
    { 0x2F98, "KANGXI RADICAL BADGER" },
    { 0x2F99, "KANGXI RADICAL SHELL" },
    { 0x2F9A, "KANGXI RADICAL RED" },
    { 0x2F9B, "KANGXI RADICAL RUN" },
    { 0x2F9C, "KANGXI RADICAL FOOT" },
    { 0x2F9D, "KANGXI RADICAL BODY" },
    { 0x2F9E, "KANGXI RADICAL CART" },
    { 0x2F9F, "KANGXI RADICAL BITTER" },
    { 0x2FA0, "KANGXI RADICAL MORNING" },
    { 0x2FA1, "KANGXI RADICAL WALK" },
    { 0x2FA2, "KANGXI RADICAL CITY" },
    { 0x2FA3, "KANGXI RADICAL WINE" },
    { 0x2FA4, "KANGXI RADICAL DISTINGUISH" },
    { 0x2FA5, "KANGXI RADICAL VILLAGE" },
    { 0x2FA6, "KANGXI RADICAL GOLD" },
    { 0x2FA7, "KANGXI RADICAL LONG" },
    { 0x2FA8, "KANGXI RADICAL GATE" },
    { 0x2FA9, "KANGXI RADICAL MOUND" },
    { 0x2FAA, "KANGXI RADICAL SLAVE" },
    { 0x2FAB, "KANGXI RADICAL SHORT TAILED BIRD" },
    { 0x2FAC, "KANGXI RADICAL RAIN" },
    { 0x2FAD, "KANGXI RADICAL BLUE" },
    { 0x2FAE, "KANGXI RADICAL WRONG" },
    { 0x2FAF, "KANGXI RADICAL FACE" },
    { 0x2FB0, "KANGXI RADICAL LEATHER" },
    { 0x2FB1, "KANGXI RADICAL TANNED LEATHER" },
    { 0x2FB2, "KANGXI RADICAL LEEK" },
    { 0x2FB3, "KANGXI RADICAL SOUND" },
    { 0x2FB4, "KANGXI RADICAL LEAF" },
    { 0x2FB5, "KANGXI RADICAL WIND" },
    { 0x2FB6, "KANGXI RADICAL FLY" },
    { 0x2FB7, "KANGXI RADICAL EAT" },
    { 0x2FB8, "KANGXI RADICAL HEAD" },
    { 0x2FB9, "KANGXI RADICAL FRAGRANT" },
    { 0x2FBA, "KANGXI RADICAL HORSE" },
    { 0x2FBB, "KANGXI RADICAL BONE" },
    { 0x2FBC, "KANGXI RADICAL TALL" },
    { 0x2FBD, "KANGXI RADICAL HAIR" },
    { 0x2FBE, "KANGXI RADICAL FIGHT" },
    { 0x2FBF, "KANGXI RADICAL SACRIFICIAL WINE" },
    { 0x2FC0, "KANGXI RADICAL CAULDRON" },
    { 0x2FC1, "KANGXI RADICAL GHOST" },
    { 0x2FC2, "KANGXI RADICAL FISH" },
    { 0x2FC3, "KANGXI RADICAL BIRD" },
    { 0x2FC4, "KANGXI RADICAL SALT" },
    { 0x2FC5, "KANGXI RADICAL DEER" },
    { 0x2FC6, "KANGXI RADICAL WHEAT" },
    { 0x2FC7, "KANGXI RADICAL HEMP" },
    { 0x2FC8, "KANGXI RADICAL YELLOW" },
    { 0x2FC9, "KANGXI RADICAL MILLET" },
    { 0x2FCA, "KANGXI RADICAL BLACK" },
    { 0x2FCB, "KANGXI RADICAL EMBROIDERY" },
    { 0x2FCC, "KANGXI RADICAL FROG" },
    { 0x2FCD, "KANGXI RADICAL TRIPOD" },
    { 0x2FCE, "KANGXI RADICAL DRUM" },
    { 0x2FCF, "KANGXI RADICAL RAT" },
    { 0x2FD0, "KANGXI RADICAL NOSE" },
    { 0x2FD1, "KANGXI RADICAL EVEN" },
    { 0x2FD2, "KANGXI RADICAL TOOTH" },
    { 0x2FD3, "KANGXI RADICAL DRAGON" },
    { 0x2FD4, "KANGXI RADICAL TURTLE" },
    { 0x2FD5, "KANGXI RADICAL FLUTE" },
    { 0x2FF0, "IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO RIGHT" },
    { 0x2FF1, "IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO BELOW" },
    { 0x2FF2, "IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO MIDDLE AND RIGHT" },
    { 0x2FF3, "IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO MIDDLE AND BELOW" },
    { 0x2FF4, "IDEOGRAPHIC DESCRIPTION CHARACTER FULL SURROUND" },
    { 0x2FF5, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM ABOVE" },
    { 0x2FF6, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM BELOW" },
    { 0x2FF7, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LEFT" },
    { 0x2FF8, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER LEFT" },
    { 0x2FF9, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER RIGHT" },
    { 0x2FFA, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER LEFT" },
    { 0x2FFB, "IDEOGRAPHIC DESCRIPTION CHARACTER OVERLAID" },
    { 0x2FFC, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM RIGHT" },
    { 0x2FFD, "IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER RIGHT" },
    { 0x2FFE, "IDEOGRAPHIC DESCRIPTION CHARACTER HORIZONTAL REFLECTION" },
    { 0x2FFF, "IDEOGRAPHIC DESCRIPTION CHARACTER ROTATION" },
    { 0x3000, "IDEOGRAPHIC SPACE" },
    { 0x3001, "IDEOGRAPHIC COMMA" },
    { 0x3002, "IDEOGRAPHIC FULL STOP" },
    { 0x3003, "DITTO MARK" },
    { 0x3004, "JAPANESE INDUSTRIAL STANDARD SYMBOL" },
    { 0x3005, "IDEOGRAPHIC ITERATION MARK" },
    { 0x3006, "IDEOGRAPHIC CLOSING MARK" },
    { 0x3007, "IDEOGRAPHIC NUMBER ZERO" },
    { 0x3008, "LEFT ANGLE BRACKET" },
    { 0x3009, "RIGHT ANGLE BRACKET" },
    { 0x300A, "LEFT DOUBLE ANGLE BRACKET" },
    { 0x300B, "RIGHT DOUBLE ANGLE BRACKET" },
    { 0x300C, "LEFT CORNER BRACKET" },
    { 0x300D, "RIGHT CORNER BRACKET" },
    { 0x300E, "LEFT WHITE CORNER BRACKET" },
    { 0x300F, "RIGHT WHITE CORNER BRACKET" },
    { 0x3010, "LEFT BLACK LENTICULAR BRACKET" },
    { 0x3011, "RIGHT BLACK LENTICULAR BRACKET" },
    { 0x3012, "POSTAL MARK" },
    { 0x3013, "GETA MARK" },
    { 0x3014, "LEFT TORTOISE SHELL BRACKET" },
    { 0x3015, "RIGHT TORTOISE SHELL BRACKET" },
    { 0x3016, "LEFT WHITE LENTICULAR BRACKET" },
    { 0x3017, "RIGHT WHITE LENTICULAR BRACKET" },
    { 0x3018, "LEFT WHITE TORTOISE SHELL BRACKET" },
    { 0x3019, "RIGHT WHITE TORTOISE SHELL BRACKET" },
    { 0x301A, "LEFT WHITE SQUARE BRACKET" },
    { 0x301B, "RIGHT WHITE SQUARE BRACKET" },
    { 0x301C, "WAVE DASH" },
    { 0x301D, "REVERSED DOUBLE PRIME QUOTATION MARK" },
    { 0x301E, "DOUBLE PRIME QUOTATION MARK" },
    { 0x301F, "LOW DOUBLE PRIME QUOTATION MARK" },
    { 0x3020, "POSTAL MARK FACE" },
    { 0x3021, "HANGZHOU NUMERAL ONE" },
    { 0x3022, "HANGZHOU NUMERAL TWO" },
    { 0x3023, "HANGZHOU NUMERAL THREE" },
    { 0x3024, "HANGZHOU NUMERAL FOUR" },
    { 0x3025, "HANGZHOU NUMERAL FIVE" },
    { 0x3026, "HANGZHOU NUMERAL SIX" },
    { 0x3027, "HANGZHOU NUMERAL SEVEN" },
    { 0x3028, "HANGZHOU NUMERAL EIGHT" },
    { 0x3029, "HANGZHOU NUMERAL NINE" },
    { 0x302A, "IDEOGRAPHIC LEVEL TONE MARK" },
    { 0x302B, "IDEOGRAPHIC RISING TONE MARK" },
    { 0x302C, "IDEOGRAPHIC DEPARTING TONE MARK" },
    { 0x302D, "IDEOGRAPHIC ENTERING TONE MARK" },
    { 0x302E, "HANGUL SINGLE DOT TONE MARK" },
    { 0x302F, "HANGUL DOUBLE DOT TONE MARK" },
    { 0x3030, "WAVY DASH" },
    { 0x3031, "VERTICAL KANA REPEAT MARK" },
    { 0x3032, "VERTICAL KANA REPEAT WITH VOICED SOUND MARK" },
    { 0x3033, "VERTICAL KANA REPEAT MARK UPPER HALF" },
    { 0x3034, "VERTICAL KANA REPEAT WITH VOICED SOUND MARK UPPER HALF" },
    { 0x3035, "VERTICAL KANA REPEAT MARK LOWER HALF" },
    { 0x3036, "CIRCLED POSTAL MARK" },
    { 0x3037, "IDEOGRAPHIC TELEGRAPH LINE FEED SEPARATOR SYMBOL" },
    { 0x3038, "HANGZHOU NUMERAL TEN" },
    { 0x3039, "HANGZHOU NUMERAL TWENTY" },
    { 0x303A, "HANGZHOU NUMERAL THIRTY" },
    { 0x303B, "VERTICAL IDEOGRAPHIC ITERATION MARK" },
    { 0x303C, "MASU MARK" },
    { 0x303D, "PART ALTERNATION MARK" },
    { 0x303E, "IDEOGRAPHIC VARIATION INDICATOR" },
    { 0x303F, "IDEOGRAPHIC HALF FILL SPACE" },
    { 0x3041, "HIRAGANA LETTER SMALL A" },
    { 0x3042, "HIRAGANA LETTER A" },
    { 0x3043, "HIRAGANA LETTER SMALL I" },
    { 0x3044, "HIRAGANA LETTER I" },
    { 0x3045, "HIRAGANA LETTER SMALL U" },
    { 0x3046, "HIRAGANA LETTER U" },
    { 0x3047, "HIRAGANA LETTER SMALL E" },
    { 0x3048, "HIRAGANA LETTER E" },
    { 0x3049, "HIRAGANA LETTER SMALL O" },
    { 0x304A, "HIRAGANA LETTER O" },
    { 0x304B, "HIRAGANA LETTER KA" },
    { 0x304C, "HIRAGANA LETTER GA" },
    { 0x304D, "HIRAGANA LETTER KI" },
    { 0x304E, "HIRAGANA LETTER GI" },
    { 0x304F, "HIRAGANA LETTER KU" },
    { 0x3050, "HIRAGANA LETTER GU" },
    { 0x3051, "HIRAGANA LETTER KE" },
    { 0x3052, "HIRAGANA LETTER GE" },
    { 0x3053, "HIRAGANA LETTER KO" },
    { 0x3054, "HIRAGANA LETTER GO" },
    { 0x3055, "HIRAGANA LETTER SA" },
    { 0x3056, "HIRAGANA LETTER ZA" },
    { 0x3057, "HIRAGANA LETTER SI" },
    { 0x3058, "HIRAGANA LETTER ZI" },
    { 0x3059, "HIRAGANA LETTER SU" },
    { 0x305A, "HIRAGANA LETTER ZU" },
    { 0x305B, "HIRAGANA LETTER SE" },
    { 0x305C, "HIRAGANA LETTER ZE" },
    { 0x305D, "HIRAGANA LETTER SO" },
    { 0x305E, "HIRAGANA LETTER ZO" },
    { 0x305F, "HIRAGANA LETTER TA" },
    { 0x3060, "HIRAGANA LETTER DA" },
    { 0x3061, "HIRAGANA LETTER TI" },
    { 0x3062, "HIRAGANA LETTER DI" },
    { 0x3063, "HIRAGANA LETTER SMALL TU" },
    { 0x3064, "HIRAGANA LETTER TU" },
    { 0x3065, "HIRAGANA LETTER DU" },
    { 0x3066, "HIRAGANA LETTER TE" },
    { 0x3067, "HIRAGANA LETTER DE" },
    { 0x3068, "HIRAGANA LETTER TO" },
    { 0x3069, "HIRAGANA LETTER DO" },
    { 0x306A, "HIRAGANA LETTER NA" },
    { 0x306B, "HIRAGANA LETTER NI" },
    { 0x306C, "HIRAGANA LETTER NU" },
    { 0x306D, "HIRAGANA LETTER NE" },
    { 0x306E, "HIRAGANA LETTER NO" },
    { 0x306F, "HIRAGANA LETTER HA" },
    { 0x3070, "HIRAGANA LETTER BA" },
    { 0x3071, "HIRAGANA LETTER PA" },
    { 0x3072, "HIRAGANA LETTER HI" },
    { 0x3073, "HIRAGANA LETTER BI" },
    { 0x3074, "HIRAGANA LETTER PI" },
    { 0x3075, "HIRAGANA LETTER HU" },
    { 0x3076, "HIRAGANA LETTER BU" },
    { 0x3077, "HIRAGANA LETTER PU" },
    { 0x3078, "HIRAGANA LETTER HE" },
    { 0x3079, "HIRAGANA LETTER BE" },
    { 0x307A, "HIRAGANA LETTER PE" },
    { 0x307B, "HIRAGANA LETTER HO" },
    { 0x307C, "HIRAGANA LETTER BO" },
    { 0x307D, "HIRAGANA LETTER PO" },
    { 0x307E, "HIRAGANA LETTER MA" },
    { 0x307F, "HIRAGANA LETTER MI" },
    { 0x3080, "HIRAGANA LETTER MU" },
    { 0x3081, "HIRAGANA LETTER ME" },
    { 0x3082, "HIRAGANA LETTER MO" },
    { 0x3083, "HIRAGANA LETTER SMALL YA" },
    { 0x3084, "HIRAGANA LETTER YA" },
    { 0x3085, "HIRAGANA LETTER SMALL YU" },
    { 0x3086, "HIRAGANA LETTER YU" },
    { 0x3087, "HIRAGANA LETTER SMALL YO" },
    { 0x3088, "HIRAGANA LETTER YO" },
    { 0x3089, "HIRAGANA LETTER RA" },
    { 0x308A, "HIRAGANA LETTER RI" },
    { 0x308B, "HIRAGANA LETTER RU" },
    { 0x308C, "HIRAGANA LETTER RE" },
    { 0x308D, "HIRAGANA LETTER RO" },
    { 0x308E, "HIRAGANA LETTER SMALL WA" },
    { 0x308F, "HIRAGANA LETTER WA" },
    { 0x3090, "HIRAGANA LETTER WI" },
    { 0x3091, "HIRAGANA LETTER WE" },
    { 0x3092, "HIRAGANA LETTER WO" },
    { 0x3093, "HIRAGANA LETTER N" },
    { 0x3094, "HIRAGANA LETTER VU" },
    { 0x3095, "HIRAGANA LETTER SMALL KA" },
    { 0x3096, "HIRAGANA LETTER SMALL KE" },
    { 0x3099, "COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK" },
    { 0x309A, "COMBINING KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK" },
    { 0x309B, "KATAKANA-HIRAGANA VOICED SOUND MARK" },
    { 0x309C, "KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK" },
    { 0x309D, "HIRAGANA ITERATION MARK" },
    { 0x309E, "HIRAGANA VOICED ITERATION MARK" },
    { 0x309F, "HIRAGANA DIGRAPH YORI" },
    { 0x30A0, "KATAKANA-HIRAGANA DOUBLE HYPHEN" },
    { 0x30A1, "KATAKANA LETTER SMALL A" },
    { 0x30A2, "KATAKANA LETTER A" },
    { 0x30A3, "KATAKANA LETTER SMALL I" },
    { 0x30A4, "KATAKANA LETTER I" },
    { 0x30A5, "KATAKANA LETTER SMALL U" },
    { 0x30A6, "KATAKANA LETTER U" },
    { 0x30A7, "KATAKANA LETTER SMALL E" },
    { 0x30A8, "KATAKANA LETTER E" },
    { 0x30A9, "KATAKANA LETTER SMALL O" },
    { 0x30AA, "KATAKANA LETTER O" },
    { 0x30AB, "KATAKANA LETTER KA" },
    { 0x30AC, "KATAKANA LETTER GA" },
    { 0x30AD, "KATAKANA LETTER KI" },
    { 0x30AE, "KATAKANA LETTER GI" },
    { 0x30AF, "KATAKANA LETTER KU" },
    { 0x30B0, "KATAKANA LETTER GU" },
    { 0x30B1, "KATAKANA LETTER KE" },
    { 0x30B2, "KATAKANA LETTER GE" },
    { 0x30B3, "KATAKANA LETTER KO" },
    { 0x30B4, "KATAKANA LETTER GO" },
    { 0x30B5, "KATAKANA LETTER SA" },
    { 0x30B6, "KATAKANA LETTER ZA" },
    { 0x30B7, "KATAKANA LETTER SI" },
    { 0x30B8, "KATAKANA LETTER ZI" },
    { 0x30B9, "KATAKANA LETTER SU" },
    { 0x30BA, "KATAKANA LETTER ZU" },
    { 0x30BB, "KATAKANA LETTER SE" },
    { 0x30BC, "KATAKANA LETTER ZE" },
    { 0x30BD, "KATAKANA LETTER SO" },
    { 0x30BE, "KATAKANA LETTER ZO" },
    { 0x30BF, "KATAKANA LETTER TA" },
    { 0x30C0, "KATAKANA LETTER DA" },
    { 0x30C1, "KATAKANA LETTER TI" },
    { 0x30C2, "KATAKANA LETTER DI" },
    { 0x30C3, "KATAKANA LETTER SMALL TU" },
    { 0x30C4, "KATAKANA LETTER TU" },
    { 0x30C5, "KATAKANA LETTER DU" },
    { 0x30C6, "KATAKANA LETTER TE" },
    { 0x30C7, "KATAKANA LETTER DE" },
    { 0x30C8, "KATAKANA LETTER TO" },
    { 0x30C9, "KATAKANA LETTER DO" },
    { 0x30CA, "KATAKANA LETTER NA" },
    { 0x30CB, "KATAKANA LETTER NI" },
    { 0x30CC, "KATAKANA LETTER NU" },
    { 0x30CD, "KATAKANA LETTER NE" },
    { 0x30CE, "KATAKANA LETTER NO" },
    { 0x30CF, "KATAKANA LETTER HA" },
    { 0x30D0, "KATAKANA LETTER BA" },
    { 0x30D1, "KATAKANA LETTER PA" },
    { 0x30D2, "KATAKANA LETTER HI" },
    { 0x30D3, "KATAKANA LETTER BI" },
    { 0x30D4, "KATAKANA LETTER PI" },
    { 0x30D5, "KATAKANA LETTER HU" },
    { 0x30D6, "KATAKANA LETTER BU" },
    { 0x30D7, "KATAKANA LETTER PU" },
    { 0x30D8, "KATAKANA LETTER HE" },
    { 0x30D9, "KATAKANA LETTER BE" },
    { 0x30DA, "KATAKANA LETTER PE" },
    { 0x30DB, "KATAKANA LETTER HO" },
    { 0x30DC, "KATAKANA LETTER BO" },
    { 0x30DD, "KATAKANA LETTER PO" },
    { 0x30DE, "KATAKANA LETTER MA" },
    { 0x30DF, "KATAKANA LETTER MI" },
    { 0x30E0, "KATAKANA LETTER MU" },
    { 0x30E1, "KATAKANA LETTER ME" },
    { 0x30E2, "KATAKANA LETTER MO" },
    { 0x30E3, "KATAKANA LETTER SMALL YA" },
    { 0x30E4, "KATAKANA LETTER YA" },
    { 0x30E5, "KATAKANA LETTER SMALL YU" },
    { 0x30E6, "KATAKANA LETTER YU" },
    { 0x30E7, "KATAKANA LETTER SMALL YO" },
    { 0x30E8, "KATAKANA LETTER YO" },
    { 0x30E9, "KATAKANA LETTER RA" },
    { 0x30EA, "KATAKANA LETTER RI" },
    { 0x30EB, "KATAKANA LETTER RU" },
    { 0x30EC, "KATAKANA LETTER RE" },
    { 0x30ED, "KATAKANA LETTER RO" },
    { 0x30EE, "KATAKANA LETTER SMALL WA" },
    { 0x30EF, "KATAKANA LETTER WA" },
    { 0x30F0, "KATAKANA LETTER WI" },
    { 0x30F1, "KATAKANA LETTER WE" },
    { 0x30F2, "KATAKANA LETTER WO" },
    { 0x30F3, "KATAKANA LETTER N" },
    { 0x30F4, "KATAKANA LETTER VU" },
    { 0x30F5, "KATAKANA LETTER SMALL KA" },
    { 0x30F6, "KATAKANA LETTER SMALL KE" },
    { 0x30F7, "KATAKANA LETTER VA" },
    { 0x30F8, "KATAKANA LETTER VI" },
    { 0x30F9, "KATAKANA LETTER VE" },
    { 0x30FA, "KATAKANA LETTER VO" },
    { 0x30FB, "KATAKANA MIDDLE DOT" },
    { 0x30FC, "KATAKANA-HIRAGANA PROLONGED SOUND MARK" },
    { 0x30FD, "KATAKANA ITERATION MARK" },
    { 0x30FE, "KATAKANA VOICED ITERATION MARK" },
    { 0x30FF, "KATAKANA DIGRAPH KOTO" },
    { 0x3105, "BOPOMOFO LETTER B" },
    { 0x3106, "BOPOMOFO LETTER P" },
    { 0x3107, "BOPOMOFO LETTER M" },
    { 0x3108, "BOPOMOFO LETTER F" },
    { 0x3109, "BOPOMOFO LETTER D" },
    { 0x310A, "BOPOMOFO LETTER T" },
    { 0x310B, "BOPOMOFO LETTER N" },
    { 0x310C, "BOPOMOFO LETTER L" },
    { 0x310D, "BOPOMOFO LETTER G" },
    { 0x310E, "BOPOMOFO LETTER K" },
    { 0x310F, "BOPOMOFO LETTER H" },
    { 0x3110, "BOPOMOFO LETTER J" },
    { 0x3111, "BOPOMOFO LETTER Q" },
    { 0x3112, "BOPOMOFO LETTER X" },
    { 0x3113, "BOPOMOFO LETTER ZH" },
    { 0x3114, "BOPOMOFO LETTER CH" },
    { 0x3115, "BOPOMOFO LETTER SH" },
    { 0x3116, "BOPOMOFO LETTER R" },
    { 0x3117, "BOPOMOFO LETTER Z" },
    { 0x3118, "BOPOMOFO LETTER C" },
    { 0x3119, "BOPOMOFO LETTER S" },
    { 0x311A, "BOPOMOFO LETTER A" },
    { 0x311B, "BOPOMOFO LETTER O" },
    { 0x311C, "BOPOMOFO LETTER E" },
    { 0x311D, "BOPOMOFO LETTER EH" },
    { 0x311E, "BOPOMOFO LETTER AI" },
    { 0x311F, "BOPOMOFO LETTER EI" },
    { 0x3120, "BOPOMOFO LETTER AU" },
    { 0x3121, "BOPOMOFO LETTER OU" },
    { 0x3122, "BOPOMOFO LETTER AN" },
    { 0x3123, "BOPOMOFO LETTER EN" },
    { 0x3124, "BOPOMOFO LETTER ANG" },
    { 0x3125, "BOPOMOFO LETTER ENG" },
    { 0x3126, "BOPOMOFO LETTER ER" },
    { 0x3127, "BOPOMOFO LETTER I" },
    { 0x3128, "BOPOMOFO LETTER U" },
    { 0x3129, "BOPOMOFO LETTER IU" },
    { 0x312A, "BOPOMOFO LETTER V" },
    { 0x312B, "BOPOMOFO LETTER NG" },
    { 0x312C, "BOPOMOFO LETTER GN" },
    { 0x312D, "BOPOMOFO LETTER IH" },
    { 0x312E, "BOPOMOFO LETTER O WITH DOT ABOVE" },
    { 0x312F, "BOPOMOFO LETTER NN" },
    { 0x3131, "HANGUL LETTER KIYEOK" },
    { 0x3132, "HANGUL LETTER SSANGKIYEOK" },
    { 0x3133, "HANGUL LETTER KIYEOK-SIOS" },
    { 0x3134, "HANGUL LETTER NIEUN" },
    { 0x3135, "HANGUL LETTER NIEUN-CIEUC" },
    { 0x3136, "HANGUL LETTER NIEUN-HIEUH" },
    { 0x3137, "HANGUL LETTER TIKEUT" },
    { 0x3138, "HANGUL LETTER SSANGTIKEUT" },
    { 0x3139, "HANGUL LETTER RIEUL" },
    { 0x313A, "HANGUL LETTER RIEUL-KIYEOK" },
    { 0x313B, "HANGUL LETTER RIEUL-MIEUM" },
    { 0x313C, "HANGUL LETTER RIEUL-PIEUP" },
    { 0x313D, "HANGUL LETTER RIEUL-SIOS" },
    { 0x313E, "HANGUL LETTER RIEUL-THIEUTH" },
    { 0x313F, "HANGUL LETTER RIEUL-PHIEUPH" },
    { 0x3140, "HANGUL LETTER RIEUL-HIEUH" },
    { 0x3141, "HANGUL LETTER MIEUM" },
    { 0x3142, "HANGUL LETTER PIEUP" },
    { 0x3143, "HANGUL LETTER SSANGPIEUP" },
    { 0x3144, "HANGUL LETTER PIEUP-SIOS" },
    { 0x3145, "HANGUL LETTER SIOS" },
    { 0x3146, "HANGUL LETTER SSANGSIOS" },
    { 0x3147, "HANGUL LETTER IEUNG" },
    { 0x3148, "HANGUL LETTER CIEUC" },
    { 0x3149, "HANGUL LETTER SSANGCIEUC" },
    { 0x314A, "HANGUL LETTER CHIEUCH" },
    { 0x314B, "HANGUL LETTER KHIEUKH" },
    { 0x314C, "HANGUL LETTER THIEUTH" },
    { 0x314D, "HANGUL LETTER PHIEUPH" },
    { 0x314E, "HANGUL LETTER HIEUH" },
    { 0x314F, "HANGUL LETTER A" },
    { 0x3150, "HANGUL LETTER AE" },
    { 0x3151, "HANGUL LETTER YA" },
    { 0x3152, "HANGUL LETTER YAE" },
    { 0x3153, "HANGUL LETTER EO" },
    { 0x3154, "HANGUL LETTER E" },
    { 0x3155, "HANGUL LETTER YEO" },
    { 0x3156, "HANGUL LETTER YE" },
    { 0x3157, "HANGUL LETTER O" },
    { 0x3158, "HANGUL LETTER WA" },
    { 0x3159, "HANGUL LETTER WAE" },
    { 0x315A, "HANGUL LETTER OE" },
    { 0x315B, "HANGUL LETTER YO" },
    { 0x315C, "HANGUL LETTER U" },
    { 0x315D, "HANGUL LETTER WEO" },
    { 0x315E, "HANGUL LETTER WE" },
    { 0x315F, "HANGUL LETTER WI" },
    { 0x3160, "HANGUL LETTER YU" },
    { 0x3161, "HANGUL LETTER EU" },
    { 0x3162, "HANGUL LETTER YI" },
    { 0x3163, "HANGUL LETTER I" },
    { 0x3164, "HANGUL FILLER" },
    { 0x3165, "HANGUL LETTER SSANGNIEUN" },
    { 0x3166, "HANGUL LETTER NIEUN-TIKEUT" },
    { 0x3167, "HANGUL LETTER NIEUN-SIOS" },
    { 0x3168, "HANGUL LETTER NIEUN-PANSIOS" },
    { 0x3169, "HANGUL LETTER RIEUL-KIYEOK-SIOS" },
    { 0x316A, "HANGUL LETTER RIEUL-TIKEUT" },
    { 0x316B, "HANGUL LETTER RIEUL-PIEUP-SIOS" },
    { 0x316C, "HANGUL LETTER RIEUL-PANSIOS" },
    { 0x316D, "HANGUL LETTER RIEUL-YEORINHIEUH" },
    { 0x316E, "HANGUL LETTER MIEUM-PIEUP" },
    { 0x316F, "HANGUL LETTER MIEUM-SIOS" },
    { 0x3170, "HANGUL LETTER MIEUM-PANSIOS" },
    { 0x3171, "HANGUL LETTER KAPYEOUNMIEUM" },
    { 0x3172, "HANGUL LETTER PIEUP-KIYEOK" },
    { 0x3173, "HANGUL LETTER PIEUP-TIKEUT" },
    { 0x3174, "HANGUL LETTER PIEUP-SIOS-KIYEOK" },
    { 0x3175, "HANGUL LETTER PIEUP-SIOS-TIKEUT" },
    { 0x3176, "HANGUL LETTER PIEUP-CIEUC" },
    { 0x3177, "HANGUL LETTER PIEUP-THIEUTH" },
    { 0x3178, "HANGUL LETTER KAPYEOUNPIEUP" },
    { 0x3179, "HANGUL LETTER KAPYEOUNSSANGPIEUP" },
    { 0x317A, "HANGUL LETTER SIOS-KIYEOK" },
    { 0x317B, "HANGUL LETTER SIOS-NIEUN" },
    { 0x317C, "HANGUL LETTER SIOS-TIKEUT" },
    { 0x317D, "HANGUL LETTER SIOS-PIEUP" },
    { 0x317E, "HANGUL LETTER SIOS-CIEUC" },
    { 0x317F, "HANGUL LETTER PANSIOS" },
    { 0x3180, "HANGUL LETTER SSANGIEUNG" },
    { 0x3181, "HANGUL LETTER YESIEUNG" },
    { 0x3182, "HANGUL LETTER YESIEUNG-SIOS" },
    { 0x3183, "HANGUL LETTER YESIEUNG-PANSIOS" },
    { 0x3184, "HANGUL LETTER KAPYEOUNPHIEUPH" },
    { 0x3185, "HANGUL LETTER SSANGHIEUH" },
    { 0x3186, "HANGUL LETTER YEORINHIEUH" },
    { 0x3187, "HANGUL LETTER YO-YA" },
    { 0x3188, "HANGUL LETTER YO-YAE" },
    { 0x3189, "HANGUL LETTER YO-I" },
    { 0x318A, "HANGUL LETTER YU-YEO" },
    { 0x318B, "HANGUL LETTER YU-YE" },
    { 0x318C, "HANGUL LETTER YU-I" },
    { 0x318D, "HANGUL LETTER ARAEA" },
    { 0x318E, "HANGUL LETTER ARAEAE" },
    { 0x3190, "IDEOGRAPHIC ANNOTATION LINKING MARK" },
    { 0x3191, "IDEOGRAPHIC ANNOTATION REVERSE MARK" },
    { 0x3192, "IDEOGRAPHIC ANNOTATION ONE MARK" },
    { 0x3193, "IDEOGRAPHIC ANNOTATION TWO MARK" },
    { 0x3194, "IDEOGRAPHIC ANNOTATION THREE MARK" },
    { 0x3195, "IDEOGRAPHIC ANNOTATION FOUR MARK" },
    { 0x3196, "IDEOGRAPHIC ANNOTATION TOP MARK" },
    { 0x3197, "IDEOGRAPHIC ANNOTATION MIDDLE MARK" },
    { 0x3198, "IDEOGRAPHIC ANNOTATION BOTTOM MARK" },
    { 0x3199, "IDEOGRAPHIC ANNOTATION FIRST MARK" },
    { 0x319A, "IDEOGRAPHIC ANNOTATION SECOND MARK" },
    { 0x319B, "IDEOGRAPHIC ANNOTATION THIRD MARK" },
    { 0x319C, "IDEOGRAPHIC ANNOTATION FOURTH MARK" },
    { 0x319D, "IDEOGRAPHIC ANNOTATION HEAVEN MARK" },
    { 0x319E, "IDEOGRAPHIC ANNOTATION EARTH MARK" },
    { 0x319F, "IDEOGRAPHIC ANNOTATION MAN MARK" },
    { 0x31A0, "BOPOMOFO LETTER BU" },
    { 0x31A1, "BOPOMOFO LETTER ZI" },
    { 0x31A2, "BOPOMOFO LETTER JI" },
    { 0x31A3, "BOPOMOFO LETTER GU" },
    { 0x31A4, "BOPOMOFO LETTER EE" },
    { 0x31A5, "BOPOMOFO LETTER ENN" },
    { 0x31A6, "BOPOMOFO LETTER OO" },
    { 0x31A7, "BOPOMOFO LETTER ONN" },
    { 0x31A8, "BOPOMOFO LETTER IR" },
    { 0x31A9, "BOPOMOFO LETTER ANN" },
    { 0x31AA, "BOPOMOFO LETTER INN" },
    { 0x31AB, "BOPOMOFO LETTER UNN" },
    { 0x31AC, "BOPOMOFO LETTER IM" },
    { 0x31AD, "BOPOMOFO LETTER NGG" },
    { 0x31AE, "BOPOMOFO LETTER AINN" },
    { 0x31AF, "BOPOMOFO LETTER AUNN" },
    { 0x31B0, "BOPOMOFO LETTER AM" },
    { 0x31B1, "BOPOMOFO LETTER OM" },
    { 0x31B2, "BOPOMOFO LETTER ONG" },
    { 0x31B3, "BOPOMOFO LETTER INNN" },
    { 0x31B4, "BOPOMOFO FINAL LETTER P" },
    { 0x31B5, "BOPOMOFO FINAL LETTER T" },
    { 0x31B6, "BOPOMOFO FINAL LETTER K" },
    { 0x31B7, "BOPOMOFO FINAL LETTER H" },
    { 0x31B8, "BOPOMOFO LETTER GH" },
    { 0x31B9, "BOPOMOFO LETTER LH" },
    { 0x31BA, "BOPOMOFO LETTER ZY" },
    { 0x31BB, "BOPOMOFO FINAL LETTER G" },
    { 0x31BC, "BOPOMOFO LETTER GW" },
    { 0x31BD, "BOPOMOFO LETTER KW" },
    { 0x31BE, "BOPOMOFO LETTER OE" },
    { 0x31BF, "BOPOMOFO LETTER AH" },
    { 0x31C0, "CJK STROKE T" },
    { 0x31C1, "CJK STROKE WG" },
    { 0x31C2, "CJK STROKE XG" },
    { 0x31C3, "CJK STROKE BXG" },
    { 0x31C4, "CJK STROKE SW" },
    { 0x31C5, "CJK STROKE HZZ" },
    { 0x31C6, "CJK STROKE HZG" },
    { 0x31C7, "CJK STROKE HP" },
    { 0x31C8, "CJK STROKE HZWG" },
    { 0x31C9, "CJK STROKE SZWG" },
    { 0x31CA, "CJK STROKE HZT" },
    { 0x31CB, "CJK STROKE HZZP" },
    { 0x31CC, "CJK STROKE HPWG" },
    { 0x31CD, "CJK STROKE HZW" },
    { 0x31CE, "CJK STROKE HZZZ" },
    { 0x31CF, "CJK STROKE N" },
    { 0x31D0, "CJK STROKE H" },
    { 0x31D1, "CJK STROKE S" },
    { 0x31D2, "CJK STROKE P" },
    { 0x31D3, "CJK STROKE SP" },
    { 0x31D4, "CJK STROKE D" },
    { 0x31D5, "CJK STROKE HZ" },
    { 0x31D6, "CJK STROKE HG" },
    { 0x31D7, "CJK STROKE SZ" },
    { 0x31D8, "CJK STROKE SWZ" },
    { 0x31D9, "CJK STROKE ST" },
    { 0x31DA, "CJK STROKE SG" },
    { 0x31DB, "CJK STROKE PD" },
    { 0x31DC, "CJK STROKE PZ" },
    { 0x31DD, "CJK STROKE TN" },
    { 0x31DE, "CJK STROKE SZZ" },
    { 0x31DF, "CJK STROKE SWG" },
    { 0x31E0, "CJK STROKE HXWG" },
    { 0x31E1, "CJK STROKE HZZZG" },
    { 0x31E2, "CJK STROKE PG" },
    { 0x31E3, "CJK STROKE Q" },
    { 0x31EF, "IDEOGRAPHIC DESCRIPTION CHARACTER SUBTRACTION" },
    { 0x31F0, "KATAKANA LETTER SMALL KU" },
    { 0x31F1, "KATAKANA LETTER SMALL SI" },
    { 0x31F2, "KATAKANA LETTER SMALL SU" },
    { 0x31F3, "KATAKANA LETTER SMALL TO" },
    { 0x31F4, "KATAKANA LETTER SMALL NU" },
    { 0x31F5, "KATAKANA LETTER SMALL HA" },
    { 0x31F6, "KATAKANA LETTER SMALL HI" },
    { 0x31F7, "KATAKANA LETTER SMALL HU" },
    { 0x31F8, "KATAKANA LETTER SMALL HE" },
    { 0x31F9, "KATAKANA LETTER SMALL HO" },
    { 0x31FA, "KATAKANA LETTER SMALL MU" },
    { 0x31FB, "KATAKANA LETTER SMALL RA" },
    { 0x31FC, "KATAKANA LETTER SMALL RI" },
    { 0x31FD, "KATAKANA LETTER SMALL RU" },
    { 0x31FE, "KATAKANA LETTER SMALL RE" },
    { 0x31FF, "KATAKANA LETTER SMALL RO" },
    { 0x3200, "PARENTHESIZED HANGUL KIYEOK" },
    { 0x3201, "PARENTHESIZED HANGUL NIEUN" },
    { 0x3202, "PARENTHESIZED HANGUL TIKEUT" },
    { 0x3203, "PARENTHESIZED HANGUL RIEUL" },
    { 0x3204, "PARENTHESIZED HANGUL MIEUM" },
    { 0x3205, "PARENTHESIZED HANGUL PIEUP" },
    { 0x3206, "PARENTHESIZED HANGUL SIOS" },
    { 0x3207, "PARENTHESIZED HANGUL IEUNG" },
    { 0x3208, "PARENTHESIZED HANGUL CIEUC" },
    { 0x3209, "PARENTHESIZED HANGUL CHIEUCH" },
    { 0x320A, "PARENTHESIZED HANGUL KHIEUKH" },
    { 0x320B, "PARENTHESIZED HANGUL THIEUTH" },
    { 0x320C, "PARENTHESIZED HANGUL PHIEUPH" },
    { 0x320D, "PARENTHESIZED HANGUL HIEUH" },
    { 0x320E, "PARENTHESIZED HANGUL KIYEOK A" },
    { 0x320F, "PARENTHESIZED HANGUL NIEUN A" },
    { 0x3210, "PARENTHESIZED HANGUL TIKEUT A" },
    { 0x3211, "PARENTHESIZED HANGUL RIEUL A" },
    { 0x3212, "PARENTHESIZED HANGUL MIEUM A" },
    { 0x3213, "PARENTHESIZED HANGUL PIEUP A" },
    { 0x3214, "PARENTHESIZED HANGUL SIOS A" },
    { 0x3215, "PARENTHESIZED HANGUL IEUNG A" },
    { 0x3216, "PARENTHESIZED HANGUL CIEUC A" },
    { 0x3217, "PARENTHESIZED HANGUL CHIEUCH A" },
    { 0x3218, "PARENTHESIZED HANGUL KHIEUKH A" },
    { 0x3219, "PARENTHESIZED HANGUL THIEUTH A" },
    { 0x321A, "PARENTHESIZED HANGUL PHIEUPH A" },
    { 0x321B, "PARENTHESIZED HANGUL HIEUH A" },
    { 0x321C, "PARENTHESIZED HANGUL CIEUC U" },
    { 0x321D, "PARENTHESIZED KOREAN CHARACTER OJEON" },
    { 0x321E, "PARENTHESIZED KOREAN CHARACTER O HU" },
    { 0x3220, "PARENTHESIZED IDEOGRAPH ONE" },
    { 0x3221, "PARENTHESIZED IDEOGRAPH TWO" },
    { 0x3222, "PARENTHESIZED IDEOGRAPH THREE" },
    { 0x3223, "PARENTHESIZED IDEOGRAPH FOUR" },
    { 0x3224, "PARENTHESIZED IDEOGRAPH FIVE" },
    { 0x3225, "PARENTHESIZED IDEOGRAPH SIX" },
    { 0x3226, "PARENTHESIZED IDEOGRAPH SEVEN" },
    { 0x3227, "PARENTHESIZED IDEOGRAPH EIGHT" },
    { 0x3228, "PARENTHESIZED IDEOGRAPH NINE" },
    { 0x3229, "PARENTHESIZED IDEOGRAPH TEN" },
    { 0x322A, "PARENTHESIZED IDEOGRAPH MOON" },
    { 0x322B, "PARENTHESIZED IDEOGRAPH FIRE" },
    { 0x322C, "PARENTHESIZED IDEOGRAPH WATER" },
    { 0x322D, "PARENTHESIZED IDEOGRAPH WOOD" },
    { 0x322E, "PARENTHESIZED IDEOGRAPH METAL" },
    { 0x322F, "PARENTHESIZED IDEOGRAPH EARTH" },
    { 0x3230, "PARENTHESIZED IDEOGRAPH SUN" },
    { 0x3231, "PARENTHESIZED IDEOGRAPH STOCK" },
    { 0x3232, "PARENTHESIZED IDEOGRAPH HAVE" },
    { 0x3233, "PARENTHESIZED IDEOGRAPH SOCIETY" },
    { 0x3234, "PARENTHESIZED IDEOGRAPH NAME" },
    { 0x3235, "PARENTHESIZED IDEOGRAPH SPECIAL" },
    { 0x3236, "PARENTHESIZED IDEOGRAPH FINANCIAL" },
    { 0x3237, "PARENTHESIZED IDEOGRAPH CONGRATULATION" },
    { 0x3238, "PARENTHESIZED IDEOGRAPH LABOR" },
    { 0x3239, "PARENTHESIZED IDEOGRAPH REPRESENT" },
    { 0x323A, "PARENTHESIZED IDEOGRAPH CALL" },
    { 0x323B, "PARENTHESIZED IDEOGRAPH STUDY" },
    { 0x323C, "PARENTHESIZED IDEOGRAPH SUPERVISE" },
    { 0x323D, "PARENTHESIZED IDEOGRAPH ENTERPRISE" },
    { 0x323E, "PARENTHESIZED IDEOGRAPH RESOURCE" },
    { 0x323F, "PARENTHESIZED IDEOGRAPH ALLIANCE" },
    { 0x3240, "PARENTHESIZED IDEOGRAPH FESTIVAL" },
    { 0x3241, "PARENTHESIZED IDEOGRAPH REST" },
    { 0x3242, "PARENTHESIZED IDEOGRAPH SELF" },
    { 0x3243, "PARENTHESIZED IDEOGRAPH REACH" },
    { 0x3244, "CIRCLED IDEOGRAPH QUESTION" },
    { 0x3245, "CIRCLED IDEOGRAPH KINDERGARTEN" },
    { 0x3246, "CIRCLED IDEOGRAPH SCHOOL" },
    { 0x3247, "CIRCLED IDEOGRAPH KOTO" },
    { 0x3248, "CIRCLED NUMBER TEN ON BLACK SQUARE" },
    { 0x3249, "CIRCLED NUMBER TWENTY ON BLACK SQUARE" },
    { 0x324A, "CIRCLED NUMBER THIRTY ON BLACK SQUARE" },
    { 0x324B, "CIRCLED NUMBER FORTY ON BLACK SQUARE" },
    { 0x324C, "CIRCLED NUMBER FIFTY ON BLACK SQUARE" },
    { 0x324D, "CIRCLED NUMBER SIXTY ON BLACK SQUARE" },
    { 0x324E, "CIRCLED NUMBER SEVENTY ON BLACK SQUARE" },
    { 0x324F, "CIRCLED NUMBER EIGHTY ON BLACK SQUARE" },
    { 0x3250, "PARTNERSHIP SIGN" },
    { 0x3251, "CIRCLED NUMBER TWENTY ONE" },
    { 0x3252, "CIRCLED NUMBER TWENTY TWO" },
    { 0x3253, "CIRCLED NUMBER TWENTY THREE" },
    { 0x3254, "CIRCLED NUMBER TWENTY FOUR" },
    { 0x3255, "CIRCLED NUMBER TWENTY FIVE" },
    { 0x3256, "CIRCLED NUMBER TWENTY SIX" },
    { 0x3257, "CIRCLED NUMBER TWENTY SEVEN" },
    { 0x3258, "CIRCLED NUMBER TWENTY EIGHT" },
    { 0x3259, "CIRCLED NUMBER TWENTY NINE" },
    { 0x325A, "CIRCLED NUMBER THIRTY" },
    { 0x325B, "CIRCLED NUMBER THIRTY ONE" },
    { 0x325C, "CIRCLED NUMBER THIRTY TWO" },
    { 0x325D, "CIRCLED NUMBER THIRTY THREE" },
    { 0x325E, "CIRCLED NUMBER THIRTY FOUR" },
    { 0x325F, "CIRCLED NUMBER THIRTY FIVE" },
    { 0x3260, "CIRCLED HANGUL KIYEOK" },
    { 0x3261, "CIRCLED HANGUL NIEUN" },
    { 0x3262, "CIRCLED HANGUL TIKEUT" },
    { 0x3263, "CIRCLED HANGUL RIEUL" },
    { 0x3264, "CIRCLED HANGUL MIEUM" },
    { 0x3265, "CIRCLED HANGUL PIEUP" },
    { 0x3266, "CIRCLED HANGUL SIOS" },
    { 0x3267, "CIRCLED HANGUL IEUNG" },
    { 0x3268, "CIRCLED HANGUL CIEUC" },
    { 0x3269, "CIRCLED HANGUL CHIEUCH" },
    { 0x326A, "CIRCLED HANGUL KHIEUKH" },
    { 0x326B, "CIRCLED HANGUL THIEUTH" },
    { 0x326C, "CIRCLED HANGUL PHIEUPH" },
    { 0x326D, "CIRCLED HANGUL HIEUH" },
    { 0x326E, "CIRCLED HANGUL KIYEOK A" },
    { 0x326F, "CIRCLED HANGUL NIEUN A" },
    { 0x3270, "CIRCLED HANGUL TIKEUT A" },
    { 0x3271, "CIRCLED HANGUL RIEUL A" },
    { 0x3272, "CIRCLED HANGUL MIEUM A" },
    { 0x3273, "CIRCLED HANGUL PIEUP A" },
    { 0x3274, "CIRCLED HANGUL SIOS A" },
    { 0x3275, "CIRCLED HANGUL IEUNG A" },
    { 0x3276, "CIRCLED HANGUL CIEUC A" },
    { 0x3277, "CIRCLED HANGUL CHIEUCH A" },
    { 0x3278, "CIRCLED HANGUL KHIEUKH A" },
    { 0x3279, "CIRCLED HANGUL THIEUTH A" },
    { 0x327A, "CIRCLED HANGUL PHIEUPH A" },
    { 0x327B, "CIRCLED HANGUL HIEUH A" },
    { 0x327C, "CIRCLED KOREAN CHARACTER CHAMKO" },
    { 0x327D, "CIRCLED KOREAN CHARACTER JUEUI" },
    { 0x327E, "CIRCLED HANGUL IEUNG U" },
    { 0x327F, "KOREAN STANDARD SYMBOL" },
    { 0x3280, "CIRCLED IDEOGRAPH ONE" },
    { 0x3281, "CIRCLED IDEOGRAPH TWO" },
    { 0x3282, "CIRCLED IDEOGRAPH THREE" },
    { 0x3283, "CIRCLED IDEOGRAPH FOUR" },
    { 0x3284, "CIRCLED IDEOGRAPH FIVE" },
    { 0x3285, "CIRCLED IDEOGRAPH SIX" },
    { 0x3286, "CIRCLED IDEOGRAPH SEVEN" },
    { 0x3287, "CIRCLED IDEOGRAPH EIGHT" },
    { 0x3288, "CIRCLED IDEOGRAPH NINE" },
    { 0x3289, "CIRCLED IDEOGRAPH TEN" },
    { 0x328A, "CIRCLED IDEOGRAPH MOON" },
    { 0x328B, "CIRCLED IDEOGRAPH FIRE" },
    { 0x328C, "CIRCLED IDEOGRAPH WATER" },
    { 0x328D, "CIRCLED IDEOGRAPH WOOD" },
    { 0x328E, "CIRCLED IDEOGRAPH METAL" },
    { 0x328F, "CIRCLED IDEOGRAPH EARTH" },
    { 0x3290, "CIRCLED IDEOGRAPH SUN" },
    { 0x3291, "CIRCLED IDEOGRAPH STOCK" },
    { 0x3292, "CIRCLED IDEOGRAPH HAVE" },
    { 0x3293, "CIRCLED IDEOGRAPH SOCIETY" },
    { 0x3294, "CIRCLED IDEOGRAPH NAME" },
    { 0x3295, "CIRCLED IDEOGRAPH SPECIAL" },
    { 0x3296, "CIRCLED IDEOGRAPH FINANCIAL" },
    { 0x3297, "CIRCLED IDEOGRAPH CONGRATULATION" },
    { 0x3298, "CIRCLED IDEOGRAPH LABOR" },
    { 0x3299, "CIRCLED IDEOGRAPH SECRET" },
    { 0x329A, "CIRCLED IDEOGRAPH MALE" },
    { 0x329B, "CIRCLED IDEOGRAPH FEMALE" },
    { 0x329C, "CIRCLED IDEOGRAPH SUITABLE" },
    { 0x329D, "CIRCLED IDEOGRAPH EXCELLENT" },
    { 0x329E, "CIRCLED IDEOGRAPH PRINT" },
    { 0x329F, "CIRCLED IDEOGRAPH ATTENTION" },
    { 0x32A0, "CIRCLED IDEOGRAPH ITEM" },
    { 0x32A1, "CIRCLED IDEOGRAPH REST" },
    { 0x32A2, "CIRCLED IDEOGRAPH COPY" },
    { 0x32A3, "CIRCLED IDEOGRAPH CORRECT" },
    { 0x32A4, "CIRCLED IDEOGRAPH HIGH" },
    { 0x32A5, "CIRCLED IDEOGRAPH CENTRE" },
    { 0x32A6, "CIRCLED IDEOGRAPH LOW" },
    { 0x32A7, "CIRCLED IDEOGRAPH LEFT" },
    { 0x32A8, "CIRCLED IDEOGRAPH RIGHT" },
    { 0x32A9, "CIRCLED IDEOGRAPH MEDICINE" },
    { 0x32AA, "CIRCLED IDEOGRAPH RELIGION" },
    { 0x32AB, "CIRCLED IDEOGRAPH STUDY" },
    { 0x32AC, "CIRCLED IDEOGRAPH SUPERVISE" },
    { 0x32AD, "CIRCLED IDEOGRAPH ENTERPRISE" },
    { 0x32AE, "CIRCLED IDEOGRAPH RESOURCE" },
    { 0x32AF, "CIRCLED IDEOGRAPH ALLIANCE" },
    { 0x32B0, "CIRCLED IDEOGRAPH NIGHT" },
    { 0x32B1, "CIRCLED NUMBER THIRTY SIX" },
    { 0x32B2, "CIRCLED NUMBER THIRTY SEVEN" },
    { 0x32B3, "CIRCLED NUMBER THIRTY EIGHT" },
    { 0x32B4, "CIRCLED NUMBER THIRTY NINE" },
    { 0x32B5, "CIRCLED NUMBER FORTY" },
    { 0x32B6, "CIRCLED NUMBER FORTY ONE" },
    { 0x32B7, "CIRCLED NUMBER FORTY TWO" },
    { 0x32B8, "CIRCLED NUMBER FORTY THREE" },
    { 0x32B9, "CIRCLED NUMBER FORTY FOUR" },
    { 0x32BA, "CIRCLED NUMBER FORTY FIVE" },
    { 0x32BB, "CIRCLED NUMBER FORTY SIX" },
    { 0x32BC, "CIRCLED NUMBER FORTY SEVEN" },
    { 0x32BD, "CIRCLED NUMBER FORTY EIGHT" },
    { 0x32BE, "CIRCLED NUMBER FORTY NINE" },
    { 0x32BF, "CIRCLED NUMBER FIFTY" },
    { 0x32C0, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR JANUARY" },
    { 0x32C1, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR FEBRUARY" },
    { 0x32C2, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR MARCH" },
    { 0x32C3, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR APRIL" },
    { 0x32C4, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR MAY" },
    { 0x32C5, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR JUNE" },
    { 0x32C6, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR JULY" },
    { 0x32C7, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR AUGUST" },
    { 0x32C8, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR SEPTEMBER" },
    { 0x32C9, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR OCTOBER" },
    { 0x32CA, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR NOVEMBER" },
    { 0x32CB, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DECEMBER" },
    { 0x32CC, "SQUARE HG" },
    { 0x32CD, "SQUARE ERG" },
    { 0x32CE, "SQUARE EV" },
    { 0x32CF, "LIMITED LIABILITY SIGN" },
    { 0x32D0, "CIRCLED KATAKANA A" },
    { 0x32D1, "CIRCLED KATAKANA I" },
    { 0x32D2, "CIRCLED KATAKANA U" },
    { 0x32D3, "CIRCLED KATAKANA E" },
    { 0x32D4, "CIRCLED KATAKANA O" },
    { 0x32D5, "CIRCLED KATAKANA KA" },
    { 0x32D6, "CIRCLED KATAKANA KI" },
    { 0x32D7, "CIRCLED KATAKANA KU" },
    { 0x32D8, "CIRCLED KATAKANA KE" },
    { 0x32D9, "CIRCLED KATAKANA KO" },
    { 0x32DA, "CIRCLED KATAKANA SA" },
    { 0x32DB, "CIRCLED KATAKANA SI" },
    { 0x32DC, "CIRCLED KATAKANA SU" },
    { 0x32DD, "CIRCLED KATAKANA SE" },
    { 0x32DE, "CIRCLED KATAKANA SO" },
    { 0x32DF, "CIRCLED KATAKANA TA" },
    { 0x32E0, "CIRCLED KATAKANA TI" },
    { 0x32E1, "CIRCLED KATAKANA TU" },
    { 0x32E2, "CIRCLED KATAKANA TE" },
    { 0x32E3, "CIRCLED KATAKANA TO" },
    { 0x32E4, "CIRCLED KATAKANA NA" },
    { 0x32E5, "CIRCLED KATAKANA NI" },
    { 0x32E6, "CIRCLED KATAKANA NU" },
    { 0x32E7, "CIRCLED KATAKANA NE" },
    { 0x32E8, "CIRCLED KATAKANA NO" },
    { 0x32E9, "CIRCLED KATAKANA HA" },
    { 0x32EA, "CIRCLED KATAKANA HI" },
    { 0x32EB, "CIRCLED KATAKANA HU" },
    { 0x32EC, "CIRCLED KATAKANA HE" },
    { 0x32ED, "CIRCLED KATAKANA HO" },
    { 0x32EE, "CIRCLED KATAKANA MA" },
    { 0x32EF, "CIRCLED KATAKANA MI" },
    { 0x32F0, "CIRCLED KATAKANA MU" },
    { 0x32F1, "CIRCLED KATAKANA ME" },
    { 0x32F2, "CIRCLED KATAKANA MO" },
    { 0x32F3, "CIRCLED KATAKANA YA" },
    { 0x32F4, "CIRCLED KATAKANA YU" },
    { 0x32F5, "CIRCLED KATAKANA YO" },
    { 0x32F6, "CIRCLED KATAKANA RA" },
    { 0x32F7, "CIRCLED KATAKANA RI" },
    { 0x32F8, "CIRCLED KATAKANA RU" },
    { 0x32F9, "CIRCLED KATAKANA RE" },
    { 0x32FA, "CIRCLED KATAKANA RO" },
    { 0x32FB, "CIRCLED KATAKANA WA" },
    { 0x32FC, "CIRCLED KATAKANA WI" },
    { 0x32FD, "CIRCLED KATAKANA WE" },
    { 0x32FE, "CIRCLED KATAKANA WO" },
    { 0x32FF, "SQUARE ERA NAME REIWA" },
    { 0x3300, "SQUARE APAATO" },
    { 0x3301, "SQUARE ARUHUA" },
    { 0x3302, "SQUARE ANPEA" },
    { 0x3303, "SQUARE AARU" },
    { 0x3304, "SQUARE ININGU" },
    { 0x3305, "SQUARE INTI" },
    { 0x3306, "SQUARE UON" },
    { 0x3307, "SQUARE ESUKUUDO" },
    { 0x3308, "SQUARE EEKAA" },
    { 0x3309, "SQUARE ONSU" },
    { 0x330A, "SQUARE OOMU" },
    { 0x330B, "SQUARE KAIRI" },
    { 0x330C, "SQUARE KARATTO" },
    { 0x330D, "SQUARE KARORII" },
    { 0x330E, "SQUARE GARON" },
    { 0x330F, "SQUARE GANMA" },
    { 0x3310, "SQUARE GIGA" },
    { 0x3311, "SQUARE GINII" },
    { 0x3312, "SQUARE KYURII" },
    { 0x3313, "SQUARE GIRUDAA" },
    { 0x3314, "SQUARE KIRO" },
    { 0x3315, "SQUARE KIROGURAMU" },
    { 0x3316, "SQUARE KIROMEETORU" },
    { 0x3317, "SQUARE KIROWATTO" },
    { 0x3318, "SQUARE GURAMU" },
    { 0x3319, "SQUARE GURAMUTON" },
    { 0x331A, "SQUARE KURUZEIRO" },
    { 0x331B, "SQUARE KUROONE" },
    { 0x331C, "SQUARE KEESU" },
    { 0x331D, "SQUARE KORUNA" },
    { 0x331E, "SQUARE KOOPO" },
    { 0x331F, "SQUARE SAIKURU" },
    { 0x3320, "SQUARE SANTIIMU" },
    { 0x3321, "SQUARE SIRINGU" },
    { 0x3322, "SQUARE SENTI" },
    { 0x3323, "SQUARE SENTO" },
    { 0x3324, "SQUARE DAASU" },
    { 0x3325, "SQUARE DESI" },
    { 0x3326, "SQUARE DORU" },
    { 0x3327, "SQUARE TON" },
    { 0x3328, "SQUARE NANO" },
    { 0x3329, "SQUARE NOTTO" },
    { 0x332A, "SQUARE HAITU" },
    { 0x332B, "SQUARE PAASENTO" },
    { 0x332C, "SQUARE PAATU" },
    { 0x332D, "SQUARE BAARERU" },
    { 0x332E, "SQUARE PIASUTORU" },
    { 0x332F, "SQUARE PIKURU" },
    { 0x3330, "SQUARE PIKO" },
    { 0x3331, "SQUARE BIRU" },
    { 0x3332, "SQUARE HUARADDO" },
    { 0x3333, "SQUARE HUIITO" },
    { 0x3334, "SQUARE BUSSYERU" },
    { 0x3335, "SQUARE HURAN" },
    { 0x3336, "SQUARE HEKUTAARU" },
    { 0x3337, "SQUARE PESO" },
    { 0x3338, "SQUARE PENIHI" },
    { 0x3339, "SQUARE HERUTU" },
    { 0x333A, "SQUARE PENSU" },
    { 0x333B, "SQUARE PEEZI" },
    { 0x333C, "SQUARE BEETA" },
    { 0x333D, "SQUARE POINTO" },
    { 0x333E, "SQUARE BORUTO" },
    { 0x333F, "SQUARE HON" },
    { 0x3340, "SQUARE PONDO" },
    { 0x3341, "SQUARE HOORU" },
    { 0x3342, "SQUARE HOON" },
    { 0x3343, "SQUARE MAIKURO" },
    { 0x3344, "SQUARE MAIRU" },
    { 0x3345, "SQUARE MAHHA" },
    { 0x3346, "SQUARE MARUKU" },
    { 0x3347, "SQUARE MANSYON" },
    { 0x3348, "SQUARE MIKURON" },
    { 0x3349, "SQUARE MIRI" },
    { 0x334A, "SQUARE MIRIBAARU" },
    { 0x334B, "SQUARE MEGA" },
    { 0x334C, "SQUARE MEGATON" },
    { 0x334D, "SQUARE MEETORU" },
    { 0x334E, "SQUARE YAADO" },
    { 0x334F, "SQUARE YAARU" },
    { 0x3350, "SQUARE YUAN" },
    { 0x3351, "SQUARE RITTORU" },
    { 0x3352, "SQUARE RIRA" },
    { 0x3353, "SQUARE RUPII" },
    { 0x3354, "SQUARE RUUBURU" },
    { 0x3355, "SQUARE REMU" },
    { 0x3356, "SQUARE RENTOGEN" },
    { 0x3357, "SQUARE WATTO" },
    { 0x3358, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ZERO" },
    { 0x3359, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ONE" },
    { 0x335A, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWO" },
    { 0x335B, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THREE" },
    { 0x335C, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOUR" },
    { 0x335D, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIVE" },
    { 0x335E, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIX" },
    { 0x335F, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVEN" },
    { 0x3360, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHT" },
    { 0x3361, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINE" },
    { 0x3362, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TEN" },
    { 0x3363, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ELEVEN" },
    { 0x3364, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWELVE" },
    { 0x3365, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THIRTEEN" },
    { 0x3366, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOURTEEN" },
    { 0x3367, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIFTEEN" },
    { 0x3368, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIXTEEN" },
    { 0x3369, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVENTEEN" },
    { 0x336A, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHTEEN" },
    { 0x336B, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINETEEN" },
    { 0x336C, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY" },
    { 0x336D, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-ONE" },
    { 0x336E, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-TWO" },
    { 0x336F, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-THREE" },
    { 0x3370, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-FOUR" },
    { 0x3371, "SQUARE HPA" },
    { 0x3372, "SQUARE DA" },
    { 0x3373, "SQUARE AU" },
    { 0x3374, "SQUARE BAR" },
    { 0x3375, "SQUARE OV" },
    { 0x3376, "SQUARE PC" },
    { 0x3377, "SQUARE DM" },
    { 0x3378, "SQUARE DM SQUARED" },
    { 0x3379, "SQUARE DM CUBED" },
    { 0x337A, "SQUARE IU" },
    { 0x337B, "SQUARE ERA NAME HEISEI" },
    { 0x337C, "SQUARE ERA NAME SYOUWA" },
    { 0x337D, "SQUARE ERA NAME TAISYOU" },
    { 0x337E, "SQUARE ERA NAME MEIZI" },
    { 0x337F, "SQUARE CORPORATION" },
    { 0x3380, "SQUARE PA AMPS" },
    { 0x3381, "SQUARE NA" },
    { 0x3382, "SQUARE MU A" },
    { 0x3383, "SQUARE MA" },
    { 0x3384, "SQUARE KA" },
    { 0x3385, "SQUARE KB" },
    { 0x3386, "SQUARE MB" },
    { 0x3387, "SQUARE GB" },
    { 0x3388, "SQUARE CAL" },
    { 0x3389, "SQUARE KCAL" },
    { 0x338A, "SQUARE PF" },
    { 0x338B, "SQUARE NF" },
    { 0x338C, "SQUARE MU F" },
    { 0x338D, "SQUARE MU G" },
    { 0x338E, "SQUARE MG" },
    { 0x338F, "SQUARE KG" },
    { 0x3390, "SQUARE HZ" },
    { 0x3391, "SQUARE KHZ" },
    { 0x3392, "SQUARE MHZ" },
    { 0x3393, "SQUARE GHZ" },
    { 0x3394, "SQUARE THZ" },
    { 0x3395, "SQUARE MU L" },
    { 0x3396, "SQUARE ML" },
    { 0x3397, "SQUARE DL" },
    { 0x3398, "SQUARE KL" },
    { 0x3399, "SQUARE FM" },
    { 0x339A, "SQUARE NM" },
    { 0x339B, "SQUARE MU M" },
    { 0x339C, "SQUARE MM" },
    { 0x339D, "SQUARE CM" },
    { 0x339E, "SQUARE KM" },
    { 0x339F, "SQUARE MM SQUARED" },
    { 0x33A0, "SQUARE CM SQUARED" },
    { 0x33A1, "SQUARE M SQUARED" },
    { 0x33A2, "SQUARE KM SQUARED" },
    { 0x33A3, "SQUARE MM CUBED" },
    { 0x33A4, "SQUARE CM CUBED" },
    { 0x33A5, "SQUARE M CUBED" },
    { 0x33A6, "SQUARE KM CUBED" },
    { 0x33A7, "SQUARE M OVER S" },
    { 0x33A8, "SQUARE M OVER S SQUARED" },
    { 0x33A9, "SQUARE PA" },
    { 0x33AA, "SQUARE KPA" },
    { 0x33AB, "SQUARE MPA" },
    { 0x33AC, "SQUARE GPA" },
    { 0x33AD, "SQUARE RAD" },
    { 0x33AE, "SQUARE RAD OVER S" },
    { 0x33AF, "SQUARE RAD OVER S SQUARED" },
    { 0x33B0, "SQUARE PS" },
    { 0x33B1, "SQUARE NS" },
    { 0x33B2, "SQUARE MU S" },
    { 0x33B3, "SQUARE MS" },
    { 0x33B4, "SQUARE PV" },
    { 0x33B5, "SQUARE NV" },
    { 0x33B6, "SQUARE MU V" },
    { 0x33B7, "SQUARE MV" },
    { 0x33B8, "SQUARE KV" },
    { 0x33B9, "SQUARE MV MEGA" },
    { 0x33BA, "SQUARE PW" },
    { 0x33BB, "SQUARE NW" },
    { 0x33BC, "SQUARE MU W" },
    { 0x33BD, "SQUARE MW" },
    { 0x33BE, "SQUARE KW" },
    { 0x33BF, "SQUARE MW MEGA" },
    { 0x33C0, "SQUARE K OHM" },
    { 0x33C1, "SQUARE M OHM" },
    { 0x33C2, "SQUARE AM" },
    { 0x33C3, "SQUARE BQ" },
    { 0x33C4, "SQUARE CC" },
    { 0x33C5, "SQUARE CD" },
    { 0x33C6, "SQUARE C OVER KG" },
    { 0x33C7, "SQUARE CO" },
    { 0x33C8, "SQUARE DB" },
    { 0x33C9, "SQUARE GY" },
    { 0x33CA, "SQUARE HA" },
    { 0x33CB, "SQUARE HP" },
    { 0x33CC, "SQUARE IN" },
    { 0x33CD, "SQUARE KK" },
    { 0x33CE, "SQUARE KM CAPITAL" },
    { 0x33CF, "SQUARE KT" },
    { 0x33D0, "SQUARE LM" },
    { 0x33D1, "SQUARE LN" },
    { 0x33D2, "SQUARE LOG" },
    { 0x33D3, "SQUARE LX" },
    { 0x33D4, "SQUARE MB SMALL" },
    { 0x33D5, "SQUARE MIL" },
    { 0x33D6, "SQUARE MOL" },
    { 0x33D7, "SQUARE PH" },
    { 0x33D8, "SQUARE PM" },
    { 0x33D9, "SQUARE PPM" },
    { 0x33DA, "SQUARE PR" },
    { 0x33DB, "SQUARE SR" },
    { 0x33DC, "SQUARE SV" },
    { 0x33DD, "SQUARE WB" },
    { 0x33DE, "SQUARE V OVER M" },
    { 0x33DF, "SQUARE A OVER M" },
    { 0x33E0, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ONE" },
    { 0x33E1, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWO" },
    { 0x33E2, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THREE" },
    { 0x33E3, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOUR" },
    { 0x33E4, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIVE" },
    { 0x33E5, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIX" },
    { 0x33E6, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVEN" },
    { 0x33E7, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHT" },
    { 0x33E8, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINE" },
    { 0x33E9, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TEN" },
    { 0x33EA, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ELEVEN" },
    { 0x33EB, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWELVE" },
    { 0x33EC, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTEEN" },
    { 0x33ED, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOURTEEN" },
    { 0x33EE, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIFTEEN" },
    { 0x33EF, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIXTEEN" },
    { 0x33F0, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVENTEEN" },
    { 0x33F1, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHTEEN" },
    { 0x33F2, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINETEEN" },
    { 0x33F3, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY" },
    { 0x33F4, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-ONE" },
    { 0x33F5, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-TWO" },
    { 0x33F6, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-THREE" },
    { 0x33F7, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FOUR" },
    { 0x33F8, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FIVE" },
    { 0x33F9, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SIX" },
    { 0x33FA, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SEVEN" },
    { 0x33FB, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-EIGHT" },
    { 0x33FC, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-NINE" },
    { 0x33FD, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY" },
    { 0x33FE, "IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY-ONE" },
    { 0x33FF, "SQUARE GAL" },
    { 0x4DC0, "HEXAGRAM FOR THE CREATIVE HEAVEN" },
    { 0x4DC1, "HEXAGRAM FOR THE RECEPTIVE EARTH" },
    { 0x4DC2, "HEXAGRAM FOR DIFFICULTY AT THE BEGINNING" },
    { 0x4DC3, "HEXAGRAM FOR YOUTHFUL FOLLY" },
    { 0x4DC4, "HEXAGRAM FOR WAITING" },
    { 0x4DC5, "HEXAGRAM FOR CONFLICT" },
    { 0x4DC6, "HEXAGRAM FOR THE ARMY" },
    { 0x4DC7, "HEXAGRAM FOR HOLDING TOGETHER" },
    { 0x4DC8, "HEXAGRAM FOR SMALL TAMING" },
    { 0x4DC9, "HEXAGRAM FOR TREADING" },
    { 0x4DCA, "HEXAGRAM FOR PEACE" },
    { 0x4DCB, "HEXAGRAM FOR STANDSTILL" },
    { 0x4DCC, "HEXAGRAM FOR FELLOWSHIP" },
    { 0x4DCD, "HEXAGRAM FOR GREAT POSSESSION" },
    { 0x4DCE, "HEXAGRAM FOR MODESTY" },
    { 0x4DCF, "HEXAGRAM FOR ENTHUSIASM" },
    { 0x4DD0, "HEXAGRAM FOR FOLLOWING" },
    { 0x4DD1, "HEXAGRAM FOR WORK ON THE DECAYED" },
    { 0x4DD2, "HEXAGRAM FOR APPROACH" },
    { 0x4DD3, "HEXAGRAM FOR CONTEMPLATION" },
    { 0x4DD4, "HEXAGRAM FOR BITING THROUGH" },
    { 0x4DD5, "HEXAGRAM FOR GRACE" },
    { 0x4DD6, "HEXAGRAM FOR SPLITTING APART" },
    { 0x4DD7, "HEXAGRAM FOR RETURN" },
    { 0x4DD8, "HEXAGRAM FOR INNOCENCE" },
    { 0x4DD9, "HEXAGRAM FOR GREAT TAMING" },
    { 0x4DDA, "HEXAGRAM FOR MOUTH CORNERS" },
    { 0x4DDB, "HEXAGRAM FOR GREAT PREPONDERANCE" },
    { 0x4DDC, "HEXAGRAM FOR THE ABYSMAL WATER" },
    { 0x4DDD, "HEXAGRAM FOR THE CLINGING FIRE" },
    { 0x4DDE, "HEXAGRAM FOR INFLUENCE" },
    { 0x4DDF, "HEXAGRAM FOR DURATION" },
    { 0x4DE0, "HEXAGRAM FOR RETREAT" },
    { 0x4DE1, "HEXAGRAM FOR GREAT POWER" },
    { 0x4DE2, "HEXAGRAM FOR PROGRESS" },
    { 0x4DE3, "HEXAGRAM FOR DARKENING OF THE LIGHT" },
    { 0x4DE4, "HEXAGRAM FOR THE FAMILY" },
    { 0x4DE5, "HEXAGRAM FOR OPPOSITION" },
    { 0x4DE6, "HEXAGRAM FOR OBSTRUCTION" },
    { 0x4DE7, "HEXAGRAM FOR DELIVERANCE" },
    { 0x4DE8, "HEXAGRAM FOR DECREASE" },
    { 0x4DE9, "HEXAGRAM FOR INCREASE" },
    { 0x4DEA, "HEXAGRAM FOR BREAKTHROUGH" },
    { 0x4DEB, "HEXAGRAM FOR COMING TO MEET" },
    { 0x4DEC, "HEXAGRAM FOR GATHERING TOGETHER" },
    { 0x4DED, "HEXAGRAM FOR PUSHING UPWARD" },
    { 0x4DEE, "HEXAGRAM FOR OPPRESSION" },
    { 0x4DEF, "HEXAGRAM FOR THE WELL" },
    { 0x4DF0, "HEXAGRAM FOR REVOLUTION" },
    { 0x4DF1, "HEXAGRAM FOR THE CAULDRON" },
    { 0x4DF2, "HEXAGRAM FOR THE AROUSING THUNDER" },
    { 0x4DF3, "HEXAGRAM FOR THE KEEPING STILL MOUNTAIN" },
    { 0x4DF4, "HEXAGRAM FOR DEVELOPMENT" },
    { 0x4DF5, "HEXAGRAM FOR THE MARRYING MAIDEN" },
    { 0x4DF6, "HEXAGRAM FOR ABUNDANCE" },
    { 0x4DF7, "HEXAGRAM FOR THE WANDERER" },
    { 0x4DF8, "HEXAGRAM FOR THE GENTLE WIND" },
    { 0x4DF9, "HEXAGRAM FOR THE JOYOUS LAKE" },
    { 0x4DFA, "HEXAGRAM FOR DISPERSION" },
    { 0x4DFB, "HEXAGRAM FOR LIMITATION" },
    { 0x4DFC, "HEXAGRAM FOR INNER TRUTH" },
    { 0x4DFD, "HEXAGRAM FOR SMALL PREPONDERANCE" },
    { 0x4DFE, "HEXAGRAM FOR AFTER COMPLETION" },
    { 0x4DFF, "HEXAGRAM FOR BEFORE COMPLETION" },
    { 0xA000, "YI SYLLABLE IT" },
    { 0xA001, "YI SYLLABLE IX" },
    { 0xA002, "YI SYLLABLE I" },
    { 0xA003, "YI SYLLABLE IP" },
    { 0xA004, "YI SYLLABLE IET" },
    { 0xA005, "YI SYLLABLE IEX" },
    { 0xA006, "YI SYLLABLE IE" },
    { 0xA007, "YI SYLLABLE IEP" },
    { 0xA008, "YI SYLLABLE AT" },
    { 0xA009, "YI SYLLABLE AX" },
    { 0xA00A, "YI SYLLABLE A" },
    { 0xA00B, "YI SYLLABLE AP" },
    { 0xA00C, "YI SYLLABLE UOX" },
    { 0xA00D, "YI SYLLABLE UO" },
    { 0xA00E, "YI SYLLABLE UOP" },
    { 0xA00F, "YI SYLLABLE OT" },
    { 0xA010, "YI SYLLABLE OX" },
    { 0xA011, "YI SYLLABLE O" },
    { 0xA012, "YI SYLLABLE OP" },
    { 0xA013, "YI SYLLABLE EX" },
    { 0xA014, "YI SYLLABLE E" },
    { 0xA015, "YI SYLLABLE WU" },
    { 0xA016, "YI SYLLABLE BIT" },
    { 0xA017, "YI SYLLABLE BIX" },
    { 0xA018, "YI SYLLABLE BI" },
    { 0xA019, "YI SYLLABLE BIP" },
    { 0xA01A, "YI SYLLABLE BIET" },
    { 0xA01B, "YI SYLLABLE BIEX" },
    { 0xA01C, "YI SYLLABLE BIE" },
    { 0xA01D, "YI SYLLABLE BIEP" },
    { 0xA01E, "YI SYLLABLE BAT" },
    { 0xA01F, "YI SYLLABLE BAX" },
    { 0xA020, "YI SYLLABLE BA" },
    { 0xA021, "YI SYLLABLE BAP" },
    { 0xA022, "YI SYLLABLE BUOX" },
    { 0xA023, "YI SYLLABLE BUO" },
    { 0xA024, "YI SYLLABLE BUOP" },
    { 0xA025, "YI SYLLABLE BOT" },
    { 0xA026, "YI SYLLABLE BOX" },
    { 0xA027, "YI SYLLABLE BO" },
    { 0xA028, "YI SYLLABLE BOP" },
    { 0xA029, "YI SYLLABLE BEX" },
    { 0xA02A, "YI SYLLABLE BE" },
    { 0xA02B, "YI SYLLABLE BEP" },
    { 0xA02C, "YI SYLLABLE BUT" },
    { 0xA02D, "YI SYLLABLE BUX" },
    { 0xA02E, "YI SYLLABLE BU" },
    { 0xA02F, "YI SYLLABLE BUP" },
    { 0xA030, "YI SYLLABLE BURX" },
    { 0xA031, "YI SYLLABLE BUR" },
    { 0xA032, "YI SYLLABLE BYT" },
    { 0xA033, "YI SYLLABLE BYX" },
    { 0xA034, "YI SYLLABLE BY" },
    { 0xA035, "YI SYLLABLE BYP" },
    { 0xA036, "YI SYLLABLE BYRX" },
    { 0xA037, "YI SYLLABLE BYR" },
    { 0xA038, "YI SYLLABLE PIT" },
    { 0xA039, "YI SYLLABLE PIX" },
    { 0xA03A, "YI SYLLABLE PI" },
    { 0xA03B, "YI SYLLABLE PIP" },
    { 0xA03C, "YI SYLLABLE PIEX" },
    { 0xA03D, "YI SYLLABLE PIE" },
    { 0xA03E, "YI SYLLABLE PIEP" },
    { 0xA03F, "YI SYLLABLE PAT" },
    { 0xA040, "YI SYLLABLE PAX" },
    { 0xA041, "YI SYLLABLE PA" },
    { 0xA042, "YI SYLLABLE PAP" },
    { 0xA043, "YI SYLLABLE PUOX" },
    { 0xA044, "YI SYLLABLE PUO" },
    { 0xA045, "YI SYLLABLE PUOP" },
    { 0xA046, "YI SYLLABLE POT" },
    { 0xA047, "YI SYLLABLE POX" },
    { 0xA048, "YI SYLLABLE PO" },
    { 0xA049, "YI SYLLABLE POP" },
    { 0xA04A, "YI SYLLABLE PUT" },
    { 0xA04B, "YI SYLLABLE PUX" },
    { 0xA04C, "YI SYLLABLE PU" },
    { 0xA04D, "YI SYLLABLE PUP" },
    { 0xA04E, "YI SYLLABLE PURX" },
    { 0xA04F, "YI SYLLABLE PUR" },
    { 0xA050, "YI SYLLABLE PYT" },
    { 0xA051, "YI SYLLABLE PYX" },
    { 0xA052, "YI SYLLABLE PY" },
    { 0xA053, "YI SYLLABLE PYP" },
    { 0xA054, "YI SYLLABLE PYRX" },
    { 0xA055, "YI SYLLABLE PYR" },
    { 0xA056, "YI SYLLABLE BBIT" },
    { 0xA057, "YI SYLLABLE BBIX" },
    { 0xA058, "YI SYLLABLE BBI" },
    { 0xA059, "YI SYLLABLE BBIP" },
    { 0xA05A, "YI SYLLABLE BBIET" },
    { 0xA05B, "YI SYLLABLE BBIEX" },
    { 0xA05C, "YI SYLLABLE BBIE" },
    { 0xA05D, "YI SYLLABLE BBIEP" },
    { 0xA05E, "YI SYLLABLE BBAT" },
    { 0xA05F, "YI SYLLABLE BBAX" },
    { 0xA060, "YI SYLLABLE BBA" },
    { 0xA061, "YI SYLLABLE BBAP" },
    { 0xA062, "YI SYLLABLE BBUOX" },
    { 0xA063, "YI SYLLABLE BBUO" },
    { 0xA064, "YI SYLLABLE BBUOP" },
    { 0xA065, "YI SYLLABLE BBOT" },
    { 0xA066, "YI SYLLABLE BBOX" },
    { 0xA067, "YI SYLLABLE BBO" },
    { 0xA068, "YI SYLLABLE BBOP" },
    { 0xA069, "YI SYLLABLE BBEX" },
    { 0xA06A, "YI SYLLABLE BBE" },
    { 0xA06B, "YI SYLLABLE BBEP" },
    { 0xA06C, "YI SYLLABLE BBUT" },
    { 0xA06D, "YI SYLLABLE BBUX" },
    { 0xA06E, "YI SYLLABLE BBU" },
    { 0xA06F, "YI SYLLABLE BBUP" },
    { 0xA070, "YI SYLLABLE BBURX" },
    { 0xA071, "YI SYLLABLE BBUR" },
    { 0xA072, "YI SYLLABLE BBYT" },
    { 0xA073, "YI SYLLABLE BBYX" },
    { 0xA074, "YI SYLLABLE BBY" },
    { 0xA075, "YI SYLLABLE BBYP" },
    { 0xA076, "YI SYLLABLE NBIT" },
    { 0xA077, "YI SYLLABLE NBIX" },
    { 0xA078, "YI SYLLABLE NBI" },
    { 0xA079, "YI SYLLABLE NBIP" },
    { 0xA07A, "YI SYLLABLE NBIEX" },
    { 0xA07B, "YI SYLLABLE NBIE" },
    { 0xA07C, "YI SYLLABLE NBIEP" },
    { 0xA07D, "YI SYLLABLE NBAT" },
    { 0xA07E, "YI SYLLABLE NBAX" },
    { 0xA07F, "YI SYLLABLE NBA" },
    { 0xA080, "YI SYLLABLE NBAP" },
    { 0xA081, "YI SYLLABLE NBOT" },
    { 0xA082, "YI SYLLABLE NBOX" },
    { 0xA083, "YI SYLLABLE NBO" },
    { 0xA084, "YI SYLLABLE NBOP" },
    { 0xA085, "YI SYLLABLE NBUT" },
    { 0xA086, "YI SYLLABLE NBUX" },
    { 0xA087, "YI SYLLABLE NBU" },
    { 0xA088, "YI SYLLABLE NBUP" },
    { 0xA089, "YI SYLLABLE NBURX" },
    { 0xA08A, "YI SYLLABLE NBUR" },
    { 0xA08B, "YI SYLLABLE NBYT" },
    { 0xA08C, "YI SYLLABLE NBYX" },
    { 0xA08D, "YI SYLLABLE NBY" },
    { 0xA08E, "YI SYLLABLE NBYP" },
    { 0xA08F, "YI SYLLABLE NBYRX" },
    { 0xA090, "YI SYLLABLE NBYR" },
    { 0xA091, "YI SYLLABLE HMIT" },
    { 0xA092, "YI SYLLABLE HMIX" },
    { 0xA093, "YI SYLLABLE HMI" },
    { 0xA094, "YI SYLLABLE HMIP" },
    { 0xA095, "YI SYLLABLE HMIEX" },
    { 0xA096, "YI SYLLABLE HMIE" },
    { 0xA097, "YI SYLLABLE HMIEP" },
    { 0xA098, "YI SYLLABLE HMAT" },
    { 0xA099, "YI SYLLABLE HMAX" },
    { 0xA09A, "YI SYLLABLE HMA" },
    { 0xA09B, "YI SYLLABLE HMAP" },
    { 0xA09C, "YI SYLLABLE HMUOX" },
    { 0xA09D, "YI SYLLABLE HMUO" },
    { 0xA09E, "YI SYLLABLE HMUOP" },
    { 0xA09F, "YI SYLLABLE HMOT" },
    { 0xA0A0, "YI SYLLABLE HMOX" },
    { 0xA0A1, "YI SYLLABLE HMO" },
    { 0xA0A2, "YI SYLLABLE HMOP" },
    { 0xA0A3, "YI SYLLABLE HMUT" },
    { 0xA0A4, "YI SYLLABLE HMUX" },
    { 0xA0A5, "YI SYLLABLE HMU" },
    { 0xA0A6, "YI SYLLABLE HMUP" },
    { 0xA0A7, "YI SYLLABLE HMURX" },
    { 0xA0A8, "YI SYLLABLE HMUR" },
    { 0xA0A9, "YI SYLLABLE HMYX" },
    { 0xA0AA, "YI SYLLABLE HMY" },
    { 0xA0AB, "YI SYLLABLE HMYP" },
    { 0xA0AC, "YI SYLLABLE HMYRX" },
    { 0xA0AD, "YI SYLLABLE HMYR" },
    { 0xA0AE, "YI SYLLABLE MIT" },
    { 0xA0AF, "YI SYLLABLE MIX" },
    { 0xA0B0, "YI SYLLABLE MI" },
    { 0xA0B1, "YI SYLLABLE MIP" },
    { 0xA0B2, "YI SYLLABLE MIEX" },
    { 0xA0B3, "YI SYLLABLE MIE" },
    { 0xA0B4, "YI SYLLABLE MIEP" },
    { 0xA0B5, "YI SYLLABLE MAT" },
    { 0xA0B6, "YI SYLLABLE MAX" },
    { 0xA0B7, "YI SYLLABLE MA" },
    { 0xA0B8, "YI SYLLABLE MAP" },
    { 0xA0B9, "YI SYLLABLE MUOT" },
    { 0xA0BA, "YI SYLLABLE MUOX" },
    { 0xA0BB, "YI SYLLABLE MUO" },
    { 0xA0BC, "YI SYLLABLE MUOP" },
    { 0xA0BD, "YI SYLLABLE MOT" },
    { 0xA0BE, "YI SYLLABLE MOX" },
    { 0xA0BF, "YI SYLLABLE MO" },
    { 0xA0C0, "YI SYLLABLE MOP" },
    { 0xA0C1, "YI SYLLABLE MEX" },
    { 0xA0C2, "YI SYLLABLE ME" },
    { 0xA0C3, "YI SYLLABLE MUT" },
    { 0xA0C4, "YI SYLLABLE MUX" },
    { 0xA0C5, "YI SYLLABLE MU" },
    { 0xA0C6, "YI SYLLABLE MUP" },
    { 0xA0C7, "YI SYLLABLE MURX" },
    { 0xA0C8, "YI SYLLABLE MUR" },
    { 0xA0C9, "YI SYLLABLE MYT" },
    { 0xA0CA, "YI SYLLABLE MYX" },
    { 0xA0CB, "YI SYLLABLE MY" },
    { 0xA0CC, "YI SYLLABLE MYP" },
    { 0xA0CD, "YI SYLLABLE FIT" },
    { 0xA0CE, "YI SYLLABLE FIX" },
    { 0xA0CF, "YI SYLLABLE FI" },
    { 0xA0D0, "YI SYLLABLE FIP" },
    { 0xA0D1, "YI SYLLABLE FAT" },
    { 0xA0D2, "YI SYLLABLE FAX" },
    { 0xA0D3, "YI SYLLABLE FA" },
    { 0xA0D4, "YI SYLLABLE FAP" },
    { 0xA0D5, "YI SYLLABLE FOX" },
    { 0xA0D6, "YI SYLLABLE FO" },
    { 0xA0D7, "YI SYLLABLE FOP" },
    { 0xA0D8, "YI SYLLABLE FUT" },
    { 0xA0D9, "YI SYLLABLE FUX" },
    { 0xA0DA, "YI SYLLABLE FU" },
    { 0xA0DB, "YI SYLLABLE FUP" },
    { 0xA0DC, "YI SYLLABLE FURX" },
    { 0xA0DD, "YI SYLLABLE FUR" },
    { 0xA0DE, "YI SYLLABLE FYT" },
    { 0xA0DF, "YI SYLLABLE FYX" },
    { 0xA0E0, "YI SYLLABLE FY" },
    { 0xA0E1, "YI SYLLABLE FYP" },
    { 0xA0E2, "YI SYLLABLE VIT" },
    { 0xA0E3, "YI SYLLABLE VIX" },
    { 0xA0E4, "YI SYLLABLE VI" },
    { 0xA0E5, "YI SYLLABLE VIP" },
    { 0xA0E6, "YI SYLLABLE VIET" },
    { 0xA0E7, "YI SYLLABLE VIEX" },
    { 0xA0E8, "YI SYLLABLE VIE" },
    { 0xA0E9, "YI SYLLABLE VIEP" },
    { 0xA0EA, "YI SYLLABLE VAT" },
    { 0xA0EB, "YI SYLLABLE VAX" },
    { 0xA0EC, "YI SYLLABLE VA" },
    { 0xA0ED, "YI SYLLABLE VAP" },
    { 0xA0EE, "YI SYLLABLE VOT" },
    { 0xA0EF, "YI SYLLABLE VOX" },
    { 0xA0F0, "YI SYLLABLE VO" },
    { 0xA0F1, "YI SYLLABLE VOP" },
    { 0xA0F2, "YI SYLLABLE VEX" },
    { 0xA0F3, "YI SYLLABLE VEP" },
    { 0xA0F4, "YI SYLLABLE VUT" },
    { 0xA0F5, "YI SYLLABLE VUX" },
    { 0xA0F6, "YI SYLLABLE VU" },
    { 0xA0F7, "YI SYLLABLE VUP" },
    { 0xA0F8, "YI SYLLABLE VURX" },
    { 0xA0F9, "YI SYLLABLE VUR" },
    { 0xA0FA, "YI SYLLABLE VYT" },
    { 0xA0FB, "YI SYLLABLE VYX" },
    { 0xA0FC, "YI SYLLABLE VY" },
    { 0xA0FD, "YI SYLLABLE VYP" },
    { 0xA0FE, "YI SYLLABLE VYRX" },
    { 0xA0FF, "YI SYLLABLE VYR" },
    { 0xA100, "YI SYLLABLE DIT" },
    { 0xA101, "YI SYLLABLE DIX" },
    { 0xA102, "YI SYLLABLE DI" },
    { 0xA103, "YI SYLLABLE DIP" },
    { 0xA104, "YI SYLLABLE DIEX" },
    { 0xA105, "YI SYLLABLE DIE" },
    { 0xA106, "YI SYLLABLE DIEP" },
    { 0xA107, "YI SYLLABLE DAT" },
    { 0xA108, "YI SYLLABLE DAX" },
    { 0xA109, "YI SYLLABLE DA" },
    { 0xA10A, "YI SYLLABLE DAP" },
    { 0xA10B, "YI SYLLABLE DUOX" },
    { 0xA10C, "YI SYLLABLE DUO" },
    { 0xA10D, "YI SYLLABLE DOT" },
    { 0xA10E, "YI SYLLABLE DOX" },
    { 0xA10F, "YI SYLLABLE DO" },
    { 0xA110, "YI SYLLABLE DOP" },
    { 0xA111, "YI SYLLABLE DEX" },
    { 0xA112, "YI SYLLABLE DE" },
    { 0xA113, "YI SYLLABLE DEP" },
    { 0xA114, "YI SYLLABLE DUT" },
    { 0xA115, "YI SYLLABLE DUX" },
    { 0xA116, "YI SYLLABLE DU" },
    { 0xA117, "YI SYLLABLE DUP" },
    { 0xA118, "YI SYLLABLE DURX" },
    { 0xA119, "YI SYLLABLE DUR" },
    { 0xA11A, "YI SYLLABLE TIT" },
    { 0xA11B, "YI SYLLABLE TIX" },
    { 0xA11C, "YI SYLLABLE TI" },
    { 0xA11D, "YI SYLLABLE TIP" },
    { 0xA11E, "YI SYLLABLE TIEX" },
    { 0xA11F, "YI SYLLABLE TIE" },
    { 0xA120, "YI SYLLABLE TIEP" },
    { 0xA121, "YI SYLLABLE TAT" },
    { 0xA122, "YI SYLLABLE TAX" },
    { 0xA123, "YI SYLLABLE TA" },
    { 0xA124, "YI SYLLABLE TAP" },
    { 0xA125, "YI SYLLABLE TUOT" },
    { 0xA126, "YI SYLLABLE TUOX" },
    { 0xA127, "YI SYLLABLE TUO" },
    { 0xA128, "YI SYLLABLE TUOP" },
    { 0xA129, "YI SYLLABLE TOT" },
    { 0xA12A, "YI SYLLABLE TOX" },
    { 0xA12B, "YI SYLLABLE TO" },
    { 0xA12C, "YI SYLLABLE TOP" },
    { 0xA12D, "YI SYLLABLE TEX" },
    { 0xA12E, "YI SYLLABLE TE" },
    { 0xA12F, "YI SYLLABLE TEP" },
    { 0xA130, "YI SYLLABLE TUT" },
    { 0xA131, "YI SYLLABLE TUX" },
    { 0xA132, "YI SYLLABLE TU" },
    { 0xA133, "YI SYLLABLE TUP" },
    { 0xA134, "YI SYLLABLE TURX" },
    { 0xA135, "YI SYLLABLE TUR" },
    { 0xA136, "YI SYLLABLE DDIT" },
    { 0xA137, "YI SYLLABLE DDIX" },
    { 0xA138, "YI SYLLABLE DDI" },
    { 0xA139, "YI SYLLABLE DDIP" },
    { 0xA13A, "YI SYLLABLE DDIEX" },
    { 0xA13B, "YI SYLLABLE DDIE" },
    { 0xA13C, "YI SYLLABLE DDIEP" },
    { 0xA13D, "YI SYLLABLE DDAT" },
    { 0xA13E, "YI SYLLABLE DDAX" },
    { 0xA13F, "YI SYLLABLE DDA" },
    { 0xA140, "YI SYLLABLE DDAP" },
    { 0xA141, "YI SYLLABLE DDUOX" },
    { 0xA142, "YI SYLLABLE DDUO" },
    { 0xA143, "YI SYLLABLE DDUOP" },
    { 0xA144, "YI SYLLABLE DDOT" },
    { 0xA145, "YI SYLLABLE DDOX" },
    { 0xA146, "YI SYLLABLE DDO" },
    { 0xA147, "YI SYLLABLE DDOP" },
    { 0xA148, "YI SYLLABLE DDEX" },
    { 0xA149, "YI SYLLABLE DDE" },
    { 0xA14A, "YI SYLLABLE DDEP" },
    { 0xA14B, "YI SYLLABLE DDUT" },
    { 0xA14C, "YI SYLLABLE DDUX" },
    { 0xA14D, "YI SYLLABLE DDU" },
    { 0xA14E, "YI SYLLABLE DDUP" },
    { 0xA14F, "YI SYLLABLE DDURX" },
    { 0xA150, "YI SYLLABLE DDUR" },
    { 0xA151, "YI SYLLABLE NDIT" },
    { 0xA152, "YI SYLLABLE NDIX" },
    { 0xA153, "YI SYLLABLE NDI" },
    { 0xA154, "YI SYLLABLE NDIP" },
    { 0xA155, "YI SYLLABLE NDIEX" },
    { 0xA156, "YI SYLLABLE NDIE" },
    { 0xA157, "YI SYLLABLE NDAT" },
    { 0xA158, "YI SYLLABLE NDAX" },
    { 0xA159, "YI SYLLABLE NDA" },
    { 0xA15A, "YI SYLLABLE NDAP" },
    { 0xA15B, "YI SYLLABLE NDOT" },
    { 0xA15C, "YI SYLLABLE NDOX" },
    { 0xA15D, "YI SYLLABLE NDO" },
    { 0xA15E, "YI SYLLABLE NDOP" },
    { 0xA15F, "YI SYLLABLE NDEX" },
    { 0xA160, "YI SYLLABLE NDE" },
    { 0xA161, "YI SYLLABLE NDEP" },
    { 0xA162, "YI SYLLABLE NDUT" },
    { 0xA163, "YI SYLLABLE NDUX" },
    { 0xA164, "YI SYLLABLE NDU" },
    { 0xA165, "YI SYLLABLE NDUP" },
    { 0xA166, "YI SYLLABLE NDURX" },
    { 0xA167, "YI SYLLABLE NDUR" },
    { 0xA168, "YI SYLLABLE HNIT" },
    { 0xA169, "YI SYLLABLE HNIX" },
    { 0xA16A, "YI SYLLABLE HNI" },
    { 0xA16B, "YI SYLLABLE HNIP" },
    { 0xA16C, "YI SYLLABLE HNIET" },
    { 0xA16D, "YI SYLLABLE HNIEX" },
    { 0xA16E, "YI SYLLABLE HNIE" },
    { 0xA16F, "YI SYLLABLE HNIEP" },
    { 0xA170, "YI SYLLABLE HNAT" },
    { 0xA171, "YI SYLLABLE HNAX" },
    { 0xA172, "YI SYLLABLE HNA" },
    { 0xA173, "YI SYLLABLE HNAP" },
    { 0xA174, "YI SYLLABLE HNUOX" },
    { 0xA175, "YI SYLLABLE HNUO" },
    { 0xA176, "YI SYLLABLE HNOT" },
    { 0xA177, "YI SYLLABLE HNOX" },
    { 0xA178, "YI SYLLABLE HNOP" },
    { 0xA179, "YI SYLLABLE HNEX" },
    { 0xA17A, "YI SYLLABLE HNE" },
    { 0xA17B, "YI SYLLABLE HNEP" },
    { 0xA17C, "YI SYLLABLE HNUT" },
    { 0xA17D, "YI SYLLABLE NIT" },
    { 0xA17E, "YI SYLLABLE NIX" },
    { 0xA17F, "YI SYLLABLE NI" },
    { 0xA180, "YI SYLLABLE NIP" },
    { 0xA181, "YI SYLLABLE NIEX" },
    { 0xA182, "YI SYLLABLE NIE" },
    { 0xA183, "YI SYLLABLE NIEP" },
    { 0xA184, "YI SYLLABLE NAX" },
    { 0xA185, "YI SYLLABLE NA" },
    { 0xA186, "YI SYLLABLE NAP" },
    { 0xA187, "YI SYLLABLE NUOX" },
    { 0xA188, "YI SYLLABLE NUO" },
    { 0xA189, "YI SYLLABLE NUOP" },
    { 0xA18A, "YI SYLLABLE NOT" },
    { 0xA18B, "YI SYLLABLE NOX" },
    { 0xA18C, "YI SYLLABLE NO" },
    { 0xA18D, "YI SYLLABLE NOP" },
    { 0xA18E, "YI SYLLABLE NEX" },
    { 0xA18F, "YI SYLLABLE NE" },
    { 0xA190, "YI SYLLABLE NEP" },
    { 0xA191, "YI SYLLABLE NUT" },
    { 0xA192, "YI SYLLABLE NUX" },
    { 0xA193, "YI SYLLABLE NU" },
    { 0xA194, "YI SYLLABLE NUP" },
    { 0xA195, "YI SYLLABLE NURX" },
    { 0xA196, "YI SYLLABLE NUR" },
    { 0xA197, "YI SYLLABLE HLIT" },
    { 0xA198, "YI SYLLABLE HLIX" },
    { 0xA199, "YI SYLLABLE HLI" },
    { 0xA19A, "YI SYLLABLE HLIP" },
    { 0xA19B, "YI SYLLABLE HLIEX" },
    { 0xA19C, "YI SYLLABLE HLIE" },
    { 0xA19D, "YI SYLLABLE HLIEP" },
    { 0xA19E, "YI SYLLABLE HLAT" },
    { 0xA19F, "YI SYLLABLE HLAX" },
    { 0xA1A0, "YI SYLLABLE HLA" },
    { 0xA1A1, "YI SYLLABLE HLAP" },
    { 0xA1A2, "YI SYLLABLE HLUOX" },
    { 0xA1A3, "YI SYLLABLE HLUO" },
    { 0xA1A4, "YI SYLLABLE HLUOP" },
    { 0xA1A5, "YI SYLLABLE HLOX" },
    { 0xA1A6, "YI SYLLABLE HLO" },
    { 0xA1A7, "YI SYLLABLE HLOP" },
    { 0xA1A8, "YI SYLLABLE HLEX" },
    { 0xA1A9, "YI SYLLABLE HLE" },
    { 0xA1AA, "YI SYLLABLE HLEP" },
    { 0xA1AB, "YI SYLLABLE HLUT" },
    { 0xA1AC, "YI SYLLABLE HLUX" },
    { 0xA1AD, "YI SYLLABLE HLU" },
    { 0xA1AE, "YI SYLLABLE HLUP" },
    { 0xA1AF, "YI SYLLABLE HLURX" },
    { 0xA1B0, "YI SYLLABLE HLUR" },
    { 0xA1B1, "YI SYLLABLE HLYT" },
    { 0xA1B2, "YI SYLLABLE HLYX" },
    { 0xA1B3, "YI SYLLABLE HLY" },
    { 0xA1B4, "YI SYLLABLE HLYP" },
    { 0xA1B5, "YI SYLLABLE HLYRX" },
    { 0xA1B6, "YI SYLLABLE HLYR" },
    { 0xA1B7, "YI SYLLABLE LIT" },
    { 0xA1B8, "YI SYLLABLE LIX" },
    { 0xA1B9, "YI SYLLABLE LI" },
    { 0xA1BA, "YI SYLLABLE LIP" },
    { 0xA1BB, "YI SYLLABLE LIET" },
    { 0xA1BC, "YI SYLLABLE LIEX" },
    { 0xA1BD, "YI SYLLABLE LIE" },
    { 0xA1BE, "YI SYLLABLE LIEP" },
    { 0xA1BF, "YI SYLLABLE LAT" },
    { 0xA1C0, "YI SYLLABLE LAX" },
    { 0xA1C1, "YI SYLLABLE LA" },
    { 0xA1C2, "YI SYLLABLE LAP" },
    { 0xA1C3, "YI SYLLABLE LUOT" },
    { 0xA1C4, "YI SYLLABLE LUOX" },
    { 0xA1C5, "YI SYLLABLE LUO" },
    { 0xA1C6, "YI SYLLABLE LUOP" },
    { 0xA1C7, "YI SYLLABLE LOT" },
    { 0xA1C8, "YI SYLLABLE LOX" },
    { 0xA1C9, "YI SYLLABLE LO" },
    { 0xA1CA, "YI SYLLABLE LOP" },
    { 0xA1CB, "YI SYLLABLE LEX" },
    { 0xA1CC, "YI SYLLABLE LE" },
    { 0xA1CD, "YI SYLLABLE LEP" },
    { 0xA1CE, "YI SYLLABLE LUT" },
    { 0xA1CF, "YI SYLLABLE LUX" },
    { 0xA1D0, "YI SYLLABLE LU" },
    { 0xA1D1, "YI SYLLABLE LUP" },
    { 0xA1D2, "YI SYLLABLE LURX" },
    { 0xA1D3, "YI SYLLABLE LUR" },
    { 0xA1D4, "YI SYLLABLE LYT" },
    { 0xA1D5, "YI SYLLABLE LYX" },
    { 0xA1D6, "YI SYLLABLE LY" },
    { 0xA1D7, "YI SYLLABLE LYP" },
    { 0xA1D8, "YI SYLLABLE LYRX" },
    { 0xA1D9, "YI SYLLABLE LYR" },
    { 0xA1DA, "YI SYLLABLE GIT" },
    { 0xA1DB, "YI SYLLABLE GIX" },
    { 0xA1DC, "YI SYLLABLE GI" },
    { 0xA1DD, "YI SYLLABLE GIP" },
    { 0xA1DE, "YI SYLLABLE GIET" },
    { 0xA1DF, "YI SYLLABLE GIEX" },
    { 0xA1E0, "YI SYLLABLE GIE" },
    { 0xA1E1, "YI SYLLABLE GIEP" },
    { 0xA1E2, "YI SYLLABLE GAT" },
    { 0xA1E3, "YI SYLLABLE GAX" },
    { 0xA1E4, "YI SYLLABLE GA" },
    { 0xA1E5, "YI SYLLABLE GAP" },
    { 0xA1E6, "YI SYLLABLE GUOT" },
    { 0xA1E7, "YI SYLLABLE GUOX" },
    { 0xA1E8, "YI SYLLABLE GUO" },
    { 0xA1E9, "YI SYLLABLE GUOP" },
    { 0xA1EA, "YI SYLLABLE GOT" },
    { 0xA1EB, "YI SYLLABLE GOX" },
    { 0xA1EC, "YI SYLLABLE GO" },
    { 0xA1ED, "YI SYLLABLE GOP" },
    { 0xA1EE, "YI SYLLABLE GET" },
    { 0xA1EF, "YI SYLLABLE GEX" },
    { 0xA1F0, "YI SYLLABLE GE" },
    { 0xA1F1, "YI SYLLABLE GEP" },
    { 0xA1F2, "YI SYLLABLE GUT" },
    { 0xA1F3, "YI SYLLABLE GUX" },
    { 0xA1F4, "YI SYLLABLE GU" },
    { 0xA1F5, "YI SYLLABLE GUP" },
    { 0xA1F6, "YI SYLLABLE GURX" },
    { 0xA1F7, "YI SYLLABLE GUR" },
    { 0xA1F8, "YI SYLLABLE KIT" },
    { 0xA1F9, "YI SYLLABLE KIX" },
    { 0xA1FA, "YI SYLLABLE KI" },
    { 0xA1FB, "YI SYLLABLE KIP" },
    { 0xA1FC, "YI SYLLABLE KIEX" },
    { 0xA1FD, "YI SYLLABLE KIE" },
    { 0xA1FE, "YI SYLLABLE KIEP" },
    { 0xA1FF, "YI SYLLABLE KAT" },
    { 0xA200, "YI SYLLABLE KAX" },
    { 0xA201, "YI SYLLABLE KA" },
    { 0xA202, "YI SYLLABLE KAP" },
    { 0xA203, "YI SYLLABLE KUOX" },
    { 0xA204, "YI SYLLABLE KUO" },
    { 0xA205, "YI SYLLABLE KUOP" },
    { 0xA206, "YI SYLLABLE KOT" },
    { 0xA207, "YI SYLLABLE KOX" },
    { 0xA208, "YI SYLLABLE KO" },
    { 0xA209, "YI SYLLABLE KOP" },
    { 0xA20A, "YI SYLLABLE KET" },
    { 0xA20B, "YI SYLLABLE KEX" },
    { 0xA20C, "YI SYLLABLE KE" },
    { 0xA20D, "YI SYLLABLE KEP" },
    { 0xA20E, "YI SYLLABLE KUT" },
    { 0xA20F, "YI SYLLABLE KUX" },
    { 0xA210, "YI SYLLABLE KU" },
    { 0xA211, "YI SYLLABLE KUP" },
    { 0xA212, "YI SYLLABLE KURX" },
    { 0xA213, "YI SYLLABLE KUR" },
    { 0xA214, "YI SYLLABLE GGIT" },
    { 0xA215, "YI SYLLABLE GGIX" },
    { 0xA216, "YI SYLLABLE GGI" },
    { 0xA217, "YI SYLLABLE GGIEX" },
    { 0xA218, "YI SYLLABLE GGIE" },
    { 0xA219, "YI SYLLABLE GGIEP" },
    { 0xA21A, "YI SYLLABLE GGAT" },
    { 0xA21B, "YI SYLLABLE GGAX" },
    { 0xA21C, "YI SYLLABLE GGA" },
    { 0xA21D, "YI SYLLABLE GGAP" },
    { 0xA21E, "YI SYLLABLE GGUOT" },
    { 0xA21F, "YI SYLLABLE GGUOX" },
    { 0xA220, "YI SYLLABLE GGUO" },
    { 0xA221, "YI SYLLABLE GGUOP" },
    { 0xA222, "YI SYLLABLE GGOT" },
    { 0xA223, "YI SYLLABLE GGOX" },
    { 0xA224, "YI SYLLABLE GGO" },
    { 0xA225, "YI SYLLABLE GGOP" },
    { 0xA226, "YI SYLLABLE GGET" },
    { 0xA227, "YI SYLLABLE GGEX" },
    { 0xA228, "YI SYLLABLE GGE" },
    { 0xA229, "YI SYLLABLE GGEP" },
    { 0xA22A, "YI SYLLABLE GGUT" },
    { 0xA22B, "YI SYLLABLE GGUX" },
    { 0xA22C, "YI SYLLABLE GGU" },
    { 0xA22D, "YI SYLLABLE GGUP" },
    { 0xA22E, "YI SYLLABLE GGURX" },
    { 0xA22F, "YI SYLLABLE GGUR" },
    { 0xA230, "YI SYLLABLE MGIEX" },
    { 0xA231, "YI SYLLABLE MGIE" },
    { 0xA232, "YI SYLLABLE MGAT" },
    { 0xA233, "YI SYLLABLE MGAX" },
    { 0xA234, "YI SYLLABLE MGA" },
    { 0xA235, "YI SYLLABLE MGAP" },
    { 0xA236, "YI SYLLABLE MGUOX" },
    { 0xA237, "YI SYLLABLE MGUO" },
    { 0xA238, "YI SYLLABLE MGUOP" },
    { 0xA239, "YI SYLLABLE MGOT" },
    { 0xA23A, "YI SYLLABLE MGOX" },
    { 0xA23B, "YI SYLLABLE MGO" },
    { 0xA23C, "YI SYLLABLE MGOP" },
    { 0xA23D, "YI SYLLABLE MGEX" },
    { 0xA23E, "YI SYLLABLE MGE" },
    { 0xA23F, "YI SYLLABLE MGEP" },
    { 0xA240, "YI SYLLABLE MGUT" },
    { 0xA241, "YI SYLLABLE MGUX" },
    { 0xA242, "YI SYLLABLE MGU" },
    { 0xA243, "YI SYLLABLE MGUP" },
    { 0xA244, "YI SYLLABLE MGURX" },
    { 0xA245, "YI SYLLABLE MGUR" },
    { 0xA246, "YI SYLLABLE HXIT" },
    { 0xA247, "YI SYLLABLE HXIX" },
    { 0xA248, "YI SYLLABLE HXI" },
    { 0xA249, "YI SYLLABLE HXIP" },
    { 0xA24A, "YI SYLLABLE HXIET" },
    { 0xA24B, "YI SYLLABLE HXIEX" },
    { 0xA24C, "YI SYLLABLE HXIE" },
    { 0xA24D, "YI SYLLABLE HXIEP" },
    { 0xA24E, "YI SYLLABLE HXAT" },
    { 0xA24F, "YI SYLLABLE HXAX" },
    { 0xA250, "YI SYLLABLE HXA" },
    { 0xA251, "YI SYLLABLE HXAP" },
    { 0xA252, "YI SYLLABLE HXUOT" },
    { 0xA253, "YI SYLLABLE HXUOX" },
    { 0xA254, "YI SYLLABLE HXUO" },
    { 0xA255, "YI SYLLABLE HXUOP" },
    { 0xA256, "YI SYLLABLE HXOT" },
    { 0xA257, "YI SYLLABLE HXOX" },
    { 0xA258, "YI SYLLABLE HXO" },
    { 0xA259, "YI SYLLABLE HXOP" },
    { 0xA25A, "YI SYLLABLE HXEX" },
    { 0xA25B, "YI SYLLABLE HXE" },
    { 0xA25C, "YI SYLLABLE HXEP" },
    { 0xA25D, "YI SYLLABLE NGIEX" },
    { 0xA25E, "YI SYLLABLE NGIE" },
    { 0xA25F, "YI SYLLABLE NGIEP" },
    { 0xA260, "YI SYLLABLE NGAT" },
    { 0xA261, "YI SYLLABLE NGAX" },
    { 0xA262, "YI SYLLABLE NGA" },
    { 0xA263, "YI SYLLABLE NGAP" },
    { 0xA264, "YI SYLLABLE NGUOT" },
    { 0xA265, "YI SYLLABLE NGUOX" },
    { 0xA266, "YI SYLLABLE NGUO" },
    { 0xA267, "YI SYLLABLE NGOT" },
    { 0xA268, "YI SYLLABLE NGOX" },
    { 0xA269, "YI SYLLABLE NGO" },
    { 0xA26A, "YI SYLLABLE NGOP" },
    { 0xA26B, "YI SYLLABLE NGEX" },
    { 0xA26C, "YI SYLLABLE NGE" },
    { 0xA26D, "YI SYLLABLE NGEP" },
    { 0xA26E, "YI SYLLABLE HIT" },
    { 0xA26F, "YI SYLLABLE HIEX" },
    { 0xA270, "YI SYLLABLE HIE" },
    { 0xA271, "YI SYLLABLE HAT" },
    { 0xA272, "YI SYLLABLE HAX" },
    { 0xA273, "YI SYLLABLE HA" },
    { 0xA274, "YI SYLLABLE HAP" },
    { 0xA275, "YI SYLLABLE HUOT" },
    { 0xA276, "YI SYLLABLE HUOX" },
    { 0xA277, "YI SYLLABLE HUO" },
    { 0xA278, "YI SYLLABLE HUOP" },
    { 0xA279, "YI SYLLABLE HOT" },
    { 0xA27A, "YI SYLLABLE HOX" },
    { 0xA27B, "YI SYLLABLE HO" },
    { 0xA27C, "YI SYLLABLE HOP" },
    { 0xA27D, "YI SYLLABLE HEX" },
    { 0xA27E, "YI SYLLABLE HE" },
    { 0xA27F, "YI SYLLABLE HEP" },
    { 0xA280, "YI SYLLABLE WAT" },
    { 0xA281, "YI SYLLABLE WAX" },
    { 0xA282, "YI SYLLABLE WA" },
    { 0xA283, "YI SYLLABLE WAP" },
    { 0xA284, "YI SYLLABLE WUOX" },
    { 0xA285, "YI SYLLABLE WUO" },
    { 0xA286, "YI SYLLABLE WUOP" },
    { 0xA287, "YI SYLLABLE WOX" },
    { 0xA288, "YI SYLLABLE WO" },
    { 0xA289, "YI SYLLABLE WOP" },
    { 0xA28A, "YI SYLLABLE WEX" },
    { 0xA28B, "YI SYLLABLE WE" },
    { 0xA28C, "YI SYLLABLE WEP" },
    { 0xA28D, "YI SYLLABLE ZIT" },
    { 0xA28E, "YI SYLLABLE ZIX" },
    { 0xA28F, "YI SYLLABLE ZI" },
    { 0xA290, "YI SYLLABLE ZIP" },
    { 0xA291, "YI SYLLABLE ZIEX" },
    { 0xA292, "YI SYLLABLE ZIE" },
    { 0xA293, "YI SYLLABLE ZIEP" },
    { 0xA294, "YI SYLLABLE ZAT" },
    { 0xA295, "YI SYLLABLE ZAX" },
    { 0xA296, "YI SYLLABLE ZA" },
    { 0xA297, "YI SYLLABLE ZAP" },
    { 0xA298, "YI SYLLABLE ZUOX" },
    { 0xA299, "YI SYLLABLE ZUO" },
    { 0xA29A, "YI SYLLABLE ZUOP" },
    { 0xA29B, "YI SYLLABLE ZOT" },
    { 0xA29C, "YI SYLLABLE ZOX" },
    { 0xA29D, "YI SYLLABLE ZO" },
    { 0xA29E, "YI SYLLABLE ZOP" },
    { 0xA29F, "YI SYLLABLE ZEX" },
    { 0xA2A0, "YI SYLLABLE ZE" },
    { 0xA2A1, "YI SYLLABLE ZEP" },
    { 0xA2A2, "YI SYLLABLE ZUT" },
    { 0xA2A3, "YI SYLLABLE ZUX" },
    { 0xA2A4, "YI SYLLABLE ZU" },
    { 0xA2A5, "YI SYLLABLE ZUP" },
    { 0xA2A6, "YI SYLLABLE ZURX" },
    { 0xA2A7, "YI SYLLABLE ZUR" },
    { 0xA2A8, "YI SYLLABLE ZYT" },
    { 0xA2A9, "YI SYLLABLE ZYX" },
    { 0xA2AA, "YI SYLLABLE ZY" },
    { 0xA2AB, "YI SYLLABLE ZYP" },
    { 0xA2AC, "YI SYLLABLE ZYRX" },
    { 0xA2AD, "YI SYLLABLE ZYR" },
    { 0xA2AE, "YI SYLLABLE CIT" },
    { 0xA2AF, "YI SYLLABLE CIX" },
    { 0xA2B0, "YI SYLLABLE CI" },
    { 0xA2B1, "YI SYLLABLE CIP" },
    { 0xA2B2, "YI SYLLABLE CIET" },
    { 0xA2B3, "YI SYLLABLE CIEX" },
    { 0xA2B4, "YI SYLLABLE CIE" },
    { 0xA2B5, "YI SYLLABLE CIEP" },
    { 0xA2B6, "YI SYLLABLE CAT" },
    { 0xA2B7, "YI SYLLABLE CAX" },
    { 0xA2B8, "YI SYLLABLE CA" },
    { 0xA2B9, "YI SYLLABLE CAP" },
    { 0xA2BA, "YI SYLLABLE CUOX" },
    { 0xA2BB, "YI SYLLABLE CUO" },
    { 0xA2BC, "YI SYLLABLE CUOP" },
    { 0xA2BD, "YI SYLLABLE COT" },
    { 0xA2BE, "YI SYLLABLE COX" },
    { 0xA2BF, "YI SYLLABLE CO" },
    { 0xA2C0, "YI SYLLABLE COP" },
    { 0xA2C1, "YI SYLLABLE CEX" },
    { 0xA2C2, "YI SYLLABLE CE" },
    { 0xA2C3, "YI SYLLABLE CEP" },
    { 0xA2C4, "YI SYLLABLE CUT" },
    { 0xA2C5, "YI SYLLABLE CUX" },
    { 0xA2C6, "YI SYLLABLE CU" },
    { 0xA2C7, "YI SYLLABLE CUP" },
    { 0xA2C8, "YI SYLLABLE CURX" },
    { 0xA2C9, "YI SYLLABLE CUR" },
    { 0xA2CA, "YI SYLLABLE CYT" },
    { 0xA2CB, "YI SYLLABLE CYX" },
    { 0xA2CC, "YI SYLLABLE CY" },
    { 0xA2CD, "YI SYLLABLE CYP" },
    { 0xA2CE, "YI SYLLABLE CYRX" },
    { 0xA2CF, "YI SYLLABLE CYR" },
    { 0xA2D0, "YI SYLLABLE ZZIT" },
    { 0xA2D1, "YI SYLLABLE ZZIX" },
    { 0xA2D2, "YI SYLLABLE ZZI" },
    { 0xA2D3, "YI SYLLABLE ZZIP" },
    { 0xA2D4, "YI SYLLABLE ZZIET" },
    { 0xA2D5, "YI SYLLABLE ZZIEX" },
    { 0xA2D6, "YI SYLLABLE ZZIE" },
    { 0xA2D7, "YI SYLLABLE ZZIEP" },
    { 0xA2D8, "YI SYLLABLE ZZAT" },
    { 0xA2D9, "YI SYLLABLE ZZAX" },
    { 0xA2DA, "YI SYLLABLE ZZA" },
    { 0xA2DB, "YI SYLLABLE ZZAP" },
    { 0xA2DC, "YI SYLLABLE ZZOX" },
    { 0xA2DD, "YI SYLLABLE ZZO" },
    { 0xA2DE, "YI SYLLABLE ZZOP" },
    { 0xA2DF, "YI SYLLABLE ZZEX" },
    { 0xA2E0, "YI SYLLABLE ZZE" },
    { 0xA2E1, "YI SYLLABLE ZZEP" },
    { 0xA2E2, "YI SYLLABLE ZZUX" },
    { 0xA2E3, "YI SYLLABLE ZZU" },
    { 0xA2E4, "YI SYLLABLE ZZUP" },
    { 0xA2E5, "YI SYLLABLE ZZURX" },
    { 0xA2E6, "YI SYLLABLE ZZUR" },
    { 0xA2E7, "YI SYLLABLE ZZYT" },
    { 0xA2E8, "YI SYLLABLE ZZYX" },
    { 0xA2E9, "YI SYLLABLE ZZY" },
    { 0xA2EA, "YI SYLLABLE ZZYP" },
    { 0xA2EB, "YI SYLLABLE ZZYRX" },
    { 0xA2EC, "YI SYLLABLE ZZYR" },
    { 0xA2ED, "YI SYLLABLE NZIT" },
    { 0xA2EE, "YI SYLLABLE NZIX" },
    { 0xA2EF, "YI SYLLABLE NZI" },
    { 0xA2F0, "YI SYLLABLE NZIP" },
    { 0xA2F1, "YI SYLLABLE NZIEX" },
    { 0xA2F2, "YI SYLLABLE NZIE" },
    { 0xA2F3, "YI SYLLABLE NZIEP" },
    { 0xA2F4, "YI SYLLABLE NZAT" },
    { 0xA2F5, "YI SYLLABLE NZAX" },
    { 0xA2F6, "YI SYLLABLE NZA" },
    { 0xA2F7, "YI SYLLABLE NZAP" },
    { 0xA2F8, "YI SYLLABLE NZUOX" },
    { 0xA2F9, "YI SYLLABLE NZUO" },
    { 0xA2FA, "YI SYLLABLE NZOX" },
    { 0xA2FB, "YI SYLLABLE NZOP" },
    { 0xA2FC, "YI SYLLABLE NZEX" },
    { 0xA2FD, "YI SYLLABLE NZE" },
    { 0xA2FE, "YI SYLLABLE NZUX" },
    { 0xA2FF, "YI SYLLABLE NZU" },
    { 0xA300, "YI SYLLABLE NZUP" },
    { 0xA301, "YI SYLLABLE NZURX" },
    { 0xA302, "YI SYLLABLE NZUR" },
    { 0xA303, "YI SYLLABLE NZYT" },
    { 0xA304, "YI SYLLABLE NZYX" },
    { 0xA305, "YI SYLLABLE NZY" },
    { 0xA306, "YI SYLLABLE NZYP" },
    { 0xA307, "YI SYLLABLE NZYRX" },
    { 0xA308, "YI SYLLABLE NZYR" },
    { 0xA309, "YI SYLLABLE SIT" },
    { 0xA30A, "YI SYLLABLE SIX" },
    { 0xA30B, "YI SYLLABLE SI" },
    { 0xA30C, "YI SYLLABLE SIP" },
    { 0xA30D, "YI SYLLABLE SIEX" },
    { 0xA30E, "YI SYLLABLE SIE" },
    { 0xA30F, "YI SYLLABLE SIEP" },
    { 0xA310, "YI SYLLABLE SAT" },
    { 0xA311, "YI SYLLABLE SAX" },
    { 0xA312, "YI SYLLABLE SA" },
    { 0xA313, "YI SYLLABLE SAP" },
    { 0xA314, "YI SYLLABLE SUOX" },
    { 0xA315, "YI SYLLABLE SUO" },
    { 0xA316, "YI SYLLABLE SUOP" },
    { 0xA317, "YI SYLLABLE SOT" },
    { 0xA318, "YI SYLLABLE SOX" },
    { 0xA319, "YI SYLLABLE SO" },
    { 0xA31A, "YI SYLLABLE SOP" },
    { 0xA31B, "YI SYLLABLE SEX" },
    { 0xA31C, "YI SYLLABLE SE" },
    { 0xA31D, "YI SYLLABLE SEP" },
    { 0xA31E, "YI SYLLABLE SUT" },
    { 0xA31F, "YI SYLLABLE SUX" },
    { 0xA320, "YI SYLLABLE SU" },
    { 0xA321, "YI SYLLABLE SUP" },
    { 0xA322, "YI SYLLABLE SURX" },
    { 0xA323, "YI SYLLABLE SUR" },
    { 0xA324, "YI SYLLABLE SYT" },
    { 0xA325, "YI SYLLABLE SYX" },
    { 0xA326, "YI SYLLABLE SY" },
    { 0xA327, "YI SYLLABLE SYP" },
    { 0xA328, "YI SYLLABLE SYRX" },
    { 0xA329, "YI SYLLABLE SYR" },
    { 0xA32A, "YI SYLLABLE SSIT" },
    { 0xA32B, "YI SYLLABLE SSIX" },
    { 0xA32C, "YI SYLLABLE SSI" },
    { 0xA32D, "YI SYLLABLE SSIP" },
    { 0xA32E, "YI SYLLABLE SSIEX" },
    { 0xA32F, "YI SYLLABLE SSIE" },
    { 0xA330, "YI SYLLABLE SSIEP" },
    { 0xA331, "YI SYLLABLE SSAT" },
    { 0xA332, "YI SYLLABLE SSAX" },
    { 0xA333, "YI SYLLABLE SSA" },
    { 0xA334, "YI SYLLABLE SSAP" },
    { 0xA335, "YI SYLLABLE SSOT" },
    { 0xA336, "YI SYLLABLE SSOX" },
    { 0xA337, "YI SYLLABLE SSO" },
    { 0xA338, "YI SYLLABLE SSOP" },
    { 0xA339, "YI SYLLABLE SSEX" },
    { 0xA33A, "YI SYLLABLE SSE" },
    { 0xA33B, "YI SYLLABLE SSEP" },
    { 0xA33C, "YI SYLLABLE SSUT" },
    { 0xA33D, "YI SYLLABLE SSUX" },
    { 0xA33E, "YI SYLLABLE SSU" },
    { 0xA33F, "YI SYLLABLE SSUP" },
    { 0xA340, "YI SYLLABLE SSYT" },
    { 0xA341, "YI SYLLABLE SSYX" },
    { 0xA342, "YI SYLLABLE SSY" },
    { 0xA343, "YI SYLLABLE SSYP" },
    { 0xA344, "YI SYLLABLE SSYRX" },
    { 0xA345, "YI SYLLABLE SSYR" },
    { 0xA346, "YI SYLLABLE ZHAT" },
    { 0xA347, "YI SYLLABLE ZHAX" },
    { 0xA348, "YI SYLLABLE ZHA" },
    { 0xA349, "YI SYLLABLE ZHAP" },
    { 0xA34A, "YI SYLLABLE ZHUOX" },
    { 0xA34B, "YI SYLLABLE ZHUO" },
    { 0xA34C, "YI SYLLABLE ZHUOP" },
    { 0xA34D, "YI SYLLABLE ZHOT" },
    { 0xA34E, "YI SYLLABLE ZHOX" },
    { 0xA34F, "YI SYLLABLE ZHO" },
    { 0xA350, "YI SYLLABLE ZHOP" },
    { 0xA351, "YI SYLLABLE ZHET" },
    { 0xA352, "YI SYLLABLE ZHEX" },
    { 0xA353, "YI SYLLABLE ZHE" },
    { 0xA354, "YI SYLLABLE ZHEP" },
    { 0xA355, "YI SYLLABLE ZHUT" },
    { 0xA356, "YI SYLLABLE ZHUX" },
    { 0xA357, "YI SYLLABLE ZHU" },
    { 0xA358, "YI SYLLABLE ZHUP" },
    { 0xA359, "YI SYLLABLE ZHURX" },
    { 0xA35A, "YI SYLLABLE ZHUR" },
    { 0xA35B, "YI SYLLABLE ZHYT" },
    { 0xA35C, "YI SYLLABLE ZHYX" },
    { 0xA35D, "YI SYLLABLE ZHY" },
    { 0xA35E, "YI SYLLABLE ZHYP" },
    { 0xA35F, "YI SYLLABLE ZHYRX" },
    { 0xA360, "YI SYLLABLE ZHYR" },
    { 0xA361, "YI SYLLABLE CHAT" },
    { 0xA362, "YI SYLLABLE CHAX" },
    { 0xA363, "YI SYLLABLE CHA" },
    { 0xA364, "YI SYLLABLE CHAP" },
    { 0xA365, "YI SYLLABLE CHUOT" },
    { 0xA366, "YI SYLLABLE CHUOX" },
    { 0xA367, "YI SYLLABLE CHUO" },
    { 0xA368, "YI SYLLABLE CHUOP" },
    { 0xA369, "YI SYLLABLE CHOT" },
    { 0xA36A, "YI SYLLABLE CHOX" },
    { 0xA36B, "YI SYLLABLE CHO" },
    { 0xA36C, "YI SYLLABLE CHOP" },
    { 0xA36D, "YI SYLLABLE CHET" },
    { 0xA36E, "YI SYLLABLE CHEX" },
    { 0xA36F, "YI SYLLABLE CHE" },
    { 0xA370, "YI SYLLABLE CHEP" },
    { 0xA371, "YI SYLLABLE CHUX" },
    { 0xA372, "YI SYLLABLE CHU" },
    { 0xA373, "YI SYLLABLE CHUP" },
    { 0xA374, "YI SYLLABLE CHURX" },
    { 0xA375, "YI SYLLABLE CHUR" },
    { 0xA376, "YI SYLLABLE CHYT" },
    { 0xA377, "YI SYLLABLE CHYX" },
    { 0xA378, "YI SYLLABLE CHY" },
    { 0xA379, "YI SYLLABLE CHYP" },
    { 0xA37A, "YI SYLLABLE CHYRX" },
    { 0xA37B, "YI SYLLABLE CHYR" },
    { 0xA37C, "YI SYLLABLE RRAX" },
    { 0xA37D, "YI SYLLABLE RRA" },
    { 0xA37E, "YI SYLLABLE RRUOX" },
    { 0xA37F, "YI SYLLABLE RRUO" },
    { 0xA380, "YI SYLLABLE RROT" },
    { 0xA381, "YI SYLLABLE RROX" },
    { 0xA382, "YI SYLLABLE RRO" },
    { 0xA383, "YI SYLLABLE RROP" },
    { 0xA384, "YI SYLLABLE RRET" },
    { 0xA385, "YI SYLLABLE RREX" },
    { 0xA386, "YI SYLLABLE RRE" },
    { 0xA387, "YI SYLLABLE RREP" },
    { 0xA388, "YI SYLLABLE RRUT" },
    { 0xA389, "YI SYLLABLE RRUX" },
    { 0xA38A, "YI SYLLABLE RRU" },
    { 0xA38B, "YI SYLLABLE RRUP" },
    { 0xA38C, "YI SYLLABLE RRURX" },
    { 0xA38D, "YI SYLLABLE RRUR" },
    { 0xA38E, "YI SYLLABLE RRYT" },
    { 0xA38F, "YI SYLLABLE RRYX" },
    { 0xA390, "YI SYLLABLE RRY" },
    { 0xA391, "YI SYLLABLE RRYP" },
    { 0xA392, "YI SYLLABLE RRYRX" },
    { 0xA393, "YI SYLLABLE RRYR" },
    { 0xA394, "YI SYLLABLE NRAT" },
    { 0xA395, "YI SYLLABLE NRAX" },
    { 0xA396, "YI SYLLABLE NRA" },
    { 0xA397, "YI SYLLABLE NRAP" },
    { 0xA398, "YI SYLLABLE NROX" },
    { 0xA399, "YI SYLLABLE NRO" },
    { 0xA39A, "YI SYLLABLE NROP" },
    { 0xA39B, "YI SYLLABLE NRET" },
    { 0xA39C, "YI SYLLABLE NREX" },
    { 0xA39D, "YI SYLLABLE NRE" },
    { 0xA39E, "YI SYLLABLE NREP" },
    { 0xA39F, "YI SYLLABLE NRUT" },
    { 0xA3A0, "YI SYLLABLE NRUX" },
    { 0xA3A1, "YI SYLLABLE NRU" },
    { 0xA3A2, "YI SYLLABLE NRUP" },
    { 0xA3A3, "YI SYLLABLE NRURX" },
    { 0xA3A4, "YI SYLLABLE NRUR" },
    { 0xA3A5, "YI SYLLABLE NRYT" },
    { 0xA3A6, "YI SYLLABLE NRYX" },
    { 0xA3A7, "YI SYLLABLE NRY" },
    { 0xA3A8, "YI SYLLABLE NRYP" },
    { 0xA3A9, "YI SYLLABLE NRYRX" },
    { 0xA3AA, "YI SYLLABLE NRYR" },
    { 0xA3AB, "YI SYLLABLE SHAT" },
    { 0xA3AC, "YI SYLLABLE SHAX" },
    { 0xA3AD, "YI SYLLABLE SHA" },
    { 0xA3AE, "YI SYLLABLE SHAP" },
    { 0xA3AF, "YI SYLLABLE SHUOX" },
    { 0xA3B0, "YI SYLLABLE SHUO" },
    { 0xA3B1, "YI SYLLABLE SHUOP" },
    { 0xA3B2, "YI SYLLABLE SHOT" },
    { 0xA3B3, "YI SYLLABLE SHOX" },
    { 0xA3B4, "YI SYLLABLE SHO" },
    { 0xA3B5, "YI SYLLABLE SHOP" },
    { 0xA3B6, "YI SYLLABLE SHET" },
    { 0xA3B7, "YI SYLLABLE SHEX" },
    { 0xA3B8, "YI SYLLABLE SHE" },
    { 0xA3B9, "YI SYLLABLE SHEP" },
    { 0xA3BA, "YI SYLLABLE SHUT" },
    { 0xA3BB, "YI SYLLABLE SHUX" },
    { 0xA3BC, "YI SYLLABLE SHU" },
    { 0xA3BD, "YI SYLLABLE SHUP" },
    { 0xA3BE, "YI SYLLABLE SHURX" },
    { 0xA3BF, "YI SYLLABLE SHUR" },
    { 0xA3C0, "YI SYLLABLE SHYT" },
    { 0xA3C1, "YI SYLLABLE SHYX" },
    { 0xA3C2, "YI SYLLABLE SHY" },
    { 0xA3C3, "YI SYLLABLE SHYP" },
    { 0xA3C4, "YI SYLLABLE SHYRX" },
    { 0xA3C5, "YI SYLLABLE SHYR" },
    { 0xA3C6, "YI SYLLABLE RAT" },
    { 0xA3C7, "YI SYLLABLE RAX" },
    { 0xA3C8, "YI SYLLABLE RA" },
    { 0xA3C9, "YI SYLLABLE RAP" },
    { 0xA3CA, "YI SYLLABLE RUOX" },
    { 0xA3CB, "YI SYLLABLE RUO" },
    { 0xA3CC, "YI SYLLABLE RUOP" },
    { 0xA3CD, "YI SYLLABLE ROT" },
    { 0xA3CE, "YI SYLLABLE ROX" },
    { 0xA3CF, "YI SYLLABLE RO" },
    { 0xA3D0, "YI SYLLABLE ROP" },
    { 0xA3D1, "YI SYLLABLE REX" },
    { 0xA3D2, "YI SYLLABLE RE" },
    { 0xA3D3, "YI SYLLABLE REP" },
    { 0xA3D4, "YI SYLLABLE RUT" },
    { 0xA3D5, "YI SYLLABLE RUX" },
    { 0xA3D6, "YI SYLLABLE RU" },
    { 0xA3D7, "YI SYLLABLE RUP" },
    { 0xA3D8, "YI SYLLABLE RURX" },
    { 0xA3D9, "YI SYLLABLE RUR" },
    { 0xA3DA, "YI SYLLABLE RYT" },
    { 0xA3DB, "YI SYLLABLE RYX" },
    { 0xA3DC, "YI SYLLABLE RY" },
    { 0xA3DD, "YI SYLLABLE RYP" },
    { 0xA3DE, "YI SYLLABLE RYRX" },
    { 0xA3DF, "YI SYLLABLE RYR" },
    { 0xA3E0, "YI SYLLABLE JIT" },
    { 0xA3E1, "YI SYLLABLE JIX" },
    { 0xA3E2, "YI SYLLABLE JI" },
    { 0xA3E3, "YI SYLLABLE JIP" },
    { 0xA3E4, "YI SYLLABLE JIET" },
    { 0xA3E5, "YI SYLLABLE JIEX" },
    { 0xA3E6, "YI SYLLABLE JIE" },
    { 0xA3E7, "YI SYLLABLE JIEP" },
    { 0xA3E8, "YI SYLLABLE JUOT" },
    { 0xA3E9, "YI SYLLABLE JUOX" },
    { 0xA3EA, "YI SYLLABLE JUO" },
    { 0xA3EB, "YI SYLLABLE JUOP" },
    { 0xA3EC, "YI SYLLABLE JOT" },
    { 0xA3ED, "YI SYLLABLE JOX" },
    { 0xA3EE, "YI SYLLABLE JO" },
    { 0xA3EF, "YI SYLLABLE JOP" },
    { 0xA3F0, "YI SYLLABLE JUT" },
    { 0xA3F1, "YI SYLLABLE JUX" },
    { 0xA3F2, "YI SYLLABLE JU" },
    { 0xA3F3, "YI SYLLABLE JUP" },
    { 0xA3F4, "YI SYLLABLE JURX" },
    { 0xA3F5, "YI SYLLABLE JUR" },
    { 0xA3F6, "YI SYLLABLE JYT" },
    { 0xA3F7, "YI SYLLABLE JYX" },
    { 0xA3F8, "YI SYLLABLE JY" },
    { 0xA3F9, "YI SYLLABLE JYP" },
    { 0xA3FA, "YI SYLLABLE JYRX" },
    { 0xA3FB, "YI SYLLABLE JYR" },
    { 0xA3FC, "YI SYLLABLE QIT" },
    { 0xA3FD, "YI SYLLABLE QIX" },
    { 0xA3FE, "YI SYLLABLE QI" },
    { 0xA3FF, "YI SYLLABLE QIP" },
    { 0xA400, "YI SYLLABLE QIET" },
    { 0xA401, "YI SYLLABLE QIEX" },
    { 0xA402, "YI SYLLABLE QIE" },
    { 0xA403, "YI SYLLABLE QIEP" },
    { 0xA404, "YI SYLLABLE QUOT" },
    { 0xA405, "YI SYLLABLE QUOX" },
    { 0xA406, "YI SYLLABLE QUO" },
    { 0xA407, "YI SYLLABLE QUOP" },
    { 0xA408, "YI SYLLABLE QOT" },
    { 0xA409, "YI SYLLABLE QOX" },
    { 0xA40A, "YI SYLLABLE QO" },
    { 0xA40B, "YI SYLLABLE QOP" },
    { 0xA40C, "YI SYLLABLE QUT" },
    { 0xA40D, "YI SYLLABLE QUX" },
    { 0xA40E, "YI SYLLABLE QU" },
    { 0xA40F, "YI SYLLABLE QUP" },
    { 0xA410, "YI SYLLABLE QURX" },
    { 0xA411, "YI SYLLABLE QUR" },
    { 0xA412, "YI SYLLABLE QYT" },
    { 0xA413, "YI SYLLABLE QYX" },
    { 0xA414, "YI SYLLABLE QY" },
    { 0xA415, "YI SYLLABLE QYP" },
    { 0xA416, "YI SYLLABLE QYRX" },
    { 0xA417, "YI SYLLABLE QYR" },
    { 0xA418, "YI SYLLABLE JJIT" },
    { 0xA419, "YI SYLLABLE JJIX" },
    { 0xA41A, "YI SYLLABLE JJI" },
    { 0xA41B, "YI SYLLABLE JJIP" },
    { 0xA41C, "YI SYLLABLE JJIET" },
    { 0xA41D, "YI SYLLABLE JJIEX" },
    { 0xA41E, "YI SYLLABLE JJIE" },
    { 0xA41F, "YI SYLLABLE JJIEP" },
    { 0xA420, "YI SYLLABLE JJUOX" },
    { 0xA421, "YI SYLLABLE JJUO" },
    { 0xA422, "YI SYLLABLE JJUOP" },
    { 0xA423, "YI SYLLABLE JJOT" },
    { 0xA424, "YI SYLLABLE JJOX" },
    { 0xA425, "YI SYLLABLE JJO" },
    { 0xA426, "YI SYLLABLE JJOP" },
    { 0xA427, "YI SYLLABLE JJUT" },
    { 0xA428, "YI SYLLABLE JJUX" },
    { 0xA429, "YI SYLLABLE JJU" },
    { 0xA42A, "YI SYLLABLE JJUP" },
    { 0xA42B, "YI SYLLABLE JJURX" },
    { 0xA42C, "YI SYLLABLE JJUR" },
    { 0xA42D, "YI SYLLABLE JJYT" },
    { 0xA42E, "YI SYLLABLE JJYX" },
    { 0xA42F, "YI SYLLABLE JJY" },
    { 0xA430, "YI SYLLABLE JJYP" },
    { 0xA431, "YI SYLLABLE NJIT" },
    { 0xA432, "YI SYLLABLE NJIX" },
    { 0xA433, "YI SYLLABLE NJI" },
    { 0xA434, "YI SYLLABLE NJIP" },
    { 0xA435, "YI SYLLABLE NJIET" },
    { 0xA436, "YI SYLLABLE NJIEX" },
    { 0xA437, "YI SYLLABLE NJIE" },
    { 0xA438, "YI SYLLABLE NJIEP" },
    { 0xA439, "YI SYLLABLE NJUOX" },
    { 0xA43A, "YI SYLLABLE NJUO" },
    { 0xA43B, "YI SYLLABLE NJOT" },
    { 0xA43C, "YI SYLLABLE NJOX" },
    { 0xA43D, "YI SYLLABLE NJO" },
    { 0xA43E, "YI SYLLABLE NJOP" },
    { 0xA43F, "YI SYLLABLE NJUX" },
    { 0xA440, "YI SYLLABLE NJU" },
    { 0xA441, "YI SYLLABLE NJUP" },
    { 0xA442, "YI SYLLABLE NJURX" },
    { 0xA443, "YI SYLLABLE NJUR" },
    { 0xA444, "YI SYLLABLE NJYT" },
    { 0xA445, "YI SYLLABLE NJYX" },
    { 0xA446, "YI SYLLABLE NJY" },
    { 0xA447, "YI SYLLABLE NJYP" },
    { 0xA448, "YI SYLLABLE NJYRX" },
    { 0xA449, "YI SYLLABLE NJYR" },
    { 0xA44A, "YI SYLLABLE NYIT" },
    { 0xA44B, "YI SYLLABLE NYIX" },
    { 0xA44C, "YI SYLLABLE NYI" },
    { 0xA44D, "YI SYLLABLE NYIP" },
    { 0xA44E, "YI SYLLABLE NYIET" },
    { 0xA44F, "YI SYLLABLE NYIEX" },
    { 0xA450, "YI SYLLABLE NYIE" },
    { 0xA451, "YI SYLLABLE NYIEP" },
    { 0xA452, "YI SYLLABLE NYUOX" },
    { 0xA453, "YI SYLLABLE NYUO" },
    { 0xA454, "YI SYLLABLE NYUOP" },
    { 0xA455, "YI SYLLABLE NYOT" },
    { 0xA456, "YI SYLLABLE NYOX" },
    { 0xA457, "YI SYLLABLE NYO" },
    { 0xA458, "YI SYLLABLE NYOP" },
    { 0xA459, "YI SYLLABLE NYUT" },
    { 0xA45A, "YI SYLLABLE NYUX" },
    { 0xA45B, "YI SYLLABLE NYU" },
    { 0xA45C, "YI SYLLABLE NYUP" },
    { 0xA45D, "YI SYLLABLE XIT" },
    { 0xA45E, "YI SYLLABLE XIX" },
    { 0xA45F, "YI SYLLABLE XI" },
    { 0xA460, "YI SYLLABLE XIP" },
    { 0xA461, "YI SYLLABLE XIET" },
    { 0xA462, "YI SYLLABLE XIEX" },
    { 0xA463, "YI SYLLABLE XIE" },
    { 0xA464, "YI SYLLABLE XIEP" },
    { 0xA465, "YI SYLLABLE XUOX" },
    { 0xA466, "YI SYLLABLE XUO" },
    { 0xA467, "YI SYLLABLE XOT" },
    { 0xA468, "YI SYLLABLE XOX" },
    { 0xA469, "YI SYLLABLE XO" },
    { 0xA46A, "YI SYLLABLE XOP" },
    { 0xA46B, "YI SYLLABLE XYT" },
    { 0xA46C, "YI SYLLABLE XYX" },
    { 0xA46D, "YI SYLLABLE XY" },
    { 0xA46E, "YI SYLLABLE XYP" },
    { 0xA46F, "YI SYLLABLE XYRX" },
    { 0xA470, "YI SYLLABLE XYR" },
    { 0xA471, "YI SYLLABLE YIT" },
    { 0xA472, "YI SYLLABLE YIX" },
    { 0xA473, "YI SYLLABLE YI" },
    { 0xA474, "YI SYLLABLE YIP" },
    { 0xA475, "YI SYLLABLE YIET" },
    { 0xA476, "YI SYLLABLE YIEX" },
    { 0xA477, "YI SYLLABLE YIE" },
    { 0xA478, "YI SYLLABLE YIEP" },
    { 0xA479, "YI SYLLABLE YUOT" },
    { 0xA47A, "YI SYLLABLE YUOX" },
    { 0xA47B, "YI SYLLABLE YUO" },
    { 0xA47C, "YI SYLLABLE YUOP" },
    { 0xA47D, "YI SYLLABLE YOT" },
    { 0xA47E, "YI SYLLABLE YOX" },
    { 0xA47F, "YI SYLLABLE YO" },
    { 0xA480, "YI SYLLABLE YOP" },
    { 0xA481, "YI SYLLABLE YUT" },
    { 0xA482, "YI SYLLABLE YUX" },
    { 0xA483, "YI SYLLABLE YU" },
    { 0xA484, "YI SYLLABLE YUP" },
    { 0xA485, "YI SYLLABLE YURX" },
    { 0xA486, "YI SYLLABLE YUR" },
    { 0xA487, "YI SYLLABLE YYT" },
    { 0xA488, "YI SYLLABLE YYX" },
    { 0xA489, "YI SYLLABLE YY" },
    { 0xA48A, "YI SYLLABLE YYP" },
    { 0xA48B, "YI SYLLABLE YYRX" },
    { 0xA48C, "YI SYLLABLE YYR" },
    { 0xA490, "YI RADICAL QOT" },
    { 0xA491, "YI RADICAL LI" },
    { 0xA492, "YI RADICAL KIT" },
    { 0xA493, "YI RADICAL NYIP" },
    { 0xA494, "YI RADICAL CYP" },
    { 0xA495, "YI RADICAL SSI" },
    { 0xA496, "YI RADICAL GGOP" },
    { 0xA497, "YI RADICAL GEP" },
    { 0xA498, "YI RADICAL MI" },
    { 0xA499, "YI RADICAL HXIT" },
    { 0xA49A, "YI RADICAL LYR" },
    { 0xA49B, "YI RADICAL BBUT" },
    { 0xA49C, "YI RADICAL MOP" },
    { 0xA49D, "YI RADICAL YO" },
    { 0xA49E, "YI RADICAL PUT" },
    { 0xA49F, "YI RADICAL HXUO" },
    { 0xA4A0, "YI RADICAL TAT" },
    { 0xA4A1, "YI RADICAL GA" },
    { 0xA4A2, "YI RADICAL ZUP" },
    { 0xA4A3, "YI RADICAL CYT" },
    { 0xA4A4, "YI RADICAL DDUR" },
    { 0xA4A5, "YI RADICAL BUR" },
    { 0xA4A6, "YI RADICAL GGUO" },
    { 0xA4A7, "YI RADICAL NYOP" },
    { 0xA4A8, "YI RADICAL TU" },
    { 0xA4A9, "YI RADICAL OP" },
    { 0xA4AA, "YI RADICAL JJUT" },
    { 0xA4AB, "YI RADICAL ZOT" },
    { 0xA4AC, "YI RADICAL PYT" },
    { 0xA4AD, "YI RADICAL HMO" },
    { 0xA4AE, "YI RADICAL YIT" },
    { 0xA4AF, "YI RADICAL VUR" },
    { 0xA4B0, "YI RADICAL SHY" },
    { 0xA4B1, "YI RADICAL VEP" },
    { 0xA4B2, "YI RADICAL ZA" },
    { 0xA4B3, "YI RADICAL JO" },
    { 0xA4B4, "YI RADICAL NZUP" },
    { 0xA4B5, "YI RADICAL JJY" },
    { 0xA4B6, "YI RADICAL GOT" },
    { 0xA4B7, "YI RADICAL JJIE" },
    { 0xA4B8, "YI RADICAL WO" },
    { 0xA4B9, "YI RADICAL DU" },
    { 0xA4BA, "YI RADICAL SHUR" },
    { 0xA4BB, "YI RADICAL LIE" },
    { 0xA4BC, "YI RADICAL CY" },
    { 0xA4BD, "YI RADICAL CUOP" },
    { 0xA4BE, "YI RADICAL CIP" },
    { 0xA4BF, "YI RADICAL HXOP" },
    { 0xA4C0, "YI RADICAL SHAT" },
    { 0xA4C1, "YI RADICAL ZUR" },
    { 0xA4C2, "YI RADICAL SHOP" },
    { 0xA4C3, "YI RADICAL CHE" },
    { 0xA4C4, "YI RADICAL ZZIET" },
    { 0xA4C5, "YI RADICAL NBIE" },
    { 0xA4C6, "YI RADICAL KE" },
    { 0xA4D0, "LISU LETTER BA" },
    { 0xA4D1, "LISU LETTER PA" },
    { 0xA4D2, "LISU LETTER PHA" },
    { 0xA4D3, "LISU LETTER DA" },
    { 0xA4D4, "LISU LETTER TA" },
    { 0xA4D5, "LISU LETTER THA" },
    { 0xA4D6, "LISU LETTER GA" },
    { 0xA4D7, "LISU LETTER KA" },
    { 0xA4D8, "LISU LETTER KHA" },
    { 0xA4D9, "LISU LETTER JA" },
    { 0xA4DA, "LISU LETTER CA" },
    { 0xA4DB, "LISU LETTER CHA" },
    { 0xA4DC, "LISU LETTER DZA" },
    { 0xA4DD, "LISU LETTER TSA" },
    { 0xA4DE, "LISU LETTER TSHA" },
    { 0xA4DF, "LISU LETTER MA" },
    { 0xA4E0, "LISU LETTER NA" },
    { 0xA4E1, "LISU LETTER LA" },
    { 0xA4E2, "LISU LETTER SA" },
    { 0xA4E3, "LISU LETTER ZHA" },
    { 0xA4E4, "LISU LETTER ZA" },
    { 0xA4E5, "LISU LETTER NGA" },
    { 0xA4E6, "LISU LETTER HA" },
    { 0xA4E7, "LISU LETTER XA" },
    { 0xA4E8, "LISU LETTER HHA" },
    { 0xA4E9, "LISU LETTER FA" },
    { 0xA4EA, "LISU LETTER WA" },
    { 0xA4EB, "LISU LETTER SHA" },
    { 0xA4EC, "LISU LETTER YA" },
    { 0xA4ED, "LISU LETTER GHA" },
    { 0xA4EE, "LISU LETTER A" },
    { 0xA4EF, "LISU LETTER AE" },
    { 0xA4F0, "LISU LETTER E" },
    { 0xA4F1, "LISU LETTER EU" },
    { 0xA4F2, "LISU LETTER I" },
    { 0xA4F3, "LISU LETTER O" },
    { 0xA4F4, "LISU LETTER U" },
    { 0xA4F5, "LISU LETTER UE" },
    { 0xA4F6, "LISU LETTER UH" },
    { 0xA4F7, "LISU LETTER OE" },
    { 0xA4F8, "LISU LETTER TONE MYA TI" },
    { 0xA4F9, "LISU LETTER TONE NA PO" },
    { 0xA4FA, "LISU LETTER TONE MYA CYA" },
    { 0xA4FB, "LISU LETTER TONE MYA BO" },
    { 0xA4FC, "LISU LETTER TONE MYA NA" },
    { 0xA4FD, "LISU LETTER TONE MYA JEU" },
    { 0xA4FE, "LISU PUNCTUATION COMMA" },
    { 0xA4FF, "LISU PUNCTUATION FULL STOP" },
    { 0xA500, "VAI SYLLABLE EE" },
    { 0xA501, "VAI SYLLABLE EEN" },
    { 0xA502, "VAI SYLLABLE HEE" },
    { 0xA503, "VAI SYLLABLE WEE" },
    { 0xA504, "VAI SYLLABLE WEEN" },
    { 0xA505, "VAI SYLLABLE PEE" },
    { 0xA506, "VAI SYLLABLE BHEE" },
    { 0xA507, "VAI SYLLABLE BEE" },
    { 0xA508, "VAI SYLLABLE MBEE" },
    { 0xA509, "VAI SYLLABLE KPEE" },
    { 0xA50A, "VAI SYLLABLE MGBEE" },
    { 0xA50B, "VAI SYLLABLE GBEE" },
    { 0xA50C, "VAI SYLLABLE FEE" },
    { 0xA50D, "VAI SYLLABLE VEE" },
    { 0xA50E, "VAI SYLLABLE TEE" },
    { 0xA50F, "VAI SYLLABLE THEE" },
    { 0xA510, "VAI SYLLABLE DHEE" },
    { 0xA511, "VAI SYLLABLE DHHEE" },
    { 0xA512, "VAI SYLLABLE LEE" },
    { 0xA513, "VAI SYLLABLE REE" },
    { 0xA514, "VAI SYLLABLE DEE" },
    { 0xA515, "VAI SYLLABLE NDEE" },
    { 0xA516, "VAI SYLLABLE SEE" },
    { 0xA517, "VAI SYLLABLE SHEE" },
    { 0xA518, "VAI SYLLABLE ZEE" },
    { 0xA519, "VAI SYLLABLE ZHEE" },
    { 0xA51A, "VAI SYLLABLE CEE" },
    { 0xA51B, "VAI SYLLABLE JEE" },
    { 0xA51C, "VAI SYLLABLE NJEE" },
    { 0xA51D, "VAI SYLLABLE YEE" },
    { 0xA51E, "VAI SYLLABLE KEE" },
    { 0xA51F, "VAI SYLLABLE NGGEE" },
    { 0xA520, "VAI SYLLABLE GEE" },
    { 0xA521, "VAI SYLLABLE MEE" },
    { 0xA522, "VAI SYLLABLE NEE" },
    { 0xA523, "VAI SYLLABLE NYEE" },
    { 0xA524, "VAI SYLLABLE I" },
    { 0xA525, "VAI SYLLABLE IN" },
    { 0xA526, "VAI SYLLABLE HI" },
    { 0xA527, "VAI SYLLABLE HIN" },
    { 0xA528, "VAI SYLLABLE WI" },
    { 0xA529, "VAI SYLLABLE WIN" },
    { 0xA52A, "VAI SYLLABLE PI" },
    { 0xA52B, "VAI SYLLABLE BHI" },
    { 0xA52C, "VAI SYLLABLE BI" },
    { 0xA52D, "VAI SYLLABLE MBI" },
    { 0xA52E, "VAI SYLLABLE KPI" },
    { 0xA52F, "VAI SYLLABLE MGBI" },
    { 0xA530, "VAI SYLLABLE GBI" },
    { 0xA531, "VAI SYLLABLE FI" },
    { 0xA532, "VAI SYLLABLE VI" },
    { 0xA533, "VAI SYLLABLE TI" },
    { 0xA534, "VAI SYLLABLE THI" },
    { 0xA535, "VAI SYLLABLE DHI" },
    { 0xA536, "VAI SYLLABLE DHHI" },
    { 0xA537, "VAI SYLLABLE LI" },
    { 0xA538, "VAI SYLLABLE RI" },
    { 0xA539, "VAI SYLLABLE DI" },
    { 0xA53A, "VAI SYLLABLE NDI" },
    { 0xA53B, "VAI SYLLABLE SI" },
    { 0xA53C, "VAI SYLLABLE SHI" },
    { 0xA53D, "VAI SYLLABLE ZI" },
    { 0xA53E, "VAI SYLLABLE ZHI" },
    { 0xA53F, "VAI SYLLABLE CI" },
    { 0xA540, "VAI SYLLABLE JI" },
    { 0xA541, "VAI SYLLABLE NJI" },
    { 0xA542, "VAI SYLLABLE YI" },
    { 0xA543, "VAI SYLLABLE KI" },
    { 0xA544, "VAI SYLLABLE NGGI" },
    { 0xA545, "VAI SYLLABLE GI" },
    { 0xA546, "VAI SYLLABLE MI" },
    { 0xA547, "VAI SYLLABLE NI" },
    { 0xA548, "VAI SYLLABLE NYI" },
    { 0xA549, "VAI SYLLABLE A" },
    { 0xA54A, "VAI SYLLABLE AN" },
    { 0xA54B, "VAI SYLLABLE NGAN" },
    { 0xA54C, "VAI SYLLABLE HA" },
    { 0xA54D, "VAI SYLLABLE HAN" },
    { 0xA54E, "VAI SYLLABLE WA" },
    { 0xA54F, "VAI SYLLABLE WAN" },
    { 0xA550, "VAI SYLLABLE PA" },
    { 0xA551, "VAI SYLLABLE BHA" },
    { 0xA552, "VAI SYLLABLE BA" },
    { 0xA553, "VAI SYLLABLE MBA" },
    { 0xA554, "VAI SYLLABLE KPA" },
    { 0xA555, "VAI SYLLABLE KPAN" },
    { 0xA556, "VAI SYLLABLE MGBA" },
    { 0xA557, "VAI SYLLABLE GBA" },
    { 0xA558, "VAI SYLLABLE FA" },
    { 0xA559, "VAI SYLLABLE VA" },
    { 0xA55A, "VAI SYLLABLE TA" },
    { 0xA55B, "VAI SYLLABLE THA" },
    { 0xA55C, "VAI SYLLABLE DHA" },
    { 0xA55D, "VAI SYLLABLE DHHA" },
    { 0xA55E, "VAI SYLLABLE LA" },
    { 0xA55F, "VAI SYLLABLE RA" },
    { 0xA560, "VAI SYLLABLE DA" },
    { 0xA561, "VAI SYLLABLE NDA" },
    { 0xA562, "VAI SYLLABLE SA" },
    { 0xA563, "VAI SYLLABLE SHA" },
    { 0xA564, "VAI SYLLABLE ZA" },
    { 0xA565, "VAI SYLLABLE ZHA" },
    { 0xA566, "VAI SYLLABLE CA" },
    { 0xA567, "VAI SYLLABLE JA" },
    { 0xA568, "VAI SYLLABLE NJA" },
    { 0xA569, "VAI SYLLABLE YA" },
    { 0xA56A, "VAI SYLLABLE KA" },
    { 0xA56B, "VAI SYLLABLE KAN" },
    { 0xA56C, "VAI SYLLABLE NGGA" },
    { 0xA56D, "VAI SYLLABLE GA" },
    { 0xA56E, "VAI SYLLABLE MA" },
    { 0xA56F, "VAI SYLLABLE NA" },
    { 0xA570, "VAI SYLLABLE NYA" },
    { 0xA571, "VAI SYLLABLE OO" },
    { 0xA572, "VAI SYLLABLE OON" },
    { 0xA573, "VAI SYLLABLE HOO" },
    { 0xA574, "VAI SYLLABLE WOO" },
    { 0xA575, "VAI SYLLABLE WOON" },
    { 0xA576, "VAI SYLLABLE POO" },
    { 0xA577, "VAI SYLLABLE BHOO" },
    { 0xA578, "VAI SYLLABLE BOO" },
    { 0xA579, "VAI SYLLABLE MBOO" },
    { 0xA57A, "VAI SYLLABLE KPOO" },
    { 0xA57B, "VAI SYLLABLE MGBOO" },
    { 0xA57C, "VAI SYLLABLE GBOO" },
    { 0xA57D, "VAI SYLLABLE FOO" },
    { 0xA57E, "VAI SYLLABLE VOO" },
    { 0xA57F, "VAI SYLLABLE TOO" },
    { 0xA580, "VAI SYLLABLE THOO" },
    { 0xA581, "VAI SYLLABLE DHOO" },
    { 0xA582, "VAI SYLLABLE DHHOO" },
    { 0xA583, "VAI SYLLABLE LOO" },
    { 0xA584, "VAI SYLLABLE ROO" },
    { 0xA585, "VAI SYLLABLE DOO" },
    { 0xA586, "VAI SYLLABLE NDOO" },
    { 0xA587, "VAI SYLLABLE SOO" },
    { 0xA588, "VAI SYLLABLE SHOO" },
    { 0xA589, "VAI SYLLABLE ZOO" },
    { 0xA58A, "VAI SYLLABLE ZHOO" },
    { 0xA58B, "VAI SYLLABLE COO" },
    { 0xA58C, "VAI SYLLABLE JOO" },
    { 0xA58D, "VAI SYLLABLE NJOO" },
    { 0xA58E, "VAI SYLLABLE YOO" },
    { 0xA58F, "VAI SYLLABLE KOO" },
    { 0xA590, "VAI SYLLABLE NGGOO" },
    { 0xA591, "VAI SYLLABLE GOO" },
    { 0xA592, "VAI SYLLABLE MOO" },
    { 0xA593, "VAI SYLLABLE NOO" },
    { 0xA594, "VAI SYLLABLE NYOO" },
    { 0xA595, "VAI SYLLABLE U" },
    { 0xA596, "VAI SYLLABLE UN" },
    { 0xA597, "VAI SYLLABLE HU" },
    { 0xA598, "VAI SYLLABLE HUN" },
    { 0xA599, "VAI SYLLABLE WU" },
    { 0xA59A, "VAI SYLLABLE WUN" },
    { 0xA59B, "VAI SYLLABLE PU" },
    { 0xA59C, "VAI SYLLABLE BHU" },
    { 0xA59D, "VAI SYLLABLE BU" },
    { 0xA59E, "VAI SYLLABLE MBU" },
    { 0xA59F, "VAI SYLLABLE KPU" },
    { 0xA5A0, "VAI SYLLABLE MGBU" },
    { 0xA5A1, "VAI SYLLABLE GBU" },
    { 0xA5A2, "VAI SYLLABLE FU" },
    { 0xA5A3, "VAI SYLLABLE VU" },
    { 0xA5A4, "VAI SYLLABLE TU" },
    { 0xA5A5, "VAI SYLLABLE THU" },
    { 0xA5A6, "VAI SYLLABLE DHU" },
    { 0xA5A7, "VAI SYLLABLE DHHU" },
    { 0xA5A8, "VAI SYLLABLE LU" },
    { 0xA5A9, "VAI SYLLABLE RU" },
    { 0xA5AA, "VAI SYLLABLE DU" },
    { 0xA5AB, "VAI SYLLABLE NDU" },
    { 0xA5AC, "VAI SYLLABLE SU" },
    { 0xA5AD, "VAI SYLLABLE SHU" },
    { 0xA5AE, "VAI SYLLABLE ZU" },
    { 0xA5AF, "VAI SYLLABLE ZHU" },
    { 0xA5B0, "VAI SYLLABLE CU" },
    { 0xA5B1, "VAI SYLLABLE JU" },
    { 0xA5B2, "VAI SYLLABLE NJU" },
    { 0xA5B3, "VAI SYLLABLE YU" },
    { 0xA5B4, "VAI SYLLABLE KU" },
    { 0xA5B5, "VAI SYLLABLE NGGU" },
    { 0xA5B6, "VAI SYLLABLE GU" },
    { 0xA5B7, "VAI SYLLABLE MU" },
    { 0xA5B8, "VAI SYLLABLE NU" },
    { 0xA5B9, "VAI SYLLABLE NYU" },
    { 0xA5BA, "VAI SYLLABLE O" },
    { 0xA5BB, "VAI SYLLABLE ON" },
    { 0xA5BC, "VAI SYLLABLE NGON" },
    { 0xA5BD, "VAI SYLLABLE HO" },
    { 0xA5BE, "VAI SYLLABLE HON" },
    { 0xA5BF, "VAI SYLLABLE WO" },
    { 0xA5C0, "VAI SYLLABLE WON" },
    { 0xA5C1, "VAI SYLLABLE PO" },
    { 0xA5C2, "VAI SYLLABLE BHO" },
    { 0xA5C3, "VAI SYLLABLE BO" },
    { 0xA5C4, "VAI SYLLABLE MBO" },
    { 0xA5C5, "VAI SYLLABLE KPO" },
    { 0xA5C6, "VAI SYLLABLE MGBO" },
    { 0xA5C7, "VAI SYLLABLE GBO" },
    { 0xA5C8, "VAI SYLLABLE GBON" },
    { 0xA5C9, "VAI SYLLABLE FO" },
    { 0xA5CA, "VAI SYLLABLE VO" },
    { 0xA5CB, "VAI SYLLABLE TO" },
    { 0xA5CC, "VAI SYLLABLE THO" },
    { 0xA5CD, "VAI SYLLABLE DHO" },
    { 0xA5CE, "VAI SYLLABLE DHHO" },
    { 0xA5CF, "VAI SYLLABLE LO" },
    { 0xA5D0, "VAI SYLLABLE RO" },
    { 0xA5D1, "VAI SYLLABLE DO" },
    { 0xA5D2, "VAI SYLLABLE NDO" },
    { 0xA5D3, "VAI SYLLABLE SO" },
    { 0xA5D4, "VAI SYLLABLE SHO" },
    { 0xA5D5, "VAI SYLLABLE ZO" },
    { 0xA5D6, "VAI SYLLABLE ZHO" },
    { 0xA5D7, "VAI SYLLABLE CO" },
    { 0xA5D8, "VAI SYLLABLE JO" },
    { 0xA5D9, "VAI SYLLABLE NJO" },
    { 0xA5DA, "VAI SYLLABLE YO" },
    { 0xA5DB, "VAI SYLLABLE KO" },
    { 0xA5DC, "VAI SYLLABLE NGGO" },
    { 0xA5DD, "VAI SYLLABLE GO" },
    { 0xA5DE, "VAI SYLLABLE MO" },
    { 0xA5DF, "VAI SYLLABLE NO" },
    { 0xA5E0, "VAI SYLLABLE NYO" },
    { 0xA5E1, "VAI SYLLABLE E" },
    { 0xA5E2, "VAI SYLLABLE EN" },
    { 0xA5E3, "VAI SYLLABLE NGEN" },
    { 0xA5E4, "VAI SYLLABLE HE" },
    { 0xA5E5, "VAI SYLLABLE HEN" },
    { 0xA5E6, "VAI SYLLABLE WE" },
    { 0xA5E7, "VAI SYLLABLE WEN" },
    { 0xA5E8, "VAI SYLLABLE PE" },
    { 0xA5E9, "VAI SYLLABLE BHE" },
    { 0xA5EA, "VAI SYLLABLE BE" },
    { 0xA5EB, "VAI SYLLABLE MBE" },
    { 0xA5EC, "VAI SYLLABLE KPE" },
    { 0xA5ED, "VAI SYLLABLE KPEN" },
    { 0xA5EE, "VAI SYLLABLE MGBE" },
    { 0xA5EF, "VAI SYLLABLE GBE" },
    { 0xA5F0, "VAI SYLLABLE GBEN" },
    { 0xA5F1, "VAI SYLLABLE FE" },
    { 0xA5F2, "VAI SYLLABLE VE" },
    { 0xA5F3, "VAI SYLLABLE TE" },
    { 0xA5F4, "VAI SYLLABLE THE" },
    { 0xA5F5, "VAI SYLLABLE DHE" },
    { 0xA5F6, "VAI SYLLABLE DHHE" },
    { 0xA5F7, "VAI SYLLABLE LE" },
    { 0xA5F8, "VAI SYLLABLE RE" },
    { 0xA5F9, "VAI SYLLABLE DE" },
    { 0xA5FA, "VAI SYLLABLE NDE" },
    { 0xA5FB, "VAI SYLLABLE SE" },
    { 0xA5FC, "VAI SYLLABLE SHE" },
    { 0xA5FD, "VAI SYLLABLE ZE" },
    { 0xA5FE, "VAI SYLLABLE ZHE" },
    { 0xA5FF, "VAI SYLLABLE CE" },
    { 0xA600, "VAI SYLLABLE JE" },
    { 0xA601, "VAI SYLLABLE NJE" },
    { 0xA602, "VAI SYLLABLE YE" },
    { 0xA603, "VAI SYLLABLE KE" },
    { 0xA604, "VAI SYLLABLE NGGE" },
    { 0xA605, "VAI SYLLABLE NGGEN" },
    { 0xA606, "VAI SYLLABLE GE" },
    { 0xA607, "VAI SYLLABLE GEN" },
    { 0xA608, "VAI SYLLABLE ME" },
    { 0xA609, "VAI SYLLABLE NE" },
    { 0xA60A, "VAI SYLLABLE NYE" },
    { 0xA60B, "VAI SYLLABLE NG" },
    { 0xA60C, "VAI SYLLABLE LENGTHENER" },
    { 0xA60D, "VAI COMMA" },
    { 0xA60E, "VAI FULL STOP" },
    { 0xA60F, "VAI QUESTION MARK" },
    { 0xA610, "VAI SYLLABLE NDOLE FA" },
    { 0xA611, "VAI SYLLABLE NDOLE KA" },
    { 0xA612, "VAI SYLLABLE NDOLE SOO" },
    { 0xA613, "VAI SYMBOL FEENG" },
    { 0xA614, "VAI SYMBOL KEENG" },
    { 0xA615, "VAI SYMBOL TING" },
    { 0xA616, "VAI SYMBOL NII" },
    { 0xA617, "VAI SYMBOL BANG" },
    { 0xA618, "VAI SYMBOL FAA" },
    { 0xA619, "VAI SYMBOL TAA" },
    { 0xA61A, "VAI SYMBOL DANG" },
    { 0xA61B, "VAI SYMBOL DOONG" },
    { 0xA61C, "VAI SYMBOL KUNG" },
    { 0xA61D, "VAI SYMBOL TONG" },
    { 0xA61E, "VAI SYMBOL DO-O" },
    { 0xA61F, "VAI SYMBOL JONG" },
    { 0xA620, "VAI DIGIT ZERO" },
    { 0xA621, "VAI DIGIT ONE" },
    { 0xA622, "VAI DIGIT TWO" },
    { 0xA623, "VAI DIGIT THREE" },
    { 0xA624, "VAI DIGIT FOUR" },
    { 0xA625, "VAI DIGIT FIVE" },
    { 0xA626, "VAI DIGIT SIX" },
    { 0xA627, "VAI DIGIT SEVEN" },
    { 0xA628, "VAI DIGIT EIGHT" },
    { 0xA629, "VAI DIGIT NINE" },
    { 0xA62A, "VAI SYLLABLE NDOLE MA" },
    { 0xA62B, "VAI SYLLABLE NDOLE DO" },
    { 0xA640, "CYRILLIC CAPITAL LETTER ZEMLYA" },
    { 0xA641, "CYRILLIC SMALL LETTER ZEMLYA" },
    { 0xA642, "CYRILLIC CAPITAL LETTER DZELO" },
    { 0xA643, "CYRILLIC SMALL LETTER DZELO" },
    { 0xA644, "CYRILLIC CAPITAL LETTER REVERSED DZE" },
    { 0xA645, "CYRILLIC SMALL LETTER REVERSED DZE" },
    { 0xA646, "CYRILLIC CAPITAL LETTER IOTA" },
    { 0xA647, "CYRILLIC SMALL LETTER IOTA" },
    { 0xA648, "CYRILLIC CAPITAL LETTER DJERV" },
    { 0xA649, "CYRILLIC SMALL LETTER DJERV" },
    { 0xA64A, "CYRILLIC CAPITAL LETTER MONOGRAPH UK" },
    { 0xA64B, "CYRILLIC SMALL LETTER MONOGRAPH UK" },
    { 0xA64C, "CYRILLIC CAPITAL LETTER BROAD OMEGA" },
    { 0xA64D, "CYRILLIC SMALL LETTER BROAD OMEGA" },
    { 0xA64E, "CYRILLIC CAPITAL LETTER NEUTRAL YER" },
    { 0xA64F, "CYRILLIC SMALL LETTER NEUTRAL YER" },
    { 0xA650, "CYRILLIC CAPITAL LETTER YERU WITH BACK YER" },
    { 0xA651, "CYRILLIC SMALL LETTER YERU WITH BACK YER" },
    { 0xA652, "CYRILLIC CAPITAL LETTER IOTIFIED YAT" },
    { 0xA653, "CYRILLIC SMALL LETTER IOTIFIED YAT" },
    { 0xA654, "CYRILLIC CAPITAL LETTER REVERSED YU" },
    { 0xA655, "CYRILLIC SMALL LETTER REVERSED YU" },
    { 0xA656, "CYRILLIC CAPITAL LETTER IOTIFIED A" },
    { 0xA657, "CYRILLIC SMALL LETTER IOTIFIED A" },
    { 0xA658, "CYRILLIC CAPITAL LETTER CLOSED LITTLE YUS" },
    { 0xA659, "CYRILLIC SMALL LETTER CLOSED LITTLE YUS" },
    { 0xA65A, "CYRILLIC CAPITAL LETTER BLENDED YUS" },
    { 0xA65B, "CYRILLIC SMALL LETTER BLENDED YUS" },
    { 0xA65C, "CYRILLIC CAPITAL LETTER IOTIFIED CLOSED LITTLE YUS" },
    { 0xA65D, "CYRILLIC SMALL LETTER IOTIFIED CLOSED LITTLE YUS" },
    { 0xA65E, "CYRILLIC CAPITAL LETTER YN" },
    { 0xA65F, "CYRILLIC SMALL LETTER YN" },
    { 0xA660, "CYRILLIC CAPITAL LETTER REVERSED TSE" },
    { 0xA661, "CYRILLIC SMALL LETTER REVERSED TSE" },
    { 0xA662, "CYRILLIC CAPITAL LETTER SOFT DE" },
    { 0xA663, "CYRILLIC SMALL LETTER SOFT DE" },
    { 0xA664, "CYRILLIC CAPITAL LETTER SOFT EL" },
    { 0xA665, "CYRILLIC SMALL LETTER SOFT EL" },
    { 0xA666, "CYRILLIC CAPITAL LETTER SOFT EM" },
    { 0xA667, "CYRILLIC SMALL LETTER SOFT EM" },
    { 0xA668, "CYRILLIC CAPITAL LETTER MONOCULAR O" },
    { 0xA669, "CYRILLIC SMALL LETTER MONOCULAR O" },
    { 0xA66A, "CYRILLIC CAPITAL LETTER BINOCULAR O" },
    { 0xA66B, "CYRILLIC SMALL LETTER BINOCULAR O" },
    { 0xA66C, "CYRILLIC CAPITAL LETTER DOUBLE MONOCULAR O" },
    { 0xA66D, "CYRILLIC SMALL LETTER DOUBLE MONOCULAR O" },
    { 0xA66E, "CYRILLIC LETTER MULTIOCULAR O" },
    { 0xA66F, "COMBINING CYRILLIC VZMET" },
    { 0xA670, "COMBINING CYRILLIC TEN MILLIONS SIGN" },
    { 0xA671, "COMBINING CYRILLIC HUNDRED MILLIONS SIGN" },
    { 0xA672, "COMBINING CYRILLIC THOUSAND MILLIONS SIGN" },
    { 0xA673, "SLAVONIC ASTERISK" },
    { 0xA674, "COMBINING CYRILLIC LETTER UKRAINIAN IE" },
    { 0xA675, "COMBINING CYRILLIC LETTER I" },
    { 0xA676, "COMBINING CYRILLIC LETTER YI" },
    { 0xA677, "COMBINING CYRILLIC LETTER U" },
    { 0xA678, "COMBINING CYRILLIC LETTER HARD SIGN" },
    { 0xA679, "COMBINING CYRILLIC LETTER YERU" },
    { 0xA67A, "COMBINING CYRILLIC LETTER SOFT SIGN" },
    { 0xA67B, "COMBINING CYRILLIC LETTER OMEGA" },
    { 0xA67C, "COMBINING CYRILLIC KAVYKA" },
    { 0xA67D, "COMBINING CYRILLIC PAYEROK" },
    { 0xA67E, "CYRILLIC KAVYKA" },
    { 0xA67F, "CYRILLIC PAYEROK" },
    { 0xA680, "CYRILLIC CAPITAL LETTER DWE" },
    { 0xA681, "CYRILLIC SMALL LETTER DWE" },
    { 0xA682, "CYRILLIC CAPITAL LETTER DZWE" },
    { 0xA683, "CYRILLIC SMALL LETTER DZWE" },
    { 0xA684, "CYRILLIC CAPITAL LETTER ZHWE" },
    { 0xA685, "CYRILLIC SMALL LETTER ZHWE" },
    { 0xA686, "CYRILLIC CAPITAL LETTER CCHE" },
    { 0xA687, "CYRILLIC SMALL LETTER CCHE" },
    { 0xA688, "CYRILLIC CAPITAL LETTER DZZE" },
    { 0xA689, "CYRILLIC SMALL LETTER DZZE" },
    { 0xA68A, "CYRILLIC CAPITAL LETTER TE WITH MIDDLE HOOK" },
    { 0xA68B, "CYRILLIC SMALL LETTER TE WITH MIDDLE HOOK" },
    { 0xA68C, "CYRILLIC CAPITAL LETTER TWE" },
    { 0xA68D, "CYRILLIC SMALL LETTER TWE" },
    { 0xA68E, "CYRILLIC CAPITAL LETTER TSWE" },
    { 0xA68F, "CYRILLIC SMALL LETTER TSWE" },
    { 0xA690, "CYRILLIC CAPITAL LETTER TSSE" },
    { 0xA691, "CYRILLIC SMALL LETTER TSSE" },
    { 0xA692, "CYRILLIC CAPITAL LETTER TCHE" },
    { 0xA693, "CYRILLIC SMALL LETTER TCHE" },
    { 0xA694, "CYRILLIC CAPITAL LETTER HWE" },
    { 0xA695, "CYRILLIC SMALL LETTER HWE" },
    { 0xA696, "CYRILLIC CAPITAL LETTER SHWE" },
    { 0xA697, "CYRILLIC SMALL LETTER SHWE" },
    { 0xA698, "CYRILLIC CAPITAL LETTER DOUBLE O" },
    { 0xA699, "CYRILLIC SMALL LETTER DOUBLE O" },
    { 0xA69A, "CYRILLIC CAPITAL LETTER CROSSED O" },
    { 0xA69B, "CYRILLIC SMALL LETTER CROSSED O" },
    { 0xA69C, "MODIFIER LETTER CYRILLIC HARD SIGN" },
    { 0xA69D, "MODIFIER LETTER CYRILLIC SOFT SIGN" },
    { 0xA69E, "COMBINING CYRILLIC LETTER EF" },
    { 0xA69F, "COMBINING CYRILLIC LETTER IOTIFIED E" },
    { 0xA6A0, "BAMUM LETTER A" },
    { 0xA6A1, "BAMUM LETTER KA" },
    { 0xA6A2, "BAMUM LETTER U" },
    { 0xA6A3, "BAMUM LETTER KU" },
    { 0xA6A4, "BAMUM LETTER EE" },
    { 0xA6A5, "BAMUM LETTER REE" },
    { 0xA6A6, "BAMUM LETTER TAE" },
    { 0xA6A7, "BAMUM LETTER O" },
    { 0xA6A8, "BAMUM LETTER NYI" },
    { 0xA6A9, "BAMUM LETTER I" },
    { 0xA6AA, "BAMUM LETTER LA" },
    { 0xA6AB, "BAMUM LETTER PA" },
    { 0xA6AC, "BAMUM LETTER RII" },
    { 0xA6AD, "BAMUM LETTER RIEE" },
    { 0xA6AE, "BAMUM LETTER LEEEE" },
    { 0xA6AF, "BAMUM LETTER MEEEE" },
    { 0xA6B0, "BAMUM LETTER TAA" },
    { 0xA6B1, "BAMUM LETTER NDAA" },
    { 0xA6B2, "BAMUM LETTER NJAEM" },
    { 0xA6B3, "BAMUM LETTER M" },
    { 0xA6B4, "BAMUM LETTER SUU" },
    { 0xA6B5, "BAMUM LETTER MU" },
    { 0xA6B6, "BAMUM LETTER SHII" },
    { 0xA6B7, "BAMUM LETTER SI" },
    { 0xA6B8, "BAMUM LETTER SHEUX" },
    { 0xA6B9, "BAMUM LETTER SEUX" },
    { 0xA6BA, "BAMUM LETTER KYEE" },
    { 0xA6BB, "BAMUM LETTER KET" },
    { 0xA6BC, "BAMUM LETTER NUAE" },
    { 0xA6BD, "BAMUM LETTER NU" },
    { 0xA6BE, "BAMUM LETTER NJUAE" },
    { 0xA6BF, "BAMUM LETTER YOQ" },
    { 0xA6C0, "BAMUM LETTER SHU" },
    { 0xA6C1, "BAMUM LETTER YUQ" },
    { 0xA6C2, "BAMUM LETTER YA" },
    { 0xA6C3, "BAMUM LETTER NSHA" },
    { 0xA6C4, "BAMUM LETTER KEUX" },
    { 0xA6C5, "BAMUM LETTER PEUX" },
    { 0xA6C6, "BAMUM LETTER NJEE" },
    { 0xA6C7, "BAMUM LETTER NTEE" },
    { 0xA6C8, "BAMUM LETTER PUE" },
    { 0xA6C9, "BAMUM LETTER WUE" },
    { 0xA6CA, "BAMUM LETTER PEE" },
    { 0xA6CB, "BAMUM LETTER FEE" },
    { 0xA6CC, "BAMUM LETTER RU" },
    { 0xA6CD, "BAMUM LETTER LU" },
    { 0xA6CE, "BAMUM LETTER MI" },
    { 0xA6CF, "BAMUM LETTER NI" },
    { 0xA6D0, "BAMUM LETTER REUX" },
    { 0xA6D1, "BAMUM LETTER RAE" },
    { 0xA6D2, "BAMUM LETTER KEN" },
    { 0xA6D3, "BAMUM LETTER NGKWAEN" },
    { 0xA6D4, "BAMUM LETTER NGGA" },
    { 0xA6D5, "BAMUM LETTER NGA" },
    { 0xA6D6, "BAMUM LETTER SHO" },
    { 0xA6D7, "BAMUM LETTER PUAE" },
    { 0xA6D8, "BAMUM LETTER FU" },
    { 0xA6D9, "BAMUM LETTER FOM" },
    { 0xA6DA, "BAMUM LETTER WA" },
    { 0xA6DB, "BAMUM LETTER NA" },
    { 0xA6DC, "BAMUM LETTER LI" },
    { 0xA6DD, "BAMUM LETTER PI" },
    { 0xA6DE, "BAMUM LETTER LOQ" },
    { 0xA6DF, "BAMUM LETTER KO" },
    { 0xA6E0, "BAMUM LETTER MBEN" },
    { 0xA6E1, "BAMUM LETTER REN" },
    { 0xA6E2, "BAMUM LETTER MEN" },
    { 0xA6E3, "BAMUM LETTER MA" },
    { 0xA6E4, "BAMUM LETTER TI" },
    { 0xA6E5, "BAMUM LETTER KI" },
    { 0xA6E6, "BAMUM LETTER MO" },
    { 0xA6E7, "BAMUM LETTER MBAA" },
    { 0xA6E8, "BAMUM LETTER TET" },
    { 0xA6E9, "BAMUM LETTER KPA" },
    { 0xA6EA, "BAMUM LETTER TEN" },
    { 0xA6EB, "BAMUM LETTER NTUU" },
    { 0xA6EC, "BAMUM LETTER SAMBA" },
    { 0xA6ED, "BAMUM LETTER FAAMAE" },
    { 0xA6EE, "BAMUM LETTER KOVUU" },
    { 0xA6EF, "BAMUM LETTER KOGHOM" },
    { 0xA6F0, "BAMUM COMBINING MARK KOQNDON" },
    { 0xA6F1, "BAMUM COMBINING MARK TUKWENTIS" },
    { 0xA6F2, "BAMUM NJAEMLI" },
    { 0xA6F3, "BAMUM FULL STOP" },
    { 0xA6F4, "BAMUM COLON" },
    { 0xA6F5, "BAMUM COMMA" },
    { 0xA6F6, "BAMUM SEMICOLON" },
    { 0xA6F7, "BAMUM QUESTION MARK" },
    { 0xA700, "MODIFIER LETTER CHINESE TONE YIN PING" },
    { 0xA701, "MODIFIER LETTER CHINESE TONE YANG PING" },
    { 0xA702, "MODIFIER LETTER CHINESE TONE YIN SHANG" },
    { 0xA703, "MODIFIER LETTER CHINESE TONE YANG SHANG" },
    { 0xA704, "MODIFIER LETTER CHINESE TONE YIN QU" },
    { 0xA705, "MODIFIER LETTER CHINESE TONE YANG QU" },
    { 0xA706, "MODIFIER LETTER CHINESE TONE YIN RU" },
    { 0xA707, "MODIFIER LETTER CHINESE TONE YANG RU" },
    { 0xA708, "MODIFIER LETTER EXTRA-HIGH DOTTED TONE BAR" },
    { 0xA709, "MODIFIER LETTER HIGH DOTTED TONE BAR" },
    { 0xA70A, "MODIFIER LETTER MID DOTTED TONE BAR" },
    { 0xA70B, "MODIFIER LETTER LOW DOTTED TONE BAR" },
    { 0xA70C, "MODIFIER LETTER EXTRA-LOW DOTTED TONE BAR" },
    { 0xA70D, "MODIFIER LETTER EXTRA-HIGH DOTTED LEFT-STEM TONE BAR" },
    { 0xA70E, "MODIFIER LETTER HIGH DOTTED LEFT-STEM TONE BAR" },
    { 0xA70F, "MODIFIER LETTER MID DOTTED LEFT-STEM TONE BAR" },
    { 0xA710, "MODIFIER LETTER LOW DOTTED LEFT-STEM TONE BAR" },
    { 0xA711, "MODIFIER LETTER EXTRA-LOW DOTTED LEFT-STEM TONE BAR" },
    { 0xA712, "MODIFIER LETTER EXTRA-HIGH LEFT-STEM TONE BAR" },
    { 0xA713, "MODIFIER LETTER HIGH LEFT-STEM TONE BAR" },
    { 0xA714, "MODIFIER LETTER MID LEFT-STEM TONE BAR" },
    { 0xA715, "MODIFIER LETTER LOW LEFT-STEM TONE BAR" },
    { 0xA716, "MODIFIER LETTER EXTRA-LOW LEFT-STEM TONE BAR" },
    { 0xA717, "MODIFIER LETTER DOT VERTICAL BAR" },
    { 0xA718, "MODIFIER LETTER DOT SLASH" },
    { 0xA719, "MODIFIER LETTER DOT HORIZONTAL BAR" },
    { 0xA71A, "MODIFIER LETTER LOWER RIGHT CORNER ANGLE" },
    { 0xA71B, "MODIFIER LETTER RAISED UP ARROW" },
    { 0xA71C, "MODIFIER LETTER RAISED DOWN ARROW" },
    { 0xA71D, "MODIFIER LETTER RAISED EXCLAMATION MARK" },
    { 0xA71E, "MODIFIER LETTER RAISED INVERTED EXCLAMATION MARK" },
    { 0xA71F, "MODIFIER LETTER LOW INVERTED EXCLAMATION MARK" },
    { 0xA720, "MODIFIER LETTER STRESS AND HIGH TONE" },
    { 0xA721, "MODIFIER LETTER STRESS AND LOW TONE" },
    { 0xA722, "LATIN CAPITAL LETTER EGYPTOLOGICAL ALEF" },
    { 0xA723, "LATIN SMALL LETTER EGYPTOLOGICAL ALEF" },
    { 0xA724, "LATIN CAPITAL LETTER EGYPTOLOGICAL AIN" },
    { 0xA725, "LATIN SMALL LETTER EGYPTOLOGICAL AIN" },
    { 0xA726, "LATIN CAPITAL LETTER HENG" },
    { 0xA727, "LATIN SMALL LETTER HENG" },
    { 0xA728, "LATIN CAPITAL LETTER TZ" },
    { 0xA729, "LATIN SMALL LETTER TZ" },
    { 0xA72A, "LATIN CAPITAL LETTER TRESILLO" },
    { 0xA72B, "LATIN SMALL LETTER TRESILLO" },
    { 0xA72C, "LATIN CAPITAL LETTER CUATRILLO" },
    { 0xA72D, "LATIN SMALL LETTER CUATRILLO" },
    { 0xA72E, "LATIN CAPITAL LETTER CUATRILLO WITH COMMA" },
    { 0xA72F, "LATIN SMALL LETTER CUATRILLO WITH COMMA" },
    { 0xA730, "LATIN LETTER SMALL CAPITAL F" },
    { 0xA731, "LATIN LETTER SMALL CAPITAL S" },
    { 0xA732, "LATIN CAPITAL LETTER AA" },
    { 0xA733, "LATIN SMALL LETTER AA" },
    { 0xA734, "LATIN CAPITAL LETTER AO" },
    { 0xA735, "LATIN SMALL LETTER AO" },
    { 0xA736, "LATIN CAPITAL LETTER AU" },
    { 0xA737, "LATIN SMALL LETTER AU" },
    { 0xA738, "LATIN CAPITAL LETTER AV" },
    { 0xA739, "LATIN SMALL LETTER AV" },
    { 0xA73A, "LATIN CAPITAL LETTER AV WITH HORIZONTAL BAR" },
    { 0xA73B, "LATIN SMALL LETTER AV WITH HORIZONTAL BAR" },
    { 0xA73C, "LATIN CAPITAL LETTER AY" },
    { 0xA73D, "LATIN SMALL LETTER AY" },
    { 0xA73E, "LATIN CAPITAL LETTER REVERSED C WITH DOT" },
    { 0xA73F, "LATIN SMALL LETTER REVERSED C WITH DOT" },
    { 0xA740, "LATIN CAPITAL LETTER K WITH STROKE" },
    { 0xA741, "LATIN SMALL LETTER K WITH STROKE" },
    { 0xA742, "LATIN CAPITAL LETTER K WITH DIAGONAL STROKE" },
    { 0xA743, "LATIN SMALL LETTER K WITH DIAGONAL STROKE" },
    { 0xA744, "LATIN CAPITAL LETTER K WITH STROKE AND DIAGONAL STROKE" },
    { 0xA745, "LATIN SMALL LETTER K WITH STROKE AND DIAGONAL STROKE" },
    { 0xA746, "LATIN CAPITAL LETTER BROKEN L" },
    { 0xA747, "LATIN SMALL LETTER BROKEN L" },
    { 0xA748, "LATIN CAPITAL LETTER L WITH HIGH STROKE" },
    { 0xA749, "LATIN SMALL LETTER L WITH HIGH STROKE" },
    { 0xA74A, "LATIN CAPITAL LETTER O WITH LONG STROKE OVERLAY" },
    { 0xA74B, "LATIN SMALL LETTER O WITH LONG STROKE OVERLAY" },
    { 0xA74C, "LATIN CAPITAL LETTER O WITH LOOP" },
    { 0xA74D, "LATIN SMALL LETTER O WITH LOOP" },
    { 0xA74E, "LATIN CAPITAL LETTER OO" },
    { 0xA74F, "LATIN SMALL LETTER OO" },
    { 0xA750, "LATIN CAPITAL LETTER P WITH STROKE THROUGH DESCENDER" },
    { 0xA751, "LATIN SMALL LETTER P WITH STROKE THROUGH DESCENDER" },
    { 0xA752, "LATIN CAPITAL LETTER P WITH FLOURISH" },
    { 0xA753, "LATIN SMALL LETTER P WITH FLOURISH" },
    { 0xA754, "LATIN CAPITAL LETTER P WITH SQUIRREL TAIL" },
    { 0xA755, "LATIN SMALL LETTER P WITH SQUIRREL TAIL" },
    { 0xA756, "LATIN CAPITAL LETTER Q WITH STROKE THROUGH DESCENDER" },
    { 0xA757, "LATIN SMALL LETTER Q WITH STROKE THROUGH DESCENDER" },
    { 0xA758, "LATIN CAPITAL LETTER Q WITH DIAGONAL STROKE" },
    { 0xA759, "LATIN SMALL LETTER Q WITH DIAGONAL STROKE" },
    { 0xA75A, "LATIN CAPITAL LETTER R ROTUNDA" },
    { 0xA75B, "LATIN SMALL LETTER R ROTUNDA" },
    { 0xA75C, "LATIN CAPITAL LETTER RUM ROTUNDA" },
    { 0xA75D, "LATIN SMALL LETTER RUM ROTUNDA" },
    { 0xA75E, "LATIN CAPITAL LETTER V WITH DIAGONAL STROKE" },
    { 0xA75F, "LATIN SMALL LETTER V WITH DIAGONAL STROKE" },
    { 0xA760, "LATIN CAPITAL LETTER VY" },
    { 0xA761, "LATIN SMALL LETTER VY" },
    { 0xA762, "LATIN CAPITAL LETTER VISIGOTHIC Z" },
    { 0xA763, "LATIN SMALL LETTER VISIGOTHIC Z" },
    { 0xA764, "LATIN CAPITAL LETTER THORN WITH STROKE" },
    { 0xA765, "LATIN SMALL LETTER THORN WITH STROKE" },
    { 0xA766, "LATIN CAPITAL LETTER THORN WITH STROKE THROUGH DESCENDER" },
    { 0xA767, "LATIN SMALL LETTER THORN WITH STROKE THROUGH DESCENDER" },
    { 0xA768, "LATIN CAPITAL LETTER VEND" },
    { 0xA769, "LATIN SMALL LETTER VEND" },
    { 0xA76A, "LATIN CAPITAL LETTER ET" },
    { 0xA76B, "LATIN SMALL LETTER ET" },
    { 0xA76C, "LATIN CAPITAL LETTER IS" },
    { 0xA76D, "LATIN SMALL LETTER IS" },
    { 0xA76E, "LATIN CAPITAL LETTER CON" },
    { 0xA76F, "LATIN SMALL LETTER CON" },
    { 0xA770, "MODIFIER LETTER US" },
    { 0xA771, "LATIN SMALL LETTER DUM" },
    { 0xA772, "LATIN SMALL LETTER LUM" },
    { 0xA773, "LATIN SMALL LETTER MUM" },
    { 0xA774, "LATIN SMALL LETTER NUM" },
    { 0xA775, "LATIN SMALL LETTER RUM" },
    { 0xA776, "LATIN LETTER SMALL CAPITAL RUM" },
    { 0xA777, "LATIN SMALL LETTER TUM" },
    { 0xA778, "LATIN SMALL LETTER UM" },
    { 0xA779, "LATIN CAPITAL LETTER INSULAR D" },
    { 0xA77A, "LATIN SMALL LETTER INSULAR D" },
    { 0xA77B, "LATIN CAPITAL LETTER INSULAR F" },
    { 0xA77C, "LATIN SMALL LETTER INSULAR F" },
    { 0xA77D, "LATIN CAPITAL LETTER INSULAR G" },
    { 0xA77E, "LATIN CAPITAL LETTER TURNED INSULAR G" },
    { 0xA77F, "LATIN SMALL LETTER TURNED INSULAR G" },
    { 0xA780, "LATIN CAPITAL LETTER TURNED L" },
    { 0xA781, "LATIN SMALL LETTER TURNED L" },
    { 0xA782, "LATIN CAPITAL LETTER INSULAR R" },
    { 0xA783, "LATIN SMALL LETTER INSULAR R" },
    { 0xA784, "LATIN CAPITAL LETTER INSULAR S" },
    { 0xA785, "LATIN SMALL LETTER INSULAR S" },
    { 0xA786, "LATIN CAPITAL LETTER INSULAR T" },
    { 0xA787, "LATIN SMALL LETTER INSULAR T" },
    { 0xA788, "MODIFIER LETTER LOW CIRCUMFLEX ACCENT" },
    { 0xA789, "MODIFIER LETTER COLON" },
    { 0xA78A, "MODIFIER LETTER SHORT EQUALS SIGN" },
    { 0xA78B, "LATIN CAPITAL LETTER SALTILLO" },
    { 0xA78C, "LATIN SMALL LETTER SALTILLO" },
    { 0xA78D, "LATIN CAPITAL LETTER TURNED H" },
    { 0xA78E, "LATIN SMALL LETTER L WITH RETROFLEX HOOK AND BELT" },
    { 0xA78F, "LATIN LETTER SINOLOGICAL DOT" },
    { 0xA790, "LATIN CAPITAL LETTER N WITH DESCENDER" },
    { 0xA791, "LATIN SMALL LETTER N WITH DESCENDER" },
    { 0xA792, "LATIN CAPITAL LETTER C WITH BAR" },
    { 0xA793, "LATIN SMALL LETTER C WITH BAR" },
    { 0xA794, "LATIN SMALL LETTER C WITH PALATAL HOOK" },
    { 0xA795, "LATIN SMALL LETTER H WITH PALATAL HOOK" },
    { 0xA796, "LATIN CAPITAL LETTER B WITH FLOURISH" },
    { 0xA797, "LATIN SMALL LETTER B WITH FLOURISH" },
    { 0xA798, "LATIN CAPITAL LETTER F WITH STROKE" },
    { 0xA799, "LATIN SMALL LETTER F WITH STROKE" },
    { 0xA79A, "LATIN CAPITAL LETTER VOLAPUK AE" },
    { 0xA79B, "LATIN SMALL LETTER VOLAPUK AE" },
    { 0xA79C, "LATIN CAPITAL LETTER VOLAPUK OE" },
    { 0xA79D, "LATIN SMALL LETTER VOLAPUK OE" },
    { 0xA79E, "LATIN CAPITAL LETTER VOLAPUK UE" },
    { 0xA79F, "LATIN SMALL LETTER VOLAPUK UE" },
    { 0xA7A0, "LATIN CAPITAL LETTER G WITH OBLIQUE STROKE" },
    { 0xA7A1, "LATIN SMALL LETTER G WITH OBLIQUE STROKE" },
    { 0xA7A2, "LATIN CAPITAL LETTER K WITH OBLIQUE STROKE" },
    { 0xA7A3, "LATIN SMALL LETTER K WITH OBLIQUE STROKE" },
    { 0xA7A4, "LATIN CAPITAL LETTER N WITH OBLIQUE STROKE" },
    { 0xA7A5, "LATIN SMALL LETTER N WITH OBLIQUE STROKE" },
    { 0xA7A6, "LATIN CAPITAL LETTER R WITH OBLIQUE STROKE" },
    { 0xA7A7, "LATIN SMALL LETTER R WITH OBLIQUE STROKE" },
    { 0xA7A8, "LATIN CAPITAL LETTER S WITH OBLIQUE STROKE" },
    { 0xA7A9, "LATIN SMALL LETTER S WITH OBLIQUE STROKE" },
    { 0xA7AA, "LATIN CAPITAL LETTER H WITH HOOK" },
    { 0xA7AB, "LATIN CAPITAL LETTER REVERSED OPEN E" },
    { 0xA7AC, "LATIN CAPITAL LETTER SCRIPT G" },
    { 0xA7AD, "LATIN CAPITAL LETTER L WITH BELT" },
    { 0xA7AE, "LATIN CAPITAL LETTER SMALL CAPITAL I" },
    { 0xA7AF, "LATIN LETTER SMALL CAPITAL Q" },
    { 0xA7B0, "LATIN CAPITAL LETTER TURNED K" },
    { 0xA7B1, "LATIN CAPITAL LETTER TURNED T" },
    { 0xA7B2, "LATIN CAPITAL LETTER J WITH CROSSED-TAIL" },
    { 0xA7B3, "LATIN CAPITAL LETTER CHI" },
    { 0xA7B4, "LATIN CAPITAL LETTER BETA" },
    { 0xA7B5, "LATIN SMALL LETTER BETA" },
    { 0xA7B6, "LATIN CAPITAL LETTER OMEGA" },
    { 0xA7B7, "LATIN SMALL LETTER OMEGA" },
    { 0xA7B8, "LATIN CAPITAL LETTER U WITH STROKE" },
    { 0xA7B9, "LATIN SMALL LETTER U WITH STROKE" },
    { 0xA7BA, "LATIN CAPITAL LETTER GLOTTAL A" },
    { 0xA7BB, "LATIN SMALL LETTER GLOTTAL A" },
    { 0xA7BC, "LATIN CAPITAL LETTER GLOTTAL I" },
    { 0xA7BD, "LATIN SMALL LETTER GLOTTAL I" },
    { 0xA7BE, "LATIN CAPITAL LETTER GLOTTAL U" },
    { 0xA7BF, "LATIN SMALL LETTER GLOTTAL U" },
    { 0xA7C0, "LATIN CAPITAL LETTER OLD POLISH O" },
    { 0xA7C1, "LATIN SMALL LETTER OLD POLISH O" },
    { 0xA7C2, "LATIN CAPITAL LETTER ANGLICANA W" },
    { 0xA7C3, "LATIN SMALL LETTER ANGLICANA W" },
    { 0xA7C4, "LATIN CAPITAL LETTER C WITH PALATAL HOOK" },
    { 0xA7C5, "LATIN CAPITAL LETTER S WITH HOOK" },
    { 0xA7C6, "LATIN CAPITAL LETTER Z WITH PALATAL HOOK" },
    { 0xA7C7, "LATIN CAPITAL LETTER D WITH SHORT STROKE OVERLAY" },
    { 0xA7C8, "LATIN SMALL LETTER D WITH SHORT STROKE OVERLAY" },
    { 0xA7C9, "LATIN CAPITAL LETTER S WITH SHORT STROKE OVERLAY" },
    { 0xA7CA, "LATIN SMALL LETTER S WITH SHORT STROKE OVERLAY" },
    { 0xA7D0, "LATIN CAPITAL LETTER CLOSED INSULAR G" },
    { 0xA7D1, "LATIN SMALL LETTER CLOSED INSULAR G" },
    { 0xA7D3, "LATIN SMALL LETTER DOUBLE THORN" },
    { 0xA7D5, "LATIN SMALL LETTER DOUBLE WYNN" },
    { 0xA7D6, "LATIN CAPITAL LETTER MIDDLE SCOTS S" },
    { 0xA7D7, "LATIN SMALL LETTER MIDDLE SCOTS S" },
    { 0xA7D8, "LATIN CAPITAL LETTER SIGMOID S" },
    { 0xA7D9, "LATIN SMALL LETTER SIGMOID S" },
    { 0xA7F2, "MODIFIER LETTER CAPITAL C" },
    { 0xA7F3, "MODIFIER LETTER CAPITAL F" },
    { 0xA7F4, "MODIFIER LETTER CAPITAL Q" },
    { 0xA7F5, "LATIN CAPITAL LETTER REVERSED HALF H" },
    { 0xA7F6, "LATIN SMALL LETTER REVERSED HALF H" },
    { 0xA7F7, "LATIN EPIGRAPHIC LETTER SIDEWAYS I" },
    { 0xA7F8, "MODIFIER LETTER CAPITAL H WITH STROKE" },
    { 0xA7F9, "MODIFIER LETTER SMALL LIGATURE OE" },
    { 0xA7FA, "LATIN LETTER SMALL CAPITAL TURNED M" },
    { 0xA7FB, "LATIN EPIGRAPHIC LETTER REVERSED F" },
    { 0xA7FC, "LATIN EPIGRAPHIC LETTER REVERSED P" },
    { 0xA7FD, "LATIN EPIGRAPHIC LETTER INVERTED M" },
    { 0xA7FE, "LATIN EPIGRAPHIC LETTER I LONGA" },
    { 0xA7FF, "LATIN EPIGRAPHIC LETTER ARCHAIC M" },
    { 0xA800, "SYLOTI NAGRI LETTER A" },
    { 0xA801, "SYLOTI NAGRI LETTER I" },
    { 0xA802, "SYLOTI NAGRI SIGN DVISVARA" },
    { 0xA803, "SYLOTI NAGRI LETTER U" },
    { 0xA804, "SYLOTI NAGRI LETTER E" },
    { 0xA805, "SYLOTI NAGRI LETTER O" },
    { 0xA806, "SYLOTI NAGRI SIGN HASANTA" },
    { 0xA807, "SYLOTI NAGRI LETTER KO" },
    { 0xA808, "SYLOTI NAGRI LETTER KHO" },
    { 0xA809, "SYLOTI NAGRI LETTER GO" },
    { 0xA80A, "SYLOTI NAGRI LETTER GHO" },
    { 0xA80B, "SYLOTI NAGRI SIGN ANUSVARA" },
    { 0xA80C, "SYLOTI NAGRI LETTER CO" },
    { 0xA80D, "SYLOTI NAGRI LETTER CHO" },
    { 0xA80E, "SYLOTI NAGRI LETTER JO" },
    { 0xA80F, "SYLOTI NAGRI LETTER JHO" },
    { 0xA810, "SYLOTI NAGRI LETTER TTO" },
    { 0xA811, "SYLOTI NAGRI LETTER TTHO" },
    { 0xA812, "SYLOTI NAGRI LETTER DDO" },
    { 0xA813, "SYLOTI NAGRI LETTER DDHO" },
    { 0xA814, "SYLOTI NAGRI LETTER TO" },
    { 0xA815, "SYLOTI NAGRI LETTER THO" },
    { 0xA816, "SYLOTI NAGRI LETTER DO" },
    { 0xA817, "SYLOTI NAGRI LETTER DHO" },
    { 0xA818, "SYLOTI NAGRI LETTER NO" },
    { 0xA819, "SYLOTI NAGRI LETTER PO" },
    { 0xA81A, "SYLOTI NAGRI LETTER PHO" },
    { 0xA81B, "SYLOTI NAGRI LETTER BO" },
    { 0xA81C, "SYLOTI NAGRI LETTER BHO" },
    { 0xA81D, "SYLOTI NAGRI LETTER MO" },
    { 0xA81E, "SYLOTI NAGRI LETTER RO" },
    { 0xA81F, "SYLOTI NAGRI LETTER LO" },
    { 0xA820, "SYLOTI NAGRI LETTER RRO" },
    { 0xA821, "SYLOTI NAGRI LETTER SO" },
    { 0xA822, "SYLOTI NAGRI LETTER HO" },
    { 0xA823, "SYLOTI NAGRI VOWEL SIGN A" },
    { 0xA824, "SYLOTI NAGRI VOWEL SIGN I" },
    { 0xA825, "SYLOTI NAGRI VOWEL SIGN U" },
    { 0xA826, "SYLOTI NAGRI VOWEL SIGN E" },
    { 0xA827, "SYLOTI NAGRI VOWEL SIGN OO" },
    { 0xA828, "SYLOTI NAGRI POETRY MARK-1" },
    { 0xA829, "SYLOTI NAGRI POETRY MARK-2" },
    { 0xA82A, "SYLOTI NAGRI POETRY MARK-3" },
    { 0xA82B, "SYLOTI NAGRI POETRY MARK-4" },
    { 0xA82C, "SYLOTI NAGRI SIGN ALTERNATE HASANTA" },
    { 0xA830, "NORTH INDIC FRACTION ONE QUARTER" },
    { 0xA831, "NORTH INDIC FRACTION ONE HALF" },
    { 0xA832, "NORTH INDIC FRACTION THREE QUARTERS" },
    { 0xA833, "NORTH INDIC FRACTION ONE SIXTEENTH" },
    { 0xA834, "NORTH INDIC FRACTION ONE EIGHTH" },
    { 0xA835, "NORTH INDIC FRACTION THREE SIXTEENTHS" },
    { 0xA836, "NORTH INDIC QUARTER MARK" },
    { 0xA837, "NORTH INDIC PLACEHOLDER MARK" },
    { 0xA838, "NORTH INDIC RUPEE MARK" },
    { 0xA839, "NORTH INDIC QUANTITY MARK" },
    { 0xA840, "PHAGS-PA LETTER KA" },
    { 0xA841, "PHAGS-PA LETTER KHA" },
    { 0xA842, "PHAGS-PA LETTER GA" },
    { 0xA843, "PHAGS-PA LETTER NGA" },
    { 0xA844, "PHAGS-PA LETTER CA" },
    { 0xA845, "PHAGS-PA LETTER CHA" },
    { 0xA846, "PHAGS-PA LETTER JA" },
    { 0xA847, "PHAGS-PA LETTER NYA" },
    { 0xA848, "PHAGS-PA LETTER TA" },
    { 0xA849, "PHAGS-PA LETTER THA" },
    { 0xA84A, "PHAGS-PA LETTER DA" },
    { 0xA84B, "PHAGS-PA LETTER NA" },
    { 0xA84C, "PHAGS-PA LETTER PA" },
    { 0xA84D, "PHAGS-PA LETTER PHA" },
    { 0xA84E, "PHAGS-PA LETTER BA" },
    { 0xA84F, "PHAGS-PA LETTER MA" },
    { 0xA850, "PHAGS-PA LETTER TSA" },
    { 0xA851, "PHAGS-PA LETTER TSHA" },
    { 0xA852, "PHAGS-PA LETTER DZA" },
    { 0xA853, "PHAGS-PA LETTER WA" },
    { 0xA854, "PHAGS-PA LETTER ZHA" },
    { 0xA855, "PHAGS-PA LETTER ZA" },
    { 0xA856, "PHAGS-PA LETTER SMALL A" },
    { 0xA857, "PHAGS-PA LETTER YA" },
    { 0xA858, "PHAGS-PA LETTER RA" },
    { 0xA859, "PHAGS-PA LETTER LA" },
    { 0xA85A, "PHAGS-PA LETTER SHA" },
    { 0xA85B, "PHAGS-PA LETTER SA" },
    { 0xA85C, "PHAGS-PA LETTER HA" },
    { 0xA85D, "PHAGS-PA LETTER A" },
    { 0xA85E, "PHAGS-PA LETTER I" },
    { 0xA85F, "PHAGS-PA LETTER U" },
    { 0xA860, "PHAGS-PA LETTER E" },
    { 0xA861, "PHAGS-PA LETTER O" },
    { 0xA862, "PHAGS-PA LETTER QA" },
    { 0xA863, "PHAGS-PA LETTER XA" },
    { 0xA864, "PHAGS-PA LETTER FA" },
    { 0xA865, "PHAGS-PA LETTER GGA" },
    { 0xA866, "PHAGS-PA LETTER EE" },
    { 0xA867, "PHAGS-PA SUBJOINED LETTER WA" },
    { 0xA868, "PHAGS-PA SUBJOINED LETTER YA" },
    { 0xA869, "PHAGS-PA LETTER TTA" },
    { 0xA86A, "PHAGS-PA LETTER TTHA" },
    { 0xA86B, "PHAGS-PA LETTER DDA" },
    { 0xA86C, "PHAGS-PA LETTER NNA" },
    { 0xA86D, "PHAGS-PA LETTER ALTERNATE YA" },
    { 0xA86E, "PHAGS-PA LETTER VOICELESS SHA" },
    { 0xA86F, "PHAGS-PA LETTER VOICED HA" },
    { 0xA870, "PHAGS-PA LETTER ASPIRATED FA" },
    { 0xA871, "PHAGS-PA SUBJOINED LETTER RA" },
    { 0xA872, "PHAGS-PA SUPERFIXED LETTER RA" },
    { 0xA873, "PHAGS-PA LETTER CANDRABINDU" },
    { 0xA874, "PHAGS-PA SINGLE HEAD MARK" },
    { 0xA875, "PHAGS-PA DOUBLE HEAD MARK" },
    { 0xA876, "PHAGS-PA MARK SHAD" },
    { 0xA877, "PHAGS-PA MARK DOUBLE SHAD" },
    { 0xA880, "SAURASHTRA SIGN ANUSVARA" },
    { 0xA881, "SAURASHTRA SIGN VISARGA" },
    { 0xA882, "SAURASHTRA LETTER A" },
    { 0xA883, "SAURASHTRA LETTER AA" },
    { 0xA884, "SAURASHTRA LETTER I" },
    { 0xA885, "SAURASHTRA LETTER II" },
    { 0xA886, "SAURASHTRA LETTER U" },
    { 0xA887, "SAURASHTRA LETTER UU" },
    { 0xA888, "SAURASHTRA LETTER VOCALIC R" },
    { 0xA889, "SAURASHTRA LETTER VOCALIC RR" },
    { 0xA88A, "SAURASHTRA LETTER VOCALIC L" },
    { 0xA88B, "SAURASHTRA LETTER VOCALIC LL" },
    { 0xA88C, "SAURASHTRA LETTER E" },
    { 0xA88D, "SAURASHTRA LETTER EE" },
    { 0xA88E, "SAURASHTRA LETTER AI" },
    { 0xA88F, "SAURASHTRA LETTER O" },
    { 0xA890, "SAURASHTRA LETTER OO" },
    { 0xA891, "SAURASHTRA LETTER AU" },
    { 0xA892, "SAURASHTRA LETTER KA" },
    { 0xA893, "SAURASHTRA LETTER KHA" },
    { 0xA894, "SAURASHTRA LETTER GA" },
    { 0xA895, "SAURASHTRA LETTER GHA" },
    { 0xA896, "SAURASHTRA LETTER NGA" },
    { 0xA897, "SAURASHTRA LETTER CA" },
    { 0xA898, "SAURASHTRA LETTER CHA" },
    { 0xA899, "SAURASHTRA LETTER JA" },
    { 0xA89A, "SAURASHTRA LETTER JHA" },
    { 0xA89B, "SAURASHTRA LETTER NYA" },
    { 0xA89C, "SAURASHTRA LETTER TTA" },
    { 0xA89D, "SAURASHTRA LETTER TTHA" },
    { 0xA89E, "SAURASHTRA LETTER DDA" },
    { 0xA89F, "SAURASHTRA LETTER DDHA" },
    { 0xA8A0, "SAURASHTRA LETTER NNA" },
    { 0xA8A1, "SAURASHTRA LETTER TA" },
    { 0xA8A2, "SAURASHTRA LETTER THA" },
    { 0xA8A3, "SAURASHTRA LETTER DA" },
    { 0xA8A4, "SAURASHTRA LETTER DHA" },
    { 0xA8A5, "SAURASHTRA LETTER NA" },
    { 0xA8A6, "SAURASHTRA LETTER PA" },
    { 0xA8A7, "SAURASHTRA LETTER PHA" },
    { 0xA8A8, "SAURASHTRA LETTER BA" },
    { 0xA8A9, "SAURASHTRA LETTER BHA" },
    { 0xA8AA, "SAURASHTRA LETTER MA" },
    { 0xA8AB, "SAURASHTRA LETTER YA" },
    { 0xA8AC, "SAURASHTRA LETTER RA" },
    { 0xA8AD, "SAURASHTRA LETTER LA" },
    { 0xA8AE, "SAURASHTRA LETTER VA" },
    { 0xA8AF, "SAURASHTRA LETTER SHA" },
    { 0xA8B0, "SAURASHTRA LETTER SSA" },
    { 0xA8B1, "SAURASHTRA LETTER SA" },
    { 0xA8B2, "SAURASHTRA LETTER HA" },
    { 0xA8B3, "SAURASHTRA LETTER LLA" },
    { 0xA8B4, "SAURASHTRA CONSONANT SIGN HAARU" },
    { 0xA8B5, "SAURASHTRA VOWEL SIGN AA" },
    { 0xA8B6, "SAURASHTRA VOWEL SIGN I" },
    { 0xA8B7, "SAURASHTRA VOWEL SIGN II" },
    { 0xA8B8, "SAURASHTRA VOWEL SIGN U" },
    { 0xA8B9, "SAURASHTRA VOWEL SIGN UU" },
    { 0xA8BA, "SAURASHTRA VOWEL SIGN VOCALIC R" },
    { 0xA8BB, "SAURASHTRA VOWEL SIGN VOCALIC RR" },
    { 0xA8BC, "SAURASHTRA VOWEL SIGN VOCALIC L" },
    { 0xA8BD, "SAURASHTRA VOWEL SIGN VOCALIC LL" },
    { 0xA8BE, "SAURASHTRA VOWEL SIGN E" },
    { 0xA8BF, "SAURASHTRA VOWEL SIGN EE" },
    { 0xA8C0, "SAURASHTRA VOWEL SIGN AI" },
    { 0xA8C1, "SAURASHTRA VOWEL SIGN O" },
    { 0xA8C2, "SAURASHTRA VOWEL SIGN OO" },
    { 0xA8C3, "SAURASHTRA VOWEL SIGN AU" },
    { 0xA8C4, "SAURASHTRA SIGN VIRAMA" },
    { 0xA8C5, "SAURASHTRA SIGN CANDRABINDU" },
    { 0xA8CE, "SAURASHTRA DANDA" },
    { 0xA8CF, "SAURASHTRA DOUBLE DANDA" },
    { 0xA8D0, "SAURASHTRA DIGIT ZERO" },
    { 0xA8D1, "SAURASHTRA DIGIT ONE" },
    { 0xA8D2, "SAURASHTRA DIGIT TWO" },
    { 0xA8D3, "SAURASHTRA DIGIT THREE" },
    { 0xA8D4, "SAURASHTRA DIGIT FOUR" },
    { 0xA8D5, "SAURASHTRA DIGIT FIVE" },
    { 0xA8D6, "SAURASHTRA DIGIT SIX" },
    { 0xA8D7, "SAURASHTRA DIGIT SEVEN" },
    { 0xA8D8, "SAURASHTRA DIGIT EIGHT" },
    { 0xA8D9, "SAURASHTRA DIGIT NINE" },
    { 0xA8E0, "COMBINING DEVANAGARI DIGIT ZERO" },
    { 0xA8E1, "COMBINING DEVANAGARI DIGIT ONE" },
    { 0xA8E2, "COMBINING DEVANAGARI DIGIT TWO" },
    { 0xA8E3, "COMBINING DEVANAGARI DIGIT THREE" },
    { 0xA8E4, "COMBINING DEVANAGARI DIGIT FOUR" },
    { 0xA8E5, "COMBINING DEVANAGARI DIGIT FIVE" },
    { 0xA8E6, "COMBINING DEVANAGARI DIGIT SIX" },
    { 0xA8E7, "COMBINING DEVANAGARI DIGIT SEVEN" },
    { 0xA8E8, "COMBINING DEVANAGARI DIGIT EIGHT" },
    { 0xA8E9, "COMBINING DEVANAGARI DIGIT NINE" },
    { 0xA8EA, "COMBINING DEVANAGARI LETTER A" },
    { 0xA8EB, "COMBINING DEVANAGARI LETTER U" },
    { 0xA8EC, "COMBINING DEVANAGARI LETTER KA" },
    { 0xA8ED, "COMBINING DEVANAGARI LETTER NA" },
    { 0xA8EE, "COMBINING DEVANAGARI LETTER PA" },
    { 0xA8EF, "COMBINING DEVANAGARI LETTER RA" },
    { 0xA8F0, "COMBINING DEVANAGARI LETTER VI" },
    { 0xA8F1, "COMBINING DEVANAGARI SIGN AVAGRAHA" },
    { 0xA8F2, "DEVANAGARI SIGN SPACING CANDRABINDU" },
    { 0xA8F3, "DEVANAGARI SIGN CANDRABINDU VIRAMA" },
    { 0xA8F4, "DEVANAGARI SIGN DOUBLE CANDRABINDU VIRAMA" },
    { 0xA8F5, "DEVANAGARI SIGN CANDRABINDU TWO" },
    { 0xA8F6, "DEVANAGARI SIGN CANDRABINDU THREE" },
    { 0xA8F7, "DEVANAGARI SIGN CANDRABINDU AVAGRAHA" },
    { 0xA8F8, "DEVANAGARI SIGN PUSHPIKA" },
    { 0xA8F9, "DEVANAGARI GAP FILLER" },
    { 0xA8FA, "DEVANAGARI CARET" },
    { 0xA8FB, "DEVANAGARI HEADSTROKE" },
    { 0xA8FC, "DEVANAGARI SIGN SIDDHAM" },
    { 0xA8FD, "DEVANAGARI JAIN OM" },
    { 0xA8FE, "DEVANAGARI LETTER AY" },
    { 0xA8FF, "DEVANAGARI VOWEL SIGN AY" },
    { 0xA900, "KAYAH LI DIGIT ZERO" },
    { 0xA901, "KAYAH LI DIGIT ONE" },
    { 0xA902, "KAYAH LI DIGIT TWO" },
    { 0xA903, "KAYAH LI DIGIT THREE" },
    { 0xA904, "KAYAH LI DIGIT FOUR" },
    { 0xA905, "KAYAH LI DIGIT FIVE" },
    { 0xA906, "KAYAH LI DIGIT SIX" },
    { 0xA907, "KAYAH LI DIGIT SEVEN" },
    { 0xA908, "KAYAH LI DIGIT EIGHT" },
    { 0xA909, "KAYAH LI DIGIT NINE" },
    { 0xA90A, "KAYAH LI LETTER KA" },
    { 0xA90B, "KAYAH LI LETTER KHA" },
    { 0xA90C, "KAYAH LI LETTER GA" },
    { 0xA90D, "KAYAH LI LETTER NGA" },
    { 0xA90E, "KAYAH LI LETTER SA" },
    { 0xA90F, "KAYAH LI LETTER SHA" },
    { 0xA910, "KAYAH LI LETTER ZA" },
    { 0xA911, "KAYAH LI LETTER NYA" },
    { 0xA912, "KAYAH LI LETTER TA" },
    { 0xA913, "KAYAH LI LETTER HTA" },
    { 0xA914, "KAYAH LI LETTER NA" },
    { 0xA915, "KAYAH LI LETTER PA" },
    { 0xA916, "KAYAH LI LETTER PHA" },
    { 0xA917, "KAYAH LI LETTER MA" },
    { 0xA918, "KAYAH LI LETTER DA" },
    { 0xA919, "KAYAH LI LETTER BA" },
    { 0xA91A, "KAYAH LI LETTER RA" },
    { 0xA91B, "KAYAH LI LETTER YA" },
    { 0xA91C, "KAYAH LI LETTER LA" },
    { 0xA91D, "KAYAH LI LETTER WA" },
    { 0xA91E, "KAYAH LI LETTER THA" },
    { 0xA91F, "KAYAH LI LETTER HA" },
    { 0xA920, "KAYAH LI LETTER VA" },
    { 0xA921, "KAYAH LI LETTER CA" },
    { 0xA922, "KAYAH LI LETTER A" },
    { 0xA923, "KAYAH LI LETTER OE" },
    { 0xA924, "KAYAH LI LETTER I" },
    { 0xA925, "KAYAH LI LETTER OO" },
    { 0xA926, "KAYAH LI VOWEL UE" },
    { 0xA927, "KAYAH LI VOWEL E" },
    { 0xA928, "KAYAH LI VOWEL U" },
    { 0xA929, "KAYAH LI VOWEL EE" },
    { 0xA92A, "KAYAH LI VOWEL O" },
    { 0xA92B, "KAYAH LI TONE PLOPHU" },
    { 0xA92C, "KAYAH LI TONE CALYA" },
    { 0xA92D, "KAYAH LI TONE CALYA PLOPHU" },
    { 0xA92E, "KAYAH LI SIGN CWI" },
    { 0xA92F, "KAYAH LI SIGN SHYA" },
    { 0xA930, "REJANG LETTER KA" },
    { 0xA931, "REJANG LETTER GA" },
    { 0xA932, "REJANG LETTER NGA" },
    { 0xA933, "REJANG LETTER TA" },
    { 0xA934, "REJANG LETTER DA" },
    { 0xA935, "REJANG LETTER NA" },
    { 0xA936, "REJANG LETTER PA" },
    { 0xA937, "REJANG LETTER BA" },
    { 0xA938, "REJANG LETTER MA" },
    { 0xA939, "REJANG LETTER CA" },
    { 0xA93A, "REJANG LETTER JA" },
    { 0xA93B, "REJANG LETTER NYA" },
    { 0xA93C, "REJANG LETTER SA" },
    { 0xA93D, "REJANG LETTER RA" },
    { 0xA93E, "REJANG LETTER LA" },
    { 0xA93F, "REJANG LETTER YA" },
    { 0xA940, "REJANG LETTER WA" },
    { 0xA941, "REJANG LETTER HA" },
    { 0xA942, "REJANG LETTER MBA" },
    { 0xA943, "REJANG LETTER NGGA" },
    { 0xA944, "REJANG LETTER NDA" },
    { 0xA945, "REJANG LETTER NYJA" },
    { 0xA946, "REJANG LETTER A" },
    { 0xA947, "REJANG VOWEL SIGN I" },
    { 0xA948, "REJANG VOWEL SIGN U" },
    { 0xA949, "REJANG VOWEL SIGN E" },
    { 0xA94A, "REJANG VOWEL SIGN AI" },
    { 0xA94B, "REJANG VOWEL SIGN O" },
    { 0xA94C, "REJANG VOWEL SIGN AU" },
    { 0xA94D, "REJANG VOWEL SIGN EU" },
    { 0xA94E, "REJANG VOWEL SIGN EA" },
    { 0xA94F, "REJANG CONSONANT SIGN NG" },
    { 0xA950, "REJANG CONSONANT SIGN N" },
    { 0xA951, "REJANG CONSONANT SIGN R" },
    { 0xA952, "REJANG CONSONANT SIGN H" },
    { 0xA953, "REJANG VIRAMA" },
    { 0xA95F, "REJANG SECTION MARK" },
    { 0xA960, "HANGUL CHOSEONG TIKEUT-MIEUM" },
    { 0xA961, "HANGUL CHOSEONG TIKEUT-PIEUP" },
    { 0xA962, "HANGUL CHOSEONG TIKEUT-SIOS" },
    { 0xA963, "HANGUL CHOSEONG TIKEUT-CIEUC" },
    { 0xA964, "HANGUL CHOSEONG RIEUL-KIYEOK" },
    { 0xA965, "HANGUL CHOSEONG RIEUL-SSANGKIYEOK" },
    { 0xA966, "HANGUL CHOSEONG RIEUL-TIKEUT" },
    { 0xA967, "HANGUL CHOSEONG RIEUL-SSANGTIKEUT" },
    { 0xA968, "HANGUL CHOSEONG RIEUL-MIEUM" },
    { 0xA969, "HANGUL CHOSEONG RIEUL-PIEUP" },
    { 0xA96A, "HANGUL CHOSEONG RIEUL-SSANGPIEUP" },
    { 0xA96B, "HANGUL CHOSEONG RIEUL-KAPYEOUNPIEUP" },
    { 0xA96C, "HANGUL CHOSEONG RIEUL-SIOS" },
    { 0xA96D, "HANGUL CHOSEONG RIEUL-CIEUC" },
    { 0xA96E, "HANGUL CHOSEONG RIEUL-KHIEUKH" },
    { 0xA96F, "HANGUL CHOSEONG MIEUM-KIYEOK" },
    { 0xA970, "HANGUL CHOSEONG MIEUM-TIKEUT" },
    { 0xA971, "HANGUL CHOSEONG MIEUM-SIOS" },
    { 0xA972, "HANGUL CHOSEONG PIEUP-SIOS-THIEUTH" },
    { 0xA973, "HANGUL CHOSEONG PIEUP-KHIEUKH" },
    { 0xA974, "HANGUL CHOSEONG PIEUP-HIEUH" },
    { 0xA975, "HANGUL CHOSEONG SSANGSIOS-PIEUP" },
    { 0xA976, "HANGUL CHOSEONG IEUNG-RIEUL" },
    { 0xA977, "HANGUL CHOSEONG IEUNG-HIEUH" },
    { 0xA978, "HANGUL CHOSEONG SSANGCIEUC-HIEUH" },
    { 0xA979, "HANGUL CHOSEONG SSANGTHIEUTH" },
    { 0xA97A, "HANGUL CHOSEONG PHIEUPH-HIEUH" },
    { 0xA97B, "HANGUL CHOSEONG HIEUH-SIOS" },
    { 0xA97C, "HANGUL CHOSEONG SSANGYEORINHIEUH" },
    { 0xA980, "JAVANESE SIGN PANYANGGA" },
    { 0xA981, "JAVANESE SIGN CECAK" },
    { 0xA982, "JAVANESE SIGN LAYAR" },
    { 0xA983, "JAVANESE SIGN WIGNYAN" },
    { 0xA984, "JAVANESE LETTER A" },
    { 0xA985, "JAVANESE LETTER I KAWI" },
    { 0xA986, "JAVANESE LETTER I" },
    { 0xA987, "JAVANESE LETTER II" },
    { 0xA988, "JAVANESE LETTER U" },
    { 0xA989, "JAVANESE LETTER PA CEREK" },
    { 0xA98A, "JAVANESE LETTER NGA LELET" },
    { 0xA98B, "JAVANESE LETTER NGA LELET RASWADI" },
    { 0xA98C, "JAVANESE LETTER E" },
    { 0xA98D, "JAVANESE LETTER AI" },
    { 0xA98E, "JAVANESE LETTER O" },
    { 0xA98F, "JAVANESE LETTER KA" },
    { 0xA990, "JAVANESE LETTER KA SASAK" },
    { 0xA991, "JAVANESE LETTER KA MURDA" },
    { 0xA992, "JAVANESE LETTER GA" },
    { 0xA993, "JAVANESE LETTER GA MURDA" },
    { 0xA994, "JAVANESE LETTER NGA" },
    { 0xA995, "JAVANESE LETTER CA" },
    { 0xA996, "JAVANESE LETTER CA MURDA" },
    { 0xA997, "JAVANESE LETTER JA" },
    { 0xA998, "JAVANESE LETTER NYA MURDA" },
    { 0xA999, "JAVANESE LETTER JA MAHAPRANA" },
    { 0xA99A, "JAVANESE LETTER NYA" },
    { 0xA99B, "JAVANESE LETTER TTA" },
    { 0xA99C, "JAVANESE LETTER TTA MAHAPRANA" },
    { 0xA99D, "JAVANESE LETTER DDA" },
    { 0xA99E, "JAVANESE LETTER DDA MAHAPRANA" },
    { 0xA99F, "JAVANESE LETTER NA MURDA" },
    { 0xA9A0, "JAVANESE LETTER TA" },
    { 0xA9A1, "JAVANESE LETTER TA MURDA" },
    { 0xA9A2, "JAVANESE LETTER DA" },
    { 0xA9A3, "JAVANESE LETTER DA MAHAPRANA" },
    { 0xA9A4, "JAVANESE LETTER NA" },
    { 0xA9A5, "JAVANESE LETTER PA" },
    { 0xA9A6, "JAVANESE LETTER PA MURDA" },
    { 0xA9A7, "JAVANESE LETTER BA" },
    { 0xA9A8, "JAVANESE LETTER BA MURDA" },
    { 0xA9A9, "JAVANESE LETTER MA" },
    { 0xA9AA, "JAVANESE LETTER YA" },
    { 0xA9AB, "JAVANESE LETTER RA" },
    { 0xA9AC, "JAVANESE LETTER RA AGUNG" },
    { 0xA9AD, "JAVANESE LETTER LA" },
    { 0xA9AE, "JAVANESE LETTER WA" },
    { 0xA9AF, "JAVANESE LETTER SA MURDA" },
    { 0xA9B0, "JAVANESE LETTER SA MAHAPRANA" },
    { 0xA9B1, "JAVANESE LETTER SA" },
    { 0xA9B2, "JAVANESE LETTER HA" },
    { 0xA9B3, "JAVANESE SIGN CECAK TELU" },
    { 0xA9B4, "JAVANESE VOWEL SIGN TARUNG" },
    { 0xA9B5, "JAVANESE VOWEL SIGN TOLONG" },
    { 0xA9B6, "JAVANESE VOWEL SIGN WULU" },
    { 0xA9B7, "JAVANESE VOWEL SIGN WULU MELIK" },
    { 0xA9B8, "JAVANESE VOWEL SIGN SUKU" },
    { 0xA9B9, "JAVANESE VOWEL SIGN SUKU MENDUT" },
    { 0xA9BA, "JAVANESE VOWEL SIGN TALING" },
    { 0xA9BB, "JAVANESE VOWEL SIGN DIRGA MURE" },
    { 0xA9BC, "JAVANESE VOWEL SIGN PEPET" },
    { 0xA9BD, "JAVANESE CONSONANT SIGN KERET" },
    { 0xA9BE, "JAVANESE CONSONANT SIGN PENGKAL" },
    { 0xA9BF, "JAVANESE CONSONANT SIGN CAKRA" },
    { 0xA9C0, "JAVANESE PANGKON" },
    { 0xA9C1, "JAVANESE LEFT RERENGGAN" },
    { 0xA9C2, "JAVANESE RIGHT RERENGGAN" },
    { 0xA9C3, "JAVANESE PADA ANDAP" },
    { 0xA9C4, "JAVANESE PADA MADYA" },
    { 0xA9C5, "JAVANESE PADA LUHUR" },
    { 0xA9C6, "JAVANESE PADA WINDU" },
    { 0xA9C7, "JAVANESE PADA PANGKAT" },
    { 0xA9C8, "JAVANESE PADA LINGSA" },
    { 0xA9C9, "JAVANESE PADA LUNGSI" },
    { 0xA9CA, "JAVANESE PADA ADEG" },
    { 0xA9CB, "JAVANESE PADA ADEG ADEG" },
    { 0xA9CC, "JAVANESE PADA PISELEH" },
    { 0xA9CD, "JAVANESE TURNED PADA PISELEH" },
    { 0xA9CF, "JAVANESE PANGRANGKEP" },
    { 0xA9D0, "JAVANESE DIGIT ZERO" },
    { 0xA9D1, "JAVANESE DIGIT ONE" },
    { 0xA9D2, "JAVANESE DIGIT TWO" },
    { 0xA9D3, "JAVANESE DIGIT THREE" },
    { 0xA9D4, "JAVANESE DIGIT FOUR" },
    { 0xA9D5, "JAVANESE DIGIT FIVE" },
    { 0xA9D6, "JAVANESE DIGIT SIX" },
    { 0xA9D7, "JAVANESE DIGIT SEVEN" },
    { 0xA9D8, "JAVANESE DIGIT EIGHT" },
    { 0xA9D9, "JAVANESE DIGIT NINE" },
    { 0xA9DE, "JAVANESE PADA TIRTA TUMETES" },
    { 0xA9DF, "JAVANESE PADA ISEN-ISEN" },
    { 0xA9E0, "MYANMAR LETTER SHAN GHA" },
    { 0xA9E1, "MYANMAR LETTER SHAN CHA" },
    { 0xA9E2, "MYANMAR LETTER SHAN JHA" },
    { 0xA9E3, "MYANMAR LETTER SHAN NNA" },
    { 0xA9E4, "MYANMAR LETTER SHAN BHA" },
    { 0xA9E5, "MYANMAR SIGN SHAN SAW" },
    { 0xA9E6, "MYANMAR MODIFIER LETTER SHAN REDUPLICATION" },
    { 0xA9E7, "MYANMAR LETTER TAI LAING NYA" },
    { 0xA9E8, "MYANMAR LETTER TAI LAING FA" },
    { 0xA9E9, "MYANMAR LETTER TAI LAING GA" },
    { 0xA9EA, "MYANMAR LETTER TAI LAING GHA" },
    { 0xA9EB, "MYANMAR LETTER TAI LAING JA" },
    { 0xA9EC, "MYANMAR LETTER TAI LAING JHA" },
    { 0xA9ED, "MYANMAR LETTER TAI LAING DDA" },
    { 0xA9EE, "MYANMAR LETTER TAI LAING DDHA" },
    { 0xA9EF, "MYANMAR LETTER TAI LAING NNA" },
    { 0xA9F0, "MYANMAR TAI LAING DIGIT ZERO" },
    { 0xA9F1, "MYANMAR TAI LAING DIGIT ONE" },
    { 0xA9F2, "MYANMAR TAI LAING DIGIT TWO" },
    { 0xA9F3, "MYANMAR TAI LAING DIGIT THREE" },
    { 0xA9F4, "MYANMAR TAI LAING DIGIT FOUR" },
    { 0xA9F5, "MYANMAR TAI LAING DIGIT FIVE" },
    { 0xA9F6, "MYANMAR TAI LAING DIGIT SIX" },
    { 0xA9F7, "MYANMAR TAI LAING DIGIT SEVEN" },
    { 0xA9F8, "MYANMAR TAI LAING DIGIT EIGHT" },
    { 0xA9F9, "MYANMAR TAI LAING DIGIT NINE" },
    { 0xA9FA, "MYANMAR LETTER TAI LAING LLA" },
    { 0xA9FB, "MYANMAR LETTER TAI LAING DA" },
    { 0xA9FC, "MYANMAR LETTER TAI LAING DHA" },
    { 0xA9FD, "MYANMAR LETTER TAI LAING BA" },
    { 0xA9FE, "MYANMAR LETTER TAI LAING BHA" },
    { 0xAA00, "CHAM LETTER A" },
    { 0xAA01, "CHAM LETTER I" },
    { 0xAA02, "CHAM LETTER U" },
    { 0xAA03, "CHAM LETTER E" },
    { 0xAA04, "CHAM LETTER AI" },
    { 0xAA05, "CHAM LETTER O" },
    { 0xAA06, "CHAM LETTER KA" },
    { 0xAA07, "CHAM LETTER KHA" },
    { 0xAA08, "CHAM LETTER GA" },
    { 0xAA09, "CHAM LETTER GHA" },
    { 0xAA0A, "CHAM LETTER NGUE" },
    { 0xAA0B, "CHAM LETTER NGA" },
    { 0xAA0C, "CHAM LETTER CHA" },
    { 0xAA0D, "CHAM LETTER CHHA" },
    { 0xAA0E, "CHAM LETTER JA" },
    { 0xAA0F, "CHAM LETTER JHA" },
    { 0xAA10, "CHAM LETTER NHUE" },
    { 0xAA11, "CHAM LETTER NHA" },
    { 0xAA12, "CHAM LETTER NHJA" },
    { 0xAA13, "CHAM LETTER TA" },
    { 0xAA14, "CHAM LETTER THA" },
    { 0xAA15, "CHAM LETTER DA" },
    { 0xAA16, "CHAM LETTER DHA" },
    { 0xAA17, "CHAM LETTER NUE" },
    { 0xAA18, "CHAM LETTER NA" },
    { 0xAA19, "CHAM LETTER DDA" },
    { 0xAA1A, "CHAM LETTER PA" },
    { 0xAA1B, "CHAM LETTER PPA" },
    { 0xAA1C, "CHAM LETTER PHA" },
    { 0xAA1D, "CHAM LETTER BA" },
    { 0xAA1E, "CHAM LETTER BHA" },
    { 0xAA1F, "CHAM LETTER MUE" },
    { 0xAA20, "CHAM LETTER MA" },
    { 0xAA21, "CHAM LETTER BBA" },
    { 0xAA22, "CHAM LETTER YA" },
    { 0xAA23, "CHAM LETTER RA" },
    { 0xAA24, "CHAM LETTER LA" },
    { 0xAA25, "CHAM LETTER VA" },
    { 0xAA26, "CHAM LETTER SSA" },
    { 0xAA27, "CHAM LETTER SA" },
    { 0xAA28, "CHAM LETTER HA" },
    { 0xAA29, "CHAM VOWEL SIGN AA" },
    { 0xAA2A, "CHAM VOWEL SIGN I" },
    { 0xAA2B, "CHAM VOWEL SIGN II" },
    { 0xAA2C, "CHAM VOWEL SIGN EI" },
    { 0xAA2D, "CHAM VOWEL SIGN U" },
    { 0xAA2E, "CHAM VOWEL SIGN OE" },
    { 0xAA2F, "CHAM VOWEL SIGN O" },
    { 0xAA30, "CHAM VOWEL SIGN AI" },
    { 0xAA31, "CHAM VOWEL SIGN AU" },
    { 0xAA32, "CHAM VOWEL SIGN UE" },
    { 0xAA33, "CHAM CONSONANT SIGN YA" },
    { 0xAA34, "CHAM CONSONANT SIGN RA" },
    { 0xAA35, "CHAM CONSONANT SIGN LA" },
    { 0xAA36, "CHAM CONSONANT SIGN WA" },
    { 0xAA40, "CHAM LETTER FINAL K" },
    { 0xAA41, "CHAM LETTER FINAL G" },
    { 0xAA42, "CHAM LETTER FINAL NG" },
    { 0xAA43, "CHAM CONSONANT SIGN FINAL NG" },
    { 0xAA44, "CHAM LETTER FINAL CH" },
    { 0xAA45, "CHAM LETTER FINAL T" },
    { 0xAA46, "CHAM LETTER FINAL N" },
    { 0xAA47, "CHAM LETTER FINAL P" },
    { 0xAA48, "CHAM LETTER FINAL Y" },
    { 0xAA49, "CHAM LETTER FINAL R" },
    { 0xAA4A, "CHAM LETTER FINAL L" },
    { 0xAA4B, "CHAM LETTER FINAL SS" },
    { 0xAA4C, "CHAM CONSONANT SIGN FINAL M" },
    { 0xAA4D, "CHAM CONSONANT SIGN FINAL H" },
    { 0xAA50, "CHAM DIGIT ZERO" },
    { 0xAA51, "CHAM DIGIT ONE" },
    { 0xAA52, "CHAM DIGIT TWO" },
    { 0xAA53, "CHAM DIGIT THREE" },
    { 0xAA54, "CHAM DIGIT FOUR" },
    { 0xAA55, "CHAM DIGIT FIVE" },
    { 0xAA56, "CHAM DIGIT SIX" },
    { 0xAA57, "CHAM DIGIT SEVEN" },
    { 0xAA58, "CHAM DIGIT EIGHT" },
    { 0xAA59, "CHAM DIGIT NINE" },
    { 0xAA5C, "CHAM PUNCTUATION SPIRAL" },
    { 0xAA5D, "CHAM PUNCTUATION DANDA" },
    { 0xAA5E, "CHAM PUNCTUATION DOUBLE DANDA" },
    { 0xAA5F, "CHAM PUNCTUATION TRIPLE DANDA" },
    { 0xAA60, "MYANMAR LETTER KHAMTI GA" },
    { 0xAA61, "MYANMAR LETTER KHAMTI CA" },
    { 0xAA62, "MYANMAR LETTER KHAMTI CHA" },
    { 0xAA63, "MYANMAR LETTER KHAMTI JA" },
    { 0xAA64, "MYANMAR LETTER KHAMTI JHA" },
    { 0xAA65, "MYANMAR LETTER KHAMTI NYA" },
    { 0xAA66, "MYANMAR LETTER KHAMTI TTA" },
    { 0xAA67, "MYANMAR LETTER KHAMTI TTHA" },
    { 0xAA68, "MYANMAR LETTER KHAMTI DDA" },
    { 0xAA69, "MYANMAR LETTER KHAMTI DDHA" },
    { 0xAA6A, "MYANMAR LETTER KHAMTI DHA" },
    { 0xAA6B, "MYANMAR LETTER KHAMTI NA" },
    { 0xAA6C, "MYANMAR LETTER KHAMTI SA" },
    { 0xAA6D, "MYANMAR LETTER KHAMTI HA" },
    { 0xAA6E, "MYANMAR LETTER KHAMTI HHA" },
    { 0xAA6F, "MYANMAR LETTER KHAMTI FA" },
    { 0xAA70, "MYANMAR MODIFIER LETTER KHAMTI REDUPLICATION" },
    { 0xAA71, "MYANMAR LETTER KHAMTI XA" },
    { 0xAA72, "MYANMAR LETTER KHAMTI ZA" },
    { 0xAA73, "MYANMAR LETTER KHAMTI RA" },
    { 0xAA74, "MYANMAR LOGOGRAM KHAMTI OAY" },
    { 0xAA75, "MYANMAR LOGOGRAM KHAMTI QN" },
    { 0xAA76, "MYANMAR LOGOGRAM KHAMTI HM" },
    { 0xAA77, "MYANMAR SYMBOL AITON EXCLAMATION" },
    { 0xAA78, "MYANMAR SYMBOL AITON ONE" },
    { 0xAA79, "MYANMAR SYMBOL AITON TWO" },
    { 0xAA7A, "MYANMAR LETTER AITON RA" },
    { 0xAA7B, "MYANMAR SIGN PAO KAREN TONE" },
    { 0xAA7C, "MYANMAR SIGN TAI LAING TONE-2" },
    { 0xAA7D, "MYANMAR SIGN TAI LAING TONE-5" },
    { 0xAA7E, "MYANMAR LETTER SHWE PALAUNG CHA" },
    { 0xAA7F, "MYANMAR LETTER SHWE PALAUNG SHA" },
    { 0xAA80, "TAI VIET LETTER LOW KO" },
    { 0xAA81, "TAI VIET LETTER HIGH KO" },
    { 0xAA82, "TAI VIET LETTER LOW KHO" },
    { 0xAA83, "TAI VIET LETTER HIGH KHO" },
    { 0xAA84, "TAI VIET LETTER LOW KHHO" },
    { 0xAA85, "TAI VIET LETTER HIGH KHHO" },
    { 0xAA86, "TAI VIET LETTER LOW GO" },
    { 0xAA87, "TAI VIET LETTER HIGH GO" },
    { 0xAA88, "TAI VIET LETTER LOW NGO" },
    { 0xAA89, "TAI VIET LETTER HIGH NGO" },
    { 0xAA8A, "TAI VIET LETTER LOW CO" },
    { 0xAA8B, "TAI VIET LETTER HIGH CO" },
    { 0xAA8C, "TAI VIET LETTER LOW CHO" },
    { 0xAA8D, "TAI VIET LETTER HIGH CHO" },
    { 0xAA8E, "TAI VIET LETTER LOW SO" },
    { 0xAA8F, "TAI VIET LETTER HIGH SO" },
    { 0xAA90, "TAI VIET LETTER LOW NYO" },
    { 0xAA91, "TAI VIET LETTER HIGH NYO" },
    { 0xAA92, "TAI VIET LETTER LOW DO" },
    { 0xAA93, "TAI VIET LETTER HIGH DO" },
    { 0xAA94, "TAI VIET LETTER LOW TO" },
    { 0xAA95, "TAI VIET LETTER HIGH TO" },
    { 0xAA96, "TAI VIET LETTER LOW THO" },
    { 0xAA97, "TAI VIET LETTER HIGH THO" },
    { 0xAA98, "TAI VIET LETTER LOW NO" },
    { 0xAA99, "TAI VIET LETTER HIGH NO" },
    { 0xAA9A, "TAI VIET LETTER LOW BO" },
    { 0xAA9B, "TAI VIET LETTER HIGH BO" },
    { 0xAA9C, "TAI VIET LETTER LOW PO" },
    { 0xAA9D, "TAI VIET LETTER HIGH PO" },
    { 0xAA9E, "TAI VIET LETTER LOW PHO" },
    { 0xAA9F, "TAI VIET LETTER HIGH PHO" },
    { 0xAAA0, "TAI VIET LETTER LOW FO" },
    { 0xAAA1, "TAI VIET LETTER HIGH FO" },
    { 0xAAA2, "TAI VIET LETTER LOW MO" },
    { 0xAAA3, "TAI VIET LETTER HIGH MO" },
    { 0xAAA4, "TAI VIET LETTER LOW YO" },
    { 0xAAA5, "TAI VIET LETTER HIGH YO" },
    { 0xAAA6, "TAI VIET LETTER LOW RO" },
    { 0xAAA7, "TAI VIET LETTER HIGH RO" },
    { 0xAAA8, "TAI VIET LETTER LOW LO" },
    { 0xAAA9, "TAI VIET LETTER HIGH LO" },
    { 0xAAAA, "TAI VIET LETTER LOW VO" },
    { 0xAAAB, "TAI VIET LETTER HIGH VO" },
    { 0xAAAC, "TAI VIET LETTER LOW HO" },
    { 0xAAAD, "TAI VIET LETTER HIGH HO" },
    { 0xAAAE, "TAI VIET LETTER LOW O" },
    { 0xAAAF, "TAI VIET LETTER HIGH O" },
    { 0xAAB0, "TAI VIET MAI KANG" },
    { 0xAAB1, "TAI VIET VOWEL AA" },
    { 0xAAB2, "TAI VIET VOWEL I" },
    { 0xAAB3, "TAI VIET VOWEL UE" },
    { 0xAAB4, "TAI VIET VOWEL U" },
    { 0xAAB5, "TAI VIET VOWEL E" },
    { 0xAAB6, "TAI VIET VOWEL O" },
    { 0xAAB7, "TAI VIET MAI KHIT" },
    { 0xAAB8, "TAI VIET VOWEL IA" },
    { 0xAAB9, "TAI VIET VOWEL UEA" },
    { 0xAABA, "TAI VIET VOWEL UA" },
    { 0xAABB, "TAI VIET VOWEL AUE" },
    { 0xAABC, "TAI VIET VOWEL AY" },
    { 0xAABD, "TAI VIET VOWEL AN" },
    { 0xAABE, "TAI VIET VOWEL AM" },
    { 0xAABF, "TAI VIET TONE MAI EK" },
    { 0xAAC0, "TAI VIET TONE MAI NUENG" },
    { 0xAAC1, "TAI VIET TONE MAI THO" },
    { 0xAAC2, "TAI VIET TONE MAI SONG" },
    { 0xAADB, "TAI VIET SYMBOL KON" },
    { 0xAADC, "TAI VIET SYMBOL NUENG" },
    { 0xAADD, "TAI VIET SYMBOL SAM" },
    { 0xAADE, "TAI VIET SYMBOL HO HOI" },
    { 0xAADF, "TAI VIET SYMBOL KOI KOI" },
    { 0xAAE0, "MEETEI MAYEK LETTER E" },
    { 0xAAE1, "MEETEI MAYEK LETTER O" },
    { 0xAAE2, "MEETEI MAYEK LETTER CHA" },
    { 0xAAE3, "MEETEI MAYEK LETTER NYA" },
    { 0xAAE4, "MEETEI MAYEK LETTER TTA" },
    { 0xAAE5, "MEETEI MAYEK LETTER TTHA" },
    { 0xAAE6, "MEETEI MAYEK LETTER DDA" },
    { 0xAAE7, "MEETEI MAYEK LETTER DDHA" },
    { 0xAAE8, "MEETEI MAYEK LETTER NNA" },
    { 0xAAE9, "MEETEI MAYEK LETTER SHA" },
    { 0xAAEA, "MEETEI MAYEK LETTER SSA" },
    { 0xAAEB, "MEETEI MAYEK VOWEL SIGN II" },
    { 0xAAEC, "MEETEI MAYEK VOWEL SIGN UU" },
    { 0xAAED, "MEETEI MAYEK VOWEL SIGN AAI" },
    { 0xAAEE, "MEETEI MAYEK VOWEL SIGN AU" },
    { 0xAAEF, "MEETEI MAYEK VOWEL SIGN AAU" },
    { 0xAAF0, "MEETEI MAYEK CHEIKHAN" },
    { 0xAAF1, "MEETEI MAYEK AHANG KHUDAM" },
    { 0xAAF2, "MEETEI MAYEK ANJI" },
    { 0xAAF3, "MEETEI MAYEK SYLLABLE REPETITION MARK" },
    { 0xAAF4, "MEETEI MAYEK WORD REPETITION MARK" },
    { 0xAAF5, "MEETEI MAYEK VOWEL SIGN VISARGA" },
    { 0xAAF6, "MEETEI MAYEK VIRAMA" },
    { 0xAB01, "ETHIOPIC SYLLABLE TTHU" },
    { 0xAB02, "ETHIOPIC SYLLABLE TTHI" },
    { 0xAB03, "ETHIOPIC SYLLABLE TTHAA" },
    { 0xAB04, "ETHIOPIC SYLLABLE TTHEE" },
    { 0xAB05, "ETHIOPIC SYLLABLE TTHE" },
    { 0xAB06, "ETHIOPIC SYLLABLE TTHO" },
    { 0xAB09, "ETHIOPIC SYLLABLE DDHU" },
    { 0xAB0A, "ETHIOPIC SYLLABLE DDHI" },
    { 0xAB0B, "ETHIOPIC SYLLABLE DDHAA" },
    { 0xAB0C, "ETHIOPIC SYLLABLE DDHEE" },
    { 0xAB0D, "ETHIOPIC SYLLABLE DDHE" },
    { 0xAB0E, "ETHIOPIC SYLLABLE DDHO" },
    { 0xAB11, "ETHIOPIC SYLLABLE DZU" },
    { 0xAB12, "ETHIOPIC SYLLABLE DZI" },
    { 0xAB13, "ETHIOPIC SYLLABLE DZAA" },
    { 0xAB14, "ETHIOPIC SYLLABLE DZEE" },
    { 0xAB15, "ETHIOPIC SYLLABLE DZE" },
    { 0xAB16, "ETHIOPIC SYLLABLE DZO" },
    { 0xAB20, "ETHIOPIC SYLLABLE CCHHA" },
    { 0xAB21, "ETHIOPIC SYLLABLE CCHHU" },
    { 0xAB22, "ETHIOPIC SYLLABLE CCHHI" },
    { 0xAB23, "ETHIOPIC SYLLABLE CCHHAA" },
    { 0xAB24, "ETHIOPIC SYLLABLE CCHHEE" },
    { 0xAB25, "ETHIOPIC SYLLABLE CCHHE" },
    { 0xAB26, "ETHIOPIC SYLLABLE CCHHO" },
    { 0xAB28, "ETHIOPIC SYLLABLE BBA" },
    { 0xAB29, "ETHIOPIC SYLLABLE BBU" },
    { 0xAB2A, "ETHIOPIC SYLLABLE BBI" },
    { 0xAB2B, "ETHIOPIC SYLLABLE BBAA" },
    { 0xAB2C, "ETHIOPIC SYLLABLE BBEE" },
    { 0xAB2D, "ETHIOPIC SYLLABLE BBE" },
    { 0xAB2E, "ETHIOPIC SYLLABLE BBO" },
    { 0xAB30, "LATIN SMALL LETTER BARRED ALPHA" },
    { 0xAB31, "LATIN SMALL LETTER A REVERSED-SCHWA" },
    { 0xAB32, "LATIN SMALL LETTER BLACKLETTER E" },
    { 0xAB33, "LATIN SMALL LETTER BARRED E" },
    { 0xAB34, "LATIN SMALL LETTER E WITH FLOURISH" },
    { 0xAB35, "LATIN SMALL LETTER LENIS F" },
    { 0xAB36, "LATIN SMALL LETTER SCRIPT G WITH CROSSED-TAIL" },
    { 0xAB37, "LATIN SMALL LETTER L WITH INVERTED LAZY S" },
    { 0xAB38, "LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE" },
    { 0xAB39, "LATIN SMALL LETTER L WITH MIDDLE RING" },
    { 0xAB3A, "LATIN SMALL LETTER M WITH CROSSED-TAIL" },
    { 0xAB3B, "LATIN SMALL LETTER N WITH CROSSED-TAIL" },
    { 0xAB3C, "LATIN SMALL LETTER ENG WITH CROSSED-TAIL" },
    { 0xAB3D, "LATIN SMALL LETTER BLACKLETTER O" },
    { 0xAB3E, "LATIN SMALL LETTER BLACKLETTER O WITH STROKE" },
    { 0xAB3F, "LATIN SMALL LETTER OPEN O WITH STROKE" },
    { 0xAB40, "LATIN SMALL LETTER INVERTED OE" },
    { 0xAB41, "LATIN SMALL LETTER TURNED OE WITH STROKE" },
    { 0xAB42, "LATIN SMALL LETTER TURNED OE WITH HORIZONTAL STROKE" },
    { 0xAB43, "LATIN SMALL LETTER TURNED O OPEN-O" },
    { 0xAB44, "LATIN SMALL LETTER TURNED O OPEN-O WITH STROKE" },
    { 0xAB45, "LATIN SMALL LETTER STIRRUP R" },
    { 0xAB46, "LATIN LETTER SMALL CAPITAL R WITH RIGHT LEG" },
    { 0xAB47, "LATIN SMALL LETTER R WITHOUT HANDLE" },
    { 0xAB48, "LATIN SMALL LETTER DOUBLE R" },
    { 0xAB49, "LATIN SMALL LETTER R WITH CROSSED-TAIL" },
    { 0xAB4A, "LATIN SMALL LETTER DOUBLE R WITH CROSSED-TAIL" },
    { 0xAB4B, "LATIN SMALL LETTER SCRIPT R" },
    { 0xAB4C, "LATIN SMALL LETTER SCRIPT R WITH RING" },
    { 0xAB4D, "LATIN SMALL LETTER BASELINE ESH" },
    { 0xAB4E, "LATIN SMALL LETTER U WITH SHORT RIGHT LEG" },
    { 0xAB4F, "LATIN SMALL LETTER U BAR WITH SHORT RIGHT LEG" },
    { 0xAB50, "LATIN SMALL LETTER UI" },
    { 0xAB51, "LATIN SMALL LETTER TURNED UI" },
    { 0xAB52, "LATIN SMALL LETTER U WITH LEFT HOOK" },
    { 0xAB53, "LATIN SMALL LETTER CHI" },
    { 0xAB54, "LATIN SMALL LETTER CHI WITH LOW RIGHT RING" },
    { 0xAB55, "LATIN SMALL LETTER CHI WITH LOW LEFT SERIF" },
    { 0xAB56, "LATIN SMALL LETTER X WITH LOW RIGHT RING" },
    { 0xAB57, "LATIN SMALL LETTER X WITH LONG LEFT LEG" },
    { 0xAB58, "LATIN SMALL LETTER X WITH LONG LEFT LEG AND LOW RIGHT RING" },
    { 0xAB59, "LATIN SMALL LETTER X WITH LONG LEFT LEG WITH SERIF" },
    { 0xAB5A, "LATIN SMALL LETTER Y WITH SHORT RIGHT LEG" },
    { 0xAB5B, "MODIFIER BREVE WITH INVERTED BREVE" },
    { 0xAB5C, "MODIFIER LETTER SMALL HENG" },
    { 0xAB5D, "MODIFIER LETTER SMALL L WITH INVERTED LAZY S" },
    { 0xAB5E, "MODIFIER LETTER SMALL L WITH MIDDLE TILDE" },
    { 0xAB5F, "MODIFIER LETTER SMALL U WITH LEFT HOOK" },
    { 0xAB60, "LATIN SMALL LETTER SAKHA YAT" },
    { 0xAB61, "LATIN SMALL LETTER IOTIFIED E" },
    { 0xAB62, "LATIN SMALL LETTER OPEN OE" },
    { 0xAB63, "LATIN SMALL LETTER UO" },
    { 0xAB64, "LATIN SMALL LETTER INVERTED ALPHA" },
    { 0xAB65, "GREEK LETTER SMALL CAPITAL OMEGA" },
    { 0xAB66, "LATIN SMALL LETTER DZ DIGRAPH WITH RETROFLEX HOOK" },
    { 0xAB67, "LATIN SMALL LETTER TS DIGRAPH WITH RETROFLEX HOOK" },
    { 0xAB68, "LATIN SMALL LETTER TURNED R WITH MIDDLE TILDE" },
    { 0xAB69, "MODIFIER LETTER SMALL TURNED W" },
    { 0xAB6A, "MODIFIER LETTER LEFT TACK" },
    { 0xAB6B, "MODIFIER LETTER RIGHT TACK" },
    { 0xAB70, "CHEROKEE SMALL LETTER A" },
    { 0xAB71, "CHEROKEE SMALL LETTER E" },
    { 0xAB72, "CHEROKEE SMALL LETTER I" },
    { 0xAB73, "CHEROKEE SMALL LETTER O" },
    { 0xAB74, "CHEROKEE SMALL LETTER U" },
    { 0xAB75, "CHEROKEE SMALL LETTER V" },
    { 0xAB76, "CHEROKEE SMALL LETTER GA" },
    { 0xAB77, "CHEROKEE SMALL LETTER KA" },
    { 0xAB78, "CHEROKEE SMALL LETTER GE" },
    { 0xAB79, "CHEROKEE SMALL LETTER GI" },
    { 0xAB7A, "CHEROKEE SMALL LETTER GO" },
    { 0xAB7B, "CHEROKEE SMALL LETTER GU" },
    { 0xAB7C, "CHEROKEE SMALL LETTER GV" },
    { 0xAB7D, "CHEROKEE SMALL LETTER HA" },
    { 0xAB7E, "CHEROKEE SMALL LETTER HE" },
    { 0xAB7F, "CHEROKEE SMALL LETTER HI" },
    { 0xAB80, "CHEROKEE SMALL LETTER HO" },
    { 0xAB81, "CHEROKEE SMALL LETTER HU" },
    { 0xAB82, "CHEROKEE SMALL LETTER HV" },
    { 0xAB83, "CHEROKEE SMALL LETTER LA" },
    { 0xAB84, "CHEROKEE SMALL LETTER LE" },
    { 0xAB85, "CHEROKEE SMALL LETTER LI" },
    { 0xAB86, "CHEROKEE SMALL LETTER LO" },
    { 0xAB87, "CHEROKEE SMALL LETTER LU" },
    { 0xAB88, "CHEROKEE SMALL LETTER LV" },
    { 0xAB89, "CHEROKEE SMALL LETTER MA" },
    { 0xAB8A, "CHEROKEE SMALL LETTER ME" },
    { 0xAB8B, "CHEROKEE SMALL LETTER MI" },
    { 0xAB8C, "CHEROKEE SMALL LETTER MO" },
    { 0xAB8D, "CHEROKEE SMALL LETTER MU" },
    { 0xAB8E, "CHEROKEE SMALL LETTER NA" },
    { 0xAB8F, "CHEROKEE SMALL LETTER HNA" },
    { 0xAB90, "CHEROKEE SMALL LETTER NAH" },
    { 0xAB91, "CHEROKEE SMALL LETTER NE" },
    { 0xAB92, "CHEROKEE SMALL LETTER NI" },
    { 0xAB93, "CHEROKEE SMALL LETTER NO" },
    { 0xAB94, "CHEROKEE SMALL LETTER NU" },
    { 0xAB95, "CHEROKEE SMALL LETTER NV" },
    { 0xAB96, "CHEROKEE SMALL LETTER QUA" },
    { 0xAB97, "CHEROKEE SMALL LETTER QUE" },
    { 0xAB98, "CHEROKEE SMALL LETTER QUI" },
    { 0xAB99, "CHEROKEE SMALL LETTER QUO" },
    { 0xAB9A, "CHEROKEE SMALL LETTER QUU" },
    { 0xAB9B, "CHEROKEE SMALL LETTER QUV" },
    { 0xAB9C, "CHEROKEE SMALL LETTER SA" },
    { 0xAB9D, "CHEROKEE SMALL LETTER S" },
    { 0xAB9E, "CHEROKEE SMALL LETTER SE" },
    { 0xAB9F, "CHEROKEE SMALL LETTER SI" },
    { 0xABA0, "CHEROKEE SMALL LETTER SO" },
    { 0xABA1, "CHEROKEE SMALL LETTER SU" },
    { 0xABA2, "CHEROKEE SMALL LETTER SV" },
    { 0xABA3, "CHEROKEE SMALL LETTER DA" },
    { 0xABA4, "CHEROKEE SMALL LETTER TA" },
    { 0xABA5, "CHEROKEE SMALL LETTER DE" },
    { 0xABA6, "CHEROKEE SMALL LETTER TE" },
    { 0xABA7, "CHEROKEE SMALL LETTER DI" },
    { 0xABA8, "CHEROKEE SMALL LETTER TI" },
    { 0xABA9, "CHEROKEE SMALL LETTER DO" },
    { 0xABAA, "CHEROKEE SMALL LETTER DU" },
    { 0xABAB, "CHEROKEE SMALL LETTER DV" },
    { 0xABAC, "CHEROKEE SMALL LETTER DLA" },
    { 0xABAD, "CHEROKEE SMALL LETTER TLA" },
    { 0xABAE, "CHEROKEE SMALL LETTER TLE" },
    { 0xABAF, "CHEROKEE SMALL LETTER TLI" },
    { 0xABB0, "CHEROKEE SMALL LETTER TLO" },
    { 0xABB1, "CHEROKEE SMALL LETTER TLU" },
    { 0xABB2, "CHEROKEE SMALL LETTER TLV" },
    { 0xABB3, "CHEROKEE SMALL LETTER TSA" },
    { 0xABB4, "CHEROKEE SMALL LETTER TSE" },
    { 0xABB5, "CHEROKEE SMALL LETTER TSI" },
    { 0xABB6, "CHEROKEE SMALL LETTER TSO" },
    { 0xABB7, "CHEROKEE SMALL LETTER TSU" },
    { 0xABB8, "CHEROKEE SMALL LETTER TSV" },
    { 0xABB9, "CHEROKEE SMALL LETTER WA" },
    { 0xABBA, "CHEROKEE SMALL LETTER WE" },
    { 0xABBB, "CHEROKEE SMALL LETTER WI" },
    { 0xABBC, "CHEROKEE SMALL LETTER WO" },
    { 0xABBD, "CHEROKEE SMALL LETTER WU" },
    { 0xABBE, "CHEROKEE SMALL LETTER WV" },
    { 0xABBF, "CHEROKEE SMALL LETTER YA" },
    { 0xABC0, "MEETEI MAYEK LETTER KOK" },
    { 0xABC1, "MEETEI MAYEK LETTER SAM" },
    { 0xABC2, "MEETEI MAYEK LETTER LAI" },
    { 0xABC3, "MEETEI MAYEK LETTER MIT" },
    { 0xABC4, "MEETEI MAYEK LETTER PA" },
    { 0xABC5, "MEETEI MAYEK LETTER NA" },
    { 0xABC6, "MEETEI MAYEK LETTER CHIL" },
    { 0xABC7, "MEETEI MAYEK LETTER TIL" },
    { 0xABC8, "MEETEI MAYEK LETTER KHOU" },
    { 0xABC9, "MEETEI MAYEK LETTER NGOU" },
    { 0xABCA, "MEETEI MAYEK LETTER THOU" },
    { 0xABCB, "MEETEI MAYEK LETTER WAI" },
    { 0xABCC, "MEETEI MAYEK LETTER YANG" },
    { 0xABCD, "MEETEI MAYEK LETTER HUK" },
    { 0xABCE, "MEETEI MAYEK LETTER UN" },
    { 0xABCF, "MEETEI MAYEK LETTER I" },
    { 0xABD0, "MEETEI MAYEK LETTER PHAM" },
    { 0xABD1, "MEETEI MAYEK LETTER ATIYA" },
    { 0xABD2, "MEETEI MAYEK LETTER GOK" },
    { 0xABD3, "MEETEI MAYEK LETTER JHAM" },
    { 0xABD4, "MEETEI MAYEK LETTER RAI" },
    { 0xABD5, "MEETEI MAYEK LETTER BA" },
    { 0xABD6, "MEETEI MAYEK LETTER JIL" },
    { 0xABD7, "MEETEI MAYEK LETTER DIL" },
    { 0xABD8, "MEETEI MAYEK LETTER GHOU" },
    { 0xABD9, "MEETEI MAYEK LETTER DHOU" },
    { 0xABDA, "MEETEI MAYEK LETTER BHAM" },
    { 0xABDB, "MEETEI MAYEK LETTER KOK LONSUM" },
    { 0xABDC, "MEETEI MAYEK LETTER LAI LONSUM" },
    { 0xABDD, "MEETEI MAYEK LETTER MIT LONSUM" },
    { 0xABDE, "MEETEI MAYEK LETTER PA LONSUM" },
    { 0xABDF, "MEETEI MAYEK LETTER NA LONSUM" },
    { 0xABE0, "MEETEI MAYEK LETTER TIL LONSUM" },
    { 0xABE1, "MEETEI MAYEK LETTER NGOU LONSUM" },
    { 0xABE2, "MEETEI MAYEK LETTER I LONSUM" },
    { 0xABE3, "MEETEI MAYEK VOWEL SIGN ONAP" },
    { 0xABE4, "MEETEI MAYEK VOWEL SIGN INAP" },
    { 0xABE5, "MEETEI MAYEK VOWEL SIGN ANAP" },
    { 0xABE6, "MEETEI MAYEK VOWEL SIGN YENAP" },
    { 0xABE7, "MEETEI MAYEK VOWEL SIGN SOUNAP" },
    { 0xABE8, "MEETEI MAYEK VOWEL SIGN UNAP" },
    { 0xABE9, "MEETEI MAYEK VOWEL SIGN CHEINAP" },
    { 0xABEA, "MEETEI MAYEK VOWEL SIGN NUNG" },
    { 0xABEB, "MEETEI MAYEK CHEIKHEI" },
    { 0xABEC, "MEETEI MAYEK LUM IYEK" },
    { 0xABED, "MEETEI MAYEK APUN IYEK" },
    { 0xABF0, "MEETEI MAYEK DIGIT ZERO" },
    { 0xABF1, "MEETEI MAYEK DIGIT ONE" },
    { 0xABF2, "MEETEI MAYEK DIGIT TWO" },
    { 0xABF3, "MEETEI MAYEK DIGIT THREE" },
    { 0xABF4, "MEETEI MAYEK DIGIT FOUR" },
    { 0xABF5, "MEETEI MAYEK DIGIT FIVE" },
    { 0xABF6, "MEETEI MAYEK DIGIT SIX" },
    { 0xABF7, "MEETEI MAYEK DIGIT SEVEN" },
    { 0xABF8, "MEETEI MAYEK DIGIT EIGHT" },
    { 0xABF9, "MEETEI MAYEK DIGIT NINE" },
    { 0xD7B0, "HANGUL JUNGSEONG O-YEO" },
    { 0xD7B1, "HANGUL JUNGSEONG O-O-I" },
    { 0xD7B2, "HANGUL JUNGSEONG YO-A" },
    { 0xD7B3, "HANGUL JUNGSEONG YO-AE" },
    { 0xD7B4, "HANGUL JUNGSEONG YO-EO" },
    { 0xD7B5, "HANGUL JUNGSEONG U-YEO" },
    { 0xD7B6, "HANGUL JUNGSEONG U-I-I" },
    { 0xD7B7, "HANGUL JUNGSEONG YU-AE" },
    { 0xD7B8, "HANGUL JUNGSEONG YU-O" },
    { 0xD7B9, "HANGUL JUNGSEONG EU-A" },
    { 0xD7BA, "HANGUL JUNGSEONG EU-EO" },
    { 0xD7BB, "HANGUL JUNGSEONG EU-E" },
    { 0xD7BC, "HANGUL JUNGSEONG EU-O" },
    { 0xD7BD, "HANGUL JUNGSEONG I-YA-O" },
    { 0xD7BE, "HANGUL JUNGSEONG I-YAE" },
    { 0xD7BF, "HANGUL JUNGSEONG I-YEO" },
    { 0xD7C0, "HANGUL JUNGSEONG I-YE" },
    { 0xD7C1, "HANGUL JUNGSEONG I-O-I" },
    { 0xD7C2, "HANGUL JUNGSEONG I-YO" },
    { 0xD7C3, "HANGUL JUNGSEONG I-YU" },
    { 0xD7C4, "HANGUL JUNGSEONG I-I" },
    { 0xD7C5, "HANGUL JUNGSEONG ARAEA-A" },
    { 0xD7C6, "HANGUL JUNGSEONG ARAEA-E" },
    { 0xD7CB, "HANGUL JONGSEONG NIEUN-RIEUL" },
    { 0xD7CC, "HANGUL JONGSEONG NIEUN-CHIEUCH" },
    { 0xD7CD, "HANGUL JONGSEONG SSANGTIKEUT" },
    { 0xD7CE, "HANGUL JONGSEONG SSANGTIKEUT-PIEUP" },
    { 0xD7CF, "HANGUL JONGSEONG TIKEUT-PIEUP" },
    { 0xD7D0, "HANGUL JONGSEONG TIKEUT-SIOS" },
    { 0xD7D1, "HANGUL JONGSEONG TIKEUT-SIOS-KIYEOK" },
    { 0xD7D2, "HANGUL JONGSEONG TIKEUT-CIEUC" },
    { 0xD7D3, "HANGUL JONGSEONG TIKEUT-CHIEUCH" },
    { 0xD7D4, "HANGUL JONGSEONG TIKEUT-THIEUTH" },
    { 0xD7D5, "HANGUL JONGSEONG RIEUL-SSANGKIYEOK" },
    { 0xD7D6, "HANGUL JONGSEONG RIEUL-KIYEOK-HIEUH" },
    { 0xD7D7, "HANGUL JONGSEONG SSANGRIEUL-KHIEUKH" },
    { 0xD7D8, "HANGUL JONGSEONG RIEUL-MIEUM-HIEUH" },
    { 0xD7D9, "HANGUL JONGSEONG RIEUL-PIEUP-TIKEUT" },
    { 0xD7DA, "HANGUL JONGSEONG RIEUL-PIEUP-PHIEUPH" },
    { 0xD7DB, "HANGUL JONGSEONG RIEUL-YESIEUNG" },
    { 0xD7DC, "HANGUL JONGSEONG RIEUL-YEORINHIEUH-HIEUH" },
    { 0xD7DD, "HANGUL JONGSEONG KAPYEOUNRIEUL" },
    { 0xD7DE, "HANGUL JONGSEONG MIEUM-NIEUN" },
    { 0xD7DF, "HANGUL JONGSEONG MIEUM-SSANGNIEUN" },
    { 0xD7E0, "HANGUL JONGSEONG SSANGMIEUM" },
    { 0xD7E1, "HANGUL JONGSEONG MIEUM-PIEUP-SIOS" },
    { 0xD7E2, "HANGUL JONGSEONG MIEUM-CIEUC" },
    { 0xD7E3, "HANGUL JONGSEONG PIEUP-TIKEUT" },
    { 0xD7E4, "HANGUL JONGSEONG PIEUP-RIEUL-PHIEUPH" },
    { 0xD7E5, "HANGUL JONGSEONG PIEUP-MIEUM" },
    { 0xD7E6, "HANGUL JONGSEONG SSANGPIEUP" },
    { 0xD7E7, "HANGUL JONGSEONG PIEUP-SIOS-TIKEUT" },
    { 0xD7E8, "HANGUL JONGSEONG PIEUP-CIEUC" },
    { 0xD7E9, "HANGUL JONGSEONG PIEUP-CHIEUCH" },
    { 0xD7EA, "HANGUL JONGSEONG SIOS-MIEUM" },
    { 0xD7EB, "HANGUL JONGSEONG SIOS-KAPYEOUNPIEUP" },
    { 0xD7EC, "HANGUL JONGSEONG SSANGSIOS-KIYEOK" },
    { 0xD7ED, "HANGUL JONGSEONG SSANGSIOS-TIKEUT" },
    { 0xD7EE, "HANGUL JONGSEONG SIOS-PANSIOS" },
    { 0xD7EF, "HANGUL JONGSEONG SIOS-CIEUC" },
    { 0xD7F0, "HANGUL JONGSEONG SIOS-CHIEUCH" },
    { 0xD7F1, "HANGUL JONGSEONG SIOS-THIEUTH" },
    { 0xD7F2, "HANGUL JONGSEONG SIOS-HIEUH" },
    { 0xD7F3, "HANGUL JONGSEONG PANSIOS-PIEUP" },
    { 0xD7F4, "HANGUL JONGSEONG PANSIOS-KAPYEOUNPIEUP" },
    { 0xD7F5, "HANGUL JONGSEONG YESIEUNG-MIEUM" },
    { 0xD7F6, "HANGUL JONGSEONG YESIEUNG-HIEUH" },
    { 0xD7F7, "HANGUL JONGSEONG CIEUC-PIEUP" },
    { 0xD7F8, "HANGUL JONGSEONG CIEUC-SSANGPIEUP" },
    { 0xD7F9, "HANGUL JONGSEONG SSANGCIEUC" },
    { 0xD7FA, "HANGUL JONGSEONG PHIEUPH-SIOS" },
    { 0xD7FB, "HANGUL JONGSEONG PHIEUPH-THIEUTH" },
    { 0xF900, "CJK COMPATIBILITY IDEOGRAPH-F900" },
    { 0xF901, "CJK COMPATIBILITY IDEOGRAPH-F901" },
    { 0xF902, "CJK COMPATIBILITY IDEOGRAPH-F902" },
    { 0xF903, "CJK COMPATIBILITY IDEOGRAPH-F903" },
    { 0xF904, "CJK COMPATIBILITY IDEOGRAPH-F904" },
    { 0xF905, "CJK COMPATIBILITY IDEOGRAPH-F905" },
    { 0xF906, "CJK COMPATIBILITY IDEOGRAPH-F906" },
    { 0xF907, "CJK COMPATIBILITY IDEOGRAPH-F907" },
    { 0xF908, "CJK COMPATIBILITY IDEOGRAPH-F908" },
    { 0xF909, "CJK COMPATIBILITY IDEOGRAPH-F909" },
    { 0xF90A, "CJK COMPATIBILITY IDEOGRAPH-F90A" },
    { 0xF90B, "CJK COMPATIBILITY IDEOGRAPH-F90B" },
    { 0xF90C, "CJK COMPATIBILITY IDEOGRAPH-F90C" },
    { 0xF90D, "CJK COMPATIBILITY IDEOGRAPH-F90D" },
    { 0xF90E, "CJK COMPATIBILITY IDEOGRAPH-F90E" },
    { 0xF90F, "CJK COMPATIBILITY IDEOGRAPH-F90F" },
    { 0xF910, "CJK COMPATIBILITY IDEOGRAPH-F910" },
    { 0xF911, "CJK COMPATIBILITY IDEOGRAPH-F911" },
    { 0xF912, "CJK COMPATIBILITY IDEOGRAPH-F912" },
    { 0xF913, "CJK COMPATIBILITY IDEOGRAPH-F913" },
    { 0xF914, "CJK COMPATIBILITY IDEOGRAPH-F914" },
    { 0xF915, "CJK COMPATIBILITY IDEOGRAPH-F915" },
    { 0xF916, "CJK COMPATIBILITY IDEOGRAPH-F916" },
    { 0xF917, "CJK COMPATIBILITY IDEOGRAPH-F917" },
    { 0xF918, "CJK COMPATIBILITY IDEOGRAPH-F918" },
    { 0xF919, "CJK COMPATIBILITY IDEOGRAPH-F919" },
    { 0xF91A, "CJK COMPATIBILITY IDEOGRAPH-F91A" },
    { 0xF91B, "CJK COMPATIBILITY IDEOGRAPH-F91B" },
    { 0xF91C, "CJK COMPATIBILITY IDEOGRAPH-F91C" },
    { 0xF91D, "CJK COMPATIBILITY IDEOGRAPH-F91D" },
    { 0xF91E, "CJK COMPATIBILITY IDEOGRAPH-F91E" },
    { 0xF91F, "CJK COMPATIBILITY IDEOGRAPH-F91F" },
    { 0xF920, "CJK COMPATIBILITY IDEOGRAPH-F920" },
    { 0xF921, "CJK COMPATIBILITY IDEOGRAPH-F921" },
    { 0xF922, "CJK COMPATIBILITY IDEOGRAPH-F922" },
    { 0xF923, "CJK COMPATIBILITY IDEOGRAPH-F923" },
    { 0xF924, "CJK COMPATIBILITY IDEOGRAPH-F924" },
    { 0xF925, "CJK COMPATIBILITY IDEOGRAPH-F925" },
    { 0xF926, "CJK COMPATIBILITY IDEOGRAPH-F926" },
    { 0xF927, "CJK COMPATIBILITY IDEOGRAPH-F927" },
    { 0xF928, "CJK COMPATIBILITY IDEOGRAPH-F928" },
    { 0xF929, "CJK COMPATIBILITY IDEOGRAPH-F929" },
    { 0xF92A, "CJK COMPATIBILITY IDEOGRAPH-F92A" },
    { 0xF92B, "CJK COMPATIBILITY IDEOGRAPH-F92B" },
    { 0xF92C, "CJK COMPATIBILITY IDEOGRAPH-F92C" },
    { 0xF92D, "CJK COMPATIBILITY IDEOGRAPH-F92D" },
    { 0xF92E, "CJK COMPATIBILITY IDEOGRAPH-F92E" },
    { 0xF92F, "CJK COMPATIBILITY IDEOGRAPH-F92F" },
    { 0xF930, "CJK COMPATIBILITY IDEOGRAPH-F930" },
    { 0xF931, "CJK COMPATIBILITY IDEOGRAPH-F931" },
    { 0xF932, "CJK COMPATIBILITY IDEOGRAPH-F932" },
    { 0xF933, "CJK COMPATIBILITY IDEOGRAPH-F933" },
    { 0xF934, "CJK COMPATIBILITY IDEOGRAPH-F934" },
    { 0xF935, "CJK COMPATIBILITY IDEOGRAPH-F935" },
    { 0xF936, "CJK COMPATIBILITY IDEOGRAPH-F936" },
    { 0xF937, "CJK COMPATIBILITY IDEOGRAPH-F937" },
    { 0xF938, "CJK COMPATIBILITY IDEOGRAPH-F938" },
    { 0xF939, "CJK COMPATIBILITY IDEOGRAPH-F939" },
    { 0xF93A, "CJK COMPATIBILITY IDEOGRAPH-F93A" },
    { 0xF93B, "CJK COMPATIBILITY IDEOGRAPH-F93B" },
    { 0xF93C, "CJK COMPATIBILITY IDEOGRAPH-F93C" },
    { 0xF93D, "CJK COMPATIBILITY IDEOGRAPH-F93D" },
    { 0xF93E, "CJK COMPATIBILITY IDEOGRAPH-F93E" },
    { 0xF93F, "CJK COMPATIBILITY IDEOGRAPH-F93F" },
    { 0xF940, "CJK COMPATIBILITY IDEOGRAPH-F940" },
    { 0xF941, "CJK COMPATIBILITY IDEOGRAPH-F941" },
    { 0xF942, "CJK COMPATIBILITY IDEOGRAPH-F942" },
    { 0xF943, "CJK COMPATIBILITY IDEOGRAPH-F943" },
    { 0xF944, "CJK COMPATIBILITY IDEOGRAPH-F944" },
    { 0xF945, "CJK COMPATIBILITY IDEOGRAPH-F945" },
    { 0xF946, "CJK COMPATIBILITY IDEOGRAPH-F946" },
    { 0xF947, "CJK COMPATIBILITY IDEOGRAPH-F947" },
    { 0xF948, "CJK COMPATIBILITY IDEOGRAPH-F948" },
    { 0xF949, "CJK COMPATIBILITY IDEOGRAPH-F949" },
    { 0xF94A, "CJK COMPATIBILITY IDEOGRAPH-F94A" },
    { 0xF94B, "CJK COMPATIBILITY IDEOGRAPH-F94B" },
    { 0xF94C, "CJK COMPATIBILITY IDEOGRAPH-F94C" },
    { 0xF94D, "CJK COMPATIBILITY IDEOGRAPH-F94D" },
    { 0xF94E, "CJK COMPATIBILITY IDEOGRAPH-F94E" },
    { 0xF94F, "CJK COMPATIBILITY IDEOGRAPH-F94F" },
    { 0xF950, "CJK COMPATIBILITY IDEOGRAPH-F950" },
    { 0xF951, "CJK COMPATIBILITY IDEOGRAPH-F951" },
    { 0xF952, "CJK COMPATIBILITY IDEOGRAPH-F952" },
    { 0xF953, "CJK COMPATIBILITY IDEOGRAPH-F953" },
    { 0xF954, "CJK COMPATIBILITY IDEOGRAPH-F954" },
    { 0xF955, "CJK COMPATIBILITY IDEOGRAPH-F955" },
    { 0xF956, "CJK COMPATIBILITY IDEOGRAPH-F956" },
    { 0xF957, "CJK COMPATIBILITY IDEOGRAPH-F957" },
    { 0xF958, "CJK COMPATIBILITY IDEOGRAPH-F958" },
    { 0xF959, "CJK COMPATIBILITY IDEOGRAPH-F959" },
    { 0xF95A, "CJK COMPATIBILITY IDEOGRAPH-F95A" },
    { 0xF95B, "CJK COMPATIBILITY IDEOGRAPH-F95B" },
    { 0xF95C, "CJK COMPATIBILITY IDEOGRAPH-F95C" },
    { 0xF95D, "CJK COMPATIBILITY IDEOGRAPH-F95D" },
    { 0xF95E, "CJK COMPATIBILITY IDEOGRAPH-F95E" },
    { 0xF95F, "CJK COMPATIBILITY IDEOGRAPH-F95F" },
    { 0xF960, "CJK COMPATIBILITY IDEOGRAPH-F960" },
    { 0xF961, "CJK COMPATIBILITY IDEOGRAPH-F961" },
    { 0xF962, "CJK COMPATIBILITY IDEOGRAPH-F962" },
    { 0xF963, "CJK COMPATIBILITY IDEOGRAPH-F963" },
    { 0xF964, "CJK COMPATIBILITY IDEOGRAPH-F964" },
    { 0xF965, "CJK COMPATIBILITY IDEOGRAPH-F965" },
    { 0xF966, "CJK COMPATIBILITY IDEOGRAPH-F966" },
    { 0xF967, "CJK COMPATIBILITY IDEOGRAPH-F967" },
    { 0xF968, "CJK COMPATIBILITY IDEOGRAPH-F968" },
    { 0xF969, "CJK COMPATIBILITY IDEOGRAPH-F969" },
    { 0xF96A, "CJK COMPATIBILITY IDEOGRAPH-F96A" },
    { 0xF96B, "CJK COMPATIBILITY IDEOGRAPH-F96B" },
    { 0xF96C, "CJK COMPATIBILITY IDEOGRAPH-F96C" },
    { 0xF96D, "CJK COMPATIBILITY IDEOGRAPH-F96D" },
    { 0xF96E, "CJK COMPATIBILITY IDEOGRAPH-F96E" },
    { 0xF96F, "CJK COMPATIBILITY IDEOGRAPH-F96F" },
    { 0xF970, "CJK COMPATIBILITY IDEOGRAPH-F970" },
    { 0xF971, "CJK COMPATIBILITY IDEOGRAPH-F971" },
    { 0xF972, "CJK COMPATIBILITY IDEOGRAPH-F972" },
    { 0xF973, "CJK COMPATIBILITY IDEOGRAPH-F973" },
    { 0xF974, "CJK COMPATIBILITY IDEOGRAPH-F974" },
    { 0xF975, "CJK COMPATIBILITY IDEOGRAPH-F975" },
    { 0xF976, "CJK COMPATIBILITY IDEOGRAPH-F976" },
    { 0xF977, "CJK COMPATIBILITY IDEOGRAPH-F977" },
    { 0xF978, "CJK COMPATIBILITY IDEOGRAPH-F978" },
    { 0xF979, "CJK COMPATIBILITY IDEOGRAPH-F979" },
    { 0xF97A, "CJK COMPATIBILITY IDEOGRAPH-F97A" },
    { 0xF97B, "CJK COMPATIBILITY IDEOGRAPH-F97B" },
    { 0xF97C, "CJK COMPATIBILITY IDEOGRAPH-F97C" },
    { 0xF97D, "CJK COMPATIBILITY IDEOGRAPH-F97D" },
    { 0xF97E, "CJK COMPATIBILITY IDEOGRAPH-F97E" },
    { 0xF97F, "CJK COMPATIBILITY IDEOGRAPH-F97F" },
    { 0xF980, "CJK COMPATIBILITY IDEOGRAPH-F980" },
    { 0xF981, "CJK COMPATIBILITY IDEOGRAPH-F981" },
    { 0xF982, "CJK COMPATIBILITY IDEOGRAPH-F982" },
    { 0xF983, "CJK COMPATIBILITY IDEOGRAPH-F983" },
    { 0xF984, "CJK COMPATIBILITY IDEOGRAPH-F984" },
    { 0xF985, "CJK COMPATIBILITY IDEOGRAPH-F985" },
    { 0xF986, "CJK COMPATIBILITY IDEOGRAPH-F986" },
    { 0xF987, "CJK COMPATIBILITY IDEOGRAPH-F987" },
    { 0xF988, "CJK COMPATIBILITY IDEOGRAPH-F988" },
    { 0xF989, "CJK COMPATIBILITY IDEOGRAPH-F989" },
    { 0xF98A, "CJK COMPATIBILITY IDEOGRAPH-F98A" },
    { 0xF98B, "CJK COMPATIBILITY IDEOGRAPH-F98B" },
    { 0xF98C, "CJK COMPATIBILITY IDEOGRAPH-F98C" },
    { 0xF98D, "CJK COMPATIBILITY IDEOGRAPH-F98D" },
    { 0xF98E, "CJK COMPATIBILITY IDEOGRAPH-F98E" },
    { 0xF98F, "CJK COMPATIBILITY IDEOGRAPH-F98F" },
    { 0xF990, "CJK COMPATIBILITY IDEOGRAPH-F990" },
    { 0xF991, "CJK COMPATIBILITY IDEOGRAPH-F991" },
    { 0xF992, "CJK COMPATIBILITY IDEOGRAPH-F992" },
    { 0xF993, "CJK COMPATIBILITY IDEOGRAPH-F993" },
    { 0xF994, "CJK COMPATIBILITY IDEOGRAPH-F994" },
    { 0xF995, "CJK COMPATIBILITY IDEOGRAPH-F995" },
    { 0xF996, "CJK COMPATIBILITY IDEOGRAPH-F996" },
    { 0xF997, "CJK COMPATIBILITY IDEOGRAPH-F997" },
    { 0xF998, "CJK COMPATIBILITY IDEOGRAPH-F998" },
    { 0xF999, "CJK COMPATIBILITY IDEOGRAPH-F999" },
    { 0xF99A, "CJK COMPATIBILITY IDEOGRAPH-F99A" },
    { 0xF99B, "CJK COMPATIBILITY IDEOGRAPH-F99B" },
    { 0xF99C, "CJK COMPATIBILITY IDEOGRAPH-F99C" },
    { 0xF99D, "CJK COMPATIBILITY IDEOGRAPH-F99D" },
    { 0xF99E, "CJK COMPATIBILITY IDEOGRAPH-F99E" },
    { 0xF99F, "CJK COMPATIBILITY IDEOGRAPH-F99F" },
    { 0xF9A0, "CJK COMPATIBILITY IDEOGRAPH-F9A0" },
    { 0xF9A1, "CJK COMPATIBILITY IDEOGRAPH-F9A1" },
    { 0xF9A2, "CJK COMPATIBILITY IDEOGRAPH-F9A2" },
    { 0xF9A3, "CJK COMPATIBILITY IDEOGRAPH-F9A3" },
    { 0xF9A4, "CJK COMPATIBILITY IDEOGRAPH-F9A4" },
    { 0xF9A5, "CJK COMPATIBILITY IDEOGRAPH-F9A5" },
    { 0xF9A6, "CJK COMPATIBILITY IDEOGRAPH-F9A6" },
    { 0xF9A7, "CJK COMPATIBILITY IDEOGRAPH-F9A7" },
    { 0xF9A8, "CJK COMPATIBILITY IDEOGRAPH-F9A8" },
    { 0xF9A9, "CJK COMPATIBILITY IDEOGRAPH-F9A9" },
    { 0xF9AA, "CJK COMPATIBILITY IDEOGRAPH-F9AA" },
    { 0xF9AB, "CJK COMPATIBILITY IDEOGRAPH-F9AB" },
    { 0xF9AC, "CJK COMPATIBILITY IDEOGRAPH-F9AC" },
    { 0xF9AD, "CJK COMPATIBILITY IDEOGRAPH-F9AD" },
    { 0xF9AE, "CJK COMPATIBILITY IDEOGRAPH-F9AE" },
    { 0xF9AF, "CJK COMPATIBILITY IDEOGRAPH-F9AF" },
    { 0xF9B0, "CJK COMPATIBILITY IDEOGRAPH-F9B0" },
    { 0xF9B1, "CJK COMPATIBILITY IDEOGRAPH-F9B1" },
    { 0xF9B2, "CJK COMPATIBILITY IDEOGRAPH-F9B2" },
    { 0xF9B3, "CJK COMPATIBILITY IDEOGRAPH-F9B3" },
    { 0xF9B4, "CJK COMPATIBILITY IDEOGRAPH-F9B4" },
    { 0xF9B5, "CJK COMPATIBILITY IDEOGRAPH-F9B5" },
    { 0xF9B6, "CJK COMPATIBILITY IDEOGRAPH-F9B6" },
    { 0xF9B7, "CJK COMPATIBILITY IDEOGRAPH-F9B7" },
    { 0xF9B8, "CJK COMPATIBILITY IDEOGRAPH-F9B8" },
    { 0xF9B9, "CJK COMPATIBILITY IDEOGRAPH-F9B9" },
    { 0xF9BA, "CJK COMPATIBILITY IDEOGRAPH-F9BA" },
    { 0xF9BB, "CJK COMPATIBILITY IDEOGRAPH-F9BB" },
    { 0xF9BC, "CJK COMPATIBILITY IDEOGRAPH-F9BC" },
    { 0xF9BD, "CJK COMPATIBILITY IDEOGRAPH-F9BD" },
    { 0xF9BE, "CJK COMPATIBILITY IDEOGRAPH-F9BE" },
    { 0xF9BF, "CJK COMPATIBILITY IDEOGRAPH-F9BF" },
    { 0xF9C0, "CJK COMPATIBILITY IDEOGRAPH-F9C0" },
    { 0xF9C1, "CJK COMPATIBILITY IDEOGRAPH-F9C1" },
    { 0xF9C2, "CJK COMPATIBILITY IDEOGRAPH-F9C2" },
    { 0xF9C3, "CJK COMPATIBILITY IDEOGRAPH-F9C3" },
    { 0xF9C4, "CJK COMPATIBILITY IDEOGRAPH-F9C4" },
    { 0xF9C5, "CJK COMPATIBILITY IDEOGRAPH-F9C5" },
    { 0xF9C6, "CJK COMPATIBILITY IDEOGRAPH-F9C6" },
    { 0xF9C7, "CJK COMPATIBILITY IDEOGRAPH-F9C7" },
    { 0xF9C8, "CJK COMPATIBILITY IDEOGRAPH-F9C8" },
    { 0xF9C9, "CJK COMPATIBILITY IDEOGRAPH-F9C9" },
    { 0xF9CA, "CJK COMPATIBILITY IDEOGRAPH-F9CA" },
    { 0xF9CB, "CJK COMPATIBILITY IDEOGRAPH-F9CB" },
    { 0xF9CC, "CJK COMPATIBILITY IDEOGRAPH-F9CC" },
    { 0xF9CD, "CJK COMPATIBILITY IDEOGRAPH-F9CD" },
    { 0xF9CE, "CJK COMPATIBILITY IDEOGRAPH-F9CE" },
    { 0xF9CF, "CJK COMPATIBILITY IDEOGRAPH-F9CF" },
    { 0xF9D0, "CJK COMPATIBILITY IDEOGRAPH-F9D0" },
    { 0xF9D1, "CJK COMPATIBILITY IDEOGRAPH-F9D1" },
    { 0xF9D2, "CJK COMPATIBILITY IDEOGRAPH-F9D2" },
    { 0xF9D3, "CJK COMPATIBILITY IDEOGRAPH-F9D3" },
    { 0xF9D4, "CJK COMPATIBILITY IDEOGRAPH-F9D4" },
    { 0xF9D5, "CJK COMPATIBILITY IDEOGRAPH-F9D5" },
    { 0xF9D6, "CJK COMPATIBILITY IDEOGRAPH-F9D6" },
    { 0xF9D7, "CJK COMPATIBILITY IDEOGRAPH-F9D7" },
    { 0xF9D8, "CJK COMPATIBILITY IDEOGRAPH-F9D8" },
    { 0xF9D9, "CJK COMPATIBILITY IDEOGRAPH-F9D9" },
    { 0xF9DA, "CJK COMPATIBILITY IDEOGRAPH-F9DA" },
    { 0xF9DB, "CJK COMPATIBILITY IDEOGRAPH-F9DB" },
    { 0xF9DC, "CJK COMPATIBILITY IDEOGRAPH-F9DC" },
    { 0xF9DD, "CJK COMPATIBILITY IDEOGRAPH-F9DD" },
    { 0xF9DE, "CJK COMPATIBILITY IDEOGRAPH-F9DE" },
    { 0xF9DF, "CJK COMPATIBILITY IDEOGRAPH-F9DF" },
    { 0xF9E0, "CJK COMPATIBILITY IDEOGRAPH-F9E0" },
    { 0xF9E1, "CJK COMPATIBILITY IDEOGRAPH-F9E1" },
    { 0xF9E2, "CJK COMPATIBILITY IDEOGRAPH-F9E2" },
    { 0xF9E3, "CJK COMPATIBILITY IDEOGRAPH-F9E3" },
    { 0xF9E4, "CJK COMPATIBILITY IDEOGRAPH-F9E4" },
    { 0xF9E5, "CJK COMPATIBILITY IDEOGRAPH-F9E5" },
    { 0xF9E6, "CJK COMPATIBILITY IDEOGRAPH-F9E6" },
    { 0xF9E7, "CJK COMPATIBILITY IDEOGRAPH-F9E7" },
    { 0xF9E8, "CJK COMPATIBILITY IDEOGRAPH-F9E8" },
    { 0xF9E9, "CJK COMPATIBILITY IDEOGRAPH-F9E9" },
    { 0xF9EA, "CJK COMPATIBILITY IDEOGRAPH-F9EA" },
    { 0xF9EB, "CJK COMPATIBILITY IDEOGRAPH-F9EB" },
    { 0xF9EC, "CJK COMPATIBILITY IDEOGRAPH-F9EC" },
    { 0xF9ED, "CJK COMPATIBILITY IDEOGRAPH-F9ED" },
    { 0xF9EE, "CJK COMPATIBILITY IDEOGRAPH-F9EE" },
    { 0xF9EF, "CJK COMPATIBILITY IDEOGRAPH-F9EF" },
    { 0xF9F0, "CJK COMPATIBILITY IDEOGRAPH-F9F0" },
    { 0xF9F1, "CJK COMPATIBILITY IDEOGRAPH-F9F1" },
    { 0xF9F2, "CJK COMPATIBILITY IDEOGRAPH-F9F2" },
    { 0xF9F3, "CJK COMPATIBILITY IDEOGRAPH-F9F3" },
    { 0xF9F4, "CJK COMPATIBILITY IDEOGRAPH-F9F4" },
    { 0xF9F5, "CJK COMPATIBILITY IDEOGRAPH-F9F5" },
    { 0xF9F6, "CJK COMPATIBILITY IDEOGRAPH-F9F6" },
    { 0xF9F7, "CJK COMPATIBILITY IDEOGRAPH-F9F7" },
    { 0xF9F8, "CJK COMPATIBILITY IDEOGRAPH-F9F8" },
    { 0xF9F9, "CJK COMPATIBILITY IDEOGRAPH-F9F9" },
    { 0xF9FA, "CJK COMPATIBILITY IDEOGRAPH-F9FA" },
    { 0xF9FB, "CJK COMPATIBILITY IDEOGRAPH-F9FB" },
    { 0xF9FC, "CJK COMPATIBILITY IDEOGRAPH-F9FC" },
    { 0xF9FD, "CJK COMPATIBILITY IDEOGRAPH-F9FD" },
    { 0xF9FE, "CJK COMPATIBILITY IDEOGRAPH-F9FE" },
    { 0xF9FF, "CJK COMPATIBILITY IDEOGRAPH-F9FF" },
    { 0xFA00, "CJK COMPATIBILITY IDEOGRAPH-FA00" },
    { 0xFA01, "CJK COMPATIBILITY IDEOGRAPH-FA01" },
    { 0xFA02, "CJK COMPATIBILITY IDEOGRAPH-FA02" },
    { 0xFA03, "CJK COMPATIBILITY IDEOGRAPH-FA03" },
    { 0xFA04, "CJK COMPATIBILITY IDEOGRAPH-FA04" },
    { 0xFA05, "CJK COMPATIBILITY IDEOGRAPH-FA05" },
    { 0xFA06, "CJK COMPATIBILITY IDEOGRAPH-FA06" },
    { 0xFA07, "CJK COMPATIBILITY IDEOGRAPH-FA07" },
    { 0xFA08, "CJK COMPATIBILITY IDEOGRAPH-FA08" },
    { 0xFA09, "CJK COMPATIBILITY IDEOGRAPH-FA09" },
    { 0xFA0A, "CJK COMPATIBILITY IDEOGRAPH-FA0A" },
    { 0xFA0B, "CJK COMPATIBILITY IDEOGRAPH-FA0B" },
    { 0xFA0C, "CJK COMPATIBILITY IDEOGRAPH-FA0C" },
    { 0xFA0D, "CJK COMPATIBILITY IDEOGRAPH-FA0D" },
    { 0xFA0E, "CJK COMPATIBILITY IDEOGRAPH-FA0E" },
    { 0xFA0F, "CJK COMPATIBILITY IDEOGRAPH-FA0F" },
    { 0xFA10, "CJK COMPATIBILITY IDEOGRAPH-FA10" },
    { 0xFA11, "CJK COMPATIBILITY IDEOGRAPH-FA11" },
    { 0xFA12, "CJK COMPATIBILITY IDEOGRAPH-FA12" },
    { 0xFA13, "CJK COMPATIBILITY IDEOGRAPH-FA13" },
    { 0xFA14, "CJK COMPATIBILITY IDEOGRAPH-FA14" },
    { 0xFA15, "CJK COMPATIBILITY IDEOGRAPH-FA15" },
    { 0xFA16, "CJK COMPATIBILITY IDEOGRAPH-FA16" },
    { 0xFA17, "CJK COMPATIBILITY IDEOGRAPH-FA17" },
    { 0xFA18, "CJK COMPATIBILITY IDEOGRAPH-FA18" },
    { 0xFA19, "CJK COMPATIBILITY IDEOGRAPH-FA19" },
    { 0xFA1A, "CJK COMPATIBILITY IDEOGRAPH-FA1A" },
    { 0xFA1B, "CJK COMPATIBILITY IDEOGRAPH-FA1B" },
    { 0xFA1C, "CJK COMPATIBILITY IDEOGRAPH-FA1C" },
    { 0xFA1D, "CJK COMPATIBILITY IDEOGRAPH-FA1D" },
    { 0xFA1E, "CJK COMPATIBILITY IDEOGRAPH-FA1E" },
    { 0xFA1F, "CJK COMPATIBILITY IDEOGRAPH-FA1F" },
    { 0xFA20, "CJK COMPATIBILITY IDEOGRAPH-FA20" },
    { 0xFA21, "CJK COMPATIBILITY IDEOGRAPH-FA21" },
    { 0xFA22, "CJK COMPATIBILITY IDEOGRAPH-FA22" },
    { 0xFA23, "CJK COMPATIBILITY IDEOGRAPH-FA23" },
    { 0xFA24, "CJK COMPATIBILITY IDEOGRAPH-FA24" },
    { 0xFA25, "CJK COMPATIBILITY IDEOGRAPH-FA25" },
    { 0xFA26, "CJK COMPATIBILITY IDEOGRAPH-FA26" },
    { 0xFA27, "CJK COMPATIBILITY IDEOGRAPH-FA27" },
    { 0xFA28, "CJK COMPATIBILITY IDEOGRAPH-FA28" },
    { 0xFA29, "CJK COMPATIBILITY IDEOGRAPH-FA29" },
    { 0xFA2A, "CJK COMPATIBILITY IDEOGRAPH-FA2A" },
    { 0xFA2B, "CJK COMPATIBILITY IDEOGRAPH-FA2B" },
    { 0xFA2C, "CJK COMPATIBILITY IDEOGRAPH-FA2C" },
    { 0xFA2D, "CJK COMPATIBILITY IDEOGRAPH-FA2D" },
    { 0xFA2E, "CJK COMPATIBILITY IDEOGRAPH-FA2E" },
    { 0xFA2F, "CJK COMPATIBILITY IDEOGRAPH-FA2F" },
    { 0xFA30, "CJK COMPATIBILITY IDEOGRAPH-FA30" },
    { 0xFA31, "CJK COMPATIBILITY IDEOGRAPH-FA31" },
    { 0xFA32, "CJK COMPATIBILITY IDEOGRAPH-FA32" },
    { 0xFA33, "CJK COMPATIBILITY IDEOGRAPH-FA33" },
    { 0xFA34, "CJK COMPATIBILITY IDEOGRAPH-FA34" },
    { 0xFA35, "CJK COMPATIBILITY IDEOGRAPH-FA35" },
    { 0xFA36, "CJK COMPATIBILITY IDEOGRAPH-FA36" },
    { 0xFA37, "CJK COMPATIBILITY IDEOGRAPH-FA37" },
    { 0xFA38, "CJK COMPATIBILITY IDEOGRAPH-FA38" },
    { 0xFA39, "CJK COMPATIBILITY IDEOGRAPH-FA39" },
    { 0xFA3A, "CJK COMPATIBILITY IDEOGRAPH-FA3A" },
    { 0xFA3B, "CJK COMPATIBILITY IDEOGRAPH-FA3B" },
    { 0xFA3C, "CJK COMPATIBILITY IDEOGRAPH-FA3C" },
    { 0xFA3D, "CJK COMPATIBILITY IDEOGRAPH-FA3D" },
    { 0xFA3E, "CJK COMPATIBILITY IDEOGRAPH-FA3E" },
    { 0xFA3F, "CJK COMPATIBILITY IDEOGRAPH-FA3F" },
    { 0xFA40, "CJK COMPATIBILITY IDEOGRAPH-FA40" },
    { 0xFA41, "CJK COMPATIBILITY IDEOGRAPH-FA41" },
    { 0xFA42, "CJK COMPATIBILITY IDEOGRAPH-FA42" },
    { 0xFA43, "CJK COMPATIBILITY IDEOGRAPH-FA43" },
    { 0xFA44, "CJK COMPATIBILITY IDEOGRAPH-FA44" },
    { 0xFA45, "CJK COMPATIBILITY IDEOGRAPH-FA45" },
    { 0xFA46, "CJK COMPATIBILITY IDEOGRAPH-FA46" },
    { 0xFA47, "CJK COMPATIBILITY IDEOGRAPH-FA47" },
    { 0xFA48, "CJK COMPATIBILITY IDEOGRAPH-FA48" },
    { 0xFA49, "CJK COMPATIBILITY IDEOGRAPH-FA49" },
    { 0xFA4A, "CJK COMPATIBILITY IDEOGRAPH-FA4A" },
    { 0xFA4B, "CJK COMPATIBILITY IDEOGRAPH-FA4B" },
    { 0xFA4C, "CJK COMPATIBILITY IDEOGRAPH-FA4C" },
    { 0xFA4D, "CJK COMPATIBILITY IDEOGRAPH-FA4D" },
    { 0xFA4E, "CJK COMPATIBILITY IDEOGRAPH-FA4E" },
    { 0xFA4F, "CJK COMPATIBILITY IDEOGRAPH-FA4F" },
    { 0xFA50, "CJK COMPATIBILITY IDEOGRAPH-FA50" },
    { 0xFA51, "CJK COMPATIBILITY IDEOGRAPH-FA51" },
    { 0xFA52, "CJK COMPATIBILITY IDEOGRAPH-FA52" },
    { 0xFA53, "CJK COMPATIBILITY IDEOGRAPH-FA53" },
    { 0xFA54, "CJK COMPATIBILITY IDEOGRAPH-FA54" },
    { 0xFA55, "CJK COMPATIBILITY IDEOGRAPH-FA55" },
    { 0xFA56, "CJK COMPATIBILITY IDEOGRAPH-FA56" },
    { 0xFA57, "CJK COMPATIBILITY IDEOGRAPH-FA57" },
    { 0xFA58, "CJK COMPATIBILITY IDEOGRAPH-FA58" },
    { 0xFA59, "CJK COMPATIBILITY IDEOGRAPH-FA59" },
    { 0xFA5A, "CJK COMPATIBILITY IDEOGRAPH-FA5A" },
    { 0xFA5B, "CJK COMPATIBILITY IDEOGRAPH-FA5B" },
    { 0xFA5C, "CJK COMPATIBILITY IDEOGRAPH-FA5C" },
    { 0xFA5D, "CJK COMPATIBILITY IDEOGRAPH-FA5D" },
    { 0xFA5E, "CJK COMPATIBILITY IDEOGRAPH-FA5E" },
    { 0xFA5F, "CJK COMPATIBILITY IDEOGRAPH-FA5F" },
    { 0xFA60, "CJK COMPATIBILITY IDEOGRAPH-FA60" },
    { 0xFA61, "CJK COMPATIBILITY IDEOGRAPH-FA61" },
    { 0xFA62, "CJK COMPATIBILITY IDEOGRAPH-FA62" },
    { 0xFA63, "CJK COMPATIBILITY IDEOGRAPH-FA63" },
    { 0xFA64, "CJK COMPATIBILITY IDEOGRAPH-FA64" },
    { 0xFA65, "CJK COMPATIBILITY IDEOGRAPH-FA65" },
    { 0xFA66, "CJK COMPATIBILITY IDEOGRAPH-FA66" },
    { 0xFA67, "CJK COMPATIBILITY IDEOGRAPH-FA67" },
    { 0xFA68, "CJK COMPATIBILITY IDEOGRAPH-FA68" },
    { 0xFA69, "CJK COMPATIBILITY IDEOGRAPH-FA69" },
    { 0xFA6A, "CJK COMPATIBILITY IDEOGRAPH-FA6A" },
    { 0xFA6B, "CJK COMPATIBILITY IDEOGRAPH-FA6B" },
    { 0xFA6C, "CJK COMPATIBILITY IDEOGRAPH-FA6C" },
    { 0xFA6D, "CJK COMPATIBILITY IDEOGRAPH-FA6D" },
    { 0xFA70, "CJK COMPATIBILITY IDEOGRAPH-FA70" },
    { 0xFA71, "CJK COMPATIBILITY IDEOGRAPH-FA71" },
    { 0xFA72, "CJK COMPATIBILITY IDEOGRAPH-FA72" },
    { 0xFA73, "CJK COMPATIBILITY IDEOGRAPH-FA73" },
    { 0xFA74, "CJK COMPATIBILITY IDEOGRAPH-FA74" },
    { 0xFA75, "CJK COMPATIBILITY IDEOGRAPH-FA75" },
    { 0xFA76, "CJK COMPATIBILITY IDEOGRAPH-FA76" },
    { 0xFA77, "CJK COMPATIBILITY IDEOGRAPH-FA77" },
    { 0xFA78, "CJK COMPATIBILITY IDEOGRAPH-FA78" },
    { 0xFA79, "CJK COMPATIBILITY IDEOGRAPH-FA79" },
    { 0xFA7A, "CJK COMPATIBILITY IDEOGRAPH-FA7A" },
    { 0xFA7B, "CJK COMPATIBILITY IDEOGRAPH-FA7B" },
    { 0xFA7C, "CJK COMPATIBILITY IDEOGRAPH-FA7C" },
    { 0xFA7D, "CJK COMPATIBILITY IDEOGRAPH-FA7D" },
    { 0xFA7E, "CJK COMPATIBILITY IDEOGRAPH-FA7E" },
    { 0xFA7F, "CJK COMPATIBILITY IDEOGRAPH-FA7F" },
    { 0xFA80, "CJK COMPATIBILITY IDEOGRAPH-FA80" },
    { 0xFA81, "CJK COMPATIBILITY IDEOGRAPH-FA81" },
    { 0xFA82, "CJK COMPATIBILITY IDEOGRAPH-FA82" },
    { 0xFA83, "CJK COMPATIBILITY IDEOGRAPH-FA83" },
    { 0xFA84, "CJK COMPATIBILITY IDEOGRAPH-FA84" },
    { 0xFA85, "CJK COMPATIBILITY IDEOGRAPH-FA85" },
    { 0xFA86, "CJK COMPATIBILITY IDEOGRAPH-FA86" },
    { 0xFA87, "CJK COMPATIBILITY IDEOGRAPH-FA87" },
    { 0xFA88, "CJK COMPATIBILITY IDEOGRAPH-FA88" },
    { 0xFA89, "CJK COMPATIBILITY IDEOGRAPH-FA89" },
    { 0xFA8A, "CJK COMPATIBILITY IDEOGRAPH-FA8A" },
    { 0xFA8B, "CJK COMPATIBILITY IDEOGRAPH-FA8B" },
    { 0xFA8C, "CJK COMPATIBILITY IDEOGRAPH-FA8C" },
    { 0xFA8D, "CJK COMPATIBILITY IDEOGRAPH-FA8D" },
    { 0xFA8E, "CJK COMPATIBILITY IDEOGRAPH-FA8E" },
    { 0xFA8F, "CJK COMPATIBILITY IDEOGRAPH-FA8F" },
    { 0xFA90, "CJK COMPATIBILITY IDEOGRAPH-FA90" },
    { 0xFA91, "CJK COMPATIBILITY IDEOGRAPH-FA91" },
    { 0xFA92, "CJK COMPATIBILITY IDEOGRAPH-FA92" },
    { 0xFA93, "CJK COMPATIBILITY IDEOGRAPH-FA93" },
    { 0xFA94, "CJK COMPATIBILITY IDEOGRAPH-FA94" },
    { 0xFA95, "CJK COMPATIBILITY IDEOGRAPH-FA95" },
    { 0xFA96, "CJK COMPATIBILITY IDEOGRAPH-FA96" },
    { 0xFA97, "CJK COMPATIBILITY IDEOGRAPH-FA97" },
    { 0xFA98, "CJK COMPATIBILITY IDEOGRAPH-FA98" },
    { 0xFA99, "CJK COMPATIBILITY IDEOGRAPH-FA99" },
    { 0xFA9A, "CJK COMPATIBILITY IDEOGRAPH-FA9A" },
    { 0xFA9B, "CJK COMPATIBILITY IDEOGRAPH-FA9B" },
    { 0xFA9C, "CJK COMPATIBILITY IDEOGRAPH-FA9C" },
    { 0xFA9D, "CJK COMPATIBILITY IDEOGRAPH-FA9D" },
    { 0xFA9E, "CJK COMPATIBILITY IDEOGRAPH-FA9E" },
    { 0xFA9F, "CJK COMPATIBILITY IDEOGRAPH-FA9F" },
    { 0xFAA0, "CJK COMPATIBILITY IDEOGRAPH-FAA0" },
    { 0xFAA1, "CJK COMPATIBILITY IDEOGRAPH-FAA1" },
    { 0xFAA2, "CJK COMPATIBILITY IDEOGRAPH-FAA2" },
    { 0xFAA3, "CJK COMPATIBILITY IDEOGRAPH-FAA3" },
    { 0xFAA4, "CJK COMPATIBILITY IDEOGRAPH-FAA4" },
    { 0xFAA5, "CJK COMPATIBILITY IDEOGRAPH-FAA5" },
    { 0xFAA6, "CJK COMPATIBILITY IDEOGRAPH-FAA6" },
    { 0xFAA7, "CJK COMPATIBILITY IDEOGRAPH-FAA7" },
    { 0xFAA8, "CJK COMPATIBILITY IDEOGRAPH-FAA8" },
    { 0xFAA9, "CJK COMPATIBILITY IDEOGRAPH-FAA9" },
    { 0xFAAA, "CJK COMPATIBILITY IDEOGRAPH-FAAA" },
    { 0xFAAB, "CJK COMPATIBILITY IDEOGRAPH-FAAB" },
    { 0xFAAC, "CJK COMPATIBILITY IDEOGRAPH-FAAC" },
    { 0xFAAD, "CJK COMPATIBILITY IDEOGRAPH-FAAD" },
    { 0xFAAE, "CJK COMPATIBILITY IDEOGRAPH-FAAE" },
    { 0xFAAF, "CJK COMPATIBILITY IDEOGRAPH-FAAF" },
    { 0xFAB0, "CJK COMPATIBILITY IDEOGRAPH-FAB0" },
    { 0xFAB1, "CJK COMPATIBILITY IDEOGRAPH-FAB1" },
    { 0xFAB2, "CJK COMPATIBILITY IDEOGRAPH-FAB2" },
    { 0xFAB3, "CJK COMPATIBILITY IDEOGRAPH-FAB3" },
    { 0xFAB4, "CJK COMPATIBILITY IDEOGRAPH-FAB4" },
    { 0xFAB5, "CJK COMPATIBILITY IDEOGRAPH-FAB5" },
    { 0xFAB6, "CJK COMPATIBILITY IDEOGRAPH-FAB6" },
    { 0xFAB7, "CJK COMPATIBILITY IDEOGRAPH-FAB7" },
    { 0xFAB8, "CJK COMPATIBILITY IDEOGRAPH-FAB8" },
    { 0xFAB9, "CJK COMPATIBILITY IDEOGRAPH-FAB9" },
    { 0xFABA, "CJK COMPATIBILITY IDEOGRAPH-FABA" },
    { 0xFABB, "CJK COMPATIBILITY IDEOGRAPH-FABB" },
    { 0xFABC, "CJK COMPATIBILITY IDEOGRAPH-FABC" },
    { 0xFABD, "CJK COMPATIBILITY IDEOGRAPH-FABD" },
    { 0xFABE, "CJK COMPATIBILITY IDEOGRAPH-FABE" },
    { 0xFABF, "CJK COMPATIBILITY IDEOGRAPH-FABF" },
    { 0xFAC0, "CJK COMPATIBILITY IDEOGRAPH-FAC0" },
    { 0xFAC1, "CJK COMPATIBILITY IDEOGRAPH-FAC1" },
    { 0xFAC2, "CJK COMPATIBILITY IDEOGRAPH-FAC2" },
    { 0xFAC3, "CJK COMPATIBILITY IDEOGRAPH-FAC3" },
    { 0xFAC4, "CJK COMPATIBILITY IDEOGRAPH-FAC4" },
    { 0xFAC5, "CJK COMPATIBILITY IDEOGRAPH-FAC5" },
    { 0xFAC6, "CJK COMPATIBILITY IDEOGRAPH-FAC6" },
    { 0xFAC7, "CJK COMPATIBILITY IDEOGRAPH-FAC7" },
    { 0xFAC8, "CJK COMPATIBILITY IDEOGRAPH-FAC8" },
    { 0xFAC9, "CJK COMPATIBILITY IDEOGRAPH-FAC9" },
    { 0xFACA, "CJK COMPATIBILITY IDEOGRAPH-FACA" },
    { 0xFACB, "CJK COMPATIBILITY IDEOGRAPH-FACB" },
    { 0xFACC, "CJK COMPATIBILITY IDEOGRAPH-FACC" },
    { 0xFACD, "CJK COMPATIBILITY IDEOGRAPH-FACD" },
    { 0xFACE, "CJK COMPATIBILITY IDEOGRAPH-FACE" },
    { 0xFACF, "CJK COMPATIBILITY IDEOGRAPH-FACF" },
    { 0xFAD0, "CJK COMPATIBILITY IDEOGRAPH-FAD0" },
    { 0xFAD1, "CJK COMPATIBILITY IDEOGRAPH-FAD1" },
    { 0xFAD2, "CJK COMPATIBILITY IDEOGRAPH-FAD2" },
    { 0xFAD3, "CJK COMPATIBILITY IDEOGRAPH-FAD3" },
    { 0xFAD4, "CJK COMPATIBILITY IDEOGRAPH-FAD4" },
    { 0xFAD5, "CJK COMPATIBILITY IDEOGRAPH-FAD5" },
    { 0xFAD6, "CJK COMPATIBILITY IDEOGRAPH-FAD6" },
    { 0xFAD7, "CJK COMPATIBILITY IDEOGRAPH-FAD7" },
    { 0xFAD8, "CJK COMPATIBILITY IDEOGRAPH-FAD8" },
    { 0xFAD9, "CJK COMPATIBILITY IDEOGRAPH-FAD9" },
    { 0xFB00, "LATIN SMALL LIGATURE FF" },
    { 0xFB01, "LATIN SMALL LIGATURE FI" },
    { 0xFB02, "LATIN SMALL LIGATURE FL" },
    { 0xFB03, "LATIN SMALL LIGATURE FFI" },
    { 0xFB04, "LATIN SMALL LIGATURE FFL" },
    { 0xFB05, "LATIN SMALL LIGATURE LONG S T" },
    { 0xFB06, "LATIN SMALL LIGATURE ST" },
    { 0xFB13, "ARMENIAN SMALL LIGATURE MEN NOW" },
    { 0xFB14, "ARMENIAN SMALL LIGATURE MEN ECH" },
    { 0xFB15, "ARMENIAN SMALL LIGATURE MEN INI" },
    { 0xFB16, "ARMENIAN SMALL LIGATURE VEW NOW" },
    { 0xFB17, "ARMENIAN SMALL LIGATURE MEN XEH" },
    { 0xFB1D, "HEBREW LETTER YOD WITH HIRIQ" },
    { 0xFB1E, "HEBREW POINT JUDEO-SPANISH VARIKA" },
    { 0xFB1F, "HEBREW LIGATURE YIDDISH YOD YOD PATAH" },
    { 0xFB20, "HEBREW LETTER ALTERNATIVE AYIN" },
    { 0xFB21, "HEBREW LETTER WIDE ALEF" },
    { 0xFB22, "HEBREW LETTER WIDE DALET" },
    { 0xFB23, "HEBREW LETTER WIDE HE" },
    { 0xFB24, "HEBREW LETTER WIDE KAF" },
    { 0xFB25, "HEBREW LETTER WIDE LAMED" },
    { 0xFB26, "HEBREW LETTER WIDE FINAL MEM" },
    { 0xFB27, "HEBREW LETTER WIDE RESH" },
    { 0xFB28, "HEBREW LETTER WIDE TAV" },
    { 0xFB29, "HEBREW LETTER ALTERNATIVE PLUS SIGN" },
    { 0xFB2A, "HEBREW LETTER SHIN WITH SHIN DOT" },
    { 0xFB2B, "HEBREW LETTER SHIN WITH SIN DOT" },
    { 0xFB2C, "HEBREW LETTER SHIN WITH DAGESH AND SHIN DOT" },
    { 0xFB2D, "HEBREW LETTER SHIN WITH DAGESH AND SIN DOT" },
    { 0xFB2E, "HEBREW LETTER ALEF WITH PATAH" },
    { 0xFB2F, "HEBREW LETTER ALEF WITH QAMATS" },
    { 0xFB30, "HEBREW LETTER ALEF WITH MAPIQ" },
    { 0xFB31, "HEBREW LETTER BET WITH DAGESH" },
    { 0xFB32, "HEBREW LETTER GIMEL WITH DAGESH" },
    { 0xFB33, "HEBREW LETTER DALET WITH DAGESH" },
    { 0xFB34, "HEBREW LETTER HE WITH MAPIQ" },
    { 0xFB35, "HEBREW LETTER VAV WITH DAGESH" },
    { 0xFB36, "HEBREW LETTER ZAYIN WITH DAGESH" },
    { 0xFB38, "HEBREW LETTER TET WITH DAGESH" },
    { 0xFB39, "HEBREW LETTER YOD WITH DAGESH" },
    { 0xFB3A, "HEBREW LETTER FINAL KAF WITH DAGESH" },
    { 0xFB3B, "HEBREW LETTER KAF WITH DAGESH" },
    { 0xFB3C, "HEBREW LETTER LAMED WITH DAGESH" },
    { 0xFB3E, "HEBREW LETTER MEM WITH DAGESH" },
    { 0xFB40, "HEBREW LETTER NUN WITH DAGESH" },
    { 0xFB41, "HEBREW LETTER SAMEKH WITH DAGESH" },
    { 0xFB43, "HEBREW LETTER FINAL PE WITH DAGESH" },
    { 0xFB44, "HEBREW LETTER PE WITH DAGESH" },
    { 0xFB46, "HEBREW LETTER TSADI WITH DAGESH" },
    { 0xFB47, "HEBREW LETTER QOF WITH DAGESH" },
    { 0xFB48, "HEBREW LETTER RESH WITH DAGESH" },
    { 0xFB49, "HEBREW LETTER SHIN WITH DAGESH" },
    { 0xFB4A, "HEBREW LETTER TAV WITH DAGESH" },
    { 0xFB4B, "HEBREW LETTER VAV WITH HOLAM" },
    { 0xFB4C, "HEBREW LETTER BET WITH RAFE" },
    { 0xFB4D, "HEBREW LETTER KAF WITH RAFE" },
    { 0xFB4E, "HEBREW LETTER PE WITH RAFE" },
    { 0xFB4F, "HEBREW LIGATURE ALEF LAMED" },
    { 0xFB50, "ARABIC LETTER ALEF WASLA ISOLATED FORM" },
    { 0xFB51, "ARABIC LETTER ALEF WASLA FINAL FORM" },
    { 0xFB52, "ARABIC LETTER BEEH ISOLATED FORM" },
    { 0xFB53, "ARABIC LETTER BEEH FINAL FORM" },
    { 0xFB54, "ARABIC LETTER BEEH INITIAL FORM" },
    { 0xFB55, "ARABIC LETTER BEEH MEDIAL FORM" },
    { 0xFB56, "ARABIC LETTER PEH ISOLATED FORM" },
    { 0xFB57, "ARABIC LETTER PEH FINAL FORM" },
    { 0xFB58, "ARABIC LETTER PEH INITIAL FORM" },
    { 0xFB59, "ARABIC LETTER PEH MEDIAL FORM" },
    { 0xFB5A, "ARABIC LETTER BEHEH ISOLATED FORM" },
    { 0xFB5B, "ARABIC LETTER BEHEH FINAL FORM" },
    { 0xFB5C, "ARABIC LETTER BEHEH INITIAL FORM" },
    { 0xFB5D, "ARABIC LETTER BEHEH MEDIAL FORM" },
    { 0xFB5E, "ARABIC LETTER TTEHEH ISOLATED FORM" },
    { 0xFB5F, "ARABIC LETTER TTEHEH FINAL FORM" },
    { 0xFB60, "ARABIC LETTER TTEHEH INITIAL FORM" },
    { 0xFB61, "ARABIC LETTER TTEHEH MEDIAL FORM" },
    { 0xFB62, "ARABIC LETTER TEHEH ISOLATED FORM" },
    { 0xFB63, "ARABIC LETTER TEHEH FINAL FORM" },
    { 0xFB64, "ARABIC LETTER TEHEH INITIAL FORM" },
    { 0xFB65, "ARABIC LETTER TEHEH MEDIAL FORM" },
    { 0xFB66, "ARABIC LETTER TTEH ISOLATED FORM" },
    { 0xFB67, "ARABIC LETTER TTEH FINAL FORM" },
    { 0xFB68, "ARABIC LETTER TTEH INITIAL FORM" },
    { 0xFB69, "ARABIC LETTER TTEH MEDIAL FORM" },
    { 0xFB6A, "ARABIC LETTER VEH ISOLATED FORM" },
    { 0xFB6B, "ARABIC LETTER VEH FINAL FORM" },
    { 0xFB6C, "ARABIC LETTER VEH INITIAL FORM" },
    { 0xFB6D, "ARABIC LETTER VEH MEDIAL FORM" },
    { 0xFB6E, "ARABIC LETTER PEHEH ISOLATED FORM" },
    { 0xFB6F, "ARABIC LETTER PEHEH FINAL FORM" },
    { 0xFB70, "ARABIC LETTER PEHEH INITIAL FORM" },
    { 0xFB71, "ARABIC LETTER PEHEH MEDIAL FORM" },
    { 0xFB72, "ARABIC LETTER DYEH ISOLATED FORM" },
    { 0xFB73, "ARABIC LETTER DYEH FINAL FORM" },
    { 0xFB74, "ARABIC LETTER DYEH INITIAL FORM" },
    { 0xFB75, "ARABIC LETTER DYEH MEDIAL FORM" },
    { 0xFB76, "ARABIC LETTER NYEH ISOLATED FORM" },
    { 0xFB77, "ARABIC LETTER NYEH FINAL FORM" },
    { 0xFB78, "ARABIC LETTER NYEH INITIAL FORM" },
    { 0xFB79, "ARABIC LETTER NYEH MEDIAL FORM" },
    { 0xFB7A, "ARABIC LETTER TCHEH ISOLATED FORM" },
    { 0xFB7B, "ARABIC LETTER TCHEH FINAL FORM" },
    { 0xFB7C, "ARABIC LETTER TCHEH INITIAL FORM" },
    { 0xFB7D, "ARABIC LETTER TCHEH MEDIAL FORM" },
    { 0xFB7E, "ARABIC LETTER TCHEHEH ISOLATED FORM" },
    { 0xFB7F, "ARABIC LETTER TCHEHEH FINAL FORM" },
    { 0xFB80, "ARABIC LETTER TCHEHEH INITIAL FORM" },
    { 0xFB81, "ARABIC LETTER TCHEHEH MEDIAL FORM" },
    { 0xFB82, "ARABIC LETTER DDAHAL ISOLATED FORM" },
    { 0xFB83, "ARABIC LETTER DDAHAL FINAL FORM" },
    { 0xFB84, "ARABIC LETTER DAHAL ISOLATED FORM" },
    { 0xFB85, "ARABIC LETTER DAHAL FINAL FORM" },
    { 0xFB86, "ARABIC LETTER DUL ISOLATED FORM" },
    { 0xFB87, "ARABIC LETTER DUL FINAL FORM" },
    { 0xFB88, "ARABIC LETTER DDAL ISOLATED FORM" },
    { 0xFB89, "ARABIC LETTER DDAL FINAL FORM" },
    { 0xFB8A, "ARABIC LETTER JEH ISOLATED FORM" },
    { 0xFB8B, "ARABIC LETTER JEH FINAL FORM" },
    { 0xFB8C, "ARABIC LETTER RREH ISOLATED FORM" },
    { 0xFB8D, "ARABIC LETTER RREH FINAL FORM" },
    { 0xFB8E, "ARABIC LETTER KEHEH ISOLATED FORM" },
    { 0xFB8F, "ARABIC LETTER KEHEH FINAL FORM" },
    { 0xFB90, "ARABIC LETTER KEHEH INITIAL FORM" },
    { 0xFB91, "ARABIC LETTER KEHEH MEDIAL FORM" },
    { 0xFB92, "ARABIC LETTER GAF ISOLATED FORM" },
    { 0xFB93, "ARABIC LETTER GAF FINAL FORM" },
    { 0xFB94, "ARABIC LETTER GAF INITIAL FORM" },
    { 0xFB95, "ARABIC LETTER GAF MEDIAL FORM" },
    { 0xFB96, "ARABIC LETTER GUEH ISOLATED FORM" },
    { 0xFB97, "ARABIC LETTER GUEH FINAL FORM" },
    { 0xFB98, "ARABIC LETTER GUEH INITIAL FORM" },
    { 0xFB99, "ARABIC LETTER GUEH MEDIAL FORM" },
    { 0xFB9A, "ARABIC LETTER NGOEH ISOLATED FORM" },
    { 0xFB9B, "ARABIC LETTER NGOEH FINAL FORM" },
    { 0xFB9C, "ARABIC LETTER NGOEH INITIAL FORM" },
    { 0xFB9D, "ARABIC LETTER NGOEH MEDIAL FORM" },
    { 0xFB9E, "ARABIC LETTER NOON GHUNNA ISOLATED FORM" },
    { 0xFB9F, "ARABIC LETTER NOON GHUNNA FINAL FORM" },
    { 0xFBA0, "ARABIC LETTER RNOON ISOLATED FORM" },
    { 0xFBA1, "ARABIC LETTER RNOON FINAL FORM" },
    { 0xFBA2, "ARABIC LETTER RNOON INITIAL FORM" },
    { 0xFBA3, "ARABIC LETTER RNOON MEDIAL FORM" },
    { 0xFBA4, "ARABIC LETTER HEH WITH YEH ABOVE ISOLATED FORM" },
    { 0xFBA5, "ARABIC LETTER HEH WITH YEH ABOVE FINAL FORM" },
    { 0xFBA6, "ARABIC LETTER HEH GOAL ISOLATED FORM" },
    { 0xFBA7, "ARABIC LETTER HEH GOAL FINAL FORM" },
    { 0xFBA8, "ARABIC LETTER HEH GOAL INITIAL FORM" },
    { 0xFBA9, "ARABIC LETTER HEH GOAL MEDIAL FORM" },
    { 0xFBAA, "ARABIC LETTER HEH DOACHASHMEE ISOLATED FORM" },
    { 0xFBAB, "ARABIC LETTER HEH DOACHASHMEE FINAL FORM" },
    { 0xFBAC, "ARABIC LETTER HEH DOACHASHMEE INITIAL FORM" },
    { 0xFBAD, "ARABIC LETTER HEH DOACHASHMEE MEDIAL FORM" },
    { 0xFBAE, "ARABIC LETTER YEH BARREE ISOLATED FORM" },
    { 0xFBAF, "ARABIC LETTER YEH BARREE FINAL FORM" },
    { 0xFBB0, "ARABIC LETTER YEH BARREE WITH HAMZA ABOVE ISOLATED FORM" },
    { 0xFBB1, "ARABIC LETTER YEH BARREE WITH HAMZA ABOVE FINAL FORM" },
    { 0xFBB2, "ARABIC SYMBOL DOT ABOVE" },
    { 0xFBB3, "ARABIC SYMBOL DOT BELOW" },
    { 0xFBB4, "ARABIC SYMBOL TWO DOTS ABOVE" },
    { 0xFBB5, "ARABIC SYMBOL TWO DOTS BELOW" },
    { 0xFBB6, "ARABIC SYMBOL THREE DOTS ABOVE" },
    { 0xFBB7, "ARABIC SYMBOL THREE DOTS BELOW" },
    { 0xFBB8, "ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS ABOVE" },
    { 0xFBB9, "ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS BELOW" },
    { 0xFBBA, "ARABIC SYMBOL FOUR DOTS ABOVE" },
    { 0xFBBB, "ARABIC SYMBOL FOUR DOTS BELOW" },
    { 0xFBBC, "ARABIC SYMBOL DOUBLE VERTICAL BAR BELOW" },
    { 0xFBBD, "ARABIC SYMBOL TWO DOTS VERTICALLY ABOVE" },
    { 0xFBBE, "ARABIC SYMBOL TWO DOTS VERTICALLY BELOW" },
    { 0xFBBF, "ARABIC SYMBOL RING" },
    { 0xFBC0, "ARABIC SYMBOL SMALL TAH ABOVE" },
    { 0xFBC1, "ARABIC SYMBOL SMALL TAH BELOW" },
    { 0xFBC2, "ARABIC SYMBOL WASLA ABOVE" },
    { 0xFBD3, "ARABIC LETTER NG ISOLATED FORM" },
    { 0xFBD4, "ARABIC LETTER NG FINAL FORM" },
    { 0xFBD5, "ARABIC LETTER NG INITIAL FORM" },
    { 0xFBD6, "ARABIC LETTER NG MEDIAL FORM" },
    { 0xFBD7, "ARABIC LETTER U ISOLATED FORM" },
    { 0xFBD8, "ARABIC LETTER U FINAL FORM" },
    { 0xFBD9, "ARABIC LETTER OE ISOLATED FORM" },
    { 0xFBDA, "ARABIC LETTER OE FINAL FORM" },
    { 0xFBDB, "ARABIC LETTER YU ISOLATED FORM" },
    { 0xFBDC, "ARABIC LETTER YU FINAL FORM" },
    { 0xFBDD, "ARABIC LETTER U WITH HAMZA ABOVE ISOLATED FORM" },
    { 0xFBDE, "ARABIC LETTER VE ISOLATED FORM" },
    { 0xFBDF, "ARABIC LETTER VE FINAL FORM" },
    { 0xFBE0, "ARABIC LETTER KIRGHIZ OE ISOLATED FORM" },
    { 0xFBE1, "ARABIC LETTER KIRGHIZ OE FINAL FORM" },
    { 0xFBE2, "ARABIC LETTER KIRGHIZ YU ISOLATED FORM" },
    { 0xFBE3, "ARABIC LETTER KIRGHIZ YU FINAL FORM" },
    { 0xFBE4, "ARABIC LETTER E ISOLATED FORM" },
    { 0xFBE5, "ARABIC LETTER E FINAL FORM" },
    { 0xFBE6, "ARABIC LETTER E INITIAL FORM" },
    { 0xFBE7, "ARABIC LETTER E MEDIAL FORM" },
    { 0xFBE8, "ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA INITIAL FORM" },
    { 0xFBE9, "ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA MEDIAL FORM" },
    { 0xFBEA, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF ISOLATED FORM" },
    { 0xFBEB, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF FINAL FORM" },
    { 0xFBEC, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE ISOLATED FORM" },
    { 0xFBED, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE FINAL FORM" },
    { 0xFBEE, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW ISOLATED FORM" },
    { 0xFBEF, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW FINAL FORM" },
    { 0xFBF0, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U ISOLATED FORM" },
    { 0xFBF1, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U FINAL FORM" },
    { 0xFBF2, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE ISOLATED FORM" },
    { 0xFBF3, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE FINAL FORM" },
    { 0xFBF4, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU ISOLATED FORM" },
    { 0xFBF5, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU FINAL FORM" },
    { 0xFBF6, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E ISOLATED FORM" },
    { 0xFBF7, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E FINAL FORM" },
    { 0xFBF8, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E INITIAL FORM" },
    { 0xFBF9, "ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFBFA, "ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM" },
    { 0xFBFB, "ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA INITIAL FORM" },
    { 0xFBFC, "ARABIC LETTER FARSI YEH ISOLATED FORM" },
    { 0xFBFD, "ARABIC LETTER FARSI YEH FINAL FORM" },
    { 0xFBFE, "ARABIC LETTER FARSI YEH INITIAL FORM" },
    { 0xFBFF, "ARABIC LETTER FARSI YEH MEDIAL FORM" },
    { 0xFC00, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM ISOLATED FORM" },
    { 0xFC01, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH ISOLATED FORM" },
    { 0xFC02, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM ISOLATED FORM" },
    { 0xFC03, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC04, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH ISOLATED FORM" },
    { 0xFC05, "ARABIC LIGATURE BEH WITH JEEM ISOLATED FORM" },
    { 0xFC06, "ARABIC LIGATURE BEH WITH HAH ISOLATED FORM" },
    { 0xFC07, "ARABIC LIGATURE BEH WITH KHAH ISOLATED FORM" },
    { 0xFC08, "ARABIC LIGATURE BEH WITH MEEM ISOLATED FORM" },
    { 0xFC09, "ARABIC LIGATURE BEH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC0A, "ARABIC LIGATURE BEH WITH YEH ISOLATED FORM" },
    { 0xFC0B, "ARABIC LIGATURE TEH WITH JEEM ISOLATED FORM" },
    { 0xFC0C, "ARABIC LIGATURE TEH WITH HAH ISOLATED FORM" },
    { 0xFC0D, "ARABIC LIGATURE TEH WITH KHAH ISOLATED FORM" },
    { 0xFC0E, "ARABIC LIGATURE TEH WITH MEEM ISOLATED FORM" },
    { 0xFC0F, "ARABIC LIGATURE TEH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC10, "ARABIC LIGATURE TEH WITH YEH ISOLATED FORM" },
    { 0xFC11, "ARABIC LIGATURE THEH WITH JEEM ISOLATED FORM" },
    { 0xFC12, "ARABIC LIGATURE THEH WITH MEEM ISOLATED FORM" },
    { 0xFC13, "ARABIC LIGATURE THEH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC14, "ARABIC LIGATURE THEH WITH YEH ISOLATED FORM" },
    { 0xFC15, "ARABIC LIGATURE JEEM WITH HAH ISOLATED FORM" },
    { 0xFC16, "ARABIC LIGATURE JEEM WITH MEEM ISOLATED FORM" },
    { 0xFC17, "ARABIC LIGATURE HAH WITH JEEM ISOLATED FORM" },
    { 0xFC18, "ARABIC LIGATURE HAH WITH MEEM ISOLATED FORM" },
    { 0xFC19, "ARABIC LIGATURE KHAH WITH JEEM ISOLATED FORM" },
    { 0xFC1A, "ARABIC LIGATURE KHAH WITH HAH ISOLATED FORM" },
    { 0xFC1B, "ARABIC LIGATURE KHAH WITH MEEM ISOLATED FORM" },
    { 0xFC1C, "ARABIC LIGATURE SEEN WITH JEEM ISOLATED FORM" },
    { 0xFC1D, "ARABIC LIGATURE SEEN WITH HAH ISOLATED FORM" },
    { 0xFC1E, "ARABIC LIGATURE SEEN WITH KHAH ISOLATED FORM" },
    { 0xFC1F, "ARABIC LIGATURE SEEN WITH MEEM ISOLATED FORM" },
    { 0xFC20, "ARABIC LIGATURE SAD WITH HAH ISOLATED FORM" },
    { 0xFC21, "ARABIC LIGATURE SAD WITH MEEM ISOLATED FORM" },
    { 0xFC22, "ARABIC LIGATURE DAD WITH JEEM ISOLATED FORM" },
    { 0xFC23, "ARABIC LIGATURE DAD WITH HAH ISOLATED FORM" },
    { 0xFC24, "ARABIC LIGATURE DAD WITH KHAH ISOLATED FORM" },
    { 0xFC25, "ARABIC LIGATURE DAD WITH MEEM ISOLATED FORM" },
    { 0xFC26, "ARABIC LIGATURE TAH WITH HAH ISOLATED FORM" },
    { 0xFC27, "ARABIC LIGATURE TAH WITH MEEM ISOLATED FORM" },
    { 0xFC28, "ARABIC LIGATURE ZAH WITH MEEM ISOLATED FORM" },
    { 0xFC29, "ARABIC LIGATURE AIN WITH JEEM ISOLATED FORM" },
    { 0xFC2A, "ARABIC LIGATURE AIN WITH MEEM ISOLATED FORM" },
    { 0xFC2B, "ARABIC LIGATURE GHAIN WITH JEEM ISOLATED FORM" },
    { 0xFC2C, "ARABIC LIGATURE GHAIN WITH MEEM ISOLATED FORM" },
    { 0xFC2D, "ARABIC LIGATURE FEH WITH JEEM ISOLATED FORM" },
    { 0xFC2E, "ARABIC LIGATURE FEH WITH HAH ISOLATED FORM" },
    { 0xFC2F, "ARABIC LIGATURE FEH WITH KHAH ISOLATED FORM" },
    { 0xFC30, "ARABIC LIGATURE FEH WITH MEEM ISOLATED FORM" },
    { 0xFC31, "ARABIC LIGATURE FEH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC32, "ARABIC LIGATURE FEH WITH YEH ISOLATED FORM" },
    { 0xFC33, "ARABIC LIGATURE QAF WITH HAH ISOLATED FORM" },
    { 0xFC34, "ARABIC LIGATURE QAF WITH MEEM ISOLATED FORM" },
    { 0xFC35, "ARABIC LIGATURE QAF WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC36, "ARABIC LIGATURE QAF WITH YEH ISOLATED FORM" },
    { 0xFC37, "ARABIC LIGATURE KAF WITH ALEF ISOLATED FORM" },
    { 0xFC38, "ARABIC LIGATURE KAF WITH JEEM ISOLATED FORM" },
    { 0xFC39, "ARABIC LIGATURE KAF WITH HAH ISOLATED FORM" },
    { 0xFC3A, "ARABIC LIGATURE KAF WITH KHAH ISOLATED FORM" },
    { 0xFC3B, "ARABIC LIGATURE KAF WITH LAM ISOLATED FORM" },
    { 0xFC3C, "ARABIC LIGATURE KAF WITH MEEM ISOLATED FORM" },
    { 0xFC3D, "ARABIC LIGATURE KAF WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC3E, "ARABIC LIGATURE KAF WITH YEH ISOLATED FORM" },
    { 0xFC3F, "ARABIC LIGATURE LAM WITH JEEM ISOLATED FORM" },
    { 0xFC40, "ARABIC LIGATURE LAM WITH HAH ISOLATED FORM" },
    { 0xFC41, "ARABIC LIGATURE LAM WITH KHAH ISOLATED FORM" },
    { 0xFC42, "ARABIC LIGATURE LAM WITH MEEM ISOLATED FORM" },
    { 0xFC43, "ARABIC LIGATURE LAM WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC44, "ARABIC LIGATURE LAM WITH YEH ISOLATED FORM" },
    { 0xFC45, "ARABIC LIGATURE MEEM WITH JEEM ISOLATED FORM" },
    { 0xFC46, "ARABIC LIGATURE MEEM WITH HAH ISOLATED FORM" },
    { 0xFC47, "ARABIC LIGATURE MEEM WITH KHAH ISOLATED FORM" },
    { 0xFC48, "ARABIC LIGATURE MEEM WITH MEEM ISOLATED FORM" },
    { 0xFC49, "ARABIC LIGATURE MEEM WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC4A, "ARABIC LIGATURE MEEM WITH YEH ISOLATED FORM" },
    { 0xFC4B, "ARABIC LIGATURE NOON WITH JEEM ISOLATED FORM" },
    { 0xFC4C, "ARABIC LIGATURE NOON WITH HAH ISOLATED FORM" },
    { 0xFC4D, "ARABIC LIGATURE NOON WITH KHAH ISOLATED FORM" },
    { 0xFC4E, "ARABIC LIGATURE NOON WITH MEEM ISOLATED FORM" },
    { 0xFC4F, "ARABIC LIGATURE NOON WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC50, "ARABIC LIGATURE NOON WITH YEH ISOLATED FORM" },
    { 0xFC51, "ARABIC LIGATURE HEH WITH JEEM ISOLATED FORM" },
    { 0xFC52, "ARABIC LIGATURE HEH WITH MEEM ISOLATED FORM" },
    { 0xFC53, "ARABIC LIGATURE HEH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC54, "ARABIC LIGATURE HEH WITH YEH ISOLATED FORM" },
    { 0xFC55, "ARABIC LIGATURE YEH WITH JEEM ISOLATED FORM" },
    { 0xFC56, "ARABIC LIGATURE YEH WITH HAH ISOLATED FORM" },
    { 0xFC57, "ARABIC LIGATURE YEH WITH KHAH ISOLATED FORM" },
    { 0xFC58, "ARABIC LIGATURE YEH WITH MEEM ISOLATED FORM" },
    { 0xFC59, "ARABIC LIGATURE YEH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFC5A, "ARABIC LIGATURE YEH WITH YEH ISOLATED FORM" },
    { 0xFC5B, "ARABIC LIGATURE THAL WITH SUPERSCRIPT ALEF ISOLATED FORM" },
    { 0xFC5C, "ARABIC LIGATURE REH WITH SUPERSCRIPT ALEF ISOLATED FORM" },
    { 0xFC5D, "ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF ISOLATED FORM" },
    { 0xFC5E, "ARABIC LIGATURE SHADDA WITH DAMMATAN ISOLATED FORM" },
    { 0xFC5F, "ARABIC LIGATURE SHADDA WITH KASRATAN ISOLATED FORM" },
    { 0xFC60, "ARABIC LIGATURE SHADDA WITH FATHA ISOLATED FORM" },
    { 0xFC61, "ARABIC LIGATURE SHADDA WITH DAMMA ISOLATED FORM" },
    { 0xFC62, "ARABIC LIGATURE SHADDA WITH KASRA ISOLATED FORM" },
    { 0xFC63, "ARABIC LIGATURE SHADDA WITH SUPERSCRIPT ALEF ISOLATED FORM" },
    { 0xFC64, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH REH FINAL FORM" },
    { 0xFC65, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ZAIN FINAL FORM" },
    { 0xFC66, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM FINAL FORM" },
    { 0xFC67, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH NOON FINAL FORM" },
    { 0xFC68, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC69, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH FINAL FORM" },
    { 0xFC6A, "ARABIC LIGATURE BEH WITH REH FINAL FORM" },
    { 0xFC6B, "ARABIC LIGATURE BEH WITH ZAIN FINAL FORM" },
    { 0xFC6C, "ARABIC LIGATURE BEH WITH MEEM FINAL FORM" },
    { 0xFC6D, "ARABIC LIGATURE BEH WITH NOON FINAL FORM" },
    { 0xFC6E, "ARABIC LIGATURE BEH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC6F, "ARABIC LIGATURE BEH WITH YEH FINAL FORM" },
    { 0xFC70, "ARABIC LIGATURE TEH WITH REH FINAL FORM" },
    { 0xFC71, "ARABIC LIGATURE TEH WITH ZAIN FINAL FORM" },
    { 0xFC72, "ARABIC LIGATURE TEH WITH MEEM FINAL FORM" },
    { 0xFC73, "ARABIC LIGATURE TEH WITH NOON FINAL FORM" },
    { 0xFC74, "ARABIC LIGATURE TEH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC75, "ARABIC LIGATURE TEH WITH YEH FINAL FORM" },
    { 0xFC76, "ARABIC LIGATURE THEH WITH REH FINAL FORM" },
    { 0xFC77, "ARABIC LIGATURE THEH WITH ZAIN FINAL FORM" },
    { 0xFC78, "ARABIC LIGATURE THEH WITH MEEM FINAL FORM" },
    { 0xFC79, "ARABIC LIGATURE THEH WITH NOON FINAL FORM" },
    { 0xFC7A, "ARABIC LIGATURE THEH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC7B, "ARABIC LIGATURE THEH WITH YEH FINAL FORM" },
    { 0xFC7C, "ARABIC LIGATURE FEH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC7D, "ARABIC LIGATURE FEH WITH YEH FINAL FORM" },
    { 0xFC7E, "ARABIC LIGATURE QAF WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC7F, "ARABIC LIGATURE QAF WITH YEH FINAL FORM" },
    { 0xFC80, "ARABIC LIGATURE KAF WITH ALEF FINAL FORM" },
    { 0xFC81, "ARABIC LIGATURE KAF WITH LAM FINAL FORM" },
    { 0xFC82, "ARABIC LIGATURE KAF WITH MEEM FINAL FORM" },
    { 0xFC83, "ARABIC LIGATURE KAF WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC84, "ARABIC LIGATURE KAF WITH YEH FINAL FORM" },
    { 0xFC85, "ARABIC LIGATURE LAM WITH MEEM FINAL FORM" },
    { 0xFC86, "ARABIC LIGATURE LAM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC87, "ARABIC LIGATURE LAM WITH YEH FINAL FORM" },
    { 0xFC88, "ARABIC LIGATURE MEEM WITH ALEF FINAL FORM" },
    { 0xFC89, "ARABIC LIGATURE MEEM WITH MEEM FINAL FORM" },
    { 0xFC8A, "ARABIC LIGATURE NOON WITH REH FINAL FORM" },
    { 0xFC8B, "ARABIC LIGATURE NOON WITH ZAIN FINAL FORM" },
    { 0xFC8C, "ARABIC LIGATURE NOON WITH MEEM FINAL FORM" },
    { 0xFC8D, "ARABIC LIGATURE NOON WITH NOON FINAL FORM" },
    { 0xFC8E, "ARABIC LIGATURE NOON WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC8F, "ARABIC LIGATURE NOON WITH YEH FINAL FORM" },
    { 0xFC90, "ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF FINAL FORM" },
    { 0xFC91, "ARABIC LIGATURE YEH WITH REH FINAL FORM" },
    { 0xFC92, "ARABIC LIGATURE YEH WITH ZAIN FINAL FORM" },
    { 0xFC93, "ARABIC LIGATURE YEH WITH MEEM FINAL FORM" },
    { 0xFC94, "ARABIC LIGATURE YEH WITH NOON FINAL FORM" },
    { 0xFC95, "ARABIC LIGATURE YEH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFC96, "ARABIC LIGATURE YEH WITH YEH FINAL FORM" },
    { 0xFC97, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM INITIAL FORM" },
    { 0xFC98, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH INITIAL FORM" },
    { 0xFC99, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH KHAH INITIAL FORM" },
    { 0xFC9A, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM INITIAL FORM" },
    { 0xFC9B, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH INITIAL FORM" },
    { 0xFC9C, "ARABIC LIGATURE BEH WITH JEEM INITIAL FORM" },
    { 0xFC9D, "ARABIC LIGATURE BEH WITH HAH INITIAL FORM" },
    { 0xFC9E, "ARABIC LIGATURE BEH WITH KHAH INITIAL FORM" },
    { 0xFC9F, "ARABIC LIGATURE BEH WITH MEEM INITIAL FORM" },
    { 0xFCA0, "ARABIC LIGATURE BEH WITH HEH INITIAL FORM" },
    { 0xFCA1, "ARABIC LIGATURE TEH WITH JEEM INITIAL FORM" },
    { 0xFCA2, "ARABIC LIGATURE TEH WITH HAH INITIAL FORM" },
    { 0xFCA3, "ARABIC LIGATURE TEH WITH KHAH INITIAL FORM" },
    { 0xFCA4, "ARABIC LIGATURE TEH WITH MEEM INITIAL FORM" },
    { 0xFCA5, "ARABIC LIGATURE TEH WITH HEH INITIAL FORM" },
    { 0xFCA6, "ARABIC LIGATURE THEH WITH MEEM INITIAL FORM" },
    { 0xFCA7, "ARABIC LIGATURE JEEM WITH HAH INITIAL FORM" },
    { 0xFCA8, "ARABIC LIGATURE JEEM WITH MEEM INITIAL FORM" },
    { 0xFCA9, "ARABIC LIGATURE HAH WITH JEEM INITIAL FORM" },
    { 0xFCAA, "ARABIC LIGATURE HAH WITH MEEM INITIAL FORM" },
    { 0xFCAB, "ARABIC LIGATURE KHAH WITH JEEM INITIAL FORM" },
    { 0xFCAC, "ARABIC LIGATURE KHAH WITH MEEM INITIAL FORM" },
    { 0xFCAD, "ARABIC LIGATURE SEEN WITH JEEM INITIAL FORM" },
    { 0xFCAE, "ARABIC LIGATURE SEEN WITH HAH INITIAL FORM" },
    { 0xFCAF, "ARABIC LIGATURE SEEN WITH KHAH INITIAL FORM" },
    { 0xFCB0, "ARABIC LIGATURE SEEN WITH MEEM INITIAL FORM" },
    { 0xFCB1, "ARABIC LIGATURE SAD WITH HAH INITIAL FORM" },
    { 0xFCB2, "ARABIC LIGATURE SAD WITH KHAH INITIAL FORM" },
    { 0xFCB3, "ARABIC LIGATURE SAD WITH MEEM INITIAL FORM" },
    { 0xFCB4, "ARABIC LIGATURE DAD WITH JEEM INITIAL FORM" },
    { 0xFCB5, "ARABIC LIGATURE DAD WITH HAH INITIAL FORM" },
    { 0xFCB6, "ARABIC LIGATURE DAD WITH KHAH INITIAL FORM" },
    { 0xFCB7, "ARABIC LIGATURE DAD WITH MEEM INITIAL FORM" },
    { 0xFCB8, "ARABIC LIGATURE TAH WITH HAH INITIAL FORM" },
    { 0xFCB9, "ARABIC LIGATURE ZAH WITH MEEM INITIAL FORM" },
    { 0xFCBA, "ARABIC LIGATURE AIN WITH JEEM INITIAL FORM" },
    { 0xFCBB, "ARABIC LIGATURE AIN WITH MEEM INITIAL FORM" },
    { 0xFCBC, "ARABIC LIGATURE GHAIN WITH JEEM INITIAL FORM" },
    { 0xFCBD, "ARABIC LIGATURE GHAIN WITH MEEM INITIAL FORM" },
    { 0xFCBE, "ARABIC LIGATURE FEH WITH JEEM INITIAL FORM" },
    { 0xFCBF, "ARABIC LIGATURE FEH WITH HAH INITIAL FORM" },
    { 0xFCC0, "ARABIC LIGATURE FEH WITH KHAH INITIAL FORM" },
    { 0xFCC1, "ARABIC LIGATURE FEH WITH MEEM INITIAL FORM" },
    { 0xFCC2, "ARABIC LIGATURE QAF WITH HAH INITIAL FORM" },
    { 0xFCC3, "ARABIC LIGATURE QAF WITH MEEM INITIAL FORM" },
    { 0xFCC4, "ARABIC LIGATURE KAF WITH JEEM INITIAL FORM" },
    { 0xFCC5, "ARABIC LIGATURE KAF WITH HAH INITIAL FORM" },
    { 0xFCC6, "ARABIC LIGATURE KAF WITH KHAH INITIAL FORM" },
    { 0xFCC7, "ARABIC LIGATURE KAF WITH LAM INITIAL FORM" },
    { 0xFCC8, "ARABIC LIGATURE KAF WITH MEEM INITIAL FORM" },
    { 0xFCC9, "ARABIC LIGATURE LAM WITH JEEM INITIAL FORM" },
    { 0xFCCA, "ARABIC LIGATURE LAM WITH HAH INITIAL FORM" },
    { 0xFCCB, "ARABIC LIGATURE LAM WITH KHAH INITIAL FORM" },
    { 0xFCCC, "ARABIC LIGATURE LAM WITH MEEM INITIAL FORM" },
    { 0xFCCD, "ARABIC LIGATURE LAM WITH HEH INITIAL FORM" },
    { 0xFCCE, "ARABIC LIGATURE MEEM WITH JEEM INITIAL FORM" },
    { 0xFCCF, "ARABIC LIGATURE MEEM WITH HAH INITIAL FORM" },
    { 0xFCD0, "ARABIC LIGATURE MEEM WITH KHAH INITIAL FORM" },
    { 0xFCD1, "ARABIC LIGATURE MEEM WITH MEEM INITIAL FORM" },
    { 0xFCD2, "ARABIC LIGATURE NOON WITH JEEM INITIAL FORM" },
    { 0xFCD3, "ARABIC LIGATURE NOON WITH HAH INITIAL FORM" },
    { 0xFCD4, "ARABIC LIGATURE NOON WITH KHAH INITIAL FORM" },
    { 0xFCD5, "ARABIC LIGATURE NOON WITH MEEM INITIAL FORM" },
    { 0xFCD6, "ARABIC LIGATURE NOON WITH HEH INITIAL FORM" },
    { 0xFCD7, "ARABIC LIGATURE HEH WITH JEEM INITIAL FORM" },
    { 0xFCD8, "ARABIC LIGATURE HEH WITH MEEM INITIAL FORM" },
    { 0xFCD9, "ARABIC LIGATURE HEH WITH SUPERSCRIPT ALEF INITIAL FORM" },
    { 0xFCDA, "ARABIC LIGATURE YEH WITH JEEM INITIAL FORM" },
    { 0xFCDB, "ARABIC LIGATURE YEH WITH HAH INITIAL FORM" },
    { 0xFCDC, "ARABIC LIGATURE YEH WITH KHAH INITIAL FORM" },
    { 0xFCDD, "ARABIC LIGATURE YEH WITH MEEM INITIAL FORM" },
    { 0xFCDE, "ARABIC LIGATURE YEH WITH HEH INITIAL FORM" },
    { 0xFCDF, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM MEDIAL FORM" },
    { 0xFCE0, "ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH MEDIAL FORM" },
    { 0xFCE1, "ARABIC LIGATURE BEH WITH MEEM MEDIAL FORM" },
    { 0xFCE2, "ARABIC LIGATURE BEH WITH HEH MEDIAL FORM" },
    { 0xFCE3, "ARABIC LIGATURE TEH WITH MEEM MEDIAL FORM" },
    { 0xFCE4, "ARABIC LIGATURE TEH WITH HEH MEDIAL FORM" },
    { 0xFCE5, "ARABIC LIGATURE THEH WITH MEEM MEDIAL FORM" },
    { 0xFCE6, "ARABIC LIGATURE THEH WITH HEH MEDIAL FORM" },
    { 0xFCE7, "ARABIC LIGATURE SEEN WITH MEEM MEDIAL FORM" },
    { 0xFCE8, "ARABIC LIGATURE SEEN WITH HEH MEDIAL FORM" },
    { 0xFCE9, "ARABIC LIGATURE SHEEN WITH MEEM MEDIAL FORM" },
    { 0xFCEA, "ARABIC LIGATURE SHEEN WITH HEH MEDIAL FORM" },
    { 0xFCEB, "ARABIC LIGATURE KAF WITH LAM MEDIAL FORM" },
    { 0xFCEC, "ARABIC LIGATURE KAF WITH MEEM MEDIAL FORM" },
    { 0xFCED, "ARABIC LIGATURE LAM WITH MEEM MEDIAL FORM" },
    { 0xFCEE, "ARABIC LIGATURE NOON WITH MEEM MEDIAL FORM" },
    { 0xFCEF, "ARABIC LIGATURE NOON WITH HEH MEDIAL FORM" },
    { 0xFCF0, "ARABIC LIGATURE YEH WITH MEEM MEDIAL FORM" },
    { 0xFCF1, "ARABIC LIGATURE YEH WITH HEH MEDIAL FORM" },
    { 0xFCF2, "ARABIC LIGATURE SHADDA WITH FATHA MEDIAL FORM" },
    { 0xFCF3, "ARABIC LIGATURE SHADDA WITH DAMMA MEDIAL FORM" },
    { 0xFCF4, "ARABIC LIGATURE SHADDA WITH KASRA MEDIAL FORM" },
    { 0xFCF5, "ARABIC LIGATURE TAH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFCF6, "ARABIC LIGATURE TAH WITH YEH ISOLATED FORM" },
    { 0xFCF7, "ARABIC LIGATURE AIN WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFCF8, "ARABIC LIGATURE AIN WITH YEH ISOLATED FORM" },
    { 0xFCF9, "ARABIC LIGATURE GHAIN WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFCFA, "ARABIC LIGATURE GHAIN WITH YEH ISOLATED FORM" },
    { 0xFCFB, "ARABIC LIGATURE SEEN WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFCFC, "ARABIC LIGATURE SEEN WITH YEH ISOLATED FORM" },
    { 0xFCFD, "ARABIC LIGATURE SHEEN WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFCFE, "ARABIC LIGATURE SHEEN WITH YEH ISOLATED FORM" },
    { 0xFCFF, "ARABIC LIGATURE HAH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFD00, "ARABIC LIGATURE HAH WITH YEH ISOLATED FORM" },
    { 0xFD01, "ARABIC LIGATURE JEEM WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFD02, "ARABIC LIGATURE JEEM WITH YEH ISOLATED FORM" },
    { 0xFD03, "ARABIC LIGATURE KHAH WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFD04, "ARABIC LIGATURE KHAH WITH YEH ISOLATED FORM" },
    { 0xFD05, "ARABIC LIGATURE SAD WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFD06, "ARABIC LIGATURE SAD WITH YEH ISOLATED FORM" },
    { 0xFD07, "ARABIC LIGATURE DAD WITH ALEF MAKSURA ISOLATED FORM" },
    { 0xFD08, "ARABIC LIGATURE DAD WITH YEH ISOLATED FORM" },
    { 0xFD09, "ARABIC LIGATURE SHEEN WITH JEEM ISOLATED FORM" },
    { 0xFD0A, "ARABIC LIGATURE SHEEN WITH HAH ISOLATED FORM" },
    { 0xFD0B, "ARABIC LIGATURE SHEEN WITH KHAH ISOLATED FORM" },
    { 0xFD0C, "ARABIC LIGATURE SHEEN WITH MEEM ISOLATED FORM" },
    { 0xFD0D, "ARABIC LIGATURE SHEEN WITH REH ISOLATED FORM" },
    { 0xFD0E, "ARABIC LIGATURE SEEN WITH REH ISOLATED FORM" },
    { 0xFD0F, "ARABIC LIGATURE SAD WITH REH ISOLATED FORM" },
    { 0xFD10, "ARABIC LIGATURE DAD WITH REH ISOLATED FORM" },
    { 0xFD11, "ARABIC LIGATURE TAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD12, "ARABIC LIGATURE TAH WITH YEH FINAL FORM" },
    { 0xFD13, "ARABIC LIGATURE AIN WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD14, "ARABIC LIGATURE AIN WITH YEH FINAL FORM" },
    { 0xFD15, "ARABIC LIGATURE GHAIN WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD16, "ARABIC LIGATURE GHAIN WITH YEH FINAL FORM" },
    { 0xFD17, "ARABIC LIGATURE SEEN WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD18, "ARABIC LIGATURE SEEN WITH YEH FINAL FORM" },
    { 0xFD19, "ARABIC LIGATURE SHEEN WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD1A, "ARABIC LIGATURE SHEEN WITH YEH FINAL FORM" },
    { 0xFD1B, "ARABIC LIGATURE HAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD1C, "ARABIC LIGATURE HAH WITH YEH FINAL FORM" },
    { 0xFD1D, "ARABIC LIGATURE JEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD1E, "ARABIC LIGATURE JEEM WITH YEH FINAL FORM" },
    { 0xFD1F, "ARABIC LIGATURE KHAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD20, "ARABIC LIGATURE KHAH WITH YEH FINAL FORM" },
    { 0xFD21, "ARABIC LIGATURE SAD WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD22, "ARABIC LIGATURE SAD WITH YEH FINAL FORM" },
    { 0xFD23, "ARABIC LIGATURE DAD WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD24, "ARABIC LIGATURE DAD WITH YEH FINAL FORM" },
    { 0xFD25, "ARABIC LIGATURE SHEEN WITH JEEM FINAL FORM" },
    { 0xFD26, "ARABIC LIGATURE SHEEN WITH HAH FINAL FORM" },
    { 0xFD27, "ARABIC LIGATURE SHEEN WITH KHAH FINAL FORM" },
    { 0xFD28, "ARABIC LIGATURE SHEEN WITH MEEM FINAL FORM" },
    { 0xFD29, "ARABIC LIGATURE SHEEN WITH REH FINAL FORM" },
    { 0xFD2A, "ARABIC LIGATURE SEEN WITH REH FINAL FORM" },
    { 0xFD2B, "ARABIC LIGATURE SAD WITH REH FINAL FORM" },
    { 0xFD2C, "ARABIC LIGATURE DAD WITH REH FINAL FORM" },
    { 0xFD2D, "ARABIC LIGATURE SHEEN WITH JEEM INITIAL FORM" },
    { 0xFD2E, "ARABIC LIGATURE SHEEN WITH HAH INITIAL FORM" },
    { 0xFD2F, "ARABIC LIGATURE SHEEN WITH KHAH INITIAL FORM" },
    { 0xFD30, "ARABIC LIGATURE SHEEN WITH MEEM INITIAL FORM" },
    { 0xFD31, "ARABIC LIGATURE SEEN WITH HEH INITIAL FORM" },
    { 0xFD32, "ARABIC LIGATURE SHEEN WITH HEH INITIAL FORM" },
    { 0xFD33, "ARABIC LIGATURE TAH WITH MEEM INITIAL FORM" },
    { 0xFD34, "ARABIC LIGATURE SEEN WITH JEEM MEDIAL FORM" },
    { 0xFD35, "ARABIC LIGATURE SEEN WITH HAH MEDIAL FORM" },
    { 0xFD36, "ARABIC LIGATURE SEEN WITH KHAH MEDIAL FORM" },
    { 0xFD37, "ARABIC LIGATURE SHEEN WITH JEEM MEDIAL FORM" },
    { 0xFD38, "ARABIC LIGATURE SHEEN WITH HAH MEDIAL FORM" },
    { 0xFD39, "ARABIC LIGATURE SHEEN WITH KHAH MEDIAL FORM" },
    { 0xFD3A, "ARABIC LIGATURE TAH WITH MEEM MEDIAL FORM" },
    { 0xFD3B, "ARABIC LIGATURE ZAH WITH MEEM MEDIAL FORM" },
    { 0xFD3C, "ARABIC LIGATURE ALEF WITH FATHATAN FINAL FORM" },
    { 0xFD3D, "ARABIC LIGATURE ALEF WITH FATHATAN ISOLATED FORM" },
    { 0xFD3E, "ORNATE LEFT PARENTHESIS" },
    { 0xFD3F, "ORNATE RIGHT PARENTHESIS" },
    { 0xFD40, "ARABIC LIGATURE RAHIMAHU ALLAAH" },
    { 0xFD41, "ARABIC LIGATURE RADI ALLAAHU ANH" },
    { 0xFD42, "ARABIC LIGATURE RADI ALLAAHU ANHAA" },
    { 0xFD43, "ARABIC LIGATURE RADI ALLAAHU ANHUM" },
    { 0xFD44, "ARABIC LIGATURE RADI ALLAAHU ANHUMAA" },
    { 0xFD45, "ARABIC LIGATURE RADI ALLAAHU ANHUNNA" },
    { 0xFD46, "ARABIC LIGATURE SALLALLAAHU ALAYHI WA-AALIH" },
    { 0xFD47, "ARABIC LIGATURE ALAYHI AS-SALAAM" },
    { 0xFD48, "ARABIC LIGATURE ALAYHIM AS-SALAAM" },
    { 0xFD49, "ARABIC LIGATURE ALAYHIMAA AS-SALAAM" },
    { 0xFD4A, "ARABIC LIGATURE ALAYHI AS-SALAATU WAS-SALAAM" },
    { 0xFD4B, "ARABIC LIGATURE QUDDISA SIRRAH" },
    { 0xFD4C, "ARABIC LIGATURE SALLALLAHU ALAYHI WAAALIHEE WA-SALLAM" },
    { 0xFD4D, "ARABIC LIGATURE ALAYHAA AS-SALAAM" },
    { 0xFD4E, "ARABIC LIGATURE TABAARAKA WA-TAAALAA" },
    { 0xFD4F, "ARABIC LIGATURE RAHIMAHUM ALLAAH" },
    { 0xFD50, "ARABIC LIGATURE TEH WITH JEEM WITH MEEM INITIAL FORM" },
    { 0xFD51, "ARABIC LIGATURE TEH WITH HAH WITH JEEM FINAL FORM" },
    { 0xFD52, "ARABIC LIGATURE TEH WITH HAH WITH JEEM INITIAL FORM" },
    { 0xFD53, "ARABIC LIGATURE TEH WITH HAH WITH MEEM INITIAL FORM" },
    { 0xFD54, "ARABIC LIGATURE TEH WITH KHAH WITH MEEM INITIAL FORM" },
    { 0xFD55, "ARABIC LIGATURE TEH WITH MEEM WITH JEEM INITIAL FORM" },
    { 0xFD56, "ARABIC LIGATURE TEH WITH MEEM WITH HAH INITIAL FORM" },
    { 0xFD57, "ARABIC LIGATURE TEH WITH MEEM WITH KHAH INITIAL FORM" },
    { 0xFD58, "ARABIC LIGATURE JEEM WITH MEEM WITH HAH FINAL FORM" },
    { 0xFD59, "ARABIC LIGATURE JEEM WITH MEEM WITH HAH INITIAL FORM" },
    { 0xFD5A, "ARABIC LIGATURE HAH WITH MEEM WITH YEH FINAL FORM" },
    { 0xFD5B, "ARABIC LIGATURE HAH WITH MEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD5C, "ARABIC LIGATURE SEEN WITH HAH WITH JEEM INITIAL FORM" },
    { 0xFD5D, "ARABIC LIGATURE SEEN WITH JEEM WITH HAH INITIAL FORM" },
    { 0xFD5E, "ARABIC LIGATURE SEEN WITH JEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD5F, "ARABIC LIGATURE SEEN WITH MEEM WITH HAH FINAL FORM" },
    { 0xFD60, "ARABIC LIGATURE SEEN WITH MEEM WITH HAH INITIAL FORM" },
    { 0xFD61, "ARABIC LIGATURE SEEN WITH MEEM WITH JEEM INITIAL FORM" },
    { 0xFD62, "ARABIC LIGATURE SEEN WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD63, "ARABIC LIGATURE SEEN WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFD64, "ARABIC LIGATURE SAD WITH HAH WITH HAH FINAL FORM" },
    { 0xFD65, "ARABIC LIGATURE SAD WITH HAH WITH HAH INITIAL FORM" },
    { 0xFD66, "ARABIC LIGATURE SAD WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD67, "ARABIC LIGATURE SHEEN WITH HAH WITH MEEM FINAL FORM" },
    { 0xFD68, "ARABIC LIGATURE SHEEN WITH HAH WITH MEEM INITIAL FORM" },
    { 0xFD69, "ARABIC LIGATURE SHEEN WITH JEEM WITH YEH FINAL FORM" },
    { 0xFD6A, "ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH FINAL FORM" },
    { 0xFD6B, "ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH INITIAL FORM" },
    { 0xFD6C, "ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD6D, "ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFD6E, "ARABIC LIGATURE DAD WITH HAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD6F, "ARABIC LIGATURE DAD WITH KHAH WITH MEEM FINAL FORM" },
    { 0xFD70, "ARABIC LIGATURE DAD WITH KHAH WITH MEEM INITIAL FORM" },
    { 0xFD71, "ARABIC LIGATURE TAH WITH MEEM WITH HAH FINAL FORM" },
    { 0xFD72, "ARABIC LIGATURE TAH WITH MEEM WITH HAH INITIAL FORM" },
    { 0xFD73, "ARABIC LIGATURE TAH WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFD74, "ARABIC LIGATURE TAH WITH MEEM WITH YEH FINAL FORM" },
    { 0xFD75, "ARABIC LIGATURE AIN WITH JEEM WITH MEEM FINAL FORM" },
    { 0xFD76, "ARABIC LIGATURE AIN WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD77, "ARABIC LIGATURE AIN WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFD78, "ARABIC LIGATURE AIN WITH MEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD79, "ARABIC LIGATURE GHAIN WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD7A, "ARABIC LIGATURE GHAIN WITH MEEM WITH YEH FINAL FORM" },
    { 0xFD7B, "ARABIC LIGATURE GHAIN WITH MEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD7C, "ARABIC LIGATURE FEH WITH KHAH WITH MEEM FINAL FORM" },
    { 0xFD7D, "ARABIC LIGATURE FEH WITH KHAH WITH MEEM INITIAL FORM" },
    { 0xFD7E, "ARABIC LIGATURE QAF WITH MEEM WITH HAH FINAL FORM" },
    { 0xFD7F, "ARABIC LIGATURE QAF WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD80, "ARABIC LIGATURE LAM WITH HAH WITH MEEM FINAL FORM" },
    { 0xFD81, "ARABIC LIGATURE LAM WITH HAH WITH YEH FINAL FORM" },
    { 0xFD82, "ARABIC LIGATURE LAM WITH HAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD83, "ARABIC LIGATURE LAM WITH JEEM WITH JEEM INITIAL FORM" },
    { 0xFD84, "ARABIC LIGATURE LAM WITH JEEM WITH JEEM FINAL FORM" },
    { 0xFD85, "ARABIC LIGATURE LAM WITH KHAH WITH MEEM FINAL FORM" },
    { 0xFD86, "ARABIC LIGATURE LAM WITH KHAH WITH MEEM INITIAL FORM" },
    { 0xFD87, "ARABIC LIGATURE LAM WITH MEEM WITH HAH FINAL FORM" },
    { 0xFD88, "ARABIC LIGATURE LAM WITH MEEM WITH HAH INITIAL FORM" },
    { 0xFD89, "ARABIC LIGATURE MEEM WITH HAH WITH JEEM INITIAL FORM" },
    { 0xFD8A, "ARABIC LIGATURE MEEM WITH HAH WITH MEEM INITIAL FORM" },
    { 0xFD8B, "ARABIC LIGATURE MEEM WITH HAH WITH YEH FINAL FORM" },
    { 0xFD8C, "ARABIC LIGATURE MEEM WITH JEEM WITH HAH INITIAL FORM" },
    { 0xFD8D, "ARABIC LIGATURE MEEM WITH JEEM WITH MEEM INITIAL FORM" },
    { 0xFD8E, "ARABIC LIGATURE MEEM WITH KHAH WITH JEEM INITIAL FORM" },
    { 0xFD8F, "ARABIC LIGATURE MEEM WITH KHAH WITH MEEM INITIAL FORM" },
    { 0xFD92, "ARABIC LIGATURE MEEM WITH JEEM WITH KHAH INITIAL FORM" },
    { 0xFD93, "ARABIC LIGATURE HEH WITH MEEM WITH JEEM INITIAL FORM" },
    { 0xFD94, "ARABIC LIGATURE HEH WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFD95, "ARABIC LIGATURE NOON WITH HAH WITH MEEM INITIAL FORM" },
    { 0xFD96, "ARABIC LIGATURE NOON WITH HAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD97, "ARABIC LIGATURE NOON WITH JEEM WITH MEEM FINAL FORM" },
    { 0xFD98, "ARABIC LIGATURE NOON WITH JEEM WITH MEEM INITIAL FORM" },
    { 0xFD99, "ARABIC LIGATURE NOON WITH JEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD9A, "ARABIC LIGATURE NOON WITH MEEM WITH YEH FINAL FORM" },
    { 0xFD9B, "ARABIC LIGATURE NOON WITH MEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFD9C, "ARABIC LIGATURE YEH WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFD9D, "ARABIC LIGATURE YEH WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFD9E, "ARABIC LIGATURE BEH WITH KHAH WITH YEH FINAL FORM" },
    { 0xFD9F, "ARABIC LIGATURE TEH WITH JEEM WITH YEH FINAL FORM" },
    { 0xFDA0, "ARABIC LIGATURE TEH WITH JEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFDA1, "ARABIC LIGATURE TEH WITH KHAH WITH YEH FINAL FORM" },
    { 0xFDA2, "ARABIC LIGATURE TEH WITH KHAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFDA3, "ARABIC LIGATURE TEH WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDA4, "ARABIC LIGATURE TEH WITH MEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFDA5, "ARABIC LIGATURE JEEM WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDA6, "ARABIC LIGATURE JEEM WITH HAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFDA7, "ARABIC LIGATURE JEEM WITH MEEM WITH ALEF MAKSURA FINAL FORM" },
    { 0xFDA8, "ARABIC LIGATURE SEEN WITH KHAH WITH ALEF MAKSURA FINAL FORM" },
    { 0xFDA9, "ARABIC LIGATURE SAD WITH HAH WITH YEH FINAL FORM" },
    { 0xFDAA, "ARABIC LIGATURE SHEEN WITH HAH WITH YEH FINAL FORM" },
    { 0xFDAB, "ARABIC LIGATURE DAD WITH HAH WITH YEH FINAL FORM" },
    { 0xFDAC, "ARABIC LIGATURE LAM WITH JEEM WITH YEH FINAL FORM" },
    { 0xFDAD, "ARABIC LIGATURE LAM WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDAE, "ARABIC LIGATURE YEH WITH HAH WITH YEH FINAL FORM" },
    { 0xFDAF, "ARABIC LIGATURE YEH WITH JEEM WITH YEH FINAL FORM" },
    { 0xFDB0, "ARABIC LIGATURE YEH WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDB1, "ARABIC LIGATURE MEEM WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDB2, "ARABIC LIGATURE QAF WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDB3, "ARABIC LIGATURE NOON WITH HAH WITH YEH FINAL FORM" },
    { 0xFDB4, "ARABIC LIGATURE QAF WITH MEEM WITH HAH INITIAL FORM" },
    { 0xFDB5, "ARABIC LIGATURE LAM WITH HAH WITH MEEM INITIAL FORM" },
    { 0xFDB6, "ARABIC LIGATURE AIN WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDB7, "ARABIC LIGATURE KAF WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDB8, "ARABIC LIGATURE NOON WITH JEEM WITH HAH INITIAL FORM" },
    { 0xFDB9, "ARABIC LIGATURE MEEM WITH KHAH WITH YEH FINAL FORM" },
    { 0xFDBA, "ARABIC LIGATURE LAM WITH JEEM WITH MEEM INITIAL FORM" },
    { 0xFDBB, "ARABIC LIGATURE KAF WITH MEEM WITH MEEM FINAL FORM" },
    { 0xFDBC, "ARABIC LIGATURE LAM WITH JEEM WITH MEEM FINAL FORM" },
    { 0xFDBD, "ARABIC LIGATURE NOON WITH JEEM WITH HAH FINAL FORM" },
    { 0xFDBE, "ARABIC LIGATURE JEEM WITH HAH WITH YEH FINAL FORM" },
    { 0xFDBF, "ARABIC LIGATURE HAH WITH JEEM WITH YEH FINAL FORM" },
    { 0xFDC0, "ARABIC LIGATURE MEEM WITH JEEM WITH YEH FINAL FORM" },
    { 0xFDC1, "ARABIC LIGATURE FEH WITH MEEM WITH YEH FINAL FORM" },
    { 0xFDC2, "ARABIC LIGATURE BEH WITH HAH WITH YEH FINAL FORM" },
    { 0xFDC3, "ARABIC LIGATURE KAF WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFDC4, "ARABIC LIGATURE AIN WITH JEEM WITH MEEM INITIAL FORM" },
    { 0xFDC5, "ARABIC LIGATURE SAD WITH MEEM WITH MEEM INITIAL FORM" },
    { 0xFDC6, "ARABIC LIGATURE SEEN WITH KHAH WITH YEH FINAL FORM" },
    { 0xFDC7, "ARABIC LIGATURE NOON WITH JEEM WITH YEH FINAL FORM" },
    { 0xFDCF, "ARABIC LIGATURE SALAAMUHU ALAYNAA" },
    { 0xFDF0, "ARABIC LIGATURE SALLA USED AS KORANIC STOP SIGN ISOLATED FORM" },
    { 0xFDF1, "ARABIC LIGATURE QALA USED AS KORANIC STOP SIGN ISOLATED FORM" },
    { 0xFDF2, "ARABIC LIGATURE ALLAH ISOLATED FORM" },
    { 0xFDF3, "ARABIC LIGATURE AKBAR ISOLATED FORM" },
    { 0xFDF4, "ARABIC LIGATURE MOHAMMAD ISOLATED FORM" },
    { 0xFDF5, "ARABIC LIGATURE SALAM ISOLATED FORM" },
    { 0xFDF6, "ARABIC LIGATURE RASOUL ISOLATED FORM" },
    { 0xFDF7, "ARABIC LIGATURE ALAYHE ISOLATED FORM" },
    { 0xFDF8, "ARABIC LIGATURE WASALLAM ISOLATED FORM" },
    { 0xFDF9, "ARABIC LIGATURE SALLA ISOLATED FORM" },
    { 0xFDFA, "ARABIC LIGATURE SALLALLAHOU ALAYHE WASALLAM" },
    { 0xFDFB, "ARABIC LIGATURE JALLAJALALOUHOU" },
    { 0xFDFC, "RIAL SIGN" },
    { 0xFDFD, "ARABIC LIGATURE BISMILLAH AR-RAHMAN AR-RAHEEM" },
    { 0xFDFE, "ARABIC LIGATURE SUBHAANAHU WA TAAALAA" },
    { 0xFDFF, "ARABIC LIGATURE AZZA WA JALL" },
    { 0xFE00, "VARIATION SELECTOR-1" },
    { 0xFE01, "VARIATION SELECTOR-2" },
    { 0xFE02, "VARIATION SELECTOR-3" },
    { 0xFE03, "VARIATION SELECTOR-4" },
    { 0xFE04, "VARIATION SELECTOR-5" },
    { 0xFE05, "VARIATION SELECTOR-6" },
    { 0xFE06, "VARIATION SELECTOR-7" },
    { 0xFE07, "VARIATION SELECTOR-8" },
    { 0xFE08, "VARIATION SELECTOR-9" },
    { 0xFE09, "VARIATION SELECTOR-10" },
    { 0xFE0A, "VARIATION SELECTOR-11" },
    { 0xFE0B, "VARIATION SELECTOR-12" },
    { 0xFE0C, "VARIATION SELECTOR-13" },
    { 0xFE0D, "VARIATION SELECTOR-14" },
    { 0xFE0E, "VARIATION SELECTOR-15" },
    { 0xFE0F, "VARIATION SELECTOR-16" },
    { 0xFE10, "PRESENTATION FORM FOR VERTICAL COMMA" },
    { 0xFE11, "PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC COMMA" },
    { 0xFE12, "PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC FULL STOP" },
    { 0xFE13, "PRESENTATION FORM FOR VERTICAL COLON" },
    { 0xFE14, "PRESENTATION FORM FOR VERTICAL SEMICOLON" },
    { 0xFE15, "PRESENTATION FORM FOR VERTICAL EXCLAMATION MARK" },
    { 0xFE16, "PRESENTATION FORM FOR VERTICAL QUESTION MARK" },
    { 0xFE17, "PRESENTATION FORM FOR VERTICAL LEFT WHITE LENTICULAR BRACKET" },
    { 0xFE18, "PRESENTATION FORM FOR VERTICAL RIGHT WHITE LENTICULAR BRAKCET" },
    { 0xFE19, "PRESENTATION FORM FOR VERTICAL HORIZONTAL ELLIPSIS" },
    { 0xFE20, "COMBINING LIGATURE LEFT HALF" },
    { 0xFE21, "COMBINING LIGATURE RIGHT HALF" },
    { 0xFE22, "COMBINING DOUBLE TILDE LEFT HALF" },
    { 0xFE23, "COMBINING DOUBLE TILDE RIGHT HALF" },
    { 0xFE24, "COMBINING MACRON LEFT HALF" },
    { 0xFE25, "COMBINING MACRON RIGHT HALF" },
    { 0xFE26, "COMBINING CONJOINING MACRON" },
    { 0xFE27, "COMBINING LIGATURE LEFT HALF BELOW" },
    { 0xFE28, "COMBINING LIGATURE RIGHT HALF BELOW" },
    { 0xFE29, "COMBINING TILDE LEFT HALF BELOW" },
    { 0xFE2A, "COMBINING TILDE RIGHT HALF BELOW" },
    { 0xFE2B, "COMBINING MACRON LEFT HALF BELOW" },
    { 0xFE2C, "COMBINING MACRON RIGHT HALF BELOW" },
    { 0xFE2D, "COMBINING CONJOINING MACRON BELOW" },
    { 0xFE2E, "COMBINING CYRILLIC TITLO LEFT HALF" },
    { 0xFE2F, "COMBINING CYRILLIC TITLO RIGHT HALF" },
    { 0xFE30, "PRESENTATION FORM FOR VERTICAL TWO DOT LEADER" },
    { 0xFE31, "PRESENTATION FORM FOR VERTICAL EM DASH" },
    { 0xFE32, "PRESENTATION FORM FOR VERTICAL EN DASH" },
    { 0xFE33, "PRESENTATION FORM FOR VERTICAL LOW LINE" },
    { 0xFE34, "PRESENTATION FORM FOR VERTICAL WAVY LOW LINE" },
    { 0xFE35, "PRESENTATION FORM FOR VERTICAL LEFT PARENTHESIS" },
    { 0xFE36, "PRESENTATION FORM FOR VERTICAL RIGHT PARENTHESIS" },
    { 0xFE37, "PRESENTATION FORM FOR VERTICAL LEFT CURLY BRACKET" },
    { 0xFE38, "PRESENTATION FORM FOR VERTICAL RIGHT CURLY BRACKET" },
    { 0xFE39, "PRESENTATION FORM FOR VERTICAL LEFT TORTOISE SHELL BRACKET" },
    { 0xFE3A, "PRESENTATION FORM FOR VERTICAL RIGHT TORTOISE SHELL BRACKET" },
    { 0xFE3B, "PRESENTATION FORM FOR VERTICAL LEFT BLACK LENTICULAR BRACKET" },
    { 0xFE3C, "PRESENTATION FORM FOR VERTICAL RIGHT BLACK LENTICULAR BRACKET" },
    { 0xFE3D, "PRESENTATION FORM FOR VERTICAL LEFT DOUBLE ANGLE BRACKET" },
    { 0xFE3E, "PRESENTATION FORM FOR VERTICAL RIGHT DOUBLE ANGLE BRACKET" },
    { 0xFE3F, "PRESENTATION FORM FOR VERTICAL LEFT ANGLE BRACKET" },
    { 0xFE40, "PRESENTATION FORM FOR VERTICAL RIGHT ANGLE BRACKET" },
    { 0xFE41, "PRESENTATION FORM FOR VERTICAL LEFT CORNER BRACKET" },
    { 0xFE42, "PRESENTATION FORM FOR VERTICAL RIGHT CORNER BRACKET" },
    { 0xFE43, "PRESENTATION FORM FOR VERTICAL LEFT WHITE CORNER BRACKET" },
    { 0xFE44, "PRESENTATION FORM FOR VERTICAL RIGHT WHITE CORNER BRACKET" },
    { 0xFE45, "SESAME DOT" },
    { 0xFE46, "WHITE SESAME DOT" },
    { 0xFE47, "PRESENTATION FORM FOR VERTICAL LEFT SQUARE BRACKET" },
    { 0xFE48, "PRESENTATION FORM FOR VERTICAL RIGHT SQUARE BRACKET" },
    { 0xFE49, "DASHED OVERLINE" },
    { 0xFE4A, "CENTRELINE OVERLINE" },
    { 0xFE4B, "WAVY OVERLINE" },
    { 0xFE4C, "DOUBLE WAVY OVERLINE" },
    { 0xFE4D, "DASHED LOW LINE" },
    { 0xFE4E, "CENTRELINE LOW LINE" },
    { 0xFE4F, "WAVY LOW LINE" },
    { 0xFE50, "SMALL COMMA" },
    { 0xFE51, "SMALL IDEOGRAPHIC COMMA" },
    { 0xFE52, "SMALL FULL STOP" },
    { 0xFE54, "SMALL SEMICOLON" },
    { 0xFE55, "SMALL COLON" },
    { 0xFE56, "SMALL QUESTION MARK" },
    { 0xFE57, "SMALL EXCLAMATION MARK" },
    { 0xFE58, "SMALL EM DASH" },
    { 0xFE59, "SMALL LEFT PARENTHESIS" },
    { 0xFE5A, "SMALL RIGHT PARENTHESIS" },
    { 0xFE5B, "SMALL LEFT CURLY BRACKET" },
    { 0xFE5C, "SMALL RIGHT CURLY BRACKET" },
    { 0xFE5D, "SMALL LEFT TORTOISE SHELL BRACKET" },
    { 0xFE5E, "SMALL RIGHT TORTOISE SHELL BRACKET" },
    { 0xFE5F, "SMALL NUMBER SIGN" },
    { 0xFE60, "SMALL AMPERSAND" },
    { 0xFE61, "SMALL ASTERISK" },
    { 0xFE62, "SMALL PLUS SIGN" },
    { 0xFE63, "SMALL HYPHEN-MINUS" },
    { 0xFE64, "SMALL LESS-THAN SIGN" },
    { 0xFE65, "SMALL GREATER-THAN SIGN" },
    { 0xFE66, "SMALL EQUALS SIGN" },
    { 0xFE68, "SMALL REVERSE SOLIDUS" },
    { 0xFE69, "SMALL DOLLAR SIGN" },
    { 0xFE6A, "SMALL PERCENT SIGN" },
    { 0xFE6B, "SMALL COMMERCIAL AT" },
    { 0xFE70, "ARABIC FATHATAN ISOLATED FORM" },
    { 0xFE71, "ARABIC TATWEEL WITH FATHATAN ABOVE" },
    { 0xFE72, "ARABIC DAMMATAN ISOLATED FORM" },
    { 0xFE73, "ARABIC TAIL FRAGMENT" },
    { 0xFE74, "ARABIC KASRATAN ISOLATED FORM" },
    { 0xFE76, "ARABIC FATHA ISOLATED FORM" },
    { 0xFE77, "ARABIC FATHA MEDIAL FORM" },
    { 0xFE78, "ARABIC DAMMA ISOLATED FORM" },
    { 0xFE79, "ARABIC DAMMA MEDIAL FORM" },
    { 0xFE7A, "ARABIC KASRA ISOLATED FORM" },
    { 0xFE7B, "ARABIC KASRA MEDIAL FORM" },
    { 0xFE7C, "ARABIC SHADDA ISOLATED FORM" },
    { 0xFE7D, "ARABIC SHADDA MEDIAL FORM" },
    { 0xFE7E, "ARABIC SUKUN ISOLATED FORM" },
    { 0xFE7F, "ARABIC SUKUN MEDIAL FORM" },
    { 0xFE80, "ARABIC LETTER HAMZA ISOLATED FORM" },
    { 0xFE81, "ARABIC LETTER ALEF WITH MADDA ABOVE ISOLATED FORM" },
    { 0xFE82, "ARABIC LETTER ALEF WITH MADDA ABOVE FINAL FORM" },
    { 0xFE83, "ARABIC LETTER ALEF WITH HAMZA ABOVE ISOLATED FORM" },
    { 0xFE84, "ARABIC LETTER ALEF WITH HAMZA ABOVE FINAL FORM" },
    { 0xFE85, "ARABIC LETTER WAW WITH HAMZA ABOVE ISOLATED FORM" },
    { 0xFE86, "ARABIC LETTER WAW WITH HAMZA ABOVE FINAL FORM" },
    { 0xFE87, "ARABIC LETTER ALEF WITH HAMZA BELOW ISOLATED FORM" },
    { 0xFE88, "ARABIC LETTER ALEF WITH HAMZA BELOW FINAL FORM" },
    { 0xFE89, "ARABIC LETTER YEH WITH HAMZA ABOVE ISOLATED FORM" },
    { 0xFE8A, "ARABIC LETTER YEH WITH HAMZA ABOVE FINAL FORM" },
    { 0xFE8B, "ARABIC LETTER YEH WITH HAMZA ABOVE INITIAL FORM" },
    { 0xFE8C, "ARABIC LETTER YEH WITH HAMZA ABOVE MEDIAL FORM" },
    { 0xFE8D, "ARABIC LETTER ALEF ISOLATED FORM" },
    { 0xFE8E, "ARABIC LETTER ALEF FINAL FORM" },
    { 0xFE8F, "ARABIC LETTER BEH ISOLATED FORM" },
    { 0xFE90, "ARABIC LETTER BEH FINAL FORM" },
    { 0xFE91, "ARABIC LETTER BEH INITIAL FORM" },
    { 0xFE92, "ARABIC LETTER BEH MEDIAL FORM" },
    { 0xFE93, "ARABIC LETTER TEH MARBUTA ISOLATED FORM" },
    { 0xFE94, "ARABIC LETTER TEH MARBUTA FINAL FORM" },
    { 0xFE95, "ARABIC LETTER TEH ISOLATED FORM" },
    { 0xFE96, "ARABIC LETTER TEH FINAL FORM" },
    { 0xFE97, "ARABIC LETTER TEH INITIAL FORM" },
    { 0xFE98, "ARABIC LETTER TEH MEDIAL FORM" },
    { 0xFE99, "ARABIC LETTER THEH ISOLATED FORM" },
    { 0xFE9A, "ARABIC LETTER THEH FINAL FORM" },
    { 0xFE9B, "ARABIC LETTER THEH INITIAL FORM" },
    { 0xFE9C, "ARABIC LETTER THEH MEDIAL FORM" },
    { 0xFE9D, "ARABIC LETTER JEEM ISOLATED FORM" },
    { 0xFE9E, "ARABIC LETTER JEEM FINAL FORM" },
    { 0xFE9F, "ARABIC LETTER JEEM INITIAL FORM" },
    { 0xFEA0, "ARABIC LETTER JEEM MEDIAL FORM" },
    { 0xFEA1, "ARABIC LETTER HAH ISOLATED FORM" },
    { 0xFEA2, "ARABIC LETTER HAH FINAL FORM" },
    { 0xFEA3, "ARABIC LETTER HAH INITIAL FORM" },
    { 0xFEA4, "ARABIC LETTER HAH MEDIAL FORM" },
    { 0xFEA5, "ARABIC LETTER KHAH ISOLATED FORM" },
    { 0xFEA6, "ARABIC LETTER KHAH FINAL FORM" },
    { 0xFEA7, "ARABIC LETTER KHAH INITIAL FORM" },
    { 0xFEA8, "ARABIC LETTER KHAH MEDIAL FORM" },
    { 0xFEA9, "ARABIC LETTER DAL ISOLATED FORM" },
    { 0xFEAA, "ARABIC LETTER DAL FINAL FORM" },
    { 0xFEAB, "ARABIC LETTER THAL ISOLATED FORM" },
    { 0xFEAC, "ARABIC LETTER THAL FINAL FORM" },
    { 0xFEAD, "ARABIC LETTER REH ISOLATED FORM" },
    { 0xFEAE, "ARABIC LETTER REH FINAL FORM" },
    { 0xFEAF, "ARABIC LETTER ZAIN ISOLATED FORM" },
    { 0xFEB0, "ARABIC LETTER ZAIN FINAL FORM" },
    { 0xFEB1, "ARABIC LETTER SEEN ISOLATED FORM" },
    { 0xFEB2, "ARABIC LETTER SEEN FINAL FORM" },
    { 0xFEB3, "ARABIC LETTER SEEN INITIAL FORM" },
    { 0xFEB4, "ARABIC LETTER SEEN MEDIAL FORM" },
    { 0xFEB5, "ARABIC LETTER SHEEN ISOLATED FORM" },
    { 0xFEB6, "ARABIC LETTER SHEEN FINAL FORM" },
    { 0xFEB7, "ARABIC LETTER SHEEN INITIAL FORM" },
    { 0xFEB8, "ARABIC LETTER SHEEN MEDIAL FORM" },
    { 0xFEB9, "ARABIC LETTER SAD ISOLATED FORM" },
    { 0xFEBA, "ARABIC LETTER SAD FINAL FORM" },
    { 0xFEBB, "ARABIC LETTER SAD INITIAL FORM" },
    { 0xFEBC, "ARABIC LETTER SAD MEDIAL FORM" },
    { 0xFEBD, "ARABIC LETTER DAD ISOLATED FORM" },
    { 0xFEBE, "ARABIC LETTER DAD FINAL FORM" },
    { 0xFEBF, "ARABIC LETTER DAD INITIAL FORM" },
    { 0xFEC0, "ARABIC LETTER DAD MEDIAL FORM" },
    { 0xFEC1, "ARABIC LETTER TAH ISOLATED FORM" },
    { 0xFEC2, "ARABIC LETTER TAH FINAL FORM" },
    { 0xFEC3, "ARABIC LETTER TAH INITIAL FORM" },
    { 0xFEC4, "ARABIC LETTER TAH MEDIAL FORM" },
    { 0xFEC5, "ARABIC LETTER ZAH ISOLATED FORM" },
    { 0xFEC6, "ARABIC LETTER ZAH FINAL FORM" },
    { 0xFEC7, "ARABIC LETTER ZAH INITIAL FORM" },
    { 0xFEC8, "ARABIC LETTER ZAH MEDIAL FORM" },
    { 0xFEC9, "ARABIC LETTER AIN ISOLATED FORM" },
    { 0xFECA, "ARABIC LETTER AIN FINAL FORM" },
    { 0xFECB, "ARABIC LETTER AIN INITIAL FORM" },
    { 0xFECC, "ARABIC LETTER AIN MEDIAL FORM" },
    { 0xFECD, "ARABIC LETTER GHAIN ISOLATED FORM" },
    { 0xFECE, "ARABIC LETTER GHAIN FINAL FORM" },
    { 0xFECF, "ARABIC LETTER GHAIN INITIAL FORM" },
    { 0xFED0, "ARABIC LETTER GHAIN MEDIAL FORM" },
    { 0xFED1, "ARABIC LETTER FEH ISOLATED FORM" },
    { 0xFED2, "ARABIC LETTER FEH FINAL FORM" },
    { 0xFED3, "ARABIC LETTER FEH INITIAL FORM" },
    { 0xFED4, "ARABIC LETTER FEH MEDIAL FORM" },
    { 0xFED5, "ARABIC LETTER QAF ISOLATED FORM" },
    { 0xFED6, "ARABIC LETTER QAF FINAL FORM" },
    { 0xFED7, "ARABIC LETTER QAF INITIAL FORM" },
    { 0xFED8, "ARABIC LETTER QAF MEDIAL FORM" },
    { 0xFED9, "ARABIC LETTER KAF ISOLATED FORM" },
    { 0xFEDA, "ARABIC LETTER KAF FINAL FORM" },
    { 0xFEDB, "ARABIC LETTER KAF INITIAL FORM" },
    { 0xFEDC, "ARABIC LETTER KAF MEDIAL FORM" },
    { 0xFEDD, "ARABIC LETTER LAM ISOLATED FORM" },
    { 0xFEDE, "ARABIC LETTER LAM FINAL FORM" },
    { 0xFEDF, "ARABIC LETTER LAM INITIAL FORM" },
    { 0xFEE0, "ARABIC LETTER LAM MEDIAL FORM" },
    { 0xFEE1, "ARABIC LETTER MEEM ISOLATED FORM" },
    { 0xFEE2, "ARABIC LETTER MEEM FINAL FORM" },
    { 0xFEE3, "ARABIC LETTER MEEM INITIAL FORM" },
    { 0xFEE4, "ARABIC LETTER MEEM MEDIAL FORM" },
    { 0xFEE5, "ARABIC LETTER NOON ISOLATED FORM" },
    { 0xFEE6, "ARABIC LETTER NOON FINAL FORM" },
    { 0xFEE7, "ARABIC LETTER NOON INITIAL FORM" },
    { 0xFEE8, "ARABIC LETTER NOON MEDIAL FORM" },
    { 0xFEE9, "ARABIC LETTER HEH ISOLATED FORM" },
    { 0xFEEA, "ARABIC LETTER HEH FINAL FORM" },
    { 0xFEEB, "ARABIC LETTER HEH INITIAL FORM" },
    { 0xFEEC, "ARABIC LETTER HEH MEDIAL FORM" },
    { 0xFEED, "ARABIC LETTER WAW ISOLATED FORM" },
    { 0xFEEE, "ARABIC LETTER WAW FINAL FORM" },
    { 0xFEEF, "ARABIC LETTER ALEF MAKSURA ISOLATED FORM" },
    { 0xFEF0, "ARABIC LETTER ALEF MAKSURA FINAL FORM" },
    { 0xFEF1, "ARABIC LETTER YEH ISOLATED FORM" },
    { 0xFEF2, "ARABIC LETTER YEH FINAL FORM" },
    { 0xFEF3, "ARABIC LETTER YEH INITIAL FORM" },
    { 0xFEF4, "ARABIC LETTER YEH MEDIAL FORM" },
    { 0xFEF5, "ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE ISOLATED FORM" },
    { 0xFEF6, "ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE FINAL FORM" },
    { 0xFEF7, "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE ISOLATED FORM" },
    { 0xFEF8, "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE FINAL FORM" },
    { 0xFEF9, "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW ISOLATED FORM" },
    { 0xFEFA, "ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW FINAL FORM" },
    { 0xFEFB, "ARABIC LIGATURE LAM WITH ALEF ISOLATED FORM" },
    { 0xFEFC, "ARABIC LIGATURE LAM WITH ALEF FINAL FORM" },
    { 0xFEFF, "ZERO WIDTH NO-BREAK SPACE" },
    { 0xFF01, "FULLWIDTH EXCLAMATION MARK" },
    { 0xFF02, "FULLWIDTH QUOTATION MARK" },
    { 0xFF03, "FULLWIDTH NUMBER SIGN" },
    { 0xFF04, "FULLWIDTH DOLLAR SIGN" },
    { 0xFF05, "FULLWIDTH PERCENT SIGN" },
    { 0xFF06, "FULLWIDTH AMPERSAND" },
    { 0xFF07, "FULLWIDTH APOSTROPHE" },
    { 0xFF08, "FULLWIDTH LEFT PARENTHESIS" },
    { 0xFF09, "FULLWIDTH RIGHT PARENTHESIS" },
    { 0xFF0A, "FULLWIDTH ASTERISK" },
    { 0xFF0B, "FULLWIDTH PLUS SIGN" },
    { 0xFF0C, "FULLWIDTH COMMA" },
    { 0xFF0D, "FULLWIDTH HYPHEN-MINUS" },
    { 0xFF0E, "FULLWIDTH FULL STOP" },
    { 0xFF0F, "FULLWIDTH SOLIDUS" },
    { 0xFF10, "FULLWIDTH DIGIT ZERO" },
    { 0xFF11, "FULLWIDTH DIGIT ONE" },
    { 0xFF12, "FULLWIDTH DIGIT TWO" },
    { 0xFF13, "FULLWIDTH DIGIT THREE" },
    { 0xFF14, "FULLWIDTH DIGIT FOUR" },
    { 0xFF15, "FULLWIDTH DIGIT FIVE" },
    { 0xFF16, "FULLWIDTH DIGIT SIX" },
    { 0xFF17, "FULLWIDTH DIGIT SEVEN" },
    { 0xFF18, "FULLWIDTH DIGIT EIGHT" },
    { 0xFF19, "FULLWIDTH DIGIT NINE" },
    { 0xFF1A, "FULLWIDTH COLON" },
    { 0xFF1B, "FULLWIDTH SEMICOLON" },
    { 0xFF1C, "FULLWIDTH LESS-THAN SIGN" },
    { 0xFF1D, "FULLWIDTH EQUALS SIGN" },
    { 0xFF1E, "FULLWIDTH GREATER-THAN SIGN" },
    { 0xFF1F, "FULLWIDTH QUESTION MARK" },
    { 0xFF20, "FULLWIDTH COMMERCIAL AT" },
    { 0xFF21, "FULLWIDTH LATIN CAPITAL LETTER A" },
    { 0xFF22, "FULLWIDTH LATIN CAPITAL LETTER B" },
    { 0xFF23, "FULLWIDTH LATIN CAPITAL LETTER C" },
    { 0xFF24, "FULLWIDTH LATIN CAPITAL LETTER D" },
    { 0xFF25, "FULLWIDTH LATIN CAPITAL LETTER E" },
    { 0xFF26, "FULLWIDTH LATIN CAPITAL LETTER F" },
    { 0xFF27, "FULLWIDTH LATIN CAPITAL LETTER G" },
    { 0xFF28, "FULLWIDTH LATIN CAPITAL LETTER H" },
    { 0xFF29, "FULLWIDTH LATIN CAPITAL LETTER I" },
    { 0xFF2A, "FULLWIDTH LATIN CAPITAL LETTER J" },
    { 0xFF2B, "FULLWIDTH LATIN CAPITAL LETTER K" },
    { 0xFF2C, "FULLWIDTH LATIN CAPITAL LETTER L" },
    { 0xFF2D, "FULLWIDTH LATIN CAPITAL LETTER M" },
    { 0xFF2E, "FULLWIDTH LATIN CAPITAL LETTER N" },
    { 0xFF2F, "FULLWIDTH LATIN CAPITAL LETTER O" },
    { 0xFF30, "FULLWIDTH LATIN CAPITAL LETTER P" },
    { 0xFF31, "FULLWIDTH LATIN CAPITAL LETTER Q" },
    { 0xFF32, "FULLWIDTH LATIN CAPITAL LETTER R" },
    { 0xFF33, "FULLWIDTH LATIN CAPITAL LETTER S" },
    { 0xFF34, "FULLWIDTH LATIN CAPITAL LETTER T" },
    { 0xFF35, "FULLWIDTH LATIN CAPITAL LETTER U" },
    { 0xFF36, "FULLWIDTH LATIN CAPITAL LETTER V" },
    { 0xFF37, "FULLWIDTH LATIN CAPITAL LETTER W" },
    { 0xFF38, "FULLWIDTH LATIN CAPITAL LETTER X" },
    { 0xFF39, "FULLWIDTH LATIN CAPITAL LETTER Y" },
    { 0xFF3A, "FULLWIDTH LATIN CAPITAL LETTER Z" },
    { 0xFF3B, "FULLWIDTH LEFT SQUARE BRACKET" },
    { 0xFF3C, "FULLWIDTH REVERSE SOLIDUS" },
    { 0xFF3D, "FULLWIDTH RIGHT SQUARE BRACKET" },
    { 0xFF3E, "FULLWIDTH CIRCUMFLEX ACCENT" },
    { 0xFF3F, "FULLWIDTH LOW LINE" },
    { 0xFF40, "FULLWIDTH GRAVE ACCENT" },
    { 0xFF41, "FULLWIDTH LATIN SMALL LETTER A" },
    { 0xFF42, "FULLWIDTH LATIN SMALL LETTER B" },
    { 0xFF43, "FULLWIDTH LATIN SMALL LETTER C" },
    { 0xFF44, "FULLWIDTH LATIN SMALL LETTER D" },
    { 0xFF45, "FULLWIDTH LATIN SMALL LETTER E" },
    { 0xFF46, "FULLWIDTH LATIN SMALL LETTER F" },
    { 0xFF47, "FULLWIDTH LATIN SMALL LETTER G" },
    { 0xFF48, "FULLWIDTH LATIN SMALL LETTER H" },
    { 0xFF49, "FULLWIDTH LATIN SMALL LETTER I" },
    { 0xFF4A, "FULLWIDTH LATIN SMALL LETTER J" },
    { 0xFF4B, "FULLWIDTH LATIN SMALL LETTER K" },
    { 0xFF4C, "FULLWIDTH LATIN SMALL LETTER L" },
    { 0xFF4D, "FULLWIDTH LATIN SMALL LETTER M" },
    { 0xFF4E, "FULLWIDTH LATIN SMALL LETTER N" },
    { 0xFF4F, "FULLWIDTH LATIN SMALL LETTER O" },
    { 0xFF50, "FULLWIDTH LATIN SMALL LETTER P" },
    { 0xFF51, "FULLWIDTH LATIN SMALL LETTER Q" },
    { 0xFF52, "FULLWIDTH LATIN SMALL LETTER R" },
    { 0xFF53, "FULLWIDTH LATIN SMALL LETTER S" },
    { 0xFF54, "FULLWIDTH LATIN SMALL LETTER T" },
    { 0xFF55, "FULLWIDTH LATIN SMALL LETTER U" },
    { 0xFF56, "FULLWIDTH LATIN SMALL LETTER V" },
    { 0xFF57, "FULLWIDTH LATIN SMALL LETTER W" },
    { 0xFF58, "FULLWIDTH LATIN SMALL LETTER X" },
    { 0xFF59, "FULLWIDTH LATIN SMALL LETTER Y" },
    { 0xFF5A, "FULLWIDTH LATIN SMALL LETTER Z" },
    { 0xFF5B, "FULLWIDTH LEFT CURLY BRACKET" },
    { 0xFF5C, "FULLWIDTH VERTICAL LINE" },
    { 0xFF5D, "FULLWIDTH RIGHT CURLY BRACKET" },
    { 0xFF5E, "FULLWIDTH TILDE" },
    { 0xFF5F, "FULLWIDTH LEFT WHITE PARENTHESIS" },
    { 0xFF60, "FULLWIDTH RIGHT WHITE PARENTHESIS" },
    { 0xFF61, "HALFWIDTH IDEOGRAPHIC FULL STOP" },
    { 0xFF62, "HALFWIDTH LEFT CORNER BRACKET" },
    { 0xFF63, "HALFWIDTH RIGHT CORNER BRACKET" },
    { 0xFF64, "HALFWIDTH IDEOGRAPHIC COMMA" },
    { 0xFF65, "HALFWIDTH KATAKANA MIDDLE DOT" },
    { 0xFF66, "HALFWIDTH KATAKANA LETTER WO" },
    { 0xFF67, "HALFWIDTH KATAKANA LETTER SMALL A" },
    { 0xFF68, "HALFWIDTH KATAKANA LETTER SMALL I" },
    { 0xFF69, "HALFWIDTH KATAKANA LETTER SMALL U" },
    { 0xFF6A, "HALFWIDTH KATAKANA LETTER SMALL E" },
    { 0xFF6B, "HALFWIDTH KATAKANA LETTER SMALL O" },
    { 0xFF6C, "HALFWIDTH KATAKANA LETTER SMALL YA" },
    { 0xFF6D, "HALFWIDTH KATAKANA LETTER SMALL YU" },
    { 0xFF6E, "HALFWIDTH KATAKANA LETTER SMALL YO" },
    { 0xFF6F, "HALFWIDTH KATAKANA LETTER SMALL TU" },
    { 0xFF70, "HALFWIDTH KATAKANA-HIRAGANA PROLONGED SOUND MARK" },
    { 0xFF71, "HALFWIDTH KATAKANA LETTER A" },
    { 0xFF72, "HALFWIDTH KATAKANA LETTER I" },
    { 0xFF73, "HALFWIDTH KATAKANA LETTER U" },
    { 0xFF74, "HALFWIDTH KATAKANA LETTER E" },
    { 0xFF75, "HALFWIDTH KATAKANA LETTER O" },
    { 0xFF76, "HALFWIDTH KATAKANA LETTER KA" },
    { 0xFF77, "HALFWIDTH KATAKANA LETTER KI" },
    { 0xFF78, "HALFWIDTH KATAKANA LETTER KU" },
    { 0xFF79, "HALFWIDTH KATAKANA LETTER KE" },
    { 0xFF7A, "HALFWIDTH KATAKANA LETTER KO" },
    { 0xFF7B, "HALFWIDTH KATAKANA LETTER SA" },
    { 0xFF7C, "HALFWIDTH KATAKANA LETTER SI" },
    { 0xFF7D, "HALFWIDTH KATAKANA LETTER SU" },
    { 0xFF7E, "HALFWIDTH KATAKANA LETTER SE" },
    { 0xFF7F, "HALFWIDTH KATAKANA LETTER SO" },
    { 0xFF80, "HALFWIDTH KATAKANA LETTER TA" },
    { 0xFF81, "HALFWIDTH KATAKANA LETTER TI" },
    { 0xFF82, "HALFWIDTH KATAKANA LETTER TU" },
    { 0xFF83, "HALFWIDTH KATAKANA LETTER TE" },
    { 0xFF84, "HALFWIDTH KATAKANA LETTER TO" },
    { 0xFF85, "HALFWIDTH KATAKANA LETTER NA" },
    { 0xFF86, "HALFWIDTH KATAKANA LETTER NI" },
    { 0xFF87, "HALFWIDTH KATAKANA LETTER NU" },
    { 0xFF88, "HALFWIDTH KATAKANA LETTER NE" },
    { 0xFF89, "HALFWIDTH KATAKANA LETTER NO" },
    { 0xFF8A, "HALFWIDTH KATAKANA LETTER HA" },
    { 0xFF8B, "HALFWIDTH KATAKANA LETTER HI" },
    { 0xFF8C, "HALFWIDTH KATAKANA LETTER HU" },
    { 0xFF8D, "HALFWIDTH KATAKANA LETTER HE" },
    { 0xFF8E, "HALFWIDTH KATAKANA LETTER HO" },
    { 0xFF8F, "HALFWIDTH KATAKANA LETTER MA" },
    { 0xFF90, "HALFWIDTH KATAKANA LETTER MI" },
    { 0xFF91, "HALFWIDTH KATAKANA LETTER MU" },
    { 0xFF92, "HALFWIDTH KATAKANA LETTER ME" },
    { 0xFF93, "HALFWIDTH KATAKANA LETTER MO" },
    { 0xFF94, "HALFWIDTH KATAKANA LETTER YA" },
    { 0xFF95, "HALFWIDTH KATAKANA LETTER YU" },
    { 0xFF96, "HALFWIDTH KATAKANA LETTER YO" },
    { 0xFF97, "HALFWIDTH KATAKANA LETTER RA" },
    { 0xFF98, "HALFWIDTH KATAKANA LETTER RI" },
    { 0xFF99, "HALFWIDTH KATAKANA LETTER RU" },
    { 0xFF9A, "HALFWIDTH KATAKANA LETTER RE" },
    { 0xFF9B, "HALFWIDTH KATAKANA LETTER RO" },
    { 0xFF9C, "HALFWIDTH KATAKANA LETTER WA" },
    { 0xFF9D, "HALFWIDTH KATAKANA LETTER N" },
    { 0xFF9E, "HALFWIDTH KATAKANA VOICED SOUND MARK" },
    { 0xFF9F, "HALFWIDTH KATAKANA SEMI-VOICED SOUND MARK" },
    { 0xFFA0, "HALFWIDTH HANGUL FILLER" },
    { 0xFFA1, "HALFWIDTH HANGUL LETTER KIYEOK" },
    { 0xFFA2, "HALFWIDTH HANGUL LETTER SSANGKIYEOK" },
    { 0xFFA3, "HALFWIDTH HANGUL LETTER KIYEOK-SIOS" },
    { 0xFFA4, "HALFWIDTH HANGUL LETTER NIEUN" },
    { 0xFFA5, "HALFWIDTH HANGUL LETTER NIEUN-CIEUC" },
    { 0xFFA6, "HALFWIDTH HANGUL LETTER NIEUN-HIEUH" },
    { 0xFFA7, "HALFWIDTH HANGUL LETTER TIKEUT" },
    { 0xFFA8, "HALFWIDTH HANGUL LETTER SSANGTIKEUT" },
    { 0xFFA9, "HALFWIDTH HANGUL LETTER RIEUL" },
    { 0xFFAA, "HALFWIDTH HANGUL LETTER RIEUL-KIYEOK" },
    { 0xFFAB, "HALFWIDTH HANGUL LETTER RIEUL-MIEUM" },
    { 0xFFAC, "HALFWIDTH HANGUL LETTER RIEUL-PIEUP" },
    { 0xFFAD, "HALFWIDTH HANGUL LETTER RIEUL-SIOS" },
    { 0xFFAE, "HALFWIDTH HANGUL LETTER RIEUL-THIEUTH" },
    { 0xFFAF, "HALFWIDTH HANGUL LETTER RIEUL-PHIEUPH" },
    { 0xFFB0, "HALFWIDTH HANGUL LETTER RIEUL-HIEUH" },
    { 0xFFB1, "HALFWIDTH HANGUL LETTER MIEUM" },
    { 0xFFB2, "HALFWIDTH HANGUL LETTER PIEUP" },
    { 0xFFB3, "HALFWIDTH HANGUL LETTER SSANGPIEUP" },
    { 0xFFB4, "HALFWIDTH HANGUL LETTER PIEUP-SIOS" },
    { 0xFFB5, "HALFWIDTH HANGUL LETTER SIOS" },
    { 0xFFB6, "HALFWIDTH HANGUL LETTER SSANGSIOS" },
    { 0xFFB7, "HALFWIDTH HANGUL LETTER IEUNG" },
    { 0xFFB8, "HALFWIDTH HANGUL LETTER CIEUC" },
    { 0xFFB9, "HALFWIDTH HANGUL LETTER SSANGCIEUC" },
    { 0xFFBA, "HALFWIDTH HANGUL LETTER CHIEUCH" },
    { 0xFFBB, "HALFWIDTH HANGUL LETTER KHIEUKH" },
    { 0xFFBC, "HALFWIDTH HANGUL LETTER THIEUTH" },
    { 0xFFBD, "HALFWIDTH HANGUL LETTER PHIEUPH" },
    { 0xFFBE, "HALFWIDTH HANGUL LETTER HIEUH" },
    { 0xFFC2, "HALFWIDTH HANGUL LETTER A" },
    { 0xFFC3, "HALFWIDTH HANGUL LETTER AE" },
    { 0xFFC4, "HALFWIDTH HANGUL LETTER YA" },
    { 0xFFC5, "HALFWIDTH HANGUL LETTER YAE" },
    { 0xFFC6, "HALFWIDTH HANGUL LETTER EO" },
    { 0xFFC7, "HALFWIDTH HANGUL LETTER E" },
    { 0xFFCA, "HALFWIDTH HANGUL LETTER YEO" },
    { 0xFFCB, "HALFWIDTH HANGUL LETTER YE" },
    { 0xFFCC, "HALFWIDTH HANGUL LETTER O" },
    { 0xFFCD, "HALFWIDTH HANGUL LETTER WA" },
    { 0xFFCE, "HALFWIDTH HANGUL LETTER WAE" },
    { 0xFFCF, "HALFWIDTH HANGUL LETTER OE" },
    { 0xFFD2, "HALFWIDTH HANGUL LETTER YO" },
    { 0xFFD3, "HALFWIDTH HANGUL LETTER U" },
    { 0xFFD4, "HALFWIDTH HANGUL LETTER WEO" },
    { 0xFFD5, "HALFWIDTH HANGUL LETTER WE" },
    { 0xFFD6, "HALFWIDTH HANGUL LETTER WI" },
    { 0xFFD7, "HALFWIDTH HANGUL LETTER YU" },
    { 0xFFDA, "HALFWIDTH HANGUL LETTER EU" },
    { 0xFFDB, "HALFWIDTH HANGUL LETTER YI" },
    { 0xFFDC, "HALFWIDTH HANGUL LETTER I" },
    { 0xFFE0, "FULLWIDTH CENT SIGN" },
    { 0xFFE1, "FULLWIDTH POUND SIGN" },
    { 0xFFE2, "FULLWIDTH NOT SIGN" },
    { 0xFFE3, "FULLWIDTH MACRON" },
    { 0xFFE4, "FULLWIDTH BROKEN BAR" },
    { 0xFFE5, "FULLWIDTH YEN SIGN" },
    { 0xFFE6, "FULLWIDTH WON SIGN" },
    { 0xFFE8, "HALFWIDTH FORMS LIGHT VERTICAL" },
    { 0xFFE9, "HALFWIDTH LEFTWARDS ARROW" },
    { 0xFFEA, "HALFWIDTH UPWARDS ARROW" },
    { 0xFFEB, "HALFWIDTH RIGHTWARDS ARROW" },
    { 0xFFEC, "HALFWIDTH DOWNWARDS ARROW" },
    { 0xFFED, "HALFWIDTH BLACK SQUARE" },
    { 0xFFEE, "HALFWIDTH WHITE CIRCLE" },
    { 0xFFF9, "INTERLINEAR ANNOTATION ANCHOR" },
    { 0xFFFA, "INTERLINEAR ANNOTATION SEPARATOR" },
    { 0xFFFB, "INTERLINEAR ANNOTATION TERMINATOR" },
    { 0xFFFC, "OBJECT REPLACEMENT CHARACTER" },
    { 0xFFFD, "REPLACEMENT CHARACTER" },
    { 0x10000, "LINEAR B SYLLABLE B008 A" },
    { 0x10001, "LINEAR B SYLLABLE B038 E" },
    { 0x10002, "LINEAR B SYLLABLE B028 I" },
    { 0x10003, "LINEAR B SYLLABLE B061 O" },
    { 0x10004, "LINEAR B SYLLABLE B010 U" },
    { 0x10005, "LINEAR B SYLLABLE B001 DA" },
    { 0x10006, "LINEAR B SYLLABLE B045 DE" },
    { 0x10007, "LINEAR B SYLLABLE B007 DI" },
    { 0x10008, "LINEAR B SYLLABLE B014 DO" },
    { 0x10009, "LINEAR B SYLLABLE B051 DU" },
    { 0x1000A, "LINEAR B SYLLABLE B057 JA" },
    { 0x1000B, "LINEAR B SYLLABLE B046 JE" },
    { 0x1000D, "LINEAR B SYLLABLE B036 JO" },
    { 0x1000E, "LINEAR B SYLLABLE B065 JU" },
    { 0x1000F, "LINEAR B SYLLABLE B077 KA" },
    { 0x10010, "LINEAR B SYLLABLE B044 KE" },
    { 0x10011, "LINEAR B SYLLABLE B067 KI" },
    { 0x10012, "LINEAR B SYLLABLE B070 KO" },
    { 0x10013, "LINEAR B SYLLABLE B081 KU" },
    { 0x10014, "LINEAR B SYLLABLE B080 MA" },
    { 0x10015, "LINEAR B SYLLABLE B013 ME" },
    { 0x10016, "LINEAR B SYLLABLE B073 MI" },
    { 0x10017, "LINEAR B SYLLABLE B015 MO" },
    { 0x10018, "LINEAR B SYLLABLE B023 MU" },
    { 0x10019, "LINEAR B SYLLABLE B006 NA" },
    { 0x1001A, "LINEAR B SYLLABLE B024 NE" },
    { 0x1001B, "LINEAR B SYLLABLE B030 NI" },
    { 0x1001C, "LINEAR B SYLLABLE B052 NO" },
    { 0x1001D, "LINEAR B SYLLABLE B055 NU" },
    { 0x1001E, "LINEAR B SYLLABLE B003 PA" },
    { 0x1001F, "LINEAR B SYLLABLE B072 PE" },
    { 0x10020, "LINEAR B SYLLABLE B039 PI" },
    { 0x10021, "LINEAR B SYLLABLE B011 PO" },
    { 0x10022, "LINEAR B SYLLABLE B050 PU" },
    { 0x10023, "LINEAR B SYLLABLE B016 QA" },
    { 0x10024, "LINEAR B SYLLABLE B078 QE" },
    { 0x10025, "LINEAR B SYLLABLE B021 QI" },
    { 0x10026, "LINEAR B SYLLABLE B032 QO" },
    { 0x10028, "LINEAR B SYLLABLE B060 RA" },
    { 0x10029, "LINEAR B SYLLABLE B027 RE" },
    { 0x1002A, "LINEAR B SYLLABLE B053 RI" },
    { 0x1002B, "LINEAR B SYLLABLE B002 RO" },
    { 0x1002C, "LINEAR B SYLLABLE B026 RU" },
    { 0x1002D, "LINEAR B SYLLABLE B031 SA" },
    { 0x1002E, "LINEAR B SYLLABLE B009 SE" },
    { 0x1002F, "LINEAR B SYLLABLE B041 SI" },
    { 0x10030, "LINEAR B SYLLABLE B012 SO" },
    { 0x10031, "LINEAR B SYLLABLE B058 SU" },
    { 0x10032, "LINEAR B SYLLABLE B059 TA" },
    { 0x10033, "LINEAR B SYLLABLE B004 TE" },
    { 0x10034, "LINEAR B SYLLABLE B037 TI" },
    { 0x10035, "LINEAR B SYLLABLE B005 TO" },
    { 0x10036, "LINEAR B SYLLABLE B069 TU" },
    { 0x10037, "LINEAR B SYLLABLE B054 WA" },
    { 0x10038, "LINEAR B SYLLABLE B075 WE" },
    { 0x10039, "LINEAR B SYLLABLE B040 WI" },
    { 0x1003A, "LINEAR B SYLLABLE B042 WO" },
    { 0x1003C, "LINEAR B SYLLABLE B017 ZA" },
    { 0x1003D, "LINEAR B SYLLABLE B074 ZE" },
    { 0x1003F, "LINEAR B SYLLABLE B020 ZO" },
    { 0x10040, "LINEAR B SYLLABLE B025 A2" },
    { 0x10041, "LINEAR B SYLLABLE B043 A3" },
    { 0x10042, "LINEAR B SYLLABLE B085 AU" },
    { 0x10043, "LINEAR B SYLLABLE B071 DWE" },
    { 0x10044, "LINEAR B SYLLABLE B090 DWO" },
    { 0x10045, "LINEAR B SYLLABLE B048 NWA" },
    { 0x10046, "LINEAR B SYLLABLE B029 PU2" },
    { 0x10047, "LINEAR B SYLLABLE B062 PTE" },
    { 0x10048, "LINEAR B SYLLABLE B076 RA2" },
    { 0x10049, "LINEAR B SYLLABLE B033 RA3" },
    { 0x1004A, "LINEAR B SYLLABLE B068 RO2" },
    { 0x1004B, "LINEAR B SYLLABLE B066 TA2" },
    { 0x1004C, "LINEAR B SYLLABLE B087 TWE" },
    { 0x1004D, "LINEAR B SYLLABLE B091 TWO" },
    { 0x10050, "LINEAR B SYMBOL B018" },
    { 0x10051, "LINEAR B SYMBOL B019" },
    { 0x10052, "LINEAR B SYMBOL B022" },
    { 0x10053, "LINEAR B SYMBOL B034" },
    { 0x10054, "LINEAR B SYMBOL B047" },
    { 0x10055, "LINEAR B SYMBOL B049" },
    { 0x10056, "LINEAR B SYMBOL B056" },
    { 0x10057, "LINEAR B SYMBOL B063" },
    { 0x10058, "LINEAR B SYMBOL B064" },
    { 0x10059, "LINEAR B SYMBOL B079" },
    { 0x1005A, "LINEAR B SYMBOL B082" },
    { 0x1005B, "LINEAR B SYMBOL B083" },
    { 0x1005C, "LINEAR B SYMBOL B086" },
    { 0x1005D, "LINEAR B SYMBOL B089" },
    { 0x10080, "LINEAR B IDEOGRAM B100 MAN" },
    { 0x10081, "LINEAR B IDEOGRAM B102 WOMAN" },
    { 0x10082, "LINEAR B IDEOGRAM B104 DEER" },
    { 0x10083, "LINEAR B IDEOGRAM B105 EQUID" },
    { 0x10084, "LINEAR B IDEOGRAM B105F MARE" },
    { 0x10085, "LINEAR B IDEOGRAM B105M STALLION" },
    { 0x10086, "LINEAR B IDEOGRAM B106F EWE" },
    { 0x10087, "LINEAR B IDEOGRAM B106M RAM" },
    { 0x10088, "LINEAR B IDEOGRAM B107F SHE-GOAT" },
    { 0x10089, "LINEAR B IDEOGRAM B107M HE-GOAT" },
    { 0x1008A, "LINEAR B IDEOGRAM B108F SOW" },
    { 0x1008B, "LINEAR B IDEOGRAM B108M BOAR" },
    { 0x1008C, "LINEAR B IDEOGRAM B109F COW" },
    { 0x1008D, "LINEAR B IDEOGRAM B109M BULL" },
    { 0x1008E, "LINEAR B IDEOGRAM B120 WHEAT" },
    { 0x1008F, "LINEAR B IDEOGRAM B121 BARLEY" },
    { 0x10090, "LINEAR B IDEOGRAM B122 OLIVE" },
    { 0x10091, "LINEAR B IDEOGRAM B123 SPICE" },
    { 0x10092, "LINEAR B IDEOGRAM B125 CYPERUS" },
    { 0x10093, "LINEAR B MONOGRAM B127 KAPO" },
    { 0x10094, "LINEAR B MONOGRAM B128 KANAKO" },
    { 0x10095, "LINEAR B IDEOGRAM B130 OIL" },
    { 0x10096, "LINEAR B IDEOGRAM B131 WINE" },
    { 0x10097, "LINEAR B IDEOGRAM B132" },
    { 0x10098, "LINEAR B MONOGRAM B133 AREPA" },
    { 0x10099, "LINEAR B MONOGRAM B135 MERI" },
    { 0x1009A, "LINEAR B IDEOGRAM B140 BRONZE" },
    { 0x1009B, "LINEAR B IDEOGRAM B141 GOLD" },
    { 0x1009C, "LINEAR B IDEOGRAM B142" },
    { 0x1009D, "LINEAR B IDEOGRAM B145 WOOL" },
    { 0x1009E, "LINEAR B IDEOGRAM B146" },
    { 0x1009F, "LINEAR B IDEOGRAM B150" },
    { 0x100A0, "LINEAR B IDEOGRAM B151 HORN" },
    { 0x100A1, "LINEAR B IDEOGRAM B152" },
    { 0x100A2, "LINEAR B IDEOGRAM B153" },
    { 0x100A3, "LINEAR B IDEOGRAM B154" },
    { 0x100A4, "LINEAR B MONOGRAM B156 TURO2" },
    { 0x100A5, "LINEAR B IDEOGRAM B157" },
    { 0x100A6, "LINEAR B IDEOGRAM B158" },
    { 0x100A7, "LINEAR B IDEOGRAM B159 CLOTH" },
    { 0x100A8, "LINEAR B IDEOGRAM B160" },
    { 0x100A9, "LINEAR B IDEOGRAM B161" },
    { 0x100AA, "LINEAR B IDEOGRAM B162 GARMENT" },
    { 0x100AB, "LINEAR B IDEOGRAM B163 ARMOUR" },
    { 0x100AC, "LINEAR B IDEOGRAM B164" },
    { 0x100AD, "LINEAR B IDEOGRAM B165" },
    { 0x100AE, "LINEAR B IDEOGRAM B166" },
    { 0x100AF, "LINEAR B IDEOGRAM B167" },
    { 0x100B0, "LINEAR B IDEOGRAM B168" },
    { 0x100B1, "LINEAR B IDEOGRAM B169" },
    { 0x100B2, "LINEAR B IDEOGRAM B170" },
    { 0x100B3, "LINEAR B IDEOGRAM B171" },
    { 0x100B4, "LINEAR B IDEOGRAM B172" },
    { 0x100B5, "LINEAR B IDEOGRAM B173 MONTH" },
    { 0x100B6, "LINEAR B IDEOGRAM B174" },
    { 0x100B7, "LINEAR B IDEOGRAM B176 TREE" },
    { 0x100B8, "LINEAR B IDEOGRAM B177" },
    { 0x100B9, "LINEAR B IDEOGRAM B178" },
    { 0x100BA, "LINEAR B IDEOGRAM B179" },
    { 0x100BB, "LINEAR B IDEOGRAM B180" },
    { 0x100BC, "LINEAR B IDEOGRAM B181" },
    { 0x100BD, "LINEAR B IDEOGRAM B182" },
    { 0x100BE, "LINEAR B IDEOGRAM B183" },
    { 0x100BF, "LINEAR B IDEOGRAM B184" },
    { 0x100C0, "LINEAR B IDEOGRAM B185" },
    { 0x100C1, "LINEAR B IDEOGRAM B189" },
    { 0x100C2, "LINEAR B IDEOGRAM B190" },
    { 0x100C3, "LINEAR B IDEOGRAM B191 HELMET" },
    { 0x100C4, "LINEAR B IDEOGRAM B220 FOOTSTOOL" },
    { 0x100C5, "LINEAR B IDEOGRAM B225 BATHTUB" },
    { 0x100C6, "LINEAR B IDEOGRAM B230 SPEAR" },
    { 0x100C7, "LINEAR B IDEOGRAM B231 ARROW" },
    { 0x100C8, "LINEAR B IDEOGRAM B232" },
    { 0x100C9, "LINEAR B IDEOGRAM B233 SWORD" },
    { 0x100CA, "LINEAR B IDEOGRAM B234" },
    { 0x100CB, "LINEAR B IDEOGRAM B236" },
    { 0x100CC, "LINEAR B IDEOGRAM B240 WHEELED CHARIOT" },
    { 0x100CD, "LINEAR B IDEOGRAM B241 CHARIOT" },
    { 0x100CE, "LINEAR B IDEOGRAM B242 CHARIOT FRAME" },
    { 0x100CF, "LINEAR B IDEOGRAM B243 WHEEL" },
    { 0x100D0, "LINEAR B IDEOGRAM B245" },
    { 0x100D1, "LINEAR B IDEOGRAM B246" },
    { 0x100D2, "LINEAR B MONOGRAM B247 DIPTE" },
    { 0x100D3, "LINEAR B IDEOGRAM B248" },
    { 0x100D4, "LINEAR B IDEOGRAM B249" },
    { 0x100D5, "LINEAR B IDEOGRAM B251" },
    { 0x100D6, "LINEAR B IDEOGRAM B252" },
    { 0x100D7, "LINEAR B IDEOGRAM B253" },
    { 0x100D8, "LINEAR B IDEOGRAM B254 DART" },
    { 0x100D9, "LINEAR B IDEOGRAM B255" },
    { 0x100DA, "LINEAR B IDEOGRAM B256" },
    { 0x100DB, "LINEAR B IDEOGRAM B257" },
    { 0x100DC, "LINEAR B IDEOGRAM B258" },
    { 0x100DD, "LINEAR B IDEOGRAM B259" },
    { 0x100DE, "LINEAR B IDEOGRAM VESSEL B155" },
    { 0x100DF, "LINEAR B IDEOGRAM VESSEL B200" },
    { 0x100E0, "LINEAR B IDEOGRAM VESSEL B201" },
    { 0x100E1, "LINEAR B IDEOGRAM VESSEL B202" },
    { 0x100E2, "LINEAR B IDEOGRAM VESSEL B203" },
    { 0x100E3, "LINEAR B IDEOGRAM VESSEL B204" },
    { 0x100E4, "LINEAR B IDEOGRAM VESSEL B205" },
    { 0x100E5, "LINEAR B IDEOGRAM VESSEL B206" },
    { 0x100E6, "LINEAR B IDEOGRAM VESSEL B207" },
    { 0x100E7, "LINEAR B IDEOGRAM VESSEL B208" },
    { 0x100E8, "LINEAR B IDEOGRAM VESSEL B209" },
    { 0x100E9, "LINEAR B IDEOGRAM VESSEL B210" },
    { 0x100EA, "LINEAR B IDEOGRAM VESSEL B211" },
    { 0x100EB, "LINEAR B IDEOGRAM VESSEL B212" },
    { 0x100EC, "LINEAR B IDEOGRAM VESSEL B213" },
    { 0x100ED, "LINEAR B IDEOGRAM VESSEL B214" },
    { 0x100EE, "LINEAR B IDEOGRAM VESSEL B215" },
    { 0x100EF, "LINEAR B IDEOGRAM VESSEL B216" },
    { 0x100F0, "LINEAR B IDEOGRAM VESSEL B217" },
    { 0x100F1, "LINEAR B IDEOGRAM VESSEL B218" },
    { 0x100F2, "LINEAR B IDEOGRAM VESSEL B219" },
    { 0x100F3, "LINEAR B IDEOGRAM VESSEL B221" },
    { 0x100F4, "LINEAR B IDEOGRAM VESSEL B222" },
    { 0x100F5, "LINEAR B IDEOGRAM VESSEL B226" },
    { 0x100F6, "LINEAR B IDEOGRAM VESSEL B227" },
    { 0x100F7, "LINEAR B IDEOGRAM VESSEL B228" },
    { 0x100F8, "LINEAR B IDEOGRAM VESSEL B229" },
    { 0x100F9, "LINEAR B IDEOGRAM VESSEL B250" },
    { 0x100FA, "LINEAR B IDEOGRAM VESSEL B305" },
    { 0x10100, "AEGEAN WORD SEPARATOR LINE" },
    { 0x10101, "AEGEAN WORD SEPARATOR DOT" },
    { 0x10102, "AEGEAN CHECK MARK" },
    { 0x10107, "AEGEAN NUMBER ONE" },
    { 0x10108, "AEGEAN NUMBER TWO" },
    { 0x10109, "AEGEAN NUMBER THREE" },
    { 0x1010A, "AEGEAN NUMBER FOUR" },
    { 0x1010B, "AEGEAN NUMBER FIVE" },
    { 0x1010C, "AEGEAN NUMBER SIX" },
    { 0x1010D, "AEGEAN NUMBER SEVEN" },
    { 0x1010E, "AEGEAN NUMBER EIGHT" },
    { 0x1010F, "AEGEAN NUMBER NINE" },
    { 0x10110, "AEGEAN NUMBER TEN" },
    { 0x10111, "AEGEAN NUMBER TWENTY" },
    { 0x10112, "AEGEAN NUMBER THIRTY" },
    { 0x10113, "AEGEAN NUMBER FORTY" },
    { 0x10114, "AEGEAN NUMBER FIFTY" },
    { 0x10115, "AEGEAN NUMBER SIXTY" },
    { 0x10116, "AEGEAN NUMBER SEVENTY" },
    { 0x10117, "AEGEAN NUMBER EIGHTY" },
    { 0x10118, "AEGEAN NUMBER NINETY" },
    { 0x10119, "AEGEAN NUMBER ONE HUNDRED" },
    { 0x1011A, "AEGEAN NUMBER TWO HUNDRED" },
    { 0x1011B, "AEGEAN NUMBER THREE HUNDRED" },
    { 0x1011C, "AEGEAN NUMBER FOUR HUNDRED" },
    { 0x1011D, "AEGEAN NUMBER FIVE HUNDRED" },
    { 0x1011E, "AEGEAN NUMBER SIX HUNDRED" },
    { 0x1011F, "AEGEAN NUMBER SEVEN HUNDRED" },
    { 0x10120, "AEGEAN NUMBER EIGHT HUNDRED" },
    { 0x10121, "AEGEAN NUMBER NINE HUNDRED" },
    { 0x10122, "AEGEAN NUMBER ONE THOUSAND" },
    { 0x10123, "AEGEAN NUMBER TWO THOUSAND" },
    { 0x10124, "AEGEAN NUMBER THREE THOUSAND" },
    { 0x10125, "AEGEAN NUMBER FOUR THOUSAND" },
    { 0x10126, "AEGEAN NUMBER FIVE THOUSAND" },
    { 0x10127, "AEGEAN NUMBER SIX THOUSAND" },
    { 0x10128, "AEGEAN NUMBER SEVEN THOUSAND" },
    { 0x10129, "AEGEAN NUMBER EIGHT THOUSAND" },
    { 0x1012A, "AEGEAN NUMBER NINE THOUSAND" },
    { 0x1012B, "AEGEAN NUMBER TEN THOUSAND" },
    { 0x1012C, "AEGEAN NUMBER TWENTY THOUSAND" },
    { 0x1012D, "AEGEAN NUMBER THIRTY THOUSAND" },
    { 0x1012E, "AEGEAN NUMBER FORTY THOUSAND" },
    { 0x1012F, "AEGEAN NUMBER FIFTY THOUSAND" },
    { 0x10130, "AEGEAN NUMBER SIXTY THOUSAND" },
    { 0x10131, "AEGEAN NUMBER SEVENTY THOUSAND" },
    { 0x10132, "AEGEAN NUMBER EIGHTY THOUSAND" },
    { 0x10133, "AEGEAN NUMBER NINETY THOUSAND" },
    { 0x10137, "AEGEAN WEIGHT BASE UNIT" },
    { 0x10138, "AEGEAN WEIGHT FIRST SUBUNIT" },
    { 0x10139, "AEGEAN WEIGHT SECOND SUBUNIT" },
    { 0x1013A, "AEGEAN WEIGHT THIRD SUBUNIT" },
    { 0x1013B, "AEGEAN WEIGHT FOURTH SUBUNIT" },
    { 0x1013C, "AEGEAN DRY MEASURE FIRST SUBUNIT" },
    { 0x1013D, "AEGEAN LIQUID MEASURE FIRST SUBUNIT" },
    { 0x1013E, "AEGEAN MEASURE SECOND SUBUNIT" },
    { 0x1013F, "AEGEAN MEASURE THIRD SUBUNIT" },
    { 0x10140, "GREEK ACROPHONIC ATTIC ONE QUARTER" },
    { 0x10141, "GREEK ACROPHONIC ATTIC ONE HALF" },
    { 0x10142, "GREEK ACROPHONIC ATTIC ONE DRACHMA" },
    { 0x10143, "GREEK ACROPHONIC ATTIC FIVE" },
    { 0x10144, "GREEK ACROPHONIC ATTIC FIFTY" },
    { 0x10145, "GREEK ACROPHONIC ATTIC FIVE HUNDRED" },
    { 0x10146, "GREEK ACROPHONIC ATTIC FIVE THOUSAND" },
    { 0x10147, "GREEK ACROPHONIC ATTIC FIFTY THOUSAND" },
    { 0x10148, "GREEK ACROPHONIC ATTIC FIVE TALENTS" },
    { 0x10149, "GREEK ACROPHONIC ATTIC TEN TALENTS" },
    { 0x1014A, "GREEK ACROPHONIC ATTIC FIFTY TALENTS" },
    { 0x1014B, "GREEK ACROPHONIC ATTIC ONE HUNDRED TALENTS" },
    { 0x1014C, "GREEK ACROPHONIC ATTIC FIVE HUNDRED TALENTS" },
    { 0x1014D, "GREEK ACROPHONIC ATTIC ONE THOUSAND TALENTS" },
    { 0x1014E, "GREEK ACROPHONIC ATTIC FIVE THOUSAND TALENTS" },
    { 0x1014F, "GREEK ACROPHONIC ATTIC FIVE STATERS" },
    { 0x10150, "GREEK ACROPHONIC ATTIC TEN STATERS" },
    { 0x10151, "GREEK ACROPHONIC ATTIC FIFTY STATERS" },
    { 0x10152, "GREEK ACROPHONIC ATTIC ONE HUNDRED STATERS" },
    { 0x10153, "GREEK ACROPHONIC ATTIC FIVE HUNDRED STATERS" },
    { 0x10154, "GREEK ACROPHONIC ATTIC ONE THOUSAND STATERS" },
    { 0x10155, "GREEK ACROPHONIC ATTIC TEN THOUSAND STATERS" },
    { 0x10156, "GREEK ACROPHONIC ATTIC FIFTY THOUSAND STATERS" },
    { 0x10157, "GREEK ACROPHONIC ATTIC TEN MNAS" },
    { 0x10158, "GREEK ACROPHONIC HERAEUM ONE PLETHRON" },
    { 0x10159, "GREEK ACROPHONIC THESPIAN ONE" },
    { 0x1015A, "GREEK ACROPHONIC HERMIONIAN ONE" },
    { 0x1015B, "GREEK ACROPHONIC EPIDAUREAN TWO" },
    { 0x1015C, "GREEK ACROPHONIC THESPIAN TWO" },
    { 0x1015D, "GREEK ACROPHONIC CYRENAIC TWO DRACHMAS" },
    { 0x1015E, "GREEK ACROPHONIC EPIDAUREAN TWO DRACHMAS" },
    { 0x1015F, "GREEK ACROPHONIC TROEZENIAN FIVE" },
    { 0x10160, "GREEK ACROPHONIC TROEZENIAN TEN" },
    { 0x10161, "GREEK ACROPHONIC TROEZENIAN TEN ALTERNATE FORM" },
    { 0x10162, "GREEK ACROPHONIC HERMIONIAN TEN" },
    { 0x10163, "GREEK ACROPHONIC MESSENIAN TEN" },
    { 0x10164, "GREEK ACROPHONIC THESPIAN TEN" },
    { 0x10165, "GREEK ACROPHONIC THESPIAN THIRTY" },
    { 0x10166, "GREEK ACROPHONIC TROEZENIAN FIFTY" },
    { 0x10167, "GREEK ACROPHONIC TROEZENIAN FIFTY ALTERNATE FORM" },
    { 0x10168, "GREEK ACROPHONIC HERMIONIAN FIFTY" },
    { 0x10169, "GREEK ACROPHONIC THESPIAN FIFTY" },
    { 0x1016A, "GREEK ACROPHONIC THESPIAN ONE HUNDRED" },
    { 0x1016B, "GREEK ACROPHONIC THESPIAN THREE HUNDRED" },
    { 0x1016C, "GREEK ACROPHONIC EPIDAUREAN FIVE HUNDRED" },
    { 0x1016D, "GREEK ACROPHONIC TROEZENIAN FIVE HUNDRED" },
    { 0x1016E, "GREEK ACROPHONIC THESPIAN FIVE HUNDRED" },
    { 0x1016F, "GREEK ACROPHONIC CARYSTIAN FIVE HUNDRED" },
    { 0x10170, "GREEK ACROPHONIC NAXIAN FIVE HUNDRED" },
    { 0x10171, "GREEK ACROPHONIC THESPIAN ONE THOUSAND" },
    { 0x10172, "GREEK ACROPHONIC THESPIAN FIVE THOUSAND" },
    { 0x10173, "GREEK ACROPHONIC DELPHIC FIVE MNAS" },
    { 0x10174, "GREEK ACROPHONIC STRATIAN FIFTY MNAS" },
    { 0x10175, "GREEK ONE HALF SIGN" },
    { 0x10176, "GREEK ONE HALF SIGN ALTERNATE FORM" },
    { 0x10177, "GREEK TWO THIRDS SIGN" },
    { 0x10178, "GREEK THREE QUARTERS SIGN" },
    { 0x10179, "GREEK YEAR SIGN" },
    { 0x1017A, "GREEK TALENT SIGN" },
    { 0x1017B, "GREEK DRACHMA SIGN" },
    { 0x1017C, "GREEK OBOL SIGN" },
    { 0x1017D, "GREEK TWO OBOLS SIGN" },
    { 0x1017E, "GREEK THREE OBOLS SIGN" },
    { 0x1017F, "GREEK FOUR OBOLS SIGN" },
    { 0x10180, "GREEK FIVE OBOLS SIGN" },
    { 0x10181, "GREEK METRETES SIGN" },
    { 0x10182, "GREEK KYATHOS BASE SIGN" },
    { 0x10183, "GREEK LITRA SIGN" },
    { 0x10184, "GREEK OUNKIA SIGN" },
    { 0x10185, "GREEK XESTES SIGN" },
    { 0x10186, "GREEK ARTABE SIGN" },
    { 0x10187, "GREEK AROURA SIGN" },
    { 0x10188, "GREEK GRAMMA SIGN" },
    { 0x10189, "GREEK TRYBLION BASE SIGN" },
    { 0x1018A, "GREEK ZERO SIGN" },
    { 0x1018B, "GREEK ONE QUARTER SIGN" },
    { 0x1018C, "GREEK SINUSOID SIGN" },
    { 0x1018D, "GREEK INDICTION SIGN" },
    { 0x1018E, "NOMISMA SIGN" },
    { 0x10190, "ROMAN SEXTANS SIGN" },
    { 0x10191, "ROMAN UNCIA SIGN" },
    { 0x10192, "ROMAN SEMUNCIA SIGN" },
    { 0x10193, "ROMAN SEXTULA SIGN" },
    { 0x10194, "ROMAN DIMIDIA SEXTULA SIGN" },
    { 0x10195, "ROMAN SILIQUA SIGN" },
    { 0x10196, "ROMAN DENARIUS SIGN" },
    { 0x10197, "ROMAN QUINARIUS SIGN" },
    { 0x10198, "ROMAN SESTERTIUS SIGN" },
    { 0x10199, "ROMAN DUPONDIUS SIGN" },
    { 0x1019A, "ROMAN AS SIGN" },
    { 0x1019B, "ROMAN CENTURIAL SIGN" },
    { 0x1019C, "ASCIA SYMBOL" },
    { 0x101A0, "GREEK SYMBOL TAU RHO" },
    { 0x101D0, "PHAISTOS DISC SIGN PEDESTRIAN" },
    { 0x101D1, "PHAISTOS DISC SIGN PLUMED HEAD" },
    { 0x101D2, "PHAISTOS DISC SIGN TATTOOED HEAD" },
    { 0x101D3, "PHAISTOS DISC SIGN CAPTIVE" },
    { 0x101D4, "PHAISTOS DISC SIGN CHILD" },
    { 0x101D5, "PHAISTOS DISC SIGN WOMAN" },
    { 0x101D6, "PHAISTOS DISC SIGN HELMET" },
    { 0x101D7, "PHAISTOS DISC SIGN GAUNTLET" },
    { 0x101D8, "PHAISTOS DISC SIGN TIARA" },
    { 0x101D9, "PHAISTOS DISC SIGN ARROW" },
    { 0x101DA, "PHAISTOS DISC SIGN BOW" },
    { 0x101DB, "PHAISTOS DISC SIGN SHIELD" },
    { 0x101DC, "PHAISTOS DISC SIGN CLUB" },
    { 0x101DD, "PHAISTOS DISC SIGN MANACLES" },
    { 0x101DE, "PHAISTOS DISC SIGN MATTOCK" },
    { 0x101DF, "PHAISTOS DISC SIGN SAW" },
    { 0x101E0, "PHAISTOS DISC SIGN LID" },
    { 0x101E1, "PHAISTOS DISC SIGN BOOMERANG" },
    { 0x101E2, "PHAISTOS DISC SIGN CARPENTRY PLANE" },
    { 0x101E3, "PHAISTOS DISC SIGN DOLIUM" },
    { 0x101E4, "PHAISTOS DISC SIGN COMB" },
    { 0x101E5, "PHAISTOS DISC SIGN SLING" },
    { 0x101E6, "PHAISTOS DISC SIGN COLUMN" },
    { 0x101E7, "PHAISTOS DISC SIGN BEEHIVE" },
    { 0x101E8, "PHAISTOS DISC SIGN SHIP" },
    { 0x101E9, "PHAISTOS DISC SIGN HORN" },
    { 0x101EA, "PHAISTOS DISC SIGN HIDE" },
    { 0x101EB, "PHAISTOS DISC SIGN BULLS LEG" },
    { 0x101EC, "PHAISTOS DISC SIGN CAT" },
    { 0x101ED, "PHAISTOS DISC SIGN RAM" },
    { 0x101EE, "PHAISTOS DISC SIGN EAGLE" },
    { 0x101EF, "PHAISTOS DISC SIGN DOVE" },
    { 0x101F0, "PHAISTOS DISC SIGN TUNNY" },
    { 0x101F1, "PHAISTOS DISC SIGN BEE" },
    { 0x101F2, "PHAISTOS DISC SIGN PLANE TREE" },
    { 0x101F3, "PHAISTOS DISC SIGN VINE" },
    { 0x101F4, "PHAISTOS DISC SIGN PAPYRUS" },
    { 0x101F5, "PHAISTOS DISC SIGN ROSETTE" },
    { 0x101F6, "PHAISTOS DISC SIGN LILY" },
    { 0x101F7, "PHAISTOS DISC SIGN OX BACK" },
    { 0x101F8, "PHAISTOS DISC SIGN FLUTE" },
    { 0x101F9, "PHAISTOS DISC SIGN GRATER" },
    { 0x101FA, "PHAISTOS DISC SIGN STRAINER" },
    { 0x101FB, "PHAISTOS DISC SIGN SMALL AXE" },
    { 0x101FC, "PHAISTOS DISC SIGN WAVY BAND" },
    { 0x101FD, "PHAISTOS DISC SIGN COMBINING OBLIQUE STROKE" },
    { 0x10280, "LYCIAN LETTER A" },
    { 0x10281, "LYCIAN LETTER E" },
    { 0x10282, "LYCIAN LETTER B" },
    { 0x10283, "LYCIAN LETTER BH" },
    { 0x10284, "LYCIAN LETTER G" },
    { 0x10285, "LYCIAN LETTER D" },
    { 0x10286, "LYCIAN LETTER I" },
    { 0x10287, "LYCIAN LETTER W" },
    { 0x10288, "LYCIAN LETTER Z" },
    { 0x10289, "LYCIAN LETTER TH" },
    { 0x1028A, "LYCIAN LETTER J" },
    { 0x1028B, "LYCIAN LETTER K" },
    { 0x1028C, "LYCIAN LETTER Q" },
    { 0x1028D, "LYCIAN LETTER L" },
    { 0x1028E, "LYCIAN LETTER M" },
    { 0x1028F, "LYCIAN LETTER N" },
    { 0x10290, "LYCIAN LETTER MM" },
    { 0x10291, "LYCIAN LETTER NN" },
    { 0x10292, "LYCIAN LETTER U" },
    { 0x10293, "LYCIAN LETTER P" },
    { 0x10294, "LYCIAN LETTER KK" },
    { 0x10295, "LYCIAN LETTER R" },
    { 0x10296, "LYCIAN LETTER S" },
    { 0x10297, "LYCIAN LETTER T" },
    { 0x10298, "LYCIAN LETTER TT" },
    { 0x10299, "LYCIAN LETTER AN" },
    { 0x1029A, "LYCIAN LETTER EN" },
    { 0x1029B, "LYCIAN LETTER H" },
    { 0x1029C, "LYCIAN LETTER X" },
    { 0x102A0, "CARIAN LETTER A" },
    { 0x102A1, "CARIAN LETTER P2" },
    { 0x102A2, "CARIAN LETTER D" },
    { 0x102A3, "CARIAN LETTER L" },
    { 0x102A4, "CARIAN LETTER UUU" },
    { 0x102A5, "CARIAN LETTER R" },
    { 0x102A6, "CARIAN LETTER LD" },
    { 0x102A7, "CARIAN LETTER A2" },
    { 0x102A8, "CARIAN LETTER Q" },
    { 0x102A9, "CARIAN LETTER B" },
    { 0x102AA, "CARIAN LETTER M" },
    { 0x102AB, "CARIAN LETTER O" },
    { 0x102AC, "CARIAN LETTER D2" },
    { 0x102AD, "CARIAN LETTER T" },
    { 0x102AE, "CARIAN LETTER SH" },
    { 0x102AF, "CARIAN LETTER SH2" },
    { 0x102B0, "CARIAN LETTER S" },
    { 0x102B1, "CARIAN LETTER C-18" },
    { 0x102B2, "CARIAN LETTER U" },
    { 0x102B3, "CARIAN LETTER NN" },
    { 0x102B4, "CARIAN LETTER X" },
    { 0x102B5, "CARIAN LETTER N" },
    { 0x102B6, "CARIAN LETTER TT2" },
    { 0x102B7, "CARIAN LETTER P" },
    { 0x102B8, "CARIAN LETTER SS" },
    { 0x102B9, "CARIAN LETTER I" },
    { 0x102BA, "CARIAN LETTER E" },
    { 0x102BB, "CARIAN LETTER UUUU" },
    { 0x102BC, "CARIAN LETTER K" },
    { 0x102BD, "CARIAN LETTER K2" },
    { 0x102BE, "CARIAN LETTER ND" },
    { 0x102BF, "CARIAN LETTER UU" },
    { 0x102C0, "CARIAN LETTER G" },
    { 0x102C1, "CARIAN LETTER G2" },
    { 0x102C2, "CARIAN LETTER ST" },
    { 0x102C3, "CARIAN LETTER ST2" },
    { 0x102C4, "CARIAN LETTER NG" },
    { 0x102C5, "CARIAN LETTER II" },
    { 0x102C6, "CARIAN LETTER C-39" },
    { 0x102C7, "CARIAN LETTER TT" },
    { 0x102C8, "CARIAN LETTER UUU2" },
    { 0x102C9, "CARIAN LETTER RR" },
    { 0x102CA, "CARIAN LETTER MB" },
    { 0x102CB, "CARIAN LETTER MB2" },
    { 0x102CC, "CARIAN LETTER MB3" },
    { 0x102CD, "CARIAN LETTER MB4" },
    { 0x102CE, "CARIAN LETTER LD2" },
    { 0x102CF, "CARIAN LETTER E2" },
    { 0x102D0, "CARIAN LETTER UUU3" },
    { 0x102E0, "COPTIC EPACT THOUSANDS MARK" },
    { 0x102E1, "COPTIC EPACT DIGIT ONE" },
    { 0x102E2, "COPTIC EPACT DIGIT TWO" },
    { 0x102E3, "COPTIC EPACT DIGIT THREE" },
    { 0x102E4, "COPTIC EPACT DIGIT FOUR" },
    { 0x102E5, "COPTIC EPACT DIGIT FIVE" },
    { 0x102E6, "COPTIC EPACT DIGIT SIX" },
    { 0x102E7, "COPTIC EPACT DIGIT SEVEN" },
    { 0x102E8, "COPTIC EPACT DIGIT EIGHT" },
    { 0x102E9, "COPTIC EPACT DIGIT NINE" },
    { 0x102EA, "COPTIC EPACT NUMBER TEN" },
    { 0x102EB, "COPTIC EPACT NUMBER TWENTY" },
    { 0x102EC, "COPTIC EPACT NUMBER THIRTY" },
    { 0x102ED, "COPTIC EPACT NUMBER FORTY" },
    { 0x102EE, "COPTIC EPACT NUMBER FIFTY" },
    { 0x102EF, "COPTIC EPACT NUMBER SIXTY" },
    { 0x102F0, "COPTIC EPACT NUMBER SEVENTY" },
    { 0x102F1, "COPTIC EPACT NUMBER EIGHTY" },
    { 0x102F2, "COPTIC EPACT NUMBER NINETY" },
    { 0x102F3, "COPTIC EPACT NUMBER ONE HUNDRED" },
    { 0x102F4, "COPTIC EPACT NUMBER TWO HUNDRED" },
    { 0x102F5, "COPTIC EPACT NUMBER THREE HUNDRED" },
    { 0x102F6, "COPTIC EPACT NUMBER FOUR HUNDRED" },
    { 0x102F7, "COPTIC EPACT NUMBER FIVE HUNDRED" },
    { 0x102F8, "COPTIC EPACT NUMBER SIX HUNDRED" },
    { 0x102F9, "COPTIC EPACT NUMBER SEVEN HUNDRED" },
    { 0x102FA, "COPTIC EPACT NUMBER EIGHT HUNDRED" },
    { 0x102FB, "COPTIC EPACT NUMBER NINE HUNDRED" },
    { 0x10300, "OLD ITALIC LETTER A" },
    { 0x10301, "OLD ITALIC LETTER BE" },
    { 0x10302, "OLD ITALIC LETTER KE" },
    { 0x10303, "OLD ITALIC LETTER DE" },
    { 0x10304, "OLD ITALIC LETTER E" },
    { 0x10305, "OLD ITALIC LETTER VE" },
    { 0x10306, "OLD ITALIC LETTER ZE" },
    { 0x10307, "OLD ITALIC LETTER HE" },
    { 0x10308, "OLD ITALIC LETTER THE" },
    { 0x10309, "OLD ITALIC LETTER I" },
    { 0x1030A, "OLD ITALIC LETTER KA" },
    { 0x1030B, "OLD ITALIC LETTER EL" },
    { 0x1030C, "OLD ITALIC LETTER EM" },
    { 0x1030D, "OLD ITALIC LETTER EN" },
    { 0x1030E, "OLD ITALIC LETTER ESH" },
    { 0x1030F, "OLD ITALIC LETTER O" },
    { 0x10310, "OLD ITALIC LETTER PE" },
    { 0x10311, "OLD ITALIC LETTER SHE" },
    { 0x10312, "OLD ITALIC LETTER KU" },
    { 0x10313, "OLD ITALIC LETTER ER" },
    { 0x10314, "OLD ITALIC LETTER ES" },
    { 0x10315, "OLD ITALIC LETTER TE" },
    { 0x10316, "OLD ITALIC LETTER U" },
    { 0x10317, "OLD ITALIC LETTER EKS" },
    { 0x10318, "OLD ITALIC LETTER PHE" },
    { 0x10319, "OLD ITALIC LETTER KHE" },
    { 0x1031A, "OLD ITALIC LETTER EF" },
    { 0x1031B, "OLD ITALIC LETTER ERS" },
    { 0x1031C, "OLD ITALIC LETTER CHE" },
    { 0x1031D, "OLD ITALIC LETTER II" },
    { 0x1031E, "OLD ITALIC LETTER UU" },
    { 0x1031F, "OLD ITALIC LETTER ESS" },
    { 0x10320, "OLD ITALIC NUMERAL ONE" },
    { 0x10321, "OLD ITALIC NUMERAL FIVE" },
    { 0x10322, "OLD ITALIC NUMERAL TEN" },
    { 0x10323, "OLD ITALIC NUMERAL FIFTY" },
    { 0x1032D, "OLD ITALIC LETTER YE" },
    { 0x1032E, "OLD ITALIC LETTER NORTHERN TSE" },
    { 0x1032F, "OLD ITALIC LETTER SOUTHERN TSE" },
    { 0x10330, "GOTHIC LETTER AHSA" },
    { 0x10331, "GOTHIC LETTER BAIRKAN" },
    { 0x10332, "GOTHIC LETTER GIBA" },
    { 0x10333, "GOTHIC LETTER DAGS" },
    { 0x10334, "GOTHIC LETTER AIHVUS" },
    { 0x10335, "GOTHIC LETTER QAIRTHRA" },
    { 0x10336, "GOTHIC LETTER IUJA" },
    { 0x10337, "GOTHIC LETTER HAGL" },
    { 0x10338, "GOTHIC LETTER THIUTH" },
    { 0x10339, "GOTHIC LETTER EIS" },
    { 0x1033A, "GOTHIC LETTER KUSMA" },
    { 0x1033B, "GOTHIC LETTER LAGUS" },
    { 0x1033C, "GOTHIC LETTER MANNA" },
    { 0x1033D, "GOTHIC LETTER NAUTHS" },
    { 0x1033E, "GOTHIC LETTER JER" },
    { 0x1033F, "GOTHIC LETTER URUS" },
    { 0x10340, "GOTHIC LETTER PAIRTHRA" },
    { 0x10341, "GOTHIC LETTER NINETY" },
    { 0x10342, "GOTHIC LETTER RAIDA" },
    { 0x10343, "GOTHIC LETTER SAUIL" },
    { 0x10344, "GOTHIC LETTER TEIWS" },
    { 0x10345, "GOTHIC LETTER WINJA" },
    { 0x10346, "GOTHIC LETTER FAIHU" },
    { 0x10347, "GOTHIC LETTER IGGWS" },
    { 0x10348, "GOTHIC LETTER HWAIR" },
    { 0x10349, "GOTHIC LETTER OTHAL" },
    { 0x1034A, "GOTHIC LETTER NINE HUNDRED" },
    { 0x10350, "OLD PERMIC LETTER AN" },
    { 0x10351, "OLD PERMIC LETTER BUR" },
    { 0x10352, "OLD PERMIC LETTER GAI" },
    { 0x10353, "OLD PERMIC LETTER DOI" },
    { 0x10354, "OLD PERMIC LETTER E" },
    { 0x10355, "OLD PERMIC LETTER ZHOI" },
    { 0x10356, "OLD PERMIC LETTER DZHOI" },
    { 0x10357, "OLD PERMIC LETTER ZATA" },
    { 0x10358, "OLD PERMIC LETTER DZITA" },
    { 0x10359, "OLD PERMIC LETTER I" },
    { 0x1035A, "OLD PERMIC LETTER KOKE" },
    { 0x1035B, "OLD PERMIC LETTER LEI" },
    { 0x1035C, "OLD PERMIC LETTER MENOE" },
    { 0x1035D, "OLD PERMIC LETTER NENOE" },
    { 0x1035E, "OLD PERMIC LETTER VOOI" },
    { 0x1035F, "OLD PERMIC LETTER PEEI" },
    { 0x10360, "OLD PERMIC LETTER REI" },
    { 0x10361, "OLD PERMIC LETTER SII" },
    { 0x10362, "OLD PERMIC LETTER TAI" },
    { 0x10363, "OLD PERMIC LETTER U" },
    { 0x10364, "OLD PERMIC LETTER CHERY" },
    { 0x10365, "OLD PERMIC LETTER SHOOI" },
    { 0x10366, "OLD PERMIC LETTER SHCHOOI" },
    { 0x10367, "OLD PERMIC LETTER YRY" },
    { 0x10368, "OLD PERMIC LETTER YERU" },
    { 0x10369, "OLD PERMIC LETTER O" },
    { 0x1036A, "OLD PERMIC LETTER OO" },
    { 0x1036B, "OLD PERMIC LETTER EF" },
    { 0x1036C, "OLD PERMIC LETTER HA" },
    { 0x1036D, "OLD PERMIC LETTER TSIU" },
    { 0x1036E, "OLD PERMIC LETTER VER" },
    { 0x1036F, "OLD PERMIC LETTER YER" },
    { 0x10370, "OLD PERMIC LETTER YERI" },
    { 0x10371, "OLD PERMIC LETTER YAT" },
    { 0x10372, "OLD PERMIC LETTER IE" },
    { 0x10373, "OLD PERMIC LETTER YU" },
    { 0x10374, "OLD PERMIC LETTER YA" },
    { 0x10375, "OLD PERMIC LETTER IA" },
    { 0x10376, "COMBINING OLD PERMIC LETTER AN" },
    { 0x10377, "COMBINING OLD PERMIC LETTER DOI" },
    { 0x10378, "COMBINING OLD PERMIC LETTER ZATA" },
    { 0x10379, "COMBINING OLD PERMIC LETTER NENOE" },
    { 0x1037A, "COMBINING OLD PERMIC LETTER SII" },
    { 0x10380, "UGARITIC LETTER ALPA" },
    { 0x10381, "UGARITIC LETTER BETA" },
    { 0x10382, "UGARITIC LETTER GAMLA" },
    { 0x10383, "UGARITIC LETTER KHA" },
    { 0x10384, "UGARITIC LETTER DELTA" },
    { 0x10385, "UGARITIC LETTER HO" },
    { 0x10386, "UGARITIC LETTER WO" },
    { 0x10387, "UGARITIC LETTER ZETA" },
    { 0x10388, "UGARITIC LETTER HOTA" },
    { 0x10389, "UGARITIC LETTER TET" },
    { 0x1038A, "UGARITIC LETTER YOD" },
    { 0x1038B, "UGARITIC LETTER KAF" },
    { 0x1038C, "UGARITIC LETTER SHIN" },
    { 0x1038D, "UGARITIC LETTER LAMDA" },
    { 0x1038E, "UGARITIC LETTER MEM" },
    { 0x1038F, "UGARITIC LETTER DHAL" },
    { 0x10390, "UGARITIC LETTER NUN" },
    { 0x10391, "UGARITIC LETTER ZU" },
    { 0x10392, "UGARITIC LETTER SAMKA" },
    { 0x10393, "UGARITIC LETTER AIN" },
    { 0x10394, "UGARITIC LETTER PU" },
    { 0x10395, "UGARITIC LETTER SADE" },
    { 0x10396, "UGARITIC LETTER QOPA" },
    { 0x10397, "UGARITIC LETTER RASHA" },
    { 0x10398, "UGARITIC LETTER THANNA" },
    { 0x10399, "UGARITIC LETTER GHAIN" },
    { 0x1039A, "UGARITIC LETTER TO" },
    { 0x1039B, "UGARITIC LETTER I" },
    { 0x1039C, "UGARITIC LETTER U" },
    { 0x1039D, "UGARITIC LETTER SSU" },
    { 0x1039F, "UGARITIC WORD DIVIDER" },
    { 0x103A0, "OLD PERSIAN SIGN A" },
    { 0x103A1, "OLD PERSIAN SIGN I" },
    { 0x103A2, "OLD PERSIAN SIGN U" },
    { 0x103A3, "OLD PERSIAN SIGN KA" },
    { 0x103A4, "OLD PERSIAN SIGN KU" },
    { 0x103A5, "OLD PERSIAN SIGN GA" },
    { 0x103A6, "OLD PERSIAN SIGN GU" },
    { 0x103A7, "OLD PERSIAN SIGN XA" },
    { 0x103A8, "OLD PERSIAN SIGN CA" },
    { 0x103A9, "OLD PERSIAN SIGN JA" },
    { 0x103AA, "OLD PERSIAN SIGN JI" },
    { 0x103AB, "OLD PERSIAN SIGN TA" },
    { 0x103AC, "OLD PERSIAN SIGN TU" },
    { 0x103AD, "OLD PERSIAN SIGN DA" },
    { 0x103AE, "OLD PERSIAN SIGN DI" },
    { 0x103AF, "OLD PERSIAN SIGN DU" },
    { 0x103B0, "OLD PERSIAN SIGN THA" },
    { 0x103B1, "OLD PERSIAN SIGN PA" },
    { 0x103B2, "OLD PERSIAN SIGN BA" },
    { 0x103B3, "OLD PERSIAN SIGN FA" },
    { 0x103B4, "OLD PERSIAN SIGN NA" },
    { 0x103B5, "OLD PERSIAN SIGN NU" },
    { 0x103B6, "OLD PERSIAN SIGN MA" },
    { 0x103B7, "OLD PERSIAN SIGN MI" },
    { 0x103B8, "OLD PERSIAN SIGN MU" },
    { 0x103B9, "OLD PERSIAN SIGN YA" },
    { 0x103BA, "OLD PERSIAN SIGN VA" },
    { 0x103BB, "OLD PERSIAN SIGN VI" },
    { 0x103BC, "OLD PERSIAN SIGN RA" },
    { 0x103BD, "OLD PERSIAN SIGN RU" },
    { 0x103BE, "OLD PERSIAN SIGN LA" },
    { 0x103BF, "OLD PERSIAN SIGN SA" },
    { 0x103C0, "OLD PERSIAN SIGN ZA" },
    { 0x103C1, "OLD PERSIAN SIGN SHA" },
    { 0x103C2, "OLD PERSIAN SIGN SSA" },
    { 0x103C3, "OLD PERSIAN SIGN HA" },
    { 0x103C8, "OLD PERSIAN SIGN AURAMAZDAA" },
    { 0x103C9, "OLD PERSIAN SIGN AURAMAZDAA-2" },
    { 0x103CA, "OLD PERSIAN SIGN AURAMAZDAAHA" },
    { 0x103CB, "OLD PERSIAN SIGN XSHAAYATHIYA" },
    { 0x103CC, "OLD PERSIAN SIGN DAHYAAUSH" },
    { 0x103CD, "OLD PERSIAN SIGN DAHYAAUSH-2" },
    { 0x103CE, "OLD PERSIAN SIGN BAGA" },
    { 0x103CF, "OLD PERSIAN SIGN BUUMISH" },
    { 0x103D0, "OLD PERSIAN WORD DIVIDER" },
    { 0x103D1, "OLD PERSIAN NUMBER ONE" },
    { 0x103D2, "OLD PERSIAN NUMBER TWO" },
    { 0x103D3, "OLD PERSIAN NUMBER TEN" },
    { 0x103D4, "OLD PERSIAN NUMBER TWENTY" },
    { 0x103D5, "OLD PERSIAN NUMBER HUNDRED" },
    { 0x10400, "DESERET CAPITAL LETTER LONG I" },
    { 0x10401, "DESERET CAPITAL LETTER LONG E" },
    { 0x10402, "DESERET CAPITAL LETTER LONG A" },
    { 0x10403, "DESERET CAPITAL LETTER LONG AH" },
    { 0x10404, "DESERET CAPITAL LETTER LONG O" },
    { 0x10405, "DESERET CAPITAL LETTER LONG OO" },
    { 0x10406, "DESERET CAPITAL LETTER SHORT I" },
    { 0x10407, "DESERET CAPITAL LETTER SHORT E" },
    { 0x10408, "DESERET CAPITAL LETTER SHORT A" },
    { 0x10409, "DESERET CAPITAL LETTER SHORT AH" },
    { 0x1040A, "DESERET CAPITAL LETTER SHORT O" },
    { 0x1040B, "DESERET CAPITAL LETTER SHORT OO" },
    { 0x1040C, "DESERET CAPITAL LETTER AY" },
    { 0x1040D, "DESERET CAPITAL LETTER OW" },
    { 0x1040E, "DESERET CAPITAL LETTER WU" },
    { 0x1040F, "DESERET CAPITAL LETTER YEE" },
    { 0x10410, "DESERET CAPITAL LETTER H" },
    { 0x10411, "DESERET CAPITAL LETTER PEE" },
    { 0x10412, "DESERET CAPITAL LETTER BEE" },
    { 0x10413, "DESERET CAPITAL LETTER TEE" },
    { 0x10414, "DESERET CAPITAL LETTER DEE" },
    { 0x10415, "DESERET CAPITAL LETTER CHEE" },
    { 0x10416, "DESERET CAPITAL LETTER JEE" },
    { 0x10417, "DESERET CAPITAL LETTER KAY" },
    { 0x10418, "DESERET CAPITAL LETTER GAY" },
    { 0x10419, "DESERET CAPITAL LETTER EF" },
    { 0x1041A, "DESERET CAPITAL LETTER VEE" },
    { 0x1041B, "DESERET CAPITAL LETTER ETH" },
    { 0x1041C, "DESERET CAPITAL LETTER THEE" },
    { 0x1041D, "DESERET CAPITAL LETTER ES" },
    { 0x1041E, "DESERET CAPITAL LETTER ZEE" },
    { 0x1041F, "DESERET CAPITAL LETTER ESH" },
    { 0x10420, "DESERET CAPITAL LETTER ZHEE" },
    { 0x10421, "DESERET CAPITAL LETTER ER" },
    { 0x10422, "DESERET CAPITAL LETTER EL" },
    { 0x10423, "DESERET CAPITAL LETTER EM" },
    { 0x10424, "DESERET CAPITAL LETTER EN" },
    { 0x10425, "DESERET CAPITAL LETTER ENG" },
    { 0x10426, "DESERET CAPITAL LETTER OI" },
    { 0x10427, "DESERET CAPITAL LETTER EW" },
    { 0x10428, "DESERET SMALL LETTER LONG I" },
    { 0x10429, "DESERET SMALL LETTER LONG E" },
    { 0x1042A, "DESERET SMALL LETTER LONG A" },
    { 0x1042B, "DESERET SMALL LETTER LONG AH" },
    { 0x1042C, "DESERET SMALL LETTER LONG O" },
    { 0x1042D, "DESERET SMALL LETTER LONG OO" },
    { 0x1042E, "DESERET SMALL LETTER SHORT I" },
    { 0x1042F, "DESERET SMALL LETTER SHORT E" },
    { 0x10430, "DESERET SMALL LETTER SHORT A" },
    { 0x10431, "DESERET SMALL LETTER SHORT AH" },
    { 0x10432, "DESERET SMALL LETTER SHORT O" },
    { 0x10433, "DESERET SMALL LETTER SHORT OO" },
    { 0x10434, "DESERET SMALL LETTER AY" },
    { 0x10435, "DESERET SMALL LETTER OW" },
    { 0x10436, "DESERET SMALL LETTER WU" },
    { 0x10437, "DESERET SMALL LETTER YEE" },
    { 0x10438, "DESERET SMALL LETTER H" },
    { 0x10439, "DESERET SMALL LETTER PEE" },
    { 0x1043A, "DESERET SMALL LETTER BEE" },
    { 0x1043B, "DESERET SMALL LETTER TEE" },
    { 0x1043C, "DESERET SMALL LETTER DEE" },
    { 0x1043D, "DESERET SMALL LETTER CHEE" },
    { 0x1043E, "DESERET SMALL LETTER JEE" },
    { 0x1043F, "DESERET SMALL LETTER KAY" },
    { 0x10440, "DESERET SMALL LETTER GAY" },
    { 0x10441, "DESERET SMALL LETTER EF" },
    { 0x10442, "DESERET SMALL LETTER VEE" },
    { 0x10443, "DESERET SMALL LETTER ETH" },
    { 0x10444, "DESERET SMALL LETTER THEE" },
    { 0x10445, "DESERET SMALL LETTER ES" },
    { 0x10446, "DESERET SMALL LETTER ZEE" },
    { 0x10447, "DESERET SMALL LETTER ESH" },
    { 0x10448, "DESERET SMALL LETTER ZHEE" },
    { 0x10449, "DESERET SMALL LETTER ER" },
    { 0x1044A, "DESERET SMALL LETTER EL" },
    { 0x1044B, "DESERET SMALL LETTER EM" },
    { 0x1044C, "DESERET SMALL LETTER EN" },
    { 0x1044D, "DESERET SMALL LETTER ENG" },
    { 0x1044E, "DESERET SMALL LETTER OI" },
    { 0x1044F, "DESERET SMALL LETTER EW" },
    { 0x10450, "SHAVIAN LETTER PEEP" },
    { 0x10451, "SHAVIAN LETTER TOT" },
    { 0x10452, "SHAVIAN LETTER KICK" },
    { 0x10453, "SHAVIAN LETTER FEE" },
    { 0x10454, "SHAVIAN LETTER THIGH" },
    { 0x10455, "SHAVIAN LETTER SO" },
    { 0x10456, "SHAVIAN LETTER SURE" },
    { 0x10457, "SHAVIAN LETTER CHURCH" },
    { 0x10458, "SHAVIAN LETTER YEA" },
    { 0x10459, "SHAVIAN LETTER HUNG" },
    { 0x1045A, "SHAVIAN LETTER BIB" },
    { 0x1045B, "SHAVIAN LETTER DEAD" },
    { 0x1045C, "SHAVIAN LETTER GAG" },
    { 0x1045D, "SHAVIAN LETTER VOW" },
    { 0x1045E, "SHAVIAN LETTER THEY" },
    { 0x1045F, "SHAVIAN LETTER ZOO" },
    { 0x10460, "SHAVIAN LETTER MEASURE" },
    { 0x10461, "SHAVIAN LETTER JUDGE" },
    { 0x10462, "SHAVIAN LETTER WOE" },
    { 0x10463, "SHAVIAN LETTER HA-HA" },
    { 0x10464, "SHAVIAN LETTER LOLL" },
    { 0x10465, "SHAVIAN LETTER MIME" },
    { 0x10466, "SHAVIAN LETTER IF" },
    { 0x10467, "SHAVIAN LETTER EGG" },
    { 0x10468, "SHAVIAN LETTER ASH" },
    { 0x10469, "SHAVIAN LETTER ADO" },
    { 0x1046A, "SHAVIAN LETTER ON" },
    { 0x1046B, "SHAVIAN LETTER WOOL" },
    { 0x1046C, "SHAVIAN LETTER OUT" },
    { 0x1046D, "SHAVIAN LETTER AH" },
    { 0x1046E, "SHAVIAN LETTER ROAR" },
    { 0x1046F, "SHAVIAN LETTER NUN" },
    { 0x10470, "SHAVIAN LETTER EAT" },
    { 0x10471, "SHAVIAN LETTER AGE" },
    { 0x10472, "SHAVIAN LETTER ICE" },
    { 0x10473, "SHAVIAN LETTER UP" },
    { 0x10474, "SHAVIAN LETTER OAK" },
    { 0x10475, "SHAVIAN LETTER OOZE" },
    { 0x10476, "SHAVIAN LETTER OIL" },
    { 0x10477, "SHAVIAN LETTER AWE" },
    { 0x10478, "SHAVIAN LETTER ARE" },
    { 0x10479, "SHAVIAN LETTER OR" },
    { 0x1047A, "SHAVIAN LETTER AIR" },
    { 0x1047B, "SHAVIAN LETTER ERR" },
    { 0x1047C, "SHAVIAN LETTER ARRAY" },
    { 0x1047D, "SHAVIAN LETTER EAR" },
    { 0x1047E, "SHAVIAN LETTER IAN" },
    { 0x1047F, "SHAVIAN LETTER YEW" },
    { 0x10480, "OSMANYA LETTER ALEF" },
    { 0x10481, "OSMANYA LETTER BA" },
    { 0x10482, "OSMANYA LETTER TA" },
    { 0x10483, "OSMANYA LETTER JA" },
    { 0x10484, "OSMANYA LETTER XA" },
    { 0x10485, "OSMANYA LETTER KHA" },
    { 0x10486, "OSMANYA LETTER DEEL" },
    { 0x10487, "OSMANYA LETTER RA" },
    { 0x10488, "OSMANYA LETTER SA" },
    { 0x10489, "OSMANYA LETTER SHIIN" },
    { 0x1048A, "OSMANYA LETTER DHA" },
    { 0x1048B, "OSMANYA LETTER CAYN" },
    { 0x1048C, "OSMANYA LETTER GA" },
    { 0x1048D, "OSMANYA LETTER FA" },
    { 0x1048E, "OSMANYA LETTER QAAF" },
    { 0x1048F, "OSMANYA LETTER KAAF" },
    { 0x10490, "OSMANYA LETTER LAAN" },
    { 0x10491, "OSMANYA LETTER MIIN" },
    { 0x10492, "OSMANYA LETTER NUUN" },
    { 0x10493, "OSMANYA LETTER WAW" },
    { 0x10494, "OSMANYA LETTER HA" },
    { 0x10495, "OSMANYA LETTER YA" },
    { 0x10496, "OSMANYA LETTER A" },
    { 0x10497, "OSMANYA LETTER E" },
    { 0x10498, "OSMANYA LETTER I" },
    { 0x10499, "OSMANYA LETTER O" },
    { 0x1049A, "OSMANYA LETTER U" },
    { 0x1049B, "OSMANYA LETTER AA" },
    { 0x1049C, "OSMANYA LETTER EE" },
    { 0x1049D, "OSMANYA LETTER OO" },
    { 0x104A0, "OSMANYA DIGIT ZERO" },
    { 0x104A1, "OSMANYA DIGIT ONE" },
    { 0x104A2, "OSMANYA DIGIT TWO" },
    { 0x104A3, "OSMANYA DIGIT THREE" },
    { 0x104A4, "OSMANYA DIGIT FOUR" },
    { 0x104A5, "OSMANYA DIGIT FIVE" },
    { 0x104A6, "OSMANYA DIGIT SIX" },
    { 0x104A7, "OSMANYA DIGIT SEVEN" },
    { 0x104A8, "OSMANYA DIGIT EIGHT" },
    { 0x104A9, "OSMANYA DIGIT NINE" },
    { 0x104B0, "OSAGE CAPITAL LETTER A" },
    { 0x104B1, "OSAGE CAPITAL LETTER AI" },
    { 0x104B2, "OSAGE CAPITAL LETTER AIN" },
    { 0x104B3, "OSAGE CAPITAL LETTER AH" },
    { 0x104B4, "OSAGE CAPITAL LETTER BRA" },
    { 0x104B5, "OSAGE CAPITAL LETTER CHA" },
    { 0x104B6, "OSAGE CAPITAL LETTER EHCHA" },
    { 0x104B7, "OSAGE CAPITAL LETTER E" },
    { 0x104B8, "OSAGE CAPITAL LETTER EIN" },
    { 0x104B9, "OSAGE CAPITAL LETTER HA" },
    { 0x104BA, "OSAGE CAPITAL LETTER HYA" },
    { 0x104BB, "OSAGE CAPITAL LETTER I" },
    { 0x104BC, "OSAGE CAPITAL LETTER KA" },
    { 0x104BD, "OSAGE CAPITAL LETTER EHKA" },
    { 0x104BE, "OSAGE CAPITAL LETTER KYA" },
    { 0x104BF, "OSAGE CAPITAL LETTER LA" },
    { 0x104C0, "OSAGE CAPITAL LETTER MA" },
    { 0x104C1, "OSAGE CAPITAL LETTER NA" },
    { 0x104C2, "OSAGE CAPITAL LETTER O" },
    { 0x104C3, "OSAGE CAPITAL LETTER OIN" },
    { 0x104C4, "OSAGE CAPITAL LETTER PA" },
    { 0x104C5, "OSAGE CAPITAL LETTER EHPA" },
    { 0x104C6, "OSAGE CAPITAL LETTER SA" },
    { 0x104C7, "OSAGE CAPITAL LETTER SHA" },
    { 0x104C8, "OSAGE CAPITAL LETTER TA" },
    { 0x104C9, "OSAGE CAPITAL LETTER EHTA" },
    { 0x104CA, "OSAGE CAPITAL LETTER TSA" },
    { 0x104CB, "OSAGE CAPITAL LETTER EHTSA" },
    { 0x104CC, "OSAGE CAPITAL LETTER TSHA" },
    { 0x104CD, "OSAGE CAPITAL LETTER DHA" },
    { 0x104CE, "OSAGE CAPITAL LETTER U" },
    { 0x104CF, "OSAGE CAPITAL LETTER WA" },
    { 0x104D0, "OSAGE CAPITAL LETTER KHA" },
    { 0x104D1, "OSAGE CAPITAL LETTER GHA" },
    { 0x104D2, "OSAGE CAPITAL LETTER ZA" },
    { 0x104D3, "OSAGE CAPITAL LETTER ZHA" },
    { 0x104D8, "OSAGE SMALL LETTER A" },
    { 0x104D9, "OSAGE SMALL LETTER AI" },
    { 0x104DA, "OSAGE SMALL LETTER AIN" },
    { 0x104DB, "OSAGE SMALL LETTER AH" },
    { 0x104DC, "OSAGE SMALL LETTER BRA" },
    { 0x104DD, "OSAGE SMALL LETTER CHA" },
    { 0x104DE, "OSAGE SMALL LETTER EHCHA" },
    { 0x104DF, "OSAGE SMALL LETTER E" },
    { 0x104E0, "OSAGE SMALL LETTER EIN" },
    { 0x104E1, "OSAGE SMALL LETTER HA" },
    { 0x104E2, "OSAGE SMALL LETTER HYA" },
    { 0x104E3, "OSAGE SMALL LETTER I" },
    { 0x104E4, "OSAGE SMALL LETTER KA" },
    { 0x104E5, "OSAGE SMALL LETTER EHKA" },
    { 0x104E6, "OSAGE SMALL LETTER KYA" },
    { 0x104E7, "OSAGE SMALL LETTER LA" },
    { 0x104E8, "OSAGE SMALL LETTER MA" },
    { 0x104E9, "OSAGE SMALL LETTER NA" },
    { 0x104EA, "OSAGE SMALL LETTER O" },
    { 0x104EB, "OSAGE SMALL LETTER OIN" },
    { 0x104EC, "OSAGE SMALL LETTER PA" },
    { 0x104ED, "OSAGE SMALL LETTER EHPA" },
    { 0x104EE, "OSAGE SMALL LETTER SA" },
    { 0x104EF, "OSAGE SMALL LETTER SHA" },
    { 0x104F0, "OSAGE SMALL LETTER TA" },
    { 0x104F1, "OSAGE SMALL LETTER EHTA" },
    { 0x104F2, "OSAGE SMALL LETTER TSA" },
    { 0x104F3, "OSAGE SMALL LETTER EHTSA" },
    { 0x104F4, "OSAGE SMALL LETTER TSHA" },
    { 0x104F5, "OSAGE SMALL LETTER DHA" },
    { 0x104F6, "OSAGE SMALL LETTER U" },
    { 0x104F7, "OSAGE SMALL LETTER WA" },
    { 0x104F8, "OSAGE SMALL LETTER KHA" },
    { 0x104F9, "OSAGE SMALL LETTER GHA" },
    { 0x104FA, "OSAGE SMALL LETTER ZA" },
    { 0x104FB, "OSAGE SMALL LETTER ZHA" },
    { 0x10500, "ELBASAN LETTER A" },
    { 0x10501, "ELBASAN LETTER BE" },
    { 0x10502, "ELBASAN LETTER CE" },
    { 0x10503, "ELBASAN LETTER CHE" },
    { 0x10504, "ELBASAN LETTER DE" },
    { 0x10505, "ELBASAN LETTER NDE" },
    { 0x10506, "ELBASAN LETTER DHE" },
    { 0x10507, "ELBASAN LETTER EI" },
    { 0x10508, "ELBASAN LETTER E" },
    { 0x10509, "ELBASAN LETTER FE" },
    { 0x1050A, "ELBASAN LETTER GE" },
    { 0x1050B, "ELBASAN LETTER GJE" },
    { 0x1050C, "ELBASAN LETTER HE" },
    { 0x1050D, "ELBASAN LETTER I" },
    { 0x1050E, "ELBASAN LETTER JE" },
    { 0x1050F, "ELBASAN LETTER KE" },
    { 0x10510, "ELBASAN LETTER LE" },
    { 0x10511, "ELBASAN LETTER LLE" },
    { 0x10512, "ELBASAN LETTER ME" },
    { 0x10513, "ELBASAN LETTER NE" },
    { 0x10514, "ELBASAN LETTER NA" },
    { 0x10515, "ELBASAN LETTER NJE" },
    { 0x10516, "ELBASAN LETTER O" },
    { 0x10517, "ELBASAN LETTER PE" },
    { 0x10518, "ELBASAN LETTER QE" },
    { 0x10519, "ELBASAN LETTER RE" },
    { 0x1051A, "ELBASAN LETTER RRE" },
    { 0x1051B, "ELBASAN LETTER SE" },
    { 0x1051C, "ELBASAN LETTER SHE" },
    { 0x1051D, "ELBASAN LETTER TE" },
    { 0x1051E, "ELBASAN LETTER THE" },
    { 0x1051F, "ELBASAN LETTER U" },
    { 0x10520, "ELBASAN LETTER VE" },
    { 0x10521, "ELBASAN LETTER XE" },
    { 0x10522, "ELBASAN LETTER Y" },
    { 0x10523, "ELBASAN LETTER ZE" },
    { 0x10524, "ELBASAN LETTER ZHE" },
    { 0x10525, "ELBASAN LETTER GHE" },
    { 0x10526, "ELBASAN LETTER GHAMMA" },
    { 0x10527, "ELBASAN LETTER KHE" },
    { 0x10530, "CAUCASIAN ALBANIAN LETTER ALT" },
    { 0x10531, "CAUCASIAN ALBANIAN LETTER BET" },
    { 0x10532, "CAUCASIAN ALBANIAN LETTER GIM" },
    { 0x10533, "CAUCASIAN ALBANIAN LETTER DAT" },
    { 0x10534, "CAUCASIAN ALBANIAN LETTER EB" },
    { 0x10535, "CAUCASIAN ALBANIAN LETTER ZARL" },
    { 0x10536, "CAUCASIAN ALBANIAN LETTER EYN" },
    { 0x10537, "CAUCASIAN ALBANIAN LETTER ZHIL" },
    { 0x10538, "CAUCASIAN ALBANIAN LETTER TAS" },
    { 0x10539, "CAUCASIAN ALBANIAN LETTER CHA" },
    { 0x1053A, "CAUCASIAN ALBANIAN LETTER YOWD" },
    { 0x1053B, "CAUCASIAN ALBANIAN LETTER ZHA" },
    { 0x1053C, "CAUCASIAN ALBANIAN LETTER IRB" },
    { 0x1053D, "CAUCASIAN ALBANIAN LETTER SHA" },
    { 0x1053E, "CAUCASIAN ALBANIAN LETTER LAN" },
    { 0x1053F, "CAUCASIAN ALBANIAN LETTER INYA" },
    { 0x10540, "CAUCASIAN ALBANIAN LETTER XEYN" },
    { 0x10541, "CAUCASIAN ALBANIAN LETTER DYAN" },
    { 0x10542, "CAUCASIAN ALBANIAN LETTER CAR" },
    { 0x10543, "CAUCASIAN ALBANIAN LETTER JHOX" },
    { 0x10544, "CAUCASIAN ALBANIAN LETTER KAR" },
    { 0x10545, "CAUCASIAN ALBANIAN LETTER LYIT" },
    { 0x10546, "CAUCASIAN ALBANIAN LETTER HEYT" },
    { 0x10547, "CAUCASIAN ALBANIAN LETTER QAY" },
    { 0x10548, "CAUCASIAN ALBANIAN LETTER AOR" },
    { 0x10549, "CAUCASIAN ALBANIAN LETTER CHOY" },
    { 0x1054A, "CAUCASIAN ALBANIAN LETTER CHI" },
    { 0x1054B, "CAUCASIAN ALBANIAN LETTER CYAY" },
    { 0x1054C, "CAUCASIAN ALBANIAN LETTER MAQ" },
    { 0x1054D, "CAUCASIAN ALBANIAN LETTER QAR" },
    { 0x1054E, "CAUCASIAN ALBANIAN LETTER NOWC" },
    { 0x1054F, "CAUCASIAN ALBANIAN LETTER DZYAY" },
    { 0x10550, "CAUCASIAN ALBANIAN LETTER SHAK" },
    { 0x10551, "CAUCASIAN ALBANIAN LETTER JAYN" },
    { 0x10552, "CAUCASIAN ALBANIAN LETTER ON" },
    { 0x10553, "CAUCASIAN ALBANIAN LETTER TYAY" },
    { 0x10554, "CAUCASIAN ALBANIAN LETTER FAM" },
    { 0x10555, "CAUCASIAN ALBANIAN LETTER DZAY" },
    { 0x10556, "CAUCASIAN ALBANIAN LETTER CHAT" },
    { 0x10557, "CAUCASIAN ALBANIAN LETTER PEN" },
    { 0x10558, "CAUCASIAN ALBANIAN LETTER GHEYS" },
    { 0x10559, "CAUCASIAN ALBANIAN LETTER RAT" },
    { 0x1055A, "CAUCASIAN ALBANIAN LETTER SEYK" },
    { 0x1055B, "CAUCASIAN ALBANIAN LETTER VEYZ" },
    { 0x1055C, "CAUCASIAN ALBANIAN LETTER TIWR" },
    { 0x1055D, "CAUCASIAN ALBANIAN LETTER SHOY" },
    { 0x1055E, "CAUCASIAN ALBANIAN LETTER IWN" },
    { 0x1055F, "CAUCASIAN ALBANIAN LETTER CYAW" },
    { 0x10560, "CAUCASIAN ALBANIAN LETTER CAYN" },
    { 0x10561, "CAUCASIAN ALBANIAN LETTER YAYD" },
    { 0x10562, "CAUCASIAN ALBANIAN LETTER PIWR" },
    { 0x10563, "CAUCASIAN ALBANIAN LETTER KIW" },
    { 0x1056F, "CAUCASIAN ALBANIAN CITATION MARK" },
    { 0x10570, "VITHKUQI CAPITAL LETTER A" },
    { 0x10571, "VITHKUQI CAPITAL LETTER BBE" },
    { 0x10572, "VITHKUQI CAPITAL LETTER BE" },
    { 0x10573, "VITHKUQI CAPITAL LETTER CE" },
    { 0x10574, "VITHKUQI CAPITAL LETTER CHE" },
    { 0x10575, "VITHKUQI CAPITAL LETTER DE" },
    { 0x10576, "VITHKUQI CAPITAL LETTER DHE" },
    { 0x10577, "VITHKUQI CAPITAL LETTER EI" },
    { 0x10578, "VITHKUQI CAPITAL LETTER E" },
    { 0x10579, "VITHKUQI CAPITAL LETTER FE" },
    { 0x1057A, "VITHKUQI CAPITAL LETTER GA" },
    { 0x1057C, "VITHKUQI CAPITAL LETTER HA" },
    { 0x1057D, "VITHKUQI CAPITAL LETTER HHA" },
    { 0x1057E, "VITHKUQI CAPITAL LETTER I" },
    { 0x1057F, "VITHKUQI CAPITAL LETTER IJE" },
    { 0x10580, "VITHKUQI CAPITAL LETTER JE" },
    { 0x10581, "VITHKUQI CAPITAL LETTER KA" },
    { 0x10582, "VITHKUQI CAPITAL LETTER LA" },
    { 0x10583, "VITHKUQI CAPITAL LETTER LLA" },
    { 0x10584, "VITHKUQI CAPITAL LETTER ME" },
    { 0x10585, "VITHKUQI CAPITAL LETTER NE" },
    { 0x10586, "VITHKUQI CAPITAL LETTER NJE" },
    { 0x10587, "VITHKUQI CAPITAL LETTER O" },
    { 0x10588, "VITHKUQI CAPITAL LETTER PE" },
    { 0x10589, "VITHKUQI CAPITAL LETTER QA" },
    { 0x1058A, "VITHKUQI CAPITAL LETTER RE" },
    { 0x1058C, "VITHKUQI CAPITAL LETTER SE" },
    { 0x1058D, "VITHKUQI CAPITAL LETTER SHE" },
    { 0x1058E, "VITHKUQI CAPITAL LETTER TE" },
    { 0x1058F, "VITHKUQI CAPITAL LETTER THE" },
    { 0x10590, "VITHKUQI CAPITAL LETTER U" },
    { 0x10591, "VITHKUQI CAPITAL LETTER VE" },
    { 0x10592, "VITHKUQI CAPITAL LETTER XE" },
    { 0x10594, "VITHKUQI CAPITAL LETTER Y" },
    { 0x10595, "VITHKUQI CAPITAL LETTER ZE" },
    { 0x10597, "VITHKUQI SMALL LETTER A" },
    { 0x10598, "VITHKUQI SMALL LETTER BBE" },
    { 0x10599, "VITHKUQI SMALL LETTER BE" },
    { 0x1059A, "VITHKUQI SMALL LETTER CE" },
    { 0x1059B, "VITHKUQI SMALL LETTER CHE" },
    { 0x1059C, "VITHKUQI SMALL LETTER DE" },
    { 0x1059D, "VITHKUQI SMALL LETTER DHE" },
    { 0x1059E, "VITHKUQI SMALL LETTER EI" },
    { 0x1059F, "VITHKUQI SMALL LETTER E" },
    { 0x105A0, "VITHKUQI SMALL LETTER FE" },
    { 0x105A1, "VITHKUQI SMALL LETTER GA" },
    { 0x105A3, "VITHKUQI SMALL LETTER HA" },
    { 0x105A4, "VITHKUQI SMALL LETTER HHA" },
    { 0x105A5, "VITHKUQI SMALL LETTER I" },
    { 0x105A6, "VITHKUQI SMALL LETTER IJE" },
    { 0x105A7, "VITHKUQI SMALL LETTER JE" },
    { 0x105A8, "VITHKUQI SMALL LETTER KA" },
    { 0x105A9, "VITHKUQI SMALL LETTER LA" },
    { 0x105AA, "VITHKUQI SMALL LETTER LLA" },
    { 0x105AB, "VITHKUQI SMALL LETTER ME" },
    { 0x105AC, "VITHKUQI SMALL LETTER NE" },
    { 0x105AD, "VITHKUQI SMALL LETTER NJE" },
    { 0x105AE, "VITHKUQI SMALL LETTER O" },
    { 0x105AF, "VITHKUQI SMALL LETTER PE" },
    { 0x105B0, "VITHKUQI SMALL LETTER QA" },
    { 0x105B1, "VITHKUQI SMALL LETTER RE" },
    { 0x105B3, "VITHKUQI SMALL LETTER SE" },
    { 0x105B4, "VITHKUQI SMALL LETTER SHE" },
    { 0x105B5, "VITHKUQI SMALL LETTER TE" },
    { 0x105B6, "VITHKUQI SMALL LETTER THE" },
    { 0x105B7, "VITHKUQI SMALL LETTER U" },
    { 0x105B8, "VITHKUQI SMALL LETTER VE" },
    { 0x105B9, "VITHKUQI SMALL LETTER XE" },
    { 0x105BB, "VITHKUQI SMALL LETTER Y" },
    { 0x105BC, "VITHKUQI SMALL LETTER ZE" },
    { 0x10600, "LINEAR A SIGN AB001" },
    { 0x10601, "LINEAR A SIGN AB002" },
    { 0x10602, "LINEAR A SIGN AB003" },
    { 0x10603, "LINEAR A SIGN AB004" },
    { 0x10604, "LINEAR A SIGN AB005" },
    { 0x10605, "LINEAR A SIGN AB006" },
    { 0x10606, "LINEAR A SIGN AB007" },
    { 0x10607, "LINEAR A SIGN AB008" },
    { 0x10608, "LINEAR A SIGN AB009" },
    { 0x10609, "LINEAR A SIGN AB010" },
    { 0x1060A, "LINEAR A SIGN AB011" },
    { 0x1060B, "LINEAR A SIGN AB013" },
    { 0x1060C, "LINEAR A SIGN AB016" },
    { 0x1060D, "LINEAR A SIGN AB017" },
    { 0x1060E, "LINEAR A SIGN AB020" },
    { 0x1060F, "LINEAR A SIGN AB021" },
    { 0x10610, "LINEAR A SIGN AB021F" },
    { 0x10611, "LINEAR A SIGN AB021M" },
    { 0x10612, "LINEAR A SIGN AB022" },
    { 0x10613, "LINEAR A SIGN AB022F" },
    { 0x10614, "LINEAR A SIGN AB022M" },
    { 0x10615, "LINEAR A SIGN AB023" },
    { 0x10616, "LINEAR A SIGN AB023M" },
    { 0x10617, "LINEAR A SIGN AB024" },
    { 0x10618, "LINEAR A SIGN AB026" },
    { 0x10619, "LINEAR A SIGN AB027" },
    { 0x1061A, "LINEAR A SIGN AB028" },
    { 0x1061B, "LINEAR A SIGN A028B" },
    { 0x1061C, "LINEAR A SIGN AB029" },
    { 0x1061D, "LINEAR A SIGN AB030" },
    { 0x1061E, "LINEAR A SIGN AB031" },
    { 0x1061F, "LINEAR A SIGN AB034" },
    { 0x10620, "LINEAR A SIGN AB037" },
    { 0x10621, "LINEAR A SIGN AB038" },
    { 0x10622, "LINEAR A SIGN AB039" },
    { 0x10623, "LINEAR A SIGN AB040" },
    { 0x10624, "LINEAR A SIGN AB041" },
    { 0x10625, "LINEAR A SIGN AB044" },
    { 0x10626, "LINEAR A SIGN AB045" },
    { 0x10627, "LINEAR A SIGN AB046" },
    { 0x10628, "LINEAR A SIGN AB047" },
    { 0x10629, "LINEAR A SIGN AB048" },
    { 0x1062A, "LINEAR A SIGN AB049" },
    { 0x1062B, "LINEAR A SIGN AB050" },
    { 0x1062C, "LINEAR A SIGN AB051" },
    { 0x1062D, "LINEAR A SIGN AB053" },
    { 0x1062E, "LINEAR A SIGN AB054" },
    { 0x1062F, "LINEAR A SIGN AB055" },
    { 0x10630, "LINEAR A SIGN AB056" },
    { 0x10631, "LINEAR A SIGN AB057" },
    { 0x10632, "LINEAR A SIGN AB058" },
    { 0x10633, "LINEAR A SIGN AB059" },
    { 0x10634, "LINEAR A SIGN AB060" },
    { 0x10635, "LINEAR A SIGN AB061" },
    { 0x10636, "LINEAR A SIGN AB065" },
    { 0x10637, "LINEAR A SIGN AB066" },
    { 0x10638, "LINEAR A SIGN AB067" },
    { 0x10639, "LINEAR A SIGN AB069" },
    { 0x1063A, "LINEAR A SIGN AB070" },
    { 0x1063B, "LINEAR A SIGN AB073" },
    { 0x1063C, "LINEAR A SIGN AB074" },
    { 0x1063D, "LINEAR A SIGN AB076" },
    { 0x1063E, "LINEAR A SIGN AB077" },
    { 0x1063F, "LINEAR A SIGN AB078" },
    { 0x10640, "LINEAR A SIGN AB079" },
    { 0x10641, "LINEAR A SIGN AB080" },
    { 0x10642, "LINEAR A SIGN AB081" },
    { 0x10643, "LINEAR A SIGN AB082" },
    { 0x10644, "LINEAR A SIGN AB085" },
    { 0x10645, "LINEAR A SIGN AB086" },
    { 0x10646, "LINEAR A SIGN AB087" },
    { 0x10647, "LINEAR A SIGN A100-102" },
    { 0x10648, "LINEAR A SIGN AB118" },
    { 0x10649, "LINEAR A SIGN AB120" },
    { 0x1064A, "LINEAR A SIGN A120B" },
    { 0x1064B, "LINEAR A SIGN AB122" },
    { 0x1064C, "LINEAR A SIGN AB123" },
    { 0x1064D, "LINEAR A SIGN AB131A" },
    { 0x1064E, "LINEAR A SIGN AB131B" },
    { 0x1064F, "LINEAR A SIGN A131C" },
    { 0x10650, "LINEAR A SIGN AB164" },
    { 0x10651, "LINEAR A SIGN AB171" },
    { 0x10652, "LINEAR A SIGN AB180" },
    { 0x10653, "LINEAR A SIGN AB188" },
    { 0x10654, "LINEAR A SIGN AB191" },
    { 0x10655, "LINEAR A SIGN A301" },
    { 0x10656, "LINEAR A SIGN A302" },
    { 0x10657, "LINEAR A SIGN A303" },
    { 0x10658, "LINEAR A SIGN A304" },
    { 0x10659, "LINEAR A SIGN A305" },
    { 0x1065A, "LINEAR A SIGN A306" },
    { 0x1065B, "LINEAR A SIGN A307" },
    { 0x1065C, "LINEAR A SIGN A308" },
    { 0x1065D, "LINEAR A SIGN A309A" },
    { 0x1065E, "LINEAR A SIGN A309B" },
    { 0x1065F, "LINEAR A SIGN A309C" },
    { 0x10660, "LINEAR A SIGN A310" },
    { 0x10661, "LINEAR A SIGN A311" },
    { 0x10662, "LINEAR A SIGN A312" },
    { 0x10663, "LINEAR A SIGN A313A" },
    { 0x10664, "LINEAR A SIGN A313B" },
    { 0x10665, "LINEAR A SIGN A313C" },
    { 0x10666, "LINEAR A SIGN A314" },
    { 0x10667, "LINEAR A SIGN A315" },
    { 0x10668, "LINEAR A SIGN A316" },
    { 0x10669, "LINEAR A SIGN A317" },
    { 0x1066A, "LINEAR A SIGN A318" },
    { 0x1066B, "LINEAR A SIGN A319" },
    { 0x1066C, "LINEAR A SIGN A320" },
    { 0x1066D, "LINEAR A SIGN A321" },
    { 0x1066E, "LINEAR A SIGN A322" },
    { 0x1066F, "LINEAR A SIGN A323" },
    { 0x10670, "LINEAR A SIGN A324" },
    { 0x10671, "LINEAR A SIGN A325" },
    { 0x10672, "LINEAR A SIGN A326" },
    { 0x10673, "LINEAR A SIGN A327" },
    { 0x10674, "LINEAR A SIGN A328" },
    { 0x10675, "LINEAR A SIGN A329" },
    { 0x10676, "LINEAR A SIGN A330" },
    { 0x10677, "LINEAR A SIGN A331" },
    { 0x10678, "LINEAR A SIGN A332" },
    { 0x10679, "LINEAR A SIGN A333" },
    { 0x1067A, "LINEAR A SIGN A334" },
    { 0x1067B, "LINEAR A SIGN A335" },
    { 0x1067C, "LINEAR A SIGN A336" },
    { 0x1067D, "LINEAR A SIGN A337" },
    { 0x1067E, "LINEAR A SIGN A338" },
    { 0x1067F, "LINEAR A SIGN A339" },
    { 0x10680, "LINEAR A SIGN A340" },
    { 0x10681, "LINEAR A SIGN A341" },
    { 0x10682, "LINEAR A SIGN A342" },
    { 0x10683, "LINEAR A SIGN A343" },
    { 0x10684, "LINEAR A SIGN A344" },
    { 0x10685, "LINEAR A SIGN A345" },
    { 0x10686, "LINEAR A SIGN A346" },
    { 0x10687, "LINEAR A SIGN A347" },
    { 0x10688, "LINEAR A SIGN A348" },
    { 0x10689, "LINEAR A SIGN A349" },
    { 0x1068A, "LINEAR A SIGN A350" },
    { 0x1068B, "LINEAR A SIGN A351" },
    { 0x1068C, "LINEAR A SIGN A352" },
    { 0x1068D, "LINEAR A SIGN A353" },
    { 0x1068E, "LINEAR A SIGN A354" },
    { 0x1068F, "LINEAR A SIGN A355" },
    { 0x10690, "LINEAR A SIGN A356" },
    { 0x10691, "LINEAR A SIGN A357" },
    { 0x10692, "LINEAR A SIGN A358" },
    { 0x10693, "LINEAR A SIGN A359" },
    { 0x10694, "LINEAR A SIGN A360" },
    { 0x10695, "LINEAR A SIGN A361" },
    { 0x10696, "LINEAR A SIGN A362" },
    { 0x10697, "LINEAR A SIGN A363" },
    { 0x10698, "LINEAR A SIGN A364" },
    { 0x10699, "LINEAR A SIGN A365" },
    { 0x1069A, "LINEAR A SIGN A366" },
    { 0x1069B, "LINEAR A SIGN A367" },
    { 0x1069C, "LINEAR A SIGN A368" },
    { 0x1069D, "LINEAR A SIGN A369" },
    { 0x1069E, "LINEAR A SIGN A370" },
    { 0x1069F, "LINEAR A SIGN A371" },
    { 0x106A0, "LINEAR A SIGN A400-VAS" },
    { 0x106A1, "LINEAR A SIGN A401-VAS" },
    { 0x106A2, "LINEAR A SIGN A402-VAS" },
    { 0x106A3, "LINEAR A SIGN A403-VAS" },
    { 0x106A4, "LINEAR A SIGN A404-VAS" },
    { 0x106A5, "LINEAR A SIGN A405-VAS" },
    { 0x106A6, "LINEAR A SIGN A406-VAS" },
    { 0x106A7, "LINEAR A SIGN A407-VAS" },
    { 0x106A8, "LINEAR A SIGN A408-VAS" },
    { 0x106A9, "LINEAR A SIGN A409-VAS" },
    { 0x106AA, "LINEAR A SIGN A410-VAS" },
    { 0x106AB, "LINEAR A SIGN A411-VAS" },
    { 0x106AC, "LINEAR A SIGN A412-VAS" },
    { 0x106AD, "LINEAR A SIGN A413-VAS" },
    { 0x106AE, "LINEAR A SIGN A414-VAS" },
    { 0x106AF, "LINEAR A SIGN A415-VAS" },
    { 0x106B0, "LINEAR A SIGN A416-VAS" },
    { 0x106B1, "LINEAR A SIGN A417-VAS" },
    { 0x106B2, "LINEAR A SIGN A418-VAS" },
    { 0x106B3, "LINEAR A SIGN A501" },
    { 0x106B4, "LINEAR A SIGN A502" },
    { 0x106B5, "LINEAR A SIGN A503" },
    { 0x106B6, "LINEAR A SIGN A504" },
    { 0x106B7, "LINEAR A SIGN A505" },
    { 0x106B8, "LINEAR A SIGN A506" },
    { 0x106B9, "LINEAR A SIGN A508" },
    { 0x106BA, "LINEAR A SIGN A509" },
    { 0x106BB, "LINEAR A SIGN A510" },
    { 0x106BC, "LINEAR A SIGN A511" },
    { 0x106BD, "LINEAR A SIGN A512" },
    { 0x106BE, "LINEAR A SIGN A513" },
    { 0x106BF, "LINEAR A SIGN A515" },
    { 0x106C0, "LINEAR A SIGN A516" },
    { 0x106C1, "LINEAR A SIGN A520" },
    { 0x106C2, "LINEAR A SIGN A521" },
    { 0x106C3, "LINEAR A SIGN A523" },
    { 0x106C4, "LINEAR A SIGN A524" },
    { 0x106C5, "LINEAR A SIGN A525" },
    { 0x106C6, "LINEAR A SIGN A526" },
    { 0x106C7, "LINEAR A SIGN A527" },
    { 0x106C8, "LINEAR A SIGN A528" },
    { 0x106C9, "LINEAR A SIGN A529" },
    { 0x106CA, "LINEAR A SIGN A530" },
    { 0x106CB, "LINEAR A SIGN A531" },
    { 0x106CC, "LINEAR A SIGN A532" },
    { 0x106CD, "LINEAR A SIGN A534" },
    { 0x106CE, "LINEAR A SIGN A535" },
    { 0x106CF, "LINEAR A SIGN A536" },
    { 0x106D0, "LINEAR A SIGN A537" },
    { 0x106D1, "LINEAR A SIGN A538" },
    { 0x106D2, "LINEAR A SIGN A539" },
    { 0x106D3, "LINEAR A SIGN A540" },
    { 0x106D4, "LINEAR A SIGN A541" },
    { 0x106D5, "LINEAR A SIGN A542" },
    { 0x106D6, "LINEAR A SIGN A545" },
    { 0x106D7, "LINEAR A SIGN A547" },
    { 0x106D8, "LINEAR A SIGN A548" },
    { 0x106D9, "LINEAR A SIGN A549" },
    { 0x106DA, "LINEAR A SIGN A550" },
    { 0x106DB, "LINEAR A SIGN A551" },
    { 0x106DC, "LINEAR A SIGN A552" },
    { 0x106DD, "LINEAR A SIGN A553" },
    { 0x106DE, "LINEAR A SIGN A554" },
    { 0x106DF, "LINEAR A SIGN A555" },
    { 0x106E0, "LINEAR A SIGN A556" },
    { 0x106E1, "LINEAR A SIGN A557" },
    { 0x106E2, "LINEAR A SIGN A559" },
    { 0x106E3, "LINEAR A SIGN A563" },
    { 0x106E4, "LINEAR A SIGN A564" },
    { 0x106E5, "LINEAR A SIGN A565" },
    { 0x106E6, "LINEAR A SIGN A566" },
    { 0x106E7, "LINEAR A SIGN A568" },
    { 0x106E8, "LINEAR A SIGN A569" },
    { 0x106E9, "LINEAR A SIGN A570" },
    { 0x106EA, "LINEAR A SIGN A571" },
    { 0x106EB, "LINEAR A SIGN A572" },
    { 0x106EC, "LINEAR A SIGN A573" },
    { 0x106ED, "LINEAR A SIGN A574" },
    { 0x106EE, "LINEAR A SIGN A575" },
    { 0x106EF, "LINEAR A SIGN A576" },
    { 0x106F0, "LINEAR A SIGN A577" },
    { 0x106F1, "LINEAR A SIGN A578" },
    { 0x106F2, "LINEAR A SIGN A579" },
    { 0x106F3, "LINEAR A SIGN A580" },
    { 0x106F4, "LINEAR A SIGN A581" },
    { 0x106F5, "LINEAR A SIGN A582" },
    { 0x106F6, "LINEAR A SIGN A583" },
    { 0x106F7, "LINEAR A SIGN A584" },
    { 0x106F8, "LINEAR A SIGN A585" },
    { 0x106F9, "LINEAR A SIGN A586" },
    { 0x106FA, "LINEAR A SIGN A587" },
    { 0x106FB, "LINEAR A SIGN A588" },
    { 0x106FC, "LINEAR A SIGN A589" },
    { 0x106FD, "LINEAR A SIGN A591" },
    { 0x106FE, "LINEAR A SIGN A592" },
    { 0x106FF, "LINEAR A SIGN A594" },
    { 0x10700, "LINEAR A SIGN A595" },
    { 0x10701, "LINEAR A SIGN A596" },
    { 0x10702, "LINEAR A SIGN A598" },
    { 0x10703, "LINEAR A SIGN A600" },
    { 0x10704, "LINEAR A SIGN A601" },
    { 0x10705, "LINEAR A SIGN A602" },
    { 0x10706, "LINEAR A SIGN A603" },
    { 0x10707, "LINEAR A SIGN A604" },
    { 0x10708, "LINEAR A SIGN A606" },
    { 0x10709, "LINEAR A SIGN A608" },
    { 0x1070A, "LINEAR A SIGN A609" },
    { 0x1070B, "LINEAR A SIGN A610" },
    { 0x1070C, "LINEAR A SIGN A611" },
    { 0x1070D, "LINEAR A SIGN A612" },
    { 0x1070E, "LINEAR A SIGN A613" },
    { 0x1070F, "LINEAR A SIGN A614" },
    { 0x10710, "LINEAR A SIGN A615" },
    { 0x10711, "LINEAR A SIGN A616" },
    { 0x10712, "LINEAR A SIGN A617" },
    { 0x10713, "LINEAR A SIGN A618" },
    { 0x10714, "LINEAR A SIGN A619" },
    { 0x10715, "LINEAR A SIGN A620" },
    { 0x10716, "LINEAR A SIGN A621" },
    { 0x10717, "LINEAR A SIGN A622" },
    { 0x10718, "LINEAR A SIGN A623" },
    { 0x10719, "LINEAR A SIGN A624" },
    { 0x1071A, "LINEAR A SIGN A626" },
    { 0x1071B, "LINEAR A SIGN A627" },
    { 0x1071C, "LINEAR A SIGN A628" },
    { 0x1071D, "LINEAR A SIGN A629" },
    { 0x1071E, "LINEAR A SIGN A634" },
    { 0x1071F, "LINEAR A SIGN A637" },
    { 0x10720, "LINEAR A SIGN A638" },
    { 0x10721, "LINEAR A SIGN A640" },
    { 0x10722, "LINEAR A SIGN A642" },
    { 0x10723, "LINEAR A SIGN A643" },
    { 0x10724, "LINEAR A SIGN A644" },
    { 0x10725, "LINEAR A SIGN A645" },
    { 0x10726, "LINEAR A SIGN A646" },
    { 0x10727, "LINEAR A SIGN A648" },
    { 0x10728, "LINEAR A SIGN A649" },
    { 0x10729, "LINEAR A SIGN A651" },
    { 0x1072A, "LINEAR A SIGN A652" },
    { 0x1072B, "LINEAR A SIGN A653" },
    { 0x1072C, "LINEAR A SIGN A654" },
    { 0x1072D, "LINEAR A SIGN A655" },
    { 0x1072E, "LINEAR A SIGN A656" },
    { 0x1072F, "LINEAR A SIGN A657" },
    { 0x10730, "LINEAR A SIGN A658" },
    { 0x10731, "LINEAR A SIGN A659" },
    { 0x10732, "LINEAR A SIGN A660" },
    { 0x10733, "LINEAR A SIGN A661" },
    { 0x10734, "LINEAR A SIGN A662" },
    { 0x10735, "LINEAR A SIGN A663" },
    { 0x10736, "LINEAR A SIGN A664" },
    { 0x10740, "LINEAR A SIGN A701 A" },
    { 0x10741, "LINEAR A SIGN A702 B" },
    { 0x10742, "LINEAR A SIGN A703 D" },
    { 0x10743, "LINEAR A SIGN A704 E" },
    { 0x10744, "LINEAR A SIGN A705 F" },
    { 0x10745, "LINEAR A SIGN A706 H" },
    { 0x10746, "LINEAR A SIGN A707 J" },
    { 0x10747, "LINEAR A SIGN A708 K" },
    { 0x10748, "LINEAR A SIGN A709 L" },
    { 0x10749, "LINEAR A SIGN A709-2 L2" },
    { 0x1074A, "LINEAR A SIGN A709-3 L3" },
    { 0x1074B, "LINEAR A SIGN A709-4 L4" },
    { 0x1074C, "LINEAR A SIGN A709-6 L6" },
    { 0x1074D, "LINEAR A SIGN A710 W" },
    { 0x1074E, "LINEAR A SIGN A711 X" },
    { 0x1074F, "LINEAR A SIGN A712 Y" },
    { 0x10750, "LINEAR A SIGN A713 OMEGA" },
    { 0x10751, "LINEAR A SIGN A714 ABB" },
    { 0x10752, "LINEAR A SIGN A715 BB" },
    { 0x10753, "LINEAR A SIGN A717 DD" },
    { 0x10754, "LINEAR A SIGN A726 EYYY" },
    { 0x10755, "LINEAR A SIGN A732 JE" },
    { 0x10760, "LINEAR A SIGN A800" },
    { 0x10761, "LINEAR A SIGN A801" },
    { 0x10762, "LINEAR A SIGN A802" },
    { 0x10763, "LINEAR A SIGN A803" },
    { 0x10764, "LINEAR A SIGN A804" },
    { 0x10765, "LINEAR A SIGN A805" },
    { 0x10766, "LINEAR A SIGN A806" },
    { 0x10767, "LINEAR A SIGN A807" },
    { 0x10780, "MODIFIER LETTER SMALL CAPITAL AA" },
    { 0x10781, "MODIFIER LETTER SUPERSCRIPT TRIANGULAR COLON" },
    { 0x10782, "MODIFIER LETTER SUPERSCRIPT HALF TRIANGULAR COLON" },
    { 0x10783, "MODIFIER LETTER SMALL AE" },
    { 0x10784, "MODIFIER LETTER SMALL CAPITAL B" },
    { 0x10785, "MODIFIER LETTER SMALL B WITH HOOK" },
    { 0x10787, "MODIFIER LETTER SMALL DZ DIGRAPH" },
    { 0x10788, "MODIFIER LETTER SMALL DZ DIGRAPH WITH RETROFLEX HOOK" },
    { 0x10789, "MODIFIER LETTER SMALL DZ DIGRAPH WITH CURL" },
    { 0x1078A, "MODIFIER LETTER SMALL DEZH DIGRAPH" },
    { 0x1078B, "MODIFIER LETTER SMALL D WITH TAIL" },
    { 0x1078C, "MODIFIER LETTER SMALL D WITH HOOK" },
    { 0x1078D, "MODIFIER LETTER SMALL D WITH HOOK AND TAIL" },
    { 0x1078E, "MODIFIER LETTER SMALL REVERSED E" },
    { 0x1078F, "MODIFIER LETTER SMALL CLOSED REVERSED OPEN E" },
    { 0x10790, "MODIFIER LETTER SMALL FENG DIGRAPH" },
    { 0x10791, "MODIFIER LETTER SMALL RAMS HORN" },
    { 0x10792, "MODIFIER LETTER SMALL CAPITAL G" },
    { 0x10793, "MODIFIER LETTER SMALL G WITH HOOK" },
    { 0x10794, "MODIFIER LETTER SMALL CAPITAL G WITH HOOK" },
    { 0x10795, "MODIFIER LETTER SMALL H WITH STROKE" },
    { 0x10796, "MODIFIER LETTER SMALL CAPITAL H" },
    { 0x10797, "MODIFIER LETTER SMALL HENG WITH HOOK" },
    { 0x10798, "MODIFIER LETTER SMALL DOTLESS J WITH STROKE AND HOOK" },
    { 0x10799, "MODIFIER LETTER SMALL LS DIGRAPH" },
    { 0x1079A, "MODIFIER LETTER SMALL LZ DIGRAPH" },
    { 0x1079B, "MODIFIER LETTER SMALL L WITH BELT" },
    { 0x1079C, "MODIFIER LETTER SMALL CAPITAL L WITH BELT" },
    { 0x1079D, "MODIFIER LETTER SMALL L WITH RETROFLEX HOOK AND BELT" },
    { 0x1079E, "MODIFIER LETTER SMALL LEZH" },
    { 0x1079F, "MODIFIER LETTER SMALL LEZH WITH RETROFLEX HOOK" },
    { 0x107A0, "MODIFIER LETTER SMALL TURNED Y" },
    { 0x107A1, "MODIFIER LETTER SMALL TURNED Y WITH BELT" },
    { 0x107A2, "MODIFIER LETTER SMALL O WITH STROKE" },
    { 0x107A3, "MODIFIER LETTER SMALL CAPITAL OE" },
    { 0x107A4, "MODIFIER LETTER SMALL CLOSED OMEGA" },
    { 0x107A5, "MODIFIER LETTER SMALL Q" },
    { 0x107A6, "MODIFIER LETTER SMALL TURNED R WITH LONG LEG" },
    { 0x107A7, "MODIFIER LETTER SMALL TURNED R WITH LONG LEG AND RETROFLEX HOOK" },
    { 0x107A8, "MODIFIER LETTER SMALL R WITH TAIL" },
    { 0x107A9, "MODIFIER LETTER SMALL R WITH FISHHOOK" },
    { 0x107AA, "MODIFIER LETTER SMALL CAPITAL R" },
    { 0x107AB, "MODIFIER LETTER SMALL TC DIGRAPH WITH CURL" },
    { 0x107AC, "MODIFIER LETTER SMALL TS DIGRAPH" },
    { 0x107AD, "MODIFIER LETTER SMALL TS DIGRAPH WITH RETROFLEX HOOK" },
    { 0x107AE, "MODIFIER LETTER SMALL TESH DIGRAPH" },
    { 0x107AF, "MODIFIER LETTER SMALL T WITH RETROFLEX HOOK" },
    { 0x107B0, "MODIFIER LETTER SMALL V WITH RIGHT HOOK" },
    { 0x107B2, "MODIFIER LETTER SMALL CAPITAL Y" },
    { 0x107B3, "MODIFIER LETTER GLOTTAL STOP WITH STROKE" },
    { 0x107B4, "MODIFIER LETTER REVERSED GLOTTAL STOP WITH STROKE" },
    { 0x107B5, "MODIFIER LETTER BILABIAL CLICK" },
    { 0x107B6, "MODIFIER LETTER DENTAL CLICK" },
    { 0x107B7, "MODIFIER LETTER LATERAL CLICK" },
    { 0x107B8, "MODIFIER LETTER ALVEOLAR CLICK" },
    { 0x107B9, "MODIFIER LETTER RETROFLEX CLICK WITH RETROFLEX HOOK" },
    { 0x107BA, "MODIFIER LETTER SMALL S WITH CURL" },
    { 0x10800, "CYPRIOT SYLLABLE A" },
    { 0x10801, "CYPRIOT SYLLABLE E" },
    { 0x10802, "CYPRIOT SYLLABLE I" },
    { 0x10803, "CYPRIOT SYLLABLE O" },
    { 0x10804, "CYPRIOT SYLLABLE U" },
    { 0x10805, "CYPRIOT SYLLABLE JA" },
    { 0x10808, "CYPRIOT SYLLABLE JO" },
    { 0x1080A, "CYPRIOT SYLLABLE KA" },
    { 0x1080B, "CYPRIOT SYLLABLE KE" },
    { 0x1080C, "CYPRIOT SYLLABLE KI" },
    { 0x1080D, "CYPRIOT SYLLABLE KO" },
    { 0x1080E, "CYPRIOT SYLLABLE KU" },
    { 0x1080F, "CYPRIOT SYLLABLE LA" },
    { 0x10810, "CYPRIOT SYLLABLE LE" },
    { 0x10811, "CYPRIOT SYLLABLE LI" },
    { 0x10812, "CYPRIOT SYLLABLE LO" },
    { 0x10813, "CYPRIOT SYLLABLE LU" },
    { 0x10814, "CYPRIOT SYLLABLE MA" },
    { 0x10815, "CYPRIOT SYLLABLE ME" },
    { 0x10816, "CYPRIOT SYLLABLE MI" },
    { 0x10817, "CYPRIOT SYLLABLE MO" },
    { 0x10818, "CYPRIOT SYLLABLE MU" },
    { 0x10819, "CYPRIOT SYLLABLE NA" },
    { 0x1081A, "CYPRIOT SYLLABLE NE" },
    { 0x1081B, "CYPRIOT SYLLABLE NI" },
    { 0x1081C, "CYPRIOT SYLLABLE NO" },
    { 0x1081D, "CYPRIOT SYLLABLE NU" },
    { 0x1081E, "CYPRIOT SYLLABLE PA" },
    { 0x1081F, "CYPRIOT SYLLABLE PE" },
    { 0x10820, "CYPRIOT SYLLABLE PI" },
    { 0x10821, "CYPRIOT SYLLABLE PO" },
    { 0x10822, "CYPRIOT SYLLABLE PU" },
    { 0x10823, "CYPRIOT SYLLABLE RA" },
    { 0x10824, "CYPRIOT SYLLABLE RE" },
    { 0x10825, "CYPRIOT SYLLABLE RI" },
    { 0x10826, "CYPRIOT SYLLABLE RO" },
    { 0x10827, "CYPRIOT SYLLABLE RU" },
    { 0x10828, "CYPRIOT SYLLABLE SA" },
    { 0x10829, "CYPRIOT SYLLABLE SE" },
    { 0x1082A, "CYPRIOT SYLLABLE SI" },
    { 0x1082B, "CYPRIOT SYLLABLE SO" },
    { 0x1082C, "CYPRIOT SYLLABLE SU" },
    { 0x1082D, "CYPRIOT SYLLABLE TA" },
    { 0x1082E, "CYPRIOT SYLLABLE TE" },
    { 0x1082F, "CYPRIOT SYLLABLE TI" },
    { 0x10830, "CYPRIOT SYLLABLE TO" },
    { 0x10831, "CYPRIOT SYLLABLE TU" },
    { 0x10832, "CYPRIOT SYLLABLE WA" },
    { 0x10833, "CYPRIOT SYLLABLE WE" },
    { 0x10834, "CYPRIOT SYLLABLE WI" },
    { 0x10835, "CYPRIOT SYLLABLE WO" },
    { 0x10837, "CYPRIOT SYLLABLE XA" },
    { 0x10838, "CYPRIOT SYLLABLE XE" },
    { 0x1083C, "CYPRIOT SYLLABLE ZA" },
    { 0x1083F, "CYPRIOT SYLLABLE ZO" },
    { 0x10840, "IMPERIAL ARAMAIC LETTER ALEPH" },
    { 0x10841, "IMPERIAL ARAMAIC LETTER BETH" },
    { 0x10842, "IMPERIAL ARAMAIC LETTER GIMEL" },
    { 0x10843, "IMPERIAL ARAMAIC LETTER DALETH" },
    { 0x10844, "IMPERIAL ARAMAIC LETTER HE" },
    { 0x10845, "IMPERIAL ARAMAIC LETTER WAW" },
    { 0x10846, "IMPERIAL ARAMAIC LETTER ZAYIN" },
    { 0x10847, "IMPERIAL ARAMAIC LETTER HETH" },
    { 0x10848, "IMPERIAL ARAMAIC LETTER TETH" },
    { 0x10849, "IMPERIAL ARAMAIC LETTER YODH" },
    { 0x1084A, "IMPERIAL ARAMAIC LETTER KAPH" },
    { 0x1084B, "IMPERIAL ARAMAIC LETTER LAMEDH" },
    { 0x1084C, "IMPERIAL ARAMAIC LETTER MEM" },
    { 0x1084D, "IMPERIAL ARAMAIC LETTER NUN" },
    { 0x1084E, "IMPERIAL ARAMAIC LETTER SAMEKH" },
    { 0x1084F, "IMPERIAL ARAMAIC LETTER AYIN" },
    { 0x10850, "IMPERIAL ARAMAIC LETTER PE" },
    { 0x10851, "IMPERIAL ARAMAIC LETTER SADHE" },
    { 0x10852, "IMPERIAL ARAMAIC LETTER QOPH" },
    { 0x10853, "IMPERIAL ARAMAIC LETTER RESH" },
    { 0x10854, "IMPERIAL ARAMAIC LETTER SHIN" },
    { 0x10855, "IMPERIAL ARAMAIC LETTER TAW" },
    { 0x10857, "IMPERIAL ARAMAIC SECTION SIGN" },
    { 0x10858, "IMPERIAL ARAMAIC NUMBER ONE" },
    { 0x10859, "IMPERIAL ARAMAIC NUMBER TWO" },
    { 0x1085A, "IMPERIAL ARAMAIC NUMBER THREE" },
    { 0x1085B, "IMPERIAL ARAMAIC NUMBER TEN" },
    { 0x1085C, "IMPERIAL ARAMAIC NUMBER TWENTY" },
    { 0x1085D, "IMPERIAL ARAMAIC NUMBER ONE HUNDRED" },
    { 0x1085E, "IMPERIAL ARAMAIC NUMBER ONE THOUSAND" },
    { 0x1085F, "IMPERIAL ARAMAIC NUMBER TEN THOUSAND" },
    { 0x10860, "PALMYRENE LETTER ALEPH" },
    { 0x10861, "PALMYRENE LETTER BETH" },
    { 0x10862, "PALMYRENE LETTER GIMEL" },
    { 0x10863, "PALMYRENE LETTER DALETH" },
    { 0x10864, "PALMYRENE LETTER HE" },
    { 0x10865, "PALMYRENE LETTER WAW" },
    { 0x10866, "PALMYRENE LETTER ZAYIN" },
    { 0x10867, "PALMYRENE LETTER HETH" },
    { 0x10868, "PALMYRENE LETTER TETH" },
    { 0x10869, "PALMYRENE LETTER YODH" },
    { 0x1086A, "PALMYRENE LETTER KAPH" },
    { 0x1086B, "PALMYRENE LETTER LAMEDH" },
    { 0x1086C, "PALMYRENE LETTER MEM" },
    { 0x1086D, "PALMYRENE LETTER FINAL NUN" },
    { 0x1086E, "PALMYRENE LETTER NUN" },
    { 0x1086F, "PALMYRENE LETTER SAMEKH" },
    { 0x10870, "PALMYRENE LETTER AYIN" },
    { 0x10871, "PALMYRENE LETTER PE" },
    { 0x10872, "PALMYRENE LETTER SADHE" },
    { 0x10873, "PALMYRENE LETTER QOPH" },
    { 0x10874, "PALMYRENE LETTER RESH" },
    { 0x10875, "PALMYRENE LETTER SHIN" },
    { 0x10876, "PALMYRENE LETTER TAW" },
    { 0x10877, "PALMYRENE LEFT-POINTING FLEURON" },
    { 0x10878, "PALMYRENE RIGHT-POINTING FLEURON" },
    { 0x10879, "PALMYRENE NUMBER ONE" },
    { 0x1087A, "PALMYRENE NUMBER TWO" },
    { 0x1087B, "PALMYRENE NUMBER THREE" },
    { 0x1087C, "PALMYRENE NUMBER FOUR" },
    { 0x1087D, "PALMYRENE NUMBER FIVE" },
    { 0x1087E, "PALMYRENE NUMBER TEN" },
    { 0x1087F, "PALMYRENE NUMBER TWENTY" },
    { 0x10880, "NABATAEAN LETTER FINAL ALEPH" },
    { 0x10881, "NABATAEAN LETTER ALEPH" },
    { 0x10882, "NABATAEAN LETTER FINAL BETH" },
    { 0x10883, "NABATAEAN LETTER BETH" },
    { 0x10884, "NABATAEAN LETTER GIMEL" },
    { 0x10885, "NABATAEAN LETTER DALETH" },
    { 0x10886, "NABATAEAN LETTER FINAL HE" },
    { 0x10887, "NABATAEAN LETTER HE" },
    { 0x10888, "NABATAEAN LETTER WAW" },
    { 0x10889, "NABATAEAN LETTER ZAYIN" },
    { 0x1088A, "NABATAEAN LETTER HETH" },
    { 0x1088B, "NABATAEAN LETTER TETH" },
    { 0x1088C, "NABATAEAN LETTER FINAL YODH" },
    { 0x1088D, "NABATAEAN LETTER YODH" },
    { 0x1088E, "NABATAEAN LETTER FINAL KAPH" },
    { 0x1088F, "NABATAEAN LETTER KAPH" },
    { 0x10890, "NABATAEAN LETTER FINAL LAMEDH" },
    { 0x10891, "NABATAEAN LETTER LAMEDH" },
    { 0x10892, "NABATAEAN LETTER FINAL MEM" },
    { 0x10893, "NABATAEAN LETTER MEM" },
    { 0x10894, "NABATAEAN LETTER FINAL NUN" },
    { 0x10895, "NABATAEAN LETTER NUN" },
    { 0x10896, "NABATAEAN LETTER SAMEKH" },
    { 0x10897, "NABATAEAN LETTER AYIN" },
    { 0x10898, "NABATAEAN LETTER PE" },
    { 0x10899, "NABATAEAN LETTER SADHE" },
    { 0x1089A, "NABATAEAN LETTER QOPH" },
    { 0x1089B, "NABATAEAN LETTER RESH" },
    { 0x1089C, "NABATAEAN LETTER FINAL SHIN" },
    { 0x1089D, "NABATAEAN LETTER SHIN" },
    { 0x1089E, "NABATAEAN LETTER TAW" },
    { 0x108A7, "NABATAEAN NUMBER ONE" },
    { 0x108A8, "NABATAEAN NUMBER TWO" },
    { 0x108A9, "NABATAEAN NUMBER THREE" },
    { 0x108AA, "NABATAEAN NUMBER FOUR" },
    { 0x108AB, "NABATAEAN CRUCIFORM NUMBER FOUR" },
    { 0x108AC, "NABATAEAN NUMBER FIVE" },
    { 0x108AD, "NABATAEAN NUMBER TEN" },
    { 0x108AE, "NABATAEAN NUMBER TWENTY" },
    { 0x108AF, "NABATAEAN NUMBER ONE HUNDRED" },
    { 0x108E0, "HATRAN LETTER ALEPH" },
    { 0x108E1, "HATRAN LETTER BETH" },
    { 0x108E2, "HATRAN LETTER GIMEL" },
    { 0x108E3, "HATRAN LETTER DALETH-RESH" },
    { 0x108E4, "HATRAN LETTER HE" },
    { 0x108E5, "HATRAN LETTER WAW" },
    { 0x108E6, "HATRAN LETTER ZAYN" },
    { 0x108E7, "HATRAN LETTER HETH" },
    { 0x108E8, "HATRAN LETTER TETH" },
    { 0x108E9, "HATRAN LETTER YODH" },
    { 0x108EA, "HATRAN LETTER KAPH" },
    { 0x108EB, "HATRAN LETTER LAMEDH" },
    { 0x108EC, "HATRAN LETTER MEM" },
    { 0x108ED, "HATRAN LETTER NUN" },
    { 0x108EE, "HATRAN LETTER SAMEKH" },
    { 0x108EF, "HATRAN LETTER AYN" },
    { 0x108F0, "HATRAN LETTER PE" },
    { 0x108F1, "HATRAN LETTER SADHE" },
    { 0x108F2, "HATRAN LETTER QOPH" },
    { 0x108F4, "HATRAN LETTER SHIN" },
    { 0x108F5, "HATRAN LETTER TAW" },
    { 0x108FB, "HATRAN NUMBER ONE" },
    { 0x108FC, "HATRAN NUMBER FIVE" },
    { 0x108FD, "HATRAN NUMBER TEN" },
    { 0x108FE, "HATRAN NUMBER TWENTY" },
    { 0x108FF, "HATRAN NUMBER ONE HUNDRED" },
    { 0x10900, "PHOENICIAN LETTER ALF" },
    { 0x10901, "PHOENICIAN LETTER BET" },
    { 0x10902, "PHOENICIAN LETTER GAML" },
    { 0x10903, "PHOENICIAN LETTER DELT" },
    { 0x10904, "PHOENICIAN LETTER HE" },
    { 0x10905, "PHOENICIAN LETTER WAU" },
    { 0x10906, "PHOENICIAN LETTER ZAI" },
    { 0x10907, "PHOENICIAN LETTER HET" },
    { 0x10908, "PHOENICIAN LETTER TET" },
    { 0x10909, "PHOENICIAN LETTER YOD" },
    { 0x1090A, "PHOENICIAN LETTER KAF" },
    { 0x1090B, "PHOENICIAN LETTER LAMD" },
    { 0x1090C, "PHOENICIAN LETTER MEM" },
    { 0x1090D, "PHOENICIAN LETTER NUN" },
    { 0x1090E, "PHOENICIAN LETTER SEMK" },
    { 0x1090F, "PHOENICIAN LETTER AIN" },
    { 0x10910, "PHOENICIAN LETTER PE" },
    { 0x10911, "PHOENICIAN LETTER SADE" },
    { 0x10912, "PHOENICIAN LETTER QOF" },
    { 0x10913, "PHOENICIAN LETTER ROSH" },
    { 0x10914, "PHOENICIAN LETTER SHIN" },
    { 0x10915, "PHOENICIAN LETTER TAU" },
    { 0x10916, "PHOENICIAN NUMBER ONE" },
    { 0x10917, "PHOENICIAN NUMBER TEN" },
    { 0x10918, "PHOENICIAN NUMBER TWENTY" },
    { 0x10919, "PHOENICIAN NUMBER ONE HUNDRED" },
    { 0x1091A, "PHOENICIAN NUMBER TWO" },
    { 0x1091B, "PHOENICIAN NUMBER THREE" },
    { 0x1091F, "PHOENICIAN WORD SEPARATOR" },
    { 0x10920, "LYDIAN LETTER A" },
    { 0x10921, "LYDIAN LETTER B" },
    { 0x10922, "LYDIAN LETTER G" },
    { 0x10923, "LYDIAN LETTER D" },
    { 0x10924, "LYDIAN LETTER E" },
    { 0x10925, "LYDIAN LETTER V" },
    { 0x10926, "LYDIAN LETTER I" },
    { 0x10927, "LYDIAN LETTER Y" },
    { 0x10928, "LYDIAN LETTER K" },
    { 0x10929, "LYDIAN LETTER L" },
    { 0x1092A, "LYDIAN LETTER M" },
    { 0x1092B, "LYDIAN LETTER N" },
    { 0x1092C, "LYDIAN LETTER O" },
    { 0x1092D, "LYDIAN LETTER R" },
    { 0x1092E, "LYDIAN LETTER SS" },
    { 0x1092F, "LYDIAN LETTER T" },
    { 0x10930, "LYDIAN LETTER U" },
    { 0x10931, "LYDIAN LETTER F" },
    { 0x10932, "LYDIAN LETTER Q" },
    { 0x10933, "LYDIAN LETTER S" },
    { 0x10934, "LYDIAN LETTER TT" },
    { 0x10935, "LYDIAN LETTER AN" },
    { 0x10936, "LYDIAN LETTER EN" },
    { 0x10937, "LYDIAN LETTER LY" },
    { 0x10938, "LYDIAN LETTER NN" },
    { 0x10939, "LYDIAN LETTER C" },
    { 0x1093F, "LYDIAN TRIANGULAR MARK" },
    { 0x10980, "MEROITIC HIEROGLYPHIC LETTER A" },
    { 0x10981, "MEROITIC HIEROGLYPHIC LETTER E" },
    { 0x10982, "MEROITIC HIEROGLYPHIC LETTER I" },
    { 0x10983, "MEROITIC HIEROGLYPHIC LETTER O" },
    { 0x10984, "MEROITIC HIEROGLYPHIC LETTER YA" },
    { 0x10985, "MEROITIC HIEROGLYPHIC LETTER WA" },
    { 0x10986, "MEROITIC HIEROGLYPHIC LETTER BA" },
    { 0x10987, "MEROITIC HIEROGLYPHIC LETTER BA-2" },
    { 0x10988, "MEROITIC HIEROGLYPHIC LETTER PA" },
    { 0x10989, "MEROITIC HIEROGLYPHIC LETTER MA" },
    { 0x1098A, "MEROITIC HIEROGLYPHIC LETTER NA" },
    { 0x1098B, "MEROITIC HIEROGLYPHIC LETTER NA-2" },
    { 0x1098C, "MEROITIC HIEROGLYPHIC LETTER NE" },
    { 0x1098D, "MEROITIC HIEROGLYPHIC LETTER NE-2" },
    { 0x1098E, "MEROITIC HIEROGLYPHIC LETTER RA" },
    { 0x1098F, "MEROITIC HIEROGLYPHIC LETTER RA-2" },
    { 0x10990, "MEROITIC HIEROGLYPHIC LETTER LA" },
    { 0x10991, "MEROITIC HIEROGLYPHIC LETTER KHA" },
    { 0x10992, "MEROITIC HIEROGLYPHIC LETTER HHA" },
    { 0x10993, "MEROITIC HIEROGLYPHIC LETTER SA" },
    { 0x10994, "MEROITIC HIEROGLYPHIC LETTER SA-2" },
    { 0x10995, "MEROITIC HIEROGLYPHIC LETTER SE" },
    { 0x10996, "MEROITIC HIEROGLYPHIC LETTER KA" },
    { 0x10997, "MEROITIC HIEROGLYPHIC LETTER QA" },
    { 0x10998, "MEROITIC HIEROGLYPHIC LETTER TA" },
    { 0x10999, "MEROITIC HIEROGLYPHIC LETTER TA-2" },
    { 0x1099A, "MEROITIC HIEROGLYPHIC LETTER TE" },
    { 0x1099B, "MEROITIC HIEROGLYPHIC LETTER TE-2" },
    { 0x1099C, "MEROITIC HIEROGLYPHIC LETTER TO" },
    { 0x1099D, "MEROITIC HIEROGLYPHIC LETTER DA" },
    { 0x1099E, "MEROITIC HIEROGLYPHIC SYMBOL VIDJ" },
    { 0x1099F, "MEROITIC HIEROGLYPHIC SYMBOL VIDJ-2" },
    { 0x109A0, "MEROITIC CURSIVE LETTER A" },
    { 0x109A1, "MEROITIC CURSIVE LETTER E" },
    { 0x109A2, "MEROITIC CURSIVE LETTER I" },
    { 0x109A3, "MEROITIC CURSIVE LETTER O" },
    { 0x109A4, "MEROITIC CURSIVE LETTER YA" },
    { 0x109A5, "MEROITIC CURSIVE LETTER WA" },
    { 0x109A6, "MEROITIC CURSIVE LETTER BA" },
    { 0x109A7, "MEROITIC CURSIVE LETTER PA" },
    { 0x109A8, "MEROITIC CURSIVE LETTER MA" },
    { 0x109A9, "MEROITIC CURSIVE LETTER NA" },
    { 0x109AA, "MEROITIC CURSIVE LETTER NE" },
    { 0x109AB, "MEROITIC CURSIVE LETTER RA" },
    { 0x109AC, "MEROITIC CURSIVE LETTER LA" },
    { 0x109AD, "MEROITIC CURSIVE LETTER KHA" },
    { 0x109AE, "MEROITIC CURSIVE LETTER HHA" },
    { 0x109AF, "MEROITIC CURSIVE LETTER SA" },
    { 0x109B0, "MEROITIC CURSIVE LETTER ARCHAIC SA" },
    { 0x109B1, "MEROITIC CURSIVE LETTER SE" },
    { 0x109B2, "MEROITIC CURSIVE LETTER KA" },
    { 0x109B3, "MEROITIC CURSIVE LETTER QA" },
    { 0x109B4, "MEROITIC CURSIVE LETTER TA" },
    { 0x109B5, "MEROITIC CURSIVE LETTER TE" },
    { 0x109B6, "MEROITIC CURSIVE LETTER TO" },
    { 0x109B7, "MEROITIC CURSIVE LETTER DA" },
    { 0x109BC, "MEROITIC CURSIVE FRACTION ELEVEN TWELFTHS" },
    { 0x109BD, "MEROITIC CURSIVE FRACTION ONE HALF" },
    { 0x109BE, "MEROITIC CURSIVE LOGOGRAM RMT" },
    { 0x109BF, "MEROITIC CURSIVE LOGOGRAM IMN" },
    { 0x109C0, "MEROITIC CURSIVE NUMBER ONE" },
    { 0x109C1, "MEROITIC CURSIVE NUMBER TWO" },
    { 0x109C2, "MEROITIC CURSIVE NUMBER THREE" },
    { 0x109C3, "MEROITIC CURSIVE NUMBER FOUR" },
    { 0x109C4, "MEROITIC CURSIVE NUMBER FIVE" },
    { 0x109C5, "MEROITIC CURSIVE NUMBER SIX" },
    { 0x109C6, "MEROITIC CURSIVE NUMBER SEVEN" },
    { 0x109C7, "MEROITIC CURSIVE NUMBER EIGHT" },
    { 0x109C8, "MEROITIC CURSIVE NUMBER NINE" },
    { 0x109C9, "MEROITIC CURSIVE NUMBER TEN" },
    { 0x109CA, "MEROITIC CURSIVE NUMBER TWENTY" },
    { 0x109CB, "MEROITIC CURSIVE NUMBER THIRTY" },
    { 0x109CC, "MEROITIC CURSIVE NUMBER FORTY" },
    { 0x109CD, "MEROITIC CURSIVE NUMBER FIFTY" },
    { 0x109CE, "MEROITIC CURSIVE NUMBER SIXTY" },
    { 0x109CF, "MEROITIC CURSIVE NUMBER SEVENTY" },
    { 0x109D2, "MEROITIC CURSIVE NUMBER ONE HUNDRED" },
    { 0x109D3, "MEROITIC CURSIVE NUMBER TWO HUNDRED" },
    { 0x109D4, "MEROITIC CURSIVE NUMBER THREE HUNDRED" },
    { 0x109D5, "MEROITIC CURSIVE NUMBER FOUR HUNDRED" },
    { 0x109D6, "MEROITIC CURSIVE NUMBER FIVE HUNDRED" },
    { 0x109D7, "MEROITIC CURSIVE NUMBER SIX HUNDRED" },
    { 0x109D8, "MEROITIC CURSIVE NUMBER SEVEN HUNDRED" },
    { 0x109D9, "MEROITIC CURSIVE NUMBER EIGHT HUNDRED" },
    { 0x109DA, "MEROITIC CURSIVE NUMBER NINE HUNDRED" },
    { 0x109DB, "MEROITIC CURSIVE NUMBER ONE THOUSAND" },
    { 0x109DC, "MEROITIC CURSIVE NUMBER TWO THOUSAND" },
    { 0x109DD, "MEROITIC CURSIVE NUMBER THREE THOUSAND" },
    { 0x109DE, "MEROITIC CURSIVE NUMBER FOUR THOUSAND" },
    { 0x109DF, "MEROITIC CURSIVE NUMBER FIVE THOUSAND" },
    { 0x109E0, "MEROITIC CURSIVE NUMBER SIX THOUSAND" },
    { 0x109E1, "MEROITIC CURSIVE NUMBER SEVEN THOUSAND" },
    { 0x109E2, "MEROITIC CURSIVE NUMBER EIGHT THOUSAND" },
    { 0x109E3, "MEROITIC CURSIVE NUMBER NINE THOUSAND" },
    { 0x109E4, "MEROITIC CURSIVE NUMBER TEN THOUSAND" },
    { 0x109E5, "MEROITIC CURSIVE NUMBER TWENTY THOUSAND" },
    { 0x109E6, "MEROITIC CURSIVE NUMBER THIRTY THOUSAND" },
    { 0x109E7, "MEROITIC CURSIVE NUMBER FORTY THOUSAND" },
    { 0x109E8, "MEROITIC CURSIVE NUMBER FIFTY THOUSAND" },
    { 0x109E9, "MEROITIC CURSIVE NUMBER SIXTY THOUSAND" },
    { 0x109EA, "MEROITIC CURSIVE NUMBER SEVENTY THOUSAND" },
    { 0x109EB, "MEROITIC CURSIVE NUMBER EIGHTY THOUSAND" },
    { 0x109EC, "MEROITIC CURSIVE NUMBER NINETY THOUSAND" },
    { 0x109ED, "MEROITIC CURSIVE NUMBER ONE HUNDRED THOUSAND" },
    { 0x109EE, "MEROITIC CURSIVE NUMBER TWO HUNDRED THOUSAND" },
    { 0x109EF, "MEROITIC CURSIVE NUMBER THREE HUNDRED THOUSAND" },
    { 0x109F0, "MEROITIC CURSIVE NUMBER FOUR HUNDRED THOUSAND" },
    { 0x109F1, "MEROITIC CURSIVE NUMBER FIVE HUNDRED THOUSAND" },
    { 0x109F2, "MEROITIC CURSIVE NUMBER SIX HUNDRED THOUSAND" },
    { 0x109F3, "MEROITIC CURSIVE NUMBER SEVEN HUNDRED THOUSAND" },
    { 0x109F4, "MEROITIC CURSIVE NUMBER EIGHT HUNDRED THOUSAND" },
    { 0x109F5, "MEROITIC CURSIVE NUMBER NINE HUNDRED THOUSAND" },
    { 0x109F6, "MEROITIC CURSIVE FRACTION ONE TWELFTH" },
    { 0x109F7, "MEROITIC CURSIVE FRACTION TWO TWELFTHS" },
    { 0x109F8, "MEROITIC CURSIVE FRACTION THREE TWELFTHS" },
    { 0x109F9, "MEROITIC CURSIVE FRACTION FOUR TWELFTHS" },
    { 0x109FA, "MEROITIC CURSIVE FRACTION FIVE TWELFTHS" },
    { 0x109FB, "MEROITIC CURSIVE FRACTION SIX TWELFTHS" },
    { 0x109FC, "MEROITIC CURSIVE FRACTION SEVEN TWELFTHS" },
    { 0x109FD, "MEROITIC CURSIVE FRACTION EIGHT TWELFTHS" },
    { 0x109FE, "MEROITIC CURSIVE FRACTION NINE TWELFTHS" },
    { 0x109FF, "MEROITIC CURSIVE FRACTION TEN TWELFTHS" },
    { 0x10A00, "KHAROSHTHI LETTER A" },
    { 0x10A01, "KHAROSHTHI VOWEL SIGN I" },
    { 0x10A02, "KHAROSHTHI VOWEL SIGN U" },
    { 0x10A03, "KHAROSHTHI VOWEL SIGN VOCALIC R" },
    { 0x10A05, "KHAROSHTHI VOWEL SIGN E" },
    { 0x10A06, "KHAROSHTHI VOWEL SIGN O" },
    { 0x10A0C, "KHAROSHTHI VOWEL LENGTH MARK" },
    { 0x10A0D, "KHAROSHTHI SIGN DOUBLE RING BELOW" },
    { 0x10A0E, "KHAROSHTHI SIGN ANUSVARA" },
    { 0x10A0F, "KHAROSHTHI SIGN VISARGA" },
    { 0x10A10, "KHAROSHTHI LETTER KA" },
    { 0x10A11, "KHAROSHTHI LETTER KHA" },
    { 0x10A12, "KHAROSHTHI LETTER GA" },
    { 0x10A13, "KHAROSHTHI LETTER GHA" },
    { 0x10A15, "KHAROSHTHI LETTER CA" },
    { 0x10A16, "KHAROSHTHI LETTER CHA" },
    { 0x10A17, "KHAROSHTHI LETTER JA" },
    { 0x10A19, "KHAROSHTHI LETTER NYA" },
    { 0x10A1A, "KHAROSHTHI LETTER TTA" },
    { 0x10A1B, "KHAROSHTHI LETTER TTHA" },
    { 0x10A1C, "KHAROSHTHI LETTER DDA" },
    { 0x10A1D, "KHAROSHTHI LETTER DDHA" },
    { 0x10A1E, "KHAROSHTHI LETTER NNA" },
    { 0x10A1F, "KHAROSHTHI LETTER TA" },
    { 0x10A20, "KHAROSHTHI LETTER THA" },
    { 0x10A21, "KHAROSHTHI LETTER DA" },
    { 0x10A22, "KHAROSHTHI LETTER DHA" },
    { 0x10A23, "KHAROSHTHI LETTER NA" },
    { 0x10A24, "KHAROSHTHI LETTER PA" },
    { 0x10A25, "KHAROSHTHI LETTER PHA" },
    { 0x10A26, "KHAROSHTHI LETTER BA" },
    { 0x10A27, "KHAROSHTHI LETTER BHA" },
    { 0x10A28, "KHAROSHTHI LETTER MA" },
    { 0x10A29, "KHAROSHTHI LETTER YA" },
    { 0x10A2A, "KHAROSHTHI LETTER RA" },
    { 0x10A2B, "KHAROSHTHI LETTER LA" },
    { 0x10A2C, "KHAROSHTHI LETTER VA" },
    { 0x10A2D, "KHAROSHTHI LETTER SHA" },
    { 0x10A2E, "KHAROSHTHI LETTER SSA" },
    { 0x10A2F, "KHAROSHTHI LETTER SA" },
    { 0x10A30, "KHAROSHTHI LETTER ZA" },
    { 0x10A31, "KHAROSHTHI LETTER HA" },
    { 0x10A32, "KHAROSHTHI LETTER KKA" },
    { 0x10A33, "KHAROSHTHI LETTER TTTHA" },
    { 0x10A34, "KHAROSHTHI LETTER TTTA" },
    { 0x10A35, "KHAROSHTHI LETTER VHA" },
    { 0x10A38, "KHAROSHTHI SIGN BAR ABOVE" },
    { 0x10A39, "KHAROSHTHI SIGN CAUDA" },
    { 0x10A3A, "KHAROSHTHI SIGN DOT BELOW" },
    { 0x10A3F, "KHAROSHTHI VIRAMA" },
    { 0x10A40, "KHAROSHTHI DIGIT ONE" },
    { 0x10A41, "KHAROSHTHI DIGIT TWO" },
    { 0x10A42, "KHAROSHTHI DIGIT THREE" },
    { 0x10A43, "KHAROSHTHI DIGIT FOUR" },
    { 0x10A44, "KHAROSHTHI NUMBER TEN" },
    { 0x10A45, "KHAROSHTHI NUMBER TWENTY" },
    { 0x10A46, "KHAROSHTHI NUMBER ONE HUNDRED" },
    { 0x10A47, "KHAROSHTHI NUMBER ONE THOUSAND" },
    { 0x10A48, "KHAROSHTHI FRACTION ONE HALF" },
    { 0x10A50, "KHAROSHTHI PUNCTUATION DOT" },
    { 0x10A51, "KHAROSHTHI PUNCTUATION SMALL CIRCLE" },
    { 0x10A52, "KHAROSHTHI PUNCTUATION CIRCLE" },
    { 0x10A53, "KHAROSHTHI PUNCTUATION CRESCENT BAR" },
    { 0x10A54, "KHAROSHTHI PUNCTUATION MANGALAM" },
    { 0x10A55, "KHAROSHTHI PUNCTUATION LOTUS" },
    { 0x10A56, "KHAROSHTHI PUNCTUATION DANDA" },
    { 0x10A57, "KHAROSHTHI PUNCTUATION DOUBLE DANDA" },
    { 0x10A58, "KHAROSHTHI PUNCTUATION LINES" },
    { 0x10A60, "OLD SOUTH ARABIAN LETTER HE" },
    { 0x10A61, "OLD SOUTH ARABIAN LETTER LAMEDH" },
    { 0x10A62, "OLD SOUTH ARABIAN LETTER HETH" },
    { 0x10A63, "OLD SOUTH ARABIAN LETTER MEM" },
    { 0x10A64, "OLD SOUTH ARABIAN LETTER QOPH" },
    { 0x10A65, "OLD SOUTH ARABIAN LETTER WAW" },
    { 0x10A66, "OLD SOUTH ARABIAN LETTER SHIN" },
    { 0x10A67, "OLD SOUTH ARABIAN LETTER RESH" },
    { 0x10A68, "OLD SOUTH ARABIAN LETTER BETH" },
    { 0x10A69, "OLD SOUTH ARABIAN LETTER TAW" },
    { 0x10A6A, "OLD SOUTH ARABIAN LETTER SAT" },
    { 0x10A6B, "OLD SOUTH ARABIAN LETTER KAPH" },
    { 0x10A6C, "OLD SOUTH ARABIAN LETTER NUN" },
    { 0x10A6D, "OLD SOUTH ARABIAN LETTER KHETH" },
    { 0x10A6E, "OLD SOUTH ARABIAN LETTER SADHE" },
    { 0x10A6F, "OLD SOUTH ARABIAN LETTER SAMEKH" },
    { 0x10A70, "OLD SOUTH ARABIAN LETTER FE" },
    { 0x10A71, "OLD SOUTH ARABIAN LETTER ALEF" },
    { 0x10A72, "OLD SOUTH ARABIAN LETTER AYN" },
    { 0x10A73, "OLD SOUTH ARABIAN LETTER DHADHE" },
    { 0x10A74, "OLD SOUTH ARABIAN LETTER GIMEL" },
    { 0x10A75, "OLD SOUTH ARABIAN LETTER DALETH" },
    { 0x10A76, "OLD SOUTH ARABIAN LETTER GHAYN" },
    { 0x10A77, "OLD SOUTH ARABIAN LETTER TETH" },
    { 0x10A78, "OLD SOUTH ARABIAN LETTER ZAYN" },
    { 0x10A79, "OLD SOUTH ARABIAN LETTER DHALETH" },
    { 0x10A7A, "OLD SOUTH ARABIAN LETTER YODH" },
    { 0x10A7B, "OLD SOUTH ARABIAN LETTER THAW" },
    { 0x10A7C, "OLD SOUTH ARABIAN LETTER THETH" },
    { 0x10A7D, "OLD SOUTH ARABIAN NUMBER ONE" },
    { 0x10A7E, "OLD SOUTH ARABIAN NUMBER FIFTY" },
    { 0x10A7F, "OLD SOUTH ARABIAN NUMERIC INDICATOR" },
    { 0x10A80, "OLD NORTH ARABIAN LETTER HEH" },
    { 0x10A81, "OLD NORTH ARABIAN LETTER LAM" },
    { 0x10A82, "OLD NORTH ARABIAN LETTER HAH" },
    { 0x10A83, "OLD NORTH ARABIAN LETTER MEEM" },
    { 0x10A84, "OLD NORTH ARABIAN LETTER QAF" },
    { 0x10A85, "OLD NORTH ARABIAN LETTER WAW" },
    { 0x10A86, "OLD NORTH ARABIAN LETTER ES-2" },
    { 0x10A87, "OLD NORTH ARABIAN LETTER REH" },
    { 0x10A88, "OLD NORTH ARABIAN LETTER BEH" },
    { 0x10A89, "OLD NORTH ARABIAN LETTER TEH" },
    { 0x10A8A, "OLD NORTH ARABIAN LETTER ES-1" },
    { 0x10A8B, "OLD NORTH ARABIAN LETTER KAF" },
    { 0x10A8C, "OLD NORTH ARABIAN LETTER NOON" },
    { 0x10A8D, "OLD NORTH ARABIAN LETTER KHAH" },
    { 0x10A8E, "OLD NORTH ARABIAN LETTER SAD" },
    { 0x10A8F, "OLD NORTH ARABIAN LETTER ES-3" },
    { 0x10A90, "OLD NORTH ARABIAN LETTER FEH" },
    { 0x10A91, "OLD NORTH ARABIAN LETTER ALEF" },
    { 0x10A92, "OLD NORTH ARABIAN LETTER AIN" },
    { 0x10A93, "OLD NORTH ARABIAN LETTER DAD" },
    { 0x10A94, "OLD NORTH ARABIAN LETTER GEEM" },
    { 0x10A95, "OLD NORTH ARABIAN LETTER DAL" },
    { 0x10A96, "OLD NORTH ARABIAN LETTER GHAIN" },
    { 0x10A97, "OLD NORTH ARABIAN LETTER TAH" },
    { 0x10A98, "OLD NORTH ARABIAN LETTER ZAIN" },
    { 0x10A99, "OLD NORTH ARABIAN LETTER THAL" },
    { 0x10A9A, "OLD NORTH ARABIAN LETTER YEH" },
    { 0x10A9B, "OLD NORTH ARABIAN LETTER THEH" },
    { 0x10A9C, "OLD NORTH ARABIAN LETTER ZAH" },
    { 0x10A9D, "OLD NORTH ARABIAN NUMBER ONE" },
    { 0x10A9E, "OLD NORTH ARABIAN NUMBER TEN" },
    { 0x10A9F, "OLD NORTH ARABIAN NUMBER TWENTY" },
    { 0x10AC0, "MANICHAEAN LETTER ALEPH" },
    { 0x10AC1, "MANICHAEAN LETTER BETH" },
    { 0x10AC2, "MANICHAEAN LETTER BHETH" },
    { 0x10AC3, "MANICHAEAN LETTER GIMEL" },
    { 0x10AC4, "MANICHAEAN LETTER GHIMEL" },
    { 0x10AC5, "MANICHAEAN LETTER DALETH" },
    { 0x10AC6, "MANICHAEAN LETTER HE" },
    { 0x10AC7, "MANICHAEAN LETTER WAW" },
    { 0x10AC8, "MANICHAEAN SIGN UD" },
    { 0x10AC9, "MANICHAEAN LETTER ZAYIN" },
    { 0x10ACA, "MANICHAEAN LETTER ZHAYIN" },
    { 0x10ACB, "MANICHAEAN LETTER JAYIN" },
    { 0x10ACC, "MANICHAEAN LETTER JHAYIN" },
    { 0x10ACD, "MANICHAEAN LETTER HETH" },
    { 0x10ACE, "MANICHAEAN LETTER TETH" },
    { 0x10ACF, "MANICHAEAN LETTER YODH" },
    { 0x10AD0, "MANICHAEAN LETTER KAPH" },
    { 0x10AD1, "MANICHAEAN LETTER XAPH" },
    { 0x10AD2, "MANICHAEAN LETTER KHAPH" },
    { 0x10AD3, "MANICHAEAN LETTER LAMEDH" },
    { 0x10AD4, "MANICHAEAN LETTER DHAMEDH" },
    { 0x10AD5, "MANICHAEAN LETTER THAMEDH" },
    { 0x10AD6, "MANICHAEAN LETTER MEM" },
    { 0x10AD7, "MANICHAEAN LETTER NUN" },
    { 0x10AD8, "MANICHAEAN LETTER SAMEKH" },
    { 0x10AD9, "MANICHAEAN LETTER AYIN" },
    { 0x10ADA, "MANICHAEAN LETTER AAYIN" },
    { 0x10ADB, "MANICHAEAN LETTER PE" },
    { 0x10ADC, "MANICHAEAN LETTER FE" },
    { 0x10ADD, "MANICHAEAN LETTER SADHE" },
    { 0x10ADE, "MANICHAEAN LETTER QOPH" },
    { 0x10ADF, "MANICHAEAN LETTER XOPH" },
    { 0x10AE0, "MANICHAEAN LETTER QHOPH" },
    { 0x10AE1, "MANICHAEAN LETTER RESH" },
    { 0x10AE2, "MANICHAEAN LETTER SHIN" },
    { 0x10AE3, "MANICHAEAN LETTER SSHIN" },
    { 0x10AE4, "MANICHAEAN LETTER TAW" },
    { 0x10AE5, "MANICHAEAN ABBREVIATION MARK ABOVE" },
    { 0x10AE6, "MANICHAEAN ABBREVIATION MARK BELOW" },
    { 0x10AEB, "MANICHAEAN NUMBER ONE" },
    { 0x10AEC, "MANICHAEAN NUMBER FIVE" },
    { 0x10AED, "MANICHAEAN NUMBER TEN" },
    { 0x10AEE, "MANICHAEAN NUMBER TWENTY" },
    { 0x10AEF, "MANICHAEAN NUMBER ONE HUNDRED" },
    { 0x10AF0, "MANICHAEAN PUNCTUATION STAR" },
    { 0x10AF1, "MANICHAEAN PUNCTUATION FLEURON" },
    { 0x10AF2, "MANICHAEAN PUNCTUATION DOUBLE DOT WITHIN DOT" },
    { 0x10AF3, "MANICHAEAN PUNCTUATION DOT WITHIN DOT" },
    { 0x10AF4, "MANICHAEAN PUNCTUATION DOT" },
    { 0x10AF5, "MANICHAEAN PUNCTUATION TWO DOTS" },
    { 0x10AF6, "MANICHAEAN PUNCTUATION LINE FILLER" },
    { 0x10B00, "AVESTAN LETTER A" },
    { 0x10B01, "AVESTAN LETTER AA" },
    { 0x10B02, "AVESTAN LETTER AO" },
    { 0x10B03, "AVESTAN LETTER AAO" },
    { 0x10B04, "AVESTAN LETTER AN" },
    { 0x10B05, "AVESTAN LETTER AAN" },
    { 0x10B06, "AVESTAN LETTER AE" },
    { 0x10B07, "AVESTAN LETTER AEE" },
    { 0x10B08, "AVESTAN LETTER E" },
    { 0x10B09, "AVESTAN LETTER EE" },
    { 0x10B0A, "AVESTAN LETTER O" },
    { 0x10B0B, "AVESTAN LETTER OO" },
    { 0x10B0C, "AVESTAN LETTER I" },
    { 0x10B0D, "AVESTAN LETTER II" },
    { 0x10B0E, "AVESTAN LETTER U" },
    { 0x10B0F, "AVESTAN LETTER UU" },
    { 0x10B10, "AVESTAN LETTER KE" },
    { 0x10B11, "AVESTAN LETTER XE" },
    { 0x10B12, "AVESTAN LETTER XYE" },
    { 0x10B13, "AVESTAN LETTER XVE" },
    { 0x10B14, "AVESTAN LETTER GE" },
    { 0x10B15, "AVESTAN LETTER GGE" },
    { 0x10B16, "AVESTAN LETTER GHE" },
    { 0x10B17, "AVESTAN LETTER CE" },
    { 0x10B18, "AVESTAN LETTER JE" },
    { 0x10B19, "AVESTAN LETTER TE" },
    { 0x10B1A, "AVESTAN LETTER THE" },
    { 0x10B1B, "AVESTAN LETTER DE" },
    { 0x10B1C, "AVESTAN LETTER DHE" },
    { 0x10B1D, "AVESTAN LETTER TTE" },
    { 0x10B1E, "AVESTAN LETTER PE" },
    { 0x10B1F, "AVESTAN LETTER FE" },
    { 0x10B20, "AVESTAN LETTER BE" },
    { 0x10B21, "AVESTAN LETTER BHE" },
    { 0x10B22, "AVESTAN LETTER NGE" },
    { 0x10B23, "AVESTAN LETTER NGYE" },
    { 0x10B24, "AVESTAN LETTER NGVE" },
    { 0x10B25, "AVESTAN LETTER NE" },
    { 0x10B26, "AVESTAN LETTER NYE" },
    { 0x10B27, "AVESTAN LETTER NNE" },
    { 0x10B28, "AVESTAN LETTER ME" },
    { 0x10B29, "AVESTAN LETTER HME" },
    { 0x10B2A, "AVESTAN LETTER YYE" },
    { 0x10B2B, "AVESTAN LETTER YE" },
    { 0x10B2C, "AVESTAN LETTER VE" },
    { 0x10B2D, "AVESTAN LETTER RE" },
    { 0x10B2E, "AVESTAN LETTER LE" },
    { 0x10B2F, "AVESTAN LETTER SE" },
    { 0x10B30, "AVESTAN LETTER ZE" },
    { 0x10B31, "AVESTAN LETTER SHE" },
    { 0x10B32, "AVESTAN LETTER ZHE" },
    { 0x10B33, "AVESTAN LETTER SHYE" },
    { 0x10B34, "AVESTAN LETTER SSHE" },
    { 0x10B35, "AVESTAN LETTER HE" },
    { 0x10B39, "AVESTAN ABBREVIATION MARK" },
    { 0x10B3A, "TINY TWO DOTS OVER ONE DOT PUNCTUATION" },
    { 0x10B3B, "SMALL TWO DOTS OVER ONE DOT PUNCTUATION" },
    { 0x10B3C, "LARGE TWO DOTS OVER ONE DOT PUNCTUATION" },
    { 0x10B3D, "LARGE ONE DOT OVER TWO DOTS PUNCTUATION" },
    { 0x10B3E, "LARGE TWO RINGS OVER ONE RING PUNCTUATION" },
    { 0x10B3F, "LARGE ONE RING OVER TWO RINGS PUNCTUATION" },
    { 0x10B40, "INSCRIPTIONAL PARTHIAN LETTER ALEPH" },
    { 0x10B41, "INSCRIPTIONAL PARTHIAN LETTER BETH" },
    { 0x10B42, "INSCRIPTIONAL PARTHIAN LETTER GIMEL" },
    { 0x10B43, "INSCRIPTIONAL PARTHIAN LETTER DALETH" },
    { 0x10B44, "INSCRIPTIONAL PARTHIAN LETTER HE" },
    { 0x10B45, "INSCRIPTIONAL PARTHIAN LETTER WAW" },
    { 0x10B46, "INSCRIPTIONAL PARTHIAN LETTER ZAYIN" },
    { 0x10B47, "INSCRIPTIONAL PARTHIAN LETTER HETH" },
    { 0x10B48, "INSCRIPTIONAL PARTHIAN LETTER TETH" },
    { 0x10B49, "INSCRIPTIONAL PARTHIAN LETTER YODH" },
    { 0x10B4A, "INSCRIPTIONAL PARTHIAN LETTER KAPH" },
    { 0x10B4B, "INSCRIPTIONAL PARTHIAN LETTER LAMEDH" },
    { 0x10B4C, "INSCRIPTIONAL PARTHIAN LETTER MEM" },
    { 0x10B4D, "INSCRIPTIONAL PARTHIAN LETTER NUN" },
    { 0x10B4E, "INSCRIPTIONAL PARTHIAN LETTER SAMEKH" },
    { 0x10B4F, "INSCRIPTIONAL PARTHIAN LETTER AYIN" },
    { 0x10B50, "INSCRIPTIONAL PARTHIAN LETTER PE" },
    { 0x10B51, "INSCRIPTIONAL PARTHIAN LETTER SADHE" },
    { 0x10B52, "INSCRIPTIONAL PARTHIAN LETTER QOPH" },
    { 0x10B53, "INSCRIPTIONAL PARTHIAN LETTER RESH" },
    { 0x10B54, "INSCRIPTIONAL PARTHIAN LETTER SHIN" },
    { 0x10B55, "INSCRIPTIONAL PARTHIAN LETTER TAW" },
    { 0x10B58, "INSCRIPTIONAL PARTHIAN NUMBER ONE" },
    { 0x10B59, "INSCRIPTIONAL PARTHIAN NUMBER TWO" },
    { 0x10B5A, "INSCRIPTIONAL PARTHIAN NUMBER THREE" },
    { 0x10B5B, "INSCRIPTIONAL PARTHIAN NUMBER FOUR" },
    { 0x10B5C, "INSCRIPTIONAL PARTHIAN NUMBER TEN" },
    { 0x10B5D, "INSCRIPTIONAL PARTHIAN NUMBER TWENTY" },
    { 0x10B5E, "INSCRIPTIONAL PARTHIAN NUMBER ONE HUNDRED" },
    { 0x10B5F, "INSCRIPTIONAL PARTHIAN NUMBER ONE THOUSAND" },
    { 0x10B60, "INSCRIPTIONAL PAHLAVI LETTER ALEPH" },
    { 0x10B61, "INSCRIPTIONAL PAHLAVI LETTER BETH" },
    { 0x10B62, "INSCRIPTIONAL PAHLAVI LETTER GIMEL" },
    { 0x10B63, "INSCRIPTIONAL PAHLAVI LETTER DALETH" },
    { 0x10B64, "INSCRIPTIONAL PAHLAVI LETTER HE" },
    { 0x10B65, "INSCRIPTIONAL PAHLAVI LETTER WAW-AYIN-RESH" },
    { 0x10B66, "INSCRIPTIONAL PAHLAVI LETTER ZAYIN" },
    { 0x10B67, "INSCRIPTIONAL PAHLAVI LETTER HETH" },
    { 0x10B68, "INSCRIPTIONAL PAHLAVI LETTER TETH" },
    { 0x10B69, "INSCRIPTIONAL PAHLAVI LETTER YODH" },
    { 0x10B6A, "INSCRIPTIONAL PAHLAVI LETTER KAPH" },
    { 0x10B6B, "INSCRIPTIONAL PAHLAVI LETTER LAMEDH" },
    { 0x10B6C, "INSCRIPTIONAL PAHLAVI LETTER MEM-QOPH" },
    { 0x10B6D, "INSCRIPTIONAL PAHLAVI LETTER NUN" },
    { 0x10B6E, "INSCRIPTIONAL PAHLAVI LETTER SAMEKH" },
    { 0x10B6F, "INSCRIPTIONAL PAHLAVI LETTER PE" },
    { 0x10B70, "INSCRIPTIONAL PAHLAVI LETTER SADHE" },
    { 0x10B71, "INSCRIPTIONAL PAHLAVI LETTER SHIN" },
    { 0x10B72, "INSCRIPTIONAL PAHLAVI LETTER TAW" },
    { 0x10B78, "INSCRIPTIONAL PAHLAVI NUMBER ONE" },
    { 0x10B79, "INSCRIPTIONAL PAHLAVI NUMBER TWO" },
    { 0x10B7A, "INSCRIPTIONAL PAHLAVI NUMBER THREE" },
    { 0x10B7B, "INSCRIPTIONAL PAHLAVI NUMBER FOUR" },
    { 0x10B7C, "INSCRIPTIONAL PAHLAVI NUMBER TEN" },
    { 0x10B7D, "INSCRIPTIONAL PAHLAVI NUMBER TWENTY" },
    { 0x10B7E, "INSCRIPTIONAL PAHLAVI NUMBER ONE HUNDRED" },
    { 0x10B7F, "INSCRIPTIONAL PAHLAVI NUMBER ONE THOUSAND" },
    { 0x10B80, "PSALTER PAHLAVI LETTER ALEPH" },
    { 0x10B81, "PSALTER PAHLAVI LETTER BETH" },
    { 0x10B82, "PSALTER PAHLAVI LETTER GIMEL" },
    { 0x10B83, "PSALTER PAHLAVI LETTER DALETH" },
    { 0x10B84, "PSALTER PAHLAVI LETTER HE" },
    { 0x10B85, "PSALTER PAHLAVI LETTER WAW-AYIN-RESH" },
    { 0x10B86, "PSALTER PAHLAVI LETTER ZAYIN" },
    { 0x10B87, "PSALTER PAHLAVI LETTER HETH" },
    { 0x10B88, "PSALTER PAHLAVI LETTER YODH" },
    { 0x10B89, "PSALTER PAHLAVI LETTER KAPH" },
    { 0x10B8A, "PSALTER PAHLAVI LETTER LAMEDH" },
    { 0x10B8B, "PSALTER PAHLAVI LETTER MEM-QOPH" },
    { 0x10B8C, "PSALTER PAHLAVI LETTER NUN" },
    { 0x10B8D, "PSALTER PAHLAVI LETTER SAMEKH" },
    { 0x10B8E, "PSALTER PAHLAVI LETTER PE" },
    { 0x10B8F, "PSALTER PAHLAVI LETTER SADHE" },
    { 0x10B90, "PSALTER PAHLAVI LETTER SHIN" },
    { 0x10B91, "PSALTER PAHLAVI LETTER TAW" },
    { 0x10B99, "PSALTER PAHLAVI SECTION MARK" },
    { 0x10B9A, "PSALTER PAHLAVI TURNED SECTION MARK" },
    { 0x10B9B, "PSALTER PAHLAVI FOUR DOTS WITH CROSS" },
    { 0x10B9C, "PSALTER PAHLAVI FOUR DOTS WITH DOT" },
    { 0x10BA9, "PSALTER PAHLAVI NUMBER ONE" },
    { 0x10BAA, "PSALTER PAHLAVI NUMBER TWO" },
    { 0x10BAB, "PSALTER PAHLAVI NUMBER THREE" },
    { 0x10BAC, "PSALTER PAHLAVI NUMBER FOUR" },
    { 0x10BAD, "PSALTER PAHLAVI NUMBER TEN" },
    { 0x10BAE, "PSALTER PAHLAVI NUMBER TWENTY" },
    { 0x10BAF, "PSALTER PAHLAVI NUMBER ONE HUNDRED" },
    { 0x10C00, "OLD TURKIC LETTER ORKHON A" },
    { 0x10C01, "OLD TURKIC LETTER YENISEI A" },
    { 0x10C02, "OLD TURKIC LETTER YENISEI AE" },
    { 0x10C03, "OLD TURKIC LETTER ORKHON I" },
    { 0x10C04, "OLD TURKIC LETTER YENISEI I" },
    { 0x10C05, "OLD TURKIC LETTER YENISEI E" },
    { 0x10C06, "OLD TURKIC LETTER ORKHON O" },
    { 0x10C07, "OLD TURKIC LETTER ORKHON OE" },
    { 0x10C08, "OLD TURKIC LETTER YENISEI OE" },
    { 0x10C09, "OLD TURKIC LETTER ORKHON AB" },
    { 0x10C0A, "OLD TURKIC LETTER YENISEI AB" },
    { 0x10C0B, "OLD TURKIC LETTER ORKHON AEB" },
    { 0x10C0C, "OLD TURKIC LETTER YENISEI AEB" },
    { 0x10C0D, "OLD TURKIC LETTER ORKHON AG" },
    { 0x10C0E, "OLD TURKIC LETTER YENISEI AG" },
    { 0x10C0F, "OLD TURKIC LETTER ORKHON AEG" },
    { 0x10C10, "OLD TURKIC LETTER YENISEI AEG" },
    { 0x10C11, "OLD TURKIC LETTER ORKHON AD" },
    { 0x10C12, "OLD TURKIC LETTER YENISEI AD" },
    { 0x10C13, "OLD TURKIC LETTER ORKHON AED" },
    { 0x10C14, "OLD TURKIC LETTER ORKHON EZ" },
    { 0x10C15, "OLD TURKIC LETTER YENISEI EZ" },
    { 0x10C16, "OLD TURKIC LETTER ORKHON AY" },
    { 0x10C17, "OLD TURKIC LETTER YENISEI AY" },
    { 0x10C18, "OLD TURKIC LETTER ORKHON AEY" },
    { 0x10C19, "OLD TURKIC LETTER YENISEI AEY" },
    { 0x10C1A, "OLD TURKIC LETTER ORKHON AEK" },
    { 0x10C1B, "OLD TURKIC LETTER YENISEI AEK" },
    { 0x10C1C, "OLD TURKIC LETTER ORKHON OEK" },
    { 0x10C1D, "OLD TURKIC LETTER YENISEI OEK" },
    { 0x10C1E, "OLD TURKIC LETTER ORKHON AL" },
    { 0x10C1F, "OLD TURKIC LETTER YENISEI AL" },
    { 0x10C20, "OLD TURKIC LETTER ORKHON AEL" },
    { 0x10C21, "OLD TURKIC LETTER ORKHON ELT" },
    { 0x10C22, "OLD TURKIC LETTER ORKHON EM" },
    { 0x10C23, "OLD TURKIC LETTER ORKHON AN" },
    { 0x10C24, "OLD TURKIC LETTER ORKHON AEN" },
    { 0x10C25, "OLD TURKIC LETTER YENISEI AEN" },
    { 0x10C26, "OLD TURKIC LETTER ORKHON ENT" },
    { 0x10C27, "OLD TURKIC LETTER YENISEI ENT" },
    { 0x10C28, "OLD TURKIC LETTER ORKHON ENC" },
    { 0x10C29, "OLD TURKIC LETTER YENISEI ENC" },
    { 0x10C2A, "OLD TURKIC LETTER ORKHON ENY" },
    { 0x10C2B, "OLD TURKIC LETTER YENISEI ENY" },
    { 0x10C2C, "OLD TURKIC LETTER YENISEI ANG" },
    { 0x10C2D, "OLD TURKIC LETTER ORKHON ENG" },
    { 0x10C2E, "OLD TURKIC LETTER YENISEI AENG" },
    { 0x10C2F, "OLD TURKIC LETTER ORKHON EP" },
    { 0x10C30, "OLD TURKIC LETTER ORKHON OP" },
    { 0x10C31, "OLD TURKIC LETTER ORKHON IC" },
    { 0x10C32, "OLD TURKIC LETTER ORKHON EC" },
    { 0x10C33, "OLD TURKIC LETTER YENISEI EC" },
    { 0x10C34, "OLD TURKIC LETTER ORKHON AQ" },
    { 0x10C35, "OLD TURKIC LETTER YENISEI AQ" },
    { 0x10C36, "OLD TURKIC LETTER ORKHON IQ" },
    { 0x10C37, "OLD TURKIC LETTER YENISEI IQ" },
    { 0x10C38, "OLD TURKIC LETTER ORKHON OQ" },
    { 0x10C39, "OLD TURKIC LETTER YENISEI OQ" },
    { 0x10C3A, "OLD TURKIC LETTER ORKHON AR" },
    { 0x10C3B, "OLD TURKIC LETTER YENISEI AR" },
    { 0x10C3C, "OLD TURKIC LETTER ORKHON AER" },
    { 0x10C3D, "OLD TURKIC LETTER ORKHON AS" },
    { 0x10C3E, "OLD TURKIC LETTER ORKHON AES" },
    { 0x10C3F, "OLD TURKIC LETTER ORKHON ASH" },
    { 0x10C40, "OLD TURKIC LETTER YENISEI ASH" },
    { 0x10C41, "OLD TURKIC LETTER ORKHON ESH" },
    { 0x10C42, "OLD TURKIC LETTER YENISEI ESH" },
    { 0x10C43, "OLD TURKIC LETTER ORKHON AT" },
    { 0x10C44, "OLD TURKIC LETTER YENISEI AT" },
    { 0x10C45, "OLD TURKIC LETTER ORKHON AET" },
    { 0x10C46, "OLD TURKIC LETTER YENISEI AET" },
    { 0x10C47, "OLD TURKIC LETTER ORKHON OT" },
    { 0x10C48, "OLD TURKIC LETTER ORKHON BASH" },
    { 0x10C80, "OLD HUNGARIAN CAPITAL LETTER A" },
    { 0x10C81, "OLD HUNGARIAN CAPITAL LETTER AA" },
    { 0x10C82, "OLD HUNGARIAN CAPITAL LETTER EB" },
    { 0x10C83, "OLD HUNGARIAN CAPITAL LETTER AMB" },
    { 0x10C84, "OLD HUNGARIAN CAPITAL LETTER EC" },
    { 0x10C85, "OLD HUNGARIAN CAPITAL LETTER ENC" },
    { 0x10C86, "OLD HUNGARIAN CAPITAL LETTER ECS" },
    { 0x10C87, "OLD HUNGARIAN CAPITAL LETTER ED" },
    { 0x10C88, "OLD HUNGARIAN CAPITAL LETTER AND" },
    { 0x10C89, "OLD HUNGARIAN CAPITAL LETTER E" },
    { 0x10C8A, "OLD HUNGARIAN CAPITAL LETTER CLOSE E" },
    { 0x10C8B, "OLD HUNGARIAN CAPITAL LETTER EE" },
    { 0x10C8C, "OLD HUNGARIAN CAPITAL LETTER EF" },
    { 0x10C8D, "OLD HUNGARIAN CAPITAL LETTER EG" },
    { 0x10C8E, "OLD HUNGARIAN CAPITAL LETTER EGY" },
    { 0x10C8F, "OLD HUNGARIAN CAPITAL LETTER EH" },
    { 0x10C90, "OLD HUNGARIAN CAPITAL LETTER I" },
    { 0x10C91, "OLD HUNGARIAN CAPITAL LETTER II" },
    { 0x10C92, "OLD HUNGARIAN CAPITAL LETTER EJ" },
    { 0x10C93, "OLD HUNGARIAN CAPITAL LETTER EK" },
    { 0x10C94, "OLD HUNGARIAN CAPITAL LETTER AK" },
    { 0x10C95, "OLD HUNGARIAN CAPITAL LETTER UNK" },
    { 0x10C96, "OLD HUNGARIAN CAPITAL LETTER EL" },
    { 0x10C97, "OLD HUNGARIAN CAPITAL LETTER ELY" },
    { 0x10C98, "OLD HUNGARIAN CAPITAL LETTER EM" },
    { 0x10C99, "OLD HUNGARIAN CAPITAL LETTER EN" },
    { 0x10C9A, "OLD HUNGARIAN CAPITAL LETTER ENY" },
    { 0x10C9B, "OLD HUNGARIAN CAPITAL LETTER O" },
    { 0x10C9C, "OLD HUNGARIAN CAPITAL LETTER OO" },
    { 0x10C9D, "OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG OE" },
    { 0x10C9E, "OLD HUNGARIAN CAPITAL LETTER RUDIMENTA OE" },
    { 0x10C9F, "OLD HUNGARIAN CAPITAL LETTER OEE" },
    { 0x10CA0, "OLD HUNGARIAN CAPITAL LETTER EP" },
    { 0x10CA1, "OLD HUNGARIAN CAPITAL LETTER EMP" },
    { 0x10CA2, "OLD HUNGARIAN CAPITAL LETTER ER" },
    { 0x10CA3, "OLD HUNGARIAN CAPITAL LETTER SHORT ER" },
    { 0x10CA4, "OLD HUNGARIAN CAPITAL LETTER ES" },
    { 0x10CA5, "OLD HUNGARIAN CAPITAL LETTER ESZ" },
    { 0x10CA6, "OLD HUNGARIAN CAPITAL LETTER ET" },
    { 0x10CA7, "OLD HUNGARIAN CAPITAL LETTER ENT" },
    { 0x10CA8, "OLD HUNGARIAN CAPITAL LETTER ETY" },
    { 0x10CA9, "OLD HUNGARIAN CAPITAL LETTER ECH" },
    { 0x10CAA, "OLD HUNGARIAN CAPITAL LETTER U" },
    { 0x10CAB, "OLD HUNGARIAN CAPITAL LETTER UU" },
    { 0x10CAC, "OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG UE" },
    { 0x10CAD, "OLD HUNGARIAN CAPITAL LETTER RUDIMENTA UE" },
    { 0x10CAE, "OLD HUNGARIAN CAPITAL LETTER EV" },
    { 0x10CAF, "OLD HUNGARIAN CAPITAL LETTER EZ" },
    { 0x10CB0, "OLD HUNGARIAN CAPITAL LETTER EZS" },
    { 0x10CB1, "OLD HUNGARIAN CAPITAL LETTER ENT-SHAPED SIGN" },
    { 0x10CB2, "OLD HUNGARIAN CAPITAL LETTER US" },
    { 0x10CC0, "OLD HUNGARIAN SMALL LETTER A" },
    { 0x10CC1, "OLD HUNGARIAN SMALL LETTER AA" },
    { 0x10CC2, "OLD HUNGARIAN SMALL LETTER EB" },
    { 0x10CC3, "OLD HUNGARIAN SMALL LETTER AMB" },
    { 0x10CC4, "OLD HUNGARIAN SMALL LETTER EC" },
    { 0x10CC5, "OLD HUNGARIAN SMALL LETTER ENC" },
    { 0x10CC6, "OLD HUNGARIAN SMALL LETTER ECS" },
    { 0x10CC7, "OLD HUNGARIAN SMALL LETTER ED" },
    { 0x10CC8, "OLD HUNGARIAN SMALL LETTER AND" },
    { 0x10CC9, "OLD HUNGARIAN SMALL LETTER E" },
    { 0x10CCA, "OLD HUNGARIAN SMALL LETTER CLOSE E" },
    { 0x10CCB, "OLD HUNGARIAN SMALL LETTER EE" },
    { 0x10CCC, "OLD HUNGARIAN SMALL LETTER EF" },
    { 0x10CCD, "OLD HUNGARIAN SMALL LETTER EG" },
    { 0x10CCE, "OLD HUNGARIAN SMALL LETTER EGY" },
    { 0x10CCF, "OLD HUNGARIAN SMALL LETTER EH" },
    { 0x10CD0, "OLD HUNGARIAN SMALL LETTER I" },
    { 0x10CD1, "OLD HUNGARIAN SMALL LETTER II" },
    { 0x10CD2, "OLD HUNGARIAN SMALL LETTER EJ" },
    { 0x10CD3, "OLD HUNGARIAN SMALL LETTER EK" },
    { 0x10CD4, "OLD HUNGARIAN SMALL LETTER AK" },
    { 0x10CD5, "OLD HUNGARIAN SMALL LETTER UNK" },
    { 0x10CD6, "OLD HUNGARIAN SMALL LETTER EL" },
    { 0x10CD7, "OLD HUNGARIAN SMALL LETTER ELY" },
    { 0x10CD8, "OLD HUNGARIAN SMALL LETTER EM" },
    { 0x10CD9, "OLD HUNGARIAN SMALL LETTER EN" },
    { 0x10CDA, "OLD HUNGARIAN SMALL LETTER ENY" },
    { 0x10CDB, "OLD HUNGARIAN SMALL LETTER O" },
    { 0x10CDC, "OLD HUNGARIAN SMALL LETTER OO" },
    { 0x10CDD, "OLD HUNGARIAN SMALL LETTER NIKOLSBURG OE" },
    { 0x10CDE, "OLD HUNGARIAN SMALL LETTER RUDIMENTA OE" },
    { 0x10CDF, "OLD HUNGARIAN SMALL LETTER OEE" },
    { 0x10CE0, "OLD HUNGARIAN SMALL LETTER EP" },
    { 0x10CE1, "OLD HUNGARIAN SMALL LETTER EMP" },
    { 0x10CE2, "OLD HUNGARIAN SMALL LETTER ER" },
    { 0x10CE3, "OLD HUNGARIAN SMALL LETTER SHORT ER" },
    { 0x10CE4, "OLD HUNGARIAN SMALL LETTER ES" },
    { 0x10CE5, "OLD HUNGARIAN SMALL LETTER ESZ" },
    { 0x10CE6, "OLD HUNGARIAN SMALL LETTER ET" },
    { 0x10CE7, "OLD HUNGARIAN SMALL LETTER ENT" },
    { 0x10CE8, "OLD HUNGARIAN SMALL LETTER ETY" },
    { 0x10CE9, "OLD HUNGARIAN SMALL LETTER ECH" },
    { 0x10CEA, "OLD HUNGARIAN SMALL LETTER U" },
    { 0x10CEB, "OLD HUNGARIAN SMALL LETTER UU" },
    { 0x10CEC, "OLD HUNGARIAN SMALL LETTER NIKOLSBURG UE" },
    { 0x10CED, "OLD HUNGARIAN SMALL LETTER RUDIMENTA UE" },
    { 0x10CEE, "OLD HUNGARIAN SMALL LETTER EV" },
    { 0x10CEF, "OLD HUNGARIAN SMALL LETTER EZ" },
    { 0x10CF0, "OLD HUNGARIAN SMALL LETTER EZS" },
    { 0x10CF1, "OLD HUNGARIAN SMALL LETTER ENT-SHAPED SIGN" },
    { 0x10CF2, "OLD HUNGARIAN SMALL LETTER US" },
    { 0x10CFA, "OLD HUNGARIAN NUMBER ONE" },
    { 0x10CFB, "OLD HUNGARIAN NUMBER FIVE" },
    { 0x10CFC, "OLD HUNGARIAN NUMBER TEN" },
    { 0x10CFD, "OLD HUNGARIAN NUMBER FIFTY" },
    { 0x10CFE, "OLD HUNGARIAN NUMBER ONE HUNDRED" },
    { 0x10CFF, "OLD HUNGARIAN NUMBER ONE THOUSAND" },
    { 0x10D00, "HANIFI ROHINGYA LETTER A" },
    { 0x10D01, "HANIFI ROHINGYA LETTER BA" },
    { 0x10D02, "HANIFI ROHINGYA LETTER PA" },
    { 0x10D03, "HANIFI ROHINGYA LETTER TA" },
    { 0x10D04, "HANIFI ROHINGYA LETTER TTA" },
    { 0x10D05, "HANIFI ROHINGYA LETTER JA" },
    { 0x10D06, "HANIFI ROHINGYA LETTER CA" },
    { 0x10D07, "HANIFI ROHINGYA LETTER HA" },
    { 0x10D08, "HANIFI ROHINGYA LETTER KHA" },
    { 0x10D09, "HANIFI ROHINGYA LETTER FA" },
    { 0x10D0A, "HANIFI ROHINGYA LETTER DA" },
    { 0x10D0B, "HANIFI ROHINGYA LETTER DDA" },
    { 0x10D0C, "HANIFI ROHINGYA LETTER RA" },
    { 0x10D0D, "HANIFI ROHINGYA LETTER RRA" },
    { 0x10D0E, "HANIFI ROHINGYA LETTER ZA" },
    { 0x10D0F, "HANIFI ROHINGYA LETTER SA" },
    { 0x10D10, "HANIFI ROHINGYA LETTER SHA" },
    { 0x10D11, "HANIFI ROHINGYA LETTER KA" },
    { 0x10D12, "HANIFI ROHINGYA LETTER GA" },
    { 0x10D13, "HANIFI ROHINGYA LETTER LA" },
    { 0x10D14, "HANIFI ROHINGYA LETTER MA" },
    { 0x10D15, "HANIFI ROHINGYA LETTER NA" },
    { 0x10D16, "HANIFI ROHINGYA LETTER WA" },
    { 0x10D17, "HANIFI ROHINGYA LETTER KINNA WA" },
    { 0x10D18, "HANIFI ROHINGYA LETTER YA" },
    { 0x10D19, "HANIFI ROHINGYA LETTER KINNA YA" },
    { 0x10D1A, "HANIFI ROHINGYA LETTER NGA" },
    { 0x10D1B, "HANIFI ROHINGYA LETTER NYA" },
    { 0x10D1C, "HANIFI ROHINGYA LETTER VA" },
    { 0x10D1D, "HANIFI ROHINGYA VOWEL A" },
    { 0x10D1E, "HANIFI ROHINGYA VOWEL I" },
    { 0x10D1F, "HANIFI ROHINGYA VOWEL U" },
    { 0x10D20, "HANIFI ROHINGYA VOWEL E" },
    { 0x10D21, "HANIFI ROHINGYA VOWEL O" },
    { 0x10D22, "HANIFI ROHINGYA MARK SAKIN" },
    { 0x10D23, "HANIFI ROHINGYA MARK NA KHONNA" },
    { 0x10D24, "HANIFI ROHINGYA SIGN HARBAHAY" },
    { 0x10D25, "HANIFI ROHINGYA SIGN TAHALA" },
    { 0x10D26, "HANIFI ROHINGYA SIGN TANA" },
    { 0x10D27, "HANIFI ROHINGYA SIGN TASSI" },
    { 0x10D30, "HANIFI ROHINGYA DIGIT ZERO" },
    { 0x10D31, "HANIFI ROHINGYA DIGIT ONE" },
    { 0x10D32, "HANIFI ROHINGYA DIGIT TWO" },
    { 0x10D33, "HANIFI ROHINGYA DIGIT THREE" },
    { 0x10D34, "HANIFI ROHINGYA DIGIT FOUR" },
    { 0x10D35, "HANIFI ROHINGYA DIGIT FIVE" },
    { 0x10D36, "HANIFI ROHINGYA DIGIT SIX" },
    { 0x10D37, "HANIFI ROHINGYA DIGIT SEVEN" },
    { 0x10D38, "HANIFI ROHINGYA DIGIT EIGHT" },
    { 0x10D39, "HANIFI ROHINGYA DIGIT NINE" },
    { 0x10E60, "RUMI DIGIT ONE" },
    { 0x10E61, "RUMI DIGIT TWO" },
    { 0x10E62, "RUMI DIGIT THREE" },
    { 0x10E63, "RUMI DIGIT FOUR" },
    { 0x10E64, "RUMI DIGIT FIVE" },
    { 0x10E65, "RUMI DIGIT SIX" },
    { 0x10E66, "RUMI DIGIT SEVEN" },
    { 0x10E67, "RUMI DIGIT EIGHT" },
    { 0x10E68, "RUMI DIGIT NINE" },
    { 0x10E69, "RUMI NUMBER TEN" },
    { 0x10E6A, "RUMI NUMBER TWENTY" },
    { 0x10E6B, "RUMI NUMBER THIRTY" },
    { 0x10E6C, "RUMI NUMBER FORTY" },
    { 0x10E6D, "RUMI NUMBER FIFTY" },
    { 0x10E6E, "RUMI NUMBER SIXTY" },
    { 0x10E6F, "RUMI NUMBER SEVENTY" },
    { 0x10E70, "RUMI NUMBER EIGHTY" },
    { 0x10E71, "RUMI NUMBER NINETY" },
    { 0x10E72, "RUMI NUMBER ONE HUNDRED" },
    { 0x10E73, "RUMI NUMBER TWO HUNDRED" },
    { 0x10E74, "RUMI NUMBER THREE HUNDRED" },
    { 0x10E75, "RUMI NUMBER FOUR HUNDRED" },
    { 0x10E76, "RUMI NUMBER FIVE HUNDRED" },
    { 0x10E77, "RUMI NUMBER SIX HUNDRED" },
    { 0x10E78, "RUMI NUMBER SEVEN HUNDRED" },
    { 0x10E79, "RUMI NUMBER EIGHT HUNDRED" },
    { 0x10E7A, "RUMI NUMBER NINE HUNDRED" },
    { 0x10E7B, "RUMI FRACTION ONE HALF" },
    { 0x10E7C, "RUMI FRACTION ONE QUARTER" },
    { 0x10E7D, "RUMI FRACTION ONE THIRD" },
    { 0x10E7E, "RUMI FRACTION TWO THIRDS" },
    { 0x10E80, "YEZIDI LETTER ELIF" },
    { 0x10E81, "YEZIDI LETTER BE" },
    { 0x10E82, "YEZIDI LETTER PE" },
    { 0x10E83, "YEZIDI LETTER PHE" },
    { 0x10E84, "YEZIDI LETTER THE" },
    { 0x10E85, "YEZIDI LETTER SE" },
    { 0x10E86, "YEZIDI LETTER CIM" },
    { 0x10E87, "YEZIDI LETTER CHIM" },
    { 0x10E88, "YEZIDI LETTER CHHIM" },
    { 0x10E89, "YEZIDI LETTER HHA" },
    { 0x10E8A, "YEZIDI LETTER XA" },
    { 0x10E8B, "YEZIDI LETTER DAL" },
    { 0x10E8C, "YEZIDI LETTER ZAL" },
    { 0x10E8D, "YEZIDI LETTER RA" },
    { 0x10E8E, "YEZIDI LETTER RHA" },
    { 0x10E8F, "YEZIDI LETTER ZA" },
    { 0x10E90, "YEZIDI LETTER JA" },
    { 0x10E91, "YEZIDI LETTER SIN" },
    { 0x10E92, "YEZIDI LETTER SHIN" },
    { 0x10E93, "YEZIDI LETTER SAD" },
    { 0x10E94, "YEZIDI LETTER DAD" },
    { 0x10E95, "YEZIDI LETTER TA" },
    { 0x10E96, "YEZIDI LETTER ZE" },
    { 0x10E97, "YEZIDI LETTER EYN" },
    { 0x10E98, "YEZIDI LETTER XHEYN" },
    { 0x10E99, "YEZIDI LETTER FA" },
    { 0x10E9A, "YEZIDI LETTER VA" },
    { 0x10E9B, "YEZIDI LETTER VA ALTERNATE FORM" },
    { 0x10E9C, "YEZIDI LETTER QAF" },
    { 0x10E9D, "YEZIDI LETTER KAF" },
    { 0x10E9E, "YEZIDI LETTER KHAF" },
    { 0x10E9F, "YEZIDI LETTER GAF" },
    { 0x10EA0, "YEZIDI LETTER LAM" },
    { 0x10EA1, "YEZIDI LETTER MIM" },
    { 0x10EA2, "YEZIDI LETTER NUN" },
    { 0x10EA3, "YEZIDI LETTER UM" },
    { 0x10EA4, "YEZIDI LETTER WAW" },
    { 0x10EA5, "YEZIDI LETTER OW" },
    { 0x10EA6, "YEZIDI LETTER EW" },
    { 0x10EA7, "YEZIDI LETTER HAY" },
    { 0x10EA8, "YEZIDI LETTER YOT" },
    { 0x10EA9, "YEZIDI LETTER ET" },
    { 0x10EAB, "YEZIDI COMBINING HAMZA MARK" },
    { 0x10EAC, "YEZIDI COMBINING MADDA MARK" },
    { 0x10EAD, "YEZIDI HYPHENATION MARK" },
    { 0x10EB0, "YEZIDI LETTER LAM WITH DOT ABOVE" },
    { 0x10EB1, "YEZIDI LETTER YOT WITH CIRCUMFLEX ABOVE" },
    { 0x10EFD, "ARABIC SMALL LOW WORD SAKTA" },
    { 0x10EFE, "ARABIC SMALL LOW WORD QASR" },
    { 0x10EFF, "ARABIC SMALL LOW WORD MADDA" },
    { 0x10F00, "OLD SOGDIAN LETTER ALEPH" },
    { 0x10F01, "OLD SOGDIAN LETTER FINAL ALEPH" },
    { 0x10F02, "OLD SOGDIAN LETTER BETH" },
    { 0x10F03, "OLD SOGDIAN LETTER FINAL BETH" },
    { 0x10F04, "OLD SOGDIAN LETTER GIMEL" },
    { 0x10F05, "OLD SOGDIAN LETTER HE" },
    { 0x10F06, "OLD SOGDIAN LETTER FINAL HE" },
    { 0x10F07, "OLD SOGDIAN LETTER WAW" },
    { 0x10F08, "OLD SOGDIAN LETTER ZAYIN" },
    { 0x10F09, "OLD SOGDIAN LETTER HETH" },
    { 0x10F0A, "OLD SOGDIAN LETTER YODH" },
    { 0x10F0B, "OLD SOGDIAN LETTER KAPH" },
    { 0x10F0C, "OLD SOGDIAN LETTER LAMEDH" },
    { 0x10F0D, "OLD SOGDIAN LETTER MEM" },
    { 0x10F0E, "OLD SOGDIAN LETTER NUN" },
    { 0x10F0F, "OLD SOGDIAN LETTER FINAL NUN" },
    { 0x10F10, "OLD SOGDIAN LETTER FINAL NUN WITH VERTICAL TAIL" },
    { 0x10F11, "OLD SOGDIAN LETTER SAMEKH" },
    { 0x10F12, "OLD SOGDIAN LETTER AYIN" },
    { 0x10F13, "OLD SOGDIAN LETTER ALTERNATE AYIN" },
    { 0x10F14, "OLD SOGDIAN LETTER PE" },
    { 0x10F15, "OLD SOGDIAN LETTER SADHE" },
    { 0x10F16, "OLD SOGDIAN LETTER FINAL SADHE" },
    { 0x10F17, "OLD SOGDIAN LETTER FINAL SADHE WITH VERTICAL TAIL" },
    { 0x10F18, "OLD SOGDIAN LETTER RESH-AYIN-DALETH" },
    { 0x10F19, "OLD SOGDIAN LETTER SHIN" },
    { 0x10F1A, "OLD SOGDIAN LETTER TAW" },
    { 0x10F1B, "OLD SOGDIAN LETTER FINAL TAW" },
    { 0x10F1C, "OLD SOGDIAN LETTER FINAL TAW WITH VERTICAL TAIL" },
    { 0x10F1D, "OLD SOGDIAN NUMBER ONE" },
    { 0x10F1E, "OLD SOGDIAN NUMBER TWO" },
    { 0x10F1F, "OLD SOGDIAN NUMBER THREE" },
    { 0x10F20, "OLD SOGDIAN NUMBER FOUR" },
    { 0x10F21, "OLD SOGDIAN NUMBER FIVE" },
    { 0x10F22, "OLD SOGDIAN NUMBER TEN" },
    { 0x10F23, "OLD SOGDIAN NUMBER TWENTY" },
    { 0x10F24, "OLD SOGDIAN NUMBER THIRTY" },
    { 0x10F25, "OLD SOGDIAN NUMBER ONE HUNDRED" },
    { 0x10F26, "OLD SOGDIAN FRACTION ONE HALF" },
    { 0x10F27, "OLD SOGDIAN LIGATURE AYIN-DALETH" },
    { 0x10F30, "SOGDIAN LETTER ALEPH" },
    { 0x10F31, "SOGDIAN LETTER BETH" },
    { 0x10F32, "SOGDIAN LETTER GIMEL" },
    { 0x10F33, "SOGDIAN LETTER HE" },
    { 0x10F34, "SOGDIAN LETTER WAW" },
    { 0x10F35, "SOGDIAN LETTER ZAYIN" },
    { 0x10F36, "SOGDIAN LETTER HETH" },
    { 0x10F37, "SOGDIAN LETTER YODH" },
    { 0x10F38, "SOGDIAN LETTER KAPH" },
    { 0x10F39, "SOGDIAN LETTER LAMEDH" },
    { 0x10F3A, "SOGDIAN LETTER MEM" },
    { 0x10F3B, "SOGDIAN LETTER NUN" },
    { 0x10F3C, "SOGDIAN LETTER SAMEKH" },
    { 0x10F3D, "SOGDIAN LETTER AYIN" },
    { 0x10F3E, "SOGDIAN LETTER PE" },
    { 0x10F3F, "SOGDIAN LETTER SADHE" },
    { 0x10F40, "SOGDIAN LETTER RESH-AYIN" },
    { 0x10F41, "SOGDIAN LETTER SHIN" },
    { 0x10F42, "SOGDIAN LETTER TAW" },
    { 0x10F43, "SOGDIAN LETTER FETH" },
    { 0x10F44, "SOGDIAN LETTER LESH" },
    { 0x10F45, "SOGDIAN INDEPENDENT SHIN" },
    { 0x10F46, "SOGDIAN COMBINING DOT BELOW" },
    { 0x10F47, "SOGDIAN COMBINING TWO DOTS BELOW" },
    { 0x10F48, "SOGDIAN COMBINING DOT ABOVE" },
    { 0x10F49, "SOGDIAN COMBINING TWO DOTS ABOVE" },
    { 0x10F4A, "SOGDIAN COMBINING CURVE ABOVE" },
    { 0x10F4B, "SOGDIAN COMBINING CURVE BELOW" },
    { 0x10F4C, "SOGDIAN COMBINING HOOK ABOVE" },
    { 0x10F4D, "SOGDIAN COMBINING HOOK BELOW" },
    { 0x10F4E, "SOGDIAN COMBINING LONG HOOK BELOW" },
    { 0x10F4F, "SOGDIAN COMBINING RESH BELOW" },
    { 0x10F50, "SOGDIAN COMBINING STROKE BELOW" },
    { 0x10F51, "SOGDIAN NUMBER ONE" },
    { 0x10F52, "SOGDIAN NUMBER TEN" },
    { 0x10F53, "SOGDIAN NUMBER TWENTY" },
    { 0x10F54, "SOGDIAN NUMBER ONE HUNDRED" },
    { 0x10F55, "SOGDIAN PUNCTUATION TWO VERTICAL BARS" },
    { 0x10F56, "SOGDIAN PUNCTUATION TWO VERTICAL BARS WITH DOTS" },
    { 0x10F57, "SOGDIAN PUNCTUATION CIRCLE WITH DOT" },
    { 0x10F58, "SOGDIAN PUNCTUATION TWO CIRCLES WITH DOTS" },
    { 0x10F59, "SOGDIAN PUNCTUATION HALF CIRCLE WITH DOT" },
    { 0x10F70, "OLD UYGHUR LETTER ALEPH" },
    { 0x10F71, "OLD UYGHUR LETTER BETH" },
    { 0x10F72, "OLD UYGHUR LETTER GIMEL-HETH" },
    { 0x10F73, "OLD UYGHUR LETTER WAW" },
    { 0x10F74, "OLD UYGHUR LETTER ZAYIN" },
    { 0x10F75, "OLD UYGHUR LETTER FINAL HETH" },
    { 0x10F76, "OLD UYGHUR LETTER YODH" },
    { 0x10F77, "OLD UYGHUR LETTER KAPH" },
    { 0x10F78, "OLD UYGHUR LETTER LAMEDH" },
    { 0x10F79, "OLD UYGHUR LETTER MEM" },
    { 0x10F7A, "OLD UYGHUR LETTER NUN" },
    { 0x10F7B, "OLD UYGHUR LETTER SAMEKH" },
    { 0x10F7C, "OLD UYGHUR LETTER PE" },
    { 0x10F7D, "OLD UYGHUR LETTER SADHE" },
    { 0x10F7E, "OLD UYGHUR LETTER RESH" },
    { 0x10F7F, "OLD UYGHUR LETTER SHIN" },
    { 0x10F80, "OLD UYGHUR LETTER TAW" },
    { 0x10F81, "OLD UYGHUR LETTER LESH" },
    { 0x10F82, "OLD UYGHUR COMBINING DOT ABOVE" },
    { 0x10F83, "OLD UYGHUR COMBINING DOT BELOW" },
    { 0x10F84, "OLD UYGHUR COMBINING TWO DOTS ABOVE" },
    { 0x10F85, "OLD UYGHUR COMBINING TWO DOTS BELOW" },
    { 0x10F86, "OLD UYGHUR PUNCTUATION BAR" },
    { 0x10F87, "OLD UYGHUR PUNCTUATION TWO BARS" },
    { 0x10F88, "OLD UYGHUR PUNCTUATION TWO DOTS" },
    { 0x10F89, "OLD UYGHUR PUNCTUATION FOUR DOTS" },
    { 0x10FB0, "CHORASMIAN LETTER ALEPH" },
    { 0x10FB1, "CHORASMIAN LETTER SMALL ALEPH" },
    { 0x10FB2, "CHORASMIAN LETTER BETH" },
    { 0x10FB3, "CHORASMIAN LETTER GIMEL" },
    { 0x10FB4, "CHORASMIAN LETTER DALETH" },
    { 0x10FB5, "CHORASMIAN LETTER HE" },
    { 0x10FB6, "CHORASMIAN LETTER WAW" },
    { 0x10FB7, "CHORASMIAN LETTER CURLED WAW" },
    { 0x10FB8, "CHORASMIAN LETTER ZAYIN" },
    { 0x10FB9, "CHORASMIAN LETTER HETH" },
    { 0x10FBA, "CHORASMIAN LETTER YODH" },
    { 0x10FBB, "CHORASMIAN LETTER KAPH" },
    { 0x10FBC, "CHORASMIAN LETTER LAMEDH" },
    { 0x10FBD, "CHORASMIAN LETTER MEM" },
    { 0x10FBE, "CHORASMIAN LETTER NUN" },
    { 0x10FBF, "CHORASMIAN LETTER SAMEKH" },
    { 0x10FC0, "CHORASMIAN LETTER AYIN" },
    { 0x10FC1, "CHORASMIAN LETTER PE" },
    { 0x10FC2, "CHORASMIAN LETTER RESH" },
    { 0x10FC3, "CHORASMIAN LETTER SHIN" },
    { 0x10FC4, "CHORASMIAN LETTER TAW" },
    { 0x10FC5, "CHORASMIAN NUMBER ONE" },
    { 0x10FC6, "CHORASMIAN NUMBER TWO" },
    { 0x10FC7, "CHORASMIAN NUMBER THREE" },
    { 0x10FC8, "CHORASMIAN NUMBER FOUR" },
    { 0x10FC9, "CHORASMIAN NUMBER TEN" },
    { 0x10FCA, "CHORASMIAN NUMBER TWENTY" },
    { 0x10FCB, "CHORASMIAN NUMBER ONE HUNDRED" },
    { 0x10FE0, "ELYMAIC LETTER ALEPH" },
    { 0x10FE1, "ELYMAIC LETTER BETH" },
    { 0x10FE2, "ELYMAIC LETTER GIMEL" },
    { 0x10FE3, "ELYMAIC LETTER DALETH" },
    { 0x10FE4, "ELYMAIC LETTER HE" },
    { 0x10FE5, "ELYMAIC LETTER WAW" },
    { 0x10FE6, "ELYMAIC LETTER ZAYIN" },
    { 0x10FE7, "ELYMAIC LETTER HETH" },
    { 0x10FE8, "ELYMAIC LETTER TETH" },
    { 0x10FE9, "ELYMAIC LETTER YODH" },
    { 0x10FEA, "ELYMAIC LETTER KAPH" },
    { 0x10FEB, "ELYMAIC LETTER LAMEDH" },
    { 0x10FEC, "ELYMAIC LETTER MEM" },
    { 0x10FED, "ELYMAIC LETTER NUN" },
    { 0x10FEE, "ELYMAIC LETTER SAMEKH" },
    { 0x10FEF, "ELYMAIC LETTER AYIN" },
    { 0x10FF0, "ELYMAIC LETTER PE" },
    { 0x10FF1, "ELYMAIC LETTER SADHE" },
    { 0x10FF2, "ELYMAIC LETTER QOPH" },
    { 0x10FF3, "ELYMAIC LETTER RESH" },
    { 0x10FF4, "ELYMAIC LETTER SHIN" },
    { 0x10FF5, "ELYMAIC LETTER TAW" },
    { 0x10FF6, "ELYMAIC LIGATURE ZAYIN-YODH" },
    { 0x11000, "BRAHMI SIGN CANDRABINDU" },
    { 0x11001, "BRAHMI SIGN ANUSVARA" },
    { 0x11002, "BRAHMI SIGN VISARGA" },
    { 0x11003, "BRAHMI SIGN JIHVAMULIYA" },
    { 0x11004, "BRAHMI SIGN UPADHMANIYA" },
    { 0x11005, "BRAHMI LETTER A" },
    { 0x11006, "BRAHMI LETTER AA" },
    { 0x11007, "BRAHMI LETTER I" },
    { 0x11008, "BRAHMI LETTER II" },
    { 0x11009, "BRAHMI LETTER U" },
    { 0x1100A, "BRAHMI LETTER UU" },
    { 0x1100B, "BRAHMI LETTER VOCALIC R" },
    { 0x1100C, "BRAHMI LETTER VOCALIC RR" },
    { 0x1100D, "BRAHMI LETTER VOCALIC L" },
    { 0x1100E, "BRAHMI LETTER VOCALIC LL" },
    { 0x1100F, "BRAHMI LETTER E" },
    { 0x11010, "BRAHMI LETTER AI" },
    { 0x11011, "BRAHMI LETTER O" },
    { 0x11012, "BRAHMI LETTER AU" },
    { 0x11013, "BRAHMI LETTER KA" },
    { 0x11014, "BRAHMI LETTER KHA" },
    { 0x11015, "BRAHMI LETTER GA" },
    { 0x11016, "BRAHMI LETTER GHA" },
    { 0x11017, "BRAHMI LETTER NGA" },
    { 0x11018, "BRAHMI LETTER CA" },
    { 0x11019, "BRAHMI LETTER CHA" },
    { 0x1101A, "BRAHMI LETTER JA" },
    { 0x1101B, "BRAHMI LETTER JHA" },
    { 0x1101C, "BRAHMI LETTER NYA" },
    { 0x1101D, "BRAHMI LETTER TTA" },
    { 0x1101E, "BRAHMI LETTER TTHA" },
    { 0x1101F, "BRAHMI LETTER DDA" },
    { 0x11020, "BRAHMI LETTER DDHA" },
    { 0x11021, "BRAHMI LETTER NNA" },
    { 0x11022, "BRAHMI LETTER TA" },
    { 0x11023, "BRAHMI LETTER THA" },
    { 0x11024, "BRAHMI LETTER DA" },
    { 0x11025, "BRAHMI LETTER DHA" },
    { 0x11026, "BRAHMI LETTER NA" },
    { 0x11027, "BRAHMI LETTER PA" },
    { 0x11028, "BRAHMI LETTER PHA" },
    { 0x11029, "BRAHMI LETTER BA" },
    { 0x1102A, "BRAHMI LETTER BHA" },
    { 0x1102B, "BRAHMI LETTER MA" },
    { 0x1102C, "BRAHMI LETTER YA" },
    { 0x1102D, "BRAHMI LETTER RA" },
    { 0x1102E, "BRAHMI LETTER LA" },
    { 0x1102F, "BRAHMI LETTER VA" },
    { 0x11030, "BRAHMI LETTER SHA" },
    { 0x11031, "BRAHMI LETTER SSA" },
    { 0x11032, "BRAHMI LETTER SA" },
    { 0x11033, "BRAHMI LETTER HA" },
    { 0x11034, "BRAHMI LETTER LLA" },
    { 0x11035, "BRAHMI LETTER OLD TAMIL LLLA" },
    { 0x11036, "BRAHMI LETTER OLD TAMIL RRA" },
    { 0x11037, "BRAHMI LETTER OLD TAMIL NNNA" },
    { 0x11038, "BRAHMI VOWEL SIGN AA" },
    { 0x11039, "BRAHMI VOWEL SIGN BHATTIPROLU AA" },
    { 0x1103A, "BRAHMI VOWEL SIGN I" },
    { 0x1103B, "BRAHMI VOWEL SIGN II" },
    { 0x1103C, "BRAHMI VOWEL SIGN U" },
    { 0x1103D, "BRAHMI VOWEL SIGN UU" },
    { 0x1103E, "BRAHMI VOWEL SIGN VOCALIC R" },
    { 0x1103F, "BRAHMI VOWEL SIGN VOCALIC RR" },
    { 0x11040, "BRAHMI VOWEL SIGN VOCALIC L" },
    { 0x11041, "BRAHMI VOWEL SIGN VOCALIC LL" },
    { 0x11042, "BRAHMI VOWEL SIGN E" },
    { 0x11043, "BRAHMI VOWEL SIGN AI" },
    { 0x11044, "BRAHMI VOWEL SIGN O" },
    { 0x11045, "BRAHMI VOWEL SIGN AU" },
    { 0x11046, "BRAHMI VIRAMA" },
    { 0x11047, "BRAHMI DANDA" },
    { 0x11048, "BRAHMI DOUBLE DANDA" },
    { 0x11049, "BRAHMI PUNCTUATION DOT" },
    { 0x1104A, "BRAHMI PUNCTUATION DOUBLE DOT" },
    { 0x1104B, "BRAHMI PUNCTUATION LINE" },
    { 0x1104C, "BRAHMI PUNCTUATION CRESCENT BAR" },
    { 0x1104D, "BRAHMI PUNCTUATION LOTUS" },
    { 0x11052, "BRAHMI NUMBER ONE" },
    { 0x11053, "BRAHMI NUMBER TWO" },
    { 0x11054, "BRAHMI NUMBER THREE" },
    { 0x11055, "BRAHMI NUMBER FOUR" },
    { 0x11056, "BRAHMI NUMBER FIVE" },
    { 0x11057, "BRAHMI NUMBER SIX" },
    { 0x11058, "BRAHMI NUMBER SEVEN" },
    { 0x11059, "BRAHMI NUMBER EIGHT" },
    { 0x1105A, "BRAHMI NUMBER NINE" },
    { 0x1105B, "BRAHMI NUMBER TEN" },
    { 0x1105C, "BRAHMI NUMBER TWENTY" },
    { 0x1105D, "BRAHMI NUMBER THIRTY" },
    { 0x1105E, "BRAHMI NUMBER FORTY" },
    { 0x1105F, "BRAHMI NUMBER FIFTY" },
    { 0x11060, "BRAHMI NUMBER SIXTY" },
    { 0x11061, "BRAHMI NUMBER SEVENTY" },
    { 0x11062, "BRAHMI NUMBER EIGHTY" },
    { 0x11063, "BRAHMI NUMBER NINETY" },
    { 0x11064, "BRAHMI NUMBER ONE HUNDRED" },
    { 0x11065, "BRAHMI NUMBER ONE THOUSAND" },
    { 0x11066, "BRAHMI DIGIT ZERO" },
    { 0x11067, "BRAHMI DIGIT ONE" },
    { 0x11068, "BRAHMI DIGIT TWO" },
    { 0x11069, "BRAHMI DIGIT THREE" },
    { 0x1106A, "BRAHMI DIGIT FOUR" },
    { 0x1106B, "BRAHMI DIGIT FIVE" },
    { 0x1106C, "BRAHMI DIGIT SIX" },
    { 0x1106D, "BRAHMI DIGIT SEVEN" },
    { 0x1106E, "BRAHMI DIGIT EIGHT" },
    { 0x1106F, "BRAHMI DIGIT NINE" },
    { 0x11070, "BRAHMI SIGN OLD TAMIL VIRAMA" },
    { 0x11071, "BRAHMI LETTER OLD TAMIL SHORT E" },
    { 0x11072, "BRAHMI LETTER OLD TAMIL SHORT O" },
    { 0x11073, "BRAHMI VOWEL SIGN OLD TAMIL SHORT E" },
    { 0x11074, "BRAHMI VOWEL SIGN OLD TAMIL SHORT O" },
    { 0x11075, "BRAHMI LETTER OLD TAMIL LLA" },
    { 0x1107F, "BRAHMI NUMBER JOINER" },
    { 0x11080, "KAITHI SIGN CANDRABINDU" },
    { 0x11081, "KAITHI SIGN ANUSVARA" },
    { 0x11082, "KAITHI SIGN VISARGA" },
    { 0x11083, "KAITHI LETTER A" },
    { 0x11084, "KAITHI LETTER AA" },
    { 0x11085, "KAITHI LETTER I" },
    { 0x11086, "KAITHI LETTER II" },
    { 0x11087, "KAITHI LETTER U" },
    { 0x11088, "KAITHI LETTER UU" },
    { 0x11089, "KAITHI LETTER E" },
    { 0x1108A, "KAITHI LETTER AI" },
    { 0x1108B, "KAITHI LETTER O" },
    { 0x1108C, "KAITHI LETTER AU" },
    { 0x1108D, "KAITHI LETTER KA" },
    { 0x1108E, "KAITHI LETTER KHA" },
    { 0x1108F, "KAITHI LETTER GA" },
    { 0x11090, "KAITHI LETTER GHA" },
    { 0x11091, "KAITHI LETTER NGA" },
    { 0x11092, "KAITHI LETTER CA" },
    { 0x11093, "KAITHI LETTER CHA" },
    { 0x11094, "KAITHI LETTER JA" },
    { 0x11095, "KAITHI LETTER JHA" },
    { 0x11096, "KAITHI LETTER NYA" },
    { 0x11097, "KAITHI LETTER TTA" },
    { 0x11098, "KAITHI LETTER TTHA" },
    { 0x11099, "KAITHI LETTER DDA" },
    { 0x1109A, "KAITHI LETTER DDDHA" },
    { 0x1109B, "KAITHI LETTER DDHA" },
    { 0x1109C, "KAITHI LETTER RHA" },
    { 0x1109D, "KAITHI LETTER NNA" },
    { 0x1109E, "KAITHI LETTER TA" },
    { 0x1109F, "KAITHI LETTER THA" },
    { 0x110A0, "KAITHI LETTER DA" },
    { 0x110A1, "KAITHI LETTER DHA" },
    { 0x110A2, "KAITHI LETTER NA" },
    { 0x110A3, "KAITHI LETTER PA" },
    { 0x110A4, "KAITHI LETTER PHA" },
    { 0x110A5, "KAITHI LETTER BA" },
    { 0x110A6, "KAITHI LETTER BHA" },
    { 0x110A7, "KAITHI LETTER MA" },
    { 0x110A8, "KAITHI LETTER YA" },
    { 0x110A9, "KAITHI LETTER RA" },
    { 0x110AA, "KAITHI LETTER LA" },
    { 0x110AB, "KAITHI LETTER VA" },
    { 0x110AC, "KAITHI LETTER SHA" },
    { 0x110AD, "KAITHI LETTER SSA" },
    { 0x110AE, "KAITHI LETTER SA" },
    { 0x110AF, "KAITHI LETTER HA" },
    { 0x110B0, "KAITHI VOWEL SIGN AA" },
    { 0x110B1, "KAITHI VOWEL SIGN I" },
    { 0x110B2, "KAITHI VOWEL SIGN II" },
    { 0x110B3, "KAITHI VOWEL SIGN U" },
    { 0x110B4, "KAITHI VOWEL SIGN UU" },
    { 0x110B5, "KAITHI VOWEL SIGN E" },
    { 0x110B6, "KAITHI VOWEL SIGN AI" },
    { 0x110B7, "KAITHI VOWEL SIGN O" },
    { 0x110B8, "KAITHI VOWEL SIGN AU" },
    { 0x110B9, "KAITHI SIGN VIRAMA" },
    { 0x110BA, "KAITHI SIGN NUKTA" },
    { 0x110BB, "KAITHI ABBREVIATION SIGN" },
    { 0x110BC, "KAITHI ENUMERATION SIGN" },
    { 0x110BD, "KAITHI NUMBER SIGN" },
    { 0x110BE, "KAITHI SECTION MARK" },
    { 0x110BF, "KAITHI DOUBLE SECTION MARK" },
    { 0x110C0, "KAITHI DANDA" },
    { 0x110C1, "KAITHI DOUBLE DANDA" },
    { 0x110C2, "KAITHI VOWEL SIGN VOCALIC R" },
    { 0x110CD, "KAITHI NUMBER SIGN ABOVE" },
    { 0x110D0, "SORA SOMPENG LETTER SAH" },
    { 0x110D1, "SORA SOMPENG LETTER TAH" },
    { 0x110D2, "SORA SOMPENG LETTER BAH" },
    { 0x110D3, "SORA SOMPENG LETTER CAH" },
    { 0x110D4, "SORA SOMPENG LETTER DAH" },
    { 0x110D5, "SORA SOMPENG LETTER GAH" },
    { 0x110D6, "SORA SOMPENG LETTER MAH" },
    { 0x110D7, "SORA SOMPENG LETTER NGAH" },
    { 0x110D8, "SORA SOMPENG LETTER LAH" },
    { 0x110D9, "SORA SOMPENG LETTER NAH" },
    { 0x110DA, "SORA SOMPENG LETTER VAH" },
    { 0x110DB, "SORA SOMPENG LETTER PAH" },
    { 0x110DC, "SORA SOMPENG LETTER YAH" },
    { 0x110DD, "SORA SOMPENG LETTER RAH" },
    { 0x110DE, "SORA SOMPENG LETTER HAH" },
    { 0x110DF, "SORA SOMPENG LETTER KAH" },
    { 0x110E0, "SORA SOMPENG LETTER JAH" },
    { 0x110E1, "SORA SOMPENG LETTER NYAH" },
    { 0x110E2, "SORA SOMPENG LETTER AH" },
    { 0x110E3, "SORA SOMPENG LETTER EEH" },
    { 0x110E4, "SORA SOMPENG LETTER IH" },
    { 0x110E5, "SORA SOMPENG LETTER UH" },
    { 0x110E6, "SORA SOMPENG LETTER OH" },
    { 0x110E7, "SORA SOMPENG LETTER EH" },
    { 0x110E8, "SORA SOMPENG LETTER MAE" },
    { 0x110F0, "SORA SOMPENG DIGIT ZERO" },
    { 0x110F1, "SORA SOMPENG DIGIT ONE" },
    { 0x110F2, "SORA SOMPENG DIGIT TWO" },
    { 0x110F3, "SORA SOMPENG DIGIT THREE" },
    { 0x110F4, "SORA SOMPENG DIGIT FOUR" },
    { 0x110F5, "SORA SOMPENG DIGIT FIVE" },
    { 0x110F6, "SORA SOMPENG DIGIT SIX" },
    { 0x110F7, "SORA SOMPENG DIGIT SEVEN" },
    { 0x110F8, "SORA SOMPENG DIGIT EIGHT" },
    { 0x110F9, "SORA SOMPENG DIGIT NINE" },
    { 0x11100, "CHAKMA SIGN CANDRABINDU" },
    { 0x11101, "CHAKMA SIGN ANUSVARA" },
    { 0x11102, "CHAKMA SIGN VISARGA" },
    { 0x11103, "CHAKMA LETTER AA" },
    { 0x11104, "CHAKMA LETTER I" },
    { 0x11105, "CHAKMA LETTER U" },
    { 0x11106, "CHAKMA LETTER E" },
    { 0x11107, "CHAKMA LETTER KAA" },
    { 0x11108, "CHAKMA LETTER KHAA" },
    { 0x11109, "CHAKMA LETTER GAA" },
    { 0x1110A, "CHAKMA LETTER GHAA" },
    { 0x1110B, "CHAKMA LETTER NGAA" },
    { 0x1110C, "CHAKMA LETTER CAA" },
    { 0x1110D, "CHAKMA LETTER CHAA" },
    { 0x1110E, "CHAKMA LETTER JAA" },
    { 0x1110F, "CHAKMA LETTER JHAA" },
    { 0x11110, "CHAKMA LETTER NYAA" },
    { 0x11111, "CHAKMA LETTER TTAA" },
    { 0x11112, "CHAKMA LETTER TTHAA" },
    { 0x11113, "CHAKMA LETTER DDAA" },
    { 0x11114, "CHAKMA LETTER DDHAA" },
    { 0x11115, "CHAKMA LETTER NNAA" },
    { 0x11116, "CHAKMA LETTER TAA" },
    { 0x11117, "CHAKMA LETTER THAA" },
    { 0x11118, "CHAKMA LETTER DAA" },
    { 0x11119, "CHAKMA LETTER DHAA" },
    { 0x1111A, "CHAKMA LETTER NAA" },
    { 0x1111B, "CHAKMA LETTER PAA" },
    { 0x1111C, "CHAKMA LETTER PHAA" },
    { 0x1111D, "CHAKMA LETTER BAA" },
    { 0x1111E, "CHAKMA LETTER BHAA" },
    { 0x1111F, "CHAKMA LETTER MAA" },
    { 0x11120, "CHAKMA LETTER YYAA" },
    { 0x11121, "CHAKMA LETTER YAA" },
    { 0x11122, "CHAKMA LETTER RAA" },
    { 0x11123, "CHAKMA LETTER LAA" },
    { 0x11124, "CHAKMA LETTER WAA" },
    { 0x11125, "CHAKMA LETTER SAA" },
    { 0x11126, "CHAKMA LETTER HAA" },
    { 0x11127, "CHAKMA VOWEL SIGN A" },
    { 0x11128, "CHAKMA VOWEL SIGN I" },
    { 0x11129, "CHAKMA VOWEL SIGN II" },
    { 0x1112A, "CHAKMA VOWEL SIGN U" },
    { 0x1112B, "CHAKMA VOWEL SIGN UU" },
    { 0x1112C, "CHAKMA VOWEL SIGN E" },
    { 0x1112D, "CHAKMA VOWEL SIGN AI" },
    { 0x1112E, "CHAKMA VOWEL SIGN O" },
    { 0x1112F, "CHAKMA VOWEL SIGN AU" },
    { 0x11130, "CHAKMA VOWEL SIGN OI" },
    { 0x11131, "CHAKMA O MARK" },
    { 0x11132, "CHAKMA AU MARK" },
    { 0x11133, "CHAKMA VIRAMA" },
    { 0x11134, "CHAKMA MAAYYAA" },
    { 0x11136, "CHAKMA DIGIT ZERO" },
    { 0x11137, "CHAKMA DIGIT ONE" },
    { 0x11138, "CHAKMA DIGIT TWO" },
    { 0x11139, "CHAKMA DIGIT THREE" },
    { 0x1113A, "CHAKMA DIGIT FOUR" },
    { 0x1113B, "CHAKMA DIGIT FIVE" },
    { 0x1113C, "CHAKMA DIGIT SIX" },
    { 0x1113D, "CHAKMA DIGIT SEVEN" },
    { 0x1113E, "CHAKMA DIGIT EIGHT" },
    { 0x1113F, "CHAKMA DIGIT NINE" },
    { 0x11140, "CHAKMA SECTION MARK" },
    { 0x11141, "CHAKMA DANDA" },
    { 0x11142, "CHAKMA DOUBLE DANDA" },
    { 0x11143, "CHAKMA QUESTION MARK" },
    { 0x11144, "CHAKMA LETTER LHAA" },
    { 0x11145, "CHAKMA VOWEL SIGN AA" },
    { 0x11146, "CHAKMA VOWEL SIGN EI" },
    { 0x11147, "CHAKMA LETTER VAA" },
    { 0x11150, "MAHAJANI LETTER A" },
    { 0x11151, "MAHAJANI LETTER I" },
    { 0x11152, "MAHAJANI LETTER U" },
    { 0x11153, "MAHAJANI LETTER E" },
    { 0x11154, "MAHAJANI LETTER O" },
    { 0x11155, "MAHAJANI LETTER KA" },
    { 0x11156, "MAHAJANI LETTER KHA" },
    { 0x11157, "MAHAJANI LETTER GA" },
    { 0x11158, "MAHAJANI LETTER GHA" },
    { 0x11159, "MAHAJANI LETTER CA" },
    { 0x1115A, "MAHAJANI LETTER CHA" },
    { 0x1115B, "MAHAJANI LETTER JA" },
    { 0x1115C, "MAHAJANI LETTER JHA" },
    { 0x1115D, "MAHAJANI LETTER NYA" },
    { 0x1115E, "MAHAJANI LETTER TTA" },
    { 0x1115F, "MAHAJANI LETTER TTHA" },
    { 0x11160, "MAHAJANI LETTER DDA" },
    { 0x11161, "MAHAJANI LETTER DDHA" },
    { 0x11162, "MAHAJANI LETTER NNA" },
    { 0x11163, "MAHAJANI LETTER TA" },
    { 0x11164, "MAHAJANI LETTER THA" },
    { 0x11165, "MAHAJANI LETTER DA" },
    { 0x11166, "MAHAJANI LETTER DHA" },
    { 0x11167, "MAHAJANI LETTER NA" },
    { 0x11168, "MAHAJANI LETTER PA" },
    { 0x11169, "MAHAJANI LETTER PHA" },
    { 0x1116A, "MAHAJANI LETTER BA" },
    { 0x1116B, "MAHAJANI LETTER BHA" },
    { 0x1116C, "MAHAJANI LETTER MA" },
    { 0x1116D, "MAHAJANI LETTER RA" },
    { 0x1116E, "MAHAJANI LETTER LA" },
    { 0x1116F, "MAHAJANI LETTER VA" },
    { 0x11170, "MAHAJANI LETTER SA" },
    { 0x11171, "MAHAJANI LETTER HA" },
    { 0x11172, "MAHAJANI LETTER RRA" },
    { 0x11173, "MAHAJANI SIGN NUKTA" },
    { 0x11174, "MAHAJANI ABBREVIATION SIGN" },
    { 0x11175, "MAHAJANI SECTION MARK" },
    { 0x11176, "MAHAJANI LIGATURE SHRI" },
    { 0x11180, "SHARADA SIGN CANDRABINDU" },
    { 0x11181, "SHARADA SIGN ANUSVARA" },
    { 0x11182, "SHARADA SIGN VISARGA" },
    { 0x11183, "SHARADA LETTER A" },
    { 0x11184, "SHARADA LETTER AA" },
    { 0x11185, "SHARADA LETTER I" },
    { 0x11186, "SHARADA LETTER II" },
    { 0x11187, "SHARADA LETTER U" },
    { 0x11188, "SHARADA LETTER UU" },
    { 0x11189, "SHARADA LETTER VOCALIC R" },
    { 0x1118A, "SHARADA LETTER VOCALIC RR" },
    { 0x1118B, "SHARADA LETTER VOCALIC L" },
    { 0x1118C, "SHARADA LETTER VOCALIC LL" },
    { 0x1118D, "SHARADA LETTER E" },
    { 0x1118E, "SHARADA LETTER AI" },
    { 0x1118F, "SHARADA LETTER O" },
    { 0x11190, "SHARADA LETTER AU" },
    { 0x11191, "SHARADA LETTER KA" },
    { 0x11192, "SHARADA LETTER KHA" },
    { 0x11193, "SHARADA LETTER GA" },
    { 0x11194, "SHARADA LETTER GHA" },
    { 0x11195, "SHARADA LETTER NGA" },
    { 0x11196, "SHARADA LETTER CA" },
    { 0x11197, "SHARADA LETTER CHA" },
    { 0x11198, "SHARADA LETTER JA" },
    { 0x11199, "SHARADA LETTER JHA" },
    { 0x1119A, "SHARADA LETTER NYA" },
    { 0x1119B, "SHARADA LETTER TTA" },
    { 0x1119C, "SHARADA LETTER TTHA" },
    { 0x1119D, "SHARADA LETTER DDA" },
    { 0x1119E, "SHARADA LETTER DDHA" },
    { 0x1119F, "SHARADA LETTER NNA" },
    { 0x111A0, "SHARADA LETTER TA" },
    { 0x111A1, "SHARADA LETTER THA" },
    { 0x111A2, "SHARADA LETTER DA" },
    { 0x111A3, "SHARADA LETTER DHA" },
    { 0x111A4, "SHARADA LETTER NA" },
    { 0x111A5, "SHARADA LETTER PA" },
    { 0x111A6, "SHARADA LETTER PHA" },
    { 0x111A7, "SHARADA LETTER BA" },
    { 0x111A8, "SHARADA LETTER BHA" },
    { 0x111A9, "SHARADA LETTER MA" },
    { 0x111AA, "SHARADA LETTER YA" },
    { 0x111AB, "SHARADA LETTER RA" },
    { 0x111AC, "SHARADA LETTER LA" },
    { 0x111AD, "SHARADA LETTER LLA" },
    { 0x111AE, "SHARADA LETTER VA" },
    { 0x111AF, "SHARADA LETTER SHA" },
    { 0x111B0, "SHARADA LETTER SSA" },
    { 0x111B1, "SHARADA LETTER SA" },
    { 0x111B2, "SHARADA LETTER HA" },
    { 0x111B3, "SHARADA VOWEL SIGN AA" },
    { 0x111B4, "SHARADA VOWEL SIGN I" },
    { 0x111B5, "SHARADA VOWEL SIGN II" },
    { 0x111B6, "SHARADA VOWEL SIGN U" },
    { 0x111B7, "SHARADA VOWEL SIGN UU" },
    { 0x111B8, "SHARADA VOWEL SIGN VOCALIC R" },
    { 0x111B9, "SHARADA VOWEL SIGN VOCALIC RR" },
    { 0x111BA, "SHARADA VOWEL SIGN VOCALIC L" },
    { 0x111BB, "SHARADA VOWEL SIGN VOCALIC LL" },
    { 0x111BC, "SHARADA VOWEL SIGN E" },
    { 0x111BD, "SHARADA VOWEL SIGN AI" },
    { 0x111BE, "SHARADA VOWEL SIGN O" },
    { 0x111BF, "SHARADA VOWEL SIGN AU" },
    { 0x111C0, "SHARADA SIGN VIRAMA" },
    { 0x111C1, "SHARADA SIGN AVAGRAHA" },
    { 0x111C2, "SHARADA SIGN JIHVAMULIYA" },
    { 0x111C3, "SHARADA SIGN UPADHMANIYA" },
    { 0x111C4, "SHARADA OM" },
    { 0x111C5, "SHARADA DANDA" },
    { 0x111C6, "SHARADA DOUBLE DANDA" },
    { 0x111C7, "SHARADA ABBREVIATION SIGN" },
    { 0x111C8, "SHARADA SEPARATOR" },
    { 0x111C9, "SHARADA SANDHI MARK" },
    { 0x111CA, "SHARADA SIGN NUKTA" },
    { 0x111CB, "SHARADA VOWEL MODIFIER MARK" },
    { 0x111CC, "SHARADA EXTRA SHORT VOWEL MARK" },
    { 0x111CD, "SHARADA SUTRA MARK" },
    { 0x111CE, "SHARADA VOWEL SIGN PRISHTHAMATRA E" },
    { 0x111CF, "SHARADA SIGN INVERTED CANDRABINDU" },
    { 0x111D0, "SHARADA DIGIT ZERO" },
    { 0x111D1, "SHARADA DIGIT ONE" },
    { 0x111D2, "SHARADA DIGIT TWO" },
    { 0x111D3, "SHARADA DIGIT THREE" },
    { 0x111D4, "SHARADA DIGIT FOUR" },
    { 0x111D5, "SHARADA DIGIT FIVE" },
    { 0x111D6, "SHARADA DIGIT SIX" },
    { 0x111D7, "SHARADA DIGIT SEVEN" },
    { 0x111D8, "SHARADA DIGIT EIGHT" },
    { 0x111D9, "SHARADA DIGIT NINE" },
    { 0x111DA, "SHARADA EKAM" },
    { 0x111DB, "SHARADA SIGN SIDDHAM" },
    { 0x111DC, "SHARADA HEADSTROKE" },
    { 0x111DD, "SHARADA CONTINUATION SIGN" },
    { 0x111DE, "SHARADA SECTION MARK-1" },
    { 0x111DF, "SHARADA SECTION MARK-2" },
    { 0x111E1, "SINHALA ARCHAIC DIGIT ONE" },
    { 0x111E2, "SINHALA ARCHAIC DIGIT TWO" },
    { 0x111E3, "SINHALA ARCHAIC DIGIT THREE" },
    { 0x111E4, "SINHALA ARCHAIC DIGIT FOUR" },
    { 0x111E5, "SINHALA ARCHAIC DIGIT FIVE" },
    { 0x111E6, "SINHALA ARCHAIC DIGIT SIX" },
    { 0x111E7, "SINHALA ARCHAIC DIGIT SEVEN" },
    { 0x111E8, "SINHALA ARCHAIC DIGIT EIGHT" },
    { 0x111E9, "SINHALA ARCHAIC DIGIT NINE" },
    { 0x111EA, "SINHALA ARCHAIC NUMBER TEN" },
    { 0x111EB, "SINHALA ARCHAIC NUMBER TWENTY" },
    { 0x111EC, "SINHALA ARCHAIC NUMBER THIRTY" },
    { 0x111ED, "SINHALA ARCHAIC NUMBER FORTY" },
    { 0x111EE, "SINHALA ARCHAIC NUMBER FIFTY" },
    { 0x111EF, "SINHALA ARCHAIC NUMBER SIXTY" },
    { 0x111F0, "SINHALA ARCHAIC NUMBER SEVENTY" },
    { 0x111F1, "SINHALA ARCHAIC NUMBER EIGHTY" },
    { 0x111F2, "SINHALA ARCHAIC NUMBER NINETY" },
    { 0x111F3, "SINHALA ARCHAIC NUMBER ONE HUNDRED" },
    { 0x111F4, "SINHALA ARCHAIC NUMBER ONE THOUSAND" },
    { 0x11200, "KHOJKI LETTER A" },
    { 0x11201, "KHOJKI LETTER AA" },
    { 0x11202, "KHOJKI LETTER I" },
    { 0x11203, "KHOJKI LETTER U" },
    { 0x11204, "KHOJKI LETTER E" },
    { 0x11205, "KHOJKI LETTER AI" },
    { 0x11206, "KHOJKI LETTER O" },
    { 0x11207, "KHOJKI LETTER AU" },
    { 0x11208, "KHOJKI LETTER KA" },
    { 0x11209, "KHOJKI LETTER KHA" },
    { 0x1120A, "KHOJKI LETTER GA" },
    { 0x1120B, "KHOJKI LETTER GGA" },
    { 0x1120C, "KHOJKI LETTER GHA" },
    { 0x1120D, "KHOJKI LETTER NGA" },
    { 0x1120E, "KHOJKI LETTER CA" },
    { 0x1120F, "KHOJKI LETTER CHA" },
    { 0x11210, "KHOJKI LETTER JA" },
    { 0x11211, "KHOJKI LETTER JJA" },
    { 0x11213, "KHOJKI LETTER NYA" },
    { 0x11214, "KHOJKI LETTER TTA" },
    { 0x11215, "KHOJKI LETTER TTHA" },
    { 0x11216, "KHOJKI LETTER DDA" },
    { 0x11217, "KHOJKI LETTER DDHA" },
    { 0x11218, "KHOJKI LETTER NNA" },
    { 0x11219, "KHOJKI LETTER TA" },
    { 0x1121A, "KHOJKI LETTER THA" },
    { 0x1121B, "KHOJKI LETTER DA" },
    { 0x1121C, "KHOJKI LETTER DDDA" },
    { 0x1121D, "KHOJKI LETTER DHA" },
    { 0x1121E, "KHOJKI LETTER NA" },
    { 0x1121F, "KHOJKI LETTER PA" },
    { 0x11220, "KHOJKI LETTER PHA" },
    { 0x11221, "KHOJKI LETTER BA" },
    { 0x11222, "KHOJKI LETTER BBA" },
    { 0x11223, "KHOJKI LETTER BHA" },
    { 0x11224, "KHOJKI LETTER MA" },
    { 0x11225, "KHOJKI LETTER YA" },
    { 0x11226, "KHOJKI LETTER RA" },
    { 0x11227, "KHOJKI LETTER LA" },
    { 0x11228, "KHOJKI LETTER VA" },
    { 0x11229, "KHOJKI LETTER SA" },
    { 0x1122A, "KHOJKI LETTER HA" },
    { 0x1122B, "KHOJKI LETTER LLA" },
    { 0x1122C, "KHOJKI VOWEL SIGN AA" },
    { 0x1122D, "KHOJKI VOWEL SIGN I" },
    { 0x1122E, "KHOJKI VOWEL SIGN II" },
    { 0x1122F, "KHOJKI VOWEL SIGN U" },
    { 0x11230, "KHOJKI VOWEL SIGN E" },
    { 0x11231, "KHOJKI VOWEL SIGN AI" },
    { 0x11232, "KHOJKI VOWEL SIGN O" },
    { 0x11233, "KHOJKI VOWEL SIGN AU" },
    { 0x11234, "KHOJKI SIGN ANUSVARA" },
    { 0x11235, "KHOJKI SIGN VIRAMA" },
    { 0x11236, "KHOJKI SIGN NUKTA" },
    { 0x11237, "KHOJKI SIGN SHADDA" },
    { 0x11238, "KHOJKI DANDA" },
    { 0x11239, "KHOJKI DOUBLE DANDA" },
    { 0x1123A, "KHOJKI WORD SEPARATOR" },
    { 0x1123B, "KHOJKI SECTION MARK" },
    { 0x1123C, "KHOJKI DOUBLE SECTION MARK" },
    { 0x1123D, "KHOJKI ABBREVIATION SIGN" },
    { 0x1123E, "KHOJKI SIGN SUKUN" },
    { 0x1123F, "KHOJKI LETTER QA" },
    { 0x11240, "KHOJKI LETTER SHORT I" },
    { 0x11241, "KHOJKI VOWEL SIGN VOCALIC R" },
    { 0x11280, "MULTANI LETTER A" },
    { 0x11281, "MULTANI LETTER I" },
    { 0x11282, "MULTANI LETTER U" },
    { 0x11283, "MULTANI LETTER E" },
    { 0x11284, "MULTANI LETTER KA" },
    { 0x11285, "MULTANI LETTER KHA" },
    { 0x11286, "MULTANI LETTER GA" },
    { 0x11288, "MULTANI LETTER GHA" },
    { 0x1128A, "MULTANI LETTER CA" },
    { 0x1128B, "MULTANI LETTER CHA" },
    { 0x1128C, "MULTANI LETTER JA" },
    { 0x1128D, "MULTANI LETTER JJA" },
    { 0x1128F, "MULTANI LETTER NYA" },
    { 0x11290, "MULTANI LETTER TTA" },
    { 0x11291, "MULTANI LETTER TTHA" },
    { 0x11292, "MULTANI LETTER DDA" },
    { 0x11293, "MULTANI LETTER DDDA" },
    { 0x11294, "MULTANI LETTER DDHA" },
    { 0x11295, "MULTANI LETTER NNA" },
    { 0x11296, "MULTANI LETTER TA" },
    { 0x11297, "MULTANI LETTER THA" },
    { 0x11298, "MULTANI LETTER DA" },
    { 0x11299, "MULTANI LETTER DHA" },
    { 0x1129A, "MULTANI LETTER NA" },
    { 0x1129B, "MULTANI LETTER PA" },
    { 0x1129C, "MULTANI LETTER PHA" },
    { 0x1129D, "MULTANI LETTER BA" },
    { 0x1129F, "MULTANI LETTER BHA" },
    { 0x112A0, "MULTANI LETTER MA" },
    { 0x112A1, "MULTANI LETTER YA" },
    { 0x112A2, "MULTANI LETTER RA" },
    { 0x112A3, "MULTANI LETTER LA" },
    { 0x112A4, "MULTANI LETTER VA" },
    { 0x112A5, "MULTANI LETTER SA" },
    { 0x112A6, "MULTANI LETTER HA" },
    { 0x112A7, "MULTANI LETTER RRA" },
    { 0x112A8, "MULTANI LETTER RHA" },
    { 0x112A9, "MULTANI SECTION MARK" },
    { 0x112B0, "KHUDAWADI LETTER A" },
    { 0x112B1, "KHUDAWADI LETTER AA" },
    { 0x112B2, "KHUDAWADI LETTER I" },
    { 0x112B3, "KHUDAWADI LETTER II" },
    { 0x112B4, "KHUDAWADI LETTER U" },
    { 0x112B5, "KHUDAWADI LETTER UU" },
    { 0x112B6, "KHUDAWADI LETTER E" },
    { 0x112B7, "KHUDAWADI LETTER AI" },
    { 0x112B8, "KHUDAWADI LETTER O" },
    { 0x112B9, "KHUDAWADI LETTER AU" },
    { 0x112BA, "KHUDAWADI LETTER KA" },
    { 0x112BB, "KHUDAWADI LETTER KHA" },
    { 0x112BC, "KHUDAWADI LETTER GA" },
    { 0x112BD, "KHUDAWADI LETTER GGA" },
    { 0x112BE, "KHUDAWADI LETTER GHA" },
    { 0x112BF, "KHUDAWADI LETTER NGA" },
    { 0x112C0, "KHUDAWADI LETTER CA" },
    { 0x112C1, "KHUDAWADI LETTER CHA" },
    { 0x112C2, "KHUDAWADI LETTER JA" },
    { 0x112C3, "KHUDAWADI LETTER JJA" },
    { 0x112C4, "KHUDAWADI LETTER JHA" },
    { 0x112C5, "KHUDAWADI LETTER NYA" },
    { 0x112C6, "KHUDAWADI LETTER TTA" },
    { 0x112C7, "KHUDAWADI LETTER TTHA" },
    { 0x112C8, "KHUDAWADI LETTER DDA" },
    { 0x112C9, "KHUDAWADI LETTER DDDA" },
    { 0x112CA, "KHUDAWADI LETTER RRA" },
    { 0x112CB, "KHUDAWADI LETTER DDHA" },
    { 0x112CC, "KHUDAWADI LETTER NNA" },
    { 0x112CD, "KHUDAWADI LETTER TA" },
    { 0x112CE, "KHUDAWADI LETTER THA" },
    { 0x112CF, "KHUDAWADI LETTER DA" },
    { 0x112D0, "KHUDAWADI LETTER DHA" },
    { 0x112D1, "KHUDAWADI LETTER NA" },
    { 0x112D2, "KHUDAWADI LETTER PA" },
    { 0x112D3, "KHUDAWADI LETTER PHA" },
    { 0x112D4, "KHUDAWADI LETTER BA" },
    { 0x112D5, "KHUDAWADI LETTER BBA" },
    { 0x112D6, "KHUDAWADI LETTER BHA" },
    { 0x112D7, "KHUDAWADI LETTER MA" },
    { 0x112D8, "KHUDAWADI LETTER YA" },
    { 0x112D9, "KHUDAWADI LETTER RA" },
    { 0x112DA, "KHUDAWADI LETTER LA" },
    { 0x112DB, "KHUDAWADI LETTER VA" },
    { 0x112DC, "KHUDAWADI LETTER SHA" },
    { 0x112DD, "KHUDAWADI LETTER SA" },
    { 0x112DE, "KHUDAWADI LETTER HA" },
    { 0x112DF, "KHUDAWADI SIGN ANUSVARA" },
    { 0x112E0, "KHUDAWADI VOWEL SIGN AA" },
    { 0x112E1, "KHUDAWADI VOWEL SIGN I" },
    { 0x112E2, "KHUDAWADI VOWEL SIGN II" },
    { 0x112E3, "KHUDAWADI VOWEL SIGN U" },
    { 0x112E4, "KHUDAWADI VOWEL SIGN UU" },
    { 0x112E5, "KHUDAWADI VOWEL SIGN E" },
    { 0x112E6, "KHUDAWADI VOWEL SIGN AI" },
    { 0x112E7, "KHUDAWADI VOWEL SIGN O" },
    { 0x112E8, "KHUDAWADI VOWEL SIGN AU" },
    { 0x112E9, "KHUDAWADI SIGN NUKTA" },
    { 0x112EA, "KHUDAWADI SIGN VIRAMA" },
    { 0x112F0, "KHUDAWADI DIGIT ZERO" },
    { 0x112F1, "KHUDAWADI DIGIT ONE" },
    { 0x112F2, "KHUDAWADI DIGIT TWO" },
    { 0x112F3, "KHUDAWADI DIGIT THREE" },
    { 0x112F4, "KHUDAWADI DIGIT FOUR" },
    { 0x112F5, "KHUDAWADI DIGIT FIVE" },
    { 0x112F6, "KHUDAWADI DIGIT SIX" },
    { 0x112F7, "KHUDAWADI DIGIT SEVEN" },
    { 0x112F8, "KHUDAWADI DIGIT EIGHT" },
    { 0x112F9, "KHUDAWADI DIGIT NINE" },
    { 0x11300, "GRANTHA SIGN COMBINING ANUSVARA ABOVE" },
    { 0x11301, "GRANTHA SIGN CANDRABINDU" },
    { 0x11302, "GRANTHA SIGN ANUSVARA" },
    { 0x11303, "GRANTHA SIGN VISARGA" },
    { 0x11305, "GRANTHA LETTER A" },
    { 0x11306, "GRANTHA LETTER AA" },
    { 0x11307, "GRANTHA LETTER I" },
    { 0x11308, "GRANTHA LETTER II" },
    { 0x11309, "GRANTHA LETTER U" },
    { 0x1130A, "GRANTHA LETTER UU" },
    { 0x1130B, "GRANTHA LETTER VOCALIC R" },
    { 0x1130C, "GRANTHA LETTER VOCALIC L" },
    { 0x1130F, "GRANTHA LETTER EE" },
    { 0x11310, "GRANTHA LETTER AI" },
    { 0x11313, "GRANTHA LETTER OO" },
    { 0x11314, "GRANTHA LETTER AU" },
    { 0x11315, "GRANTHA LETTER KA" },
    { 0x11316, "GRANTHA LETTER KHA" },
    { 0x11317, "GRANTHA LETTER GA" },
    { 0x11318, "GRANTHA LETTER GHA" },
    { 0x11319, "GRANTHA LETTER NGA" },
    { 0x1131A, "GRANTHA LETTER CA" },
    { 0x1131B, "GRANTHA LETTER CHA" },
    { 0x1131C, "GRANTHA LETTER JA" },
    { 0x1131D, "GRANTHA LETTER JHA" },
    { 0x1131E, "GRANTHA LETTER NYA" },
    { 0x1131F, "GRANTHA LETTER TTA" },
    { 0x11320, "GRANTHA LETTER TTHA" },
    { 0x11321, "GRANTHA LETTER DDA" },
    { 0x11322, "GRANTHA LETTER DDHA" },
    { 0x11323, "GRANTHA LETTER NNA" },
    { 0x11324, "GRANTHA LETTER TA" },
    { 0x11325, "GRANTHA LETTER THA" },
    { 0x11326, "GRANTHA LETTER DA" },
    { 0x11327, "GRANTHA LETTER DHA" },
    { 0x11328, "GRANTHA LETTER NA" },
    { 0x1132A, "GRANTHA LETTER PA" },
    { 0x1132B, "GRANTHA LETTER PHA" },
    { 0x1132C, "GRANTHA LETTER BA" },
    { 0x1132D, "GRANTHA LETTER BHA" },
    { 0x1132E, "GRANTHA LETTER MA" },
    { 0x1132F, "GRANTHA LETTER YA" },
    { 0x11330, "GRANTHA LETTER RA" },
    { 0x11332, "GRANTHA LETTER LA" },
    { 0x11333, "GRANTHA LETTER LLA" },
    { 0x11335, "GRANTHA LETTER VA" },
    { 0x11336, "GRANTHA LETTER SHA" },
    { 0x11337, "GRANTHA LETTER SSA" },
    { 0x11338, "GRANTHA LETTER SA" },
    { 0x11339, "GRANTHA LETTER HA" },
    { 0x1133B, "COMBINING BINDU BELOW" },
    { 0x1133C, "GRANTHA SIGN NUKTA" },
    { 0x1133D, "GRANTHA SIGN AVAGRAHA" },
    { 0x1133E, "GRANTHA VOWEL SIGN AA" },
    { 0x1133F, "GRANTHA VOWEL SIGN I" },
    { 0x11340, "GRANTHA VOWEL SIGN II" },
    { 0x11341, "GRANTHA VOWEL SIGN U" },
    { 0x11342, "GRANTHA VOWEL SIGN UU" },
    { 0x11343, "GRANTHA VOWEL SIGN VOCALIC R" },
    { 0x11344, "GRANTHA VOWEL SIGN VOCALIC RR" },
    { 0x11347, "GRANTHA VOWEL SIGN EE" },
    { 0x11348, "GRANTHA VOWEL SIGN AI" },
    { 0x1134B, "GRANTHA VOWEL SIGN OO" },
    { 0x1134C, "GRANTHA VOWEL SIGN AU" },
    { 0x1134D, "GRANTHA SIGN VIRAMA" },
    { 0x11350, "GRANTHA OM" },
    { 0x11357, "GRANTHA AU LENGTH MARK" },
    { 0x1135D, "GRANTHA SIGN PLUTA" },
    { 0x1135E, "GRANTHA LETTER VEDIC ANUSVARA" },
    { 0x1135F, "GRANTHA LETTER VEDIC DOUBLE ANUSVARA" },
    { 0x11360, "GRANTHA LETTER VOCALIC RR" },
    { 0x11361, "GRANTHA LETTER VOCALIC LL" },
    { 0x11362, "GRANTHA VOWEL SIGN VOCALIC L" },
    { 0x11363, "GRANTHA VOWEL SIGN VOCALIC LL" },
    { 0x11366, "COMBINING GRANTHA DIGIT ZERO" },
    { 0x11367, "COMBINING GRANTHA DIGIT ONE" },
    { 0x11368, "COMBINING GRANTHA DIGIT TWO" },
    { 0x11369, "COMBINING GRANTHA DIGIT THREE" },
    { 0x1136A, "COMBINING GRANTHA DIGIT FOUR" },
    { 0x1136B, "COMBINING GRANTHA DIGIT FIVE" },
    { 0x1136C, "COMBINING GRANTHA DIGIT SIX" },
    { 0x11370, "COMBINING GRANTHA LETTER A" },
    { 0x11371, "COMBINING GRANTHA LETTER KA" },
    { 0x11372, "COMBINING GRANTHA LETTER NA" },
    { 0x11373, "COMBINING GRANTHA LETTER VI" },
    { 0x11374, "COMBINING GRANTHA LETTER PA" },
    { 0x11400, "NEWA LETTER A" },
    { 0x11401, "NEWA LETTER AA" },
    { 0x11402, "NEWA LETTER I" },
    { 0x11403, "NEWA LETTER II" },
    { 0x11404, "NEWA LETTER U" },
    { 0x11405, "NEWA LETTER UU" },
    { 0x11406, "NEWA LETTER VOCALIC R" },
    { 0x11407, "NEWA LETTER VOCALIC RR" },
    { 0x11408, "NEWA LETTER VOCALIC L" },
    { 0x11409, "NEWA LETTER VOCALIC LL" },
    { 0x1140A, "NEWA LETTER E" },
    { 0x1140B, "NEWA LETTER AI" },
    { 0x1140C, "NEWA LETTER O" },
    { 0x1140D, "NEWA LETTER AU" },
    { 0x1140E, "NEWA LETTER KA" },
    { 0x1140F, "NEWA LETTER KHA" },
    { 0x11410, "NEWA LETTER GA" },
    { 0x11411, "NEWA LETTER GHA" },
    { 0x11412, "NEWA LETTER NGA" },
    { 0x11413, "NEWA LETTER NGHA" },
    { 0x11414, "NEWA LETTER CA" },
    { 0x11415, "NEWA LETTER CHA" },
    { 0x11416, "NEWA LETTER JA" },
    { 0x11417, "NEWA LETTER JHA" },
    { 0x11418, "NEWA LETTER NYA" },
    { 0x11419, "NEWA LETTER NYHA" },
    { 0x1141A, "NEWA LETTER TTA" },
    { 0x1141B, "NEWA LETTER TTHA" },
    { 0x1141C, "NEWA LETTER DDA" },
    { 0x1141D, "NEWA LETTER DDHA" },
    { 0x1141E, "NEWA LETTER NNA" },
    { 0x1141F, "NEWA LETTER TA" },
    { 0x11420, "NEWA LETTER THA" },
    { 0x11421, "NEWA LETTER DA" },
    { 0x11422, "NEWA LETTER DHA" },
    { 0x11423, "NEWA LETTER NA" },
    { 0x11424, "NEWA LETTER NHA" },
    { 0x11425, "NEWA LETTER PA" },
    { 0x11426, "NEWA LETTER PHA" },
    { 0x11427, "NEWA LETTER BA" },
    { 0x11428, "NEWA LETTER BHA" },
    { 0x11429, "NEWA LETTER MA" },
    { 0x1142A, "NEWA LETTER MHA" },
    { 0x1142B, "NEWA LETTER YA" },
    { 0x1142C, "NEWA LETTER RA" },
    { 0x1142D, "NEWA LETTER RHA" },
    { 0x1142E, "NEWA LETTER LA" },
    { 0x1142F, "NEWA LETTER LHA" },
    { 0x11430, "NEWA LETTER WA" },
    { 0x11431, "NEWA LETTER SHA" },
    { 0x11432, "NEWA LETTER SSA" },
    { 0x11433, "NEWA LETTER SA" },
    { 0x11434, "NEWA LETTER HA" },
    { 0x11435, "NEWA VOWEL SIGN AA" },
    { 0x11436, "NEWA VOWEL SIGN I" },
    { 0x11437, "NEWA VOWEL SIGN II" },
    { 0x11438, "NEWA VOWEL SIGN U" },
    { 0x11439, "NEWA VOWEL SIGN UU" },
    { 0x1143A, "NEWA VOWEL SIGN VOCALIC R" },
    { 0x1143B, "NEWA VOWEL SIGN VOCALIC RR" },
    { 0x1143C, "NEWA VOWEL SIGN VOCALIC L" },
    { 0x1143D, "NEWA VOWEL SIGN VOCALIC LL" },
    { 0x1143E, "NEWA VOWEL SIGN E" },
    { 0x1143F, "NEWA VOWEL SIGN AI" },
    { 0x11440, "NEWA VOWEL SIGN O" },
    { 0x11441, "NEWA VOWEL SIGN AU" },
    { 0x11442, "NEWA SIGN VIRAMA" },
    { 0x11443, "NEWA SIGN CANDRABINDU" },
    { 0x11444, "NEWA SIGN ANUSVARA" },
    { 0x11445, "NEWA SIGN VISARGA" },
    { 0x11446, "NEWA SIGN NUKTA" },
    { 0x11447, "NEWA SIGN AVAGRAHA" },
    { 0x11448, "NEWA SIGN FINAL ANUSVARA" },
    { 0x11449, "NEWA OM" },
    { 0x1144A, "NEWA SIDDHI" },
    { 0x1144B, "NEWA DANDA" },
    { 0x1144C, "NEWA DOUBLE DANDA" },
    { 0x1144D, "NEWA COMMA" },
    { 0x1144E, "NEWA GAP FILLER" },
    { 0x1144F, "NEWA ABBREVIATION SIGN" },
    { 0x11450, "NEWA DIGIT ZERO" },
    { 0x11451, "NEWA DIGIT ONE" },
    { 0x11452, "NEWA DIGIT TWO" },
    { 0x11453, "NEWA DIGIT THREE" },
    { 0x11454, "NEWA DIGIT FOUR" },
    { 0x11455, "NEWA DIGIT FIVE" },
    { 0x11456, "NEWA DIGIT SIX" },
    { 0x11457, "NEWA DIGIT SEVEN" },
    { 0x11458, "NEWA DIGIT EIGHT" },
    { 0x11459, "NEWA DIGIT NINE" },
    { 0x1145A, "NEWA DOUBLE COMMA" },
    { 0x1145B, "NEWA PLACEHOLDER MARK" },
    { 0x1145D, "NEWA INSERTION SIGN" },
    { 0x1145E, "NEWA SANDHI MARK" },
    { 0x1145F, "NEWA LETTER VEDIC ANUSVARA" },
    { 0x11460, "NEWA SIGN JIHVAMULIYA" },
    { 0x11461, "NEWA SIGN UPADHMANIYA" },
    { 0x11480, "TIRHUTA ANJI" },
    { 0x11481, "TIRHUTA LETTER A" },
    { 0x11482, "TIRHUTA LETTER AA" },
    { 0x11483, "TIRHUTA LETTER I" },
    { 0x11484, "TIRHUTA LETTER II" },
    { 0x11485, "TIRHUTA LETTER U" },
    { 0x11486, "TIRHUTA LETTER UU" },
    { 0x11487, "TIRHUTA LETTER VOCALIC R" },
    { 0x11488, "TIRHUTA LETTER VOCALIC RR" },
    { 0x11489, "TIRHUTA LETTER VOCALIC L" },
    { 0x1148A, "TIRHUTA LETTER VOCALIC LL" },
    { 0x1148B, "TIRHUTA LETTER E" },
    { 0x1148C, "TIRHUTA LETTER AI" },
    { 0x1148D, "TIRHUTA LETTER O" },
    { 0x1148E, "TIRHUTA LETTER AU" },
    { 0x1148F, "TIRHUTA LETTER KA" },
    { 0x11490, "TIRHUTA LETTER KHA" },
    { 0x11491, "TIRHUTA LETTER GA" },
    { 0x11492, "TIRHUTA LETTER GHA" },
    { 0x11493, "TIRHUTA LETTER NGA" },
    { 0x11494, "TIRHUTA LETTER CA" },
    { 0x11495, "TIRHUTA LETTER CHA" },
    { 0x11496, "TIRHUTA LETTER JA" },
    { 0x11497, "TIRHUTA LETTER JHA" },
    { 0x11498, "TIRHUTA LETTER NYA" },
    { 0x11499, "TIRHUTA LETTER TTA" },
    { 0x1149A, "TIRHUTA LETTER TTHA" },
    { 0x1149B, "TIRHUTA LETTER DDA" },
    { 0x1149C, "TIRHUTA LETTER DDHA" },
    { 0x1149D, "TIRHUTA LETTER NNA" },
    { 0x1149E, "TIRHUTA LETTER TA" },
    { 0x1149F, "TIRHUTA LETTER THA" },
    { 0x114A0, "TIRHUTA LETTER DA" },
    { 0x114A1, "TIRHUTA LETTER DHA" },
    { 0x114A2, "TIRHUTA LETTER NA" },
    { 0x114A3, "TIRHUTA LETTER PA" },
    { 0x114A4, "TIRHUTA LETTER PHA" },
    { 0x114A5, "TIRHUTA LETTER BA" },
    { 0x114A6, "TIRHUTA LETTER BHA" },
    { 0x114A7, "TIRHUTA LETTER MA" },
    { 0x114A8, "TIRHUTA LETTER YA" },
    { 0x114A9, "TIRHUTA LETTER RA" },
    { 0x114AA, "TIRHUTA LETTER LA" },
    { 0x114AB, "TIRHUTA LETTER VA" },
    { 0x114AC, "TIRHUTA LETTER SHA" },
    { 0x114AD, "TIRHUTA LETTER SSA" },
    { 0x114AE, "TIRHUTA LETTER SA" },
    { 0x114AF, "TIRHUTA LETTER HA" },
    { 0x114B0, "TIRHUTA VOWEL SIGN AA" },
    { 0x114B1, "TIRHUTA VOWEL SIGN I" },
    { 0x114B2, "TIRHUTA VOWEL SIGN II" },
    { 0x114B3, "TIRHUTA VOWEL SIGN U" },
    { 0x114B4, "TIRHUTA VOWEL SIGN UU" },
    { 0x114B5, "TIRHUTA VOWEL SIGN VOCALIC R" },
    { 0x114B6, "TIRHUTA VOWEL SIGN VOCALIC RR" },
    { 0x114B7, "TIRHUTA VOWEL SIGN VOCALIC L" },
    { 0x114B8, "TIRHUTA VOWEL SIGN VOCALIC LL" },
    { 0x114B9, "TIRHUTA VOWEL SIGN E" },
    { 0x114BA, "TIRHUTA VOWEL SIGN SHORT E" },
    { 0x114BB, "TIRHUTA VOWEL SIGN AI" },
    { 0x114BC, "TIRHUTA VOWEL SIGN O" },
    { 0x114BD, "TIRHUTA VOWEL SIGN SHORT O" },
    { 0x114BE, "TIRHUTA VOWEL SIGN AU" },
    { 0x114BF, "TIRHUTA SIGN CANDRABINDU" },
    { 0x114C0, "TIRHUTA SIGN ANUSVARA" },
    { 0x114C1, "TIRHUTA SIGN VISARGA" },
    { 0x114C2, "TIRHUTA SIGN VIRAMA" },
    { 0x114C3, "TIRHUTA SIGN NUKTA" },
    { 0x114C4, "TIRHUTA SIGN AVAGRAHA" },
    { 0x114C5, "TIRHUTA GVANG" },
    { 0x114C6, "TIRHUTA ABBREVIATION SIGN" },
    { 0x114C7, "TIRHUTA OM" },
    { 0x114D0, "TIRHUTA DIGIT ZERO" },
    { 0x114D1, "TIRHUTA DIGIT ONE" },
    { 0x114D2, "TIRHUTA DIGIT TWO" },
    { 0x114D3, "TIRHUTA DIGIT THREE" },
    { 0x114D4, "TIRHUTA DIGIT FOUR" },
    { 0x114D5, "TIRHUTA DIGIT FIVE" },
    { 0x114D6, "TIRHUTA DIGIT SIX" },
    { 0x114D7, "TIRHUTA DIGIT SEVEN" },
    { 0x114D8, "TIRHUTA DIGIT EIGHT" },
    { 0x114D9, "TIRHUTA DIGIT NINE" },
    { 0x11580, "SIDDHAM LETTER A" },
    { 0x11581, "SIDDHAM LETTER AA" },
    { 0x11582, "SIDDHAM LETTER I" },
    { 0x11583, "SIDDHAM LETTER II" },
    { 0x11584, "SIDDHAM LETTER U" },
    { 0x11585, "SIDDHAM LETTER UU" },
    { 0x11586, "SIDDHAM LETTER VOCALIC R" },
    { 0x11587, "SIDDHAM LETTER VOCALIC RR" },
    { 0x11588, "SIDDHAM LETTER VOCALIC L" },
    { 0x11589, "SIDDHAM LETTER VOCALIC LL" },
    { 0x1158A, "SIDDHAM LETTER E" },
    { 0x1158B, "SIDDHAM LETTER AI" },
    { 0x1158C, "SIDDHAM LETTER O" },
    { 0x1158D, "SIDDHAM LETTER AU" },
    { 0x1158E, "SIDDHAM LETTER KA" },
    { 0x1158F, "SIDDHAM LETTER KHA" },
    { 0x11590, "SIDDHAM LETTER GA" },
    { 0x11591, "SIDDHAM LETTER GHA" },
    { 0x11592, "SIDDHAM LETTER NGA" },
    { 0x11593, "SIDDHAM LETTER CA" },
    { 0x11594, "SIDDHAM LETTER CHA" },
    { 0x11595, "SIDDHAM LETTER JA" },
    { 0x11596, "SIDDHAM LETTER JHA" },
    { 0x11597, "SIDDHAM LETTER NYA" },
    { 0x11598, "SIDDHAM LETTER TTA" },
    { 0x11599, "SIDDHAM LETTER TTHA" },
    { 0x1159A, "SIDDHAM LETTER DDA" },
    { 0x1159B, "SIDDHAM LETTER DDHA" },
    { 0x1159C, "SIDDHAM LETTER NNA" },
    { 0x1159D, "SIDDHAM LETTER TA" },
    { 0x1159E, "SIDDHAM LETTER THA" },
    { 0x1159F, "SIDDHAM LETTER DA" },
    { 0x115A0, "SIDDHAM LETTER DHA" },
    { 0x115A1, "SIDDHAM LETTER NA" },
    { 0x115A2, "SIDDHAM LETTER PA" },
    { 0x115A3, "SIDDHAM LETTER PHA" },
    { 0x115A4, "SIDDHAM LETTER BA" },
    { 0x115A5, "SIDDHAM LETTER BHA" },
    { 0x115A6, "SIDDHAM LETTER MA" },
    { 0x115A7, "SIDDHAM LETTER YA" },
    { 0x115A8, "SIDDHAM LETTER RA" },
    { 0x115A9, "SIDDHAM LETTER LA" },
    { 0x115AA, "SIDDHAM LETTER VA" },
    { 0x115AB, "SIDDHAM LETTER SHA" },
    { 0x115AC, "SIDDHAM LETTER SSA" },
    { 0x115AD, "SIDDHAM LETTER SA" },
    { 0x115AE, "SIDDHAM LETTER HA" },
    { 0x115AF, "SIDDHAM VOWEL SIGN AA" },
    { 0x115B0, "SIDDHAM VOWEL SIGN I" },
    { 0x115B1, "SIDDHAM VOWEL SIGN II" },
    { 0x115B2, "SIDDHAM VOWEL SIGN U" },
    { 0x115B3, "SIDDHAM VOWEL SIGN UU" },
    { 0x115B4, "SIDDHAM VOWEL SIGN VOCALIC R" },
    { 0x115B5, "SIDDHAM VOWEL SIGN VOCALIC RR" },
    { 0x115B8, "SIDDHAM VOWEL SIGN E" },
    { 0x115B9, "SIDDHAM VOWEL SIGN AI" },
    { 0x115BA, "SIDDHAM VOWEL SIGN O" },
    { 0x115BB, "SIDDHAM VOWEL SIGN AU" },
    { 0x115BC, "SIDDHAM SIGN CANDRABINDU" },
    { 0x115BD, "SIDDHAM SIGN ANUSVARA" },
    { 0x115BE, "SIDDHAM SIGN VISARGA" },
    { 0x115BF, "SIDDHAM SIGN VIRAMA" },
    { 0x115C0, "SIDDHAM SIGN NUKTA" },
    { 0x115C1, "SIDDHAM SIGN SIDDHAM" },
    { 0x115C2, "SIDDHAM DANDA" },
    { 0x115C3, "SIDDHAM DOUBLE DANDA" },
    { 0x115C4, "SIDDHAM SEPARATOR DOT" },
    { 0x115C5, "SIDDHAM SEPARATOR BAR" },
    { 0x115C6, "SIDDHAM REPETITION MARK-1" },
    { 0x115C7, "SIDDHAM REPETITION MARK-2" },
    { 0x115C8, "SIDDHAM REPETITION MARK-3" },
    { 0x115C9, "SIDDHAM END OF TEXT MARK" },
    { 0x115CA, "SIDDHAM SECTION MARK WITH TRIDENT AND U-SHAPED ORNAMENTS" },
    { 0x115CB, "SIDDHAM SECTION MARK WITH TRIDENT AND DOTTED CRESCENTS" },
    { 0x115CC, "SIDDHAM SECTION MARK WITH RAYS AND DOTTED CRESCENTS" },
    { 0x115CD, "SIDDHAM SECTION MARK WITH RAYS AND DOTTED DOUBLE CRESCENTS" },
    { 0x115CE, "SIDDHAM SECTION MARK WITH RAYS AND DOTTED TRIPLE CRESCENTS" },
    { 0x115CF, "SIDDHAM SECTION MARK DOUBLE RING" },
    { 0x115D0, "SIDDHAM SECTION MARK DOUBLE RING WITH RAYS" },
    { 0x115D1, "SIDDHAM SECTION MARK WITH DOUBLE CRESCENTS" },
    { 0x115D2, "SIDDHAM SECTION MARK WITH TRIPLE CRESCENTS" },
    { 0x115D3, "SIDDHAM SECTION MARK WITH QUADRUPLE CRESCENTS" },
    { 0x115D4, "SIDDHAM SECTION MARK WITH SEPTUPLE CRESCENTS" },
    { 0x115D5, "SIDDHAM SECTION MARK WITH CIRCLES AND RAYS" },
    { 0x115D6, "SIDDHAM SECTION MARK WITH CIRCLES AND TWO ENCLOSURES" },
    { 0x115D7, "SIDDHAM SECTION MARK WITH CIRCLES AND FOUR ENCLOSURES" },
    { 0x115D8, "SIDDHAM LETTER THREE-CIRCLE ALTERNATE I" },
    { 0x115D9, "SIDDHAM LETTER TWO-CIRCLE ALTERNATE I" },
    { 0x115DA, "SIDDHAM LETTER TWO-CIRCLE ALTERNATE II" },
    { 0x115DB, "SIDDHAM LETTER ALTERNATE U" },
    { 0x115DC, "SIDDHAM VOWEL SIGN ALTERNATE U" },
    { 0x115DD, "SIDDHAM VOWEL SIGN ALTERNATE UU" },
    { 0x11600, "MODI LETTER A" },
    { 0x11601, "MODI LETTER AA" },
    { 0x11602, "MODI LETTER I" },
    { 0x11603, "MODI LETTER II" },
    { 0x11604, "MODI LETTER U" },
    { 0x11605, "MODI LETTER UU" },
    { 0x11606, "MODI LETTER VOCALIC R" },
    { 0x11607, "MODI LETTER VOCALIC RR" },
    { 0x11608, "MODI LETTER VOCALIC L" },
    { 0x11609, "MODI LETTER VOCALIC LL" },
    { 0x1160A, "MODI LETTER E" },
    { 0x1160B, "MODI LETTER AI" },
    { 0x1160C, "MODI LETTER O" },
    { 0x1160D, "MODI LETTER AU" },
    { 0x1160E, "MODI LETTER KA" },
    { 0x1160F, "MODI LETTER KHA" },
    { 0x11610, "MODI LETTER GA" },
    { 0x11611, "MODI LETTER GHA" },
    { 0x11612, "MODI LETTER NGA" },
    { 0x11613, "MODI LETTER CA" },
    { 0x11614, "MODI LETTER CHA" },
    { 0x11615, "MODI LETTER JA" },
    { 0x11616, "MODI LETTER JHA" },
    { 0x11617, "MODI LETTER NYA" },
    { 0x11618, "MODI LETTER TTA" },
    { 0x11619, "MODI LETTER TTHA" },
    { 0x1161A, "MODI LETTER DDA" },
    { 0x1161B, "MODI LETTER DDHA" },
    { 0x1161C, "MODI LETTER NNA" },
    { 0x1161D, "MODI LETTER TA" },
    { 0x1161E, "MODI LETTER THA" },
    { 0x1161F, "MODI LETTER DA" },
    { 0x11620, "MODI LETTER DHA" },
    { 0x11621, "MODI LETTER NA" },
    { 0x11622, "MODI LETTER PA" },
    { 0x11623, "MODI LETTER PHA" },
    { 0x11624, "MODI LETTER BA" },
    { 0x11625, "MODI LETTER BHA" },
    { 0x11626, "MODI LETTER MA" },
    { 0x11627, "MODI LETTER YA" },
    { 0x11628, "MODI LETTER RA" },
    { 0x11629, "MODI LETTER LA" },
    { 0x1162A, "MODI LETTER VA" },
    { 0x1162B, "MODI LETTER SHA" },
    { 0x1162C, "MODI LETTER SSA" },
    { 0x1162D, "MODI LETTER SA" },
    { 0x1162E, "MODI LETTER HA" },
    { 0x1162F, "MODI LETTER LLA" },
    { 0x11630, "MODI VOWEL SIGN AA" },
    { 0x11631, "MODI VOWEL SIGN I" },
    { 0x11632, "MODI VOWEL SIGN II" },
    { 0x11633, "MODI VOWEL SIGN U" },
    { 0x11634, "MODI VOWEL SIGN UU" },
    { 0x11635, "MODI VOWEL SIGN VOCALIC R" },
    { 0x11636, "MODI VOWEL SIGN VOCALIC RR" },
    { 0x11637, "MODI VOWEL SIGN VOCALIC L" },
    { 0x11638, "MODI VOWEL SIGN VOCALIC LL" },
    { 0x11639, "MODI VOWEL SIGN E" },
    { 0x1163A, "MODI VOWEL SIGN AI" },
    { 0x1163B, "MODI VOWEL SIGN O" },
    { 0x1163C, "MODI VOWEL SIGN AU" },
    { 0x1163D, "MODI SIGN ANUSVARA" },
    { 0x1163E, "MODI SIGN VISARGA" },
    { 0x1163F, "MODI SIGN VIRAMA" },
    { 0x11640, "MODI SIGN ARDHACANDRA" },
    { 0x11641, "MODI DANDA" },
    { 0x11642, "MODI DOUBLE DANDA" },
    { 0x11643, "MODI ABBREVIATION SIGN" },
    { 0x11644, "MODI SIGN HUVA" },
    { 0x11650, "MODI DIGIT ZERO" },
    { 0x11651, "MODI DIGIT ONE" },
    { 0x11652, "MODI DIGIT TWO" },
    { 0x11653, "MODI DIGIT THREE" },
    { 0x11654, "MODI DIGIT FOUR" },
    { 0x11655, "MODI DIGIT FIVE" },
    { 0x11656, "MODI DIGIT SIX" },
    { 0x11657, "MODI DIGIT SEVEN" },
    { 0x11658, "MODI DIGIT EIGHT" },
    { 0x11659, "MODI DIGIT NINE" },
    { 0x11660, "MONGOLIAN BIRGA WITH ORNAMENT" },
    { 0x11661, "MONGOLIAN ROTATED BIRGA" },
    { 0x11662, "MONGOLIAN DOUBLE BIRGA WITH ORNAMENT" },
    { 0x11663, "MONGOLIAN TRIPLE BIRGA WITH ORNAMENT" },
    { 0x11664, "MONGOLIAN BIRGA WITH DOUBLE ORNAMENT" },
    { 0x11665, "MONGOLIAN ROTATED BIRGA WITH ORNAMENT" },
    { 0x11666, "MONGOLIAN ROTATED BIRGA WITH DOUBLE ORNAMENT" },
    { 0x11667, "MONGOLIAN INVERTED BIRGA" },
    { 0x11668, "MONGOLIAN INVERTED BIRGA WITH DOUBLE ORNAMENT" },
    { 0x11669, "MONGOLIAN SWIRL BIRGA" },
    { 0x1166A, "MONGOLIAN SWIRL BIRGA WITH ORNAMENT" },
    { 0x1166B, "MONGOLIAN SWIRL BIRGA WITH DOUBLE ORNAMENT" },
    { 0x1166C, "MONGOLIAN TURNED SWIRL BIRGA WITH DOUBLE ORNAMENT" },
    { 0x11680, "TAKRI LETTER A" },
    { 0x11681, "TAKRI LETTER AA" },
    { 0x11682, "TAKRI LETTER I" },
    { 0x11683, "TAKRI LETTER II" },
    { 0x11684, "TAKRI LETTER U" },
    { 0x11685, "TAKRI LETTER UU" },
    { 0x11686, "TAKRI LETTER E" },
    { 0x11687, "TAKRI LETTER AI" },
    { 0x11688, "TAKRI LETTER O" },
    { 0x11689, "TAKRI LETTER AU" },
    { 0x1168A, "TAKRI LETTER KA" },
    { 0x1168B, "TAKRI LETTER KHA" },
    { 0x1168C, "TAKRI LETTER GA" },
    { 0x1168D, "TAKRI LETTER GHA" },
    { 0x1168E, "TAKRI LETTER NGA" },
    { 0x1168F, "TAKRI LETTER CA" },
    { 0x11690, "TAKRI LETTER CHA" },
    { 0x11691, "TAKRI LETTER JA" },
    { 0x11692, "TAKRI LETTER JHA" },
    { 0x11693, "TAKRI LETTER NYA" },
    { 0x11694, "TAKRI LETTER TTA" },
    { 0x11695, "TAKRI LETTER TTHA" },
    { 0x11696, "TAKRI LETTER DDA" },
    { 0x11697, "TAKRI LETTER DDHA" },
    { 0x11698, "TAKRI LETTER NNA" },
    { 0x11699, "TAKRI LETTER TA" },
    { 0x1169A, "TAKRI LETTER THA" },
    { 0x1169B, "TAKRI LETTER DA" },
    { 0x1169C, "TAKRI LETTER DHA" },
    { 0x1169D, "TAKRI LETTER NA" },
    { 0x1169E, "TAKRI LETTER PA" },
    { 0x1169F, "TAKRI LETTER PHA" },
    { 0x116A0, "TAKRI LETTER BA" },
    { 0x116A1, "TAKRI LETTER BHA" },
    { 0x116A2, "TAKRI LETTER MA" },
    { 0x116A3, "TAKRI LETTER YA" },
    { 0x116A4, "TAKRI LETTER RA" },
    { 0x116A5, "TAKRI LETTER LA" },
    { 0x116A6, "TAKRI LETTER VA" },
    { 0x116A7, "TAKRI LETTER SHA" },
    { 0x116A8, "TAKRI LETTER SA" },
    { 0x116A9, "TAKRI LETTER HA" },
    { 0x116AA, "TAKRI LETTER RRA" },
    { 0x116AB, "TAKRI SIGN ANUSVARA" },
    { 0x116AC, "TAKRI SIGN VISARGA" },
    { 0x116AD, "TAKRI VOWEL SIGN AA" },
    { 0x116AE, "TAKRI VOWEL SIGN I" },
    { 0x116AF, "TAKRI VOWEL SIGN II" },
    { 0x116B0, "TAKRI VOWEL SIGN U" },
    { 0x116B1, "TAKRI VOWEL SIGN UU" },
    { 0x116B2, "TAKRI VOWEL SIGN E" },
    { 0x116B3, "TAKRI VOWEL SIGN AI" },
    { 0x116B4, "TAKRI VOWEL SIGN O" },
    { 0x116B5, "TAKRI VOWEL SIGN AU" },
    { 0x116B6, "TAKRI SIGN VIRAMA" },
    { 0x116B7, "TAKRI SIGN NUKTA" },
    { 0x116B8, "TAKRI LETTER ARCHAIC KHA" },
    { 0x116B9, "TAKRI ABBREVIATION SIGN" },
    { 0x116C0, "TAKRI DIGIT ZERO" },
    { 0x116C1, "TAKRI DIGIT ONE" },
    { 0x116C2, "TAKRI DIGIT TWO" },
    { 0x116C3, "TAKRI DIGIT THREE" },
    { 0x116C4, "TAKRI DIGIT FOUR" },
    { 0x116C5, "TAKRI DIGIT FIVE" },
    { 0x116C6, "TAKRI DIGIT SIX" },
    { 0x116C7, "TAKRI DIGIT SEVEN" },
    { 0x116C8, "TAKRI DIGIT EIGHT" },
    { 0x116C9, "TAKRI DIGIT NINE" },
    { 0x11700, "AHOM LETTER KA" },
    { 0x11701, "AHOM LETTER KHA" },
    { 0x11702, "AHOM LETTER NGA" },
    { 0x11703, "AHOM LETTER NA" },
    { 0x11704, "AHOM LETTER TA" },
    { 0x11705, "AHOM LETTER ALTERNATE TA" },
    { 0x11706, "AHOM LETTER PA" },
    { 0x11707, "AHOM LETTER PHA" },
    { 0x11708, "AHOM LETTER BA" },
    { 0x11709, "AHOM LETTER MA" },
    { 0x1170A, "AHOM LETTER JA" },
    { 0x1170B, "AHOM LETTER CHA" },
    { 0x1170C, "AHOM LETTER THA" },
    { 0x1170D, "AHOM LETTER RA" },
    { 0x1170E, "AHOM LETTER LA" },
    { 0x1170F, "AHOM LETTER SA" },
    { 0x11710, "AHOM LETTER NYA" },
    { 0x11711, "AHOM LETTER HA" },
    { 0x11712, "AHOM LETTER A" },
    { 0x11713, "AHOM LETTER DA" },
    { 0x11714, "AHOM LETTER DHA" },
    { 0x11715, "AHOM LETTER GA" },
    { 0x11716, "AHOM LETTER ALTERNATE GA" },
    { 0x11717, "AHOM LETTER GHA" },
    { 0x11718, "AHOM LETTER BHA" },
    { 0x11719, "AHOM LETTER JHA" },
    { 0x1171A, "AHOM LETTER ALTERNATE BA" },
    { 0x1171D, "AHOM CONSONANT SIGN MEDIAL LA" },
    { 0x1171E, "AHOM CONSONANT SIGN MEDIAL RA" },
    { 0x1171F, "AHOM CONSONANT SIGN MEDIAL LIGATING RA" },
    { 0x11720, "AHOM VOWEL SIGN A" },
    { 0x11721, "AHOM VOWEL SIGN AA" },
    { 0x11722, "AHOM VOWEL SIGN I" },
    { 0x11723, "AHOM VOWEL SIGN II" },
    { 0x11724, "AHOM VOWEL SIGN U" },
    { 0x11725, "AHOM VOWEL SIGN UU" },
    { 0x11726, "AHOM VOWEL SIGN E" },
    { 0x11727, "AHOM VOWEL SIGN AW" },
    { 0x11728, "AHOM VOWEL SIGN O" },
    { 0x11729, "AHOM VOWEL SIGN AI" },
    { 0x1172A, "AHOM VOWEL SIGN AM" },
    { 0x1172B, "AHOM SIGN KILLER" },
    { 0x11730, "AHOM DIGIT ZERO" },
    { 0x11731, "AHOM DIGIT ONE" },
    { 0x11732, "AHOM DIGIT TWO" },
    { 0x11733, "AHOM DIGIT THREE" },
    { 0x11734, "AHOM DIGIT FOUR" },
    { 0x11735, "AHOM DIGIT FIVE" },
    { 0x11736, "AHOM DIGIT SIX" },
    { 0x11737, "AHOM DIGIT SEVEN" },
    { 0x11738, "AHOM DIGIT EIGHT" },
    { 0x11739, "AHOM DIGIT NINE" },
    { 0x1173A, "AHOM NUMBER TEN" },
    { 0x1173B, "AHOM NUMBER TWENTY" },
    { 0x1173C, "AHOM SIGN SMALL SECTION" },
    { 0x1173D, "AHOM SIGN SECTION" },
    { 0x1173E, "AHOM SIGN RULAI" },
    { 0x1173F, "AHOM SYMBOL VI" },
    { 0x11740, "AHOM LETTER CA" },
    { 0x11741, "AHOM LETTER TTA" },
    { 0x11742, "AHOM LETTER TTHA" },
    { 0x11743, "AHOM LETTER DDA" },
    { 0x11744, "AHOM LETTER DDHA" },
    { 0x11745, "AHOM LETTER NNA" },
    { 0x11746, "AHOM LETTER LLA" },
    { 0x11800, "DOGRA LETTER A" },
    { 0x11801, "DOGRA LETTER AA" },
    { 0x11802, "DOGRA LETTER I" },
    { 0x11803, "DOGRA LETTER II" },
    { 0x11804, "DOGRA LETTER U" },
    { 0x11805, "DOGRA LETTER UU" },
    { 0x11806, "DOGRA LETTER E" },
    { 0x11807, "DOGRA LETTER AI" },
    { 0x11808, "DOGRA LETTER O" },
    { 0x11809, "DOGRA LETTER AU" },
    { 0x1180A, "DOGRA LETTER KA" },
    { 0x1180B, "DOGRA LETTER KHA" },
    { 0x1180C, "DOGRA LETTER GA" },
    { 0x1180D, "DOGRA LETTER GHA" },
    { 0x1180E, "DOGRA LETTER NGA" },
    { 0x1180F, "DOGRA LETTER CA" },
    { 0x11810, "DOGRA LETTER CHA" },
    { 0x11811, "DOGRA LETTER JA" },
    { 0x11812, "DOGRA LETTER JHA" },
    { 0x11813, "DOGRA LETTER NYA" },
    { 0x11814, "DOGRA LETTER TTA" },
    { 0x11815, "DOGRA LETTER TTHA" },
    { 0x11816, "DOGRA LETTER DDA" },
    { 0x11817, "DOGRA LETTER DDHA" },
    { 0x11818, "DOGRA LETTER NNA" },
    { 0x11819, "DOGRA LETTER TA" },
    { 0x1181A, "DOGRA LETTER THA" },
    { 0x1181B, "DOGRA LETTER DA" },
    { 0x1181C, "DOGRA LETTER DHA" },
    { 0x1181D, "DOGRA LETTER NA" },
    { 0x1181E, "DOGRA LETTER PA" },
    { 0x1181F, "DOGRA LETTER PHA" },
    { 0x11820, "DOGRA LETTER BA" },
    { 0x11821, "DOGRA LETTER BHA" },
    { 0x11822, "DOGRA LETTER MA" },
    { 0x11823, "DOGRA LETTER YA" },
    { 0x11824, "DOGRA LETTER RA" },
    { 0x11825, "DOGRA LETTER LA" },
    { 0x11826, "DOGRA LETTER VA" },
    { 0x11827, "DOGRA LETTER SHA" },
    { 0x11828, "DOGRA LETTER SSA" },
    { 0x11829, "DOGRA LETTER SA" },
    { 0x1182A, "DOGRA LETTER HA" },
    { 0x1182B, "DOGRA LETTER RRA" },
    { 0x1182C, "DOGRA VOWEL SIGN AA" },
    { 0x1182D, "DOGRA VOWEL SIGN I" },
    { 0x1182E, "DOGRA VOWEL SIGN II" },
    { 0x1182F, "DOGRA VOWEL SIGN U" },
    { 0x11830, "DOGRA VOWEL SIGN UU" },
    { 0x11831, "DOGRA VOWEL SIGN VOCALIC R" },
    { 0x11832, "DOGRA VOWEL SIGN VOCALIC RR" },
    { 0x11833, "DOGRA VOWEL SIGN E" },
    { 0x11834, "DOGRA VOWEL SIGN AI" },
    { 0x11835, "DOGRA VOWEL SIGN O" },
    { 0x11836, "DOGRA VOWEL SIGN AU" },
    { 0x11837, "DOGRA SIGN ANUSVARA" },
    { 0x11838, "DOGRA SIGN VISARGA" },
    { 0x11839, "DOGRA SIGN VIRAMA" },
    { 0x1183A, "DOGRA SIGN NUKTA" },
    { 0x1183B, "DOGRA ABBREVIATION SIGN" },
    { 0x118A0, "WARANG CITI CAPITAL LETTER NGAA" },
    { 0x118A1, "WARANG CITI CAPITAL LETTER A" },
    { 0x118A2, "WARANG CITI CAPITAL LETTER WI" },
    { 0x118A3, "WARANG CITI CAPITAL LETTER YU" },
    { 0x118A4, "WARANG CITI CAPITAL LETTER YA" },
    { 0x118A5, "WARANG CITI CAPITAL LETTER YO" },
    { 0x118A6, "WARANG CITI CAPITAL LETTER II" },
    { 0x118A7, "WARANG CITI CAPITAL LETTER UU" },
    { 0x118A8, "WARANG CITI CAPITAL LETTER E" },
    { 0x118A9, "WARANG CITI CAPITAL LETTER O" },
    { 0x118AA, "WARANG CITI CAPITAL LETTER ANG" },
    { 0x118AB, "WARANG CITI CAPITAL LETTER GA" },
    { 0x118AC, "WARANG CITI CAPITAL LETTER KO" },
    { 0x118AD, "WARANG CITI CAPITAL LETTER ENY" },
    { 0x118AE, "WARANG CITI CAPITAL LETTER YUJ" },
    { 0x118AF, "WARANG CITI CAPITAL LETTER UC" },
    { 0x118B0, "WARANG CITI CAPITAL LETTER ENN" },
    { 0x118B1, "WARANG CITI CAPITAL LETTER ODD" },
    { 0x118B2, "WARANG CITI CAPITAL LETTER TTE" },
    { 0x118B3, "WARANG CITI CAPITAL LETTER NUNG" },
    { 0x118B4, "WARANG CITI CAPITAL LETTER DA" },
    { 0x118B5, "WARANG CITI CAPITAL LETTER AT" },
    { 0x118B6, "WARANG CITI CAPITAL LETTER AM" },
    { 0x118B7, "WARANG CITI CAPITAL LETTER BU" },
    { 0x118B8, "WARANG CITI CAPITAL LETTER PU" },
    { 0x118B9, "WARANG CITI CAPITAL LETTER HIYO" },
    { 0x118BA, "WARANG CITI CAPITAL LETTER HOLO" },
    { 0x118BB, "WARANG CITI CAPITAL LETTER HORR" },
    { 0x118BC, "WARANG CITI CAPITAL LETTER HAR" },
    { 0x118BD, "WARANG CITI CAPITAL LETTER SSUU" },
    { 0x118BE, "WARANG CITI CAPITAL LETTER SII" },
    { 0x118BF, "WARANG CITI CAPITAL LETTER VIYO" },
    { 0x118C0, "WARANG CITI SMALL LETTER NGAA" },
    { 0x118C1, "WARANG CITI SMALL LETTER A" },
    { 0x118C2, "WARANG CITI SMALL LETTER WI" },
    { 0x118C3, "WARANG CITI SMALL LETTER YU" },
    { 0x118C4, "WARANG CITI SMALL LETTER YA" },
    { 0x118C5, "WARANG CITI SMALL LETTER YO" },
    { 0x118C6, "WARANG CITI SMALL LETTER II" },
    { 0x118C7, "WARANG CITI SMALL LETTER UU" },
    { 0x118C8, "WARANG CITI SMALL LETTER E" },
    { 0x118C9, "WARANG CITI SMALL LETTER O" },
    { 0x118CA, "WARANG CITI SMALL LETTER ANG" },
    { 0x118CB, "WARANG CITI SMALL LETTER GA" },
    { 0x118CC, "WARANG CITI SMALL LETTER KO" },
    { 0x118CD, "WARANG CITI SMALL LETTER ENY" },
    { 0x118CE, "WARANG CITI SMALL LETTER YUJ" },
    { 0x118CF, "WARANG CITI SMALL LETTER UC" },
    { 0x118D0, "WARANG CITI SMALL LETTER ENN" },
    { 0x118D1, "WARANG CITI SMALL LETTER ODD" },
    { 0x118D2, "WARANG CITI SMALL LETTER TTE" },
    { 0x118D3, "WARANG CITI SMALL LETTER NUNG" },
    { 0x118D4, "WARANG CITI SMALL LETTER DA" },
    { 0x118D5, "WARANG CITI SMALL LETTER AT" },
    { 0x118D6, "WARANG CITI SMALL LETTER AM" },
    { 0x118D7, "WARANG CITI SMALL LETTER BU" },
    { 0x118D8, "WARANG CITI SMALL LETTER PU" },
    { 0x118D9, "WARANG CITI SMALL LETTER HIYO" },
    { 0x118DA, "WARANG CITI SMALL LETTER HOLO" },
    { 0x118DB, "WARANG CITI SMALL LETTER HORR" },
    { 0x118DC, "WARANG CITI SMALL LETTER HAR" },
    { 0x118DD, "WARANG CITI SMALL LETTER SSUU" },
    { 0x118DE, "WARANG CITI SMALL LETTER SII" },
    { 0x118DF, "WARANG CITI SMALL LETTER VIYO" },
    { 0x118E0, "WARANG CITI DIGIT ZERO" },
    { 0x118E1, "WARANG CITI DIGIT ONE" },
    { 0x118E2, "WARANG CITI DIGIT TWO" },
    { 0x118E3, "WARANG CITI DIGIT THREE" },
    { 0x118E4, "WARANG CITI DIGIT FOUR" },
    { 0x118E5, "WARANG CITI DIGIT FIVE" },
    { 0x118E6, "WARANG CITI DIGIT SIX" },
    { 0x118E7, "WARANG CITI DIGIT SEVEN" },
    { 0x118E8, "WARANG CITI DIGIT EIGHT" },
    { 0x118E9, "WARANG CITI DIGIT NINE" },
    { 0x118EA, "WARANG CITI NUMBER TEN" },
    { 0x118EB, "WARANG CITI NUMBER TWENTY" },
    { 0x118EC, "WARANG CITI NUMBER THIRTY" },
    { 0x118ED, "WARANG CITI NUMBER FORTY" },
    { 0x118EE, "WARANG CITI NUMBER FIFTY" },
    { 0x118EF, "WARANG CITI NUMBER SIXTY" },
    { 0x118F0, "WARANG CITI NUMBER SEVENTY" },
    { 0x118F1, "WARANG CITI NUMBER EIGHTY" },
    { 0x118F2, "WARANG CITI NUMBER NINETY" },
    { 0x118FF, "WARANG CITI OM" },
    { 0x11900, "DIVES AKURU LETTER A" },
    { 0x11901, "DIVES AKURU LETTER AA" },
    { 0x11902, "DIVES AKURU LETTER I" },
    { 0x11903, "DIVES AKURU LETTER II" },
    { 0x11904, "DIVES AKURU LETTER U" },
    { 0x11905, "DIVES AKURU LETTER UU" },
    { 0x11906, "DIVES AKURU LETTER E" },
    { 0x11909, "DIVES AKURU LETTER O" },
    { 0x1190C, "DIVES AKURU LETTER KA" },
    { 0x1190D, "DIVES AKURU LETTER KHA" },
    { 0x1190E, "DIVES AKURU LETTER GA" },
    { 0x1190F, "DIVES AKURU LETTER GHA" },
    { 0x11910, "DIVES AKURU LETTER NGA" },
    { 0x11911, "DIVES AKURU LETTER CA" },
    { 0x11912, "DIVES AKURU LETTER CHA" },
    { 0x11913, "DIVES AKURU LETTER JA" },
    { 0x11915, "DIVES AKURU LETTER NYA" },
    { 0x11916, "DIVES AKURU LETTER TTA" },
    { 0x11918, "DIVES AKURU LETTER DDA" },
    { 0x11919, "DIVES AKURU LETTER DDHA" },
    { 0x1191A, "DIVES AKURU LETTER NNA" },
    { 0x1191B, "DIVES AKURU LETTER TA" },
    { 0x1191C, "DIVES AKURU LETTER THA" },
    { 0x1191D, "DIVES AKURU LETTER DA" },
    { 0x1191E, "DIVES AKURU LETTER DHA" },
    { 0x1191F, "DIVES AKURU LETTER NA" },
    { 0x11920, "DIVES AKURU LETTER PA" },
    { 0x11921, "DIVES AKURU LETTER PHA" },
    { 0x11922, "DIVES AKURU LETTER BA" },
    { 0x11923, "DIVES AKURU LETTER BHA" },
    { 0x11924, "DIVES AKURU LETTER MA" },
    { 0x11925, "DIVES AKURU LETTER YA" },
    { 0x11926, "DIVES AKURU LETTER YYA" },
    { 0x11927, "DIVES AKURU LETTER RA" },
    { 0x11928, "DIVES AKURU LETTER LA" },
    { 0x11929, "DIVES AKURU LETTER VA" },
    { 0x1192A, "DIVES AKURU LETTER SHA" },
    { 0x1192B, "DIVES AKURU LETTER SSA" },
    { 0x1192C, "DIVES AKURU LETTER SA" },
    { 0x1192D, "DIVES AKURU LETTER HA" },
    { 0x1192E, "DIVES AKURU LETTER LLA" },
    { 0x1192F, "DIVES AKURU LETTER ZA" },
    { 0x11930, "DIVES AKURU VOWEL SIGN AA" },
    { 0x11931, "DIVES AKURU VOWEL SIGN I" },
    { 0x11932, "DIVES AKURU VOWEL SIGN II" },
    { 0x11933, "DIVES AKURU VOWEL SIGN U" },
    { 0x11934, "DIVES AKURU VOWEL SIGN UU" },
    { 0x11935, "DIVES AKURU VOWEL SIGN E" },
    { 0x11937, "DIVES AKURU VOWEL SIGN AI" },
    { 0x11938, "DIVES AKURU VOWEL SIGN O" },
    { 0x1193B, "DIVES AKURU SIGN ANUSVARA" },
    { 0x1193C, "DIVES AKURU SIGN CANDRABINDU" },
    { 0x1193D, "DIVES AKURU SIGN HALANTA" },
    { 0x1193E, "DIVES AKURU VIRAMA" },
    { 0x1193F, "DIVES AKURU PREFIXED NASAL SIGN" },
    { 0x11940, "DIVES AKURU MEDIAL YA" },
    { 0x11941, "DIVES AKURU INITIAL RA" },
    { 0x11942, "DIVES AKURU MEDIAL RA" },
    { 0x11943, "DIVES AKURU SIGN NUKTA" },
    { 0x11944, "DIVES AKURU DOUBLE DANDA" },
    { 0x11945, "DIVES AKURU GAP FILLER" },
    { 0x11946, "DIVES AKURU END OF TEXT MARK" },
    { 0x11950, "DIVES AKURU DIGIT ZERO" },
    { 0x11951, "DIVES AKURU DIGIT ONE" },
    { 0x11952, "DIVES AKURU DIGIT TWO" },
    { 0x11953, "DIVES AKURU DIGIT THREE" },
    { 0x11954, "DIVES AKURU DIGIT FOUR" },
    { 0x11955, "DIVES AKURU DIGIT FIVE" },
    { 0x11956, "DIVES AKURU DIGIT SIX" },
    { 0x11957, "DIVES AKURU DIGIT SEVEN" },
    { 0x11958, "DIVES AKURU DIGIT EIGHT" },
    { 0x11959, "DIVES AKURU DIGIT NINE" },
    { 0x119A0, "NANDINAGARI LETTER A" },
    { 0x119A1, "NANDINAGARI LETTER AA" },
    { 0x119A2, "NANDINAGARI LETTER I" },
    { 0x119A3, "NANDINAGARI LETTER II" },
    { 0x119A4, "NANDINAGARI LETTER U" },
    { 0x119A5, "NANDINAGARI LETTER UU" },
    { 0x119A6, "NANDINAGARI LETTER VOCALIC R" },
    { 0x119A7, "NANDINAGARI LETTER VOCALIC RR" },
    { 0x119AA, "NANDINAGARI LETTER E" },
    { 0x119AB, "NANDINAGARI LETTER AI" },
    { 0x119AC, "NANDINAGARI LETTER O" },
    { 0x119AD, "NANDINAGARI LETTER AU" },
    { 0x119AE, "NANDINAGARI LETTER KA" },
    { 0x119AF, "NANDINAGARI LETTER KHA" },
    { 0x119B0, "NANDINAGARI LETTER GA" },
    { 0x119B1, "NANDINAGARI LETTER GHA" },
    { 0x119B2, "NANDINAGARI LETTER NGA" },
    { 0x119B3, "NANDINAGARI LETTER CA" },
    { 0x119B4, "NANDINAGARI LETTER CHA" },
    { 0x119B5, "NANDINAGARI LETTER JA" },
    { 0x119B6, "NANDINAGARI LETTER JHA" },
    { 0x119B7, "NANDINAGARI LETTER NYA" },
    { 0x119B8, "NANDINAGARI LETTER TTA" },
    { 0x119B9, "NANDINAGARI LETTER TTHA" },
    { 0x119BA, "NANDINAGARI LETTER DDA" },
    { 0x119BB, "NANDINAGARI LETTER DDHA" },
    { 0x119BC, "NANDINAGARI LETTER NNA" },
    { 0x119BD, "NANDINAGARI LETTER TA" },
    { 0x119BE, "NANDINAGARI LETTER THA" },
    { 0x119BF, "NANDINAGARI LETTER DA" },
    { 0x119C0, "NANDINAGARI LETTER DHA" },
    { 0x119C1, "NANDINAGARI LETTER NA" },
    { 0x119C2, "NANDINAGARI LETTER PA" },
    { 0x119C3, "NANDINAGARI LETTER PHA" },
    { 0x119C4, "NANDINAGARI LETTER BA" },
    { 0x119C5, "NANDINAGARI LETTER BHA" },
    { 0x119C6, "NANDINAGARI LETTER MA" },
    { 0x119C7, "NANDINAGARI LETTER YA" },
    { 0x119C8, "NANDINAGARI LETTER RA" },
    { 0x119C9, "NANDINAGARI LETTER LA" },
    { 0x119CA, "NANDINAGARI LETTER VA" },
    { 0x119CB, "NANDINAGARI LETTER SHA" },
    { 0x119CC, "NANDINAGARI LETTER SSA" },
    { 0x119CD, "NANDINAGARI LETTER SA" },
    { 0x119CE, "NANDINAGARI LETTER HA" },
    { 0x119CF, "NANDINAGARI LETTER LLA" },
    { 0x119D0, "NANDINAGARI LETTER RRA" },
    { 0x119D1, "NANDINAGARI VOWEL SIGN AA" },
    { 0x119D2, "NANDINAGARI VOWEL SIGN I" },
    { 0x119D3, "NANDINAGARI VOWEL SIGN II" },
    { 0x119D4, "NANDINAGARI VOWEL SIGN U" },
    { 0x119D5, "NANDINAGARI VOWEL SIGN UU" },
    { 0x119D6, "NANDINAGARI VOWEL SIGN VOCALIC R" },
    { 0x119D7, "NANDINAGARI VOWEL SIGN VOCALIC RR" },
    { 0x119DA, "NANDINAGARI VOWEL SIGN E" },
    { 0x119DB, "NANDINAGARI VOWEL SIGN AI" },
    { 0x119DC, "NANDINAGARI VOWEL SIGN O" },
    { 0x119DD, "NANDINAGARI VOWEL SIGN AU" },
    { 0x119DE, "NANDINAGARI SIGN ANUSVARA" },
    { 0x119DF, "NANDINAGARI SIGN VISARGA" },
    { 0x119E0, "NANDINAGARI SIGN VIRAMA" },
    { 0x119E1, "NANDINAGARI SIGN AVAGRAHA" },
    { 0x119E2, "NANDINAGARI SIGN SIDDHAM" },
    { 0x119E3, "NANDINAGARI HEADSTROKE" },
    { 0x119E4, "NANDINAGARI VOWEL SIGN PRISHTHAMATRA E" },
    { 0x11A00, "ZANABAZAR SQUARE LETTER A" },
    { 0x11A01, "ZANABAZAR SQUARE VOWEL SIGN I" },
    { 0x11A02, "ZANABAZAR SQUARE VOWEL SIGN UE" },
    { 0x11A03, "ZANABAZAR SQUARE VOWEL SIGN U" },
    { 0x11A04, "ZANABAZAR SQUARE VOWEL SIGN E" },
    { 0x11A05, "ZANABAZAR SQUARE VOWEL SIGN OE" },
    { 0x11A06, "ZANABAZAR SQUARE VOWEL SIGN O" },
    { 0x11A07, "ZANABAZAR SQUARE VOWEL SIGN AI" },
    { 0x11A08, "ZANABAZAR SQUARE VOWEL SIGN AU" },
    { 0x11A09, "ZANABAZAR SQUARE VOWEL SIGN REVERSED I" },
    { 0x11A0A, "ZANABAZAR SQUARE VOWEL LENGTH MARK" },
    { 0x11A0B, "ZANABAZAR SQUARE LETTER KA" },
    { 0x11A0C, "ZANABAZAR SQUARE LETTER KHA" },
    { 0x11A0D, "ZANABAZAR SQUARE LETTER GA" },
    { 0x11A0E, "ZANABAZAR SQUARE LETTER GHA" },
    { 0x11A0F, "ZANABAZAR SQUARE LETTER NGA" },
    { 0x11A10, "ZANABAZAR SQUARE LETTER CA" },
    { 0x11A11, "ZANABAZAR SQUARE LETTER CHA" },
    { 0x11A12, "ZANABAZAR SQUARE LETTER JA" },
    { 0x11A13, "ZANABAZAR SQUARE LETTER NYA" },
    { 0x11A14, "ZANABAZAR SQUARE LETTER TTA" },
    { 0x11A15, "ZANABAZAR SQUARE LETTER TTHA" },
    { 0x11A16, "ZANABAZAR SQUARE LETTER DDA" },
    { 0x11A17, "ZANABAZAR SQUARE LETTER DDHA" },
    { 0x11A18, "ZANABAZAR SQUARE LETTER NNA" },
    { 0x11A19, "ZANABAZAR SQUARE LETTER TA" },
    { 0x11A1A, "ZANABAZAR SQUARE LETTER THA" },
    { 0x11A1B, "ZANABAZAR SQUARE LETTER DA" },
    { 0x11A1C, "ZANABAZAR SQUARE LETTER DHA" },
    { 0x11A1D, "ZANABAZAR SQUARE LETTER NA" },
    { 0x11A1E, "ZANABAZAR SQUARE LETTER PA" },
    { 0x11A1F, "ZANABAZAR SQUARE LETTER PHA" },
    { 0x11A20, "ZANABAZAR SQUARE LETTER BA" },
    { 0x11A21, "ZANABAZAR SQUARE LETTER BHA" },
    { 0x11A22, "ZANABAZAR SQUARE LETTER MA" },
    { 0x11A23, "ZANABAZAR SQUARE LETTER TSA" },
    { 0x11A24, "ZANABAZAR SQUARE LETTER TSHA" },
    { 0x11A25, "ZANABAZAR SQUARE LETTER DZA" },
    { 0x11A26, "ZANABAZAR SQUARE LETTER DZHA" },
    { 0x11A27, "ZANABAZAR SQUARE LETTER ZHA" },
    { 0x11A28, "ZANABAZAR SQUARE LETTER ZA" },
    { 0x11A29, "ZANABAZAR SQUARE LETTER -A" },
    { 0x11A2A, "ZANABAZAR SQUARE LETTER YA" },
    { 0x11A2B, "ZANABAZAR SQUARE LETTER RA" },
    { 0x11A2C, "ZANABAZAR SQUARE LETTER LA" },
    { 0x11A2D, "ZANABAZAR SQUARE LETTER VA" },
    { 0x11A2E, "ZANABAZAR SQUARE LETTER SHA" },
    { 0x11A2F, "ZANABAZAR SQUARE LETTER SSA" },
    { 0x11A30, "ZANABAZAR SQUARE LETTER SA" },
    { 0x11A31, "ZANABAZAR SQUARE LETTER HA" },
    { 0x11A32, "ZANABAZAR SQUARE LETTER KSSA" },
    { 0x11A33, "ZANABAZAR SQUARE FINAL CONSONANT MARK" },
    { 0x11A34, "ZANABAZAR SQUARE SIGN VIRAMA" },
    { 0x11A35, "ZANABAZAR SQUARE SIGN CANDRABINDU" },
    { 0x11A36, "ZANABAZAR SQUARE SIGN CANDRABINDU WITH ORNAMENT" },
    { 0x11A37, "ZANABAZAR SQUARE SIGN CANDRA WITH ORNAMENT" },
    { 0x11A38, "ZANABAZAR SQUARE SIGN ANUSVARA" },
    { 0x11A39, "ZANABAZAR SQUARE SIGN VISARGA" },
    { 0x11A3A, "ZANABAZAR SQUARE CLUSTER-INITIAL LETTER RA" },
    { 0x11A3B, "ZANABAZAR SQUARE CLUSTER-FINAL LETTER YA" },
    { 0x11A3C, "ZANABAZAR SQUARE CLUSTER-FINAL LETTER RA" },
    { 0x11A3D, "ZANABAZAR SQUARE CLUSTER-FINAL LETTER LA" },
    { 0x11A3E, "ZANABAZAR SQUARE CLUSTER-FINAL LETTER VA" },
    { 0x11A3F, "ZANABAZAR SQUARE INITIAL HEAD MARK" },
    { 0x11A40, "ZANABAZAR SQUARE CLOSING HEAD MARK" },
    { 0x11A41, "ZANABAZAR SQUARE MARK TSHEG" },
    { 0x11A42, "ZANABAZAR SQUARE MARK SHAD" },
    { 0x11A43, "ZANABAZAR SQUARE MARK DOUBLE SHAD" },
    { 0x11A44, "ZANABAZAR SQUARE MARK LONG TSHEG" },
    { 0x11A45, "ZANABAZAR SQUARE INITIAL DOUBLE-LINED HEAD MARK" },
    { 0x11A46, "ZANABAZAR SQUARE CLOSING DOUBLE-LINED HEAD MARK" },
    { 0x11A47, "ZANABAZAR SQUARE SUBJOINER" },
    { 0x11A50, "SOYOMBO LETTER A" },
    { 0x11A51, "SOYOMBO VOWEL SIGN I" },
    { 0x11A52, "SOYOMBO VOWEL SIGN UE" },
    { 0x11A53, "SOYOMBO VOWEL SIGN U" },
    { 0x11A54, "SOYOMBO VOWEL SIGN E" },
    { 0x11A55, "SOYOMBO VOWEL SIGN O" },
    { 0x11A56, "SOYOMBO VOWEL SIGN OE" },
    { 0x11A57, "SOYOMBO VOWEL SIGN AI" },
    { 0x11A58, "SOYOMBO VOWEL SIGN AU" },
    { 0x11A59, "SOYOMBO VOWEL SIGN VOCALIC R" },
    { 0x11A5A, "SOYOMBO VOWEL SIGN VOCALIC L" },
    { 0x11A5B, "SOYOMBO VOWEL LENGTH MARK" },
    { 0x11A5C, "SOYOMBO LETTER KA" },
    { 0x11A5D, "SOYOMBO LETTER KHA" },
    { 0x11A5E, "SOYOMBO LETTER GA" },
    { 0x11A5F, "SOYOMBO LETTER GHA" },
    { 0x11A60, "SOYOMBO LETTER NGA" },
    { 0x11A61, "SOYOMBO LETTER CA" },
    { 0x11A62, "SOYOMBO LETTER CHA" },
    { 0x11A63, "SOYOMBO LETTER JA" },
    { 0x11A64, "SOYOMBO LETTER JHA" },
    { 0x11A65, "SOYOMBO LETTER NYA" },
    { 0x11A66, "SOYOMBO LETTER TTA" },
    { 0x11A67, "SOYOMBO LETTER TTHA" },
    { 0x11A68, "SOYOMBO LETTER DDA" },
    { 0x11A69, "SOYOMBO LETTER DDHA" },
    { 0x11A6A, "SOYOMBO LETTER NNA" },
    { 0x11A6B, "SOYOMBO LETTER TA" },
    { 0x11A6C, "SOYOMBO LETTER THA" },
    { 0x11A6D, "SOYOMBO LETTER DA" },
    { 0x11A6E, "SOYOMBO LETTER DHA" },
    { 0x11A6F, "SOYOMBO LETTER NA" },
    { 0x11A70, "SOYOMBO LETTER PA" },
    { 0x11A71, "SOYOMBO LETTER PHA" },
    { 0x11A72, "SOYOMBO LETTER BA" },
    { 0x11A73, "SOYOMBO LETTER BHA" },
    { 0x11A74, "SOYOMBO LETTER MA" },
    { 0x11A75, "SOYOMBO LETTER TSA" },
    { 0x11A76, "SOYOMBO LETTER TSHA" },
    { 0x11A77, "SOYOMBO LETTER DZA" },
    { 0x11A78, "SOYOMBO LETTER ZHA" },
    { 0x11A79, "SOYOMBO LETTER ZA" },
    { 0x11A7A, "SOYOMBO LETTER -A" },
    { 0x11A7B, "SOYOMBO LETTER YA" },
    { 0x11A7C, "SOYOMBO LETTER RA" },
    { 0x11A7D, "SOYOMBO LETTER LA" },
    { 0x11A7E, "SOYOMBO LETTER VA" },
    { 0x11A7F, "SOYOMBO LETTER SHA" },
    { 0x11A80, "SOYOMBO LETTER SSA" },
    { 0x11A81, "SOYOMBO LETTER SA" },
    { 0x11A82, "SOYOMBO LETTER HA" },
    { 0x11A83, "SOYOMBO LETTER KSSA" },
    { 0x11A84, "SOYOMBO SIGN JIHVAMULIYA" },
    { 0x11A85, "SOYOMBO SIGN UPADHMANIYA" },
    { 0x11A86, "SOYOMBO CLUSTER-INITIAL LETTER RA" },
    { 0x11A87, "SOYOMBO CLUSTER-INITIAL LETTER LA" },
    { 0x11A88, "SOYOMBO CLUSTER-INITIAL LETTER SHA" },
    { 0x11A89, "SOYOMBO CLUSTER-INITIAL LETTER SA" },
    { 0x11A8A, "SOYOMBO FINAL CONSONANT SIGN G" },
    { 0x11A8B, "SOYOMBO FINAL CONSONANT SIGN K" },
    { 0x11A8C, "SOYOMBO FINAL CONSONANT SIGN NG" },
    { 0x11A8D, "SOYOMBO FINAL CONSONANT SIGN D" },
    { 0x11A8E, "SOYOMBO FINAL CONSONANT SIGN N" },
    { 0x11A8F, "SOYOMBO FINAL CONSONANT SIGN B" },
    { 0x11A90, "SOYOMBO FINAL CONSONANT SIGN M" },
    { 0x11A91, "SOYOMBO FINAL CONSONANT SIGN R" },
    { 0x11A92, "SOYOMBO FINAL CONSONANT SIGN L" },
    { 0x11A93, "SOYOMBO FINAL CONSONANT SIGN SH" },
    { 0x11A94, "SOYOMBO FINAL CONSONANT SIGN S" },
    { 0x11A95, "SOYOMBO FINAL CONSONANT SIGN -A" },
    { 0x11A96, "SOYOMBO SIGN ANUSVARA" },
    { 0x11A97, "SOYOMBO SIGN VISARGA" },
    { 0x11A98, "SOYOMBO GEMINATION MARK" },
    { 0x11A99, "SOYOMBO SUBJOINER" },
    { 0x11A9A, "SOYOMBO MARK TSHEG" },
    { 0x11A9B, "SOYOMBO MARK SHAD" },
    { 0x11A9C, "SOYOMBO MARK DOUBLE SHAD" },
    { 0x11A9D, "SOYOMBO MARK PLUTA" },
    { 0x11A9E, "SOYOMBO HEAD MARK WITH MOON AND SUN AND TRIPLE FLAME" },
    { 0x11A9F, "SOYOMBO HEAD MARK WITH MOON AND SUN AND FLAME" },
    { 0x11AA0, "SOYOMBO HEAD MARK WITH MOON AND SUN" },
    { 0x11AA1, "SOYOMBO TERMINAL MARK-1" },
    { 0x11AA2, "SOYOMBO TERMINAL MARK-2" },
    { 0x11AB0, "CANADIAN SYLLABICS NATTILIK HI" },
    { 0x11AB1, "CANADIAN SYLLABICS NATTILIK HII" },
    { 0x11AB2, "CANADIAN SYLLABICS NATTILIK HO" },
    { 0x11AB3, "CANADIAN SYLLABICS NATTILIK HOO" },
    { 0x11AB4, "CANADIAN SYLLABICS NATTILIK HA" },
    { 0x11AB5, "CANADIAN SYLLABICS NATTILIK HAA" },
    { 0x11AB6, "CANADIAN SYLLABICS NATTILIK SHRI" },
    { 0x11AB7, "CANADIAN SYLLABICS NATTILIK SHRII" },
    { 0x11AB8, "CANADIAN SYLLABICS NATTILIK SHRO" },
    { 0x11AB9, "CANADIAN SYLLABICS NATTILIK SHROO" },
    { 0x11ABA, "CANADIAN SYLLABICS NATTILIK SHRA" },
    { 0x11ABB, "CANADIAN SYLLABICS NATTILIK SHRAA" },
    { 0x11ABC, "CANADIAN SYLLABICS SPE" },
    { 0x11ABD, "CANADIAN SYLLABICS SPI" },
    { 0x11ABE, "CANADIAN SYLLABICS SPO" },
    { 0x11ABF, "CANADIAN SYLLABICS SPA" },
    { 0x11AC0, "PAU CIN HAU LETTER PA" },
    { 0x11AC1, "PAU CIN HAU LETTER KA" },
    { 0x11AC2, "PAU CIN HAU LETTER LA" },
    { 0x11AC3, "PAU CIN HAU LETTER MA" },
    { 0x11AC4, "PAU CIN HAU LETTER DA" },
    { 0x11AC5, "PAU CIN HAU LETTER ZA" },
    { 0x11AC6, "PAU CIN HAU LETTER VA" },
    { 0x11AC7, "PAU CIN HAU LETTER NGA" },
    { 0x11AC8, "PAU CIN HAU LETTER HA" },
    { 0x11AC9, "PAU CIN HAU LETTER GA" },
    { 0x11ACA, "PAU CIN HAU LETTER KHA" },
    { 0x11ACB, "PAU CIN HAU LETTER SA" },
    { 0x11ACC, "PAU CIN HAU LETTER BA" },
    { 0x11ACD, "PAU CIN HAU LETTER CA" },
    { 0x11ACE, "PAU CIN HAU LETTER TA" },
    { 0x11ACF, "PAU CIN HAU LETTER THA" },
    { 0x11AD0, "PAU CIN HAU LETTER NA" },
    { 0x11AD1, "PAU CIN HAU LETTER PHA" },
    { 0x11AD2, "PAU CIN HAU LETTER RA" },
    { 0x11AD3, "PAU CIN HAU LETTER FA" },
    { 0x11AD4, "PAU CIN HAU LETTER CHA" },
    { 0x11AD5, "PAU CIN HAU LETTER A" },
    { 0x11AD6, "PAU CIN HAU LETTER E" },
    { 0x11AD7, "PAU CIN HAU LETTER I" },
    { 0x11AD8, "PAU CIN HAU LETTER O" },
    { 0x11AD9, "PAU CIN HAU LETTER U" },
    { 0x11ADA, "PAU CIN HAU LETTER UA" },
    { 0x11ADB, "PAU CIN HAU LETTER IA" },
    { 0x11ADC, "PAU CIN HAU LETTER FINAL P" },
    { 0x11ADD, "PAU CIN HAU LETTER FINAL K" },
    { 0x11ADE, "PAU CIN HAU LETTER FINAL T" },
    { 0x11ADF, "PAU CIN HAU LETTER FINAL M" },
    { 0x11AE0, "PAU CIN HAU LETTER FINAL N" },
    { 0x11AE1, "PAU CIN HAU LETTER FINAL L" },
    { 0x11AE2, "PAU CIN HAU LETTER FINAL W" },
    { 0x11AE3, "PAU CIN HAU LETTER FINAL NG" },
    { 0x11AE4, "PAU CIN HAU LETTER FINAL Y" },
    { 0x11AE5, "PAU CIN HAU RISING TONE LONG" },
    { 0x11AE6, "PAU CIN HAU RISING TONE" },
    { 0x11AE7, "PAU CIN HAU SANDHI GLOTTAL STOP" },
    { 0x11AE8, "PAU CIN HAU RISING TONE LONG FINAL" },
    { 0x11AE9, "PAU CIN HAU RISING TONE FINAL" },
    { 0x11AEA, "PAU CIN HAU SANDHI GLOTTAL STOP FINAL" },
    { 0x11AEB, "PAU CIN HAU SANDHI TONE LONG" },
    { 0x11AEC, "PAU CIN HAU SANDHI TONE" },
    { 0x11AED, "PAU CIN HAU SANDHI TONE LONG FINAL" },
    { 0x11AEE, "PAU CIN HAU SANDHI TONE FINAL" },
    { 0x11AEF, "PAU CIN HAU MID-LEVEL TONE" },
    { 0x11AF0, "PAU CIN HAU GLOTTAL STOP VARIANT" },
    { 0x11AF1, "PAU CIN HAU MID-LEVEL TONE LONG FINAL" },
    { 0x11AF2, "PAU CIN HAU MID-LEVEL TONE FINAL" },
    { 0x11AF3, "PAU CIN HAU LOW-FALLING TONE LONG" },
    { 0x11AF4, "PAU CIN HAU LOW-FALLING TONE" },
    { 0x11AF5, "PAU CIN HAU GLOTTAL STOP" },
    { 0x11AF6, "PAU CIN HAU LOW-FALLING TONE LONG FINAL" },
    { 0x11AF7, "PAU CIN HAU LOW-FALLING TONE FINAL" },
    { 0x11AF8, "PAU CIN HAU GLOTTAL STOP FINAL" },
    { 0x11B00, "DEVANAGARI HEAD MARK" },
    { 0x11B01, "DEVANAGARI HEAD MARK WITH HEADSTROKE" },
    { 0x11B02, "DEVANAGARI SIGN BHALE" },
    { 0x11B03, "DEVANAGARI SIGN BHALE WITH HOOK" },
    { 0x11B04, "DEVANAGARI SIGN EXTENDED BHALE" },
    { 0x11B05, "DEVANAGARI SIGN EXTENDED BHALE WITH HOOK" },
    { 0x11B06, "DEVANAGARI SIGN WESTERN FIVE-LIKE BHALE" },
    { 0x11B07, "DEVANAGARI SIGN WESTERN NINE-LIKE BHALE" },
    { 0x11B08, "DEVANAGARI SIGN REVERSED NINE-LIKE BHALE" },
    { 0x11B09, "DEVANAGARI SIGN MINDU" },
    { 0x11C00, "BHAIKSUKI LETTER A" },
    { 0x11C01, "BHAIKSUKI LETTER AA" },
    { 0x11C02, "BHAIKSUKI LETTER I" },
    { 0x11C03, "BHAIKSUKI LETTER II" },
    { 0x11C04, "BHAIKSUKI LETTER U" },
    { 0x11C05, "BHAIKSUKI LETTER UU" },
    { 0x11C06, "BHAIKSUKI LETTER VOCALIC R" },
    { 0x11C07, "BHAIKSUKI LETTER VOCALIC RR" },
    { 0x11C08, "BHAIKSUKI LETTER VOCALIC L" },
    { 0x11C0A, "BHAIKSUKI LETTER E" },
    { 0x11C0B, "BHAIKSUKI LETTER AI" },
    { 0x11C0C, "BHAIKSUKI LETTER O" },
    { 0x11C0D, "BHAIKSUKI LETTER AU" },
    { 0x11C0E, "BHAIKSUKI LETTER KA" },
    { 0x11C0F, "BHAIKSUKI LETTER KHA" },
    { 0x11C10, "BHAIKSUKI LETTER GA" },
    { 0x11C11, "BHAIKSUKI LETTER GHA" },
    { 0x11C12, "BHAIKSUKI LETTER NGA" },
    { 0x11C13, "BHAIKSUKI LETTER CA" },
    { 0x11C14, "BHAIKSUKI LETTER CHA" },
    { 0x11C15, "BHAIKSUKI LETTER JA" },
    { 0x11C16, "BHAIKSUKI LETTER JHA" },
    { 0x11C17, "BHAIKSUKI LETTER NYA" },
    { 0x11C18, "BHAIKSUKI LETTER TTA" },
    { 0x11C19, "BHAIKSUKI LETTER TTHA" },
    { 0x11C1A, "BHAIKSUKI LETTER DDA" },
    { 0x11C1B, "BHAIKSUKI LETTER DDHA" },
    { 0x11C1C, "BHAIKSUKI LETTER NNA" },
    { 0x11C1D, "BHAIKSUKI LETTER TA" },
    { 0x11C1E, "BHAIKSUKI LETTER THA" },
    { 0x11C1F, "BHAIKSUKI LETTER DA" },
    { 0x11C20, "BHAIKSUKI LETTER DHA" },
    { 0x11C21, "BHAIKSUKI LETTER NA" },
    { 0x11C22, "BHAIKSUKI LETTER PA" },
    { 0x11C23, "BHAIKSUKI LETTER PHA" },
    { 0x11C24, "BHAIKSUKI LETTER BA" },
    { 0x11C25, "BHAIKSUKI LETTER BHA" },
    { 0x11C26, "BHAIKSUKI LETTER MA" },
    { 0x11C27, "BHAIKSUKI LETTER YA" },
    { 0x11C28, "BHAIKSUKI LETTER RA" },
    { 0x11C29, "BHAIKSUKI LETTER LA" },
    { 0x11C2A, "BHAIKSUKI LETTER VA" },
    { 0x11C2B, "BHAIKSUKI LETTER SHA" },
    { 0x11C2C, "BHAIKSUKI LETTER SSA" },
    { 0x11C2D, "BHAIKSUKI LETTER SA" },
    { 0x11C2E, "BHAIKSUKI LETTER HA" },
    { 0x11C2F, "BHAIKSUKI VOWEL SIGN AA" },
    { 0x11C30, "BHAIKSUKI VOWEL SIGN I" },
    { 0x11C31, "BHAIKSUKI VOWEL SIGN II" },
    { 0x11C32, "BHAIKSUKI VOWEL SIGN U" },
    { 0x11C33, "BHAIKSUKI VOWEL SIGN UU" },
    { 0x11C34, "BHAIKSUKI VOWEL SIGN VOCALIC R" },
    { 0x11C35, "BHAIKSUKI VOWEL SIGN VOCALIC RR" },
    { 0x11C36, "BHAIKSUKI VOWEL SIGN VOCALIC L" },
    { 0x11C38, "BHAIKSUKI VOWEL SIGN E" },
    { 0x11C39, "BHAIKSUKI VOWEL SIGN AI" },
    { 0x11C3A, "BHAIKSUKI VOWEL SIGN O" },
    { 0x11C3B, "BHAIKSUKI VOWEL SIGN AU" },
    { 0x11C3C, "BHAIKSUKI SIGN CANDRABINDU" },
    { 0x11C3D, "BHAIKSUKI SIGN ANUSVARA" },
    { 0x11C3E, "BHAIKSUKI SIGN VISARGA" },
    { 0x11C3F, "BHAIKSUKI SIGN VIRAMA" },
    { 0x11C40, "BHAIKSUKI SIGN AVAGRAHA" },
    { 0x11C41, "BHAIKSUKI DANDA" },
    { 0x11C42, "BHAIKSUKI DOUBLE DANDA" },
    { 0x11C43, "BHAIKSUKI WORD SEPARATOR" },
    { 0x11C44, "BHAIKSUKI GAP FILLER-1" },
    { 0x11C45, "BHAIKSUKI GAP FILLER-2" },
    { 0x11C50, "BHAIKSUKI DIGIT ZERO" },
    { 0x11C51, "BHAIKSUKI DIGIT ONE" },
    { 0x11C52, "BHAIKSUKI DIGIT TWO" },
    { 0x11C53, "BHAIKSUKI DIGIT THREE" },
    { 0x11C54, "BHAIKSUKI DIGIT FOUR" },
    { 0x11C55, "BHAIKSUKI DIGIT FIVE" },
    { 0x11C56, "BHAIKSUKI DIGIT SIX" },
    { 0x11C57, "BHAIKSUKI DIGIT SEVEN" },
    { 0x11C58, "BHAIKSUKI DIGIT EIGHT" },
    { 0x11C59, "BHAIKSUKI DIGIT NINE" },
    { 0x11C5A, "BHAIKSUKI NUMBER ONE" },
    { 0x11C5B, "BHAIKSUKI NUMBER TWO" },
    { 0x11C5C, "BHAIKSUKI NUMBER THREE" },
    { 0x11C5D, "BHAIKSUKI NUMBER FOUR" },
    { 0x11C5E, "BHAIKSUKI NUMBER FIVE" },
    { 0x11C5F, "BHAIKSUKI NUMBER SIX" },
    { 0x11C60, "BHAIKSUKI NUMBER SEVEN" },
    { 0x11C61, "BHAIKSUKI NUMBER EIGHT" },
    { 0x11C62, "BHAIKSUKI NUMBER NINE" },
    { 0x11C63, "BHAIKSUKI NUMBER TEN" },
    { 0x11C64, "BHAIKSUKI NUMBER TWENTY" },
    { 0x11C65, "BHAIKSUKI NUMBER THIRTY" },
    { 0x11C66, "BHAIKSUKI NUMBER FORTY" },
    { 0x11C67, "BHAIKSUKI NUMBER FIFTY" },
    { 0x11C68, "BHAIKSUKI NUMBER SIXTY" },
    { 0x11C69, "BHAIKSUKI NUMBER SEVENTY" },
    { 0x11C6A, "BHAIKSUKI NUMBER EIGHTY" },
    { 0x11C6B, "BHAIKSUKI NUMBER NINETY" },
    { 0x11C6C, "BHAIKSUKI HUNDREDS UNIT MARK" },
    { 0x11C70, "MARCHEN HEAD MARK" },
    { 0x11C71, "MARCHEN MARK SHAD" },
    { 0x11C72, "MARCHEN LETTER KA" },
    { 0x11C73, "MARCHEN LETTER KHA" },
    { 0x11C74, "MARCHEN LETTER GA" },
    { 0x11C75, "MARCHEN LETTER NGA" },
    { 0x11C76, "MARCHEN LETTER CA" },
    { 0x11C77, "MARCHEN LETTER CHA" },
    { 0x11C78, "MARCHEN LETTER JA" },
    { 0x11C79, "MARCHEN LETTER NYA" },
    { 0x11C7A, "MARCHEN LETTER TA" },
    { 0x11C7B, "MARCHEN LETTER THA" },
    { 0x11C7C, "MARCHEN LETTER DA" },
    { 0x11C7D, "MARCHEN LETTER NA" },
    { 0x11C7E, "MARCHEN LETTER PA" },
    { 0x11C7F, "MARCHEN LETTER PHA" },
    { 0x11C80, "MARCHEN LETTER BA" },
    { 0x11C81, "MARCHEN LETTER MA" },
    { 0x11C82, "MARCHEN LETTER TSA" },
    { 0x11C83, "MARCHEN LETTER TSHA" },
    { 0x11C84, "MARCHEN LETTER DZA" },
    { 0x11C85, "MARCHEN LETTER WA" },
    { 0x11C86, "MARCHEN LETTER ZHA" },
    { 0x11C87, "MARCHEN LETTER ZA" },
    { 0x11C88, "MARCHEN LETTER -A" },
    { 0x11C89, "MARCHEN LETTER YA" },
    { 0x11C8A, "MARCHEN LETTER RA" },
    { 0x11C8B, "MARCHEN LETTER LA" },
    { 0x11C8C, "MARCHEN LETTER SHA" },
    { 0x11C8D, "MARCHEN LETTER SA" },
    { 0x11C8E, "MARCHEN LETTER HA" },
    { 0x11C8F, "MARCHEN LETTER A" },
    { 0x11C92, "MARCHEN SUBJOINED LETTER KA" },
    { 0x11C93, "MARCHEN SUBJOINED LETTER KHA" },
    { 0x11C94, "MARCHEN SUBJOINED LETTER GA" },
    { 0x11C95, "MARCHEN SUBJOINED LETTER NGA" },
    { 0x11C96, "MARCHEN SUBJOINED LETTER CA" },
    { 0x11C97, "MARCHEN SUBJOINED LETTER CHA" },
    { 0x11C98, "MARCHEN SUBJOINED LETTER JA" },
    { 0x11C99, "MARCHEN SUBJOINED LETTER NYA" },
    { 0x11C9A, "MARCHEN SUBJOINED LETTER TA" },
    { 0x11C9B, "MARCHEN SUBJOINED LETTER THA" },
    { 0x11C9C, "MARCHEN SUBJOINED LETTER DA" },
    { 0x11C9D, "MARCHEN SUBJOINED LETTER NA" },
    { 0x11C9E, "MARCHEN SUBJOINED LETTER PA" },
    { 0x11C9F, "MARCHEN SUBJOINED LETTER PHA" },
    { 0x11CA0, "MARCHEN SUBJOINED LETTER BA" },
    { 0x11CA1, "MARCHEN SUBJOINED LETTER MA" },
    { 0x11CA2, "MARCHEN SUBJOINED LETTER TSA" },
    { 0x11CA3, "MARCHEN SUBJOINED LETTER TSHA" },
    { 0x11CA4, "MARCHEN SUBJOINED LETTER DZA" },
    { 0x11CA5, "MARCHEN SUBJOINED LETTER WA" },
    { 0x11CA6, "MARCHEN SUBJOINED LETTER ZHA" },
    { 0x11CA7, "MARCHEN SUBJOINED LETTER ZA" },
    { 0x11CA9, "MARCHEN SUBJOINED LETTER YA" },
    { 0x11CAA, "MARCHEN SUBJOINED LETTER RA" },
    { 0x11CAB, "MARCHEN SUBJOINED LETTER LA" },
    { 0x11CAC, "MARCHEN SUBJOINED LETTER SHA" },
    { 0x11CAD, "MARCHEN SUBJOINED LETTER SA" },
    { 0x11CAE, "MARCHEN SUBJOINED LETTER HA" },
    { 0x11CAF, "MARCHEN SUBJOINED LETTER A" },
    { 0x11CB0, "MARCHEN VOWEL SIGN AA" },
    { 0x11CB1, "MARCHEN VOWEL SIGN I" },
    { 0x11CB2, "MARCHEN VOWEL SIGN U" },
    { 0x11CB3, "MARCHEN VOWEL SIGN E" },
    { 0x11CB4, "MARCHEN VOWEL SIGN O" },
    { 0x11CB5, "MARCHEN SIGN ANUSVARA" },
    { 0x11CB6, "MARCHEN SIGN CANDRABINDU" },
    { 0x11D00, "MASARAM GONDI LETTER A" },
    { 0x11D01, "MASARAM GONDI LETTER AA" },
    { 0x11D02, "MASARAM GONDI LETTER I" },
    { 0x11D03, "MASARAM GONDI LETTER II" },
    { 0x11D04, "MASARAM GONDI LETTER U" },
    { 0x11D05, "MASARAM GONDI LETTER UU" },
    { 0x11D06, "MASARAM GONDI LETTER E" },
    { 0x11D08, "MASARAM GONDI LETTER AI" },
    { 0x11D09, "MASARAM GONDI LETTER O" },
    { 0x11D0B, "MASARAM GONDI LETTER AU" },
    { 0x11D0C, "MASARAM GONDI LETTER KA" },
    { 0x11D0D, "MASARAM GONDI LETTER KHA" },
    { 0x11D0E, "MASARAM GONDI LETTER GA" },
    { 0x11D0F, "MASARAM GONDI LETTER GHA" },
    { 0x11D10, "MASARAM GONDI LETTER NGA" },
    { 0x11D11, "MASARAM GONDI LETTER CA" },
    { 0x11D12, "MASARAM GONDI LETTER CHA" },
    { 0x11D13, "MASARAM GONDI LETTER JA" },
    { 0x11D14, "MASARAM GONDI LETTER JHA" },
    { 0x11D15, "MASARAM GONDI LETTER NYA" },
    { 0x11D16, "MASARAM GONDI LETTER TTA" },
    { 0x11D17, "MASARAM GONDI LETTER TTHA" },
    { 0x11D18, "MASARAM GONDI LETTER DDA" },
    { 0x11D19, "MASARAM GONDI LETTER DDHA" },
    { 0x11D1A, "MASARAM GONDI LETTER NNA" },
    { 0x11D1B, "MASARAM GONDI LETTER TA" },
    { 0x11D1C, "MASARAM GONDI LETTER THA" },
    { 0x11D1D, "MASARAM GONDI LETTER DA" },
    { 0x11D1E, "MASARAM GONDI LETTER DHA" },
    { 0x11D1F, "MASARAM GONDI LETTER NA" },
    { 0x11D20, "MASARAM GONDI LETTER PA" },
    { 0x11D21, "MASARAM GONDI LETTER PHA" },
    { 0x11D22, "MASARAM GONDI LETTER BA" },
    { 0x11D23, "MASARAM GONDI LETTER BHA" },
    { 0x11D24, "MASARAM GONDI LETTER MA" },
    { 0x11D25, "MASARAM GONDI LETTER YA" },
    { 0x11D26, "MASARAM GONDI LETTER RA" },
    { 0x11D27, "MASARAM GONDI LETTER LA" },
    { 0x11D28, "MASARAM GONDI LETTER VA" },
    { 0x11D29, "MASARAM GONDI LETTER SHA" },
    { 0x11D2A, "MASARAM GONDI LETTER SSA" },
    { 0x11D2B, "MASARAM GONDI LETTER SA" },
    { 0x11D2C, "MASARAM GONDI LETTER HA" },
    { 0x11D2D, "MASARAM GONDI LETTER LLA" },
    { 0x11D2E, "MASARAM GONDI LETTER KSSA" },
    { 0x11D2F, "MASARAM GONDI LETTER JNYA" },
    { 0x11D30, "MASARAM GONDI LETTER TRA" },
    { 0x11D31, "MASARAM GONDI VOWEL SIGN AA" },
    { 0x11D32, "MASARAM GONDI VOWEL SIGN I" },
    { 0x11D33, "MASARAM GONDI VOWEL SIGN II" },
    { 0x11D34, "MASARAM GONDI VOWEL SIGN U" },
    { 0x11D35, "MASARAM GONDI VOWEL SIGN UU" },
    { 0x11D36, "MASARAM GONDI VOWEL SIGN VOCALIC R" },
    { 0x11D3A, "MASARAM GONDI VOWEL SIGN E" },
    { 0x11D3C, "MASARAM GONDI VOWEL SIGN AI" },
    { 0x11D3D, "MASARAM GONDI VOWEL SIGN O" },
    { 0x11D3F, "MASARAM GONDI VOWEL SIGN AU" },
    { 0x11D40, "MASARAM GONDI SIGN ANUSVARA" },
    { 0x11D41, "MASARAM GONDI SIGN VISARGA" },
    { 0x11D42, "MASARAM GONDI SIGN NUKTA" },
    { 0x11D43, "MASARAM GONDI SIGN CANDRA" },
    { 0x11D44, "MASARAM GONDI SIGN HALANTA" },
    { 0x11D45, "MASARAM GONDI VIRAMA" },
    { 0x11D46, "MASARAM GONDI REPHA" },
    { 0x11D47, "MASARAM GONDI RA-KARA" },
    { 0x11D50, "MASARAM GONDI DIGIT ZERO" },
    { 0x11D51, "MASARAM GONDI DIGIT ONE" },
    { 0x11D52, "MASARAM GONDI DIGIT TWO" },
    { 0x11D53, "MASARAM GONDI DIGIT THREE" },
    { 0x11D54, "MASARAM GONDI DIGIT FOUR" },
    { 0x11D55, "MASARAM GONDI DIGIT FIVE" },
    { 0x11D56, "MASARAM GONDI DIGIT SIX" },
    { 0x11D57, "MASARAM GONDI DIGIT SEVEN" },
    { 0x11D58, "MASARAM GONDI DIGIT EIGHT" },
    { 0x11D59, "MASARAM GONDI DIGIT NINE" },
    { 0x11D60, "GUNJALA GONDI LETTER A" },
    { 0x11D61, "GUNJALA GONDI LETTER AA" },
    { 0x11D62, "GUNJALA GONDI LETTER I" },
    { 0x11D63, "GUNJALA GONDI LETTER II" },
    { 0x11D64, "GUNJALA GONDI LETTER U" },
    { 0x11D65, "GUNJALA GONDI LETTER UU" },
    { 0x11D67, "GUNJALA GONDI LETTER EE" },
    { 0x11D68, "GUNJALA GONDI LETTER AI" },
    { 0x11D6A, "GUNJALA GONDI LETTER OO" },
    { 0x11D6B, "GUNJALA GONDI LETTER AU" },
    { 0x11D6C, "GUNJALA GONDI LETTER YA" },
    { 0x11D6D, "GUNJALA GONDI LETTER VA" },
    { 0x11D6E, "GUNJALA GONDI LETTER BA" },
    { 0x11D6F, "GUNJALA GONDI LETTER BHA" },
    { 0x11D70, "GUNJALA GONDI LETTER MA" },
    { 0x11D71, "GUNJALA GONDI LETTER KA" },
    { 0x11D72, "GUNJALA GONDI LETTER KHA" },
    { 0x11D73, "GUNJALA GONDI LETTER TA" },
    { 0x11D74, "GUNJALA GONDI LETTER THA" },
    { 0x11D75, "GUNJALA GONDI LETTER LA" },
    { 0x11D76, "GUNJALA GONDI LETTER GA" },
    { 0x11D77, "GUNJALA GONDI LETTER GHA" },
    { 0x11D78, "GUNJALA GONDI LETTER DA" },
    { 0x11D79, "GUNJALA GONDI LETTER DHA" },
    { 0x11D7A, "GUNJALA GONDI LETTER NA" },
    { 0x11D7B, "GUNJALA GONDI LETTER CA" },
    { 0x11D7C, "GUNJALA GONDI LETTER CHA" },
    { 0x11D7D, "GUNJALA GONDI LETTER TTA" },
    { 0x11D7E, "GUNJALA GONDI LETTER TTHA" },
    { 0x11D7F, "GUNJALA GONDI LETTER LLA" },
    { 0x11D80, "GUNJALA GONDI LETTER JA" },
    { 0x11D81, "GUNJALA GONDI LETTER JHA" },
    { 0x11D82, "GUNJALA GONDI LETTER DDA" },
    { 0x11D83, "GUNJALA GONDI LETTER DDHA" },
    { 0x11D84, "GUNJALA GONDI LETTER NGA" },
    { 0x11D85, "GUNJALA GONDI LETTER PA" },
    { 0x11D86, "GUNJALA GONDI LETTER PHA" },
    { 0x11D87, "GUNJALA GONDI LETTER HA" },
    { 0x11D88, "GUNJALA GONDI LETTER RA" },
    { 0x11D89, "GUNJALA GONDI LETTER SA" },
    { 0x11D8A, "GUNJALA GONDI VOWEL SIGN AA" },
    { 0x11D8B, "GUNJALA GONDI VOWEL SIGN I" },
    { 0x11D8C, "GUNJALA GONDI VOWEL SIGN II" },
    { 0x11D8D, "GUNJALA GONDI VOWEL SIGN U" },
    { 0x11D8E, "GUNJALA GONDI VOWEL SIGN UU" },
    { 0x11D90, "GUNJALA GONDI VOWEL SIGN EE" },
    { 0x11D91, "GUNJALA GONDI VOWEL SIGN AI" },
    { 0x11D93, "GUNJALA GONDI VOWEL SIGN OO" },
    { 0x11D94, "GUNJALA GONDI VOWEL SIGN AU" },
    { 0x11D95, "GUNJALA GONDI SIGN ANUSVARA" },
    { 0x11D96, "GUNJALA GONDI SIGN VISARGA" },
    { 0x11D97, "GUNJALA GONDI VIRAMA" },
    { 0x11D98, "GUNJALA GONDI OM" },
    { 0x11DA0, "GUNJALA GONDI DIGIT ZERO" },
    { 0x11DA1, "GUNJALA GONDI DIGIT ONE" },
    { 0x11DA2, "GUNJALA GONDI DIGIT TWO" },
    { 0x11DA3, "GUNJALA GONDI DIGIT THREE" },
    { 0x11DA4, "GUNJALA GONDI DIGIT FOUR" },
    { 0x11DA5, "GUNJALA GONDI DIGIT FIVE" },
    { 0x11DA6, "GUNJALA GONDI DIGIT SIX" },
    { 0x11DA7, "GUNJALA GONDI DIGIT SEVEN" },
    { 0x11DA8, "GUNJALA GONDI DIGIT EIGHT" },
    { 0x11DA9, "GUNJALA GONDI DIGIT NINE" },
    { 0x11EE0, "MAKASAR LETTER KA" },
    { 0x11EE1, "MAKASAR LETTER GA" },
    { 0x11EE2, "MAKASAR LETTER NGA" },
    { 0x11EE3, "MAKASAR LETTER PA" },
    { 0x11EE4, "MAKASAR LETTER BA" },
    { 0x11EE5, "MAKASAR LETTER MA" },
    { 0x11EE6, "MAKASAR LETTER TA" },
    { 0x11EE7, "MAKASAR LETTER DA" },
    { 0x11EE8, "MAKASAR LETTER NA" },
    { 0x11EE9, "MAKASAR LETTER CA" },
    { 0x11EEA, "MAKASAR LETTER JA" },
    { 0x11EEB, "MAKASAR LETTER NYA" },
    { 0x11EEC, "MAKASAR LETTER YA" },
    { 0x11EED, "MAKASAR LETTER RA" },
    { 0x11EEE, "MAKASAR LETTER LA" },
    { 0x11EEF, "MAKASAR LETTER VA" },
    { 0x11EF0, "MAKASAR LETTER SA" },
    { 0x11EF1, "MAKASAR LETTER A" },
    { 0x11EF2, "MAKASAR ANGKA" },
    { 0x11EF3, "MAKASAR VOWEL SIGN I" },
    { 0x11EF4, "MAKASAR VOWEL SIGN U" },
    { 0x11EF5, "MAKASAR VOWEL SIGN E" },
    { 0x11EF6, "MAKASAR VOWEL SIGN O" },
    { 0x11EF7, "MAKASAR PASSIMBANG" },
    { 0x11EF8, "MAKASAR END OF SECTION" },
    { 0x11F00, "KAWI SIGN CANDRABINDU" },
    { 0x11F01, "KAWI SIGN ANUSVARA" },
    { 0x11F02, "KAWI SIGN REPHA" },
    { 0x11F03, "KAWI SIGN VISARGA" },
    { 0x11F04, "KAWI LETTER A" },
    { 0x11F05, "KAWI LETTER AA" },
    { 0x11F06, "KAWI LETTER I" },
    { 0x11F07, "KAWI LETTER II" },
    { 0x11F08, "KAWI LETTER U" },
    { 0x11F09, "KAWI LETTER UU" },
    { 0x11F0A, "KAWI LETTER VOCALIC R" },
    { 0x11F0B, "KAWI LETTER VOCALIC RR" },
    { 0x11F0C, "KAWI LETTER VOCALIC L" },
    { 0x11F0D, "KAWI LETTER VOCALIC LL" },
    { 0x11F0E, "KAWI LETTER E" },
    { 0x11F0F, "KAWI LETTER AI" },
    { 0x11F10, "KAWI LETTER O" },
    { 0x11F12, "KAWI LETTER KA" },
    { 0x11F13, "KAWI LETTER KHA" },
    { 0x11F14, "KAWI LETTER GA" },
    { 0x11F15, "KAWI LETTER GHA" },
    { 0x11F16, "KAWI LETTER NGA" },
    { 0x11F17, "KAWI LETTER CA" },
    { 0x11F18, "KAWI LETTER CHA" },
    { 0x11F19, "KAWI LETTER JA" },
    { 0x11F1A, "KAWI LETTER JHA" },
    { 0x11F1B, "KAWI LETTER NYA" },
    { 0x11F1C, "KAWI LETTER TTA" },
    { 0x11F1D, "KAWI LETTER TTHA" },
    { 0x11F1E, "KAWI LETTER DDA" },
    { 0x11F1F, "KAWI LETTER DDHA" },
    { 0x11F20, "KAWI LETTER NNA" },
    { 0x11F21, "KAWI LETTER TA" },
    { 0x11F22, "KAWI LETTER THA" },
    { 0x11F23, "KAWI LETTER DA" },
    { 0x11F24, "KAWI LETTER DHA" },
    { 0x11F25, "KAWI LETTER NA" },
    { 0x11F26, "KAWI LETTER PA" },
    { 0x11F27, "KAWI LETTER PHA" },
    { 0x11F28, "KAWI LETTER BA" },
    { 0x11F29, "KAWI LETTER BHA" },
    { 0x11F2A, "KAWI LETTER MA" },
    { 0x11F2B, "KAWI LETTER YA" },
    { 0x11F2C, "KAWI LETTER RA" },
    { 0x11F2D, "KAWI LETTER LA" },
    { 0x11F2E, "KAWI LETTER WA" },
    { 0x11F2F, "KAWI LETTER SHA" },
    { 0x11F30, "KAWI LETTER SSA" },
    { 0x11F31, "KAWI LETTER SA" },
    { 0x11F32, "KAWI LETTER HA" },
    { 0x11F33, "KAWI LETTER JNYA" },
    { 0x11F34, "KAWI VOWEL SIGN AA" },
    { 0x11F35, "KAWI VOWEL SIGN ALTERNATE AA" },
    { 0x11F36, "KAWI VOWEL SIGN I" },
    { 0x11F37, "KAWI VOWEL SIGN II" },
    { 0x11F38, "KAWI VOWEL SIGN U" },
    { 0x11F39, "KAWI VOWEL SIGN UU" },
    { 0x11F3A, "KAWI VOWEL SIGN VOCALIC R" },
    { 0x11F3E, "KAWI VOWEL SIGN E" },
    { 0x11F3F, "KAWI VOWEL SIGN AI" },
    { 0x11F40, "KAWI VOWEL SIGN EU" },
    { 0x11F41, "KAWI SIGN KILLER" },
    { 0x11F42, "KAWI CONJOINER" },
    { 0x11F43, "KAWI DANDA" },
    { 0x11F44, "KAWI DOUBLE DANDA" },
    { 0x11F45, "KAWI PUNCTUATION SECTION MARKER" },
    { 0x11F46, "KAWI PUNCTUATION ALTERNATE SECTION MARKER" },
    { 0x11F47, "KAWI PUNCTUATION FLOWER" },
    { 0x11F48, "KAWI PUNCTUATION SPACE FILLER" },
    { 0x11F49, "KAWI PUNCTUATION DOT" },
    { 0x11F4A, "KAWI PUNCTUATION DOUBLE DOT" },
    { 0x11F4B, "KAWI PUNCTUATION TRIPLE DOT" },
    { 0x11F4C, "KAWI PUNCTUATION CIRCLE" },
    { 0x11F4D, "KAWI PUNCTUATION FILLED CIRCLE" },
    { 0x11F4E, "KAWI PUNCTUATION SPIRAL" },
    { 0x11F4F, "KAWI PUNCTUATION CLOSING SPIRAL" },
    { 0x11F50, "KAWI DIGIT ZERO" },
    { 0x11F51, "KAWI DIGIT ONE" },
    { 0x11F52, "KAWI DIGIT TWO" },
    { 0x11F53, "KAWI DIGIT THREE" },
    { 0x11F54, "KAWI DIGIT FOUR" },
    { 0x11F55, "KAWI DIGIT FIVE" },
    { 0x11F56, "KAWI DIGIT SIX" },
    { 0x11F57, "KAWI DIGIT SEVEN" },
    { 0x11F58, "KAWI DIGIT EIGHT" },
    { 0x11F59, "KAWI DIGIT NINE" },
    { 0x11FB0, "LISU LETTER YHA" },
    { 0x11FC0, "TAMIL FRACTION ONE THREE-HUNDRED-AND-TWENTIETH" },
    { 0x11FC1, "TAMIL FRACTION ONE ONE-HUNDRED-AND-SIXTIETH" },
    { 0x11FC2, "TAMIL FRACTION ONE EIGHTIETH" },
    { 0x11FC3, "TAMIL FRACTION ONE SIXTY-FOURTH" },
    { 0x11FC4, "TAMIL FRACTION ONE FORTIETH" },
    { 0x11FC5, "TAMIL FRACTION ONE THIRTY-SECOND" },
    { 0x11FC6, "TAMIL FRACTION THREE EIGHTIETHS" },
    { 0x11FC7, "TAMIL FRACTION THREE SIXTY-FOURTHS" },
    { 0x11FC8, "TAMIL FRACTION ONE TWENTIETH" },
    { 0x11FC9, "TAMIL FRACTION ONE SIXTEENTH-1" },
    { 0x11FCA, "TAMIL FRACTION ONE SIXTEENTH-2" },
    { 0x11FCB, "TAMIL FRACTION ONE TENTH" },
    { 0x11FCC, "TAMIL FRACTION ONE EIGHTH" },
    { 0x11FCD, "TAMIL FRACTION THREE TWENTIETHS" },
    { 0x11FCE, "TAMIL FRACTION THREE SIXTEENTHS" },
    { 0x11FCF, "TAMIL FRACTION ONE FIFTH" },
    { 0x11FD0, "TAMIL FRACTION ONE QUARTER" },
    { 0x11FD1, "TAMIL FRACTION ONE HALF-1" },
    { 0x11FD2, "TAMIL FRACTION ONE HALF-2" },
    { 0x11FD3, "TAMIL FRACTION THREE QUARTERS" },
    { 0x11FD4, "TAMIL FRACTION DOWNSCALING FACTOR KIIZH" },
    { 0x11FD5, "TAMIL SIGN NEL" },
    { 0x11FD6, "TAMIL SIGN CEVITU" },
    { 0x11FD7, "TAMIL SIGN AAZHAAKKU" },
    { 0x11FD8, "TAMIL SIGN UZHAKKU" },
    { 0x11FD9, "TAMIL SIGN MUUVUZHAKKU" },
    { 0x11FDA, "TAMIL SIGN KURUNI" },
    { 0x11FDB, "TAMIL SIGN PATHAKKU" },
    { 0x11FDC, "TAMIL SIGN MUKKURUNI" },
    { 0x11FDD, "TAMIL SIGN KAACU" },
    { 0x11FDE, "TAMIL SIGN PANAM" },
    { 0x11FDF, "TAMIL SIGN PON" },
    { 0x11FE0, "TAMIL SIGN VARAAKAN" },
    { 0x11FE1, "TAMIL SIGN PAARAM" },
    { 0x11FE2, "TAMIL SIGN KUZHI" },
    { 0x11FE3, "TAMIL SIGN VELI" },
    { 0x11FE4, "TAMIL WET CULTIVATION SIGN" },
    { 0x11FE5, "TAMIL DRY CULTIVATION SIGN" },
    { 0x11FE6, "TAMIL LAND SIGN" },
    { 0x11FE7, "TAMIL SALT PAN SIGN" },
    { 0x11FE8, "TAMIL TRADITIONAL CREDIT SIGN" },
    { 0x11FE9, "TAMIL TRADITIONAL NUMBER SIGN" },
    { 0x11FEA, "TAMIL CURRENT SIGN" },
    { 0x11FEB, "TAMIL AND ODD SIGN" },
    { 0x11FEC, "TAMIL SPENT SIGN" },
    { 0x11FED, "TAMIL TOTAL SIGN" },
    { 0x11FEE, "TAMIL IN POSSESSION SIGN" },
    { 0x11FEF, "TAMIL STARTING FROM SIGN" },
    { 0x11FF0, "TAMIL SIGN MUTHALIYA" },
    { 0x11FF1, "TAMIL SIGN VAKAIYARAA" },
    { 0x11FFF, "TAMIL PUNCTUATION END OF TEXT" },
    { 0x12000, "CUNEIFORM SIGN A" },
    { 0x12001, "CUNEIFORM SIGN A TIMES A" },
    { 0x12002, "CUNEIFORM SIGN A TIMES BAD" },
    { 0x12003, "CUNEIFORM SIGN A TIMES GAN2 TENU" },
    { 0x12004, "CUNEIFORM SIGN A TIMES HA" },
    { 0x12005, "CUNEIFORM SIGN A TIMES IGI" },
    { 0x12006, "CUNEIFORM SIGN A TIMES LAGAR GUNU" },
    { 0x12007, "CUNEIFORM SIGN A TIMES MUSH" },
    { 0x12008, "CUNEIFORM SIGN A TIMES SAG" },
    { 0x12009, "CUNEIFORM SIGN A2" },
    { 0x1200A, "CUNEIFORM SIGN AB" },
    { 0x1200B, "CUNEIFORM SIGN AB TIMES ASH2" },
    { 0x1200C, "CUNEIFORM SIGN AB TIMES DUN3 GUNU" },
    { 0x1200D, "CUNEIFORM SIGN AB TIMES GAL" },
    { 0x1200E, "CUNEIFORM SIGN AB TIMES GAN2 TENU" },
    { 0x1200F, "CUNEIFORM SIGN AB TIMES HA" },
    { 0x12010, "CUNEIFORM SIGN AB TIMES IGI GUNU" },
    { 0x12011, "CUNEIFORM SIGN AB TIMES IMIN" },
    { 0x12012, "CUNEIFORM SIGN AB TIMES LAGAB" },
    { 0x12013, "CUNEIFORM SIGN AB TIMES SHESH" },
    { 0x12014, "CUNEIFORM SIGN AB TIMES U PLUS U PLUS U" },
    { 0x12015, "CUNEIFORM SIGN AB GUNU" },
    { 0x12016, "CUNEIFORM SIGN AB2" },
    { 0x12017, "CUNEIFORM SIGN AB2 TIMES BALAG" },
    { 0x12018, "CUNEIFORM SIGN AB2 TIMES GAN2 TENU" },
    { 0x12019, "CUNEIFORM SIGN AB2 TIMES ME PLUS EN" },
    { 0x1201A, "CUNEIFORM SIGN AB2 TIMES SHA3" },
    { 0x1201B, "CUNEIFORM SIGN AB2 TIMES TAK4" },
    { 0x1201C, "CUNEIFORM SIGN AD" },
    { 0x1201D, "CUNEIFORM SIGN AK" },
    { 0x1201E, "CUNEIFORM SIGN AK TIMES ERIN2" },
    { 0x1201F, "CUNEIFORM SIGN AK TIMES SHITA PLUS GISH" },
    { 0x12020, "CUNEIFORM SIGN AL" },
    { 0x12021, "CUNEIFORM SIGN AL TIMES AL" },
    { 0x12022, "CUNEIFORM SIGN AL TIMES DIM2" },
    { 0x12023, "CUNEIFORM SIGN AL TIMES GISH" },
    { 0x12024, "CUNEIFORM SIGN AL TIMES HA" },
    { 0x12025, "CUNEIFORM SIGN AL TIMES KAD3" },
    { 0x12026, "CUNEIFORM SIGN AL TIMES KI" },
    { 0x12027, "CUNEIFORM SIGN AL TIMES SHE" },
    { 0x12028, "CUNEIFORM SIGN AL TIMES USH" },
    { 0x12029, "CUNEIFORM SIGN ALAN" },
    { 0x1202A, "CUNEIFORM SIGN ALEPH" },
    { 0x1202B, "CUNEIFORM SIGN AMAR" },
    { 0x1202C, "CUNEIFORM SIGN AMAR TIMES SHE" },
    { 0x1202D, "CUNEIFORM SIGN AN" },
    { 0x1202E, "CUNEIFORM SIGN AN OVER AN" },
    { 0x1202F, "CUNEIFORM SIGN AN THREE TIMES" },
    { 0x12030, "CUNEIFORM SIGN AN PLUS NAGA OPPOSING AN PLUS NAGA" },
    { 0x12031, "CUNEIFORM SIGN AN PLUS NAGA SQUARED" },
    { 0x12032, "CUNEIFORM SIGN ANSHE" },
    { 0x12033, "CUNEIFORM SIGN APIN" },
    { 0x12034, "CUNEIFORM SIGN ARAD" },
    { 0x12035, "CUNEIFORM SIGN ARAD TIMES KUR" },
    { 0x12036, "CUNEIFORM SIGN ARKAB" },
    { 0x12037, "CUNEIFORM SIGN ASAL2" },
    { 0x12038, "CUNEIFORM SIGN ASH" },
    { 0x12039, "CUNEIFORM SIGN ASH ZIDA TENU" },
    { 0x1203A, "CUNEIFORM SIGN ASH KABA TENU" },
    { 0x1203B, "CUNEIFORM SIGN ASH OVER ASH TUG2 OVER TUG2 TUG2 OVER TUG2 PAP" },
    { 0x1203C, "CUNEIFORM SIGN ASH OVER ASH OVER ASH" },
    { 0x1203D, "CUNEIFORM SIGN ASH OVER ASH OVER ASH CROSSING ASH OVER ASH OVER ASH" },
    { 0x1203E, "CUNEIFORM SIGN ASH2" },
    { 0x1203F, "CUNEIFORM SIGN ASHGAB" },
    { 0x12040, "CUNEIFORM SIGN BA" },
    { 0x12041, "CUNEIFORM SIGN BAD" },
    { 0x12042, "CUNEIFORM SIGN BAG3" },
    { 0x12043, "CUNEIFORM SIGN BAHAR2" },
    { 0x12044, "CUNEIFORM SIGN BAL" },
    { 0x12045, "CUNEIFORM SIGN BAL OVER BAL" },
    { 0x12046, "CUNEIFORM SIGN BALAG" },
    { 0x12047, "CUNEIFORM SIGN BAR" },
    { 0x12048, "CUNEIFORM SIGN BARA2" },
    { 0x12049, "CUNEIFORM SIGN BI" },
    { 0x1204A, "CUNEIFORM SIGN BI TIMES A" },
    { 0x1204B, "CUNEIFORM SIGN BI TIMES GAR" },
    { 0x1204C, "CUNEIFORM SIGN BI TIMES IGI GUNU" },
    { 0x1204D, "CUNEIFORM SIGN BU" },
    { 0x1204E, "CUNEIFORM SIGN BU OVER BU AB" },
    { 0x1204F, "CUNEIFORM SIGN BU OVER BU UN" },
    { 0x12050, "CUNEIFORM SIGN BU CROSSING BU" },
    { 0x12051, "CUNEIFORM SIGN BULUG" },
    { 0x12052, "CUNEIFORM SIGN BULUG OVER BULUG" },
    { 0x12053, "CUNEIFORM SIGN BUR" },
    { 0x12054, "CUNEIFORM SIGN BUR2" },
    { 0x12055, "CUNEIFORM SIGN DA" },
    { 0x12056, "CUNEIFORM SIGN DAG" },
    { 0x12057, "CUNEIFORM SIGN DAG KISIM5 TIMES A PLUS MASH" },
    { 0x12058, "CUNEIFORM SIGN DAG KISIM5 TIMES AMAR" },
    { 0x12059, "CUNEIFORM SIGN DAG KISIM5 TIMES BALAG" },
    { 0x1205A, "CUNEIFORM SIGN DAG KISIM5 TIMES BI" },
    { 0x1205B, "CUNEIFORM SIGN DAG KISIM5 TIMES GA" },
    { 0x1205C, "CUNEIFORM SIGN DAG KISIM5 TIMES GA PLUS MASH" },
    { 0x1205D, "CUNEIFORM SIGN DAG KISIM5 TIMES GI" },
    { 0x1205E, "CUNEIFORM SIGN DAG KISIM5 TIMES GIR2" },
    { 0x1205F, "CUNEIFORM SIGN DAG KISIM5 TIMES GUD" },
    { 0x12060, "CUNEIFORM SIGN DAG KISIM5 TIMES HA" },
    { 0x12061, "CUNEIFORM SIGN DAG KISIM5 TIMES IR" },
    { 0x12062, "CUNEIFORM SIGN DAG KISIM5 TIMES IR PLUS LU" },
    { 0x12063, "CUNEIFORM SIGN DAG KISIM5 TIMES KAK" },
    { 0x12064, "CUNEIFORM SIGN DAG KISIM5 TIMES LA" },
    { 0x12065, "CUNEIFORM SIGN DAG KISIM5 TIMES LU" },
    { 0x12066, "CUNEIFORM SIGN DAG KISIM5 TIMES LU PLUS MASH2" },
    { 0x12067, "CUNEIFORM SIGN DAG KISIM5 TIMES LUM" },
    { 0x12068, "CUNEIFORM SIGN DAG KISIM5 TIMES NE" },
    { 0x12069, "CUNEIFORM SIGN DAG KISIM5 TIMES PAP PLUS PAP" },
    { 0x1206A, "CUNEIFORM SIGN DAG KISIM5 TIMES SI" },
    { 0x1206B, "CUNEIFORM SIGN DAG KISIM5 TIMES TAK4" },
    { 0x1206C, "CUNEIFORM SIGN DAG KISIM5 TIMES U2 PLUS GIR2" },
    { 0x1206D, "CUNEIFORM SIGN DAG KISIM5 TIMES USH" },
    { 0x1206E, "CUNEIFORM SIGN DAM" },
    { 0x1206F, "CUNEIFORM SIGN DAR" },
    { 0x12070, "CUNEIFORM SIGN DARA3" },
    { 0x12071, "CUNEIFORM SIGN DARA4" },
    { 0x12072, "CUNEIFORM SIGN DI" },
    { 0x12073, "CUNEIFORM SIGN DIB" },
    { 0x12074, "CUNEIFORM SIGN DIM" },
    { 0x12075, "CUNEIFORM SIGN DIM TIMES SHE" },
    { 0x12076, "CUNEIFORM SIGN DIM2" },
    { 0x12077, "CUNEIFORM SIGN DIN" },
    { 0x12078, "CUNEIFORM SIGN DIN KASKAL U GUNU DISH" },
    { 0x12079, "CUNEIFORM SIGN DISH" },
    { 0x1207A, "CUNEIFORM SIGN DU" },
    { 0x1207B, "CUNEIFORM SIGN DU OVER DU" },
    { 0x1207C, "CUNEIFORM SIGN DU GUNU" },
    { 0x1207D, "CUNEIFORM SIGN DU SHESHIG" },
    { 0x1207E, "CUNEIFORM SIGN DUB" },
    { 0x1207F, "CUNEIFORM SIGN DUB TIMES ESH2" },
    { 0x12080, "CUNEIFORM SIGN DUB2" },
    { 0x12081, "CUNEIFORM SIGN DUG" },
    { 0x12082, "CUNEIFORM SIGN DUGUD" },
    { 0x12083, "CUNEIFORM SIGN DUH" },
    { 0x12084, "CUNEIFORM SIGN DUN" },
    { 0x12085, "CUNEIFORM SIGN DUN3" },
    { 0x12086, "CUNEIFORM SIGN DUN3 GUNU" },
    { 0x12087, "CUNEIFORM SIGN DUN3 GUNU GUNU" },
    { 0x12088, "CUNEIFORM SIGN DUN4" },
    { 0x12089, "CUNEIFORM SIGN DUR2" },
    { 0x1208A, "CUNEIFORM SIGN E" },
    { 0x1208B, "CUNEIFORM SIGN E TIMES PAP" },
    { 0x1208C, "CUNEIFORM SIGN E OVER E NUN OVER NUN" },
    { 0x1208D, "CUNEIFORM SIGN E2" },
    { 0x1208E, "CUNEIFORM SIGN E2 TIMES A PLUS HA PLUS DA" },
    { 0x1208F, "CUNEIFORM SIGN E2 TIMES GAR" },
    { 0x12090, "CUNEIFORM SIGN E2 TIMES MI" },
    { 0x12091, "CUNEIFORM SIGN E2 TIMES SAL" },
    { 0x12092, "CUNEIFORM SIGN E2 TIMES SHE" },
    { 0x12093, "CUNEIFORM SIGN E2 TIMES U" },
    { 0x12094, "CUNEIFORM SIGN EDIN" },
    { 0x12095, "CUNEIFORM SIGN EGIR" },
    { 0x12096, "CUNEIFORM SIGN EL" },
    { 0x12097, "CUNEIFORM SIGN EN" },
    { 0x12098, "CUNEIFORM SIGN EN TIMES GAN2" },
    { 0x12099, "CUNEIFORM SIGN EN TIMES GAN2 TENU" },
    { 0x1209A, "CUNEIFORM SIGN EN TIMES ME" },
    { 0x1209B, "CUNEIFORM SIGN EN CROSSING EN" },
    { 0x1209C, "CUNEIFORM SIGN EN OPPOSING EN" },
    { 0x1209D, "CUNEIFORM SIGN EN SQUARED" },
    { 0x1209E, "CUNEIFORM SIGN EREN" },
    { 0x1209F, "CUNEIFORM SIGN ERIN2" },
    { 0x120A0, "CUNEIFORM SIGN ESH2" },
    { 0x120A1, "CUNEIFORM SIGN EZEN" },
    { 0x120A2, "CUNEIFORM SIGN EZEN TIMES A" },
    { 0x120A3, "CUNEIFORM SIGN EZEN TIMES A PLUS LAL" },
    { 0x120A4, "CUNEIFORM SIGN EZEN TIMES A PLUS LAL TIMES LAL" },
    { 0x120A5, "CUNEIFORM SIGN EZEN TIMES AN" },
    { 0x120A6, "CUNEIFORM SIGN EZEN TIMES BAD" },
    { 0x120A7, "CUNEIFORM SIGN EZEN TIMES DUN3 GUNU" },
    { 0x120A8, "CUNEIFORM SIGN EZEN TIMES DUN3 GUNU GUNU" },
    { 0x120A9, "CUNEIFORM SIGN EZEN TIMES HA" },
    { 0x120AA, "CUNEIFORM SIGN EZEN TIMES HA GUNU" },
    { 0x120AB, "CUNEIFORM SIGN EZEN TIMES IGI GUNU" },
    { 0x120AC, "CUNEIFORM SIGN EZEN TIMES KASKAL" },
    { 0x120AD, "CUNEIFORM SIGN EZEN TIMES KASKAL SQUARED" },
    { 0x120AE, "CUNEIFORM SIGN EZEN TIMES KU3" },
    { 0x120AF, "CUNEIFORM SIGN EZEN TIMES LA" },
    { 0x120B0, "CUNEIFORM SIGN EZEN TIMES LAL TIMES LAL" },
    { 0x120B1, "CUNEIFORM SIGN EZEN TIMES LI" },
    { 0x120B2, "CUNEIFORM SIGN EZEN TIMES LU" },
    { 0x120B3, "CUNEIFORM SIGN EZEN TIMES U2" },
    { 0x120B4, "CUNEIFORM SIGN EZEN TIMES UD" },
    { 0x120B5, "CUNEIFORM SIGN GA" },
    { 0x120B6, "CUNEIFORM SIGN GA GUNU" },
    { 0x120B7, "CUNEIFORM SIGN GA2" },
    { 0x120B8, "CUNEIFORM SIGN GA2 TIMES A PLUS DA PLUS HA" },
    { 0x120B9, "CUNEIFORM SIGN GA2 TIMES A PLUS HA" },
    { 0x120BA, "CUNEIFORM SIGN GA2 TIMES A PLUS IGI" },
    { 0x120BB, "CUNEIFORM SIGN GA2 TIMES AB2 TENU PLUS TAB" },
    { 0x120BC, "CUNEIFORM SIGN GA2 TIMES AN" },
    { 0x120BD, "CUNEIFORM SIGN GA2 TIMES ASH" },
    { 0x120BE, "CUNEIFORM SIGN GA2 TIMES ASH2 PLUS GAL" },
    { 0x120BF, "CUNEIFORM SIGN GA2 TIMES BAD" },
    { 0x120C0, "CUNEIFORM SIGN GA2 TIMES BAR PLUS RA" },
    { 0x120C1, "CUNEIFORM SIGN GA2 TIMES BUR" },
    { 0x120C2, "CUNEIFORM SIGN GA2 TIMES BUR PLUS RA" },
    { 0x120C3, "CUNEIFORM SIGN GA2 TIMES DA" },
    { 0x120C4, "CUNEIFORM SIGN GA2 TIMES DI" },
    { 0x120C5, "CUNEIFORM SIGN GA2 TIMES DIM TIMES SHE" },
    { 0x120C6, "CUNEIFORM SIGN GA2 TIMES DUB" },
    { 0x120C7, "CUNEIFORM SIGN GA2 TIMES EL" },
    { 0x120C8, "CUNEIFORM SIGN GA2 TIMES EL PLUS LA" },
    { 0x120C9, "CUNEIFORM SIGN GA2 TIMES EN" },
    { 0x120CA, "CUNEIFORM SIGN GA2 TIMES EN TIMES GAN2 TENU" },
    { 0x120CB, "CUNEIFORM SIGN GA2 TIMES GAN2 TENU" },
    { 0x120CC, "CUNEIFORM SIGN GA2 TIMES GAR" },
    { 0x120CD, "CUNEIFORM SIGN GA2 TIMES GI" },
    { 0x120CE, "CUNEIFORM SIGN GA2 TIMES GI4" },
    { 0x120CF, "CUNEIFORM SIGN GA2 TIMES GI4 PLUS A" },
    { 0x120D0, "CUNEIFORM SIGN GA2 TIMES GIR2 PLUS SU" },
    { 0x120D1, "CUNEIFORM SIGN GA2 TIMES HA PLUS LU PLUS ESH2" },
    { 0x120D2, "CUNEIFORM SIGN GA2 TIMES HAL" },
    { 0x120D3, "CUNEIFORM SIGN GA2 TIMES HAL PLUS LA" },
    { 0x120D4, "CUNEIFORM SIGN GA2 TIMES HI PLUS LI" },
    { 0x120D5, "CUNEIFORM SIGN GA2 TIMES HUB2" },
    { 0x120D6, "CUNEIFORM SIGN GA2 TIMES IGI GUNU" },
    { 0x120D7, "CUNEIFORM SIGN GA2 TIMES ISH PLUS HU PLUS ASH" },
    { 0x120D8, "CUNEIFORM SIGN GA2 TIMES KAK" },
    { 0x120D9, "CUNEIFORM SIGN GA2 TIMES KASKAL" },
    { 0x120DA, "CUNEIFORM SIGN GA2 TIMES KID" },
    { 0x120DB, "CUNEIFORM SIGN GA2 TIMES KID PLUS LAL" },
    { 0x120DC, "CUNEIFORM SIGN GA2 TIMES KU3 PLUS AN" },
    { 0x120DD, "CUNEIFORM SIGN GA2 TIMES LA" },
    { 0x120DE, "CUNEIFORM SIGN GA2 TIMES ME PLUS EN" },
    { 0x120DF, "CUNEIFORM SIGN GA2 TIMES MI" },
    { 0x120E0, "CUNEIFORM SIGN GA2 TIMES NUN" },
    { 0x120E1, "CUNEIFORM SIGN GA2 TIMES NUN OVER NUN" },
    { 0x120E2, "CUNEIFORM SIGN GA2 TIMES PA" },
    { 0x120E3, "CUNEIFORM SIGN GA2 TIMES SAL" },
    { 0x120E4, "CUNEIFORM SIGN GA2 TIMES SAR" },
    { 0x120E5, "CUNEIFORM SIGN GA2 TIMES SHE" },
    { 0x120E6, "CUNEIFORM SIGN GA2 TIMES SHE PLUS TUR" },
    { 0x120E7, "CUNEIFORM SIGN GA2 TIMES SHID" },
    { 0x120E8, "CUNEIFORM SIGN GA2 TIMES SUM" },
    { 0x120E9, "CUNEIFORM SIGN GA2 TIMES TAK4" },
    { 0x120EA, "CUNEIFORM SIGN GA2 TIMES U" },
    { 0x120EB, "CUNEIFORM SIGN GA2 TIMES UD" },
    { 0x120EC, "CUNEIFORM SIGN GA2 TIMES UD PLUS DU" },
    { 0x120ED, "CUNEIFORM SIGN GA2 OVER GA2" },
    { 0x120EE, "CUNEIFORM SIGN GABA" },
    { 0x120EF, "CUNEIFORM SIGN GABA CROSSING GABA" },
    { 0x120F0, "CUNEIFORM SIGN GAD" },
    { 0x120F1, "CUNEIFORM SIGN GAD OVER GAD GAR OVER GAR" },
    { 0x120F2, "CUNEIFORM SIGN GAL" },
    { 0x120F3, "CUNEIFORM SIGN GAL GAD OVER GAD GAR OVER GAR" },
    { 0x120F4, "CUNEIFORM SIGN GALAM" },
    { 0x120F5, "CUNEIFORM SIGN GAM" },
    { 0x120F6, "CUNEIFORM SIGN GAN" },
    { 0x120F7, "CUNEIFORM SIGN GAN2" },
    { 0x120F8, "CUNEIFORM SIGN GAN2 TENU" },
    { 0x120F9, "CUNEIFORM SIGN GAN2 OVER GAN2" },
    { 0x120FA, "CUNEIFORM SIGN GAN2 CROSSING GAN2" },
    { 0x120FB, "CUNEIFORM SIGN GAR" },
    { 0x120FC, "CUNEIFORM SIGN GAR3" },
    { 0x120FD, "CUNEIFORM SIGN GASHAN" },
    { 0x120FE, "CUNEIFORM SIGN GESHTIN" },
    { 0x120FF, "CUNEIFORM SIGN GESHTIN TIMES KUR" },
    { 0x12100, "CUNEIFORM SIGN GI" },
    { 0x12101, "CUNEIFORM SIGN GI TIMES E" },
    { 0x12102, "CUNEIFORM SIGN GI TIMES U" },
    { 0x12103, "CUNEIFORM SIGN GI CROSSING GI" },
    { 0x12104, "CUNEIFORM SIGN GI4" },
    { 0x12105, "CUNEIFORM SIGN GI4 OVER GI4" },
    { 0x12106, "CUNEIFORM SIGN GI4 CROSSING GI4" },
    { 0x12107, "CUNEIFORM SIGN GIDIM" },
    { 0x12108, "CUNEIFORM SIGN GIR2" },
    { 0x12109, "CUNEIFORM SIGN GIR2 GUNU" },
    { 0x1210A, "CUNEIFORM SIGN GIR3" },
    { 0x1210B, "CUNEIFORM SIGN GIR3 TIMES A PLUS IGI" },
    { 0x1210C, "CUNEIFORM SIGN GIR3 TIMES GAN2 TENU" },
    { 0x1210D, "CUNEIFORM SIGN GIR3 TIMES IGI" },
    { 0x1210E, "CUNEIFORM SIGN GIR3 TIMES LU PLUS IGI" },
    { 0x1210F, "CUNEIFORM SIGN GIR3 TIMES PA" },
    { 0x12110, "CUNEIFORM SIGN GISAL" },
    { 0x12111, "CUNEIFORM SIGN GISH" },
    { 0x12112, "CUNEIFORM SIGN GISH CROSSING GISH" },
    { 0x12113, "CUNEIFORM SIGN GISH TIMES BAD" },
    { 0x12114, "CUNEIFORM SIGN GISH TIMES TAK4" },
    { 0x12115, "CUNEIFORM SIGN GISH TENU" },
    { 0x12116, "CUNEIFORM SIGN GU" },
    { 0x12117, "CUNEIFORM SIGN GU CROSSING GU" },
    { 0x12118, "CUNEIFORM SIGN GU2" },
    { 0x12119, "CUNEIFORM SIGN GU2 TIMES KAK" },
    { 0x1211A, "CUNEIFORM SIGN GU2 TIMES KAK TIMES IGI GUNU" },
    { 0x1211B, "CUNEIFORM SIGN GU2 TIMES NUN" },
    { 0x1211C, "CUNEIFORM SIGN GU2 TIMES SAL PLUS TUG2" },
    { 0x1211D, "CUNEIFORM SIGN GU2 GUNU" },
    { 0x1211E, "CUNEIFORM SIGN GUD" },
    { 0x1211F, "CUNEIFORM SIGN GUD TIMES A PLUS KUR" },
    { 0x12120, "CUNEIFORM SIGN GUD TIMES KUR" },
    { 0x12121, "CUNEIFORM SIGN GUD OVER GUD LUGAL" },
    { 0x12122, "CUNEIFORM SIGN GUL" },
    { 0x12123, "CUNEIFORM SIGN GUM" },
    { 0x12124, "CUNEIFORM SIGN GUM TIMES SHE" },
    { 0x12125, "CUNEIFORM SIGN GUR" },
    { 0x12126, "CUNEIFORM SIGN GUR7" },
    { 0x12127, "CUNEIFORM SIGN GURUN" },
    { 0x12128, "CUNEIFORM SIGN GURUSH" },
    { 0x12129, "CUNEIFORM SIGN HA" },
    { 0x1212A, "CUNEIFORM SIGN HA TENU" },
    { 0x1212B, "CUNEIFORM SIGN HA GUNU" },
    { 0x1212C, "CUNEIFORM SIGN HAL" },
    { 0x1212D, "CUNEIFORM SIGN HI" },
    { 0x1212E, "CUNEIFORM SIGN HI TIMES ASH" },
    { 0x1212F, "CUNEIFORM SIGN HI TIMES ASH2" },
    { 0x12130, "CUNEIFORM SIGN HI TIMES BAD" },
    { 0x12131, "CUNEIFORM SIGN HI TIMES DISH" },
    { 0x12132, "CUNEIFORM SIGN HI TIMES GAD" },
    { 0x12133, "CUNEIFORM SIGN HI TIMES KIN" },
    { 0x12134, "CUNEIFORM SIGN HI TIMES NUN" },
    { 0x12135, "CUNEIFORM SIGN HI TIMES SHE" },
    { 0x12136, "CUNEIFORM SIGN HI TIMES U" },
    { 0x12137, "CUNEIFORM SIGN HU" },
    { 0x12138, "CUNEIFORM SIGN HUB2" },
    { 0x12139, "CUNEIFORM SIGN HUB2 TIMES AN" },
    { 0x1213A, "CUNEIFORM SIGN HUB2 TIMES HAL" },
    { 0x1213B, "CUNEIFORM SIGN HUB2 TIMES KASKAL" },
    { 0x1213C, "CUNEIFORM SIGN HUB2 TIMES LISH" },
    { 0x1213D, "CUNEIFORM SIGN HUB2 TIMES UD" },
    { 0x1213E, "CUNEIFORM SIGN HUL2" },
    { 0x1213F, "CUNEIFORM SIGN I" },
    { 0x12140, "CUNEIFORM SIGN I A" },
    { 0x12141, "CUNEIFORM SIGN IB" },
    { 0x12142, "CUNEIFORM SIGN IDIM" },
    { 0x12143, "CUNEIFORM SIGN IDIM OVER IDIM BUR" },
    { 0x12144, "CUNEIFORM SIGN IDIM OVER IDIM SQUARED" },
    { 0x12145, "CUNEIFORM SIGN IG" },
    { 0x12146, "CUNEIFORM SIGN IGI" },
    { 0x12147, "CUNEIFORM SIGN IGI DIB" },
    { 0x12148, "CUNEIFORM SIGN IGI RI" },
    { 0x12149, "CUNEIFORM SIGN IGI OVER IGI SHIR OVER SHIR UD OVER UD" },
    { 0x1214A, "CUNEIFORM SIGN IGI GUNU" },
    { 0x1214B, "CUNEIFORM SIGN IL" },
    { 0x1214C, "CUNEIFORM SIGN IL TIMES GAN2 TENU" },
    { 0x1214D, "CUNEIFORM SIGN IL2" },
    { 0x1214E, "CUNEIFORM SIGN IM" },
    { 0x1214F, "CUNEIFORM SIGN IM TIMES TAK4" },
    { 0x12150, "CUNEIFORM SIGN IM CROSSING IM" },
    { 0x12151, "CUNEIFORM SIGN IM OPPOSING IM" },
    { 0x12152, "CUNEIFORM SIGN IM SQUARED" },
    { 0x12153, "CUNEIFORM SIGN IMIN" },
    { 0x12154, "CUNEIFORM SIGN IN" },
    { 0x12155, "CUNEIFORM SIGN IR" },
    { 0x12156, "CUNEIFORM SIGN ISH" },
    { 0x12157, "CUNEIFORM SIGN KA" },
    { 0x12158, "CUNEIFORM SIGN KA TIMES A" },
    { 0x12159, "CUNEIFORM SIGN KA TIMES AD" },
    { 0x1215A, "CUNEIFORM SIGN KA TIMES AD PLUS KU3" },
    { 0x1215B, "CUNEIFORM SIGN KA TIMES ASH2" },
    { 0x1215C, "CUNEIFORM SIGN KA TIMES BAD" },
    { 0x1215D, "CUNEIFORM SIGN KA TIMES BALAG" },
    { 0x1215E, "CUNEIFORM SIGN KA TIMES BAR" },
    { 0x1215F, "CUNEIFORM SIGN KA TIMES BI" },
    { 0x12160, "CUNEIFORM SIGN KA TIMES ERIN2" },
    { 0x12161, "CUNEIFORM SIGN KA TIMES ESH2" },
    { 0x12162, "CUNEIFORM SIGN KA TIMES GA" },
    { 0x12163, "CUNEIFORM SIGN KA TIMES GAL" },
    { 0x12164, "CUNEIFORM SIGN KA TIMES GAN2 TENU" },
    { 0x12165, "CUNEIFORM SIGN KA TIMES GAR" },
    { 0x12166, "CUNEIFORM SIGN KA TIMES GAR PLUS SHA3 PLUS A" },
    { 0x12167, "CUNEIFORM SIGN KA TIMES GI" },
    { 0x12168, "CUNEIFORM SIGN KA TIMES GIR2" },
    { 0x12169, "CUNEIFORM SIGN KA TIMES GISH PLUS SAR" },
    { 0x1216A, "CUNEIFORM SIGN KA TIMES GISH CROSSING GISH" },
    { 0x1216B, "CUNEIFORM SIGN KA TIMES GU" },
    { 0x1216C, "CUNEIFORM SIGN KA TIMES GUR7" },
    { 0x1216D, "CUNEIFORM SIGN KA TIMES IGI" },
    { 0x1216E, "CUNEIFORM SIGN KA TIMES IM" },
    { 0x1216F, "CUNEIFORM SIGN KA TIMES KAK" },
    { 0x12170, "CUNEIFORM SIGN KA TIMES KI" },
    { 0x12171, "CUNEIFORM SIGN KA TIMES KID" },
    { 0x12172, "CUNEIFORM SIGN KA TIMES LI" },
    { 0x12173, "CUNEIFORM SIGN KA TIMES LU" },
    { 0x12174, "CUNEIFORM SIGN KA TIMES ME" },
    { 0x12175, "CUNEIFORM SIGN KA TIMES ME PLUS DU" },
    { 0x12176, "CUNEIFORM SIGN KA TIMES ME PLUS GI" },
    { 0x12177, "CUNEIFORM SIGN KA TIMES ME PLUS TE" },
    { 0x12178, "CUNEIFORM SIGN KA TIMES MI" },
    { 0x12179, "CUNEIFORM SIGN KA TIMES MI PLUS NUNUZ" },
    { 0x1217A, "CUNEIFORM SIGN KA TIMES NE" },
    { 0x1217B, "CUNEIFORM SIGN KA TIMES NUN" },
    { 0x1217C, "CUNEIFORM SIGN KA TIMES PI" },
    { 0x1217D, "CUNEIFORM SIGN KA TIMES RU" },
    { 0x1217E, "CUNEIFORM SIGN KA TIMES SA" },
    { 0x1217F, "CUNEIFORM SIGN KA TIMES SAR" },
    { 0x12180, "CUNEIFORM SIGN KA TIMES SHA" },
    { 0x12181, "CUNEIFORM SIGN KA TIMES SHE" },
    { 0x12182, "CUNEIFORM SIGN KA TIMES SHID" },
    { 0x12183, "CUNEIFORM SIGN KA TIMES SHU" },
    { 0x12184, "CUNEIFORM SIGN KA TIMES SIG" },
    { 0x12185, "CUNEIFORM SIGN KA TIMES SUHUR" },
    { 0x12186, "CUNEIFORM SIGN KA TIMES TAR" },
    { 0x12187, "CUNEIFORM SIGN KA TIMES U" },
    { 0x12188, "CUNEIFORM SIGN KA TIMES U2" },
    { 0x12189, "CUNEIFORM SIGN KA TIMES UD" },
    { 0x1218A, "CUNEIFORM SIGN KA TIMES UMUM TIMES PA" },
    { 0x1218B, "CUNEIFORM SIGN KA TIMES USH" },
    { 0x1218C, "CUNEIFORM SIGN KA TIMES ZI" },
    { 0x1218D, "CUNEIFORM SIGN KA2" },
    { 0x1218E, "CUNEIFORM SIGN KA2 CROSSING KA2" },
    { 0x1218F, "CUNEIFORM SIGN KAB" },
    { 0x12190, "CUNEIFORM SIGN KAD2" },
    { 0x12191, "CUNEIFORM SIGN KAD3" },
    { 0x12192, "CUNEIFORM SIGN KAD4" },
    { 0x12193, "CUNEIFORM SIGN KAD5" },
    { 0x12194, "CUNEIFORM SIGN KAD5 OVER KAD5" },
    { 0x12195, "CUNEIFORM SIGN KAK" },
    { 0x12196, "CUNEIFORM SIGN KAK TIMES IGI GUNU" },
    { 0x12197, "CUNEIFORM SIGN KAL" },
    { 0x12198, "CUNEIFORM SIGN KAL TIMES BAD" },
    { 0x12199, "CUNEIFORM SIGN KAL CROSSING KAL" },
    { 0x1219A, "CUNEIFORM SIGN KAM2" },
    { 0x1219B, "CUNEIFORM SIGN KAM4" },
    { 0x1219C, "CUNEIFORM SIGN KASKAL" },
    { 0x1219D, "CUNEIFORM SIGN KASKAL LAGAB TIMES U OVER LAGAB TIMES U" },
    { 0x1219E, "CUNEIFORM SIGN KASKAL OVER KASKAL LAGAB TIMES U OVER LAGAB TIMES U" },
    { 0x1219F, "CUNEIFORM SIGN KESH2" },
    { 0x121A0, "CUNEIFORM SIGN KI" },
    { 0x121A1, "CUNEIFORM SIGN KI TIMES BAD" },
    { 0x121A2, "CUNEIFORM SIGN KI TIMES U" },
    { 0x121A3, "CUNEIFORM SIGN KI TIMES UD" },
    { 0x121A4, "CUNEIFORM SIGN KID" },
    { 0x121A5, "CUNEIFORM SIGN KIN" },
    { 0x121A6, "CUNEIFORM SIGN KISAL" },
    { 0x121A7, "CUNEIFORM SIGN KISH" },
    { 0x121A8, "CUNEIFORM SIGN KISIM5" },
    { 0x121A9, "CUNEIFORM SIGN KISIM5 OVER KISIM5" },
    { 0x121AA, "CUNEIFORM SIGN KU" },
    { 0x121AB, "CUNEIFORM SIGN KU OVER HI TIMES ASH2 KU OVER HI TIMES ASH2" },
    { 0x121AC, "CUNEIFORM SIGN KU3" },
    { 0x121AD, "CUNEIFORM SIGN KU4" },
    { 0x121AE, "CUNEIFORM SIGN KU4 VARIANT FORM" },
    { 0x121AF, "CUNEIFORM SIGN KU7" },
    { 0x121B0, "CUNEIFORM SIGN KUL" },
    { 0x121B1, "CUNEIFORM SIGN KUL GUNU" },
    { 0x121B2, "CUNEIFORM SIGN KUN" },
    { 0x121B3, "CUNEIFORM SIGN KUR" },
    { 0x121B4, "CUNEIFORM SIGN KUR OPPOSING KUR" },
    { 0x121B5, "CUNEIFORM SIGN KUSHU2" },
    { 0x121B6, "CUNEIFORM SIGN KWU318" },
    { 0x121B7, "CUNEIFORM SIGN LA" },
    { 0x121B8, "CUNEIFORM SIGN LAGAB" },
    { 0x121B9, "CUNEIFORM SIGN LAGAB TIMES A" },
    { 0x121BA, "CUNEIFORM SIGN LAGAB TIMES A PLUS DA PLUS HA" },
    { 0x121BB, "CUNEIFORM SIGN LAGAB TIMES A PLUS GAR" },
    { 0x121BC, "CUNEIFORM SIGN LAGAB TIMES A PLUS LAL" },
    { 0x121BD, "CUNEIFORM SIGN LAGAB TIMES AL" },
    { 0x121BE, "CUNEIFORM SIGN LAGAB TIMES AN" },
    { 0x121BF, "CUNEIFORM SIGN LAGAB TIMES ASH ZIDA TENU" },
    { 0x121C0, "CUNEIFORM SIGN LAGAB TIMES BAD" },
    { 0x121C1, "CUNEIFORM SIGN LAGAB TIMES BI" },
    { 0x121C2, "CUNEIFORM SIGN LAGAB TIMES DAR" },
    { 0x121C3, "CUNEIFORM SIGN LAGAB TIMES EN" },
    { 0x121C4, "CUNEIFORM SIGN LAGAB TIMES GA" },
    { 0x121C5, "CUNEIFORM SIGN LAGAB TIMES GAR" },
    { 0x121C6, "CUNEIFORM SIGN LAGAB TIMES GUD" },
    { 0x121C7, "CUNEIFORM SIGN LAGAB TIMES GUD PLUS GUD" },
    { 0x121C8, "CUNEIFORM SIGN LAGAB TIMES HA" },
    { 0x121C9, "CUNEIFORM SIGN LAGAB TIMES HAL" },
    { 0x121CA, "CUNEIFORM SIGN LAGAB TIMES HI TIMES NUN" },
    { 0x121CB, "CUNEIFORM SIGN LAGAB TIMES IGI GUNU" },
    { 0x121CC, "CUNEIFORM SIGN LAGAB TIMES IM" },
    { 0x121CD, "CUNEIFORM SIGN LAGAB TIMES IM PLUS HA" },
    { 0x121CE, "CUNEIFORM SIGN LAGAB TIMES IM PLUS LU" },
    { 0x121CF, "CUNEIFORM SIGN LAGAB TIMES KI" },
    { 0x121D0, "CUNEIFORM SIGN LAGAB TIMES KIN" },
    { 0x121D1, "CUNEIFORM SIGN LAGAB TIMES KU3" },
    { 0x121D2, "CUNEIFORM SIGN LAGAB TIMES KUL" },
    { 0x121D3, "CUNEIFORM SIGN LAGAB TIMES KUL PLUS HI PLUS A" },
    { 0x121D4, "CUNEIFORM SIGN LAGAB TIMES LAGAB" },
    { 0x121D5, "CUNEIFORM SIGN LAGAB TIMES LISH" },
    { 0x121D6, "CUNEIFORM SIGN LAGAB TIMES LU" },
    { 0x121D7, "CUNEIFORM SIGN LAGAB TIMES LUL" },
    { 0x121D8, "CUNEIFORM SIGN LAGAB TIMES ME" },
    { 0x121D9, "CUNEIFORM SIGN LAGAB TIMES ME PLUS EN" },
    { 0x121DA, "CUNEIFORM SIGN LAGAB TIMES MUSH" },
    { 0x121DB, "CUNEIFORM SIGN LAGAB TIMES NE" },
    { 0x121DC, "CUNEIFORM SIGN LAGAB TIMES SHE PLUS SUM" },
    { 0x121DD, "CUNEIFORM SIGN LAGAB TIMES SHITA PLUS GISH PLUS ERIN2" },
    { 0x121DE, "CUNEIFORM SIGN LAGAB TIMES SHITA PLUS GISH TENU" },
    { 0x121DF, "CUNEIFORM SIGN LAGAB TIMES SHU2" },
    { 0x121E0, "CUNEIFORM SIGN LAGAB TIMES SHU2 PLUS SHU2" },
    { 0x121E1, "CUNEIFORM SIGN LAGAB TIMES SUM" },
    { 0x121E2, "CUNEIFORM SIGN LAGAB TIMES TAG" },
    { 0x121E3, "CUNEIFORM SIGN LAGAB TIMES TAK4" },
    { 0x121E4, "CUNEIFORM SIGN LAGAB TIMES TE PLUS A PLUS SU PLUS NA" },
    { 0x121E5, "CUNEIFORM SIGN LAGAB TIMES U" },
    { 0x121E6, "CUNEIFORM SIGN LAGAB TIMES U PLUS A" },
    { 0x121E7, "CUNEIFORM SIGN LAGAB TIMES U PLUS U PLUS U" },
    { 0x121E8, "CUNEIFORM SIGN LAGAB TIMES U2 PLUS ASH" },
    { 0x121E9, "CUNEIFORM SIGN LAGAB TIMES UD" },
    { 0x121EA, "CUNEIFORM SIGN LAGAB TIMES USH" },
    { 0x121EB, "CUNEIFORM SIGN LAGAB SQUARED" },
    { 0x121EC, "CUNEIFORM SIGN LAGAR" },
    { 0x121ED, "CUNEIFORM SIGN LAGAR TIMES SHE" },
    { 0x121EE, "CUNEIFORM SIGN LAGAR TIMES SHE PLUS SUM" },
    { 0x121EF, "CUNEIFORM SIGN LAGAR GUNU" },
    { 0x121F0, "CUNEIFORM SIGN LAGAR GUNU OVER LAGAR GUNU SHE" },
    { 0x121F1, "CUNEIFORM SIGN LAHSHU" },
    { 0x121F2, "CUNEIFORM SIGN LAL" },
    { 0x121F3, "CUNEIFORM SIGN LAL TIMES LAL" },
    { 0x121F4, "CUNEIFORM SIGN LAM" },
    { 0x121F5, "CUNEIFORM SIGN LAM TIMES KUR" },
    { 0x121F6, "CUNEIFORM SIGN LAM TIMES KUR PLUS RU" },
    { 0x121F7, "CUNEIFORM SIGN LI" },
    { 0x121F8, "CUNEIFORM SIGN LIL" },
    { 0x121F9, "CUNEIFORM SIGN LIMMU2" },
    { 0x121FA, "CUNEIFORM SIGN LISH" },
    { 0x121FB, "CUNEIFORM SIGN LU" },
    { 0x121FC, "CUNEIFORM SIGN LU TIMES BAD" },
    { 0x121FD, "CUNEIFORM SIGN LU2" },
    { 0x121FE, "CUNEIFORM SIGN LU2 TIMES AL" },
    { 0x121FF, "CUNEIFORM SIGN LU2 TIMES BAD" },
    { 0x12200, "CUNEIFORM SIGN LU2 TIMES ESH2" },
    { 0x12201, "CUNEIFORM SIGN LU2 TIMES ESH2 TENU" },
    { 0x12202, "CUNEIFORM SIGN LU2 TIMES GAN2 TENU" },
    { 0x12203, "CUNEIFORM SIGN LU2 TIMES HI TIMES BAD" },
    { 0x12204, "CUNEIFORM SIGN LU2 TIMES IM" },
    { 0x12205, "CUNEIFORM SIGN LU2 TIMES KAD2" },
    { 0x12206, "CUNEIFORM SIGN LU2 TIMES KAD3" },
    { 0x12207, "CUNEIFORM SIGN LU2 TIMES KAD3 PLUS ASH" },
    { 0x12208, "CUNEIFORM SIGN LU2 TIMES KI" },
    { 0x12209, "CUNEIFORM SIGN LU2 TIMES LA PLUS ASH" },
    { 0x1220A, "CUNEIFORM SIGN LU2 TIMES LAGAB" },
    { 0x1220B, "CUNEIFORM SIGN LU2 TIMES ME PLUS EN" },
    { 0x1220C, "CUNEIFORM SIGN LU2 TIMES NE" },
    { 0x1220D, "CUNEIFORM SIGN LU2 TIMES NU" },
    { 0x1220E, "CUNEIFORM SIGN LU2 TIMES SI PLUS ASH" },
    { 0x1220F, "CUNEIFORM SIGN LU2 TIMES SIK2 PLUS BU" },
    { 0x12210, "CUNEIFORM SIGN LU2 TIMES TUG2" },
    { 0x12211, "CUNEIFORM SIGN LU2 TENU" },
    { 0x12212, "CUNEIFORM SIGN LU2 CROSSING LU2" },
    { 0x12213, "CUNEIFORM SIGN LU2 OPPOSING LU2" },
    { 0x12214, "CUNEIFORM SIGN LU2 SQUARED" },
    { 0x12215, "CUNEIFORM SIGN LU2 SHESHIG" },
    { 0x12216, "CUNEIFORM SIGN LU3" },
    { 0x12217, "CUNEIFORM SIGN LUGAL" },
    { 0x12218, "CUNEIFORM SIGN LUGAL OVER LUGAL" },
    { 0x12219, "CUNEIFORM SIGN LUGAL OPPOSING LUGAL" },
    { 0x1221A, "CUNEIFORM SIGN LUGAL SHESHIG" },
    { 0x1221B, "CUNEIFORM SIGN LUH" },
    { 0x1221C, "CUNEIFORM SIGN LUL" },
    { 0x1221D, "CUNEIFORM SIGN LUM" },
    { 0x1221E, "CUNEIFORM SIGN LUM OVER LUM" },
    { 0x1221F, "CUNEIFORM SIGN LUM OVER LUM GAR OVER GAR" },
    { 0x12220, "CUNEIFORM SIGN MA" },
    { 0x12221, "CUNEIFORM SIGN MA TIMES TAK4" },
    { 0x12222, "CUNEIFORM SIGN MA GUNU" },
    { 0x12223, "CUNEIFORM SIGN MA2" },
    { 0x12224, "CUNEIFORM SIGN MAH" },
    { 0x12225, "CUNEIFORM SIGN MAR" },
    { 0x12226, "CUNEIFORM SIGN MASH" },
    { 0x12227, "CUNEIFORM SIGN MASH2" },
    { 0x12228, "CUNEIFORM SIGN ME" },
    { 0x12229, "CUNEIFORM SIGN MES" },
    { 0x1222A, "CUNEIFORM SIGN MI" },
    { 0x1222B, "CUNEIFORM SIGN MIN" },
    { 0x1222C, "CUNEIFORM SIGN MU" },
    { 0x1222D, "CUNEIFORM SIGN MU OVER MU" },
    { 0x1222E, "CUNEIFORM SIGN MUG" },
    { 0x1222F, "CUNEIFORM SIGN MUG GUNU" },
    { 0x12230, "CUNEIFORM SIGN MUNSUB" },
    { 0x12231, "CUNEIFORM SIGN MURGU2" },
    { 0x12232, "CUNEIFORM SIGN MUSH" },
    { 0x12233, "CUNEIFORM SIGN MUSH TIMES A" },
    { 0x12234, "CUNEIFORM SIGN MUSH TIMES KUR" },
    { 0x12235, "CUNEIFORM SIGN MUSH TIMES ZA" },
    { 0x12236, "CUNEIFORM SIGN MUSH OVER MUSH" },
    { 0x12237, "CUNEIFORM SIGN MUSH OVER MUSH TIMES A PLUS NA" },
    { 0x12238, "CUNEIFORM SIGN MUSH CROSSING MUSH" },
    { 0x12239, "CUNEIFORM SIGN MUSH3" },
    { 0x1223A, "CUNEIFORM SIGN MUSH3 TIMES A" },
    { 0x1223B, "CUNEIFORM SIGN MUSH3 TIMES A PLUS DI" },
    { 0x1223C, "CUNEIFORM SIGN MUSH3 TIMES DI" },
    { 0x1223D, "CUNEIFORM SIGN MUSH3 GUNU" },
    { 0x1223E, "CUNEIFORM SIGN NA" },
    { 0x1223F, "CUNEIFORM SIGN NA2" },
    { 0x12240, "CUNEIFORM SIGN NAGA" },
    { 0x12241, "CUNEIFORM SIGN NAGA INVERTED" },
    { 0x12242, "CUNEIFORM SIGN NAGA TIMES SHU TENU" },
    { 0x12243, "CUNEIFORM SIGN NAGA OPPOSING NAGA" },
    { 0x12244, "CUNEIFORM SIGN NAGAR" },
    { 0x12245, "CUNEIFORM SIGN NAM NUTILLU" },
    { 0x12246, "CUNEIFORM SIGN NAM" },
    { 0x12247, "CUNEIFORM SIGN NAM2" },
    { 0x12248, "CUNEIFORM SIGN NE" },
    { 0x12249, "CUNEIFORM SIGN NE TIMES A" },
    { 0x1224A, "CUNEIFORM SIGN NE TIMES UD" },
    { 0x1224B, "CUNEIFORM SIGN NE SHESHIG" },
    { 0x1224C, "CUNEIFORM SIGN NI" },
    { 0x1224D, "CUNEIFORM SIGN NI TIMES E" },
    { 0x1224E, "CUNEIFORM SIGN NI2" },
    { 0x1224F, "CUNEIFORM SIGN NIM" },
    { 0x12250, "CUNEIFORM SIGN NIM TIMES GAN2 TENU" },
    { 0x12251, "CUNEIFORM SIGN NIM TIMES GAR PLUS GAN2 TENU" },
    { 0x12252, "CUNEIFORM SIGN NINDA2" },
    { 0x12253, "CUNEIFORM SIGN NINDA2 TIMES AN" },
    { 0x12254, "CUNEIFORM SIGN NINDA2 TIMES ASH" },
    { 0x12255, "CUNEIFORM SIGN NINDA2 TIMES ASH PLUS ASH" },
    { 0x12256, "CUNEIFORM SIGN NINDA2 TIMES GUD" },
    { 0x12257, "CUNEIFORM SIGN NINDA2 TIMES ME PLUS GAN2 TENU" },
    { 0x12258, "CUNEIFORM SIGN NINDA2 TIMES NE" },
    { 0x12259, "CUNEIFORM SIGN NINDA2 TIMES NUN" },
    { 0x1225A, "CUNEIFORM SIGN NINDA2 TIMES SHE" },
    { 0x1225B, "CUNEIFORM SIGN NINDA2 TIMES SHE PLUS A AN" },
    { 0x1225C, "CUNEIFORM SIGN NINDA2 TIMES SHE PLUS ASH" },
    { 0x1225D, "CUNEIFORM SIGN NINDA2 TIMES SHE PLUS ASH PLUS ASH" },
    { 0x1225E, "CUNEIFORM SIGN NINDA2 TIMES U2 PLUS ASH" },
    { 0x1225F, "CUNEIFORM SIGN NINDA2 TIMES USH" },
    { 0x12260, "CUNEIFORM SIGN NISAG" },
    { 0x12261, "CUNEIFORM SIGN NU" },
    { 0x12262, "CUNEIFORM SIGN NU11" },
    { 0x12263, "CUNEIFORM SIGN NUN" },
    { 0x12264, "CUNEIFORM SIGN NUN LAGAR TIMES GAR" },
    { 0x12265, "CUNEIFORM SIGN NUN LAGAR TIMES MASH" },
    { 0x12266, "CUNEIFORM SIGN NUN LAGAR TIMES SAL" },
    { 0x12267, "CUNEIFORM SIGN NUN LAGAR TIMES SAL OVER NUN LAGAR TIMES SAL" },
    { 0x12268, "CUNEIFORM SIGN NUN LAGAR TIMES USH" },
    { 0x12269, "CUNEIFORM SIGN NUN TENU" },
    { 0x1226A, "CUNEIFORM SIGN NUN OVER NUN" },
    { 0x1226B, "CUNEIFORM SIGN NUN CROSSING NUN" },
    { 0x1226C, "CUNEIFORM SIGN NUN CROSSING NUN LAGAR OVER LAGAR" },
    { 0x1226D, "CUNEIFORM SIGN NUNUZ" },
    { 0x1226E, "CUNEIFORM SIGN NUNUZ AB2 TIMES ASHGAB" },
    { 0x1226F, "CUNEIFORM SIGN NUNUZ AB2 TIMES BI" },
    { 0x12270, "CUNEIFORM SIGN NUNUZ AB2 TIMES DUG" },
    { 0x12271, "CUNEIFORM SIGN NUNUZ AB2 TIMES GUD" },
    { 0x12272, "CUNEIFORM SIGN NUNUZ AB2 TIMES IGI GUNU" },
    { 0x12273, "CUNEIFORM SIGN NUNUZ AB2 TIMES KAD3" },
    { 0x12274, "CUNEIFORM SIGN NUNUZ AB2 TIMES LA" },
    { 0x12275, "CUNEIFORM SIGN NUNUZ AB2 TIMES NE" },
    { 0x12276, "CUNEIFORM SIGN NUNUZ AB2 TIMES SILA3" },
    { 0x12277, "CUNEIFORM SIGN NUNUZ AB2 TIMES U2" },
    { 0x12278, "CUNEIFORM SIGN NUNUZ KISIM5 TIMES BI" },
    { 0x12279, "CUNEIFORM SIGN NUNUZ KISIM5 TIMES BI U" },
    { 0x1227A, "CUNEIFORM SIGN PA" },
    { 0x1227B, "CUNEIFORM SIGN PAD" },
    { 0x1227C, "CUNEIFORM SIGN PAN" },
    { 0x1227D, "CUNEIFORM SIGN PAP" },
    { 0x1227E, "CUNEIFORM SIGN PESH2" },
    { 0x1227F, "CUNEIFORM SIGN PI" },
    { 0x12280, "CUNEIFORM SIGN PI TIMES A" },
    { 0x12281, "CUNEIFORM SIGN PI TIMES AB" },
    { 0x12282, "CUNEIFORM SIGN PI TIMES BI" },
    { 0x12283, "CUNEIFORM SIGN PI TIMES BU" },
    { 0x12284, "CUNEIFORM SIGN PI TIMES E" },
    { 0x12285, "CUNEIFORM SIGN PI TIMES I" },
    { 0x12286, "CUNEIFORM SIGN PI TIMES IB" },
    { 0x12287, "CUNEIFORM SIGN PI TIMES U" },
    { 0x12288, "CUNEIFORM SIGN PI TIMES U2" },
    { 0x12289, "CUNEIFORM SIGN PI CROSSING PI" },
    { 0x1228A, "CUNEIFORM SIGN PIRIG" },
    { 0x1228B, "CUNEIFORM SIGN PIRIG TIMES KAL" },
    { 0x1228C, "CUNEIFORM SIGN PIRIG TIMES UD" },
    { 0x1228D, "CUNEIFORM SIGN PIRIG TIMES ZA" },
    { 0x1228E, "CUNEIFORM SIGN PIRIG OPPOSING PIRIG" },
    { 0x1228F, "CUNEIFORM SIGN RA" },
    { 0x12290, "CUNEIFORM SIGN RAB" },
    { 0x12291, "CUNEIFORM SIGN RI" },
    { 0x12292, "CUNEIFORM SIGN RU" },
    { 0x12293, "CUNEIFORM SIGN SA" },
    { 0x12294, "CUNEIFORM SIGN SAG NUTILLU" },
    { 0x12295, "CUNEIFORM SIGN SAG" },
    { 0x12296, "CUNEIFORM SIGN SAG TIMES A" },
    { 0x12297, "CUNEIFORM SIGN SAG TIMES DU" },
    { 0x12298, "CUNEIFORM SIGN SAG TIMES DUB" },
    { 0x12299, "CUNEIFORM SIGN SAG TIMES HA" },
    { 0x1229A, "CUNEIFORM SIGN SAG TIMES KAK" },
    { 0x1229B, "CUNEIFORM SIGN SAG TIMES KUR" },
    { 0x1229C, "CUNEIFORM SIGN SAG TIMES LUM" },
    { 0x1229D, "CUNEIFORM SIGN SAG TIMES MI" },
    { 0x1229E, "CUNEIFORM SIGN SAG TIMES NUN" },
    { 0x1229F, "CUNEIFORM SIGN SAG TIMES SAL" },
    { 0x122A0, "CUNEIFORM SIGN SAG TIMES SHID" },
    { 0x122A1, "CUNEIFORM SIGN SAG TIMES TAB" },
    { 0x122A2, "CUNEIFORM SIGN SAG TIMES U2" },
    { 0x122A3, "CUNEIFORM SIGN SAG TIMES UB" },
    { 0x122A4, "CUNEIFORM SIGN SAG TIMES UM" },
    { 0x122A5, "CUNEIFORM SIGN SAG TIMES UR" },
    { 0x122A6, "CUNEIFORM SIGN SAG TIMES USH" },
    { 0x122A7, "CUNEIFORM SIGN SAG OVER SAG" },
    { 0x122A8, "CUNEIFORM SIGN SAG GUNU" },
    { 0x122A9, "CUNEIFORM SIGN SAL" },
    { 0x122AA, "CUNEIFORM SIGN SAL LAGAB TIMES ASH2" },
    { 0x122AB, "CUNEIFORM SIGN SANGA2" },
    { 0x122AC, "CUNEIFORM SIGN SAR" },
    { 0x122AD, "CUNEIFORM SIGN SHA" },
    { 0x122AE, "CUNEIFORM SIGN SHA3" },
    { 0x122AF, "CUNEIFORM SIGN SHA3 TIMES A" },
    { 0x122B0, "CUNEIFORM SIGN SHA3 TIMES BAD" },
    { 0x122B1, "CUNEIFORM SIGN SHA3 TIMES GISH" },
    { 0x122B2, "CUNEIFORM SIGN SHA3 TIMES NE" },
    { 0x122B3, "CUNEIFORM SIGN SHA3 TIMES SHU2" },
    { 0x122B4, "CUNEIFORM SIGN SHA3 TIMES TUR" },
    { 0x122B5, "CUNEIFORM SIGN SHA3 TIMES U" },
    { 0x122B6, "CUNEIFORM SIGN SHA3 TIMES U PLUS A" },
    { 0x122B7, "CUNEIFORM SIGN SHA6" },
    { 0x122B8, "CUNEIFORM SIGN SHAB6" },
    { 0x122B9, "CUNEIFORM SIGN SHAR2" },
    { 0x122BA, "CUNEIFORM SIGN SHE" },
    { 0x122BB, "CUNEIFORM SIGN SHE HU" },
    { 0x122BC, "CUNEIFORM SIGN SHE OVER SHE GAD OVER GAD GAR OVER GAR" },
    { 0x122BD, "CUNEIFORM SIGN SHE OVER SHE TAB OVER TAB GAR OVER GAR" },
    { 0x122BE, "CUNEIFORM SIGN SHEG9" },
    { 0x122BF, "CUNEIFORM SIGN SHEN" },
    { 0x122C0, "CUNEIFORM SIGN SHESH" },
    { 0x122C1, "CUNEIFORM SIGN SHESH2" },
    { 0x122C2, "CUNEIFORM SIGN SHESHLAM" },
    { 0x122C3, "CUNEIFORM SIGN SHID" },
    { 0x122C4, "CUNEIFORM SIGN SHID TIMES A" },
    { 0x122C5, "CUNEIFORM SIGN SHID TIMES IM" },
    { 0x122C6, "CUNEIFORM SIGN SHIM" },
    { 0x122C7, "CUNEIFORM SIGN SHIM TIMES A" },
    { 0x122C8, "CUNEIFORM SIGN SHIM TIMES BAL" },
    { 0x122C9, "CUNEIFORM SIGN SHIM TIMES BULUG" },
    { 0x122CA, "CUNEIFORM SIGN SHIM TIMES DIN" },
    { 0x122CB, "CUNEIFORM SIGN SHIM TIMES GAR" },
    { 0x122CC, "CUNEIFORM SIGN SHIM TIMES IGI" },
    { 0x122CD, "CUNEIFORM SIGN SHIM TIMES IGI GUNU" },
    { 0x122CE, "CUNEIFORM SIGN SHIM TIMES KUSHU2" },
    { 0x122CF, "CUNEIFORM SIGN SHIM TIMES LUL" },
    { 0x122D0, "CUNEIFORM SIGN SHIM TIMES MUG" },
    { 0x122D1, "CUNEIFORM SIGN SHIM TIMES SAL" },
    { 0x122D2, "CUNEIFORM SIGN SHINIG" },
    { 0x122D3, "CUNEIFORM SIGN SHIR" },
    { 0x122D4, "CUNEIFORM SIGN SHIR TENU" },
    { 0x122D5, "CUNEIFORM SIGN SHIR OVER SHIR BUR OVER BUR" },
    { 0x122D6, "CUNEIFORM SIGN SHITA" },
    { 0x122D7, "CUNEIFORM SIGN SHU" },
    { 0x122D8, "CUNEIFORM SIGN SHU OVER INVERTED SHU" },
    { 0x122D9, "CUNEIFORM SIGN SHU2" },
    { 0x122DA, "CUNEIFORM SIGN SHUBUR" },
    { 0x122DB, "CUNEIFORM SIGN SI" },
    { 0x122DC, "CUNEIFORM SIGN SI GUNU" },
    { 0x122DD, "CUNEIFORM SIGN SIG" },
    { 0x122DE, "CUNEIFORM SIGN SIG4" },
    { 0x122DF, "CUNEIFORM SIGN SIG4 OVER SIG4 SHU2" },
    { 0x122E0, "CUNEIFORM SIGN SIK2" },
    { 0x122E1, "CUNEIFORM SIGN SILA3" },
    { 0x122E2, "CUNEIFORM SIGN SU" },
    { 0x122E3, "CUNEIFORM SIGN SU OVER SU" },
    { 0x122E4, "CUNEIFORM SIGN SUD" },
    { 0x122E5, "CUNEIFORM SIGN SUD2" },
    { 0x122E6, "CUNEIFORM SIGN SUHUR" },
    { 0x122E7, "CUNEIFORM SIGN SUM" },
    { 0x122E8, "CUNEIFORM SIGN SUMASH" },
    { 0x122E9, "CUNEIFORM SIGN SUR" },
    { 0x122EA, "CUNEIFORM SIGN SUR9" },
    { 0x122EB, "CUNEIFORM SIGN TA" },
    { 0x122EC, "CUNEIFORM SIGN TA ASTERISK" },
    { 0x122ED, "CUNEIFORM SIGN TA TIMES HI" },
    { 0x122EE, "CUNEIFORM SIGN TA TIMES MI" },
    { 0x122EF, "CUNEIFORM SIGN TA GUNU" },
    { 0x122F0, "CUNEIFORM SIGN TAB" },
    { 0x122F1, "CUNEIFORM SIGN TAB OVER TAB NI OVER NI DISH OVER DISH" },
    { 0x122F2, "CUNEIFORM SIGN TAB SQUARED" },
    { 0x122F3, "CUNEIFORM SIGN TAG" },
    { 0x122F4, "CUNEIFORM SIGN TAG TIMES BI" },
    { 0x122F5, "CUNEIFORM SIGN TAG TIMES GUD" },
    { 0x122F6, "CUNEIFORM SIGN TAG TIMES SHE" },
    { 0x122F7, "CUNEIFORM SIGN TAG TIMES SHU" },
    { 0x122F8, "CUNEIFORM SIGN TAG TIMES TUG2" },
    { 0x122F9, "CUNEIFORM SIGN TAG TIMES UD" },
    { 0x122FA, "CUNEIFORM SIGN TAK4" },
    { 0x122FB, "CUNEIFORM SIGN TAR" },
    { 0x122FC, "CUNEIFORM SIGN TE" },
    { 0x122FD, "CUNEIFORM SIGN TE GUNU" },
    { 0x122FE, "CUNEIFORM SIGN TI" },
    { 0x122FF, "CUNEIFORM SIGN TI TENU" },
    { 0x12300, "CUNEIFORM SIGN TIL" },
    { 0x12301, "CUNEIFORM SIGN TIR" },
    { 0x12302, "CUNEIFORM SIGN TIR TIMES TAK4" },
    { 0x12303, "CUNEIFORM SIGN TIR OVER TIR" },
    { 0x12304, "CUNEIFORM SIGN TIR OVER TIR GAD OVER GAD GAR OVER GAR" },
    { 0x12305, "CUNEIFORM SIGN TU" },
    { 0x12306, "CUNEIFORM SIGN TUG2" },
    { 0x12307, "CUNEIFORM SIGN TUK" },
    { 0x12308, "CUNEIFORM SIGN TUM" },
    { 0x12309, "CUNEIFORM SIGN TUR" },
    { 0x1230A, "CUNEIFORM SIGN TUR OVER TUR ZA OVER ZA" },
    { 0x1230B, "CUNEIFORM SIGN U" },
    { 0x1230C, "CUNEIFORM SIGN U GUD" },
    { 0x1230D, "CUNEIFORM SIGN U U U" },
    { 0x1230E, "CUNEIFORM SIGN U OVER U PA OVER PA GAR OVER GAR" },
    { 0x1230F, "CUNEIFORM SIGN U OVER U SUR OVER SUR" },
    { 0x12310, "CUNEIFORM SIGN U OVER U U REVERSED OVER U REVERSED" },
    { 0x12311, "CUNEIFORM SIGN U2" },
    { 0x12312, "CUNEIFORM SIGN UB" },
    { 0x12313, "CUNEIFORM SIGN UD" },
    { 0x12314, "CUNEIFORM SIGN UD KUSHU2" },
    { 0x12315, "CUNEIFORM SIGN UD TIMES BAD" },
    { 0x12316, "CUNEIFORM SIGN UD TIMES MI" },
    { 0x12317, "CUNEIFORM SIGN UD TIMES U PLUS U PLUS U" },
    { 0x12318, "CUNEIFORM SIGN UD TIMES U PLUS U PLUS U GUNU" },
    { 0x12319, "CUNEIFORM SIGN UD GUNU" },
    { 0x1231A, "CUNEIFORM SIGN UD SHESHIG" },
    { 0x1231B, "CUNEIFORM SIGN UD SHESHIG TIMES BAD" },
    { 0x1231C, "CUNEIFORM SIGN UDUG" },
    { 0x1231D, "CUNEIFORM SIGN UM" },
    { 0x1231E, "CUNEIFORM SIGN UM TIMES LAGAB" },
    { 0x1231F, "CUNEIFORM SIGN UM TIMES ME PLUS DA" },
    { 0x12320, "CUNEIFORM SIGN UM TIMES SHA3" },
    { 0x12321, "CUNEIFORM SIGN UM TIMES U" },
    { 0x12322, "CUNEIFORM SIGN UMBIN" },
    { 0x12323, "CUNEIFORM SIGN UMUM" },
    { 0x12324, "CUNEIFORM SIGN UMUM TIMES KASKAL" },
    { 0x12325, "CUNEIFORM SIGN UMUM TIMES PA" },
    { 0x12326, "CUNEIFORM SIGN UN" },
    { 0x12327, "CUNEIFORM SIGN UN GUNU" },
    { 0x12328, "CUNEIFORM SIGN UR" },
    { 0x12329, "CUNEIFORM SIGN UR CROSSING UR" },
    { 0x1232A, "CUNEIFORM SIGN UR SHESHIG" },
    { 0x1232B, "CUNEIFORM SIGN UR2" },
    { 0x1232C, "CUNEIFORM SIGN UR2 TIMES A PLUS HA" },
    { 0x1232D, "CUNEIFORM SIGN UR2 TIMES A PLUS NA" },
    { 0x1232E, "CUNEIFORM SIGN UR2 TIMES AL" },
    { 0x1232F, "CUNEIFORM SIGN UR2 TIMES HA" },
    { 0x12330, "CUNEIFORM SIGN UR2 TIMES NUN" },
    { 0x12331, "CUNEIFORM SIGN UR2 TIMES U2" },
    { 0x12332, "CUNEIFORM SIGN UR2 TIMES U2 PLUS ASH" },
    { 0x12333, "CUNEIFORM SIGN UR2 TIMES U2 PLUS BI" },
    { 0x12334, "CUNEIFORM SIGN UR4" },
    { 0x12335, "CUNEIFORM SIGN URI" },
    { 0x12336, "CUNEIFORM SIGN URI3" },
    { 0x12337, "CUNEIFORM SIGN URU" },
    { 0x12338, "CUNEIFORM SIGN URU TIMES A" },
    { 0x12339, "CUNEIFORM SIGN URU TIMES ASHGAB" },
    { 0x1233A, "CUNEIFORM SIGN URU TIMES BAR" },
    { 0x1233B, "CUNEIFORM SIGN URU TIMES DUN" },
    { 0x1233C, "CUNEIFORM SIGN URU TIMES GA" },
    { 0x1233D, "CUNEIFORM SIGN URU TIMES GAL" },
    { 0x1233E, "CUNEIFORM SIGN URU TIMES GAN2 TENU" },
    { 0x1233F, "CUNEIFORM SIGN URU TIMES GAR" },
    { 0x12340, "CUNEIFORM SIGN URU TIMES GU" },
    { 0x12341, "CUNEIFORM SIGN URU TIMES HA" },
    { 0x12342, "CUNEIFORM SIGN URU TIMES IGI" },
    { 0x12343, "CUNEIFORM SIGN URU TIMES IM" },
    { 0x12344, "CUNEIFORM SIGN URU TIMES ISH" },
    { 0x12345, "CUNEIFORM SIGN URU TIMES KI" },
    { 0x12346, "CUNEIFORM SIGN URU TIMES LUM" },
    { 0x12347, "CUNEIFORM SIGN URU TIMES MIN" },
    { 0x12348, "CUNEIFORM SIGN URU TIMES PA" },
    { 0x12349, "CUNEIFORM SIGN URU TIMES SHE" },
    { 0x1234A, "CUNEIFORM SIGN URU TIMES SIG4" },
    { 0x1234B, "CUNEIFORM SIGN URU TIMES TU" },
    { 0x1234C, "CUNEIFORM SIGN URU TIMES U PLUS GUD" },
    { 0x1234D, "CUNEIFORM SIGN URU TIMES UD" },
    { 0x1234E, "CUNEIFORM SIGN URU TIMES URUDA" },
    { 0x1234F, "CUNEIFORM SIGN URUDA" },
    { 0x12350, "CUNEIFORM SIGN URUDA TIMES U" },
    { 0x12351, "CUNEIFORM SIGN USH" },
    { 0x12352, "CUNEIFORM SIGN USH TIMES A" },
    { 0x12353, "CUNEIFORM SIGN USH TIMES KU" },
    { 0x12354, "CUNEIFORM SIGN USH TIMES KUR" },
    { 0x12355, "CUNEIFORM SIGN USH TIMES TAK4" },
    { 0x12356, "CUNEIFORM SIGN USHX" },
    { 0x12357, "CUNEIFORM SIGN USH2" },
    { 0x12358, "CUNEIFORM SIGN USHUMX" },
    { 0x12359, "CUNEIFORM SIGN UTUKI" },
    { 0x1235A, "CUNEIFORM SIGN UZ3" },
    { 0x1235B, "CUNEIFORM SIGN UZ3 TIMES KASKAL" },
    { 0x1235C, "CUNEIFORM SIGN UZU" },
    { 0x1235D, "CUNEIFORM SIGN ZA" },
    { 0x1235E, "CUNEIFORM SIGN ZA TENU" },
    { 0x1235F, "CUNEIFORM SIGN ZA SQUARED TIMES KUR" },
    { 0x12360, "CUNEIFORM SIGN ZAG" },
    { 0x12361, "CUNEIFORM SIGN ZAMX" },
    { 0x12362, "CUNEIFORM SIGN ZE2" },
    { 0x12363, "CUNEIFORM SIGN ZI" },
    { 0x12364, "CUNEIFORM SIGN ZI OVER ZI" },
    { 0x12365, "CUNEIFORM SIGN ZI3" },
    { 0x12366, "CUNEIFORM SIGN ZIB" },
    { 0x12367, "CUNEIFORM SIGN ZIB KABA TENU" },
    { 0x12368, "CUNEIFORM SIGN ZIG" },
    { 0x12369, "CUNEIFORM SIGN ZIZ2" },
    { 0x1236A, "CUNEIFORM SIGN ZU" },
    { 0x1236B, "CUNEIFORM SIGN ZU5" },
    { 0x1236C, "CUNEIFORM SIGN ZU5 TIMES A" },
    { 0x1236D, "CUNEIFORM SIGN ZUBUR" },
    { 0x1236E, "CUNEIFORM SIGN ZUM" },
    { 0x1236F, "CUNEIFORM SIGN KAP ELAMITE" },
    { 0x12370, "CUNEIFORM SIGN AB TIMES NUN" },
    { 0x12371, "CUNEIFORM SIGN AB2 TIMES A" },
    { 0x12372, "CUNEIFORM SIGN AMAR TIMES KUG" },
    { 0x12373, "CUNEIFORM SIGN DAG KISIM5 TIMES U2 PLUS MASH" },
    { 0x12374, "CUNEIFORM SIGN DAG3" },
    { 0x12375, "CUNEIFORM SIGN DISH PLUS SHU" },
    { 0x12376, "CUNEIFORM SIGN DUB TIMES SHE" },
    { 0x12377, "CUNEIFORM SIGN EZEN TIMES GUD" },
    { 0x12378, "CUNEIFORM SIGN EZEN TIMES SHE" },
    { 0x12379, "CUNEIFORM SIGN GA2 TIMES AN PLUS KAK PLUS A" },
    { 0x1237A, "CUNEIFORM SIGN GA2 TIMES ASH2" },
    { 0x1237B, "CUNEIFORM SIGN GE22" },
    { 0x1237C, "CUNEIFORM SIGN GIG" },
    { 0x1237D, "CUNEIFORM SIGN HUSH" },
    { 0x1237E, "CUNEIFORM SIGN KA TIMES ANSHE" },
    { 0x1237F, "CUNEIFORM SIGN KA TIMES ASH3" },
    { 0x12380, "CUNEIFORM SIGN KA TIMES GISH" },
    { 0x12381, "CUNEIFORM SIGN KA TIMES GUD" },
    { 0x12382, "CUNEIFORM SIGN KA TIMES HI TIMES ASH2" },
    { 0x12383, "CUNEIFORM SIGN KA TIMES LUM" },
    { 0x12384, "CUNEIFORM SIGN KA TIMES PA" },
    { 0x12385, "CUNEIFORM SIGN KA TIMES SHUL" },
    { 0x12386, "CUNEIFORM SIGN KA TIMES TU" },
    { 0x12387, "CUNEIFORM SIGN KA TIMES UR2" },
    { 0x12388, "CUNEIFORM SIGN LAGAB TIMES GI" },
    { 0x12389, "CUNEIFORM SIGN LU2 SHESHIG TIMES BAD" },
    { 0x1238A, "CUNEIFORM SIGN LU2 TIMES ESH2 PLUS LAL" },
    { 0x1238B, "CUNEIFORM SIGN LU2 TIMES SHU" },
    { 0x1238C, "CUNEIFORM SIGN MESH" },
    { 0x1238D, "CUNEIFORM SIGN MUSH3 TIMES ZA" },
    { 0x1238E, "CUNEIFORM SIGN NA4" },
    { 0x1238F, "CUNEIFORM SIGN NIN" },
    { 0x12390, "CUNEIFORM SIGN NIN9" },
    { 0x12391, "CUNEIFORM SIGN NINDA2 TIMES BAL" },
    { 0x12392, "CUNEIFORM SIGN NINDA2 TIMES GI" },
    { 0x12393, "CUNEIFORM SIGN NU11 ROTATED NINETY DEGREES" },
    { 0x12394, "CUNEIFORM SIGN PESH2 ASTERISK" },
    { 0x12395, "CUNEIFORM SIGN PIR2" },
    { 0x12396, "CUNEIFORM SIGN SAG TIMES IGI GUNU" },
    { 0x12397, "CUNEIFORM SIGN TI2" },
    { 0x12398, "CUNEIFORM SIGN UM TIMES ME" },
    { 0x12399, "CUNEIFORM SIGN U U" },
    { 0x12400, "CUNEIFORM NUMERIC SIGN TWO ASH" },
    { 0x12401, "CUNEIFORM NUMERIC SIGN THREE ASH" },
    { 0x12402, "CUNEIFORM NUMERIC SIGN FOUR ASH" },
    { 0x12403, "CUNEIFORM NUMERIC SIGN FIVE ASH" },
    { 0x12404, "CUNEIFORM NUMERIC SIGN SIX ASH" },
    { 0x12405, "CUNEIFORM NUMERIC SIGN SEVEN ASH" },
    { 0x12406, "CUNEIFORM NUMERIC SIGN EIGHT ASH" },
    { 0x12407, "CUNEIFORM NUMERIC SIGN NINE ASH" },
    { 0x12408, "CUNEIFORM NUMERIC SIGN THREE DISH" },
    { 0x12409, "CUNEIFORM NUMERIC SIGN FOUR DISH" },
    { 0x1240A, "CUNEIFORM NUMERIC SIGN FIVE DISH" },
    { 0x1240B, "CUNEIFORM NUMERIC SIGN SIX DISH" },
    { 0x1240C, "CUNEIFORM NUMERIC SIGN SEVEN DISH" },
    { 0x1240D, "CUNEIFORM NUMERIC SIGN EIGHT DISH" },
    { 0x1240E, "CUNEIFORM NUMERIC SIGN NINE DISH" },
    { 0x1240F, "CUNEIFORM NUMERIC SIGN FOUR U" },
    { 0x12410, "CUNEIFORM NUMERIC SIGN FIVE U" },
    { 0x12411, "CUNEIFORM NUMERIC SIGN SIX U" },
    { 0x12412, "CUNEIFORM NUMERIC SIGN SEVEN U" },
    { 0x12413, "CUNEIFORM NUMERIC SIGN EIGHT U" },
    { 0x12414, "CUNEIFORM NUMERIC SIGN NINE U" },
    { 0x12415, "CUNEIFORM NUMERIC SIGN ONE GESH2" },
    { 0x12416, "CUNEIFORM NUMERIC SIGN TWO GESH2" },
    { 0x12417, "CUNEIFORM NUMERIC SIGN THREE GESH2" },
    { 0x12418, "CUNEIFORM NUMERIC SIGN FOUR GESH2" },
    { 0x12419, "CUNEIFORM NUMERIC SIGN FIVE GESH2" },
    { 0x1241A, "CUNEIFORM NUMERIC SIGN SIX GESH2" },
    { 0x1241B, "CUNEIFORM NUMERIC SIGN SEVEN GESH2" },
    { 0x1241C, "CUNEIFORM NUMERIC SIGN EIGHT GESH2" },
    { 0x1241D, "CUNEIFORM NUMERIC SIGN NINE GESH2" },
    { 0x1241E, "CUNEIFORM NUMERIC SIGN ONE GESHU" },
    { 0x1241F, "CUNEIFORM NUMERIC SIGN TWO GESHU" },
    { 0x12420, "CUNEIFORM NUMERIC SIGN THREE GESHU" },
    { 0x12421, "CUNEIFORM NUMERIC SIGN FOUR GESHU" },
    { 0x12422, "CUNEIFORM NUMERIC SIGN FIVE GESHU" },
    { 0x12423, "CUNEIFORM NUMERIC SIGN TWO SHAR2" },
    { 0x12424, "CUNEIFORM NUMERIC SIGN THREE SHAR2" },
    { 0x12425, "CUNEIFORM NUMERIC SIGN THREE SHAR2 VARIANT FORM" },
    { 0x12426, "CUNEIFORM NUMERIC SIGN FOUR SHAR2" },
    { 0x12427, "CUNEIFORM NUMERIC SIGN FIVE SHAR2" },
    { 0x12428, "CUNEIFORM NUMERIC SIGN SIX SHAR2" },
    { 0x12429, "CUNEIFORM NUMERIC SIGN SEVEN SHAR2" },
    { 0x1242A, "CUNEIFORM NUMERIC SIGN EIGHT SHAR2" },
    { 0x1242B, "CUNEIFORM NUMERIC SIGN NINE SHAR2" },
    { 0x1242C, "CUNEIFORM NUMERIC SIGN ONE SHARU" },
    { 0x1242D, "CUNEIFORM NUMERIC SIGN TWO SHARU" },
    { 0x1242E, "CUNEIFORM NUMERIC SIGN THREE SHARU" },
    { 0x1242F, "CUNEIFORM NUMERIC SIGN THREE SHARU VARIANT FORM" },
    { 0x12430, "CUNEIFORM NUMERIC SIGN FOUR SHARU" },
    { 0x12431, "CUNEIFORM NUMERIC SIGN FIVE SHARU" },
    { 0x12432, "CUNEIFORM NUMERIC SIGN SHAR2 TIMES GAL PLUS DISH" },
    { 0x12433, "CUNEIFORM NUMERIC SIGN SHAR2 TIMES GAL PLUS MIN" },
    { 0x12434, "CUNEIFORM NUMERIC SIGN ONE BURU" },
    { 0x12435, "CUNEIFORM NUMERIC SIGN TWO BURU" },
    { 0x12436, "CUNEIFORM NUMERIC SIGN THREE BURU" },
    { 0x12437, "CUNEIFORM NUMERIC SIGN THREE BURU VARIANT FORM" },
    { 0x12438, "CUNEIFORM NUMERIC SIGN FOUR BURU" },
    { 0x12439, "CUNEIFORM NUMERIC SIGN FIVE BURU" },
    { 0x1243A, "CUNEIFORM NUMERIC SIGN THREE VARIANT FORM ESH16" },
    { 0x1243B, "CUNEIFORM NUMERIC SIGN THREE VARIANT FORM ESH21" },
    { 0x1243C, "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU" },
    { 0x1243D, "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU4" },
    { 0x1243E, "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU A" },
    { 0x1243F, "CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU B" },
    { 0x12440, "CUNEIFORM NUMERIC SIGN SIX VARIANT FORM ASH9" },
    { 0x12441, "CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN3" },
    { 0x12442, "CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN A" },
    { 0x12443, "CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN B" },
    { 0x12444, "CUNEIFORM NUMERIC SIGN EIGHT VARIANT FORM USSU" },
    { 0x12445, "CUNEIFORM NUMERIC SIGN EIGHT VARIANT FORM USSU3" },
    { 0x12446, "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU" },
    { 0x12447, "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU3" },
    { 0x12448, "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU4" },
    { 0x12449, "CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU A" },
    { 0x1244A, "CUNEIFORM NUMERIC SIGN TWO ASH TENU" },
    { 0x1244B, "CUNEIFORM NUMERIC SIGN THREE ASH TENU" },
    { 0x1244C, "CUNEIFORM NUMERIC SIGN FOUR ASH TENU" },
    { 0x1244D, "CUNEIFORM NUMERIC SIGN FIVE ASH TENU" },
    { 0x1244E, "CUNEIFORM NUMERIC SIGN SIX ASH TENU" },
    { 0x1244F, "CUNEIFORM NUMERIC SIGN ONE BAN2" },
    { 0x12450, "CUNEIFORM NUMERIC SIGN TWO BAN2" },
    { 0x12451, "CUNEIFORM NUMERIC SIGN THREE BAN2" },
    { 0x12452, "CUNEIFORM NUMERIC SIGN FOUR BAN2" },
    { 0x12453, "CUNEIFORM NUMERIC SIGN FOUR BAN2 VARIANT FORM" },
    { 0x12454, "CUNEIFORM NUMERIC SIGN FIVE BAN2" },
    { 0x12455, "CUNEIFORM NUMERIC SIGN FIVE BAN2 VARIANT FORM" },
    { 0x12456, "CUNEIFORM NUMERIC SIGN NIGIDAMIN" },
    { 0x12457, "CUNEIFORM NUMERIC SIGN NIGIDAESH" },
    { 0x12458, "CUNEIFORM NUMERIC SIGN ONE ESHE3" },
    { 0x12459, "CUNEIFORM NUMERIC SIGN TWO ESHE3" },
    { 0x1245A, "CUNEIFORM NUMERIC SIGN ONE THIRD DISH" },
    { 0x1245B, "CUNEIFORM NUMERIC SIGN TWO THIRDS DISH" },
    { 0x1245C, "CUNEIFORM NUMERIC SIGN FIVE SIXTHS DISH" },
    { 0x1245D, "CUNEIFORM NUMERIC SIGN ONE THIRD VARIANT FORM A" },
    { 0x1245E, "CUNEIFORM NUMERIC SIGN TWO THIRDS VARIANT FORM A" },
    { 0x1245F, "CUNEIFORM NUMERIC SIGN ONE EIGHTH ASH" },
    { 0x12460, "CUNEIFORM NUMERIC SIGN ONE QUARTER ASH" },
    { 0x12461, "CUNEIFORM NUMERIC SIGN OLD ASSYRIAN ONE SIXTH" },
    { 0x12462, "CUNEIFORM NUMERIC SIGN OLD ASSYRIAN ONE QUARTER" },
    { 0x12463, "CUNEIFORM NUMERIC SIGN ONE QUARTER GUR" },
    { 0x12464, "CUNEIFORM NUMERIC SIGN ONE HALF GUR" },
    { 0x12465, "CUNEIFORM NUMERIC SIGN ELAMITE ONE THIRD" },
    { 0x12466, "CUNEIFORM NUMERIC SIGN ELAMITE TWO THIRDS" },
    { 0x12467, "CUNEIFORM NUMERIC SIGN ELAMITE FORTY" },
    { 0x12468, "CUNEIFORM NUMERIC SIGN ELAMITE FIFTY" },
    { 0x12469, "CUNEIFORM NUMERIC SIGN FOUR U VARIANT FORM" },
    { 0x1246A, "CUNEIFORM NUMERIC SIGN FIVE U VARIANT FORM" },
    { 0x1246B, "CUNEIFORM NUMERIC SIGN SIX U VARIANT FORM" },
    { 0x1246C, "CUNEIFORM NUMERIC SIGN SEVEN U VARIANT FORM" },
    { 0x1246D, "CUNEIFORM NUMERIC SIGN EIGHT U VARIANT FORM" },
    { 0x1246E, "CUNEIFORM NUMERIC SIGN NINE U VARIANT FORM" },
    { 0x12470, "CUNEIFORM PUNCTUATION SIGN OLD ASSYRIAN WORD DIVIDER" },
    { 0x12471, "CUNEIFORM PUNCTUATION SIGN VERTICAL COLON" },
    { 0x12472, "CUNEIFORM PUNCTUATION SIGN DIAGONAL COLON" },
    { 0x12473, "CUNEIFORM PUNCTUATION SIGN DIAGONAL TRICOLON" },
    { 0x12474, "CUNEIFORM PUNCTUATION SIGN DIAGONAL QUADCOLON" },
    { 0x12480, "CUNEIFORM SIGN AB TIMES NUN TENU" },
    { 0x12481, "CUNEIFORM SIGN AB TIMES SHU2" },
    { 0x12482, "CUNEIFORM SIGN AD TIMES ESH2" },
    { 0x12483, "CUNEIFORM SIGN BAD TIMES DISH TENU" },
    { 0x12484, "CUNEIFORM SIGN BAHAR2 TIMES AB2" },
    { 0x12485, "CUNEIFORM SIGN BAHAR2 TIMES NI" },
    { 0x12486, "CUNEIFORM SIGN BAHAR2 TIMES ZA" },
    { 0x12487, "CUNEIFORM SIGN BU OVER BU TIMES NA2" },
    { 0x12488, "CUNEIFORM SIGN DA TIMES TAK4" },
    { 0x12489, "CUNEIFORM SIGN DAG TIMES KUR" },
    { 0x1248A, "CUNEIFORM SIGN DIM TIMES IGI" },
    { 0x1248B, "CUNEIFORM SIGN DIM TIMES U U U" },
    { 0x1248C, "CUNEIFORM SIGN DIM2 TIMES UD" },
    { 0x1248D, "CUNEIFORM SIGN DUG TIMES ANSHE" },
    { 0x1248E, "CUNEIFORM SIGN DUG TIMES ASH" },
    { 0x1248F, "CUNEIFORM SIGN DUG TIMES ASH AT LEFT" },
    { 0x12490, "CUNEIFORM SIGN DUG TIMES DIN" },
    { 0x12491, "CUNEIFORM SIGN DUG TIMES DUN" },
    { 0x12492, "CUNEIFORM SIGN DUG TIMES ERIN2" },
    { 0x12493, "CUNEIFORM SIGN DUG TIMES GA" },
    { 0x12494, "CUNEIFORM SIGN DUG TIMES GI" },
    { 0x12495, "CUNEIFORM SIGN DUG TIMES GIR2 GUNU" },
    { 0x12496, "CUNEIFORM SIGN DUG TIMES GISH" },
    { 0x12497, "CUNEIFORM SIGN DUG TIMES HA" },
    { 0x12498, "CUNEIFORM SIGN DUG TIMES HI" },
    { 0x12499, "CUNEIFORM SIGN DUG TIMES IGI GUNU" },
    { 0x1249A, "CUNEIFORM SIGN DUG TIMES KASKAL" },
    { 0x1249B, "CUNEIFORM SIGN DUG TIMES KUR" },
    { 0x1249C, "CUNEIFORM SIGN DUG TIMES KUSHU2" },
    { 0x1249D, "CUNEIFORM SIGN DUG TIMES KUSHU2 PLUS KASKAL" },
    { 0x1249E, "CUNEIFORM SIGN DUG TIMES LAK-020" },
    { 0x1249F, "CUNEIFORM SIGN DUG TIMES LAM" },
    { 0x124A0, "CUNEIFORM SIGN DUG TIMES LAM TIMES KUR" },
    { 0x124A1, "CUNEIFORM SIGN DUG TIMES LUH PLUS GISH" },
    { 0x124A2, "CUNEIFORM SIGN DUG TIMES MASH" },
    { 0x124A3, "CUNEIFORM SIGN DUG TIMES MES" },
    { 0x124A4, "CUNEIFORM SIGN DUG TIMES MI" },
    { 0x124A5, "CUNEIFORM SIGN DUG TIMES NI" },
    { 0x124A6, "CUNEIFORM SIGN DUG TIMES PI" },
    { 0x124A7, "CUNEIFORM SIGN DUG TIMES SHE" },
    { 0x124A8, "CUNEIFORM SIGN DUG TIMES SI GUNU" },
    { 0x124A9, "CUNEIFORM SIGN E2 TIMES KUR" },
    { 0x124AA, "CUNEIFORM SIGN E2 TIMES PAP" },
    { 0x124AB, "CUNEIFORM SIGN ERIN2 X" },
    { 0x124AC, "CUNEIFORM SIGN ESH2 CROSSING ESH2" },
    { 0x124AD, "CUNEIFORM SIGN EZEN SHESHIG TIMES ASH" },
    { 0x124AE, "CUNEIFORM SIGN EZEN SHESHIG TIMES HI" },
    { 0x124AF, "CUNEIFORM SIGN EZEN SHESHIG TIMES IGI GUNU" },
    { 0x124B0, "CUNEIFORM SIGN EZEN SHESHIG TIMES LA" },
    { 0x124B1, "CUNEIFORM SIGN EZEN SHESHIG TIMES LAL" },
    { 0x124B2, "CUNEIFORM SIGN EZEN SHESHIG TIMES ME" },
    { 0x124B3, "CUNEIFORM SIGN EZEN SHESHIG TIMES MES" },
    { 0x124B4, "CUNEIFORM SIGN EZEN SHESHIG TIMES SU" },
    { 0x124B5, "CUNEIFORM SIGN EZEN TIMES SU" },
    { 0x124B6, "CUNEIFORM SIGN GA2 TIMES BAHAR2" },
    { 0x124B7, "CUNEIFORM SIGN GA2 TIMES DIM GUNU" },
    { 0x124B8, "CUNEIFORM SIGN GA2 TIMES DUG TIMES IGI GUNU" },
    { 0x124B9, "CUNEIFORM SIGN GA2 TIMES DUG TIMES KASKAL" },
    { 0x124BA, "CUNEIFORM SIGN GA2 TIMES EREN" },
    { 0x124BB, "CUNEIFORM SIGN GA2 TIMES GA" },
    { 0x124BC, "CUNEIFORM SIGN GA2 TIMES GAR PLUS DI" },
    { 0x124BD, "CUNEIFORM SIGN GA2 TIMES GAR PLUS NE" },
    { 0x124BE, "CUNEIFORM SIGN GA2 TIMES HA PLUS A" },
    { 0x124BF, "CUNEIFORM SIGN GA2 TIMES KUSHU2 PLUS KASKAL" },
    { 0x124C0, "CUNEIFORM SIGN GA2 TIMES LAM" },
    { 0x124C1, "CUNEIFORM SIGN GA2 TIMES LAM TIMES KUR" },
    { 0x124C2, "CUNEIFORM SIGN GA2 TIMES LUH" },
    { 0x124C3, "CUNEIFORM SIGN GA2 TIMES MUSH" },
    { 0x124C4, "CUNEIFORM SIGN GA2 TIMES NE" },
    { 0x124C5, "CUNEIFORM SIGN GA2 TIMES NE PLUS E2" },
    { 0x124C6, "CUNEIFORM SIGN GA2 TIMES NE PLUS GI" },
    { 0x124C7, "CUNEIFORM SIGN GA2 TIMES SHIM" },
    { 0x124C8, "CUNEIFORM SIGN GA2 TIMES ZIZ2" },
    { 0x124C9, "CUNEIFORM SIGN GABA ROTATED NINETY DEGREES" },
    { 0x124CA, "CUNEIFORM SIGN GESHTIN TIMES U" },
    { 0x124CB, "CUNEIFORM SIGN GISH TIMES GISH CROSSING GISH" },
    { 0x124CC, "CUNEIFORM SIGN GU2 TIMES IGI GUNU" },
    { 0x124CD, "CUNEIFORM SIGN GUD PLUS GISH TIMES TAK4" },
    { 0x124CE, "CUNEIFORM SIGN HA TENU GUNU" },
    { 0x124CF, "CUNEIFORM SIGN HI TIMES ASH OVER HI TIMES ASH" },
    { 0x124D0, "CUNEIFORM SIGN KA TIMES BU" },
    { 0x124D1, "CUNEIFORM SIGN KA TIMES KA" },
    { 0x124D2, "CUNEIFORM SIGN KA TIMES U U U" },
    { 0x124D3, "CUNEIFORM SIGN KA TIMES UR" },
    { 0x124D4, "CUNEIFORM SIGN LAGAB TIMES ZU OVER ZU" },
    { 0x124D5, "CUNEIFORM SIGN LAK-003" },
    { 0x124D6, "CUNEIFORM SIGN LAK-021" },
    { 0x124D7, "CUNEIFORM SIGN LAK-025" },
    { 0x124D8, "CUNEIFORM SIGN LAK-030" },
    { 0x124D9, "CUNEIFORM SIGN LAK-050" },
    { 0x124DA, "CUNEIFORM SIGN LAK-051" },
    { 0x124DB, "CUNEIFORM SIGN LAK-062" },
    { 0x124DC, "CUNEIFORM SIGN LAK-079 OVER LAK-079 GUNU" },
    { 0x124DD, "CUNEIFORM SIGN LAK-080" },
    { 0x124DE, "CUNEIFORM SIGN LAK-081 OVER LAK-081" },
    { 0x124DF, "CUNEIFORM SIGN LAK-092" },
    { 0x124E0, "CUNEIFORM SIGN LAK-130" },
    { 0x124E1, "CUNEIFORM SIGN LAK-142" },
    { 0x124E2, "CUNEIFORM SIGN LAK-210" },
    { 0x124E3, "CUNEIFORM SIGN LAK-219" },
    { 0x124E4, "CUNEIFORM SIGN LAK-220" },
    { 0x124E5, "CUNEIFORM SIGN LAK-225" },
    { 0x124E6, "CUNEIFORM SIGN LAK-228" },
    { 0x124E7, "CUNEIFORM SIGN LAK-238" },
    { 0x124E8, "CUNEIFORM SIGN LAK-265" },
    { 0x124E9, "CUNEIFORM SIGN LAK-266" },
    { 0x124EA, "CUNEIFORM SIGN LAK-343" },
    { 0x124EB, "CUNEIFORM SIGN LAK-347" },
    { 0x124EC, "CUNEIFORM SIGN LAK-348" },
    { 0x124ED, "CUNEIFORM SIGN LAK-383" },
    { 0x124EE, "CUNEIFORM SIGN LAK-384" },
    { 0x124EF, "CUNEIFORM SIGN LAK-390" },
    { 0x124F0, "CUNEIFORM SIGN LAK-441" },
    { 0x124F1, "CUNEIFORM SIGN LAK-449" },
    { 0x124F2, "CUNEIFORM SIGN LAK-449 TIMES GU" },
    { 0x124F3, "CUNEIFORM SIGN LAK-449 TIMES IGI" },
    { 0x124F4, "CUNEIFORM SIGN LAK-449 TIMES PAP PLUS LU3" },
    { 0x124F5, "CUNEIFORM SIGN LAK-449 TIMES PAP PLUS PAP PLUS LU3" },
    { 0x124F6, "CUNEIFORM SIGN LAK-449 TIMES U2 PLUS BA" },
    { 0x124F7, "CUNEIFORM SIGN LAK-450" },
    { 0x124F8, "CUNEIFORM SIGN LAK-457" },
    { 0x124F9, "CUNEIFORM SIGN LAK-470" },
    { 0x124FA, "CUNEIFORM SIGN LAK-483" },
    { 0x124FB, "CUNEIFORM SIGN LAK-490" },
    { 0x124FC, "CUNEIFORM SIGN LAK-492" },
    { 0x124FD, "CUNEIFORM SIGN LAK-493" },
    { 0x124FE, "CUNEIFORM SIGN LAK-495" },
    { 0x124FF, "CUNEIFORM SIGN LAK-550" },
    { 0x12500, "CUNEIFORM SIGN LAK-608" },
    { 0x12501, "CUNEIFORM SIGN LAK-617" },
    { 0x12502, "CUNEIFORM SIGN LAK-617 TIMES ASH" },
    { 0x12503, "CUNEIFORM SIGN LAK-617 TIMES BAD" },
    { 0x12504, "CUNEIFORM SIGN LAK-617 TIMES DUN3 GUNU GUNU" },
    { 0x12505, "CUNEIFORM SIGN LAK-617 TIMES KU3" },
    { 0x12506, "CUNEIFORM SIGN LAK-617 TIMES LA" },
    { 0x12507, "CUNEIFORM SIGN LAK-617 TIMES TAR" },
    { 0x12508, "CUNEIFORM SIGN LAK-617 TIMES TE" },
    { 0x12509, "CUNEIFORM SIGN LAK-617 TIMES U2" },
    { 0x1250A, "CUNEIFORM SIGN LAK-617 TIMES UD" },
    { 0x1250B, "CUNEIFORM SIGN LAK-617 TIMES URUDA" },
    { 0x1250C, "CUNEIFORM SIGN LAK-636" },
    { 0x1250D, "CUNEIFORM SIGN LAK-648" },
    { 0x1250E, "CUNEIFORM SIGN LAK-648 TIMES DUB" },
    { 0x1250F, "CUNEIFORM SIGN LAK-648 TIMES GA" },
    { 0x12510, "CUNEIFORM SIGN LAK-648 TIMES IGI" },
    { 0x12511, "CUNEIFORM SIGN LAK-648 TIMES IGI GUNU" },
    { 0x12512, "CUNEIFORM SIGN LAK-648 TIMES NI" },
    { 0x12513, "CUNEIFORM SIGN LAK-648 TIMES PAP PLUS PAP PLUS LU3" },
    { 0x12514, "CUNEIFORM SIGN LAK-648 TIMES SHESH PLUS KI" },
    { 0x12515, "CUNEIFORM SIGN LAK-648 TIMES UD" },
    { 0x12516, "CUNEIFORM SIGN LAK-648 TIMES URUDA" },
    { 0x12517, "CUNEIFORM SIGN LAK-724" },
    { 0x12518, "CUNEIFORM SIGN LAK-749" },
    { 0x12519, "CUNEIFORM SIGN LU2 GUNU TIMES ASH" },
    { 0x1251A, "CUNEIFORM SIGN LU2 TIMES DISH" },
    { 0x1251B, "CUNEIFORM SIGN LU2 TIMES HAL" },
    { 0x1251C, "CUNEIFORM SIGN LU2 TIMES PAP" },
    { 0x1251D, "CUNEIFORM SIGN LU2 TIMES PAP PLUS PAP PLUS LU3" },
    { 0x1251E, "CUNEIFORM SIGN LU2 TIMES TAK4" },
    { 0x1251F, "CUNEIFORM SIGN MI PLUS ZA7" },
    { 0x12520, "CUNEIFORM SIGN MUSH OVER MUSH TIMES GA" },
    { 0x12521, "CUNEIFORM SIGN MUSH OVER MUSH TIMES KAK" },
    { 0x12522, "CUNEIFORM SIGN NINDA2 TIMES DIM GUNU" },
    { 0x12523, "CUNEIFORM SIGN NINDA2 TIMES GISH" },
    { 0x12524, "CUNEIFORM SIGN NINDA2 TIMES GUL" },
    { 0x12525, "CUNEIFORM SIGN NINDA2 TIMES HI" },
    { 0x12526, "CUNEIFORM SIGN NINDA2 TIMES KESH2" },
    { 0x12527, "CUNEIFORM SIGN NINDA2 TIMES LAK-050" },
    { 0x12528, "CUNEIFORM SIGN NINDA2 TIMES MASH" },
    { 0x12529, "CUNEIFORM SIGN NINDA2 TIMES PAP PLUS PAP" },
    { 0x1252A, "CUNEIFORM SIGN NINDA2 TIMES U" },
    { 0x1252B, "CUNEIFORM SIGN NINDA2 TIMES U PLUS U" },
    { 0x1252C, "CUNEIFORM SIGN NINDA2 TIMES URUDA" },
    { 0x1252D, "CUNEIFORM SIGN SAG GUNU TIMES HA" },
    { 0x1252E, "CUNEIFORM SIGN SAG TIMES EN" },
    { 0x1252F, "CUNEIFORM SIGN SAG TIMES SHE AT LEFT" },
    { 0x12530, "CUNEIFORM SIGN SAG TIMES TAK4" },
    { 0x12531, "CUNEIFORM SIGN SHA6 TENU" },
    { 0x12532, "CUNEIFORM SIGN SHE OVER SHE" },
    { 0x12533, "CUNEIFORM SIGN SHE PLUS HUB2" },
    { 0x12534, "CUNEIFORM SIGN SHE PLUS NAM2" },
    { 0x12535, "CUNEIFORM SIGN SHE PLUS SAR" },
    { 0x12536, "CUNEIFORM SIGN SHU2 PLUS DUG TIMES NI" },
    { 0x12537, "CUNEIFORM SIGN SHU2 PLUS E2 TIMES AN" },
    { 0x12538, "CUNEIFORM SIGN SI TIMES TAK4" },
    { 0x12539, "CUNEIFORM SIGN TAK4 PLUS SAG" },
    { 0x1253A, "CUNEIFORM SIGN TUM TIMES GAN2 TENU" },
    { 0x1253B, "CUNEIFORM SIGN TUM TIMES THREE DISH" },
    { 0x1253C, "CUNEIFORM SIGN UR2 INVERTED" },
    { 0x1253D, "CUNEIFORM SIGN UR2 TIMES UD" },
    { 0x1253E, "CUNEIFORM SIGN URU TIMES DARA3" },
    { 0x1253F, "CUNEIFORM SIGN URU TIMES LAK-668" },
    { 0x12540, "CUNEIFORM SIGN URU TIMES LU3" },
    { 0x12541, "CUNEIFORM SIGN ZA7" },
    { 0x12542, "CUNEIFORM SIGN ZU OVER ZU PLUS SAR" },
    { 0x12543, "CUNEIFORM SIGN ZU5 TIMES THREE DISH TENU" },
    { 0x12F90, "CYPRO-MINOAN SIGN CM001" },
    { 0x12F91, "CYPRO-MINOAN SIGN CM002" },
    { 0x12F92, "CYPRO-MINOAN SIGN CM004" },
    { 0x12F93, "CYPRO-MINOAN SIGN CM005" },
    { 0x12F94, "CYPRO-MINOAN SIGN CM006" },
    { 0x12F95, "CYPRO-MINOAN SIGN CM007" },
    { 0x12F96, "CYPRO-MINOAN SIGN CM008" },
    { 0x12F97, "CYPRO-MINOAN SIGN CM009" },
    { 0x12F98, "CYPRO-MINOAN SIGN CM010" },
    { 0x12F99, "CYPRO-MINOAN SIGN CM011" },
    { 0x12F9A, "CYPRO-MINOAN SIGN CM012" },
    { 0x12F9B, "CYPRO-MINOAN SIGN CM012B" },
    { 0x12F9C, "CYPRO-MINOAN SIGN CM013" },
    { 0x12F9D, "CYPRO-MINOAN SIGN CM015" },
    { 0x12F9E, "CYPRO-MINOAN SIGN CM017" },
    { 0x12F9F, "CYPRO-MINOAN SIGN CM019" },
    { 0x12FA0, "CYPRO-MINOAN SIGN CM021" },
    { 0x12FA1, "CYPRO-MINOAN SIGN CM023" },
    { 0x12FA2, "CYPRO-MINOAN SIGN CM024" },
    { 0x12FA3, "CYPRO-MINOAN SIGN CM025" },
    { 0x12FA4, "CYPRO-MINOAN SIGN CM026" },
    { 0x12FA5, "CYPRO-MINOAN SIGN CM027" },
    { 0x12FA6, "CYPRO-MINOAN SIGN CM028" },
    { 0x12FA7, "CYPRO-MINOAN SIGN CM029" },
    { 0x12FA8, "CYPRO-MINOAN SIGN CM030" },
    { 0x12FA9, "CYPRO-MINOAN SIGN CM033" },
    { 0x12FAA, "CYPRO-MINOAN SIGN CM034" },
    { 0x12FAB, "CYPRO-MINOAN SIGN CM035" },
    { 0x12FAC, "CYPRO-MINOAN SIGN CM036" },
    { 0x12FAD, "CYPRO-MINOAN SIGN CM037" },
    { 0x12FAE, "CYPRO-MINOAN SIGN CM038" },
    { 0x12FAF, "CYPRO-MINOAN SIGN CM039" },
    { 0x12FB0, "CYPRO-MINOAN SIGN CM040" },
    { 0x12FB1, "CYPRO-MINOAN SIGN CM041" },
    { 0x12FB2, "CYPRO-MINOAN SIGN CM044" },
    { 0x12FB3, "CYPRO-MINOAN SIGN CM046" },
    { 0x12FB4, "CYPRO-MINOAN SIGN CM047" },
    { 0x12FB5, "CYPRO-MINOAN SIGN CM049" },
    { 0x12FB6, "CYPRO-MINOAN SIGN CM050" },
    { 0x12FB7, "CYPRO-MINOAN SIGN CM051" },
    { 0x12FB8, "CYPRO-MINOAN SIGN CM052" },
    { 0x12FB9, "CYPRO-MINOAN SIGN CM053" },
    { 0x12FBA, "CYPRO-MINOAN SIGN CM054" },
    { 0x12FBB, "CYPRO-MINOAN SIGN CM055" },
    { 0x12FBC, "CYPRO-MINOAN SIGN CM056" },
    { 0x12FBD, "CYPRO-MINOAN SIGN CM058" },
    { 0x12FBE, "CYPRO-MINOAN SIGN CM059" },
    { 0x12FBF, "CYPRO-MINOAN SIGN CM060" },
    { 0x12FC0, "CYPRO-MINOAN SIGN CM061" },
    { 0x12FC1, "CYPRO-MINOAN SIGN CM062" },
    { 0x12FC2, "CYPRO-MINOAN SIGN CM063" },
    { 0x12FC3, "CYPRO-MINOAN SIGN CM064" },
    { 0x12FC4, "CYPRO-MINOAN SIGN CM066" },
    { 0x12FC5, "CYPRO-MINOAN SIGN CM067" },
    { 0x12FC6, "CYPRO-MINOAN SIGN CM068" },
    { 0x12FC7, "CYPRO-MINOAN SIGN CM069" },
    { 0x12FC8, "CYPRO-MINOAN SIGN CM070" },
    { 0x12FC9, "CYPRO-MINOAN SIGN CM071" },
    { 0x12FCA, "CYPRO-MINOAN SIGN CM072" },
    { 0x12FCB, "CYPRO-MINOAN SIGN CM073" },
    { 0x12FCC, "CYPRO-MINOAN SIGN CM074" },
    { 0x12FCD, "CYPRO-MINOAN SIGN CM075" },
    { 0x12FCE, "CYPRO-MINOAN SIGN CM075B" },
    { 0x12FCF, "CYPRO-MINOAN SIGN CM076" },
    { 0x12FD0, "CYPRO-MINOAN SIGN CM078" },
    { 0x12FD1, "CYPRO-MINOAN SIGN CM079" },
    { 0x12FD2, "CYPRO-MINOAN SIGN CM080" },
    { 0x12FD3, "CYPRO-MINOAN SIGN CM081" },
    { 0x12FD4, "CYPRO-MINOAN SIGN CM082" },
    { 0x12FD5, "CYPRO-MINOAN SIGN CM083" },
    { 0x12FD6, "CYPRO-MINOAN SIGN CM084" },
    { 0x12FD7, "CYPRO-MINOAN SIGN CM085" },
    { 0x12FD8, "CYPRO-MINOAN SIGN CM086" },
    { 0x12FD9, "CYPRO-MINOAN SIGN CM087" },
    { 0x12FDA, "CYPRO-MINOAN SIGN CM088" },
    { 0x12FDB, "CYPRO-MINOAN SIGN CM089" },
    { 0x12FDC, "CYPRO-MINOAN SIGN CM090" },
    { 0x12FDD, "CYPRO-MINOAN SIGN CM091" },
    { 0x12FDE, "CYPRO-MINOAN SIGN CM092" },
    { 0x12FDF, "CYPRO-MINOAN SIGN CM094" },
    { 0x12FE0, "CYPRO-MINOAN SIGN CM095" },
    { 0x12FE1, "CYPRO-MINOAN SIGN CM096" },
    { 0x12FE2, "CYPRO-MINOAN SIGN CM097" },
    { 0x12FE3, "CYPRO-MINOAN SIGN CM098" },
    { 0x12FE4, "CYPRO-MINOAN SIGN CM099" },
    { 0x12FE5, "CYPRO-MINOAN SIGN CM100" },
    { 0x12FE6, "CYPRO-MINOAN SIGN CM101" },
    { 0x12FE7, "CYPRO-MINOAN SIGN CM102" },
    { 0x12FE8, "CYPRO-MINOAN SIGN CM103" },
    { 0x12FE9, "CYPRO-MINOAN SIGN CM104" },
    { 0x12FEA, "CYPRO-MINOAN SIGN CM105" },
    { 0x12FEB, "CYPRO-MINOAN SIGN CM107" },
    { 0x12FEC, "CYPRO-MINOAN SIGN CM108" },
    { 0x12FED, "CYPRO-MINOAN SIGN CM109" },
    { 0x12FEE, "CYPRO-MINOAN SIGN CM110" },
    { 0x12FEF, "CYPRO-MINOAN SIGN CM112" },
    { 0x12FF0, "CYPRO-MINOAN SIGN CM114" },
    { 0x12FF1, "CYPRO-MINOAN SIGN CM301" },
    { 0x12FF2, "CYPRO-MINOAN SIGN CM302" },
    { 0x13000, "EGYPTIAN HIEROGLYPH A001" },
    { 0x13001, "EGYPTIAN HIEROGLYPH A002" },
    { 0x13002, "EGYPTIAN HIEROGLYPH A003" },
    { 0x13003, "EGYPTIAN HIEROGLYPH A004" },
    { 0x13004, "EGYPTIAN HIEROGLYPH A005" },
    { 0x13005, "EGYPTIAN HIEROGLYPH A005A" },
    { 0x13006, "EGYPTIAN HIEROGLYPH A006" },
    { 0x13007, "EGYPTIAN HIEROGLYPH A006A" },
    { 0x13008, "EGYPTIAN HIEROGLYPH A006B" },
    { 0x13009, "EGYPTIAN HIEROGLYPH A007" },
    { 0x1300A, "EGYPTIAN HIEROGLYPH A008" },
    { 0x1300B, "EGYPTIAN HIEROGLYPH A009" },
    { 0x1300C, "EGYPTIAN HIEROGLYPH A010" },
    { 0x1300D, "EGYPTIAN HIEROGLYPH A011" },
    { 0x1300E, "EGYPTIAN HIEROGLYPH A012" },
    { 0x1300F, "EGYPTIAN HIEROGLYPH A013" },
    { 0x13010, "EGYPTIAN HIEROGLYPH A014" },
    { 0x13011, "EGYPTIAN HIEROGLYPH A014A" },
    { 0x13012, "EGYPTIAN HIEROGLYPH A015" },
    { 0x13013, "EGYPTIAN HIEROGLYPH A016" },
    { 0x13014, "EGYPTIAN HIEROGLYPH A017" },
    { 0x13015, "EGYPTIAN HIEROGLYPH A017A" },
    { 0x13016, "EGYPTIAN HIEROGLYPH A018" },
    { 0x13017, "EGYPTIAN HIEROGLYPH A019" },
    { 0x13018, "EGYPTIAN HIEROGLYPH A020" },
    { 0x13019, "EGYPTIAN HIEROGLYPH A021" },
    { 0x1301A, "EGYPTIAN HIEROGLYPH A022" },
    { 0x1301B, "EGYPTIAN HIEROGLYPH A023" },
    { 0x1301C, "EGYPTIAN HIEROGLYPH A024" },
    { 0x1301D, "EGYPTIAN HIEROGLYPH A025" },
    { 0x1301E, "EGYPTIAN HIEROGLYPH A026" },
    { 0x1301F, "EGYPTIAN HIEROGLYPH A027" },
    { 0x13020, "EGYPTIAN HIEROGLYPH A028" },
    { 0x13021, "EGYPTIAN HIEROGLYPH A029" },
    { 0x13022, "EGYPTIAN HIEROGLYPH A030" },
    { 0x13023, "EGYPTIAN HIEROGLYPH A031" },
    { 0x13024, "EGYPTIAN HIEROGLYPH A032" },
    { 0x13025, "EGYPTIAN HIEROGLYPH A032A" },
    { 0x13026, "EGYPTIAN HIEROGLYPH A033" },
    { 0x13027, "EGYPTIAN HIEROGLYPH A034" },
    { 0x13028, "EGYPTIAN HIEROGLYPH A035" },
    { 0x13029, "EGYPTIAN HIEROGLYPH A036" },
    { 0x1302A, "EGYPTIAN HIEROGLYPH A037" },
    { 0x1302B, "EGYPTIAN HIEROGLYPH A038" },
    { 0x1302C, "EGYPTIAN HIEROGLYPH A039" },
    { 0x1302D, "EGYPTIAN HIEROGLYPH A040" },
    { 0x1302E, "EGYPTIAN HIEROGLYPH A040A" },
    { 0x1302F, "EGYPTIAN HIEROGLYPH A041" },
    { 0x13030, "EGYPTIAN HIEROGLYPH A042" },
    { 0x13031, "EGYPTIAN HIEROGLYPH A042A" },
    { 0x13032, "EGYPTIAN HIEROGLYPH A043" },
    { 0x13033, "EGYPTIAN HIEROGLYPH A043A" },
    { 0x13034, "EGYPTIAN HIEROGLYPH A044" },
    { 0x13035, "EGYPTIAN HIEROGLYPH A045" },
    { 0x13036, "EGYPTIAN HIEROGLYPH A045A" },
    { 0x13037, "EGYPTIAN HIEROGLYPH A046" },
    { 0x13038, "EGYPTIAN HIEROGLYPH A047" },
    { 0x13039, "EGYPTIAN HIEROGLYPH A048" },
    { 0x1303A, "EGYPTIAN HIEROGLYPH A049" },
    { 0x1303B, "EGYPTIAN HIEROGLYPH A050" },
    { 0x1303C, "EGYPTIAN HIEROGLYPH A051" },
    { 0x1303D, "EGYPTIAN HIEROGLYPH A052" },
    { 0x1303E, "EGYPTIAN HIEROGLYPH A053" },
    { 0x1303F, "EGYPTIAN HIEROGLYPH A054" },
    { 0x13040, "EGYPTIAN HIEROGLYPH A055" },
    { 0x13041, "EGYPTIAN HIEROGLYPH A056" },
    { 0x13042, "EGYPTIAN HIEROGLYPH A057" },
    { 0x13043, "EGYPTIAN HIEROGLYPH A058" },
    { 0x13044, "EGYPTIAN HIEROGLYPH A059" },
    { 0x13045, "EGYPTIAN HIEROGLYPH A060" },
    { 0x13046, "EGYPTIAN HIEROGLYPH A061" },
    { 0x13047, "EGYPTIAN HIEROGLYPH A062" },
    { 0x13048, "EGYPTIAN HIEROGLYPH A063" },
    { 0x13049, "EGYPTIAN HIEROGLYPH A064" },
    { 0x1304A, "EGYPTIAN HIEROGLYPH A065" },
    { 0x1304B, "EGYPTIAN HIEROGLYPH A066" },
    { 0x1304C, "EGYPTIAN HIEROGLYPH A067" },
    { 0x1304D, "EGYPTIAN HIEROGLYPH A068" },
    { 0x1304E, "EGYPTIAN HIEROGLYPH A069" },
    { 0x1304F, "EGYPTIAN HIEROGLYPH A070" },
    { 0x13050, "EGYPTIAN HIEROGLYPH B001" },
    { 0x13051, "EGYPTIAN HIEROGLYPH B002" },
    { 0x13052, "EGYPTIAN HIEROGLYPH B003" },
    { 0x13053, "EGYPTIAN HIEROGLYPH B004" },
    { 0x13054, "EGYPTIAN HIEROGLYPH B005" },
    { 0x13055, "EGYPTIAN HIEROGLYPH B005A" },
    { 0x13056, "EGYPTIAN HIEROGLYPH B006" },
    { 0x13057, "EGYPTIAN HIEROGLYPH B007" },
    { 0x13058, "EGYPTIAN HIEROGLYPH B008" },
    { 0x13059, "EGYPTIAN HIEROGLYPH B009" },
    { 0x1305A, "EGYPTIAN HIEROGLYPH C001" },
    { 0x1305B, "EGYPTIAN HIEROGLYPH C002" },
    { 0x1305C, "EGYPTIAN HIEROGLYPH C002A" },
    { 0x1305D, "EGYPTIAN HIEROGLYPH C002B" },
    { 0x1305E, "EGYPTIAN HIEROGLYPH C002C" },
    { 0x1305F, "EGYPTIAN HIEROGLYPH C003" },
    { 0x13060, "EGYPTIAN HIEROGLYPH C004" },
    { 0x13061, "EGYPTIAN HIEROGLYPH C005" },
    { 0x13062, "EGYPTIAN HIEROGLYPH C006" },
    { 0x13063, "EGYPTIAN HIEROGLYPH C007" },
    { 0x13064, "EGYPTIAN HIEROGLYPH C008" },
    { 0x13065, "EGYPTIAN HIEROGLYPH C009" },
    { 0x13066, "EGYPTIAN HIEROGLYPH C010" },
    { 0x13067, "EGYPTIAN HIEROGLYPH C010A" },
    { 0x13068, "EGYPTIAN HIEROGLYPH C011" },
    { 0x13069, "EGYPTIAN HIEROGLYPH C012" },
    { 0x1306A, "EGYPTIAN HIEROGLYPH C013" },
    { 0x1306B, "EGYPTIAN HIEROGLYPH C014" },
    { 0x1306C, "EGYPTIAN HIEROGLYPH C015" },
    { 0x1306D, "EGYPTIAN HIEROGLYPH C016" },
    { 0x1306E, "EGYPTIAN HIEROGLYPH C017" },
    { 0x1306F, "EGYPTIAN HIEROGLYPH C018" },
    { 0x13070, "EGYPTIAN HIEROGLYPH C019" },
    { 0x13071, "EGYPTIAN HIEROGLYPH C020" },
    { 0x13072, "EGYPTIAN HIEROGLYPH C021" },
    { 0x13073, "EGYPTIAN HIEROGLYPH C022" },
    { 0x13074, "EGYPTIAN HIEROGLYPH C023" },
    { 0x13075, "EGYPTIAN HIEROGLYPH C024" },
    { 0x13076, "EGYPTIAN HIEROGLYPH D001" },
    { 0x13077, "EGYPTIAN HIEROGLYPH D002" },
    { 0x13078, "EGYPTIAN HIEROGLYPH D003" },
    { 0x13079, "EGYPTIAN HIEROGLYPH D004" },
    { 0x1307A, "EGYPTIAN HIEROGLYPH D005" },
    { 0x1307B, "EGYPTIAN HIEROGLYPH D006" },
    { 0x1307C, "EGYPTIAN HIEROGLYPH D007" },
    { 0x1307D, "EGYPTIAN HIEROGLYPH D008" },
    { 0x1307E, "EGYPTIAN HIEROGLYPH D008A" },
    { 0x1307F, "EGYPTIAN HIEROGLYPH D009" },
    { 0x13080, "EGYPTIAN HIEROGLYPH D010" },
    { 0x13081, "EGYPTIAN HIEROGLYPH D011" },
    { 0x13082, "EGYPTIAN HIEROGLYPH D012" },
    { 0x13083, "EGYPTIAN HIEROGLYPH D013" },
    { 0x13084, "EGYPTIAN HIEROGLYPH D014" },
    { 0x13085, "EGYPTIAN HIEROGLYPH D015" },
    { 0x13086, "EGYPTIAN HIEROGLYPH D016" },
    { 0x13087, "EGYPTIAN HIEROGLYPH D017" },
    { 0x13088, "EGYPTIAN HIEROGLYPH D018" },
    { 0x13089, "EGYPTIAN HIEROGLYPH D019" },
    { 0x1308A, "EGYPTIAN HIEROGLYPH D020" },
    { 0x1308B, "EGYPTIAN HIEROGLYPH D021" },
    { 0x1308C, "EGYPTIAN HIEROGLYPH D022" },
    { 0x1308D, "EGYPTIAN HIEROGLYPH D023" },
    { 0x1308E, "EGYPTIAN HIEROGLYPH D024" },
    { 0x1308F, "EGYPTIAN HIEROGLYPH D025" },
    { 0x13090, "EGYPTIAN HIEROGLYPH D026" },
    { 0x13091, "EGYPTIAN HIEROGLYPH D027" },
    { 0x13092, "EGYPTIAN HIEROGLYPH D027A" },
    { 0x13093, "EGYPTIAN HIEROGLYPH D028" },
    { 0x13094, "EGYPTIAN HIEROGLYPH D029" },
    { 0x13095, "EGYPTIAN HIEROGLYPH D030" },
    { 0x13096, "EGYPTIAN HIEROGLYPH D031" },
    { 0x13097, "EGYPTIAN HIEROGLYPH D031A" },
    { 0x13098, "EGYPTIAN HIEROGLYPH D032" },
    { 0x13099, "EGYPTIAN HIEROGLYPH D033" },
    { 0x1309A, "EGYPTIAN HIEROGLYPH D034" },
    { 0x1309B, "EGYPTIAN HIEROGLYPH D034A" },
    { 0x1309C, "EGYPTIAN HIEROGLYPH D035" },
    { 0x1309D, "EGYPTIAN HIEROGLYPH D036" },
    { 0x1309E, "EGYPTIAN HIEROGLYPH D037" },
    { 0x1309F, "EGYPTIAN HIEROGLYPH D038" },
    { 0x130A0, "EGYPTIAN HIEROGLYPH D039" },
    { 0x130A1, "EGYPTIAN HIEROGLYPH D040" },
    { 0x130A2, "EGYPTIAN HIEROGLYPH D041" },
    { 0x130A3, "EGYPTIAN HIEROGLYPH D042" },
    { 0x130A4, "EGYPTIAN HIEROGLYPH D043" },
    { 0x130A5, "EGYPTIAN HIEROGLYPH D044" },
    { 0x130A6, "EGYPTIAN HIEROGLYPH D045" },
    { 0x130A7, "EGYPTIAN HIEROGLYPH D046" },
    { 0x130A8, "EGYPTIAN HIEROGLYPH D046A" },
    { 0x130A9, "EGYPTIAN HIEROGLYPH D047" },
    { 0x130AA, "EGYPTIAN HIEROGLYPH D048" },
    { 0x130AB, "EGYPTIAN HIEROGLYPH D048A" },
    { 0x130AC, "EGYPTIAN HIEROGLYPH D049" },
    { 0x130AD, "EGYPTIAN HIEROGLYPH D050" },
    { 0x130AE, "EGYPTIAN HIEROGLYPH D050A" },
    { 0x130AF, "EGYPTIAN HIEROGLYPH D050B" },
    { 0x130B0, "EGYPTIAN HIEROGLYPH D050C" },
    { 0x130B1, "EGYPTIAN HIEROGLYPH D050D" },
    { 0x130B2, "EGYPTIAN HIEROGLYPH D050E" },
    { 0x130B3, "EGYPTIAN HIEROGLYPH D050F" },
    { 0x130B4, "EGYPTIAN HIEROGLYPH D050G" },
    { 0x130B5, "EGYPTIAN HIEROGLYPH D050H" },
    { 0x130B6, "EGYPTIAN HIEROGLYPH D050I" },
    { 0x130B7, "EGYPTIAN HIEROGLYPH D051" },
    { 0x130B8, "EGYPTIAN HIEROGLYPH D052" },
    { 0x130B9, "EGYPTIAN HIEROGLYPH D052A" },
    { 0x130BA, "EGYPTIAN HIEROGLYPH D053" },
    { 0x130BB, "EGYPTIAN HIEROGLYPH D054" },
    { 0x130BC, "EGYPTIAN HIEROGLYPH D054A" },
    { 0x130BD, "EGYPTIAN HIEROGLYPH D055" },
    { 0x130BE, "EGYPTIAN HIEROGLYPH D056" },
    { 0x130BF, "EGYPTIAN HIEROGLYPH D057" },
    { 0x130C0, "EGYPTIAN HIEROGLYPH D058" },
    { 0x130C1, "EGYPTIAN HIEROGLYPH D059" },
    { 0x130C2, "EGYPTIAN HIEROGLYPH D060" },
    { 0x130C3, "EGYPTIAN HIEROGLYPH D061" },
    { 0x130C4, "EGYPTIAN HIEROGLYPH D062" },
    { 0x130C5, "EGYPTIAN HIEROGLYPH D063" },
    { 0x130C6, "EGYPTIAN HIEROGLYPH D064" },
    { 0x130C7, "EGYPTIAN HIEROGLYPH D065" },
    { 0x130C8, "EGYPTIAN HIEROGLYPH D066" },
    { 0x130C9, "EGYPTIAN HIEROGLYPH D067" },
    { 0x130CA, "EGYPTIAN HIEROGLYPH D067A" },
    { 0x130CB, "EGYPTIAN HIEROGLYPH D067B" },
    { 0x130CC, "EGYPTIAN HIEROGLYPH D067C" },
    { 0x130CD, "EGYPTIAN HIEROGLYPH D067D" },
    { 0x130CE, "EGYPTIAN HIEROGLYPH D067E" },
    { 0x130CF, "EGYPTIAN HIEROGLYPH D067F" },
    { 0x130D0, "EGYPTIAN HIEROGLYPH D067G" },
    { 0x130D1, "EGYPTIAN HIEROGLYPH D067H" },
    { 0x130D2, "EGYPTIAN HIEROGLYPH E001" },
    { 0x130D3, "EGYPTIAN HIEROGLYPH E002" },
    { 0x130D4, "EGYPTIAN HIEROGLYPH E003" },
    { 0x130D5, "EGYPTIAN HIEROGLYPH E004" },
    { 0x130D6, "EGYPTIAN HIEROGLYPH E005" },
    { 0x130D7, "EGYPTIAN HIEROGLYPH E006" },
    { 0x130D8, "EGYPTIAN HIEROGLYPH E007" },
    { 0x130D9, "EGYPTIAN HIEROGLYPH E008" },
    { 0x130DA, "EGYPTIAN HIEROGLYPH E008A" },
    { 0x130DB, "EGYPTIAN HIEROGLYPH E009" },
    { 0x130DC, "EGYPTIAN HIEROGLYPH E009A" },
    { 0x130DD, "EGYPTIAN HIEROGLYPH E010" },
    { 0x130DE, "EGYPTIAN HIEROGLYPH E011" },
    { 0x130DF, "EGYPTIAN HIEROGLYPH E012" },
    { 0x130E0, "EGYPTIAN HIEROGLYPH E013" },
    { 0x130E1, "EGYPTIAN HIEROGLYPH E014" },
    { 0x130E2, "EGYPTIAN HIEROGLYPH E015" },
    { 0x130E3, "EGYPTIAN HIEROGLYPH E016" },
    { 0x130E4, "EGYPTIAN HIEROGLYPH E016A" },
    { 0x130E5, "EGYPTIAN HIEROGLYPH E017" },
    { 0x130E6, "EGYPTIAN HIEROGLYPH E017A" },
    { 0x130E7, "EGYPTIAN HIEROGLYPH E018" },
    { 0x130E8, "EGYPTIAN HIEROGLYPH E019" },
    { 0x130E9, "EGYPTIAN HIEROGLYPH E020" },
    { 0x130EA, "EGYPTIAN HIEROGLYPH E020A" },
    { 0x130EB, "EGYPTIAN HIEROGLYPH E021" },
    { 0x130EC, "EGYPTIAN HIEROGLYPH E022" },
    { 0x130ED, "EGYPTIAN HIEROGLYPH E023" },
    { 0x130EE, "EGYPTIAN HIEROGLYPH E024" },
    { 0x130EF, "EGYPTIAN HIEROGLYPH E025" },
    { 0x130F0, "EGYPTIAN HIEROGLYPH E026" },
    { 0x130F1, "EGYPTIAN HIEROGLYPH E027" },
    { 0x130F2, "EGYPTIAN HIEROGLYPH E028" },
    { 0x130F3, "EGYPTIAN HIEROGLYPH E028A" },
    { 0x130F4, "EGYPTIAN HIEROGLYPH E029" },
    { 0x130F5, "EGYPTIAN HIEROGLYPH E030" },
    { 0x130F6, "EGYPTIAN HIEROGLYPH E031" },
    { 0x130F7, "EGYPTIAN HIEROGLYPH E032" },
    { 0x130F8, "EGYPTIAN HIEROGLYPH E033" },
    { 0x130F9, "EGYPTIAN HIEROGLYPH E034" },
    { 0x130FA, "EGYPTIAN HIEROGLYPH E034A" },
    { 0x130FB, "EGYPTIAN HIEROGLYPH E036" },
    { 0x130FC, "EGYPTIAN HIEROGLYPH E037" },
    { 0x130FD, "EGYPTIAN HIEROGLYPH E038" },
    { 0x130FE, "EGYPTIAN HIEROGLYPH F001" },
    { 0x130FF, "EGYPTIAN HIEROGLYPH F001A" },
    { 0x13100, "EGYPTIAN HIEROGLYPH F002" },
    { 0x13101, "EGYPTIAN HIEROGLYPH F003" },
    { 0x13102, "EGYPTIAN HIEROGLYPH F004" },
    { 0x13103, "EGYPTIAN HIEROGLYPH F005" },
    { 0x13104, "EGYPTIAN HIEROGLYPH F006" },
    { 0x13105, "EGYPTIAN HIEROGLYPH F007" },
    { 0x13106, "EGYPTIAN HIEROGLYPH F008" },
    { 0x13107, "EGYPTIAN HIEROGLYPH F009" },
    { 0x13108, "EGYPTIAN HIEROGLYPH F010" },
    { 0x13109, "EGYPTIAN HIEROGLYPH F011" },
    { 0x1310A, "EGYPTIAN HIEROGLYPH F012" },
    { 0x1310B, "EGYPTIAN HIEROGLYPH F013" },
    { 0x1310C, "EGYPTIAN HIEROGLYPH F013A" },
    { 0x1310D, "EGYPTIAN HIEROGLYPH F014" },
    { 0x1310E, "EGYPTIAN HIEROGLYPH F015" },
    { 0x1310F, "EGYPTIAN HIEROGLYPH F016" },
    { 0x13110, "EGYPTIAN HIEROGLYPH F017" },
    { 0x13111, "EGYPTIAN HIEROGLYPH F018" },
    { 0x13112, "EGYPTIAN HIEROGLYPH F019" },
    { 0x13113, "EGYPTIAN HIEROGLYPH F020" },
    { 0x13114, "EGYPTIAN HIEROGLYPH F021" },
    { 0x13115, "EGYPTIAN HIEROGLYPH F021A" },
    { 0x13116, "EGYPTIAN HIEROGLYPH F022" },
    { 0x13117, "EGYPTIAN HIEROGLYPH F023" },
    { 0x13118, "EGYPTIAN HIEROGLYPH F024" },
    { 0x13119, "EGYPTIAN HIEROGLYPH F025" },
    { 0x1311A, "EGYPTIAN HIEROGLYPH F026" },
    { 0x1311B, "EGYPTIAN HIEROGLYPH F027" },
    { 0x1311C, "EGYPTIAN HIEROGLYPH F028" },
    { 0x1311D, "EGYPTIAN HIEROGLYPH F029" },
    { 0x1311E, "EGYPTIAN HIEROGLYPH F030" },
    { 0x1311F, "EGYPTIAN HIEROGLYPH F031" },
    { 0x13120, "EGYPTIAN HIEROGLYPH F031A" },
    { 0x13121, "EGYPTIAN HIEROGLYPH F032" },
    { 0x13122, "EGYPTIAN HIEROGLYPH F033" },
    { 0x13123, "EGYPTIAN HIEROGLYPH F034" },
    { 0x13124, "EGYPTIAN HIEROGLYPH F035" },
    { 0x13125, "EGYPTIAN HIEROGLYPH F036" },
    { 0x13126, "EGYPTIAN HIEROGLYPH F037" },
    { 0x13127, "EGYPTIAN HIEROGLYPH F037A" },
    { 0x13128, "EGYPTIAN HIEROGLYPH F038" },
    { 0x13129, "EGYPTIAN HIEROGLYPH F038A" },
    { 0x1312A, "EGYPTIAN HIEROGLYPH F039" },
    { 0x1312B, "EGYPTIAN HIEROGLYPH F040" },
    { 0x1312C, "EGYPTIAN HIEROGLYPH F041" },
    { 0x1312D, "EGYPTIAN HIEROGLYPH F042" },
    { 0x1312E, "EGYPTIAN HIEROGLYPH F043" },
    { 0x1312F, "EGYPTIAN HIEROGLYPH F044" },
    { 0x13130, "EGYPTIAN HIEROGLYPH F045" },
    { 0x13131, "EGYPTIAN HIEROGLYPH F045A" },
    { 0x13132, "EGYPTIAN HIEROGLYPH F046" },
    { 0x13133, "EGYPTIAN HIEROGLYPH F046A" },
    { 0x13134, "EGYPTIAN HIEROGLYPH F047" },
    { 0x13135, "EGYPTIAN HIEROGLYPH F047A" },
    { 0x13136, "EGYPTIAN HIEROGLYPH F048" },
    { 0x13137, "EGYPTIAN HIEROGLYPH F049" },
    { 0x13138, "EGYPTIAN HIEROGLYPH F050" },
    { 0x13139, "EGYPTIAN HIEROGLYPH F051" },
    { 0x1313A, "EGYPTIAN HIEROGLYPH F051A" },
    { 0x1313B, "EGYPTIAN HIEROGLYPH F051B" },
    { 0x1313C, "EGYPTIAN HIEROGLYPH F051C" },
    { 0x1313D, "EGYPTIAN HIEROGLYPH F052" },
    { 0x1313E, "EGYPTIAN HIEROGLYPH F053" },
    { 0x1313F, "EGYPTIAN HIEROGLYPH G001" },
    { 0x13140, "EGYPTIAN HIEROGLYPH G002" },
    { 0x13141, "EGYPTIAN HIEROGLYPH G003" },
    { 0x13142, "EGYPTIAN HIEROGLYPH G004" },
    { 0x13143, "EGYPTIAN HIEROGLYPH G005" },
    { 0x13144, "EGYPTIAN HIEROGLYPH G006" },
    { 0x13145, "EGYPTIAN HIEROGLYPH G006A" },
    { 0x13146, "EGYPTIAN HIEROGLYPH G007" },
    { 0x13147, "EGYPTIAN HIEROGLYPH G007A" },
    { 0x13148, "EGYPTIAN HIEROGLYPH G007B" },
    { 0x13149, "EGYPTIAN HIEROGLYPH G008" },
    { 0x1314A, "EGYPTIAN HIEROGLYPH G009" },
    { 0x1314B, "EGYPTIAN HIEROGLYPH G010" },
    { 0x1314C, "EGYPTIAN HIEROGLYPH G011" },
    { 0x1314D, "EGYPTIAN HIEROGLYPH G011A" },
    { 0x1314E, "EGYPTIAN HIEROGLYPH G012" },
    { 0x1314F, "EGYPTIAN HIEROGLYPH G013" },
    { 0x13150, "EGYPTIAN HIEROGLYPH G014" },
    { 0x13151, "EGYPTIAN HIEROGLYPH G015" },
    { 0x13152, "EGYPTIAN HIEROGLYPH G016" },
    { 0x13153, "EGYPTIAN HIEROGLYPH G017" },
    { 0x13154, "EGYPTIAN HIEROGLYPH G018" },
    { 0x13155, "EGYPTIAN HIEROGLYPH G019" },
    { 0x13156, "EGYPTIAN HIEROGLYPH G020" },
    { 0x13157, "EGYPTIAN HIEROGLYPH G020A" },
    { 0x13158, "EGYPTIAN HIEROGLYPH G021" },
    { 0x13159, "EGYPTIAN HIEROGLYPH G022" },
    { 0x1315A, "EGYPTIAN HIEROGLYPH G023" },
    { 0x1315B, "EGYPTIAN HIEROGLYPH G024" },
    { 0x1315C, "EGYPTIAN HIEROGLYPH G025" },
    { 0x1315D, "EGYPTIAN HIEROGLYPH G026" },
    { 0x1315E, "EGYPTIAN HIEROGLYPH G026A" },
    { 0x1315F, "EGYPTIAN HIEROGLYPH G027" },
    { 0x13160, "EGYPTIAN HIEROGLYPH G028" },
    { 0x13161, "EGYPTIAN HIEROGLYPH G029" },
    { 0x13162, "EGYPTIAN HIEROGLYPH G030" },
    { 0x13163, "EGYPTIAN HIEROGLYPH G031" },
    { 0x13164, "EGYPTIAN HIEROGLYPH G032" },
    { 0x13165, "EGYPTIAN HIEROGLYPH G033" },
    { 0x13166, "EGYPTIAN HIEROGLYPH G034" },
    { 0x13167, "EGYPTIAN HIEROGLYPH G035" },
    { 0x13168, "EGYPTIAN HIEROGLYPH G036" },
    { 0x13169, "EGYPTIAN HIEROGLYPH G036A" },
    { 0x1316A, "EGYPTIAN HIEROGLYPH G037" },
    { 0x1316B, "EGYPTIAN HIEROGLYPH G037A" },
    { 0x1316C, "EGYPTIAN HIEROGLYPH G038" },
    { 0x1316D, "EGYPTIAN HIEROGLYPH G039" },
    { 0x1316E, "EGYPTIAN HIEROGLYPH G040" },
    { 0x1316F, "EGYPTIAN HIEROGLYPH G041" },
    { 0x13170, "EGYPTIAN HIEROGLYPH G042" },
    { 0x13171, "EGYPTIAN HIEROGLYPH G043" },
    { 0x13172, "EGYPTIAN HIEROGLYPH G043A" },
    { 0x13173, "EGYPTIAN HIEROGLYPH G044" },
    { 0x13174, "EGYPTIAN HIEROGLYPH G045" },
    { 0x13175, "EGYPTIAN HIEROGLYPH G045A" },
    { 0x13176, "EGYPTIAN HIEROGLYPH G046" },
    { 0x13177, "EGYPTIAN HIEROGLYPH G047" },
    { 0x13178, "EGYPTIAN HIEROGLYPH G048" },
    { 0x13179, "EGYPTIAN HIEROGLYPH G049" },
    { 0x1317A, "EGYPTIAN HIEROGLYPH G050" },
    { 0x1317B, "EGYPTIAN HIEROGLYPH G051" },
    { 0x1317C, "EGYPTIAN HIEROGLYPH G052" },
    { 0x1317D, "EGYPTIAN HIEROGLYPH G053" },
    { 0x1317E, "EGYPTIAN HIEROGLYPH G054" },
    { 0x1317F, "EGYPTIAN HIEROGLYPH H001" },
    { 0x13180, "EGYPTIAN HIEROGLYPH H002" },
    { 0x13181, "EGYPTIAN HIEROGLYPH H003" },
    { 0x13182, "EGYPTIAN HIEROGLYPH H004" },
    { 0x13183, "EGYPTIAN HIEROGLYPH H005" },
    { 0x13184, "EGYPTIAN HIEROGLYPH H006" },
    { 0x13185, "EGYPTIAN HIEROGLYPH H006A" },
    { 0x13186, "EGYPTIAN HIEROGLYPH H007" },
    { 0x13187, "EGYPTIAN HIEROGLYPH H008" },
    { 0x13188, "EGYPTIAN HIEROGLYPH I001" },
    { 0x13189, "EGYPTIAN HIEROGLYPH I002" },
    { 0x1318A, "EGYPTIAN HIEROGLYPH I003" },
    { 0x1318B, "EGYPTIAN HIEROGLYPH I004" },
    { 0x1318C, "EGYPTIAN HIEROGLYPH I005" },
    { 0x1318D, "EGYPTIAN HIEROGLYPH I005A" },
    { 0x1318E, "EGYPTIAN HIEROGLYPH I006" },
    { 0x1318F, "EGYPTIAN HIEROGLYPH I007" },
    { 0x13190, "EGYPTIAN HIEROGLYPH I008" },
    { 0x13191, "EGYPTIAN HIEROGLYPH I009" },
    { 0x13192, "EGYPTIAN HIEROGLYPH I009A" },
    { 0x13193, "EGYPTIAN HIEROGLYPH I010" },
    { 0x13194, "EGYPTIAN HIEROGLYPH I010A" },
    { 0x13195, "EGYPTIAN HIEROGLYPH I011" },
    { 0x13196, "EGYPTIAN HIEROGLYPH I011A" },
    { 0x13197, "EGYPTIAN HIEROGLYPH I012" },
    { 0x13198, "EGYPTIAN HIEROGLYPH I013" },
    { 0x13199, "EGYPTIAN HIEROGLYPH I014" },
    { 0x1319A, "EGYPTIAN HIEROGLYPH I015" },
    { 0x1319B, "EGYPTIAN HIEROGLYPH K001" },
    { 0x1319C, "EGYPTIAN HIEROGLYPH K002" },
    { 0x1319D, "EGYPTIAN HIEROGLYPH K003" },
    { 0x1319E, "EGYPTIAN HIEROGLYPH K004" },
    { 0x1319F, "EGYPTIAN HIEROGLYPH K005" },
    { 0x131A0, "EGYPTIAN HIEROGLYPH K006" },
    { 0x131A1, "EGYPTIAN HIEROGLYPH K007" },
    { 0x131A2, "EGYPTIAN HIEROGLYPH K008" },
    { 0x131A3, "EGYPTIAN HIEROGLYPH L001" },
    { 0x131A4, "EGYPTIAN HIEROGLYPH L002" },
    { 0x131A5, "EGYPTIAN HIEROGLYPH L002A" },
    { 0x131A6, "EGYPTIAN HIEROGLYPH L003" },
    { 0x131A7, "EGYPTIAN HIEROGLYPH L004" },
    { 0x131A8, "EGYPTIAN HIEROGLYPH L005" },
    { 0x131A9, "EGYPTIAN HIEROGLYPH L006" },
    { 0x131AA, "EGYPTIAN HIEROGLYPH L006A" },
    { 0x131AB, "EGYPTIAN HIEROGLYPH L007" },
    { 0x131AC, "EGYPTIAN HIEROGLYPH L008" },
    { 0x131AD, "EGYPTIAN HIEROGLYPH M001" },
    { 0x131AE, "EGYPTIAN HIEROGLYPH M001A" },
    { 0x131AF, "EGYPTIAN HIEROGLYPH M001B" },
    { 0x131B0, "EGYPTIAN HIEROGLYPH M002" },
    { 0x131B1, "EGYPTIAN HIEROGLYPH M003" },
    { 0x131B2, "EGYPTIAN HIEROGLYPH M003A" },
    { 0x131B3, "EGYPTIAN HIEROGLYPH M004" },
    { 0x131B4, "EGYPTIAN HIEROGLYPH M005" },
    { 0x131B5, "EGYPTIAN HIEROGLYPH M006" },
    { 0x131B6, "EGYPTIAN HIEROGLYPH M007" },
    { 0x131B7, "EGYPTIAN HIEROGLYPH M008" },
    { 0x131B8, "EGYPTIAN HIEROGLYPH M009" },
    { 0x131B9, "EGYPTIAN HIEROGLYPH M010" },
    { 0x131BA, "EGYPTIAN HIEROGLYPH M010A" },
    { 0x131BB, "EGYPTIAN HIEROGLYPH M011" },
    { 0x131BC, "EGYPTIAN HIEROGLYPH M012" },
    { 0x131BD, "EGYPTIAN HIEROGLYPH M012A" },
    { 0x131BE, "EGYPTIAN HIEROGLYPH M012B" },
    { 0x131BF, "EGYPTIAN HIEROGLYPH M012C" },
    { 0x131C0, "EGYPTIAN HIEROGLYPH M012D" },
    { 0x131C1, "EGYPTIAN HIEROGLYPH M012E" },
    { 0x131C2, "EGYPTIAN HIEROGLYPH M012F" },
    { 0x131C3, "EGYPTIAN HIEROGLYPH M012G" },
    { 0x131C4, "EGYPTIAN HIEROGLYPH M012H" },
    { 0x131C5, "EGYPTIAN HIEROGLYPH M013" },
    { 0x131C6, "EGYPTIAN HIEROGLYPH M014" },
    { 0x131C7, "EGYPTIAN HIEROGLYPH M015" },
    { 0x131C8, "EGYPTIAN HIEROGLYPH M015A" },
    { 0x131C9, "EGYPTIAN HIEROGLYPH M016" },
    { 0x131CA, "EGYPTIAN HIEROGLYPH M016A" },
    { 0x131CB, "EGYPTIAN HIEROGLYPH M017" },
    { 0x131CC, "EGYPTIAN HIEROGLYPH M017A" },
    { 0x131CD, "EGYPTIAN HIEROGLYPH M018" },
    { 0x131CE, "EGYPTIAN HIEROGLYPH M019" },
    { 0x131CF, "EGYPTIAN HIEROGLYPH M020" },
    { 0x131D0, "EGYPTIAN HIEROGLYPH M021" },
    { 0x131D1, "EGYPTIAN HIEROGLYPH M022" },
    { 0x131D2, "EGYPTIAN HIEROGLYPH M022A" },
    { 0x131D3, "EGYPTIAN HIEROGLYPH M023" },
    { 0x131D4, "EGYPTIAN HIEROGLYPH M024" },
    { 0x131D5, "EGYPTIAN HIEROGLYPH M024A" },
    { 0x131D6, "EGYPTIAN HIEROGLYPH M025" },
    { 0x131D7, "EGYPTIAN HIEROGLYPH M026" },
    { 0x131D8, "EGYPTIAN HIEROGLYPH M027" },
    { 0x131D9, "EGYPTIAN HIEROGLYPH M028" },
    { 0x131DA, "EGYPTIAN HIEROGLYPH M028A" },
    { 0x131DB, "EGYPTIAN HIEROGLYPH M029" },
    { 0x131DC, "EGYPTIAN HIEROGLYPH M030" },
    { 0x131DD, "EGYPTIAN HIEROGLYPH M031" },
    { 0x131DE, "EGYPTIAN HIEROGLYPH M031A" },
    { 0x131DF, "EGYPTIAN HIEROGLYPH M032" },
    { 0x131E0, "EGYPTIAN HIEROGLYPH M033" },
    { 0x131E1, "EGYPTIAN HIEROGLYPH M033A" },
    { 0x131E2, "EGYPTIAN HIEROGLYPH M033B" },
    { 0x131E3, "EGYPTIAN HIEROGLYPH M034" },
    { 0x131E4, "EGYPTIAN HIEROGLYPH M035" },
    { 0x131E5, "EGYPTIAN HIEROGLYPH M036" },
    { 0x131E6, "EGYPTIAN HIEROGLYPH M037" },
    { 0x131E7, "EGYPTIAN HIEROGLYPH M038" },
    { 0x131E8, "EGYPTIAN HIEROGLYPH M039" },
    { 0x131E9, "EGYPTIAN HIEROGLYPH M040" },
    { 0x131EA, "EGYPTIAN HIEROGLYPH M040A" },
    { 0x131EB, "EGYPTIAN HIEROGLYPH M041" },
    { 0x131EC, "EGYPTIAN HIEROGLYPH M042" },
    { 0x131ED, "EGYPTIAN HIEROGLYPH M043" },
    { 0x131EE, "EGYPTIAN HIEROGLYPH M044" },
    { 0x131EF, "EGYPTIAN HIEROGLYPH N001" },
    { 0x131F0, "EGYPTIAN HIEROGLYPH N002" },
    { 0x131F1, "EGYPTIAN HIEROGLYPH N003" },
    { 0x131F2, "EGYPTIAN HIEROGLYPH N004" },
    { 0x131F3, "EGYPTIAN HIEROGLYPH N005" },
    { 0x131F4, "EGYPTIAN HIEROGLYPH N006" },
    { 0x131F5, "EGYPTIAN HIEROGLYPH N007" },
    { 0x131F6, "EGYPTIAN HIEROGLYPH N008" },
    { 0x131F7, "EGYPTIAN HIEROGLYPH N009" },
    { 0x131F8, "EGYPTIAN HIEROGLYPH N010" },
    { 0x131F9, "EGYPTIAN HIEROGLYPH N011" },
    { 0x131FA, "EGYPTIAN HIEROGLYPH N012" },
    { 0x131FB, "EGYPTIAN HIEROGLYPH N013" },
    { 0x131FC, "EGYPTIAN HIEROGLYPH N014" },
    { 0x131FD, "EGYPTIAN HIEROGLYPH N015" },
    { 0x131FE, "EGYPTIAN HIEROGLYPH N016" },
    { 0x131FF, "EGYPTIAN HIEROGLYPH N017" },
    { 0x13200, "EGYPTIAN HIEROGLYPH N018" },
    { 0x13201, "EGYPTIAN HIEROGLYPH N018A" },
    { 0x13202, "EGYPTIAN HIEROGLYPH N018B" },
    { 0x13203, "EGYPTIAN HIEROGLYPH N019" },
    { 0x13204, "EGYPTIAN HIEROGLYPH N020" },
    { 0x13205, "EGYPTIAN HIEROGLYPH N021" },
    { 0x13206, "EGYPTIAN HIEROGLYPH N022" },
    { 0x13207, "EGYPTIAN HIEROGLYPH N023" },
    { 0x13208, "EGYPTIAN HIEROGLYPH N024" },
    { 0x13209, "EGYPTIAN HIEROGLYPH N025" },
    { 0x1320A, "EGYPTIAN HIEROGLYPH N025A" },
    { 0x1320B, "EGYPTIAN HIEROGLYPH N026" },
    { 0x1320C, "EGYPTIAN HIEROGLYPH N027" },
    { 0x1320D, "EGYPTIAN HIEROGLYPH N028" },
    { 0x1320E, "EGYPTIAN HIEROGLYPH N029" },
    { 0x1320F, "EGYPTIAN HIEROGLYPH N030" },
    { 0x13210, "EGYPTIAN HIEROGLYPH N031" },
    { 0x13211, "EGYPTIAN HIEROGLYPH N032" },
    { 0x13212, "EGYPTIAN HIEROGLYPH N033" },
    { 0x13213, "EGYPTIAN HIEROGLYPH N033A" },
    { 0x13214, "EGYPTIAN HIEROGLYPH N034" },
    { 0x13215, "EGYPTIAN HIEROGLYPH N034A" },
    { 0x13216, "EGYPTIAN HIEROGLYPH N035" },
    { 0x13217, "EGYPTIAN HIEROGLYPH N035A" },
    { 0x13218, "EGYPTIAN HIEROGLYPH N036" },
    { 0x13219, "EGYPTIAN HIEROGLYPH N037" },
    { 0x1321A, "EGYPTIAN HIEROGLYPH N037A" },
    { 0x1321B, "EGYPTIAN HIEROGLYPH N038" },
    { 0x1321C, "EGYPTIAN HIEROGLYPH N039" },
    { 0x1321D, "EGYPTIAN HIEROGLYPH N040" },
    { 0x1321E, "EGYPTIAN HIEROGLYPH N041" },
    { 0x1321F, "EGYPTIAN HIEROGLYPH N042" },
    { 0x13220, "EGYPTIAN HIEROGLYPH NL001" },
    { 0x13221, "EGYPTIAN HIEROGLYPH NL002" },
    { 0x13222, "EGYPTIAN HIEROGLYPH NL003" },
    { 0x13223, "EGYPTIAN HIEROGLYPH NL004" },
    { 0x13224, "EGYPTIAN HIEROGLYPH NL005" },
    { 0x13225, "EGYPTIAN HIEROGLYPH NL005A" },
    { 0x13226, "EGYPTIAN HIEROGLYPH NL006" },
    { 0x13227, "EGYPTIAN HIEROGLYPH NL007" },
    { 0x13228, "EGYPTIAN HIEROGLYPH NL008" },
    { 0x13229, "EGYPTIAN HIEROGLYPH NL009" },
    { 0x1322A, "EGYPTIAN HIEROGLYPH NL010" },
    { 0x1322B, "EGYPTIAN HIEROGLYPH NL011" },
    { 0x1322C, "EGYPTIAN HIEROGLYPH NL012" },
    { 0x1322D, "EGYPTIAN HIEROGLYPH NL013" },
    { 0x1322E, "EGYPTIAN HIEROGLYPH NL014" },
    { 0x1322F, "EGYPTIAN HIEROGLYPH NL015" },
    { 0x13230, "EGYPTIAN HIEROGLYPH NL016" },
    { 0x13231, "EGYPTIAN HIEROGLYPH NL017" },
    { 0x13232, "EGYPTIAN HIEROGLYPH NL017A" },
    { 0x13233, "EGYPTIAN HIEROGLYPH NL018" },
    { 0x13234, "EGYPTIAN HIEROGLYPH NL019" },
    { 0x13235, "EGYPTIAN HIEROGLYPH NL020" },
    { 0x13236, "EGYPTIAN HIEROGLYPH NU001" },
    { 0x13237, "EGYPTIAN HIEROGLYPH NU002" },
    { 0x13238, "EGYPTIAN HIEROGLYPH NU003" },
    { 0x13239, "EGYPTIAN HIEROGLYPH NU004" },
    { 0x1323A, "EGYPTIAN HIEROGLYPH NU005" },
    { 0x1323B, "EGYPTIAN HIEROGLYPH NU006" },
    { 0x1323C, "EGYPTIAN HIEROGLYPH NU007" },
    { 0x1323D, "EGYPTIAN HIEROGLYPH NU008" },
    { 0x1323E, "EGYPTIAN HIEROGLYPH NU009" },
    { 0x1323F, "EGYPTIAN HIEROGLYPH NU010" },
    { 0x13240, "EGYPTIAN HIEROGLYPH NU010A" },
    { 0x13241, "EGYPTIAN HIEROGLYPH NU011" },
    { 0x13242, "EGYPTIAN HIEROGLYPH NU011A" },
    { 0x13243, "EGYPTIAN HIEROGLYPH NU012" },
    { 0x13244, "EGYPTIAN HIEROGLYPH NU013" },
    { 0x13245, "EGYPTIAN HIEROGLYPH NU014" },
    { 0x13246, "EGYPTIAN HIEROGLYPH NU015" },
    { 0x13247, "EGYPTIAN HIEROGLYPH NU016" },
    { 0x13248, "EGYPTIAN HIEROGLYPH NU017" },
    { 0x13249, "EGYPTIAN HIEROGLYPH NU018" },
    { 0x1324A, "EGYPTIAN HIEROGLYPH NU018A" },
    { 0x1324B, "EGYPTIAN HIEROGLYPH NU019" },
    { 0x1324C, "EGYPTIAN HIEROGLYPH NU020" },
    { 0x1324D, "EGYPTIAN HIEROGLYPH NU021" },
    { 0x1324E, "EGYPTIAN HIEROGLYPH NU022" },
    { 0x1324F, "EGYPTIAN HIEROGLYPH NU022A" },
    { 0x13250, "EGYPTIAN HIEROGLYPH O001" },
    { 0x13251, "EGYPTIAN HIEROGLYPH O001A" },
    { 0x13252, "EGYPTIAN HIEROGLYPH O002" },
    { 0x13253, "EGYPTIAN HIEROGLYPH O003" },
    { 0x13254, "EGYPTIAN HIEROGLYPH O004" },
    { 0x13255, "EGYPTIAN HIEROGLYPH O005" },
    { 0x13256, "EGYPTIAN HIEROGLYPH O005A" },
    { 0x13257, "EGYPTIAN HIEROGLYPH O006" },
    { 0x13258, "EGYPTIAN HIEROGLYPH O006A" },
    { 0x13259, "EGYPTIAN HIEROGLYPH O006B" },
    { 0x1325A, "EGYPTIAN HIEROGLYPH O006C" },
    { 0x1325B, "EGYPTIAN HIEROGLYPH O006D" },
    { 0x1325C, "EGYPTIAN HIEROGLYPH O006E" },
    { 0x1325D, "EGYPTIAN HIEROGLYPH O006F" },
    { 0x1325E, "EGYPTIAN HIEROGLYPH O007" },
    { 0x1325F, "EGYPTIAN HIEROGLYPH O008" },
    { 0x13260, "EGYPTIAN HIEROGLYPH O009" },
    { 0x13261, "EGYPTIAN HIEROGLYPH O010" },
    { 0x13262, "EGYPTIAN HIEROGLYPH O010A" },
    { 0x13263, "EGYPTIAN HIEROGLYPH O010B" },
    { 0x13264, "EGYPTIAN HIEROGLYPH O010C" },
    { 0x13265, "EGYPTIAN HIEROGLYPH O011" },
    { 0x13266, "EGYPTIAN HIEROGLYPH O012" },
    { 0x13267, "EGYPTIAN HIEROGLYPH O013" },
    { 0x13268, "EGYPTIAN HIEROGLYPH O014" },
    { 0x13269, "EGYPTIAN HIEROGLYPH O015" },
    { 0x1326A, "EGYPTIAN HIEROGLYPH O016" },
    { 0x1326B, "EGYPTIAN HIEROGLYPH O017" },
    { 0x1326C, "EGYPTIAN HIEROGLYPH O018" },
    { 0x1326D, "EGYPTIAN HIEROGLYPH O019" },
    { 0x1326E, "EGYPTIAN HIEROGLYPH O019A" },
    { 0x1326F, "EGYPTIAN HIEROGLYPH O020" },
    { 0x13270, "EGYPTIAN HIEROGLYPH O020A" },
    { 0x13271, "EGYPTIAN HIEROGLYPH O021" },
    { 0x13272, "EGYPTIAN HIEROGLYPH O022" },
    { 0x13273, "EGYPTIAN HIEROGLYPH O023" },
    { 0x13274, "EGYPTIAN HIEROGLYPH O024" },
    { 0x13275, "EGYPTIAN HIEROGLYPH O024A" },
    { 0x13276, "EGYPTIAN HIEROGLYPH O025" },
    { 0x13277, "EGYPTIAN HIEROGLYPH O025A" },
    { 0x13278, "EGYPTIAN HIEROGLYPH O026" },
    { 0x13279, "EGYPTIAN HIEROGLYPH O027" },
    { 0x1327A, "EGYPTIAN HIEROGLYPH O028" },
    { 0x1327B, "EGYPTIAN HIEROGLYPH O029" },
    { 0x1327C, "EGYPTIAN HIEROGLYPH O029A" },
    { 0x1327D, "EGYPTIAN HIEROGLYPH O030" },
    { 0x1327E, "EGYPTIAN HIEROGLYPH O030A" },
    { 0x1327F, "EGYPTIAN HIEROGLYPH O031" },
    { 0x13280, "EGYPTIAN HIEROGLYPH O032" },
    { 0x13281, "EGYPTIAN HIEROGLYPH O033" },
    { 0x13282, "EGYPTIAN HIEROGLYPH O033A" },
    { 0x13283, "EGYPTIAN HIEROGLYPH O034" },
    { 0x13284, "EGYPTIAN HIEROGLYPH O035" },
    { 0x13285, "EGYPTIAN HIEROGLYPH O036" },
    { 0x13286, "EGYPTIAN HIEROGLYPH O036A" },
    { 0x13287, "EGYPTIAN HIEROGLYPH O036B" },
    { 0x13288, "EGYPTIAN HIEROGLYPH O036C" },
    { 0x13289, "EGYPTIAN HIEROGLYPH O036D" },
    { 0x1328A, "EGYPTIAN HIEROGLYPH O037" },
    { 0x1328B, "EGYPTIAN HIEROGLYPH O038" },
    { 0x1328C, "EGYPTIAN HIEROGLYPH O039" },
    { 0x1328D, "EGYPTIAN HIEROGLYPH O040" },
    { 0x1328E, "EGYPTIAN HIEROGLYPH O041" },
    { 0x1328F, "EGYPTIAN HIEROGLYPH O042" },
    { 0x13290, "EGYPTIAN HIEROGLYPH O043" },
    { 0x13291, "EGYPTIAN HIEROGLYPH O044" },
    { 0x13292, "EGYPTIAN HIEROGLYPH O045" },
    { 0x13293, "EGYPTIAN HIEROGLYPH O046" },
    { 0x13294, "EGYPTIAN HIEROGLYPH O047" },
    { 0x13295, "EGYPTIAN HIEROGLYPH O048" },
    { 0x13296, "EGYPTIAN HIEROGLYPH O049" },
    { 0x13297, "EGYPTIAN HIEROGLYPH O050" },
    { 0x13298, "EGYPTIAN HIEROGLYPH O050A" },
    { 0x13299, "EGYPTIAN HIEROGLYPH O050B" },
    { 0x1329A, "EGYPTIAN HIEROGLYPH O051" },
    { 0x1329B, "EGYPTIAN HIEROGLYPH P001" },
    { 0x1329C, "EGYPTIAN HIEROGLYPH P001A" },
    { 0x1329D, "EGYPTIAN HIEROGLYPH P002" },
    { 0x1329E, "EGYPTIAN HIEROGLYPH P003" },
    { 0x1329F, "EGYPTIAN HIEROGLYPH P003A" },
    { 0x132A0, "EGYPTIAN HIEROGLYPH P004" },
    { 0x132A1, "EGYPTIAN HIEROGLYPH P005" },
    { 0x132A2, "EGYPTIAN HIEROGLYPH P006" },
    { 0x132A3, "EGYPTIAN HIEROGLYPH P007" },
    { 0x132A4, "EGYPTIAN HIEROGLYPH P008" },
    { 0x132A5, "EGYPTIAN HIEROGLYPH P009" },
    { 0x132A6, "EGYPTIAN HIEROGLYPH P010" },
    { 0x132A7, "EGYPTIAN HIEROGLYPH P011" },
    { 0x132A8, "EGYPTIAN HIEROGLYPH Q001" },
    { 0x132A9, "EGYPTIAN HIEROGLYPH Q002" },
    { 0x132AA, "EGYPTIAN HIEROGLYPH Q003" },
    { 0x132AB, "EGYPTIAN HIEROGLYPH Q004" },
    { 0x132AC, "EGYPTIAN HIEROGLYPH Q005" },
    { 0x132AD, "EGYPTIAN HIEROGLYPH Q006" },
    { 0x132AE, "EGYPTIAN HIEROGLYPH Q007" },
    { 0x132AF, "EGYPTIAN HIEROGLYPH R001" },
    { 0x132B0, "EGYPTIAN HIEROGLYPH R002" },
    { 0x132B1, "EGYPTIAN HIEROGLYPH R002A" },
    { 0x132B2, "EGYPTIAN HIEROGLYPH R003" },
    { 0x132B3, "EGYPTIAN HIEROGLYPH R003A" },
    { 0x132B4, "EGYPTIAN HIEROGLYPH R003B" },
    { 0x132B5, "EGYPTIAN HIEROGLYPH R004" },
    { 0x132B6, "EGYPTIAN HIEROGLYPH R005" },
    { 0x132B7, "EGYPTIAN HIEROGLYPH R006" },
    { 0x132B8, "EGYPTIAN HIEROGLYPH R007" },
    { 0x132B9, "EGYPTIAN HIEROGLYPH R008" },
    { 0x132BA, "EGYPTIAN HIEROGLYPH R009" },
    { 0x132BB, "EGYPTIAN HIEROGLYPH R010" },
    { 0x132BC, "EGYPTIAN HIEROGLYPH R010A" },
    { 0x132BD, "EGYPTIAN HIEROGLYPH R011" },
    { 0x132BE, "EGYPTIAN HIEROGLYPH R012" },
    { 0x132BF, "EGYPTIAN HIEROGLYPH R013" },
    { 0x132C0, "EGYPTIAN HIEROGLYPH R014" },
    { 0x132C1, "EGYPTIAN HIEROGLYPH R015" },
    { 0x132C2, "EGYPTIAN HIEROGLYPH R016" },
    { 0x132C3, "EGYPTIAN HIEROGLYPH R016A" },
    { 0x132C4, "EGYPTIAN HIEROGLYPH R017" },
    { 0x132C5, "EGYPTIAN HIEROGLYPH R018" },
    { 0x132C6, "EGYPTIAN HIEROGLYPH R019" },
    { 0x132C7, "EGYPTIAN HIEROGLYPH R020" },
    { 0x132C8, "EGYPTIAN HIEROGLYPH R021" },
    { 0x132C9, "EGYPTIAN HIEROGLYPH R022" },
    { 0x132CA, "EGYPTIAN HIEROGLYPH R023" },
    { 0x132CB, "EGYPTIAN HIEROGLYPH R024" },
    { 0x132CC, "EGYPTIAN HIEROGLYPH R025" },
    { 0x132CD, "EGYPTIAN HIEROGLYPH R026" },
    { 0x132CE, "EGYPTIAN HIEROGLYPH R027" },
    { 0x132CF, "EGYPTIAN HIEROGLYPH R028" },
    { 0x132D0, "EGYPTIAN HIEROGLYPH R029" },
    { 0x132D1, "EGYPTIAN HIEROGLYPH S001" },
    { 0x132D2, "EGYPTIAN HIEROGLYPH S002" },
    { 0x132D3, "EGYPTIAN HIEROGLYPH S002A" },
    { 0x132D4, "EGYPTIAN HIEROGLYPH S003" },
    { 0x132D5, "EGYPTIAN HIEROGLYPH S004" },
    { 0x132D6, "EGYPTIAN HIEROGLYPH S005" },
    { 0x132D7, "EGYPTIAN HIEROGLYPH S006" },
    { 0x132D8, "EGYPTIAN HIEROGLYPH S006A" },
    { 0x132D9, "EGYPTIAN HIEROGLYPH S007" },
    { 0x132DA, "EGYPTIAN HIEROGLYPH S008" },
    { 0x132DB, "EGYPTIAN HIEROGLYPH S009" },
    { 0x132DC, "EGYPTIAN HIEROGLYPH S010" },
    { 0x132DD, "EGYPTIAN HIEROGLYPH S011" },
    { 0x132DE, "EGYPTIAN HIEROGLYPH S012" },
    { 0x132DF, "EGYPTIAN HIEROGLYPH S013" },
    { 0x132E0, "EGYPTIAN HIEROGLYPH S014" },
    { 0x132E1, "EGYPTIAN HIEROGLYPH S014A" },
    { 0x132E2, "EGYPTIAN HIEROGLYPH S014B" },
    { 0x132E3, "EGYPTIAN HIEROGLYPH S015" },
    { 0x132E4, "EGYPTIAN HIEROGLYPH S016" },
    { 0x132E5, "EGYPTIAN HIEROGLYPH S017" },
    { 0x132E6, "EGYPTIAN HIEROGLYPH S017A" },
    { 0x132E7, "EGYPTIAN HIEROGLYPH S018" },
    { 0x132E8, "EGYPTIAN HIEROGLYPH S019" },
    { 0x132E9, "EGYPTIAN HIEROGLYPH S020" },
    { 0x132EA, "EGYPTIAN HIEROGLYPH S021" },
    { 0x132EB, "EGYPTIAN HIEROGLYPH S022" },
    { 0x132EC, "EGYPTIAN HIEROGLYPH S023" },
    { 0x132ED, "EGYPTIAN HIEROGLYPH S024" },
    { 0x132EE, "EGYPTIAN HIEROGLYPH S025" },
    { 0x132EF, "EGYPTIAN HIEROGLYPH S026" },
    { 0x132F0, "EGYPTIAN HIEROGLYPH S026A" },
    { 0x132F1, "EGYPTIAN HIEROGLYPH S026B" },
    { 0x132F2, "EGYPTIAN HIEROGLYPH S027" },
    { 0x132F3, "EGYPTIAN HIEROGLYPH S028" },
    { 0x132F4, "EGYPTIAN HIEROGLYPH S029" },
    { 0x132F5, "EGYPTIAN HIEROGLYPH S030" },
    { 0x132F6, "EGYPTIAN HIEROGLYPH S031" },
    { 0x132F7, "EGYPTIAN HIEROGLYPH S032" },
    { 0x132F8, "EGYPTIAN HIEROGLYPH S033" },
    { 0x132F9, "EGYPTIAN HIEROGLYPH S034" },
    { 0x132FA, "EGYPTIAN HIEROGLYPH S035" },
    { 0x132FB, "EGYPTIAN HIEROGLYPH S035A" },
    { 0x132FC, "EGYPTIAN HIEROGLYPH S036" },
    { 0x132FD, "EGYPTIAN HIEROGLYPH S037" },
    { 0x132FE, "EGYPTIAN HIEROGLYPH S038" },
    { 0x132FF, "EGYPTIAN HIEROGLYPH S039" },
    { 0x13300, "EGYPTIAN HIEROGLYPH S040" },
    { 0x13301, "EGYPTIAN HIEROGLYPH S041" },
    { 0x13302, "EGYPTIAN HIEROGLYPH S042" },
    { 0x13303, "EGYPTIAN HIEROGLYPH S043" },
    { 0x13304, "EGYPTIAN HIEROGLYPH S044" },
    { 0x13305, "EGYPTIAN HIEROGLYPH S045" },
    { 0x13306, "EGYPTIAN HIEROGLYPH S046" },
    { 0x13307, "EGYPTIAN HIEROGLYPH T001" },
    { 0x13308, "EGYPTIAN HIEROGLYPH T002" },
    { 0x13309, "EGYPTIAN HIEROGLYPH T003" },
    { 0x1330A, "EGYPTIAN HIEROGLYPH T003A" },
    { 0x1330B, "EGYPTIAN HIEROGLYPH T004" },
    { 0x1330C, "EGYPTIAN HIEROGLYPH T005" },
    { 0x1330D, "EGYPTIAN HIEROGLYPH T006" },
    { 0x1330E, "EGYPTIAN HIEROGLYPH T007" },
    { 0x1330F, "EGYPTIAN HIEROGLYPH T007A" },
    { 0x13310, "EGYPTIAN HIEROGLYPH T008" },
    { 0x13311, "EGYPTIAN HIEROGLYPH T008A" },
    { 0x13312, "EGYPTIAN HIEROGLYPH T009" },
    { 0x13313, "EGYPTIAN HIEROGLYPH T009A" },
    { 0x13314, "EGYPTIAN HIEROGLYPH T010" },
    { 0x13315, "EGYPTIAN HIEROGLYPH T011" },
    { 0x13316, "EGYPTIAN HIEROGLYPH T011A" },
    { 0x13317, "EGYPTIAN HIEROGLYPH T012" },
    { 0x13318, "EGYPTIAN HIEROGLYPH T013" },
    { 0x13319, "EGYPTIAN HIEROGLYPH T014" },
    { 0x1331A, "EGYPTIAN HIEROGLYPH T015" },
    { 0x1331B, "EGYPTIAN HIEROGLYPH T016" },
    { 0x1331C, "EGYPTIAN HIEROGLYPH T016A" },
    { 0x1331D, "EGYPTIAN HIEROGLYPH T017" },
    { 0x1331E, "EGYPTIAN HIEROGLYPH T018" },
    { 0x1331F, "EGYPTIAN HIEROGLYPH T019" },
    { 0x13320, "EGYPTIAN HIEROGLYPH T020" },
    { 0x13321, "EGYPTIAN HIEROGLYPH T021" },
    { 0x13322, "EGYPTIAN HIEROGLYPH T022" },
    { 0x13323, "EGYPTIAN HIEROGLYPH T023" },
    { 0x13324, "EGYPTIAN HIEROGLYPH T024" },
    { 0x13325, "EGYPTIAN HIEROGLYPH T025" },
    { 0x13326, "EGYPTIAN HIEROGLYPH T026" },
    { 0x13327, "EGYPTIAN HIEROGLYPH T027" },
    { 0x13328, "EGYPTIAN HIEROGLYPH T028" },
    { 0x13329, "EGYPTIAN HIEROGLYPH T029" },
    { 0x1332A, "EGYPTIAN HIEROGLYPH T030" },
    { 0x1332B, "EGYPTIAN HIEROGLYPH T031" },
    { 0x1332C, "EGYPTIAN HIEROGLYPH T032" },
    { 0x1332D, "EGYPTIAN HIEROGLYPH T032A" },
    { 0x1332E, "EGYPTIAN HIEROGLYPH T033" },
    { 0x1332F, "EGYPTIAN HIEROGLYPH T033A" },
    { 0x13330, "EGYPTIAN HIEROGLYPH T034" },
    { 0x13331, "EGYPTIAN HIEROGLYPH T035" },
    { 0x13332, "EGYPTIAN HIEROGLYPH T036" },
    { 0x13333, "EGYPTIAN HIEROGLYPH U001" },
    { 0x13334, "EGYPTIAN HIEROGLYPH U002" },
    { 0x13335, "EGYPTIAN HIEROGLYPH U003" },
    { 0x13336, "EGYPTIAN HIEROGLYPH U004" },
    { 0x13337, "EGYPTIAN HIEROGLYPH U005" },
    { 0x13338, "EGYPTIAN HIEROGLYPH U006" },
    { 0x13339, "EGYPTIAN HIEROGLYPH U006A" },
    { 0x1333A, "EGYPTIAN HIEROGLYPH U006B" },
    { 0x1333B, "EGYPTIAN HIEROGLYPH U007" },
    { 0x1333C, "EGYPTIAN HIEROGLYPH U008" },
    { 0x1333D, "EGYPTIAN HIEROGLYPH U009" },
    { 0x1333E, "EGYPTIAN HIEROGLYPH U010" },
    { 0x1333F, "EGYPTIAN HIEROGLYPH U011" },
    { 0x13340, "EGYPTIAN HIEROGLYPH U012" },
    { 0x13341, "EGYPTIAN HIEROGLYPH U013" },
    { 0x13342, "EGYPTIAN HIEROGLYPH U014" },
    { 0x13343, "EGYPTIAN HIEROGLYPH U015" },
    { 0x13344, "EGYPTIAN HIEROGLYPH U016" },
    { 0x13345, "EGYPTIAN HIEROGLYPH U017" },
    { 0x13346, "EGYPTIAN HIEROGLYPH U018" },
    { 0x13347, "EGYPTIAN HIEROGLYPH U019" },
    { 0x13348, "EGYPTIAN HIEROGLYPH U020" },
    { 0x13349, "EGYPTIAN HIEROGLYPH U021" },
    { 0x1334A, "EGYPTIAN HIEROGLYPH U022" },
    { 0x1334B, "EGYPTIAN HIEROGLYPH U023" },
    { 0x1334C, "EGYPTIAN HIEROGLYPH U023A" },
    { 0x1334D, "EGYPTIAN HIEROGLYPH U024" },
    { 0x1334E, "EGYPTIAN HIEROGLYPH U025" },
    { 0x1334F, "EGYPTIAN HIEROGLYPH U026" },
    { 0x13350, "EGYPTIAN HIEROGLYPH U027" },
    { 0x13351, "EGYPTIAN HIEROGLYPH U028" },
    { 0x13352, "EGYPTIAN HIEROGLYPH U029" },
    { 0x13353, "EGYPTIAN HIEROGLYPH U029A" },
    { 0x13354, "EGYPTIAN HIEROGLYPH U030" },
    { 0x13355, "EGYPTIAN HIEROGLYPH U031" },
    { 0x13356, "EGYPTIAN HIEROGLYPH U032" },
    { 0x13357, "EGYPTIAN HIEROGLYPH U032A" },
    { 0x13358, "EGYPTIAN HIEROGLYPH U033" },
    { 0x13359, "EGYPTIAN HIEROGLYPH U034" },
    { 0x1335A, "EGYPTIAN HIEROGLYPH U035" },
    { 0x1335B, "EGYPTIAN HIEROGLYPH U036" },
    { 0x1335C, "EGYPTIAN HIEROGLYPH U037" },
    { 0x1335D, "EGYPTIAN HIEROGLYPH U038" },
    { 0x1335E, "EGYPTIAN HIEROGLYPH U039" },
    { 0x1335F, "EGYPTIAN HIEROGLYPH U040" },
    { 0x13360, "EGYPTIAN HIEROGLYPH U041" },
    { 0x13361, "EGYPTIAN HIEROGLYPH U042" },
    { 0x13362, "EGYPTIAN HIEROGLYPH V001" },
    { 0x13363, "EGYPTIAN HIEROGLYPH V001A" },
    { 0x13364, "EGYPTIAN HIEROGLYPH V001B" },
    { 0x13365, "EGYPTIAN HIEROGLYPH V001C" },
    { 0x13366, "EGYPTIAN HIEROGLYPH V001D" },
    { 0x13367, "EGYPTIAN HIEROGLYPH V001E" },
    { 0x13368, "EGYPTIAN HIEROGLYPH V001F" },
    { 0x13369, "EGYPTIAN HIEROGLYPH V001G" },
    { 0x1336A, "EGYPTIAN HIEROGLYPH V001H" },
    { 0x1336B, "EGYPTIAN HIEROGLYPH V001I" },
    { 0x1336C, "EGYPTIAN HIEROGLYPH V002" },
    { 0x1336D, "EGYPTIAN HIEROGLYPH V002A" },
    { 0x1336E, "EGYPTIAN HIEROGLYPH V003" },
    { 0x1336F, "EGYPTIAN HIEROGLYPH V004" },
    { 0x13370, "EGYPTIAN HIEROGLYPH V005" },
    { 0x13371, "EGYPTIAN HIEROGLYPH V006" },
    { 0x13372, "EGYPTIAN HIEROGLYPH V007" },
    { 0x13373, "EGYPTIAN HIEROGLYPH V007A" },
    { 0x13374, "EGYPTIAN HIEROGLYPH V007B" },
    { 0x13375, "EGYPTIAN HIEROGLYPH V008" },
    { 0x13376, "EGYPTIAN HIEROGLYPH V009" },
    { 0x13377, "EGYPTIAN HIEROGLYPH V010" },
    { 0x13378, "EGYPTIAN HIEROGLYPH V011" },
    { 0x13379, "EGYPTIAN HIEROGLYPH V011A" },
    { 0x1337A, "EGYPTIAN HIEROGLYPH V011B" },
    { 0x1337B, "EGYPTIAN HIEROGLYPH V011C" },
    { 0x1337C, "EGYPTIAN HIEROGLYPH V012" },
    { 0x1337D, "EGYPTIAN HIEROGLYPH V012A" },
    { 0x1337E, "EGYPTIAN HIEROGLYPH V012B" },
    { 0x1337F, "EGYPTIAN HIEROGLYPH V013" },
    { 0x13380, "EGYPTIAN HIEROGLYPH V014" },
    { 0x13381, "EGYPTIAN HIEROGLYPH V015" },
    { 0x13382, "EGYPTIAN HIEROGLYPH V016" },
    { 0x13383, "EGYPTIAN HIEROGLYPH V017" },
    { 0x13384, "EGYPTIAN HIEROGLYPH V018" },
    { 0x13385, "EGYPTIAN HIEROGLYPH V019" },
    { 0x13386, "EGYPTIAN HIEROGLYPH V020" },
    { 0x13387, "EGYPTIAN HIEROGLYPH V020A" },
    { 0x13388, "EGYPTIAN HIEROGLYPH V020B" },
    { 0x13389, "EGYPTIAN HIEROGLYPH V020C" },
    { 0x1338A, "EGYPTIAN HIEROGLYPH V020D" },
    { 0x1338B, "EGYPTIAN HIEROGLYPH V020E" },
    { 0x1338C, "EGYPTIAN HIEROGLYPH V020F" },
    { 0x1338D, "EGYPTIAN HIEROGLYPH V020G" },
    { 0x1338E, "EGYPTIAN HIEROGLYPH V020H" },
    { 0x1338F, "EGYPTIAN HIEROGLYPH V020I" },
    { 0x13390, "EGYPTIAN HIEROGLYPH V020J" },
    { 0x13391, "EGYPTIAN HIEROGLYPH V020K" },
    { 0x13392, "EGYPTIAN HIEROGLYPH V020L" },
    { 0x13393, "EGYPTIAN HIEROGLYPH V021" },
    { 0x13394, "EGYPTIAN HIEROGLYPH V022" },
    { 0x13395, "EGYPTIAN HIEROGLYPH V023" },
    { 0x13396, "EGYPTIAN HIEROGLYPH V023A" },
    { 0x13397, "EGYPTIAN HIEROGLYPH V024" },
    { 0x13398, "EGYPTIAN HIEROGLYPH V025" },
    { 0x13399, "EGYPTIAN HIEROGLYPH V026" },
    { 0x1339A, "EGYPTIAN HIEROGLYPH V027" },
    { 0x1339B, "EGYPTIAN HIEROGLYPH V028" },
    { 0x1339C, "EGYPTIAN HIEROGLYPH V028A" },
    { 0x1339D, "EGYPTIAN HIEROGLYPH V029" },
    { 0x1339E, "EGYPTIAN HIEROGLYPH V029A" },
    { 0x1339F, "EGYPTIAN HIEROGLYPH V030" },
    { 0x133A0, "EGYPTIAN HIEROGLYPH V030A" },
    { 0x133A1, "EGYPTIAN HIEROGLYPH V031" },
    { 0x133A2, "EGYPTIAN HIEROGLYPH V031A" },
    { 0x133A3, "EGYPTIAN HIEROGLYPH V032" },
    { 0x133A4, "EGYPTIAN HIEROGLYPH V033" },
    { 0x133A5, "EGYPTIAN HIEROGLYPH V033A" },
    { 0x133A6, "EGYPTIAN HIEROGLYPH V034" },
    { 0x133A7, "EGYPTIAN HIEROGLYPH V035" },
    { 0x133A8, "EGYPTIAN HIEROGLYPH V036" },
    { 0x133A9, "EGYPTIAN HIEROGLYPH V037" },
    { 0x133AA, "EGYPTIAN HIEROGLYPH V037A" },
    { 0x133AB, "EGYPTIAN HIEROGLYPH V038" },
    { 0x133AC, "EGYPTIAN HIEROGLYPH V039" },
    { 0x133AD, "EGYPTIAN HIEROGLYPH V040" },
    { 0x133AE, "EGYPTIAN HIEROGLYPH V040A" },
    { 0x133AF, "EGYPTIAN HIEROGLYPH W001" },
    { 0x133B0, "EGYPTIAN HIEROGLYPH W002" },
    { 0x133B1, "EGYPTIAN HIEROGLYPH W003" },
    { 0x133B2, "EGYPTIAN HIEROGLYPH W003A" },
    { 0x133B3, "EGYPTIAN HIEROGLYPH W004" },
    { 0x133B4, "EGYPTIAN HIEROGLYPH W005" },
    { 0x133B5, "EGYPTIAN HIEROGLYPH W006" },
    { 0x133B6, "EGYPTIAN HIEROGLYPH W007" },
    { 0x133B7, "EGYPTIAN HIEROGLYPH W008" },
    { 0x133B8, "EGYPTIAN HIEROGLYPH W009" },
    { 0x133B9, "EGYPTIAN HIEROGLYPH W009A" },
    { 0x133BA, "EGYPTIAN HIEROGLYPH W010" },
    { 0x133BB, "EGYPTIAN HIEROGLYPH W010A" },
    { 0x133BC, "EGYPTIAN HIEROGLYPH W011" },
    { 0x133BD, "EGYPTIAN HIEROGLYPH W012" },
    { 0x133BE, "EGYPTIAN HIEROGLYPH W013" },
    { 0x133BF, "EGYPTIAN HIEROGLYPH W014" },
    { 0x133C0, "EGYPTIAN HIEROGLYPH W014A" },
    { 0x133C1, "EGYPTIAN HIEROGLYPH W015" },
    { 0x133C2, "EGYPTIAN HIEROGLYPH W016" },
    { 0x133C3, "EGYPTIAN HIEROGLYPH W017" },
    { 0x133C4, "EGYPTIAN HIEROGLYPH W017A" },
    { 0x133C5, "EGYPTIAN HIEROGLYPH W018" },
    { 0x133C6, "EGYPTIAN HIEROGLYPH W018A" },
    { 0x133C7, "EGYPTIAN HIEROGLYPH W019" },
    { 0x133C8, "EGYPTIAN HIEROGLYPH W020" },
    { 0x133C9, "EGYPTIAN HIEROGLYPH W021" },
    { 0x133CA, "EGYPTIAN HIEROGLYPH W022" },
    { 0x133CB, "EGYPTIAN HIEROGLYPH W023" },
    { 0x133CC, "EGYPTIAN HIEROGLYPH W024" },
    { 0x133CD, "EGYPTIAN HIEROGLYPH W024A" },
    { 0x133CE, "EGYPTIAN HIEROGLYPH W025" },
    { 0x133CF, "EGYPTIAN HIEROGLYPH X001" },
    { 0x133D0, "EGYPTIAN HIEROGLYPH X002" },
    { 0x133D1, "EGYPTIAN HIEROGLYPH X003" },
    { 0x133D2, "EGYPTIAN HIEROGLYPH X004" },
    { 0x133D3, "EGYPTIAN HIEROGLYPH X004A" },
    { 0x133D4, "EGYPTIAN HIEROGLYPH X004B" },
    { 0x133D5, "EGYPTIAN HIEROGLYPH X005" },
    { 0x133D6, "EGYPTIAN HIEROGLYPH X006" },
    { 0x133D7, "EGYPTIAN HIEROGLYPH X006A" },
    { 0x133D8, "EGYPTIAN HIEROGLYPH X007" },
    { 0x133D9, "EGYPTIAN HIEROGLYPH X008" },
    { 0x133DA, "EGYPTIAN HIEROGLYPH X008A" },
    { 0x133DB, "EGYPTIAN HIEROGLYPH Y001" },
    { 0x133DC, "EGYPTIAN HIEROGLYPH Y001A" },
    { 0x133DD, "EGYPTIAN HIEROGLYPH Y002" },
    { 0x133DE, "EGYPTIAN HIEROGLYPH Y003" },
    { 0x133DF, "EGYPTIAN HIEROGLYPH Y004" },
    { 0x133E0, "EGYPTIAN HIEROGLYPH Y005" },
    { 0x133E1, "EGYPTIAN HIEROGLYPH Y006" },
    { 0x133E2, "EGYPTIAN HIEROGLYPH Y007" },
    { 0x133E3, "EGYPTIAN HIEROGLYPH Y008" },
    { 0x133E4, "EGYPTIAN HIEROGLYPH Z001" },
    { 0x133E5, "EGYPTIAN HIEROGLYPH Z002" },
    { 0x133E6, "EGYPTIAN HIEROGLYPH Z002A" },
    { 0x133E7, "EGYPTIAN HIEROGLYPH Z002B" },
    { 0x133E8, "EGYPTIAN HIEROGLYPH Z002C" },
    { 0x133E9, "EGYPTIAN HIEROGLYPH Z002D" },
    { 0x133EA, "EGYPTIAN HIEROGLYPH Z003" },
    { 0x133EB, "EGYPTIAN HIEROGLYPH Z003A" },
    { 0x133EC, "EGYPTIAN HIEROGLYPH Z003B" },
    { 0x133ED, "EGYPTIAN HIEROGLYPH Z004" },
    { 0x133EE, "EGYPTIAN HIEROGLYPH Z004A" },
    { 0x133EF, "EGYPTIAN HIEROGLYPH Z005" },
    { 0x133F0, "EGYPTIAN HIEROGLYPH Z005A" },
    { 0x133F1, "EGYPTIAN HIEROGLYPH Z006" },
    { 0x133F2, "EGYPTIAN HIEROGLYPH Z007" },
    { 0x133F3, "EGYPTIAN HIEROGLYPH Z008" },
    { 0x133F4, "EGYPTIAN HIEROGLYPH Z009" },
    { 0x133F5, "EGYPTIAN HIEROGLYPH Z010" },
    { 0x133F6, "EGYPTIAN HIEROGLYPH Z011" },
    { 0x133F7, "EGYPTIAN HIEROGLYPH Z012" },
    { 0x133F8, "EGYPTIAN HIEROGLYPH Z013" },
    { 0x133F9, "EGYPTIAN HIEROGLYPH Z014" },
    { 0x133FA, "EGYPTIAN HIEROGLYPH Z015" },
    { 0x133FB, "EGYPTIAN HIEROGLYPH Z015A" },
    { 0x133FC, "EGYPTIAN HIEROGLYPH Z015B" },
    { 0x133FD, "EGYPTIAN HIEROGLYPH Z015C" },
    { 0x133FE, "EGYPTIAN HIEROGLYPH Z015D" },
    { 0x133FF, "EGYPTIAN HIEROGLYPH Z015E" },
    { 0x13400, "EGYPTIAN HIEROGLYPH Z015F" },
    { 0x13401, "EGYPTIAN HIEROGLYPH Z015G" },
    { 0x13402, "EGYPTIAN HIEROGLYPH Z015H" },
    { 0x13403, "EGYPTIAN HIEROGLYPH Z015I" },
    { 0x13404, "EGYPTIAN HIEROGLYPH Z016" },
    { 0x13405, "EGYPTIAN HIEROGLYPH Z016A" },
    { 0x13406, "EGYPTIAN HIEROGLYPH Z016B" },
    { 0x13407, "EGYPTIAN HIEROGLYPH Z016C" },
    { 0x13408, "EGYPTIAN HIEROGLYPH Z016D" },
    { 0x13409, "EGYPTIAN HIEROGLYPH Z016E" },
    { 0x1340A, "EGYPTIAN HIEROGLYPH Z016F" },
    { 0x1340B, "EGYPTIAN HIEROGLYPH Z016G" },
    { 0x1340C, "EGYPTIAN HIEROGLYPH Z016H" },
    { 0x1340D, "EGYPTIAN HIEROGLYPH AA001" },
    { 0x1340E, "EGYPTIAN HIEROGLYPH AA002" },
    { 0x1340F, "EGYPTIAN HIEROGLYPH AA003" },
    { 0x13410, "EGYPTIAN HIEROGLYPH AA004" },
    { 0x13411, "EGYPTIAN HIEROGLYPH AA005" },
    { 0x13412, "EGYPTIAN HIEROGLYPH AA006" },
    { 0x13413, "EGYPTIAN HIEROGLYPH AA007" },
    { 0x13414, "EGYPTIAN HIEROGLYPH AA007A" },
    { 0x13415, "EGYPTIAN HIEROGLYPH AA007B" },
    { 0x13416, "EGYPTIAN HIEROGLYPH AA008" },
    { 0x13417, "EGYPTIAN HIEROGLYPH AA009" },
    { 0x13418, "EGYPTIAN HIEROGLYPH AA010" },
    { 0x13419, "EGYPTIAN HIEROGLYPH AA011" },
    { 0x1341A, "EGYPTIAN HIEROGLYPH AA012" },
    { 0x1341B, "EGYPTIAN HIEROGLYPH AA013" },
    { 0x1341C, "EGYPTIAN HIEROGLYPH AA014" },
    { 0x1341D, "EGYPTIAN HIEROGLYPH AA015" },
    { 0x1341E, "EGYPTIAN HIEROGLYPH AA016" },
    { 0x1341F, "EGYPTIAN HIEROGLYPH AA017" },
    { 0x13420, "EGYPTIAN HIEROGLYPH AA018" },
    { 0x13421, "EGYPTIAN HIEROGLYPH AA019" },
    { 0x13422, "EGYPTIAN HIEROGLYPH AA020" },
    { 0x13423, "EGYPTIAN HIEROGLYPH AA021" },
    { 0x13424, "EGYPTIAN HIEROGLYPH AA022" },
    { 0x13425, "EGYPTIAN HIEROGLYPH AA023" },
    { 0x13426, "EGYPTIAN HIEROGLYPH AA024" },
    { 0x13427, "EGYPTIAN HIEROGLYPH AA025" },
    { 0x13428, "EGYPTIAN HIEROGLYPH AA026" },
    { 0x13429, "EGYPTIAN HIEROGLYPH AA027" },
    { 0x1342A, "EGYPTIAN HIEROGLYPH AA028" },
    { 0x1342B, "EGYPTIAN HIEROGLYPH AA029" },
    { 0x1342C, "EGYPTIAN HIEROGLYPH AA030" },
    { 0x1342D, "EGYPTIAN HIEROGLYPH AA031" },
    { 0x1342E, "EGYPTIAN HIEROGLYPH AA032" },
    { 0x1342F, "EGYPTIAN HIEROGLYPH V011D" },
    { 0x13430, "EGYPTIAN HIEROGLYPH VERTICAL JOINER" },
    { 0x13431, "EGYPTIAN HIEROGLYPH HORIZONTAL JOINER" },
    { 0x13432, "EGYPTIAN HIEROGLYPH INSERT AT TOP START" },
    { 0x13433, "EGYPTIAN HIEROGLYPH INSERT AT BOTTOM START" },
    { 0x13434, "EGYPTIAN HIEROGLYPH INSERT AT TOP END" },
    { 0x13435, "EGYPTIAN HIEROGLYPH INSERT AT BOTTOM END" },
    { 0x13436, "EGYPTIAN HIEROGLYPH OVERLAY MIDDLE" },
    { 0x13437, "EGYPTIAN HIEROGLYPH BEGIN SEGMENT" },
    { 0x13438, "EGYPTIAN HIEROGLYPH END SEGMENT" },
    { 0x13439, "EGYPTIAN HIEROGLYPH INSERT AT MIDDLE" },
    { 0x1343A, "EGYPTIAN HIEROGLYPH INSERT AT TOP" },
    { 0x1343B, "EGYPTIAN HIEROGLYPH INSERT AT BOTTOM" },
    { 0x1343C, "EGYPTIAN HIEROGLYPH BEGIN ENCLOSURE" },
    { 0x1343D, "EGYPTIAN HIEROGLYPH END ENCLOSURE" },
    { 0x1343E, "EGYPTIAN HIEROGLYPH BEGIN WALLED ENCLOSURE" },
    { 0x1343F, "EGYPTIAN HIEROGLYPH END WALLED ENCLOSURE" },
    { 0x13440, "EGYPTIAN HIEROGLYPH MIRROR HORIZONTALLY" },
    { 0x13441, "EGYPTIAN HIEROGLYPH FULL BLANK" },
    { 0x13442, "EGYPTIAN HIEROGLYPH HALF BLANK" },
    { 0x13443, "EGYPTIAN HIEROGLYPH LOST SIGN" },
    { 0x13444, "EGYPTIAN HIEROGLYPH HALF LOST SIGN" },
    { 0x13445, "EGYPTIAN HIEROGLYPH TALL LOST SIGN" },
    { 0x13446, "EGYPTIAN HIEROGLYPH WIDE LOST SIGN" },
    { 0x13447, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP START" },
    { 0x13448, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM START" },
    { 0x13449, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START" },
    { 0x1344A, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP END" },
    { 0x1344B, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP" },
    { 0x1344C, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM START AND TOP END" },
    { 0x1344D, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START AND TOP" },
    { 0x1344E, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM END" },
    { 0x1344F, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP START AND BOTTOM END" },
    { 0x13450, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM" },
    { 0x13451, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START AND BOTTOM" },
    { 0x13452, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT END" },
    { 0x13453, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP AND END" },
    { 0x13454, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM AND END" },
    { 0x13455, "EGYPTIAN HIEROGLYPH MODIFIER DAMAGED" },
    { 0x14400, "ANATOLIAN HIEROGLYPH A001" },
    { 0x14401, "ANATOLIAN HIEROGLYPH A002" },
    { 0x14402, "ANATOLIAN HIEROGLYPH A003" },
    { 0x14403, "ANATOLIAN HIEROGLYPH A004" },
    { 0x14404, "ANATOLIAN HIEROGLYPH A005" },
    { 0x14405, "ANATOLIAN HIEROGLYPH A006" },
    { 0x14406, "ANATOLIAN HIEROGLYPH A007" },
    { 0x14407, "ANATOLIAN HIEROGLYPH A008" },
    { 0x14408, "ANATOLIAN HIEROGLYPH A009" },
    { 0x14409, "ANATOLIAN HIEROGLYPH A010" },
    { 0x1440A, "ANATOLIAN HIEROGLYPH A010A" },
    { 0x1440B, "ANATOLIAN HIEROGLYPH A011" },
    { 0x1440C, "ANATOLIAN HIEROGLYPH A012" },
    { 0x1440D, "ANATOLIAN HIEROGLYPH A013" },
    { 0x1440E, "ANATOLIAN HIEROGLYPH A014" },
    { 0x1440F, "ANATOLIAN HIEROGLYPH A015" },
    { 0x14410, "ANATOLIAN HIEROGLYPH A016" },
    { 0x14411, "ANATOLIAN HIEROGLYPH A017" },
    { 0x14412, "ANATOLIAN HIEROGLYPH A018" },
    { 0x14413, "ANATOLIAN HIEROGLYPH A019" },
    { 0x14414, "ANATOLIAN HIEROGLYPH A020" },
    { 0x14415, "ANATOLIAN HIEROGLYPH A021" },
    { 0x14416, "ANATOLIAN HIEROGLYPH A022" },
    { 0x14417, "ANATOLIAN HIEROGLYPH A023" },
    { 0x14418, "ANATOLIAN HIEROGLYPH A024" },
    { 0x14419, "ANATOLIAN HIEROGLYPH A025" },
    { 0x1441A, "ANATOLIAN HIEROGLYPH A026" },
    { 0x1441B, "ANATOLIAN HIEROGLYPH A026A" },
    { 0x1441C, "ANATOLIAN HIEROGLYPH A027" },
    { 0x1441D, "ANATOLIAN HIEROGLYPH A028" },
    { 0x1441E, "ANATOLIAN HIEROGLYPH A029" },
    { 0x1441F, "ANATOLIAN HIEROGLYPH A030" },
    { 0x14420, "ANATOLIAN HIEROGLYPH A031" },
    { 0x14421, "ANATOLIAN HIEROGLYPH A032" },
    { 0x14422, "ANATOLIAN HIEROGLYPH A033" },
    { 0x14423, "ANATOLIAN HIEROGLYPH A034" },
    { 0x14424, "ANATOLIAN HIEROGLYPH A035" },
    { 0x14425, "ANATOLIAN HIEROGLYPH A036" },
    { 0x14426, "ANATOLIAN HIEROGLYPH A037" },
    { 0x14427, "ANATOLIAN HIEROGLYPH A038" },
    { 0x14428, "ANATOLIAN HIEROGLYPH A039" },
    { 0x14429, "ANATOLIAN HIEROGLYPH A039A" },
    { 0x1442A, "ANATOLIAN HIEROGLYPH A040" },
    { 0x1442B, "ANATOLIAN HIEROGLYPH A041" },
    { 0x1442C, "ANATOLIAN HIEROGLYPH A041A" },
    { 0x1442D, "ANATOLIAN HIEROGLYPH A042" },
    { 0x1442E, "ANATOLIAN HIEROGLYPH A043" },
    { 0x1442F, "ANATOLIAN HIEROGLYPH A044" },
    { 0x14430, "ANATOLIAN HIEROGLYPH A045" },
    { 0x14431, "ANATOLIAN HIEROGLYPH A045A" },
    { 0x14432, "ANATOLIAN HIEROGLYPH A046" },
    { 0x14433, "ANATOLIAN HIEROGLYPH A046A" },
    { 0x14434, "ANATOLIAN HIEROGLYPH A046B" },
    { 0x14435, "ANATOLIAN HIEROGLYPH A047" },
    { 0x14436, "ANATOLIAN HIEROGLYPH A048" },
    { 0x14437, "ANATOLIAN HIEROGLYPH A049" },
    { 0x14438, "ANATOLIAN HIEROGLYPH A050" },
    { 0x14439, "ANATOLIAN HIEROGLYPH A051" },
    { 0x1443A, "ANATOLIAN HIEROGLYPH A052" },
    { 0x1443B, "ANATOLIAN HIEROGLYPH A053" },
    { 0x1443C, "ANATOLIAN HIEROGLYPH A054" },
    { 0x1443D, "ANATOLIAN HIEROGLYPH A055" },
    { 0x1443E, "ANATOLIAN HIEROGLYPH A056" },
    { 0x1443F, "ANATOLIAN HIEROGLYPH A057" },
    { 0x14440, "ANATOLIAN HIEROGLYPH A058" },
    { 0x14441, "ANATOLIAN HIEROGLYPH A059" },
    { 0x14442, "ANATOLIAN HIEROGLYPH A060" },
    { 0x14443, "ANATOLIAN HIEROGLYPH A061" },
    { 0x14444, "ANATOLIAN HIEROGLYPH A062" },
    { 0x14445, "ANATOLIAN HIEROGLYPH A063" },
    { 0x14446, "ANATOLIAN HIEROGLYPH A064" },
    { 0x14447, "ANATOLIAN HIEROGLYPH A065" },
    { 0x14448, "ANATOLIAN HIEROGLYPH A066" },
    { 0x14449, "ANATOLIAN HIEROGLYPH A066A" },
    { 0x1444A, "ANATOLIAN HIEROGLYPH A066B" },
    { 0x1444B, "ANATOLIAN HIEROGLYPH A066C" },
    { 0x1444C, "ANATOLIAN HIEROGLYPH A067" },
    { 0x1444D, "ANATOLIAN HIEROGLYPH A068" },
    { 0x1444E, "ANATOLIAN HIEROGLYPH A069" },
    { 0x1444F, "ANATOLIAN HIEROGLYPH A070" },
    { 0x14450, "ANATOLIAN HIEROGLYPH A071" },
    { 0x14451, "ANATOLIAN HIEROGLYPH A072" },
    { 0x14452, "ANATOLIAN HIEROGLYPH A073" },
    { 0x14453, "ANATOLIAN HIEROGLYPH A074" },
    { 0x14454, "ANATOLIAN HIEROGLYPH A075" },
    { 0x14455, "ANATOLIAN HIEROGLYPH A076" },
    { 0x14456, "ANATOLIAN HIEROGLYPH A077" },
    { 0x14457, "ANATOLIAN HIEROGLYPH A078" },
    { 0x14458, "ANATOLIAN HIEROGLYPH A079" },
    { 0x14459, "ANATOLIAN HIEROGLYPH A080" },
    { 0x1445A, "ANATOLIAN HIEROGLYPH A081" },
    { 0x1445B, "ANATOLIAN HIEROGLYPH A082" },
    { 0x1445C, "ANATOLIAN HIEROGLYPH A083" },
    { 0x1445D, "ANATOLIAN HIEROGLYPH A084" },
    { 0x1445E, "ANATOLIAN HIEROGLYPH A085" },
    { 0x1445F, "ANATOLIAN HIEROGLYPH A086" },
    { 0x14460, "ANATOLIAN HIEROGLYPH A087" },
    { 0x14461, "ANATOLIAN HIEROGLYPH A088" },
    { 0x14462, "ANATOLIAN HIEROGLYPH A089" },
    { 0x14463, "ANATOLIAN HIEROGLYPH A090" },
    { 0x14464, "ANATOLIAN HIEROGLYPH A091" },
    { 0x14465, "ANATOLIAN HIEROGLYPH A092" },
    { 0x14466, "ANATOLIAN HIEROGLYPH A093" },
    { 0x14467, "ANATOLIAN HIEROGLYPH A094" },
    { 0x14468, "ANATOLIAN HIEROGLYPH A095" },
    { 0x14469, "ANATOLIAN HIEROGLYPH A096" },
    { 0x1446A, "ANATOLIAN HIEROGLYPH A097" },
    { 0x1446B, "ANATOLIAN HIEROGLYPH A097A" },
    { 0x1446C, "ANATOLIAN HIEROGLYPH A098" },
    { 0x1446D, "ANATOLIAN HIEROGLYPH A098A" },
    { 0x1446E, "ANATOLIAN HIEROGLYPH A099" },
    { 0x1446F, "ANATOLIAN HIEROGLYPH A100" },
    { 0x14470, "ANATOLIAN HIEROGLYPH A100A" },
    { 0x14471, "ANATOLIAN HIEROGLYPH A101" },
    { 0x14472, "ANATOLIAN HIEROGLYPH A101A" },
    { 0x14473, "ANATOLIAN HIEROGLYPH A102" },
    { 0x14474, "ANATOLIAN HIEROGLYPH A102A" },
    { 0x14475, "ANATOLIAN HIEROGLYPH A103" },
    { 0x14476, "ANATOLIAN HIEROGLYPH A104" },
    { 0x14477, "ANATOLIAN HIEROGLYPH A104A" },
    { 0x14478, "ANATOLIAN HIEROGLYPH A104B" },
    { 0x14479, "ANATOLIAN HIEROGLYPH A104C" },
    { 0x1447A, "ANATOLIAN HIEROGLYPH A105" },
    { 0x1447B, "ANATOLIAN HIEROGLYPH A105A" },
    { 0x1447C, "ANATOLIAN HIEROGLYPH A105B" },
    { 0x1447D, "ANATOLIAN HIEROGLYPH A106" },
    { 0x1447E, "ANATOLIAN HIEROGLYPH A107" },
    { 0x1447F, "ANATOLIAN HIEROGLYPH A107A" },
    { 0x14480, "ANATOLIAN HIEROGLYPH A107B" },
    { 0x14481, "ANATOLIAN HIEROGLYPH A107C" },
    { 0x14482, "ANATOLIAN HIEROGLYPH A108" },
    { 0x14483, "ANATOLIAN HIEROGLYPH A109" },
    { 0x14484, "ANATOLIAN HIEROGLYPH A110" },
    { 0x14485, "ANATOLIAN HIEROGLYPH A110A" },
    { 0x14486, "ANATOLIAN HIEROGLYPH A110B" },
    { 0x14487, "ANATOLIAN HIEROGLYPH A111" },
    { 0x14488, "ANATOLIAN HIEROGLYPH A112" },
    { 0x14489, "ANATOLIAN HIEROGLYPH A113" },
    { 0x1448A, "ANATOLIAN HIEROGLYPH A114" },
    { 0x1448B, "ANATOLIAN HIEROGLYPH A115" },
    { 0x1448C, "ANATOLIAN HIEROGLYPH A115A" },
    { 0x1448D, "ANATOLIAN HIEROGLYPH A116" },
    { 0x1448E, "ANATOLIAN HIEROGLYPH A117" },
    { 0x1448F, "ANATOLIAN HIEROGLYPH A118" },
    { 0x14490, "ANATOLIAN HIEROGLYPH A119" },
    { 0x14491, "ANATOLIAN HIEROGLYPH A120" },
    { 0x14492, "ANATOLIAN HIEROGLYPH A121" },
    { 0x14493, "ANATOLIAN HIEROGLYPH A122" },
    { 0x14494, "ANATOLIAN HIEROGLYPH A123" },
    { 0x14495, "ANATOLIAN HIEROGLYPH A124" },
    { 0x14496, "ANATOLIAN HIEROGLYPH A125" },
    { 0x14497, "ANATOLIAN HIEROGLYPH A125A" },
    { 0x14498, "ANATOLIAN HIEROGLYPH A126" },
    { 0x14499, "ANATOLIAN HIEROGLYPH A127" },
    { 0x1449A, "ANATOLIAN HIEROGLYPH A128" },
    { 0x1449B, "ANATOLIAN HIEROGLYPH A129" },
    { 0x1449C, "ANATOLIAN HIEROGLYPH A130" },
    { 0x1449D, "ANATOLIAN HIEROGLYPH A131" },
    { 0x1449E, "ANATOLIAN HIEROGLYPH A132" },
    { 0x1449F, "ANATOLIAN HIEROGLYPH A133" },
    { 0x144A0, "ANATOLIAN HIEROGLYPH A134" },
    { 0x144A1, "ANATOLIAN HIEROGLYPH A135" },
    { 0x144A2, "ANATOLIAN HIEROGLYPH A135A" },
    { 0x144A3, "ANATOLIAN HIEROGLYPH A136" },
    { 0x144A4, "ANATOLIAN HIEROGLYPH A137" },
    { 0x144A5, "ANATOLIAN HIEROGLYPH A138" },
    { 0x144A6, "ANATOLIAN HIEROGLYPH A139" },
    { 0x144A7, "ANATOLIAN HIEROGLYPH A140" },
    { 0x144A8, "ANATOLIAN HIEROGLYPH A141" },
    { 0x144A9, "ANATOLIAN HIEROGLYPH A142" },
    { 0x144AA, "ANATOLIAN HIEROGLYPH A143" },
    { 0x144AB, "ANATOLIAN HIEROGLYPH A144" },
    { 0x144AC, "ANATOLIAN HIEROGLYPH A145" },
    { 0x144AD, "ANATOLIAN HIEROGLYPH A146" },
    { 0x144AE, "ANATOLIAN HIEROGLYPH A147" },
    { 0x144AF, "ANATOLIAN HIEROGLYPH A148" },
    { 0x144B0, "ANATOLIAN HIEROGLYPH A149" },
    { 0x144B1, "ANATOLIAN HIEROGLYPH A150" },
    { 0x144B2, "ANATOLIAN HIEROGLYPH A151" },
    { 0x144B3, "ANATOLIAN HIEROGLYPH A152" },
    { 0x144B4, "ANATOLIAN HIEROGLYPH A153" },
    { 0x144B5, "ANATOLIAN HIEROGLYPH A154" },
    { 0x144B6, "ANATOLIAN HIEROGLYPH A155" },
    { 0x144B7, "ANATOLIAN HIEROGLYPH A156" },
    { 0x144B8, "ANATOLIAN HIEROGLYPH A157" },
    { 0x144B9, "ANATOLIAN HIEROGLYPH A158" },
    { 0x144BA, "ANATOLIAN HIEROGLYPH A159" },
    { 0x144BB, "ANATOLIAN HIEROGLYPH A160" },
    { 0x144BC, "ANATOLIAN HIEROGLYPH A161" },
    { 0x144BD, "ANATOLIAN HIEROGLYPH A162" },
    { 0x144BE, "ANATOLIAN HIEROGLYPH A163" },
    { 0x144BF, "ANATOLIAN HIEROGLYPH A164" },
    { 0x144C0, "ANATOLIAN HIEROGLYPH A165" },
    { 0x144C1, "ANATOLIAN HIEROGLYPH A166" },
    { 0x144C2, "ANATOLIAN HIEROGLYPH A167" },
    { 0x144C3, "ANATOLIAN HIEROGLYPH A168" },
    { 0x144C4, "ANATOLIAN HIEROGLYPH A169" },
    { 0x144C5, "ANATOLIAN HIEROGLYPH A170" },
    { 0x144C6, "ANATOLIAN HIEROGLYPH A171" },
    { 0x144C7, "ANATOLIAN HIEROGLYPH A172" },
    { 0x144C8, "ANATOLIAN HIEROGLYPH A173" },
    { 0x144C9, "ANATOLIAN HIEROGLYPH A174" },
    { 0x144CA, "ANATOLIAN HIEROGLYPH A175" },
    { 0x144CB, "ANATOLIAN HIEROGLYPH A176" },
    { 0x144CC, "ANATOLIAN HIEROGLYPH A177" },
    { 0x144CD, "ANATOLIAN HIEROGLYPH A178" },
    { 0x144CE, "ANATOLIAN HIEROGLYPH A179" },
    { 0x144CF, "ANATOLIAN HIEROGLYPH A180" },
    { 0x144D0, "ANATOLIAN HIEROGLYPH A181" },
    { 0x144D1, "ANATOLIAN HIEROGLYPH A182" },
    { 0x144D2, "ANATOLIAN HIEROGLYPH A183" },
    { 0x144D3, "ANATOLIAN HIEROGLYPH A184" },
    { 0x144D4, "ANATOLIAN HIEROGLYPH A185" },
    { 0x144D5, "ANATOLIAN HIEROGLYPH A186" },
    { 0x144D6, "ANATOLIAN HIEROGLYPH A187" },
    { 0x144D7, "ANATOLIAN HIEROGLYPH A188" },
    { 0x144D8, "ANATOLIAN HIEROGLYPH A189" },
    { 0x144D9, "ANATOLIAN HIEROGLYPH A190" },
    { 0x144DA, "ANATOLIAN HIEROGLYPH A191" },
    { 0x144DB, "ANATOLIAN HIEROGLYPH A192" },
    { 0x144DC, "ANATOLIAN HIEROGLYPH A193" },
    { 0x144DD, "ANATOLIAN HIEROGLYPH A194" },
    { 0x144DE, "ANATOLIAN HIEROGLYPH A195" },
    { 0x144DF, "ANATOLIAN HIEROGLYPH A196" },
    { 0x144E0, "ANATOLIAN HIEROGLYPH A197" },
    { 0x144E1, "ANATOLIAN HIEROGLYPH A198" },
    { 0x144E2, "ANATOLIAN HIEROGLYPH A199" },
    { 0x144E3, "ANATOLIAN HIEROGLYPH A200" },
    { 0x144E4, "ANATOLIAN HIEROGLYPH A201" },
    { 0x144E5, "ANATOLIAN HIEROGLYPH A202" },
    { 0x144E6, "ANATOLIAN HIEROGLYPH A202A" },
    { 0x144E7, "ANATOLIAN HIEROGLYPH A202B" },
    { 0x144E8, "ANATOLIAN HIEROGLYPH A203" },
    { 0x144E9, "ANATOLIAN HIEROGLYPH A204" },
    { 0x144EA, "ANATOLIAN HIEROGLYPH A205" },
    { 0x144EB, "ANATOLIAN HIEROGLYPH A206" },
    { 0x144EC, "ANATOLIAN HIEROGLYPH A207" },
    { 0x144ED, "ANATOLIAN HIEROGLYPH A207A" },
    { 0x144EE, "ANATOLIAN HIEROGLYPH A208" },
    { 0x144EF, "ANATOLIAN HIEROGLYPH A209" },
    { 0x144F0, "ANATOLIAN HIEROGLYPH A209A" },
    { 0x144F1, "ANATOLIAN HIEROGLYPH A210" },
    { 0x144F2, "ANATOLIAN HIEROGLYPH A211" },
    { 0x144F3, "ANATOLIAN HIEROGLYPH A212" },
    { 0x144F4, "ANATOLIAN HIEROGLYPH A213" },
    { 0x144F5, "ANATOLIAN HIEROGLYPH A214" },
    { 0x144F6, "ANATOLIAN HIEROGLYPH A215" },
    { 0x144F7, "ANATOLIAN HIEROGLYPH A215A" },
    { 0x144F8, "ANATOLIAN HIEROGLYPH A216" },
    { 0x144F9, "ANATOLIAN HIEROGLYPH A216A" },
    { 0x144FA, "ANATOLIAN HIEROGLYPH A217" },
    { 0x144FB, "ANATOLIAN HIEROGLYPH A218" },
    { 0x144FC, "ANATOLIAN HIEROGLYPH A219" },
    { 0x144FD, "ANATOLIAN HIEROGLYPH A220" },
    { 0x144FE, "ANATOLIAN HIEROGLYPH A221" },
    { 0x144FF, "ANATOLIAN HIEROGLYPH A222" },
    { 0x14500, "ANATOLIAN HIEROGLYPH A223" },
    { 0x14501, "ANATOLIAN HIEROGLYPH A224" },
    { 0x14502, "ANATOLIAN HIEROGLYPH A225" },
    { 0x14503, "ANATOLIAN HIEROGLYPH A226" },
    { 0x14504, "ANATOLIAN HIEROGLYPH A227" },
    { 0x14505, "ANATOLIAN HIEROGLYPH A227A" },
    { 0x14506, "ANATOLIAN HIEROGLYPH A228" },
    { 0x14507, "ANATOLIAN HIEROGLYPH A229" },
    { 0x14508, "ANATOLIAN HIEROGLYPH A230" },
    { 0x14509, "ANATOLIAN HIEROGLYPH A231" },
    { 0x1450A, "ANATOLIAN HIEROGLYPH A232" },
    { 0x1450B, "ANATOLIAN HIEROGLYPH A233" },
    { 0x1450C, "ANATOLIAN HIEROGLYPH A234" },
    { 0x1450D, "ANATOLIAN HIEROGLYPH A235" },
    { 0x1450E, "ANATOLIAN HIEROGLYPH A236" },
    { 0x1450F, "ANATOLIAN HIEROGLYPH A237" },
    { 0x14510, "ANATOLIAN HIEROGLYPH A238" },
    { 0x14511, "ANATOLIAN HIEROGLYPH A239" },
    { 0x14512, "ANATOLIAN HIEROGLYPH A240" },
    { 0x14513, "ANATOLIAN HIEROGLYPH A241" },
    { 0x14514, "ANATOLIAN HIEROGLYPH A242" },
    { 0x14515, "ANATOLIAN HIEROGLYPH A243" },
    { 0x14516, "ANATOLIAN HIEROGLYPH A244" },
    { 0x14517, "ANATOLIAN HIEROGLYPH A245" },
    { 0x14518, "ANATOLIAN HIEROGLYPH A246" },
    { 0x14519, "ANATOLIAN HIEROGLYPH A247" },
    { 0x1451A, "ANATOLIAN HIEROGLYPH A248" },
    { 0x1451B, "ANATOLIAN HIEROGLYPH A249" },
    { 0x1451C, "ANATOLIAN HIEROGLYPH A250" },
    { 0x1451D, "ANATOLIAN HIEROGLYPH A251" },
    { 0x1451E, "ANATOLIAN HIEROGLYPH A252" },
    { 0x1451F, "ANATOLIAN HIEROGLYPH A253" },
    { 0x14520, "ANATOLIAN HIEROGLYPH A254" },
    { 0x14521, "ANATOLIAN HIEROGLYPH A255" },
    { 0x14522, "ANATOLIAN HIEROGLYPH A256" },
    { 0x14523, "ANATOLIAN HIEROGLYPH A257" },
    { 0x14524, "ANATOLIAN HIEROGLYPH A258" },
    { 0x14525, "ANATOLIAN HIEROGLYPH A259" },
    { 0x14526, "ANATOLIAN HIEROGLYPH A260" },
    { 0x14527, "ANATOLIAN HIEROGLYPH A261" },
    { 0x14528, "ANATOLIAN HIEROGLYPH A262" },
    { 0x14529, "ANATOLIAN HIEROGLYPH A263" },
    { 0x1452A, "ANATOLIAN HIEROGLYPH A264" },
    { 0x1452B, "ANATOLIAN HIEROGLYPH A265" },
    { 0x1452C, "ANATOLIAN HIEROGLYPH A266" },
    { 0x1452D, "ANATOLIAN HIEROGLYPH A267" },
    { 0x1452E, "ANATOLIAN HIEROGLYPH A267A" },
    { 0x1452F, "ANATOLIAN HIEROGLYPH A268" },
    { 0x14530, "ANATOLIAN HIEROGLYPH A269" },
    { 0x14531, "ANATOLIAN HIEROGLYPH A270" },
    { 0x14532, "ANATOLIAN HIEROGLYPH A271" },
    { 0x14533, "ANATOLIAN HIEROGLYPH A272" },
    { 0x14534, "ANATOLIAN HIEROGLYPH A273" },
    { 0x14535, "ANATOLIAN HIEROGLYPH A274" },
    { 0x14536, "ANATOLIAN HIEROGLYPH A275" },
    { 0x14537, "ANATOLIAN HIEROGLYPH A276" },
    { 0x14538, "ANATOLIAN HIEROGLYPH A277" },
    { 0x14539, "ANATOLIAN HIEROGLYPH A278" },
    { 0x1453A, "ANATOLIAN HIEROGLYPH A279" },
    { 0x1453B, "ANATOLIAN HIEROGLYPH A280" },
    { 0x1453C, "ANATOLIAN HIEROGLYPH A281" },
    { 0x1453D, "ANATOLIAN HIEROGLYPH A282" },
    { 0x1453E, "ANATOLIAN HIEROGLYPH A283" },
    { 0x1453F, "ANATOLIAN HIEROGLYPH A284" },
    { 0x14540, "ANATOLIAN HIEROGLYPH A285" },
    { 0x14541, "ANATOLIAN HIEROGLYPH A286" },
    { 0x14542, "ANATOLIAN HIEROGLYPH A287" },
    { 0x14543, "ANATOLIAN HIEROGLYPH A288" },
    { 0x14544, "ANATOLIAN HIEROGLYPH A289" },
    { 0x14545, "ANATOLIAN HIEROGLYPH A289A" },
    { 0x14546, "ANATOLIAN HIEROGLYPH A290" },
    { 0x14547, "ANATOLIAN HIEROGLYPH A291" },
    { 0x14548, "ANATOLIAN HIEROGLYPH A292" },
    { 0x14549, "ANATOLIAN HIEROGLYPH A293" },
    { 0x1454A, "ANATOLIAN HIEROGLYPH A294" },
    { 0x1454B, "ANATOLIAN HIEROGLYPH A294A" },
    { 0x1454C, "ANATOLIAN HIEROGLYPH A295" },
    { 0x1454D, "ANATOLIAN HIEROGLYPH A296" },
    { 0x1454E, "ANATOLIAN HIEROGLYPH A297" },
    { 0x1454F, "ANATOLIAN HIEROGLYPH A298" },
    { 0x14550, "ANATOLIAN HIEROGLYPH A299" },
    { 0x14551, "ANATOLIAN HIEROGLYPH A299A" },
    { 0x14552, "ANATOLIAN HIEROGLYPH A300" },
    { 0x14553, "ANATOLIAN HIEROGLYPH A301" },
    { 0x14554, "ANATOLIAN HIEROGLYPH A302" },
    { 0x14555, "ANATOLIAN HIEROGLYPH A303" },
    { 0x14556, "ANATOLIAN HIEROGLYPH A304" },
    { 0x14557, "ANATOLIAN HIEROGLYPH A305" },
    { 0x14558, "ANATOLIAN HIEROGLYPH A306" },
    { 0x14559, "ANATOLIAN HIEROGLYPH A307" },
    { 0x1455A, "ANATOLIAN HIEROGLYPH A308" },
    { 0x1455B, "ANATOLIAN HIEROGLYPH A309" },
    { 0x1455C, "ANATOLIAN HIEROGLYPH A309A" },
    { 0x1455D, "ANATOLIAN HIEROGLYPH A310" },
    { 0x1455E, "ANATOLIAN HIEROGLYPH A311" },
    { 0x1455F, "ANATOLIAN HIEROGLYPH A312" },
    { 0x14560, "ANATOLIAN HIEROGLYPH A313" },
    { 0x14561, "ANATOLIAN HIEROGLYPH A314" },
    { 0x14562, "ANATOLIAN HIEROGLYPH A315" },
    { 0x14563, "ANATOLIAN HIEROGLYPH A316" },
    { 0x14564, "ANATOLIAN HIEROGLYPH A317" },
    { 0x14565, "ANATOLIAN HIEROGLYPH A318" },
    { 0x14566, "ANATOLIAN HIEROGLYPH A319" },
    { 0x14567, "ANATOLIAN HIEROGLYPH A320" },
    { 0x14568, "ANATOLIAN HIEROGLYPH A321" },
    { 0x14569, "ANATOLIAN HIEROGLYPH A322" },
    { 0x1456A, "ANATOLIAN HIEROGLYPH A323" },
    { 0x1456B, "ANATOLIAN HIEROGLYPH A324" },
    { 0x1456C, "ANATOLIAN HIEROGLYPH A325" },
    { 0x1456D, "ANATOLIAN HIEROGLYPH A326" },
    { 0x1456E, "ANATOLIAN HIEROGLYPH A327" },
    { 0x1456F, "ANATOLIAN HIEROGLYPH A328" },
    { 0x14570, "ANATOLIAN HIEROGLYPH A329" },
    { 0x14571, "ANATOLIAN HIEROGLYPH A329A" },
    { 0x14572, "ANATOLIAN HIEROGLYPH A330" },
    { 0x14573, "ANATOLIAN HIEROGLYPH A331" },
    { 0x14574, "ANATOLIAN HIEROGLYPH A332A" },
    { 0x14575, "ANATOLIAN HIEROGLYPH A332B" },
    { 0x14576, "ANATOLIAN HIEROGLYPH A332C" },
    { 0x14577, "ANATOLIAN HIEROGLYPH A333" },
    { 0x14578, "ANATOLIAN HIEROGLYPH A334" },
    { 0x14579, "ANATOLIAN HIEROGLYPH A335" },
    { 0x1457A, "ANATOLIAN HIEROGLYPH A336" },
    { 0x1457B, "ANATOLIAN HIEROGLYPH A336A" },
    { 0x1457C, "ANATOLIAN HIEROGLYPH A336B" },
    { 0x1457D, "ANATOLIAN HIEROGLYPH A336C" },
    { 0x1457E, "ANATOLIAN HIEROGLYPH A337" },
    { 0x1457F, "ANATOLIAN HIEROGLYPH A338" },
    { 0x14580, "ANATOLIAN HIEROGLYPH A339" },
    { 0x14581, "ANATOLIAN HIEROGLYPH A340" },
    { 0x14582, "ANATOLIAN HIEROGLYPH A341" },
    { 0x14583, "ANATOLIAN HIEROGLYPH A342" },
    { 0x14584, "ANATOLIAN HIEROGLYPH A343" },
    { 0x14585, "ANATOLIAN HIEROGLYPH A344" },
    { 0x14586, "ANATOLIAN HIEROGLYPH A345" },
    { 0x14587, "ANATOLIAN HIEROGLYPH A346" },
    { 0x14588, "ANATOLIAN HIEROGLYPH A347" },
    { 0x14589, "ANATOLIAN HIEROGLYPH A348" },
    { 0x1458A, "ANATOLIAN HIEROGLYPH A349" },
    { 0x1458B, "ANATOLIAN HIEROGLYPH A350" },
    { 0x1458C, "ANATOLIAN HIEROGLYPH A351" },
    { 0x1458D, "ANATOLIAN HIEROGLYPH A352" },
    { 0x1458E, "ANATOLIAN HIEROGLYPH A353" },
    { 0x1458F, "ANATOLIAN HIEROGLYPH A354" },
    { 0x14590, "ANATOLIAN HIEROGLYPH A355" },
    { 0x14591, "ANATOLIAN HIEROGLYPH A356" },
    { 0x14592, "ANATOLIAN HIEROGLYPH A357" },
    { 0x14593, "ANATOLIAN HIEROGLYPH A358" },
    { 0x14594, "ANATOLIAN HIEROGLYPH A359" },
    { 0x14595, "ANATOLIAN HIEROGLYPH A359A" },
    { 0x14596, "ANATOLIAN HIEROGLYPH A360" },
    { 0x14597, "ANATOLIAN HIEROGLYPH A361" },
    { 0x14598, "ANATOLIAN HIEROGLYPH A362" },
    { 0x14599, "ANATOLIAN HIEROGLYPH A363" },
    { 0x1459A, "ANATOLIAN HIEROGLYPH A364" },
    { 0x1459B, "ANATOLIAN HIEROGLYPH A364A" },
    { 0x1459C, "ANATOLIAN HIEROGLYPH A365" },
    { 0x1459D, "ANATOLIAN HIEROGLYPH A366" },
    { 0x1459E, "ANATOLIAN HIEROGLYPH A367" },
    { 0x1459F, "ANATOLIAN HIEROGLYPH A368" },
    { 0x145A0, "ANATOLIAN HIEROGLYPH A368A" },
    { 0x145A1, "ANATOLIAN HIEROGLYPH A369" },
    { 0x145A2, "ANATOLIAN HIEROGLYPH A370" },
    { 0x145A3, "ANATOLIAN HIEROGLYPH A371" },
    { 0x145A4, "ANATOLIAN HIEROGLYPH A371A" },
    { 0x145A5, "ANATOLIAN HIEROGLYPH A372" },
    { 0x145A6, "ANATOLIAN HIEROGLYPH A373" },
    { 0x145A7, "ANATOLIAN HIEROGLYPH A374" },
    { 0x145A8, "ANATOLIAN HIEROGLYPH A375" },
    { 0x145A9, "ANATOLIAN HIEROGLYPH A376" },
    { 0x145AA, "ANATOLIAN HIEROGLYPH A377" },
    { 0x145AB, "ANATOLIAN HIEROGLYPH A378" },
    { 0x145AC, "ANATOLIAN HIEROGLYPH A379" },
    { 0x145AD, "ANATOLIAN HIEROGLYPH A380" },
    { 0x145AE, "ANATOLIAN HIEROGLYPH A381" },
    { 0x145AF, "ANATOLIAN HIEROGLYPH A381A" },
    { 0x145B0, "ANATOLIAN HIEROGLYPH A382" },
    { 0x145B1, "ANATOLIAN HIEROGLYPH A383 RA OR RI" },
    { 0x145B2, "ANATOLIAN HIEROGLYPH A383A" },
    { 0x145B3, "ANATOLIAN HIEROGLYPH A384" },
    { 0x145B4, "ANATOLIAN HIEROGLYPH A385" },
    { 0x145B5, "ANATOLIAN HIEROGLYPH A386" },
    { 0x145B6, "ANATOLIAN HIEROGLYPH A386A" },
    { 0x145B7, "ANATOLIAN HIEROGLYPH A387" },
    { 0x145B8, "ANATOLIAN HIEROGLYPH A388" },
    { 0x145B9, "ANATOLIAN HIEROGLYPH A389" },
    { 0x145BA, "ANATOLIAN HIEROGLYPH A390" },
    { 0x145BB, "ANATOLIAN HIEROGLYPH A391" },
    { 0x145BC, "ANATOLIAN HIEROGLYPH A392" },
    { 0x145BD, "ANATOLIAN HIEROGLYPH A393 EIGHT" },
    { 0x145BE, "ANATOLIAN HIEROGLYPH A394" },
    { 0x145BF, "ANATOLIAN HIEROGLYPH A395" },
    { 0x145C0, "ANATOLIAN HIEROGLYPH A396" },
    { 0x145C1, "ANATOLIAN HIEROGLYPH A397" },
    { 0x145C2, "ANATOLIAN HIEROGLYPH A398" },
    { 0x145C3, "ANATOLIAN HIEROGLYPH A399" },
    { 0x145C4, "ANATOLIAN HIEROGLYPH A400" },
    { 0x145C5, "ANATOLIAN HIEROGLYPH A401" },
    { 0x145C6, "ANATOLIAN HIEROGLYPH A402" },
    { 0x145C7, "ANATOLIAN HIEROGLYPH A403" },
    { 0x145C8, "ANATOLIAN HIEROGLYPH A404" },
    { 0x145C9, "ANATOLIAN HIEROGLYPH A405" },
    { 0x145CA, "ANATOLIAN HIEROGLYPH A406" },
    { 0x145CB, "ANATOLIAN HIEROGLYPH A407" },
    { 0x145CC, "ANATOLIAN HIEROGLYPH A408" },
    { 0x145CD, "ANATOLIAN HIEROGLYPH A409" },
    { 0x145CE, "ANATOLIAN HIEROGLYPH A410 BEGIN LOGOGRAM MARK" },
    { 0x145CF, "ANATOLIAN HIEROGLYPH A410A END LOGOGRAM MARK" },
    { 0x145D0, "ANATOLIAN HIEROGLYPH A411" },
    { 0x145D1, "ANATOLIAN HIEROGLYPH A412" },
    { 0x145D2, "ANATOLIAN HIEROGLYPH A413" },
    { 0x145D3, "ANATOLIAN HIEROGLYPH A414" },
    { 0x145D4, "ANATOLIAN HIEROGLYPH A415" },
    { 0x145D5, "ANATOLIAN HIEROGLYPH A416" },
    { 0x145D6, "ANATOLIAN HIEROGLYPH A417" },
    { 0x145D7, "ANATOLIAN HIEROGLYPH A418" },
    { 0x145D8, "ANATOLIAN HIEROGLYPH A419" },
    { 0x145D9, "ANATOLIAN HIEROGLYPH A420" },
    { 0x145DA, "ANATOLIAN HIEROGLYPH A421" },
    { 0x145DB, "ANATOLIAN HIEROGLYPH A422" },
    { 0x145DC, "ANATOLIAN HIEROGLYPH A423" },
    { 0x145DD, "ANATOLIAN HIEROGLYPH A424" },
    { 0x145DE, "ANATOLIAN HIEROGLYPH A425" },
    { 0x145DF, "ANATOLIAN HIEROGLYPH A426" },
    { 0x145E0, "ANATOLIAN HIEROGLYPH A427" },
    { 0x145E1, "ANATOLIAN HIEROGLYPH A428" },
    { 0x145E2, "ANATOLIAN HIEROGLYPH A429" },
    { 0x145E3, "ANATOLIAN HIEROGLYPH A430" },
    { 0x145E4, "ANATOLIAN HIEROGLYPH A431" },
    { 0x145E5, "ANATOLIAN HIEROGLYPH A432" },
    { 0x145E6, "ANATOLIAN HIEROGLYPH A433" },
    { 0x145E7, "ANATOLIAN HIEROGLYPH A434" },
    { 0x145E8, "ANATOLIAN HIEROGLYPH A435" },
    { 0x145E9, "ANATOLIAN HIEROGLYPH A436" },
    { 0x145EA, "ANATOLIAN HIEROGLYPH A437" },
    { 0x145EB, "ANATOLIAN HIEROGLYPH A438" },
    { 0x145EC, "ANATOLIAN HIEROGLYPH A439" },
    { 0x145ED, "ANATOLIAN HIEROGLYPH A440" },
    { 0x145EE, "ANATOLIAN HIEROGLYPH A441" },
    { 0x145EF, "ANATOLIAN HIEROGLYPH A442" },
    { 0x145F0, "ANATOLIAN HIEROGLYPH A443" },
    { 0x145F1, "ANATOLIAN HIEROGLYPH A444" },
    { 0x145F2, "ANATOLIAN HIEROGLYPH A445" },
    { 0x145F3, "ANATOLIAN HIEROGLYPH A446" },
    { 0x145F4, "ANATOLIAN HIEROGLYPH A447" },
    { 0x145F5, "ANATOLIAN HIEROGLYPH A448" },
    { 0x145F6, "ANATOLIAN HIEROGLYPH A449" },
    { 0x145F7, "ANATOLIAN HIEROGLYPH A450" },
    { 0x145F8, "ANATOLIAN HIEROGLYPH A450A" },
    { 0x145F9, "ANATOLIAN HIEROGLYPH A451" },
    { 0x145FA, "ANATOLIAN HIEROGLYPH A452" },
    { 0x145FB, "ANATOLIAN HIEROGLYPH A453" },
    { 0x145FC, "ANATOLIAN HIEROGLYPH A454" },
    { 0x145FD, "ANATOLIAN HIEROGLYPH A455" },
    { 0x145FE, "ANATOLIAN HIEROGLYPH A456" },
    { 0x145FF, "ANATOLIAN HIEROGLYPH A457" },
    { 0x14600, "ANATOLIAN HIEROGLYPH A457A" },
    { 0x14601, "ANATOLIAN HIEROGLYPH A458" },
    { 0x14602, "ANATOLIAN HIEROGLYPH A459" },
    { 0x14603, "ANATOLIAN HIEROGLYPH A460" },
    { 0x14604, "ANATOLIAN HIEROGLYPH A461" },
    { 0x14605, "ANATOLIAN HIEROGLYPH A462" },
    { 0x14606, "ANATOLIAN HIEROGLYPH A463" },
    { 0x14607, "ANATOLIAN HIEROGLYPH A464" },
    { 0x14608, "ANATOLIAN HIEROGLYPH A465" },
    { 0x14609, "ANATOLIAN HIEROGLYPH A466" },
    { 0x1460A, "ANATOLIAN HIEROGLYPH A467" },
    { 0x1460B, "ANATOLIAN HIEROGLYPH A468" },
    { 0x1460C, "ANATOLIAN HIEROGLYPH A469" },
    { 0x1460D, "ANATOLIAN HIEROGLYPH A470" },
    { 0x1460E, "ANATOLIAN HIEROGLYPH A471" },
    { 0x1460F, "ANATOLIAN HIEROGLYPH A472" },
    { 0x14610, "ANATOLIAN HIEROGLYPH A473" },
    { 0x14611, "ANATOLIAN HIEROGLYPH A474" },
    { 0x14612, "ANATOLIAN HIEROGLYPH A475" },
    { 0x14613, "ANATOLIAN HIEROGLYPH A476" },
    { 0x14614, "ANATOLIAN HIEROGLYPH A477" },
    { 0x14615, "ANATOLIAN HIEROGLYPH A478" },
    { 0x14616, "ANATOLIAN HIEROGLYPH A479" },
    { 0x14617, "ANATOLIAN HIEROGLYPH A480" },
    { 0x14618, "ANATOLIAN HIEROGLYPH A481" },
    { 0x14619, "ANATOLIAN HIEROGLYPH A482" },
    { 0x1461A, "ANATOLIAN HIEROGLYPH A483" },
    { 0x1461B, "ANATOLIAN HIEROGLYPH A484" },
    { 0x1461C, "ANATOLIAN HIEROGLYPH A485" },
    { 0x1461D, "ANATOLIAN HIEROGLYPH A486" },
    { 0x1461E, "ANATOLIAN HIEROGLYPH A487" },
    { 0x1461F, "ANATOLIAN HIEROGLYPH A488" },
    { 0x14620, "ANATOLIAN HIEROGLYPH A489" },
    { 0x14621, "ANATOLIAN HIEROGLYPH A490" },
    { 0x14622, "ANATOLIAN HIEROGLYPH A491" },
    { 0x14623, "ANATOLIAN HIEROGLYPH A492" },
    { 0x14624, "ANATOLIAN HIEROGLYPH A493" },
    { 0x14625, "ANATOLIAN HIEROGLYPH A494" },
    { 0x14626, "ANATOLIAN HIEROGLYPH A495" },
    { 0x14627, "ANATOLIAN HIEROGLYPH A496" },
    { 0x14628, "ANATOLIAN HIEROGLYPH A497" },
    { 0x14629, "ANATOLIAN HIEROGLYPH A501" },
    { 0x1462A, "ANATOLIAN HIEROGLYPH A502" },
    { 0x1462B, "ANATOLIAN HIEROGLYPH A503" },
    { 0x1462C, "ANATOLIAN HIEROGLYPH A504" },
    { 0x1462D, "ANATOLIAN HIEROGLYPH A505" },
    { 0x1462E, "ANATOLIAN HIEROGLYPH A506" },
    { 0x1462F, "ANATOLIAN HIEROGLYPH A507" },
    { 0x14630, "ANATOLIAN HIEROGLYPH A508" },
    { 0x14631, "ANATOLIAN HIEROGLYPH A509" },
    { 0x14632, "ANATOLIAN HIEROGLYPH A510" },
    { 0x14633, "ANATOLIAN HIEROGLYPH A511" },
    { 0x14634, "ANATOLIAN HIEROGLYPH A512" },
    { 0x14635, "ANATOLIAN HIEROGLYPH A513" },
    { 0x14636, "ANATOLIAN HIEROGLYPH A514" },
    { 0x14637, "ANATOLIAN HIEROGLYPH A515" },
    { 0x14638, "ANATOLIAN HIEROGLYPH A516" },
    { 0x14639, "ANATOLIAN HIEROGLYPH A517" },
    { 0x1463A, "ANATOLIAN HIEROGLYPH A518" },
    { 0x1463B, "ANATOLIAN HIEROGLYPH A519" },
    { 0x1463C, "ANATOLIAN HIEROGLYPH A520" },
    { 0x1463D, "ANATOLIAN HIEROGLYPH A521" },
    { 0x1463E, "ANATOLIAN HIEROGLYPH A522" },
    { 0x1463F, "ANATOLIAN HIEROGLYPH A523" },
    { 0x14640, "ANATOLIAN HIEROGLYPH A524" },
    { 0x14641, "ANATOLIAN HIEROGLYPH A525" },
    { 0x14642, "ANATOLIAN HIEROGLYPH A526" },
    { 0x14643, "ANATOLIAN HIEROGLYPH A527" },
    { 0x14644, "ANATOLIAN HIEROGLYPH A528" },
    { 0x14645, "ANATOLIAN HIEROGLYPH A529" },
    { 0x14646, "ANATOLIAN HIEROGLYPH A530" },
    { 0x16800, "BAMUM LETTER PHASE-A NGKUE MFON" },
    { 0x16801, "BAMUM LETTER PHASE-A GBIEE FON" },
    { 0x16802, "BAMUM LETTER PHASE-A PON MFON PIPAEMGBIEE" },
    { 0x16803, "BAMUM LETTER PHASE-A PON MFON PIPAEMBA" },
    { 0x16804, "BAMUM LETTER PHASE-A NAA MFON" },
    { 0x16805, "BAMUM LETTER PHASE-A SHUENSHUET" },
    { 0x16806, "BAMUM LETTER PHASE-A TITA MFON" },
    { 0x16807, "BAMUM LETTER PHASE-A NZA MFON" },
    { 0x16808, "BAMUM LETTER PHASE-A SHINDA PA NJI" },
    { 0x16809, "BAMUM LETTER PHASE-A PON PA NJI PIPAEMGBIEE" },
    { 0x1680A, "BAMUM LETTER PHASE-A PON PA NJI PIPAEMBA" },
    { 0x1680B, "BAMUM LETTER PHASE-A MAEMBGBIEE" },
    { 0x1680C, "BAMUM LETTER PHASE-A TU MAEMBA" },
    { 0x1680D, "BAMUM LETTER PHASE-A NGANGU" },
    { 0x1680E, "BAMUM LETTER PHASE-A MAEMVEUX" },
    { 0x1680F, "BAMUM LETTER PHASE-A MANSUAE" },
    { 0x16810, "BAMUM LETTER PHASE-A MVEUAENGAM" },
    { 0x16811, "BAMUM LETTER PHASE-A SEUNYAM" },
    { 0x16812, "BAMUM LETTER PHASE-A NTOQPEN" },
    { 0x16813, "BAMUM LETTER PHASE-A KEUKEUTNDA" },
    { 0x16814, "BAMUM LETTER PHASE-A NKINDI" },
    { 0x16815, "BAMUM LETTER PHASE-A SUU" },
    { 0x16816, "BAMUM LETTER PHASE-A NGKUENZEUM" },
    { 0x16817, "BAMUM LETTER PHASE-A LAPAQ" },
    { 0x16818, "BAMUM LETTER PHASE-A LET KUT" },
    { 0x16819, "BAMUM LETTER PHASE-A NTAP MFAA" },
    { 0x1681A, "BAMUM LETTER PHASE-A MAEKEUP" },
    { 0x1681B, "BAMUM LETTER PHASE-A PASHAE" },
    { 0x1681C, "BAMUM LETTER PHASE-A GHEUAERAE" },
    { 0x1681D, "BAMUM LETTER PHASE-A PAMSHAE" },
    { 0x1681E, "BAMUM LETTER PHASE-A MON NGGEUAET" },
    { 0x1681F, "BAMUM LETTER PHASE-A NZUN MEUT" },
    { 0x16820, "BAMUM LETTER PHASE-A U YUQ NAE" },
    { 0x16821, "BAMUM LETTER PHASE-A GHEUAEGHEUAE" },
    { 0x16822, "BAMUM LETTER PHASE-A NTAP NTAA" },
    { 0x16823, "BAMUM LETTER PHASE-A SISA" },
    { 0x16824, "BAMUM LETTER PHASE-A MGBASA" },
    { 0x16825, "BAMUM LETTER PHASE-A MEUNJOMNDEUQ" },
    { 0x16826, "BAMUM LETTER PHASE-A MOOMPUQ" },
    { 0x16827, "BAMUM LETTER PHASE-A KAFA" },
    { 0x16828, "BAMUM LETTER PHASE-A PA LEERAEWA" },
    { 0x16829, "BAMUM LETTER PHASE-A NDA LEERAEWA" },
    { 0x1682A, "BAMUM LETTER PHASE-A PET" },
    { 0x1682B, "BAMUM LETTER PHASE-A MAEMKPEN" },
    { 0x1682C, "BAMUM LETTER PHASE-A NIKA" },
    { 0x1682D, "BAMUM LETTER PHASE-A PUP" },
    { 0x1682E, "BAMUM LETTER PHASE-A TUAEP" },
    { 0x1682F, "BAMUM LETTER PHASE-A LUAEP" },
    { 0x16830, "BAMUM LETTER PHASE-A SONJAM" },
    { 0x16831, "BAMUM LETTER PHASE-A TEUTEUWEN" },
    { 0x16832, "BAMUM LETTER PHASE-A MAENYI" },
    { 0x16833, "BAMUM LETTER PHASE-A KET" },
    { 0x16834, "BAMUM LETTER PHASE-A NDAANGGEUAET" },
    { 0x16835, "BAMUM LETTER PHASE-A KUOQ" },
    { 0x16836, "BAMUM LETTER PHASE-A MOOMEUT" },
    { 0x16837, "BAMUM LETTER PHASE-A SHUM" },
    { 0x16838, "BAMUM LETTER PHASE-A LOMMAE" },
    { 0x16839, "BAMUM LETTER PHASE-A FIRI" },
    { 0x1683A, "BAMUM LETTER PHASE-A ROM" },
    { 0x1683B, "BAMUM LETTER PHASE-A KPOQ" },
    { 0x1683C, "BAMUM LETTER PHASE-A SOQ" },
    { 0x1683D, "BAMUM LETTER PHASE-A MAP PIEET" },
    { 0x1683E, "BAMUM LETTER PHASE-A SHIRAE" },
    { 0x1683F, "BAMUM LETTER PHASE-A NTAP" },
    { 0x16840, "BAMUM LETTER PHASE-A SHOQ NSHUT YUM" },
    { 0x16841, "BAMUM LETTER PHASE-A NYIT MONGKEUAEQ" },
    { 0x16842, "BAMUM LETTER PHASE-A PAARAE" },
    { 0x16843, "BAMUM LETTER PHASE-A NKAARAE" },
    { 0x16844, "BAMUM LETTER PHASE-A UNKNOWN" },
    { 0x16845, "BAMUM LETTER PHASE-A NGGEN" },
    { 0x16846, "BAMUM LETTER PHASE-A MAESI" },
    { 0x16847, "BAMUM LETTER PHASE-A NJAM" },
    { 0x16848, "BAMUM LETTER PHASE-A MBANYI" },
    { 0x16849, "BAMUM LETTER PHASE-A NYET" },
    { 0x1684A, "BAMUM LETTER PHASE-A TEUAEN" },
    { 0x1684B, "BAMUM LETTER PHASE-A SOT" },
    { 0x1684C, "BAMUM LETTER PHASE-A PAAM" },
    { 0x1684D, "BAMUM LETTER PHASE-A NSHIEE" },
    { 0x1684E, "BAMUM LETTER PHASE-A MAEM" },
    { 0x1684F, "BAMUM LETTER PHASE-A NYI" },
    { 0x16850, "BAMUM LETTER PHASE-A KAQ" },
    { 0x16851, "BAMUM LETTER PHASE-A NSHA" },
    { 0x16852, "BAMUM LETTER PHASE-A VEE" },
    { 0x16853, "BAMUM LETTER PHASE-A LU" },
    { 0x16854, "BAMUM LETTER PHASE-A NEN" },
    { 0x16855, "BAMUM LETTER PHASE-A NAQ" },
    { 0x16856, "BAMUM LETTER PHASE-A MBAQ" },
    { 0x16857, "BAMUM LETTER PHASE-B NSHUET" },
    { 0x16858, "BAMUM LETTER PHASE-B TU MAEMGBIEE" },
    { 0x16859, "BAMUM LETTER PHASE-B SIEE" },
    { 0x1685A, "BAMUM LETTER PHASE-B SET TU" },
    { 0x1685B, "BAMUM LETTER PHASE-B LOM NTEUM" },
    { 0x1685C, "BAMUM LETTER PHASE-B MBA MAELEE" },
    { 0x1685D, "BAMUM LETTER PHASE-B KIEEM" },
    { 0x1685E, "BAMUM LETTER PHASE-B YEURAE" },
    { 0x1685F, "BAMUM LETTER PHASE-B MBAARAE" },
    { 0x16860, "BAMUM LETTER PHASE-B KAM" },
    { 0x16861, "BAMUM LETTER PHASE-B PEESHI" },
    { 0x16862, "BAMUM LETTER PHASE-B YAFU LEERAEWA" },
    { 0x16863, "BAMUM LETTER PHASE-B LAM NSHUT NYAM" },
    { 0x16864, "BAMUM LETTER PHASE-B NTIEE SHEUOQ" },
    { 0x16865, "BAMUM LETTER PHASE-B NDU NJAA" },
    { 0x16866, "BAMUM LETTER PHASE-B GHEUGHEUAEM" },
    { 0x16867, "BAMUM LETTER PHASE-B PIT" },
    { 0x16868, "BAMUM LETTER PHASE-B TU NSIEE" },
    { 0x16869, "BAMUM LETTER PHASE-B SHET NJAQ" },
    { 0x1686A, "BAMUM LETTER PHASE-B SHEUAEQTU" },
    { 0x1686B, "BAMUM LETTER PHASE-B MFON TEUAEQ" },
    { 0x1686C, "BAMUM LETTER PHASE-B MBIT MBAAKET" },
    { 0x1686D, "BAMUM LETTER PHASE-B NYI NTEUM" },
    { 0x1686E, "BAMUM LETTER PHASE-B KEUPUQ" },
    { 0x1686F, "BAMUM LETTER PHASE-B GHEUGHEN" },
    { 0x16870, "BAMUM LETTER PHASE-B KEUYEUX" },
    { 0x16871, "BAMUM LETTER PHASE-B LAANAE" },
    { 0x16872, "BAMUM LETTER PHASE-B PARUM" },
    { 0x16873, "BAMUM LETTER PHASE-B VEUM" },
    { 0x16874, "BAMUM LETTER PHASE-B NGKINDI MVOP" },
    { 0x16875, "BAMUM LETTER PHASE-B NGGEU MBU" },
    { 0x16876, "BAMUM LETTER PHASE-B WUAET" },
    { 0x16877, "BAMUM LETTER PHASE-B SAKEUAE" },
    { 0x16878, "BAMUM LETTER PHASE-B TAAM" },
    { 0x16879, "BAMUM LETTER PHASE-B MEUQ" },
    { 0x1687A, "BAMUM LETTER PHASE-B NGGUOQ" },
    { 0x1687B, "BAMUM LETTER PHASE-B NGGUOQ LARGE" },
    { 0x1687C, "BAMUM LETTER PHASE-B MFIYAQ" },
    { 0x1687D, "BAMUM LETTER PHASE-B SUE" },
    { 0x1687E, "BAMUM LETTER PHASE-B MBEURI" },
    { 0x1687F, "BAMUM LETTER PHASE-B MONTIEEN" },
    { 0x16880, "BAMUM LETTER PHASE-B NYAEMAE" },
    { 0x16881, "BAMUM LETTER PHASE-B PUNGAAM" },
    { 0x16882, "BAMUM LETTER PHASE-B MEUT NGGEET" },
    { 0x16883, "BAMUM LETTER PHASE-B FEUX" },
    { 0x16884, "BAMUM LETTER PHASE-B MBUOQ" },
    { 0x16885, "BAMUM LETTER PHASE-B FEE" },
    { 0x16886, "BAMUM LETTER PHASE-B KEUAEM" },
    { 0x16887, "BAMUM LETTER PHASE-B MA NJEUAENA" },
    { 0x16888, "BAMUM LETTER PHASE-B MA NJUQA" },
    { 0x16889, "BAMUM LETTER PHASE-B LET" },
    { 0x1688A, "BAMUM LETTER PHASE-B NGGAAM" },
    { 0x1688B, "BAMUM LETTER PHASE-B NSEN" },
    { 0x1688C, "BAMUM LETTER PHASE-B MA" },
    { 0x1688D, "BAMUM LETTER PHASE-B KIQ" },
    { 0x1688E, "BAMUM LETTER PHASE-B NGOM" },
    { 0x1688F, "BAMUM LETTER PHASE-C NGKUE MAEMBA" },
    { 0x16890, "BAMUM LETTER PHASE-C NZA" },
    { 0x16891, "BAMUM LETTER PHASE-C YUM" },
    { 0x16892, "BAMUM LETTER PHASE-C WANGKUOQ" },
    { 0x16893, "BAMUM LETTER PHASE-C NGGEN" },
    { 0x16894, "BAMUM LETTER PHASE-C NDEUAEREE" },
    { 0x16895, "BAMUM LETTER PHASE-C NGKAQ" },
    { 0x16896, "BAMUM LETTER PHASE-C GHARAE" },
    { 0x16897, "BAMUM LETTER PHASE-C MBEEKEET" },
    { 0x16898, "BAMUM LETTER PHASE-C GBAYI" },
    { 0x16899, "BAMUM LETTER PHASE-C NYIR MKPARAQ MEUN" },
    { 0x1689A, "BAMUM LETTER PHASE-C NTU MBIT" },
    { 0x1689B, "BAMUM LETTER PHASE-C MBEUM" },
    { 0x1689C, "BAMUM LETTER PHASE-C PIRIEEN" },
    { 0x1689D, "BAMUM LETTER PHASE-C NDOMBU" },
    { 0x1689E, "BAMUM LETTER PHASE-C MBAA CABBAGE-TREE" },
    { 0x1689F, "BAMUM LETTER PHASE-C KEUSHEUAEP" },
    { 0x168A0, "BAMUM LETTER PHASE-C GHAP" },
    { 0x168A1, "BAMUM LETTER PHASE-C KEUKAQ" },
    { 0x168A2, "BAMUM LETTER PHASE-C YU MUOMAE" },
    { 0x168A3, "BAMUM LETTER PHASE-C NZEUM" },
    { 0x168A4, "BAMUM LETTER PHASE-C MBUE" },
    { 0x168A5, "BAMUM LETTER PHASE-C NSEUAEN" },
    { 0x168A6, "BAMUM LETTER PHASE-C MBIT" },
    { 0x168A7, "BAMUM LETTER PHASE-C YEUQ" },
    { 0x168A8, "BAMUM LETTER PHASE-C KPARAQ" },
    { 0x168A9, "BAMUM LETTER PHASE-C KAA" },
    { 0x168AA, "BAMUM LETTER PHASE-C SEUX" },
    { 0x168AB, "BAMUM LETTER PHASE-C NDIDA" },
    { 0x168AC, "BAMUM LETTER PHASE-C TAASHAE" },
    { 0x168AD, "BAMUM LETTER PHASE-C NJUEQ" },
    { 0x168AE, "BAMUM LETTER PHASE-C TITA YUE" },
    { 0x168AF, "BAMUM LETTER PHASE-C SUAET" },
    { 0x168B0, "BAMUM LETTER PHASE-C NGGUAEN NYAM" },
    { 0x168B1, "BAMUM LETTER PHASE-C VEUX" },
    { 0x168B2, "BAMUM LETTER PHASE-C NANSANAQ" },
    { 0x168B3, "BAMUM LETTER PHASE-C MA KEUAERI" },
    { 0x168B4, "BAMUM LETTER PHASE-C NTAA" },
    { 0x168B5, "BAMUM LETTER PHASE-C NGGUON" },
    { 0x168B6, "BAMUM LETTER PHASE-C LAP" },
    { 0x168B7, "BAMUM LETTER PHASE-C MBIRIEEN" },
    { 0x168B8, "BAMUM LETTER PHASE-C MGBASAQ" },
    { 0x168B9, "BAMUM LETTER PHASE-C NTEUNGBA" },
    { 0x168BA, "BAMUM LETTER PHASE-C TEUTEUX" },
    { 0x168BB, "BAMUM LETTER PHASE-C NGGUM" },
    { 0x168BC, "BAMUM LETTER PHASE-C FUE" },
    { 0x168BD, "BAMUM LETTER PHASE-C NDEUT" },
    { 0x168BE, "BAMUM LETTER PHASE-C NSA" },
    { 0x168BF, "BAMUM LETTER PHASE-C NSHAQ" },
    { 0x168C0, "BAMUM LETTER PHASE-C BUNG" },
    { 0x168C1, "BAMUM LETTER PHASE-C VEUAEPEN" },
    { 0x168C2, "BAMUM LETTER PHASE-C MBERAE" },
    { 0x168C3, "BAMUM LETTER PHASE-C RU" },
    { 0x168C4, "BAMUM LETTER PHASE-C NJAEM" },
    { 0x168C5, "BAMUM LETTER PHASE-C LAM" },
    { 0x168C6, "BAMUM LETTER PHASE-C TITUAEP" },
    { 0x168C7, "BAMUM LETTER PHASE-C NSUOT NGOM" },
    { 0x168C8, "BAMUM LETTER PHASE-C NJEEEE" },
    { 0x168C9, "BAMUM LETTER PHASE-C KET" },
    { 0x168CA, "BAMUM LETTER PHASE-C NGGU" },
    { 0x168CB, "BAMUM LETTER PHASE-C MAESI" },
    { 0x168CC, "BAMUM LETTER PHASE-C MBUAEM" },
    { 0x168CD, "BAMUM LETTER PHASE-C LU" },
    { 0x168CE, "BAMUM LETTER PHASE-C KUT" },
    { 0x168CF, "BAMUM LETTER PHASE-C NJAM" },
    { 0x168D0, "BAMUM LETTER PHASE-C NGOM" },
    { 0x168D1, "BAMUM LETTER PHASE-C WUP" },
    { 0x168D2, "BAMUM LETTER PHASE-C NGGUEET" },
    { 0x168D3, "BAMUM LETTER PHASE-C NSOM" },
    { 0x168D4, "BAMUM LETTER PHASE-C NTEN" },
    { 0x168D5, "BAMUM LETTER PHASE-C KUOP NKAARAE" },
    { 0x168D6, "BAMUM LETTER PHASE-C NSUN" },
    { 0x168D7, "BAMUM LETTER PHASE-C NDAM" },
    { 0x168D8, "BAMUM LETTER PHASE-C MA NSIEE" },
    { 0x168D9, "BAMUM LETTER PHASE-C YAA" },
    { 0x168DA, "BAMUM LETTER PHASE-C NDAP" },
    { 0x168DB, "BAMUM LETTER PHASE-C SHUEQ" },
    { 0x168DC, "BAMUM LETTER PHASE-C SETFON" },
    { 0x168DD, "BAMUM LETTER PHASE-C MBI" },
    { 0x168DE, "BAMUM LETTER PHASE-C MAEMBA" },
    { 0x168DF, "BAMUM LETTER PHASE-C MBANYI" },
    { 0x168E0, "BAMUM LETTER PHASE-C KEUSEUX" },
    { 0x168E1, "BAMUM LETTER PHASE-C MBEUX" },
    { 0x168E2, "BAMUM LETTER PHASE-C KEUM" },
    { 0x168E3, "BAMUM LETTER PHASE-C MBAA PICKET" },
    { 0x168E4, "BAMUM LETTER PHASE-C YUWOQ" },
    { 0x168E5, "BAMUM LETTER PHASE-C NJEUX" },
    { 0x168E6, "BAMUM LETTER PHASE-C MIEE" },
    { 0x168E7, "BAMUM LETTER PHASE-C MUAE" },
    { 0x168E8, "BAMUM LETTER PHASE-C SHIQ" },
    { 0x168E9, "BAMUM LETTER PHASE-C KEN LAW" },
    { 0x168EA, "BAMUM LETTER PHASE-C KEN FATIGUE" },
    { 0x168EB, "BAMUM LETTER PHASE-C NGAQ" },
    { 0x168EC, "BAMUM LETTER PHASE-C NAQ" },
    { 0x168ED, "BAMUM LETTER PHASE-C LIQ" },
    { 0x168EE, "BAMUM LETTER PHASE-C PIN" },
    { 0x168EF, "BAMUM LETTER PHASE-C PEN" },
    { 0x168F0, "BAMUM LETTER PHASE-C TET" },
    { 0x168F1, "BAMUM LETTER PHASE-D MBUO" },
    { 0x168F2, "BAMUM LETTER PHASE-D WAP" },
    { 0x168F3, "BAMUM LETTER PHASE-D NJI" },
    { 0x168F4, "BAMUM LETTER PHASE-D MFON" },
    { 0x168F5, "BAMUM LETTER PHASE-D NJIEE" },
    { 0x168F6, "BAMUM LETTER PHASE-D LIEE" },
    { 0x168F7, "BAMUM LETTER PHASE-D NJEUT" },
    { 0x168F8, "BAMUM LETTER PHASE-D NSHEE" },
    { 0x168F9, "BAMUM LETTER PHASE-D NGGAAMAE" },
    { 0x168FA, "BAMUM LETTER PHASE-D NYAM" },
    { 0x168FB, "BAMUM LETTER PHASE-D WUAEN" },
    { 0x168FC, "BAMUM LETTER PHASE-D NGKUN" },
    { 0x168FD, "BAMUM LETTER PHASE-D SHEE" },
    { 0x168FE, "BAMUM LETTER PHASE-D NGKAP" },
    { 0x168FF, "BAMUM LETTER PHASE-D KEUAETMEUN" },
    { 0x16900, "BAMUM LETTER PHASE-D TEUT" },
    { 0x16901, "BAMUM LETTER PHASE-D SHEUAE" },
    { 0x16902, "BAMUM LETTER PHASE-D NJAP" },
    { 0x16903, "BAMUM LETTER PHASE-D SUE" },
    { 0x16904, "BAMUM LETTER PHASE-D KET" },
    { 0x16905, "BAMUM LETTER PHASE-D YAEMMAE" },
    { 0x16906, "BAMUM LETTER PHASE-D KUOM" },
    { 0x16907, "BAMUM LETTER PHASE-D SAP" },
    { 0x16908, "BAMUM LETTER PHASE-D MFEUT" },
    { 0x16909, "BAMUM LETTER PHASE-D NDEUX" },
    { 0x1690A, "BAMUM LETTER PHASE-D MALEERI" },
    { 0x1690B, "BAMUM LETTER PHASE-D MEUT" },
    { 0x1690C, "BAMUM LETTER PHASE-D SEUAEQ" },
    { 0x1690D, "BAMUM LETTER PHASE-D YEN" },
    { 0x1690E, "BAMUM LETTER PHASE-D NJEUAEM" },
    { 0x1690F, "BAMUM LETTER PHASE-D KEUOT MBUAE" },
    { 0x16910, "BAMUM LETTER PHASE-D NGKEURI" },
    { 0x16911, "BAMUM LETTER PHASE-D TU" },
    { 0x16912, "BAMUM LETTER PHASE-D GHAA" },
    { 0x16913, "BAMUM LETTER PHASE-D NGKYEE" },
    { 0x16914, "BAMUM LETTER PHASE-D FEUFEUAET" },
    { 0x16915, "BAMUM LETTER PHASE-D NDEE" },
    { 0x16916, "BAMUM LETTER PHASE-D MGBOFUM" },
    { 0x16917, "BAMUM LETTER PHASE-D LEUAEP" },
    { 0x16918, "BAMUM LETTER PHASE-D NDON" },
    { 0x16919, "BAMUM LETTER PHASE-D MONI" },
    { 0x1691A, "BAMUM LETTER PHASE-D MGBEUN" },
    { 0x1691B, "BAMUM LETTER PHASE-D PUUT" },
    { 0x1691C, "BAMUM LETTER PHASE-D MGBIEE" },
    { 0x1691D, "BAMUM LETTER PHASE-D MFO" },
    { 0x1691E, "BAMUM LETTER PHASE-D LUM" },
    { 0x1691F, "BAMUM LETTER PHASE-D NSIEEP" },
    { 0x16920, "BAMUM LETTER PHASE-D MBAA" },
    { 0x16921, "BAMUM LETTER PHASE-D KWAET" },
    { 0x16922, "BAMUM LETTER PHASE-D NYET" },
    { 0x16923, "BAMUM LETTER PHASE-D TEUAEN" },
    { 0x16924, "BAMUM LETTER PHASE-D SOT" },
    { 0x16925, "BAMUM LETTER PHASE-D YUWOQ" },
    { 0x16926, "BAMUM LETTER PHASE-D KEUM" },
    { 0x16927, "BAMUM LETTER PHASE-D RAEM" },
    { 0x16928, "BAMUM LETTER PHASE-D TEEEE" },
    { 0x16929, "BAMUM LETTER PHASE-D NGKEUAEQ" },
    { 0x1692A, "BAMUM LETTER PHASE-D MFEUAE" },
    { 0x1692B, "BAMUM LETTER PHASE-D NSIEET" },
    { 0x1692C, "BAMUM LETTER PHASE-D KEUP" },
    { 0x1692D, "BAMUM LETTER PHASE-D PIP" },
    { 0x1692E, "BAMUM LETTER PHASE-D PEUTAE" },
    { 0x1692F, "BAMUM LETTER PHASE-D NYUE" },
    { 0x16930, "BAMUM LETTER PHASE-D LET" },
    { 0x16931, "BAMUM LETTER PHASE-D NGGAAM" },
    { 0x16932, "BAMUM LETTER PHASE-D MFIEE" },
    { 0x16933, "BAMUM LETTER PHASE-D NGGWAEN" },
    { 0x16934, "BAMUM LETTER PHASE-D YUOM" },
    { 0x16935, "BAMUM LETTER PHASE-D PAP" },
    { 0x16936, "BAMUM LETTER PHASE-D YUOP" },
    { 0x16937, "BAMUM LETTER PHASE-D NDAM" },
    { 0x16938, "BAMUM LETTER PHASE-D NTEUM" },
    { 0x16939, "BAMUM LETTER PHASE-D SUAE" },
    { 0x1693A, "BAMUM LETTER PHASE-D KUN" },
    { 0x1693B, "BAMUM LETTER PHASE-D NGGEUX" },
    { 0x1693C, "BAMUM LETTER PHASE-D NGKIEE" },
    { 0x1693D, "BAMUM LETTER PHASE-D TUOT" },
    { 0x1693E, "BAMUM LETTER PHASE-D MEUN" },
    { 0x1693F, "BAMUM LETTER PHASE-D KUQ" },
    { 0x16940, "BAMUM LETTER PHASE-D NSUM" },
    { 0x16941, "BAMUM LETTER PHASE-D TEUN" },
    { 0x16942, "BAMUM LETTER PHASE-D MAENJET" },
    { 0x16943, "BAMUM LETTER PHASE-D NGGAP" },
    { 0x16944, "BAMUM LETTER PHASE-D LEUM" },
    { 0x16945, "BAMUM LETTER PHASE-D NGGUOM" },
    { 0x16946, "BAMUM LETTER PHASE-D NSHUT" },
    { 0x16947, "BAMUM LETTER PHASE-D NJUEQ" },
    { 0x16948, "BAMUM LETTER PHASE-D GHEUAE" },
    { 0x16949, "BAMUM LETTER PHASE-D KU" },
    { 0x1694A, "BAMUM LETTER PHASE-D REN OLD" },
    { 0x1694B, "BAMUM LETTER PHASE-D TAE" },
    { 0x1694C, "BAMUM LETTER PHASE-D TOQ" },
    { 0x1694D, "BAMUM LETTER PHASE-D NYI" },
    { 0x1694E, "BAMUM LETTER PHASE-D RII" },
    { 0x1694F, "BAMUM LETTER PHASE-D LEEEE" },
    { 0x16950, "BAMUM LETTER PHASE-D MEEEE" },
    { 0x16951, "BAMUM LETTER PHASE-D M" },
    { 0x16952, "BAMUM LETTER PHASE-D SUU" },
    { 0x16953, "BAMUM LETTER PHASE-D MU" },
    { 0x16954, "BAMUM LETTER PHASE-D SHII" },
    { 0x16955, "BAMUM LETTER PHASE-D SHEUX" },
    { 0x16956, "BAMUM LETTER PHASE-D KYEE" },
    { 0x16957, "BAMUM LETTER PHASE-D NU" },
    { 0x16958, "BAMUM LETTER PHASE-D SHU" },
    { 0x16959, "BAMUM LETTER PHASE-D NTEE" },
    { 0x1695A, "BAMUM LETTER PHASE-D PEE" },
    { 0x1695B, "BAMUM LETTER PHASE-D NI" },
    { 0x1695C, "BAMUM LETTER PHASE-D SHOQ" },
    { 0x1695D, "BAMUM LETTER PHASE-D PUQ" },
    { 0x1695E, "BAMUM LETTER PHASE-D MVOP" },
    { 0x1695F, "BAMUM LETTER PHASE-D LOQ" },
    { 0x16960, "BAMUM LETTER PHASE-D REN MUCH" },
    { 0x16961, "BAMUM LETTER PHASE-D TI" },
    { 0x16962, "BAMUM LETTER PHASE-D NTUU" },
    { 0x16963, "BAMUM LETTER PHASE-D MBAA SEVEN" },
    { 0x16964, "BAMUM LETTER PHASE-D SAQ" },
    { 0x16965, "BAMUM LETTER PHASE-D FAA" },
    { 0x16966, "BAMUM LETTER PHASE-E NDAP" },
    { 0x16967, "BAMUM LETTER PHASE-E TOON" },
    { 0x16968, "BAMUM LETTER PHASE-E MBEUM" },
    { 0x16969, "BAMUM LETTER PHASE-E LAP" },
    { 0x1696A, "BAMUM LETTER PHASE-E VOM" },
    { 0x1696B, "BAMUM LETTER PHASE-E LOON" },
    { 0x1696C, "BAMUM LETTER PHASE-E PAA" },
    { 0x1696D, "BAMUM LETTER PHASE-E SOM" },
    { 0x1696E, "BAMUM LETTER PHASE-E RAQ" },
    { 0x1696F, "BAMUM LETTER PHASE-E NSHUOP" },
    { 0x16970, "BAMUM LETTER PHASE-E NDUN" },
    { 0x16971, "BAMUM LETTER PHASE-E PUAE" },
    { 0x16972, "BAMUM LETTER PHASE-E TAM" },
    { 0x16973, "BAMUM LETTER PHASE-E NGKA" },
    { 0x16974, "BAMUM LETTER PHASE-E KPEUX" },
    { 0x16975, "BAMUM LETTER PHASE-E WUO" },
    { 0x16976, "BAMUM LETTER PHASE-E SEE" },
    { 0x16977, "BAMUM LETTER PHASE-E NGGEUAET" },
    { 0x16978, "BAMUM LETTER PHASE-E PAAM" },
    { 0x16979, "BAMUM LETTER PHASE-E TOO" },
    { 0x1697A, "BAMUM LETTER PHASE-E KUOP" },
    { 0x1697B, "BAMUM LETTER PHASE-E LOM" },
    { 0x1697C, "BAMUM LETTER PHASE-E NSHIEE" },
    { 0x1697D, "BAMUM LETTER PHASE-E NGOP" },
    { 0x1697E, "BAMUM LETTER PHASE-E MAEM" },
    { 0x1697F, "BAMUM LETTER PHASE-E NGKEUX" },
    { 0x16980, "BAMUM LETTER PHASE-E NGOQ" },
    { 0x16981, "BAMUM LETTER PHASE-E NSHUE" },
    { 0x16982, "BAMUM LETTER PHASE-E RIMGBA" },
    { 0x16983, "BAMUM LETTER PHASE-E NJEUX" },
    { 0x16984, "BAMUM LETTER PHASE-E PEEM" },
    { 0x16985, "BAMUM LETTER PHASE-E SAA" },
    { 0x16986, "BAMUM LETTER PHASE-E NGGURAE" },
    { 0x16987, "BAMUM LETTER PHASE-E MGBA" },
    { 0x16988, "BAMUM LETTER PHASE-E GHEUX" },
    { 0x16989, "BAMUM LETTER PHASE-E NGKEUAEM" },
    { 0x1698A, "BAMUM LETTER PHASE-E NJAEMLI" },
    { 0x1698B, "BAMUM LETTER PHASE-E MAP" },
    { 0x1698C, "BAMUM LETTER PHASE-E LOOT" },
    { 0x1698D, "BAMUM LETTER PHASE-E NGGEEEE" },
    { 0x1698E, "BAMUM LETTER PHASE-E NDIQ" },
    { 0x1698F, "BAMUM LETTER PHASE-E TAEN NTEUM" },
    { 0x16990, "BAMUM LETTER PHASE-E SET" },
    { 0x16991, "BAMUM LETTER PHASE-E PUM" },
    { 0x16992, "BAMUM LETTER PHASE-E NDAA SOFTNESS" },
    { 0x16993, "BAMUM LETTER PHASE-E NGGUAESHAE NYAM" },
    { 0x16994, "BAMUM LETTER PHASE-E YIEE" },
    { 0x16995, "BAMUM LETTER PHASE-E GHEUN" },
    { 0x16996, "BAMUM LETTER PHASE-E TUAE" },
    { 0x16997, "BAMUM LETTER PHASE-E YEUAE" },
    { 0x16998, "BAMUM LETTER PHASE-E PO" },
    { 0x16999, "BAMUM LETTER PHASE-E TUMAE" },
    { 0x1699A, "BAMUM LETTER PHASE-E KEUAE" },
    { 0x1699B, "BAMUM LETTER PHASE-E SUAEN" },
    { 0x1699C, "BAMUM LETTER PHASE-E TEUAEQ" },
    { 0x1699D, "BAMUM LETTER PHASE-E VEUAE" },
    { 0x1699E, "BAMUM LETTER PHASE-E WEUX" },
    { 0x1699F, "BAMUM LETTER PHASE-E LAAM" },
    { 0x169A0, "BAMUM LETTER PHASE-E PU" },
    { 0x169A1, "BAMUM LETTER PHASE-E TAAQ" },
    { 0x169A2, "BAMUM LETTER PHASE-E GHAAMAE" },
    { 0x169A3, "BAMUM LETTER PHASE-E NGEUREUT" },
    { 0x169A4, "BAMUM LETTER PHASE-E SHEUAEQ" },
    { 0x169A5, "BAMUM LETTER PHASE-E MGBEN" },
    { 0x169A6, "BAMUM LETTER PHASE-E MBEE" },
    { 0x169A7, "BAMUM LETTER PHASE-E NZAQ" },
    { 0x169A8, "BAMUM LETTER PHASE-E NKOM" },
    { 0x169A9, "BAMUM LETTER PHASE-E GBET" },
    { 0x169AA, "BAMUM LETTER PHASE-E TUM" },
    { 0x169AB, "BAMUM LETTER PHASE-E KUET" },
    { 0x169AC, "BAMUM LETTER PHASE-E YAP" },
    { 0x169AD, "BAMUM LETTER PHASE-E NYI CLEAVER" },
    { 0x169AE, "BAMUM LETTER PHASE-E YIT" },
    { 0x169AF, "BAMUM LETTER PHASE-E MFEUQ" },
    { 0x169B0, "BAMUM LETTER PHASE-E NDIAQ" },
    { 0x169B1, "BAMUM LETTER PHASE-E PIEEQ" },
    { 0x169B2, "BAMUM LETTER PHASE-E YUEQ" },
    { 0x169B3, "BAMUM LETTER PHASE-E LEUAEM" },
    { 0x169B4, "BAMUM LETTER PHASE-E FUE" },
    { 0x169B5, "BAMUM LETTER PHASE-E GBEUX" },
    { 0x169B6, "BAMUM LETTER PHASE-E NGKUP" },
    { 0x169B7, "BAMUM LETTER PHASE-E KET" },
    { 0x169B8, "BAMUM LETTER PHASE-E MAE" },
    { 0x169B9, "BAMUM LETTER PHASE-E NGKAAMI" },
    { 0x169BA, "BAMUM LETTER PHASE-E GHET" },
    { 0x169BB, "BAMUM LETTER PHASE-E FA" },
    { 0x169BC, "BAMUM LETTER PHASE-E NTUM" },
    { 0x169BD, "BAMUM LETTER PHASE-E PEUT" },
    { 0x169BE, "BAMUM LETTER PHASE-E YEUM" },
    { 0x169BF, "BAMUM LETTER PHASE-E NGGEUAE" },
    { 0x169C0, "BAMUM LETTER PHASE-E NYI BETWEEN" },
    { 0x169C1, "BAMUM LETTER PHASE-E NZUQ" },
    { 0x169C2, "BAMUM LETTER PHASE-E POON" },
    { 0x169C3, "BAMUM LETTER PHASE-E MIEE" },
    { 0x169C4, "BAMUM LETTER PHASE-E FUET" },
    { 0x169C5, "BAMUM LETTER PHASE-E NAE" },
    { 0x169C6, "BAMUM LETTER PHASE-E MUAE" },
    { 0x169C7, "BAMUM LETTER PHASE-E GHEUAE" },
    { 0x169C8, "BAMUM LETTER PHASE-E FU I" },
    { 0x169C9, "BAMUM LETTER PHASE-E MVI" },
    { 0x169CA, "BAMUM LETTER PHASE-E PUAQ" },
    { 0x169CB, "BAMUM LETTER PHASE-E NGKUM" },
    { 0x169CC, "BAMUM LETTER PHASE-E KUT" },
    { 0x169CD, "BAMUM LETTER PHASE-E PIET" },
    { 0x169CE, "BAMUM LETTER PHASE-E NTAP" },
    { 0x169CF, "BAMUM LETTER PHASE-E YEUAET" },
    { 0x169D0, "BAMUM LETTER PHASE-E NGGUP" },
    { 0x169D1, "BAMUM LETTER PHASE-E PA PEOPLE" },
    { 0x169D2, "BAMUM LETTER PHASE-E FU CALL" },
    { 0x169D3, "BAMUM LETTER PHASE-E FOM" },
    { 0x169D4, "BAMUM LETTER PHASE-E NJEE" },
    { 0x169D5, "BAMUM LETTER PHASE-E A" },
    { 0x169D6, "BAMUM LETTER PHASE-E TOQ" },
    { 0x169D7, "BAMUM LETTER PHASE-E O" },
    { 0x169D8, "BAMUM LETTER PHASE-E I" },
    { 0x169D9, "BAMUM LETTER PHASE-E LAQ" },
    { 0x169DA, "BAMUM LETTER PHASE-E PA PLURAL" },
    { 0x169DB, "BAMUM LETTER PHASE-E TAA" },
    { 0x169DC, "BAMUM LETTER PHASE-E TAQ" },
    { 0x169DD, "BAMUM LETTER PHASE-E NDAA MY HOUSE" },
    { 0x169DE, "BAMUM LETTER PHASE-E SHIQ" },
    { 0x169DF, "BAMUM LETTER PHASE-E YEUX" },
    { 0x169E0, "BAMUM LETTER PHASE-E NGUAE" },
    { 0x169E1, "BAMUM LETTER PHASE-E YUAEN" },
    { 0x169E2, "BAMUM LETTER PHASE-E YOQ SWIMMING" },
    { 0x169E3, "BAMUM LETTER PHASE-E YOQ COVER" },
    { 0x169E4, "BAMUM LETTER PHASE-E YUQ" },
    { 0x169E5, "BAMUM LETTER PHASE-E YUN" },
    { 0x169E6, "BAMUM LETTER PHASE-E KEUX" },
    { 0x169E7, "BAMUM LETTER PHASE-E PEUX" },
    { 0x169E8, "BAMUM LETTER PHASE-E NJEE EPOCH" },
    { 0x169E9, "BAMUM LETTER PHASE-E PUE" },
    { 0x169EA, "BAMUM LETTER PHASE-E WUE" },
    { 0x169EB, "BAMUM LETTER PHASE-E FEE" },
    { 0x169EC, "BAMUM LETTER PHASE-E VEE" },
    { 0x169ED, "BAMUM LETTER PHASE-E LU" },
    { 0x169EE, "BAMUM LETTER PHASE-E MI" },
    { 0x169EF, "BAMUM LETTER PHASE-E REUX" },
    { 0x169F0, "BAMUM LETTER PHASE-E RAE" },
    { 0x169F1, "BAMUM LETTER PHASE-E NGUAET" },
    { 0x169F2, "BAMUM LETTER PHASE-E NGA" },
    { 0x169F3, "BAMUM LETTER PHASE-E SHO" },
    { 0x169F4, "BAMUM LETTER PHASE-E SHOQ" },
    { 0x169F5, "BAMUM LETTER PHASE-E FU REMEDY" },
    { 0x169F6, "BAMUM LETTER PHASE-E NA" },
    { 0x169F7, "BAMUM LETTER PHASE-E PI" },
    { 0x169F8, "BAMUM LETTER PHASE-E LOQ" },
    { 0x169F9, "BAMUM LETTER PHASE-E KO" },
    { 0x169FA, "BAMUM LETTER PHASE-E MEN" },
    { 0x169FB, "BAMUM LETTER PHASE-E MA" },
    { 0x169FC, "BAMUM LETTER PHASE-E MAQ" },
    { 0x169FD, "BAMUM LETTER PHASE-E TEU" },
    { 0x169FE, "BAMUM LETTER PHASE-E KI" },
    { 0x169FF, "BAMUM LETTER PHASE-E MON" },
    { 0x16A00, "BAMUM LETTER PHASE-E TEN" },
    { 0x16A01, "BAMUM LETTER PHASE-E FAQ" },
    { 0x16A02, "BAMUM LETTER PHASE-E GHOM" },
    { 0x16A03, "BAMUM LETTER PHASE-F KA" },
    { 0x16A04, "BAMUM LETTER PHASE-F U" },
    { 0x16A05, "BAMUM LETTER PHASE-F KU" },
    { 0x16A06, "BAMUM LETTER PHASE-F EE" },
    { 0x16A07, "BAMUM LETTER PHASE-F REE" },
    { 0x16A08, "BAMUM LETTER PHASE-F TAE" },
    { 0x16A09, "BAMUM LETTER PHASE-F NYI" },
    { 0x16A0A, "BAMUM LETTER PHASE-F LA" },
    { 0x16A0B, "BAMUM LETTER PHASE-F RII" },
    { 0x16A0C, "BAMUM LETTER PHASE-F RIEE" },
    { 0x16A0D, "BAMUM LETTER PHASE-F MEEEE" },
    { 0x16A0E, "BAMUM LETTER PHASE-F TAA" },
    { 0x16A0F, "BAMUM LETTER PHASE-F NDAA" },
    { 0x16A10, "BAMUM LETTER PHASE-F NJAEM" },
    { 0x16A11, "BAMUM LETTER PHASE-F M" },
    { 0x16A12, "BAMUM LETTER PHASE-F SUU" },
    { 0x16A13, "BAMUM LETTER PHASE-F SHII" },
    { 0x16A14, "BAMUM LETTER PHASE-F SI" },
    { 0x16A15, "BAMUM LETTER PHASE-F SEUX" },
    { 0x16A16, "BAMUM LETTER PHASE-F KYEE" },
    { 0x16A17, "BAMUM LETTER PHASE-F KET" },
    { 0x16A18, "BAMUM LETTER PHASE-F NUAE" },
    { 0x16A19, "BAMUM LETTER PHASE-F NU" },
    { 0x16A1A, "BAMUM LETTER PHASE-F NJUAE" },
    { 0x16A1B, "BAMUM LETTER PHASE-F YOQ" },
    { 0x16A1C, "BAMUM LETTER PHASE-F SHU" },
    { 0x16A1D, "BAMUM LETTER PHASE-F YA" },
    { 0x16A1E, "BAMUM LETTER PHASE-F NSHA" },
    { 0x16A1F, "BAMUM LETTER PHASE-F PEUX" },
    { 0x16A20, "BAMUM LETTER PHASE-F NTEE" },
    { 0x16A21, "BAMUM LETTER PHASE-F WUE" },
    { 0x16A22, "BAMUM LETTER PHASE-F PEE" },
    { 0x16A23, "BAMUM LETTER PHASE-F RU" },
    { 0x16A24, "BAMUM LETTER PHASE-F NI" },
    { 0x16A25, "BAMUM LETTER PHASE-F REUX" },
    { 0x16A26, "BAMUM LETTER PHASE-F KEN" },
    { 0x16A27, "BAMUM LETTER PHASE-F NGKWAEN" },
    { 0x16A28, "BAMUM LETTER PHASE-F NGGA" },
    { 0x16A29, "BAMUM LETTER PHASE-F SHO" },
    { 0x16A2A, "BAMUM LETTER PHASE-F PUAE" },
    { 0x16A2B, "BAMUM LETTER PHASE-F FOM" },
    { 0x16A2C, "BAMUM LETTER PHASE-F WA" },
    { 0x16A2D, "BAMUM LETTER PHASE-F LI" },
    { 0x16A2E, "BAMUM LETTER PHASE-F LOQ" },
    { 0x16A2F, "BAMUM LETTER PHASE-F KO" },
    { 0x16A30, "BAMUM LETTER PHASE-F MBEN" },
    { 0x16A31, "BAMUM LETTER PHASE-F REN" },
    { 0x16A32, "BAMUM LETTER PHASE-F MA" },
    { 0x16A33, "BAMUM LETTER PHASE-F MO" },
    { 0x16A34, "BAMUM LETTER PHASE-F MBAA" },
    { 0x16A35, "BAMUM LETTER PHASE-F TET" },
    { 0x16A36, "BAMUM LETTER PHASE-F KPA" },
    { 0x16A37, "BAMUM LETTER PHASE-F SAMBA" },
    { 0x16A38, "BAMUM LETTER PHASE-F VUEQ" },
    { 0x16A40, "MRO LETTER TA" },
    { 0x16A41, "MRO LETTER NGI" },
    { 0x16A42, "MRO LETTER YO" },
    { 0x16A43, "MRO LETTER MIM" },
    { 0x16A44, "MRO LETTER BA" },
    { 0x16A45, "MRO LETTER DA" },
    { 0x16A46, "MRO LETTER A" },
    { 0x16A47, "MRO LETTER PHI" },
    { 0x16A48, "MRO LETTER KHAI" },
    { 0x16A49, "MRO LETTER HAO" },
    { 0x16A4A, "MRO LETTER DAI" },
    { 0x16A4B, "MRO LETTER CHU" },
    { 0x16A4C, "MRO LETTER KEAAE" },
    { 0x16A4D, "MRO LETTER OL" },
    { 0x16A4E, "MRO LETTER MAEM" },
    { 0x16A4F, "MRO LETTER NIN" },
    { 0x16A50, "MRO LETTER PA" },
    { 0x16A51, "MRO LETTER OO" },
    { 0x16A52, "MRO LETTER O" },
    { 0x16A53, "MRO LETTER RO" },
    { 0x16A54, "MRO LETTER SHI" },
    { 0x16A55, "MRO LETTER THEA" },
    { 0x16A56, "MRO LETTER EA" },
    { 0x16A57, "MRO LETTER WA" },
    { 0x16A58, "MRO LETTER E" },
    { 0x16A59, "MRO LETTER KO" },
    { 0x16A5A, "MRO LETTER LAN" },
    { 0x16A5B, "MRO LETTER LA" },
    { 0x16A5C, "MRO LETTER HAI" },
    { 0x16A5D, "MRO LETTER RI" },
    { 0x16A5E, "MRO LETTER TEK" },
    { 0x16A60, "MRO DIGIT ZERO" },
    { 0x16A61, "MRO DIGIT ONE" },
    { 0x16A62, "MRO DIGIT TWO" },
    { 0x16A63, "MRO DIGIT THREE" },
    { 0x16A64, "MRO DIGIT FOUR" },
    { 0x16A65, "MRO DIGIT FIVE" },
    { 0x16A66, "MRO DIGIT SIX" },
    { 0x16A67, "MRO DIGIT SEVEN" },
    { 0x16A68, "MRO DIGIT EIGHT" },
    { 0x16A69, "MRO DIGIT NINE" },
    { 0x16A6E, "MRO DANDA" },
    { 0x16A6F, "MRO DOUBLE DANDA" },
    { 0x16A70, "TANGSA LETTER OZ" },
    { 0x16A71, "TANGSA LETTER OC" },
    { 0x16A72, "TANGSA LETTER OQ" },
    { 0x16A73, "TANGSA LETTER OX" },
    { 0x16A74, "TANGSA LETTER AZ" },
    { 0x16A75, "TANGSA LETTER AC" },
    { 0x16A76, "TANGSA LETTER AQ" },
    { 0x16A77, "TANGSA LETTER AX" },
    { 0x16A78, "TANGSA LETTER VZ" },
    { 0x16A79, "TANGSA LETTER VC" },
    { 0x16A7A, "TANGSA LETTER VQ" },
    { 0x16A7B, "TANGSA LETTER VX" },
    { 0x16A7C, "TANGSA LETTER EZ" },
    { 0x16A7D, "TANGSA LETTER EC" },
    { 0x16A7E, "TANGSA LETTER EQ" },
    { 0x16A7F, "TANGSA LETTER EX" },
    { 0x16A80, "TANGSA LETTER IZ" },
    { 0x16A81, "TANGSA LETTER IC" },
    { 0x16A82, "TANGSA LETTER IQ" },
    { 0x16A83, "TANGSA LETTER IX" },
    { 0x16A84, "TANGSA LETTER UZ" },
    { 0x16A85, "TANGSA LETTER UC" },
    { 0x16A86, "TANGSA LETTER UQ" },
    { 0x16A87, "TANGSA LETTER UX" },
    { 0x16A88, "TANGSA LETTER AWZ" },
    { 0x16A89, "TANGSA LETTER AWC" },
    { 0x16A8A, "TANGSA LETTER AWQ" },
    { 0x16A8B, "TANGSA LETTER AWX" },
    { 0x16A8C, "TANGSA LETTER UIZ" },
    { 0x16A8D, "TANGSA LETTER UIC" },
    { 0x16A8E, "TANGSA LETTER UIQ" },
    { 0x16A8F, "TANGSA LETTER UIX" },
    { 0x16A90, "TANGSA LETTER FINAL NG" },
    { 0x16A91, "TANGSA LETTER LONG UEX" },
    { 0x16A92, "TANGSA LETTER SHORT UEZ" },
    { 0x16A93, "TANGSA LETTER SHORT AWX" },
    { 0x16A94, "TANGSA LETTER UEC" },
    { 0x16A95, "TANGSA LETTER UEZ" },
    { 0x16A96, "TANGSA LETTER UEQ" },
    { 0x16A97, "TANGSA LETTER UEX" },
    { 0x16A98, "TANGSA LETTER UIUZ" },
    { 0x16A99, "TANGSA LETTER UIUC" },
    { 0x16A9A, "TANGSA LETTER UIUQ" },
    { 0x16A9B, "TANGSA LETTER UIUX" },
    { 0x16A9C, "TANGSA LETTER MZ" },
    { 0x16A9D, "TANGSA LETTER MC" },
    { 0x16A9E, "TANGSA LETTER MQ" },
    { 0x16A9F, "TANGSA LETTER MX" },
    { 0x16AA0, "TANGSA LETTER KA" },
    { 0x16AA1, "TANGSA LETTER KHA" },
    { 0x16AA2, "TANGSA LETTER GA" },
    { 0x16AA3, "TANGSA LETTER NGA" },
    { 0x16AA4, "TANGSA LETTER SA" },
    { 0x16AA5, "TANGSA LETTER YA" },
    { 0x16AA6, "TANGSA LETTER WA" },
    { 0x16AA7, "TANGSA LETTER PA" },
    { 0x16AA8, "TANGSA LETTER NYA" },
    { 0x16AA9, "TANGSA LETTER PHA" },
    { 0x16AAA, "TANGSA LETTER BA" },
    { 0x16AAB, "TANGSA LETTER MA" },
    { 0x16AAC, "TANGSA LETTER NA" },
    { 0x16AAD, "TANGSA LETTER HA" },
    { 0x16AAE, "TANGSA LETTER LA" },
    { 0x16AAF, "TANGSA LETTER HTA" },
    { 0x16AB0, "TANGSA LETTER TA" },
    { 0x16AB1, "TANGSA LETTER DA" },
    { 0x16AB2, "TANGSA LETTER RA" },
    { 0x16AB3, "TANGSA LETTER NHA" },
    { 0x16AB4, "TANGSA LETTER SHA" },
    { 0x16AB5, "TANGSA LETTER CA" },
    { 0x16AB6, "TANGSA LETTER TSA" },
    { 0x16AB7, "TANGSA LETTER GHA" },
    { 0x16AB8, "TANGSA LETTER HTTA" },
    { 0x16AB9, "TANGSA LETTER THA" },
    { 0x16ABA, "TANGSA LETTER XA" },
    { 0x16ABB, "TANGSA LETTER FA" },
    { 0x16ABC, "TANGSA LETTER DHA" },
    { 0x16ABD, "TANGSA LETTER CHA" },
    { 0x16ABE, "TANGSA LETTER ZA" },
    { 0x16AC0, "TANGSA DIGIT ZERO" },
    { 0x16AC1, "TANGSA DIGIT ONE" },
    { 0x16AC2, "TANGSA DIGIT TWO" },
    { 0x16AC3, "TANGSA DIGIT THREE" },
    { 0x16AC4, "TANGSA DIGIT FOUR" },
    { 0x16AC5, "TANGSA DIGIT FIVE" },
    { 0x16AC6, "TANGSA DIGIT SIX" },
    { 0x16AC7, "TANGSA DIGIT SEVEN" },
    { 0x16AC8, "TANGSA DIGIT EIGHT" },
    { 0x16AC9, "TANGSA DIGIT NINE" },
    { 0x16AD0, "BASSA VAH LETTER ENNI" },
    { 0x16AD1, "BASSA VAH LETTER KA" },
    { 0x16AD2, "BASSA VAH LETTER SE" },
    { 0x16AD3, "BASSA VAH LETTER FA" },
    { 0x16AD4, "BASSA VAH LETTER MBE" },
    { 0x16AD5, "BASSA VAH LETTER YIE" },
    { 0x16AD6, "BASSA VAH LETTER GAH" },
    { 0x16AD7, "BASSA VAH LETTER DHII" },
    { 0x16AD8, "BASSA VAH LETTER KPAH" },
    { 0x16AD9, "BASSA VAH LETTER JO" },
    { 0x16ADA, "BASSA VAH LETTER HWAH" },
    { 0x16ADB, "BASSA VAH LETTER WA" },
    { 0x16ADC, "BASSA VAH LETTER ZO" },
    { 0x16ADD, "BASSA VAH LETTER GBU" },
    { 0x16ADE, "BASSA VAH LETTER DO" },
    { 0x16ADF, "BASSA VAH LETTER CE" },
    { 0x16AE0, "BASSA VAH LETTER UWU" },
    { 0x16AE1, "BASSA VAH LETTER TO" },
    { 0x16AE2, "BASSA VAH LETTER BA" },
    { 0x16AE3, "BASSA VAH LETTER VU" },
    { 0x16AE4, "BASSA VAH LETTER YEIN" },
    { 0x16AE5, "BASSA VAH LETTER PA" },
    { 0x16AE6, "BASSA VAH LETTER WADDA" },
    { 0x16AE7, "BASSA VAH LETTER A" },
    { 0x16AE8, "BASSA VAH LETTER O" },
    { 0x16AE9, "BASSA VAH LETTER OO" },
    { 0x16AEA, "BASSA VAH LETTER U" },
    { 0x16AEB, "BASSA VAH LETTER EE" },
    { 0x16AEC, "BASSA VAH LETTER E" },
    { 0x16AED, "BASSA VAH LETTER I" },
    { 0x16AF0, "BASSA VAH COMBINING HIGH TONE" },
    { 0x16AF1, "BASSA VAH COMBINING LOW TONE" },
    { 0x16AF2, "BASSA VAH COMBINING MID TONE" },
    { 0x16AF3, "BASSA VAH COMBINING LOW-MID TONE" },
    { 0x16AF4, "BASSA VAH COMBINING HIGH-LOW TONE" },
    { 0x16AF5, "BASSA VAH FULL STOP" },
    { 0x16B00, "PAHAWH HMONG VOWEL KEEB" },
    { 0x16B01, "PAHAWH HMONG VOWEL KEEV" },
    { 0x16B02, "PAHAWH HMONG VOWEL KIB" },
    { 0x16B03, "PAHAWH HMONG VOWEL KIV" },
    { 0x16B04, "PAHAWH HMONG VOWEL KAUB" },
    { 0x16B05, "PAHAWH HMONG VOWEL KAUV" },
    { 0x16B06, "PAHAWH HMONG VOWEL KUB" },
    { 0x16B07, "PAHAWH HMONG VOWEL KUV" },
    { 0x16B08, "PAHAWH HMONG VOWEL KEB" },
    { 0x16B09, "PAHAWH HMONG VOWEL KEV" },
    { 0x16B0A, "PAHAWH HMONG VOWEL KAIB" },
    { 0x16B0B, "PAHAWH HMONG VOWEL KAIV" },
    { 0x16B0C, "PAHAWH HMONG VOWEL KOOB" },
    { 0x16B0D, "PAHAWH HMONG VOWEL KOOV" },
    { 0x16B0E, "PAHAWH HMONG VOWEL KAWB" },
    { 0x16B0F, "PAHAWH HMONG VOWEL KAWV" },
    { 0x16B10, "PAHAWH HMONG VOWEL KUAB" },
    { 0x16B11, "PAHAWH HMONG VOWEL KUAV" },
    { 0x16B12, "PAHAWH HMONG VOWEL KOB" },
    { 0x16B13, "PAHAWH HMONG VOWEL KOV" },
    { 0x16B14, "PAHAWH HMONG VOWEL KIAB" },
    { 0x16B15, "PAHAWH HMONG VOWEL KIAV" },
    { 0x16B16, "PAHAWH HMONG VOWEL KAB" },
    { 0x16B17, "PAHAWH HMONG VOWEL KAV" },
    { 0x16B18, "PAHAWH HMONG VOWEL KWB" },
    { 0x16B19, "PAHAWH HMONG VOWEL KWV" },
    { 0x16B1A, "PAHAWH HMONG VOWEL KAAB" },
    { 0x16B1B, "PAHAWH HMONG VOWEL KAAV" },
    { 0x16B1C, "PAHAWH HMONG CONSONANT VAU" },
    { 0x16B1D, "PAHAWH HMONG CONSONANT NTSAU" },
    { 0x16B1E, "PAHAWH HMONG CONSONANT LAU" },
    { 0x16B1F, "PAHAWH HMONG CONSONANT HAU" },
    { 0x16B20, "PAHAWH HMONG CONSONANT NLAU" },
    { 0x16B21, "PAHAWH HMONG CONSONANT RAU" },
    { 0x16B22, "PAHAWH HMONG CONSONANT NKAU" },
    { 0x16B23, "PAHAWH HMONG CONSONANT QHAU" },
    { 0x16B24, "PAHAWH HMONG CONSONANT YAU" },
    { 0x16B25, "PAHAWH HMONG CONSONANT HLAU" },
    { 0x16B26, "PAHAWH HMONG CONSONANT MAU" },
    { 0x16B27, "PAHAWH HMONG CONSONANT CHAU" },
    { 0x16B28, "PAHAWH HMONG CONSONANT NCHAU" },
    { 0x16B29, "PAHAWH HMONG CONSONANT HNAU" },
    { 0x16B2A, "PAHAWH HMONG CONSONANT PLHAU" },
    { 0x16B2B, "PAHAWH HMONG CONSONANT NTHAU" },
    { 0x16B2C, "PAHAWH HMONG CONSONANT NAU" },
    { 0x16B2D, "PAHAWH HMONG CONSONANT AU" },
    { 0x16B2E, "PAHAWH HMONG CONSONANT XAU" },
    { 0x16B2F, "PAHAWH HMONG CONSONANT CAU" },
    { 0x16B30, "PAHAWH HMONG MARK CIM TUB" },
    { 0x16B31, "PAHAWH HMONG MARK CIM SO" },
    { 0x16B32, "PAHAWH HMONG MARK CIM KES" },
    { 0x16B33, "PAHAWH HMONG MARK CIM KHAV" },
    { 0x16B34, "PAHAWH HMONG MARK CIM SUAM" },
    { 0x16B35, "PAHAWH HMONG MARK CIM HOM" },
    { 0x16B36, "PAHAWH HMONG MARK CIM TAUM" },
    { 0x16B37, "PAHAWH HMONG SIGN VOS THOM" },
    { 0x16B38, "PAHAWH HMONG SIGN VOS TSHAB CEEB" },
    { 0x16B39, "PAHAWH HMONG SIGN CIM CHEEM" },
    { 0x16B3A, "PAHAWH HMONG SIGN VOS THIAB" },
    { 0x16B3B, "PAHAWH HMONG SIGN VOS FEEM" },
    { 0x16B3C, "PAHAWH HMONG SIGN XYEEM NTXIV" },
    { 0x16B3D, "PAHAWH HMONG SIGN XYEEM RHO" },
    { 0x16B3E, "PAHAWH HMONG SIGN XYEEM TOV" },
    { 0x16B3F, "PAHAWH HMONG SIGN XYEEM FAIB" },
    { 0x16B40, "PAHAWH HMONG SIGN VOS SEEV" },
    { 0x16B41, "PAHAWH HMONG SIGN MEEJ SUAB" },
    { 0x16B42, "PAHAWH HMONG SIGN VOS NRUA" },
    { 0x16B43, "PAHAWH HMONG SIGN IB YAM" },
    { 0x16B44, "PAHAWH HMONG SIGN XAUS" },
    { 0x16B45, "PAHAWH HMONG SIGN CIM TSOV ROG" },
    { 0x16B50, "PAHAWH HMONG DIGIT ZERO" },
    { 0x16B51, "PAHAWH HMONG DIGIT ONE" },
    { 0x16B52, "PAHAWH HMONG DIGIT TWO" },
    { 0x16B53, "PAHAWH HMONG DIGIT THREE" },
    { 0x16B54, "PAHAWH HMONG DIGIT FOUR" },
    { 0x16B55, "PAHAWH HMONG DIGIT FIVE" },
    { 0x16B56, "PAHAWH HMONG DIGIT SIX" },
    { 0x16B57, "PAHAWH HMONG DIGIT SEVEN" },
    { 0x16B58, "PAHAWH HMONG DIGIT EIGHT" },
    { 0x16B59, "PAHAWH HMONG DIGIT NINE" },
    { 0x16B5B, "PAHAWH HMONG NUMBER TENS" },
    { 0x16B5C, "PAHAWH HMONG NUMBER HUNDREDS" },
    { 0x16B5D, "PAHAWH HMONG NUMBER TEN THOUSANDS" },
    { 0x16B5E, "PAHAWH HMONG NUMBER MILLIONS" },
    { 0x16B5F, "PAHAWH HMONG NUMBER HUNDRED MILLIONS" },
    { 0x16B60, "PAHAWH HMONG NUMBER TEN BILLIONS" },
    { 0x16B61, "PAHAWH HMONG NUMBER TRILLIONS" },
    { 0x16B63, "PAHAWH HMONG SIGN VOS LUB" },
    { 0x16B64, "PAHAWH HMONG SIGN XYOO" },
    { 0x16B65, "PAHAWH HMONG SIGN HLI" },
    { 0x16B66, "PAHAWH HMONG SIGN THIRD-STAGE HLI" },
    { 0x16B67, "PAHAWH HMONG SIGN ZWJ THAJ" },
    { 0x16B68, "PAHAWH HMONG SIGN HNUB" },
    { 0x16B69, "PAHAWH HMONG SIGN NQIG" },
    { 0x16B6A, "PAHAWH HMONG SIGN XIAB" },
    { 0x16B6B, "PAHAWH HMONG SIGN NTUJ" },
    { 0x16B6C, "PAHAWH HMONG SIGN AV" },
    { 0x16B6D, "PAHAWH HMONG SIGN TXHEEJ CEEV" },
    { 0x16B6E, "PAHAWH HMONG SIGN MEEJ TSEEB" },
    { 0x16B6F, "PAHAWH HMONG SIGN TAU" },
    { 0x16B70, "PAHAWH HMONG SIGN LOS" },
    { 0x16B71, "PAHAWH HMONG SIGN MUS" },
    { 0x16B72, "PAHAWH HMONG SIGN CIM HAIS LUS NTOG NTOG" },
    { 0x16B73, "PAHAWH HMONG SIGN CIM CUAM TSHOOJ" },
    { 0x16B74, "PAHAWH HMONG SIGN CIM TXWV" },
    { 0x16B75, "PAHAWH HMONG SIGN CIM TXWV CHWV" },
    { 0x16B76, "PAHAWH HMONG SIGN CIM PUB DAWB" },
    { 0x16B77, "PAHAWH HMONG SIGN CIM NRES TOS" },
    { 0x16B7D, "PAHAWH HMONG CLAN SIGN TSHEEJ" },
    { 0x16B7E, "PAHAWH HMONG CLAN SIGN YEEG" },
    { 0x16B7F, "PAHAWH HMONG CLAN SIGN LIS" },
    { 0x16B80, "PAHAWH HMONG CLAN SIGN LAUJ" },
    { 0x16B81, "PAHAWH HMONG CLAN SIGN XYOOJ" },
    { 0x16B82, "PAHAWH HMONG CLAN SIGN KOO" },
    { 0x16B83, "PAHAWH HMONG CLAN SIGN HAWJ" },
    { 0x16B84, "PAHAWH HMONG CLAN SIGN MUAS" },
    { 0x16B85, "PAHAWH HMONG CLAN SIGN THOJ" },
    { 0x16B86, "PAHAWH HMONG CLAN SIGN TSAB" },
    { 0x16B87, "PAHAWH HMONG CLAN SIGN PHAB" },
    { 0x16B88, "PAHAWH HMONG CLAN SIGN KHAB" },
    { 0x16B89, "PAHAWH HMONG CLAN SIGN HAM" },
    { 0x16B8A, "PAHAWH HMONG CLAN SIGN VAJ" },
    { 0x16B8B, "PAHAWH HMONG CLAN SIGN FAJ" },
    { 0x16B8C, "PAHAWH HMONG CLAN SIGN YAJ" },
    { 0x16B8D, "PAHAWH HMONG CLAN SIGN TSWB" },
    { 0x16B8E, "PAHAWH HMONG CLAN SIGN KWM" },
    { 0x16B8F, "PAHAWH HMONG CLAN SIGN VWJ" },
    { 0x16E40, "MEDEFAIDRIN CAPITAL LETTER M" },
    { 0x16E41, "MEDEFAIDRIN CAPITAL LETTER S" },
    { 0x16E42, "MEDEFAIDRIN CAPITAL LETTER V" },
    { 0x16E43, "MEDEFAIDRIN CAPITAL LETTER W" },
    { 0x16E44, "MEDEFAIDRIN CAPITAL LETTER ATIU" },
    { 0x16E45, "MEDEFAIDRIN CAPITAL LETTER Z" },
    { 0x16E46, "MEDEFAIDRIN CAPITAL LETTER KP" },
    { 0x16E47, "MEDEFAIDRIN CAPITAL LETTER P" },
    { 0x16E48, "MEDEFAIDRIN CAPITAL LETTER T" },
    { 0x16E49, "MEDEFAIDRIN CAPITAL LETTER G" },
    { 0x16E4A, "MEDEFAIDRIN CAPITAL LETTER F" },
    { 0x16E4B, "MEDEFAIDRIN CAPITAL LETTER I" },
    { 0x16E4C, "MEDEFAIDRIN CAPITAL LETTER K" },
    { 0x16E4D, "MEDEFAIDRIN CAPITAL LETTER A" },
    { 0x16E4E, "MEDEFAIDRIN CAPITAL LETTER J" },
    { 0x16E4F, "MEDEFAIDRIN CAPITAL LETTER E" },
    { 0x16E50, "MEDEFAIDRIN CAPITAL LETTER B" },
    { 0x16E51, "MEDEFAIDRIN CAPITAL LETTER C" },
    { 0x16E52, "MEDEFAIDRIN CAPITAL LETTER U" },
    { 0x16E53, "MEDEFAIDRIN CAPITAL LETTER YU" },
    { 0x16E54, "MEDEFAIDRIN CAPITAL LETTER L" },
    { 0x16E55, "MEDEFAIDRIN CAPITAL LETTER Q" },
    { 0x16E56, "MEDEFAIDRIN CAPITAL LETTER HP" },
    { 0x16E57, "MEDEFAIDRIN CAPITAL LETTER NY" },
    { 0x16E58, "MEDEFAIDRIN CAPITAL LETTER X" },
    { 0x16E59, "MEDEFAIDRIN CAPITAL LETTER D" },
    { 0x16E5A, "MEDEFAIDRIN CAPITAL LETTER OE" },
    { 0x16E5B, "MEDEFAIDRIN CAPITAL LETTER N" },
    { 0x16E5C, "MEDEFAIDRIN CAPITAL LETTER R" },
    { 0x16E5D, "MEDEFAIDRIN CAPITAL LETTER O" },
    { 0x16E5E, "MEDEFAIDRIN CAPITAL LETTER AI" },
    { 0x16E5F, "MEDEFAIDRIN CAPITAL LETTER Y" },
    { 0x16E60, "MEDEFAIDRIN SMALL LETTER M" },
    { 0x16E61, "MEDEFAIDRIN SMALL LETTER S" },
    { 0x16E62, "MEDEFAIDRIN SMALL LETTER V" },
    { 0x16E63, "MEDEFAIDRIN SMALL LETTER W" },
    { 0x16E64, "MEDEFAIDRIN SMALL LETTER ATIU" },
    { 0x16E65, "MEDEFAIDRIN SMALL LETTER Z" },
    { 0x16E66, "MEDEFAIDRIN SMALL LETTER KP" },
    { 0x16E67, "MEDEFAIDRIN SMALL LETTER P" },
    { 0x16E68, "MEDEFAIDRIN SMALL LETTER T" },
    { 0x16E69, "MEDEFAIDRIN SMALL LETTER G" },
    { 0x16E6A, "MEDEFAIDRIN SMALL LETTER F" },
    { 0x16E6B, "MEDEFAIDRIN SMALL LETTER I" },
    { 0x16E6C, "MEDEFAIDRIN SMALL LETTER K" },
    { 0x16E6D, "MEDEFAIDRIN SMALL LETTER A" },
    { 0x16E6E, "MEDEFAIDRIN SMALL LETTER J" },
    { 0x16E6F, "MEDEFAIDRIN SMALL LETTER E" },
    { 0x16E70, "MEDEFAIDRIN SMALL LETTER B" },
    { 0x16E71, "MEDEFAIDRIN SMALL LETTER C" },
    { 0x16E72, "MEDEFAIDRIN SMALL LETTER U" },
    { 0x16E73, "MEDEFAIDRIN SMALL LETTER YU" },
    { 0x16E74, "MEDEFAIDRIN SMALL LETTER L" },
    { 0x16E75, "MEDEFAIDRIN SMALL LETTER Q" },
    { 0x16E76, "MEDEFAIDRIN SMALL LETTER HP" },
    { 0x16E77, "MEDEFAIDRIN SMALL LETTER NY" },
    { 0x16E78, "MEDEFAIDRIN SMALL LETTER X" },
    { 0x16E79, "MEDEFAIDRIN SMALL LETTER D" },
    { 0x16E7A, "MEDEFAIDRIN SMALL LETTER OE" },
    { 0x16E7B, "MEDEFAIDRIN SMALL LETTER N" },
    { 0x16E7C, "MEDEFAIDRIN SMALL LETTER R" },
    { 0x16E7D, "MEDEFAIDRIN SMALL LETTER O" },
    { 0x16E7E, "MEDEFAIDRIN SMALL LETTER AI" },
    { 0x16E7F, "MEDEFAIDRIN SMALL LETTER Y" },
    { 0x16E80, "MEDEFAIDRIN DIGIT ZERO" },
    { 0x16E81, "MEDEFAIDRIN DIGIT ONE" },
    { 0x16E82, "MEDEFAIDRIN DIGIT TWO" },
    { 0x16E83, "MEDEFAIDRIN DIGIT THREE" },
    { 0x16E84, "MEDEFAIDRIN DIGIT FOUR" },
    { 0x16E85, "MEDEFAIDRIN DIGIT FIVE" },
    { 0x16E86, "MEDEFAIDRIN DIGIT SIX" },
    { 0x16E87, "MEDEFAIDRIN DIGIT SEVEN" },
    { 0x16E88, "MEDEFAIDRIN DIGIT EIGHT" },
    { 0x16E89, "MEDEFAIDRIN DIGIT NINE" },
    { 0x16E8A, "MEDEFAIDRIN NUMBER TEN" },
    { 0x16E8B, "MEDEFAIDRIN NUMBER ELEVEN" },
    { 0x16E8C, "MEDEFAIDRIN NUMBER TWELVE" },
    { 0x16E8D, "MEDEFAIDRIN NUMBER THIRTEEN" },
    { 0x16E8E, "MEDEFAIDRIN NUMBER FOURTEEN" },
    { 0x16E8F, "MEDEFAIDRIN NUMBER FIFTEEN" },
    { 0x16E90, "MEDEFAIDRIN NUMBER SIXTEEN" },
    { 0x16E91, "MEDEFAIDRIN NUMBER SEVENTEEN" },
    { 0x16E92, "MEDEFAIDRIN NUMBER EIGHTEEN" },
    { 0x16E93, "MEDEFAIDRIN NUMBER NINETEEN" },
    { 0x16E94, "MEDEFAIDRIN DIGIT ONE ALTERNATE FORM" },
    { 0x16E95, "MEDEFAIDRIN DIGIT TWO ALTERNATE FORM" },
    { 0x16E96, "MEDEFAIDRIN DIGIT THREE ALTERNATE FORM" },
    { 0x16E97, "MEDEFAIDRIN COMMA" },
    { 0x16E98, "MEDEFAIDRIN FULL STOP" },
    { 0x16E99, "MEDEFAIDRIN SYMBOL AIVA" },
    { 0x16E9A, "MEDEFAIDRIN EXCLAMATION OH" },
    { 0x16F00, "MIAO LETTER PA" },
    { 0x16F01, "MIAO LETTER BA" },
    { 0x16F02, "MIAO LETTER YI PA" },
    { 0x16F03, "MIAO LETTER PLA" },
    { 0x16F04, "MIAO LETTER MA" },
    { 0x16F05, "MIAO LETTER MHA" },
    { 0x16F06, "MIAO LETTER ARCHAIC MA" },
    { 0x16F07, "MIAO LETTER FA" },
    { 0x16F08, "MIAO LETTER VA" },
    { 0x16F09, "MIAO LETTER VFA" },
    { 0x16F0A, "MIAO LETTER TA" },
    { 0x16F0B, "MIAO LETTER DA" },
    { 0x16F0C, "MIAO LETTER YI TTA" },
    { 0x16F0D, "MIAO LETTER YI TA" },
    { 0x16F0E, "MIAO LETTER TTA" },
    { 0x16F0F, "MIAO LETTER DDA" },
    { 0x16F10, "MIAO LETTER NA" },
    { 0x16F11, "MIAO LETTER NHA" },
    { 0x16F12, "MIAO LETTER YI NNA" },
    { 0x16F13, "MIAO LETTER ARCHAIC NA" },
    { 0x16F14, "MIAO LETTER NNA" },
    { 0x16F15, "MIAO LETTER NNHA" },
    { 0x16F16, "MIAO LETTER LA" },
    { 0x16F17, "MIAO LETTER LYA" },
    { 0x16F18, "MIAO LETTER LHA" },
    { 0x16F19, "MIAO LETTER LHYA" },
    { 0x16F1A, "MIAO LETTER TLHA" },
    { 0x16F1B, "MIAO LETTER DLHA" },
    { 0x16F1C, "MIAO LETTER TLHYA" },
    { 0x16F1D, "MIAO LETTER DLHYA" },
    { 0x16F1E, "MIAO LETTER KA" },
    { 0x16F1F, "MIAO LETTER GA" },
    { 0x16F20, "MIAO LETTER YI KA" },
    { 0x16F21, "MIAO LETTER QA" },
    { 0x16F22, "MIAO LETTER QGA" },
    { 0x16F23, "MIAO LETTER NGA" },
    { 0x16F24, "MIAO LETTER NGHA" },
    { 0x16F25, "MIAO LETTER ARCHAIC NGA" },
    { 0x16F26, "MIAO LETTER HA" },
    { 0x16F27, "MIAO LETTER XA" },
    { 0x16F28, "MIAO LETTER GHA" },
    { 0x16F29, "MIAO LETTER GHHA" },
    { 0x16F2A, "MIAO LETTER TSSA" },
    { 0x16F2B, "MIAO LETTER DZZA" },
    { 0x16F2C, "MIAO LETTER NYA" },
    { 0x16F2D, "MIAO LETTER NYHA" },
    { 0x16F2E, "MIAO LETTER TSHA" },
    { 0x16F2F, "MIAO LETTER DZHA" },
    { 0x16F30, "MIAO LETTER YI TSHA" },
    { 0x16F31, "MIAO LETTER YI DZHA" },
    { 0x16F32, "MIAO LETTER REFORMED TSHA" },
    { 0x16F33, "MIAO LETTER SHA" },
    { 0x16F34, "MIAO LETTER SSA" },
    { 0x16F35, "MIAO LETTER ZHA" },
    { 0x16F36, "MIAO LETTER ZSHA" },
    { 0x16F37, "MIAO LETTER TSA" },
    { 0x16F38, "MIAO LETTER DZA" },
    { 0x16F39, "MIAO LETTER YI TSA" },
    { 0x16F3A, "MIAO LETTER SA" },
    { 0x16F3B, "MIAO LETTER ZA" },
    { 0x16F3C, "MIAO LETTER ZSA" },
    { 0x16F3D, "MIAO LETTER ZZA" },
    { 0x16F3E, "MIAO LETTER ZZSA" },
    { 0x16F3F, "MIAO LETTER ARCHAIC ZZA" },
    { 0x16F40, "MIAO LETTER ZZYA" },
    { 0x16F41, "MIAO LETTER ZZSYA" },
    { 0x16F42, "MIAO LETTER WA" },
    { 0x16F43, "MIAO LETTER AH" },
    { 0x16F44, "MIAO LETTER HHA" },
    { 0x16F45, "MIAO LETTER BRI" },
    { 0x16F46, "MIAO LETTER SYI" },
    { 0x16F47, "MIAO LETTER DZYI" },
    { 0x16F48, "MIAO LETTER TE" },
    { 0x16F49, "MIAO LETTER TSE" },
    { 0x16F4A, "MIAO LETTER RTE" },
    { 0x16F4F, "MIAO SIGN CONSONANT MODIFIER BAR" },
    { 0x16F50, "MIAO LETTER NASALIZATION" },
    { 0x16F51, "MIAO SIGN ASPIRATION" },
    { 0x16F52, "MIAO SIGN REFORMED VOICING" },
    { 0x16F53, "MIAO SIGN REFORMED ASPIRATION" },
    { 0x16F54, "MIAO VOWEL SIGN A" },
    { 0x16F55, "MIAO VOWEL SIGN AA" },
    { 0x16F56, "MIAO VOWEL SIGN AHH" },
    { 0x16F57, "MIAO VOWEL SIGN AN" },
    { 0x16F58, "MIAO VOWEL SIGN ANG" },
    { 0x16F59, "MIAO VOWEL SIGN O" },
    { 0x16F5A, "MIAO VOWEL SIGN OO" },
    { 0x16F5B, "MIAO VOWEL SIGN WO" },
    { 0x16F5C, "MIAO VOWEL SIGN W" },
    { 0x16F5D, "MIAO VOWEL SIGN E" },
    { 0x16F5E, "MIAO VOWEL SIGN EN" },
    { 0x16F5F, "MIAO VOWEL SIGN ENG" },
    { 0x16F60, "MIAO VOWEL SIGN OEY" },
    { 0x16F61, "MIAO VOWEL SIGN I" },
    { 0x16F62, "MIAO VOWEL SIGN IA" },
    { 0x16F63, "MIAO VOWEL SIGN IAN" },
    { 0x16F64, "MIAO VOWEL SIGN IANG" },
    { 0x16F65, "MIAO VOWEL SIGN IO" },
    { 0x16F66, "MIAO VOWEL SIGN IE" },
    { 0x16F67, "MIAO VOWEL SIGN II" },
    { 0x16F68, "MIAO VOWEL SIGN IU" },
    { 0x16F69, "MIAO VOWEL SIGN ING" },
    { 0x16F6A, "MIAO VOWEL SIGN U" },
    { 0x16F6B, "MIAO VOWEL SIGN UA" },
    { 0x16F6C, "MIAO VOWEL SIGN UAN" },
    { 0x16F6D, "MIAO VOWEL SIGN UANG" },
    { 0x16F6E, "MIAO VOWEL SIGN UU" },
    { 0x16F6F, "MIAO VOWEL SIGN UEI" },
    { 0x16F70, "MIAO VOWEL SIGN UNG" },
    { 0x16F71, "MIAO VOWEL SIGN Y" },
    { 0x16F72, "MIAO VOWEL SIGN YI" },
    { 0x16F73, "MIAO VOWEL SIGN AE" },
    { 0x16F74, "MIAO VOWEL SIGN AEE" },
    { 0x16F75, "MIAO VOWEL SIGN ERR" },
    { 0x16F76, "MIAO VOWEL SIGN ROUNDED ERR" },
    { 0x16F77, "MIAO VOWEL SIGN ER" },
    { 0x16F78, "MIAO VOWEL SIGN ROUNDED ER" },
    { 0x16F79, "MIAO VOWEL SIGN AI" },
    { 0x16F7A, "MIAO VOWEL SIGN EI" },
    { 0x16F7B, "MIAO VOWEL SIGN AU" },
    { 0x16F7C, "MIAO VOWEL SIGN OU" },
    { 0x16F7D, "MIAO VOWEL SIGN N" },
    { 0x16F7E, "MIAO VOWEL SIGN NG" },
    { 0x16F7F, "MIAO VOWEL SIGN UOG" },
    { 0x16F80, "MIAO VOWEL SIGN YUI" },
    { 0x16F81, "MIAO VOWEL SIGN OG" },
    { 0x16F82, "MIAO VOWEL SIGN OER" },
    { 0x16F83, "MIAO VOWEL SIGN VW" },
    { 0x16F84, "MIAO VOWEL SIGN IG" },
    { 0x16F85, "MIAO VOWEL SIGN EA" },
    { 0x16F86, "MIAO VOWEL SIGN IONG" },
    { 0x16F87, "MIAO VOWEL SIGN UI" },
    { 0x16F8F, "MIAO TONE RIGHT" },
    { 0x16F90, "MIAO TONE TOP RIGHT" },
    { 0x16F91, "MIAO TONE ABOVE" },
    { 0x16F92, "MIAO TONE BELOW" },
    { 0x16F93, "MIAO LETTER TONE-2" },
    { 0x16F94, "MIAO LETTER TONE-3" },
    { 0x16F95, "MIAO LETTER TONE-4" },
    { 0x16F96, "MIAO LETTER TONE-5" },
    { 0x16F97, "MIAO LETTER TONE-6" },
    { 0x16F98, "MIAO LETTER TONE-7" },
    { 0x16F99, "MIAO LETTER TONE-8" },
    { 0x16F9A, "MIAO LETTER REFORMED TONE-1" },
    { 0x16F9B, "MIAO LETTER REFORMED TONE-2" },
    { 0x16F9C, "MIAO LETTER REFORMED TONE-4" },
    { 0x16F9D, "MIAO LETTER REFORMED TONE-5" },
    { 0x16F9E, "MIAO LETTER REFORMED TONE-6" },
    { 0x16F9F, "MIAO LETTER REFORMED TONE-8" },
    { 0x16FE0, "TANGUT ITERATION MARK" },
    { 0x16FE1, "NUSHU ITERATION MARK" },
    { 0x16FE2, "OLD CHINESE HOOK MARK" },
    { 0x16FE3, "OLD CHINESE ITERATION MARK" },
    { 0x16FE4, "KHITAN SMALL SCRIPT FILLER" },
    { 0x16FF0, "VIETNAMESE ALTERNATE READING MARK CA" },
    { 0x16FF1, "VIETNAMESE ALTERNATE READING MARK NHAY" },
    { 0x18800, "TANGUT COMPONENT-001" },
    { 0x18801, "TANGUT COMPONENT-002" },
    { 0x18802, "TANGUT COMPONENT-003" },
    { 0x18803, "TANGUT COMPONENT-004" },
    { 0x18804, "TANGUT COMPONENT-005" },
    { 0x18805, "TANGUT COMPONENT-006" },
    { 0x18806, "TANGUT COMPONENT-007" },
    { 0x18807, "TANGUT COMPONENT-008" },
    { 0x18808, "TANGUT COMPONENT-009" },
    { 0x18809, "TANGUT COMPONENT-010" },
    { 0x1880A, "TANGUT COMPONENT-011" },
    { 0x1880B, "TANGUT COMPONENT-012" },
    { 0x1880C, "TANGUT COMPONENT-013" },
    { 0x1880D, "TANGUT COMPONENT-014" },
    { 0x1880E, "TANGUT COMPONENT-015" },
    { 0x1880F, "TANGUT COMPONENT-016" },
    { 0x18810, "TANGUT COMPONENT-017" },
    { 0x18811, "TANGUT COMPONENT-018" },
    { 0x18812, "TANGUT COMPONENT-019" },
    { 0x18813, "TANGUT COMPONENT-020" },
    { 0x18814, "TANGUT COMPONENT-021" },
    { 0x18815, "TANGUT COMPONENT-022" },
    { 0x18816, "TANGUT COMPONENT-023" },
    { 0x18817, "TANGUT COMPONENT-024" },
    { 0x18818, "TANGUT COMPONENT-025" },
    { 0x18819, "TANGUT COMPONENT-026" },
    { 0x1881A, "TANGUT COMPONENT-027" },
    { 0x1881B, "TANGUT COMPONENT-028" },
    { 0x1881C, "TANGUT COMPONENT-029" },
    { 0x1881D, "TANGUT COMPONENT-030" },
    { 0x1881E, "TANGUT COMPONENT-031" },
    { 0x1881F, "TANGUT COMPONENT-032" },
    { 0x18820, "TANGUT COMPONENT-033" },
    { 0x18821, "TANGUT COMPONENT-034" },
    { 0x18822, "TANGUT COMPONENT-035" },
    { 0x18823, "TANGUT COMPONENT-036" },
    { 0x18824, "TANGUT COMPONENT-037" },
    { 0x18825, "TANGUT COMPONENT-038" },
    { 0x18826, "TANGUT COMPONENT-039" },
    { 0x18827, "TANGUT COMPONENT-040" },
    { 0x18828, "TANGUT COMPONENT-041" },
    { 0x18829, "TANGUT COMPONENT-042" },
    { 0x1882A, "TANGUT COMPONENT-043" },
    { 0x1882B, "TANGUT COMPONENT-044" },
    { 0x1882C, "TANGUT COMPONENT-045" },
    { 0x1882D, "TANGUT COMPONENT-046" },
    { 0x1882E, "TANGUT COMPONENT-047" },
    { 0x1882F, "TANGUT COMPONENT-048" },
    { 0x18830, "TANGUT COMPONENT-049" },
    { 0x18831, "TANGUT COMPONENT-050" },
    { 0x18832, "TANGUT COMPONENT-051" },
    { 0x18833, "TANGUT COMPONENT-052" },
    { 0x18834, "TANGUT COMPONENT-053" },
    { 0x18835, "TANGUT COMPONENT-054" },
    { 0x18836, "TANGUT COMPONENT-055" },
    { 0x18837, "TANGUT COMPONENT-056" },
    { 0x18838, "TANGUT COMPONENT-057" },
    { 0x18839, "TANGUT COMPONENT-058" },
    { 0x1883A, "TANGUT COMPONENT-059" },
    { 0x1883B, "TANGUT COMPONENT-060" },
    { 0x1883C, "TANGUT COMPONENT-061" },
    { 0x1883D, "TANGUT COMPONENT-062" },
    { 0x1883E, "TANGUT COMPONENT-063" },
    { 0x1883F, "TANGUT COMPONENT-064" },
    { 0x18840, "TANGUT COMPONENT-065" },
    { 0x18841, "TANGUT COMPONENT-066" },
    { 0x18842, "TANGUT COMPONENT-067" },
    { 0x18843, "TANGUT COMPONENT-068" },
    { 0x18844, "TANGUT COMPONENT-069" },
    { 0x18845, "TANGUT COMPONENT-070" },
    { 0x18846, "TANGUT COMPONENT-071" },
    { 0x18847, "TANGUT COMPONENT-072" },
    { 0x18848, "TANGUT COMPONENT-073" },
    { 0x18849, "TANGUT COMPONENT-074" },
    { 0x1884A, "TANGUT COMPONENT-075" },
    { 0x1884B, "TANGUT COMPONENT-076" },
    { 0x1884C, "TANGUT COMPONENT-077" },
    { 0x1884D, "TANGUT COMPONENT-078" },
    { 0x1884E, "TANGUT COMPONENT-079" },
    { 0x1884F, "TANGUT COMPONENT-080" },
    { 0x18850, "TANGUT COMPONENT-081" },
    { 0x18851, "TANGUT COMPONENT-082" },
    { 0x18852, "TANGUT COMPONENT-083" },
    { 0x18853, "TANGUT COMPONENT-084" },
    { 0x18854, "TANGUT COMPONENT-085" },
    { 0x18855, "TANGUT COMPONENT-086" },
    { 0x18856, "TANGUT COMPONENT-087" },
    { 0x18857, "TANGUT COMPONENT-088" },
    { 0x18858, "TANGUT COMPONENT-089" },
    { 0x18859, "TANGUT COMPONENT-090" },
    { 0x1885A, "TANGUT COMPONENT-091" },
    { 0x1885B, "TANGUT COMPONENT-092" },
    { 0x1885C, "TANGUT COMPONENT-093" },
    { 0x1885D, "TANGUT COMPONENT-094" },
    { 0x1885E, "TANGUT COMPONENT-095" },
    { 0x1885F, "TANGUT COMPONENT-096" },
    { 0x18860, "TANGUT COMPONENT-097" },
    { 0x18861, "TANGUT COMPONENT-098" },
    { 0x18862, "TANGUT COMPONENT-099" },
    { 0x18863, "TANGUT COMPONENT-100" },
    { 0x18864, "TANGUT COMPONENT-101" },
    { 0x18865, "TANGUT COMPONENT-102" },
    { 0x18866, "TANGUT COMPONENT-103" },
    { 0x18867, "TANGUT COMPONENT-104" },
    { 0x18868, "TANGUT COMPONENT-105" },
    { 0x18869, "TANGUT COMPONENT-106" },
    { 0x1886A, "TANGUT COMPONENT-107" },
    { 0x1886B, "TANGUT COMPONENT-108" },
    { 0x1886C, "TANGUT COMPONENT-109" },
    { 0x1886D, "TANGUT COMPONENT-110" },
    { 0x1886E, "TANGUT COMPONENT-111" },
    { 0x1886F, "TANGUT COMPONENT-112" },
    { 0x18870, "TANGUT COMPONENT-113" },
    { 0x18871, "TANGUT COMPONENT-114" },
    { 0x18872, "TANGUT COMPONENT-115" },
    { 0x18873, "TANGUT COMPONENT-116" },
    { 0x18874, "TANGUT COMPONENT-117" },
    { 0x18875, "TANGUT COMPONENT-118" },
    { 0x18876, "TANGUT COMPONENT-119" },
    { 0x18877, "TANGUT COMPONENT-120" },
    { 0x18878, "TANGUT COMPONENT-121" },
    { 0x18879, "TANGUT COMPONENT-122" },
    { 0x1887A, "TANGUT COMPONENT-123" },
    { 0x1887B, "TANGUT COMPONENT-124" },
    { 0x1887C, "TANGUT COMPONENT-125" },
    { 0x1887D, "TANGUT COMPONENT-126" },
    { 0x1887E, "TANGUT COMPONENT-127" },
    { 0x1887F, "TANGUT COMPONENT-128" },
    { 0x18880, "TANGUT COMPONENT-129" },
    { 0x18881, "TANGUT COMPONENT-130" },
    { 0x18882, "TANGUT COMPONENT-131" },
    { 0x18883, "TANGUT COMPONENT-132" },
    { 0x18884, "TANGUT COMPONENT-133" },
    { 0x18885, "TANGUT COMPONENT-134" },
    { 0x18886, "TANGUT COMPONENT-135" },
    { 0x18887, "TANGUT COMPONENT-136" },
    { 0x18888, "TANGUT COMPONENT-137" },
    { 0x18889, "TANGUT COMPONENT-138" },
    { 0x1888A, "TANGUT COMPONENT-139" },
    { 0x1888B, "TANGUT COMPONENT-140" },
    { 0x1888C, "TANGUT COMPONENT-141" },
    { 0x1888D, "TANGUT COMPONENT-142" },
    { 0x1888E, "TANGUT COMPONENT-143" },
    { 0x1888F, "TANGUT COMPONENT-144" },
    { 0x18890, "TANGUT COMPONENT-145" },
    { 0x18891, "TANGUT COMPONENT-146" },
    { 0x18892, "TANGUT COMPONENT-147" },
    { 0x18893, "TANGUT COMPONENT-148" },
    { 0x18894, "TANGUT COMPONENT-149" },
    { 0x18895, "TANGUT COMPONENT-150" },
    { 0x18896, "TANGUT COMPONENT-151" },
    { 0x18897, "TANGUT COMPONENT-152" },
    { 0x18898, "TANGUT COMPONENT-153" },
    { 0x18899, "TANGUT COMPONENT-154" },
    { 0x1889A, "TANGUT COMPONENT-155" },
    { 0x1889B, "TANGUT COMPONENT-156" },
    { 0x1889C, "TANGUT COMPONENT-157" },
    { 0x1889D, "TANGUT COMPONENT-158" },
    { 0x1889E, "TANGUT COMPONENT-159" },
    { 0x1889F, "TANGUT COMPONENT-160" },
    { 0x188A0, "TANGUT COMPONENT-161" },
    { 0x188A1, "TANGUT COMPONENT-162" },
    { 0x188A2, "TANGUT COMPONENT-163" },
    { 0x188A3, "TANGUT COMPONENT-164" },
    { 0x188A4, "TANGUT COMPONENT-165" },
    { 0x188A5, "TANGUT COMPONENT-166" },
    { 0x188A6, "TANGUT COMPONENT-167" },
    { 0x188A7, "TANGUT COMPONENT-168" },
    { 0x188A8, "TANGUT COMPONENT-169" },
    { 0x188A9, "TANGUT COMPONENT-170" },
    { 0x188AA, "TANGUT COMPONENT-171" },
    { 0x188AB, "TANGUT COMPONENT-172" },
    { 0x188AC, "TANGUT COMPONENT-173" },
    { 0x188AD, "TANGUT COMPONENT-174" },
    { 0x188AE, "TANGUT COMPONENT-175" },
    { 0x188AF, "TANGUT COMPONENT-176" },
    { 0x188B0, "TANGUT COMPONENT-177" },
    { 0x188B1, "TANGUT COMPONENT-178" },
    { 0x188B2, "TANGUT COMPONENT-179" },
    { 0x188B3, "TANGUT COMPONENT-180" },
    { 0x188B4, "TANGUT COMPONENT-181" },
    { 0x188B5, "TANGUT COMPONENT-182" },
    { 0x188B6, "TANGUT COMPONENT-183" },
    { 0x188B7, "TANGUT COMPONENT-184" },
    { 0x188B8, "TANGUT COMPONENT-185" },
    { 0x188B9, "TANGUT COMPONENT-186" },
    { 0x188BA, "TANGUT COMPONENT-187" },
    { 0x188BB, "TANGUT COMPONENT-188" },
    { 0x188BC, "TANGUT COMPONENT-189" },
    { 0x188BD, "TANGUT COMPONENT-190" },
    { 0x188BE, "TANGUT COMPONENT-191" },
    { 0x188BF, "TANGUT COMPONENT-192" },
    { 0x188C0, "TANGUT COMPONENT-193" },
    { 0x188C1, "TANGUT COMPONENT-194" },
    { 0x188C2, "TANGUT COMPONENT-195" },
    { 0x188C3, "TANGUT COMPONENT-196" },
    { 0x188C4, "TANGUT COMPONENT-197" },
    { 0x188C5, "TANGUT COMPONENT-198" },
    { 0x188C6, "TANGUT COMPONENT-199" },
    { 0x188C7, "TANGUT COMPONENT-200" },
    { 0x188C8, "TANGUT COMPONENT-201" },
    { 0x188C9, "TANGUT COMPONENT-202" },
    { 0x188CA, "TANGUT COMPONENT-203" },
    { 0x188CB, "TANGUT COMPONENT-204" },
    { 0x188CC, "TANGUT COMPONENT-205" },
    { 0x188CD, "TANGUT COMPONENT-206" },
    { 0x188CE, "TANGUT COMPONENT-207" },
    { 0x188CF, "TANGUT COMPONENT-208" },
    { 0x188D0, "TANGUT COMPONENT-209" },
    { 0x188D1, "TANGUT COMPONENT-210" },
    { 0x188D2, "TANGUT COMPONENT-211" },
    { 0x188D3, "TANGUT COMPONENT-212" },
    { 0x188D4, "TANGUT COMPONENT-213" },
    { 0x188D5, "TANGUT COMPONENT-214" },
    { 0x188D6, "TANGUT COMPONENT-215" },
    { 0x188D7, "TANGUT COMPONENT-216" },
    { 0x188D8, "TANGUT COMPONENT-217" },
    { 0x188D9, "TANGUT COMPONENT-218" },
    { 0x188DA, "TANGUT COMPONENT-219" },
    { 0x188DB, "TANGUT COMPONENT-220" },
    { 0x188DC, "TANGUT COMPONENT-221" },
    { 0x188DD, "TANGUT COMPONENT-222" },
    { 0x188DE, "TANGUT COMPONENT-223" },
    { 0x188DF, "TANGUT COMPONENT-224" },
    { 0x188E0, "TANGUT COMPONENT-225" },
    { 0x188E1, "TANGUT COMPONENT-226" },
    { 0x188E2, "TANGUT COMPONENT-227" },
    { 0x188E3, "TANGUT COMPONENT-228" },
    { 0x188E4, "TANGUT COMPONENT-229" },
    { 0x188E5, "TANGUT COMPONENT-230" },
    { 0x188E6, "TANGUT COMPONENT-231" },
    { 0x188E7, "TANGUT COMPONENT-232" },
    { 0x188E8, "TANGUT COMPONENT-233" },
    { 0x188E9, "TANGUT COMPONENT-234" },
    { 0x188EA, "TANGUT COMPONENT-235" },
    { 0x188EB, "TANGUT COMPONENT-236" },
    { 0x188EC, "TANGUT COMPONENT-237" },
    { 0x188ED, "TANGUT COMPONENT-238" },
    { 0x188EE, "TANGUT COMPONENT-239" },
    { 0x188EF, "TANGUT COMPONENT-240" },
    { 0x188F0, "TANGUT COMPONENT-241" },
    { 0x188F1, "TANGUT COMPONENT-242" },
    { 0x188F2, "TANGUT COMPONENT-243" },
    { 0x188F3, "TANGUT COMPONENT-244" },
    { 0x188F4, "TANGUT COMPONENT-245" },
    { 0x188F5, "TANGUT COMPONENT-246" },
    { 0x188F6, "TANGUT COMPONENT-247" },
    { 0x188F7, "TANGUT COMPONENT-248" },
    { 0x188F8, "TANGUT COMPONENT-249" },
    { 0x188F9, "TANGUT COMPONENT-250" },
    { 0x188FA, "TANGUT COMPONENT-251" },
    { 0x188FB, "TANGUT COMPONENT-252" },
    { 0x188FC, "TANGUT COMPONENT-253" },
    { 0x188FD, "TANGUT COMPONENT-254" },
    { 0x188FE, "TANGUT COMPONENT-255" },
    { 0x188FF, "TANGUT COMPONENT-256" },
    { 0x18900, "TANGUT COMPONENT-257" },
    { 0x18901, "TANGUT COMPONENT-258" },
    { 0x18902, "TANGUT COMPONENT-259" },
    { 0x18903, "TANGUT COMPONENT-260" },
    { 0x18904, "TANGUT COMPONENT-261" },
    { 0x18905, "TANGUT COMPONENT-262" },
    { 0x18906, "TANGUT COMPONENT-263" },
    { 0x18907, "TANGUT COMPONENT-264" },
    { 0x18908, "TANGUT COMPONENT-265" },
    { 0x18909, "TANGUT COMPONENT-266" },
    { 0x1890A, "TANGUT COMPONENT-267" },
    { 0x1890B, "TANGUT COMPONENT-268" },
    { 0x1890C, "TANGUT COMPONENT-269" },
    { 0x1890D, "TANGUT COMPONENT-270" },
    { 0x1890E, "TANGUT COMPONENT-271" },
    { 0x1890F, "TANGUT COMPONENT-272" },
    { 0x18910, "TANGUT COMPONENT-273" },
    { 0x18911, "TANGUT COMPONENT-274" },
    { 0x18912, "TANGUT COMPONENT-275" },
    { 0x18913, "TANGUT COMPONENT-276" },
    { 0x18914, "TANGUT COMPONENT-277" },
    { 0x18915, "TANGUT COMPONENT-278" },
    { 0x18916, "TANGUT COMPONENT-279" },
    { 0x18917, "TANGUT COMPONENT-280" },
    { 0x18918, "TANGUT COMPONENT-281" },
    { 0x18919, "TANGUT COMPONENT-282" },
    { 0x1891A, "TANGUT COMPONENT-283" },
    { 0x1891B, "TANGUT COMPONENT-284" },
    { 0x1891C, "TANGUT COMPONENT-285" },
    { 0x1891D, "TANGUT COMPONENT-286" },
    { 0x1891E, "TANGUT COMPONENT-287" },
    { 0x1891F, "TANGUT COMPONENT-288" },
    { 0x18920, "TANGUT COMPONENT-289" },
    { 0x18921, "TANGUT COMPONENT-290" },
    { 0x18922, "TANGUT COMPONENT-291" },
    { 0x18923, "TANGUT COMPONENT-292" },
    { 0x18924, "TANGUT COMPONENT-293" },
    { 0x18925, "TANGUT COMPONENT-294" },
    { 0x18926, "TANGUT COMPONENT-295" },
    { 0x18927, "TANGUT COMPONENT-296" },
    { 0x18928, "TANGUT COMPONENT-297" },
    { 0x18929, "TANGUT COMPONENT-298" },
    { 0x1892A, "TANGUT COMPONENT-299" },
    { 0x1892B, "TANGUT COMPONENT-300" },
    { 0x1892C, "TANGUT COMPONENT-301" },
    { 0x1892D, "TANGUT COMPONENT-302" },
    { 0x1892E, "TANGUT COMPONENT-303" },
    { 0x1892F, "TANGUT COMPONENT-304" },
    { 0x18930, "TANGUT COMPONENT-305" },
    { 0x18931, "TANGUT COMPONENT-306" },
    { 0x18932, "TANGUT COMPONENT-307" },
    { 0x18933, "TANGUT COMPONENT-308" },
    { 0x18934, "TANGUT COMPONENT-309" },
    { 0x18935, "TANGUT COMPONENT-310" },
    { 0x18936, "TANGUT COMPONENT-311" },
    { 0x18937, "TANGUT COMPONENT-312" },
    { 0x18938, "TANGUT COMPONENT-313" },
    { 0x18939, "TANGUT COMPONENT-314" },
    { 0x1893A, "TANGUT COMPONENT-315" },
    { 0x1893B, "TANGUT COMPONENT-316" },
    { 0x1893C, "TANGUT COMPONENT-317" },
    { 0x1893D, "TANGUT COMPONENT-318" },
    { 0x1893E, "TANGUT COMPONENT-319" },
    { 0x1893F, "TANGUT COMPONENT-320" },
    { 0x18940, "TANGUT COMPONENT-321" },
    { 0x18941, "TANGUT COMPONENT-322" },
    { 0x18942, "TANGUT COMPONENT-323" },
    { 0x18943, "TANGUT COMPONENT-324" },
    { 0x18944, "TANGUT COMPONENT-325" },
    { 0x18945, "TANGUT COMPONENT-326" },
    { 0x18946, "TANGUT COMPONENT-327" },
    { 0x18947, "TANGUT COMPONENT-328" },
    { 0x18948, "TANGUT COMPONENT-329" },
    { 0x18949, "TANGUT COMPONENT-330" },
    { 0x1894A, "TANGUT COMPONENT-331" },
    { 0x1894B, "TANGUT COMPONENT-332" },
    { 0x1894C, "TANGUT COMPONENT-333" },
    { 0x1894D, "TANGUT COMPONENT-334" },
    { 0x1894E, "TANGUT COMPONENT-335" },
    { 0x1894F, "TANGUT COMPONENT-336" },
    { 0x18950, "TANGUT COMPONENT-337" },
    { 0x18951, "TANGUT COMPONENT-338" },
    { 0x18952, "TANGUT COMPONENT-339" },
    { 0x18953, "TANGUT COMPONENT-340" },
    { 0x18954, "TANGUT COMPONENT-341" },
    { 0x18955, "TANGUT COMPONENT-342" },
    { 0x18956, "TANGUT COMPONENT-343" },
    { 0x18957, "TANGUT COMPONENT-344" },
    { 0x18958, "TANGUT COMPONENT-345" },
    { 0x18959, "TANGUT COMPONENT-346" },
    { 0x1895A, "TANGUT COMPONENT-347" },
    { 0x1895B, "TANGUT COMPONENT-348" },
    { 0x1895C, "TANGUT COMPONENT-349" },
    { 0x1895D, "TANGUT COMPONENT-350" },
    { 0x1895E, "TANGUT COMPONENT-351" },
    { 0x1895F, "TANGUT COMPONENT-352" },
    { 0x18960, "TANGUT COMPONENT-353" },
    { 0x18961, "TANGUT COMPONENT-354" },
    { 0x18962, "TANGUT COMPONENT-355" },
    { 0x18963, "TANGUT COMPONENT-356" },
    { 0x18964, "TANGUT COMPONENT-357" },
    { 0x18965, "TANGUT COMPONENT-358" },
    { 0x18966, "TANGUT COMPONENT-359" },
    { 0x18967, "TANGUT COMPONENT-360" },
    { 0x18968, "TANGUT COMPONENT-361" },
    { 0x18969, "TANGUT COMPONENT-362" },
    { 0x1896A, "TANGUT COMPONENT-363" },
    { 0x1896B, "TANGUT COMPONENT-364" },
    { 0x1896C, "TANGUT COMPONENT-365" },
    { 0x1896D, "TANGUT COMPONENT-366" },
    { 0x1896E, "TANGUT COMPONENT-367" },
    { 0x1896F, "TANGUT COMPONENT-368" },
    { 0x18970, "TANGUT COMPONENT-369" },
    { 0x18971, "TANGUT COMPONENT-370" },
    { 0x18972, "TANGUT COMPONENT-371" },
    { 0x18973, "TANGUT COMPONENT-372" },
    { 0x18974, "TANGUT COMPONENT-373" },
    { 0x18975, "TANGUT COMPONENT-374" },
    { 0x18976, "TANGUT COMPONENT-375" },
    { 0x18977, "TANGUT COMPONENT-376" },
    { 0x18978, "TANGUT COMPONENT-377" },
    { 0x18979, "TANGUT COMPONENT-378" },
    { 0x1897A, "TANGUT COMPONENT-379" },
    { 0x1897B, "TANGUT COMPONENT-380" },
    { 0x1897C, "TANGUT COMPONENT-381" },
    { 0x1897D, "TANGUT COMPONENT-382" },
    { 0x1897E, "TANGUT COMPONENT-383" },
    { 0x1897F, "TANGUT COMPONENT-384" },
    { 0x18980, "TANGUT COMPONENT-385" },
    { 0x18981, "TANGUT COMPONENT-386" },
    { 0x18982, "TANGUT COMPONENT-387" },
    { 0x18983, "TANGUT COMPONENT-388" },
    { 0x18984, "TANGUT COMPONENT-389" },
    { 0x18985, "TANGUT COMPONENT-390" },
    { 0x18986, "TANGUT COMPONENT-391" },
    { 0x18987, "TANGUT COMPONENT-392" },
    { 0x18988, "TANGUT COMPONENT-393" },
    { 0x18989, "TANGUT COMPONENT-394" },
    { 0x1898A, "TANGUT COMPONENT-395" },
    { 0x1898B, "TANGUT COMPONENT-396" },
    { 0x1898C, "TANGUT COMPONENT-397" },
    { 0x1898D, "TANGUT COMPONENT-398" },
    { 0x1898E, "TANGUT COMPONENT-399" },
    { 0x1898F, "TANGUT COMPONENT-400" },
    { 0x18990, "TANGUT COMPONENT-401" },
    { 0x18991, "TANGUT COMPONENT-402" },
    { 0x18992, "TANGUT COMPONENT-403" },
    { 0x18993, "TANGUT COMPONENT-404" },
    { 0x18994, "TANGUT COMPONENT-405" },
    { 0x18995, "TANGUT COMPONENT-406" },
    { 0x18996, "TANGUT COMPONENT-407" },
    { 0x18997, "TANGUT COMPONENT-408" },
    { 0x18998, "TANGUT COMPONENT-409" },
    { 0x18999, "TANGUT COMPONENT-410" },
    { 0x1899A, "TANGUT COMPONENT-411" },
    { 0x1899B, "TANGUT COMPONENT-412" },
    { 0x1899C, "TANGUT COMPONENT-413" },
    { 0x1899D, "TANGUT COMPONENT-414" },
    { 0x1899E, "TANGUT COMPONENT-415" },
    { 0x1899F, "TANGUT COMPONENT-416" },
    { 0x189A0, "TANGUT COMPONENT-417" },
    { 0x189A1, "TANGUT COMPONENT-418" },
    { 0x189A2, "TANGUT COMPONENT-419" },
    { 0x189A3, "TANGUT COMPONENT-420" },
    { 0x189A4, "TANGUT COMPONENT-421" },
    { 0x189A5, "TANGUT COMPONENT-422" },
    { 0x189A6, "TANGUT COMPONENT-423" },
    { 0x189A7, "TANGUT COMPONENT-424" },
    { 0x189A8, "TANGUT COMPONENT-425" },
    { 0x189A9, "TANGUT COMPONENT-426" },
    { 0x189AA, "TANGUT COMPONENT-427" },
    { 0x189AB, "TANGUT COMPONENT-428" },
    { 0x189AC, "TANGUT COMPONENT-429" },
    { 0x189AD, "TANGUT COMPONENT-430" },
    { 0x189AE, "TANGUT COMPONENT-431" },
    { 0x189AF, "TANGUT COMPONENT-432" },
    { 0x189B0, "TANGUT COMPONENT-433" },
    { 0x189B1, "TANGUT COMPONENT-434" },
    { 0x189B2, "TANGUT COMPONENT-435" },
    { 0x189B3, "TANGUT COMPONENT-436" },
    { 0x189B4, "TANGUT COMPONENT-437" },
    { 0x189B5, "TANGUT COMPONENT-438" },
    { 0x189B6, "TANGUT COMPONENT-439" },
    { 0x189B7, "TANGUT COMPONENT-440" },
    { 0x189B8, "TANGUT COMPONENT-441" },
    { 0x189B9, "TANGUT COMPONENT-442" },
    { 0x189BA, "TANGUT COMPONENT-443" },
    { 0x189BB, "TANGUT COMPONENT-444" },
    { 0x189BC, "TANGUT COMPONENT-445" },
    { 0x189BD, "TANGUT COMPONENT-446" },
    { 0x189BE, "TANGUT COMPONENT-447" },
    { 0x189BF, "TANGUT COMPONENT-448" },
    { 0x189C0, "TANGUT COMPONENT-449" },
    { 0x189C1, "TANGUT COMPONENT-450" },
    { 0x189C2, "TANGUT COMPONENT-451" },
    { 0x189C3, "TANGUT COMPONENT-452" },
    { 0x189C4, "TANGUT COMPONENT-453" },
    { 0x189C5, "TANGUT COMPONENT-454" },
    { 0x189C6, "TANGUT COMPONENT-455" },
    { 0x189C7, "TANGUT COMPONENT-456" },
    { 0x189C8, "TANGUT COMPONENT-457" },
    { 0x189C9, "TANGUT COMPONENT-458" },
    { 0x189CA, "TANGUT COMPONENT-459" },
    { 0x189CB, "TANGUT COMPONENT-460" },
    { 0x189CC, "TANGUT COMPONENT-461" },
    { 0x189CD, "TANGUT COMPONENT-462" },
    { 0x189CE, "TANGUT COMPONENT-463" },
    { 0x189CF, "TANGUT COMPONENT-464" },
    { 0x189D0, "TANGUT COMPONENT-465" },
    { 0x189D1, "TANGUT COMPONENT-466" },
    { 0x189D2, "TANGUT COMPONENT-467" },
    { 0x189D3, "TANGUT COMPONENT-468" },
    { 0x189D4, "TANGUT COMPONENT-469" },
    { 0x189D5, "TANGUT COMPONENT-470" },
    { 0x189D6, "TANGUT COMPONENT-471" },
    { 0x189D7, "TANGUT COMPONENT-472" },
    { 0x189D8, "TANGUT COMPONENT-473" },
    { 0x189D9, "TANGUT COMPONENT-474" },
    { 0x189DA, "TANGUT COMPONENT-475" },
    { 0x189DB, "TANGUT COMPONENT-476" },
    { 0x189DC, "TANGUT COMPONENT-477" },
    { 0x189DD, "TANGUT COMPONENT-478" },
    { 0x189DE, "TANGUT COMPONENT-479" },
    { 0x189DF, "TANGUT COMPONENT-480" },
    { 0x189E0, "TANGUT COMPONENT-481" },
    { 0x189E1, "TANGUT COMPONENT-482" },
    { 0x189E2, "TANGUT COMPONENT-483" },
    { 0x189E3, "TANGUT COMPONENT-484" },
    { 0x189E4, "TANGUT COMPONENT-485" },
    { 0x189E5, "TANGUT COMPONENT-486" },
    { 0x189E6, "TANGUT COMPONENT-487" },
    { 0x189E7, "TANGUT COMPONENT-488" },
    { 0x189E8, "TANGUT COMPONENT-489" },
    { 0x189E9, "TANGUT COMPONENT-490" },
    { 0x189EA, "TANGUT COMPONENT-491" },
    { 0x189EB, "TANGUT COMPONENT-492" },
    { 0x189EC, "TANGUT COMPONENT-493" },
    { 0x189ED, "TANGUT COMPONENT-494" },
    { 0x189EE, "TANGUT COMPONENT-495" },
    { 0x189EF, "TANGUT COMPONENT-496" },
    { 0x189F0, "TANGUT COMPONENT-497" },
    { 0x189F1, "TANGUT COMPONENT-498" },
    { 0x189F2, "TANGUT COMPONENT-499" },
    { 0x189F3, "TANGUT COMPONENT-500" },
    { 0x189F4, "TANGUT COMPONENT-501" },
    { 0x189F5, "TANGUT COMPONENT-502" },
    { 0x189F6, "TANGUT COMPONENT-503" },
    { 0x189F7, "TANGUT COMPONENT-504" },
    { 0x189F8, "TANGUT COMPONENT-505" },
    { 0x189F9, "TANGUT COMPONENT-506" },
    { 0x189FA, "TANGUT COMPONENT-507" },
    { 0x189FB, "TANGUT COMPONENT-508" },
    { 0x189FC, "TANGUT COMPONENT-509" },
    { 0x189FD, "TANGUT COMPONENT-510" },
    { 0x189FE, "TANGUT COMPONENT-511" },
    { 0x189FF, "TANGUT COMPONENT-512" },
    { 0x18A00, "TANGUT COMPONENT-513" },
    { 0x18A01, "TANGUT COMPONENT-514" },
    { 0x18A02, "TANGUT COMPONENT-515" },
    { 0x18A03, "TANGUT COMPONENT-516" },
    { 0x18A04, "TANGUT COMPONENT-517" },
    { 0x18A05, "TANGUT COMPONENT-518" },
    { 0x18A06, "TANGUT COMPONENT-519" },
    { 0x18A07, "TANGUT COMPONENT-520" },
    { 0x18A08, "TANGUT COMPONENT-521" },
    { 0x18A09, "TANGUT COMPONENT-522" },
    { 0x18A0A, "TANGUT COMPONENT-523" },
    { 0x18A0B, "TANGUT COMPONENT-524" },
    { 0x18A0C, "TANGUT COMPONENT-525" },
    { 0x18A0D, "TANGUT COMPONENT-526" },
    { 0x18A0E, "TANGUT COMPONENT-527" },
    { 0x18A0F, "TANGUT COMPONENT-528" },
    { 0x18A10, "TANGUT COMPONENT-529" },
    { 0x18A11, "TANGUT COMPONENT-530" },
    { 0x18A12, "TANGUT COMPONENT-531" },
    { 0x18A13, "TANGUT COMPONENT-532" },
    { 0x18A14, "TANGUT COMPONENT-533" },
    { 0x18A15, "TANGUT COMPONENT-534" },
    { 0x18A16, "TANGUT COMPONENT-535" },
    { 0x18A17, "TANGUT COMPONENT-536" },
    { 0x18A18, "TANGUT COMPONENT-537" },
    { 0x18A19, "TANGUT COMPONENT-538" },
    { 0x18A1A, "TANGUT COMPONENT-539" },
    { 0x18A1B, "TANGUT COMPONENT-540" },
    { 0x18A1C, "TANGUT COMPONENT-541" },
    { 0x18A1D, "TANGUT COMPONENT-542" },
    { 0x18A1E, "TANGUT COMPONENT-543" },
    { 0x18A1F, "TANGUT COMPONENT-544" },
    { 0x18A20, "TANGUT COMPONENT-545" },
    { 0x18A21, "TANGUT COMPONENT-546" },
    { 0x18A22, "TANGUT COMPONENT-547" },
    { 0x18A23, "TANGUT COMPONENT-548" },
    { 0x18A24, "TANGUT COMPONENT-549" },
    { 0x18A25, "TANGUT COMPONENT-550" },
    { 0x18A26, "TANGUT COMPONENT-551" },
    { 0x18A27, "TANGUT COMPONENT-552" },
    { 0x18A28, "TANGUT COMPONENT-553" },
    { 0x18A29, "TANGUT COMPONENT-554" },
    { 0x18A2A, "TANGUT COMPONENT-555" },
    { 0x18A2B, "TANGUT COMPONENT-556" },
    { 0x18A2C, "TANGUT COMPONENT-557" },
    { 0x18A2D, "TANGUT COMPONENT-558" },
    { 0x18A2E, "TANGUT COMPONENT-559" },
    { 0x18A2F, "TANGUT COMPONENT-560" },
    { 0x18A30, "TANGUT COMPONENT-561" },
    { 0x18A31, "TANGUT COMPONENT-562" },
    { 0x18A32, "TANGUT COMPONENT-563" },
    { 0x18A33, "TANGUT COMPONENT-564" },
    { 0x18A34, "TANGUT COMPONENT-565" },
    { 0x18A35, "TANGUT COMPONENT-566" },
    { 0x18A36, "TANGUT COMPONENT-567" },
    { 0x18A37, "TANGUT COMPONENT-568" },
    { 0x18A38, "TANGUT COMPONENT-569" },
    { 0x18A39, "TANGUT COMPONENT-570" },
    { 0x18A3A, "TANGUT COMPONENT-571" },
    { 0x18A3B, "TANGUT COMPONENT-572" },
    { 0x18A3C, "TANGUT COMPONENT-573" },
    { 0x18A3D, "TANGUT COMPONENT-574" },
    { 0x18A3E, "TANGUT COMPONENT-575" },
    { 0x18A3F, "TANGUT COMPONENT-576" },
    { 0x18A40, "TANGUT COMPONENT-577" },
    { 0x18A41, "TANGUT COMPONENT-578" },
    { 0x18A42, "TANGUT COMPONENT-579" },
    { 0x18A43, "TANGUT COMPONENT-580" },
    { 0x18A44, "TANGUT COMPONENT-581" },
    { 0x18A45, "TANGUT COMPONENT-582" },
    { 0x18A46, "TANGUT COMPONENT-583" },
    { 0x18A47, "TANGUT COMPONENT-584" },
    { 0x18A48, "TANGUT COMPONENT-585" },
    { 0x18A49, "TANGUT COMPONENT-586" },
    { 0x18A4A, "TANGUT COMPONENT-587" },
    { 0x18A4B, "TANGUT COMPONENT-588" },
    { 0x18A4C, "TANGUT COMPONENT-589" },
    { 0x18A4D, "TANGUT COMPONENT-590" },
    { 0x18A4E, "TANGUT COMPONENT-591" },
    { 0x18A4F, "TANGUT COMPONENT-592" },
    { 0x18A50, "TANGUT COMPONENT-593" },
    { 0x18A51, "TANGUT COMPONENT-594" },
    { 0x18A52, "TANGUT COMPONENT-595" },
    { 0x18A53, "TANGUT COMPONENT-596" },
    { 0x18A54, "TANGUT COMPONENT-597" },
    { 0x18A55, "TANGUT COMPONENT-598" },
    { 0x18A56, "TANGUT COMPONENT-599" },
    { 0x18A57, "TANGUT COMPONENT-600" },
    { 0x18A58, "TANGUT COMPONENT-601" },
    { 0x18A59, "TANGUT COMPONENT-602" },
    { 0x18A5A, "TANGUT COMPONENT-603" },
    { 0x18A5B, "TANGUT COMPONENT-604" },
    { 0x18A5C, "TANGUT COMPONENT-605" },
    { 0x18A5D, "TANGUT COMPONENT-606" },
    { 0x18A5E, "TANGUT COMPONENT-607" },
    { 0x18A5F, "TANGUT COMPONENT-608" },
    { 0x18A60, "TANGUT COMPONENT-609" },
    { 0x18A61, "TANGUT COMPONENT-610" },
    { 0x18A62, "TANGUT COMPONENT-611" },
    { 0x18A63, "TANGUT COMPONENT-612" },
    { 0x18A64, "TANGUT COMPONENT-613" },
    { 0x18A65, "TANGUT COMPONENT-614" },
    { 0x18A66, "TANGUT COMPONENT-615" },
    { 0x18A67, "TANGUT COMPONENT-616" },
    { 0x18A68, "TANGUT COMPONENT-617" },
    { 0x18A69, "TANGUT COMPONENT-618" },
    { 0x18A6A, "TANGUT COMPONENT-619" },
    { 0x18A6B, "TANGUT COMPONENT-620" },
    { 0x18A6C, "TANGUT COMPONENT-621" },
    { 0x18A6D, "TANGUT COMPONENT-622" },
    { 0x18A6E, "TANGUT COMPONENT-623" },
    { 0x18A6F, "TANGUT COMPONENT-624" },
    { 0x18A70, "TANGUT COMPONENT-625" },
    { 0x18A71, "TANGUT COMPONENT-626" },
    { 0x18A72, "TANGUT COMPONENT-627" },
    { 0x18A73, "TANGUT COMPONENT-628" },
    { 0x18A74, "TANGUT COMPONENT-629" },
    { 0x18A75, "TANGUT COMPONENT-630" },
    { 0x18A76, "TANGUT COMPONENT-631" },
    { 0x18A77, "TANGUT COMPONENT-632" },
    { 0x18A78, "TANGUT COMPONENT-633" },
    { 0x18A79, "TANGUT COMPONENT-634" },
    { 0x18A7A, "TANGUT COMPONENT-635" },
    { 0x18A7B, "TANGUT COMPONENT-636" },
    { 0x18A7C, "TANGUT COMPONENT-637" },
    { 0x18A7D, "TANGUT COMPONENT-638" },
    { 0x18A7E, "TANGUT COMPONENT-639" },
    { 0x18A7F, "TANGUT COMPONENT-640" },
    { 0x18A80, "TANGUT COMPONENT-641" },
    { 0x18A81, "TANGUT COMPONENT-642" },
    { 0x18A82, "TANGUT COMPONENT-643" },
    { 0x18A83, "TANGUT COMPONENT-644" },
    { 0x18A84, "TANGUT COMPONENT-645" },
    { 0x18A85, "TANGUT COMPONENT-646" },
    { 0x18A86, "TANGUT COMPONENT-647" },
    { 0x18A87, "TANGUT COMPONENT-648" },
    { 0x18A88, "TANGUT COMPONENT-649" },
    { 0x18A89, "TANGUT COMPONENT-650" },
    { 0x18A8A, "TANGUT COMPONENT-651" },
    { 0x18A8B, "TANGUT COMPONENT-652" },
    { 0x18A8C, "TANGUT COMPONENT-653" },
    { 0x18A8D, "TANGUT COMPONENT-654" },
    { 0x18A8E, "TANGUT COMPONENT-655" },
    { 0x18A8F, "TANGUT COMPONENT-656" },
    { 0x18A90, "TANGUT COMPONENT-657" },
    { 0x18A91, "TANGUT COMPONENT-658" },
    { 0x18A92, "TANGUT COMPONENT-659" },
    { 0x18A93, "TANGUT COMPONENT-660" },
    { 0x18A94, "TANGUT COMPONENT-661" },
    { 0x18A95, "TANGUT COMPONENT-662" },
    { 0x18A96, "TANGUT COMPONENT-663" },
    { 0x18A97, "TANGUT COMPONENT-664" },
    { 0x18A98, "TANGUT COMPONENT-665" },
    { 0x18A99, "TANGUT COMPONENT-666" },
    { 0x18A9A, "TANGUT COMPONENT-667" },
    { 0x18A9B, "TANGUT COMPONENT-668" },
    { 0x18A9C, "TANGUT COMPONENT-669" },
    { 0x18A9D, "TANGUT COMPONENT-670" },
    { 0x18A9E, "TANGUT COMPONENT-671" },
    { 0x18A9F, "TANGUT COMPONENT-672" },
    { 0x18AA0, "TANGUT COMPONENT-673" },
    { 0x18AA1, "TANGUT COMPONENT-674" },
    { 0x18AA2, "TANGUT COMPONENT-675" },
    { 0x18AA3, "TANGUT COMPONENT-676" },
    { 0x18AA4, "TANGUT COMPONENT-677" },
    { 0x18AA5, "TANGUT COMPONENT-678" },
    { 0x18AA6, "TANGUT COMPONENT-679" },
    { 0x18AA7, "TANGUT COMPONENT-680" },
    { 0x18AA8, "TANGUT COMPONENT-681" },
    { 0x18AA9, "TANGUT COMPONENT-682" },
    { 0x18AAA, "TANGUT COMPONENT-683" },
    { 0x18AAB, "TANGUT COMPONENT-684" },
    { 0x18AAC, "TANGUT COMPONENT-685" },
    { 0x18AAD, "TANGUT COMPONENT-686" },
    { 0x18AAE, "TANGUT COMPONENT-687" },
    { 0x18AAF, "TANGUT COMPONENT-688" },
    { 0x18AB0, "TANGUT COMPONENT-689" },
    { 0x18AB1, "TANGUT COMPONENT-690" },
    { 0x18AB2, "TANGUT COMPONENT-691" },
    { 0x18AB3, "TANGUT COMPONENT-692" },
    { 0x18AB4, "TANGUT COMPONENT-693" },
    { 0x18AB5, "TANGUT COMPONENT-694" },
    { 0x18AB6, "TANGUT COMPONENT-695" },
    { 0x18AB7, "TANGUT COMPONENT-696" },
    { 0x18AB8, "TANGUT COMPONENT-697" },
    { 0x18AB9, "TANGUT COMPONENT-698" },
    { 0x18ABA, "TANGUT COMPONENT-699" },
    { 0x18ABB, "TANGUT COMPONENT-700" },
    { 0x18ABC, "TANGUT COMPONENT-701" },
    { 0x18ABD, "TANGUT COMPONENT-702" },
    { 0x18ABE, "TANGUT COMPONENT-703" },
    { 0x18ABF, "TANGUT COMPONENT-704" },
    { 0x18AC0, "TANGUT COMPONENT-705" },
    { 0x18AC1, "TANGUT COMPONENT-706" },
    { 0x18AC2, "TANGUT COMPONENT-707" },
    { 0x18AC3, "TANGUT COMPONENT-708" },
    { 0x18AC4, "TANGUT COMPONENT-709" },
    { 0x18AC5, "TANGUT COMPONENT-710" },
    { 0x18AC6, "TANGUT COMPONENT-711" },
    { 0x18AC7, "TANGUT COMPONENT-712" },
    { 0x18AC8, "TANGUT COMPONENT-713" },
    { 0x18AC9, "TANGUT COMPONENT-714" },
    { 0x18ACA, "TANGUT COMPONENT-715" },
    { 0x18ACB, "TANGUT COMPONENT-716" },
    { 0x18ACC, "TANGUT COMPONENT-717" },
    { 0x18ACD, "TANGUT COMPONENT-718" },
    { 0x18ACE, "TANGUT COMPONENT-719" },
    { 0x18ACF, "TANGUT COMPONENT-720" },
    { 0x18AD0, "TANGUT COMPONENT-721" },
    { 0x18AD1, "TANGUT COMPONENT-722" },
    { 0x18AD2, "TANGUT COMPONENT-723" },
    { 0x18AD3, "TANGUT COMPONENT-724" },
    { 0x18AD4, "TANGUT COMPONENT-725" },
    { 0x18AD5, "TANGUT COMPONENT-726" },
    { 0x18AD6, "TANGUT COMPONENT-727" },
    { 0x18AD7, "TANGUT COMPONENT-728" },
    { 0x18AD8, "TANGUT COMPONENT-729" },
    { 0x18AD9, "TANGUT COMPONENT-730" },
    { 0x18ADA, "TANGUT COMPONENT-731" },
    { 0x18ADB, "TANGUT COMPONENT-732" },
    { 0x18ADC, "TANGUT COMPONENT-733" },
    { 0x18ADD, "TANGUT COMPONENT-734" },
    { 0x18ADE, "TANGUT COMPONENT-735" },
    { 0x18ADF, "TANGUT COMPONENT-736" },
    { 0x18AE0, "TANGUT COMPONENT-737" },
    { 0x18AE1, "TANGUT COMPONENT-738" },
    { 0x18AE2, "TANGUT COMPONENT-739" },
    { 0x18AE3, "TANGUT COMPONENT-740" },
    { 0x18AE4, "TANGUT COMPONENT-741" },
    { 0x18AE5, "TANGUT COMPONENT-742" },
    { 0x18AE6, "TANGUT COMPONENT-743" },
    { 0x18AE7, "TANGUT COMPONENT-744" },
    { 0x18AE8, "TANGUT COMPONENT-745" },
    { 0x18AE9, "TANGUT COMPONENT-746" },
    { 0x18AEA, "TANGUT COMPONENT-747" },
    { 0x18AEB, "TANGUT COMPONENT-748" },
    { 0x18AEC, "TANGUT COMPONENT-749" },
    { 0x18AED, "TANGUT COMPONENT-750" },
    { 0x18AEE, "TANGUT COMPONENT-751" },
    { 0x18AEF, "TANGUT COMPONENT-752" },
    { 0x18AF0, "TANGUT COMPONENT-753" },
    { 0x18AF1, "TANGUT COMPONENT-754" },
    { 0x18AF2, "TANGUT COMPONENT-755" },
    { 0x18AF3, "TANGUT COMPONENT-756" },
    { 0x18AF4, "TANGUT COMPONENT-757" },
    { 0x18AF5, "TANGUT COMPONENT-758" },
    { 0x18AF6, "TANGUT COMPONENT-759" },
    { 0x18AF7, "TANGUT COMPONENT-760" },
    { 0x18AF8, "TANGUT COMPONENT-761" },
    { 0x18AF9, "TANGUT COMPONENT-762" },
    { 0x18AFA, "TANGUT COMPONENT-763" },
    { 0x18AFB, "TANGUT COMPONENT-764" },
    { 0x18AFC, "TANGUT COMPONENT-765" },
    { 0x18AFD, "TANGUT COMPONENT-766" },
    { 0x18AFE, "TANGUT COMPONENT-767" },
    { 0x18AFF, "TANGUT COMPONENT-768" },
    { 0x18B00, "KHITAN SMALL SCRIPT CHARACTER-18B00" },
    { 0x18B01, "KHITAN SMALL SCRIPT CHARACTER-18B01" },
    { 0x18B02, "KHITAN SMALL SCRIPT CHARACTER-18B02" },
    { 0x18B03, "KHITAN SMALL SCRIPT CHARACTER-18B03" },
    { 0x18B04, "KHITAN SMALL SCRIPT CHARACTER-18B04" },
    { 0x18B05, "KHITAN SMALL SCRIPT CHARACTER-18B05" },
    { 0x18B06, "KHITAN SMALL SCRIPT CHARACTER-18B06" },
    { 0x18B07, "KHITAN SMALL SCRIPT CHARACTER-18B07" },
    { 0x18B08, "KHITAN SMALL SCRIPT CHARACTER-18B08" },
    { 0x18B09, "KHITAN SMALL SCRIPT CHARACTER-18B09" },
    { 0x18B0A, "KHITAN SMALL SCRIPT CHARACTER-18B0A" },
    { 0x18B0B, "KHITAN SMALL SCRIPT CHARACTER-18B0B" },
    { 0x18B0C, "KHITAN SMALL SCRIPT CHARACTER-18B0C" },
    { 0x18B0D, "KHITAN SMALL SCRIPT CHARACTER-18B0D" },
    { 0x18B0E, "KHITAN SMALL SCRIPT CHARACTER-18B0E" },
    { 0x18B0F, "KHITAN SMALL SCRIPT CHARACTER-18B0F" },
    { 0x18B10, "KHITAN SMALL SCRIPT CHARACTER-18B10" },
    { 0x18B11, "KHITAN SMALL SCRIPT CHARACTER-18B11" },
    { 0x18B12, "KHITAN SMALL SCRIPT CHARACTER-18B12" },
    { 0x18B13, "KHITAN SMALL SCRIPT CHARACTER-18B13" },
    { 0x18B14, "KHITAN SMALL SCRIPT CHARACTER-18B14" },
    { 0x18B15, "KHITAN SMALL SCRIPT CHARACTER-18B15" },
    { 0x18B16, "KHITAN SMALL SCRIPT CHARACTER-18B16" },
    { 0x18B17, "KHITAN SMALL SCRIPT CHARACTER-18B17" },
    { 0x18B18, "KHITAN SMALL SCRIPT CHARACTER-18B18" },
    { 0x18B19, "KHITAN SMALL SCRIPT CHARACTER-18B19" },
    { 0x18B1A, "KHITAN SMALL SCRIPT CHARACTER-18B1A" },
    { 0x18B1B, "KHITAN SMALL SCRIPT CHARACTER-18B1B" },
    { 0x18B1C, "KHITAN SMALL SCRIPT CHARACTER-18B1C" },
    { 0x18B1D, "KHITAN SMALL SCRIPT CHARACTER-18B1D" },
    { 0x18B1E, "KHITAN SMALL SCRIPT CHARACTER-18B1E" },
    { 0x18B1F, "KHITAN SMALL SCRIPT CHARACTER-18B1F" },
    { 0x18B20, "KHITAN SMALL SCRIPT CHARACTER-18B20" },
    { 0x18B21, "KHITAN SMALL SCRIPT CHARACTER-18B21" },
    { 0x18B22, "KHITAN SMALL SCRIPT CHARACTER-18B22" },
    { 0x18B23, "KHITAN SMALL SCRIPT CHARACTER-18B23" },
    { 0x18B24, "KHITAN SMALL SCRIPT CHARACTER-18B24" },
    { 0x18B25, "KHITAN SMALL SCRIPT CHARACTER-18B25" },
    { 0x18B26, "KHITAN SMALL SCRIPT CHARACTER-18B26" },
    { 0x18B27, "KHITAN SMALL SCRIPT CHARACTER-18B27" },
    { 0x18B28, "KHITAN SMALL SCRIPT CHARACTER-18B28" },
    { 0x18B29, "KHITAN SMALL SCRIPT CHARACTER-18B29" },
    { 0x18B2A, "KHITAN SMALL SCRIPT CHARACTER-18B2A" },
    { 0x18B2B, "KHITAN SMALL SCRIPT CHARACTER-18B2B" },
    { 0x18B2C, "KHITAN SMALL SCRIPT CHARACTER-18B2C" },
    { 0x18B2D, "KHITAN SMALL SCRIPT CHARACTER-18B2D" },
    { 0x18B2E, "KHITAN SMALL SCRIPT CHARACTER-18B2E" },
    { 0x18B2F, "KHITAN SMALL SCRIPT CHARACTER-18B2F" },
    { 0x18B30, "KHITAN SMALL SCRIPT CHARACTER-18B30" },
    { 0x18B31, "KHITAN SMALL SCRIPT CHARACTER-18B31" },
    { 0x18B32, "KHITAN SMALL SCRIPT CHARACTER-18B32" },
    { 0x18B33, "KHITAN SMALL SCRIPT CHARACTER-18B33" },
    { 0x18B34, "KHITAN SMALL SCRIPT CHARACTER-18B34" },
    { 0x18B35, "KHITAN SMALL SCRIPT CHARACTER-18B35" },
    { 0x18B36, "KHITAN SMALL SCRIPT CHARACTER-18B36" },
    { 0x18B37, "KHITAN SMALL SCRIPT CHARACTER-18B37" },
    { 0x18B38, "KHITAN SMALL SCRIPT CHARACTER-18B38" },
    { 0x18B39, "KHITAN SMALL SCRIPT CHARACTER-18B39" },
    { 0x18B3A, "KHITAN SMALL SCRIPT CHARACTER-18B3A" },
    { 0x18B3B, "KHITAN SMALL SCRIPT CHARACTER-18B3B" },
    { 0x18B3C, "KHITAN SMALL SCRIPT CHARACTER-18B3C" },
    { 0x18B3D, "KHITAN SMALL SCRIPT CHARACTER-18B3D" },
    { 0x18B3E, "KHITAN SMALL SCRIPT CHARACTER-18B3E" },
    { 0x18B3F, "KHITAN SMALL SCRIPT CHARACTER-18B3F" },
    { 0x18B40, "KHITAN SMALL SCRIPT CHARACTER-18B40" },
    { 0x18B41, "KHITAN SMALL SCRIPT CHARACTER-18B41" },
    { 0x18B42, "KHITAN SMALL SCRIPT CHARACTER-18B42" },
    { 0x18B43, "KHITAN SMALL SCRIPT CHARACTER-18B43" },
    { 0x18B44, "KHITAN SMALL SCRIPT CHARACTER-18B44" },
    { 0x18B45, "KHITAN SMALL SCRIPT CHARACTER-18B45" },
    { 0x18B46, "KHITAN SMALL SCRIPT CHARACTER-18B46" },
    { 0x18B47, "KHITAN SMALL SCRIPT CHARACTER-18B47" },
    { 0x18B48, "KHITAN SMALL SCRIPT CHARACTER-18B48" },
    { 0x18B49, "KHITAN SMALL SCRIPT CHARACTER-18B49" },
    { 0x18B4A, "KHITAN SMALL SCRIPT CHARACTER-18B4A" },
    { 0x18B4B, "KHITAN SMALL SCRIPT CHARACTER-18B4B" },
    { 0x18B4C, "KHITAN SMALL SCRIPT CHARACTER-18B4C" },
    { 0x18B4D, "KHITAN SMALL SCRIPT CHARACTER-18B4D" },
    { 0x18B4E, "KHITAN SMALL SCRIPT CHARACTER-18B4E" },
    { 0x18B4F, "KHITAN SMALL SCRIPT CHARACTER-18B4F" },
    { 0x18B50, "KHITAN SMALL SCRIPT CHARACTER-18B50" },
    { 0x18B51, "KHITAN SMALL SCRIPT CHARACTER-18B51" },
    { 0x18B52, "KHITAN SMALL SCRIPT CHARACTER-18B52" },
    { 0x18B53, "KHITAN SMALL SCRIPT CHARACTER-18B53" },
    { 0x18B54, "KHITAN SMALL SCRIPT CHARACTER-18B54" },
    { 0x18B55, "KHITAN SMALL SCRIPT CHARACTER-18B55" },
    { 0x18B56, "KHITAN SMALL SCRIPT CHARACTER-18B56" },
    { 0x18B57, "KHITAN SMALL SCRIPT CHARACTER-18B57" },
    { 0x18B58, "KHITAN SMALL SCRIPT CHARACTER-18B58" },
    { 0x18B59, "KHITAN SMALL SCRIPT CHARACTER-18B59" },
    { 0x18B5A, "KHITAN SMALL SCRIPT CHARACTER-18B5A" },
    { 0x18B5B, "KHITAN SMALL SCRIPT CHARACTER-18B5B" },
    { 0x18B5C, "KHITAN SMALL SCRIPT CHARACTER-18B5C" },
    { 0x18B5D, "KHITAN SMALL SCRIPT CHARACTER-18B5D" },
    { 0x18B5E, "KHITAN SMALL SCRIPT CHARACTER-18B5E" },
    { 0x18B5F, "KHITAN SMALL SCRIPT CHARACTER-18B5F" },
    { 0x18B60, "KHITAN SMALL SCRIPT CHARACTER-18B60" },
    { 0x18B61, "KHITAN SMALL SCRIPT CHARACTER-18B61" },
    { 0x18B62, "KHITAN SMALL SCRIPT CHARACTER-18B62" },
    { 0x18B63, "KHITAN SMALL SCRIPT CHARACTER-18B63" },
    { 0x18B64, "KHITAN SMALL SCRIPT CHARACTER-18B64" },
    { 0x18B65, "KHITAN SMALL SCRIPT CHARACTER-18B65" },
    { 0x18B66, "KHITAN SMALL SCRIPT CHARACTER-18B66" },
    { 0x18B67, "KHITAN SMALL SCRIPT CHARACTER-18B67" },
    { 0x18B68, "KHITAN SMALL SCRIPT CHARACTER-18B68" },
    { 0x18B69, "KHITAN SMALL SCRIPT CHARACTER-18B69" },
    { 0x18B6A, "KHITAN SMALL SCRIPT CHARACTER-18B6A" },
    { 0x18B6B, "KHITAN SMALL SCRIPT CHARACTER-18B6B" },
    { 0x18B6C, "KHITAN SMALL SCRIPT CHARACTER-18B6C" },
    { 0x18B6D, "KHITAN SMALL SCRIPT CHARACTER-18B6D" },
    { 0x18B6E, "KHITAN SMALL SCRIPT CHARACTER-18B6E" },
    { 0x18B6F, "KHITAN SMALL SCRIPT CHARACTER-18B6F" },
    { 0x18B70, "KHITAN SMALL SCRIPT CHARACTER-18B70" },
    { 0x18B71, "KHITAN SMALL SCRIPT CHARACTER-18B71" },
    { 0x18B72, "KHITAN SMALL SCRIPT CHARACTER-18B72" },
    { 0x18B73, "KHITAN SMALL SCRIPT CHARACTER-18B73" },
    { 0x18B74, "KHITAN SMALL SCRIPT CHARACTER-18B74" },
    { 0x18B75, "KHITAN SMALL SCRIPT CHARACTER-18B75" },
    { 0x18B76, "KHITAN SMALL SCRIPT CHARACTER-18B76" },
    { 0x18B77, "KHITAN SMALL SCRIPT CHARACTER-18B77" },
    { 0x18B78, "KHITAN SMALL SCRIPT CHARACTER-18B78" },
    { 0x18B79, "KHITAN SMALL SCRIPT CHARACTER-18B79" },
    { 0x18B7A, "KHITAN SMALL SCRIPT CHARACTER-18B7A" },
    { 0x18B7B, "KHITAN SMALL SCRIPT CHARACTER-18B7B" },
    { 0x18B7C, "KHITAN SMALL SCRIPT CHARACTER-18B7C" },
    { 0x18B7D, "KHITAN SMALL SCRIPT CHARACTER-18B7D" },
    { 0x18B7E, "KHITAN SMALL SCRIPT CHARACTER-18B7E" },
    { 0x18B7F, "KHITAN SMALL SCRIPT CHARACTER-18B7F" },
    { 0x18B80, "KHITAN SMALL SCRIPT CHARACTER-18B80" },
    { 0x18B81, "KHITAN SMALL SCRIPT CHARACTER-18B81" },
    { 0x18B82, "KHITAN SMALL SCRIPT CHARACTER-18B82" },
    { 0x18B83, "KHITAN SMALL SCRIPT CHARACTER-18B83" },
    { 0x18B84, "KHITAN SMALL SCRIPT CHARACTER-18B84" },
    { 0x18B85, "KHITAN SMALL SCRIPT CHARACTER-18B85" },
    { 0x18B86, "KHITAN SMALL SCRIPT CHARACTER-18B86" },
    { 0x18B87, "KHITAN SMALL SCRIPT CHARACTER-18B87" },
    { 0x18B88, "KHITAN SMALL SCRIPT CHARACTER-18B88" },
    { 0x18B89, "KHITAN SMALL SCRIPT CHARACTER-18B89" },
    { 0x18B8A, "KHITAN SMALL SCRIPT CHARACTER-18B8A" },
    { 0x18B8B, "KHITAN SMALL SCRIPT CHARACTER-18B8B" },
    { 0x18B8C, "KHITAN SMALL SCRIPT CHARACTER-18B8C" },
    { 0x18B8D, "KHITAN SMALL SCRIPT CHARACTER-18B8D" },
    { 0x18B8E, "KHITAN SMALL SCRIPT CHARACTER-18B8E" },
    { 0x18B8F, "KHITAN SMALL SCRIPT CHARACTER-18B8F" },
    { 0x18B90, "KHITAN SMALL SCRIPT CHARACTER-18B90" },
    { 0x18B91, "KHITAN SMALL SCRIPT CHARACTER-18B91" },
    { 0x18B92, "KHITAN SMALL SCRIPT CHARACTER-18B92" },
    { 0x18B93, "KHITAN SMALL SCRIPT CHARACTER-18B93" },
    { 0x18B94, "KHITAN SMALL SCRIPT CHARACTER-18B94" },
    { 0x18B95, "KHITAN SMALL SCRIPT CHARACTER-18B95" },
    { 0x18B96, "KHITAN SMALL SCRIPT CHARACTER-18B96" },
    { 0x18B97, "KHITAN SMALL SCRIPT CHARACTER-18B97" },
    { 0x18B98, "KHITAN SMALL SCRIPT CHARACTER-18B98" },
    { 0x18B99, "KHITAN SMALL SCRIPT CHARACTER-18B99" },
    { 0x18B9A, "KHITAN SMALL SCRIPT CHARACTER-18B9A" },
    { 0x18B9B, "KHITAN SMALL SCRIPT CHARACTER-18B9B" },
    { 0x18B9C, "KHITAN SMALL SCRIPT CHARACTER-18B9C" },
    { 0x18B9D, "KHITAN SMALL SCRIPT CHARACTER-18B9D" },
    { 0x18B9E, "KHITAN SMALL SCRIPT CHARACTER-18B9E" },
    { 0x18B9F, "KHITAN SMALL SCRIPT CHARACTER-18B9F" },
    { 0x18BA0, "KHITAN SMALL SCRIPT CHARACTER-18BA0" },
    { 0x18BA1, "KHITAN SMALL SCRIPT CHARACTER-18BA1" },
    { 0x18BA2, "KHITAN SMALL SCRIPT CHARACTER-18BA2" },
    { 0x18BA3, "KHITAN SMALL SCRIPT CHARACTER-18BA3" },
    { 0x18BA4, "KHITAN SMALL SCRIPT CHARACTER-18BA4" },
    { 0x18BA5, "KHITAN SMALL SCRIPT CHARACTER-18BA5" },
    { 0x18BA6, "KHITAN SMALL SCRIPT CHARACTER-18BA6" },
    { 0x18BA7, "KHITAN SMALL SCRIPT CHARACTER-18BA7" },
    { 0x18BA8, "KHITAN SMALL SCRIPT CHARACTER-18BA8" },
    { 0x18BA9, "KHITAN SMALL SCRIPT CHARACTER-18BA9" },
    { 0x18BAA, "KHITAN SMALL SCRIPT CHARACTER-18BAA" },
    { 0x18BAB, "KHITAN SMALL SCRIPT CHARACTER-18BAB" },
    { 0x18BAC, "KHITAN SMALL SCRIPT CHARACTER-18BAC" },
    { 0x18BAD, "KHITAN SMALL SCRIPT CHARACTER-18BAD" },
    { 0x18BAE, "KHITAN SMALL SCRIPT CHARACTER-18BAE" },
    { 0x18BAF, "KHITAN SMALL SCRIPT CHARACTER-18BAF" },
    { 0x18BB0, "KHITAN SMALL SCRIPT CHARACTER-18BB0" },
    { 0x18BB1, "KHITAN SMALL SCRIPT CHARACTER-18BB1" },
    { 0x18BB2, "KHITAN SMALL SCRIPT CHARACTER-18BB2" },
    { 0x18BB3, "KHITAN SMALL SCRIPT CHARACTER-18BB3" },
    { 0x18BB4, "KHITAN SMALL SCRIPT CHARACTER-18BB4" },
    { 0x18BB5, "KHITAN SMALL SCRIPT CHARACTER-18BB5" },
    { 0x18BB6, "KHITAN SMALL SCRIPT CHARACTER-18BB6" },
    { 0x18BB7, "KHITAN SMALL SCRIPT CHARACTER-18BB7" },
    { 0x18BB8, "KHITAN SMALL SCRIPT CHARACTER-18BB8" },
    { 0x18BB9, "KHITAN SMALL SCRIPT CHARACTER-18BB9" },
    { 0x18BBA, "KHITAN SMALL SCRIPT CHARACTER-18BBA" },
    { 0x18BBB, "KHITAN SMALL SCRIPT CHARACTER-18BBB" },
    { 0x18BBC, "KHITAN SMALL SCRIPT CHARACTER-18BBC" },
    { 0x18BBD, "KHITAN SMALL SCRIPT CHARACTER-18BBD" },
    { 0x18BBE, "KHITAN SMALL SCRIPT CHARACTER-18BBE" },
    { 0x18BBF, "KHITAN SMALL SCRIPT CHARACTER-18BBF" },
    { 0x18BC0, "KHITAN SMALL SCRIPT CHARACTER-18BC0" },
    { 0x18BC1, "KHITAN SMALL SCRIPT CHARACTER-18BC1" },
    { 0x18BC2, "KHITAN SMALL SCRIPT CHARACTER-18BC2" },
    { 0x18BC3, "KHITAN SMALL SCRIPT CHARACTER-18BC3" },
    { 0x18BC4, "KHITAN SMALL SCRIPT CHARACTER-18BC4" },
    { 0x18BC5, "KHITAN SMALL SCRIPT CHARACTER-18BC5" },
    { 0x18BC6, "KHITAN SMALL SCRIPT CHARACTER-18BC6" },
    { 0x18BC7, "KHITAN SMALL SCRIPT CHARACTER-18BC7" },
    { 0x18BC8, "KHITAN SMALL SCRIPT CHARACTER-18BC8" },
    { 0x18BC9, "KHITAN SMALL SCRIPT CHARACTER-18BC9" },
    { 0x18BCA, "KHITAN SMALL SCRIPT CHARACTER-18BCA" },
    { 0x18BCB, "KHITAN SMALL SCRIPT CHARACTER-18BCB" },
    { 0x18BCC, "KHITAN SMALL SCRIPT CHARACTER-18BCC" },
    { 0x18BCD, "KHITAN SMALL SCRIPT CHARACTER-18BCD" },
    { 0x18BCE, "KHITAN SMALL SCRIPT CHARACTER-18BCE" },
    { 0x18BCF, "KHITAN SMALL SCRIPT CHARACTER-18BCF" },
    { 0x18BD0, "KHITAN SMALL SCRIPT CHARACTER-18BD0" },
    { 0x18BD1, "KHITAN SMALL SCRIPT CHARACTER-18BD1" },
    { 0x18BD2, "KHITAN SMALL SCRIPT CHARACTER-18BD2" },
    { 0x18BD3, "KHITAN SMALL SCRIPT CHARACTER-18BD3" },
    { 0x18BD4, "KHITAN SMALL SCRIPT CHARACTER-18BD4" },
    { 0x18BD5, "KHITAN SMALL SCRIPT CHARACTER-18BD5" },
    { 0x18BD6, "KHITAN SMALL SCRIPT CHARACTER-18BD6" },
    { 0x18BD7, "KHITAN SMALL SCRIPT CHARACTER-18BD7" },
    { 0x18BD8, "KHITAN SMALL SCRIPT CHARACTER-18BD8" },
    { 0x18BD9, "KHITAN SMALL SCRIPT CHARACTER-18BD9" },
    { 0x18BDA, "KHITAN SMALL SCRIPT CHARACTER-18BDA" },
    { 0x18BDB, "KHITAN SMALL SCRIPT CHARACTER-18BDB" },
    { 0x18BDC, "KHITAN SMALL SCRIPT CHARACTER-18BDC" },
    { 0x18BDD, "KHITAN SMALL SCRIPT CHARACTER-18BDD" },
    { 0x18BDE, "KHITAN SMALL SCRIPT CHARACTER-18BDE" },
    { 0x18BDF, "KHITAN SMALL SCRIPT CHARACTER-18BDF" },
    { 0x18BE0, "KHITAN SMALL SCRIPT CHARACTER-18BE0" },
    { 0x18BE1, "KHITAN SMALL SCRIPT CHARACTER-18BE1" },
    { 0x18BE2, "KHITAN SMALL SCRIPT CHARACTER-18BE2" },
    { 0x18BE3, "KHITAN SMALL SCRIPT CHARACTER-18BE3" },
    { 0x18BE4, "KHITAN SMALL SCRIPT CHARACTER-18BE4" },
    { 0x18BE5, "KHITAN SMALL SCRIPT CHARACTER-18BE5" },
    { 0x18BE6, "KHITAN SMALL SCRIPT CHARACTER-18BE6" },
    { 0x18BE7, "KHITAN SMALL SCRIPT CHARACTER-18BE7" },
    { 0x18BE8, "KHITAN SMALL SCRIPT CHARACTER-18BE8" },
    { 0x18BE9, "KHITAN SMALL SCRIPT CHARACTER-18BE9" },
    { 0x18BEA, "KHITAN SMALL SCRIPT CHARACTER-18BEA" },
    { 0x18BEB, "KHITAN SMALL SCRIPT CHARACTER-18BEB" },
    { 0x18BEC, "KHITAN SMALL SCRIPT CHARACTER-18BEC" },
    { 0x18BED, "KHITAN SMALL SCRIPT CHARACTER-18BED" },
    { 0x18BEE, "KHITAN SMALL SCRIPT CHARACTER-18BEE" },
    { 0x18BEF, "KHITAN SMALL SCRIPT CHARACTER-18BEF" },
    { 0x18BF0, "KHITAN SMALL SCRIPT CHARACTER-18BF0" },
    { 0x18BF1, "KHITAN SMALL SCRIPT CHARACTER-18BF1" },
    { 0x18BF2, "KHITAN SMALL SCRIPT CHARACTER-18BF2" },
    { 0x18BF3, "KHITAN SMALL SCRIPT CHARACTER-18BF3" },
    { 0x18BF4, "KHITAN SMALL SCRIPT CHARACTER-18BF4" },
    { 0x18BF5, "KHITAN SMALL SCRIPT CHARACTER-18BF5" },
    { 0x18BF6, "KHITAN SMALL SCRIPT CHARACTER-18BF6" },
    { 0x18BF7, "KHITAN SMALL SCRIPT CHARACTER-18BF7" },
    { 0x18BF8, "KHITAN SMALL SCRIPT CHARACTER-18BF8" },
    { 0x18BF9, "KHITAN SMALL SCRIPT CHARACTER-18BF9" },
    { 0x18BFA, "KHITAN SMALL SCRIPT CHARACTER-18BFA" },
    { 0x18BFB, "KHITAN SMALL SCRIPT CHARACTER-18BFB" },
    { 0x18BFC, "KHITAN SMALL SCRIPT CHARACTER-18BFC" },
    { 0x18BFD, "KHITAN SMALL SCRIPT CHARACTER-18BFD" },
    { 0x18BFE, "KHITAN SMALL SCRIPT CHARACTER-18BFE" },
    { 0x18BFF, "KHITAN SMALL SCRIPT CHARACTER-18BFF" },
    { 0x18C00, "KHITAN SMALL SCRIPT CHARACTER-18C00" },
    { 0x18C01, "KHITAN SMALL SCRIPT CHARACTER-18C01" },
    { 0x18C02, "KHITAN SMALL SCRIPT CHARACTER-18C02" },
    { 0x18C03, "KHITAN SMALL SCRIPT CHARACTER-18C03" },
    { 0x18C04, "KHITAN SMALL SCRIPT CHARACTER-18C04" },
    { 0x18C05, "KHITAN SMALL SCRIPT CHARACTER-18C05" },
    { 0x18C06, "KHITAN SMALL SCRIPT CHARACTER-18C06" },
    { 0x18C07, "KHITAN SMALL SCRIPT CHARACTER-18C07" },
    { 0x18C08, "KHITAN SMALL SCRIPT CHARACTER-18C08" },
    { 0x18C09, "KHITAN SMALL SCRIPT CHARACTER-18C09" },
    { 0x18C0A, "KHITAN SMALL SCRIPT CHARACTER-18C0A" },
    { 0x18C0B, "KHITAN SMALL SCRIPT CHARACTER-18C0B" },
    { 0x18C0C, "KHITAN SMALL SCRIPT CHARACTER-18C0C" },
    { 0x18C0D, "KHITAN SMALL SCRIPT CHARACTER-18C0D" },
    { 0x18C0E, "KHITAN SMALL SCRIPT CHARACTER-18C0E" },
    { 0x18C0F, "KHITAN SMALL SCRIPT CHARACTER-18C0F" },
    { 0x18C10, "KHITAN SMALL SCRIPT CHARACTER-18C10" },
    { 0x18C11, "KHITAN SMALL SCRIPT CHARACTER-18C11" },
    { 0x18C12, "KHITAN SMALL SCRIPT CHARACTER-18C12" },
    { 0x18C13, "KHITAN SMALL SCRIPT CHARACTER-18C13" },
    { 0x18C14, "KHITAN SMALL SCRIPT CHARACTER-18C14" },
    { 0x18C15, "KHITAN SMALL SCRIPT CHARACTER-18C15" },
    { 0x18C16, "KHITAN SMALL SCRIPT CHARACTER-18C16" },
    { 0x18C17, "KHITAN SMALL SCRIPT CHARACTER-18C17" },
    { 0x18C18, "KHITAN SMALL SCRIPT CHARACTER-18C18" },
    { 0x18C19, "KHITAN SMALL SCRIPT CHARACTER-18C19" },
    { 0x18C1A, "KHITAN SMALL SCRIPT CHARACTER-18C1A" },
    { 0x18C1B, "KHITAN SMALL SCRIPT CHARACTER-18C1B" },
    { 0x18C1C, "KHITAN SMALL SCRIPT CHARACTER-18C1C" },
    { 0x18C1D, "KHITAN SMALL SCRIPT CHARACTER-18C1D" },
    { 0x18C1E, "KHITAN SMALL SCRIPT CHARACTER-18C1E" },
    { 0x18C1F, "KHITAN SMALL SCRIPT CHARACTER-18C1F" },
    { 0x18C20, "KHITAN SMALL SCRIPT CHARACTER-18C20" },
    { 0x18C21, "KHITAN SMALL SCRIPT CHARACTER-18C21" },
    { 0x18C22, "KHITAN SMALL SCRIPT CHARACTER-18C22" },
    { 0x18C23, "KHITAN SMALL SCRIPT CHARACTER-18C23" },
    { 0x18C24, "KHITAN SMALL SCRIPT CHARACTER-18C24" },
    { 0x18C25, "KHITAN SMALL SCRIPT CHARACTER-18C25" },
    { 0x18C26, "KHITAN SMALL SCRIPT CHARACTER-18C26" },
    { 0x18C27, "KHITAN SMALL SCRIPT CHARACTER-18C27" },
    { 0x18C28, "KHITAN SMALL SCRIPT CHARACTER-18C28" },
    { 0x18C29, "KHITAN SMALL SCRIPT CHARACTER-18C29" },
    { 0x18C2A, "KHITAN SMALL SCRIPT CHARACTER-18C2A" },
    { 0x18C2B, "KHITAN SMALL SCRIPT CHARACTER-18C2B" },
    { 0x18C2C, "KHITAN SMALL SCRIPT CHARACTER-18C2C" },
    { 0x18C2D, "KHITAN SMALL SCRIPT CHARACTER-18C2D" },
    { 0x18C2E, "KHITAN SMALL SCRIPT CHARACTER-18C2E" },
    { 0x18C2F, "KHITAN SMALL SCRIPT CHARACTER-18C2F" },
    { 0x18C30, "KHITAN SMALL SCRIPT CHARACTER-18C30" },
    { 0x18C31, "KHITAN SMALL SCRIPT CHARACTER-18C31" },
    { 0x18C32, "KHITAN SMALL SCRIPT CHARACTER-18C32" },
    { 0x18C33, "KHITAN SMALL SCRIPT CHARACTER-18C33" },
    { 0x18C34, "KHITAN SMALL SCRIPT CHARACTER-18C34" },
    { 0x18C35, "KHITAN SMALL SCRIPT CHARACTER-18C35" },
    { 0x18C36, "KHITAN SMALL SCRIPT CHARACTER-18C36" },
    { 0x18C37, "KHITAN SMALL SCRIPT CHARACTER-18C37" },
    { 0x18C38, "KHITAN SMALL SCRIPT CHARACTER-18C38" },
    { 0x18C39, "KHITAN SMALL SCRIPT CHARACTER-18C39" },
    { 0x18C3A, "KHITAN SMALL SCRIPT CHARACTER-18C3A" },
    { 0x18C3B, "KHITAN SMALL SCRIPT CHARACTER-18C3B" },
    { 0x18C3C, "KHITAN SMALL SCRIPT CHARACTER-18C3C" },
    { 0x18C3D, "KHITAN SMALL SCRIPT CHARACTER-18C3D" },
    { 0x18C3E, "KHITAN SMALL SCRIPT CHARACTER-18C3E" },
    { 0x18C3F, "KHITAN SMALL SCRIPT CHARACTER-18C3F" },
    { 0x18C40, "KHITAN SMALL SCRIPT CHARACTER-18C40" },
    { 0x18C41, "KHITAN SMALL SCRIPT CHARACTER-18C41" },
    { 0x18C42, "KHITAN SMALL SCRIPT CHARACTER-18C42" },
    { 0x18C43, "KHITAN SMALL SCRIPT CHARACTER-18C43" },
    { 0x18C44, "KHITAN SMALL SCRIPT CHARACTER-18C44" },
    { 0x18C45, "KHITAN SMALL SCRIPT CHARACTER-18C45" },
    { 0x18C46, "KHITAN SMALL SCRIPT CHARACTER-18C46" },
    { 0x18C47, "KHITAN SMALL SCRIPT CHARACTER-18C47" },
    { 0x18C48, "KHITAN SMALL SCRIPT CHARACTER-18C48" },
    { 0x18C49, "KHITAN SMALL SCRIPT CHARACTER-18C49" },
    { 0x18C4A, "KHITAN SMALL SCRIPT CHARACTER-18C4A" },
    { 0x18C4B, "KHITAN SMALL SCRIPT CHARACTER-18C4B" },
    { 0x18C4C, "KHITAN SMALL SCRIPT CHARACTER-18C4C" },
    { 0x18C4D, "KHITAN SMALL SCRIPT CHARACTER-18C4D" },
    { 0x18C4E, "KHITAN SMALL SCRIPT CHARACTER-18C4E" },
    { 0x18C4F, "KHITAN SMALL SCRIPT CHARACTER-18C4F" },
    { 0x18C50, "KHITAN SMALL SCRIPT CHARACTER-18C50" },
    { 0x18C51, "KHITAN SMALL SCRIPT CHARACTER-18C51" },
    { 0x18C52, "KHITAN SMALL SCRIPT CHARACTER-18C52" },
    { 0x18C53, "KHITAN SMALL SCRIPT CHARACTER-18C53" },
    { 0x18C54, "KHITAN SMALL SCRIPT CHARACTER-18C54" },
    { 0x18C55, "KHITAN SMALL SCRIPT CHARACTER-18C55" },
    { 0x18C56, "KHITAN SMALL SCRIPT CHARACTER-18C56" },
    { 0x18C57, "KHITAN SMALL SCRIPT CHARACTER-18C57" },
    { 0x18C58, "KHITAN SMALL SCRIPT CHARACTER-18C58" },
    { 0x18C59, "KHITAN SMALL SCRIPT CHARACTER-18C59" },
    { 0x18C5A, "KHITAN SMALL SCRIPT CHARACTER-18C5A" },
    { 0x18C5B, "KHITAN SMALL SCRIPT CHARACTER-18C5B" },
    { 0x18C5C, "KHITAN SMALL SCRIPT CHARACTER-18C5C" },
    { 0x18C5D, "KHITAN SMALL SCRIPT CHARACTER-18C5D" },
    { 0x18C5E, "KHITAN SMALL SCRIPT CHARACTER-18C5E" },
    { 0x18C5F, "KHITAN SMALL SCRIPT CHARACTER-18C5F" },
    { 0x18C60, "KHITAN SMALL SCRIPT CHARACTER-18C60" },
    { 0x18C61, "KHITAN SMALL SCRIPT CHARACTER-18C61" },
    { 0x18C62, "KHITAN SMALL SCRIPT CHARACTER-18C62" },
    { 0x18C63, "KHITAN SMALL SCRIPT CHARACTER-18C63" },
    { 0x18C64, "KHITAN SMALL SCRIPT CHARACTER-18C64" },
    { 0x18C65, "KHITAN SMALL SCRIPT CHARACTER-18C65" },
    { 0x18C66, "KHITAN SMALL SCRIPT CHARACTER-18C66" },
    { 0x18C67, "KHITAN SMALL SCRIPT CHARACTER-18C67" },
    { 0x18C68, "KHITAN SMALL SCRIPT CHARACTER-18C68" },
    { 0x18C69, "KHITAN SMALL SCRIPT CHARACTER-18C69" },
    { 0x18C6A, "KHITAN SMALL SCRIPT CHARACTER-18C6A" },
    { 0x18C6B, "KHITAN SMALL SCRIPT CHARACTER-18C6B" },
    { 0x18C6C, "KHITAN SMALL SCRIPT CHARACTER-18C6C" },
    { 0x18C6D, "KHITAN SMALL SCRIPT CHARACTER-18C6D" },
    { 0x18C6E, "KHITAN SMALL SCRIPT CHARACTER-18C6E" },
    { 0x18C6F, "KHITAN SMALL SCRIPT CHARACTER-18C6F" },
    { 0x18C70, "KHITAN SMALL SCRIPT CHARACTER-18C70" },
    { 0x18C71, "KHITAN SMALL SCRIPT CHARACTER-18C71" },
    { 0x18C72, "KHITAN SMALL SCRIPT CHARACTER-18C72" },
    { 0x18C73, "KHITAN SMALL SCRIPT CHARACTER-18C73" },
    { 0x18C74, "KHITAN SMALL SCRIPT CHARACTER-18C74" },
    { 0x18C75, "KHITAN SMALL SCRIPT CHARACTER-18C75" },
    { 0x18C76, "KHITAN SMALL SCRIPT CHARACTER-18C76" },
    { 0x18C77, "KHITAN SMALL SCRIPT CHARACTER-18C77" },
    { 0x18C78, "KHITAN SMALL SCRIPT CHARACTER-18C78" },
    { 0x18C79, "KHITAN SMALL SCRIPT CHARACTER-18C79" },
    { 0x18C7A, "KHITAN SMALL SCRIPT CHARACTER-18C7A" },
    { 0x18C7B, "KHITAN SMALL SCRIPT CHARACTER-18C7B" },
    { 0x18C7C, "KHITAN SMALL SCRIPT CHARACTER-18C7C" },
    { 0x18C7D, "KHITAN SMALL SCRIPT CHARACTER-18C7D" },
    { 0x18C7E, "KHITAN SMALL SCRIPT CHARACTER-18C7E" },
    { 0x18C7F, "KHITAN SMALL SCRIPT CHARACTER-18C7F" },
    { 0x18C80, "KHITAN SMALL SCRIPT CHARACTER-18C80" },
    { 0x18C81, "KHITAN SMALL SCRIPT CHARACTER-18C81" },
    { 0x18C82, "KHITAN SMALL SCRIPT CHARACTER-18C82" },
    { 0x18C83, "KHITAN SMALL SCRIPT CHARACTER-18C83" },
    { 0x18C84, "KHITAN SMALL SCRIPT CHARACTER-18C84" },
    { 0x18C85, "KHITAN SMALL SCRIPT CHARACTER-18C85" },
    { 0x18C86, "KHITAN SMALL SCRIPT CHARACTER-18C86" },
    { 0x18C87, "KHITAN SMALL SCRIPT CHARACTER-18C87" },
    { 0x18C88, "KHITAN SMALL SCRIPT CHARACTER-18C88" },
    { 0x18C89, "KHITAN SMALL SCRIPT CHARACTER-18C89" },
    { 0x18C8A, "KHITAN SMALL SCRIPT CHARACTER-18C8A" },
    { 0x18C8B, "KHITAN SMALL SCRIPT CHARACTER-18C8B" },
    { 0x18C8C, "KHITAN SMALL SCRIPT CHARACTER-18C8C" },
    { 0x18C8D, "KHITAN SMALL SCRIPT CHARACTER-18C8D" },
    { 0x18C8E, "KHITAN SMALL SCRIPT CHARACTER-18C8E" },
    { 0x18C8F, "KHITAN SMALL SCRIPT CHARACTER-18C8F" },
    { 0x18C90, "KHITAN SMALL SCRIPT CHARACTER-18C90" },
    { 0x18C91, "KHITAN SMALL SCRIPT CHARACTER-18C91" },
    { 0x18C92, "KHITAN SMALL SCRIPT CHARACTER-18C92" },
    { 0x18C93, "KHITAN SMALL SCRIPT CHARACTER-18C93" },
    { 0x18C94, "KHITAN SMALL SCRIPT CHARACTER-18C94" },
    { 0x18C95, "KHITAN SMALL SCRIPT CHARACTER-18C95" },
    { 0x18C96, "KHITAN SMALL SCRIPT CHARACTER-18C96" },
    { 0x18C97, "KHITAN SMALL SCRIPT CHARACTER-18C97" },
    { 0x18C98, "KHITAN SMALL SCRIPT CHARACTER-18C98" },
    { 0x18C99, "KHITAN SMALL SCRIPT CHARACTER-18C99" },
    { 0x18C9A, "KHITAN SMALL SCRIPT CHARACTER-18C9A" },
    { 0x18C9B, "KHITAN SMALL SCRIPT CHARACTER-18C9B" },
    { 0x18C9C, "KHITAN SMALL SCRIPT CHARACTER-18C9C" },
    { 0x18C9D, "KHITAN SMALL SCRIPT CHARACTER-18C9D" },
    { 0x18C9E, "KHITAN SMALL SCRIPT CHARACTER-18C9E" },
    { 0x18C9F, "KHITAN SMALL SCRIPT CHARACTER-18C9F" },
    { 0x18CA0, "KHITAN SMALL SCRIPT CHARACTER-18CA0" },
    { 0x18CA1, "KHITAN SMALL SCRIPT CHARACTER-18CA1" },
    { 0x18CA2, "KHITAN SMALL SCRIPT CHARACTER-18CA2" },
    { 0x18CA3, "KHITAN SMALL SCRIPT CHARACTER-18CA3" },
    { 0x18CA4, "KHITAN SMALL SCRIPT CHARACTER-18CA4" },
    { 0x18CA5, "KHITAN SMALL SCRIPT CHARACTER-18CA5" },
    { 0x18CA6, "KHITAN SMALL SCRIPT CHARACTER-18CA6" },
    { 0x18CA7, "KHITAN SMALL SCRIPT CHARACTER-18CA7" },
    { 0x18CA8, "KHITAN SMALL SCRIPT CHARACTER-18CA8" },
    { 0x18CA9, "KHITAN SMALL SCRIPT CHARACTER-18CA9" },
    { 0x18CAA, "KHITAN SMALL SCRIPT CHARACTER-18CAA" },
    { 0x18CAB, "KHITAN SMALL SCRIPT CHARACTER-18CAB" },
    { 0x18CAC, "KHITAN SMALL SCRIPT CHARACTER-18CAC" },
    { 0x18CAD, "KHITAN SMALL SCRIPT CHARACTER-18CAD" },
    { 0x18CAE, "KHITAN SMALL SCRIPT CHARACTER-18CAE" },
    { 0x18CAF, "KHITAN SMALL SCRIPT CHARACTER-18CAF" },
    { 0x18CB0, "KHITAN SMALL SCRIPT CHARACTER-18CB0" },
    { 0x18CB1, "KHITAN SMALL SCRIPT CHARACTER-18CB1" },
    { 0x18CB2, "KHITAN SMALL SCRIPT CHARACTER-18CB2" },
    { 0x18CB3, "KHITAN SMALL SCRIPT CHARACTER-18CB3" },
    { 0x18CB4, "KHITAN SMALL SCRIPT CHARACTER-18CB4" },
    { 0x18CB5, "KHITAN SMALL SCRIPT CHARACTER-18CB5" },
    { 0x18CB6, "KHITAN SMALL SCRIPT CHARACTER-18CB6" },
    { 0x18CB7, "KHITAN SMALL SCRIPT CHARACTER-18CB7" },
    { 0x18CB8, "KHITAN SMALL SCRIPT CHARACTER-18CB8" },
    { 0x18CB9, "KHITAN SMALL SCRIPT CHARACTER-18CB9" },
    { 0x18CBA, "KHITAN SMALL SCRIPT CHARACTER-18CBA" },
    { 0x18CBB, "KHITAN SMALL SCRIPT CHARACTER-18CBB" },
    { 0x18CBC, "KHITAN SMALL SCRIPT CHARACTER-18CBC" },
    { 0x18CBD, "KHITAN SMALL SCRIPT CHARACTER-18CBD" },
    { 0x18CBE, "KHITAN SMALL SCRIPT CHARACTER-18CBE" },
    { 0x18CBF, "KHITAN SMALL SCRIPT CHARACTER-18CBF" },
    { 0x18CC0, "KHITAN SMALL SCRIPT CHARACTER-18CC0" },
    { 0x18CC1, "KHITAN SMALL SCRIPT CHARACTER-18CC1" },
    { 0x18CC2, "KHITAN SMALL SCRIPT CHARACTER-18CC2" },
    { 0x18CC3, "KHITAN SMALL SCRIPT CHARACTER-18CC3" },
    { 0x18CC4, "KHITAN SMALL SCRIPT CHARACTER-18CC4" },
    { 0x18CC5, "KHITAN SMALL SCRIPT CHARACTER-18CC5" },
    { 0x18CC6, "KHITAN SMALL SCRIPT CHARACTER-18CC6" },
    { 0x18CC7, "KHITAN SMALL SCRIPT CHARACTER-18CC7" },
    { 0x18CC8, "KHITAN SMALL SCRIPT CHARACTER-18CC8" },
    { 0x18CC9, "KHITAN SMALL SCRIPT CHARACTER-18CC9" },
    { 0x18CCA, "KHITAN SMALL SCRIPT CHARACTER-18CCA" },
    { 0x18CCB, "KHITAN SMALL SCRIPT CHARACTER-18CCB" },
    { 0x18CCC, "KHITAN SMALL SCRIPT CHARACTER-18CCC" },
    { 0x18CCD, "KHITAN SMALL SCRIPT CHARACTER-18CCD" },
    { 0x18CCE, "KHITAN SMALL SCRIPT CHARACTER-18CCE" },
    { 0x18CCF, "KHITAN SMALL SCRIPT CHARACTER-18CCF" },
    { 0x18CD0, "KHITAN SMALL SCRIPT CHARACTER-18CD0" },
    { 0x18CD1, "KHITAN SMALL SCRIPT CHARACTER-18CD1" },
    { 0x18CD2, "KHITAN SMALL SCRIPT CHARACTER-18CD2" },
    { 0x18CD3, "KHITAN SMALL SCRIPT CHARACTER-18CD3" },
    { 0x18CD4, "KHITAN SMALL SCRIPT CHARACTER-18CD4" },
    { 0x18CD5, "KHITAN SMALL SCRIPT CHARACTER-18CD5" },
    { 0x1AFF0, "KATAKANA LETTER MINNAN TONE-2" },
    { 0x1AFF1, "KATAKANA LETTER MINNAN TONE-3" },
    { 0x1AFF2, "KATAKANA LETTER MINNAN TONE-4" },
    { 0x1AFF3, "KATAKANA LETTER MINNAN TONE-5" },
    { 0x1AFF5, "KATAKANA LETTER MINNAN TONE-7" },
    { 0x1AFF6, "KATAKANA LETTER MINNAN TONE-8" },
    { 0x1AFF7, "KATAKANA LETTER MINNAN NASALIZED TONE-1" },
    { 0x1AFF8, "KATAKANA LETTER MINNAN NASALIZED TONE-2" },
    { 0x1AFF9, "KATAKANA LETTER MINNAN NASALIZED TONE-3" },
    { 0x1AFFA, "KATAKANA LETTER MINNAN NASALIZED TONE-4" },
    { 0x1AFFB, "KATAKANA LETTER MINNAN NASALIZED TONE-5" },
    { 0x1AFFD, "KATAKANA LETTER MINNAN NASALIZED TONE-7" },
    { 0x1AFFE, "KATAKANA LETTER MINNAN NASALIZED TONE-8" },
    { 0x1B000, "KATAKANA LETTER ARCHAIC E" },
    { 0x1B001, "HIRAGANA LETTER ARCHAIC YE" },
    { 0x1B002, "HENTAIGANA LETTER A-1" },
    { 0x1B003, "HENTAIGANA LETTER A-2" },
    { 0x1B004, "HENTAIGANA LETTER A-3" },
    { 0x1B005, "HENTAIGANA LETTER A-WO" },
    { 0x1B006, "HENTAIGANA LETTER I-1" },
    { 0x1B007, "HENTAIGANA LETTER I-2" },
    { 0x1B008, "HENTAIGANA LETTER I-3" },
    { 0x1B009, "HENTAIGANA LETTER I-4" },
    { 0x1B00A, "HENTAIGANA LETTER U-1" },
    { 0x1B00B, "HENTAIGANA LETTER U-2" },
    { 0x1B00C, "HENTAIGANA LETTER U-3" },
    { 0x1B00D, "HENTAIGANA LETTER U-4" },
    { 0x1B00E, "HENTAIGANA LETTER U-5" },
    { 0x1B00F, "HENTAIGANA LETTER E-2" },
    { 0x1B010, "HENTAIGANA LETTER E-3" },
    { 0x1B011, "HENTAIGANA LETTER E-4" },
    { 0x1B012, "HENTAIGANA LETTER E-5" },
    { 0x1B013, "HENTAIGANA LETTER E-6" },
    { 0x1B014, "HENTAIGANA LETTER O-1" },
    { 0x1B015, "HENTAIGANA LETTER O-2" },
    { 0x1B016, "HENTAIGANA LETTER O-3" },
    { 0x1B017, "HENTAIGANA LETTER KA-1" },
    { 0x1B018, "HENTAIGANA LETTER KA-2" },
    { 0x1B019, "HENTAIGANA LETTER KA-3" },
    { 0x1B01A, "HENTAIGANA LETTER KA-4" },
    { 0x1B01B, "HENTAIGANA LETTER KA-5" },
    { 0x1B01C, "HENTAIGANA LETTER KA-6" },
    { 0x1B01D, "HENTAIGANA LETTER KA-7" },
    { 0x1B01E, "HENTAIGANA LETTER KA-8" },
    { 0x1B01F, "HENTAIGANA LETTER KA-9" },
    { 0x1B020, "HENTAIGANA LETTER KA-10" },
    { 0x1B021, "HENTAIGANA LETTER KA-11" },
    { 0x1B022, "HENTAIGANA LETTER KA-KE" },
    { 0x1B023, "HENTAIGANA LETTER KI-1" },
    { 0x1B024, "HENTAIGANA LETTER KI-2" },
    { 0x1B025, "HENTAIGANA LETTER KI-3" },
    { 0x1B026, "HENTAIGANA LETTER KI-4" },
    { 0x1B027, "HENTAIGANA LETTER KI-5" },
    { 0x1B028, "HENTAIGANA LETTER KI-6" },
    { 0x1B029, "HENTAIGANA LETTER KI-7" },
    { 0x1B02A, "HENTAIGANA LETTER KI-8" },
    { 0x1B02B, "HENTAIGANA LETTER KU-1" },
    { 0x1B02C, "HENTAIGANA LETTER KU-2" },
    { 0x1B02D, "HENTAIGANA LETTER KU-3" },
    { 0x1B02E, "HENTAIGANA LETTER KU-4" },
    { 0x1B02F, "HENTAIGANA LETTER KU-5" },
    { 0x1B030, "HENTAIGANA LETTER KU-6" },
    { 0x1B031, "HENTAIGANA LETTER KU-7" },
    { 0x1B032, "HENTAIGANA LETTER KE-1" },
    { 0x1B033, "HENTAIGANA LETTER KE-2" },
    { 0x1B034, "HENTAIGANA LETTER KE-3" },
    { 0x1B035, "HENTAIGANA LETTER KE-4" },
    { 0x1B036, "HENTAIGANA LETTER KE-5" },
    { 0x1B037, "HENTAIGANA LETTER KE-6" },
    { 0x1B038, "HENTAIGANA LETTER KO-1" },
    { 0x1B039, "HENTAIGANA LETTER KO-2" },
    { 0x1B03A, "HENTAIGANA LETTER KO-3" },
    { 0x1B03B, "HENTAIGANA LETTER KO-KI" },
    { 0x1B03C, "HENTAIGANA LETTER SA-1" },
    { 0x1B03D, "HENTAIGANA LETTER SA-2" },
    { 0x1B03E, "HENTAIGANA LETTER SA-3" },
    { 0x1B03F, "HENTAIGANA LETTER SA-4" },
    { 0x1B040, "HENTAIGANA LETTER SA-5" },
    { 0x1B041, "HENTAIGANA LETTER SA-6" },
    { 0x1B042, "HENTAIGANA LETTER SA-7" },
    { 0x1B043, "HENTAIGANA LETTER SA-8" },
    { 0x1B044, "HENTAIGANA LETTER SI-1" },
    { 0x1B045, "HENTAIGANA LETTER SI-2" },
    { 0x1B046, "HENTAIGANA LETTER SI-3" },
    { 0x1B047, "HENTAIGANA LETTER SI-4" },
    { 0x1B048, "HENTAIGANA LETTER SI-5" },
    { 0x1B049, "HENTAIGANA LETTER SI-6" },
    { 0x1B04A, "HENTAIGANA LETTER SU-1" },
    { 0x1B04B, "HENTAIGANA LETTER SU-2" },
    { 0x1B04C, "HENTAIGANA LETTER SU-3" },
    { 0x1B04D, "HENTAIGANA LETTER SU-4" },
    { 0x1B04E, "HENTAIGANA LETTER SU-5" },
    { 0x1B04F, "HENTAIGANA LETTER SU-6" },
    { 0x1B050, "HENTAIGANA LETTER SU-7" },
    { 0x1B051, "HENTAIGANA LETTER SU-8" },
    { 0x1B052, "HENTAIGANA LETTER SE-1" },
    { 0x1B053, "HENTAIGANA LETTER SE-2" },
    { 0x1B054, "HENTAIGANA LETTER SE-3" },
    { 0x1B055, "HENTAIGANA LETTER SE-4" },
    { 0x1B056, "HENTAIGANA LETTER SE-5" },
    { 0x1B057, "HENTAIGANA LETTER SO-1" },
    { 0x1B058, "HENTAIGANA LETTER SO-2" },
    { 0x1B059, "HENTAIGANA LETTER SO-3" },
    { 0x1B05A, "HENTAIGANA LETTER SO-4" },
    { 0x1B05B, "HENTAIGANA LETTER SO-5" },
    { 0x1B05C, "HENTAIGANA LETTER SO-6" },
    { 0x1B05D, "HENTAIGANA LETTER SO-7" },
    { 0x1B05E, "HENTAIGANA LETTER TA-1" },
    { 0x1B05F, "HENTAIGANA LETTER TA-2" },
    { 0x1B060, "HENTAIGANA LETTER TA-3" },
    { 0x1B061, "HENTAIGANA LETTER TA-4" },
    { 0x1B062, "HENTAIGANA LETTER TI-1" },
    { 0x1B063, "HENTAIGANA LETTER TI-2" },
    { 0x1B064, "HENTAIGANA LETTER TI-3" },
    { 0x1B065, "HENTAIGANA LETTER TI-4" },
    { 0x1B066, "HENTAIGANA LETTER TI-5" },
    { 0x1B067, "HENTAIGANA LETTER TI-6" },
    { 0x1B068, "HENTAIGANA LETTER TI-7" },
    { 0x1B069, "HENTAIGANA LETTER TU-1" },
    { 0x1B06A, "HENTAIGANA LETTER TU-2" },
    { 0x1B06B, "HENTAIGANA LETTER TU-3" },
    { 0x1B06C, "HENTAIGANA LETTER TU-4" },
    { 0x1B06D, "HENTAIGANA LETTER TU-TO" },
    { 0x1B06E, "HENTAIGANA LETTER TE-1" },
    { 0x1B06F, "HENTAIGANA LETTER TE-2" },
    { 0x1B070, "HENTAIGANA LETTER TE-3" },
    { 0x1B071, "HENTAIGANA LETTER TE-4" },
    { 0x1B072, "HENTAIGANA LETTER TE-5" },
    { 0x1B073, "HENTAIGANA LETTER TE-6" },
    { 0x1B074, "HENTAIGANA LETTER TE-7" },
    { 0x1B075, "HENTAIGANA LETTER TE-8" },
    { 0x1B076, "HENTAIGANA LETTER TE-9" },
    { 0x1B077, "HENTAIGANA LETTER TO-1" },
    { 0x1B078, "HENTAIGANA LETTER TO-2" },
    { 0x1B079, "HENTAIGANA LETTER TO-3" },
    { 0x1B07A, "HENTAIGANA LETTER TO-4" },
    { 0x1B07B, "HENTAIGANA LETTER TO-5" },
    { 0x1B07C, "HENTAIGANA LETTER TO-6" },
    { 0x1B07D, "HENTAIGANA LETTER TO-RA" },
    { 0x1B07E, "HENTAIGANA LETTER NA-1" },
    { 0x1B07F, "HENTAIGANA LETTER NA-2" },
    { 0x1B080, "HENTAIGANA LETTER NA-3" },
    { 0x1B081, "HENTAIGANA LETTER NA-4" },
    { 0x1B082, "HENTAIGANA LETTER NA-5" },
    { 0x1B083, "HENTAIGANA LETTER NA-6" },
    { 0x1B084, "HENTAIGANA LETTER NA-7" },
    { 0x1B085, "HENTAIGANA LETTER NA-8" },
    { 0x1B086, "HENTAIGANA LETTER NA-9" },
    { 0x1B087, "HENTAIGANA LETTER NI-1" },
    { 0x1B088, "HENTAIGANA LETTER NI-2" },
    { 0x1B089, "HENTAIGANA LETTER NI-3" },
    { 0x1B08A, "HENTAIGANA LETTER NI-4" },
    { 0x1B08B, "HENTAIGANA LETTER NI-5" },
    { 0x1B08C, "HENTAIGANA LETTER NI-6" },
    { 0x1B08D, "HENTAIGANA LETTER NI-7" },
    { 0x1B08E, "HENTAIGANA LETTER NI-TE" },
    { 0x1B08F, "HENTAIGANA LETTER NU-1" },
    { 0x1B090, "HENTAIGANA LETTER NU-2" },
    { 0x1B091, "HENTAIGANA LETTER NU-3" },
    { 0x1B092, "HENTAIGANA LETTER NE-1" },
    { 0x1B093, "HENTAIGANA LETTER NE-2" },
    { 0x1B094, "HENTAIGANA LETTER NE-3" },
    { 0x1B095, "HENTAIGANA LETTER NE-4" },
    { 0x1B096, "HENTAIGANA LETTER NE-5" },
    { 0x1B097, "HENTAIGANA LETTER NE-6" },
    { 0x1B098, "HENTAIGANA LETTER NE-KO" },
    { 0x1B099, "HENTAIGANA LETTER NO-1" },
    { 0x1B09A, "HENTAIGANA LETTER NO-2" },
    { 0x1B09B, "HENTAIGANA LETTER NO-3" },
    { 0x1B09C, "HENTAIGANA LETTER NO-4" },
    { 0x1B09D, "HENTAIGANA LETTER NO-5" },
    { 0x1B09E, "HENTAIGANA LETTER HA-1" },
    { 0x1B09F, "HENTAIGANA LETTER HA-2" },
    { 0x1B0A0, "HENTAIGANA LETTER HA-3" },
    { 0x1B0A1, "HENTAIGANA LETTER HA-4" },
    { 0x1B0A2, "HENTAIGANA LETTER HA-5" },
    { 0x1B0A3, "HENTAIGANA LETTER HA-6" },
    { 0x1B0A4, "HENTAIGANA LETTER HA-7" },
    { 0x1B0A5, "HENTAIGANA LETTER HA-8" },
    { 0x1B0A6, "HENTAIGANA LETTER HA-9" },
    { 0x1B0A7, "HENTAIGANA LETTER HA-10" },
    { 0x1B0A8, "HENTAIGANA LETTER HA-11" },
    { 0x1B0A9, "HENTAIGANA LETTER HI-1" },
    { 0x1B0AA, "HENTAIGANA LETTER HI-2" },
    { 0x1B0AB, "HENTAIGANA LETTER HI-3" },
    { 0x1B0AC, "HENTAIGANA LETTER HI-4" },
    { 0x1B0AD, "HENTAIGANA LETTER HI-5" },
    { 0x1B0AE, "HENTAIGANA LETTER HI-6" },
    { 0x1B0AF, "HENTAIGANA LETTER HI-7" },
    { 0x1B0B0, "HENTAIGANA LETTER HU-1" },
    { 0x1B0B1, "HENTAIGANA LETTER HU-2" },
    { 0x1B0B2, "HENTAIGANA LETTER HU-3" },
    { 0x1B0B3, "HENTAIGANA LETTER HE-1" },
    { 0x1B0B4, "HENTAIGANA LETTER HE-2" },
    { 0x1B0B5, "HENTAIGANA LETTER HE-3" },
    { 0x1B0B6, "HENTAIGANA LETTER HE-4" },
    { 0x1B0B7, "HENTAIGANA LETTER HE-5" },
    { 0x1B0B8, "HENTAIGANA LETTER HE-6" },
    { 0x1B0B9, "HENTAIGANA LETTER HE-7" },
    { 0x1B0BA, "HENTAIGANA LETTER HO-1" },
    { 0x1B0BB, "HENTAIGANA LETTER HO-2" },
    { 0x1B0BC, "HENTAIGANA LETTER HO-3" },
    { 0x1B0BD, "HENTAIGANA LETTER HO-4" },
    { 0x1B0BE, "HENTAIGANA LETTER HO-5" },
    { 0x1B0BF, "HENTAIGANA LETTER HO-6" },
    { 0x1B0C0, "HENTAIGANA LETTER HO-7" },
    { 0x1B0C1, "HENTAIGANA LETTER HO-8" },
    { 0x1B0C2, "HENTAIGANA LETTER MA-1" },
    { 0x1B0C3, "HENTAIGANA LETTER MA-2" },
    { 0x1B0C4, "HENTAIGANA LETTER MA-3" },
    { 0x1B0C5, "HENTAIGANA LETTER MA-4" },
    { 0x1B0C6, "HENTAIGANA LETTER MA-5" },
    { 0x1B0C7, "HENTAIGANA LETTER MA-6" },
    { 0x1B0C8, "HENTAIGANA LETTER MA-7" },
    { 0x1B0C9, "HENTAIGANA LETTER MI-1" },
    { 0x1B0CA, "HENTAIGANA LETTER MI-2" },
    { 0x1B0CB, "HENTAIGANA LETTER MI-3" },
    { 0x1B0CC, "HENTAIGANA LETTER MI-4" },
    { 0x1B0CD, "HENTAIGANA LETTER MI-5" },
    { 0x1B0CE, "HENTAIGANA LETTER MI-6" },
    { 0x1B0CF, "HENTAIGANA LETTER MI-7" },
    { 0x1B0D0, "HENTAIGANA LETTER MU-1" },
    { 0x1B0D1, "HENTAIGANA LETTER MU-2" },
    { 0x1B0D2, "HENTAIGANA LETTER MU-3" },
    { 0x1B0D3, "HENTAIGANA LETTER MU-4" },
    { 0x1B0D4, "HENTAIGANA LETTER ME-1" },
    { 0x1B0D5, "HENTAIGANA LETTER ME-2" },
    { 0x1B0D6, "HENTAIGANA LETTER ME-MA" },
    { 0x1B0D7, "HENTAIGANA LETTER MO-1" },
    { 0x1B0D8, "HENTAIGANA LETTER MO-2" },
    { 0x1B0D9, "HENTAIGANA LETTER MO-3" },
    { 0x1B0DA, "HENTAIGANA LETTER MO-4" },
    { 0x1B0DB, "HENTAIGANA LETTER MO-5" },
    { 0x1B0DC, "HENTAIGANA LETTER MO-6" },
    { 0x1B0DD, "HENTAIGANA LETTER YA-1" },
    { 0x1B0DE, "HENTAIGANA LETTER YA-2" },
    { 0x1B0DF, "HENTAIGANA LETTER YA-3" },
    { 0x1B0E0, "HENTAIGANA LETTER YA-4" },
    { 0x1B0E1, "HENTAIGANA LETTER YA-5" },
    { 0x1B0E2, "HENTAIGANA LETTER YA-YO" },
    { 0x1B0E3, "HENTAIGANA LETTER YU-1" },
    { 0x1B0E4, "HENTAIGANA LETTER YU-2" },
    { 0x1B0E5, "HENTAIGANA LETTER YU-3" },
    { 0x1B0E6, "HENTAIGANA LETTER YU-4" },
    { 0x1B0E7, "HENTAIGANA LETTER YO-1" },
    { 0x1B0E8, "HENTAIGANA LETTER YO-2" },
    { 0x1B0E9, "HENTAIGANA LETTER YO-3" },
    { 0x1B0EA, "HENTAIGANA LETTER YO-4" },
    { 0x1B0EB, "HENTAIGANA LETTER YO-5" },
    { 0x1B0EC, "HENTAIGANA LETTER YO-6" },
    { 0x1B0ED, "HENTAIGANA LETTER RA-1" },
    { 0x1B0EE, "HENTAIGANA LETTER RA-2" },
    { 0x1B0EF, "HENTAIGANA LETTER RA-3" },
    { 0x1B0F0, "HENTAIGANA LETTER RA-4" },
    { 0x1B0F1, "HENTAIGANA LETTER RI-1" },
    { 0x1B0F2, "HENTAIGANA LETTER RI-2" },
    { 0x1B0F3, "HENTAIGANA LETTER RI-3" },
    { 0x1B0F4, "HENTAIGANA LETTER RI-4" },
    { 0x1B0F5, "HENTAIGANA LETTER RI-5" },
    { 0x1B0F6, "HENTAIGANA LETTER RI-6" },
    { 0x1B0F7, "HENTAIGANA LETTER RI-7" },
    { 0x1B0F8, "HENTAIGANA LETTER RU-1" },
    { 0x1B0F9, "HENTAIGANA LETTER RU-2" },
    { 0x1B0FA, "HENTAIGANA LETTER RU-3" },
    { 0x1B0FB, "HENTAIGANA LETTER RU-4" },
    { 0x1B0FC, "HENTAIGANA LETTER RU-5" },
    { 0x1B0FD, "HENTAIGANA LETTER RU-6" },
    { 0x1B0FE, "HENTAIGANA LETTER RE-1" },
    { 0x1B0FF, "HENTAIGANA LETTER RE-2" },
    { 0x1B100, "HENTAIGANA LETTER RE-3" },
    { 0x1B101, "HENTAIGANA LETTER RE-4" },
    { 0x1B102, "HENTAIGANA LETTER RO-1" },
    { 0x1B103, "HENTAIGANA LETTER RO-2" },
    { 0x1B104, "HENTAIGANA LETTER RO-3" },
    { 0x1B105, "HENTAIGANA LETTER RO-4" },
    { 0x1B106, "HENTAIGANA LETTER RO-5" },
    { 0x1B107, "HENTAIGANA LETTER RO-6" },
    { 0x1B108, "HENTAIGANA LETTER WA-1" },
    { 0x1B109, "HENTAIGANA LETTER WA-2" },
    { 0x1B10A, "HENTAIGANA LETTER WA-3" },
    { 0x1B10B, "HENTAIGANA LETTER WA-4" },
    { 0x1B10C, "HENTAIGANA LETTER WA-5" },
    { 0x1B10D, "HENTAIGANA LETTER WI-1" },
    { 0x1B10E, "HENTAIGANA LETTER WI-2" },
    { 0x1B10F, "HENTAIGANA LETTER WI-3" },
    { 0x1B110, "HENTAIGANA LETTER WI-4" },
    { 0x1B111, "HENTAIGANA LETTER WI-5" },
    { 0x1B112, "HENTAIGANA LETTER WE-1" },
    { 0x1B113, "HENTAIGANA LETTER WE-2" },
    { 0x1B114, "HENTAIGANA LETTER WE-3" },
    { 0x1B115, "HENTAIGANA LETTER WE-4" },
    { 0x1B116, "HENTAIGANA LETTER WO-1" },
    { 0x1B117, "HENTAIGANA LETTER WO-2" },
    { 0x1B118, "HENTAIGANA LETTER WO-3" },
    { 0x1B119, "HENTAIGANA LETTER WO-4" },
    { 0x1B11A, "HENTAIGANA LETTER WO-5" },
    { 0x1B11B, "HENTAIGANA LETTER WO-6" },
    { 0x1B11C, "HENTAIGANA LETTER WO-7" },
    { 0x1B11D, "HENTAIGANA LETTER N-MU-MO-1" },
    { 0x1B11E, "HENTAIGANA LETTER N-MU-MO-2" },
    { 0x1B11F, "HIRAGANA LETTER ARCHAIC WU" },
    { 0x1B120, "KATAKANA LETTER ARCHAIC YI" },
    { 0x1B121, "KATAKANA LETTER ARCHAIC YE" },
    { 0x1B122, "KATAKANA LETTER ARCHAIC WU" },
    { 0x1B132, "HIRAGANA LETTER SMALL KO" },
    { 0x1B150, "HIRAGANA LETTER SMALL WI" },
    { 0x1B151, "HIRAGANA LETTER SMALL WE" },
    { 0x1B152, "HIRAGANA LETTER SMALL WO" },
    { 0x1B155, "KATAKANA LETTER SMALL KO" },
    { 0x1B164, "KATAKANA LETTER SMALL WI" },
    { 0x1B165, "KATAKANA LETTER SMALL WE" },
    { 0x1B166, "KATAKANA LETTER SMALL WO" },
    { 0x1B167, "KATAKANA LETTER SMALL N" },
    { 0x1B170, "NUSHU CHARACTER-1B170" },
    { 0x1B171, "NUSHU CHARACTER-1B171" },
    { 0x1B172, "NUSHU CHARACTER-1B172" },
    { 0x1B173, "NUSHU CHARACTER-1B173" },
    { 0x1B174, "NUSHU CHARACTER-1B174" },
    { 0x1B175, "NUSHU CHARACTER-1B175" },
    { 0x1B176, "NUSHU CHARACTER-1B176" },
    { 0x1B177, "NUSHU CHARACTER-1B177" },
    { 0x1B178, "NUSHU CHARACTER-1B178" },
    { 0x1B179, "NUSHU CHARACTER-1B179" },
    { 0x1B17A, "NUSHU CHARACTER-1B17A" },
    { 0x1B17B, "NUSHU CHARACTER-1B17B" },
    { 0x1B17C, "NUSHU CHARACTER-1B17C" },
    { 0x1B17D, "NUSHU CHARACTER-1B17D" },
    { 0x1B17E, "NUSHU CHARACTER-1B17E" },
    { 0x1B17F, "NUSHU CHARACTER-1B17F" },
    { 0x1B180, "NUSHU CHARACTER-1B180" },
    { 0x1B181, "NUSHU CHARACTER-1B181" },
    { 0x1B182, "NUSHU CHARACTER-1B182" },
    { 0x1B183, "NUSHU CHARACTER-1B183" },
    { 0x1B184, "NUSHU CHARACTER-1B184" },
    { 0x1B185, "NUSHU CHARACTER-1B185" },
    { 0x1B186, "NUSHU CHARACTER-1B186" },
    { 0x1B187, "NUSHU CHARACTER-1B187" },
    { 0x1B188, "NUSHU CHARACTER-1B188" },
    { 0x1B189, "NUSHU CHARACTER-1B189" },
    { 0x1B18A, "NUSHU CHARACTER-1B18A" },
    { 0x1B18B, "NUSHU CHARACTER-1B18B" },
    { 0x1B18C, "NUSHU CHARACTER-1B18C" },
    { 0x1B18D, "NUSHU CHARACTER-1B18D" },
    { 0x1B18E, "NUSHU CHARACTER-1B18E" },
    { 0x1B18F, "NUSHU CHARACTER-1B18F" },
    { 0x1B190, "NUSHU CHARACTER-1B190" },
    { 0x1B191, "NUSHU CHARACTER-1B191" },
    { 0x1B192, "NUSHU CHARACTER-1B192" },
    { 0x1B193, "NUSHU CHARACTER-1B193" },
    { 0x1B194, "NUSHU CHARACTER-1B194" },
    { 0x1B195, "NUSHU CHARACTER-1B195" },
    { 0x1B196, "NUSHU CHARACTER-1B196" },
    { 0x1B197, "NUSHU CHARACTER-1B197" },
    { 0x1B198, "NUSHU CHARACTER-1B198" },
    { 0x1B199, "NUSHU CHARACTER-1B199" },
    { 0x1B19A, "NUSHU CHARACTER-1B19A" },
    { 0x1B19B, "NUSHU CHARACTER-1B19B" },
    { 0x1B19C, "NUSHU CHARACTER-1B19C" },
    { 0x1B19D, "NUSHU CHARACTER-1B19D" },
    { 0x1B19E, "NUSHU CHARACTER-1B19E" },
    { 0x1B19F, "NUSHU CHARACTER-1B19F" },
    { 0x1B1A0, "NUSHU CHARACTER-1B1A0" },
    { 0x1B1A1, "NUSHU CHARACTER-1B1A1" },
    { 0x1B1A2, "NUSHU CHARACTER-1B1A2" },
    { 0x1B1A3, "NUSHU CHARACTER-1B1A3" },
    { 0x1B1A4, "NUSHU CHARACTER-1B1A4" },
    { 0x1B1A5, "NUSHU CHARACTER-1B1A5" },
    { 0x1B1A6, "NUSHU CHARACTER-1B1A6" },
    { 0x1B1A7, "NUSHU CHARACTER-1B1A7" },
    { 0x1B1A8, "NUSHU CHARACTER-1B1A8" },
    { 0x1B1A9, "NUSHU CHARACTER-1B1A9" },
    { 0x1B1AA, "NUSHU CHARACTER-1B1AA" },
    { 0x1B1AB, "NUSHU CHARACTER-1B1AB" },
    { 0x1B1AC, "NUSHU CHARACTER-1B1AC" },
    { 0x1B1AD, "NUSHU CHARACTER-1B1AD" },
    { 0x1B1AE, "NUSHU CHARACTER-1B1AE" },
    { 0x1B1AF, "NUSHU CHARACTER-1B1AF" },
    { 0x1B1B0, "NUSHU CHARACTER-1B1B0" },
    { 0x1B1B1, "NUSHU CHARACTER-1B1B1" },
    { 0x1B1B2, "NUSHU CHARACTER-1B1B2" },
    { 0x1B1B3, "NUSHU CHARACTER-1B1B3" },
    { 0x1B1B4, "NUSHU CHARACTER-1B1B4" },
    { 0x1B1B5, "NUSHU CHARACTER-1B1B5" },
    { 0x1B1B6, "NUSHU CHARACTER-1B1B6" },
    { 0x1B1B7, "NUSHU CHARACTER-1B1B7" },
    { 0x1B1B8, "NUSHU CHARACTER-1B1B8" },
    { 0x1B1B9, "NUSHU CHARACTER-1B1B9" },
    { 0x1B1BA, "NUSHU CHARACTER-1B1BA" },
    { 0x1B1BB, "NUSHU CHARACTER-1B1BB" },
    { 0x1B1BC, "NUSHU CHARACTER-1B1BC" },
    { 0x1B1BD, "NUSHU CHARACTER-1B1BD" },
    { 0x1B1BE, "NUSHU CHARACTER-1B1BE" },
    { 0x1B1BF, "NUSHU CHARACTER-1B1BF" },
    { 0x1B1C0, "NUSHU CHARACTER-1B1C0" },
    { 0x1B1C1, "NUSHU CHARACTER-1B1C1" },
    { 0x1B1C2, "NUSHU CHARACTER-1B1C2" },
    { 0x1B1C3, "NUSHU CHARACTER-1B1C3" },
    { 0x1B1C4, "NUSHU CHARACTER-1B1C4" },
    { 0x1B1C5, "NUSHU CHARACTER-1B1C5" },
    { 0x1B1C6, "NUSHU CHARACTER-1B1C6" },
    { 0x1B1C7, "NUSHU CHARACTER-1B1C7" },
    { 0x1B1C8, "NUSHU CHARACTER-1B1C8" },
    { 0x1B1C9, "NUSHU CHARACTER-1B1C9" },
    { 0x1B1CA, "NUSHU CHARACTER-1B1CA" },
    { 0x1B1CB, "NUSHU CHARACTER-1B1CB" },
    { 0x1B1CC, "NUSHU CHARACTER-1B1CC" },
    { 0x1B1CD, "NUSHU CHARACTER-1B1CD" },
    { 0x1B1CE, "NUSHU CHARACTER-1B1CE" },
    { 0x1B1CF, "NUSHU CHARACTER-1B1CF" },
    { 0x1B1D0, "NUSHU CHARACTER-1B1D0" },
    { 0x1B1D1, "NUSHU CHARACTER-1B1D1" },
    { 0x1B1D2, "NUSHU CHARACTER-1B1D2" },
    { 0x1B1D3, "NUSHU CHARACTER-1B1D3" },
    { 0x1B1D4, "NUSHU CHARACTER-1B1D4" },
    { 0x1B1D5, "NUSHU CHARACTER-1B1D5" },
    { 0x1B1D6, "NUSHU CHARACTER-1B1D6" },
    { 0x1B1D7, "NUSHU CHARACTER-1B1D7" },
    { 0x1B1D8, "NUSHU CHARACTER-1B1D8" },
    { 0x1B1D9, "NUSHU CHARACTER-1B1D9" },
    { 0x1B1DA, "NUSHU CHARACTER-1B1DA" },
    { 0x1B1DB, "NUSHU CHARACTER-1B1DB" },
    { 0x1B1DC, "NUSHU CHARACTER-1B1DC" },
    { 0x1B1DD, "NUSHU CHARACTER-1B1DD" },
    { 0x1B1DE, "NUSHU CHARACTER-1B1DE" },
    { 0x1B1DF, "NUSHU CHARACTER-1B1DF" },
    { 0x1B1E0, "NUSHU CHARACTER-1B1E0" },
    { 0x1B1E1, "NUSHU CHARACTER-1B1E1" },
    { 0x1B1E2, "NUSHU CHARACTER-1B1E2" },
    { 0x1B1E3, "NUSHU CHARACTER-1B1E3" },
    { 0x1B1E4, "NUSHU CHARACTER-1B1E4" },
    { 0x1B1E5, "NUSHU CHARACTER-1B1E5" },
    { 0x1B1E6, "NUSHU CHARACTER-1B1E6" },
    { 0x1B1E7, "NUSHU CHARACTER-1B1E7" },
    { 0x1B1E8, "NUSHU CHARACTER-1B1E8" },
    { 0x1B1E9, "NUSHU CHARACTER-1B1E9" },
    { 0x1B1EA, "NUSHU CHARACTER-1B1EA" },
    { 0x1B1EB, "NUSHU CHARACTER-1B1EB" },
    { 0x1B1EC, "NUSHU CHARACTER-1B1EC" },
    { 0x1B1ED, "NUSHU CHARACTER-1B1ED" },
    { 0x1B1EE, "NUSHU CHARACTER-1B1EE" },
    { 0x1B1EF, "NUSHU CHARACTER-1B1EF" },
    { 0x1B1F0, "NUSHU CHARACTER-1B1F0" },
    { 0x1B1F1, "NUSHU CHARACTER-1B1F1" },
    { 0x1B1F2, "NUSHU CHARACTER-1B1F2" },
    { 0x1B1F3, "NUSHU CHARACTER-1B1F3" },
    { 0x1B1F4, "NUSHU CHARACTER-1B1F4" },
    { 0x1B1F5, "NUSHU CHARACTER-1B1F5" },
    { 0x1B1F6, "NUSHU CHARACTER-1B1F6" },
    { 0x1B1F7, "NUSHU CHARACTER-1B1F7" },
    { 0x1B1F8, "NUSHU CHARACTER-1B1F8" },
    { 0x1B1F9, "NUSHU CHARACTER-1B1F9" },
    { 0x1B1FA, "NUSHU CHARACTER-1B1FA" },
    { 0x1B1FB, "NUSHU CHARACTER-1B1FB" },
    { 0x1B1FC, "NUSHU CHARACTER-1B1FC" },
    { 0x1B1FD, "NUSHU CHARACTER-1B1FD" },
    { 0x1B1FE, "NUSHU CHARACTER-1B1FE" },
    { 0x1B1FF, "NUSHU CHARACTER-1B1FF" },
    { 0x1B200, "NUSHU CHARACTER-1B200" },
    { 0x1B201, "NUSHU CHARACTER-1B201" },
    { 0x1B202, "NUSHU CHARACTER-1B202" },
    { 0x1B203, "NUSHU CHARACTER-1B203" },
    { 0x1B204, "NUSHU CHARACTER-1B204" },
    { 0x1B205, "NUSHU CHARACTER-1B205" },
    { 0x1B206, "NUSHU CHARACTER-1B206" },
    { 0x1B207, "NUSHU CHARACTER-1B207" },
    { 0x1B208, "NUSHU CHARACTER-1B208" },
    { 0x1B209, "NUSHU CHARACTER-1B209" },
    { 0x1B20A, "NUSHU CHARACTER-1B20A" },
    { 0x1B20B, "NUSHU CHARACTER-1B20B" },
    { 0x1B20C, "NUSHU CHARACTER-1B20C" },
    { 0x1B20D, "NUSHU CHARACTER-1B20D" },
    { 0x1B20E, "NUSHU CHARACTER-1B20E" },
    { 0x1B20F, "NUSHU CHARACTER-1B20F" },
    { 0x1B210, "NUSHU CHARACTER-1B210" },
    { 0x1B211, "NUSHU CHARACTER-1B211" },
    { 0x1B212, "NUSHU CHARACTER-1B212" },
    { 0x1B213, "NUSHU CHARACTER-1B213" },
    { 0x1B214, "NUSHU CHARACTER-1B214" },
    { 0x1B215, "NUSHU CHARACTER-1B215" },
    { 0x1B216, "NUSHU CHARACTER-1B216" },
    { 0x1B217, "NUSHU CHARACTER-1B217" },
    { 0x1B218, "NUSHU CHARACTER-1B218" },
    { 0x1B219, "NUSHU CHARACTER-1B219" },
    { 0x1B21A, "NUSHU CHARACTER-1B21A" },
    { 0x1B21B, "NUSHU CHARACTER-1B21B" },
    { 0x1B21C, "NUSHU CHARACTER-1B21C" },
    { 0x1B21D, "NUSHU CHARACTER-1B21D" },
    { 0x1B21E, "NUSHU CHARACTER-1B21E" },
    { 0x1B21F, "NUSHU CHARACTER-1B21F" },
    { 0x1B220, "NUSHU CHARACTER-1B220" },
    { 0x1B221, "NUSHU CHARACTER-1B221" },
    { 0x1B222, "NUSHU CHARACTER-1B222" },
    { 0x1B223, "NUSHU CHARACTER-1B223" },
    { 0x1B224, "NUSHU CHARACTER-1B224" },
    { 0x1B225, "NUSHU CHARACTER-1B225" },
    { 0x1B226, "NUSHU CHARACTER-1B226" },
    { 0x1B227, "NUSHU CHARACTER-1B227" },
    { 0x1B228, "NUSHU CHARACTER-1B228" },
    { 0x1B229, "NUSHU CHARACTER-1B229" },
    { 0x1B22A, "NUSHU CHARACTER-1B22A" },
    { 0x1B22B, "NUSHU CHARACTER-1B22B" },
    { 0x1B22C, "NUSHU CHARACTER-1B22C" },
    { 0x1B22D, "NUSHU CHARACTER-1B22D" },
    { 0x1B22E, "NUSHU CHARACTER-1B22E" },
    { 0x1B22F, "NUSHU CHARACTER-1B22F" },
    { 0x1B230, "NUSHU CHARACTER-1B230" },
    { 0x1B231, "NUSHU CHARACTER-1B231" },
    { 0x1B232, "NUSHU CHARACTER-1B232" },
    { 0x1B233, "NUSHU CHARACTER-1B233" },
    { 0x1B234, "NUSHU CHARACTER-1B234" },
    { 0x1B235, "NUSHU CHARACTER-1B235" },
    { 0x1B236, "NUSHU CHARACTER-1B236" },
    { 0x1B237, "NUSHU CHARACTER-1B237" },
    { 0x1B238, "NUSHU CHARACTER-1B238" },
    { 0x1B239, "NUSHU CHARACTER-1B239" },
    { 0x1B23A, "NUSHU CHARACTER-1B23A" },
    { 0x1B23B, "NUSHU CHARACTER-1B23B" },
    { 0x1B23C, "NUSHU CHARACTER-1B23C" },
    { 0x1B23D, "NUSHU CHARACTER-1B23D" },
    { 0x1B23E, "NUSHU CHARACTER-1B23E" },
    { 0x1B23F, "NUSHU CHARACTER-1B23F" },
    { 0x1B240, "NUSHU CHARACTER-1B240" },
    { 0x1B241, "NUSHU CHARACTER-1B241" },
    { 0x1B242, "NUSHU CHARACTER-1B242" },
    { 0x1B243, "NUSHU CHARACTER-1B243" },
    { 0x1B244, "NUSHU CHARACTER-1B244" },
    { 0x1B245, "NUSHU CHARACTER-1B245" },
    { 0x1B246, "NUSHU CHARACTER-1B246" },
    { 0x1B247, "NUSHU CHARACTER-1B247" },
    { 0x1B248, "NUSHU CHARACTER-1B248" },
    { 0x1B249, "NUSHU CHARACTER-1B249" },
    { 0x1B24A, "NUSHU CHARACTER-1B24A" },
    { 0x1B24B, "NUSHU CHARACTER-1B24B" },
    { 0x1B24C, "NUSHU CHARACTER-1B24C" },
    { 0x1B24D, "NUSHU CHARACTER-1B24D" },
    { 0x1B24E, "NUSHU CHARACTER-1B24E" },
    { 0x1B24F, "NUSHU CHARACTER-1B24F" },
    { 0x1B250, "NUSHU CHARACTER-1B250" },
    { 0x1B251, "NUSHU CHARACTER-1B251" },
    { 0x1B252, "NUSHU CHARACTER-1B252" },
    { 0x1B253, "NUSHU CHARACTER-1B253" },
    { 0x1B254, "NUSHU CHARACTER-1B254" },
    { 0x1B255, "NUSHU CHARACTER-1B255" },
    { 0x1B256, "NUSHU CHARACTER-1B256" },
    { 0x1B257, "NUSHU CHARACTER-1B257" },
    { 0x1B258, "NUSHU CHARACTER-1B258" },
    { 0x1B259, "NUSHU CHARACTER-1B259" },
    { 0x1B25A, "NUSHU CHARACTER-1B25A" },
    { 0x1B25B, "NUSHU CHARACTER-1B25B" },
    { 0x1B25C, "NUSHU CHARACTER-1B25C" },
    { 0x1B25D, "NUSHU CHARACTER-1B25D" },
    { 0x1B25E, "NUSHU CHARACTER-1B25E" },
    { 0x1B25F, "NUSHU CHARACTER-1B25F" },
    { 0x1B260, "NUSHU CHARACTER-1B260" },
    { 0x1B261, "NUSHU CHARACTER-1B261" },
    { 0x1B262, "NUSHU CHARACTER-1B262" },
    { 0x1B263, "NUSHU CHARACTER-1B263" },
    { 0x1B264, "NUSHU CHARACTER-1B264" },
    { 0x1B265, "NUSHU CHARACTER-1B265" },
    { 0x1B266, "NUSHU CHARACTER-1B266" },
    { 0x1B267, "NUSHU CHARACTER-1B267" },
    { 0x1B268, "NUSHU CHARACTER-1B268" },
    { 0x1B269, "NUSHU CHARACTER-1B269" },
    { 0x1B26A, "NUSHU CHARACTER-1B26A" },
    { 0x1B26B, "NUSHU CHARACTER-1B26B" },
    { 0x1B26C, "NUSHU CHARACTER-1B26C" },
    { 0x1B26D, "NUSHU CHARACTER-1B26D" },
    { 0x1B26E, "NUSHU CHARACTER-1B26E" },
    { 0x1B26F, "NUSHU CHARACTER-1B26F" },
    { 0x1B270, "NUSHU CHARACTER-1B270" },
    { 0x1B271, "NUSHU CHARACTER-1B271" },
    { 0x1B272, "NUSHU CHARACTER-1B272" },
    { 0x1B273, "NUSHU CHARACTER-1B273" },
    { 0x1B274, "NUSHU CHARACTER-1B274" },
    { 0x1B275, "NUSHU CHARACTER-1B275" },
    { 0x1B276, "NUSHU CHARACTER-1B276" },
    { 0x1B277, "NUSHU CHARACTER-1B277" },
    { 0x1B278, "NUSHU CHARACTER-1B278" },
    { 0x1B279, "NUSHU CHARACTER-1B279" },
    { 0x1B27A, "NUSHU CHARACTER-1B27A" },
    { 0x1B27B, "NUSHU CHARACTER-1B27B" },
    { 0x1B27C, "NUSHU CHARACTER-1B27C" },
    { 0x1B27D, "NUSHU CHARACTER-1B27D" },
    { 0x1B27E, "NUSHU CHARACTER-1B27E" },
    { 0x1B27F, "NUSHU CHARACTER-1B27F" },
    { 0x1B280, "NUSHU CHARACTER-1B280" },
    { 0x1B281, "NUSHU CHARACTER-1B281" },
    { 0x1B282, "NUSHU CHARACTER-1B282" },
    { 0x1B283, "NUSHU CHARACTER-1B283" },
    { 0x1B284, "NUSHU CHARACTER-1B284" },
    { 0x1B285, "NUSHU CHARACTER-1B285" },
    { 0x1B286, "NUSHU CHARACTER-1B286" },
    { 0x1B287, "NUSHU CHARACTER-1B287" },
    { 0x1B288, "NUSHU CHARACTER-1B288" },
    { 0x1B289, "NUSHU CHARACTER-1B289" },
    { 0x1B28A, "NUSHU CHARACTER-1B28A" },
    { 0x1B28B, "NUSHU CHARACTER-1B28B" },
    { 0x1B28C, "NUSHU CHARACTER-1B28C" },
    { 0x1B28D, "NUSHU CHARACTER-1B28D" },
    { 0x1B28E, "NUSHU CHARACTER-1B28E" },
    { 0x1B28F, "NUSHU CHARACTER-1B28F" },
    { 0x1B290, "NUSHU CHARACTER-1B290" },
    { 0x1B291, "NUSHU CHARACTER-1B291" },
    { 0x1B292, "NUSHU CHARACTER-1B292" },
    { 0x1B293, "NUSHU CHARACTER-1B293" },
    { 0x1B294, "NUSHU CHARACTER-1B294" },
    { 0x1B295, "NUSHU CHARACTER-1B295" },
    { 0x1B296, "NUSHU CHARACTER-1B296" },
    { 0x1B297, "NUSHU CHARACTER-1B297" },
    { 0x1B298, "NUSHU CHARACTER-1B298" },
    { 0x1B299, "NUSHU CHARACTER-1B299" },
    { 0x1B29A, "NUSHU CHARACTER-1B29A" },
    { 0x1B29B, "NUSHU CHARACTER-1B29B" },
    { 0x1B29C, "NUSHU CHARACTER-1B29C" },
    { 0x1B29D, "NUSHU CHARACTER-1B29D" },
    { 0x1B29E, "NUSHU CHARACTER-1B29E" },
    { 0x1B29F, "NUSHU CHARACTER-1B29F" },
    { 0x1B2A0, "NUSHU CHARACTER-1B2A0" },
    { 0x1B2A1, "NUSHU CHARACTER-1B2A1" },
    { 0x1B2A2, "NUSHU CHARACTER-1B2A2" },
    { 0x1B2A3, "NUSHU CHARACTER-1B2A3" },
    { 0x1B2A4, "NUSHU CHARACTER-1B2A4" },
    { 0x1B2A5, "NUSHU CHARACTER-1B2A5" },
    { 0x1B2A6, "NUSHU CHARACTER-1B2A6" },
    { 0x1B2A7, "NUSHU CHARACTER-1B2A7" },
    { 0x1B2A8, "NUSHU CHARACTER-1B2A8" },
    { 0x1B2A9, "NUSHU CHARACTER-1B2A9" },
    { 0x1B2AA, "NUSHU CHARACTER-1B2AA" },
    { 0x1B2AB, "NUSHU CHARACTER-1B2AB" },
    { 0x1B2AC, "NUSHU CHARACTER-1B2AC" },
    { 0x1B2AD, "NUSHU CHARACTER-1B2AD" },
    { 0x1B2AE, "NUSHU CHARACTER-1B2AE" },
    { 0x1B2AF, "NUSHU CHARACTER-1B2AF" },
    { 0x1B2B0, "NUSHU CHARACTER-1B2B0" },
    { 0x1B2B1, "NUSHU CHARACTER-1B2B1" },
    { 0x1B2B2, "NUSHU CHARACTER-1B2B2" },
    { 0x1B2B3, "NUSHU CHARACTER-1B2B3" },
    { 0x1B2B4, "NUSHU CHARACTER-1B2B4" },
    { 0x1B2B5, "NUSHU CHARACTER-1B2B5" },
    { 0x1B2B6, "NUSHU CHARACTER-1B2B6" },
    { 0x1B2B7, "NUSHU CHARACTER-1B2B7" },
    { 0x1B2B8, "NUSHU CHARACTER-1B2B8" },
    { 0x1B2B9, "NUSHU CHARACTER-1B2B9" },
    { 0x1B2BA, "NUSHU CHARACTER-1B2BA" },
    { 0x1B2BB, "NUSHU CHARACTER-1B2BB" },
    { 0x1B2BC, "NUSHU CHARACTER-1B2BC" },
    { 0x1B2BD, "NUSHU CHARACTER-1B2BD" },
    { 0x1B2BE, "NUSHU CHARACTER-1B2BE" },
    { 0x1B2BF, "NUSHU CHARACTER-1B2BF" },
    { 0x1B2C0, "NUSHU CHARACTER-1B2C0" },
    { 0x1B2C1, "NUSHU CHARACTER-1B2C1" },
    { 0x1B2C2, "NUSHU CHARACTER-1B2C2" },
    { 0x1B2C3, "NUSHU CHARACTER-1B2C3" },
    { 0x1B2C4, "NUSHU CHARACTER-1B2C4" },
    { 0x1B2C5, "NUSHU CHARACTER-1B2C5" },
    { 0x1B2C6, "NUSHU CHARACTER-1B2C6" },
    { 0x1B2C7, "NUSHU CHARACTER-1B2C7" },
    { 0x1B2C8, "NUSHU CHARACTER-1B2C8" },
    { 0x1B2C9, "NUSHU CHARACTER-1B2C9" },
    { 0x1B2CA, "NUSHU CHARACTER-1B2CA" },
    { 0x1B2CB, "NUSHU CHARACTER-1B2CB" },
    { 0x1B2CC, "NUSHU CHARACTER-1B2CC" },
    { 0x1B2CD, "NUSHU CHARACTER-1B2CD" },
    { 0x1B2CE, "NUSHU CHARACTER-1B2CE" },
    { 0x1B2CF, "NUSHU CHARACTER-1B2CF" },
    { 0x1B2D0, "NUSHU CHARACTER-1B2D0" },
    { 0x1B2D1, "NUSHU CHARACTER-1B2D1" },
    { 0x1B2D2, "NUSHU CHARACTER-1B2D2" },
    { 0x1B2D3, "NUSHU CHARACTER-1B2D3" },
    { 0x1B2D4, "NUSHU CHARACTER-1B2D4" },
    { 0x1B2D5, "NUSHU CHARACTER-1B2D5" },
    { 0x1B2D6, "NUSHU CHARACTER-1B2D6" },
    { 0x1B2D7, "NUSHU CHARACTER-1B2D7" },
    { 0x1B2D8, "NUSHU CHARACTER-1B2D8" },
    { 0x1B2D9, "NUSHU CHARACTER-1B2D9" },
    { 0x1B2DA, "NUSHU CHARACTER-1B2DA" },
    { 0x1B2DB, "NUSHU CHARACTER-1B2DB" },
    { 0x1B2DC, "NUSHU CHARACTER-1B2DC" },
    { 0x1B2DD, "NUSHU CHARACTER-1B2DD" },
    { 0x1B2DE, "NUSHU CHARACTER-1B2DE" },
    { 0x1B2DF, "NUSHU CHARACTER-1B2DF" },
    { 0x1B2E0, "NUSHU CHARACTER-1B2E0" },
    { 0x1B2E1, "NUSHU CHARACTER-1B2E1" },
    { 0x1B2E2, "NUSHU CHARACTER-1B2E2" },
    { 0x1B2E3, "NUSHU CHARACTER-1B2E3" },
    { 0x1B2E4, "NUSHU CHARACTER-1B2E4" },
    { 0x1B2E5, "NUSHU CHARACTER-1B2E5" },
    { 0x1B2E6, "NUSHU CHARACTER-1B2E6" },
    { 0x1B2E7, "NUSHU CHARACTER-1B2E7" },
    { 0x1B2E8, "NUSHU CHARACTER-1B2E8" },
    { 0x1B2E9, "NUSHU CHARACTER-1B2E9" },
    { 0x1B2EA, "NUSHU CHARACTER-1B2EA" },
    { 0x1B2EB, "NUSHU CHARACTER-1B2EB" },
    { 0x1B2EC, "NUSHU CHARACTER-1B2EC" },
    { 0x1B2ED, "NUSHU CHARACTER-1B2ED" },
    { 0x1B2EE, "NUSHU CHARACTER-1B2EE" },
    { 0x1B2EF, "NUSHU CHARACTER-1B2EF" },
    { 0x1B2F0, "NUSHU CHARACTER-1B2F0" },
    { 0x1B2F1, "NUSHU CHARACTER-1B2F1" },
    { 0x1B2F2, "NUSHU CHARACTER-1B2F2" },
    { 0x1B2F3, "NUSHU CHARACTER-1B2F3" },
    { 0x1B2F4, "NUSHU CHARACTER-1B2F4" },
    { 0x1B2F5, "NUSHU CHARACTER-1B2F5" },
    { 0x1B2F6, "NUSHU CHARACTER-1B2F6" },
    { 0x1B2F7, "NUSHU CHARACTER-1B2F7" },
    { 0x1B2F8, "NUSHU CHARACTER-1B2F8" },
    { 0x1B2F9, "NUSHU CHARACTER-1B2F9" },
    { 0x1B2FA, "NUSHU CHARACTER-1B2FA" },
    { 0x1B2FB, "NUSHU CHARACTER-1B2FB" },
    { 0x1BC00, "DUPLOYAN LETTER H" },
    { 0x1BC01, "DUPLOYAN LETTER X" },
    { 0x1BC02, "DUPLOYAN LETTER P" },
    { 0x1BC03, "DUPLOYAN LETTER T" },
    { 0x1BC04, "DUPLOYAN LETTER F" },
    { 0x1BC05, "DUPLOYAN LETTER K" },
    { 0x1BC06, "DUPLOYAN LETTER L" },
    { 0x1BC07, "DUPLOYAN LETTER B" },
    { 0x1BC08, "DUPLOYAN LETTER D" },
    { 0x1BC09, "DUPLOYAN LETTER V" },
    { 0x1BC0A, "DUPLOYAN LETTER G" },
    { 0x1BC0B, "DUPLOYAN LETTER R" },
    { 0x1BC0C, "DUPLOYAN LETTER P N" },
    { 0x1BC0D, "DUPLOYAN LETTER D S" },
    { 0x1BC0E, "DUPLOYAN LETTER F N" },
    { 0x1BC0F, "DUPLOYAN LETTER K M" },
    { 0x1BC10, "DUPLOYAN LETTER R S" },
    { 0x1BC11, "DUPLOYAN LETTER TH" },
    { 0x1BC12, "DUPLOYAN LETTER SLOAN DH" },
    { 0x1BC13, "DUPLOYAN LETTER DH" },
    { 0x1BC14, "DUPLOYAN LETTER KK" },
    { 0x1BC15, "DUPLOYAN LETTER SLOAN J" },
    { 0x1BC16, "DUPLOYAN LETTER HL" },
    { 0x1BC17, "DUPLOYAN LETTER LH" },
    { 0x1BC18, "DUPLOYAN LETTER RH" },
    { 0x1BC19, "DUPLOYAN LETTER M" },
    { 0x1BC1A, "DUPLOYAN LETTER N" },
    { 0x1BC1B, "DUPLOYAN LETTER J" },
    { 0x1BC1C, "DUPLOYAN LETTER S" },
    { 0x1BC1D, "DUPLOYAN LETTER M N" },
    { 0x1BC1E, "DUPLOYAN LETTER N M" },
    { 0x1BC1F, "DUPLOYAN LETTER J M" },
    { 0x1BC20, "DUPLOYAN LETTER S J" },
    { 0x1BC21, "DUPLOYAN LETTER M WITH DOT" },
    { 0x1BC22, "DUPLOYAN LETTER N WITH DOT" },
    { 0x1BC23, "DUPLOYAN LETTER J WITH DOT" },
    { 0x1BC24, "DUPLOYAN LETTER J WITH DOTS INSIDE AND ABOVE" },
    { 0x1BC25, "DUPLOYAN LETTER S WITH DOT" },
    { 0x1BC26, "DUPLOYAN LETTER S WITH DOT BELOW" },
    { 0x1BC27, "DUPLOYAN LETTER M S" },
    { 0x1BC28, "DUPLOYAN LETTER N S" },
    { 0x1BC29, "DUPLOYAN LETTER J S" },
    { 0x1BC2A, "DUPLOYAN LETTER S S" },
    { 0x1BC2B, "DUPLOYAN LETTER M N S" },
    { 0x1BC2C, "DUPLOYAN LETTER N M S" },
    { 0x1BC2D, "DUPLOYAN LETTER J M S" },
    { 0x1BC2E, "DUPLOYAN LETTER S J S" },
    { 0x1BC2F, "DUPLOYAN LETTER J S WITH DOT" },
    { 0x1BC30, "DUPLOYAN LETTER J N" },
    { 0x1BC31, "DUPLOYAN LETTER J N S" },
    { 0x1BC32, "DUPLOYAN LETTER S T" },
    { 0x1BC33, "DUPLOYAN LETTER S T R" },
    { 0x1BC34, "DUPLOYAN LETTER S P" },
    { 0x1BC35, "DUPLOYAN LETTER S P R" },
    { 0x1BC36, "DUPLOYAN LETTER T S" },
    { 0x1BC37, "DUPLOYAN LETTER T R S" },
    { 0x1BC38, "DUPLOYAN LETTER W" },
    { 0x1BC39, "DUPLOYAN LETTER WH" },
    { 0x1BC3A, "DUPLOYAN LETTER W R" },
    { 0x1BC3B, "DUPLOYAN LETTER S N" },
    { 0x1BC3C, "DUPLOYAN LETTER S M" },
    { 0x1BC3D, "DUPLOYAN LETTER K R S" },
    { 0x1BC3E, "DUPLOYAN LETTER G R S" },
    { 0x1BC3F, "DUPLOYAN LETTER S K" },
    { 0x1BC40, "DUPLOYAN LETTER S K R" },
    { 0x1BC41, "DUPLOYAN LETTER A" },
    { 0x1BC42, "DUPLOYAN LETTER SLOAN OW" },
    { 0x1BC43, "DUPLOYAN LETTER OA" },
    { 0x1BC44, "DUPLOYAN LETTER O" },
    { 0x1BC45, "DUPLOYAN LETTER AOU" },
    { 0x1BC46, "DUPLOYAN LETTER I" },
    { 0x1BC47, "DUPLOYAN LETTER E" },
    { 0x1BC48, "DUPLOYAN LETTER IE" },
    { 0x1BC49, "DUPLOYAN LETTER SHORT I" },
    { 0x1BC4A, "DUPLOYAN LETTER UI" },
    { 0x1BC4B, "DUPLOYAN LETTER EE" },
    { 0x1BC4C, "DUPLOYAN LETTER SLOAN EH" },
    { 0x1BC4D, "DUPLOYAN LETTER ROMANIAN I" },
    { 0x1BC4E, "DUPLOYAN LETTER SLOAN EE" },
    { 0x1BC4F, "DUPLOYAN LETTER LONG I" },
    { 0x1BC50, "DUPLOYAN LETTER YE" },
    { 0x1BC51, "DUPLOYAN LETTER U" },
    { 0x1BC52, "DUPLOYAN LETTER EU" },
    { 0x1BC53, "DUPLOYAN LETTER XW" },
    { 0x1BC54, "DUPLOYAN LETTER U N" },
    { 0x1BC55, "DUPLOYAN LETTER LONG U" },
    { 0x1BC56, "DUPLOYAN LETTER ROMANIAN U" },
    { 0x1BC57, "DUPLOYAN LETTER UH" },
    { 0x1BC58, "DUPLOYAN LETTER SLOAN U" },
    { 0x1BC59, "DUPLOYAN LETTER OOH" },
    { 0x1BC5A, "DUPLOYAN LETTER OW" },
    { 0x1BC5B, "DUPLOYAN LETTER OU" },
    { 0x1BC5C, "DUPLOYAN LETTER WA" },
    { 0x1BC5D, "DUPLOYAN LETTER WO" },
    { 0x1BC5E, "DUPLOYAN LETTER WI" },
    { 0x1BC5F, "DUPLOYAN LETTER WEI" },
    { 0x1BC60, "DUPLOYAN LETTER WOW" },
    { 0x1BC61, "DUPLOYAN LETTER NASAL U" },
    { 0x1BC62, "DUPLOYAN LETTER NASAL O" },
    { 0x1BC63, "DUPLOYAN LETTER NASAL I" },
    { 0x1BC64, "DUPLOYAN LETTER NASAL A" },
    { 0x1BC65, "DUPLOYAN LETTER PERNIN AN" },
    { 0x1BC66, "DUPLOYAN LETTER PERNIN AM" },
    { 0x1BC67, "DUPLOYAN LETTER SLOAN EN" },
    { 0x1BC68, "DUPLOYAN LETTER SLOAN AN" },
    { 0x1BC69, "DUPLOYAN LETTER SLOAN ON" },
    { 0x1BC6A, "DUPLOYAN LETTER VOCALIC M" },
    { 0x1BC70, "DUPLOYAN AFFIX LEFT HORIZONTAL SECANT" },
    { 0x1BC71, "DUPLOYAN AFFIX MID HORIZONTAL SECANT" },
    { 0x1BC72, "DUPLOYAN AFFIX RIGHT HORIZONTAL SECANT" },
    { 0x1BC73, "DUPLOYAN AFFIX LOW VERTICAL SECANT" },
    { 0x1BC74, "DUPLOYAN AFFIX MID VERTICAL SECANT" },
    { 0x1BC75, "DUPLOYAN AFFIX HIGH VERTICAL SECANT" },
    { 0x1BC76, "DUPLOYAN AFFIX ATTACHED SECANT" },
    { 0x1BC77, "DUPLOYAN AFFIX ATTACHED LEFT-TO-RIGHT SECANT" },
    { 0x1BC78, "DUPLOYAN AFFIX ATTACHED TANGENT" },
    { 0x1BC79, "DUPLOYAN AFFIX ATTACHED TAIL" },
    { 0x1BC7A, "DUPLOYAN AFFIX ATTACHED E HOOK" },
    { 0x1BC7B, "DUPLOYAN AFFIX ATTACHED I HOOK" },
    { 0x1BC7C, "DUPLOYAN AFFIX ATTACHED TANGENT HOOK" },
    { 0x1BC80, "DUPLOYAN AFFIX HIGH ACUTE" },
    { 0x1BC81, "DUPLOYAN AFFIX HIGH TIGHT ACUTE" },
    { 0x1BC82, "DUPLOYAN AFFIX HIGH GRAVE" },
    { 0x1BC83, "DUPLOYAN AFFIX HIGH LONG GRAVE" },
    { 0x1BC84, "DUPLOYAN AFFIX HIGH DOT" },
    { 0x1BC85, "DUPLOYAN AFFIX HIGH CIRCLE" },
    { 0x1BC86, "DUPLOYAN AFFIX HIGH LINE" },
    { 0x1BC87, "DUPLOYAN AFFIX HIGH WAVE" },
    { 0x1BC88, "DUPLOYAN AFFIX HIGH VERTICAL" },
    { 0x1BC90, "DUPLOYAN AFFIX LOW ACUTE" },
    { 0x1BC91, "DUPLOYAN AFFIX LOW TIGHT ACUTE" },
    { 0x1BC92, "DUPLOYAN AFFIX LOW GRAVE" },
    { 0x1BC93, "DUPLOYAN AFFIX LOW LONG GRAVE" },
    { 0x1BC94, "DUPLOYAN AFFIX LOW DOT" },
    { 0x1BC95, "DUPLOYAN AFFIX LOW CIRCLE" },
    { 0x1BC96, "DUPLOYAN AFFIX LOW LINE" },
    { 0x1BC97, "DUPLOYAN AFFIX LOW WAVE" },
    { 0x1BC98, "DUPLOYAN AFFIX LOW VERTICAL" },
    { 0x1BC99, "DUPLOYAN AFFIX LOW ARROW" },
    { 0x1BC9C, "DUPLOYAN SIGN O WITH CROSS" },
    { 0x1BC9D, "DUPLOYAN THICK LETTER SELECTOR" },
    { 0x1BC9E, "DUPLOYAN DOUBLE MARK" },
    { 0x1BC9F, "DUPLOYAN PUNCTUATION CHINOOK FULL STOP" },
    { 0x1BCA0, "SHORTHAND FORMAT LETTER OVERLAP" },
    { 0x1BCA1, "SHORTHAND FORMAT CONTINUING OVERLAP" },
    { 0x1BCA2, "SHORTHAND FORMAT DOWN STEP" },
    { 0x1BCA3, "SHORTHAND FORMAT UP STEP" },
    { 0x1CF00, "ZNAMENNY COMBINING MARK GORAZDO NIZKO S KRYZHEM ON LEFT" },
    { 0x1CF01, "ZNAMENNY COMBINING MARK NIZKO S KRYZHEM ON LEFT" },
    { 0x1CF02, "ZNAMENNY COMBINING MARK TSATA ON LEFT" },
    { 0x1CF03, "ZNAMENNY COMBINING MARK GORAZDO NIZKO ON LEFT" },
    { 0x1CF04, "ZNAMENNY COMBINING MARK NIZKO ON LEFT" },
    { 0x1CF05, "ZNAMENNY COMBINING MARK SREDNE ON LEFT" },
    { 0x1CF06, "ZNAMENNY COMBINING MARK MALO POVYSHE ON LEFT" },
    { 0x1CF07, "ZNAMENNY COMBINING MARK POVYSHE ON LEFT" },
    { 0x1CF08, "ZNAMENNY COMBINING MARK VYSOKO ON LEFT" },
    { 0x1CF09, "ZNAMENNY COMBINING MARK MALO POVYSHE S KHOKHLOM ON LEFT" },
    { 0x1CF0A, "ZNAMENNY COMBINING MARK POVYSHE S KHOKHLOM ON LEFT" },
    { 0x1CF0B, "ZNAMENNY COMBINING MARK VYSOKO S KHOKHLOM ON LEFT" },
    { 0x1CF0C, "ZNAMENNY COMBINING MARK GORAZDO NIZKO S KRYZHEM ON RIGHT" },
    { 0x1CF0D, "ZNAMENNY COMBINING MARK NIZKO S KRYZHEM ON RIGHT" },
    { 0x1CF0E, "ZNAMENNY COMBINING MARK TSATA ON RIGHT" },
    { 0x1CF0F, "ZNAMENNY COMBINING MARK GORAZDO NIZKO ON RIGHT" },
    { 0x1CF10, "ZNAMENNY COMBINING MARK NIZKO ON RIGHT" },
    { 0x1CF11, "ZNAMENNY COMBINING MARK SREDNE ON RIGHT" },
    { 0x1CF12, "ZNAMENNY COMBINING MARK MALO POVYSHE ON RIGHT" },
    { 0x1CF13, "ZNAMENNY COMBINING MARK POVYSHE ON RIGHT" },
    { 0x1CF14, "ZNAMENNY COMBINING MARK VYSOKO ON RIGHT" },
    { 0x1CF15, "ZNAMENNY COMBINING MARK MALO POVYSHE S KHOKHLOM ON RIGHT" },
    { 0x1CF16, "ZNAMENNY COMBINING MARK POVYSHE S KHOKHLOM ON RIGHT" },
    { 0x1CF17, "ZNAMENNY COMBINING MARK VYSOKO S KHOKHLOM ON RIGHT" },
    { 0x1CF18, "ZNAMENNY COMBINING MARK TSATA S KRYZHEM" },
    { 0x1CF19, "ZNAMENNY COMBINING MARK MALO POVYSHE S KRYZHEM" },
    { 0x1CF1A, "ZNAMENNY COMBINING MARK STRANNO MALO POVYSHE" },
    { 0x1CF1B, "ZNAMENNY COMBINING MARK POVYSHE S KRYZHEM" },
    { 0x1CF1C, "ZNAMENNY COMBINING MARK POVYSHE STRANNO" },
    { 0x1CF1D, "ZNAMENNY COMBINING MARK VYSOKO S KRYZHEM" },
    { 0x1CF1E, "ZNAMENNY COMBINING MARK MALO POVYSHE STRANNO" },
    { 0x1CF1F, "ZNAMENNY COMBINING MARK GORAZDO VYSOKO" },
    { 0x1CF20, "ZNAMENNY COMBINING MARK ZELO" },
    { 0x1CF21, "ZNAMENNY COMBINING MARK ON" },
    { 0x1CF22, "ZNAMENNY COMBINING MARK RAVNO" },
    { 0x1CF23, "ZNAMENNY COMBINING MARK TIKHAYA" },
    { 0x1CF24, "ZNAMENNY COMBINING MARK BORZAYA" },
    { 0x1CF25, "ZNAMENNY COMBINING MARK UDARKA" },
    { 0x1CF26, "ZNAMENNY COMBINING MARK PODVERTKA" },
    { 0x1CF27, "ZNAMENNY COMBINING MARK LOMKA" },
    { 0x1CF28, "ZNAMENNY COMBINING MARK KUPNAYA" },
    { 0x1CF29, "ZNAMENNY COMBINING MARK KACHKA" },
    { 0x1CF2A, "ZNAMENNY COMBINING MARK ZEVOK" },
    { 0x1CF2B, "ZNAMENNY COMBINING MARK SKOBA" },
    { 0x1CF2C, "ZNAMENNY COMBINING MARK RAZSEKA" },
    { 0x1CF2D, "ZNAMENNY COMBINING MARK KRYZH ON LEFT" },
    { 0x1CF30, "ZNAMENNY COMBINING TONAL RANGE MARK MRACHNO" },
    { 0x1CF31, "ZNAMENNY COMBINING TONAL RANGE MARK SVETLO" },
    { 0x1CF32, "ZNAMENNY COMBINING TONAL RANGE MARK TRESVETLO" },
    { 0x1CF33, "ZNAMENNY COMBINING MARK ZADERZHKA" },
    { 0x1CF34, "ZNAMENNY COMBINING MARK DEMESTVENNY ZADERZHKA" },
    { 0x1CF35, "ZNAMENNY COMBINING MARK OTSECHKA" },
    { 0x1CF36, "ZNAMENNY COMBINING MARK PODCHASHIE" },
    { 0x1CF37, "ZNAMENNY COMBINING MARK PODCHASHIE WITH VERTICAL STROKE" },
    { 0x1CF38, "ZNAMENNY COMBINING MARK CHASHKA" },
    { 0x1CF39, "ZNAMENNY COMBINING MARK CHASHKA POLNAYA" },
    { 0x1CF3A, "ZNAMENNY COMBINING MARK OBLACHKO" },
    { 0x1CF3B, "ZNAMENNY COMBINING MARK SOROCHYA NOZHKA" },
    { 0x1CF3C, "ZNAMENNY COMBINING MARK TOCHKA" },
    { 0x1CF3D, "ZNAMENNY COMBINING MARK DVOETOCHIE" },
    { 0x1CF3E, "ZNAMENNY COMBINING ATTACHING VERTICAL OMET" },
    { 0x1CF3F, "ZNAMENNY COMBINING MARK CURVED OMET" },
    { 0x1CF40, "ZNAMENNY COMBINING MARK KRYZH" },
    { 0x1CF41, "ZNAMENNY COMBINING LOWER TONAL RANGE INDICATOR" },
    { 0x1CF42, "ZNAMENNY PRIZNAK MODIFIER LEVEL-2" },
    { 0x1CF43, "ZNAMENNY PRIZNAK MODIFIER LEVEL-3" },
    { 0x1CF44, "ZNAMENNY PRIZNAK MODIFIER DIRECTION FLIP" },
    { 0x1CF45, "ZNAMENNY PRIZNAK MODIFIER KRYZH" },
    { 0x1CF46, "ZNAMENNY PRIZNAK MODIFIER ROG" },
    { 0x1CF50, "ZNAMENNY NEUME KRYUK" },
    { 0x1CF51, "ZNAMENNY NEUME KRYUK TIKHY" },
    { 0x1CF52, "ZNAMENNY NEUME PARAKLIT" },
    { 0x1CF53, "ZNAMENNY NEUME DVA V CHELNU" },
    { 0x1CF54, "ZNAMENNY NEUME KLYUCH" },
    { 0x1CF55, "ZNAMENNY NEUME ZANOZHEK" },
    { 0x1CF56, "ZNAMENNY NEUME STOPITSA" },
    { 0x1CF57, "ZNAMENNY NEUME STOPITSA S OCHKOM" },
    { 0x1CF58, "ZNAMENNY NEUME PEREVODKA" },
    { 0x1CF59, "ZNAMENNY NEUME PEREVODKA NEPOSTOYANNAYA" },
    { 0x1CF5A, "ZNAMENNY NEUME STOPITSA WITH SOROCHYA NOZHKA" },
    { 0x1CF5B, "ZNAMENNY NEUME CHELYUSTKA" },
    { 0x1CF5C, "ZNAMENNY NEUME PALKA" },
    { 0x1CF5D, "ZNAMENNY NEUME ZAPYATAYA" },
    { 0x1CF5E, "ZNAMENNY NEUME GOLUBCHIK BORZY" },
    { 0x1CF5F, "ZNAMENNY NEUME GOLUBCHIK TIKHY" },
    { 0x1CF60, "ZNAMENNY NEUME GOLUBCHIK MRACHNY" },
    { 0x1CF61, "ZNAMENNY NEUME GOLUBCHIK SVETLY" },
    { 0x1CF62, "ZNAMENNY NEUME GOLUBCHIK TRESVETLY" },
    { 0x1CF63, "ZNAMENNY NEUME VRAKHIYA PROSTAYA" },
    { 0x1CF64, "ZNAMENNY NEUME VRAKHIYA MRACHNAYA" },
    { 0x1CF65, "ZNAMENNY NEUME VRAKHIYA SVETLAYA" },
    { 0x1CF66, "ZNAMENNY NEUME VRAKHIYA TRESVETLAYA" },
    { 0x1CF67, "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA PROSTAYA" },
    { 0x1CF68, "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA MRACHNAYA" },
    { 0x1CF69, "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA SVETLAYA" },
    { 0x1CF6A, "ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA TRESVETLAYA" },
    { 0x1CF6B, "ZNAMENNY NEUME DOUBLE ZAPYATAYA" },
    { 0x1CF6C, "ZNAMENNY NEUME REVERSED CHELYUSTKA" },
    { 0x1CF6D, "ZNAMENNY NEUME DERBITSA" },
    { 0x1CF6E, "ZNAMENNY NEUME KHAMILO" },
    { 0x1CF6F, "ZNAMENNY NEUME CHASHKA" },
    { 0x1CF70, "ZNAMENNY NEUME PODCHASHIE" },
    { 0x1CF71, "ZNAMENNY NEUME SKAMEYTSA MRACHNAYA" },
    { 0x1CF72, "ZNAMENNY NEUME SKAMEYTSA SVETLAYA" },
    { 0x1CF73, "ZNAMENNY NEUME SKAMEYTSA TRESVETLAYA" },
    { 0x1CF74, "ZNAMENNY NEUME SKAMEYTSA TIKHAYA" },
    { 0x1CF75, "ZNAMENNY NEUME DEMESTVENNY KLYUCH" },
    { 0x1CF76, "ZNAMENNY NEUME SKAMEYTSA KLYUCHEVAYA SVETLAYA" },
    { 0x1CF77, "ZNAMENNY NEUME SKAMEYTSA KLYUCHENEPOSTOYANNAYA" },
    { 0x1CF78, "ZNAMENNY NEUME SKAMEYTSA KLYUCHEVAYA TIKHAYA" },
    { 0x1CF79, "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA PROSTAYA" },
    { 0x1CF7A, "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA SVETLAYA" },
    { 0x1CF7B, "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA NEPOSTOYANNAYA" },
    { 0x1CF7C, "ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA KLYUCHEVAYA" },
    { 0x1CF7D, "ZNAMENNY NEUME SLOZHITIE" },
    { 0x1CF7E, "ZNAMENNY NEUME SLOZHITIE S ZAPYATOY" },
    { 0x1CF7F, "ZNAMENNY NEUME SLOZHITIE ZAKRYTOE" },
    { 0x1CF80, "ZNAMENNY NEUME SLOZHITIE S KRYZHEM" },
    { 0x1CF81, "ZNAMENNY NEUME KRYZH" },
    { 0x1CF82, "ZNAMENNY NEUME ROG" },
    { 0x1CF83, "ZNAMENNY NEUME FITA" },
    { 0x1CF84, "ZNAMENNY NEUME KOBYLA" },
    { 0x1CF85, "ZNAMENNY NEUME ZMEYTSA" },
    { 0x1CF86, "ZNAMENNY NEUME STATYA" },
    { 0x1CF87, "ZNAMENNY NEUME STATYA S ZAPYATOY" },
    { 0x1CF88, "ZNAMENNY NEUME STATYA S KRYZHEM" },
    { 0x1CF89, "ZNAMENNY NEUME STATYA S ZAPYATOY I KRYZHEM" },
    { 0x1CF8A, "ZNAMENNY NEUME STATYA S KRYZHEM I ZAPYATOY" },
    { 0x1CF8B, "ZNAMENNY NEUME STATYA ZAKRYTAYA" },
    { 0x1CF8C, "ZNAMENNY NEUME STATYA ZAKRYTAYA S ZAPYATOY" },
    { 0x1CF8D, "ZNAMENNY NEUME STATYA S ROGOM" },
    { 0x1CF8E, "ZNAMENNY NEUME STATYA S DVUMYA ZAPYATYMI" },
    { 0x1CF8F, "ZNAMENNY NEUME STATYA S ZAPYATOY I PODCHASHIEM" },
    { 0x1CF90, "ZNAMENNY NEUME POLKULIZMY" },
    { 0x1CF91, "ZNAMENNY NEUME STATYA NEPOSTOYANNAYA" },
    { 0x1CF92, "ZNAMENNY NEUME STRELA PROSTAYA" },
    { 0x1CF93, "ZNAMENNY NEUME STRELA MRACHNOTIKHAYA" },
    { 0x1CF94, "ZNAMENNY NEUME STRELA KRYZHEVAYA" },
    { 0x1CF95, "ZNAMENNY NEUME STRELA POLUPOVODNAYA" },
    { 0x1CF96, "ZNAMENNY NEUME STRELA POVODNAYA" },
    { 0x1CF97, "ZNAMENNY NEUME STRELA NEPOSTOYANNAYA" },
    { 0x1CF98, "ZNAMENNY NEUME STRELA KLYUCHEPOVODNAYA" },
    { 0x1CF99, "ZNAMENNY NEUME STRELA KLYUCHENEPOSTOYANNAYA" },
    { 0x1CF9A, "ZNAMENNY NEUME STRELA TIKHAYA PUTNAYA" },
    { 0x1CF9B, "ZNAMENNY NEUME STRELA DVOECHELNAYA" },
    { 0x1CF9C, "ZNAMENNY NEUME STRELA DVOECHELNOKRYZHEVAYA" },
    { 0x1CF9D, "ZNAMENNY NEUME STRELA DVOECHELNOPOVODNAYA" },
    { 0x1CF9E, "ZNAMENNY NEUME STRELA DVOECHELNAYA KLYUCHEVAYA" },
    { 0x1CF9F, "ZNAMENNY NEUME STRELA DVOECHELNOPOVODNAYA KLYUCHEVAYA" },
    { 0x1CFA0, "ZNAMENNY NEUME STRELA GROMNAYA WITH SINGLE ZAPYATAYA" },
    { 0x1CFA1, "ZNAMENNY NEUME STRELA GROMOPOVODNAYA WITH SINGLE ZAPYATAYA" },
    { 0x1CFA2, "ZNAMENNY NEUME STRELA GROMNAYA" },
    { 0x1CFA3, "ZNAMENNY NEUME STRELA GROMOPOVODNAYA" },
    { 0x1CFA4, "ZNAMENNY NEUME STRELA GROMOPOVODNAYA WITH DOUBLE ZAPYATAYA" },
    { 0x1CFA5, "ZNAMENNY NEUME STRELA GROMOKRYZHEVAYA" },
    { 0x1CFA6, "ZNAMENNY NEUME STRELA GROMOKRYZHEVAYA POVODNAYA" },
    { 0x1CFA7, "ZNAMENNY NEUME MECHIK" },
    { 0x1CFA8, "ZNAMENNY NEUME MECHIK POVODNY" },
    { 0x1CFA9, "ZNAMENNY NEUME MECHIK KLYUCHEVOY" },
    { 0x1CFAA, "ZNAMENNY NEUME MECHIK KLYUCHEPOVODNY" },
    { 0x1CFAB, "ZNAMENNY NEUME MECHIK KLYUCHENEPOSTOYANNY" },
    { 0x1CFAC, "ZNAMENNY NEUME STRELA TRYASOGLASNAYA" },
    { 0x1CFAD, "ZNAMENNY NEUME STRELA TRYASOPOVODNAYA" },
    { 0x1CFAE, "ZNAMENNY NEUME STRELA TRYASOSTRELNAYA" },
    { 0x1CFAF, "ZNAMENNY NEUME OSOKA" },
    { 0x1CFB0, "ZNAMENNY NEUME OSOKA SVETLAYA" },
    { 0x1CFB1, "ZNAMENNY NEUME OSOKA TRESVETLAYA" },
    { 0x1CFB2, "ZNAMENNY NEUME OSOKA KRYUKOVAYA SVETLAYA" },
    { 0x1CFB3, "ZNAMENNY NEUME OSOKA KLYUCHEVAYA SVETLAYA" },
    { 0x1CFB4, "ZNAMENNY NEUME OSOKA KLYUCHEVAYA NEPOSTOYANNAYA" },
    { 0x1CFB5, "ZNAMENNY NEUME STRELA KRYUKOVAYA" },
    { 0x1CFB6, "ZNAMENNY NEUME STRELA KRYUKOVAYA POVODNAYA" },
    { 0x1CFB7, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMNAYA WITH SINGLE ZAPYATAYA" },
    { 0x1CFB8, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA WITH SINGLE ZAPYATAYA" },
    { 0x1CFB9, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMNAYA" },
    { 0x1CFBA, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA" },
    { 0x1CFBB, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA WITH DOUBLE ZAPYATAYA" },
    { 0x1CFBC, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOKRYZHEVAYA" },
    { 0x1CFBD, "ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOKRYZHEVAYA POVODNAYA" },
    { 0x1CFBE, "ZNAMENNY NEUME STRELA KRYUKOVAYA TRYASKA" },
    { 0x1CFBF, "ZNAMENNY NEUME KUFISMA" },
    { 0x1CFC0, "ZNAMENNY NEUME OBLAKO" },
    { 0x1CFC1, "ZNAMENNY NEUME DUDA" },
    { 0x1CFC2, "ZNAMENNY NEUME NEMKA" },
    { 0x1CFC3, "ZNAMENNY NEUME PAUK" },
    { 0x1D000, "BYZANTINE MUSICAL SYMBOL PSILI" },
    { 0x1D001, "BYZANTINE MUSICAL SYMBOL DASEIA" },
    { 0x1D002, "BYZANTINE MUSICAL SYMBOL PERISPOMENI" },
    { 0x1D003, "BYZANTINE MUSICAL SYMBOL OXEIA EKFONITIKON" },
    { 0x1D004, "BYZANTINE MUSICAL SYMBOL OXEIA DIPLI" },
    { 0x1D005, "BYZANTINE MUSICAL SYMBOL VAREIA EKFONITIKON" },
    { 0x1D006, "BYZANTINE MUSICAL SYMBOL VAREIA DIPLI" },
    { 0x1D007, "BYZANTINE MUSICAL SYMBOL KATHISTI" },
    { 0x1D008, "BYZANTINE MUSICAL SYMBOL SYRMATIKI" },
    { 0x1D009, "BYZANTINE MUSICAL SYMBOL PARAKLITIKI" },
    { 0x1D00A, "BYZANTINE MUSICAL SYMBOL YPOKRISIS" },
    { 0x1D00B, "BYZANTINE MUSICAL SYMBOL YPOKRISIS DIPLI" },
    { 0x1D00C, "BYZANTINE MUSICAL SYMBOL KREMASTI" },
    { 0x1D00D, "BYZANTINE MUSICAL SYMBOL APESO EKFONITIKON" },
    { 0x1D00E, "BYZANTINE MUSICAL SYMBOL EXO EKFONITIKON" },
    { 0x1D00F, "BYZANTINE MUSICAL SYMBOL TELEIA" },
    { 0x1D010, "BYZANTINE MUSICAL SYMBOL KENTIMATA" },
    { 0x1D011, "BYZANTINE MUSICAL SYMBOL APOSTROFOS" },
    { 0x1D012, "BYZANTINE MUSICAL SYMBOL APOSTROFOS DIPLI" },
    { 0x1D013, "BYZANTINE MUSICAL SYMBOL SYNEVMA" },
    { 0x1D014, "BYZANTINE MUSICAL SYMBOL THITA" },
    { 0x1D015, "BYZANTINE MUSICAL SYMBOL OLIGON ARCHAION" },
    { 0x1D016, "BYZANTINE MUSICAL SYMBOL GORGON ARCHAION" },
    { 0x1D017, "BYZANTINE MUSICAL SYMBOL PSILON" },
    { 0x1D018, "BYZANTINE MUSICAL SYMBOL CHAMILON" },
    { 0x1D019, "BYZANTINE MUSICAL SYMBOL VATHY" },
    { 0x1D01A, "BYZANTINE MUSICAL SYMBOL ISON ARCHAION" },
    { 0x1D01B, "BYZANTINE MUSICAL SYMBOL KENTIMA ARCHAION" },
    { 0x1D01C, "BYZANTINE MUSICAL SYMBOL KENTIMATA ARCHAION" },
    { 0x1D01D, "BYZANTINE MUSICAL SYMBOL SAXIMATA" },
    { 0x1D01E, "BYZANTINE MUSICAL SYMBOL PARICHON" },
    { 0x1D01F, "BYZANTINE MUSICAL SYMBOL STAVROS APODEXIA" },
    { 0x1D020, "BYZANTINE MUSICAL SYMBOL OXEIAI ARCHAION" },
    { 0x1D021, "BYZANTINE MUSICAL SYMBOL VAREIAI ARCHAION" },
    { 0x1D022, "BYZANTINE MUSICAL SYMBOL APODERMA ARCHAION" },
    { 0x1D023, "BYZANTINE MUSICAL SYMBOL APOTHEMA" },
    { 0x1D024, "BYZANTINE MUSICAL SYMBOL KLASMA" },
    { 0x1D025, "BYZANTINE MUSICAL SYMBOL REVMA" },
    { 0x1D026, "BYZANTINE MUSICAL SYMBOL PIASMA ARCHAION" },
    { 0x1D027, "BYZANTINE MUSICAL SYMBOL TINAGMA" },
    { 0x1D028, "BYZANTINE MUSICAL SYMBOL ANATRICHISMA" },
    { 0x1D029, "BYZANTINE MUSICAL SYMBOL SEISMA" },
    { 0x1D02A, "BYZANTINE MUSICAL SYMBOL SYNAGMA ARCHAION" },
    { 0x1D02B, "BYZANTINE MUSICAL SYMBOL SYNAGMA META STAVROU" },
    { 0x1D02C, "BYZANTINE MUSICAL SYMBOL OYRANISMA ARCHAION" },
    { 0x1D02D, "BYZANTINE MUSICAL SYMBOL THEMA" },
    { 0x1D02E, "BYZANTINE MUSICAL SYMBOL LEMOI" },
    { 0x1D02F, "BYZANTINE MUSICAL SYMBOL DYO" },
    { 0x1D030, "BYZANTINE MUSICAL SYMBOL TRIA" },
    { 0x1D031, "BYZANTINE MUSICAL SYMBOL TESSERA" },
    { 0x1D032, "BYZANTINE MUSICAL SYMBOL KRATIMATA" },
    { 0x1D033, "BYZANTINE MUSICAL SYMBOL APESO EXO NEO" },
    { 0x1D034, "BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION" },
    { 0x1D035, "BYZANTINE MUSICAL SYMBOL IMIFTHORA" },
    { 0x1D036, "BYZANTINE MUSICAL SYMBOL TROMIKON ARCHAION" },
    { 0x1D037, "BYZANTINE MUSICAL SYMBOL KATAVA TROMIKON" },
    { 0x1D038, "BYZANTINE MUSICAL SYMBOL PELASTON" },
    { 0x1D039, "BYZANTINE MUSICAL SYMBOL PSIFISTON" },
    { 0x1D03A, "BYZANTINE MUSICAL SYMBOL KONTEVMA" },
    { 0x1D03B, "BYZANTINE MUSICAL SYMBOL CHOREVMA ARCHAION" },
    { 0x1D03C, "BYZANTINE MUSICAL SYMBOL RAPISMA" },
    { 0x1D03D, "BYZANTINE MUSICAL SYMBOL PARAKALESMA ARCHAION" },
    { 0x1D03E, "BYZANTINE MUSICAL SYMBOL PARAKLITIKI ARCHAION" },
    { 0x1D03F, "BYZANTINE MUSICAL SYMBOL ICHADIN" },
    { 0x1D040, "BYZANTINE MUSICAL SYMBOL NANA" },
    { 0x1D041, "BYZANTINE MUSICAL SYMBOL PETASMA" },
    { 0x1D042, "BYZANTINE MUSICAL SYMBOL KONTEVMA ALLO" },
    { 0x1D043, "BYZANTINE MUSICAL SYMBOL TROMIKON ALLO" },
    { 0x1D044, "BYZANTINE MUSICAL SYMBOL STRAGGISMATA" },
    { 0x1D045, "BYZANTINE MUSICAL SYMBOL GRONTHISMATA" },
    { 0x1D046, "BYZANTINE MUSICAL SYMBOL ISON NEO" },
    { 0x1D047, "BYZANTINE MUSICAL SYMBOL OLIGON NEO" },
    { 0x1D048, "BYZANTINE MUSICAL SYMBOL OXEIA NEO" },
    { 0x1D049, "BYZANTINE MUSICAL SYMBOL PETASTI" },
    { 0x1D04A, "BYZANTINE MUSICAL SYMBOL KOUFISMA" },
    { 0x1D04B, "BYZANTINE MUSICAL SYMBOL PETASTOKOUFISMA" },
    { 0x1D04C, "BYZANTINE MUSICAL SYMBOL KRATIMOKOUFISMA" },
    { 0x1D04D, "BYZANTINE MUSICAL SYMBOL PELASTON NEO" },
    { 0x1D04E, "BYZANTINE MUSICAL SYMBOL KENTIMATA NEO ANO" },
    { 0x1D04F, "BYZANTINE MUSICAL SYMBOL KENTIMA NEO ANO" },
    { 0x1D050, "BYZANTINE MUSICAL SYMBOL YPSILI" },
    { 0x1D051, "BYZANTINE MUSICAL SYMBOL APOSTROFOS NEO" },
    { 0x1D052, "BYZANTINE MUSICAL SYMBOL APOSTROFOI SYNDESMOS NEO" },
    { 0x1D053, "BYZANTINE MUSICAL SYMBOL YPORROI" },
    { 0x1D054, "BYZANTINE MUSICAL SYMBOL KRATIMOYPORROON" },
    { 0x1D055, "BYZANTINE MUSICAL SYMBOL ELAFRON" },
    { 0x1D056, "BYZANTINE MUSICAL SYMBOL CHAMILI" },
    { 0x1D057, "BYZANTINE MUSICAL SYMBOL MIKRON ISON" },
    { 0x1D058, "BYZANTINE MUSICAL SYMBOL VAREIA NEO" },
    { 0x1D059, "BYZANTINE MUSICAL SYMBOL PIASMA NEO" },
    { 0x1D05A, "BYZANTINE MUSICAL SYMBOL PSIFISTON NEO" },
    { 0x1D05B, "BYZANTINE MUSICAL SYMBOL OMALON" },
    { 0x1D05C, "BYZANTINE MUSICAL SYMBOL ANTIKENOMA" },
    { 0x1D05D, "BYZANTINE MUSICAL SYMBOL LYGISMA" },
    { 0x1D05E, "BYZANTINE MUSICAL SYMBOL PARAKLITIKI NEO" },
    { 0x1D05F, "BYZANTINE MUSICAL SYMBOL PARAKALESMA NEO" },
    { 0x1D060, "BYZANTINE MUSICAL SYMBOL ETERON PARAKALESMA" },
    { 0x1D061, "BYZANTINE MUSICAL SYMBOL KYLISMA" },
    { 0x1D062, "BYZANTINE MUSICAL SYMBOL ANTIKENOKYLISMA" },
    { 0x1D063, "BYZANTINE MUSICAL SYMBOL TROMIKON NEO" },
    { 0x1D064, "BYZANTINE MUSICAL SYMBOL EKSTREPTON" },
    { 0x1D065, "BYZANTINE MUSICAL SYMBOL SYNAGMA NEO" },
    { 0x1D066, "BYZANTINE MUSICAL SYMBOL SYRMA" },
    { 0x1D067, "BYZANTINE MUSICAL SYMBOL CHOREVMA NEO" },
    { 0x1D068, "BYZANTINE MUSICAL SYMBOL EPEGERMA" },
    { 0x1D069, "BYZANTINE MUSICAL SYMBOL SEISMA NEO" },
    { 0x1D06A, "BYZANTINE MUSICAL SYMBOL XIRON KLASMA" },
    { 0x1D06B, "BYZANTINE MUSICAL SYMBOL TROMIKOPSIFISTON" },
    { 0x1D06C, "BYZANTINE MUSICAL SYMBOL PSIFISTOLYGISMA" },
    { 0x1D06D, "BYZANTINE MUSICAL SYMBOL TROMIKOLYGISMA" },
    { 0x1D06E, "BYZANTINE MUSICAL SYMBOL TROMIKOPARAKALESMA" },
    { 0x1D06F, "BYZANTINE MUSICAL SYMBOL PSIFISTOPARAKALESMA" },
    { 0x1D070, "BYZANTINE MUSICAL SYMBOL TROMIKOSYNAGMA" },
    { 0x1D071, "BYZANTINE MUSICAL SYMBOL PSIFISTOSYNAGMA" },
    { 0x1D072, "BYZANTINE MUSICAL SYMBOL GORGOSYNTHETON" },
    { 0x1D073, "BYZANTINE MUSICAL SYMBOL ARGOSYNTHETON" },
    { 0x1D074, "BYZANTINE MUSICAL SYMBOL ETERON ARGOSYNTHETON" },
    { 0x1D075, "BYZANTINE MUSICAL SYMBOL OYRANISMA NEO" },
    { 0x1D076, "BYZANTINE MUSICAL SYMBOL THEMATISMOS ESO" },
    { 0x1D077, "BYZANTINE MUSICAL SYMBOL THEMATISMOS EXO" },
    { 0x1D078, "BYZANTINE MUSICAL SYMBOL THEMA APLOUN" },
    { 0x1D079, "BYZANTINE MUSICAL SYMBOL THES KAI APOTHES" },
    { 0x1D07A, "BYZANTINE MUSICAL SYMBOL KATAVASMA" },
    { 0x1D07B, "BYZANTINE MUSICAL SYMBOL ENDOFONON" },
    { 0x1D07C, "BYZANTINE MUSICAL SYMBOL YFEN KATO" },
    { 0x1D07D, "BYZANTINE MUSICAL SYMBOL YFEN ANO" },
    { 0x1D07E, "BYZANTINE MUSICAL SYMBOL STAVROS" },
    { 0x1D07F, "BYZANTINE MUSICAL SYMBOL KLASMA ANO" },
    { 0x1D080, "BYZANTINE MUSICAL SYMBOL DIPLI ARCHAION" },
    { 0x1D081, "BYZANTINE MUSICAL SYMBOL KRATIMA ARCHAION" },
    { 0x1D082, "BYZANTINE MUSICAL SYMBOL KRATIMA ALLO" },
    { 0x1D083, "BYZANTINE MUSICAL SYMBOL KRATIMA NEO" },
    { 0x1D084, "BYZANTINE MUSICAL SYMBOL APODERMA NEO" },
    { 0x1D085, "BYZANTINE MUSICAL SYMBOL APLI" },
    { 0x1D086, "BYZANTINE MUSICAL SYMBOL DIPLI" },
    { 0x1D087, "BYZANTINE MUSICAL SYMBOL TRIPLI" },
    { 0x1D088, "BYZANTINE MUSICAL SYMBOL TETRAPLI" },
    { 0x1D089, "BYZANTINE MUSICAL SYMBOL KORONIS" },
    { 0x1D08A, "BYZANTINE MUSICAL SYMBOL LEIMMA ENOS CHRONOU" },
    { 0x1D08B, "BYZANTINE MUSICAL SYMBOL LEIMMA DYO CHRONON" },
    { 0x1D08C, "BYZANTINE MUSICAL SYMBOL LEIMMA TRION CHRONON" },
    { 0x1D08D, "BYZANTINE MUSICAL SYMBOL LEIMMA TESSARON CHRONON" },
    { 0x1D08E, "BYZANTINE MUSICAL SYMBOL LEIMMA IMISEOS CHRONOU" },
    { 0x1D08F, "BYZANTINE MUSICAL SYMBOL GORGON NEO ANO" },
    { 0x1D090, "BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON ARISTERA" },
    { 0x1D091, "BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON DEXIA" },
    { 0x1D092, "BYZANTINE MUSICAL SYMBOL DIGORGON" },
    { 0x1D093, "BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA KATO" },
    { 0x1D094, "BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA ANO" },
    { 0x1D095, "BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON DEXIA" },
    { 0x1D096, "BYZANTINE MUSICAL SYMBOL TRIGORGON" },
    { 0x1D097, "BYZANTINE MUSICAL SYMBOL ARGON" },
    { 0x1D098, "BYZANTINE MUSICAL SYMBOL IMIDIARGON" },
    { 0x1D099, "BYZANTINE MUSICAL SYMBOL DIARGON" },
    { 0x1D09A, "BYZANTINE MUSICAL SYMBOL AGOGI POLI ARGI" },
    { 0x1D09B, "BYZANTINE MUSICAL SYMBOL AGOGI ARGOTERI" },
    { 0x1D09C, "BYZANTINE MUSICAL SYMBOL AGOGI ARGI" },
    { 0x1D09D, "BYZANTINE MUSICAL SYMBOL AGOGI METRIA" },
    { 0x1D09E, "BYZANTINE MUSICAL SYMBOL AGOGI MESI" },
    { 0x1D09F, "BYZANTINE MUSICAL SYMBOL AGOGI GORGI" },
    { 0x1D0A0, "BYZANTINE MUSICAL SYMBOL AGOGI GORGOTERI" },
    { 0x1D0A1, "BYZANTINE MUSICAL SYMBOL AGOGI POLI GORGI" },
    { 0x1D0A2, "BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOS ICHOS" },
    { 0x1D0A3, "BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI PROTOS ICHOS" },
    { 0x1D0A4, "BYZANTINE MUSICAL SYMBOL MARTYRIA DEYTEROS ICHOS" },
    { 0x1D0A5, "BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI DEYTEROS ICHOS" },
    { 0x1D0A6, "BYZANTINE MUSICAL SYMBOL MARTYRIA TRITOS ICHOS" },
    { 0x1D0A7, "BYZANTINE MUSICAL SYMBOL MARTYRIA TRIFONIAS" },
    { 0x1D0A8, "BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS ICHOS" },
    { 0x1D0A9, "BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS LEGETOS ICHOS" },
    { 0x1D0AA, "BYZANTINE MUSICAL SYMBOL MARTYRIA LEGETOS ICHOS" },
    { 0x1D0AB, "BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS ICHOS" },
    { 0x1D0AC, "BYZANTINE MUSICAL SYMBOL ISAKIA TELOUS ICHIMATOS" },
    { 0x1D0AD, "BYZANTINE MUSICAL SYMBOL APOSTROFOI TELOUS ICHIMATOS" },
    { 0x1D0AE, "BYZANTINE MUSICAL SYMBOL FANEROSIS TETRAFONIAS" },
    { 0x1D0AF, "BYZANTINE MUSICAL SYMBOL FANEROSIS MONOFONIAS" },
    { 0x1D0B0, "BYZANTINE MUSICAL SYMBOL FANEROSIS DIFONIAS" },
    { 0x1D0B1, "BYZANTINE MUSICAL SYMBOL MARTYRIA VARYS ICHOS" },
    { 0x1D0B2, "BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOVARYS ICHOS" },
    { 0x1D0B3, "BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS TETARTOS ICHOS" },
    { 0x1D0B4, "BYZANTINE MUSICAL SYMBOL GORTHMIKON N APLOUN" },
    { 0x1D0B5, "BYZANTINE MUSICAL SYMBOL GORTHMIKON N DIPLOUN" },
    { 0x1D0B6, "BYZANTINE MUSICAL SYMBOL ENARXIS KAI FTHORA VOU" },
    { 0x1D0B7, "BYZANTINE MUSICAL SYMBOL IMIFONON" },
    { 0x1D0B8, "BYZANTINE MUSICAL SYMBOL IMIFTHORON" },
    { 0x1D0B9, "BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION DEYTEROU ICHOU" },
    { 0x1D0BA, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI PA" },
    { 0x1D0BB, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NANA" },
    { 0x1D0BC, "BYZANTINE MUSICAL SYMBOL FTHORA NAOS ICHOS" },
    { 0x1D0BD, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI DI" },
    { 0x1D0BE, "BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON DIATONON DI" },
    { 0x1D0BF, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI KE" },
    { 0x1D0C0, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI ZO" },
    { 0x1D0C1, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI KATO" },
    { 0x1D0C2, "BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI ANO" },
    { 0x1D0C3, "BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA DIFONIAS" },
    { 0x1D0C4, "BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA MONOFONIAS" },
    { 0x1D0C5, "BYZANTINE MUSICAL SYMBOL FHTORA SKLIRON CHROMA VASIS" },
    { 0x1D0C6, "BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON CHROMA SYNAFI" },
    { 0x1D0C7, "BYZANTINE MUSICAL SYMBOL FTHORA NENANO" },
    { 0x1D0C8, "BYZANTINE MUSICAL SYMBOL CHROA ZYGOS" },
    { 0x1D0C9, "BYZANTINE MUSICAL SYMBOL CHROA KLITON" },
    { 0x1D0CA, "BYZANTINE MUSICAL SYMBOL CHROA SPATHI" },
    { 0x1D0CB, "BYZANTINE MUSICAL SYMBOL FTHORA I YFESIS TETARTIMORION" },
    { 0x1D0CC, "BYZANTINE MUSICAL SYMBOL FTHORA ENARMONIOS ANTIFONIA" },
    { 0x1D0CD, "BYZANTINE MUSICAL SYMBOL YFESIS TRITIMORION" },
    { 0x1D0CE, "BYZANTINE MUSICAL SYMBOL DIESIS TRITIMORION" },
    { 0x1D0CF, "BYZANTINE MUSICAL SYMBOL DIESIS TETARTIMORION" },
    { 0x1D0D0, "BYZANTINE MUSICAL SYMBOL DIESIS APLI DYO DODEKATA" },
    { 0x1D0D1, "BYZANTINE MUSICAL SYMBOL DIESIS MONOGRAMMOS TESSERA DODEKATA" },
    { 0x1D0D2, "BYZANTINE MUSICAL SYMBOL DIESIS DIGRAMMOS EX DODEKATA" },
    { 0x1D0D3, "BYZANTINE MUSICAL SYMBOL DIESIS TRIGRAMMOS OKTO DODEKATA" },
    { 0x1D0D4, "BYZANTINE MUSICAL SYMBOL YFESIS APLI DYO DODEKATA" },
    { 0x1D0D5, "BYZANTINE MUSICAL SYMBOL YFESIS MONOGRAMMOS TESSERA DODEKATA" },
    { 0x1D0D6, "BYZANTINE MUSICAL SYMBOL YFESIS DIGRAMMOS EX DODEKATA" },
    { 0x1D0D7, "BYZANTINE MUSICAL SYMBOL YFESIS TRIGRAMMOS OKTO DODEKATA" },
    { 0x1D0D8, "BYZANTINE MUSICAL SYMBOL GENIKI DIESIS" },
    { 0x1D0D9, "BYZANTINE MUSICAL SYMBOL GENIKI YFESIS" },
    { 0x1D0DA, "BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MIKRI" },
    { 0x1D0DB, "BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MEGALI" },
    { 0x1D0DC, "BYZANTINE MUSICAL SYMBOL DIASTOLI DIPLI" },
    { 0x1D0DD, "BYZANTINE MUSICAL SYMBOL DIASTOLI THESEOS" },
    { 0x1D0DE, "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS" },
    { 0x1D0DF, "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS DISIMOU" },
    { 0x1D0E0, "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TRISIMOU" },
    { 0x1D0E1, "BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TETRASIMOU" },
    { 0x1D0E2, "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS" },
    { 0x1D0E3, "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS DISIMOU" },
    { 0x1D0E4, "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TRISIMOU" },
    { 0x1D0E5, "BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TETRASIMOU" },
    { 0x1D0E6, "BYZANTINE MUSICAL SYMBOL DIGRAMMA GG" },
    { 0x1D0E7, "BYZANTINE MUSICAL SYMBOL DIFTOGGOS OU" },
    { 0x1D0E8, "BYZANTINE MUSICAL SYMBOL STIGMA" },
    { 0x1D0E9, "BYZANTINE MUSICAL SYMBOL ARKTIKO PA" },
    { 0x1D0EA, "BYZANTINE MUSICAL SYMBOL ARKTIKO VOU" },
    { 0x1D0EB, "BYZANTINE MUSICAL SYMBOL ARKTIKO GA" },
    { 0x1D0EC, "BYZANTINE MUSICAL SYMBOL ARKTIKO DI" },
    { 0x1D0ED, "BYZANTINE MUSICAL SYMBOL ARKTIKO KE" },
    { 0x1D0EE, "BYZANTINE MUSICAL SYMBOL ARKTIKO ZO" },
    { 0x1D0EF, "BYZANTINE MUSICAL SYMBOL ARKTIKO NI" },
    { 0x1D0F0, "BYZANTINE MUSICAL SYMBOL KENTIMATA NEO MESO" },
    { 0x1D0F1, "BYZANTINE MUSICAL SYMBOL KENTIMA NEO MESO" },
    { 0x1D0F2, "BYZANTINE MUSICAL SYMBOL KENTIMATA NEO KATO" },
    { 0x1D0F3, "BYZANTINE MUSICAL SYMBOL KENTIMA NEO KATO" },
    { 0x1D0F4, "BYZANTINE MUSICAL SYMBOL KLASMA KATO" },
    { 0x1D0F5, "BYZANTINE MUSICAL SYMBOL GORGON NEO KATO" },
    { 0x1D100, "MUSICAL SYMBOL SINGLE BARLINE" },
    { 0x1D101, "MUSICAL SYMBOL DOUBLE BARLINE" },
    { 0x1D102, "MUSICAL SYMBOL FINAL BARLINE" },
    { 0x1D103, "MUSICAL SYMBOL REVERSE FINAL BARLINE" },
    { 0x1D104, "MUSICAL SYMBOL DASHED BARLINE" },
    { 0x1D105, "MUSICAL SYMBOL SHORT BARLINE" },
    { 0x1D106, "MUSICAL SYMBOL LEFT REPEAT SIGN" },
    { 0x1D107, "MUSICAL SYMBOL RIGHT REPEAT SIGN" },
    { 0x1D108, "MUSICAL SYMBOL REPEAT DOTS" },
    { 0x1D109, "MUSICAL SYMBOL DAL SEGNO" },
    { 0x1D10A, "MUSICAL SYMBOL DA CAPO" },
    { 0x1D10B, "MUSICAL SYMBOL SEGNO" },
    { 0x1D10C, "MUSICAL SYMBOL CODA" },
    { 0x1D10D, "MUSICAL SYMBOL REPEATED FIGURE-1" },
    { 0x1D10E, "MUSICAL SYMBOL REPEATED FIGURE-2" },
    { 0x1D10F, "MUSICAL SYMBOL REPEATED FIGURE-3" },
    { 0x1D110, "MUSICAL SYMBOL FERMATA" },
    { 0x1D111, "MUSICAL SYMBOL FERMATA BELOW" },
    { 0x1D112, "MUSICAL SYMBOL BREATH MARK" },
    { 0x1D113, "MUSICAL SYMBOL CAESURA" },
    { 0x1D114, "MUSICAL SYMBOL BRACE" },
    { 0x1D115, "MUSICAL SYMBOL BRACKET" },
    { 0x1D116, "MUSICAL SYMBOL ONE-LINE STAFF" },
    { 0x1D117, "MUSICAL SYMBOL TWO-LINE STAFF" },
    { 0x1D118, "MUSICAL SYMBOL THREE-LINE STAFF" },
    { 0x1D119, "MUSICAL SYMBOL FOUR-LINE STAFF" },
    { 0x1D11A, "MUSICAL SYMBOL FIVE-LINE STAFF" },
    { 0x1D11B, "MUSICAL SYMBOL SIX-LINE STAFF" },
    { 0x1D11C, "MUSICAL SYMBOL SIX-STRING FRETBOARD" },
    { 0x1D11D, "MUSICAL SYMBOL FOUR-STRING FRETBOARD" },
    { 0x1D11E, "MUSICAL SYMBOL G CLEF" },
    { 0x1D11F, "MUSICAL SYMBOL G CLEF OTTAVA ALTA" },
    { 0x1D120, "MUSICAL SYMBOL G CLEF OTTAVA BASSA" },
    { 0x1D121, "MUSICAL SYMBOL C CLEF" },
    { 0x1D122, "MUSICAL SYMBOL F CLEF" },
    { 0x1D123, "MUSICAL SYMBOL F CLEF OTTAVA ALTA" },
    { 0x1D124, "MUSICAL SYMBOL F CLEF OTTAVA BASSA" },
    { 0x1D125, "MUSICAL SYMBOL DRUM CLEF-1" },
    { 0x1D126, "MUSICAL SYMBOL DRUM CLEF-2" },
    { 0x1D129, "MUSICAL SYMBOL MULTIPLE MEASURE REST" },
    { 0x1D12A, "MUSICAL SYMBOL DOUBLE SHARP" },
    { 0x1D12B, "MUSICAL SYMBOL DOUBLE FLAT" },
    { 0x1D12C, "MUSICAL SYMBOL FLAT UP" },
    { 0x1D12D, "MUSICAL SYMBOL FLAT DOWN" },
    { 0x1D12E, "MUSICAL SYMBOL NATURAL UP" },
    { 0x1D12F, "MUSICAL SYMBOL NATURAL DOWN" },
    { 0x1D130, "MUSICAL SYMBOL SHARP UP" },
    { 0x1D131, "MUSICAL SYMBOL SHARP DOWN" },
    { 0x1D132, "MUSICAL SYMBOL QUARTER TONE SHARP" },
    { 0x1D133, "MUSICAL SYMBOL QUARTER TONE FLAT" },
    { 0x1D134, "MUSICAL SYMBOL COMMON TIME" },
    { 0x1D135, "MUSICAL SYMBOL CUT TIME" },
    { 0x1D136, "MUSICAL SYMBOL OTTAVA ALTA" },
    { 0x1D137, "MUSICAL SYMBOL OTTAVA BASSA" },
    { 0x1D138, "MUSICAL SYMBOL QUINDICESIMA ALTA" },
    { 0x1D139, "MUSICAL SYMBOL QUINDICESIMA BASSA" },
    { 0x1D13A, "MUSICAL SYMBOL MULTI REST" },
    { 0x1D13B, "MUSICAL SYMBOL WHOLE REST" },
    { 0x1D13C, "MUSICAL SYMBOL HALF REST" },
    { 0x1D13D, "MUSICAL SYMBOL QUARTER REST" },
    { 0x1D13E, "MUSICAL SYMBOL EIGHTH REST" },
    { 0x1D13F, "MUSICAL SYMBOL SIXTEENTH REST" },
    { 0x1D140, "MUSICAL SYMBOL THIRTY-SECOND REST" },
    { 0x1D141, "MUSICAL SYMBOL SIXTY-FOURTH REST" },
    { 0x1D142, "MUSICAL SYMBOL ONE HUNDRED TWENTY-EIGHTH REST" },
    { 0x1D143, "MUSICAL SYMBOL X NOTEHEAD" },
    { 0x1D144, "MUSICAL SYMBOL PLUS NOTEHEAD" },
    { 0x1D145, "MUSICAL SYMBOL CIRCLE X NOTEHEAD" },
    { 0x1D146, "MUSICAL SYMBOL SQUARE NOTEHEAD WHITE" },
    { 0x1D147, "MUSICAL SYMBOL SQUARE NOTEHEAD BLACK" },
    { 0x1D148, "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP WHITE" },
    { 0x1D149, "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP BLACK" },
    { 0x1D14A, "MUSICAL SYMBOL TRIANGLE NOTEHEAD LEFT WHITE" },
    { 0x1D14B, "MUSICAL SYMBOL TRIANGLE NOTEHEAD LEFT BLACK" },
    { 0x1D14C, "MUSICAL SYMBOL TRIANGLE NOTEHEAD RIGHT WHITE" },
    { 0x1D14D, "MUSICAL SYMBOL TRIANGLE NOTEHEAD RIGHT BLACK" },
    { 0x1D14E, "MUSICAL SYMBOL TRIANGLE NOTEHEAD DOWN WHITE" },
    { 0x1D14F, "MUSICAL SYMBOL TRIANGLE NOTEHEAD DOWN BLACK" },
    { 0x1D150, "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP RIGHT WHITE" },
    { 0x1D151, "MUSICAL SYMBOL TRIANGLE NOTEHEAD UP RIGHT BLACK" },
    { 0x1D152, "MUSICAL SYMBOL MOON NOTEHEAD WHITE" },
    { 0x1D153, "MUSICAL SYMBOL MOON NOTEHEAD BLACK" },
    { 0x1D154, "MUSICAL SYMBOL TRIANGLE-ROUND NOTEHEAD DOWN WHITE" },
    { 0x1D155, "MUSICAL SYMBOL TRIANGLE-ROUND NOTEHEAD DOWN BLACK" },
    { 0x1D156, "MUSICAL SYMBOL PARENTHESIS NOTEHEAD" },
    { 0x1D157, "MUSICAL SYMBOL VOID NOTEHEAD" },
    { 0x1D158, "MUSICAL SYMBOL NOTEHEAD BLACK" },
    { 0x1D159, "MUSICAL SYMBOL NULL NOTEHEAD" },
    { 0x1D15A, "MUSICAL SYMBOL CLUSTER NOTEHEAD WHITE" },
    { 0x1D15B, "MUSICAL SYMBOL CLUSTER NOTEHEAD BLACK" },
    { 0x1D15C, "MUSICAL SYMBOL BREVE" },
    { 0x1D15D, "MUSICAL SYMBOL WHOLE NOTE" },
    { 0x1D15E, "MUSICAL SYMBOL HALF NOTE" },
    { 0x1D15F, "MUSICAL SYMBOL QUARTER NOTE" },
    { 0x1D160, "MUSICAL SYMBOL EIGHTH NOTE" },
    { 0x1D161, "MUSICAL SYMBOL SIXTEENTH NOTE" },
    { 0x1D162, "MUSICAL SYMBOL THIRTY-SECOND NOTE" },
    { 0x1D163, "MUSICAL SYMBOL SIXTY-FOURTH NOTE" },
    { 0x1D164, "MUSICAL SYMBOL ONE HUNDRED TWENTY-EIGHTH NOTE" },
    { 0x1D165, "MUSICAL SYMBOL COMBINING STEM" },
    { 0x1D166, "MUSICAL SYMBOL COMBINING SPRECHGESANG STEM" },
    { 0x1D167, "MUSICAL SYMBOL COMBINING TREMOLO-1" },
    { 0x1D168, "MUSICAL SYMBOL COMBINING TREMOLO-2" },
    { 0x1D169, "MUSICAL SYMBOL COMBINING TREMOLO-3" },
    { 0x1D16A, "MUSICAL SYMBOL FINGERED TREMOLO-1" },
    { 0x1D16B, "MUSICAL SYMBOL FINGERED TREMOLO-2" },
    { 0x1D16C, "MUSICAL SYMBOL FINGERED TREMOLO-3" },
    { 0x1D16D, "MUSICAL SYMBOL COMBINING AUGMENTATION DOT" },
    { 0x1D16E, "MUSICAL SYMBOL COMBINING FLAG-1" },
    { 0x1D16F, "MUSICAL SYMBOL COMBINING FLAG-2" },
    { 0x1D170, "MUSICAL SYMBOL COMBINING FLAG-3" },
    { 0x1D171, "MUSICAL SYMBOL COMBINING FLAG-4" },
    { 0x1D172, "MUSICAL SYMBOL COMBINING FLAG-5" },
    { 0x1D173, "MUSICAL SYMBOL BEGIN BEAM" },
    { 0x1D174, "MUSICAL SYMBOL END BEAM" },
    { 0x1D175, "MUSICAL SYMBOL BEGIN TIE" },
    { 0x1D176, "MUSICAL SYMBOL END TIE" },
    { 0x1D177, "MUSICAL SYMBOL BEGIN SLUR" },
    { 0x1D178, "MUSICAL SYMBOL END SLUR" },
    { 0x1D179, "MUSICAL SYMBOL BEGIN PHRASE" },
    { 0x1D17A, "MUSICAL SYMBOL END PHRASE" },
    { 0x1D17B, "MUSICAL SYMBOL COMBINING ACCENT" },
    { 0x1D17C, "MUSICAL SYMBOL COMBINING STACCATO" },
    { 0x1D17D, "MUSICAL SYMBOL COMBINING TENUTO" },
    { 0x1D17E, "MUSICAL SYMBOL COMBINING STACCATISSIMO" },
    { 0x1D17F, "MUSICAL SYMBOL COMBINING MARCATO" },
    { 0x1D180, "MUSICAL SYMBOL COMBINING MARCATO-STACCATO" },
    { 0x1D181, "MUSICAL SYMBOL COMBINING ACCENT-STACCATO" },
    { 0x1D182, "MUSICAL SYMBOL COMBINING LOURE" },
    { 0x1D183, "MUSICAL SYMBOL ARPEGGIATO UP" },
    { 0x1D184, "MUSICAL SYMBOL ARPEGGIATO DOWN" },
    { 0x1D185, "MUSICAL SYMBOL COMBINING DOIT" },
    { 0x1D186, "MUSICAL SYMBOL COMBINING RIP" },
    { 0x1D187, "MUSICAL SYMBOL COMBINING FLIP" },
    { 0x1D188, "MUSICAL SYMBOL COMBINING SMEAR" },
    { 0x1D189, "MUSICAL SYMBOL COMBINING BEND" },
    { 0x1D18A, "MUSICAL SYMBOL COMBINING DOUBLE TONGUE" },
    { 0x1D18B, "MUSICAL SYMBOL COMBINING TRIPLE TONGUE" },
    { 0x1D18C, "MUSICAL SYMBOL RINFORZANDO" },
    { 0x1D18D, "MUSICAL SYMBOL SUBITO" },
    { 0x1D18E, "MUSICAL SYMBOL Z" },
    { 0x1D18F, "MUSICAL SYMBOL PIANO" },
    { 0x1D190, "MUSICAL SYMBOL MEZZO" },
    { 0x1D191, "MUSICAL SYMBOL FORTE" },
    { 0x1D192, "MUSICAL SYMBOL CRESCENDO" },
    { 0x1D193, "MUSICAL SYMBOL DECRESCENDO" },
    { 0x1D194, "MUSICAL SYMBOL GRACE NOTE SLASH" },
    { 0x1D195, "MUSICAL SYMBOL GRACE NOTE NO SLASH" },
    { 0x1D196, "MUSICAL SYMBOL TR" },
    { 0x1D197, "MUSICAL SYMBOL TURN" },
    { 0x1D198, "MUSICAL SYMBOL INVERTED TURN" },
    { 0x1D199, "MUSICAL SYMBOL TURN SLASH" },
    { 0x1D19A, "MUSICAL SYMBOL TURN UP" },
    { 0x1D19B, "MUSICAL SYMBOL ORNAMENT STROKE-1" },
    { 0x1D19C, "MUSICAL SYMBOL ORNAMENT STROKE-2" },
    { 0x1D19D, "MUSICAL SYMBOL ORNAMENT STROKE-3" },
    { 0x1D19E, "MUSICAL SYMBOL ORNAMENT STROKE-4" },
    { 0x1D19F, "MUSICAL SYMBOL ORNAMENT STROKE-5" },
    { 0x1D1A0, "MUSICAL SYMBOL ORNAMENT STROKE-6" },
    { 0x1D1A1, "MUSICAL SYMBOL ORNAMENT STROKE-7" },
    { 0x1D1A2, "MUSICAL SYMBOL ORNAMENT STROKE-8" },
    { 0x1D1A3, "MUSICAL SYMBOL ORNAMENT STROKE-9" },
    { 0x1D1A4, "MUSICAL SYMBOL ORNAMENT STROKE-10" },
    { 0x1D1A5, "MUSICAL SYMBOL ORNAMENT STROKE-11" },
    { 0x1D1A6, "MUSICAL SYMBOL HAUPTSTIMME" },
    { 0x1D1A7, "MUSICAL SYMBOL NEBENSTIMME" },
    { 0x1D1A8, "MUSICAL SYMBOL END OF STIMME" },
    { 0x1D1A9, "MUSICAL SYMBOL DEGREE SLASH" },
    { 0x1D1AA, "MUSICAL SYMBOL COMBINING DOWN BOW" },
    { 0x1D1AB, "MUSICAL SYMBOL COMBINING UP BOW" },
    { 0x1D1AC, "MUSICAL SYMBOL COMBINING HARMONIC" },
    { 0x1D1AD, "MUSICAL SYMBOL COMBINING SNAP PIZZICATO" },
    { 0x1D1AE, "MUSICAL SYMBOL PEDAL MARK" },
    { 0x1D1AF, "MUSICAL SYMBOL PEDAL UP MARK" },
    { 0x1D1B0, "MUSICAL SYMBOL HALF PEDAL MARK" },
    { 0x1D1B1, "MUSICAL SYMBOL GLISSANDO UP" },
    { 0x1D1B2, "MUSICAL SYMBOL GLISSANDO DOWN" },
    { 0x1D1B3, "MUSICAL SYMBOL WITH FINGERNAILS" },
    { 0x1D1B4, "MUSICAL SYMBOL DAMP" },
    { 0x1D1B5, "MUSICAL SYMBOL DAMP ALL" },
    { 0x1D1B6, "MUSICAL SYMBOL MAXIMA" },
    { 0x1D1B7, "MUSICAL SYMBOL LONGA" },
    { 0x1D1B8, "MUSICAL SYMBOL BREVIS" },
    { 0x1D1B9, "MUSICAL SYMBOL SEMIBREVIS WHITE" },
    { 0x1D1BA, "MUSICAL SYMBOL SEMIBREVIS BLACK" },
    { 0x1D1BB, "MUSICAL SYMBOL MINIMA" },
    { 0x1D1BC, "MUSICAL SYMBOL MINIMA BLACK" },
    { 0x1D1BD, "MUSICAL SYMBOL SEMIMINIMA WHITE" },
    { 0x1D1BE, "MUSICAL SYMBOL SEMIMINIMA BLACK" },
    { 0x1D1BF, "MUSICAL SYMBOL FUSA WHITE" },
    { 0x1D1C0, "MUSICAL SYMBOL FUSA BLACK" },
    { 0x1D1C1, "MUSICAL SYMBOL LONGA PERFECTA REST" },
    { 0x1D1C2, "MUSICAL SYMBOL LONGA IMPERFECTA REST" },
    { 0x1D1C3, "MUSICAL SYMBOL BREVIS REST" },
    { 0x1D1C4, "MUSICAL SYMBOL SEMIBREVIS REST" },
    { 0x1D1C5, "MUSICAL SYMBOL MINIMA REST" },
    { 0x1D1C6, "MUSICAL SYMBOL SEMIMINIMA REST" },
    { 0x1D1C7, "MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE PERFECTA" },
    { 0x1D1C8, "MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE IMPERFECTA" },
    { 0x1D1C9, "MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE PERFECTA DIMINUTION-1" },
    { 0x1D1CA, "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE PERFECTA" },
    { 0x1D1CB, "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA" },
    { 0x1D1CC, "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-1" },
    { 0x1D1CD, "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-2" },
    { 0x1D1CE, "MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-3" },
    { 0x1D1CF, "MUSICAL SYMBOL CROIX" },
    { 0x1D1D0, "MUSICAL SYMBOL GREGORIAN C CLEF" },
    { 0x1D1D1, "MUSICAL SYMBOL GREGORIAN F CLEF" },
    { 0x1D1D2, "MUSICAL SYMBOL SQUARE B" },
    { 0x1D1D3, "MUSICAL SYMBOL VIRGA" },
    { 0x1D1D4, "MUSICAL SYMBOL PODATUS" },
    { 0x1D1D5, "MUSICAL SYMBOL CLIVIS" },
    { 0x1D1D6, "MUSICAL SYMBOL SCANDICUS" },
    { 0x1D1D7, "MUSICAL SYMBOL CLIMACUS" },
    { 0x1D1D8, "MUSICAL SYMBOL TORCULUS" },
    { 0x1D1D9, "MUSICAL SYMBOL PORRECTUS" },
    { 0x1D1DA, "MUSICAL SYMBOL PORRECTUS FLEXUS" },
    { 0x1D1DB, "MUSICAL SYMBOL SCANDICUS FLEXUS" },
    { 0x1D1DC, "MUSICAL SYMBOL TORCULUS RESUPINUS" },
    { 0x1D1DD, "MUSICAL SYMBOL PES SUBPUNCTIS" },
    { 0x1D1DE, "MUSICAL SYMBOL KIEVAN C CLEF" },
    { 0x1D1DF, "MUSICAL SYMBOL KIEVAN END OF PIECE" },
    { 0x1D1E0, "MUSICAL SYMBOL KIEVAN FINAL NOTE" },
    { 0x1D1E1, "MUSICAL SYMBOL KIEVAN RECITATIVE MARK" },
    { 0x1D1E2, "MUSICAL SYMBOL KIEVAN WHOLE NOTE" },
    { 0x1D1E3, "MUSICAL SYMBOL KIEVAN HALF NOTE" },
    { 0x1D1E4, "MUSICAL SYMBOL KIEVAN QUARTER NOTE STEM DOWN" },
    { 0x1D1E5, "MUSICAL SYMBOL KIEVAN QUARTER NOTE STEM UP" },
    { 0x1D1E6, "MUSICAL SYMBOL KIEVAN EIGHTH NOTE STEM DOWN" },
    { 0x1D1E7, "MUSICAL SYMBOL KIEVAN EIGHTH NOTE STEM UP" },
    { 0x1D1E8, "MUSICAL SYMBOL KIEVAN FLAT SIGN" },
    { 0x1D1E9, "MUSICAL SYMBOL SORI" },
    { 0x1D1EA, "MUSICAL SYMBOL KORON" },
    { 0x1D200, "GREEK VOCAL NOTATION SYMBOL-1" },
    { 0x1D201, "GREEK VOCAL NOTATION SYMBOL-2" },
    { 0x1D202, "GREEK VOCAL NOTATION SYMBOL-3" },
    { 0x1D203, "GREEK VOCAL NOTATION SYMBOL-4" },
    { 0x1D204, "GREEK VOCAL NOTATION SYMBOL-5" },
    { 0x1D205, "GREEK VOCAL NOTATION SYMBOL-6" },
    { 0x1D206, "GREEK VOCAL NOTATION SYMBOL-7" },
    { 0x1D207, "GREEK VOCAL NOTATION SYMBOL-8" },
    { 0x1D208, "GREEK VOCAL NOTATION SYMBOL-9" },
    { 0x1D209, "GREEK VOCAL NOTATION SYMBOL-10" },
    { 0x1D20A, "GREEK VOCAL NOTATION SYMBOL-11" },
    { 0x1D20B, "GREEK VOCAL NOTATION SYMBOL-12" },
    { 0x1D20C, "GREEK VOCAL NOTATION SYMBOL-13" },
    { 0x1D20D, "GREEK VOCAL NOTATION SYMBOL-14" },
    { 0x1D20E, "GREEK VOCAL NOTATION SYMBOL-15" },
    { 0x1D20F, "GREEK VOCAL NOTATION SYMBOL-16" },
    { 0x1D210, "GREEK VOCAL NOTATION SYMBOL-17" },
    { 0x1D211, "GREEK VOCAL NOTATION SYMBOL-18" },
    { 0x1D212, "GREEK VOCAL NOTATION SYMBOL-19" },
    { 0x1D213, "GREEK VOCAL NOTATION SYMBOL-20" },
    { 0x1D214, "GREEK VOCAL NOTATION SYMBOL-21" },
    { 0x1D215, "GREEK VOCAL NOTATION SYMBOL-22" },
    { 0x1D216, "GREEK VOCAL NOTATION SYMBOL-23" },
    { 0x1D217, "GREEK VOCAL NOTATION SYMBOL-24" },
    { 0x1D218, "GREEK VOCAL NOTATION SYMBOL-50" },
    { 0x1D219, "GREEK VOCAL NOTATION SYMBOL-51" },
    { 0x1D21A, "GREEK VOCAL NOTATION SYMBOL-52" },
    { 0x1D21B, "GREEK VOCAL NOTATION SYMBOL-53" },
    { 0x1D21C, "GREEK VOCAL NOTATION SYMBOL-54" },
    { 0x1D21D, "GREEK INSTRUMENTAL NOTATION SYMBOL-1" },
    { 0x1D21E, "GREEK INSTRUMENTAL NOTATION SYMBOL-2" },
    { 0x1D21F, "GREEK INSTRUMENTAL NOTATION SYMBOL-4" },
    { 0x1D220, "GREEK INSTRUMENTAL NOTATION SYMBOL-5" },
    { 0x1D221, "GREEK INSTRUMENTAL NOTATION SYMBOL-7" },
    { 0x1D222, "GREEK INSTRUMENTAL NOTATION SYMBOL-8" },
    { 0x1D223, "GREEK INSTRUMENTAL NOTATION SYMBOL-11" },
    { 0x1D224, "GREEK INSTRUMENTAL NOTATION SYMBOL-12" },
    { 0x1D225, "GREEK INSTRUMENTAL NOTATION SYMBOL-13" },
    { 0x1D226, "GREEK INSTRUMENTAL NOTATION SYMBOL-14" },
    { 0x1D227, "GREEK INSTRUMENTAL NOTATION SYMBOL-17" },
    { 0x1D228, "GREEK INSTRUMENTAL NOTATION SYMBOL-18" },
    { 0x1D229, "GREEK INSTRUMENTAL NOTATION SYMBOL-19" },
    { 0x1D22A, "GREEK INSTRUMENTAL NOTATION SYMBOL-23" },
    { 0x1D22B, "GREEK INSTRUMENTAL NOTATION SYMBOL-24" },
    { 0x1D22C, "GREEK INSTRUMENTAL NOTATION SYMBOL-25" },
    { 0x1D22D, "GREEK INSTRUMENTAL NOTATION SYMBOL-26" },
    { 0x1D22E, "GREEK INSTRUMENTAL NOTATION SYMBOL-27" },
    { 0x1D22F, "GREEK INSTRUMENTAL NOTATION SYMBOL-29" },
    { 0x1D230, "GREEK INSTRUMENTAL NOTATION SYMBOL-30" },
    { 0x1D231, "GREEK INSTRUMENTAL NOTATION SYMBOL-32" },
    { 0x1D232, "GREEK INSTRUMENTAL NOTATION SYMBOL-36" },
    { 0x1D233, "GREEK INSTRUMENTAL NOTATION SYMBOL-37" },
    { 0x1D234, "GREEK INSTRUMENTAL NOTATION SYMBOL-38" },
    { 0x1D235, "GREEK INSTRUMENTAL NOTATION SYMBOL-39" },
    { 0x1D236, "GREEK INSTRUMENTAL NOTATION SYMBOL-40" },
    { 0x1D237, "GREEK INSTRUMENTAL NOTATION SYMBOL-42" },
    { 0x1D238, "GREEK INSTRUMENTAL NOTATION SYMBOL-43" },
    { 0x1D239, "GREEK INSTRUMENTAL NOTATION SYMBOL-45" },
    { 0x1D23A, "GREEK INSTRUMENTAL NOTATION SYMBOL-47" },
    { 0x1D23B, "GREEK INSTRUMENTAL NOTATION SYMBOL-48" },
    { 0x1D23C, "GREEK INSTRUMENTAL NOTATION SYMBOL-49" },
    { 0x1D23D, "GREEK INSTRUMENTAL NOTATION SYMBOL-50" },
    { 0x1D23E, "GREEK INSTRUMENTAL NOTATION SYMBOL-51" },
    { 0x1D23F, "GREEK INSTRUMENTAL NOTATION SYMBOL-52" },
    { 0x1D240, "GREEK INSTRUMENTAL NOTATION SYMBOL-53" },
    { 0x1D241, "GREEK INSTRUMENTAL NOTATION SYMBOL-54" },
    { 0x1D242, "COMBINING GREEK MUSICAL TRISEME" },
    { 0x1D243, "COMBINING GREEK MUSICAL TETRASEME" },
    { 0x1D244, "COMBINING GREEK MUSICAL PENTASEME" },
    { 0x1D245, "GREEK MUSICAL LEIMMA" },
    { 0x1D2C0, "KAKTOVIK NUMERAL ZERO" },
    { 0x1D2C1, "KAKTOVIK NUMERAL ONE" },
    { 0x1D2C2, "KAKTOVIK NUMERAL TWO" },
    { 0x1D2C3, "KAKTOVIK NUMERAL THREE" },
    { 0x1D2C4, "KAKTOVIK NUMERAL FOUR" },
    { 0x1D2C5, "KAKTOVIK NUMERAL FIVE" },
    { 0x1D2C6, "KAKTOVIK NUMERAL SIX" },
    { 0x1D2C7, "KAKTOVIK NUMERAL SEVEN" },
    { 0x1D2C8, "KAKTOVIK NUMERAL EIGHT" },
    { 0x1D2C9, "KAKTOVIK NUMERAL NINE" },
    { 0x1D2CA, "KAKTOVIK NUMERAL TEN" },
    { 0x1D2CB, "KAKTOVIK NUMERAL ELEVEN" },
    { 0x1D2CC, "KAKTOVIK NUMERAL TWELVE" },
    { 0x1D2CD, "KAKTOVIK NUMERAL THIRTEEN" },
    { 0x1D2CE, "KAKTOVIK NUMERAL FOURTEEN" },
    { 0x1D2CF, "KAKTOVIK NUMERAL FIFTEEN" },
    { 0x1D2D0, "KAKTOVIK NUMERAL SIXTEEN" },
    { 0x1D2D1, "KAKTOVIK NUMERAL SEVENTEEN" },
    { 0x1D2D2, "KAKTOVIK NUMERAL EIGHTEEN" },
    { 0x1D2D3, "KAKTOVIK NUMERAL NINETEEN" },
    { 0x1D2E0, "MAYAN NUMERAL ZERO" },
    { 0x1D2E1, "MAYAN NUMERAL ONE" },
    { 0x1D2E2, "MAYAN NUMERAL TWO" },
    { 0x1D2E3, "MAYAN NUMERAL THREE" },
    { 0x1D2E4, "MAYAN NUMERAL FOUR" },
    { 0x1D2E5, "MAYAN NUMERAL FIVE" },
    { 0x1D2E6, "MAYAN NUMERAL SIX" },
    { 0x1D2E7, "MAYAN NUMERAL SEVEN" },
    { 0x1D2E8, "MAYAN NUMERAL EIGHT" },
    { 0x1D2E9, "MAYAN NUMERAL NINE" },
    { 0x1D2EA, "MAYAN NUMERAL TEN" },
    { 0x1D2EB, "MAYAN NUMERAL ELEVEN" },
    { 0x1D2EC, "MAYAN NUMERAL TWELVE" },
    { 0x1D2ED, "MAYAN NUMERAL THIRTEEN" },
    { 0x1D2EE, "MAYAN NUMERAL FOURTEEN" },
    { 0x1D2EF, "MAYAN NUMERAL FIFTEEN" },
    { 0x1D2F0, "MAYAN NUMERAL SIXTEEN" },
    { 0x1D2F1, "MAYAN NUMERAL SEVENTEEN" },
    { 0x1D2F2, "MAYAN NUMERAL EIGHTEEN" },
    { 0x1D2F3, "MAYAN NUMERAL NINETEEN" },
    { 0x1D300, "MONOGRAM FOR EARTH" },
    { 0x1D301, "DIGRAM FOR HEAVENLY EARTH" },
    { 0x1D302, "DIGRAM FOR HUMAN EARTH" },
    { 0x1D303, "DIGRAM FOR EARTHLY HEAVEN" },
    { 0x1D304, "DIGRAM FOR EARTHLY HUMAN" },
    { 0x1D305, "DIGRAM FOR EARTH" },
    { 0x1D306, "TETRAGRAM FOR CENTRE" },
    { 0x1D307, "TETRAGRAM FOR FULL CIRCLE" },
    { 0x1D308, "TETRAGRAM FOR MIRED" },
    { 0x1D309, "TETRAGRAM FOR BARRIER" },
    { 0x1D30A, "TETRAGRAM FOR KEEPING SMALL" },
    { 0x1D30B, "TETRAGRAM FOR CONTRARIETY" },
    { 0x1D30C, "TETRAGRAM FOR ASCENT" },
    { 0x1D30D, "TETRAGRAM FOR OPPOSITION" },
    { 0x1D30E, "TETRAGRAM FOR BRANCHING OUT" },
    { 0x1D30F, "TETRAGRAM FOR DEFECTIVENESS OR DISTORTION" },
    { 0x1D310, "TETRAGRAM FOR DIVERGENCE" },
    { 0x1D311, "TETRAGRAM FOR YOUTHFULNESS" },
    { 0x1D312, "TETRAGRAM FOR INCREASE" },
    { 0x1D313, "TETRAGRAM FOR PENETRATION" },
    { 0x1D314, "TETRAGRAM FOR REACH" },
    { 0x1D315, "TETRAGRAM FOR CONTACT" },
    { 0x1D316, "TETRAGRAM FOR HOLDING BACK" },
    { 0x1D317, "TETRAGRAM FOR WAITING" },
    { 0x1D318, "TETRAGRAM FOR FOLLOWING" },
    { 0x1D319, "TETRAGRAM FOR ADVANCE" },
    { 0x1D31A, "TETRAGRAM FOR RELEASE" },
    { 0x1D31B, "TETRAGRAM FOR RESISTANCE" },
    { 0x1D31C, "TETRAGRAM FOR EASE" },
    { 0x1D31D, "TETRAGRAM FOR JOY" },
    { 0x1D31E, "TETRAGRAM FOR CONTENTION" },
    { 0x1D31F, "TETRAGRAM FOR ENDEAVOUR" },
    { 0x1D320, "TETRAGRAM FOR DUTIES" },
    { 0x1D321, "TETRAGRAM FOR CHANGE" },
    { 0x1D322, "TETRAGRAM FOR DECISIVENESS" },
    { 0x1D323, "TETRAGRAM FOR BOLD RESOLUTION" },
    { 0x1D324, "TETRAGRAM FOR PACKING" },
    { 0x1D325, "TETRAGRAM FOR LEGION" },
    { 0x1D326, "TETRAGRAM FOR CLOSENESS" },
    { 0x1D327, "TETRAGRAM FOR KINSHIP" },
    { 0x1D328, "TETRAGRAM FOR GATHERING" },
    { 0x1D329, "TETRAGRAM FOR STRENGTH" },
    { 0x1D32A, "TETRAGRAM FOR PURITY" },
    { 0x1D32B, "TETRAGRAM FOR FULLNESS" },
    { 0x1D32C, "TETRAGRAM FOR RESIDENCE" },
    { 0x1D32D, "TETRAGRAM FOR LAW OR MODEL" },
    { 0x1D32E, "TETRAGRAM FOR RESPONSE" },
    { 0x1D32F, "TETRAGRAM FOR GOING TO MEET" },
    { 0x1D330, "TETRAGRAM FOR ENCOUNTERS" },
    { 0x1D331, "TETRAGRAM FOR STOVE" },
    { 0x1D332, "TETRAGRAM FOR GREATNESS" },
    { 0x1D333, "TETRAGRAM FOR ENLARGEMENT" },
    { 0x1D334, "TETRAGRAM FOR PATTERN" },
    { 0x1D335, "TETRAGRAM FOR RITUAL" },
    { 0x1D336, "TETRAGRAM FOR FLIGHT" },
    { 0x1D337, "TETRAGRAM FOR VASTNESS OR WASTING" },
    { 0x1D338, "TETRAGRAM FOR CONSTANCY" },
    { 0x1D339, "TETRAGRAM FOR MEASURE" },
    { 0x1D33A, "TETRAGRAM FOR ETERNITY" },
    { 0x1D33B, "TETRAGRAM FOR UNITY" },
    { 0x1D33C, "TETRAGRAM FOR DIMINISHMENT" },
    { 0x1D33D, "TETRAGRAM FOR CLOSED MOUTH" },
    { 0x1D33E, "TETRAGRAM FOR GUARDEDNESS" },
    { 0x1D33F, "TETRAGRAM FOR GATHERING IN" },
    { 0x1D340, "TETRAGRAM FOR MASSING" },
    { 0x1D341, "TETRAGRAM FOR ACCUMULATION" },
    { 0x1D342, "TETRAGRAM FOR EMBELLISHMENT" },
    { 0x1D343, "TETRAGRAM FOR DOUBT" },
    { 0x1D344, "TETRAGRAM FOR WATCH" },
    { 0x1D345, "TETRAGRAM FOR SINKING" },
    { 0x1D346, "TETRAGRAM FOR INNER" },
    { 0x1D347, "TETRAGRAM FOR DEPARTURE" },
    { 0x1D348, "TETRAGRAM FOR DARKENING" },
    { 0x1D349, "TETRAGRAM FOR DIMMING" },
    { 0x1D34A, "TETRAGRAM FOR EXHAUSTION" },
    { 0x1D34B, "TETRAGRAM FOR SEVERANCE" },
    { 0x1D34C, "TETRAGRAM FOR STOPPAGE" },
    { 0x1D34D, "TETRAGRAM FOR HARDNESS" },
    { 0x1D34E, "TETRAGRAM FOR COMPLETION" },
    { 0x1D34F, "TETRAGRAM FOR CLOSURE" },
    { 0x1D350, "TETRAGRAM FOR FAILURE" },
    { 0x1D351, "TETRAGRAM FOR AGGRAVATION" },
    { 0x1D352, "TETRAGRAM FOR COMPLIANCE" },
    { 0x1D353, "TETRAGRAM FOR ON THE VERGE" },
    { 0x1D354, "TETRAGRAM FOR DIFFICULTIES" },
    { 0x1D355, "TETRAGRAM FOR LABOURING" },
    { 0x1D356, "TETRAGRAM FOR FOSTERING" },
    { 0x1D360, "COUNTING ROD UNIT DIGIT ONE" },
    { 0x1D361, "COUNTING ROD UNIT DIGIT TWO" },
    { 0x1D362, "COUNTING ROD UNIT DIGIT THREE" },
    { 0x1D363, "COUNTING ROD UNIT DIGIT FOUR" },
    { 0x1D364, "COUNTING ROD UNIT DIGIT FIVE" },
    { 0x1D365, "COUNTING ROD UNIT DIGIT SIX" },
    { 0x1D366, "COUNTING ROD UNIT DIGIT SEVEN" },
    { 0x1D367, "COUNTING ROD UNIT DIGIT EIGHT" },
    { 0x1D368, "COUNTING ROD UNIT DIGIT NINE" },
    { 0x1D369, "COUNTING ROD TENS DIGIT ONE" },
    { 0x1D36A, "COUNTING ROD TENS DIGIT TWO" },
    { 0x1D36B, "COUNTING ROD TENS DIGIT THREE" },
    { 0x1D36C, "COUNTING ROD TENS DIGIT FOUR" },
    { 0x1D36D, "COUNTING ROD TENS DIGIT FIVE" },
    { 0x1D36E, "COUNTING ROD TENS DIGIT SIX" },
    { 0x1D36F, "COUNTING ROD TENS DIGIT SEVEN" },
    { 0x1D370, "COUNTING ROD TENS DIGIT EIGHT" },
    { 0x1D371, "COUNTING ROD TENS DIGIT NINE" },
    { 0x1D372, "IDEOGRAPHIC TALLY MARK ONE" },
    { 0x1D373, "IDEOGRAPHIC TALLY MARK TWO" },
    { 0x1D374, "IDEOGRAPHIC TALLY MARK THREE" },
    { 0x1D375, "IDEOGRAPHIC TALLY MARK FOUR" },
    { 0x1D376, "IDEOGRAPHIC TALLY MARK FIVE" },
    { 0x1D377, "TALLY MARK ONE" },
    { 0x1D378, "TALLY MARK FIVE" },
    { 0x1D400, "MATHEMATICAL BOLD CAPITAL A" },
    { 0x1D401, "MATHEMATICAL BOLD CAPITAL B" },
    { 0x1D402, "MATHEMATICAL BOLD CAPITAL C" },
    { 0x1D403, "MATHEMATICAL BOLD CAPITAL D" },
    { 0x1D404, "MATHEMATICAL BOLD CAPITAL E" },
    { 0x1D405, "MATHEMATICAL BOLD CAPITAL F" },
    { 0x1D406, "MATHEMATICAL BOLD CAPITAL G" },
    { 0x1D407, "MATHEMATICAL BOLD CAPITAL H" },
    { 0x1D408, "MATHEMATICAL BOLD CAPITAL I" },
    { 0x1D409, "MATHEMATICAL BOLD CAPITAL J" },
    { 0x1D40A, "MATHEMATICAL BOLD CAPITAL K" },
    { 0x1D40B, "MATHEMATICAL BOLD CAPITAL L" },
    { 0x1D40C, "MATHEMATICAL BOLD CAPITAL M" },
    { 0x1D40D, "MATHEMATICAL BOLD CAPITAL N" },
    { 0x1D40E, "MATHEMATICAL BOLD CAPITAL O" },
    { 0x1D40F, "MATHEMATICAL BOLD CAPITAL P" },
    { 0x1D410, "MATHEMATICAL BOLD CAPITAL Q" },
    { 0x1D411, "MATHEMATICAL BOLD CAPITAL R" },
    { 0x1D412, "MATHEMATICAL BOLD CAPITAL S" },
    { 0x1D413, "MATHEMATICAL BOLD CAPITAL T" },
    { 0x1D414, "MATHEMATICAL BOLD CAPITAL U" },
    { 0x1D415, "MATHEMATICAL BOLD CAPITAL V" },
    { 0x1D416, "MATHEMATICAL BOLD CAPITAL W" },
    { 0x1D417, "MATHEMATICAL BOLD CAPITAL X" },
    { 0x1D418, "MATHEMATICAL BOLD CAPITAL Y" },
    { 0x1D419, "MATHEMATICAL BOLD CAPITAL Z" },
    { 0x1D41A, "MATHEMATICAL BOLD SMALL A" },
    { 0x1D41B, "MATHEMATICAL BOLD SMALL B" },
    { 0x1D41C, "MATHEMATICAL BOLD SMALL C" },
    { 0x1D41D, "MATHEMATICAL BOLD SMALL D" },
    { 0x1D41E, "MATHEMATICAL BOLD SMALL E" },
    { 0x1D41F, "MATHEMATICAL BOLD SMALL F" },
    { 0x1D420, "MATHEMATICAL BOLD SMALL G" },
    { 0x1D421, "MATHEMATICAL BOLD SMALL H" },
    { 0x1D422, "MATHEMATICAL BOLD SMALL I" },
    { 0x1D423, "MATHEMATICAL BOLD SMALL J" },
    { 0x1D424, "MATHEMATICAL BOLD SMALL K" },
    { 0x1D425, "MATHEMATICAL BOLD SMALL L" },
    { 0x1D426, "MATHEMATICAL BOLD SMALL M" },
    { 0x1D427, "MATHEMATICAL BOLD SMALL N" },
    { 0x1D428, "MATHEMATICAL BOLD SMALL O" },
    { 0x1D429, "MATHEMATICAL BOLD SMALL P" },
    { 0x1D42A, "MATHEMATICAL BOLD SMALL Q" },
    { 0x1D42B, "MATHEMATICAL BOLD SMALL R" },
    { 0x1D42C, "MATHEMATICAL BOLD SMALL S" },
    { 0x1D42D, "MATHEMATICAL BOLD SMALL T" },
    { 0x1D42E, "MATHEMATICAL BOLD SMALL U" },
    { 0x1D42F, "MATHEMATICAL BOLD SMALL V" },
    { 0x1D430, "MATHEMATICAL BOLD SMALL W" },
    { 0x1D431, "MATHEMATICAL BOLD SMALL X" },
    { 0x1D432, "MATHEMATICAL BOLD SMALL Y" },
    { 0x1D433, "MATHEMATICAL BOLD SMALL Z" },
    { 0x1D434, "MATHEMATICAL ITALIC CAPITAL A" },
    { 0x1D435, "MATHEMATICAL ITALIC CAPITAL B" },
    { 0x1D436, "MATHEMATICAL ITALIC CAPITAL C" },
    { 0x1D437, "MATHEMATICAL ITALIC CAPITAL D" },
    { 0x1D438, "MATHEMATICAL ITALIC CAPITAL E" },
    { 0x1D439, "MATHEMATICAL ITALIC CAPITAL F" },
    { 0x1D43A, "MATHEMATICAL ITALIC CAPITAL G" },
    { 0x1D43B, "MATHEMATICAL ITALIC CAPITAL H" },
    { 0x1D43C, "MATHEMATICAL ITALIC CAPITAL I" },
    { 0x1D43D, "MATHEMATICAL ITALIC CAPITAL J" },
    { 0x1D43E, "MATHEMATICAL ITALIC CAPITAL K" },
    { 0x1D43F, "MATHEMATICAL ITALIC CAPITAL L" },
    { 0x1D440, "MATHEMATICAL ITALIC CAPITAL M" },
    { 0x1D441, "MATHEMATICAL ITALIC CAPITAL N" },
    { 0x1D442, "MATHEMATICAL ITALIC CAPITAL O" },
    { 0x1D443, "MATHEMATICAL ITALIC CAPITAL P" },
    { 0x1D444, "MATHEMATICAL ITALIC CAPITAL Q" },
    { 0x1D445, "MATHEMATICAL ITALIC CAPITAL R" },
    { 0x1D446, "MATHEMATICAL ITALIC CAPITAL S" },
    { 0x1D447, "MATHEMATICAL ITALIC CAPITAL T" },
    { 0x1D448, "MATHEMATICAL ITALIC CAPITAL U" },
    { 0x1D449, "MATHEMATICAL ITALIC CAPITAL V" },
    { 0x1D44A, "MATHEMATICAL ITALIC CAPITAL W" },
    { 0x1D44B, "MATHEMATICAL ITALIC CAPITAL X" },
    { 0x1D44C, "MATHEMATICAL ITALIC CAPITAL Y" },
    { 0x1D44D, "MATHEMATICAL ITALIC CAPITAL Z" },
    { 0x1D44E, "MATHEMATICAL ITALIC SMALL A" },
    { 0x1D44F, "MATHEMATICAL ITALIC SMALL B" },
    { 0x1D450, "MATHEMATICAL ITALIC SMALL C" },
    { 0x1D451, "MATHEMATICAL ITALIC SMALL D" },
    { 0x1D452, "MATHEMATICAL ITALIC SMALL E" },
    { 0x1D453, "MATHEMATICAL ITALIC SMALL F" },
    { 0x1D454, "MATHEMATICAL ITALIC SMALL G" },
    { 0x1D456, "MATHEMATICAL ITALIC SMALL I" },
    { 0x1D457, "MATHEMATICAL ITALIC SMALL J" },
    { 0x1D458, "MATHEMATICAL ITALIC SMALL K" },
    { 0x1D459, "MATHEMATICAL ITALIC SMALL L" },
    { 0x1D45A, "MATHEMATICAL ITALIC SMALL M" },
    { 0x1D45B, "MATHEMATICAL ITALIC SMALL N" },
    { 0x1D45C, "MATHEMATICAL ITALIC SMALL O" },
    { 0x1D45D, "MATHEMATICAL ITALIC SMALL P" },
    { 0x1D45E, "MATHEMATICAL ITALIC SMALL Q" },
    { 0x1D45F, "MATHEMATICAL ITALIC SMALL R" },
    { 0x1D460, "MATHEMATICAL ITALIC SMALL S" },
    { 0x1D461, "MATHEMATICAL ITALIC SMALL T" },
    { 0x1D462, "MATHEMATICAL ITALIC SMALL U" },
    { 0x1D463, "MATHEMATICAL ITALIC SMALL V" },
    { 0x1D464, "MATHEMATICAL ITALIC SMALL W" },
    { 0x1D465, "MATHEMATICAL ITALIC SMALL X" },
    { 0x1D466, "MATHEMATICAL ITALIC SMALL Y" },
    { 0x1D467, "MATHEMATICAL ITALIC SMALL Z" },
    { 0x1D468, "MATHEMATICAL BOLD ITALIC CAPITAL A" },
    { 0x1D469, "MATHEMATICAL BOLD ITALIC CAPITAL B" },
    { 0x1D46A, "MATHEMATICAL BOLD ITALIC CAPITAL C" },
    { 0x1D46B, "MATHEMATICAL BOLD ITALIC CAPITAL D" },
    { 0x1D46C, "MATHEMATICAL BOLD ITALIC CAPITAL E" },
    { 0x1D46D, "MATHEMATICAL BOLD ITALIC CAPITAL F" },
    { 0x1D46E, "MATHEMATICAL BOLD ITALIC CAPITAL G" },
    { 0x1D46F, "MATHEMATICAL BOLD ITALIC CAPITAL H" },
    { 0x1D470, "MATHEMATICAL BOLD ITALIC CAPITAL I" },
    { 0x1D471, "MATHEMATICAL BOLD ITALIC CAPITAL J" },
    { 0x1D472, "MATHEMATICAL BOLD ITALIC CAPITAL K" },
    { 0x1D473, "MATHEMATICAL BOLD ITALIC CAPITAL L" },
    { 0x1D474, "MATHEMATICAL BOLD ITALIC CAPITAL M" },
    { 0x1D475, "MATHEMATICAL BOLD ITALIC CAPITAL N" },
    { 0x1D476, "MATHEMATICAL BOLD ITALIC CAPITAL O" },
    { 0x1D477, "MATHEMATICAL BOLD ITALIC CAPITAL P" },
    { 0x1D478, "MATHEMATICAL BOLD ITALIC CAPITAL Q" },
    { 0x1D479, "MATHEMATICAL BOLD ITALIC CAPITAL R" },
    { 0x1D47A, "MATHEMATICAL BOLD ITALIC CAPITAL S" },
    { 0x1D47B, "MATHEMATICAL BOLD ITALIC CAPITAL T" },
    { 0x1D47C, "MATHEMATICAL BOLD ITALIC CAPITAL U" },
    { 0x1D47D, "MATHEMATICAL BOLD ITALIC CAPITAL V" },
    { 0x1D47E, "MATHEMATICAL BOLD ITALIC CAPITAL W" },
    { 0x1D47F, "MATHEMATICAL BOLD ITALIC CAPITAL X" },
    { 0x1D480, "MATHEMATICAL BOLD ITALIC CAPITAL Y" },
    { 0x1D481, "MATHEMATICAL BOLD ITALIC CAPITAL Z" },
    { 0x1D482, "MATHEMATICAL BOLD ITALIC SMALL A" },
    { 0x1D483, "MATHEMATICAL BOLD ITALIC SMALL B" },
    { 0x1D484, "MATHEMATICAL BOLD ITALIC SMALL C" },
    { 0x1D485, "MATHEMATICAL BOLD ITALIC SMALL D" },
    { 0x1D486, "MATHEMATICAL BOLD ITALIC SMALL E" },
    { 0x1D487, "MATHEMATICAL BOLD ITALIC SMALL F" },
    { 0x1D488, "MATHEMATICAL BOLD ITALIC SMALL G" },
    { 0x1D489, "MATHEMATICAL BOLD ITALIC SMALL H" },
    { 0x1D48A, "MATHEMATICAL BOLD ITALIC SMALL I" },
    { 0x1D48B, "MATHEMATICAL BOLD ITALIC SMALL J" },
    { 0x1D48C, "MATHEMATICAL BOLD ITALIC SMALL K" },
    { 0x1D48D, "MATHEMATICAL BOLD ITALIC SMALL L" },
    { 0x1D48E, "MATHEMATICAL BOLD ITALIC SMALL M" },
    { 0x1D48F, "MATHEMATICAL BOLD ITALIC SMALL N" },
    { 0x1D490, "MATHEMATICAL BOLD ITALIC SMALL O" },
    { 0x1D491, "MATHEMATICAL BOLD ITALIC SMALL P" },
    { 0x1D492, "MATHEMATICAL BOLD ITALIC SMALL Q" },
    { 0x1D493, "MATHEMATICAL BOLD ITALIC SMALL R" },
    { 0x1D494, "MATHEMATICAL BOLD ITALIC SMALL S" },
    { 0x1D495, "MATHEMATICAL BOLD ITALIC SMALL T" },
    { 0x1D496, "MATHEMATICAL BOLD ITALIC SMALL U" },
    { 0x1D497, "MATHEMATICAL BOLD ITALIC SMALL V" },
    { 0x1D498, "MATHEMATICAL BOLD ITALIC SMALL W" },
    { 0x1D499, "MATHEMATICAL BOLD ITALIC SMALL X" },
    { 0x1D49A, "MATHEMATICAL BOLD ITALIC SMALL Y" },
    { 0x1D49B, "MATHEMATICAL BOLD ITALIC SMALL Z" },
    { 0x1D49C, "MATHEMATICAL SCRIPT CAPITAL A" },
    { 0x1D49E, "MATHEMATICAL SCRIPT CAPITAL C" },
    { 0x1D49F, "MATHEMATICAL SCRIPT CAPITAL D" },
    { 0x1D4A2, "MATHEMATICAL SCRIPT CAPITAL G" },
    { 0x1D4A5, "MATHEMATICAL SCRIPT CAPITAL J" },
    { 0x1D4A6, "MATHEMATICAL SCRIPT CAPITAL K" },
    { 0x1D4A9, "MATHEMATICAL SCRIPT CAPITAL N" },
    { 0x1D4AA, "MATHEMATICAL SCRIPT CAPITAL O" },
    { 0x1D4AB, "MATHEMATICAL SCRIPT CAPITAL P" },
    { 0x1D4AC, "MATHEMATICAL SCRIPT CAPITAL Q" },
    { 0x1D4AE, "MATHEMATICAL SCRIPT CAPITAL S" },
    { 0x1D4AF, "MATHEMATICAL SCRIPT CAPITAL T" },
    { 0x1D4B0, "MATHEMATICAL SCRIPT CAPITAL U" },
    { 0x1D4B1, "MATHEMATICAL SCRIPT CAPITAL V" },
    { 0x1D4B2, "MATHEMATICAL SCRIPT CAPITAL W" },
    { 0x1D4B3, "MATHEMATICAL SCRIPT CAPITAL X" },
    { 0x1D4B4, "MATHEMATICAL SCRIPT CAPITAL Y" },
    { 0x1D4B5, "MATHEMATICAL SCRIPT CAPITAL Z" },
    { 0x1D4B6, "MATHEMATICAL SCRIPT SMALL A" },
    { 0x1D4B7, "MATHEMATICAL SCRIPT SMALL B" },
    { 0x1D4B8, "MATHEMATICAL SCRIPT SMALL C" },
    { 0x1D4B9, "MATHEMATICAL SCRIPT SMALL D" },
    { 0x1D4BB, "MATHEMATICAL SCRIPT SMALL F" },
    { 0x1D4BD, "MATHEMATICAL SCRIPT SMALL H" },
    { 0x1D4BE, "MATHEMATICAL SCRIPT SMALL I" },
    { 0x1D4BF, "MATHEMATICAL SCRIPT SMALL J" },
    { 0x1D4C0, "MATHEMATICAL SCRIPT SMALL K" },
    { 0x1D4C1, "MATHEMATICAL SCRIPT SMALL L" },
    { 0x1D4C2, "MATHEMATICAL SCRIPT SMALL M" },
    { 0x1D4C3, "MATHEMATICAL SCRIPT SMALL N" },
    { 0x1D4C5, "MATHEMATICAL SCRIPT SMALL P" },
    { 0x1D4C6, "MATHEMATICAL SCRIPT SMALL Q" },
    { 0x1D4C7, "MATHEMATICAL SCRIPT SMALL R" },
    { 0x1D4C8, "MATHEMATICAL SCRIPT SMALL S" },
    { 0x1D4C9, "MATHEMATICAL SCRIPT SMALL T" },
    { 0x1D4CA, "MATHEMATICAL SCRIPT SMALL U" },
    { 0x1D4CB, "MATHEMATICAL SCRIPT SMALL V" },
    { 0x1D4CC, "MATHEMATICAL SCRIPT SMALL W" },
    { 0x1D4CD, "MATHEMATICAL SCRIPT SMALL X" },
    { 0x1D4CE, "MATHEMATICAL SCRIPT SMALL Y" },
    { 0x1D4CF, "MATHEMATICAL SCRIPT SMALL Z" },
    { 0x1D4D0, "MATHEMATICAL BOLD SCRIPT CAPITAL A" },
    { 0x1D4D1, "MATHEMATICAL BOLD SCRIPT CAPITAL B" },
    { 0x1D4D2, "MATHEMATICAL BOLD SCRIPT CAPITAL C" },
    { 0x1D4D3, "MATHEMATICAL BOLD SCRIPT CAPITAL D" },
    { 0x1D4D4, "MATHEMATICAL BOLD SCRIPT CAPITAL E" },
    { 0x1D4D5, "MATHEMATICAL BOLD SCRIPT CAPITAL F" },
    { 0x1D4D6, "MATHEMATICAL BOLD SCRIPT CAPITAL G" },
    { 0x1D4D7, "MATHEMATICAL BOLD SCRIPT CAPITAL H" },
    { 0x1D4D8, "MATHEMATICAL BOLD SCRIPT CAPITAL I" },
    { 0x1D4D9, "MATHEMATICAL BOLD SCRIPT CAPITAL J" },
    { 0x1D4DA, "MATHEMATICAL BOLD SCRIPT CAPITAL K" },
    { 0x1D4DB, "MATHEMATICAL BOLD SCRIPT CAPITAL L" },
    { 0x1D4DC, "MATHEMATICAL BOLD SCRIPT CAPITAL M" },
    { 0x1D4DD, "MATHEMATICAL BOLD SCRIPT CAPITAL N" },
    { 0x1D4DE, "MATHEMATICAL BOLD SCRIPT CAPITAL O" },
    { 0x1D4DF, "MATHEMATICAL BOLD SCRIPT CAPITAL P" },
    { 0x1D4E0, "MATHEMATICAL BOLD SCRIPT CAPITAL Q" },
    { 0x1D4E1, "MATHEMATICAL BOLD SCRIPT CAPITAL R" },
    { 0x1D4E2, "MATHEMATICAL BOLD SCRIPT CAPITAL S" },
    { 0x1D4E3, "MATHEMATICAL BOLD SCRIPT CAPITAL T" },
    { 0x1D4E4, "MATHEMATICAL BOLD SCRIPT CAPITAL U" },
    { 0x1D4E5, "MATHEMATICAL BOLD SCRIPT CAPITAL V" },
    { 0x1D4E6, "MATHEMATICAL BOLD SCRIPT CAPITAL W" },
    { 0x1D4E7, "MATHEMATICAL BOLD SCRIPT CAPITAL X" },
    { 0x1D4E8, "MATHEMATICAL BOLD SCRIPT CAPITAL Y" },
    { 0x1D4E9, "MATHEMATICAL BOLD SCRIPT CAPITAL Z" },
    { 0x1D4EA, "MATHEMATICAL BOLD SCRIPT SMALL A" },
    { 0x1D4EB, "MATHEMATICAL BOLD SCRIPT SMALL B" },
    { 0x1D4EC, "MATHEMATICAL BOLD SCRIPT SMALL C" },
    { 0x1D4ED, "MATHEMATICAL BOLD SCRIPT SMALL D" },
    { 0x1D4EE, "MATHEMATICAL BOLD SCRIPT SMALL E" },
    { 0x1D4EF, "MATHEMATICAL BOLD SCRIPT SMALL F" },
    { 0x1D4F0, "MATHEMATICAL BOLD SCRIPT SMALL G" },
    { 0x1D4F1, "MATHEMATICAL BOLD SCRIPT SMALL H" },
    { 0x1D4F2, "MATHEMATICAL BOLD SCRIPT SMALL I" },
    { 0x1D4F3, "MATHEMATICAL BOLD SCRIPT SMALL J" },
    { 0x1D4F4, "MATHEMATICAL BOLD SCRIPT SMALL K" },
    { 0x1D4F5, "MATHEMATICAL BOLD SCRIPT SMALL L" },
    { 0x1D4F6, "MATHEMATICAL BOLD SCRIPT SMALL M" },
    { 0x1D4F7, "MATHEMATICAL BOLD SCRIPT SMALL N" },
    { 0x1D4F8, "MATHEMATICAL BOLD SCRIPT SMALL O" },
    { 0x1D4F9, "MATHEMATICAL BOLD SCRIPT SMALL P" },
    { 0x1D4FA, "MATHEMATICAL BOLD SCRIPT SMALL Q" },
    { 0x1D4FB, "MATHEMATICAL BOLD SCRIPT SMALL R" },
    { 0x1D4FC, "MATHEMATICAL BOLD SCRIPT SMALL S" },
    { 0x1D4FD, "MATHEMATICAL BOLD SCRIPT SMALL T" },
    { 0x1D4FE, "MATHEMATICAL BOLD SCRIPT SMALL U" },
    { 0x1D4FF, "MATHEMATICAL BOLD SCRIPT SMALL V" },
    { 0x1D500, "MATHEMATICAL BOLD SCRIPT SMALL W" },
    { 0x1D501, "MATHEMATICAL BOLD SCRIPT SMALL X" },
    { 0x1D502, "MATHEMATICAL BOLD SCRIPT SMALL Y" },
    { 0x1D503, "MATHEMATICAL BOLD SCRIPT SMALL Z" },
    { 0x1D504, "MATHEMATICAL FRAKTUR CAPITAL A" },
    { 0x1D505, "MATHEMATICAL FRAKTUR CAPITAL B" },
    { 0x1D507, "MATHEMATICAL FRAKTUR CAPITAL D" },
    { 0x1D508, "MATHEMATICAL FRAKTUR CAPITAL E" },
    { 0x1D509, "MATHEMATICAL FRAKTUR CAPITAL F" },
    { 0x1D50A, "MATHEMATICAL FRAKTUR CAPITAL G" },
    { 0x1D50D, "MATHEMATICAL FRAKTUR CAPITAL J" },
    { 0x1D50E, "MATHEMATICAL FRAKTUR CAPITAL K" },
    { 0x1D50F, "MATHEMATICAL FRAKTUR CAPITAL L" },
    { 0x1D510, "MATHEMATICAL FRAKTUR CAPITAL M" },
    { 0x1D511, "MATHEMATICAL FRAKTUR CAPITAL N" },
    { 0x1D512, "MATHEMATICAL FRAKTUR CAPITAL O" },
    { 0x1D513, "MATHEMATICAL FRAKTUR CAPITAL P" },
    { 0x1D514, "MATHEMATICAL FRAKTUR CAPITAL Q" },
    { 0x1D516, "MATHEMATICAL FRAKTUR CAPITAL S" },
    { 0x1D517, "MATHEMATICAL FRAKTUR CAPITAL T" },
    { 0x1D518, "MATHEMATICAL FRAKTUR CAPITAL U" },
    { 0x1D519, "MATHEMATICAL FRAKTUR CAPITAL V" },
    { 0x1D51A, "MATHEMATICAL FRAKTUR CAPITAL W" },
    { 0x1D51B, "MATHEMATICAL FRAKTUR CAPITAL X" },
    { 0x1D51C, "MATHEMATICAL FRAKTUR CAPITAL Y" },
    { 0x1D51E, "MATHEMATICAL FRAKTUR SMALL A" },
    { 0x1D51F, "MATHEMATICAL FRAKTUR SMALL B" },
    { 0x1D520, "MATHEMATICAL FRAKTUR SMALL C" },
    { 0x1D521, "MATHEMATICAL FRAKTUR SMALL D" },
    { 0x1D522, "MATHEMATICAL FRAKTUR SMALL E" },
    { 0x1D523, "MATHEMATICAL FRAKTUR SMALL F" },
    { 0x1D524, "MATHEMATICAL FRAKTUR SMALL G" },
    { 0x1D525, "MATHEMATICAL FRAKTUR SMALL H" },
    { 0x1D526, "MATHEMATICAL FRAKTUR SMALL I" },
    { 0x1D527, "MATHEMATICAL FRAKTUR SMALL J" },
    { 0x1D528, "MATHEMATICAL FRAKTUR SMALL K" },
    { 0x1D529, "MATHEMATICAL FRAKTUR SMALL L" },
    { 0x1D52A, "MATHEMATICAL FRAKTUR SMALL M" },
    { 0x1D52B, "MATHEMATICAL FRAKTUR SMALL N" },
    { 0x1D52C, "MATHEMATICAL FRAKTUR SMALL O" },
    { 0x1D52D, "MATHEMATICAL FRAKTUR SMALL P" },
    { 0x1D52E, "MATHEMATICAL FRAKTUR SMALL Q" },
    { 0x1D52F, "MATHEMATICAL FRAKTUR SMALL R" },
    { 0x1D530, "MATHEMATICAL FRAKTUR SMALL S" },
    { 0x1D531, "MATHEMATICAL FRAKTUR SMALL T" },
    { 0x1D532, "MATHEMATICAL FRAKTUR SMALL U" },
    { 0x1D533, "MATHEMATICAL FRAKTUR SMALL V" },
    { 0x1D534, "MATHEMATICAL FRAKTUR SMALL W" },
    { 0x1D535, "MATHEMATICAL FRAKTUR SMALL X" },
    { 0x1D536, "MATHEMATICAL FRAKTUR SMALL Y" },
    { 0x1D537, "MATHEMATICAL FRAKTUR SMALL Z" },
    { 0x1D538, "MATHEMATICAL DOUBLE-STRUCK CAPITAL A" },
    { 0x1D539, "MATHEMATICAL DOUBLE-STRUCK CAPITAL B" },
    { 0x1D53B, "MATHEMATICAL DOUBLE-STRUCK CAPITAL D" },
    { 0x1D53C, "MATHEMATICAL DOUBLE-STRUCK CAPITAL E" },
    { 0x1D53D, "MATHEMATICAL DOUBLE-STRUCK CAPITAL F" },
    { 0x1D53E, "MATHEMATICAL DOUBLE-STRUCK CAPITAL G" },
    { 0x1D540, "MATHEMATICAL DOUBLE-STRUCK CAPITAL I" },
    { 0x1D541, "MATHEMATICAL DOUBLE-STRUCK CAPITAL J" },
    { 0x1D542, "MATHEMATICAL DOUBLE-STRUCK CAPITAL K" },
    { 0x1D543, "MATHEMATICAL DOUBLE-STRUCK CAPITAL L" },
    { 0x1D544, "MATHEMATICAL DOUBLE-STRUCK CAPITAL M" },
    { 0x1D546, "MATHEMATICAL DOUBLE-STRUCK CAPITAL O" },
    { 0x1D54A, "MATHEMATICAL DOUBLE-STRUCK CAPITAL S" },
    { 0x1D54B, "MATHEMATICAL DOUBLE-STRUCK CAPITAL T" },
    { 0x1D54C, "MATHEMATICAL DOUBLE-STRUCK CAPITAL U" },
    { 0x1D54D, "MATHEMATICAL DOUBLE-STRUCK CAPITAL V" },
    { 0x1D54E, "MATHEMATICAL DOUBLE-STRUCK CAPITAL W" },
    { 0x1D54F, "MATHEMATICAL DOUBLE-STRUCK CAPITAL X" },
    { 0x1D550, "MATHEMATICAL DOUBLE-STRUCK CAPITAL Y" },
    { 0x1D552, "MATHEMATICAL DOUBLE-STRUCK SMALL A" },
    { 0x1D553, "MATHEMATICAL DOUBLE-STRUCK SMALL B" },
    { 0x1D554, "MATHEMATICAL DOUBLE-STRUCK SMALL C" },
    { 0x1D555, "MATHEMATICAL DOUBLE-STRUCK SMALL D" },
    { 0x1D556, "MATHEMATICAL DOUBLE-STRUCK SMALL E" },
    { 0x1D557, "MATHEMATICAL DOUBLE-STRUCK SMALL F" },
    { 0x1D558, "MATHEMATICAL DOUBLE-STRUCK SMALL G" },
    { 0x1D559, "MATHEMATICAL DOUBLE-STRUCK SMALL H" },
    { 0x1D55A, "MATHEMATICAL DOUBLE-STRUCK SMALL I" },
    { 0x1D55B, "MATHEMATICAL DOUBLE-STRUCK SMALL J" },
    { 0x1D55C, "MATHEMATICAL DOUBLE-STRUCK SMALL K" },
    { 0x1D55D, "MATHEMATICAL DOUBLE-STRUCK SMALL L" },
    { 0x1D55E, "MATHEMATICAL DOUBLE-STRUCK SMALL M" },
    { 0x1D55F, "MATHEMATICAL DOUBLE-STRUCK SMALL N" },
    { 0x1D560, "MATHEMATICAL DOUBLE-STRUCK SMALL O" },
    { 0x1D561, "MATHEMATICAL DOUBLE-STRUCK SMALL P" },
    { 0x1D562, "MATHEMATICAL DOUBLE-STRUCK SMALL Q" },
    { 0x1D563, "MATHEMATICAL DOUBLE-STRUCK SMALL R" },
    { 0x1D564, "MATHEMATICAL DOUBLE-STRUCK SMALL S" },
    { 0x1D565, "MATHEMATICAL DOUBLE-STRUCK SMALL T" },
    { 0x1D566, "MATHEMATICAL DOUBLE-STRUCK SMALL U" },
    { 0x1D567, "MATHEMATICAL DOUBLE-STRUCK SMALL V" },
    { 0x1D568, "MATHEMATICAL DOUBLE-STRUCK SMALL W" },
    { 0x1D569, "MATHEMATICAL DOUBLE-STRUCK SMALL X" },
    { 0x1D56A, "MATHEMATICAL DOUBLE-STRUCK SMALL Y" },
    { 0x1D56B, "MATHEMATICAL DOUBLE-STRUCK SMALL Z" },
    { 0x1D56C, "MATHEMATICAL BOLD FRAKTUR CAPITAL A" },
    { 0x1D56D, "MATHEMATICAL BOLD FRAKTUR CAPITAL B" },
    { 0x1D56E, "MATHEMATICAL BOLD FRAKTUR CAPITAL C" },
    { 0x1D56F, "MATHEMATICAL BOLD FRAKTUR CAPITAL D" },
    { 0x1D570, "MATHEMATICAL BOLD FRAKTUR CAPITAL E" },
    { 0x1D571, "MATHEMATICAL BOLD FRAKTUR CAPITAL F" },
    { 0x1D572, "MATHEMATICAL BOLD FRAKTUR CAPITAL G" },
    { 0x1D573, "MATHEMATICAL BOLD FRAKTUR CAPITAL H" },
    { 0x1D574, "MATHEMATICAL BOLD FRAKTUR CAPITAL I" },
    { 0x1D575, "MATHEMATICAL BOLD FRAKTUR CAPITAL J" },
    { 0x1D576, "MATHEMATICAL BOLD FRAKTUR CAPITAL K" },
    { 0x1D577, "MATHEMATICAL BOLD FRAKTUR CAPITAL L" },
    { 0x1D578, "MATHEMATICAL BOLD FRAKTUR CAPITAL M" },
    { 0x1D579, "MATHEMATICAL BOLD FRAKTUR CAPITAL N" },
    { 0x1D57A, "MATHEMATICAL BOLD FRAKTUR CAPITAL O" },
    { 0x1D57B, "MATHEMATICAL BOLD FRAKTUR CAPITAL P" },
    { 0x1D57C, "MATHEMATICAL BOLD FRAKTUR CAPITAL Q" },
    { 0x1D57D, "MATHEMATICAL BOLD FRAKTUR CAPITAL R" },
    { 0x1D57E, "MATHEMATICAL BOLD FRAKTUR CAPITAL S" },
    { 0x1D57F, "MATHEMATICAL BOLD FRAKTUR CAPITAL T" },
    { 0x1D580, "MATHEMATICAL BOLD FRAKTUR CAPITAL U" },
    { 0x1D581, "MATHEMATICAL BOLD FRAKTUR CAPITAL V" },
    { 0x1D582, "MATHEMATICAL BOLD FRAKTUR CAPITAL W" },
    { 0x1D583, "MATHEMATICAL BOLD FRAKTUR CAPITAL X" },
    { 0x1D584, "MATHEMATICAL BOLD FRAKTUR CAPITAL Y" },
    { 0x1D585, "MATHEMATICAL BOLD FRAKTUR CAPITAL Z" },
    { 0x1D586, "MATHEMATICAL BOLD FRAKTUR SMALL A" },
    { 0x1D587, "MATHEMATICAL BOLD FRAKTUR SMALL B" },
    { 0x1D588, "MATHEMATICAL BOLD FRAKTUR SMALL C" },
    { 0x1D589, "MATHEMATICAL BOLD FRAKTUR SMALL D" },
    { 0x1D58A, "MATHEMATICAL BOLD FRAKTUR SMALL E" },
    { 0x1D58B, "MATHEMATICAL BOLD FRAKTUR SMALL F" },
    { 0x1D58C, "MATHEMATICAL BOLD FRAKTUR SMALL G" },
    { 0x1D58D, "MATHEMATICAL BOLD FRAKTUR SMALL H" },
    { 0x1D58E, "MATHEMATICAL BOLD FRAKTUR SMALL I" },
    { 0x1D58F, "MATHEMATICAL BOLD FRAKTUR SMALL J" },
    { 0x1D590, "MATHEMATICAL BOLD FRAKTUR SMALL K" },
    { 0x1D591, "MATHEMATICAL BOLD FRAKTUR SMALL L" },
    { 0x1D592, "MATHEMATICAL BOLD FRAKTUR SMALL M" },
    { 0x1D593, "MATHEMATICAL BOLD FRAKTUR SMALL N" },
    { 0x1D594, "MATHEMATICAL BOLD FRAKTUR SMALL O" },
    { 0x1D595, "MATHEMATICAL BOLD FRAKTUR SMALL P" },
    { 0x1D596, "MATHEMATICAL BOLD FRAKTUR SMALL Q" },
    { 0x1D597, "MATHEMATICAL BOLD FRAKTUR SMALL R" },
    { 0x1D598, "MATHEMATICAL BOLD FRAKTUR SMALL S" },
    { 0x1D599, "MATHEMATICAL BOLD FRAKTUR SMALL T" },
    { 0x1D59A, "MATHEMATICAL BOLD FRAKTUR SMALL U" },
    { 0x1D59B, "MATHEMATICAL BOLD FRAKTUR SMALL V" },
    { 0x1D59C, "MATHEMATICAL BOLD FRAKTUR SMALL W" },
    { 0x1D59D, "MATHEMATICAL BOLD FRAKTUR SMALL X" },
    { 0x1D59E, "MATHEMATICAL BOLD FRAKTUR SMALL Y" },
    { 0x1D59F, "MATHEMATICAL BOLD FRAKTUR SMALL Z" },
    { 0x1D5A0, "MATHEMATICAL SANS-SERIF CAPITAL A" },
    { 0x1D5A1, "MATHEMATICAL SANS-SERIF CAPITAL B" },
    { 0x1D5A2, "MATHEMATICAL SANS-SERIF CAPITAL C" },
    { 0x1D5A3, "MATHEMATICAL SANS-SERIF CAPITAL D" },
    { 0x1D5A4, "MATHEMATICAL SANS-SERIF CAPITAL E" },
    { 0x1D5A5, "MATHEMATICAL SANS-SERIF CAPITAL F" },
    { 0x1D5A6, "MATHEMATICAL SANS-SERIF CAPITAL G" },
    { 0x1D5A7, "MATHEMATICAL SANS-SERIF CAPITAL H" },
    { 0x1D5A8, "MATHEMATICAL SANS-SERIF CAPITAL I" },
    { 0x1D5A9, "MATHEMATICAL SANS-SERIF CAPITAL J" },
    { 0x1D5AA, "MATHEMATICAL SANS-SERIF CAPITAL K" },
    { 0x1D5AB, "MATHEMATICAL SANS-SERIF CAPITAL L" },
    { 0x1D5AC, "MATHEMATICAL SANS-SERIF CAPITAL M" },
    { 0x1D5AD, "MATHEMATICAL SANS-SERIF CAPITAL N" },
    { 0x1D5AE, "MATHEMATICAL SANS-SERIF CAPITAL O" },
    { 0x1D5AF, "MATHEMATICAL SANS-SERIF CAPITAL P" },
    { 0x1D5B0, "MATHEMATICAL SANS-SERIF CAPITAL Q" },
    { 0x1D5B1, "MATHEMATICAL SANS-SERIF CAPITAL R" },
    { 0x1D5B2, "MATHEMATICAL SANS-SERIF CAPITAL S" },
    { 0x1D5B3, "MATHEMATICAL SANS-SERIF CAPITAL T" },
    { 0x1D5B4, "MATHEMATICAL SANS-SERIF CAPITAL U" },
    { 0x1D5B5, "MATHEMATICAL SANS-SERIF CAPITAL V" },
    { 0x1D5B6, "MATHEMATICAL SANS-SERIF CAPITAL W" },
    { 0x1D5B7, "MATHEMATICAL SANS-SERIF CAPITAL X" },
    { 0x1D5B8, "MATHEMATICAL SANS-SERIF CAPITAL Y" },
    { 0x1D5B9, "MATHEMATICAL SANS-SERIF CAPITAL Z" },
    { 0x1D5BA, "MATHEMATICAL SANS-SERIF SMALL A" },
    { 0x1D5BB, "MATHEMATICAL SANS-SERIF SMALL B" },
    { 0x1D5BC, "MATHEMATICAL SANS-SERIF SMALL C" },
    { 0x1D5BD, "MATHEMATICAL SANS-SERIF SMALL D" },
    { 0x1D5BE, "MATHEMATICAL SANS-SERIF SMALL E" },
    { 0x1D5BF, "MATHEMATICAL SANS-SERIF SMALL F" },
    { 0x1D5C0, "MATHEMATICAL SANS-SERIF SMALL G" },
    { 0x1D5C1, "MATHEMATICAL SANS-SERIF SMALL H" },
    { 0x1D5C2, "MATHEMATICAL SANS-SERIF SMALL I" },
    { 0x1D5C3, "MATHEMATICAL SANS-SERIF SMALL J" },
    { 0x1D5C4, "MATHEMATICAL SANS-SERIF SMALL K" },
    { 0x1D5C5, "MATHEMATICAL SANS-SERIF SMALL L" },
    { 0x1D5C6, "MATHEMATICAL SANS-SERIF SMALL M" },
    { 0x1D5C7, "MATHEMATICAL SANS-SERIF SMALL N" },
    { 0x1D5C8, "MATHEMATICAL SANS-SERIF SMALL O" },
    { 0x1D5C9, "MATHEMATICAL SANS-SERIF SMALL P" },
    { 0x1D5CA, "MATHEMATICAL SANS-SERIF SMALL Q" },
    { 0x1D5CB, "MATHEMATICAL SANS-SERIF SMALL R" },
    { 0x1D5CC, "MATHEMATICAL SANS-SERIF SMALL S" },
    { 0x1D5CD, "MATHEMATICAL SANS-SERIF SMALL T" },
    { 0x1D5CE, "MATHEMATICAL SANS-SERIF SMALL U" },
    { 0x1D5CF, "MATHEMATICAL SANS-SERIF SMALL V" },
    { 0x1D5D0, "MATHEMATICAL SANS-SERIF SMALL W" },
    { 0x1D5D1, "MATHEMATICAL SANS-SERIF SMALL X" },
    { 0x1D5D2, "MATHEMATICAL SANS-SERIF SMALL Y" },
    { 0x1D5D3, "MATHEMATICAL SANS-SERIF SMALL Z" },
    { 0x1D5D4, "MATHEMATICAL SANS-SERIF BOLD CAPITAL A" },
    { 0x1D5D5, "MATHEMATICAL SANS-SERIF BOLD CAPITAL B" },
    { 0x1D5D6, "MATHEMATICAL SANS-SERIF BOLD CAPITAL C" },
    { 0x1D5D7, "MATHEMATICAL SANS-SERIF BOLD CAPITAL D" },
    { 0x1D5D8, "MATHEMATICAL SANS-SERIF BOLD CAPITAL E" },
    { 0x1D5D9, "MATHEMATICAL SANS-SERIF BOLD CAPITAL F" },
    { 0x1D5DA, "MATHEMATICAL SANS-SERIF BOLD CAPITAL G" },
    { 0x1D5DB, "MATHEMATICAL SANS-SERIF BOLD CAPITAL H" },
    { 0x1D5DC, "MATHEMATICAL SANS-SERIF BOLD CAPITAL I" },
    { 0x1D5DD, "MATHEMATICAL SANS-SERIF BOLD CAPITAL J" },
    { 0x1D5DE, "MATHEMATICAL SANS-SERIF BOLD CAPITAL K" },
    { 0x1D5DF, "MATHEMATICAL SANS-SERIF BOLD CAPITAL L" },
    { 0x1D5E0, "MATHEMATICAL SANS-SERIF BOLD CAPITAL M" },
    { 0x1D5E1, "MATHEMATICAL SANS-SERIF BOLD CAPITAL N" },
    { 0x1D5E2, "MATHEMATICAL SANS-SERIF BOLD CAPITAL O" },
    { 0x1D5E3, "MATHEMATICAL SANS-SERIF BOLD CAPITAL P" },
    { 0x1D5E4, "MATHEMATICAL SANS-SERIF BOLD CAPITAL Q" },
    { 0x1D5E5, "MATHEMATICAL SANS-SERIF BOLD CAPITAL R" },
    { 0x1D5E6, "MATHEMATICAL SANS-SERIF BOLD CAPITAL S" },
    { 0x1D5E7, "MATHEMATICAL SANS-SERIF BOLD CAPITAL T" },
    { 0x1D5E8, "MATHEMATICAL SANS-SERIF BOLD CAPITAL U" },
    { 0x1D5E9, "MATHEMATICAL SANS-SERIF BOLD CAPITAL V" },
    { 0x1D5EA, "MATHEMATICAL SANS-SERIF BOLD CAPITAL W" },
    { 0x1D5EB, "MATHEMATICAL SANS-SERIF BOLD CAPITAL X" },
    { 0x1D5EC, "MATHEMATICAL SANS-SERIF BOLD CAPITAL Y" },
    { 0x1D5ED, "MATHEMATICAL SANS-SERIF BOLD CAPITAL Z" },
    { 0x1D5EE, "MATHEMATICAL SANS-SERIF BOLD SMALL A" },
    { 0x1D5EF, "MATHEMATICAL SANS-SERIF BOLD SMALL B" },
    { 0x1D5F0, "MATHEMATICAL SANS-SERIF BOLD SMALL C" },
    { 0x1D5F1, "MATHEMATICAL SANS-SERIF BOLD SMALL D" },
    { 0x1D5F2, "MATHEMATICAL SANS-SERIF BOLD SMALL E" },
    { 0x1D5F3, "MATHEMATICAL SANS-SERIF BOLD SMALL F" },
    { 0x1D5F4, "MATHEMATICAL SANS-SERIF BOLD SMALL G" },
    { 0x1D5F5, "MATHEMATICAL SANS-SERIF BOLD SMALL H" },
    { 0x1D5F6, "MATHEMATICAL SANS-SERIF BOLD SMALL I" },
    { 0x1D5F7, "MATHEMATICAL SANS-SERIF BOLD SMALL J" },
    { 0x1D5F8, "MATHEMATICAL SANS-SERIF BOLD SMALL K" },
    { 0x1D5F9, "MATHEMATICAL SANS-SERIF BOLD SMALL L" },
    { 0x1D5FA, "MATHEMATICAL SANS-SERIF BOLD SMALL M" },
    { 0x1D5FB, "MATHEMATICAL SANS-SERIF BOLD SMALL N" },
    { 0x1D5FC, "MATHEMATICAL SANS-SERIF BOLD SMALL O" },
    { 0x1D5FD, "MATHEMATICAL SANS-SERIF BOLD SMALL P" },
    { 0x1D5FE, "MATHEMATICAL SANS-SERIF BOLD SMALL Q" },
    { 0x1D5FF, "MATHEMATICAL SANS-SERIF BOLD SMALL R" },
    { 0x1D600, "MATHEMATICAL SANS-SERIF BOLD SMALL S" },
    { 0x1D601, "MATHEMATICAL SANS-SERIF BOLD SMALL T" },
    { 0x1D602, "MATHEMATICAL SANS-SERIF BOLD SMALL U" },
    { 0x1D603, "MATHEMATICAL SANS-SERIF BOLD SMALL V" },
    { 0x1D604, "MATHEMATICAL SANS-SERIF BOLD SMALL W" },
    { 0x1D605, "MATHEMATICAL SANS-SERIF BOLD SMALL X" },
    { 0x1D606, "MATHEMATICAL SANS-SERIF BOLD SMALL Y" },
    { 0x1D607, "MATHEMATICAL SANS-SERIF BOLD SMALL Z" },
    { 0x1D608, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL A" },
    { 0x1D609, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL B" },
    { 0x1D60A, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL C" },
    { 0x1D60B, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL D" },
    { 0x1D60C, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL E" },
    { 0x1D60D, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL F" },
    { 0x1D60E, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL G" },
    { 0x1D60F, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL H" },
    { 0x1D610, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL I" },
    { 0x1D611, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL J" },
    { 0x1D612, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL K" },
    { 0x1D613, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL L" },
    { 0x1D614, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL M" },
    { 0x1D615, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL N" },
    { 0x1D616, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL O" },
    { 0x1D617, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL P" },
    { 0x1D618, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL Q" },
    { 0x1D619, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL R" },
    { 0x1D61A, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL S" },
    { 0x1D61B, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL T" },
    { 0x1D61C, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL U" },
    { 0x1D61D, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL V" },
    { 0x1D61E, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL W" },
    { 0x1D61F, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL X" },
    { 0x1D620, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL Y" },
    { 0x1D621, "MATHEMATICAL SANS-SERIF ITALIC CAPITAL Z" },
    { 0x1D622, "MATHEMATICAL SANS-SERIF ITALIC SMALL A" },
    { 0x1D623, "MATHEMATICAL SANS-SERIF ITALIC SMALL B" },
    { 0x1D624, "MATHEMATICAL SANS-SERIF ITALIC SMALL C" },
    { 0x1D625, "MATHEMATICAL SANS-SERIF ITALIC SMALL D" },
    { 0x1D626, "MATHEMATICAL SANS-SERIF ITALIC SMALL E" },
    { 0x1D627, "MATHEMATICAL SANS-SERIF ITALIC SMALL F" },
    { 0x1D628, "MATHEMATICAL SANS-SERIF ITALIC SMALL G" },
    { 0x1D629, "MATHEMATICAL SANS-SERIF ITALIC SMALL H" },
    { 0x1D62A, "MATHEMATICAL SANS-SERIF ITALIC SMALL I" },
    { 0x1D62B, "MATHEMATICAL SANS-SERIF ITALIC SMALL J" },
    { 0x1D62C, "MATHEMATICAL SANS-SERIF ITALIC SMALL K" },
    { 0x1D62D, "MATHEMATICAL SANS-SERIF ITALIC SMALL L" },
    { 0x1D62E, "MATHEMATICAL SANS-SERIF ITALIC SMALL M" },
    { 0x1D62F, "MATHEMATICAL SANS-SERIF ITALIC SMALL N" },
    { 0x1D630, "MATHEMATICAL SANS-SERIF ITALIC SMALL O" },
    { 0x1D631, "MATHEMATICAL SANS-SERIF ITALIC SMALL P" },
    { 0x1D632, "MATHEMATICAL SANS-SERIF ITALIC SMALL Q" },
    { 0x1D633, "MATHEMATICAL SANS-SERIF ITALIC SMALL R" },
    { 0x1D634, "MATHEMATICAL SANS-SERIF ITALIC SMALL S" },
    { 0x1D635, "MATHEMATICAL SANS-SERIF ITALIC SMALL T" },
    { 0x1D636, "MATHEMATICAL SANS-SERIF ITALIC SMALL U" },
    { 0x1D637, "MATHEMATICAL SANS-SERIF ITALIC SMALL V" },
    { 0x1D638, "MATHEMATICAL SANS-SERIF ITALIC SMALL W" },
    { 0x1D639, "MATHEMATICAL SANS-SERIF ITALIC SMALL X" },
    { 0x1D63A, "MATHEMATICAL SANS-SERIF ITALIC SMALL Y" },
    { 0x1D63B, "MATHEMATICAL SANS-SERIF ITALIC SMALL Z" },
    { 0x1D63C, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL A" },
    { 0x1D63D, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL B" },
    { 0x1D63E, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL C" },
    { 0x1D63F, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL D" },
    { 0x1D640, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL E" },
    { 0x1D641, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL F" },
    { 0x1D642, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL G" },
    { 0x1D643, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL H" },
    { 0x1D644, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL I" },
    { 0x1D645, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL J" },
    { 0x1D646, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL K" },
    { 0x1D647, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL L" },
    { 0x1D648, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL M" },
    { 0x1D649, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL N" },
    { 0x1D64A, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL O" },
    { 0x1D64B, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL P" },
    { 0x1D64C, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Q" },
    { 0x1D64D, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL R" },
    { 0x1D64E, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL S" },
    { 0x1D64F, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL T" },
    { 0x1D650, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL U" },
    { 0x1D651, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL V" },
    { 0x1D652, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL W" },
    { 0x1D653, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL X" },
    { 0x1D654, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Y" },
    { 0x1D655, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Z" },
    { 0x1D656, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL A" },
    { 0x1D657, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL B" },
    { 0x1D658, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL C" },
    { 0x1D659, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL D" },
    { 0x1D65A, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL E" },
    { 0x1D65B, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL F" },
    { 0x1D65C, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL G" },
    { 0x1D65D, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL H" },
    { 0x1D65E, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL I" },
    { 0x1D65F, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL J" },
    { 0x1D660, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL K" },
    { 0x1D661, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL L" },
    { 0x1D662, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL M" },
    { 0x1D663, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL N" },
    { 0x1D664, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL O" },
    { 0x1D665, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL P" },
    { 0x1D666, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Q" },
    { 0x1D667, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL R" },
    { 0x1D668, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL S" },
    { 0x1D669, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL T" },
    { 0x1D66A, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL U" },
    { 0x1D66B, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL V" },
    { 0x1D66C, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL W" },
    { 0x1D66D, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL X" },
    { 0x1D66E, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Y" },
    { 0x1D66F, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Z" },
    { 0x1D670, "MATHEMATICAL MONOSPACE CAPITAL A" },
    { 0x1D671, "MATHEMATICAL MONOSPACE CAPITAL B" },
    { 0x1D672, "MATHEMATICAL MONOSPACE CAPITAL C" },
    { 0x1D673, "MATHEMATICAL MONOSPACE CAPITAL D" },
    { 0x1D674, "MATHEMATICAL MONOSPACE CAPITAL E" },
    { 0x1D675, "MATHEMATICAL MONOSPACE CAPITAL F" },
    { 0x1D676, "MATHEMATICAL MONOSPACE CAPITAL G" },
    { 0x1D677, "MATHEMATICAL MONOSPACE CAPITAL H" },
    { 0x1D678, "MATHEMATICAL MONOSPACE CAPITAL I" },
    { 0x1D679, "MATHEMATICAL MONOSPACE CAPITAL J" },
    { 0x1D67A, "MATHEMATICAL MONOSPACE CAPITAL K" },
    { 0x1D67B, "MATHEMATICAL MONOSPACE CAPITAL L" },
    { 0x1D67C, "MATHEMATICAL MONOSPACE CAPITAL M" },
    { 0x1D67D, "MATHEMATICAL MONOSPACE CAPITAL N" },
    { 0x1D67E, "MATHEMATICAL MONOSPACE CAPITAL O" },
    { 0x1D67F, "MATHEMATICAL MONOSPACE CAPITAL P" },
    { 0x1D680, "MATHEMATICAL MONOSPACE CAPITAL Q" },
    { 0x1D681, "MATHEMATICAL MONOSPACE CAPITAL R" },
    { 0x1D682, "MATHEMATICAL MONOSPACE CAPITAL S" },
    { 0x1D683, "MATHEMATICAL MONOSPACE CAPITAL T" },
    { 0x1D684, "MATHEMATICAL MONOSPACE CAPITAL U" },
    { 0x1D685, "MATHEMATICAL MONOSPACE CAPITAL V" },
    { 0x1D686, "MATHEMATICAL MONOSPACE CAPITAL W" },
    { 0x1D687, "MATHEMATICAL MONOSPACE CAPITAL X" },
    { 0x1D688, "MATHEMATICAL MONOSPACE CAPITAL Y" },
    { 0x1D689, "MATHEMATICAL MONOSPACE CAPITAL Z" },
    { 0x1D68A, "MATHEMATICAL MONOSPACE SMALL A" },
    { 0x1D68B, "MATHEMATICAL MONOSPACE SMALL B" },
    { 0x1D68C, "MATHEMATICAL MONOSPACE SMALL C" },
    { 0x1D68D, "MATHEMATICAL MONOSPACE SMALL D" },
    { 0x1D68E, "MATHEMATICAL MONOSPACE SMALL E" },
    { 0x1D68F, "MATHEMATICAL MONOSPACE SMALL F" },
    { 0x1D690, "MATHEMATICAL MONOSPACE SMALL G" },
    { 0x1D691, "MATHEMATICAL MONOSPACE SMALL H" },
    { 0x1D692, "MATHEMATICAL MONOSPACE SMALL I" },
    { 0x1D693, "MATHEMATICAL MONOSPACE SMALL J" },
    { 0x1D694, "MATHEMATICAL MONOSPACE SMALL K" },
    { 0x1D695, "MATHEMATICAL MONOSPACE SMALL L" },
    { 0x1D696, "MATHEMATICAL MONOSPACE SMALL M" },
    { 0x1D697, "MATHEMATICAL MONOSPACE SMALL N" },
    { 0x1D698, "MATHEMATICAL MONOSPACE SMALL O" },
    { 0x1D699, "MATHEMATICAL MONOSPACE SMALL P" },
    { 0x1D69A, "MATHEMATICAL MONOSPACE SMALL Q" },
    { 0x1D69B, "MATHEMATICAL MONOSPACE SMALL R" },
    { 0x1D69C, "MATHEMATICAL MONOSPACE SMALL S" },
    { 0x1D69D, "MATHEMATICAL MONOSPACE SMALL T" },
    { 0x1D69E, "MATHEMATICAL MONOSPACE SMALL U" },
    { 0x1D69F, "MATHEMATICAL MONOSPACE SMALL V" },
    { 0x1D6A0, "MATHEMATICAL MONOSPACE SMALL W" },
    { 0x1D6A1, "MATHEMATICAL MONOSPACE SMALL X" },
    { 0x1D6A2, "MATHEMATICAL MONOSPACE SMALL Y" },
    { 0x1D6A3, "MATHEMATICAL MONOSPACE SMALL Z" },
    { 0x1D6A4, "MATHEMATICAL ITALIC SMALL DOTLESS I" },
    { 0x1D6A5, "MATHEMATICAL ITALIC SMALL DOTLESS J" },
    { 0x1D6A8, "MATHEMATICAL BOLD CAPITAL ALPHA" },
    { 0x1D6A9, "MATHEMATICAL BOLD CAPITAL BETA" },
    { 0x1D6AA, "MATHEMATICAL BOLD CAPITAL GAMMA" },
    { 0x1D6AB, "MATHEMATICAL BOLD CAPITAL DELTA" },
    { 0x1D6AC, "MATHEMATICAL BOLD CAPITAL EPSILON" },
    { 0x1D6AD, "MATHEMATICAL BOLD CAPITAL ZETA" },
    { 0x1D6AE, "MATHEMATICAL BOLD CAPITAL ETA" },
    { 0x1D6AF, "MATHEMATICAL BOLD CAPITAL THETA" },
    { 0x1D6B0, "MATHEMATICAL BOLD CAPITAL IOTA" },
    { 0x1D6B1, "MATHEMATICAL BOLD CAPITAL KAPPA" },
    { 0x1D6B2, "MATHEMATICAL BOLD CAPITAL LAMDA" },
    { 0x1D6B3, "MATHEMATICAL BOLD CAPITAL MU" },
    { 0x1D6B4, "MATHEMATICAL BOLD CAPITAL NU" },
    { 0x1D6B5, "MATHEMATICAL BOLD CAPITAL XI" },
    { 0x1D6B6, "MATHEMATICAL BOLD CAPITAL OMICRON" },
    { 0x1D6B7, "MATHEMATICAL BOLD CAPITAL PI" },
    { 0x1D6B8, "MATHEMATICAL BOLD CAPITAL RHO" },
    { 0x1D6B9, "MATHEMATICAL BOLD CAPITAL THETA SYMBOL" },
    { 0x1D6BA, "MATHEMATICAL BOLD CAPITAL SIGMA" },
    { 0x1D6BB, "MATHEMATICAL BOLD CAPITAL TAU" },
    { 0x1D6BC, "MATHEMATICAL BOLD CAPITAL UPSILON" },
    { 0x1D6BD, "MATHEMATICAL BOLD CAPITAL PHI" },
    { 0x1D6BE, "MATHEMATICAL BOLD CAPITAL CHI" },
    { 0x1D6BF, "MATHEMATICAL BOLD CAPITAL PSI" },
    { 0x1D6C0, "MATHEMATICAL BOLD CAPITAL OMEGA" },
    { 0x1D6C1, "MATHEMATICAL BOLD NABLA" },
    { 0x1D6C2, "MATHEMATICAL BOLD SMALL ALPHA" },
    { 0x1D6C3, "MATHEMATICAL BOLD SMALL BETA" },
    { 0x1D6C4, "MATHEMATICAL BOLD SMALL GAMMA" },
    { 0x1D6C5, "MATHEMATICAL BOLD SMALL DELTA" },
    { 0x1D6C6, "MATHEMATICAL BOLD SMALL EPSILON" },
    { 0x1D6C7, "MATHEMATICAL BOLD SMALL ZETA" },
    { 0x1D6C8, "MATHEMATICAL BOLD SMALL ETA" },
    { 0x1D6C9, "MATHEMATICAL BOLD SMALL THETA" },
    { 0x1D6CA, "MATHEMATICAL BOLD SMALL IOTA" },
    { 0x1D6CB, "MATHEMATICAL BOLD SMALL KAPPA" },
    { 0x1D6CC, "MATHEMATICAL BOLD SMALL LAMDA" },
    { 0x1D6CD, "MATHEMATICAL BOLD SMALL MU" },
    { 0x1D6CE, "MATHEMATICAL BOLD SMALL NU" },
    { 0x1D6CF, "MATHEMATICAL BOLD SMALL XI" },
    { 0x1D6D0, "MATHEMATICAL BOLD SMALL OMICRON" },
    { 0x1D6D1, "MATHEMATICAL BOLD SMALL PI" },
    { 0x1D6D2, "MATHEMATICAL BOLD SMALL RHO" },
    { 0x1D6D3, "MATHEMATICAL BOLD SMALL FINAL SIGMA" },
    { 0x1D6D4, "MATHEMATICAL BOLD SMALL SIGMA" },
    { 0x1D6D5, "MATHEMATICAL BOLD SMALL TAU" },
    { 0x1D6D6, "MATHEMATICAL BOLD SMALL UPSILON" },
    { 0x1D6D7, "MATHEMATICAL BOLD SMALL PHI" },
    { 0x1D6D8, "MATHEMATICAL BOLD SMALL CHI" },
    { 0x1D6D9, "MATHEMATICAL BOLD SMALL PSI" },
    { 0x1D6DA, "MATHEMATICAL BOLD SMALL OMEGA" },
    { 0x1D6DB, "MATHEMATICAL BOLD PARTIAL DIFFERENTIAL" },
    { 0x1D6DC, "MATHEMATICAL BOLD EPSILON SYMBOL" },
    { 0x1D6DD, "MATHEMATICAL BOLD THETA SYMBOL" },
    { 0x1D6DE, "MATHEMATICAL BOLD KAPPA SYMBOL" },
    { 0x1D6DF, "MATHEMATICAL BOLD PHI SYMBOL" },
    { 0x1D6E0, "MATHEMATICAL BOLD RHO SYMBOL" },
    { 0x1D6E1, "MATHEMATICAL BOLD PI SYMBOL" },
    { 0x1D6E2, "MATHEMATICAL ITALIC CAPITAL ALPHA" },
    { 0x1D6E3, "MATHEMATICAL ITALIC CAPITAL BETA" },
    { 0x1D6E4, "MATHEMATICAL ITALIC CAPITAL GAMMA" },
    { 0x1D6E5, "MATHEMATICAL ITALIC CAPITAL DELTA" },
    { 0x1D6E6, "MATHEMATICAL ITALIC CAPITAL EPSILON" },
    { 0x1D6E7, "MATHEMATICAL ITALIC CAPITAL ZETA" },
    { 0x1D6E8, "MATHEMATICAL ITALIC CAPITAL ETA" },
    { 0x1D6E9, "MATHEMATICAL ITALIC CAPITAL THETA" },
    { 0x1D6EA, "MATHEMATICAL ITALIC CAPITAL IOTA" },
    { 0x1D6EB, "MATHEMATICAL ITALIC CAPITAL KAPPA" },
    { 0x1D6EC, "MATHEMATICAL ITALIC CAPITAL LAMDA" },
    { 0x1D6ED, "MATHEMATICAL ITALIC CAPITAL MU" },
    { 0x1D6EE, "MATHEMATICAL ITALIC CAPITAL NU" },
    { 0x1D6EF, "MATHEMATICAL ITALIC CAPITAL XI" },
    { 0x1D6F0, "MATHEMATICAL ITALIC CAPITAL OMICRON" },
    { 0x1D6F1, "MATHEMATICAL ITALIC CAPITAL PI" },
    { 0x1D6F2, "MATHEMATICAL ITALIC CAPITAL RHO" },
    { 0x1D6F3, "MATHEMATICAL ITALIC CAPITAL THETA SYMBOL" },
    { 0x1D6F4, "MATHEMATICAL ITALIC CAPITAL SIGMA" },
    { 0x1D6F5, "MATHEMATICAL ITALIC CAPITAL TAU" },
    { 0x1D6F6, "MATHEMATICAL ITALIC CAPITAL UPSILON" },
    { 0x1D6F7, "MATHEMATICAL ITALIC CAPITAL PHI" },
    { 0x1D6F8, "MATHEMATICAL ITALIC CAPITAL CHI" },
    { 0x1D6F9, "MATHEMATICAL ITALIC CAPITAL PSI" },
    { 0x1D6FA, "MATHEMATICAL ITALIC CAPITAL OMEGA" },
    { 0x1D6FB, "MATHEMATICAL ITALIC NABLA" },
    { 0x1D6FC, "MATHEMATICAL ITALIC SMALL ALPHA" },
    { 0x1D6FD, "MATHEMATICAL ITALIC SMALL BETA" },
    { 0x1D6FE, "MATHEMATICAL ITALIC SMALL GAMMA" },
    { 0x1D6FF, "MATHEMATICAL ITALIC SMALL DELTA" },
    { 0x1D700, "MATHEMATICAL ITALIC SMALL EPSILON" },
    { 0x1D701, "MATHEMATICAL ITALIC SMALL ZETA" },
    { 0x1D702, "MATHEMATICAL ITALIC SMALL ETA" },
    { 0x1D703, "MATHEMATICAL ITALIC SMALL THETA" },
    { 0x1D704, "MATHEMATICAL ITALIC SMALL IOTA" },
    { 0x1D705, "MATHEMATICAL ITALIC SMALL KAPPA" },
    { 0x1D706, "MATHEMATICAL ITALIC SMALL LAMDA" },
    { 0x1D707, "MATHEMATICAL ITALIC SMALL MU" },
    { 0x1D708, "MATHEMATICAL ITALIC SMALL NU" },
    { 0x1D709, "MATHEMATICAL ITALIC SMALL XI" },
    { 0x1D70A, "MATHEMATICAL ITALIC SMALL OMICRON" },
    { 0x1D70B, "MATHEMATICAL ITALIC SMALL PI" },
    { 0x1D70C, "MATHEMATICAL ITALIC SMALL RHO" },
    { 0x1D70D, "MATHEMATICAL ITALIC SMALL FINAL SIGMA" },
    { 0x1D70E, "MATHEMATICAL ITALIC SMALL SIGMA" },
    { 0x1D70F, "MATHEMATICAL ITALIC SMALL TAU" },
    { 0x1D710, "MATHEMATICAL ITALIC SMALL UPSILON" },
    { 0x1D711, "MATHEMATICAL ITALIC SMALL PHI" },
    { 0x1D712, "MATHEMATICAL ITALIC SMALL CHI" },
    { 0x1D713, "MATHEMATICAL ITALIC SMALL PSI" },
    { 0x1D714, "MATHEMATICAL ITALIC SMALL OMEGA" },
    { 0x1D715, "MATHEMATICAL ITALIC PARTIAL DIFFERENTIAL" },
    { 0x1D716, "MATHEMATICAL ITALIC EPSILON SYMBOL" },
    { 0x1D717, "MATHEMATICAL ITALIC THETA SYMBOL" },
    { 0x1D718, "MATHEMATICAL ITALIC KAPPA SYMBOL" },
    { 0x1D719, "MATHEMATICAL ITALIC PHI SYMBOL" },
    { 0x1D71A, "MATHEMATICAL ITALIC RHO SYMBOL" },
    { 0x1D71B, "MATHEMATICAL ITALIC PI SYMBOL" },
    { 0x1D71C, "MATHEMATICAL BOLD ITALIC CAPITAL ALPHA" },
    { 0x1D71D, "MATHEMATICAL BOLD ITALIC CAPITAL BETA" },
    { 0x1D71E, "MATHEMATICAL BOLD ITALIC CAPITAL GAMMA" },
    { 0x1D71F, "MATHEMATICAL BOLD ITALIC CAPITAL DELTA" },
    { 0x1D720, "MATHEMATICAL BOLD ITALIC CAPITAL EPSILON" },
    { 0x1D721, "MATHEMATICAL BOLD ITALIC CAPITAL ZETA" },
    { 0x1D722, "MATHEMATICAL BOLD ITALIC CAPITAL ETA" },
    { 0x1D723, "MATHEMATICAL BOLD ITALIC CAPITAL THETA" },
    { 0x1D724, "MATHEMATICAL BOLD ITALIC CAPITAL IOTA" },
    { 0x1D725, "MATHEMATICAL BOLD ITALIC CAPITAL KAPPA" },
    { 0x1D726, "MATHEMATICAL BOLD ITALIC CAPITAL LAMDA" },
    { 0x1D727, "MATHEMATICAL BOLD ITALIC CAPITAL MU" },
    { 0x1D728, "MATHEMATICAL BOLD ITALIC CAPITAL NU" },
    { 0x1D729, "MATHEMATICAL BOLD ITALIC CAPITAL XI" },
    { 0x1D72A, "MATHEMATICAL BOLD ITALIC CAPITAL OMICRON" },
    { 0x1D72B, "MATHEMATICAL BOLD ITALIC CAPITAL PI" },
    { 0x1D72C, "MATHEMATICAL BOLD ITALIC CAPITAL RHO" },
    { 0x1D72D, "MATHEMATICAL BOLD ITALIC CAPITAL THETA SYMBOL" },
    { 0x1D72E, "MATHEMATICAL BOLD ITALIC CAPITAL SIGMA" },
    { 0x1D72F, "MATHEMATICAL BOLD ITALIC CAPITAL TAU" },
    { 0x1D730, "MATHEMATICAL BOLD ITALIC CAPITAL UPSILON" },
    { 0x1D731, "MATHEMATICAL BOLD ITALIC CAPITAL PHI" },
    { 0x1D732, "MATHEMATICAL BOLD ITALIC CAPITAL CHI" },
    { 0x1D733, "MATHEMATICAL BOLD ITALIC CAPITAL PSI" },
    { 0x1D734, "MATHEMATICAL BOLD ITALIC CAPITAL OMEGA" },
    { 0x1D735, "MATHEMATICAL BOLD ITALIC NABLA" },
    { 0x1D736, "MATHEMATICAL BOLD ITALIC SMALL ALPHA" },
    { 0x1D737, "MATHEMATICAL BOLD ITALIC SMALL BETA" },
    { 0x1D738, "MATHEMATICAL BOLD ITALIC SMALL GAMMA" },
    { 0x1D739, "MATHEMATICAL BOLD ITALIC SMALL DELTA" },
    { 0x1D73A, "MATHEMATICAL BOLD ITALIC SMALL EPSILON" },
    { 0x1D73B, "MATHEMATICAL BOLD ITALIC SMALL ZETA" },
    { 0x1D73C, "MATHEMATICAL BOLD ITALIC SMALL ETA" },
    { 0x1D73D, "MATHEMATICAL BOLD ITALIC SMALL THETA" },
    { 0x1D73E, "MATHEMATICAL BOLD ITALIC SMALL IOTA" },
    { 0x1D73F, "MATHEMATICAL BOLD ITALIC SMALL KAPPA" },
    { 0x1D740, "MATHEMATICAL BOLD ITALIC SMALL LAMDA" },
    { 0x1D741, "MATHEMATICAL BOLD ITALIC SMALL MU" },
    { 0x1D742, "MATHEMATICAL BOLD ITALIC SMALL NU" },
    { 0x1D743, "MATHEMATICAL BOLD ITALIC SMALL XI" },
    { 0x1D744, "MATHEMATICAL BOLD ITALIC SMALL OMICRON" },
    { 0x1D745, "MATHEMATICAL BOLD ITALIC SMALL PI" },
    { 0x1D746, "MATHEMATICAL BOLD ITALIC SMALL RHO" },
    { 0x1D747, "MATHEMATICAL BOLD ITALIC SMALL FINAL SIGMA" },
    { 0x1D748, "MATHEMATICAL BOLD ITALIC SMALL SIGMA" },
    { 0x1D749, "MATHEMATICAL BOLD ITALIC SMALL TAU" },
    { 0x1D74A, "MATHEMATICAL BOLD ITALIC SMALL UPSILON" },
    { 0x1D74B, "MATHEMATICAL BOLD ITALIC SMALL PHI" },
    { 0x1D74C, "MATHEMATICAL BOLD ITALIC SMALL CHI" },
    { 0x1D74D, "MATHEMATICAL BOLD ITALIC SMALL PSI" },
    { 0x1D74E, "MATHEMATICAL BOLD ITALIC SMALL OMEGA" },
    { 0x1D74F, "MATHEMATICAL BOLD ITALIC PARTIAL DIFFERENTIAL" },
    { 0x1D750, "MATHEMATICAL BOLD ITALIC EPSILON SYMBOL" },
    { 0x1D751, "MATHEMATICAL BOLD ITALIC THETA SYMBOL" },
    { 0x1D752, "MATHEMATICAL BOLD ITALIC KAPPA SYMBOL" },
    { 0x1D753, "MATHEMATICAL BOLD ITALIC PHI SYMBOL" },
    { 0x1D754, "MATHEMATICAL BOLD ITALIC RHO SYMBOL" },
    { 0x1D755, "MATHEMATICAL BOLD ITALIC PI SYMBOL" },
    { 0x1D756, "MATHEMATICAL SANS-SERIF BOLD CAPITAL ALPHA" },
    { 0x1D757, "MATHEMATICAL SANS-SERIF BOLD CAPITAL BETA" },
    { 0x1D758, "MATHEMATICAL SANS-SERIF BOLD CAPITAL GAMMA" },
    { 0x1D759, "MATHEMATICAL SANS-SERIF BOLD CAPITAL DELTA" },
    { 0x1D75A, "MATHEMATICAL SANS-SERIF BOLD CAPITAL EPSILON" },
    { 0x1D75B, "MATHEMATICAL SANS-SERIF BOLD CAPITAL ZETA" },
    { 0x1D75C, "MATHEMATICAL SANS-SERIF BOLD CAPITAL ETA" },
    { 0x1D75D, "MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA" },
    { 0x1D75E, "MATHEMATICAL SANS-SERIF BOLD CAPITAL IOTA" },
    { 0x1D75F, "MATHEMATICAL SANS-SERIF BOLD CAPITAL KAPPA" },
    { 0x1D760, "MATHEMATICAL SANS-SERIF BOLD CAPITAL LAMDA" },
    { 0x1D761, "MATHEMATICAL SANS-SERIF BOLD CAPITAL MU" },
    { 0x1D762, "MATHEMATICAL SANS-SERIF BOLD CAPITAL NU" },
    { 0x1D763, "MATHEMATICAL SANS-SERIF BOLD CAPITAL XI" },
    { 0x1D764, "MATHEMATICAL SANS-SERIF BOLD CAPITAL OMICRON" },
    { 0x1D765, "MATHEMATICAL SANS-SERIF BOLD CAPITAL PI" },
    { 0x1D766, "MATHEMATICAL SANS-SERIF BOLD CAPITAL RHO" },
    { 0x1D767, "MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA SYMBOL" },
    { 0x1D768, "MATHEMATICAL SANS-SERIF BOLD CAPITAL SIGMA" },
    { 0x1D769, "MATHEMATICAL SANS-SERIF BOLD CAPITAL TAU" },
    { 0x1D76A, "MATHEMATICAL SANS-SERIF BOLD CAPITAL UPSILON" },
    { 0x1D76B, "MATHEMATICAL SANS-SERIF BOLD CAPITAL PHI" },
    { 0x1D76C, "MATHEMATICAL SANS-SERIF BOLD CAPITAL CHI" },
    { 0x1D76D, "MATHEMATICAL SANS-SERIF BOLD CAPITAL PSI" },
    { 0x1D76E, "MATHEMATICAL SANS-SERIF BOLD CAPITAL OMEGA" },
    { 0x1D76F, "MATHEMATICAL SANS-SERIF BOLD NABLA" },
    { 0x1D770, "MATHEMATICAL SANS-SERIF BOLD SMALL ALPHA" },
    { 0x1D771, "MATHEMATICAL SANS-SERIF BOLD SMALL BETA" },
    { 0x1D772, "MATHEMATICAL SANS-SERIF BOLD SMALL GAMMA" },
    { 0x1D773, "MATHEMATICAL SANS-SERIF BOLD SMALL DELTA" },
    { 0x1D774, "MATHEMATICAL SANS-SERIF BOLD SMALL EPSILON" },
    { 0x1D775, "MATHEMATICAL SANS-SERIF BOLD SMALL ZETA" },
    { 0x1D776, "MATHEMATICAL SANS-SERIF BOLD SMALL ETA" },
    { 0x1D777, "MATHEMATICAL SANS-SERIF BOLD SMALL THETA" },
    { 0x1D778, "MATHEMATICAL SANS-SERIF BOLD SMALL IOTA" },
    { 0x1D779, "MATHEMATICAL SANS-SERIF BOLD SMALL KAPPA" },
    { 0x1D77A, "MATHEMATICAL SANS-SERIF BOLD SMALL LAMDA" },
    { 0x1D77B, "MATHEMATICAL SANS-SERIF BOLD SMALL MU" },
    { 0x1D77C, "MATHEMATICAL SANS-SERIF BOLD SMALL NU" },
    { 0x1D77D, "MATHEMATICAL SANS-SERIF BOLD SMALL XI" },
    { 0x1D77E, "MATHEMATICAL SANS-SERIF BOLD SMALL OMICRON" },
    { 0x1D77F, "MATHEMATICAL SANS-SERIF BOLD SMALL PI" },
    { 0x1D780, "MATHEMATICAL SANS-SERIF BOLD SMALL RHO" },
    { 0x1D781, "MATHEMATICAL SANS-SERIF BOLD SMALL FINAL SIGMA" },
    { 0x1D782, "MATHEMATICAL SANS-SERIF BOLD SMALL SIGMA" },
    { 0x1D783, "MATHEMATICAL SANS-SERIF BOLD SMALL TAU" },
    { 0x1D784, "MATHEMATICAL SANS-SERIF BOLD SMALL UPSILON" },
    { 0x1D785, "MATHEMATICAL SANS-SERIF BOLD SMALL PHI" },
    { 0x1D786, "MATHEMATICAL SANS-SERIF BOLD SMALL CHI" },
    { 0x1D787, "MATHEMATICAL SANS-SERIF BOLD SMALL PSI" },
    { 0x1D788, "MATHEMATICAL SANS-SERIF BOLD SMALL OMEGA" },
    { 0x1D789, "MATHEMATICAL SANS-SERIF BOLD PARTIAL DIFFERENTIAL" },
    { 0x1D78A, "MATHEMATICAL SANS-SERIF BOLD EPSILON SYMBOL" },
    { 0x1D78B, "MATHEMATICAL SANS-SERIF BOLD THETA SYMBOL" },
    { 0x1D78C, "MATHEMATICAL SANS-SERIF BOLD KAPPA SYMBOL" },
    { 0x1D78D, "MATHEMATICAL SANS-SERIF BOLD PHI SYMBOL" },
    { 0x1D78E, "MATHEMATICAL SANS-SERIF BOLD RHO SYMBOL" },
    { 0x1D78F, "MATHEMATICAL SANS-SERIF BOLD PI SYMBOL" },
    { 0x1D790, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ALPHA" },
    { 0x1D791, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL BETA" },
    { 0x1D792, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL GAMMA" },
    { 0x1D793, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL DELTA" },
    { 0x1D794, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL EPSILON" },
    { 0x1D795, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ZETA" },
    { 0x1D796, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ETA" },
    { 0x1D797, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA" },
    { 0x1D798, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL IOTA" },
    { 0x1D799, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL KAPPA" },
    { 0x1D79A, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL LAMDA" },
    { 0x1D79B, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL MU" },
    { 0x1D79C, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL NU" },
    { 0x1D79D, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL XI" },
    { 0x1D79E, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMICRON" },
    { 0x1D79F, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PI" },
    { 0x1D7A0, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL RHO" },
    { 0x1D7A1, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA SYMBOL" },
    { 0x1D7A2, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL SIGMA" },
    { 0x1D7A3, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL TAU" },
    { 0x1D7A4, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL UPSILON" },
    { 0x1D7A5, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PHI" },
    { 0x1D7A6, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL CHI" },
    { 0x1D7A7, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PSI" },
    { 0x1D7A8, "MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMEGA" },
    { 0x1D7A9, "MATHEMATICAL SANS-SERIF BOLD ITALIC NABLA" },
    { 0x1D7AA, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ALPHA" },
    { 0x1D7AB, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL BETA" },
    { 0x1D7AC, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL GAMMA" },
    { 0x1D7AD, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL DELTA" },
    { 0x1D7AE, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL EPSILON" },
    { 0x1D7AF, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ZETA" },
    { 0x1D7B0, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ETA" },
    { 0x1D7B1, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL THETA" },
    { 0x1D7B2, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL IOTA" },
    { 0x1D7B3, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL KAPPA" },
    { 0x1D7B4, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL LAMDA" },
    { 0x1D7B5, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL MU" },
    { 0x1D7B6, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL NU" },
    { 0x1D7B7, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL XI" },
    { 0x1D7B8, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMICRON" },
    { 0x1D7B9, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PI" },
    { 0x1D7BA, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL RHO" },
    { 0x1D7BB, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL FINAL SIGMA" },
    { 0x1D7BC, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL SIGMA" },
    { 0x1D7BD, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL TAU" },
    { 0x1D7BE, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL UPSILON" },
    { 0x1D7BF, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PHI" },
    { 0x1D7C0, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL CHI" },
    { 0x1D7C1, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PSI" },
    { 0x1D7C2, "MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMEGA" },
    { 0x1D7C3, "MATHEMATICAL SANS-SERIF BOLD ITALIC PARTIAL DIFFERENTIAL" },
    { 0x1D7C4, "MATHEMATICAL SANS-SERIF BOLD ITALIC EPSILON SYMBOL" },
    { 0x1D7C5, "MATHEMATICAL SANS-SERIF BOLD ITALIC THETA SYMBOL" },
    { 0x1D7C6, "MATHEMATICAL SANS-SERIF BOLD ITALIC KAPPA SYMBOL" },
    { 0x1D7C7, "MATHEMATICAL SANS-SERIF BOLD ITALIC PHI SYMBOL" },
    { 0x1D7C8, "MATHEMATICAL SANS-SERIF BOLD ITALIC RHO SYMBOL" },
    { 0x1D7C9, "MATHEMATICAL SANS-SERIF BOLD ITALIC PI SYMBOL" },
    { 0x1D7CA, "MATHEMATICAL BOLD CAPITAL DIGAMMA" },
    { 0x1D7CB, "MATHEMATICAL BOLD SMALL DIGAMMA" },
    { 0x1D7CE, "MATHEMATICAL BOLD DIGIT ZERO" },
    { 0x1D7CF, "MATHEMATICAL BOLD DIGIT ONE" },
    { 0x1D7D0, "MATHEMATICAL BOLD DIGIT TWO" },
    { 0x1D7D1, "MATHEMATICAL BOLD DIGIT THREE" },
    { 0x1D7D2, "MATHEMATICAL BOLD DIGIT FOUR" },
    { 0x1D7D3, "MATHEMATICAL BOLD DIGIT FIVE" },
    { 0x1D7D4, "MATHEMATICAL BOLD DIGIT SIX" },
    { 0x1D7D5, "MATHEMATICAL BOLD DIGIT SEVEN" },
    { 0x1D7D6, "MATHEMATICAL BOLD DIGIT EIGHT" },
    { 0x1D7D7, "MATHEMATICAL BOLD DIGIT NINE" },
    { 0x1D7D8, "MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO" },
    { 0x1D7D9, "MATHEMATICAL DOUBLE-STRUCK DIGIT ONE" },
    { 0x1D7DA, "MATHEMATICAL DOUBLE-STRUCK DIGIT TWO" },
    { 0x1D7DB, "MATHEMATICAL DOUBLE-STRUCK DIGIT THREE" },
    { 0x1D7DC, "MATHEMATICAL DOUBLE-STRUCK DIGIT FOUR" },
    { 0x1D7DD, "MATHEMATICAL DOUBLE-STRUCK DIGIT FIVE" },
    { 0x1D7DE, "MATHEMATICAL DOUBLE-STRUCK DIGIT SIX" },
    { 0x1D7DF, "MATHEMATICAL DOUBLE-STRUCK DIGIT SEVEN" },
    { 0x1D7E0, "MATHEMATICAL DOUBLE-STRUCK DIGIT EIGHT" },
    { 0x1D7E1, "MATHEMATICAL DOUBLE-STRUCK DIGIT NINE" },
    { 0x1D7E2, "MATHEMATICAL SANS-SERIF DIGIT ZERO" },
    { 0x1D7E3, "MATHEMATICAL SANS-SERIF DIGIT ONE" },
    { 0x1D7E4, "MATHEMATICAL SANS-SERIF DIGIT TWO" },
    { 0x1D7E5, "MATHEMATICAL SANS-SERIF DIGIT THREE" },
    { 0x1D7E6, "MATHEMATICAL SANS-SERIF DIGIT FOUR" },
    { 0x1D7E7, "MATHEMATICAL SANS-SERIF DIGIT FIVE" },
    { 0x1D7E8, "MATHEMATICAL SANS-SERIF DIGIT SIX" },
    { 0x1D7E9, "MATHEMATICAL SANS-SERIF DIGIT SEVEN" },
    { 0x1D7EA, "MATHEMATICAL SANS-SERIF DIGIT EIGHT" },
    { 0x1D7EB, "MATHEMATICAL SANS-SERIF DIGIT NINE" },
    { 0x1D7EC, "MATHEMATICAL SANS-SERIF BOLD DIGIT ZERO" },
    { 0x1D7ED, "MATHEMATICAL SANS-SERIF BOLD DIGIT ONE" },
    { 0x1D7EE, "MATHEMATICAL SANS-SERIF BOLD DIGIT TWO" },
    { 0x1D7EF, "MATHEMATICAL SANS-SERIF BOLD DIGIT THREE" },
    { 0x1D7F0, "MATHEMATICAL SANS-SERIF BOLD DIGIT FOUR" },
    { 0x1D7F1, "MATHEMATICAL SANS-SERIF BOLD DIGIT FIVE" },
    { 0x1D7F2, "MATHEMATICAL SANS-SERIF BOLD DIGIT SIX" },
    { 0x1D7F3, "MATHEMATICAL SANS-SERIF BOLD DIGIT SEVEN" },
    { 0x1D7F4, "MATHEMATICAL SANS-SERIF BOLD DIGIT EIGHT" },
    { 0x1D7F5, "MATHEMATICAL SANS-SERIF BOLD DIGIT NINE" },
    { 0x1D7F6, "MATHEMATICAL MONOSPACE DIGIT ZERO" },
    { 0x1D7F7, "MATHEMATICAL MONOSPACE DIGIT ONE" },
    { 0x1D7F8, "MATHEMATICAL MONOSPACE DIGIT TWO" },
    { 0x1D7F9, "MATHEMATICAL MONOSPACE DIGIT THREE" },
    { 0x1D7FA, "MATHEMATICAL MONOSPACE DIGIT FOUR" },
    { 0x1D7FB, "MATHEMATICAL MONOSPACE DIGIT FIVE" },
    { 0x1D7FC, "MATHEMATICAL MONOSPACE DIGIT SIX" },
    { 0x1D7FD, "MATHEMATICAL MONOSPACE DIGIT SEVEN" },
    { 0x1D7FE, "MATHEMATICAL MONOSPACE DIGIT EIGHT" },
    { 0x1D7FF, "MATHEMATICAL MONOSPACE DIGIT NINE" },
    { 0x1D800, "SIGNWRITING HAND-FIST INDEX" },
    { 0x1D801, "SIGNWRITING HAND-CIRCLE INDEX" },
    { 0x1D802, "SIGNWRITING HAND-CUP INDEX" },
    { 0x1D803, "SIGNWRITING HAND-OVAL INDEX" },
    { 0x1D804, "SIGNWRITING HAND-HINGE INDEX" },
    { 0x1D805, "SIGNWRITING HAND-ANGLE INDEX" },
    { 0x1D806, "SIGNWRITING HAND-FIST INDEX BENT" },
    { 0x1D807, "SIGNWRITING HAND-CIRCLE INDEX BENT" },
    { 0x1D808, "SIGNWRITING HAND-FIST THUMB UNDER INDEX BENT" },
    { 0x1D809, "SIGNWRITING HAND-FIST INDEX RAISED KNUCKLE" },
    { 0x1D80A, "SIGNWRITING HAND-FIST INDEX CUPPED" },
    { 0x1D80B, "SIGNWRITING HAND-FIST INDEX HINGED" },
    { 0x1D80C, "SIGNWRITING HAND-FIST INDEX HINGED LOW" },
    { 0x1D80D, "SIGNWRITING HAND-CIRCLE INDEX HINGE" },
    { 0x1D80E, "SIGNWRITING HAND-FIST INDEX MIDDLE" },
    { 0x1D80F, "SIGNWRITING HAND-CIRCLE INDEX MIDDLE" },
    { 0x1D810, "SIGNWRITING HAND-FIST INDEX MIDDLE BENT" },
    { 0x1D811, "SIGNWRITING HAND-FIST INDEX MIDDLE RAISED KNUCKLES" },
    { 0x1D812, "SIGNWRITING HAND-FIST INDEX MIDDLE HINGED" },
    { 0x1D813, "SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED" },
    { 0x1D814, "SIGNWRITING HAND-FIST INDEX HINGED MIDDLE UP" },
    { 0x1D815, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED" },
    { 0x1D816, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED INDEX BENT" },
    { 0x1D817, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED MIDDLE BENT" },
    { 0x1D818, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED CUPPED" },
    { 0x1D819, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED HINGED" },
    { 0x1D81A, "SIGNWRITING HAND-FIST INDEX MIDDLE CROSSED" },
    { 0x1D81B, "SIGNWRITING HAND-CIRCLE INDEX MIDDLE CROSSED" },
    { 0x1D81C, "SIGNWRITING HAND-FIST MIDDLE BENT OVER INDEX" },
    { 0x1D81D, "SIGNWRITING HAND-FIST INDEX BENT OVER MIDDLE" },
    { 0x1D81E, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB" },
    { 0x1D81F, "SIGNWRITING HAND-CIRCLE INDEX MIDDLE THUMB" },
    { 0x1D820, "SIGNWRITING HAND-FIST INDEX MIDDLE STRAIGHT THUMB BENT" },
    { 0x1D821, "SIGNWRITING HAND-FIST INDEX MIDDLE BENT THUMB STRAIGHT" },
    { 0x1D822, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB BENT" },
    { 0x1D823, "SIGNWRITING HAND-FIST INDEX MIDDLE HINGED SPREAD THUMB SIDE" },
    { 0x1D824, "SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED THUMB SIDE" },
    { 0x1D825, "SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED THUMB CONJOINED" },
    { 0x1D826, "SIGNWRITING HAND-FIST INDEX HINGED MIDDLE UP THUMB SIDE" },
    { 0x1D827, "SIGNWRITING HAND-FIST INDEX MIDDLE UP SPREAD THUMB FORWARD" },
    { 0x1D828, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CUPPED" },
    { 0x1D829, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CIRCLED" },
    { 0x1D82A, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB HOOKED" },
    { 0x1D82B, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB HINGED" },
    { 0x1D82C, "SIGNWRITING HAND-FIST THUMB BETWEEN INDEX MIDDLE STRAIGHT" },
    { 0x1D82D, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE" },
    { 0x1D82E, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE CONJOINED" },
    { 0x1D82F, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE BENT" },
    { 0x1D830, "SIGNWRITING HAND-FIST MIDDLE THUMB HOOKED INDEX UP" },
    { 0x1D831, "SIGNWRITING HAND-FIST INDEX THUMB HOOKED MIDDLE UP" },
    { 0x1D832, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED HINGED THUMB SIDE" },
    { 0x1D833, "SIGNWRITING HAND-FIST INDEX MIDDLE CROSSED THUMB SIDE" },
    { 0x1D834, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB FORWARD" },
    { 0x1D835, "SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED CUPPED THUMB FORWARD" },
    { 0x1D836, "SIGNWRITING HAND-FIST MIDDLE THUMB CUPPED INDEX UP" },
    { 0x1D837, "SIGNWRITING HAND-FIST INDEX THUMB CUPPED MIDDLE UP" },
    { 0x1D838, "SIGNWRITING HAND-FIST MIDDLE THUMB CIRCLED INDEX UP" },
    { 0x1D839, "SIGNWRITING HAND-FIST MIDDLE THUMB CIRCLED INDEX HINGED" },
    { 0x1D83A, "SIGNWRITING HAND-FIST INDEX THUMB ANGLED OUT MIDDLE UP" },
    { 0x1D83B, "SIGNWRITING HAND-FIST INDEX THUMB ANGLED IN MIDDLE UP" },
    { 0x1D83C, "SIGNWRITING HAND-FIST INDEX THUMB CIRCLED MIDDLE UP" },
    { 0x1D83D, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CONJOINED HINGED" },
    { 0x1D83E, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB ANGLED OUT" },
    { 0x1D83F, "SIGNWRITING HAND-FIST INDEX MIDDLE THUMB ANGLED" },
    { 0x1D840, "SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED OUT INDEX UP" },
    { 0x1D841, "SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED OUT INDEX CROSSED" },
    { 0x1D842, "SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED INDEX UP" },
    { 0x1D843, "SIGNWRITING HAND-FIST INDEX THUMB HOOKED MIDDLE HINGED" },
    { 0x1D844, "SIGNWRITING HAND-FLAT FOUR FINGERS" },
    { 0x1D845, "SIGNWRITING HAND-FLAT FOUR FINGERS BENT" },
    { 0x1D846, "SIGNWRITING HAND-FLAT FOUR FINGERS HINGED" },
    { 0x1D847, "SIGNWRITING HAND-FLAT FOUR FINGERS CONJOINED" },
    { 0x1D848, "SIGNWRITING HAND-FLAT FOUR FINGERS CONJOINED SPLIT" },
    { 0x1D849, "SIGNWRITING HAND-CLAW FOUR FINGERS CONJOINED" },
    { 0x1D84A, "SIGNWRITING HAND-FIST FOUR FINGERS CONJOINED BENT" },
    { 0x1D84B, "SIGNWRITING HAND-HINGE FOUR FINGERS CONJOINED" },
    { 0x1D84C, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD" },
    { 0x1D84D, "SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD" },
    { 0x1D84E, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD FOUR BENT" },
    { 0x1D84F, "SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD FOUR BENT" },
    { 0x1D850, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD BENT" },
    { 0x1D851, "SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD BENT" },
    { 0x1D852, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD THUMB FORWARD" },
    { 0x1D853, "SIGNWRITING HAND-CUP FIVE FINGERS SPREAD" },
    { 0x1D854, "SIGNWRITING HAND-CUP FIVE FINGERS SPREAD OPEN" },
    { 0x1D855, "SIGNWRITING HAND-HINGE FIVE FINGERS SPREAD OPEN" },
    { 0x1D856, "SIGNWRITING HAND-OVAL FIVE FINGERS SPREAD" },
    { 0x1D857, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED" },
    { 0x1D858, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED THUMB SIDE" },
    { 0x1D859, "SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED NO THUMB" },
    { 0x1D85A, "SIGNWRITING HAND-FLAT" },
    { 0x1D85B, "SIGNWRITING HAND-FLAT BETWEEN PALM FACINGS" },
    { 0x1D85C, "SIGNWRITING HAND-FLAT HEEL" },
    { 0x1D85D, "SIGNWRITING HAND-FLAT THUMB SIDE" },
    { 0x1D85E, "SIGNWRITING HAND-FLAT HEEL THUMB SIDE" },
    { 0x1D85F, "SIGNWRITING HAND-FLAT THUMB BENT" },
    { 0x1D860, "SIGNWRITING HAND-FLAT THUMB FORWARD" },
    { 0x1D861, "SIGNWRITING HAND-FLAT SPLIT INDEX THUMB SIDE" },
    { 0x1D862, "SIGNWRITING HAND-FLAT SPLIT CENTRE" },
    { 0x1D863, "SIGNWRITING HAND-FLAT SPLIT CENTRE THUMB SIDE" },
    { 0x1D864, "SIGNWRITING HAND-FLAT SPLIT CENTRE THUMB SIDE BENT" },
    { 0x1D865, "SIGNWRITING HAND-FLAT SPLIT LITTLE" },
    { 0x1D866, "SIGNWRITING HAND-CLAW" },
    { 0x1D867, "SIGNWRITING HAND-CLAW THUMB SIDE" },
    { 0x1D868, "SIGNWRITING HAND-CLAW NO THUMB" },
    { 0x1D869, "SIGNWRITING HAND-CLAW THUMB FORWARD" },
    { 0x1D86A, "SIGNWRITING HAND-HOOK CURLICUE" },
    { 0x1D86B, "SIGNWRITING HAND-HOOK" },
    { 0x1D86C, "SIGNWRITING HAND-CUP OPEN" },
    { 0x1D86D, "SIGNWRITING HAND-CUP" },
    { 0x1D86E, "SIGNWRITING HAND-CUP OPEN THUMB SIDE" },
    { 0x1D86F, "SIGNWRITING HAND-CUP THUMB SIDE" },
    { 0x1D870, "SIGNWRITING HAND-CUP OPEN NO THUMB" },
    { 0x1D871, "SIGNWRITING HAND-CUP NO THUMB" },
    { 0x1D872, "SIGNWRITING HAND-CUP OPEN THUMB FORWARD" },
    { 0x1D873, "SIGNWRITING HAND-CUP THUMB FORWARD" },
    { 0x1D874, "SIGNWRITING HAND-CURLICUE OPEN" },
    { 0x1D875, "SIGNWRITING HAND-CURLICUE" },
    { 0x1D876, "SIGNWRITING HAND-CIRCLE" },
    { 0x1D877, "SIGNWRITING HAND-OVAL" },
    { 0x1D878, "SIGNWRITING HAND-OVAL THUMB SIDE" },
    { 0x1D879, "SIGNWRITING HAND-OVAL NO THUMB" },
    { 0x1D87A, "SIGNWRITING HAND-OVAL THUMB FORWARD" },
    { 0x1D87B, "SIGNWRITING HAND-HINGE OPEN" },
    { 0x1D87C, "SIGNWRITING HAND-HINGE OPEN THUMB FORWARD" },
    { 0x1D87D, "SIGNWRITING HAND-HINGE" },
    { 0x1D87E, "SIGNWRITING HAND-HINGE SMALL" },
    { 0x1D87F, "SIGNWRITING HAND-HINGE OPEN THUMB SIDE" },
    { 0x1D880, "SIGNWRITING HAND-HINGE THUMB SIDE" },
    { 0x1D881, "SIGNWRITING HAND-HINGE OPEN NO THUMB" },
    { 0x1D882, "SIGNWRITING HAND-HINGE NO THUMB" },
    { 0x1D883, "SIGNWRITING HAND-HINGE THUMB SIDE TOUCHING INDEX" },
    { 0x1D884, "SIGNWRITING HAND-HINGE THUMB BETWEEN MIDDLE RING" },
    { 0x1D885, "SIGNWRITING HAND-ANGLE" },
    { 0x1D886, "SIGNWRITING HAND-FIST INDEX MIDDLE RING" },
    { 0x1D887, "SIGNWRITING HAND-CIRCLE INDEX MIDDLE RING" },
    { 0x1D888, "SIGNWRITING HAND-HINGE INDEX MIDDLE RING" },
    { 0x1D889, "SIGNWRITING HAND-ANGLE INDEX MIDDLE RING" },
    { 0x1D88A, "SIGNWRITING HAND-HINGE LITTLE" },
    { 0x1D88B, "SIGNWRITING HAND-FIST INDEX MIDDLE RING BENT" },
    { 0x1D88C, "SIGNWRITING HAND-FIST INDEX MIDDLE RING CONJOINED" },
    { 0x1D88D, "SIGNWRITING HAND-HINGE INDEX MIDDLE RING CONJOINED" },
    { 0x1D88E, "SIGNWRITING HAND-FIST LITTLE DOWN" },
    { 0x1D88F, "SIGNWRITING HAND-FIST LITTLE DOWN RIPPLE STRAIGHT" },
    { 0x1D890, "SIGNWRITING HAND-FIST LITTLE DOWN RIPPLE CURVED" },
    { 0x1D891, "SIGNWRITING HAND-FIST LITTLE DOWN OTHERS CIRCLED" },
    { 0x1D892, "SIGNWRITING HAND-FIST LITTLE UP" },
    { 0x1D893, "SIGNWRITING HAND-FIST THUMB UNDER LITTLE UP" },
    { 0x1D894, "SIGNWRITING HAND-CIRCLE LITTLE UP" },
    { 0x1D895, "SIGNWRITING HAND-OVAL LITTLE UP" },
    { 0x1D896, "SIGNWRITING HAND-ANGLE LITTLE UP" },
    { 0x1D897, "SIGNWRITING HAND-FIST LITTLE RAISED KNUCKLE" },
    { 0x1D898, "SIGNWRITING HAND-FIST LITTLE BENT" },
    { 0x1D899, "SIGNWRITING HAND-FIST LITTLE TOUCHES THUMB" },
    { 0x1D89A, "SIGNWRITING HAND-FIST LITTLE THUMB" },
    { 0x1D89B, "SIGNWRITING HAND-HINGE LITTLE THUMB" },
    { 0x1D89C, "SIGNWRITING HAND-FIST LITTLE INDEX THUMB" },
    { 0x1D89D, "SIGNWRITING HAND-HINGE LITTLE INDEX THUMB" },
    { 0x1D89E, "SIGNWRITING HAND-ANGLE LITTLE INDEX THUMB INDEX THUMB OUT" },
    { 0x1D89F, "SIGNWRITING HAND-ANGLE LITTLE INDEX THUMB INDEX THUMB" },
    { 0x1D8A0, "SIGNWRITING HAND-FIST LITTLE INDEX" },
    { 0x1D8A1, "SIGNWRITING HAND-CIRCLE LITTLE INDEX" },
    { 0x1D8A2, "SIGNWRITING HAND-HINGE LITTLE INDEX" },
    { 0x1D8A3, "SIGNWRITING HAND-ANGLE LITTLE INDEX" },
    { 0x1D8A4, "SIGNWRITING HAND-FIST INDEX MIDDLE LITTLE" },
    { 0x1D8A5, "SIGNWRITING HAND-CIRCLE INDEX MIDDLE LITTLE" },
    { 0x1D8A6, "SIGNWRITING HAND-HINGE INDEX MIDDLE LITTLE" },
    { 0x1D8A7, "SIGNWRITING HAND-HINGE RING" },
    { 0x1D8A8, "SIGNWRITING HAND-ANGLE INDEX MIDDLE LITTLE" },
    { 0x1D8A9, "SIGNWRITING HAND-FIST INDEX MIDDLE CROSS LITTLE" },
    { 0x1D8AA, "SIGNWRITING HAND-CIRCLE INDEX MIDDLE CROSS LITTLE" },
    { 0x1D8AB, "SIGNWRITING HAND-FIST RING DOWN" },
    { 0x1D8AC, "SIGNWRITING HAND-HINGE RING DOWN INDEX THUMB HOOK MIDDLE" },
    { 0x1D8AD, "SIGNWRITING HAND-ANGLE RING DOWN MIDDLE THUMB INDEX CROSS" },
    { 0x1D8AE, "SIGNWRITING HAND-FIST RING UP" },
    { 0x1D8AF, "SIGNWRITING HAND-FIST RING RAISED KNUCKLE" },
    { 0x1D8B0, "SIGNWRITING HAND-FIST RING LITTLE" },
    { 0x1D8B1, "SIGNWRITING HAND-CIRCLE RING LITTLE" },
    { 0x1D8B2, "SIGNWRITING HAND-OVAL RING LITTLE" },
    { 0x1D8B3, "SIGNWRITING HAND-ANGLE RING LITTLE" },
    { 0x1D8B4, "SIGNWRITING HAND-FIST RING MIDDLE" },
    { 0x1D8B5, "SIGNWRITING HAND-FIST RING MIDDLE CONJOINED" },
    { 0x1D8B6, "SIGNWRITING HAND-FIST RING MIDDLE RAISED KNUCKLES" },
    { 0x1D8B7, "SIGNWRITING HAND-FIST RING INDEX" },
    { 0x1D8B8, "SIGNWRITING HAND-FIST RING THUMB" },
    { 0x1D8B9, "SIGNWRITING HAND-HOOK RING THUMB" },
    { 0x1D8BA, "SIGNWRITING HAND-FIST INDEX RING LITTLE" },
    { 0x1D8BB, "SIGNWRITING HAND-CIRCLE INDEX RING LITTLE" },
    { 0x1D8BC, "SIGNWRITING HAND-CURLICUE INDEX RING LITTLE ON" },
    { 0x1D8BD, "SIGNWRITING HAND-HOOK INDEX RING LITTLE OUT" },
    { 0x1D8BE, "SIGNWRITING HAND-HOOK INDEX RING LITTLE IN" },
    { 0x1D8BF, "SIGNWRITING HAND-HOOK INDEX RING LITTLE UNDER" },
    { 0x1D8C0, "SIGNWRITING HAND-CUP INDEX RING LITTLE" },
    { 0x1D8C1, "SIGNWRITING HAND-HINGE INDEX RING LITTLE" },
    { 0x1D8C2, "SIGNWRITING HAND-ANGLE INDEX RING LITTLE OUT" },
    { 0x1D8C3, "SIGNWRITING HAND-ANGLE INDEX RING LITTLE" },
    { 0x1D8C4, "SIGNWRITING HAND-FIST MIDDLE DOWN" },
    { 0x1D8C5, "SIGNWRITING HAND-HINGE MIDDLE" },
    { 0x1D8C6, "SIGNWRITING HAND-FIST MIDDLE UP" },
    { 0x1D8C7, "SIGNWRITING HAND-CIRCLE MIDDLE UP" },
    { 0x1D8C8, "SIGNWRITING HAND-FIST MIDDLE RAISED KNUCKLE" },
    { 0x1D8C9, "SIGNWRITING HAND-FIST MIDDLE UP THUMB SIDE" },
    { 0x1D8CA, "SIGNWRITING HAND-HOOK MIDDLE THUMB" },
    { 0x1D8CB, "SIGNWRITING HAND-FIST MIDDLE THUMB LITTLE" },
    { 0x1D8CC, "SIGNWRITING HAND-FIST MIDDLE LITTLE" },
    { 0x1D8CD, "SIGNWRITING HAND-FIST MIDDLE RING LITTLE" },
    { 0x1D8CE, "SIGNWRITING HAND-CIRCLE MIDDLE RING LITTLE" },
    { 0x1D8CF, "SIGNWRITING HAND-CURLICUE MIDDLE RING LITTLE ON" },
    { 0x1D8D0, "SIGNWRITING HAND-CUP MIDDLE RING LITTLE" },
    { 0x1D8D1, "SIGNWRITING HAND-HINGE MIDDLE RING LITTLE" },
    { 0x1D8D2, "SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE OUT" },
    { 0x1D8D3, "SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE IN" },
    { 0x1D8D4, "SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE" },
    { 0x1D8D5, "SIGNWRITING HAND-CIRCLE MIDDLE RING LITTLE BENT" },
    { 0x1D8D6, "SIGNWRITING HAND-CLAW MIDDLE RING LITTLE CONJOINED" },
    { 0x1D8D7, "SIGNWRITING HAND-CLAW MIDDLE RING LITTLE CONJOINED SIDE" },
    { 0x1D8D8, "SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED OUT" },
    { 0x1D8D9, "SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED IN" },
    { 0x1D8DA, "SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED" },
    { 0x1D8DB, "SIGNWRITING HAND-HINGE INDEX HINGED" },
    { 0x1D8DC, "SIGNWRITING HAND-FIST INDEX THUMB SIDE" },
    { 0x1D8DD, "SIGNWRITING HAND-HINGE INDEX THUMB SIDE" },
    { 0x1D8DE, "SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB DIAGONAL" },
    { 0x1D8DF, "SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB CONJOINED" },
    { 0x1D8E0, "SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB BENT" },
    { 0x1D8E1, "SIGNWRITING HAND-FIST INDEX THUMB SIDE INDEX BENT" },
    { 0x1D8E2, "SIGNWRITING HAND-FIST INDEX THUMB SIDE BOTH BENT" },
    { 0x1D8E3, "SIGNWRITING HAND-FIST INDEX THUMB SIDE INDEX HINGE" },
    { 0x1D8E4, "SIGNWRITING HAND-FIST INDEX THUMB FORWARD INDEX STRAIGHT" },
    { 0x1D8E5, "SIGNWRITING HAND-FIST INDEX THUMB FORWARD INDEX BENT" },
    { 0x1D8E6, "SIGNWRITING HAND-FIST INDEX THUMB HOOK" },
    { 0x1D8E7, "SIGNWRITING HAND-FIST INDEX THUMB CURLICUE" },
    { 0x1D8E8, "SIGNWRITING HAND-FIST INDEX THUMB CURVE THUMB INSIDE" },
    { 0x1D8E9, "SIGNWRITING HAND-CLAW INDEX THUMB CURVE THUMB INSIDE" },
    { 0x1D8EA, "SIGNWRITING HAND-FIST INDEX THUMB CURVE THUMB UNDER" },
    { 0x1D8EB, "SIGNWRITING HAND-FIST INDEX THUMB CIRCLE" },
    { 0x1D8EC, "SIGNWRITING HAND-CUP INDEX THUMB" },
    { 0x1D8ED, "SIGNWRITING HAND-CUP INDEX THUMB OPEN" },
    { 0x1D8EE, "SIGNWRITING HAND-HINGE INDEX THUMB OPEN" },
    { 0x1D8EF, "SIGNWRITING HAND-HINGE INDEX THUMB LARGE" },
    { 0x1D8F0, "SIGNWRITING HAND-HINGE INDEX THUMB" },
    { 0x1D8F1, "SIGNWRITING HAND-HINGE INDEX THUMB SMALL" },
    { 0x1D8F2, "SIGNWRITING HAND-ANGLE INDEX THUMB OUT" },
    { 0x1D8F3, "SIGNWRITING HAND-ANGLE INDEX THUMB IN" },
    { 0x1D8F4, "SIGNWRITING HAND-ANGLE INDEX THUMB" },
    { 0x1D8F5, "SIGNWRITING HAND-FIST THUMB" },
    { 0x1D8F6, "SIGNWRITING HAND-FIST THUMB HEEL" },
    { 0x1D8F7, "SIGNWRITING HAND-FIST THUMB SIDE DIAGONAL" },
    { 0x1D8F8, "SIGNWRITING HAND-FIST THUMB SIDE CONJOINED" },
    { 0x1D8F9, "SIGNWRITING HAND-FIST THUMB SIDE BENT" },
    { 0x1D8FA, "SIGNWRITING HAND-FIST THUMB FORWARD" },
    { 0x1D8FB, "SIGNWRITING HAND-FIST THUMB BETWEEN INDEX MIDDLE" },
    { 0x1D8FC, "SIGNWRITING HAND-FIST THUMB BETWEEN MIDDLE RING" },
    { 0x1D8FD, "SIGNWRITING HAND-FIST THUMB BETWEEN RING LITTLE" },
    { 0x1D8FE, "SIGNWRITING HAND-FIST THUMB UNDER TWO FINGERS" },
    { 0x1D8FF, "SIGNWRITING HAND-FIST THUMB OVER TWO FINGERS" },
    { 0x1D900, "SIGNWRITING HAND-FIST THUMB UNDER THREE FINGERS" },
    { 0x1D901, "SIGNWRITING HAND-FIST THUMB UNDER FOUR FINGERS" },
    { 0x1D902, "SIGNWRITING HAND-FIST THUMB OVER FOUR RAISED KNUCKLES" },
    { 0x1D903, "SIGNWRITING HAND-FIST" },
    { 0x1D904, "SIGNWRITING HAND-FIST HEEL" },
    { 0x1D905, "SIGNWRITING TOUCH SINGLE" },
    { 0x1D906, "SIGNWRITING TOUCH MULTIPLE" },
    { 0x1D907, "SIGNWRITING TOUCH BETWEEN" },
    { 0x1D908, "SIGNWRITING GRASP SINGLE" },
    { 0x1D909, "SIGNWRITING GRASP MULTIPLE" },
    { 0x1D90A, "SIGNWRITING GRASP BETWEEN" },
    { 0x1D90B, "SIGNWRITING STRIKE SINGLE" },
    { 0x1D90C, "SIGNWRITING STRIKE MULTIPLE" },
    { 0x1D90D, "SIGNWRITING STRIKE BETWEEN" },
    { 0x1D90E, "SIGNWRITING BRUSH SINGLE" },
    { 0x1D90F, "SIGNWRITING BRUSH MULTIPLE" },
    { 0x1D910, "SIGNWRITING BRUSH BETWEEN" },
    { 0x1D911, "SIGNWRITING RUB SINGLE" },
    { 0x1D912, "SIGNWRITING RUB MULTIPLE" },
    { 0x1D913, "SIGNWRITING RUB BETWEEN" },
    { 0x1D914, "SIGNWRITING SURFACE SYMBOLS" },
    { 0x1D915, "SIGNWRITING SURFACE BETWEEN" },
    { 0x1D916, "SIGNWRITING SQUEEZE LARGE SINGLE" },
    { 0x1D917, "SIGNWRITING SQUEEZE SMALL SINGLE" },
    { 0x1D918, "SIGNWRITING SQUEEZE LARGE MULTIPLE" },
    { 0x1D919, "SIGNWRITING SQUEEZE SMALL MULTIPLE" },
    { 0x1D91A, "SIGNWRITING SQUEEZE SEQUENTIAL" },
    { 0x1D91B, "SIGNWRITING FLICK LARGE SINGLE" },
    { 0x1D91C, "SIGNWRITING FLICK SMALL SINGLE" },
    { 0x1D91D, "SIGNWRITING FLICK LARGE MULTIPLE" },
    { 0x1D91E, "SIGNWRITING FLICK SMALL MULTIPLE" },
    { 0x1D91F, "SIGNWRITING FLICK SEQUENTIAL" },
    { 0x1D920, "SIGNWRITING SQUEEZE FLICK ALTERNATING" },
    { 0x1D921, "SIGNWRITING MOVEMENT-HINGE UP DOWN LARGE" },
    { 0x1D922, "SIGNWRITING MOVEMENT-HINGE UP DOWN SMALL" },
    { 0x1D923, "SIGNWRITING MOVEMENT-HINGE UP SEQUENTIAL" },
    { 0x1D924, "SIGNWRITING MOVEMENT-HINGE DOWN SEQUENTIAL" },
    { 0x1D925, "SIGNWRITING MOVEMENT-HINGE UP DOWN ALTERNATING LARGE" },
    { 0x1D926, "SIGNWRITING MOVEMENT-HINGE UP DOWN ALTERNATING SMALL" },
    { 0x1D927, "SIGNWRITING MOVEMENT-HINGE SIDE TO SIDE SCISSORS" },
    { 0x1D928, "SIGNWRITING MOVEMENT-WALLPLANE FINGER CONTACT" },
    { 0x1D929, "SIGNWRITING MOVEMENT-FLOORPLANE FINGER CONTACT" },
    { 0x1D92A, "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT SMALL" },
    { 0x1D92B, "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT MEDIUM" },
    { 0x1D92C, "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT LARGE" },
    { 0x1D92D, "SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT LARGEST" },
    { 0x1D92E, "SIGNWRITING MOVEMENT-WALLPLANE SINGLE WRIST FLEX" },
    { 0x1D92F, "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE STRAIGHT" },
    { 0x1D930, "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE WRIST FLEX" },
    { 0x1D931, "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE ALTERNATING" },
    { 0x1D932, "SIGNWRITING MOVEMENT-WALLPLANE DOUBLE ALTERNATING WRIST FLEX" },
    { 0x1D933, "SIGNWRITING MOVEMENT-WALLPLANE CROSS" },
    { 0x1D934, "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE STRAIGHT MOVEMENT" },
    { 0x1D935, "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE WRIST FLEX" },
    { 0x1D936, "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE ALTERNATING" },
    { 0x1D937, "SIGNWRITING MOVEMENT-WALLPLANE TRIPLE ALTERNATING WRIST FLEX" },
    { 0x1D938, "SIGNWRITING MOVEMENT-WALLPLANE BEND SMALL" },
    { 0x1D939, "SIGNWRITING MOVEMENT-WALLPLANE BEND MEDIUM" },
    { 0x1D93A, "SIGNWRITING MOVEMENT-WALLPLANE BEND LARGE" },
    { 0x1D93B, "SIGNWRITING MOVEMENT-WALLPLANE CORNER SMALL" },
    { 0x1D93C, "SIGNWRITING MOVEMENT-WALLPLANE CORNER MEDIUM" },
    { 0x1D93D, "SIGNWRITING MOVEMENT-WALLPLANE CORNER LARGE" },
    { 0x1D93E, "SIGNWRITING MOVEMENT-WALLPLANE CORNER ROTATION" },
    { 0x1D93F, "SIGNWRITING MOVEMENT-WALLPLANE CHECK SMALL" },
    { 0x1D940, "SIGNWRITING MOVEMENT-WALLPLANE CHECK MEDIUM" },
    { 0x1D941, "SIGNWRITING MOVEMENT-WALLPLANE CHECK LARGE" },
    { 0x1D942, "SIGNWRITING MOVEMENT-WALLPLANE BOX SMALL" },
    { 0x1D943, "SIGNWRITING MOVEMENT-WALLPLANE BOX MEDIUM" },
    { 0x1D944, "SIGNWRITING MOVEMENT-WALLPLANE BOX LARGE" },
    { 0x1D945, "SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG SMALL" },
    { 0x1D946, "SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG MEDIUM" },
    { 0x1D947, "SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG LARGE" },
    { 0x1D948, "SIGNWRITING MOVEMENT-WALLPLANE PEAKS SMALL" },
    { 0x1D949, "SIGNWRITING MOVEMENT-WALLPLANE PEAKS MEDIUM" },
    { 0x1D94A, "SIGNWRITING MOVEMENT-WALLPLANE PEAKS LARGE" },
    { 0x1D94B, "SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE SINGLE" },
    { 0x1D94C, "SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE DOUBLE" },
    { 0x1D94D, "SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE ALTERNATING" },
    { 0x1D94E, "SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE SINGLE" },
    { 0x1D94F, "SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE DOUBLE" },
    { 0x1D950, "SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE ALTERNATING" },
    { 0x1D951, "SIGNWRITING TRAVEL-WALLPLANE SHAKING" },
    { 0x1D952, "SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL SINGLE" },
    { 0x1D953, "SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL DOUBLE" },
    { 0x1D954, "SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL TRIPLE" },
    { 0x1D955, "SIGNWRITING MOVEMENT-DIAGONAL AWAY SMALL" },
    { 0x1D956, "SIGNWRITING MOVEMENT-DIAGONAL AWAY MEDIUM" },
    { 0x1D957, "SIGNWRITING MOVEMENT-DIAGONAL AWAY LARGE" },
    { 0x1D958, "SIGNWRITING MOVEMENT-DIAGONAL AWAY LARGEST" },
    { 0x1D959, "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS SMALL" },
    { 0x1D95A, "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS MEDIUM" },
    { 0x1D95B, "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS LARGE" },
    { 0x1D95C, "SIGNWRITING MOVEMENT-DIAGONAL TOWARDS LARGEST" },
    { 0x1D95D, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY SMALL" },
    { 0x1D95E, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY MEDIUM" },
    { 0x1D95F, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY LARGE" },
    { 0x1D960, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY LARGEST" },
    { 0x1D961, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS SMALL" },
    { 0x1D962, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS MEDIUM" },
    { 0x1D963, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS LARGE" },
    { 0x1D964, "SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS LARGEST" },
    { 0x1D965, "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT SMALL" },
    { 0x1D966, "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT MEDIUM" },
    { 0x1D967, "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT LARGE" },
    { 0x1D968, "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT LARGEST" },
    { 0x1D969, "SIGNWRITING MOVEMENT-FLOORPLANE SINGLE WRIST FLEX" },
    { 0x1D96A, "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE STRAIGHT" },
    { 0x1D96B, "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE WRIST FLEX" },
    { 0x1D96C, "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE ALTERNATING" },
    { 0x1D96D, "SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE ALTERNATING WRIST FLEX" },
    { 0x1D96E, "SIGNWRITING MOVEMENT-FLOORPLANE CROSS" },
    { 0x1D96F, "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE STRAIGHT MOVEMENT" },
    { 0x1D970, "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE WRIST FLEX" },
    { 0x1D971, "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE ALTERNATING MOVEMENT" },
    { 0x1D972, "SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE ALTERNATING WRIST FLEX" },
    { 0x1D973, "SIGNWRITING MOVEMENT-FLOORPLANE BEND" },
    { 0x1D974, "SIGNWRITING MOVEMENT-FLOORPLANE CORNER SMALL" },
    { 0x1D975, "SIGNWRITING MOVEMENT-FLOORPLANE CORNER MEDIUM" },
    { 0x1D976, "SIGNWRITING MOVEMENT-FLOORPLANE CORNER LARGE" },
    { 0x1D977, "SIGNWRITING MOVEMENT-FLOORPLANE CHECK" },
    { 0x1D978, "SIGNWRITING MOVEMENT-FLOORPLANE BOX SMALL" },
    { 0x1D979, "SIGNWRITING MOVEMENT-FLOORPLANE BOX MEDIUM" },
    { 0x1D97A, "SIGNWRITING MOVEMENT-FLOORPLANE BOX LARGE" },
    { 0x1D97B, "SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG SMALL" },
    { 0x1D97C, "SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG MEDIUM" },
    { 0x1D97D, "SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG LARGE" },
    { 0x1D97E, "SIGNWRITING MOVEMENT-FLOORPLANE PEAKS SMALL" },
    { 0x1D97F, "SIGNWRITING MOVEMENT-FLOORPLANE PEAKS MEDIUM" },
    { 0x1D980, "SIGNWRITING MOVEMENT-FLOORPLANE PEAKS LARGE" },
    { 0x1D981, "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE SINGLE" },
    { 0x1D982, "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE DOUBLE" },
    { 0x1D983, "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE ALTERNATING" },
    { 0x1D984, "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE SINGLE" },
    { 0x1D985, "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE DOUBLE" },
    { 0x1D986, "SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE ALTERNATING" },
    { 0x1D987, "SIGNWRITING TRAVEL-FLOORPLANE SHAKING" },
    { 0x1D988, "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER SMALL" },
    { 0x1D989, "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER MEDIUM" },
    { 0x1D98A, "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER LARGE" },
    { 0x1D98B, "SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER LARGEST" },
    { 0x1D98C, "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE SMALL" },
    { 0x1D98D, "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE MEDIUM" },
    { 0x1D98E, "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE LARGE" },
    { 0x1D98F, "SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE LARGEST" },
    { 0x1D990, "SIGNWRITING MOVEMENT-WALLPLANE CURVE THREE-QUARTER CIRCLE SMALL" },
    { 0x1D991, "SIGNWRITING MOVEMENT-WALLPLANE CURVE THREE-QUARTER CIRCLE MEDIUM" },
    { 0x1D992, "SIGNWRITING MOVEMENT-WALLPLANE HUMP SMALL" },
    { 0x1D993, "SIGNWRITING MOVEMENT-WALLPLANE HUMP MEDIUM" },
    { 0x1D994, "SIGNWRITING MOVEMENT-WALLPLANE HUMP LARGE" },
    { 0x1D995, "SIGNWRITING MOVEMENT-WALLPLANE LOOP SMALL" },
    { 0x1D996, "SIGNWRITING MOVEMENT-WALLPLANE LOOP MEDIUM" },
    { 0x1D997, "SIGNWRITING MOVEMENT-WALLPLANE LOOP LARGE" },
    { 0x1D998, "SIGNWRITING MOVEMENT-WALLPLANE LOOP SMALL DOUBLE" },
    { 0x1D999, "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE SMALL" },
    { 0x1D99A, "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE MEDIUM" },
    { 0x1D99B, "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE LARGE" },
    { 0x1D99C, "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE SMALL" },
    { 0x1D99D, "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE MEDIUM" },
    { 0x1D99E, "SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE LARGE" },
    { 0x1D99F, "SIGNWRITING MOVEMENT-WALLPLANE CURVE THEN STRAIGHT" },
    { 0x1D9A0, "SIGNWRITING MOVEMENT-WALLPLANE CURVED CROSS SMALL" },
    { 0x1D9A1, "SIGNWRITING MOVEMENT-WALLPLANE CURVED CROSS MEDIUM" },
    { 0x1D9A2, "SIGNWRITING ROTATION-WALLPLANE SINGLE" },
    { 0x1D9A3, "SIGNWRITING ROTATION-WALLPLANE DOUBLE" },
    { 0x1D9A4, "SIGNWRITING ROTATION-WALLPLANE ALTERNATE" },
    { 0x1D9A5, "SIGNWRITING MOVEMENT-WALLPLANE SHAKING" },
    { 0x1D9A6, "SIGNWRITING MOVEMENT-WALLPLANE CURVE HITTING FRONT WALL" },
    { 0x1D9A7, "SIGNWRITING MOVEMENT-WALLPLANE HUMP HITTING FRONT WALL" },
    { 0x1D9A8, "SIGNWRITING MOVEMENT-WALLPLANE LOOP HITTING FRONT WALL" },
    { 0x1D9A9, "SIGNWRITING MOVEMENT-WALLPLANE WAVE HITTING FRONT WALL" },
    { 0x1D9AA, "SIGNWRITING ROTATION-WALLPLANE SINGLE HITTING FRONT WALL" },
    { 0x1D9AB, "SIGNWRITING ROTATION-WALLPLANE DOUBLE HITTING FRONT WALL" },
    { 0x1D9AC, "SIGNWRITING ROTATION-WALLPLANE ALTERNATING HITTING FRONT WALL" },
    { 0x1D9AD, "SIGNWRITING MOVEMENT-WALLPLANE CURVE HITTING CHEST" },
    { 0x1D9AE, "SIGNWRITING MOVEMENT-WALLPLANE HUMP HITTING CHEST" },
    { 0x1D9AF, "SIGNWRITING MOVEMENT-WALLPLANE LOOP HITTING CHEST" },
    { 0x1D9B0, "SIGNWRITING MOVEMENT-WALLPLANE WAVE HITTING CHEST" },
    { 0x1D9B1, "SIGNWRITING ROTATION-WALLPLANE SINGLE HITTING CHEST" },
    { 0x1D9B2, "SIGNWRITING ROTATION-WALLPLANE DOUBLE HITTING CHEST" },
    { 0x1D9B3, "SIGNWRITING ROTATION-WALLPLANE ALTERNATING HITTING CHEST" },
    { 0x1D9B4, "SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH SMALL" },
    { 0x1D9B5, "SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH MEDIUM" },
    { 0x1D9B6, "SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH LARGE" },
    { 0x1D9B7, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING CEILING SMALL" },
    { 0x1D9B8, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING CEILING LARGE" },
    { 0x1D9B9, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING SMALL DOUBLE" },
    { 0x1D9BA, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING LARGE DOUBLE" },
    { 0x1D9BB, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING SMALL TRIPLE" },
    { 0x1D9BC, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING LARGE TRIPLE" },
    { 0x1D9BD, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING SMALL SINGLE" },
    { 0x1D9BE, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING LARGE SINGLE" },
    { 0x1D9BF, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING SMALL DOUBLE" },
    { 0x1D9C0, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING LARGE DOUBLE" },
    { 0x1D9C1, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING CEILING SMALL" },
    { 0x1D9C2, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING CEILING LARGE" },
    { 0x1D9C3, "SIGNWRITING ROTATION-FLOORPLANE SINGLE HITTING CEILING" },
    { 0x1D9C4, "SIGNWRITING ROTATION-FLOORPLANE DOUBLE HITTING CEILING" },
    { 0x1D9C5, "SIGNWRITING ROTATION-FLOORPLANE ALTERNATING HITTING CEILING" },
    { 0x1D9C6, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING FLOOR SMALL" },
    { 0x1D9C7, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING FLOOR LARGE" },
    { 0x1D9C8, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR SMALL DOUBLE" },
    { 0x1D9C9, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR LARGE DOUBLE" },
    { 0x1D9CA, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR TRIPLE SMALL TRIPLE" },
    { 0x1D9CB, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR TRIPLE LARGE TRIPLE" },
    { 0x1D9CC, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR SMALL SINGLE" },
    { 0x1D9CD, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR LARGE SINGLE" },
    { 0x1D9CE, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR SMALL DOUBLE" },
    { 0x1D9CF, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR LARGE DOUBLE" },
    { 0x1D9D0, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING FLOOR SMALL" },
    { 0x1D9D1, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING FLOOR LARGE" },
    { 0x1D9D2, "SIGNWRITING ROTATION-FLOORPLANE SINGLE HITTING FLOOR" },
    { 0x1D9D3, "SIGNWRITING ROTATION-FLOORPLANE DOUBLE HITTING FLOOR" },
    { 0x1D9D4, "SIGNWRITING ROTATION-FLOORPLANE ALTERNATING HITTING FLOOR" },
    { 0x1D9D5, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE SMALL" },
    { 0x1D9D6, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE MEDIUM" },
    { 0x1D9D7, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE LARGE" },
    { 0x1D9D8, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE LARGEST" },
    { 0x1D9D9, "SIGNWRITING MOVEMENT-FLOORPLANE CURVE COMBINED" },
    { 0x1D9DA, "SIGNWRITING MOVEMENT-FLOORPLANE HUMP SMALL" },
    { 0x1D9DB, "SIGNWRITING MOVEMENT-FLOORPLANE LOOP SMALL" },
    { 0x1D9DC, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE SNAKE" },
    { 0x1D9DD, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE SMALL" },
    { 0x1D9DE, "SIGNWRITING MOVEMENT-FLOORPLANE WAVE LARGE" },
    { 0x1D9DF, "SIGNWRITING ROTATION-FLOORPLANE SINGLE" },
    { 0x1D9E0, "SIGNWRITING ROTATION-FLOORPLANE DOUBLE" },
    { 0x1D9E1, "SIGNWRITING ROTATION-FLOORPLANE ALTERNATING" },
    { 0x1D9E2, "SIGNWRITING MOVEMENT-FLOORPLANE SHAKING PARALLEL" },
    { 0x1D9E3, "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE SMALL SINGLE" },
    { 0x1D9E4, "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE MEDIUM SINGLE" },
    { 0x1D9E5, "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE SMALL DOUBLE" },
    { 0x1D9E6, "SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE MEDIUM DOUBLE" },
    { 0x1D9E7, "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL SMALL SINGLE" },
    { 0x1D9E8, "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL MEDIUM SINGLE" },
    { 0x1D9E9, "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL LARGE SINGLE" },
    { 0x1D9EA, "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL SMALL DOUBLE" },
    { 0x1D9EB, "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL MEDIUM DOUBLE" },
    { 0x1D9EC, "SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL LARGE DOUBLE" },
    { 0x1D9ED, "SIGNWRITING MOVEMENT-WALLPLANE WRIST CIRCLE FRONT SINGLE" },
    { 0x1D9EE, "SIGNWRITING MOVEMENT-WALLPLANE WRIST CIRCLE FRONT DOUBLE" },
    { 0x1D9EF, "SIGNWRITING MOVEMENT-FLOORPLANE WRIST CIRCLE HITTING WALL SINGLE" },
    { 0x1D9F0, "SIGNWRITING MOVEMENT-FLOORPLANE WRIST CIRCLE HITTING WALL DOUBLE" },
    { 0x1D9F1, "SIGNWRITING MOVEMENT-WALLPLANE FINGER CIRCLES SINGLE" },
    { 0x1D9F2, "SIGNWRITING MOVEMENT-WALLPLANE FINGER CIRCLES DOUBLE" },
    { 0x1D9F3, "SIGNWRITING MOVEMENT-FLOORPLANE FINGER CIRCLES HITTING WALL SINGLE" },
    { 0x1D9F4, "SIGNWRITING MOVEMENT-FLOORPLANE FINGER CIRCLES HITTING WALL DOUBLE" },
    { 0x1D9F5, "SIGNWRITING DYNAMIC ARROWHEAD SMALL" },
    { 0x1D9F6, "SIGNWRITING DYNAMIC ARROWHEAD LARGE" },
    { 0x1D9F7, "SIGNWRITING DYNAMIC FAST" },
    { 0x1D9F8, "SIGNWRITING DYNAMIC SLOW" },
    { 0x1D9F9, "SIGNWRITING DYNAMIC TENSE" },
    { 0x1D9FA, "SIGNWRITING DYNAMIC RELAXED" },
    { 0x1D9FB, "SIGNWRITING DYNAMIC SIMULTANEOUS" },
    { 0x1D9FC, "SIGNWRITING DYNAMIC SIMULTANEOUS ALTERNATING" },
    { 0x1D9FD, "SIGNWRITING DYNAMIC EVERY OTHER TIME" },
    { 0x1D9FE, "SIGNWRITING DYNAMIC GRADUAL" },
    { 0x1D9FF, "SIGNWRITING HEAD" },
    { 0x1DA00, "SIGNWRITING HEAD RIM" },
    { 0x1DA01, "SIGNWRITING HEAD MOVEMENT-WALLPLANE STRAIGHT" },
    { 0x1DA02, "SIGNWRITING HEAD MOVEMENT-WALLPLANE TILT" },
    { 0x1DA03, "SIGNWRITING HEAD MOVEMENT-FLOORPLANE STRAIGHT" },
    { 0x1DA04, "SIGNWRITING HEAD MOVEMENT-WALLPLANE CURVE" },
    { 0x1DA05, "SIGNWRITING HEAD MOVEMENT-FLOORPLANE CURVE" },
    { 0x1DA06, "SIGNWRITING HEAD MOVEMENT CIRCLE" },
    { 0x1DA07, "SIGNWRITING FACE DIRECTION POSITION NOSE FORWARD TILTING" },
    { 0x1DA08, "SIGNWRITING FACE DIRECTION POSITION NOSE UP OR DOWN" },
    { 0x1DA09, "SIGNWRITING FACE DIRECTION POSITION NOSE UP OR DOWN TILTING" },
    { 0x1DA0A, "SIGNWRITING EYEBROWS STRAIGHT UP" },
    { 0x1DA0B, "SIGNWRITING EYEBROWS STRAIGHT NEUTRAL" },
    { 0x1DA0C, "SIGNWRITING EYEBROWS STRAIGHT DOWN" },
    { 0x1DA0D, "SIGNWRITING DREAMY EYEBROWS NEUTRAL DOWN" },
    { 0x1DA0E, "SIGNWRITING DREAMY EYEBROWS DOWN NEUTRAL" },
    { 0x1DA0F, "SIGNWRITING DREAMY EYEBROWS UP NEUTRAL" },
    { 0x1DA10, "SIGNWRITING DREAMY EYEBROWS NEUTRAL UP" },
    { 0x1DA11, "SIGNWRITING FOREHEAD NEUTRAL" },
    { 0x1DA12, "SIGNWRITING FOREHEAD CONTACT" },
    { 0x1DA13, "SIGNWRITING FOREHEAD WRINKLED" },
    { 0x1DA14, "SIGNWRITING EYES OPEN" },
    { 0x1DA15, "SIGNWRITING EYES SQUEEZED" },
    { 0x1DA16, "SIGNWRITING EYES CLOSED" },
    { 0x1DA17, "SIGNWRITING EYE BLINK SINGLE" },
    { 0x1DA18, "SIGNWRITING EYE BLINK MULTIPLE" },
    { 0x1DA19, "SIGNWRITING EYES HALF OPEN" },
    { 0x1DA1A, "SIGNWRITING EYES WIDE OPEN" },
    { 0x1DA1B, "SIGNWRITING EYES HALF CLOSED" },
    { 0x1DA1C, "SIGNWRITING EYES WIDENING MOVEMENT" },
    { 0x1DA1D, "SIGNWRITING EYE WINK" },
    { 0x1DA1E, "SIGNWRITING EYELASHES UP" },
    { 0x1DA1F, "SIGNWRITING EYELASHES DOWN" },
    { 0x1DA20, "SIGNWRITING EYELASHES FLUTTERING" },
    { 0x1DA21, "SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT" },
    { 0x1DA22, "SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT DOUBLE" },
    { 0x1DA23, "SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT ALTERNATING" },
    { 0x1DA24, "SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT" },
    { 0x1DA25, "SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT DOUBLE" },
    { 0x1DA26, "SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT ALTERNATING" },
    { 0x1DA27, "SIGNWRITING EYEGAZE-WALLPLANE CURVED" },
    { 0x1DA28, "SIGNWRITING EYEGAZE-FLOORPLANE CURVED" },
    { 0x1DA29, "SIGNWRITING EYEGAZE-WALLPLANE CIRCLING" },
    { 0x1DA2A, "SIGNWRITING CHEEKS PUFFED" },
    { 0x1DA2B, "SIGNWRITING CHEEKS NEUTRAL" },
    { 0x1DA2C, "SIGNWRITING CHEEKS SUCKED" },
    { 0x1DA2D, "SIGNWRITING TENSE CHEEKS HIGH" },
    { 0x1DA2E, "SIGNWRITING TENSE CHEEKS MIDDLE" },
    { 0x1DA2F, "SIGNWRITING TENSE CHEEKS LOW" },
    { 0x1DA30, "SIGNWRITING EARS" },
    { 0x1DA31, "SIGNWRITING NOSE NEUTRAL" },
    { 0x1DA32, "SIGNWRITING NOSE CONTACT" },
    { 0x1DA33, "SIGNWRITING NOSE WRINKLES" },
    { 0x1DA34, "SIGNWRITING NOSE WIGGLES" },
    { 0x1DA35, "SIGNWRITING AIR BLOWING OUT" },
    { 0x1DA36, "SIGNWRITING AIR SUCKING IN" },
    { 0x1DA37, "SIGNWRITING AIR BLOW SMALL ROTATIONS" },
    { 0x1DA38, "SIGNWRITING AIR SUCK SMALL ROTATIONS" },
    { 0x1DA39, "SIGNWRITING BREATH INHALE" },
    { 0x1DA3A, "SIGNWRITING BREATH EXHALE" },
    { 0x1DA3B, "SIGNWRITING MOUTH CLOSED NEUTRAL" },
    { 0x1DA3C, "SIGNWRITING MOUTH CLOSED FORWARD" },
    { 0x1DA3D, "SIGNWRITING MOUTH CLOSED CONTACT" },
    { 0x1DA3E, "SIGNWRITING MOUTH SMILE" },
    { 0x1DA3F, "SIGNWRITING MOUTH SMILE WRINKLED" },
    { 0x1DA40, "SIGNWRITING MOUTH SMILE OPEN" },
    { 0x1DA41, "SIGNWRITING MOUTH FROWN" },
    { 0x1DA42, "SIGNWRITING MOUTH FROWN WRINKLED" },
    { 0x1DA43, "SIGNWRITING MOUTH FROWN OPEN" },
    { 0x1DA44, "SIGNWRITING MOUTH OPEN CIRCLE" },
    { 0x1DA45, "SIGNWRITING MOUTH OPEN FORWARD" },
    { 0x1DA46, "SIGNWRITING MOUTH OPEN WRINKLED" },
    { 0x1DA47, "SIGNWRITING MOUTH OPEN OVAL" },
    { 0x1DA48, "SIGNWRITING MOUTH OPEN OVAL WRINKLED" },
    { 0x1DA49, "SIGNWRITING MOUTH OPEN OVAL YAWN" },
    { 0x1DA4A, "SIGNWRITING MOUTH OPEN RECTANGLE" },
    { 0x1DA4B, "SIGNWRITING MOUTH OPEN RECTANGLE WRINKLED" },
    { 0x1DA4C, "SIGNWRITING MOUTH OPEN RECTANGLE YAWN" },
    { 0x1DA4D, "SIGNWRITING MOUTH KISS" },
    { 0x1DA4E, "SIGNWRITING MOUTH KISS FORWARD" },
    { 0x1DA4F, "SIGNWRITING MOUTH KISS WRINKLED" },
    { 0x1DA50, "SIGNWRITING MOUTH TENSE" },
    { 0x1DA51, "SIGNWRITING MOUTH TENSE FORWARD" },
    { 0x1DA52, "SIGNWRITING MOUTH TENSE SUCKED" },
    { 0x1DA53, "SIGNWRITING LIPS PRESSED TOGETHER" },
    { 0x1DA54, "SIGNWRITING LIP LOWER OVER UPPER" },
    { 0x1DA55, "SIGNWRITING LIP UPPER OVER LOWER" },
    { 0x1DA56, "SIGNWRITING MOUTH CORNERS" },
    { 0x1DA57, "SIGNWRITING MOUTH WRINKLES SINGLE" },
    { 0x1DA58, "SIGNWRITING MOUTH WRINKLES DOUBLE" },
    { 0x1DA59, "SIGNWRITING TONGUE STICKING OUT FAR" },
    { 0x1DA5A, "SIGNWRITING TONGUE LICKING LIPS" },
    { 0x1DA5B, "SIGNWRITING TONGUE TIP BETWEEN LIPS" },
    { 0x1DA5C, "SIGNWRITING TONGUE TIP TOUCHING INSIDE MOUTH" },
    { 0x1DA5D, "SIGNWRITING TONGUE INSIDE MOUTH RELAXED" },
    { 0x1DA5E, "SIGNWRITING TONGUE MOVES AGAINST CHEEK" },
    { 0x1DA5F, "SIGNWRITING TONGUE CENTRE STICKING OUT" },
    { 0x1DA60, "SIGNWRITING TONGUE CENTRE INSIDE MOUTH" },
    { 0x1DA61, "SIGNWRITING TEETH" },
    { 0x1DA62, "SIGNWRITING TEETH MOVEMENT" },
    { 0x1DA63, "SIGNWRITING TEETH ON TONGUE" },
    { 0x1DA64, "SIGNWRITING TEETH ON TONGUE MOVEMENT" },
    { 0x1DA65, "SIGNWRITING TEETH ON LIPS" },
    { 0x1DA66, "SIGNWRITING TEETH ON LIPS MOVEMENT" },
    { 0x1DA67, "SIGNWRITING TEETH BITE LIPS" },
    { 0x1DA68, "SIGNWRITING MOVEMENT-WALLPLANE JAW" },
    { 0x1DA69, "SIGNWRITING MOVEMENT-FLOORPLANE JAW" },
    { 0x1DA6A, "SIGNWRITING NECK" },
    { 0x1DA6B, "SIGNWRITING HAIR" },
    { 0x1DA6C, "SIGNWRITING EXCITEMENT" },
    { 0x1DA6D, "SIGNWRITING SHOULDER HIP SPINE" },
    { 0x1DA6E, "SIGNWRITING SHOULDER HIP POSITIONS" },
    { 0x1DA6F, "SIGNWRITING WALLPLANE SHOULDER HIP MOVE" },
    { 0x1DA70, "SIGNWRITING FLOORPLANE SHOULDER HIP MOVE" },
    { 0x1DA71, "SIGNWRITING SHOULDER TILTING FROM WAIST" },
    { 0x1DA72, "SIGNWRITING TORSO-WALLPLANE STRAIGHT STRETCH" },
    { 0x1DA73, "SIGNWRITING TORSO-WALLPLANE CURVED BEND" },
    { 0x1DA74, "SIGNWRITING TORSO-FLOORPLANE TWISTING" },
    { 0x1DA75, "SIGNWRITING UPPER BODY TILTING FROM HIP JOINTS" },
    { 0x1DA76, "SIGNWRITING LIMB COMBINATION" },
    { 0x1DA77, "SIGNWRITING LIMB LENGTH-1" },
    { 0x1DA78, "SIGNWRITING LIMB LENGTH-2" },
    { 0x1DA79, "SIGNWRITING LIMB LENGTH-3" },
    { 0x1DA7A, "SIGNWRITING LIMB LENGTH-4" },
    { 0x1DA7B, "SIGNWRITING LIMB LENGTH-5" },
    { 0x1DA7C, "SIGNWRITING LIMB LENGTH-6" },
    { 0x1DA7D, "SIGNWRITING LIMB LENGTH-7" },
    { 0x1DA7E, "SIGNWRITING FINGER" },
    { 0x1DA7F, "SIGNWRITING LOCATION-WALLPLANE SPACE" },
    { 0x1DA80, "SIGNWRITING LOCATION-FLOORPLANE SPACE" },
    { 0x1DA81, "SIGNWRITING LOCATION HEIGHT" },
    { 0x1DA82, "SIGNWRITING LOCATION WIDTH" },
    { 0x1DA83, "SIGNWRITING LOCATION DEPTH" },
    { 0x1DA84, "SIGNWRITING LOCATION HEAD NECK" },
    { 0x1DA85, "SIGNWRITING LOCATION TORSO" },
    { 0x1DA86, "SIGNWRITING LOCATION LIMBS DIGITS" },
    { 0x1DA87, "SIGNWRITING COMMA" },
    { 0x1DA88, "SIGNWRITING FULL STOP" },
    { 0x1DA89, "SIGNWRITING SEMICOLON" },
    { 0x1DA8A, "SIGNWRITING COLON" },
    { 0x1DA8B, "SIGNWRITING PARENTHESIS" },
    { 0x1DA9B, "SIGNWRITING FILL MODIFIER-2" },
    { 0x1DA9C, "SIGNWRITING FILL MODIFIER-3" },
    { 0x1DA9D, "SIGNWRITING FILL MODIFIER-4" },
    { 0x1DA9E, "SIGNWRITING FILL MODIFIER-5" },
    { 0x1DA9F, "SIGNWRITING FILL MODIFIER-6" },
    { 0x1DAA1, "SIGNWRITING ROTATION MODIFIER-2" },
    { 0x1DAA2, "SIGNWRITING ROTATION MODIFIER-3" },
    { 0x1DAA3, "SIGNWRITING ROTATION MODIFIER-4" },
    { 0x1DAA4, "SIGNWRITING ROTATION MODIFIER-5" },
    { 0x1DAA5, "SIGNWRITING ROTATION MODIFIER-6" },
    { 0x1DAA6, "SIGNWRITING ROTATION MODIFIER-7" },
    { 0x1DAA7, "SIGNWRITING ROTATION MODIFIER-8" },
    { 0x1DAA8, "SIGNWRITING ROTATION MODIFIER-9" },
    { 0x1DAA9, "SIGNWRITING ROTATION MODIFIER-10" },
    { 0x1DAAA, "SIGNWRITING ROTATION MODIFIER-11" },
    { 0x1DAAB, "SIGNWRITING ROTATION MODIFIER-12" },
    { 0x1DAAC, "SIGNWRITING ROTATION MODIFIER-13" },
    { 0x1DAAD, "SIGNWRITING ROTATION MODIFIER-14" },
    { 0x1DAAE, "SIGNWRITING ROTATION MODIFIER-15" },
    { 0x1DAAF, "SIGNWRITING ROTATION MODIFIER-16" },
    { 0x1DF00, "LATIN SMALL LETTER FENG DIGRAPH WITH TRILL" },
    { 0x1DF01, "LATIN SMALL LETTER REVERSED SCRIPT G" },
    { 0x1DF02, "LATIN LETTER SMALL CAPITAL TURNED G" },
    { 0x1DF03, "LATIN SMALL LETTER REVERSED K" },
    { 0x1DF04, "LATIN LETTER SMALL CAPITAL L WITH BELT" },
    { 0x1DF05, "LATIN SMALL LETTER LEZH WITH RETROFLEX HOOK" },
    { 0x1DF06, "LATIN SMALL LETTER TURNED Y WITH BELT" },
    { 0x1DF07, "LATIN SMALL LETTER REVERSED ENG" },
    { 0x1DF08, "LATIN SMALL LETTER TURNED R WITH LONG LEG AND RETROFLEX HOOK" },
    { 0x1DF09, "LATIN SMALL LETTER T WITH HOOK AND RETROFLEX HOOK" },
    { 0x1DF0A, "LATIN LETTER RETROFLEX CLICK WITH RETROFLEX HOOK" },
    { 0x1DF0B, "LATIN SMALL LETTER ESH WITH DOUBLE BAR" },
    { 0x1DF0C, "LATIN SMALL LETTER ESH WITH DOUBLE BAR AND CURL" },
    { 0x1DF0D, "LATIN SMALL LETTER TURNED T WITH CURL" },
    { 0x1DF0E, "LATIN LETTER INVERTED GLOTTAL STOP WITH CURL" },
    { 0x1DF0F, "LATIN LETTER STRETCHED C WITH CURL" },
    { 0x1DF10, "LATIN LETTER SMALL CAPITAL TURNED K" },
    { 0x1DF11, "LATIN SMALL LETTER L WITH FISHHOOK" },
    { 0x1DF12, "LATIN SMALL LETTER DEZH DIGRAPH WITH PALATAL HOOK" },
    { 0x1DF13, "LATIN SMALL LETTER L WITH BELT AND PALATAL HOOK" },
    { 0x1DF14, "LATIN SMALL LETTER ENG WITH PALATAL HOOK" },
    { 0x1DF15, "LATIN SMALL LETTER TURNED R WITH PALATAL HOOK" },
    { 0x1DF16, "LATIN SMALL LETTER R WITH FISHHOOK AND PALATAL HOOK" },
    { 0x1DF17, "LATIN SMALL LETTER TESH DIGRAPH WITH PALATAL HOOK" },
    { 0x1DF18, "LATIN SMALL LETTER EZH WITH PALATAL HOOK" },
    { 0x1DF19, "LATIN SMALL LETTER DEZH DIGRAPH WITH RETROFLEX HOOK" },
    { 0x1DF1A, "LATIN SMALL LETTER I WITH STROKE AND RETROFLEX HOOK" },
    { 0x1DF1B, "LATIN SMALL LETTER O WITH RETROFLEX HOOK" },
    { 0x1DF1C, "LATIN SMALL LETTER TESH DIGRAPH WITH RETROFLEX HOOK" },
    { 0x1DF1D, "LATIN SMALL LETTER C WITH RETROFLEX HOOK" },
    { 0x1DF1E, "LATIN SMALL LETTER S WITH CURL" },
    { 0x1DF25, "LATIN SMALL LETTER D WITH MID-HEIGHT LEFT HOOK" },
    { 0x1DF26, "LATIN SMALL LETTER L WITH MID-HEIGHT LEFT HOOK" },
    { 0x1DF27, "LATIN SMALL LETTER N WITH MID-HEIGHT LEFT HOOK" },
    { 0x1DF28, "LATIN SMALL LETTER R WITH MID-HEIGHT LEFT HOOK" },
    { 0x1DF29, "LATIN SMALL LETTER S WITH MID-HEIGHT LEFT HOOK" },
    { 0x1DF2A, "LATIN SMALL LETTER T WITH MID-HEIGHT LEFT HOOK" },
    { 0x1E000, "COMBINING GLAGOLITIC LETTER AZU" },
    { 0x1E001, "COMBINING GLAGOLITIC LETTER BUKY" },
    { 0x1E002, "COMBINING GLAGOLITIC LETTER VEDE" },
    { 0x1E003, "COMBINING GLAGOLITIC LETTER GLAGOLI" },
    { 0x1E004, "COMBINING GLAGOLITIC LETTER DOBRO" },
    { 0x1E005, "COMBINING GLAGOLITIC LETTER YESTU" },
    { 0x1E006, "COMBINING GLAGOLITIC LETTER ZHIVETE" },
    { 0x1E008, "COMBINING GLAGOLITIC LETTER ZEMLJA" },
    { 0x1E009, "COMBINING GLAGOLITIC LETTER IZHE" },
    { 0x1E00A, "COMBINING GLAGOLITIC LETTER INITIAL IZHE" },
    { 0x1E00B, "COMBINING GLAGOLITIC LETTER I" },
    { 0x1E00C, "COMBINING GLAGOLITIC LETTER DJERVI" },
    { 0x1E00D, "COMBINING GLAGOLITIC LETTER KAKO" },
    { 0x1E00E, "COMBINING GLAGOLITIC LETTER LJUDIJE" },
    { 0x1E00F, "COMBINING GLAGOLITIC LETTER MYSLITE" },
    { 0x1E010, "COMBINING GLAGOLITIC LETTER NASHI" },
    { 0x1E011, "COMBINING GLAGOLITIC LETTER ONU" },
    { 0x1E012, "COMBINING GLAGOLITIC LETTER POKOJI" },
    { 0x1E013, "COMBINING GLAGOLITIC LETTER RITSI" },
    { 0x1E014, "COMBINING GLAGOLITIC LETTER SLOVO" },
    { 0x1E015, "COMBINING GLAGOLITIC LETTER TVRIDO" },
    { 0x1E016, "COMBINING GLAGOLITIC LETTER UKU" },
    { 0x1E017, "COMBINING GLAGOLITIC LETTER FRITU" },
    { 0x1E018, "COMBINING GLAGOLITIC LETTER HERU" },
    { 0x1E01B, "COMBINING GLAGOLITIC LETTER SHTA" },
    { 0x1E01C, "COMBINING GLAGOLITIC LETTER TSI" },
    { 0x1E01D, "COMBINING GLAGOLITIC LETTER CHRIVI" },
    { 0x1E01E, "COMBINING GLAGOLITIC LETTER SHA" },
    { 0x1E01F, "COMBINING GLAGOLITIC LETTER YERU" },
    { 0x1E020, "COMBINING GLAGOLITIC LETTER YERI" },
    { 0x1E021, "COMBINING GLAGOLITIC LETTER YATI" },
    { 0x1E023, "COMBINING GLAGOLITIC LETTER YU" },
    { 0x1E024, "COMBINING GLAGOLITIC LETTER SMALL YUS" },
    { 0x1E026, "COMBINING GLAGOLITIC LETTER YO" },
    { 0x1E027, "COMBINING GLAGOLITIC LETTER IOTATED SMALL YUS" },
    { 0x1E028, "COMBINING GLAGOLITIC LETTER BIG YUS" },
    { 0x1E029, "COMBINING GLAGOLITIC LETTER IOTATED BIG YUS" },
    { 0x1E02A, "COMBINING GLAGOLITIC LETTER FITA" },
    { 0x1E030, "MODIFIER LETTER CYRILLIC SMALL A" },
    { 0x1E031, "MODIFIER LETTER CYRILLIC SMALL BE" },
    { 0x1E032, "MODIFIER LETTER CYRILLIC SMALL VE" },
    { 0x1E033, "MODIFIER LETTER CYRILLIC SMALL GHE" },
    { 0x1E034, "MODIFIER LETTER CYRILLIC SMALL DE" },
    { 0x1E035, "MODIFIER LETTER CYRILLIC SMALL IE" },
    { 0x1E036, "MODIFIER LETTER CYRILLIC SMALL ZHE" },
    { 0x1E037, "MODIFIER LETTER CYRILLIC SMALL ZE" },
    { 0x1E038, "MODIFIER LETTER CYRILLIC SMALL I" },
    { 0x1E039, "MODIFIER LETTER CYRILLIC SMALL KA" },
    { 0x1E03A, "MODIFIER LETTER CYRILLIC SMALL EL" },
    { 0x1E03B, "MODIFIER LETTER CYRILLIC SMALL EM" },
    { 0x1E03C, "MODIFIER LETTER CYRILLIC SMALL O" },
    { 0x1E03D, "MODIFIER LETTER CYRILLIC SMALL PE" },
    { 0x1E03E, "MODIFIER LETTER CYRILLIC SMALL ER" },
    { 0x1E03F, "MODIFIER LETTER CYRILLIC SMALL ES" },
    { 0x1E040, "MODIFIER LETTER CYRILLIC SMALL TE" },
    { 0x1E041, "MODIFIER LETTER CYRILLIC SMALL U" },
    { 0x1E042, "MODIFIER LETTER CYRILLIC SMALL EF" },
    { 0x1E043, "MODIFIER LETTER CYRILLIC SMALL HA" },
    { 0x1E044, "MODIFIER LETTER CYRILLIC SMALL TSE" },
    { 0x1E045, "MODIFIER LETTER CYRILLIC SMALL CHE" },
    { 0x1E046, "MODIFIER LETTER CYRILLIC SMALL SHA" },
    { 0x1E047, "MODIFIER LETTER CYRILLIC SMALL YERU" },
    { 0x1E048, "MODIFIER LETTER CYRILLIC SMALL E" },
    { 0x1E049, "MODIFIER LETTER CYRILLIC SMALL YU" },
    { 0x1E04A, "MODIFIER LETTER CYRILLIC SMALL DZZE" },
    { 0x1E04B, "MODIFIER LETTER CYRILLIC SMALL SCHWA" },
    { 0x1E04C, "MODIFIER LETTER CYRILLIC SMALL BYELORUSSIAN-UKRAINIAN I" },
    { 0x1E04D, "MODIFIER LETTER CYRILLIC SMALL JE" },
    { 0x1E04E, "MODIFIER LETTER CYRILLIC SMALL BARRED O" },
    { 0x1E04F, "MODIFIER LETTER CYRILLIC SMALL STRAIGHT U" },
    { 0x1E050, "MODIFIER LETTER CYRILLIC SMALL PALOCHKA" },
    { 0x1E051, "CYRILLIC SUBSCRIPT SMALL LETTER A" },
    { 0x1E052, "CYRILLIC SUBSCRIPT SMALL LETTER BE" },
    { 0x1E053, "CYRILLIC SUBSCRIPT SMALL LETTER VE" },
    { 0x1E054, "CYRILLIC SUBSCRIPT SMALL LETTER GHE" },
    { 0x1E055, "CYRILLIC SUBSCRIPT SMALL LETTER DE" },
    { 0x1E056, "CYRILLIC SUBSCRIPT SMALL LETTER IE" },
    { 0x1E057, "CYRILLIC SUBSCRIPT SMALL LETTER ZHE" },
    { 0x1E058, "CYRILLIC SUBSCRIPT SMALL LETTER ZE" },
    { 0x1E059, "CYRILLIC SUBSCRIPT SMALL LETTER I" },
    { 0x1E05A, "CYRILLIC SUBSCRIPT SMALL LETTER KA" },
    { 0x1E05B, "CYRILLIC SUBSCRIPT SMALL LETTER EL" },
    { 0x1E05C, "CYRILLIC SUBSCRIPT SMALL LETTER O" },
    { 0x1E05D, "CYRILLIC SUBSCRIPT SMALL LETTER PE" },
    { 0x1E05E, "CYRILLIC SUBSCRIPT SMALL LETTER ES" },
    { 0x1E05F, "CYRILLIC SUBSCRIPT SMALL LETTER U" },
    { 0x1E060, "CYRILLIC SUBSCRIPT SMALL LETTER EF" },
    { 0x1E061, "CYRILLIC SUBSCRIPT SMALL LETTER HA" },
    { 0x1E062, "CYRILLIC SUBSCRIPT SMALL LETTER TSE" },
    { 0x1E063, "CYRILLIC SUBSCRIPT SMALL LETTER CHE" },
    { 0x1E064, "CYRILLIC SUBSCRIPT SMALL LETTER SHA" },
    { 0x1E065, "CYRILLIC SUBSCRIPT SMALL LETTER HARD SIGN" },
    { 0x1E066, "CYRILLIC SUBSCRIPT SMALL LETTER YERU" },
    { 0x1E067, "CYRILLIC SUBSCRIPT SMALL LETTER GHE WITH UPTURN" },
    { 0x1E068, "CYRILLIC SUBSCRIPT SMALL LETTER BYELORUSSIAN-UKRAINIAN I" },
    { 0x1E069, "CYRILLIC SUBSCRIPT SMALL LETTER DZE" },
    { 0x1E06A, "CYRILLIC SUBSCRIPT SMALL LETTER DZHE" },
    { 0x1E06B, "MODIFIER LETTER CYRILLIC SMALL ES WITH DESCENDER" },
    { 0x1E06C, "MODIFIER LETTER CYRILLIC SMALL YERU WITH BACK YER" },
    { 0x1E06D, "MODIFIER LETTER CYRILLIC SMALL STRAIGHT U WITH STROKE" },
    { 0x1E08F, "COMBINING CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I" },
    { 0x1E100, "NYIAKENG PUACHUE HMONG LETTER MA" },
    { 0x1E101, "NYIAKENG PUACHUE HMONG LETTER TSA" },
    { 0x1E102, "NYIAKENG PUACHUE HMONG LETTER NTA" },
    { 0x1E103, "NYIAKENG PUACHUE HMONG LETTER TA" },
    { 0x1E104, "NYIAKENG PUACHUE HMONG LETTER HA" },
    { 0x1E105, "NYIAKENG PUACHUE HMONG LETTER NA" },
    { 0x1E106, "NYIAKENG PUACHUE HMONG LETTER XA" },
    { 0x1E107, "NYIAKENG PUACHUE HMONG LETTER NKA" },
    { 0x1E108, "NYIAKENG PUACHUE HMONG LETTER CA" },
    { 0x1E109, "NYIAKENG PUACHUE HMONG LETTER LA" },
    { 0x1E10A, "NYIAKENG PUACHUE HMONG LETTER SA" },
    { 0x1E10B, "NYIAKENG PUACHUE HMONG LETTER ZA" },
    { 0x1E10C, "NYIAKENG PUACHUE HMONG LETTER NCA" },
    { 0x1E10D, "NYIAKENG PUACHUE HMONG LETTER NTSA" },
    { 0x1E10E, "NYIAKENG PUACHUE HMONG LETTER KA" },
    { 0x1E10F, "NYIAKENG PUACHUE HMONG LETTER DA" },
    { 0x1E110, "NYIAKENG PUACHUE HMONG LETTER NYA" },
    { 0x1E111, "NYIAKENG PUACHUE HMONG LETTER NRA" },
    { 0x1E112, "NYIAKENG PUACHUE HMONG LETTER VA" },
    { 0x1E113, "NYIAKENG PUACHUE HMONG LETTER NTXA" },
    { 0x1E114, "NYIAKENG PUACHUE HMONG LETTER TXA" },
    { 0x1E115, "NYIAKENG PUACHUE HMONG LETTER FA" },
    { 0x1E116, "NYIAKENG PUACHUE HMONG LETTER RA" },
    { 0x1E117, "NYIAKENG PUACHUE HMONG LETTER QA" },
    { 0x1E118, "NYIAKENG PUACHUE HMONG LETTER YA" },
    { 0x1E119, "NYIAKENG PUACHUE HMONG LETTER NQA" },
    { 0x1E11A, "NYIAKENG PUACHUE HMONG LETTER PA" },
    { 0x1E11B, "NYIAKENG PUACHUE HMONG LETTER XYA" },
    { 0x1E11C, "NYIAKENG PUACHUE HMONG LETTER NPA" },
    { 0x1E11D, "NYIAKENG PUACHUE HMONG LETTER DLA" },
    { 0x1E11E, "NYIAKENG PUACHUE HMONG LETTER NPLA" },
    { 0x1E11F, "NYIAKENG PUACHUE HMONG LETTER HAH" },
    { 0x1E120, "NYIAKENG PUACHUE HMONG LETTER MLA" },
    { 0x1E121, "NYIAKENG PUACHUE HMONG LETTER PLA" },
    { 0x1E122, "NYIAKENG PUACHUE HMONG LETTER GA" },
    { 0x1E123, "NYIAKENG PUACHUE HMONG LETTER RRA" },
    { 0x1E124, "NYIAKENG PUACHUE HMONG LETTER A" },
    { 0x1E125, "NYIAKENG PUACHUE HMONG LETTER AA" },
    { 0x1E126, "NYIAKENG PUACHUE HMONG LETTER I" },
    { 0x1E127, "NYIAKENG PUACHUE HMONG LETTER U" },
    { 0x1E128, "NYIAKENG PUACHUE HMONG LETTER O" },
    { 0x1E129, "NYIAKENG PUACHUE HMONG LETTER OO" },
    { 0x1E12A, "NYIAKENG PUACHUE HMONG LETTER E" },
    { 0x1E12B, "NYIAKENG PUACHUE HMONG LETTER EE" },
    { 0x1E12C, "NYIAKENG PUACHUE HMONG LETTER W" },
    { 0x1E130, "NYIAKENG PUACHUE HMONG TONE-B" },
    { 0x1E131, "NYIAKENG PUACHUE HMONG TONE-M" },
    { 0x1E132, "NYIAKENG PUACHUE HMONG TONE-J" },
    { 0x1E133, "NYIAKENG PUACHUE HMONG TONE-V" },
    { 0x1E134, "NYIAKENG PUACHUE HMONG TONE-S" },
    { 0x1E135, "NYIAKENG PUACHUE HMONG TONE-G" },
    { 0x1E136, "NYIAKENG PUACHUE HMONG TONE-D" },
    { 0x1E137, "NYIAKENG PUACHUE HMONG SIGN FOR PERSON" },
    { 0x1E138, "NYIAKENG PUACHUE HMONG SIGN FOR THING" },
    { 0x1E139, "NYIAKENG PUACHUE HMONG SIGN FOR LOCATION" },
    { 0x1E13A, "NYIAKENG PUACHUE HMONG SIGN FOR ANIMAL" },
    { 0x1E13B, "NYIAKENG PUACHUE HMONG SIGN FOR INVERTEBRATE" },
    { 0x1E13C, "NYIAKENG PUACHUE HMONG SIGN XW XW" },
    { 0x1E13D, "NYIAKENG PUACHUE HMONG SYLLABLE LENGTHENER" },
    { 0x1E140, "NYIAKENG PUACHUE HMONG DIGIT ZERO" },
    { 0x1E141, "NYIAKENG PUACHUE HMONG DIGIT ONE" },
    { 0x1E142, "NYIAKENG PUACHUE HMONG DIGIT TWO" },
    { 0x1E143, "NYIAKENG PUACHUE HMONG DIGIT THREE" },
    { 0x1E144, "NYIAKENG PUACHUE HMONG DIGIT FOUR" },
    { 0x1E145, "NYIAKENG PUACHUE HMONG DIGIT FIVE" },
    { 0x1E146, "NYIAKENG PUACHUE HMONG DIGIT SIX" },
    { 0x1E147, "NYIAKENG PUACHUE HMONG DIGIT SEVEN" },
    { 0x1E148, "NYIAKENG PUACHUE HMONG DIGIT EIGHT" },
    { 0x1E149, "NYIAKENG PUACHUE HMONG DIGIT NINE" },
    { 0x1E14E, "NYIAKENG PUACHUE HMONG LOGOGRAM NYAJ" },
    { 0x1E14F, "NYIAKENG PUACHUE HMONG CIRCLED CA" },
    { 0x1E290, "TOTO LETTER PA" },
    { 0x1E291, "TOTO LETTER BA" },
    { 0x1E292, "TOTO LETTER TA" },
    { 0x1E293, "TOTO LETTER DA" },
    { 0x1E294, "TOTO LETTER KA" },
    { 0x1E295, "TOTO LETTER GA" },
    { 0x1E296, "TOTO LETTER MA" },
    { 0x1E297, "TOTO LETTER NA" },
    { 0x1E298, "TOTO LETTER NGA" },
    { 0x1E299, "TOTO LETTER SA" },
    { 0x1E29A, "TOTO LETTER CHA" },
    { 0x1E29B, "TOTO LETTER YA" },
    { 0x1E29C, "TOTO LETTER WA" },
    { 0x1E29D, "TOTO LETTER JA" },
    { 0x1E29E, "TOTO LETTER HA" },
    { 0x1E29F, "TOTO LETTER RA" },
    { 0x1E2A0, "TOTO LETTER LA" },
    { 0x1E2A1, "TOTO LETTER I" },
    { 0x1E2A2, "TOTO LETTER BREATHY I" },
    { 0x1E2A3, "TOTO LETTER IU" },
    { 0x1E2A4, "TOTO LETTER BREATHY IU" },
    { 0x1E2A5, "TOTO LETTER U" },
    { 0x1E2A6, "TOTO LETTER E" },
    { 0x1E2A7, "TOTO LETTER BREATHY E" },
    { 0x1E2A8, "TOTO LETTER EO" },
    { 0x1E2A9, "TOTO LETTER BREATHY EO" },
    { 0x1E2AA, "TOTO LETTER O" },
    { 0x1E2AB, "TOTO LETTER AE" },
    { 0x1E2AC, "TOTO LETTER BREATHY AE" },
    { 0x1E2AD, "TOTO LETTER A" },
    { 0x1E2AE, "TOTO SIGN RISING TONE" },
    { 0x1E2C0, "WANCHO LETTER AA" },
    { 0x1E2C1, "WANCHO LETTER A" },
    { 0x1E2C2, "WANCHO LETTER BA" },
    { 0x1E2C3, "WANCHO LETTER CA" },
    { 0x1E2C4, "WANCHO LETTER DA" },
    { 0x1E2C5, "WANCHO LETTER GA" },
    { 0x1E2C6, "WANCHO LETTER YA" },
    { 0x1E2C7, "WANCHO LETTER PHA" },
    { 0x1E2C8, "WANCHO LETTER LA" },
    { 0x1E2C9, "WANCHO LETTER NA" },
    { 0x1E2CA, "WANCHO LETTER PA" },
    { 0x1E2CB, "WANCHO LETTER TA" },
    { 0x1E2CC, "WANCHO LETTER THA" },
    { 0x1E2CD, "WANCHO LETTER FA" },
    { 0x1E2CE, "WANCHO LETTER SA" },
    { 0x1E2CF, "WANCHO LETTER SHA" },
    { 0x1E2D0, "WANCHO LETTER JA" },
    { 0x1E2D1, "WANCHO LETTER ZA" },
    { 0x1E2D2, "WANCHO LETTER WA" },
    { 0x1E2D3, "WANCHO LETTER VA" },
    { 0x1E2D4, "WANCHO LETTER KA" },
    { 0x1E2D5, "WANCHO LETTER O" },
    { 0x1E2D6, "WANCHO LETTER AU" },
    { 0x1E2D7, "WANCHO LETTER RA" },
    { 0x1E2D8, "WANCHO LETTER MA" },
    { 0x1E2D9, "WANCHO LETTER KHA" },
    { 0x1E2DA, "WANCHO LETTER HA" },
    { 0x1E2DB, "WANCHO LETTER E" },
    { 0x1E2DC, "WANCHO LETTER I" },
    { 0x1E2DD, "WANCHO LETTER NGA" },
    { 0x1E2DE, "WANCHO LETTER U" },
    { 0x1E2DF, "WANCHO LETTER LLHA" },
    { 0x1E2E0, "WANCHO LETTER TSA" },
    { 0x1E2E1, "WANCHO LETTER TRA" },
    { 0x1E2E2, "WANCHO LETTER ONG" },
    { 0x1E2E3, "WANCHO LETTER AANG" },
    { 0x1E2E4, "WANCHO LETTER ANG" },
    { 0x1E2E5, "WANCHO LETTER ING" },
    { 0x1E2E6, "WANCHO LETTER ON" },
    { 0x1E2E7, "WANCHO LETTER EN" },
    { 0x1E2E8, "WANCHO LETTER AAN" },
    { 0x1E2E9, "WANCHO LETTER NYA" },
    { 0x1E2EA, "WANCHO LETTER UEN" },
    { 0x1E2EB, "WANCHO LETTER YIH" },
    { 0x1E2EC, "WANCHO TONE TUP" },
    { 0x1E2ED, "WANCHO TONE TUPNI" },
    { 0x1E2EE, "WANCHO TONE KOI" },
    { 0x1E2EF, "WANCHO TONE KOINI" },
    { 0x1E2F0, "WANCHO DIGIT ZERO" },
    { 0x1E2F1, "WANCHO DIGIT ONE" },
    { 0x1E2F2, "WANCHO DIGIT TWO" },
    { 0x1E2F3, "WANCHO DIGIT THREE" },
    { 0x1E2F4, "WANCHO DIGIT FOUR" },
    { 0x1E2F5, "WANCHO DIGIT FIVE" },
    { 0x1E2F6, "WANCHO DIGIT SIX" },
    { 0x1E2F7, "WANCHO DIGIT SEVEN" },
    { 0x1E2F8, "WANCHO DIGIT EIGHT" },
    { 0x1E2F9, "WANCHO DIGIT NINE" },
    { 0x1E2FF, "WANCHO NGUN SIGN" },
    { 0x1E4D0, "NAG MUNDARI LETTER O" },
    { 0x1E4D1, "NAG MUNDARI LETTER OP" },
    { 0x1E4D2, "NAG MUNDARI LETTER OL" },
    { 0x1E4D3, "NAG MUNDARI LETTER OY" },
    { 0x1E4D4, "NAG MUNDARI LETTER ONG" },
    { 0x1E4D5, "NAG MUNDARI LETTER A" },
    { 0x1E4D6, "NAG MUNDARI LETTER AJ" },
    { 0x1E4D7, "NAG MUNDARI LETTER AB" },
    { 0x1E4D8, "NAG MUNDARI LETTER ANY" },
    { 0x1E4D9, "NAG MUNDARI LETTER AH" },
    { 0x1E4DA, "NAG MUNDARI LETTER I" },
    { 0x1E4DB, "NAG MUNDARI LETTER IS" },
    { 0x1E4DC, "NAG MUNDARI LETTER IDD" },
    { 0x1E4DD, "NAG MUNDARI LETTER IT" },
    { 0x1E4DE, "NAG MUNDARI LETTER IH" },
    { 0x1E4DF, "NAG MUNDARI LETTER U" },
    { 0x1E4E0, "NAG MUNDARI LETTER UC" },
    { 0x1E4E1, "NAG MUNDARI LETTER UD" },
    { 0x1E4E2, "NAG MUNDARI LETTER UK" },
    { 0x1E4E3, "NAG MUNDARI LETTER UR" },
    { 0x1E4E4, "NAG MUNDARI LETTER E" },
    { 0x1E4E5, "NAG MUNDARI LETTER ENN" },
    { 0x1E4E6, "NAG MUNDARI LETTER EG" },
    { 0x1E4E7, "NAG MUNDARI LETTER EM" },
    { 0x1E4E8, "NAG MUNDARI LETTER EN" },
    { 0x1E4E9, "NAG MUNDARI LETTER ETT" },
    { 0x1E4EA, "NAG MUNDARI LETTER ELL" },
    { 0x1E4EB, "NAG MUNDARI SIGN OJOD" },
    { 0x1E4EC, "NAG MUNDARI SIGN MUHOR" },
    { 0x1E4ED, "NAG MUNDARI SIGN TOYOR" },
    { 0x1E4EE, "NAG MUNDARI SIGN IKIR" },
    { 0x1E4EF, "NAG MUNDARI SIGN SUTUH" },
    { 0x1E4F0, "NAG MUNDARI DIGIT ZERO" },
    { 0x1E4F1, "NAG MUNDARI DIGIT ONE" },
    { 0x1E4F2, "NAG MUNDARI DIGIT TWO" },
    { 0x1E4F3, "NAG MUNDARI DIGIT THREE" },
    { 0x1E4F4, "NAG MUNDARI DIGIT FOUR" },
    { 0x1E4F5, "NAG MUNDARI DIGIT FIVE" },
    { 0x1E4F6, "NAG MUNDARI DIGIT SIX" },
    { 0x1E4F7, "NAG MUNDARI DIGIT SEVEN" },
    { 0x1E4F8, "NAG MUNDARI DIGIT EIGHT" },
    { 0x1E4F9, "NAG MUNDARI DIGIT NINE" },
    { 0x1E7E0, "ETHIOPIC SYLLABLE HHYA" },
    { 0x1E7E1, "ETHIOPIC SYLLABLE HHYU" },
    { 0x1E7E2, "ETHIOPIC SYLLABLE HHYI" },
    { 0x1E7E3, "ETHIOPIC SYLLABLE HHYAA" },
    { 0x1E7E4, "ETHIOPIC SYLLABLE HHYEE" },
    { 0x1E7E5, "ETHIOPIC SYLLABLE HHYE" },
    { 0x1E7E6, "ETHIOPIC SYLLABLE HHYO" },
    { 0x1E7E8, "ETHIOPIC SYLLABLE GURAGE HHWA" },
    { 0x1E7E9, "ETHIOPIC SYLLABLE HHWI" },
    { 0x1E7EA, "ETHIOPIC SYLLABLE HHWEE" },
    { 0x1E7EB, "ETHIOPIC SYLLABLE HHWE" },
    { 0x1E7ED, "ETHIOPIC SYLLABLE GURAGE MWI" },
    { 0x1E7EE, "ETHIOPIC SYLLABLE GURAGE MWEE" },
    { 0x1E7F0, "ETHIOPIC SYLLABLE GURAGE QWI" },
    { 0x1E7F1, "ETHIOPIC SYLLABLE GURAGE QWEE" },
    { 0x1E7F2, "ETHIOPIC SYLLABLE GURAGE QWE" },
    { 0x1E7F3, "ETHIOPIC SYLLABLE GURAGE BWI" },
    { 0x1E7F4, "ETHIOPIC SYLLABLE GURAGE BWEE" },
    { 0x1E7F5, "ETHIOPIC SYLLABLE GURAGE KWI" },
    { 0x1E7F6, "ETHIOPIC SYLLABLE GURAGE KWEE" },
    { 0x1E7F7, "ETHIOPIC SYLLABLE GURAGE KWE" },
    { 0x1E7F8, "ETHIOPIC SYLLABLE GURAGE GWI" },
    { 0x1E7F9, "ETHIOPIC SYLLABLE GURAGE GWEE" },
    { 0x1E7FA, "ETHIOPIC SYLLABLE GURAGE GWE" },
    { 0x1E7FB, "ETHIOPIC SYLLABLE GURAGE FWI" },
    { 0x1E7FC, "ETHIOPIC SYLLABLE GURAGE FWEE" },
    { 0x1E7FD, "ETHIOPIC SYLLABLE GURAGE PWI" },
    { 0x1E7FE, "ETHIOPIC SYLLABLE GURAGE PWEE" },
    { 0x1E800, "MENDE KIKAKUI SYLLABLE M001 KI" },
    { 0x1E801, "MENDE KIKAKUI SYLLABLE M002 KA" },
    { 0x1E802, "MENDE KIKAKUI SYLLABLE M003 KU" },
    { 0x1E803, "MENDE KIKAKUI SYLLABLE M065 KEE" },
    { 0x1E804, "MENDE KIKAKUI SYLLABLE M095 KE" },
    { 0x1E805, "MENDE KIKAKUI SYLLABLE M076 KOO" },
    { 0x1E806, "MENDE KIKAKUI SYLLABLE M048 KO" },
    { 0x1E807, "MENDE KIKAKUI SYLLABLE M179 KUA" },
    { 0x1E808, "MENDE KIKAKUI SYLLABLE M004 WI" },
    { 0x1E809, "MENDE KIKAKUI SYLLABLE M005 WA" },
    { 0x1E80A, "MENDE KIKAKUI SYLLABLE M006 WU" },
    { 0x1E80B, "MENDE KIKAKUI SYLLABLE M126 WEE" },
    { 0x1E80C, "MENDE KIKAKUI SYLLABLE M118 WE" },
    { 0x1E80D, "MENDE KIKAKUI SYLLABLE M114 WOO" },
    { 0x1E80E, "MENDE KIKAKUI SYLLABLE M045 WO" },
    { 0x1E80F, "MENDE KIKAKUI SYLLABLE M194 WUI" },
    { 0x1E810, "MENDE KIKAKUI SYLLABLE M143 WEI" },
    { 0x1E811, "MENDE KIKAKUI SYLLABLE M061 WVI" },
    { 0x1E812, "MENDE KIKAKUI SYLLABLE M049 WVA" },
    { 0x1E813, "MENDE KIKAKUI SYLLABLE M139 WVE" },
    { 0x1E814, "MENDE KIKAKUI SYLLABLE M007 MIN" },
    { 0x1E815, "MENDE KIKAKUI SYLLABLE M008 MAN" },
    { 0x1E816, "MENDE KIKAKUI SYLLABLE M009 MUN" },
    { 0x1E817, "MENDE KIKAKUI SYLLABLE M059 MEN" },
    { 0x1E818, "MENDE KIKAKUI SYLLABLE M094 MON" },
    { 0x1E819, "MENDE KIKAKUI SYLLABLE M154 MUAN" },
    { 0x1E81A, "MENDE KIKAKUI SYLLABLE M189 MUEN" },
    { 0x1E81B, "MENDE KIKAKUI SYLLABLE M010 BI" },
    { 0x1E81C, "MENDE KIKAKUI SYLLABLE M011 BA" },
    { 0x1E81D, "MENDE KIKAKUI SYLLABLE M012 BU" },
    { 0x1E81E, "MENDE KIKAKUI SYLLABLE M150 BEE" },
    { 0x1E81F, "MENDE KIKAKUI SYLLABLE M097 BE" },
    { 0x1E820, "MENDE KIKAKUI SYLLABLE M103 BOO" },
    { 0x1E821, "MENDE KIKAKUI SYLLABLE M138 BO" },
    { 0x1E822, "MENDE KIKAKUI SYLLABLE M013 I" },
    { 0x1E823, "MENDE KIKAKUI SYLLABLE M014 A" },
    { 0x1E824, "MENDE KIKAKUI SYLLABLE M015 U" },
    { 0x1E825, "MENDE KIKAKUI SYLLABLE M163 EE" },
    { 0x1E826, "MENDE KIKAKUI SYLLABLE M100 E" },
    { 0x1E827, "MENDE KIKAKUI SYLLABLE M165 OO" },
    { 0x1E828, "MENDE KIKAKUI SYLLABLE M147 O" },
    { 0x1E829, "MENDE KIKAKUI SYLLABLE M137 EI" },
    { 0x1E82A, "MENDE KIKAKUI SYLLABLE M131 IN" },
    { 0x1E82B, "MENDE KIKAKUI SYLLABLE M135 IN" },
    { 0x1E82C, "MENDE KIKAKUI SYLLABLE M195 AN" },
    { 0x1E82D, "MENDE KIKAKUI SYLLABLE M178 EN" },
    { 0x1E82E, "MENDE KIKAKUI SYLLABLE M019 SI" },
    { 0x1E82F, "MENDE KIKAKUI SYLLABLE M020 SA" },
    { 0x1E830, "MENDE KIKAKUI SYLLABLE M021 SU" },
    { 0x1E831, "MENDE KIKAKUI SYLLABLE M162 SEE" },
    { 0x1E832, "MENDE KIKAKUI SYLLABLE M116 SE" },
    { 0x1E833, "MENDE KIKAKUI SYLLABLE M136 SOO" },
    { 0x1E834, "MENDE KIKAKUI SYLLABLE M079 SO" },
    { 0x1E835, "MENDE KIKAKUI SYLLABLE M196 SIA" },
    { 0x1E836, "MENDE KIKAKUI SYLLABLE M025 LI" },
    { 0x1E837, "MENDE KIKAKUI SYLLABLE M026 LA" },
    { 0x1E838, "MENDE KIKAKUI SYLLABLE M027 LU" },
    { 0x1E839, "MENDE KIKAKUI SYLLABLE M084 LEE" },
    { 0x1E83A, "MENDE KIKAKUI SYLLABLE M073 LE" },
    { 0x1E83B, "MENDE KIKAKUI SYLLABLE M054 LOO" },
    { 0x1E83C, "MENDE KIKAKUI SYLLABLE M153 LO" },
    { 0x1E83D, "MENDE KIKAKUI SYLLABLE M110 LONG LE" },
    { 0x1E83E, "MENDE KIKAKUI SYLLABLE M016 DI" },
    { 0x1E83F, "MENDE KIKAKUI SYLLABLE M017 DA" },
    { 0x1E840, "MENDE KIKAKUI SYLLABLE M018 DU" },
    { 0x1E841, "MENDE KIKAKUI SYLLABLE M089 DEE" },
    { 0x1E842, "MENDE KIKAKUI SYLLABLE M180 DOO" },
    { 0x1E843, "MENDE KIKAKUI SYLLABLE M181 DO" },
    { 0x1E844, "MENDE KIKAKUI SYLLABLE M022 TI" },
    { 0x1E845, "MENDE KIKAKUI SYLLABLE M023 TA" },
    { 0x1E846, "MENDE KIKAKUI SYLLABLE M024 TU" },
    { 0x1E847, "MENDE KIKAKUI SYLLABLE M091 TEE" },
    { 0x1E848, "MENDE KIKAKUI SYLLABLE M055 TE" },
    { 0x1E849, "MENDE KIKAKUI SYLLABLE M104 TOO" },
    { 0x1E84A, "MENDE KIKAKUI SYLLABLE M069 TO" },
    { 0x1E84B, "MENDE KIKAKUI SYLLABLE M028 JI" },
    { 0x1E84C, "MENDE KIKAKUI SYLLABLE M029 JA" },
    { 0x1E84D, "MENDE KIKAKUI SYLLABLE M030 JU" },
    { 0x1E84E, "MENDE KIKAKUI SYLLABLE M157 JEE" },
    { 0x1E84F, "MENDE KIKAKUI SYLLABLE M113 JE" },
    { 0x1E850, "MENDE KIKAKUI SYLLABLE M160 JOO" },
    { 0x1E851, "MENDE KIKAKUI SYLLABLE M063 JO" },
    { 0x1E852, "MENDE KIKAKUI SYLLABLE M175 LONG JO" },
    { 0x1E853, "MENDE KIKAKUI SYLLABLE M031 YI" },
    { 0x1E854, "MENDE KIKAKUI SYLLABLE M032 YA" },
    { 0x1E855, "MENDE KIKAKUI SYLLABLE M033 YU" },
    { 0x1E856, "MENDE KIKAKUI SYLLABLE M109 YEE" },
    { 0x1E857, "MENDE KIKAKUI SYLLABLE M080 YE" },
    { 0x1E858, "MENDE KIKAKUI SYLLABLE M141 YOO" },
    { 0x1E859, "MENDE KIKAKUI SYLLABLE M121 YO" },
    { 0x1E85A, "MENDE KIKAKUI SYLLABLE M034 FI" },
    { 0x1E85B, "MENDE KIKAKUI SYLLABLE M035 FA" },
    { 0x1E85C, "MENDE KIKAKUI SYLLABLE M036 FU" },
    { 0x1E85D, "MENDE KIKAKUI SYLLABLE M078 FEE" },
    { 0x1E85E, "MENDE KIKAKUI SYLLABLE M075 FE" },
    { 0x1E85F, "MENDE KIKAKUI SYLLABLE M133 FOO" },
    { 0x1E860, "MENDE KIKAKUI SYLLABLE M088 FO" },
    { 0x1E861, "MENDE KIKAKUI SYLLABLE M197 FUA" },
    { 0x1E862, "MENDE KIKAKUI SYLLABLE M101 FAN" },
    { 0x1E863, "MENDE KIKAKUI SYLLABLE M037 NIN" },
    { 0x1E864, "MENDE KIKAKUI SYLLABLE M038 NAN" },
    { 0x1E865, "MENDE KIKAKUI SYLLABLE M039 NUN" },
    { 0x1E866, "MENDE KIKAKUI SYLLABLE M117 NEN" },
    { 0x1E867, "MENDE KIKAKUI SYLLABLE M169 NON" },
    { 0x1E868, "MENDE KIKAKUI SYLLABLE M176 HI" },
    { 0x1E869, "MENDE KIKAKUI SYLLABLE M041 HA" },
    { 0x1E86A, "MENDE KIKAKUI SYLLABLE M186 HU" },
    { 0x1E86B, "MENDE KIKAKUI SYLLABLE M040 HEE" },
    { 0x1E86C, "MENDE KIKAKUI SYLLABLE M096 HE" },
    { 0x1E86D, "MENDE KIKAKUI SYLLABLE M042 HOO" },
    { 0x1E86E, "MENDE KIKAKUI SYLLABLE M140 HO" },
    { 0x1E86F, "MENDE KIKAKUI SYLLABLE M083 HEEI" },
    { 0x1E870, "MENDE KIKAKUI SYLLABLE M128 HOOU" },
    { 0x1E871, "MENDE KIKAKUI SYLLABLE M053 HIN" },
    { 0x1E872, "MENDE KIKAKUI SYLLABLE M130 HAN" },
    { 0x1E873, "MENDE KIKAKUI SYLLABLE M087 HUN" },
    { 0x1E874, "MENDE KIKAKUI SYLLABLE M052 HEN" },
    { 0x1E875, "MENDE KIKAKUI SYLLABLE M193 HON" },
    { 0x1E876, "MENDE KIKAKUI SYLLABLE M046 HUAN" },
    { 0x1E877, "MENDE KIKAKUI SYLLABLE M090 NGGI" },
    { 0x1E878, "MENDE KIKAKUI SYLLABLE M043 NGGA" },
    { 0x1E879, "MENDE KIKAKUI SYLLABLE M082 NGGU" },
    { 0x1E87A, "MENDE KIKAKUI SYLLABLE M115 NGGEE" },
    { 0x1E87B, "MENDE KIKAKUI SYLLABLE M146 NGGE" },
    { 0x1E87C, "MENDE KIKAKUI SYLLABLE M156 NGGOO" },
    { 0x1E87D, "MENDE KIKAKUI SYLLABLE M120 NGGO" },
    { 0x1E87E, "MENDE KIKAKUI SYLLABLE M159 NGGAA" },
    { 0x1E87F, "MENDE KIKAKUI SYLLABLE M127 NGGUA" },
    { 0x1E880, "MENDE KIKAKUI SYLLABLE M086 LONG NGGE" },
    { 0x1E881, "MENDE KIKAKUI SYLLABLE M106 LONG NGGOO" },
    { 0x1E882, "MENDE KIKAKUI SYLLABLE M183 LONG NGGO" },
    { 0x1E883, "MENDE KIKAKUI SYLLABLE M155 GI" },
    { 0x1E884, "MENDE KIKAKUI SYLLABLE M111 GA" },
    { 0x1E885, "MENDE KIKAKUI SYLLABLE M168 GU" },
    { 0x1E886, "MENDE KIKAKUI SYLLABLE M190 GEE" },
    { 0x1E887, "MENDE KIKAKUI SYLLABLE M166 GUEI" },
    { 0x1E888, "MENDE KIKAKUI SYLLABLE M167 GUAN" },
    { 0x1E889, "MENDE KIKAKUI SYLLABLE M184 NGEN" },
    { 0x1E88A, "MENDE KIKAKUI SYLLABLE M057 NGON" },
    { 0x1E88B, "MENDE KIKAKUI SYLLABLE M177 NGUAN" },
    { 0x1E88C, "MENDE KIKAKUI SYLLABLE M068 PI" },
    { 0x1E88D, "MENDE KIKAKUI SYLLABLE M099 PA" },
    { 0x1E88E, "MENDE KIKAKUI SYLLABLE M050 PU" },
    { 0x1E88F, "MENDE KIKAKUI SYLLABLE M081 PEE" },
    { 0x1E890, "MENDE KIKAKUI SYLLABLE M051 PE" },
    { 0x1E891, "MENDE KIKAKUI SYLLABLE M102 POO" },
    { 0x1E892, "MENDE KIKAKUI SYLLABLE M066 PO" },
    { 0x1E893, "MENDE KIKAKUI SYLLABLE M145 MBI" },
    { 0x1E894, "MENDE KIKAKUI SYLLABLE M062 MBA" },
    { 0x1E895, "MENDE KIKAKUI SYLLABLE M122 MBU" },
    { 0x1E896, "MENDE KIKAKUI SYLLABLE M047 MBEE" },
    { 0x1E897, "MENDE KIKAKUI SYLLABLE M188 MBEE" },
    { 0x1E898, "MENDE KIKAKUI SYLLABLE M072 MBE" },
    { 0x1E899, "MENDE KIKAKUI SYLLABLE M172 MBOO" },
    { 0x1E89A, "MENDE KIKAKUI SYLLABLE M174 MBO" },
    { 0x1E89B, "MENDE KIKAKUI SYLLABLE M187 MBUU" },
    { 0x1E89C, "MENDE KIKAKUI SYLLABLE M161 LONG MBE" },
    { 0x1E89D, "MENDE KIKAKUI SYLLABLE M105 LONG MBOO" },
    { 0x1E89E, "MENDE KIKAKUI SYLLABLE M142 LONG MBO" },
    { 0x1E89F, "MENDE KIKAKUI SYLLABLE M132 KPI" },
    { 0x1E8A0, "MENDE KIKAKUI SYLLABLE M092 KPA" },
    { 0x1E8A1, "MENDE KIKAKUI SYLLABLE M074 KPU" },
    { 0x1E8A2, "MENDE KIKAKUI SYLLABLE M044 KPEE" },
    { 0x1E8A3, "MENDE KIKAKUI SYLLABLE M108 KPE" },
    { 0x1E8A4, "MENDE KIKAKUI SYLLABLE M112 KPOO" },
    { 0x1E8A5, "MENDE KIKAKUI SYLLABLE M158 KPO" },
    { 0x1E8A6, "MENDE KIKAKUI SYLLABLE M124 GBI" },
    { 0x1E8A7, "MENDE KIKAKUI SYLLABLE M056 GBA" },
    { 0x1E8A8, "MENDE KIKAKUI SYLLABLE M148 GBU" },
    { 0x1E8A9, "MENDE KIKAKUI SYLLABLE M093 GBEE" },
    { 0x1E8AA, "MENDE KIKAKUI SYLLABLE M107 GBE" },
    { 0x1E8AB, "MENDE KIKAKUI SYLLABLE M071 GBOO" },
    { 0x1E8AC, "MENDE KIKAKUI SYLLABLE M070 GBO" },
    { 0x1E8AD, "MENDE KIKAKUI SYLLABLE M171 RA" },
    { 0x1E8AE, "MENDE KIKAKUI SYLLABLE M123 NDI" },
    { 0x1E8AF, "MENDE KIKAKUI SYLLABLE M129 NDA" },
    { 0x1E8B0, "MENDE KIKAKUI SYLLABLE M125 NDU" },
    { 0x1E8B1, "MENDE KIKAKUI SYLLABLE M191 NDEE" },
    { 0x1E8B2, "MENDE KIKAKUI SYLLABLE M119 NDE" },
    { 0x1E8B3, "MENDE KIKAKUI SYLLABLE M067 NDOO" },
    { 0x1E8B4, "MENDE KIKAKUI SYLLABLE M064 NDO" },
    { 0x1E8B5, "MENDE KIKAKUI SYLLABLE M152 NJA" },
    { 0x1E8B6, "MENDE KIKAKUI SYLLABLE M192 NJU" },
    { 0x1E8B7, "MENDE KIKAKUI SYLLABLE M149 NJEE" },
    { 0x1E8B8, "MENDE KIKAKUI SYLLABLE M134 NJOO" },
    { 0x1E8B9, "MENDE KIKAKUI SYLLABLE M182 VI" },
    { 0x1E8BA, "MENDE KIKAKUI SYLLABLE M185 VA" },
    { 0x1E8BB, "MENDE KIKAKUI SYLLABLE M151 VU" },
    { 0x1E8BC, "MENDE KIKAKUI SYLLABLE M173 VEE" },
    { 0x1E8BD, "MENDE KIKAKUI SYLLABLE M085 VE" },
    { 0x1E8BE, "MENDE KIKAKUI SYLLABLE M144 VOO" },
    { 0x1E8BF, "MENDE KIKAKUI SYLLABLE M077 VO" },
    { 0x1E8C0, "MENDE KIKAKUI SYLLABLE M164 NYIN" },
    { 0x1E8C1, "MENDE KIKAKUI SYLLABLE M058 NYAN" },
    { 0x1E8C2, "MENDE KIKAKUI SYLLABLE M170 NYUN" },
    { 0x1E8C3, "MENDE KIKAKUI SYLLABLE M098 NYEN" },
    { 0x1E8C4, "MENDE KIKAKUI SYLLABLE M060 NYON" },
    { 0x1E8C7, "MENDE KIKAKUI DIGIT ONE" },
    { 0x1E8C8, "MENDE KIKAKUI DIGIT TWO" },
    { 0x1E8C9, "MENDE KIKAKUI DIGIT THREE" },
    { 0x1E8CA, "MENDE KIKAKUI DIGIT FOUR" },
    { 0x1E8CB, "MENDE KIKAKUI DIGIT FIVE" },
    { 0x1E8CC, "MENDE KIKAKUI DIGIT SIX" },
    { 0x1E8CD, "MENDE KIKAKUI DIGIT SEVEN" },
    { 0x1E8CE, "MENDE KIKAKUI DIGIT EIGHT" },
    { 0x1E8CF, "MENDE KIKAKUI DIGIT NINE" },
    { 0x1E8D0, "MENDE KIKAKUI COMBINING NUMBER TEENS" },
    { 0x1E8D1, "MENDE KIKAKUI COMBINING NUMBER TENS" },
    { 0x1E8D2, "MENDE KIKAKUI COMBINING NUMBER HUNDREDS" },
    { 0x1E8D3, "MENDE KIKAKUI COMBINING NUMBER THOUSANDS" },
    { 0x1E8D4, "MENDE KIKAKUI COMBINING NUMBER TEN THOUSANDS" },
    { 0x1E8D5, "MENDE KIKAKUI COMBINING NUMBER HUNDRED THOUSANDS" },
    { 0x1E8D6, "MENDE KIKAKUI COMBINING NUMBER MILLIONS" },
    { 0x1E900, "ADLAM CAPITAL LETTER ALIF" },
    { 0x1E901, "ADLAM CAPITAL LETTER DAALI" },
    { 0x1E902, "ADLAM CAPITAL LETTER LAAM" },
    { 0x1E903, "ADLAM CAPITAL LETTER MIIM" },
    { 0x1E904, "ADLAM CAPITAL LETTER BA" },
    { 0x1E905, "ADLAM CAPITAL LETTER SINNYIIYHE" },
    { 0x1E906, "ADLAM CAPITAL LETTER PE" },
    { 0x1E907, "ADLAM CAPITAL LETTER BHE" },
    { 0x1E908, "ADLAM CAPITAL LETTER RA" },
    { 0x1E909, "ADLAM CAPITAL LETTER E" },
    { 0x1E90A, "ADLAM CAPITAL LETTER FA" },
    { 0x1E90B, "ADLAM CAPITAL LETTER I" },
    { 0x1E90C, "ADLAM CAPITAL LETTER O" },
    { 0x1E90D, "ADLAM CAPITAL LETTER DHA" },
    { 0x1E90E, "ADLAM CAPITAL LETTER YHE" },
    { 0x1E90F, "ADLAM CAPITAL LETTER WAW" },
    { 0x1E910, "ADLAM CAPITAL LETTER NUN" },
    { 0x1E911, "ADLAM CAPITAL LETTER KAF" },
    { 0x1E912, "ADLAM CAPITAL LETTER YA" },
    { 0x1E913, "ADLAM CAPITAL LETTER U" },
    { 0x1E914, "ADLAM CAPITAL LETTER JIIM" },
    { 0x1E915, "ADLAM CAPITAL LETTER CHI" },
    { 0x1E916, "ADLAM CAPITAL LETTER HA" },
    { 0x1E917, "ADLAM CAPITAL LETTER QAAF" },
    { 0x1E918, "ADLAM CAPITAL LETTER GA" },
    { 0x1E919, "ADLAM CAPITAL LETTER NYA" },
    { 0x1E91A, "ADLAM CAPITAL LETTER TU" },
    { 0x1E91B, "ADLAM CAPITAL LETTER NHA" },
    { 0x1E91C, "ADLAM CAPITAL LETTER VA" },
    { 0x1E91D, "ADLAM CAPITAL LETTER KHA" },
    { 0x1E91E, "ADLAM CAPITAL LETTER GBE" },
    { 0x1E91F, "ADLAM CAPITAL LETTER ZAL" },
    { 0x1E920, "ADLAM CAPITAL LETTER KPO" },
    { 0x1E921, "ADLAM CAPITAL LETTER SHA" },
    { 0x1E922, "ADLAM SMALL LETTER ALIF" },
    { 0x1E923, "ADLAM SMALL LETTER DAALI" },
    { 0x1E924, "ADLAM SMALL LETTER LAAM" },
    { 0x1E925, "ADLAM SMALL LETTER MIIM" },
    { 0x1E926, "ADLAM SMALL LETTER BA" },
    { 0x1E927, "ADLAM SMALL LETTER SINNYIIYHE" },
    { 0x1E928, "ADLAM SMALL LETTER PE" },
    { 0x1E929, "ADLAM SMALL LETTER BHE" },
    { 0x1E92A, "ADLAM SMALL LETTER RA" },
    { 0x1E92B, "ADLAM SMALL LETTER E" },
    { 0x1E92C, "ADLAM SMALL LETTER FA" },
    { 0x1E92D, "ADLAM SMALL LETTER I" },
    { 0x1E92E, "ADLAM SMALL LETTER O" },
    { 0x1E92F, "ADLAM SMALL LETTER DHA" },
    { 0x1E930, "ADLAM SMALL LETTER YHE" },
    { 0x1E931, "ADLAM SMALL LETTER WAW" },
    { 0x1E932, "ADLAM SMALL LETTER NUN" },
    { 0x1E933, "ADLAM SMALL LETTER KAF" },
    { 0x1E934, "ADLAM SMALL LETTER YA" },
    { 0x1E935, "ADLAM SMALL LETTER U" },
    { 0x1E936, "ADLAM SMALL LETTER JIIM" },
    { 0x1E937, "ADLAM SMALL LETTER CHI" },
    { 0x1E938, "ADLAM SMALL LETTER HA" },
    { 0x1E939, "ADLAM SMALL LETTER QAAF" },
    { 0x1E93A, "ADLAM SMALL LETTER GA" },
    { 0x1E93B, "ADLAM SMALL LETTER NYA" },
    { 0x1E93C, "ADLAM SMALL LETTER TU" },
    { 0x1E93D, "ADLAM SMALL LETTER NHA" },
    { 0x1E93E, "ADLAM SMALL LETTER VA" },
    { 0x1E93F, "ADLAM SMALL LETTER KHA" },
    { 0x1E940, "ADLAM SMALL LETTER GBE" },
    { 0x1E941, "ADLAM SMALL LETTER ZAL" },
    { 0x1E942, "ADLAM SMALL LETTER KPO" },
    { 0x1E943, "ADLAM SMALL LETTER SHA" },
    { 0x1E944, "ADLAM ALIF LENGTHENER" },
    { 0x1E945, "ADLAM VOWEL LENGTHENER" },
    { 0x1E946, "ADLAM GEMINATION MARK" },
    { 0x1E947, "ADLAM HAMZA" },
    { 0x1E948, "ADLAM CONSONANT MODIFIER" },
    { 0x1E949, "ADLAM GEMINATE CONSONANT MODIFIER" },
    { 0x1E94A, "ADLAM NUKTA" },
    { 0x1E94B, "ADLAM NASALIZATION MARK" },
    { 0x1E950, "ADLAM DIGIT ZERO" },
    { 0x1E951, "ADLAM DIGIT ONE" },
    { 0x1E952, "ADLAM DIGIT TWO" },
    { 0x1E953, "ADLAM DIGIT THREE" },
    { 0x1E954, "ADLAM DIGIT FOUR" },
    { 0x1E955, "ADLAM DIGIT FIVE" },
    { 0x1E956, "ADLAM DIGIT SIX" },
    { 0x1E957, "ADLAM DIGIT SEVEN" },
    { 0x1E958, "ADLAM DIGIT EIGHT" },
    { 0x1E959, "ADLAM DIGIT NINE" },
    { 0x1E95E, "ADLAM INITIAL EXCLAMATION MARK" },
    { 0x1E95F, "ADLAM INITIAL QUESTION MARK" },
    { 0x1EC71, "INDIC SIYAQ NUMBER ONE" },
    { 0x1EC72, "INDIC SIYAQ NUMBER TWO" },
    { 0x1EC73, "INDIC SIYAQ NUMBER THREE" },
    { 0x1EC74, "INDIC SIYAQ NUMBER FOUR" },
    { 0x1EC75, "INDIC SIYAQ NUMBER FIVE" },
    { 0x1EC76, "INDIC SIYAQ NUMBER SIX" },
    { 0x1EC77, "INDIC SIYAQ NUMBER SEVEN" },
    { 0x1EC78, "INDIC SIYAQ NUMBER EIGHT" },
    { 0x1EC79, "INDIC SIYAQ NUMBER NINE" },
    { 0x1EC7A, "INDIC SIYAQ NUMBER TEN" },
    { 0x1EC7B, "INDIC SIYAQ NUMBER TWENTY" },
    { 0x1EC7C, "INDIC SIYAQ NUMBER THIRTY" },
    { 0x1EC7D, "INDIC SIYAQ NUMBER FORTY" },
    { 0x1EC7E, "INDIC SIYAQ NUMBER FIFTY" },
    { 0x1EC7F, "INDIC SIYAQ NUMBER SIXTY" },
    { 0x1EC80, "INDIC SIYAQ NUMBER SEVENTY" },
    { 0x1EC81, "INDIC SIYAQ NUMBER EIGHTY" },
    { 0x1EC82, "INDIC SIYAQ NUMBER NINETY" },
    { 0x1EC83, "INDIC SIYAQ NUMBER ONE HUNDRED" },
    { 0x1EC84, "INDIC SIYAQ NUMBER TWO HUNDRED" },
    { 0x1EC85, "INDIC SIYAQ NUMBER THREE HUNDRED" },
    { 0x1EC86, "INDIC SIYAQ NUMBER FOUR HUNDRED" },
    { 0x1EC87, "INDIC SIYAQ NUMBER FIVE HUNDRED" },
    { 0x1EC88, "INDIC SIYAQ NUMBER SIX HUNDRED" },
    { 0x1EC89, "INDIC SIYAQ NUMBER SEVEN HUNDRED" },
    { 0x1EC8A, "INDIC SIYAQ NUMBER EIGHT HUNDRED" },
    { 0x1EC8B, "INDIC SIYAQ NUMBER NINE HUNDRED" },
    { 0x1EC8C, "INDIC SIYAQ NUMBER ONE THOUSAND" },
    { 0x1EC8D, "INDIC SIYAQ NUMBER TWO THOUSAND" },
    { 0x1EC8E, "INDIC SIYAQ NUMBER THREE THOUSAND" },
    { 0x1EC8F, "INDIC SIYAQ NUMBER FOUR THOUSAND" },
    { 0x1EC90, "INDIC SIYAQ NUMBER FIVE THOUSAND" },
    { 0x1EC91, "INDIC SIYAQ NUMBER SIX THOUSAND" },
    { 0x1EC92, "INDIC SIYAQ NUMBER SEVEN THOUSAND" },
    { 0x1EC93, "INDIC SIYAQ NUMBER EIGHT THOUSAND" },
    { 0x1EC94, "INDIC SIYAQ NUMBER NINE THOUSAND" },
    { 0x1EC95, "INDIC SIYAQ NUMBER TEN THOUSAND" },
    { 0x1EC96, "INDIC SIYAQ NUMBER TWENTY THOUSAND" },
    { 0x1EC97, "INDIC SIYAQ NUMBER THIRTY THOUSAND" },
    { 0x1EC98, "INDIC SIYAQ NUMBER FORTY THOUSAND" },
    { 0x1EC99, "INDIC SIYAQ NUMBER FIFTY THOUSAND" },
    { 0x1EC9A, "INDIC SIYAQ NUMBER SIXTY THOUSAND" },
    { 0x1EC9B, "INDIC SIYAQ NUMBER SEVENTY THOUSAND" },
    { 0x1EC9C, "INDIC SIYAQ NUMBER EIGHTY THOUSAND" },
    { 0x1EC9D, "INDIC SIYAQ NUMBER NINETY THOUSAND" },
    { 0x1EC9E, "INDIC SIYAQ NUMBER LAKH" },
    { 0x1EC9F, "INDIC SIYAQ NUMBER LAKHAN" },
    { 0x1ECA0, "INDIC SIYAQ LAKH MARK" },
    { 0x1ECA1, "INDIC SIYAQ NUMBER KAROR" },
    { 0x1ECA2, "INDIC SIYAQ NUMBER KARORAN" },
    { 0x1ECA3, "INDIC SIYAQ NUMBER PREFIXED ONE" },
    { 0x1ECA4, "INDIC SIYAQ NUMBER PREFIXED TWO" },
    { 0x1ECA5, "INDIC SIYAQ NUMBER PREFIXED THREE" },
    { 0x1ECA6, "INDIC SIYAQ NUMBER PREFIXED FOUR" },
    { 0x1ECA7, "INDIC SIYAQ NUMBER PREFIXED FIVE" },
    { 0x1ECA8, "INDIC SIYAQ NUMBER PREFIXED SIX" },
    { 0x1ECA9, "INDIC SIYAQ NUMBER PREFIXED SEVEN" },
    { 0x1ECAA, "INDIC SIYAQ NUMBER PREFIXED EIGHT" },
    { 0x1ECAB, "INDIC SIYAQ NUMBER PREFIXED NINE" },
    { 0x1ECAC, "INDIC SIYAQ PLACEHOLDER" },
    { 0x1ECAD, "INDIC SIYAQ FRACTION ONE QUARTER" },
    { 0x1ECAE, "INDIC SIYAQ FRACTION ONE HALF" },
    { 0x1ECAF, "INDIC SIYAQ FRACTION THREE QUARTERS" },
    { 0x1ECB0, "INDIC SIYAQ RUPEE MARK" },
    { 0x1ECB1, "INDIC SIYAQ NUMBER ALTERNATE ONE" },
    { 0x1ECB2, "INDIC SIYAQ NUMBER ALTERNATE TWO" },
    { 0x1ECB3, "INDIC SIYAQ NUMBER ALTERNATE TEN THOUSAND" },
    { 0x1ECB4, "INDIC SIYAQ ALTERNATE LAKH MARK" },
    { 0x1ED01, "OTTOMAN SIYAQ NUMBER ONE" },
    { 0x1ED02, "OTTOMAN SIYAQ NUMBER TWO" },
    { 0x1ED03, "OTTOMAN SIYAQ NUMBER THREE" },
    { 0x1ED04, "OTTOMAN SIYAQ NUMBER FOUR" },
    { 0x1ED05, "OTTOMAN SIYAQ NUMBER FIVE" },
    { 0x1ED06, "OTTOMAN SIYAQ NUMBER SIX" },
    { 0x1ED07, "OTTOMAN SIYAQ NUMBER SEVEN" },
    { 0x1ED08, "OTTOMAN SIYAQ NUMBER EIGHT" },
    { 0x1ED09, "OTTOMAN SIYAQ NUMBER NINE" },
    { 0x1ED0A, "OTTOMAN SIYAQ NUMBER TEN" },
    { 0x1ED0B, "OTTOMAN SIYAQ NUMBER TWENTY" },
    { 0x1ED0C, "OTTOMAN SIYAQ NUMBER THIRTY" },
    { 0x1ED0D, "OTTOMAN SIYAQ NUMBER FORTY" },
    { 0x1ED0E, "OTTOMAN SIYAQ NUMBER FIFTY" },
    { 0x1ED0F, "OTTOMAN SIYAQ NUMBER SIXTY" },
    { 0x1ED10, "OTTOMAN SIYAQ NUMBER SEVENTY" },
    { 0x1ED11, "OTTOMAN SIYAQ NUMBER EIGHTY" },
    { 0x1ED12, "OTTOMAN SIYAQ NUMBER NINETY" },
    { 0x1ED13, "OTTOMAN SIYAQ NUMBER ONE HUNDRED" },
    { 0x1ED14, "OTTOMAN SIYAQ NUMBER TWO HUNDRED" },
    { 0x1ED15, "OTTOMAN SIYAQ NUMBER THREE HUNDRED" },
    { 0x1ED16, "OTTOMAN SIYAQ NUMBER FOUR HUNDRED" },
    { 0x1ED17, "OTTOMAN SIYAQ NUMBER FIVE HUNDRED" },
    { 0x1ED18, "OTTOMAN SIYAQ NUMBER SIX HUNDRED" },
    { 0x1ED19, "OTTOMAN SIYAQ NUMBER SEVEN HUNDRED" },
    { 0x1ED1A, "OTTOMAN SIYAQ NUMBER EIGHT HUNDRED" },
    { 0x1ED1B, "OTTOMAN SIYAQ NUMBER NINE HUNDRED" },
    { 0x1ED1C, "OTTOMAN SIYAQ NUMBER ONE THOUSAND" },
    { 0x1ED1D, "OTTOMAN SIYAQ NUMBER TWO THOUSAND" },
    { 0x1ED1E, "OTTOMAN SIYAQ NUMBER THREE THOUSAND" },
    { 0x1ED1F, "OTTOMAN SIYAQ NUMBER FOUR THOUSAND" },
    { 0x1ED20, "OTTOMAN SIYAQ NUMBER FIVE THOUSAND" },
    { 0x1ED21, "OTTOMAN SIYAQ NUMBER SIX THOUSAND" },
    { 0x1ED22, "OTTOMAN SIYAQ NUMBER SEVEN THOUSAND" },
    { 0x1ED23, "OTTOMAN SIYAQ NUMBER EIGHT THOUSAND" },
    { 0x1ED24, "OTTOMAN SIYAQ NUMBER NINE THOUSAND" },
    { 0x1ED25, "OTTOMAN SIYAQ NUMBER TEN THOUSAND" },
    { 0x1ED26, "OTTOMAN SIYAQ NUMBER TWENTY THOUSAND" },
    { 0x1ED27, "OTTOMAN SIYAQ NUMBER THIRTY THOUSAND" },
    { 0x1ED28, "OTTOMAN SIYAQ NUMBER FORTY THOUSAND" },
    { 0x1ED29, "OTTOMAN SIYAQ NUMBER FIFTY THOUSAND" },
    { 0x1ED2A, "OTTOMAN SIYAQ NUMBER SIXTY THOUSAND" },
    { 0x1ED2B, "OTTOMAN SIYAQ NUMBER SEVENTY THOUSAND" },
    { 0x1ED2C, "OTTOMAN SIYAQ NUMBER EIGHTY THOUSAND" },
    { 0x1ED2D, "OTTOMAN SIYAQ NUMBER NINETY THOUSAND" },
    { 0x1ED2E, "OTTOMAN SIYAQ MARRATAN" },
    { 0x1ED2F, "OTTOMAN SIYAQ ALTERNATE NUMBER TWO" },
    { 0x1ED30, "OTTOMAN SIYAQ ALTERNATE NUMBER THREE" },
    { 0x1ED31, "OTTOMAN SIYAQ ALTERNATE NUMBER FOUR" },
    { 0x1ED32, "OTTOMAN SIYAQ ALTERNATE NUMBER FIVE" },
    { 0x1ED33, "OTTOMAN SIYAQ ALTERNATE NUMBER SIX" },
    { 0x1ED34, "OTTOMAN SIYAQ ALTERNATE NUMBER SEVEN" },
    { 0x1ED35, "OTTOMAN SIYAQ ALTERNATE NUMBER EIGHT" },
    { 0x1ED36, "OTTOMAN SIYAQ ALTERNATE NUMBER NINE" },
    { 0x1ED37, "OTTOMAN SIYAQ ALTERNATE NUMBER TEN" },
    { 0x1ED38, "OTTOMAN SIYAQ ALTERNATE NUMBER FOUR HUNDRED" },
    { 0x1ED39, "OTTOMAN SIYAQ ALTERNATE NUMBER SIX HUNDRED" },
    { 0x1ED3A, "OTTOMAN SIYAQ ALTERNATE NUMBER TWO THOUSAND" },
    { 0x1ED3B, "OTTOMAN SIYAQ ALTERNATE NUMBER TEN THOUSAND" },
    { 0x1ED3C, "OTTOMAN SIYAQ FRACTION ONE HALF" },
    { 0x1ED3D, "OTTOMAN SIYAQ FRACTION ONE SIXTH" },
    { 0x1EE00, "ARABIC MATHEMATICAL ALEF" },
    { 0x1EE01, "ARABIC MATHEMATICAL BEH" },
    { 0x1EE02, "ARABIC MATHEMATICAL JEEM" },
    { 0x1EE03, "ARABIC MATHEMATICAL DAL" },
    { 0x1EE05, "ARABIC MATHEMATICAL WAW" },
    { 0x1EE06, "ARABIC MATHEMATICAL ZAIN" },
    { 0x1EE07, "ARABIC MATHEMATICAL HAH" },
    { 0x1EE08, "ARABIC MATHEMATICAL TAH" },
    { 0x1EE09, "ARABIC MATHEMATICAL YEH" },
    { 0x1EE0A, "ARABIC MATHEMATICAL KAF" },
    { 0x1EE0B, "ARABIC MATHEMATICAL LAM" },
    { 0x1EE0C, "ARABIC MATHEMATICAL MEEM" },
    { 0x1EE0D, "ARABIC MATHEMATICAL NOON" },
    { 0x1EE0E, "ARABIC MATHEMATICAL SEEN" },
    { 0x1EE0F, "ARABIC MATHEMATICAL AIN" },
    { 0x1EE10, "ARABIC MATHEMATICAL FEH" },
    { 0x1EE11, "ARABIC MATHEMATICAL SAD" },
    { 0x1EE12, "ARABIC MATHEMATICAL QAF" },
    { 0x1EE13, "ARABIC MATHEMATICAL REH" },
    { 0x1EE14, "ARABIC MATHEMATICAL SHEEN" },
    { 0x1EE15, "ARABIC MATHEMATICAL TEH" },
    { 0x1EE16, "ARABIC MATHEMATICAL THEH" },
    { 0x1EE17, "ARABIC MATHEMATICAL KHAH" },
    { 0x1EE18, "ARABIC MATHEMATICAL THAL" },
    { 0x1EE19, "ARABIC MATHEMATICAL DAD" },
    { 0x1EE1A, "ARABIC MATHEMATICAL ZAH" },
    { 0x1EE1B, "ARABIC MATHEMATICAL GHAIN" },
    { 0x1EE1C, "ARABIC MATHEMATICAL DOTLESS BEH" },
    { 0x1EE1D, "ARABIC MATHEMATICAL DOTLESS NOON" },
    { 0x1EE1E, "ARABIC MATHEMATICAL DOTLESS FEH" },
    { 0x1EE1F, "ARABIC MATHEMATICAL DOTLESS QAF" },
    { 0x1EE21, "ARABIC MATHEMATICAL INITIAL BEH" },
    { 0x1EE22, "ARABIC MATHEMATICAL INITIAL JEEM" },
    { 0x1EE24, "ARABIC MATHEMATICAL INITIAL HEH" },
    { 0x1EE27, "ARABIC MATHEMATICAL INITIAL HAH" },
    { 0x1EE29, "ARABIC MATHEMATICAL INITIAL YEH" },
    { 0x1EE2A, "ARABIC MATHEMATICAL INITIAL KAF" },
    { 0x1EE2B, "ARABIC MATHEMATICAL INITIAL LAM" },
    { 0x1EE2C, "ARABIC MATHEMATICAL INITIAL MEEM" },
    { 0x1EE2D, "ARABIC MATHEMATICAL INITIAL NOON" },
    { 0x1EE2E, "ARABIC MATHEMATICAL INITIAL SEEN" },
    { 0x1EE2F, "ARABIC MATHEMATICAL INITIAL AIN" },
    { 0x1EE30, "ARABIC MATHEMATICAL INITIAL FEH" },
    { 0x1EE31, "ARABIC MATHEMATICAL INITIAL SAD" },
    { 0x1EE32, "ARABIC MATHEMATICAL INITIAL QAF" },
    { 0x1EE34, "ARABIC MATHEMATICAL INITIAL SHEEN" },
    { 0x1EE35, "ARABIC MATHEMATICAL INITIAL TEH" },
    { 0x1EE36, "ARABIC MATHEMATICAL INITIAL THEH" },
    { 0x1EE37, "ARABIC MATHEMATICAL INITIAL KHAH" },
    { 0x1EE39, "ARABIC MATHEMATICAL INITIAL DAD" },
    { 0x1EE3B, "ARABIC MATHEMATICAL INITIAL GHAIN" },
    { 0x1EE42, "ARABIC MATHEMATICAL TAILED JEEM" },
    { 0x1EE47, "ARABIC MATHEMATICAL TAILED HAH" },
    { 0x1EE49, "ARABIC MATHEMATICAL TAILED YEH" },
    { 0x1EE4B, "ARABIC MATHEMATICAL TAILED LAM" },
    { 0x1EE4D, "ARABIC MATHEMATICAL TAILED NOON" },
    { 0x1EE4E, "ARABIC MATHEMATICAL TAILED SEEN" },
    { 0x1EE4F, "ARABIC MATHEMATICAL TAILED AIN" },
    { 0x1EE51, "ARABIC MATHEMATICAL TAILED SAD" },
    { 0x1EE52, "ARABIC MATHEMATICAL TAILED QAF" },
    { 0x1EE54, "ARABIC MATHEMATICAL TAILED SHEEN" },
    { 0x1EE57, "ARABIC MATHEMATICAL TAILED KHAH" },
    { 0x1EE59, "ARABIC MATHEMATICAL TAILED DAD" },
    { 0x1EE5B, "ARABIC MATHEMATICAL TAILED GHAIN" },
    { 0x1EE5D, "ARABIC MATHEMATICAL TAILED DOTLESS NOON" },
    { 0x1EE5F, "ARABIC MATHEMATICAL TAILED DOTLESS QAF" },
    { 0x1EE61, "ARABIC MATHEMATICAL STRETCHED BEH" },
    { 0x1EE62, "ARABIC MATHEMATICAL STRETCHED JEEM" },
    { 0x1EE64, "ARABIC MATHEMATICAL STRETCHED HEH" },
    { 0x1EE67, "ARABIC MATHEMATICAL STRETCHED HAH" },
    { 0x1EE68, "ARABIC MATHEMATICAL STRETCHED TAH" },
    { 0x1EE69, "ARABIC MATHEMATICAL STRETCHED YEH" },
    { 0x1EE6A, "ARABIC MATHEMATICAL STRETCHED KAF" },
    { 0x1EE6C, "ARABIC MATHEMATICAL STRETCHED MEEM" },
    { 0x1EE6D, "ARABIC MATHEMATICAL STRETCHED NOON" },
    { 0x1EE6E, "ARABIC MATHEMATICAL STRETCHED SEEN" },
    { 0x1EE6F, "ARABIC MATHEMATICAL STRETCHED AIN" },
    { 0x1EE70, "ARABIC MATHEMATICAL STRETCHED FEH" },
    { 0x1EE71, "ARABIC MATHEMATICAL STRETCHED SAD" },
    { 0x1EE72, "ARABIC MATHEMATICAL STRETCHED QAF" },
    { 0x1EE74, "ARABIC MATHEMATICAL STRETCHED SHEEN" },
    { 0x1EE75, "ARABIC MATHEMATICAL STRETCHED TEH" },
    { 0x1EE76, "ARABIC MATHEMATICAL STRETCHED THEH" },
    { 0x1EE77, "ARABIC MATHEMATICAL STRETCHED KHAH" },
    { 0x1EE79, "ARABIC MATHEMATICAL STRETCHED DAD" },
    { 0x1EE7A, "ARABIC MATHEMATICAL STRETCHED ZAH" },
    { 0x1EE7B, "ARABIC MATHEMATICAL STRETCHED GHAIN" },
    { 0x1EE7C, "ARABIC MATHEMATICAL STRETCHED DOTLESS BEH" },
    { 0x1EE7E, "ARABIC MATHEMATICAL STRETCHED DOTLESS FEH" },
    { 0x1EE80, "ARABIC MATHEMATICAL LOOPED ALEF" },
    { 0x1EE81, "ARABIC MATHEMATICAL LOOPED BEH" },
    { 0x1EE82, "ARABIC MATHEMATICAL LOOPED JEEM" },
    { 0x1EE83, "ARABIC MATHEMATICAL LOOPED DAL" },
    { 0x1EE84, "ARABIC MATHEMATICAL LOOPED HEH" },
    { 0x1EE85, "ARABIC MATHEMATICAL LOOPED WAW" },
    { 0x1EE86, "ARABIC MATHEMATICAL LOOPED ZAIN" },
    { 0x1EE87, "ARABIC MATHEMATICAL LOOPED HAH" },
    { 0x1EE88, "ARABIC MATHEMATICAL LOOPED TAH" },
    { 0x1EE89, "ARABIC MATHEMATICAL LOOPED YEH" },
    { 0x1EE8B, "ARABIC MATHEMATICAL LOOPED LAM" },
    { 0x1EE8C, "ARABIC MATHEMATICAL LOOPED MEEM" },
    { 0x1EE8D, "ARABIC MATHEMATICAL LOOPED NOON" },
    { 0x1EE8E, "ARABIC MATHEMATICAL LOOPED SEEN" },
    { 0x1EE8F, "ARABIC MATHEMATICAL LOOPED AIN" },
    { 0x1EE90, "ARABIC MATHEMATICAL LOOPED FEH" },
    { 0x1EE91, "ARABIC MATHEMATICAL LOOPED SAD" },
    { 0x1EE92, "ARABIC MATHEMATICAL LOOPED QAF" },
    { 0x1EE93, "ARABIC MATHEMATICAL LOOPED REH" },
    { 0x1EE94, "ARABIC MATHEMATICAL LOOPED SHEEN" },
    { 0x1EE95, "ARABIC MATHEMATICAL LOOPED TEH" },
    { 0x1EE96, "ARABIC MATHEMATICAL LOOPED THEH" },
    { 0x1EE97, "ARABIC MATHEMATICAL LOOPED KHAH" },
    { 0x1EE98, "ARABIC MATHEMATICAL LOOPED THAL" },
    { 0x1EE99, "ARABIC MATHEMATICAL LOOPED DAD" },
    { 0x1EE9A, "ARABIC MATHEMATICAL LOOPED ZAH" },
    { 0x1EE9B, "ARABIC MATHEMATICAL LOOPED GHAIN" },
    { 0x1EEA1, "ARABIC MATHEMATICAL DOUBLE-STRUCK BEH" },
    { 0x1EEA2, "ARABIC MATHEMATICAL DOUBLE-STRUCK JEEM" },
    { 0x1EEA3, "ARABIC MATHEMATICAL DOUBLE-STRUCK DAL" },
    { 0x1EEA5, "ARABIC MATHEMATICAL DOUBLE-STRUCK WAW" },
    { 0x1EEA6, "ARABIC MATHEMATICAL DOUBLE-STRUCK ZAIN" },
    { 0x1EEA7, "ARABIC MATHEMATICAL DOUBLE-STRUCK HAH" },
    { 0x1EEA8, "ARABIC MATHEMATICAL DOUBLE-STRUCK TAH" },
    { 0x1EEA9, "ARABIC MATHEMATICAL DOUBLE-STRUCK YEH" },
    { 0x1EEAB, "ARABIC MATHEMATICAL DOUBLE-STRUCK LAM" },
    { 0x1EEAC, "ARABIC MATHEMATICAL DOUBLE-STRUCK MEEM" },
    { 0x1EEAD, "ARABIC MATHEMATICAL DOUBLE-STRUCK NOON" },
    { 0x1EEAE, "ARABIC MATHEMATICAL DOUBLE-STRUCK SEEN" },
    { 0x1EEAF, "ARABIC MATHEMATICAL DOUBLE-STRUCK AIN" },
    { 0x1EEB0, "ARABIC MATHEMATICAL DOUBLE-STRUCK FEH" },
    { 0x1EEB1, "ARABIC MATHEMATICAL DOUBLE-STRUCK SAD" },
    { 0x1EEB2, "ARABIC MATHEMATICAL DOUBLE-STRUCK QAF" },
    { 0x1EEB3, "ARABIC MATHEMATICAL DOUBLE-STRUCK REH" },
    { 0x1EEB4, "ARABIC MATHEMATICAL DOUBLE-STRUCK SHEEN" },
    { 0x1EEB5, "ARABIC MATHEMATICAL DOUBLE-STRUCK TEH" },
    { 0x1EEB6, "ARABIC MATHEMATICAL DOUBLE-STRUCK THEH" },
    { 0x1EEB7, "ARABIC MATHEMATICAL DOUBLE-STRUCK KHAH" },
    { 0x1EEB8, "ARABIC MATHEMATICAL DOUBLE-STRUCK THAL" },
    { 0x1EEB9, "ARABIC MATHEMATICAL DOUBLE-STRUCK DAD" },
    { 0x1EEBA, "ARABIC MATHEMATICAL DOUBLE-STRUCK ZAH" },
    { 0x1EEBB, "ARABIC MATHEMATICAL DOUBLE-STRUCK GHAIN" },
    { 0x1EEF0, "ARABIC MATHEMATICAL OPERATOR MEEM WITH HAH WITH TATWEEL" },
    { 0x1EEF1, "ARABIC MATHEMATICAL OPERATOR HAH WITH DAL" },
    { 0x1F000, "MAHJONG TILE EAST WIND" },
    { 0x1F001, "MAHJONG TILE SOUTH WIND" },
    { 0x1F002, "MAHJONG TILE WEST WIND" },
    { 0x1F003, "MAHJONG TILE NORTH WIND" },
    { 0x1F005, "MAHJONG TILE GREEN DRAGON" },
    { 0x1F006, "MAHJONG TILE WHITE DRAGON" },
    { 0x1F007, "MAHJONG TILE ONE OF CHARACTERS" },
    { 0x1F008, "MAHJONG TILE TWO OF CHARACTERS" },
    { 0x1F009, "MAHJONG TILE THREE OF CHARACTERS" },
    { 0x1F00A, "MAHJONG TILE FOUR OF CHARACTERS" },
    { 0x1F00B, "MAHJONG TILE FIVE OF CHARACTERS" },
    { 0x1F00C, "MAHJONG TILE SIX OF CHARACTERS" },
    { 0x1F00D, "MAHJONG TILE SEVEN OF CHARACTERS" },
    { 0x1F00E, "MAHJONG TILE EIGHT OF CHARACTERS" },
    { 0x1F00F, "MAHJONG TILE NINE OF CHARACTERS" },
    { 0x1F010, "MAHJONG TILE ONE OF BAMBOOS" },
    { 0x1F011, "MAHJONG TILE TWO OF BAMBOOS" },
    { 0x1F012, "MAHJONG TILE THREE OF BAMBOOS" },
    { 0x1F013, "MAHJONG TILE FOUR OF BAMBOOS" },
    { 0x1F014, "MAHJONG TILE FIVE OF BAMBOOS" },
    { 0x1F015, "MAHJONG TILE SIX OF BAMBOOS" },
    { 0x1F016, "MAHJONG TILE SEVEN OF BAMBOOS" },
    { 0x1F017, "MAHJONG TILE EIGHT OF BAMBOOS" },
    { 0x1F018, "MAHJONG TILE NINE OF BAMBOOS" },
    { 0x1F019, "MAHJONG TILE ONE OF CIRCLES" },
    { 0x1F01A, "MAHJONG TILE TWO OF CIRCLES" },
    { 0x1F01B, "MAHJONG TILE THREE OF CIRCLES" },
    { 0x1F01C, "MAHJONG TILE FOUR OF CIRCLES" },
    { 0x1F01D, "MAHJONG TILE FIVE OF CIRCLES" },
    { 0x1F01E, "MAHJONG TILE SIX OF CIRCLES" },
    { 0x1F01F, "MAHJONG TILE SEVEN OF CIRCLES" },
    { 0x1F020, "MAHJONG TILE EIGHT OF CIRCLES" },
    { 0x1F021, "MAHJONG TILE NINE OF CIRCLES" },
    { 0x1F022, "MAHJONG TILE PLUM" },
    { 0x1F023, "MAHJONG TILE ORCHID" },
    { 0x1F024, "MAHJONG TILE BAMBOO" },
    { 0x1F025, "MAHJONG TILE CHRYSANTHEMUM" },
    { 0x1F026, "MAHJONG TILE SPRING" },
    { 0x1F027, "MAHJONG TILE SUMMER" },
    { 0x1F028, "MAHJONG TILE AUTUMN" },
    { 0x1F029, "MAHJONG TILE WINTER" },
    { 0x1F02A, "MAHJONG TILE JOKER" },
    { 0x1F02B, "MAHJONG TILE BACK" },
    { 0x1F030, "DOMINO TILE HORIZONTAL BACK" },
    { 0x1F031, "DOMINO TILE HORIZONTAL-00-00" },
    { 0x1F032, "DOMINO TILE HORIZONTAL-00-01" },
    { 0x1F033, "DOMINO TILE HORIZONTAL-00-02" },
    { 0x1F034, "DOMINO TILE HORIZONTAL-00-03" },
    { 0x1F035, "DOMINO TILE HORIZONTAL-00-04" },
    { 0x1F036, "DOMINO TILE HORIZONTAL-00-05" },
    { 0x1F037, "DOMINO TILE HORIZONTAL-00-06" },
    { 0x1F038, "DOMINO TILE HORIZONTAL-01-00" },
    { 0x1F039, "DOMINO TILE HORIZONTAL-01-01" },
    { 0x1F03A, "DOMINO TILE HORIZONTAL-01-02" },
    { 0x1F03B, "DOMINO TILE HORIZONTAL-01-03" },
    { 0x1F03C, "DOMINO TILE HORIZONTAL-01-04" },
    { 0x1F03D, "DOMINO TILE HORIZONTAL-01-05" },
    { 0x1F03E, "DOMINO TILE HORIZONTAL-01-06" },
    { 0x1F03F, "DOMINO TILE HORIZONTAL-02-00" },
    { 0x1F040, "DOMINO TILE HORIZONTAL-02-01" },
    { 0x1F041, "DOMINO TILE HORIZONTAL-02-02" },
    { 0x1F042, "DOMINO TILE HORIZONTAL-02-03" },
    { 0x1F043, "DOMINO TILE HORIZONTAL-02-04" },
    { 0x1F044, "DOMINO TILE HORIZONTAL-02-05" },
    { 0x1F045, "DOMINO TILE HORIZONTAL-02-06" },
    { 0x1F046, "DOMINO TILE HORIZONTAL-03-00" },
    { 0x1F047, "DOMINO TILE HORIZONTAL-03-01" },
    { 0x1F048, "DOMINO TILE HORIZONTAL-03-02" },
    { 0x1F049, "DOMINO TILE HORIZONTAL-03-03" },
    { 0x1F04A, "DOMINO TILE HORIZONTAL-03-04" },
    { 0x1F04B, "DOMINO TILE HORIZONTAL-03-05" },
    { 0x1F04C, "DOMINO TILE HORIZONTAL-03-06" },
    { 0x1F04D, "DOMINO TILE HORIZONTAL-04-00" },
    { 0x1F04E, "DOMINO TILE HORIZONTAL-04-01" },
    { 0x1F04F, "DOMINO TILE HORIZONTAL-04-02" },
    { 0x1F050, "DOMINO TILE HORIZONTAL-04-03" },
    { 0x1F051, "DOMINO TILE HORIZONTAL-04-04" },
    { 0x1F052, "DOMINO TILE HORIZONTAL-04-05" },
    { 0x1F053, "DOMINO TILE HORIZONTAL-04-06" },
    { 0x1F054, "DOMINO TILE HORIZONTAL-05-00" },
    { 0x1F055, "DOMINO TILE HORIZONTAL-05-01" },
    { 0x1F056, "DOMINO TILE HORIZONTAL-05-02" },
    { 0x1F057, "DOMINO TILE HORIZONTAL-05-03" },
    { 0x1F058, "DOMINO TILE HORIZONTAL-05-04" },
    { 0x1F059, "DOMINO TILE HORIZONTAL-05-05" },
    { 0x1F05A, "DOMINO TILE HORIZONTAL-05-06" },
    { 0x1F05B, "DOMINO TILE HORIZONTAL-06-00" },
    { 0x1F05C, "DOMINO TILE HORIZONTAL-06-01" },
    { 0x1F05D, "DOMINO TILE HORIZONTAL-06-02" },
    { 0x1F05E, "DOMINO TILE HORIZONTAL-06-03" },
    { 0x1F05F, "DOMINO TILE HORIZONTAL-06-04" },
    { 0x1F060, "DOMINO TILE HORIZONTAL-06-05" },
    { 0x1F061, "DOMINO TILE HORIZONTAL-06-06" },
    { 0x1F062, "DOMINO TILE VERTICAL BACK" },
    { 0x1F063, "DOMINO TILE VERTICAL-00-00" },
    { 0x1F064, "DOMINO TILE VERTICAL-00-01" },
    { 0x1F065, "DOMINO TILE VERTICAL-00-02" },
    { 0x1F066, "DOMINO TILE VERTICAL-00-03" },
    { 0x1F067, "DOMINO TILE VERTICAL-00-04" },
    { 0x1F068, "DOMINO TILE VERTICAL-00-05" },
    { 0x1F069, "DOMINO TILE VERTICAL-00-06" },
    { 0x1F06A, "DOMINO TILE VERTICAL-01-00" },
    { 0x1F06B, "DOMINO TILE VERTICAL-01-01" },
    { 0x1F06C, "DOMINO TILE VERTICAL-01-02" },
    { 0x1F06D, "DOMINO TILE VERTICAL-01-03" },
    { 0x1F06E, "DOMINO TILE VERTICAL-01-04" },
    { 0x1F06F, "DOMINO TILE VERTICAL-01-05" },
    { 0x1F070, "DOMINO TILE VERTICAL-01-06" },
    { 0x1F071, "DOMINO TILE VERTICAL-02-00" },
    { 0x1F072, "DOMINO TILE VERTICAL-02-01" },
    { 0x1F073, "DOMINO TILE VERTICAL-02-02" },
    { 0x1F074, "DOMINO TILE VERTICAL-02-03" },
    { 0x1F075, "DOMINO TILE VERTICAL-02-04" },
    { 0x1F076, "DOMINO TILE VERTICAL-02-05" },
    { 0x1F077, "DOMINO TILE VERTICAL-02-06" },
    { 0x1F078, "DOMINO TILE VERTICAL-03-00" },
    { 0x1F079, "DOMINO TILE VERTICAL-03-01" },
    { 0x1F07A, "DOMINO TILE VERTICAL-03-02" },
    { 0x1F07B, "DOMINO TILE VERTICAL-03-03" },
    { 0x1F07C, "DOMINO TILE VERTICAL-03-04" },
    { 0x1F07D, "DOMINO TILE VERTICAL-03-05" },
    { 0x1F07E, "DOMINO TILE VERTICAL-03-06" },
    { 0x1F07F, "DOMINO TILE VERTICAL-04-00" },
    { 0x1F080, "DOMINO TILE VERTICAL-04-01" },
    { 0x1F081, "DOMINO TILE VERTICAL-04-02" },
    { 0x1F082, "DOMINO TILE VERTICAL-04-03" },
    { 0x1F083, "DOMINO TILE VERTICAL-04-04" },
    { 0x1F084, "DOMINO TILE VERTICAL-04-05" },
    { 0x1F085, "DOMINO TILE VERTICAL-04-06" },
    { 0x1F086, "DOMINO TILE VERTICAL-05-00" },
    { 0x1F087, "DOMINO TILE VERTICAL-05-01" },
    { 0x1F088, "DOMINO TILE VERTICAL-05-02" },
    { 0x1F089, "DOMINO TILE VERTICAL-05-03" },
    { 0x1F08A, "DOMINO TILE VERTICAL-05-04" },
    { 0x1F08B, "DOMINO TILE VERTICAL-05-05" },
    { 0x1F08C, "DOMINO TILE VERTICAL-05-06" },
    { 0x1F08D, "DOMINO TILE VERTICAL-06-00" },
    { 0x1F08E, "DOMINO TILE VERTICAL-06-01" },
    { 0x1F08F, "DOMINO TILE VERTICAL-06-02" },
    { 0x1F090, "DOMINO TILE VERTICAL-06-03" },
    { 0x1F091, "DOMINO TILE VERTICAL-06-04" },
    { 0x1F092, "DOMINO TILE VERTICAL-06-05" },
    { 0x1F093, "DOMINO TILE VERTICAL-06-06" },
    { 0x1F0A0, "PLAYING CARD BACK" },
    { 0x1F0A1, "PLAYING CARD ACE OF SPADES" },
    { 0x1F0A2, "PLAYING CARD TWO OF SPADES" },
    { 0x1F0A3, "PLAYING CARD THREE OF SPADES" },
    { 0x1F0A4, "PLAYING CARD FOUR OF SPADES" },
    { 0x1F0A5, "PLAYING CARD FIVE OF SPADES" },
    { 0x1F0A6, "PLAYING CARD SIX OF SPADES" },
    { 0x1F0A7, "PLAYING CARD SEVEN OF SPADES" },
    { 0x1F0A8, "PLAYING CARD EIGHT OF SPADES" },
    { 0x1F0A9, "PLAYING CARD NINE OF SPADES" },
    { 0x1F0AA, "PLAYING CARD TEN OF SPADES" },
    { 0x1F0AB, "PLAYING CARD JACK OF SPADES" },
    { 0x1F0AC, "PLAYING CARD KNIGHT OF SPADES" },
    { 0x1F0AD, "PLAYING CARD QUEEN OF SPADES" },
    { 0x1F0AE, "PLAYING CARD KING OF SPADES" },
    { 0x1F0B1, "PLAYING CARD ACE OF HEARTS" },
    { 0x1F0B2, "PLAYING CARD TWO OF HEARTS" },
    { 0x1F0B3, "PLAYING CARD THREE OF HEARTS" },
    { 0x1F0B4, "PLAYING CARD FOUR OF HEARTS" },
    { 0x1F0B5, "PLAYING CARD FIVE OF HEARTS" },
    { 0x1F0B6, "PLAYING CARD SIX OF HEARTS" },
    { 0x1F0B7, "PLAYING CARD SEVEN OF HEARTS" },
    { 0x1F0B8, "PLAYING CARD EIGHT OF HEARTS" },
    { 0x1F0B9, "PLAYING CARD NINE OF HEARTS" },
    { 0x1F0BA, "PLAYING CARD TEN OF HEARTS" },
    { 0x1F0BB, "PLAYING CARD JACK OF HEARTS" },
    { 0x1F0BC, "PLAYING CARD KNIGHT OF HEARTS" },
    { 0x1F0BD, "PLAYING CARD QUEEN OF HEARTS" },
    { 0x1F0BE, "PLAYING CARD KING OF HEARTS" },
    { 0x1F0BF, "PLAYING CARD RED JOKER" },
    { 0x1F0C1, "PLAYING CARD ACE OF DIAMONDS" },
    { 0x1F0C2, "PLAYING CARD TWO OF DIAMONDS" },
    { 0x1F0C3, "PLAYING CARD THREE OF DIAMONDS" },
    { 0x1F0C4, "PLAYING CARD FOUR OF DIAMONDS" },
    { 0x1F0C5, "PLAYING CARD FIVE OF DIAMONDS" },
    { 0x1F0C6, "PLAYING CARD SIX OF DIAMONDS" },
    { 0x1F0C7, "PLAYING CARD SEVEN OF DIAMONDS" },
    { 0x1F0C8, "PLAYING CARD EIGHT OF DIAMONDS" },
    { 0x1F0C9, "PLAYING CARD NINE OF DIAMONDS" },
    { 0x1F0CA, "PLAYING CARD TEN OF DIAMONDS" },
    { 0x1F0CB, "PLAYING CARD JACK OF DIAMONDS" },
    { 0x1F0CC, "PLAYING CARD KNIGHT OF DIAMONDS" },
    { 0x1F0CD, "PLAYING CARD QUEEN OF DIAMONDS" },
    { 0x1F0CE, "PLAYING CARD KING OF DIAMONDS" },
    { 0x1F0D1, "PLAYING CARD ACE OF CLUBS" },
    { 0x1F0D2, "PLAYING CARD TWO OF CLUBS" },
    { 0x1F0D3, "PLAYING CARD THREE OF CLUBS" },
    { 0x1F0D4, "PLAYING CARD FOUR OF CLUBS" },
    { 0x1F0D5, "PLAYING CARD FIVE OF CLUBS" },
    { 0x1F0D6, "PLAYING CARD SIX OF CLUBS" },
    { 0x1F0D7, "PLAYING CARD SEVEN OF CLUBS" },
    { 0x1F0D8, "PLAYING CARD EIGHT OF CLUBS" },
    { 0x1F0D9, "PLAYING CARD NINE OF CLUBS" },
    { 0x1F0DA, "PLAYING CARD TEN OF CLUBS" },
    { 0x1F0DB, "PLAYING CARD JACK OF CLUBS" },
    { 0x1F0DC, "PLAYING CARD KNIGHT OF CLUBS" },
    { 0x1F0DD, "PLAYING CARD QUEEN OF CLUBS" },
    { 0x1F0DE, "PLAYING CARD KING OF CLUBS" },
    { 0x1F0DF, "PLAYING CARD WHITE JOKER" },
    { 0x1F0E0, "PLAYING CARD FOOL" },
    { 0x1F0E1, "PLAYING CARD TRUMP-1" },
    { 0x1F0E2, "PLAYING CARD TRUMP-2" },
    { 0x1F0E3, "PLAYING CARD TRUMP-3" },
    { 0x1F0E4, "PLAYING CARD TRUMP-4" },
    { 0x1F0E5, "PLAYING CARD TRUMP-5" },
    { 0x1F0E6, "PLAYING CARD TRUMP-6" },
    { 0x1F0E7, "PLAYING CARD TRUMP-7" },
    { 0x1F0E8, "PLAYING CARD TRUMP-8" },
    { 0x1F0E9, "PLAYING CARD TRUMP-9" },
    { 0x1F0EA, "PLAYING CARD TRUMP-10" },
    { 0x1F0EB, "PLAYING CARD TRUMP-11" },
    { 0x1F0EC, "PLAYING CARD TRUMP-12" },
    { 0x1F0ED, "PLAYING CARD TRUMP-13" },
    { 0x1F0EE, "PLAYING CARD TRUMP-14" },
    { 0x1F0EF, "PLAYING CARD TRUMP-15" },
    { 0x1F0F0, "PLAYING CARD TRUMP-16" },
    { 0x1F0F1, "PLAYING CARD TRUMP-17" },
    { 0x1F0F2, "PLAYING CARD TRUMP-18" },
    { 0x1F0F3, "PLAYING CARD TRUMP-19" },
    { 0x1F0F4, "PLAYING CARD TRUMP-20" },
    { 0x1F0F5, "PLAYING CARD TRUMP-21" },
    { 0x1F100, "DIGIT ZERO FULL STOP" },
    { 0x1F101, "DIGIT ZERO COMMA" },
    { 0x1F102, "DIGIT ONE COMMA" },
    { 0x1F103, "DIGIT TWO COMMA" },
    { 0x1F104, "DIGIT THREE COMMA" },
    { 0x1F105, "DIGIT FOUR COMMA" },
    { 0x1F106, "DIGIT FIVE COMMA" },
    { 0x1F107, "DIGIT SIX COMMA" },
    { 0x1F108, "DIGIT SEVEN COMMA" },
    { 0x1F109, "DIGIT EIGHT COMMA" },
    { 0x1F10A, "DIGIT NINE COMMA" },
    { 0x1F10B, "DINGBAT CIRCLED SANS-SERIF DIGIT ZERO" },
    { 0x1F10C, "DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ZERO" },
    { 0x1F10D, "CIRCLED ZERO WITH SLASH" },
    { 0x1F10E, "CIRCLED ANTICLOCKWISE ARROW" },
    { 0x1F10F, "CIRCLED DOLLAR SIGN WITH OVERLAID BACKSLASH" },
    { 0x1F110, "PARENTHESIZED LATIN CAPITAL LETTER A" },
    { 0x1F111, "PARENTHESIZED LATIN CAPITAL LETTER B" },
    { 0x1F112, "PARENTHESIZED LATIN CAPITAL LETTER C" },
    { 0x1F113, "PARENTHESIZED LATIN CAPITAL LETTER D" },
    { 0x1F114, "PARENTHESIZED LATIN CAPITAL LETTER E" },
    { 0x1F115, "PARENTHESIZED LATIN CAPITAL LETTER F" },
    { 0x1F116, "PARENTHESIZED LATIN CAPITAL LETTER G" },
    { 0x1F117, "PARENTHESIZED LATIN CAPITAL LETTER H" },
    { 0x1F118, "PARENTHESIZED LATIN CAPITAL LETTER I" },
    { 0x1F119, "PARENTHESIZED LATIN CAPITAL LETTER J" },
    { 0x1F11A, "PARENTHESIZED LATIN CAPITAL LETTER K" },
    { 0x1F11B, "PARENTHESIZED LATIN CAPITAL LETTER L" },
    { 0x1F11C, "PARENTHESIZED LATIN CAPITAL LETTER M" },
    { 0x1F11D, "PARENTHESIZED LATIN CAPITAL LETTER N" },
    { 0x1F11E, "PARENTHESIZED LATIN CAPITAL LETTER O" },
    { 0x1F11F, "PARENTHESIZED LATIN CAPITAL LETTER P" },
    { 0x1F120, "PARENTHESIZED LATIN CAPITAL LETTER Q" },
    { 0x1F121, "PARENTHESIZED LATIN CAPITAL LETTER R" },
    { 0x1F122, "PARENTHESIZED LATIN CAPITAL LETTER S" },
    { 0x1F123, "PARENTHESIZED LATIN CAPITAL LETTER T" },
    { 0x1F124, "PARENTHESIZED LATIN CAPITAL LETTER U" },
    { 0x1F125, "PARENTHESIZED LATIN CAPITAL LETTER V" },
    { 0x1F126, "PARENTHESIZED LATIN CAPITAL LETTER W" },
    { 0x1F127, "PARENTHESIZED LATIN CAPITAL LETTER X" },
    { 0x1F128, "PARENTHESIZED LATIN CAPITAL LETTER Y" },
    { 0x1F129, "PARENTHESIZED LATIN CAPITAL LETTER Z" },
    { 0x1F12A, "TORTOISE SHELL BRACKETED LATIN CAPITAL LETTER S" },
    { 0x1F12B, "CIRCLED ITALIC LATIN CAPITAL LETTER C" },
    { 0x1F12C, "CIRCLED ITALIC LATIN CAPITAL LETTER R" },
    { 0x1F12D, "CIRCLED CD" },
    { 0x1F12E, "CIRCLED WZ" },
    { 0x1F12F, "COPYLEFT SYMBOL" },
    { 0x1F130, "SQUARED LATIN CAPITAL LETTER A" },
    { 0x1F131, "SQUARED LATIN CAPITAL LETTER B" },
    { 0x1F132, "SQUARED LATIN CAPITAL LETTER C" },
    { 0x1F133, "SQUARED LATIN CAPITAL LETTER D" },
    { 0x1F134, "SQUARED LATIN CAPITAL LETTER E" },
    { 0x1F135, "SQUARED LATIN CAPITAL LETTER F" },
    { 0x1F136, "SQUARED LATIN CAPITAL LETTER G" },
    { 0x1F137, "SQUARED LATIN CAPITAL LETTER H" },
    { 0x1F138, "SQUARED LATIN CAPITAL LETTER I" },
    { 0x1F139, "SQUARED LATIN CAPITAL LETTER J" },
    { 0x1F13A, "SQUARED LATIN CAPITAL LETTER K" },
    { 0x1F13B, "SQUARED LATIN CAPITAL LETTER L" },
    { 0x1F13C, "SQUARED LATIN CAPITAL LETTER M" },
    { 0x1F13D, "SQUARED LATIN CAPITAL LETTER N" },
    { 0x1F13E, "SQUARED LATIN CAPITAL LETTER O" },
    { 0x1F13F, "SQUARED LATIN CAPITAL LETTER P" },
    { 0x1F140, "SQUARED LATIN CAPITAL LETTER Q" },
    { 0x1F141, "SQUARED LATIN CAPITAL LETTER R" },
    { 0x1F142, "SQUARED LATIN CAPITAL LETTER S" },
    { 0x1F143, "SQUARED LATIN CAPITAL LETTER T" },
    { 0x1F144, "SQUARED LATIN CAPITAL LETTER U" },
    { 0x1F145, "SQUARED LATIN CAPITAL LETTER V" },
    { 0x1F146, "SQUARED LATIN CAPITAL LETTER W" },
    { 0x1F147, "SQUARED LATIN CAPITAL LETTER X" },
    { 0x1F148, "SQUARED LATIN CAPITAL LETTER Y" },
    { 0x1F149, "SQUARED LATIN CAPITAL LETTER Z" },
    { 0x1F14A, "SQUARED HV" },
    { 0x1F14B, "SQUARED MV" },
    { 0x1F14C, "SQUARED SD" },
    { 0x1F14D, "SQUARED SS" },
    { 0x1F14E, "SQUARED PPV" },
    { 0x1F14F, "SQUARED WC" },
    { 0x1F150, "NEGATIVE CIRCLED LATIN CAPITAL LETTER A" },
    { 0x1F151, "NEGATIVE CIRCLED LATIN CAPITAL LETTER B" },
    { 0x1F152, "NEGATIVE CIRCLED LATIN CAPITAL LETTER C" },
    { 0x1F153, "NEGATIVE CIRCLED LATIN CAPITAL LETTER D" },
    { 0x1F154, "NEGATIVE CIRCLED LATIN CAPITAL LETTER E" },
    { 0x1F155, "NEGATIVE CIRCLED LATIN CAPITAL LETTER F" },
    { 0x1F156, "NEGATIVE CIRCLED LATIN CAPITAL LETTER G" },
    { 0x1F157, "NEGATIVE CIRCLED LATIN CAPITAL LETTER H" },
    { 0x1F158, "NEGATIVE CIRCLED LATIN CAPITAL LETTER I" },
    { 0x1F159, "NEGATIVE CIRCLED LATIN CAPITAL LETTER J" },
    { 0x1F15A, "NEGATIVE CIRCLED LATIN CAPITAL LETTER K" },
    { 0x1F15B, "NEGATIVE CIRCLED LATIN CAPITAL LETTER L" },
    { 0x1F15C, "NEGATIVE CIRCLED LATIN CAPITAL LETTER M" },
    { 0x1F15D, "NEGATIVE CIRCLED LATIN CAPITAL LETTER N" },
    { 0x1F15E, "NEGATIVE CIRCLED LATIN CAPITAL LETTER O" },
    { 0x1F15F, "NEGATIVE CIRCLED LATIN CAPITAL LETTER P" },
    { 0x1F160, "NEGATIVE CIRCLED LATIN CAPITAL LETTER Q" },
    { 0x1F161, "NEGATIVE CIRCLED LATIN CAPITAL LETTER R" },
    { 0x1F162, "NEGATIVE CIRCLED LATIN CAPITAL LETTER S" },
    { 0x1F163, "NEGATIVE CIRCLED LATIN CAPITAL LETTER T" },
    { 0x1F164, "NEGATIVE CIRCLED LATIN CAPITAL LETTER U" },
    { 0x1F165, "NEGATIVE CIRCLED LATIN CAPITAL LETTER V" },
    { 0x1F166, "NEGATIVE CIRCLED LATIN CAPITAL LETTER W" },
    { 0x1F167, "NEGATIVE CIRCLED LATIN CAPITAL LETTER X" },
    { 0x1F168, "NEGATIVE CIRCLED LATIN CAPITAL LETTER Y" },
    { 0x1F169, "NEGATIVE CIRCLED LATIN CAPITAL LETTER Z" },
    { 0x1F16A, "RAISED MC SIGN" },
    { 0x1F16B, "RAISED MD SIGN" },
    { 0x1F16C, "RAISED MR SIGN" },
    { 0x1F16D, "CIRCLED CC" },
    { 0x1F16E, "CIRCLED C WITH OVERLAID BACKSLASH" },
    { 0x1F16F, "CIRCLED HUMAN FIGURE" },
    { 0x1F170, "NEGATIVE SQUARED LATIN CAPITAL LETTER A" },
    { 0x1F171, "NEGATIVE SQUARED LATIN CAPITAL LETTER B" },
    { 0x1F172, "NEGATIVE SQUARED LATIN CAPITAL LETTER C" },
    { 0x1F173, "NEGATIVE SQUARED LATIN CAPITAL LETTER D" },
    { 0x1F174, "NEGATIVE SQUARED LATIN CAPITAL LETTER E" },
    { 0x1F175, "NEGATIVE SQUARED LATIN CAPITAL LETTER F" },
    { 0x1F176, "NEGATIVE SQUARED LATIN CAPITAL LETTER G" },
    { 0x1F177, "NEGATIVE SQUARED LATIN CAPITAL LETTER H" },
    { 0x1F178, "NEGATIVE SQUARED LATIN CAPITAL LETTER I" },
    { 0x1F179, "NEGATIVE SQUARED LATIN CAPITAL LETTER J" },
    { 0x1F17A, "NEGATIVE SQUARED LATIN CAPITAL LETTER K" },
    { 0x1F17B, "NEGATIVE SQUARED LATIN CAPITAL LETTER L" },
    { 0x1F17C, "NEGATIVE SQUARED LATIN CAPITAL LETTER M" },
    { 0x1F17D, "NEGATIVE SQUARED LATIN CAPITAL LETTER N" },
    { 0x1F17E, "NEGATIVE SQUARED LATIN CAPITAL LETTER O" },
    { 0x1F17F, "NEGATIVE SQUARED LATIN CAPITAL LETTER P" },
    { 0x1F180, "NEGATIVE SQUARED LATIN CAPITAL LETTER Q" },
    { 0x1F181, "NEGATIVE SQUARED LATIN CAPITAL LETTER R" },
    { 0x1F182, "NEGATIVE SQUARED LATIN CAPITAL LETTER S" },
    { 0x1F183, "NEGATIVE SQUARED LATIN CAPITAL LETTER T" },
    { 0x1F184, "NEGATIVE SQUARED LATIN CAPITAL LETTER U" },
    { 0x1F185, "NEGATIVE SQUARED LATIN CAPITAL LETTER V" },
    { 0x1F186, "NEGATIVE SQUARED LATIN CAPITAL LETTER W" },
    { 0x1F187, "NEGATIVE SQUARED LATIN CAPITAL LETTER X" },
    { 0x1F188, "NEGATIVE SQUARED LATIN CAPITAL LETTER Y" },
    { 0x1F189, "NEGATIVE SQUARED LATIN CAPITAL LETTER Z" },
    { 0x1F18A, "CROSSED NEGATIVE SQUARED LATIN CAPITAL LETTER P" },
    { 0x1F18B, "NEGATIVE SQUARED IC" },
    { 0x1F18C, "NEGATIVE SQUARED PA" },
    { 0x1F18D, "NEGATIVE SQUARED SA" },
    { 0x1F18F, "NEGATIVE SQUARED WC" },
    { 0x1F190, "SQUARE DJ" },
    { 0x1F19B, "SQUARED THREE D" },
    { 0x1F19C, "SQUARED SECOND SCREEN" },
    { 0x1F19D, "SQUARED TWO K" },
    { 0x1F19E, "SQUARED FOUR K" },
    { 0x1F19F, "SQUARED EIGHT K" },
    { 0x1F1A0, "SQUARED FIVE POINT ONE" },
    { 0x1F1A1, "SQUARED SEVEN POINT ONE" },
    { 0x1F1A2, "SQUARED TWENTY-TWO POINT TWO" },
    { 0x1F1A3, "SQUARED SIXTY P" },
    { 0x1F1A4, "SQUARED ONE HUNDRED TWENTY P" },
    { 0x1F1A5, "SQUARED LATIN SMALL LETTER D" },
    { 0x1F1A6, "SQUARED HC" },
    { 0x1F1A7, "SQUARED HDR" },
    { 0x1F1A8, "SQUARED HI-RES" },
    { 0x1F1A9, "SQUARED LOSSLESS" },
    { 0x1F1AA, "SQUARED SHV" },
    { 0x1F1AB, "SQUARED UHD" },
    { 0x1F1AC, "SQUARED VOD" },
    { 0x1F1AD, "MASK WORK SYMBOL" },
    { 0x1F1E6, "REGIONAL INDICATOR SYMBOL LETTER A" },
    { 0x1F1E7, "REGIONAL INDICATOR SYMBOL LETTER B" },
    { 0x1F1E8, "REGIONAL INDICATOR SYMBOL LETTER C" },
    { 0x1F1E9, "REGIONAL INDICATOR SYMBOL LETTER D" },
    { 0x1F1EA, "REGIONAL INDICATOR SYMBOL LETTER E" },
    { 0x1F1EB, "REGIONAL INDICATOR SYMBOL LETTER F" },
    { 0x1F1EC, "REGIONAL INDICATOR SYMBOL LETTER G" },
    { 0x1F1ED, "REGIONAL INDICATOR SYMBOL LETTER H" },
    { 0x1F1EE, "REGIONAL INDICATOR SYMBOL LETTER I" },
    { 0x1F1EF, "REGIONAL INDICATOR SYMBOL LETTER J" },
    { 0x1F1F0, "REGIONAL INDICATOR SYMBOL LETTER K" },
    { 0x1F1F1, "REGIONAL INDICATOR SYMBOL LETTER L" },
    { 0x1F1F2, "REGIONAL INDICATOR SYMBOL LETTER M" },
    { 0x1F1F3, "REGIONAL INDICATOR SYMBOL LETTER N" },
    { 0x1F1F4, "REGIONAL INDICATOR SYMBOL LETTER O" },
    { 0x1F1F5, "REGIONAL INDICATOR SYMBOL LETTER P" },
    { 0x1F1F6, "REGIONAL INDICATOR SYMBOL LETTER Q" },
    { 0x1F1F7, "REGIONAL INDICATOR SYMBOL LETTER R" },
    { 0x1F1F8, "REGIONAL INDICATOR SYMBOL LETTER S" },
    { 0x1F1F9, "REGIONAL INDICATOR SYMBOL LETTER T" },
    { 0x1F1FA, "REGIONAL INDICATOR SYMBOL LETTER U" },
    { 0x1F1FB, "REGIONAL INDICATOR SYMBOL LETTER V" },
    { 0x1F1FC, "REGIONAL INDICATOR SYMBOL LETTER W" },
    { 0x1F1FD, "REGIONAL INDICATOR SYMBOL LETTER X" },
    { 0x1F1FE, "REGIONAL INDICATOR SYMBOL LETTER Y" },
    { 0x1F1FF, "REGIONAL INDICATOR SYMBOL LETTER Z" },
    { 0x1F200, "SQUARE HIRAGANA HOKA" },
    { 0x1F202, "SQUARED KATAKANA SA" },
    { 0x1F210, "SQUARED CJK UNIFIED IDEOGRAPH-624B" },
    { 0x1F211, "SQUARED CJK UNIFIED IDEOGRAPH-5B57" },
    { 0x1F212, "SQUARED CJK UNIFIED IDEOGRAPH-53CC" },
    { 0x1F213, "SQUARED KATAKANA DE" },
    { 0x1F214, "SQUARED CJK UNIFIED IDEOGRAPH-4E8C" },
    { 0x1F215, "SQUARED CJK UNIFIED IDEOGRAPH-591A" },
    { 0x1F216, "SQUARED CJK UNIFIED IDEOGRAPH-89E3" },
    { 0x1F217, "SQUARED CJK UNIFIED IDEOGRAPH-5929" },
    { 0x1F218, "SQUARED CJK UNIFIED IDEOGRAPH-4EA4" },
    { 0x1F219, "SQUARED CJK UNIFIED IDEOGRAPH-6620" },
    { 0x1F21B, "SQUARED CJK UNIFIED IDEOGRAPH-6599" },
    { 0x1F21C, "SQUARED CJK UNIFIED IDEOGRAPH-524D" },
    { 0x1F21D, "SQUARED CJK UNIFIED IDEOGRAPH-5F8C" },
    { 0x1F21E, "SQUARED CJK UNIFIED IDEOGRAPH-518D" },
    { 0x1F21F, "SQUARED CJK UNIFIED IDEOGRAPH-65B0" },
    { 0x1F220, "SQUARED CJK UNIFIED IDEOGRAPH-521D" },
    { 0x1F221, "SQUARED CJK UNIFIED IDEOGRAPH-7D42" },
    { 0x1F222, "SQUARED CJK UNIFIED IDEOGRAPH-751F" },
    { 0x1F223, "SQUARED CJK UNIFIED IDEOGRAPH-8CA9" },
    { 0x1F224, "SQUARED CJK UNIFIED IDEOGRAPH-58F0" },
    { 0x1F225, "SQUARED CJK UNIFIED IDEOGRAPH-5439" },
    { 0x1F226, "SQUARED CJK UNIFIED IDEOGRAPH-6F14" },
    { 0x1F227, "SQUARED CJK UNIFIED IDEOGRAPH-6295" },
    { 0x1F228, "SQUARED CJK UNIFIED IDEOGRAPH-6355" },
    { 0x1F229, "SQUARED CJK UNIFIED IDEOGRAPH-4E00" },
    { 0x1F22A, "SQUARED CJK UNIFIED IDEOGRAPH-4E09" },
    { 0x1F22B, "SQUARED CJK UNIFIED IDEOGRAPH-904A" },
    { 0x1F22C, "SQUARED CJK UNIFIED IDEOGRAPH-5DE6" },
    { 0x1F22D, "SQUARED CJK UNIFIED IDEOGRAPH-4E2D" },
    { 0x1F22E, "SQUARED CJK UNIFIED IDEOGRAPH-53F3" },
    { 0x1F230, "SQUARED CJK UNIFIED IDEOGRAPH-8D70" },
    { 0x1F231, "SQUARED CJK UNIFIED IDEOGRAPH-6253" },
    { 0x1F237, "SQUARED CJK UNIFIED IDEOGRAPH-6708" },
    { 0x1F23B, "SQUARED CJK UNIFIED IDEOGRAPH-914D" },
    { 0x1F240, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-672C" },
    { 0x1F241, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-4E09" },
    { 0x1F242, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-4E8C" },
    { 0x1F243, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-5B89" },
    { 0x1F244, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-70B9" },
    { 0x1F245, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-6253" },
    { 0x1F246, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-76D7" },
    { 0x1F247, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-52DD" },
    { 0x1F248, "TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-6557" },
    { 0x1F260, "ROUNDED SYMBOL FOR FU" },
    { 0x1F261, "ROUNDED SYMBOL FOR LU" },
    { 0x1F262, "ROUNDED SYMBOL FOR SHOU" },
    { 0x1F263, "ROUNDED SYMBOL FOR XI" },
    { 0x1F264, "ROUNDED SYMBOL FOR SHUANGXI" },
    { 0x1F265, "ROUNDED SYMBOL FOR CAI" },
    { 0x1F321, "THERMOMETER" },
    { 0x1F322, "BLACK DROPLET" },
    { 0x1F323, "WHITE SUN" },
    { 0x1F324, "WHITE SUN WITH SMALL CLOUD" },
    { 0x1F325, "WHITE SUN BEHIND CLOUD" },
    { 0x1F326, "WHITE SUN BEHIND CLOUD WITH RAIN" },
    { 0x1F327, "CLOUD WITH RAIN" },
    { 0x1F328, "CLOUD WITH SNOW" },
    { 0x1F329, "CLOUD WITH LIGHTNING" },
    { 0x1F32A, "CLOUD WITH TORNADO" },
    { 0x1F32B, "FOG" },
    { 0x1F32C, "WIND BLOWING FACE" },
    { 0x1F336, "HOT PEPPER" },
    { 0x1F37D, "FORK AND KNIFE WITH PLATE" },
    { 0x1F394, "HEART WITH TIP ON THE LEFT" },
    { 0x1F395, "BOUQUET OF FLOWERS" },
    { 0x1F396, "MILITARY MEDAL" },
    { 0x1F397, "REMINDER RIBBON" },
    { 0x1F398, "MUSICAL KEYBOARD WITH JACKS" },
    { 0x1F399, "STUDIO MICROPHONE" },
    { 0x1F39A, "LEVEL SLIDER" },
    { 0x1F39B, "CONTROL KNOBS" },
    { 0x1F39C, "BEAMED ASCENDING MUSICAL NOTES" },
    { 0x1F39D, "BEAMED DESCENDING MUSICAL NOTES" },
    { 0x1F39E, "FILM FRAMES" },
    { 0x1F39F, "ADMISSION TICKETS" },
    { 0x1F3CB, "WEIGHT LIFTER" },
    { 0x1F3CC, "GOLFER" },
    { 0x1F3CD, "RACING MOTORCYCLE" },
    { 0x1F3CE, "RACING CAR" },
    { 0x1F3D4, "SNOW CAPPED MOUNTAIN" },
    { 0x1F3D5, "CAMPING" },
    { 0x1F3D6, "BEACH WITH UMBRELLA" },
    { 0x1F3D7, "BUILDING CONSTRUCTION" },
    { 0x1F3D8, "HOUSE BUILDINGS" },
    { 0x1F3D9, "CITYSCAPE" },
    { 0x1F3DA, "DERELICT HOUSE BUILDING" },
    { 0x1F3DB, "CLASSICAL BUILDING" },
    { 0x1F3DC, "DESERT" },
    { 0x1F3DD, "DESERT ISLAND" },
    { 0x1F3DE, "NATIONAL PARK" },
    { 0x1F3DF, "STADIUM" },
    { 0x1F3F1, "WHITE PENNANT" },
    { 0x1F3F2, "BLACK PENNANT" },
    { 0x1F3F3, "WAVING WHITE FLAG" },
    { 0x1F3F5, "ROSETTE" },
    { 0x1F3F6, "BLACK ROSETTE" },
    { 0x1F3F7, "LABEL" },
    { 0x1F3FB, "EMOJI MODIFIER FITZPATRICK TYPE-1-2" },
    { 0x1F3FC, "EMOJI MODIFIER FITZPATRICK TYPE-3" },
    { 0x1F3FD, "EMOJI MODIFIER FITZPATRICK TYPE-4" },
    { 0x1F3FE, "EMOJI MODIFIER FITZPATRICK TYPE-5" },
    { 0x1F3FF, "EMOJI MODIFIER FITZPATRICK TYPE-6" },
    { 0x1F43F, "CHIPMUNK" },
    { 0x1F441, "EYE" },
    { 0x1F4FD, "FILM PROJECTOR" },
    { 0x1F4FE, "PORTABLE STEREO" },
    { 0x1F53E, "LOWER RIGHT SHADOWED WHITE CIRCLE" },
    { 0x1F53F, "UPPER RIGHT SHADOWED WHITE CIRCLE" },
    { 0x1F540, "CIRCLED CROSS POMMEE" },
    { 0x1F541, "CROSS POMMEE WITH HALF-CIRCLE BELOW" },
    { 0x1F542, "CROSS POMMEE" },
    { 0x1F543, "NOTCHED LEFT SEMICIRCLE WITH THREE DOTS" },
    { 0x1F544, "NOTCHED RIGHT SEMICIRCLE WITH THREE DOTS" },
    { 0x1F545, "SYMBOL FOR MARKS CHAPTER" },
    { 0x1F546, "WHITE LATIN CROSS" },
    { 0x1F547, "HEAVY LATIN CROSS" },
    { 0x1F548, "CELTIC CROSS" },
    { 0x1F549, "OM SYMBOL" },
    { 0x1F54A, "DOVE OF PEACE" },
    { 0x1F54F, "BOWL OF HYGIEIA" },
    { 0x1F568, "RIGHT SPEAKER" },
    { 0x1F569, "RIGHT SPEAKER WITH ONE SOUND WAVE" },
    { 0x1F56A, "RIGHT SPEAKER WITH THREE SOUND WAVES" },
    { 0x1F56B, "BULLHORN" },
    { 0x1F56C, "BULLHORN WITH SOUND WAVES" },
    { 0x1F56D, "RINGING BELL" },
    { 0x1F56E, "BOOK" },
    { 0x1F56F, "CANDLE" },
    { 0x1F570, "MANTELPIECE CLOCK" },
    { 0x1F571, "BLACK SKULL AND CROSSBONES" },
    { 0x1F572, "NO PIRACY" },
    { 0x1F573, "HOLE" },
    { 0x1F574, "MAN IN BUSINESS SUIT LEVITATING" },
    { 0x1F575, "SLEUTH OR SPY" },
    { 0x1F576, "DARK SUNGLASSES" },
    { 0x1F577, "SPIDER" },
    { 0x1F578, "SPIDER WEB" },
    { 0x1F579, "JOYSTICK" },
    { 0x1F57B, "LEFT HAND TELEPHONE RECEIVER" },
    { 0x1F57C, "TELEPHONE RECEIVER WITH PAGE" },
    { 0x1F57D, "RIGHT HAND TELEPHONE RECEIVER" },
    { 0x1F57E, "WHITE TOUCHTONE TELEPHONE" },
    { 0x1F57F, "BLACK TOUCHTONE TELEPHONE" },
    { 0x1F580, "TELEPHONE ON TOP OF MODEM" },
    { 0x1F581, "CLAMSHELL MOBILE PHONE" },
    { 0x1F582, "BACK OF ENVELOPE" },
    { 0x1F583, "STAMPED ENVELOPE" },
    { 0x1F584, "ENVELOPE WITH LIGHTNING" },
    { 0x1F585, "FLYING ENVELOPE" },
    { 0x1F586, "PEN OVER STAMPED ENVELOPE" },
    { 0x1F587, "LINKED PAPERCLIPS" },
    { 0x1F588, "BLACK PUSHPIN" },
    { 0x1F589, "LOWER LEFT PENCIL" },
    { 0x1F58A, "LOWER LEFT BALLPOINT PEN" },
    { 0x1F58B, "LOWER LEFT FOUNTAIN PEN" },
    { 0x1F58C, "LOWER LEFT PAINTBRUSH" },
    { 0x1F58D, "LOWER LEFT CRAYON" },
    { 0x1F58E, "LEFT WRITING HAND" },
    { 0x1F58F, "TURNED OK HAND SIGN" },
    { 0x1F590, "RAISED HAND WITH FINGERS SPLAYED" },
    { 0x1F591, "REVERSED RAISED HAND WITH FINGERS SPLAYED" },
    { 0x1F592, "REVERSED THUMBS UP SIGN" },
    { 0x1F593, "REVERSED THUMBS DOWN SIGN" },
    { 0x1F594, "REVERSED VICTORY HAND" },
    { 0x1F597, "WHITE DOWN POINTING LEFT HAND INDEX" },
    { 0x1F598, "SIDEWAYS WHITE LEFT POINTING INDEX" },
    { 0x1F599, "SIDEWAYS WHITE RIGHT POINTING INDEX" },
    { 0x1F59A, "SIDEWAYS BLACK LEFT POINTING INDEX" },
    { 0x1F59B, "SIDEWAYS BLACK RIGHT POINTING INDEX" },
    { 0x1F59C, "BLACK LEFT POINTING BACKHAND INDEX" },
    { 0x1F59D, "BLACK RIGHT POINTING BACKHAND INDEX" },
    { 0x1F59E, "SIDEWAYS WHITE UP POINTING INDEX" },
    { 0x1F59F, "SIDEWAYS WHITE DOWN POINTING INDEX" },
    { 0x1F5A0, "SIDEWAYS BLACK UP POINTING INDEX" },
    { 0x1F5A1, "SIDEWAYS BLACK DOWN POINTING INDEX" },
    { 0x1F5A2, "BLACK UP POINTING BACKHAND INDEX" },
    { 0x1F5A3, "BLACK DOWN POINTING BACKHAND INDEX" },
    { 0x1F5A5, "DESKTOP COMPUTER" },
    { 0x1F5A6, "KEYBOARD AND MOUSE" },
    { 0x1F5A7, "THREE NETWORKED COMPUTERS" },
    { 0x1F5A8, "PRINTER" },
    { 0x1F5A9, "POCKET CALCULATOR" },
    { 0x1F5AA, "BLACK HARD SHELL FLOPPY DISK" },
    { 0x1F5AB, "WHITE HARD SHELL FLOPPY DISK" },
    { 0x1F5AC, "SOFT SHELL FLOPPY DISK" },
    { 0x1F5AD, "TAPE CARTRIDGE" },
    { 0x1F5AE, "WIRED KEYBOARD" },
    { 0x1F5AF, "ONE BUTTON MOUSE" },
    { 0x1F5B0, "TWO BUTTON MOUSE" },
    { 0x1F5B1, "THREE BUTTON MOUSE" },
    { 0x1F5B2, "TRACKBALL" },
    { 0x1F5B3, "OLD PERSONAL COMPUTER" },
    { 0x1F5B4, "HARD DISK" },
    { 0x1F5B5, "SCREEN" },
    { 0x1F5B6, "PRINTER ICON" },
    { 0x1F5B7, "FAX ICON" },
    { 0x1F5B8, "OPTICAL DISC ICON" },
    { 0x1F5B9, "DOCUMENT WITH TEXT" },
    { 0x1F5BA, "DOCUMENT WITH TEXT AND PICTURE" },
    { 0x1F5BB, "DOCUMENT WITH PICTURE" },
    { 0x1F5BC, "FRAME WITH PICTURE" },
    { 0x1F5BD, "FRAME WITH TILES" },
    { 0x1F5BE, "FRAME WITH AN X" },
    { 0x1F5BF, "BLACK FOLDER" },
    { 0x1F5C0, "FOLDER" },
    { 0x1F5C1, "OPEN FOLDER" },
    { 0x1F5C2, "CARD INDEX DIVIDERS" },
    { 0x1F5C3, "CARD FILE BOX" },
    { 0x1F5C4, "FILE CABINET" },
    { 0x1F5C5, "EMPTY NOTE" },
    { 0x1F5C6, "EMPTY NOTE PAGE" },
    { 0x1F5C7, "EMPTY NOTE PAD" },
    { 0x1F5C8, "NOTE" },
    { 0x1F5C9, "NOTE PAGE" },
    { 0x1F5CA, "NOTE PAD" },
    { 0x1F5CB, "EMPTY DOCUMENT" },
    { 0x1F5CC, "EMPTY PAGE" },
    { 0x1F5CD, "EMPTY PAGES" },
    { 0x1F5CE, "DOCUMENT" },
    { 0x1F5CF, "PAGE" },
    { 0x1F5D0, "PAGES" },
    { 0x1F5D1, "WASTEBASKET" },
    { 0x1F5D2, "SPIRAL NOTE PAD" },
    { 0x1F5D3, "SPIRAL CALENDAR PAD" },
    { 0x1F5D4, "DESKTOP WINDOW" },
    { 0x1F5D5, "MINIMIZE" },
    { 0x1F5D6, "MAXIMIZE" },
    { 0x1F5D7, "OVERLAP" },
    { 0x1F5D8, "CLOCKWISE RIGHT AND LEFT SEMICIRCLE ARROWS" },
    { 0x1F5D9, "CANCELLATION X" },
    { 0x1F5DA, "INCREASE FONT SIZE SYMBOL" },
    { 0x1F5DB, "DECREASE FONT SIZE SYMBOL" },
    { 0x1F5DC, "COMPRESSION" },
    { 0x1F5DD, "OLD KEY" },
    { 0x1F5DE, "ROLLED-UP NEWSPAPER" },
    { 0x1F5DF, "PAGE WITH CIRCLED TEXT" },
    { 0x1F5E0, "STOCK CHART" },
    { 0x1F5E1, "DAGGER KNIFE" },
    { 0x1F5E2, "LIPS" },
    { 0x1F5E3, "SPEAKING HEAD IN SILHOUETTE" },
    { 0x1F5E4, "THREE RAYS ABOVE" },
    { 0x1F5E5, "THREE RAYS BELOW" },
    { 0x1F5E6, "THREE RAYS LEFT" },
    { 0x1F5E7, "THREE RAYS RIGHT" },
    { 0x1F5E8, "LEFT SPEECH BUBBLE" },
    { 0x1F5E9, "RIGHT SPEECH BUBBLE" },
    { 0x1F5EA, "TWO SPEECH BUBBLES" },
    { 0x1F5EB, "THREE SPEECH BUBBLES" },
    { 0x1F5EC, "LEFT THOUGHT BUBBLE" },
    { 0x1F5ED, "RIGHT THOUGHT BUBBLE" },
    { 0x1F5EE, "LEFT ANGER BUBBLE" },
    { 0x1F5EF, "RIGHT ANGER BUBBLE" },
    { 0x1F5F0, "MOOD BUBBLE" },
    { 0x1F5F1, "LIGHTNING MOOD BUBBLE" },
    { 0x1F5F2, "LIGHTNING MOOD" },
    { 0x1F5F3, "BALLOT BOX WITH BALLOT" },
    { 0x1F5F4, "BALLOT SCRIPT X" },
    { 0x1F5F5, "BALLOT BOX WITH SCRIPT X" },
    { 0x1F5F6, "BALLOT BOLD SCRIPT X" },
    { 0x1F5F7, "BALLOT BOX WITH BOLD SCRIPT X" },
    { 0x1F5F8, "LIGHT CHECK MARK" },
    { 0x1F5F9, "BALLOT BOX WITH BOLD CHECK" },
    { 0x1F5FA, "WORLD MAP" },
    { 0x1F650, "NORTH WEST POINTING LEAF" },
    { 0x1F651, "SOUTH WEST POINTING LEAF" },
    { 0x1F652, "NORTH EAST POINTING LEAF" },
    { 0x1F653, "SOUTH EAST POINTING LEAF" },
    { 0x1F654, "TURNED NORTH WEST POINTING LEAF" },
    { 0x1F655, "TURNED SOUTH WEST POINTING LEAF" },
    { 0x1F656, "TURNED NORTH EAST POINTING LEAF" },
    { 0x1F657, "TURNED SOUTH EAST POINTING LEAF" },
    { 0x1F658, "NORTH WEST POINTING VINE LEAF" },
    { 0x1F659, "SOUTH WEST POINTING VINE LEAF" },
    { 0x1F65A, "NORTH EAST POINTING VINE LEAF" },
    { 0x1F65B, "SOUTH EAST POINTING VINE LEAF" },
    { 0x1F65C, "HEAVY NORTH WEST POINTING VINE LEAF" },
    { 0x1F65D, "HEAVY SOUTH WEST POINTING VINE LEAF" },
    { 0x1F65E, "HEAVY NORTH EAST POINTING VINE LEAF" },
    { 0x1F65F, "HEAVY SOUTH EAST POINTING VINE LEAF" },
    { 0x1F660, "NORTH WEST POINTING BUD" },
    { 0x1F661, "SOUTH WEST POINTING BUD" },
    { 0x1F662, "NORTH EAST POINTING BUD" },
    { 0x1F663, "SOUTH EAST POINTING BUD" },
    { 0x1F664, "HEAVY NORTH WEST POINTING BUD" },
    { 0x1F665, "HEAVY SOUTH WEST POINTING BUD" },
    { 0x1F666, "HEAVY NORTH EAST POINTING BUD" },
    { 0x1F667, "HEAVY SOUTH EAST POINTING BUD" },
    { 0x1F668, "HOLLOW QUILT SQUARE ORNAMENT" },
    { 0x1F669, "HOLLOW QUILT SQUARE ORNAMENT IN BLACK SQUARE" },
    { 0x1F66A, "SOLID QUILT SQUARE ORNAMENT" },
    { 0x1F66B, "SOLID QUILT SQUARE ORNAMENT IN BLACK SQUARE" },
    { 0x1F66C, "LEFTWARDS ROCKET" },
    { 0x1F66D, "UPWARDS ROCKET" },
    { 0x1F66E, "RIGHTWARDS ROCKET" },
    { 0x1F66F, "DOWNWARDS ROCKET" },
    { 0x1F670, "SCRIPT LIGATURE ET ORNAMENT" },
    { 0x1F671, "HEAVY SCRIPT LIGATURE ET ORNAMENT" },
    { 0x1F672, "LIGATURE OPEN ET ORNAMENT" },
    { 0x1F673, "HEAVY LIGATURE OPEN ET ORNAMENT" },
    { 0x1F674, "HEAVY AMPERSAND ORNAMENT" },
    { 0x1F675, "SWASH AMPERSAND ORNAMENT" },
    { 0x1F676, "SANS-SERIF HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT" },
    { 0x1F677, "SANS-SERIF HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT" },
    { 0x1F678, "SANS-SERIF HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT" },
    { 0x1F679, "HEAVY INTERROBANG ORNAMENT" },
    { 0x1F67A, "SANS-SERIF INTERROBANG ORNAMENT" },
    { 0x1F67B, "HEAVY SANS-SERIF INTERROBANG ORNAMENT" },
    { 0x1F67C, "VERY HEAVY SOLIDUS" },
    { 0x1F67D, "VERY HEAVY REVERSE SOLIDUS" },
    { 0x1F67E, "CHECKER BOARD" },
    { 0x1F67F, "REVERSE CHECKER BOARD" },
    { 0x1F6C6, "TRIANGLE WITH ROUNDED CORNERS" },
    { 0x1F6C7, "PROHIBITED SIGN" },
    { 0x1F6C8, "CIRCLED INFORMATION SOURCE" },
    { 0x1F6C9, "BOYS SYMBOL" },
    { 0x1F6CA, "GIRLS SYMBOL" },
    { 0x1F6CB, "COUCH AND LAMP" },
    { 0x1F6CD, "SHOPPING BAGS" },
    { 0x1F6CE, "BELLHOP BELL" },
    { 0x1F6CF, "BED" },
    { 0x1F6D3, "STUPA" },
    { 0x1F6D4, "PAGODA" },
    { 0x1F6E0, "HAMMER AND WRENCH" },
    { 0x1F6E1, "SHIELD" },
    { 0x1F6E2, "OIL DRUM" },
    { 0x1F6E3, "MOTORWAY" },
    { 0x1F6E4, "RAILWAY TRACK" },
    { 0x1F6E5, "MOTOR BOAT" },
    { 0x1F6E6, "UP-POINTING MILITARY AIRPLANE" },
    { 0x1F6E7, "UP-POINTING AIRPLANE" },
    { 0x1F6E8, "UP-POINTING SMALL AIRPLANE" },
    { 0x1F6E9, "SMALL AIRPLANE" },
    { 0x1F6EA, "NORTHEAST-POINTING AIRPLANE" },
    { 0x1F6F0, "SATELLITE" },
    { 0x1F6F1, "ONCOMING FIRE ENGINE" },
    { 0x1F6F2, "DIESEL LOCOMOTIVE" },
    { 0x1F6F3, "PASSENGER SHIP" },
    { 0x1F700, "ALCHEMICAL SYMBOL FOR QUINTESSENCE" },
    { 0x1F701, "ALCHEMICAL SYMBOL FOR AIR" },
    { 0x1F702, "ALCHEMICAL SYMBOL FOR FIRE" },
    { 0x1F703, "ALCHEMICAL SYMBOL FOR EARTH" },
    { 0x1F704, "ALCHEMICAL SYMBOL FOR WATER" },
    { 0x1F705, "ALCHEMICAL SYMBOL FOR AQUAFORTIS" },
    { 0x1F706, "ALCHEMICAL SYMBOL FOR AQUA REGIA" },
    { 0x1F707, "ALCHEMICAL SYMBOL FOR AQUA REGIA-2" },
    { 0x1F708, "ALCHEMICAL SYMBOL FOR AQUA VITAE" },
    { 0x1F709, "ALCHEMICAL SYMBOL FOR AQUA VITAE-2" },
    { 0x1F70A, "ALCHEMICAL SYMBOL FOR VINEGAR" },
    { 0x1F70B, "ALCHEMICAL SYMBOL FOR VINEGAR-2" },
    { 0x1F70C, "ALCHEMICAL SYMBOL FOR VINEGAR-3" },
    { 0x1F70D, "ALCHEMICAL SYMBOL FOR SULFUR" },
    { 0x1F70E, "ALCHEMICAL SYMBOL FOR PHILOSOPHERS SULFUR" },
    { 0x1F70F, "ALCHEMICAL SYMBOL FOR BLACK SULFUR" },
    { 0x1F710, "ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE" },
    { 0x1F711, "ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE-2" },
    { 0x1F712, "ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE-3" },
    { 0x1F713, "ALCHEMICAL SYMBOL FOR CINNABAR" },
    { 0x1F714, "ALCHEMICAL SYMBOL FOR SALT" },
    { 0x1F715, "ALCHEMICAL SYMBOL FOR NITRE" },
    { 0x1F716, "ALCHEMICAL SYMBOL FOR VITRIOL" },
    { 0x1F717, "ALCHEMICAL SYMBOL FOR VITRIOL-2" },
    { 0x1F718, "ALCHEMICAL SYMBOL FOR ROCK SALT" },
    { 0x1F719, "ALCHEMICAL SYMBOL FOR ROCK SALT-2" },
    { 0x1F71A, "ALCHEMICAL SYMBOL FOR GOLD" },
    { 0x1F71B, "ALCHEMICAL SYMBOL FOR SILVER" },
    { 0x1F71C, "ALCHEMICAL SYMBOL FOR IRON ORE" },
    { 0x1F71D, "ALCHEMICAL SYMBOL FOR IRON ORE-2" },
    { 0x1F71E, "ALCHEMICAL SYMBOL FOR CROCUS OF IRON" },
    { 0x1F71F, "ALCHEMICAL SYMBOL FOR REGULUS OF IRON" },
    { 0x1F720, "ALCHEMICAL SYMBOL FOR COPPER ORE" },
    { 0x1F721, "ALCHEMICAL SYMBOL FOR IRON-COPPER ORE" },
    { 0x1F722, "ALCHEMICAL SYMBOL FOR SUBLIMATE OF COPPER" },
    { 0x1F723, "ALCHEMICAL SYMBOL FOR CROCUS OF COPPER" },
    { 0x1F724, "ALCHEMICAL SYMBOL FOR CROCUS OF COPPER-2" },
    { 0x1F725, "ALCHEMICAL SYMBOL FOR COPPER ANTIMONIATE" },
    { 0x1F726, "ALCHEMICAL SYMBOL FOR SALT OF COPPER ANTIMONIATE" },
    { 0x1F727, "ALCHEMICAL SYMBOL FOR SUBLIMATE OF SALT OF COPPER" },
    { 0x1F728, "ALCHEMICAL SYMBOL FOR VERDIGRIS" },
    { 0x1F729, "ALCHEMICAL SYMBOL FOR TIN ORE" },
    { 0x1F72A, "ALCHEMICAL SYMBOL FOR LEAD ORE" },
    { 0x1F72B, "ALCHEMICAL SYMBOL FOR ANTIMONY ORE" },
    { 0x1F72C, "ALCHEMICAL SYMBOL FOR SUBLIMATE OF ANTIMONY" },
    { 0x1F72D, "ALCHEMICAL SYMBOL FOR SALT OF ANTIMONY" },
    { 0x1F72E, "ALCHEMICAL SYMBOL FOR SUBLIMATE OF SALT OF ANTIMONY" },
    { 0x1F72F, "ALCHEMICAL SYMBOL FOR VINEGAR OF ANTIMONY" },
    { 0x1F730, "ALCHEMICAL SYMBOL FOR REGULUS OF ANTIMONY" },
    { 0x1F731, "ALCHEMICAL SYMBOL FOR REGULUS OF ANTIMONY-2" },
    { 0x1F732, "ALCHEMICAL SYMBOL FOR REGULUS" },
    { 0x1F733, "ALCHEMICAL SYMBOL FOR REGULUS-2" },
    { 0x1F734, "ALCHEMICAL SYMBOL FOR REGULUS-3" },
    { 0x1F735, "ALCHEMICAL SYMBOL FOR REGULUS-4" },
    { 0x1F736, "ALCHEMICAL SYMBOL FOR ALKALI" },
    { 0x1F737, "ALCHEMICAL SYMBOL FOR ALKALI-2" },
    { 0x1F738, "ALCHEMICAL SYMBOL FOR MARCASITE" },
    { 0x1F739, "ALCHEMICAL SYMBOL FOR SAL-AMMONIAC" },
    { 0x1F73A, "ALCHEMICAL SYMBOL FOR ARSENIC" },
    { 0x1F73B, "ALCHEMICAL SYMBOL FOR REALGAR" },
    { 0x1F73C, "ALCHEMICAL SYMBOL FOR REALGAR-2" },
    { 0x1F73D, "ALCHEMICAL SYMBOL FOR AURIPIGMENT" },
    { 0x1F73E, "ALCHEMICAL SYMBOL FOR BISMUTH ORE" },
    { 0x1F73F, "ALCHEMICAL SYMBOL FOR TARTAR" },
    { 0x1F740, "ALCHEMICAL SYMBOL FOR TARTAR-2" },
    { 0x1F741, "ALCHEMICAL SYMBOL FOR QUICK LIME" },
    { 0x1F742, "ALCHEMICAL SYMBOL FOR BORAX" },
    { 0x1F743, "ALCHEMICAL SYMBOL FOR BORAX-2" },
    { 0x1F744, "ALCHEMICAL SYMBOL FOR BORAX-3" },
    { 0x1F745, "ALCHEMICAL SYMBOL FOR ALUM" },
    { 0x1F746, "ALCHEMICAL SYMBOL FOR OIL" },
    { 0x1F747, "ALCHEMICAL SYMBOL FOR SPIRIT" },
    { 0x1F748, "ALCHEMICAL SYMBOL FOR TINCTURE" },
    { 0x1F749, "ALCHEMICAL SYMBOL FOR GUM" },
    { 0x1F74A, "ALCHEMICAL SYMBOL FOR WAX" },
    { 0x1F74B, "ALCHEMICAL SYMBOL FOR POWDER" },
    { 0x1F74C, "ALCHEMICAL SYMBOL FOR CALX" },
    { 0x1F74D, "ALCHEMICAL SYMBOL FOR TUTTY" },
    { 0x1F74E, "ALCHEMICAL SYMBOL FOR CAPUT MORTUUM" },
    { 0x1F74F, "ALCHEMICAL SYMBOL FOR SCEPTER OF JOVE" },
    { 0x1F750, "ALCHEMICAL SYMBOL FOR CADUCEUS" },
    { 0x1F751, "ALCHEMICAL SYMBOL FOR TRIDENT" },
    { 0x1F752, "ALCHEMICAL SYMBOL FOR STARRED TRIDENT" },
    { 0x1F753, "ALCHEMICAL SYMBOL FOR LODESTONE" },
    { 0x1F754, "ALCHEMICAL SYMBOL FOR SOAP" },
    { 0x1F755, "ALCHEMICAL SYMBOL FOR URINE" },
    { 0x1F756, "ALCHEMICAL SYMBOL FOR HORSE DUNG" },
    { 0x1F757, "ALCHEMICAL SYMBOL FOR ASHES" },
    { 0x1F758, "ALCHEMICAL SYMBOL FOR POT ASHES" },
    { 0x1F759, "ALCHEMICAL SYMBOL FOR BRICK" },
    { 0x1F75A, "ALCHEMICAL SYMBOL FOR POWDERED BRICK" },
    { 0x1F75B, "ALCHEMICAL SYMBOL FOR AMALGAM" },
    { 0x1F75C, "ALCHEMICAL SYMBOL FOR STRATUM SUPER STRATUM" },
    { 0x1F75D, "ALCHEMICAL SYMBOL FOR STRATUM SUPER STRATUM-2" },
    { 0x1F75E, "ALCHEMICAL SYMBOL FOR SUBLIMATION" },
    { 0x1F75F, "ALCHEMICAL SYMBOL FOR PRECIPITATE" },
    { 0x1F760, "ALCHEMICAL SYMBOL FOR DISTILL" },
    { 0x1F761, "ALCHEMICAL SYMBOL FOR DISSOLVE" },
    { 0x1F762, "ALCHEMICAL SYMBOL FOR DISSOLVE-2" },
    { 0x1F763, "ALCHEMICAL SYMBOL FOR PURIFY" },
    { 0x1F764, "ALCHEMICAL SYMBOL FOR PUTREFACTION" },
    { 0x1F765, "ALCHEMICAL SYMBOL FOR CRUCIBLE" },
    { 0x1F766, "ALCHEMICAL SYMBOL FOR CRUCIBLE-2" },
    { 0x1F767, "ALCHEMICAL SYMBOL FOR CRUCIBLE-3" },
    { 0x1F768, "ALCHEMICAL SYMBOL FOR CRUCIBLE-4" },
    { 0x1F769, "ALCHEMICAL SYMBOL FOR CRUCIBLE-5" },
    { 0x1F76A, "ALCHEMICAL SYMBOL FOR ALEMBIC" },
    { 0x1F76B, "ALCHEMICAL SYMBOL FOR BATH OF MARY" },
    { 0x1F76C, "ALCHEMICAL SYMBOL FOR BATH OF VAPOURS" },
    { 0x1F76D, "ALCHEMICAL SYMBOL FOR RETORT" },
    { 0x1F76E, "ALCHEMICAL SYMBOL FOR HOUR" },
    { 0x1F76F, "ALCHEMICAL SYMBOL FOR NIGHT" },
    { 0x1F770, "ALCHEMICAL SYMBOL FOR DAY-NIGHT" },
    { 0x1F771, "ALCHEMICAL SYMBOL FOR MONTH" },
    { 0x1F772, "ALCHEMICAL SYMBOL FOR HALF DRAM" },
    { 0x1F773, "ALCHEMICAL SYMBOL FOR HALF OUNCE" },
    { 0x1F774, "LOT OF FORTUNE" },
    { 0x1F775, "OCCULTATION" },
    { 0x1F776, "LUNAR ECLIPSE" },
    { 0x1F77B, "HAUMEA" },
    { 0x1F77C, "MAKEMAKE" },
    { 0x1F77D, "GONGGONG" },
    { 0x1F77E, "QUAOAR" },
    { 0x1F77F, "ORCUS" },
    { 0x1F780, "BLACK LEFT-POINTING ISOSCELES RIGHT TRIANGLE" },
    { 0x1F781, "BLACK UP-POINTING ISOSCELES RIGHT TRIANGLE" },
    { 0x1F782, "BLACK RIGHT-POINTING ISOSCELES RIGHT TRIANGLE" },
    { 0x1F783, "BLACK DOWN-POINTING ISOSCELES RIGHT TRIANGLE" },
    { 0x1F784, "BLACK SLIGHTLY SMALL CIRCLE" },
    { 0x1F785, "MEDIUM BOLD WHITE CIRCLE" },
    { 0x1F786, "BOLD WHITE CIRCLE" },
    { 0x1F787, "HEAVY WHITE CIRCLE" },
    { 0x1F788, "VERY HEAVY WHITE CIRCLE" },
    { 0x1F789, "EXTREMELY HEAVY WHITE CIRCLE" },
    { 0x1F78A, "WHITE CIRCLE CONTAINING BLACK SMALL CIRCLE" },
    { 0x1F78B, "ROUND TARGET" },
    { 0x1F78C, "BLACK TINY SQUARE" },
    { 0x1F78D, "BLACK SLIGHTLY SMALL SQUARE" },
    { 0x1F78E, "LIGHT WHITE SQUARE" },
    { 0x1F78F, "MEDIUM WHITE SQUARE" },
    { 0x1F790, "BOLD WHITE SQUARE" },
    { 0x1F791, "HEAVY WHITE SQUARE" },
    { 0x1F792, "VERY HEAVY WHITE SQUARE" },
    { 0x1F793, "EXTREMELY HEAVY WHITE SQUARE" },
    { 0x1F794, "WHITE SQUARE CONTAINING BLACK VERY SMALL SQUARE" },
    { 0x1F795, "WHITE SQUARE CONTAINING BLACK MEDIUM SQUARE" },
    { 0x1F796, "SQUARE TARGET" },
    { 0x1F797, "BLACK TINY DIAMOND" },
    { 0x1F798, "BLACK VERY SMALL DIAMOND" },
    { 0x1F799, "BLACK MEDIUM SMALL DIAMOND" },
    { 0x1F79A, "WHITE DIAMOND CONTAINING BLACK VERY SMALL DIAMOND" },
    { 0x1F79B, "WHITE DIAMOND CONTAINING BLACK MEDIUM DIAMOND" },
    { 0x1F79C, "DIAMOND TARGET" },
    { 0x1F79D, "BLACK TINY LOZENGE" },
    { 0x1F79E, "BLACK VERY SMALL LOZENGE" },
    { 0x1F79F, "BLACK MEDIUM SMALL LOZENGE" },
    { 0x1F7A0, "WHITE LOZENGE CONTAINING BLACK SMALL LOZENGE" },
    { 0x1F7A1, "THIN GREEK CROSS" },
    { 0x1F7A2, "LIGHT GREEK CROSS" },
    { 0x1F7A3, "MEDIUM GREEK CROSS" },
    { 0x1F7A4, "BOLD GREEK CROSS" },
    { 0x1F7A5, "VERY BOLD GREEK CROSS" },
    { 0x1F7A6, "VERY HEAVY GREEK CROSS" },
    { 0x1F7A7, "EXTREMELY HEAVY GREEK CROSS" },
    { 0x1F7A8, "THIN SALTIRE" },
    { 0x1F7A9, "LIGHT SALTIRE" },
    { 0x1F7AA, "MEDIUM SALTIRE" },
    { 0x1F7AB, "BOLD SALTIRE" },
    { 0x1F7AC, "HEAVY SALTIRE" },
    { 0x1F7AD, "VERY HEAVY SALTIRE" },
    { 0x1F7AE, "EXTREMELY HEAVY SALTIRE" },
    { 0x1F7AF, "LIGHT FIVE SPOKED ASTERISK" },
    { 0x1F7B0, "MEDIUM FIVE SPOKED ASTERISK" },
    { 0x1F7B1, "BOLD FIVE SPOKED ASTERISK" },
    { 0x1F7B2, "HEAVY FIVE SPOKED ASTERISK" },
    { 0x1F7B3, "VERY HEAVY FIVE SPOKED ASTERISK" },
    { 0x1F7B4, "EXTREMELY HEAVY FIVE SPOKED ASTERISK" },
    { 0x1F7B5, "LIGHT SIX SPOKED ASTERISK" },
    { 0x1F7B6, "MEDIUM SIX SPOKED ASTERISK" },
    { 0x1F7B7, "BOLD SIX SPOKED ASTERISK" },
    { 0x1F7B8, "HEAVY SIX SPOKED ASTERISK" },
    { 0x1F7B9, "VERY HEAVY SIX SPOKED ASTERISK" },
    { 0x1F7BA, "EXTREMELY HEAVY SIX SPOKED ASTERISK" },
    { 0x1F7BB, "LIGHT EIGHT SPOKED ASTERISK" },
    { 0x1F7BC, "MEDIUM EIGHT SPOKED ASTERISK" },
    { 0x1F7BD, "BOLD EIGHT SPOKED ASTERISK" },
    { 0x1F7BE, "HEAVY EIGHT SPOKED ASTERISK" },
    { 0x1F7BF, "VERY HEAVY EIGHT SPOKED ASTERISK" },
    { 0x1F7C0, "LIGHT THREE POINTED BLACK STAR" },
    { 0x1F7C1, "MEDIUM THREE POINTED BLACK STAR" },
    { 0x1F7C2, "THREE POINTED BLACK STAR" },
    { 0x1F7C3, "MEDIUM THREE POINTED PINWHEEL STAR" },
    { 0x1F7C4, "LIGHT FOUR POINTED BLACK STAR" },
    { 0x1F7C5, "MEDIUM FOUR POINTED BLACK STAR" },
    { 0x1F7C6, "FOUR POINTED BLACK STAR" },
    { 0x1F7C7, "MEDIUM FOUR POINTED PINWHEEL STAR" },
    { 0x1F7C8, "REVERSE LIGHT FOUR POINTED PINWHEEL STAR" },
    { 0x1F7C9, "LIGHT FIVE POINTED BLACK STAR" },
    { 0x1F7CA, "HEAVY FIVE POINTED BLACK STAR" },
    { 0x1F7CB, "MEDIUM SIX POINTED BLACK STAR" },
    { 0x1F7CC, "HEAVY SIX POINTED BLACK STAR" },
    { 0x1F7CD, "SIX POINTED PINWHEEL STAR" },
    { 0x1F7CE, "MEDIUM EIGHT POINTED BLACK STAR" },
    { 0x1F7CF, "HEAVY EIGHT POINTED BLACK STAR" },
    { 0x1F7D0, "VERY HEAVY EIGHT POINTED BLACK STAR" },
    { 0x1F7D1, "HEAVY EIGHT POINTED PINWHEEL STAR" },
    { 0x1F7D2, "LIGHT TWELVE POINTED BLACK STAR" },
    { 0x1F7D3, "HEAVY TWELVE POINTED BLACK STAR" },
    { 0x1F7D4, "HEAVY TWELVE POINTED PINWHEEL STAR" },
    { 0x1F7D5, "CIRCLED TRIANGLE" },
    { 0x1F7D6, "NEGATIVE CIRCLED TRIANGLE" },
    { 0x1F7D7, "CIRCLED SQUARE" },
    { 0x1F7D8, "NEGATIVE CIRCLED SQUARE" },
    { 0x1F7D9, "NINE POINTED WHITE STAR" },
    { 0x1F800, "LEFTWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD" },
    { 0x1F801, "UPWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD" },
    { 0x1F802, "RIGHTWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD" },
    { 0x1F803, "DOWNWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD" },
    { 0x1F804, "LEFTWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD" },
    { 0x1F805, "UPWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD" },
    { 0x1F806, "RIGHTWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD" },
    { 0x1F807, "DOWNWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD" },
    { 0x1F808, "LEFTWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD" },
    { 0x1F809, "UPWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD" },
    { 0x1F80A, "RIGHTWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD" },
    { 0x1F80B, "DOWNWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD" },
    { 0x1F810, "LEFTWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD" },
    { 0x1F811, "UPWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD" },
    { 0x1F812, "RIGHTWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD" },
    { 0x1F813, "DOWNWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD" },
    { 0x1F814, "LEFTWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F815, "UPWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F816, "RIGHTWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F817, "DOWNWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F818, "HEAVY LEFTWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F819, "HEAVY UPWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F81A, "HEAVY RIGHTWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F81B, "HEAVY DOWNWARDS ARROW WITH EQUILATERAL ARROWHEAD" },
    { 0x1F81C, "HEAVY LEFTWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD" },
    { 0x1F81D, "HEAVY UPWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD" },
    { 0x1F81E, "HEAVY RIGHTWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD" },
    { 0x1F81F, "HEAVY DOWNWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD" },
    { 0x1F820, "LEFTWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT" },
    { 0x1F821, "UPWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT" },
    { 0x1F822, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT" },
    { 0x1F823, "DOWNWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT" },
    { 0x1F824, "LEFTWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT" },
    { 0x1F825, "UPWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT" },
    { 0x1F826, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT" },
    { 0x1F827, "DOWNWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT" },
    { 0x1F828, "LEFTWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT" },
    { 0x1F829, "UPWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT" },
    { 0x1F82A, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT" },
    { 0x1F82B, "DOWNWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT" },
    { 0x1F82C, "LEFTWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT" },
    { 0x1F82D, "UPWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT" },
    { 0x1F82E, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT" },
    { 0x1F82F, "DOWNWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT" },
    { 0x1F830, "LEFTWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT" },
    { 0x1F831, "UPWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT" },
    { 0x1F832, "RIGHTWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT" },
    { 0x1F833, "DOWNWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT" },
    { 0x1F834, "LEFTWARDS FINGER-POST ARROW" },
    { 0x1F835, "UPWARDS FINGER-POST ARROW" },
    { 0x1F836, "RIGHTWARDS FINGER-POST ARROW" },
    { 0x1F837, "DOWNWARDS FINGER-POST ARROW" },
    { 0x1F838, "LEFTWARDS SQUARED ARROW" },
    { 0x1F839, "UPWARDS SQUARED ARROW" },
    { 0x1F83A, "RIGHTWARDS SQUARED ARROW" },
    { 0x1F83B, "DOWNWARDS SQUARED ARROW" },
    { 0x1F83C, "LEFTWARDS COMPRESSED ARROW" },
    { 0x1F83D, "UPWARDS COMPRESSED ARROW" },
    { 0x1F83E, "RIGHTWARDS COMPRESSED ARROW" },
    { 0x1F83F, "DOWNWARDS COMPRESSED ARROW" },
    { 0x1F840, "LEFTWARDS HEAVY COMPRESSED ARROW" },
    { 0x1F841, "UPWARDS HEAVY COMPRESSED ARROW" },
    { 0x1F842, "RIGHTWARDS HEAVY COMPRESSED ARROW" },
    { 0x1F843, "DOWNWARDS HEAVY COMPRESSED ARROW" },
    { 0x1F844, "LEFTWARDS HEAVY ARROW" },
    { 0x1F845, "UPWARDS HEAVY ARROW" },
    { 0x1F846, "RIGHTWARDS HEAVY ARROW" },
    { 0x1F847, "DOWNWARDS HEAVY ARROW" },
    { 0x1F850, "LEFTWARDS SANS-SERIF ARROW" },
    { 0x1F851, "UPWARDS SANS-SERIF ARROW" },
    { 0x1F852, "RIGHTWARDS SANS-SERIF ARROW" },
    { 0x1F853, "DOWNWARDS SANS-SERIF ARROW" },
    { 0x1F854, "NORTH WEST SANS-SERIF ARROW" },
    { 0x1F855, "NORTH EAST SANS-SERIF ARROW" },
    { 0x1F856, "SOUTH EAST SANS-SERIF ARROW" },
    { 0x1F857, "SOUTH WEST SANS-SERIF ARROW" },
    { 0x1F858, "LEFT RIGHT SANS-SERIF ARROW" },
    { 0x1F859, "UP DOWN SANS-SERIF ARROW" },
    { 0x1F860, "WIDE-HEADED LEFTWARDS LIGHT BARB ARROW" },
    { 0x1F861, "WIDE-HEADED UPWARDS LIGHT BARB ARROW" },
    { 0x1F862, "WIDE-HEADED RIGHTWARDS LIGHT BARB ARROW" },
    { 0x1F863, "WIDE-HEADED DOWNWARDS LIGHT BARB ARROW" },
    { 0x1F864, "WIDE-HEADED NORTH WEST LIGHT BARB ARROW" },
    { 0x1F865, "WIDE-HEADED NORTH EAST LIGHT BARB ARROW" },
    { 0x1F866, "WIDE-HEADED SOUTH EAST LIGHT BARB ARROW" },
    { 0x1F867, "WIDE-HEADED SOUTH WEST LIGHT BARB ARROW" },
    { 0x1F868, "WIDE-HEADED LEFTWARDS BARB ARROW" },
    { 0x1F869, "WIDE-HEADED UPWARDS BARB ARROW" },
    { 0x1F86A, "WIDE-HEADED RIGHTWARDS BARB ARROW" },
    { 0x1F86B, "WIDE-HEADED DOWNWARDS BARB ARROW" },
    { 0x1F86C, "WIDE-HEADED NORTH WEST BARB ARROW" },
    { 0x1F86D, "WIDE-HEADED NORTH EAST BARB ARROW" },
    { 0x1F86E, "WIDE-HEADED SOUTH EAST BARB ARROW" },
    { 0x1F86F, "WIDE-HEADED SOUTH WEST BARB ARROW" },
    { 0x1F870, "WIDE-HEADED LEFTWARDS MEDIUM BARB ARROW" },
    { 0x1F871, "WIDE-HEADED UPWARDS MEDIUM BARB ARROW" },
    { 0x1F872, "WIDE-HEADED RIGHTWARDS MEDIUM BARB ARROW" },
    { 0x1F873, "WIDE-HEADED DOWNWARDS MEDIUM BARB ARROW" },
    { 0x1F874, "WIDE-HEADED NORTH WEST MEDIUM BARB ARROW" },
    { 0x1F875, "WIDE-HEADED NORTH EAST MEDIUM BARB ARROW" },
    { 0x1F876, "WIDE-HEADED SOUTH EAST MEDIUM BARB ARROW" },
    { 0x1F877, "WIDE-HEADED SOUTH WEST MEDIUM BARB ARROW" },
    { 0x1F878, "WIDE-HEADED LEFTWARDS HEAVY BARB ARROW" },
    { 0x1F879, "WIDE-HEADED UPWARDS HEAVY BARB ARROW" },
    { 0x1F87A, "WIDE-HEADED RIGHTWARDS HEAVY BARB ARROW" },
    { 0x1F87B, "WIDE-HEADED DOWNWARDS HEAVY BARB ARROW" },
    { 0x1F87C, "WIDE-HEADED NORTH WEST HEAVY BARB ARROW" },
    { 0x1F87D, "WIDE-HEADED NORTH EAST HEAVY BARB ARROW" },
    { 0x1F87E, "WIDE-HEADED SOUTH EAST HEAVY BARB ARROW" },
    { 0x1F87F, "WIDE-HEADED SOUTH WEST HEAVY BARB ARROW" },
    { 0x1F880, "WIDE-HEADED LEFTWARDS VERY HEAVY BARB ARROW" },
    { 0x1F881, "WIDE-HEADED UPWARDS VERY HEAVY BARB ARROW" },
    { 0x1F882, "WIDE-HEADED RIGHTWARDS VERY HEAVY BARB ARROW" },
    { 0x1F883, "WIDE-HEADED DOWNWARDS VERY HEAVY BARB ARROW" },
    { 0x1F884, "WIDE-HEADED NORTH WEST VERY HEAVY BARB ARROW" },
    { 0x1F885, "WIDE-HEADED NORTH EAST VERY HEAVY BARB ARROW" },
    { 0x1F886, "WIDE-HEADED SOUTH EAST VERY HEAVY BARB ARROW" },
    { 0x1F887, "WIDE-HEADED SOUTH WEST VERY HEAVY BARB ARROW" },
    { 0x1F890, "LEFTWARDS TRIANGLE ARROWHEAD" },
    { 0x1F891, "UPWARDS TRIANGLE ARROWHEAD" },
    { 0x1F892, "RIGHTWARDS TRIANGLE ARROWHEAD" },
    { 0x1F893, "DOWNWARDS TRIANGLE ARROWHEAD" },
    { 0x1F894, "LEFTWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD" },
    { 0x1F895, "UPWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD" },
    { 0x1F896, "RIGHTWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD" },
    { 0x1F897, "DOWNWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD" },
    { 0x1F898, "LEFTWARDS ARROW WITH NOTCHED TAIL" },
    { 0x1F899, "UPWARDS ARROW WITH NOTCHED TAIL" },
    { 0x1F89A, "RIGHTWARDS ARROW WITH NOTCHED TAIL" },
    { 0x1F89B, "DOWNWARDS ARROW WITH NOTCHED TAIL" },
    { 0x1F89C, "HEAVY ARROW SHAFT WIDTH ONE" },
    { 0x1F89D, "HEAVY ARROW SHAFT WIDTH TWO THIRDS" },
    { 0x1F89E, "HEAVY ARROW SHAFT WIDTH ONE HALF" },
    { 0x1F89F, "HEAVY ARROW SHAFT WIDTH ONE THIRD" },
    { 0x1F8A0, "LEFTWARDS BOTTOM-SHADED WHITE ARROW" },
    { 0x1F8A1, "RIGHTWARDS BOTTOM SHADED WHITE ARROW" },
    { 0x1F8A2, "LEFTWARDS TOP SHADED WHITE ARROW" },
    { 0x1F8A3, "RIGHTWARDS TOP SHADED WHITE ARROW" },
    { 0x1F8A4, "LEFTWARDS LEFT-SHADED WHITE ARROW" },
    { 0x1F8A5, "RIGHTWARDS RIGHT-SHADED WHITE ARROW" },
    { 0x1F8A6, "LEFTWARDS RIGHT-SHADED WHITE ARROW" },
    { 0x1F8A7, "RIGHTWARDS LEFT-SHADED WHITE ARROW" },
    { 0x1F8A8, "LEFTWARDS BACK-TILTED SHADOWED WHITE ARROW" },
    { 0x1F8A9, "RIGHTWARDS BACK-TILTED SHADOWED WHITE ARROW" },
    { 0x1F8AA, "LEFTWARDS FRONT-TILTED SHADOWED WHITE ARROW" },
    { 0x1F8AB, "RIGHTWARDS FRONT-TILTED SHADOWED WHITE ARROW" },
    { 0x1F8AC, "WHITE ARROW SHAFT WIDTH ONE" },
    { 0x1F8AD, "WHITE ARROW SHAFT WIDTH TWO THIRDS" },
    { 0x1F8B0, "ARROW POINTING UPWARDS THEN NORTH WEST" },
    { 0x1F8B1, "ARROW POINTING RIGHTWARDS THEN CURVING SOUTH WEST" },
    { 0x1F900, "CIRCLED CROSS FORMEE WITH FOUR DOTS" },
    { 0x1F901, "CIRCLED CROSS FORMEE WITH TWO DOTS" },
    { 0x1F902, "CIRCLED CROSS FORMEE" },
    { 0x1F903, "LEFT HALF CIRCLE WITH FOUR DOTS" },
    { 0x1F904, "LEFT HALF CIRCLE WITH THREE DOTS" },
    { 0x1F905, "LEFT HALF CIRCLE WITH TWO DOTS" },
    { 0x1F906, "LEFT HALF CIRCLE WITH DOT" },
    { 0x1F907, "LEFT HALF CIRCLE" },
    { 0x1F908, "DOWNWARD FACING HOOK" },
    { 0x1F909, "DOWNWARD FACING NOTCHED HOOK" },
    { 0x1F90A, "DOWNWARD FACING HOOK WITH DOT" },
    { 0x1F90B, "DOWNWARD FACING NOTCHED HOOK WITH DOT" },
    { 0x1F93B, "MODERN PENTATHLON" },
    { 0x1F946, "RIFLE" },
    { 0x1F9B0, "EMOJI COMPONENT RED HAIR" },
    { 0x1F9B1, "EMOJI COMPONENT CURLY HAIR" },
    { 0x1F9B2, "EMOJI COMPONENT BALD" },
    { 0x1F9B3, "EMOJI COMPONENT WHITE HAIR" },
    { 0x1FA00, "NEUTRAL CHESS KING" },
    { 0x1FA01, "NEUTRAL CHESS QUEEN" },
    { 0x1FA02, "NEUTRAL CHESS ROOK" },
    { 0x1FA03, "NEUTRAL CHESS BISHOP" },
    { 0x1FA04, "NEUTRAL CHESS KNIGHT" },
    { 0x1FA05, "NEUTRAL CHESS PAWN" },
    { 0x1FA06, "WHITE CHESS KNIGHT ROTATED FORTY-FIVE DEGREES" },
    { 0x1FA07, "BLACK CHESS KNIGHT ROTATED FORTY-FIVE DEGREES" },
    { 0x1FA08, "NEUTRAL CHESS KNIGHT ROTATED FORTY-FIVE DEGREES" },
    { 0x1FA09, "WHITE CHESS KING ROTATED NINETY DEGREES" },
    { 0x1FA0A, "WHITE CHESS QUEEN ROTATED NINETY DEGREES" },
    { 0x1FA0B, "WHITE CHESS ROOK ROTATED NINETY DEGREES" },
    { 0x1FA0C, "WHITE CHESS BISHOP ROTATED NINETY DEGREES" },
    { 0x1FA0D, "WHITE CHESS KNIGHT ROTATED NINETY DEGREES" },
    { 0x1FA0E, "WHITE CHESS PAWN ROTATED NINETY DEGREES" },
    { 0x1FA0F, "BLACK CHESS KING ROTATED NINETY DEGREES" },
    { 0x1FA10, "BLACK CHESS QUEEN ROTATED NINETY DEGREES" },
    { 0x1FA11, "BLACK CHESS ROOK ROTATED NINETY DEGREES" },
    { 0x1FA12, "BLACK CHESS BISHOP ROTATED NINETY DEGREES" },
    { 0x1FA13, "BLACK CHESS KNIGHT ROTATED NINETY DEGREES" },
    { 0x1FA14, "BLACK CHESS PAWN ROTATED NINETY DEGREES" },
    { 0x1FA15, "NEUTRAL CHESS KING ROTATED NINETY DEGREES" },
    { 0x1FA16, "NEUTRAL CHESS QUEEN ROTATED NINETY DEGREES" },
    { 0x1FA17, "NEUTRAL CHESS ROOK ROTATED NINETY DEGREES" },
    { 0x1FA18, "NEUTRAL CHESS BISHOP ROTATED NINETY DEGREES" },
    { 0x1FA19, "NEUTRAL CHESS KNIGHT ROTATED NINETY DEGREES" },
    { 0x1FA1A, "NEUTRAL CHESS PAWN ROTATED NINETY DEGREES" },
    { 0x1FA1B, "WHITE CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES" },
    { 0x1FA1C, "BLACK CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES" },
    { 0x1FA1D, "NEUTRAL CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES" },
    { 0x1FA1E, "WHITE CHESS TURNED KING" },
    { 0x1FA1F, "WHITE CHESS TURNED QUEEN" },
    { 0x1FA20, "WHITE CHESS TURNED ROOK" },
    { 0x1FA21, "WHITE CHESS TURNED BISHOP" },
    { 0x1FA22, "WHITE CHESS TURNED KNIGHT" },
    { 0x1FA23, "WHITE CHESS TURNED PAWN" },
    { 0x1FA24, "BLACK CHESS TURNED KING" },
    { 0x1FA25, "BLACK CHESS TURNED QUEEN" },
    { 0x1FA26, "BLACK CHESS TURNED ROOK" },
    { 0x1FA27, "BLACK CHESS TURNED BISHOP" },
    { 0x1FA28, "BLACK CHESS TURNED KNIGHT" },
    { 0x1FA29, "BLACK CHESS TURNED PAWN" },
    { 0x1FA2A, "NEUTRAL CHESS TURNED KING" },
    { 0x1FA2B, "NEUTRAL CHESS TURNED QUEEN" },
    { 0x1FA2C, "NEUTRAL CHESS TURNED ROOK" },
    { 0x1FA2D, "NEUTRAL CHESS TURNED BISHOP" },
    { 0x1FA2E, "NEUTRAL CHESS TURNED KNIGHT" },
    { 0x1FA2F, "NEUTRAL CHESS TURNED PAWN" },
    { 0x1FA30, "WHITE CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES" },
    { 0x1FA31, "BLACK CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES" },
    { 0x1FA32, "NEUTRAL CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES" },
    { 0x1FA33, "WHITE CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA34, "WHITE CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA35, "WHITE CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA36, "WHITE CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA37, "WHITE CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA38, "WHITE CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA39, "BLACK CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA3A, "BLACK CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA3B, "BLACK CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA3C, "BLACK CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA3D, "BLACK CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA3E, "BLACK CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA3F, "NEUTRAL CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA40, "NEUTRAL CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA41, "NEUTRAL CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA42, "NEUTRAL CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA43, "NEUTRAL CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA44, "NEUTRAL CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES" },
    { 0x1FA45, "WHITE CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES" },
    { 0x1FA46, "BLACK CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES" },
    { 0x1FA47, "NEUTRAL CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES" },
    { 0x1FA48, "WHITE CHESS EQUIHOPPER" },
    { 0x1FA49, "BLACK CHESS EQUIHOPPER" },
    { 0x1FA4A, "NEUTRAL CHESS EQUIHOPPER" },
    { 0x1FA4B, "WHITE CHESS EQUIHOPPER ROTATED NINETY DEGREES" },
    { 0x1FA4C, "BLACK CHESS EQUIHOPPER ROTATED NINETY DEGREES" },
    { 0x1FA4D, "NEUTRAL CHESS EQUIHOPPER ROTATED NINETY DEGREES" },
    { 0x1FA4E, "WHITE CHESS KNIGHT-QUEEN" },
    { 0x1FA4F, "WHITE CHESS KNIGHT-ROOK" },
    { 0x1FA50, "WHITE CHESS KNIGHT-BISHOP" },
    { 0x1FA51, "BLACK CHESS KNIGHT-QUEEN" },
    { 0x1FA52, "BLACK CHESS KNIGHT-ROOK" },
    { 0x1FA53, "BLACK CHESS KNIGHT-BISHOP" },
    { 0x1FA60, "XIANGQI RED GENERAL" },
    { 0x1FA61, "XIANGQI RED MANDARIN" },
    { 0x1FA62, "XIANGQI RED ELEPHANT" },
    { 0x1FA63, "XIANGQI RED HORSE" },
    { 0x1FA64, "XIANGQI RED CHARIOT" },
    { 0x1FA65, "XIANGQI RED CANNON" },
    { 0x1FA66, "XIANGQI RED SOLDIER" },
    { 0x1FA67, "XIANGQI BLACK GENERAL" },
    { 0x1FA68, "XIANGQI BLACK MANDARIN" },
    { 0x1FA69, "XIANGQI BLACK ELEPHANT" },
    { 0x1FA6A, "XIANGQI BLACK HORSE" },
    { 0x1FA6B, "XIANGQI BLACK CHARIOT" },
    { 0x1FA6C, "XIANGQI BLACK CANNON" },
    { 0x1FA6D, "XIANGQI BLACK SOLDIER" },
    { 0x1FB00, "BLOCK SEXTANT-1" },
    { 0x1FB01, "BLOCK SEXTANT-2" },
    { 0x1FB02, "BLOCK SEXTANT-12" },
    { 0x1FB03, "BLOCK SEXTANT-3" },
    { 0x1FB04, "BLOCK SEXTANT-13" },
    { 0x1FB05, "BLOCK SEXTANT-23" },
    { 0x1FB06, "BLOCK SEXTANT-123" },
    { 0x1FB07, "BLOCK SEXTANT-4" },
    { 0x1FB08, "BLOCK SEXTANT-14" },
    { 0x1FB09, "BLOCK SEXTANT-24" },
    { 0x1FB0A, "BLOCK SEXTANT-124" },
    { 0x1FB0B, "BLOCK SEXTANT-34" },
    { 0x1FB0C, "BLOCK SEXTANT-134" },
    { 0x1FB0D, "BLOCK SEXTANT-234" },
    { 0x1FB0E, "BLOCK SEXTANT-1234" },
    { 0x1FB0F, "BLOCK SEXTANT-5" },
    { 0x1FB10, "BLOCK SEXTANT-15" },
    { 0x1FB11, "BLOCK SEXTANT-25" },
    { 0x1FB12, "BLOCK SEXTANT-125" },
    { 0x1FB13, "BLOCK SEXTANT-35" },
    { 0x1FB14, "BLOCK SEXTANT-235" },
    { 0x1FB15, "BLOCK SEXTANT-1235" },
    { 0x1FB16, "BLOCK SEXTANT-45" },
    { 0x1FB17, "BLOCK SEXTANT-145" },
    { 0x1FB18, "BLOCK SEXTANT-245" },
    { 0x1FB19, "BLOCK SEXTANT-1245" },
    { 0x1FB1A, "BLOCK SEXTANT-345" },
    { 0x1FB1B, "BLOCK SEXTANT-1345" },
    { 0x1FB1C, "BLOCK SEXTANT-2345" },
    { 0x1FB1D, "BLOCK SEXTANT-12345" },
    { 0x1FB1E, "BLOCK SEXTANT-6" },
    { 0x1FB1F, "BLOCK SEXTANT-16" },
    { 0x1FB20, "BLOCK SEXTANT-26" },
    { 0x1FB21, "BLOCK SEXTANT-126" },
    { 0x1FB22, "BLOCK SEXTANT-36" },
    { 0x1FB23, "BLOCK SEXTANT-136" },
    { 0x1FB24, "BLOCK SEXTANT-236" },
    { 0x1FB25, "BLOCK SEXTANT-1236" },
    { 0x1FB26, "BLOCK SEXTANT-46" },
    { 0x1FB27, "BLOCK SEXTANT-146" },
    { 0x1FB28, "BLOCK SEXTANT-1246" },
    { 0x1FB29, "BLOCK SEXTANT-346" },
    { 0x1FB2A, "BLOCK SEXTANT-1346" },
    { 0x1FB2B, "BLOCK SEXTANT-2346" },
    { 0x1FB2C, "BLOCK SEXTANT-12346" },
    { 0x1FB2D, "BLOCK SEXTANT-56" },
    { 0x1FB2E, "BLOCK SEXTANT-156" },
    { 0x1FB2F, "BLOCK SEXTANT-256" },
    { 0x1FB30, "BLOCK SEXTANT-1256" },
    { 0x1FB31, "BLOCK SEXTANT-356" },
    { 0x1FB32, "BLOCK SEXTANT-1356" },
    { 0x1FB33, "BLOCK SEXTANT-2356" },
    { 0x1FB34, "BLOCK SEXTANT-12356" },
    { 0x1FB35, "BLOCK SEXTANT-456" },
    { 0x1FB36, "BLOCK SEXTANT-1456" },
    { 0x1FB37, "BLOCK SEXTANT-2456" },
    { 0x1FB38, "BLOCK SEXTANT-12456" },
    { 0x1FB39, "BLOCK SEXTANT-3456" },
    { 0x1FB3A, "BLOCK SEXTANT-13456" },
    { 0x1FB3B, "BLOCK SEXTANT-23456" },
    { 0x1FB3C, "LOWER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FB3D, "LOWER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER RIGHT" },
    { 0x1FB3E, "LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FB3F, "LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER RIGHT" },
    { 0x1FB40, "LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO LOWER CENTRE" },
    { 0x1FB41, "LOWER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER CENTRE" },
    { 0x1FB42, "LOWER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER RIGHT" },
    { 0x1FB43, "LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER CENTRE" },
    { 0x1FB44, "LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER RIGHT" },
    { 0x1FB45, "LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO UPPER CENTRE" },
    { 0x1FB46, "LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER MIDDLE RIGHT" },
    { 0x1FB47, "LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO LOWER MIDDLE RIGHT" },
    { 0x1FB48, "LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO LOWER MIDDLE RIGHT" },
    { 0x1FB49, "LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO UPPER MIDDLE RIGHT" },
    { 0x1FB4A, "LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO UPPER MIDDLE RIGHT" },
    { 0x1FB4B, "LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO UPPER RIGHT" },
    { 0x1FB4C, "LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO UPPER MIDDLE RIGHT" },
    { 0x1FB4D, "LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO UPPER MIDDLE RIGHT" },
    { 0x1FB4E, "LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO LOWER MIDDLE RIGHT" },
    { 0x1FB4F, "LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO LOWER MIDDLE RIGHT" },
    { 0x1FB50, "LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO LOWER RIGHT" },
    { 0x1FB51, "LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER MIDDLE RIGHT" },
    { 0x1FB52, "UPPER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FB53, "UPPER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER RIGHT" },
    { 0x1FB54, "UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FB55, "UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER RIGHT" },
    { 0x1FB56, "UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO LOWER CENTRE" },
    { 0x1FB57, "UPPER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER CENTRE" },
    { 0x1FB58, "UPPER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER RIGHT" },
    { 0x1FB59, "UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER CENTRE" },
    { 0x1FB5A, "UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER RIGHT" },
    { 0x1FB5B, "UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO UPPER CENTRE" },
    { 0x1FB5C, "UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER MIDDLE RIGHT" },
    { 0x1FB5D, "UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO LOWER MIDDLE RIGHT" },
    { 0x1FB5E, "UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO LOWER MIDDLE RIGHT" },
    { 0x1FB5F, "UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO UPPER MIDDLE RIGHT" },
    { 0x1FB60, "UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO UPPER MIDDLE RIGHT" },
    { 0x1FB61, "UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO UPPER RIGHT" },
    { 0x1FB62, "UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO UPPER MIDDLE RIGHT" },
    { 0x1FB63, "UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO UPPER MIDDLE RIGHT" },
    { 0x1FB64, "UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO LOWER MIDDLE RIGHT" },
    { 0x1FB65, "UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO LOWER MIDDLE RIGHT" },
    { 0x1FB66, "UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO LOWER RIGHT" },
    { 0x1FB67, "UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER MIDDLE RIGHT" },
    { 0x1FB68, "UPPER AND RIGHT AND LOWER TRIANGULAR THREE QUARTERS BLOCK" },
    { 0x1FB69, "LEFT AND LOWER AND RIGHT TRIANGULAR THREE QUARTERS BLOCK" },
    { 0x1FB6A, "UPPER AND LEFT AND LOWER TRIANGULAR THREE QUARTERS BLOCK" },
    { 0x1FB6B, "LEFT AND UPPER AND RIGHT TRIANGULAR THREE QUARTERS BLOCK" },
    { 0x1FB6C, "LEFT TRIANGULAR ONE QUARTER BLOCK" },
    { 0x1FB6D, "UPPER TRIANGULAR ONE QUARTER BLOCK" },
    { 0x1FB6E, "RIGHT TRIANGULAR ONE QUARTER BLOCK" },
    { 0x1FB6F, "LOWER TRIANGULAR ONE QUARTER BLOCK" },
    { 0x1FB70, "VERTICAL ONE EIGHTH BLOCK-2" },
    { 0x1FB71, "VERTICAL ONE EIGHTH BLOCK-3" },
    { 0x1FB72, "VERTICAL ONE EIGHTH BLOCK-4" },
    { 0x1FB73, "VERTICAL ONE EIGHTH BLOCK-5" },
    { 0x1FB74, "VERTICAL ONE EIGHTH BLOCK-6" },
    { 0x1FB75, "VERTICAL ONE EIGHTH BLOCK-7" },
    { 0x1FB76, "HORIZONTAL ONE EIGHTH BLOCK-2" },
    { 0x1FB77, "HORIZONTAL ONE EIGHTH BLOCK-3" },
    { 0x1FB78, "HORIZONTAL ONE EIGHTH BLOCK-4" },
    { 0x1FB79, "HORIZONTAL ONE EIGHTH BLOCK-5" },
    { 0x1FB7A, "HORIZONTAL ONE EIGHTH BLOCK-6" },
    { 0x1FB7B, "HORIZONTAL ONE EIGHTH BLOCK-7" },
    { 0x1FB7C, "LEFT AND LOWER ONE EIGHTH BLOCK" },
    { 0x1FB7D, "LEFT AND UPPER ONE EIGHTH BLOCK" },
    { 0x1FB7E, "RIGHT AND UPPER ONE EIGHTH BLOCK" },
    { 0x1FB7F, "RIGHT AND LOWER ONE EIGHTH BLOCK" },
    { 0x1FB80, "UPPER AND LOWER ONE EIGHTH BLOCK" },
    { 0x1FB81, "HORIZONTAL ONE EIGHTH BLOCK-1358" },
    { 0x1FB82, "UPPER ONE QUARTER BLOCK" },
    { 0x1FB83, "UPPER THREE EIGHTHS BLOCK" },
    { 0x1FB84, "UPPER FIVE EIGHTHS BLOCK" },
    { 0x1FB85, "UPPER THREE QUARTERS BLOCK" },
    { 0x1FB86, "UPPER SEVEN EIGHTHS BLOCK" },
    { 0x1FB87, "RIGHT ONE QUARTER BLOCK" },
    { 0x1FB88, "RIGHT THREE EIGHTHS BLOCK" },
    { 0x1FB89, "RIGHT FIVE EIGHTHS BLOCK" },
    { 0x1FB8A, "RIGHT THREE QUARTERS BLOCK" },
    { 0x1FB8B, "RIGHT SEVEN EIGHTHS BLOCK" },
    { 0x1FB8C, "LEFT HALF MEDIUM SHADE" },
    { 0x1FB8D, "RIGHT HALF MEDIUM SHADE" },
    { 0x1FB8E, "UPPER HALF MEDIUM SHADE" },
    { 0x1FB8F, "LOWER HALF MEDIUM SHADE" },
    { 0x1FB90, "INVERSE MEDIUM SHADE" },
    { 0x1FB91, "UPPER HALF BLOCK AND LOWER HALF INVERSE MEDIUM SHADE" },
    { 0x1FB92, "UPPER HALF INVERSE MEDIUM SHADE AND LOWER HALF BLOCK" },
    { 0x1FB94, "LEFT HALF INVERSE MEDIUM SHADE AND RIGHT HALF BLOCK" },
    { 0x1FB95, "CHECKER BOARD FILL" },
    { 0x1FB96, "INVERSE CHECKER BOARD FILL" },
    { 0x1FB97, "HEAVY HORIZONTAL FILL" },
    { 0x1FB98, "UPPER LEFT TO LOWER RIGHT FILL" },
    { 0x1FB99, "UPPER RIGHT TO LOWER LEFT FILL" },
    { 0x1FB9A, "UPPER AND LOWER TRIANGULAR HALF BLOCK" },
    { 0x1FB9B, "LEFT AND RIGHT TRIANGULAR HALF BLOCK" },
    { 0x1FB9C, "UPPER LEFT TRIANGULAR MEDIUM SHADE" },
    { 0x1FB9D, "UPPER RIGHT TRIANGULAR MEDIUM SHADE" },
    { 0x1FB9E, "LOWER RIGHT TRIANGULAR MEDIUM SHADE" },
    { 0x1FB9F, "LOWER LEFT TRIANGULAR MEDIUM SHADE" },
    { 0x1FBA0, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT" },
    { 0x1FBA1, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT" },
    { 0x1FBA2, "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FBA3, "BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO LOWER CENTRE" },
    { 0x1FBA4, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FBA5, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE" },
    { 0x1FBA6, "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER CENTRE TO MIDDLE RIGHT" },
    { 0x1FBA7, "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO UPPER CENTRE TO MIDDLE RIGHT" },
    { 0x1FBA8, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT AND MIDDLE RIGHT TO LOWER CENTRE" },
    { 0x1FBA9, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT AND MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FBAA, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE TO MIDDLE LEFT" },
    { 0x1FBAB, "BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE TO MIDDLE RIGHT" },
    { 0x1FBAC, "BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE" },
    { 0x1FBAD, "BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE" },
    { 0x1FBAE, "BOX DRAWINGS LIGHT DIAGONAL DIAMOND" },
    { 0x1FBAF, "BOX DRAWINGS LIGHT HORIZONTAL WITH VERTICAL STROKE" },
    { 0x1FBB0, "ARROWHEAD-SHAPED POINTER" },
    { 0x1FBB1, "INVERSE CHECK MARK" },
    { 0x1FBB2, "LEFT HALF RUNNING MAN" },
    { 0x1FBB3, "RIGHT HALF RUNNING MAN" },
    { 0x1FBB4, "INVERSE DOWNWARDS ARROW WITH TIP LEFTWARDS" },
    { 0x1FBB5, "LEFTWARDS ARROW AND UPPER AND LOWER ONE EIGHTH BLOCK" },
    { 0x1FBB6, "RIGHTWARDS ARROW AND UPPER AND LOWER ONE EIGHTH BLOCK" },
    { 0x1FBB7, "DOWNWARDS ARROW AND RIGHT ONE EIGHTH BLOCK" },
    { 0x1FBB8, "UPWARDS ARROW AND RIGHT ONE EIGHTH BLOCK" },
    { 0x1FBB9, "LEFT HALF FOLDER" },
    { 0x1FBBA, "RIGHT HALF FOLDER" },
    { 0x1FBBB, "VOIDED GREEK CROSS" },
    { 0x1FBBC, "RIGHT OPEN SQUARED DOT" },
    { 0x1FBBD, "NEGATIVE DIAGONAL CROSS" },
    { 0x1FBBE, "NEGATIVE DIAGONAL MIDDLE RIGHT TO LOWER CENTRE" },
    { 0x1FBBF, "NEGATIVE DIAGONAL DIAMOND" },
    { 0x1FBC0, "WHITE HEAVY SALTIRE WITH ROUNDED CORNERS" },
    { 0x1FBC1, "LEFT THIRD WHITE RIGHT POINTING INDEX" },
    { 0x1FBC2, "MIDDLE THIRD WHITE RIGHT POINTING INDEX" },
    { 0x1FBC3, "RIGHT THIRD WHITE RIGHT POINTING INDEX" },
    { 0x1FBC4, "NEGATIVE SQUARED QUESTION MARK" },
    { 0x1FBC5, "STICK FIGURE" },
    { 0x1FBC6, "STICK FIGURE WITH ARMS RAISED" },
    { 0x1FBC7, "STICK FIGURE LEANING LEFT" },
    { 0x1FBC8, "STICK FIGURE LEANING RIGHT" },
    { 0x1FBC9, "STICK FIGURE WITH DRESS" },
    { 0x1FBCA, "WHITE UP-POINTING CHEVRON" },
    { 0x1FBF0, "SEGMENTED DIGIT ZERO" },
    { 0x1FBF1, "SEGMENTED DIGIT ONE" },
    { 0x1FBF2, "SEGMENTED DIGIT TWO" },
    { 0x1FBF3, "SEGMENTED DIGIT THREE" },
    { 0x1FBF4, "SEGMENTED DIGIT FOUR" },
    { 0x1FBF5, "SEGMENTED DIGIT FIVE" },
    { 0x1FBF6, "SEGMENTED DIGIT SIX" },
    { 0x1FBF7, "SEGMENTED DIGIT SEVEN" },
    { 0x1FBF8, "SEGMENTED DIGIT EIGHT" },
    { 0x1FBF9, "SEGMENTED DIGIT NINE" },
    { 0x2F800, "CJK COMPATIBILITY IDEOGRAPH-2F800" },
    { 0x2F801, "CJK COMPATIBILITY IDEOGRAPH-2F801" },
    { 0x2F802, "CJK COMPATIBILITY IDEOGRAPH-2F802" },
    { 0x2F803, "CJK COMPATIBILITY IDEOGRAPH-2F803" },
    { 0x2F804, "CJK COMPATIBILITY IDEOGRAPH-2F804" },
    { 0x2F805, "CJK COMPATIBILITY IDEOGRAPH-2F805" },
    { 0x2F806, "CJK COMPATIBILITY IDEOGRAPH-2F806" },
    { 0x2F807, "CJK COMPATIBILITY IDEOGRAPH-2F807" },
    { 0x2F808, "CJK COMPATIBILITY IDEOGRAPH-2F808" },
    { 0x2F809, "CJK COMPATIBILITY IDEOGRAPH-2F809" },
    { 0x2F80A, "CJK COMPATIBILITY IDEOGRAPH-2F80A" },
    { 0x2F80B, "CJK COMPATIBILITY IDEOGRAPH-2F80B" },
    { 0x2F80C, "CJK COMPATIBILITY IDEOGRAPH-2F80C" },
    { 0x2F80D, "CJK COMPATIBILITY IDEOGRAPH-2F80D" },
    { 0x2F80E, "CJK COMPATIBILITY IDEOGRAPH-2F80E" },
    { 0x2F80F, "CJK COMPATIBILITY IDEOGRAPH-2F80F" },
    { 0x2F810, "CJK COMPATIBILITY IDEOGRAPH-2F810" },
    { 0x2F811, "CJK COMPATIBILITY IDEOGRAPH-2F811" },
    { 0x2F812, "CJK COMPATIBILITY IDEOGRAPH-2F812" },
    { 0x2F813, "CJK COMPATIBILITY IDEOGRAPH-2F813" },
    { 0x2F814, "CJK COMPATIBILITY IDEOGRAPH-2F814" },
    { 0x2F815, "CJK COMPATIBILITY IDEOGRAPH-2F815" },
    { 0x2F816, "CJK COMPATIBILITY IDEOGRAPH-2F816" },
    { 0x2F817, "CJK COMPATIBILITY IDEOGRAPH-2F817" },
    { 0x2F818, "CJK COMPATIBILITY IDEOGRAPH-2F818" },
    { 0x2F819, "CJK COMPATIBILITY IDEOGRAPH-2F819" },
    { 0x2F81A, "CJK COMPATIBILITY IDEOGRAPH-2F81A" },
    { 0x2F81B, "CJK COMPATIBILITY IDEOGRAPH-2F81B" },
    { 0x2F81C, "CJK COMPATIBILITY IDEOGRAPH-2F81C" },
    { 0x2F81D, "CJK COMPATIBILITY IDEOGRAPH-2F81D" },
    { 0x2F81E, "CJK COMPATIBILITY IDEOGRAPH-2F81E" },
    { 0x2F81F, "CJK COMPATIBILITY IDEOGRAPH-2F81F" },
    { 0x2F820, "CJK COMPATIBILITY IDEOGRAPH-2F820" },
    { 0x2F821, "CJK COMPATIBILITY IDEOGRAPH-2F821" },
    { 0x2F822, "CJK COMPATIBILITY IDEOGRAPH-2F822" },
    { 0x2F823, "CJK COMPATIBILITY IDEOGRAPH-2F823" },
    { 0x2F824, "CJK COMPATIBILITY IDEOGRAPH-2F824" },
    { 0x2F825, "CJK COMPATIBILITY IDEOGRAPH-2F825" },
    { 0x2F826, "CJK COMPATIBILITY IDEOGRAPH-2F826" },
    { 0x2F827, "CJK COMPATIBILITY IDEOGRAPH-2F827" },
    { 0x2F828, "CJK COMPATIBILITY IDEOGRAPH-2F828" },
    { 0x2F829, "CJK COMPATIBILITY IDEOGRAPH-2F829" },
    { 0x2F82A, "CJK COMPATIBILITY IDEOGRAPH-2F82A" },
    { 0x2F82B, "CJK COMPATIBILITY IDEOGRAPH-2F82B" },
    { 0x2F82C, "CJK COMPATIBILITY IDEOGRAPH-2F82C" },
    { 0x2F82D, "CJK COMPATIBILITY IDEOGRAPH-2F82D" },
    { 0x2F82E, "CJK COMPATIBILITY IDEOGRAPH-2F82E" },
    { 0x2F82F, "CJK COMPATIBILITY IDEOGRAPH-2F82F" },
    { 0x2F830, "CJK COMPATIBILITY IDEOGRAPH-2F830" },
    { 0x2F831, "CJK COMPATIBILITY IDEOGRAPH-2F831" },
    { 0x2F832, "CJK COMPATIBILITY IDEOGRAPH-2F832" },
    { 0x2F833, "CJK COMPATIBILITY IDEOGRAPH-2F833" },
    { 0x2F834, "CJK COMPATIBILITY IDEOGRAPH-2F834" },
    { 0x2F835, "CJK COMPATIBILITY IDEOGRAPH-2F835" },
    { 0x2F836, "CJK COMPATIBILITY IDEOGRAPH-2F836" },
    { 0x2F837, "CJK COMPATIBILITY IDEOGRAPH-2F837" },
    { 0x2F838, "CJK COMPATIBILITY IDEOGRAPH-2F838" },
    { 0x2F839, "CJK COMPATIBILITY IDEOGRAPH-2F839" },
    { 0x2F83A, "CJK COMPATIBILITY IDEOGRAPH-2F83A" },
    { 0x2F83B, "CJK COMPATIBILITY IDEOGRAPH-2F83B" },
    { 0x2F83C, "CJK COMPATIBILITY IDEOGRAPH-2F83C" },
    { 0x2F83D, "CJK COMPATIBILITY IDEOGRAPH-2F83D" },
    { 0x2F83E, "CJK COMPATIBILITY IDEOGRAPH-2F83E" },
    { 0x2F83F, "CJK COMPATIBILITY IDEOGRAPH-2F83F" },
    { 0x2F840, "CJK COMPATIBILITY IDEOGRAPH-2F840" },
    { 0x2F841, "CJK COMPATIBILITY IDEOGRAPH-2F841" },
    { 0x2F842, "CJK COMPATIBILITY IDEOGRAPH-2F842" },
    { 0x2F843, "CJK COMPATIBILITY IDEOGRAPH-2F843" },
    { 0x2F844, "CJK COMPATIBILITY IDEOGRAPH-2F844" },
    { 0x2F845, "CJK COMPATIBILITY IDEOGRAPH-2F845" },
    { 0x2F846, "CJK COMPATIBILITY IDEOGRAPH-2F846" },
    { 0x2F847, "CJK COMPATIBILITY IDEOGRAPH-2F847" },
    { 0x2F848, "CJK COMPATIBILITY IDEOGRAPH-2F848" },
    { 0x2F849, "CJK COMPATIBILITY IDEOGRAPH-2F849" },
    { 0x2F84A, "CJK COMPATIBILITY IDEOGRAPH-2F84A" },
    { 0x2F84B, "CJK COMPATIBILITY IDEOGRAPH-2F84B" },
    { 0x2F84C, "CJK COMPATIBILITY IDEOGRAPH-2F84C" },
    { 0x2F84D, "CJK COMPATIBILITY IDEOGRAPH-2F84D" },
    { 0x2F84E, "CJK COMPATIBILITY IDEOGRAPH-2F84E" },
    { 0x2F84F, "CJK COMPATIBILITY IDEOGRAPH-2F84F" },
    { 0x2F850, "CJK COMPATIBILITY IDEOGRAPH-2F850" },
    { 0x2F851, "CJK COMPATIBILITY IDEOGRAPH-2F851" },
    { 0x2F852, "CJK COMPATIBILITY IDEOGRAPH-2F852" },
    { 0x2F853, "CJK COMPATIBILITY IDEOGRAPH-2F853" },
    { 0x2F854, "CJK COMPATIBILITY IDEOGRAPH-2F854" },
    { 0x2F855, "CJK COMPATIBILITY IDEOGRAPH-2F855" },
    { 0x2F856, "CJK COMPATIBILITY IDEOGRAPH-2F856" },
    { 0x2F857, "CJK COMPATIBILITY IDEOGRAPH-2F857" },
    { 0x2F858, "CJK COMPATIBILITY IDEOGRAPH-2F858" },
    { 0x2F859, "CJK COMPATIBILITY IDEOGRAPH-2F859" },
    { 0x2F85A, "CJK COMPATIBILITY IDEOGRAPH-2F85A" },
    { 0x2F85B, "CJK COMPATIBILITY IDEOGRAPH-2F85B" },
    { 0x2F85C, "CJK COMPATIBILITY IDEOGRAPH-2F85C" },
    { 0x2F85D, "CJK COMPATIBILITY IDEOGRAPH-2F85D" },
    { 0x2F85E, "CJK COMPATIBILITY IDEOGRAPH-2F85E" },
    { 0x2F85F, "CJK COMPATIBILITY IDEOGRAPH-2F85F" },
    { 0x2F860, "CJK COMPATIBILITY IDEOGRAPH-2F860" },
    { 0x2F861, "CJK COMPATIBILITY IDEOGRAPH-2F861" },
    { 0x2F862, "CJK COMPATIBILITY IDEOGRAPH-2F862" },
    { 0x2F863, "CJK COMPATIBILITY IDEOGRAPH-2F863" },
    { 0x2F864, "CJK COMPATIBILITY IDEOGRAPH-2F864" },
    { 0x2F865, "CJK COMPATIBILITY IDEOGRAPH-2F865" },
    { 0x2F866, "CJK COMPATIBILITY IDEOGRAPH-2F866" },
    { 0x2F867, "CJK COMPATIBILITY IDEOGRAPH-2F867" },
    { 0x2F868, "CJK COMPATIBILITY IDEOGRAPH-2F868" },
    { 0x2F869, "CJK COMPATIBILITY IDEOGRAPH-2F869" },
    { 0x2F86A, "CJK COMPATIBILITY IDEOGRAPH-2F86A" },
    { 0x2F86B, "CJK COMPATIBILITY IDEOGRAPH-2F86B" },
    { 0x2F86C, "CJK COMPATIBILITY IDEOGRAPH-2F86C" },
    { 0x2F86D, "CJK COMPATIBILITY IDEOGRAPH-2F86D" },
    { 0x2F86E, "CJK COMPATIBILITY IDEOGRAPH-2F86E" },
    { 0x2F86F, "CJK COMPATIBILITY IDEOGRAPH-2F86F" },
    { 0x2F870, "CJK COMPATIBILITY IDEOGRAPH-2F870" },
    { 0x2F871, "CJK COMPATIBILITY IDEOGRAPH-2F871" },
    { 0x2F872, "CJK COMPATIBILITY IDEOGRAPH-2F872" },
    { 0x2F873, "CJK COMPATIBILITY IDEOGRAPH-2F873" },
    { 0x2F874, "CJK COMPATIBILITY IDEOGRAPH-2F874" },
    { 0x2F875, "CJK COMPATIBILITY IDEOGRAPH-2F875" },
    { 0x2F876, "CJK COMPATIBILITY IDEOGRAPH-2F876" },
    { 0x2F877, "CJK COMPATIBILITY IDEOGRAPH-2F877" },
    { 0x2F878, "CJK COMPATIBILITY IDEOGRAPH-2F878" },
    { 0x2F879, "CJK COMPATIBILITY IDEOGRAPH-2F879" },
    { 0x2F87A, "CJK COMPATIBILITY IDEOGRAPH-2F87A" },
    { 0x2F87B, "CJK COMPATIBILITY IDEOGRAPH-2F87B" },
    { 0x2F87C, "CJK COMPATIBILITY IDEOGRAPH-2F87C" },
    { 0x2F87D, "CJK COMPATIBILITY IDEOGRAPH-2F87D" },
    { 0x2F87E, "CJK COMPATIBILITY IDEOGRAPH-2F87E" },
    { 0x2F87F, "CJK COMPATIBILITY IDEOGRAPH-2F87F" },
    { 0x2F880, "CJK COMPATIBILITY IDEOGRAPH-2F880" },
    { 0x2F881, "CJK COMPATIBILITY IDEOGRAPH-2F881" },
    { 0x2F882, "CJK COMPATIBILITY IDEOGRAPH-2F882" },
    { 0x2F883, "CJK COMPATIBILITY IDEOGRAPH-2F883" },
    { 0x2F884, "CJK COMPATIBILITY IDEOGRAPH-2F884" },
    { 0x2F885, "CJK COMPATIBILITY IDEOGRAPH-2F885" },
    { 0x2F886, "CJK COMPATIBILITY IDEOGRAPH-2F886" },
    { 0x2F887, "CJK COMPATIBILITY IDEOGRAPH-2F887" },
    { 0x2F888, "CJK COMPATIBILITY IDEOGRAPH-2F888" },
    { 0x2F889, "CJK COMPATIBILITY IDEOGRAPH-2F889" },
    { 0x2F88A, "CJK COMPATIBILITY IDEOGRAPH-2F88A" },
    { 0x2F88B, "CJK COMPATIBILITY IDEOGRAPH-2F88B" },
    { 0x2F88C, "CJK COMPATIBILITY IDEOGRAPH-2F88C" },
    { 0x2F88D, "CJK COMPATIBILITY IDEOGRAPH-2F88D" },
    { 0x2F88E, "CJK COMPATIBILITY IDEOGRAPH-2F88E" },
    { 0x2F88F, "CJK COMPATIBILITY IDEOGRAPH-2F88F" },
    { 0x2F890, "CJK COMPATIBILITY IDEOGRAPH-2F890" },
    { 0x2F891, "CJK COMPATIBILITY IDEOGRAPH-2F891" },
    { 0x2F892, "CJK COMPATIBILITY IDEOGRAPH-2F892" },
    { 0x2F893, "CJK COMPATIBILITY IDEOGRAPH-2F893" },
    { 0x2F894, "CJK COMPATIBILITY IDEOGRAPH-2F894" },
    { 0x2F895, "CJK COMPATIBILITY IDEOGRAPH-2F895" },
    { 0x2F896, "CJK COMPATIBILITY IDEOGRAPH-2F896" },
    { 0x2F897, "CJK COMPATIBILITY IDEOGRAPH-2F897" },
    { 0x2F898, "CJK COMPATIBILITY IDEOGRAPH-2F898" },
    { 0x2F899, "CJK COMPATIBILITY IDEOGRAPH-2F899" },
    { 0x2F89A, "CJK COMPATIBILITY IDEOGRAPH-2F89A" },
    { 0x2F89B, "CJK COMPATIBILITY IDEOGRAPH-2F89B" },
    { 0x2F89C, "CJK COMPATIBILITY IDEOGRAPH-2F89C" },
    { 0x2F89D, "CJK COMPATIBILITY IDEOGRAPH-2F89D" },
    { 0x2F89E, "CJK COMPATIBILITY IDEOGRAPH-2F89E" },
    { 0x2F89F, "CJK COMPATIBILITY IDEOGRAPH-2F89F" },
    { 0x2F8A0, "CJK COMPATIBILITY IDEOGRAPH-2F8A0" },
    { 0x2F8A1, "CJK COMPATIBILITY IDEOGRAPH-2F8A1" },
    { 0x2F8A2, "CJK COMPATIBILITY IDEOGRAPH-2F8A2" },
    { 0x2F8A3, "CJK COMPATIBILITY IDEOGRAPH-2F8A3" },
    { 0x2F8A4, "CJK COMPATIBILITY IDEOGRAPH-2F8A4" },
    { 0x2F8A5, "CJK COMPATIBILITY IDEOGRAPH-2F8A5" },
    { 0x2F8A6, "CJK COMPATIBILITY IDEOGRAPH-2F8A6" },
    { 0x2F8A7, "CJK COMPATIBILITY IDEOGRAPH-2F8A7" },
    { 0x2F8A8, "CJK COMPATIBILITY IDEOGRAPH-2F8A8" },
    { 0x2F8A9, "CJK COMPATIBILITY IDEOGRAPH-2F8A9" },
    { 0x2F8AA, "CJK COMPATIBILITY IDEOGRAPH-2F8AA" },
    { 0x2F8AB, "CJK COMPATIBILITY IDEOGRAPH-2F8AB" },
    { 0x2F8AC, "CJK COMPATIBILITY IDEOGRAPH-2F8AC" },
    { 0x2F8AD, "CJK COMPATIBILITY IDEOGRAPH-2F8AD" },
    { 0x2F8AE, "CJK COMPATIBILITY IDEOGRAPH-2F8AE" },
    { 0x2F8AF, "CJK COMPATIBILITY IDEOGRAPH-2F8AF" },
    { 0x2F8B0, "CJK COMPATIBILITY IDEOGRAPH-2F8B0" },
    { 0x2F8B1, "CJK COMPATIBILITY IDEOGRAPH-2F8B1" },
    { 0x2F8B2, "CJK COMPATIBILITY IDEOGRAPH-2F8B2" },
    { 0x2F8B3, "CJK COMPATIBILITY IDEOGRAPH-2F8B3" },
    { 0x2F8B4, "CJK COMPATIBILITY IDEOGRAPH-2F8B4" },
    { 0x2F8B5, "CJK COMPATIBILITY IDEOGRAPH-2F8B5" },
    { 0x2F8B6, "CJK COMPATIBILITY IDEOGRAPH-2F8B6" },
    { 0x2F8B7, "CJK COMPATIBILITY IDEOGRAPH-2F8B7" },
    { 0x2F8B8, "CJK COMPATIBILITY IDEOGRAPH-2F8B8" },
    { 0x2F8B9, "CJK COMPATIBILITY IDEOGRAPH-2F8B9" },
    { 0x2F8BA, "CJK COMPATIBILITY IDEOGRAPH-2F8BA" },
    { 0x2F8BB, "CJK COMPATIBILITY IDEOGRAPH-2F8BB" },
    { 0x2F8BC, "CJK COMPATIBILITY IDEOGRAPH-2F8BC" },
    { 0x2F8BD, "CJK COMPATIBILITY IDEOGRAPH-2F8BD" },
    { 0x2F8BE, "CJK COMPATIBILITY IDEOGRAPH-2F8BE" },
    { 0x2F8BF, "CJK COMPATIBILITY IDEOGRAPH-2F8BF" },
    { 0x2F8C0, "CJK COMPATIBILITY IDEOGRAPH-2F8C0" },
    { 0x2F8C1, "CJK COMPATIBILITY IDEOGRAPH-2F8C1" },
    { 0x2F8C2, "CJK COMPATIBILITY IDEOGRAPH-2F8C2" },
    { 0x2F8C3, "CJK COMPATIBILITY IDEOGRAPH-2F8C3" },
    { 0x2F8C4, "CJK COMPATIBILITY IDEOGRAPH-2F8C4" },
    { 0x2F8C5, "CJK COMPATIBILITY IDEOGRAPH-2F8C5" },
    { 0x2F8C6, "CJK COMPATIBILITY IDEOGRAPH-2F8C6" },
    { 0x2F8C7, "CJK COMPATIBILITY IDEOGRAPH-2F8C7" },
    { 0x2F8C8, "CJK COMPATIBILITY IDEOGRAPH-2F8C8" },
    { 0x2F8C9, "CJK COMPATIBILITY IDEOGRAPH-2F8C9" },
    { 0x2F8CA, "CJK COMPATIBILITY IDEOGRAPH-2F8CA" },
    { 0x2F8CB, "CJK COMPATIBILITY IDEOGRAPH-2F8CB" },
    { 0x2F8CC, "CJK COMPATIBILITY IDEOGRAPH-2F8CC" },
    { 0x2F8CD, "CJK COMPATIBILITY IDEOGRAPH-2F8CD" },
    { 0x2F8CE, "CJK COMPATIBILITY IDEOGRAPH-2F8CE" },
    { 0x2F8CF, "CJK COMPATIBILITY IDEOGRAPH-2F8CF" },
    { 0x2F8D0, "CJK COMPATIBILITY IDEOGRAPH-2F8D0" },
    { 0x2F8D1, "CJK COMPATIBILITY IDEOGRAPH-2F8D1" },
    { 0x2F8D2, "CJK COMPATIBILITY IDEOGRAPH-2F8D2" },
    { 0x2F8D3, "CJK COMPATIBILITY IDEOGRAPH-2F8D3" },
    { 0x2F8D4, "CJK COMPATIBILITY IDEOGRAPH-2F8D4" },
    { 0x2F8D5, "CJK COMPATIBILITY IDEOGRAPH-2F8D5" },
    { 0x2F8D6, "CJK COMPATIBILITY IDEOGRAPH-2F8D6" },
    { 0x2F8D7, "CJK COMPATIBILITY IDEOGRAPH-2F8D7" },
    { 0x2F8D8, "CJK COMPATIBILITY IDEOGRAPH-2F8D8" },
    { 0x2F8D9, "CJK COMPATIBILITY IDEOGRAPH-2F8D9" },
    { 0x2F8DA, "CJK COMPATIBILITY IDEOGRAPH-2F8DA" },
    { 0x2F8DB, "CJK COMPATIBILITY IDEOGRAPH-2F8DB" },
    { 0x2F8DC, "CJK COMPATIBILITY IDEOGRAPH-2F8DC" },
    { 0x2F8DD, "CJK COMPATIBILITY IDEOGRAPH-2F8DD" },
    { 0x2F8DE, "CJK COMPATIBILITY IDEOGRAPH-2F8DE" },
    { 0x2F8DF, "CJK COMPATIBILITY IDEOGRAPH-2F8DF" },
    { 0x2F8E0, "CJK COMPATIBILITY IDEOGRAPH-2F8E0" },
    { 0x2F8E1, "CJK COMPATIBILITY IDEOGRAPH-2F8E1" },
    { 0x2F8E2, "CJK COMPATIBILITY IDEOGRAPH-2F8E2" },
    { 0x2F8E3, "CJK COMPATIBILITY IDEOGRAPH-2F8E3" },
    { 0x2F8E4, "CJK COMPATIBILITY IDEOGRAPH-2F8E4" },
    { 0x2F8E5, "CJK COMPATIBILITY IDEOGRAPH-2F8E5" },
    { 0x2F8E6, "CJK COMPATIBILITY IDEOGRAPH-2F8E6" },
    { 0x2F8E7, "CJK COMPATIBILITY IDEOGRAPH-2F8E7" },
    { 0x2F8E8, "CJK COMPATIBILITY IDEOGRAPH-2F8E8" },
    { 0x2F8E9, "CJK COMPATIBILITY IDEOGRAPH-2F8E9" },
    { 0x2F8EA, "CJK COMPATIBILITY IDEOGRAPH-2F8EA" },
    { 0x2F8EB, "CJK COMPATIBILITY IDEOGRAPH-2F8EB" },
    { 0x2F8EC, "CJK COMPATIBILITY IDEOGRAPH-2F8EC" },
    { 0x2F8ED, "CJK COMPATIBILITY IDEOGRAPH-2F8ED" },
    { 0x2F8EE, "CJK COMPATIBILITY IDEOGRAPH-2F8EE" },
    { 0x2F8EF, "CJK COMPATIBILITY IDEOGRAPH-2F8EF" },
    { 0x2F8F0, "CJK COMPATIBILITY IDEOGRAPH-2F8F0" },
    { 0x2F8F1, "CJK COMPATIBILITY IDEOGRAPH-2F8F1" },
    { 0x2F8F2, "CJK COMPATIBILITY IDEOGRAPH-2F8F2" },
    { 0x2F8F3, "CJK COMPATIBILITY IDEOGRAPH-2F8F3" },
    { 0x2F8F4, "CJK COMPATIBILITY IDEOGRAPH-2F8F4" },
    { 0x2F8F5, "CJK COMPATIBILITY IDEOGRAPH-2F8F5" },
    { 0x2F8F6, "CJK COMPATIBILITY IDEOGRAPH-2F8F6" },
    { 0x2F8F7, "CJK COMPATIBILITY IDEOGRAPH-2F8F7" },
    { 0x2F8F8, "CJK COMPATIBILITY IDEOGRAPH-2F8F8" },
    { 0x2F8F9, "CJK COMPATIBILITY IDEOGRAPH-2F8F9" },
    { 0x2F8FA, "CJK COMPATIBILITY IDEOGRAPH-2F8FA" },
    { 0x2F8FB, "CJK COMPATIBILITY IDEOGRAPH-2F8FB" },
    { 0x2F8FC, "CJK COMPATIBILITY IDEOGRAPH-2F8FC" },
    { 0x2F8FD, "CJK COMPATIBILITY IDEOGRAPH-2F8FD" },
    { 0x2F8FE, "CJK COMPATIBILITY IDEOGRAPH-2F8FE" },
    { 0x2F8FF, "CJK COMPATIBILITY IDEOGRAPH-2F8FF" },
    { 0x2F900, "CJK COMPATIBILITY IDEOGRAPH-2F900" },
    { 0x2F901, "CJK COMPATIBILITY IDEOGRAPH-2F901" },
    { 0x2F902, "CJK COMPATIBILITY IDEOGRAPH-2F902" },
    { 0x2F903, "CJK COMPATIBILITY IDEOGRAPH-2F903" },
    { 0x2F904, "CJK COMPATIBILITY IDEOGRAPH-2F904" },
    { 0x2F905, "CJK COMPATIBILITY IDEOGRAPH-2F905" },
    { 0x2F906, "CJK COMPATIBILITY IDEOGRAPH-2F906" },
    { 0x2F907, "CJK COMPATIBILITY IDEOGRAPH-2F907" },
    { 0x2F908, "CJK COMPATIBILITY IDEOGRAPH-2F908" },
    { 0x2F909, "CJK COMPATIBILITY IDEOGRAPH-2F909" },
    { 0x2F90A, "CJK COMPATIBILITY IDEOGRAPH-2F90A" },
    { 0x2F90B, "CJK COMPATIBILITY IDEOGRAPH-2F90B" },
    { 0x2F90C, "CJK COMPATIBILITY IDEOGRAPH-2F90C" },
    { 0x2F90D, "CJK COMPATIBILITY IDEOGRAPH-2F90D" },
    { 0x2F90E, "CJK COMPATIBILITY IDEOGRAPH-2F90E" },
    { 0x2F90F, "CJK COMPATIBILITY IDEOGRAPH-2F90F" },
    { 0x2F910, "CJK COMPATIBILITY IDEOGRAPH-2F910" },
    { 0x2F911, "CJK COMPATIBILITY IDEOGRAPH-2F911" },
    { 0x2F912, "CJK COMPATIBILITY IDEOGRAPH-2F912" },
    { 0x2F913, "CJK COMPATIBILITY IDEOGRAPH-2F913" },
    { 0x2F914, "CJK COMPATIBILITY IDEOGRAPH-2F914" },
    { 0x2F915, "CJK COMPATIBILITY IDEOGRAPH-2F915" },
    { 0x2F916, "CJK COMPATIBILITY IDEOGRAPH-2F916" },
    { 0x2F917, "CJK COMPATIBILITY IDEOGRAPH-2F917" },
    { 0x2F918, "CJK COMPATIBILITY IDEOGRAPH-2F918" },
    { 0x2F919, "CJK COMPATIBILITY IDEOGRAPH-2F919" },
    { 0x2F91A, "CJK COMPATIBILITY IDEOGRAPH-2F91A" },
    { 0x2F91B, "CJK COMPATIBILITY IDEOGRAPH-2F91B" },
    { 0x2F91C, "CJK COMPATIBILITY IDEOGRAPH-2F91C" },
    { 0x2F91D, "CJK COMPATIBILITY IDEOGRAPH-2F91D" },
    { 0x2F91E, "CJK COMPATIBILITY IDEOGRAPH-2F91E" },
    { 0x2F91F, "CJK COMPATIBILITY IDEOGRAPH-2F91F" },
    { 0x2F920, "CJK COMPATIBILITY IDEOGRAPH-2F920" },
    { 0x2F921, "CJK COMPATIBILITY IDEOGRAPH-2F921" },
    { 0x2F922, "CJK COMPATIBILITY IDEOGRAPH-2F922" },
    { 0x2F923, "CJK COMPATIBILITY IDEOGRAPH-2F923" },
    { 0x2F924, "CJK COMPATIBILITY IDEOGRAPH-2F924" },
    { 0x2F925, "CJK COMPATIBILITY IDEOGRAPH-2F925" },
    { 0x2F926, "CJK COMPATIBILITY IDEOGRAPH-2F926" },
    { 0x2F927, "CJK COMPATIBILITY IDEOGRAPH-2F927" },
    { 0x2F928, "CJK COMPATIBILITY IDEOGRAPH-2F928" },
    { 0x2F929, "CJK COMPATIBILITY IDEOGRAPH-2F929" },
    { 0x2F92A, "CJK COMPATIBILITY IDEOGRAPH-2F92A" },
    { 0x2F92B, "CJK COMPATIBILITY IDEOGRAPH-2F92B" },
    { 0x2F92C, "CJK COMPATIBILITY IDEOGRAPH-2F92C" },
    { 0x2F92D, "CJK COMPATIBILITY IDEOGRAPH-2F92D" },
    { 0x2F92E, "CJK COMPATIBILITY IDEOGRAPH-2F92E" },
    { 0x2F92F, "CJK COMPATIBILITY IDEOGRAPH-2F92F" },
    { 0x2F930, "CJK COMPATIBILITY IDEOGRAPH-2F930" },
    { 0x2F931, "CJK COMPATIBILITY IDEOGRAPH-2F931" },
    { 0x2F932, "CJK COMPATIBILITY IDEOGRAPH-2F932" },
    { 0x2F933, "CJK COMPATIBILITY IDEOGRAPH-2F933" },
    { 0x2F934, "CJK COMPATIBILITY IDEOGRAPH-2F934" },
    { 0x2F935, "CJK COMPATIBILITY IDEOGRAPH-2F935" },
    { 0x2F936, "CJK COMPATIBILITY IDEOGRAPH-2F936" },
    { 0x2F937, "CJK COMPATIBILITY IDEOGRAPH-2F937" },
    { 0x2F938, "CJK COMPATIBILITY IDEOGRAPH-2F938" },
    { 0x2F939, "CJK COMPATIBILITY IDEOGRAPH-2F939" },
    { 0x2F93A, "CJK COMPATIBILITY IDEOGRAPH-2F93A" },
    { 0x2F93B, "CJK COMPATIBILITY IDEOGRAPH-2F93B" },
    { 0x2F93C, "CJK COMPATIBILITY IDEOGRAPH-2F93C" },
    { 0x2F93D, "CJK COMPATIBILITY IDEOGRAPH-2F93D" },
    { 0x2F93E, "CJK COMPATIBILITY IDEOGRAPH-2F93E" },
    { 0x2F93F, "CJK COMPATIBILITY IDEOGRAPH-2F93F" },
    { 0x2F940, "CJK COMPATIBILITY IDEOGRAPH-2F940" },
    { 0x2F941, "CJK COMPATIBILITY IDEOGRAPH-2F941" },
    { 0x2F942, "CJK COMPATIBILITY IDEOGRAPH-2F942" },
    { 0x2F943, "CJK COMPATIBILITY IDEOGRAPH-2F943" },
    { 0x2F944, "CJK COMPATIBILITY IDEOGRAPH-2F944" },
    { 0x2F945, "CJK COMPATIBILITY IDEOGRAPH-2F945" },
    { 0x2F946, "CJK COMPATIBILITY IDEOGRAPH-2F946" },
    { 0x2F947, "CJK COMPATIBILITY IDEOGRAPH-2F947" },
    { 0x2F948, "CJK COMPATIBILITY IDEOGRAPH-2F948" },
    { 0x2F949, "CJK COMPATIBILITY IDEOGRAPH-2F949" },
    { 0x2F94A, "CJK COMPATIBILITY IDEOGRAPH-2F94A" },
    { 0x2F94B, "CJK COMPATIBILITY IDEOGRAPH-2F94B" },
    { 0x2F94C, "CJK COMPATIBILITY IDEOGRAPH-2F94C" },
    { 0x2F94D, "CJK COMPATIBILITY IDEOGRAPH-2F94D" },
    { 0x2F94E, "CJK COMPATIBILITY IDEOGRAPH-2F94E" },
    { 0x2F94F, "CJK COMPATIBILITY IDEOGRAPH-2F94F" },
    { 0x2F950, "CJK COMPATIBILITY IDEOGRAPH-2F950" },
    { 0x2F951, "CJK COMPATIBILITY IDEOGRAPH-2F951" },
    { 0x2F952, "CJK COMPATIBILITY IDEOGRAPH-2F952" },
    { 0x2F953, "CJK COMPATIBILITY IDEOGRAPH-2F953" },
    { 0x2F954, "CJK COMPATIBILITY IDEOGRAPH-2F954" },
    { 0x2F955, "CJK COMPATIBILITY IDEOGRAPH-2F955" },
    { 0x2F956, "CJK COMPATIBILITY IDEOGRAPH-2F956" },
    { 0x2F957, "CJK COMPATIBILITY IDEOGRAPH-2F957" },
    { 0x2F958, "CJK COMPATIBILITY IDEOGRAPH-2F958" },
    { 0x2F959, "CJK COMPATIBILITY IDEOGRAPH-2F959" },
    { 0x2F95A, "CJK COMPATIBILITY IDEOGRAPH-2F95A" },
    { 0x2F95B, "CJK COMPATIBILITY IDEOGRAPH-2F95B" },
    { 0x2F95C, "CJK COMPATIBILITY IDEOGRAPH-2F95C" },
    { 0x2F95D, "CJK COMPATIBILITY IDEOGRAPH-2F95D" },
    { 0x2F95E, "CJK COMPATIBILITY IDEOGRAPH-2F95E" },
    { 0x2F95F, "CJK COMPATIBILITY IDEOGRAPH-2F95F" },
    { 0x2F960, "CJK COMPATIBILITY IDEOGRAPH-2F960" },
    { 0x2F961, "CJK COMPATIBILITY IDEOGRAPH-2F961" },
    { 0x2F962, "CJK COMPATIBILITY IDEOGRAPH-2F962" },
    { 0x2F963, "CJK COMPATIBILITY IDEOGRAPH-2F963" },
    { 0x2F964, "CJK COMPATIBILITY IDEOGRAPH-2F964" },
    { 0x2F965, "CJK COMPATIBILITY IDEOGRAPH-2F965" },
    { 0x2F966, "CJK COMPATIBILITY IDEOGRAPH-2F966" },
    { 0x2F967, "CJK COMPATIBILITY IDEOGRAPH-2F967" },
    { 0x2F968, "CJK COMPATIBILITY IDEOGRAPH-2F968" },
    { 0x2F969, "CJK COMPATIBILITY IDEOGRAPH-2F969" },
    { 0x2F96A, "CJK COMPATIBILITY IDEOGRAPH-2F96A" },
    { 0x2F96B, "CJK COMPATIBILITY IDEOGRAPH-2F96B" },
    { 0x2F96C, "CJK COMPATIBILITY IDEOGRAPH-2F96C" },
    { 0x2F96D, "CJK COMPATIBILITY IDEOGRAPH-2F96D" },
    { 0x2F96E, "CJK COMPATIBILITY IDEOGRAPH-2F96E" },
    { 0x2F96F, "CJK COMPATIBILITY IDEOGRAPH-2F96F" },
    { 0x2F970, "CJK COMPATIBILITY IDEOGRAPH-2F970" },
    { 0x2F971, "CJK COMPATIBILITY IDEOGRAPH-2F971" },
    { 0x2F972, "CJK COMPATIBILITY IDEOGRAPH-2F972" },
    { 0x2F973, "CJK COMPATIBILITY IDEOGRAPH-2F973" },
    { 0x2F974, "CJK COMPATIBILITY IDEOGRAPH-2F974" },
    { 0x2F975, "CJK COMPATIBILITY IDEOGRAPH-2F975" },
    { 0x2F976, "CJK COMPATIBILITY IDEOGRAPH-2F976" },
    { 0x2F977, "CJK COMPATIBILITY IDEOGRAPH-2F977" },
    { 0x2F978, "CJK COMPATIBILITY IDEOGRAPH-2F978" },
    { 0x2F979, "CJK COMPATIBILITY IDEOGRAPH-2F979" },
    { 0x2F97A, "CJK COMPATIBILITY IDEOGRAPH-2F97A" },
    { 0x2F97B, "CJK COMPATIBILITY IDEOGRAPH-2F97B" },
    { 0x2F97C, "CJK COMPATIBILITY IDEOGRAPH-2F97C" },
    { 0x2F97D, "CJK COMPATIBILITY IDEOGRAPH-2F97D" },
    { 0x2F97E, "CJK COMPATIBILITY IDEOGRAPH-2F97E" },
    { 0x2F97F, "CJK COMPATIBILITY IDEOGRAPH-2F97F" },
    { 0x2F980, "CJK COMPATIBILITY IDEOGRAPH-2F980" },
    { 0x2F981, "CJK COMPATIBILITY IDEOGRAPH-2F981" },
    { 0x2F982, "CJK COMPATIBILITY IDEOGRAPH-2F982" },
    { 0x2F983, "CJK COMPATIBILITY IDEOGRAPH-2F983" },
    { 0x2F984, "CJK COMPATIBILITY IDEOGRAPH-2F984" },
    { 0x2F985, "CJK COMPATIBILITY IDEOGRAPH-2F985" },
    { 0x2F986, "CJK COMPATIBILITY IDEOGRAPH-2F986" },
    { 0x2F987, "CJK COMPATIBILITY IDEOGRAPH-2F987" },
    { 0x2F988, "CJK COMPATIBILITY IDEOGRAPH-2F988" },
    { 0x2F989, "CJK COMPATIBILITY IDEOGRAPH-2F989" },
    { 0x2F98A, "CJK COMPATIBILITY IDEOGRAPH-2F98A" },
    { 0x2F98B, "CJK COMPATIBILITY IDEOGRAPH-2F98B" },
    { 0x2F98C, "CJK COMPATIBILITY IDEOGRAPH-2F98C" },
    { 0x2F98D, "CJK COMPATIBILITY IDEOGRAPH-2F98D" },
    { 0x2F98E, "CJK COMPATIBILITY IDEOGRAPH-2F98E" },
    { 0x2F98F, "CJK COMPATIBILITY IDEOGRAPH-2F98F" },
    { 0x2F990, "CJK COMPATIBILITY IDEOGRAPH-2F990" },
    { 0x2F991, "CJK COMPATIBILITY IDEOGRAPH-2F991" },
    { 0x2F992, "CJK COMPATIBILITY IDEOGRAPH-2F992" },
    { 0x2F993, "CJK COMPATIBILITY IDEOGRAPH-2F993" },
    { 0x2F994, "CJK COMPATIBILITY IDEOGRAPH-2F994" },
    { 0x2F995, "CJK COMPATIBILITY IDEOGRAPH-2F995" },
    { 0x2F996, "CJK COMPATIBILITY IDEOGRAPH-2F996" },
    { 0x2F997, "CJK COMPATIBILITY IDEOGRAPH-2F997" },
    { 0x2F998, "CJK COMPATIBILITY IDEOGRAPH-2F998" },
    { 0x2F999, "CJK COMPATIBILITY IDEOGRAPH-2F999" },
    { 0x2F99A, "CJK COMPATIBILITY IDEOGRAPH-2F99A" },
    { 0x2F99B, "CJK COMPATIBILITY IDEOGRAPH-2F99B" },
    { 0x2F99C, "CJK COMPATIBILITY IDEOGRAPH-2F99C" },
    { 0x2F99D, "CJK COMPATIBILITY IDEOGRAPH-2F99D" },
    { 0x2F99E, "CJK COMPATIBILITY IDEOGRAPH-2F99E" },
    { 0x2F99F, "CJK COMPATIBILITY IDEOGRAPH-2F99F" },
    { 0x2F9A0, "CJK COMPATIBILITY IDEOGRAPH-2F9A0" },
    { 0x2F9A1, "CJK COMPATIBILITY IDEOGRAPH-2F9A1" },
    { 0x2F9A2, "CJK COMPATIBILITY IDEOGRAPH-2F9A2" },
    { 0x2F9A3, "CJK COMPATIBILITY IDEOGRAPH-2F9A3" },
    { 0x2F9A4, "CJK COMPATIBILITY IDEOGRAPH-2F9A4" },
    { 0x2F9A5, "CJK COMPATIBILITY IDEOGRAPH-2F9A5" },
    { 0x2F9A6, "CJK COMPATIBILITY IDEOGRAPH-2F9A6" },
    { 0x2F9A7, "CJK COMPATIBILITY IDEOGRAPH-2F9A7" },
    { 0x2F9A8, "CJK COMPATIBILITY IDEOGRAPH-2F9A8" },
    { 0x2F9A9, "CJK COMPATIBILITY IDEOGRAPH-2F9A9" },
    { 0x2F9AA, "CJK COMPATIBILITY IDEOGRAPH-2F9AA" },
    { 0x2F9AB, "CJK COMPATIBILITY IDEOGRAPH-2F9AB" },
    { 0x2F9AC, "CJK COMPATIBILITY IDEOGRAPH-2F9AC" },
    { 0x2F9AD, "CJK COMPATIBILITY IDEOGRAPH-2F9AD" },
    { 0x2F9AE, "CJK COMPATIBILITY IDEOGRAPH-2F9AE" },
    { 0x2F9AF, "CJK COMPATIBILITY IDEOGRAPH-2F9AF" },
    { 0x2F9B0, "CJK COMPATIBILITY IDEOGRAPH-2F9B0" },
    { 0x2F9B1, "CJK COMPATIBILITY IDEOGRAPH-2F9B1" },
    { 0x2F9B2, "CJK COMPATIBILITY IDEOGRAPH-2F9B2" },
    { 0x2F9B3, "CJK COMPATIBILITY IDEOGRAPH-2F9B3" },
    { 0x2F9B4, "CJK COMPATIBILITY IDEOGRAPH-2F9B4" },
    { 0x2F9B5, "CJK COMPATIBILITY IDEOGRAPH-2F9B5" },
    { 0x2F9B6, "CJK COMPATIBILITY IDEOGRAPH-2F9B6" },
    { 0x2F9B7, "CJK COMPATIBILITY IDEOGRAPH-2F9B7" },
    { 0x2F9B8, "CJK COMPATIBILITY IDEOGRAPH-2F9B8" },
    { 0x2F9B9, "CJK COMPATIBILITY IDEOGRAPH-2F9B9" },
    { 0x2F9BA, "CJK COMPATIBILITY IDEOGRAPH-2F9BA" },
    { 0x2F9BB, "CJK COMPATIBILITY IDEOGRAPH-2F9BB" },
    { 0x2F9BC, "CJK COMPATIBILITY IDEOGRAPH-2F9BC" },
    { 0x2F9BD, "CJK COMPATIBILITY IDEOGRAPH-2F9BD" },
    { 0x2F9BE, "CJK COMPATIBILITY IDEOGRAPH-2F9BE" },
    { 0x2F9BF, "CJK COMPATIBILITY IDEOGRAPH-2F9BF" },
    { 0x2F9C0, "CJK COMPATIBILITY IDEOGRAPH-2F9C0" },
    { 0x2F9C1, "CJK COMPATIBILITY IDEOGRAPH-2F9C1" },
    { 0x2F9C2, "CJK COMPATIBILITY IDEOGRAPH-2F9C2" },
    { 0x2F9C3, "CJK COMPATIBILITY IDEOGRAPH-2F9C3" },
    { 0x2F9C4, "CJK COMPATIBILITY IDEOGRAPH-2F9C4" },
    { 0x2F9C5, "CJK COMPATIBILITY IDEOGRAPH-2F9C5" },
    { 0x2F9C6, "CJK COMPATIBILITY IDEOGRAPH-2F9C6" },
    { 0x2F9C7, "CJK COMPATIBILITY IDEOGRAPH-2F9C7" },
    { 0x2F9C8, "CJK COMPATIBILITY IDEOGRAPH-2F9C8" },
    { 0x2F9C9, "CJK COMPATIBILITY IDEOGRAPH-2F9C9" },
    { 0x2F9CA, "CJK COMPATIBILITY IDEOGRAPH-2F9CA" },
    { 0x2F9CB, "CJK COMPATIBILITY IDEOGRAPH-2F9CB" },
    { 0x2F9CC, "CJK COMPATIBILITY IDEOGRAPH-2F9CC" },
    { 0x2F9CD, "CJK COMPATIBILITY IDEOGRAPH-2F9CD" },
    { 0x2F9CE, "CJK COMPATIBILITY IDEOGRAPH-2F9CE" },
    { 0x2F9CF, "CJK COMPATIBILITY IDEOGRAPH-2F9CF" },
    { 0x2F9D0, "CJK COMPATIBILITY IDEOGRAPH-2F9D0" },
    { 0x2F9D1, "CJK COMPATIBILITY IDEOGRAPH-2F9D1" },
    { 0x2F9D2, "CJK COMPATIBILITY IDEOGRAPH-2F9D2" },
    { 0x2F9D3, "CJK COMPATIBILITY IDEOGRAPH-2F9D3" },
    { 0x2F9D4, "CJK COMPATIBILITY IDEOGRAPH-2F9D4" },
    { 0x2F9D5, "CJK COMPATIBILITY IDEOGRAPH-2F9D5" },
    { 0x2F9D6, "CJK COMPATIBILITY IDEOGRAPH-2F9D6" },
    { 0x2F9D7, "CJK COMPATIBILITY IDEOGRAPH-2F9D7" },
    { 0x2F9D8, "CJK COMPATIBILITY IDEOGRAPH-2F9D8" },
    { 0x2F9D9, "CJK COMPATIBILITY IDEOGRAPH-2F9D9" },
    { 0x2F9DA, "CJK COMPATIBILITY IDEOGRAPH-2F9DA" },
    { 0x2F9DB, "CJK COMPATIBILITY IDEOGRAPH-2F9DB" },
    { 0x2F9DC, "CJK COMPATIBILITY IDEOGRAPH-2F9DC" },
    { 0x2F9DD, "CJK COMPATIBILITY IDEOGRAPH-2F9DD" },
    { 0x2F9DE, "CJK COMPATIBILITY IDEOGRAPH-2F9DE" },
    { 0x2F9DF, "CJK COMPATIBILITY IDEOGRAPH-2F9DF" },
    { 0x2F9E0, "CJK COMPATIBILITY IDEOGRAPH-2F9E0" },
    { 0x2F9E1, "CJK COMPATIBILITY IDEOGRAPH-2F9E1" },
    { 0x2F9E2, "CJK COMPATIBILITY IDEOGRAPH-2F9E2" },
    { 0x2F9E3, "CJK COMPATIBILITY IDEOGRAPH-2F9E3" },
    { 0x2F9E4, "CJK COMPATIBILITY IDEOGRAPH-2F9E4" },
    { 0x2F9E5, "CJK COMPATIBILITY IDEOGRAPH-2F9E5" },
    { 0x2F9E6, "CJK COMPATIBILITY IDEOGRAPH-2F9E6" },
    { 0x2F9E7, "CJK COMPATIBILITY IDEOGRAPH-2F9E7" },
    { 0x2F9E8, "CJK COMPATIBILITY IDEOGRAPH-2F9E8" },
    { 0x2F9E9, "CJK COMPATIBILITY IDEOGRAPH-2F9E9" },
    { 0x2F9EA, "CJK COMPATIBILITY IDEOGRAPH-2F9EA" },
    { 0x2F9EB, "CJK COMPATIBILITY IDEOGRAPH-2F9EB" },
    { 0x2F9EC, "CJK COMPATIBILITY IDEOGRAPH-2F9EC" },
    { 0x2F9ED, "CJK COMPATIBILITY IDEOGRAPH-2F9ED" },
    { 0x2F9EE, "CJK COMPATIBILITY IDEOGRAPH-2F9EE" },
    { 0x2F9EF, "CJK COMPATIBILITY IDEOGRAPH-2F9EF" },
    { 0x2F9F0, "CJK COMPATIBILITY IDEOGRAPH-2F9F0" },
    { 0x2F9F1, "CJK COMPATIBILITY IDEOGRAPH-2F9F1" },
    { 0x2F9F2, "CJK COMPATIBILITY IDEOGRAPH-2F9F2" },
    { 0x2F9F3, "CJK COMPATIBILITY IDEOGRAPH-2F9F3" },
    { 0x2F9F4, "CJK COMPATIBILITY IDEOGRAPH-2F9F4" },
    { 0x2F9F5, "CJK COMPATIBILITY IDEOGRAPH-2F9F5" },
    { 0x2F9F6, "CJK COMPATIBILITY IDEOGRAPH-2F9F6" },
    { 0x2F9F7, "CJK COMPATIBILITY IDEOGRAPH-2F9F7" },
    { 0x2F9F8, "CJK COMPATIBILITY IDEOGRAPH-2F9F8" },
    { 0x2F9F9, "CJK COMPATIBILITY IDEOGRAPH-2F9F9" },
    { 0x2F9FA, "CJK COMPATIBILITY IDEOGRAPH-2F9FA" },
    { 0x2F9FB, "CJK COMPATIBILITY IDEOGRAPH-2F9FB" },
    { 0x2F9FC, "CJK COMPATIBILITY IDEOGRAPH-2F9FC" },
    { 0x2F9FD, "CJK COMPATIBILITY IDEOGRAPH-2F9FD" },
    { 0x2F9FE, "CJK COMPATIBILITY IDEOGRAPH-2F9FE" },
    { 0x2F9FF, "CJK COMPATIBILITY IDEOGRAPH-2F9FF" },
    { 0x2FA00, "CJK COMPATIBILITY IDEOGRAPH-2FA00" },
    { 0x2FA01, "CJK COMPATIBILITY IDEOGRAPH-2FA01" },
    { 0x2FA02, "CJK COMPATIBILITY IDEOGRAPH-2FA02" },
    { 0x2FA03, "CJK COMPATIBILITY IDEOGRAPH-2FA03" },
    { 0x2FA04, "CJK COMPATIBILITY IDEOGRAPH-2FA04" },
    { 0x2FA05, "CJK COMPATIBILITY IDEOGRAPH-2FA05" },
    { 0x2FA06, "CJK COMPATIBILITY IDEOGRAPH-2FA06" },
    { 0x2FA07, "CJK COMPATIBILITY IDEOGRAPH-2FA07" },
    { 0x2FA08, "CJK COMPATIBILITY IDEOGRAPH-2FA08" },
    { 0x2FA09, "CJK COMPATIBILITY IDEOGRAPH-2FA09" },
    { 0x2FA0A, "CJK COMPATIBILITY IDEOGRAPH-2FA0A" },
    { 0x2FA0B, "CJK COMPATIBILITY IDEOGRAPH-2FA0B" },
    { 0x2FA0C, "CJK COMPATIBILITY IDEOGRAPH-2FA0C" },
    { 0x2FA0D, "CJK COMPATIBILITY IDEOGRAPH-2FA0D" },
    { 0x2FA0E, "CJK COMPATIBILITY IDEOGRAPH-2FA0E" },
    { 0x2FA0F, "CJK COMPATIBILITY IDEOGRAPH-2FA0F" },
    { 0x2FA10, "CJK COMPATIBILITY IDEOGRAPH-2FA10" },
    { 0x2FA11, "CJK COMPATIBILITY IDEOGRAPH-2FA11" },
    { 0x2FA12, "CJK COMPATIBILITY IDEOGRAPH-2FA12" },
    { 0x2FA13, "CJK COMPATIBILITY IDEOGRAPH-2FA13" },
    { 0x2FA14, "CJK COMPATIBILITY IDEOGRAPH-2FA14" },
    { 0x2FA15, "CJK COMPATIBILITY IDEOGRAPH-2FA15" },
    { 0x2FA16, "CJK COMPATIBILITY IDEOGRAPH-2FA16" },
    { 0x2FA17, "CJK COMPATIBILITY IDEOGRAPH-2FA17" },
    { 0x2FA18, "CJK COMPATIBILITY IDEOGRAPH-2FA18" },
    { 0x2FA19, "CJK COMPATIBILITY IDEOGRAPH-2FA19" },
    { 0x2FA1A, "CJK COMPATIBILITY IDEOGRAPH-2FA1A" },
    { 0x2FA1B, "CJK COMPATIBILITY IDEOGRAPH-2FA1B" },
    { 0x2FA1C, "CJK COMPATIBILITY IDEOGRAPH-2FA1C" },
    { 0x2FA1D, "CJK COMPATIBILITY IDEOGRAPH-2FA1D" },
    { 0xE0001, "LANGUAGE TAG" },
    { 0xE0020, "TAG SPACE" },
    { 0xE0021, "TAG EXCLAMATION MARK" },
    { 0xE0022, "TAG QUOTATION MARK" },
    { 0xE0023, "TAG NUMBER SIGN" },
    { 0xE0024, "TAG DOLLAR SIGN" },
    { 0xE0025, "TAG PERCENT SIGN" },
    { 0xE0026, "TAG AMPERSAND" },
    { 0xE0027, "TAG APOSTROPHE" },
    { 0xE0028, "TAG LEFT PARENTHESIS" },
    { 0xE0029, "TAG RIGHT PARENTHESIS" },
    { 0xE002A, "TAG ASTERISK" },
    { 0xE002B, "TAG PLUS SIGN" },
    { 0xE002C, "TAG COMMA" },
    { 0xE002D, "TAG HYPHEN-MINUS" },
    { 0xE002E, "TAG FULL STOP" },
    { 0xE002F, "TAG SOLIDUS" },
    { 0xE0030, "TAG DIGIT ZERO" },
    { 0xE0031, "TAG DIGIT ONE" },
    { 0xE0032, "TAG DIGIT TWO" },
    { 0xE0033, "TAG DIGIT THREE" },
    { 0xE0034, "TAG DIGIT FOUR" },
    { 0xE0035, "TAG DIGIT FIVE" },
    { 0xE0036, "TAG DIGIT SIX" },
    { 0xE0037, "TAG DIGIT SEVEN" },
    { 0xE0038, "TAG DIGIT EIGHT" },
    { 0xE0039, "TAG DIGIT NINE" },
    { 0xE003A, "TAG COLON" },
    { 0xE003B, "TAG SEMICOLON" },
    { 0xE003C, "TAG LESS-THAN SIGN" },
    { 0xE003D, "TAG EQUALS SIGN" },
    { 0xE003E, "TAG GREATER-THAN SIGN" },
    { 0xE003F, "TAG QUESTION MARK" },
    { 0xE0040, "TAG COMMERCIAL AT" },
    { 0xE0041, "TAG LATIN CAPITAL LETTER A" },
    { 0xE0042, "TAG LATIN CAPITAL LETTER B" },
    { 0xE0043, "TAG LATIN CAPITAL LETTER C" },
    { 0xE0044, "TAG LATIN CAPITAL LETTER D" },
    { 0xE0045, "TAG LATIN CAPITAL LETTER E" },
    { 0xE0046, "TAG LATIN CAPITAL LETTER F" },
    { 0xE0047, "TAG LATIN CAPITAL LETTER G" },
    { 0xE0048, "TAG LATIN CAPITAL LETTER H" },
    { 0xE0049, "TAG LATIN CAPITAL LETTER I" },
    { 0xE004A, "TAG LATIN CAPITAL LETTER J" },
    { 0xE004B, "TAG LATIN CAPITAL LETTER K" },
    { 0xE004C, "TAG LATIN CAPITAL LETTER L" },
    { 0xE004D, "TAG LATIN CAPITAL LETTER M" },
    { 0xE004E, "TAG LATIN CAPITAL LETTER N" },
    { 0xE004F, "TAG LATIN CAPITAL LETTER O" },
    { 0xE0050, "TAG LATIN CAPITAL LETTER P" },
    { 0xE0051, "TAG LATIN CAPITAL LETTER Q" },
    { 0xE0052, "TAG LATIN CAPITAL LETTER R" },
    { 0xE0053, "TAG LATIN CAPITAL LETTER S" },
    { 0xE0054, "TAG LATIN CAPITAL LETTER T" },
    { 0xE0055, "TAG LATIN CAPITAL LETTER U" },
    { 0xE0056, "TAG LATIN CAPITAL LETTER V" },
    { 0xE0057, "TAG LATIN CAPITAL LETTER W" },
    { 0xE0058, "TAG LATIN CAPITAL LETTER X" },
    { 0xE0059, "TAG LATIN CAPITAL LETTER Y" },
    { 0xE005A, "TAG LATIN CAPITAL LETTER Z" },
    { 0xE005B, "TAG LEFT SQUARE BRACKET" },
    { 0xE005C, "TAG REVERSE SOLIDUS" },
    { 0xE005D, "TAG RIGHT SQUARE BRACKET" },
    { 0xE005E, "TAG CIRCUMFLEX ACCENT" },
    { 0xE005F, "TAG LOW LINE" },
    { 0xE0060, "TAG GRAVE ACCENT" },
    { 0xE0061, "TAG LATIN SMALL LETTER A" },
    { 0xE0062, "TAG LATIN SMALL LETTER B" },
    { 0xE0063, "TAG LATIN SMALL LETTER C" },
    { 0xE0064, "TAG LATIN SMALL LETTER D" },
    { 0xE0065, "TAG LATIN SMALL LETTER E" },
    { 0xE0066, "TAG LATIN SMALL LETTER F" },
    { 0xE0067, "TAG LATIN SMALL LETTER G" },
    { 0xE0068, "TAG LATIN SMALL LETTER H" },
    { 0xE0069, "TAG LATIN SMALL LETTER I" },
    { 0xE006A, "TAG LATIN SMALL LETTER J" },
    { 0xE006B, "TAG LATIN SMALL LETTER K" },
    { 0xE006C, "TAG LATIN SMALL LETTER L" },
    { 0xE006D, "TAG LATIN SMALL LETTER M" },
    { 0xE006E, "TAG LATIN SMALL LETTER N" },
    { 0xE006F, "TAG LATIN SMALL LETTER O" },
    { 0xE0070, "TAG LATIN SMALL LETTER P" },
    { 0xE0071, "TAG LATIN SMALL LETTER Q" },
    { 0xE0072, "TAG LATIN SMALL LETTER R" },
    { 0xE0073, "TAG LATIN SMALL LETTER S" },
    { 0xE0074, "TAG LATIN SMALL LETTER T" },
    { 0xE0075, "TAG LATIN SMALL LETTER U" },
    { 0xE0076, "TAG LATIN SMALL LETTER V" },
    { 0xE0077, "TAG LATIN SMALL LETTER W" },
    { 0xE0078, "TAG LATIN SMALL LETTER X" },
    { 0xE0079, "TAG LATIN SMALL LETTER Y" },
    { 0xE007A, "TAG LATIN SMALL LETTER Z" },
    { 0xE007B, "TAG LEFT CURLY BRACKET" },
    { 0xE007C, "TAG VERTICAL LINE" },
    { 0xE007D, "TAG RIGHT CURLY BRACKET" },
    { 0xE007E, "TAG TILDE" },
    { 0xE007F, "CANCEL TAG" },
    { 0xE0100, "VARIATION SELECTOR-17" },
    { 0xE0101, "VARIATION SELECTOR-18" },
    { 0xE0102, "VARIATION SELECTOR-19" },
    { 0xE0103, "VARIATION SELECTOR-20" },
    { 0xE0104, "VARIATION SELECTOR-21" },
    { 0xE0105, "VARIATION SELECTOR-22" },
    { 0xE0106, "VARIATION SELECTOR-23" },
    { 0xE0107, "VARIATION SELECTOR-24" },
    { 0xE0108, "VARIATION SELECTOR-25" },
    { 0xE0109, "VARIATION SELECTOR-26" },
    { 0xE010A, "VARIATION SELECTOR-27" },
    { 0xE010B, "VARIATION SELECTOR-28" },
    { 0xE010C, "VARIATION SELECTOR-29" },
    { 0xE010D, "VARIATION SELECTOR-30" },
    { 0xE010E, "VARIATION SELECTOR-31" },
    { 0xE010F, "VARIATION SELECTOR-32" },
    { 0xE0110, "VARIATION SELECTOR-33" },
    { 0xE0111, "VARIATION SELECTOR-34" },
    { 0xE0112, "VARIATION SELECTOR-35" },
    { 0xE0113, "VARIATION SELECTOR-36" },
    { 0xE0114, "VARIATION SELECTOR-37" },
    { 0xE0115, "VARIATION SELECTOR-38" },
    { 0xE0116, "VARIATION SELECTOR-39" },
    { 0xE0117, "VARIATION SELECTOR-40" },
    { 0xE0118, "VARIATION SELECTOR-41" },
    { 0xE0119, "VARIATION SELECTOR-42" },
    { 0xE011A, "VARIATION SELECTOR-43" },
    { 0xE011B, "VARIATION SELECTOR-44" },
    { 0xE011C, "VARIATION SELECTOR-45" },
    { 0xE011D, "VARIATION SELECTOR-46" },
    { 0xE011E, "VARIATION SELECTOR-47" },
    { 0xE011F, "VARIATION SELECTOR-48" },
    { 0xE0120, "VARIATION SELECTOR-49" },
    { 0xE0121, "VARIATION SELECTOR-50" },
    { 0xE0122, "VARIATION SELECTOR-51" },
    { 0xE0123, "VARIATION SELECTOR-52" },
    { 0xE0124, "VARIATION SELECTOR-53" },
    { 0xE0125, "VARIATION SELECTOR-54" },
    { 0xE0126, "VARIATION SELECTOR-55" },
    { 0xE0127, "VARIATION SELECTOR-56" },
    { 0xE0128, "VARIATION SELECTOR-57" },
    { 0xE0129, "VARIATION SELECTOR-58" },
    { 0xE012A, "VARIATION SELECTOR-59" },
    { 0xE012B, "VARIATION SELECTOR-60" },
    { 0xE012C, "VARIATION SELECTOR-61" },
    { 0xE012D, "VARIATION SELECTOR-62" },
    { 0xE012E, "VARIATION SELECTOR-63" },
    { 0xE012F, "VARIATION SELECTOR-64" },
    { 0xE0130, "VARIATION SELECTOR-65" },
    { 0xE0131, "VARIATION SELECTOR-66" },
    { 0xE0132, "VARIATION SELECTOR-67" },
    { 0xE0133, "VARIATION SELECTOR-68" },
    { 0xE0134, "VARIATION SELECTOR-69" },
    { 0xE0135, "VARIATION SELECTOR-70" },
    { 0xE0136, "VARIATION SELECTOR-71" },
    { 0xE0137, "VARIATION SELECTOR-72" },
    { 0xE0138, "VARIATION SELECTOR-73" },
    { 0xE0139, "VARIATION SELECTOR-74" },
    { 0xE013A, "VARIATION SELECTOR-75" },
    { 0xE013B, "VARIATION SELECTOR-76" },
    { 0xE013C, "VARIATION SELECTOR-77" },
    { 0xE013D, "VARIATION SELECTOR-78" },
    { 0xE013E, "VARIATION SELECTOR-79" },
    { 0xE013F, "VARIATION SELECTOR-80" },
    { 0xE0140, "VARIATION SELECTOR-81" },
    { 0xE0141, "VARIATION SELECTOR-82" },
    { 0xE0142, "VARIATION SELECTOR-83" },
    { 0xE0143, "VARIATION SELECTOR-84" },
    { 0xE0144, "VARIATION SELECTOR-85" },
    { 0xE0145, "VARIATION SELECTOR-86" },
    { 0xE0146, "VARIATION SELECTOR-87" },
    { 0xE0147, "VARIATION SELECTOR-88" },
    { 0xE0148, "VARIATION SELECTOR-89" },
    { 0xE0149, "VARIATION SELECTOR-90" },
    { 0xE014A, "VARIATION SELECTOR-91" },
    { 0xE014B, "VARIATION SELECTOR-92" },
    { 0xE014C, "VARIATION SELECTOR-93" },
    { 0xE014D, "VARIATION SELECTOR-94" },
    { 0xE014E, "VARIATION SELECTOR-95" },
    { 0xE014F, "VARIATION SELECTOR-96" },
    { 0xE0150, "VARIATION SELECTOR-97" },
    { 0xE0151, "VARIATION SELECTOR-98" },
    { 0xE0152, "VARIATION SELECTOR-99" },
    { 0xE0153, "VARIATION SELECTOR-100" },
    { 0xE0154, "VARIATION SELECTOR-101" },
    { 0xE0155, "VARIATION SELECTOR-102" },
    { 0xE0156, "VARIATION SELECTOR-103" },
    { 0xE0157, "VARIATION SELECTOR-104" },
    { 0xE0158, "VARIATION SELECTOR-105" },
    { 0xE0159, "VARIATION SELECTOR-106" },
    { 0xE015A, "VARIATION SELECTOR-107" },
    { 0xE015B, "VARIATION SELECTOR-108" },
    { 0xE015C, "VARIATION SELECTOR-109" },
    { 0xE015D, "VARIATION SELECTOR-110" },
    { 0xE015E, "VARIATION SELECTOR-111" },
    { 0xE015F, "VARIATION SELECTOR-112" },
    { 0xE0160, "VARIATION SELECTOR-113" },
    { 0xE0161, "VARIATION SELECTOR-114" },
    { 0xE0162, "VARIATION SELECTOR-115" },
    { 0xE0163, "VARIATION SELECTOR-116" },
    { 0xE0164, "VARIATION SELECTOR-117" },
    { 0xE0165, "VARIATION SELECTOR-118" },
    { 0xE0166, "VARIATION SELECTOR-119" },
    { 0xE0167, "VARIATION SELECTOR-120" },
    { 0xE0168, "VARIATION SELECTOR-121" },
    { 0xE0169, "VARIATION SELECTOR-122" },
    { 0xE016A, "VARIATION SELECTOR-123" },
    { 0xE016B, "VARIATION SELECTOR-124" },
    { 0xE016C, "VARIATION SELECTOR-125" },
    { 0xE016D, "VARIATION SELECTOR-126" },
    { 0xE016E, "VARIATION SELECTOR-127" },
    { 0xE016F, "VARIATION SELECTOR-128" },
    { 0xE0170, "VARIATION SELECTOR-129" },
    { 0xE0171, "VARIATION SELECTOR-130" },
    { 0xE0172, "VARIATION SELECTOR-131" },
    { 0xE0173, "VARIATION SELECTOR-132" },
    { 0xE0174, "VARIATION SELECTOR-133" },
    { 0xE0175, "VARIATION SELECTOR-134" },
    { 0xE0176, "VARIATION SELECTOR-135" },
    { 0xE0177, "VARIATION SELECTOR-136" },
    { 0xE0178, "VARIATION SELECTOR-137" },
    { 0xE0179, "VARIATION SELECTOR-138" },
    { 0xE017A, "VARIATION SELECTOR-139" },
    { 0xE017B, "VARIATION SELECTOR-140" },
    { 0xE017C, "VARIATION SELECTOR-141" },
    { 0xE017D, "VARIATION SELECTOR-142" },
    { 0xE017E, "VARIATION SELECTOR-143" },
    { 0xE017F, "VARIATION SELECTOR-144" },
    { 0xE0180, "VARIATION SELECTOR-145" },
    { 0xE0181, "VARIATION SELECTOR-146" },
    { 0xE0182, "VARIATION SELECTOR-147" },
    { 0xE0183, "VARIATION SELECTOR-148" },
    { 0xE0184, "VARIATION SELECTOR-149" },
    { 0xE0185, "VARIATION SELECTOR-150" },
    { 0xE0186, "VARIATION SELECTOR-151" },
    { 0xE0187, "VARIATION SELECTOR-152" },
    { 0xE0188, "VARIATION SELECTOR-153" },
    { 0xE0189, "VARIATION SELECTOR-154" },
    { 0xE018A, "VARIATION SELECTOR-155" },
    { 0xE018B, "VARIATION SELECTOR-156" },
    { 0xE018C, "VARIATION SELECTOR-157" },
    { 0xE018D, "VARIATION SELECTOR-158" },
    { 0xE018E, "VARIATION SELECTOR-159" },
    { 0xE018F, "VARIATION SELECTOR-160" },
    { 0xE0190, "VARIATION SELECTOR-161" },
    { 0xE0191, "VARIATION SELECTOR-162" },
    { 0xE0192, "VARIATION SELECTOR-163" },
    { 0xE0193, "VARIATION SELECTOR-164" },
    { 0xE0194, "VARIATION SELECTOR-165" },
    { 0xE0195, "VARIATION SELECTOR-166" },
    { 0xE0196, "VARIATION SELECTOR-167" },
    { 0xE0197, "VARIATION SELECTOR-168" },
    { 0xE0198, "VARIATION SELECTOR-169" },
    { 0xE0199, "VARIATION SELECTOR-170" },
    { 0xE019A, "VARIATION SELECTOR-171" },
    { 0xE019B, "VARIATION SELECTOR-172" },
    { 0xE019C, "VARIATION SELECTOR-173" },
    { 0xE019D, "VARIATION SELECTOR-174" },
    { 0xE019E, "VARIATION SELECTOR-175" },
    { 0xE019F, "VARIATION SELECTOR-176" },
    { 0xE01A0, "VARIATION SELECTOR-177" },
    { 0xE01A1, "VARIATION SELECTOR-178" },
    { 0xE01A2, "VARIATION SELECTOR-179" },
    { 0xE01A3, "VARIATION SELECTOR-180" },
    { 0xE01A4, "VARIATION SELECTOR-181" },
    { 0xE01A5, "VARIATION SELECTOR-182" },
    { 0xE01A6, "VARIATION SELECTOR-183" },
    { 0xE01A7, "VARIATION SELECTOR-184" },
    { 0xE01A8, "VARIATION SELECTOR-185" },
    { 0xE01A9, "VARIATION SELECTOR-186" },
    { 0xE01AA, "VARIATION SELECTOR-187" },
    { 0xE01AB, "VARIATION SELECTOR-188" },
    { 0xE01AC, "VARIATION SELECTOR-189" },
    { 0xE01AD, "VARIATION SELECTOR-190" },
    { 0xE01AE, "VARIATION SELECTOR-191" },
    { 0xE01AF, "VARIATION SELECTOR-192" },
    { 0xE01B0, "VARIATION SELECTOR-193" },
    { 0xE01B1, "VARIATION SELECTOR-194" },
    { 0xE01B2, "VARIATION SELECTOR-195" },
    { 0xE01B3, "VARIATION SELECTOR-196" },
    { 0xE01B4, "VARIATION SELECTOR-197" },
    { 0xE01B5, "VARIATION SELECTOR-198" },
    { 0xE01B6, "VARIATION SELECTOR-199" },
    { 0xE01B7, "VARIATION SELECTOR-200" },
    { 0xE01B8, "VARIATION SELECTOR-201" },
    { 0xE01B9, "VARIATION SELECTOR-202" },
    { 0xE01BA, "VARIATION SELECTOR-203" },
    { 0xE01BB, "VARIATION SELECTOR-204" },
    { 0xE01BC, "VARIATION SELECTOR-205" },
    { 0xE01BD, "VARIATION SELECTOR-206" },
    { 0xE01BE, "VARIATION SELECTOR-207" },
    { 0xE01BF, "VARIATION SELECTOR-208" },
    { 0xE01C0, "VARIATION SELECTOR-209" },
    { 0xE01C1, "VARIATION SELECTOR-210" },
    { 0xE01C2, "VARIATION SELECTOR-211" },
    { 0xE01C3, "VARIATION SELECTOR-212" },
    { 0xE01C4, "VARIATION SELECTOR-213" },
    { 0xE01C5, "VARIATION SELECTOR-214" },
    { 0xE01C6, "VARIATION SELECTOR-215" },
    { 0xE01C7, "VARIATION SELECTOR-216" },
    { 0xE01C8, "VARIATION SELECTOR-217" },
    { 0xE01C9, "VARIATION SELECTOR-218" },
    { 0xE01CA, "VARIATION SELECTOR-219" },
    { 0xE01CB, "VARIATION SELECTOR-220" },
    { 0xE01CC, "VARIATION SELECTOR-221" },
    { 0xE01CD, "VARIATION SELECTOR-222" },
    { 0xE01CE, "VARIATION SELECTOR-223" },
    { 0xE01CF, "VARIATION SELECTOR-224" },
    { 0xE01D0, "VARIATION SELECTOR-225" },
    { 0xE01D1, "VARIATION SELECTOR-226" },
    { 0xE01D2, "VARIATION SELECTOR-227" },
    { 0xE01D3, "VARIATION SELECTOR-228" },
    { 0xE01D4, "VARIATION SELECTOR-229" },
    { 0xE01D5, "VARIATION SELECTOR-230" },
    { 0xE01D6, "VARIATION SELECTOR-231" },
    { 0xE01D7, "VARIATION SELECTOR-232" },
    { 0xE01D8, "VARIATION SELECTOR-233" },
    { 0xE01D9, "VARIATION SELECTOR-234" },
    { 0xE01DA, "VARIATION SELECTOR-235" },
    { 0xE01DB, "VARIATION SELECTOR-236" },
    { 0xE01DC, "VARIATION SELECTOR-237" },
    { 0xE01DD, "VARIATION SELECTOR-238" },
    { 0xE01DE, "VARIATION SELECTOR-239" },
    { 0xE01DF, "VARIATION SELECTOR-240" },
    { 0xE01E0, "VARIATION SELECTOR-241" },
    { 0xE01E1, "VARIATION SELECTOR-242" },
    { 0xE01E2, "VARIATION SELECTOR-243" },
    { 0xE01E3, "VARIATION SELECTOR-244" },
    { 0xE01E4, "VARIATION SELECTOR-245" },
    { 0xE01E5, "VARIATION SELECTOR-246" },
    { 0xE01E6, "VARIATION SELECTOR-247" },
    { 0xE01E7, "VARIATION SELECTOR-248" },
    { 0xE01E8, "VARIATION SELECTOR-249" },
    { 0xE01E9, "VARIATION SELECTOR-250" },
    { 0xE01EA, "VARIATION SELECTOR-251" },
    { 0xE01EB, "VARIATION SELECTOR-252" },
    { 0xE01EC, "VARIATION SELECTOR-253" },
    { 0xE01ED, "VARIATION SELECTOR-254" },
    { 0xE01EE, "VARIATION SELECTOR-255" },
    { 0xE01EF, "VARIATION SELECTOR-256" },
  };
